
/* grDraw.h --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * File contains procedures to draw things on the screen.
 *
 * Copyright(C) 1988 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header$
 */

#ifndef _GRDRAW
#define _GRDRAW /* once-only latch */

#ifndef BEAR_BLACK_BOX

typedef struct _DrawNet {
    Net             *netPtr;
    CellMaster      *cellMasterPtr;
    int             whichNet;       /* FPG_NW | FPG_CENTER | FPG_SE */
    Pixel           netColor;
} DrawNet;

/* Note DrawNode should have been named DrawPinOrJunct. */
typedef struct _DrawNode {
    PinOrJunct      *pinOrJunctPtr;
    Pixel           pinColor;       /* pin color */
    Pixel           junctColor;     /* junction color */
} DrawNode;

typedef struct _DrawPin {
    PinInst         *pinInstPtr;
    Pixel           pinColor;
} DrawPin;

typedef struct _DrawPort {
    PortInst        *portInstPtr;
    Pixel           portColor;
} DrawPort;

typedef struct _DrawGraph {
    Gt_Graph        graph;

    PlaneData       *planeDataPtr;
    Pixel           graphColor;
    bool            dotFlag;
    bool            widthFlag;
} DrawGraph;

typedef struct _DrawGraphNode {
    Gt_GraphNode    node;

    PlaneData       *planeDataPtr;
    Pixel           nodeColor;
    bool            dotFlag;
} DrawGraphNode;

typedef struct _DrawGraphEdge {
    Gt_GraphEdge    edge;

    PlaneData       *planeDataPtr;
    Pixel           edgeColor;
    bool            dotFlag;
    bool            widthFlag;
} DrawGraphEdge;

typedef struct _LabelNet {
    Net             *netPtr;
    PlaneData       *planeDataPtr;
    int             whichNet;       /* FPG_NW | FPG_CENTER | FPG_SE */
} LabelNet;

extern void Gre_UsePgWireDataToDrawNet(/* usePgWireData */);
/*
 * bool             usePgWireData;
 */

extern void Gre_ChooseWallJunct(/* whichJunct */);
/*
 * int              whichJunct;     (* FPG_NW | FPG_CENTER | FPG_SE *)
 */

extern BearStatus Gre_DrawNet(/* winDataPtr, drawNetPtr */);
/*
 * WindowData       *winDataPtr;
 * DrawNet          *drawNetPtr;
 */

extern BearStatus Gre_DrawGraph(/* winDataPtr, drawGraphPtr */);
/*
 * WindowData       *winDataPtr;
 * DrawGraph        *drawGraphPtr;
 */

extern BearStatus Gre_DrawGraphNode(/* winDataPtr, drawGraphNodePtr */);
/*
 * WindowData       *winDataPtr;
 * DrawGraphNode    *drawGraphNodePtr;
 */

extern BearStatus Gre_DrawGraphEdge(/* winDataPtr, drawGraphEdgePtr */);
/*
 * WindowData       *winDataPtr;
 * DrawGraphEdge    *drawGraphEdgePtr;
 */

extern BearStatus Gre_LabelNet(/* winDataPtr, labelNetPtr */);
/*
 * WindowData       *winDataPtr;
 * DrawNet          *labelNetPtr;
 */

extern BearStatus Gre_DrawNode(/* winDataPtr, drawNodePtr */);
/*
 * WindowData       *winDataPtr;
 * DrawNodeData     *drawNodePtr;
 */

extern void Gre_DrawAllPins(/* drawDataPtr */);
/*
 * DrawData         *drawDataPtr;
 */

extern BearStatus Gre_DrawPinOne(/* pinInstPtr, drawDataPtr */);
/*
 * PinInst      *pinInstPtr;
 * DrawData     *drawDataPtr; 
 */

extern BearStatus Gre_DrawPinTwo(/* winDataPtr, drawPinPtr */);
/*
 * WindowData   *winDataPtr;
 * DrawPin      *drawPinPtr;
 */

extern BearStatus Gre_ErasePin(/* pinInstPtr, drawDataPtr */);
/*
 * PinInst      *pinInstPtr;
 * DrawData     *drawDataPtr; 
 */

extern BearStatus Gre_DrawPinsOfCell(/* cellInstPtr, drawDataPtr */);
/*
 * CellInst     *cellInstPtr;
 * DrawData     *drawDataPtr;
 */

extern BearStatus Gre_DrawPinsOfNet(/* netPtr, drawDataPtr */);
/*
 * Net          *netPtr;
 * DrawData     *drawDataPtr;
 */

extern BearStatus Gre_DrawPinsOfPortOne(/* portInstPtr, drawDataPtr */);
/*
 * PortInst     *portInstPtr;
 * DrawData     *drawDataPtr;
 */

extern BearStatus Gre_DrawPinsOfPortTwo(/* winDataPtr, drawPortPtr */);
/*
 * WindowData   *winDataPtr;
 * DrawPort     *drawPortPtr;
 */

extern BearStatus Gre_DrawWallJunct(/* junctPtr, drawDataPtr */);
/*
 * WallJunction *junctPtr;
 * DrawData     *drawDataPtr;
 */

extern BearStatus Gre_DrawWallSeg(/* segPtr, drawDataPtr */);
/*
 * WallSegment  *segPtr;
 * DrawData     *drawDataPtr;
 */

extern void Gre_DrawFromJunctToJunct(/* drawData, junct1, junct2 */);
/*
 * DrawData     *drawData;
 * WallJunction *junct1, *junct2;
 */

extern void Gre_DrawFromDCellToJunct(/* drawDataPtr, pinDir, pinPoint, curJunctPtr */);
/*
 * DrawData        *drawDataPtr;
 * PinDirection    pinDir;
 * Point           pinPoint;
 * WallJunction    *curJunctPtr;
 */

extern void
Gre_DrawFromDCellToDCell(/* drawDataPtr,
    pinDir1, pinPoint1, pinDir2, pinPoint2 */);
/*
 * DrawData     *drawDataPtr;
 * PinDirection pinDir1;
 * Point        pinPoint1;
 * PinDirection pinDir2;
 * Point        pinPoint2;
 */

extern void
Gre_DrawFromPinToDCell(/* drawDataPtr, pinInst1Ptr, pinDir2, pinPoint2 */);
/*
 * DrawData     *drawDataPtr;
 * PinInst      *pinInst1Ptr;
 * PinDirection pinDir2;
 * Point        pinPoint2;
 */

extern void Gre_DrawFromPinToJunct(/* drawData, pinInst, curJunct */);
/*
 * DrawData     *drawData;
 * PinInst      *pinInst;
 * WallJunction *curJunct;
 */

extern void Gre_DrawFromPinToPin(/* drawData, pinInst1, pinInst2*/);
/*
 * DrawData     *drawData;
 * PinInst      *pinInst1, *pinInst2;
 */
 

extern void Gre_DrawFromRPinToJunct(/* drawDataPtr, rpinPtr, curJunctPtr */);
/*
 * DrawData     *drawDataPtr;
 * RoutePin     *rpinPtr;
 * WallJunction *curJunctPtr;
 */

extern void Gre_DrawFromRPinToPin(/* drawDataPtr, rpinPtr, pinInstPtr */);
/*
 * DrawData    *drawDataPtr;
 * RoutePin    *rpinPtr;
 * PinInst     *pinInstPtr;
 */

extern void Gre_DrawFromRPinToRPin(/* drawDataPtr, rpinPtr1, rpinPtr2 */);
/*
 * DrawData    *drawDataPtr;
 * RoutePin    *rpinPtr1;
 * PinInst     *rpinPtr2;
 */

#endif /* BEAR_BLACK_BOX */

#endif _GRDRAW

