
/* lrBBLInfo.h --
 *
 *     (Professor E. S. Kuh)
 *
 * Copyright (C) 1989 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header$
 */

#ifndef _LR_BBL_INFO
#define _LR_BBL_INFO

#include "clientData.h"

typedef struct locxy { /* coordinate structure */
    int xy[2];
} locxy;

struct schip {  
    struct module *mp;  
    struct designrl *desgnp;
    struct Signal *alls;    
};

typedef struct module {
    long utility;
    char *file;                 /*file name of the module*/
    char *nm;                   /*module name*/
    struct module *ansmp;       /*parent module pointer*/
    struct module *desmp;       /*descendant module chain*/
    struct module *sibmp;       /*same level module chain*/
    struct term   *mtc;         /*module terminal chain*/
    struct geom *geop;          /*pointer to module geometry*/
    struct locxy loc;           /*coordinates of module origin*/
    int rot ;                   /*rotation right=0 up=1 left=2 down=3*/
    int rfl;                    /*x reflection=0 y reflection=1*/
    int placg;                  /*decendent modules placed=1*/
    int type;                   /*routing module>0 regular module=0*/
    int globrt;                 /*module global routed yet*/

    /* newly added for interfacing with mac */
    struct sroot *rRootList;    /* List of roots inside the module */

    /* Newly added for ring router */
    char    *ptr;               /* This space for rent. */

    /* Newly added for Denis */
    char    *rCellPtr;
} module;

typedef struct rmpar {          /* parameter for routers*/
    int routbnd;                /* routing boundary mask level*/
                                /* In struct geom, the bndp array
                                 * may contain several boundaries
                                 * of the module for different 
                                 * purposes.  This routbnd is to
                                 * specify which boundary is for
                                 * this routing purpose.
                                 */

    int chdr;                   /* 0=vert 1=hor. 11=L*/
    int rtflag;                 /* router flag      */
                                /* = 0 if the parameter array is
                                 *     for channel router
                                 * = 1 if for 2-D router
                                 */
    int param[9];               /* parameter array   */
                                /* For L router,
                                   param[0] = topbound index
                                   param[1] = botbound index
                                   param[2] = x-y fold
                                   param[3] = xmin
                                   param[4] = xmax
                                   param[5] = ymin
                                   param[6] = ymax
                                   param[7] = zmin
                                   param[8] = zmax
                                 */
    int adjx;                   /* adjustment for x direction*/
    int adjy;                   /* adjustment for y direction*/
    int adjcor;                 /* L-shape corner adjustment*/
    int adjleft;                /* extend left side*/
    int adjright;               /* extend right side*/
} rmpar;

typedef struct geom {   
    long utility;
    struct gterm *gtp;          /* geometrical terminal array*/
    int lgtp;                   /* length of the array gtp*/
    struct rmpar *rpar;         /* routing parameter structure*/
    struct locxy *bndp;         /* corner coordinate array
                                 * --counter-clockwise
                                 * For different mask levels, we
                                 * can have different boundaries.
                                 * [0] element stores the mask 
                                 * level number and the number
                                 * of vertices. 'bndp' is a stack
                                 * of boundary corner coordinates
                                 */
    int lbndp;                  /* length of bndp array*/

    /* Newly added for ring router */
    struct gterm *gtinhd;       /* Head of gterm array for pins on inner
                                 * boundary.
                                 */
} geom;
        
typedef struct gterm {
        struct locxy loc;       /*coordinates to module origin*/
        struct locxy loc1;      /*extended location*/
        struct locxy loc2;
        char *nm;               /*name*/
        int eeg;                /*electrical equivalence*/
        int leg;                /*logical equivalence*/
        int rdg;                /*routing direction*/
                                /* <- 0; | 1; -> 2; ^ 3; */
        int placg;              /*placement flag*/
                                /*FLOAT,EDGE,FIX,RTFIX*/
        int clasg;              /*signal class flag - defines priority*/
                            /*>0:privileged--has priority & width caluculation*/
                                    /*0:ordinary nets*/
        /* Newly added for ring router */
        int vcflag;         /* vertical constrainst flag - YES or NO */
        /* Newly added for ring router */
        int cornerLabel;    /* set to a number between 1 to 8 if CORNER_PIN */
        float pwc;      /*power consumption data*/
                        /*unit is the same as width */
                    /*0<|ordinary nets|<=1*/
                    /* if >0 grow down or left
                       if <0 grow up or right  */
        int hwidth1;            /*half wire width on mask 1 */
        int hwidth2;        /*half wire width on mask 2 */
        int msklvl;     /*mask level*/
        /* Newly added for ring router */
        char *ptr;          /* This space for rent. */
} gterm;

/* flag for gterm placg */
#define FLOAT 0     /* floating terminal */
#define EDGE  1     /* edge assigned terminal */
#define FIX   2     /* originally fixed terminal */
#define RTFIX 3     /* terminal fixed by routing */

typedef struct Signal {
    long utility;
    struct Signal *alls;    /*all signal chain*/
    char *nm;       /*net name*/
    char *globalNetName;
    struct sroot *rtls;     /*routing root chain*/
    struct module *smp;     /*parent module*/ 
    struct term *trmls;     /*logical terminal chain*/
    /* Newly added for ring router */
    int cornerflag;     /*YES if signal has corner pad */
    int bundle;     /*bus*/
    /* Newly added for ring router (all the following five) */
    struct term *span1;
    struct term *span2;
    int track ;
    int trackdist ;
    char *ptr;          /* This space for rent. */
} Signal;

typedef struct term {   
    long utility;       /*16=135 degree extension*/
    struct term *mtc;   /*module terminal chain*/
    struct term *stc;   /*signal terminal chain*/
    struct term *subtc; /*terminal chain same net and module*/
    struct module *mp;  /*parent module*/
    struct Signal *sig;     /*parent signal*/
    struct rseg *rsp;   /*routing segment*/
    int tnum;       /*gtp index*/ 
    int mark;       
    /* Newly added for ring router */
    char *ptr;              /* This space for rent. */
    /* Newly added for link to mac route pins */
    char *genericPinPtr;
    ClientData	clientData; /* This space for rent. */
} term;

/* structures and members for routing */
union utjr {
    struct term *rt;    	/* ptr to a terminal */
    struct srjun *rj;   	/* ptr to a route junction */
};

typedef struct srjun {
    struct srjun *alljr;    /* routing junctions same root */
    struct rseg *sljr;      /* ptr into chain of routing segs */
    struct locxy locjr;     /* junction coordinates */
    short conjr;            /* 12=normal contact */
                            /* 16=135 degree turn */
                            /* 17=90 degree turn */
                            /* 18=octogon contact */
    unsigned flag;
    ClientData	clientData; /* This space for rent. */
} rjun;

typedef struct rseg {
    struct rseg *allsr;     /* routing segments same root */
    struct sroot *hsr;		/* ptr to route root */
    union utjr j0sr;    	/* ptr to junction or term, end 0 */
    union utjr j1sr;    	/* ptr to junction or term, end 1 */
    struct rseg *s0lsr;     /* ptr to another segment at end 0 */
    struct rseg *s1lsr;     /* ptr to another segment at end 1 */
    int widsr;              /* width down/left>0 up/right<0 */
    int hwidth;             /* half wire width */
    int msklvl ;            /* masklevel of this segment */
    unsigned t0fsr : 1;     /* set if end 0 at a terminal */
    unsigned t1fsr : 1;     /* set if end 1 at a terminal */
    ClientData	clientData; /* This space for rent. */
} rseg;

/* typedef for a routing root (root) */
typedef struct sroot {
    struct rseg  *allseg;       /* routing segments same root*/
    struct srjun *alljun;       /* routing junction same root*/
    /* newly added in for interfacing with mac */
    struct sroot *nextInRCell;  /* next root of the routing cell */
    /* newly added in for interfacing with mac */
    struct sroot *nextInNet;    /* next root of the net    */
    struct sroot *nrts;         /* ptr to another root, this signal */
    struct module *mp;          /* module that this root belongs to*/
    struct Signal *shr;         /* pointer to signal of this root */
} root;

/* ----------------------------------------------------------------- */

/* these centrally defined macros will aid in route-following */
#define whichend(JP,SP) (SP->j0sr.rj == (rjun *) JP ? 0 : 1)
#define farjsr(JP,SP) (SP->j0sr.rj == (rjun *) JP ? SP->j1sr.rj : SP->j0sr.rj)
#define nearslsr(JP,SP) (SP->j0sr.rj != (rjun *) JP ? SP->s1lsr : SP->s0lsr)
#define fartfsr(JP,SP) (SP->j0sr.rj == (rjun *) JP ? SP->t1fsr : SP->t0fsr)
#define neartfsr(JP,SP) (SP->j0sr.rj != (rjun *) JP ? SP->t1fsr : SP->t0fsr)

struct designrl {   /* design rule information*/
    int sp1;        /* minimum spacing on mask 1 */
    int sp2;        /* minimum spacing on mask 2 */
    int hwidth1;    /* minimum half width on mask 1 */
    int hwidth2;    /* minimum half width on mask 2 */
    int hvia;       /* half via size */
    int overlap;    /* minimum overlap width */
    int bcc;        /* boundary clearance constraint */
    int ftc;        /* floating terminal constraint */
    float w1tow2;   /* width conversion */
};


extern struct schip *chip;      /* chip pointer */

#endif _LR_BBL_INFO

