
/* macPort.h --
 *
 *     June Wang
 *     Wei-ming Dai
 *     (Prof. E. S. Kuh)
 *
 * It declares the data structures related to ports and the operations
 * used to delete and insert ports.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header: macPort.h,v 1.1 87/02/04 14:05:57 weiming Exp $
 */

#ifndef _MAC_PORT
#define _MAC_PORT

/* ---------- Ports ---------- */

typedef enum _PortDir
{
    INPUT, OUTPUT, INOUT
/* LOCAL, UNSPECIFIED */
} PortDir;

typedef enum _PortKind
{
    SIGNAL_PORT, PG_PORT, PG_SRC_PORT
} PortKind;

typedef struct _MinNomMax 
{
    int minimum;                        /* minimum value                    */
    int nominal;                        /* nominal value                    */
    int maximum;                        /* maximum value                    */
} MinNomMax;

typedef struct _SigPortProp
{
    MinNomMax          DCFanInLoad;     /* DC fan in load                   */ 
    MinNomMax          DCFanOutLoad;    /* DC fan out load                  */ 
    MinNomMax          DCMaxFanIn;      /* DC maximum fan in                */ 
    MinNomMax          DCMaxFanOut;     /* DC maximum fan out               */ 
    MinNomMax          ACLoad;          /* AC load                          */ 
} SigPortProp;

typedef struct _PGPortProp PGPortProp;
struct _PGPortProp {
    double maxVoltDrop;                 /* Maximum voltage drop             */
    double maxCurrentReq;               /* Maximum current reqirement       */
};

typedef struct _PortInst PortInst;
typedef struct _PortMaster PortMaster;

struct _PortMaster {
    BearString         name;            /* Port master name                 */
    PortDir            direction;       /* Port direction                   */
    PortKind           kind;            /* Port kind                        */
    union {
        SigPortProp    sigProp;         /* Signal port property             */
        PGPortProp     PGProp;          /* Power/ground port property       */
    } PropUn;
    PortInst           *instList;       /* The list of the port instances   */
    CellMaster         *cellMasterPtr;  /* The cell master to which the     */
                                        /* port master belongs              */
    struct _PinMaster  *pinMasterList;  /* The list of pin masters of
                                         * the port.
                                         */
    PortMaster         *nextPtr;        /* The next port master             */
    ClientData         clientData;      /* This space for rent              */
};

struct _PortInst {
    BearString         name;            /* Port instance name               */
    struct _LinkedLinkedPort *feedThrListPtr; /* The feedThr list which
                                               * the port instance is in.
                                               */
    struct _LinkedLinkedPort *eleEqvListPtr;  /* The eleEqv list which
                                               * the port instance is in.
                                               */
    PortMaster         *masterPtr;      /* The master of the port instance  */
    struct _Net        *netPtr;         /* The net to which the port instance 
                                         * belongs 
                                         */
    struct _Net        *intNetPtr;      /* The internal net to which the port 
                                         * instance belongs 
                                         */
    CellInst           *cellInstPtr;    /* The cell instance to which
                                         * the port instance belongs 
                                         */
    struct _PinInst    *pinList;        /* The list of the pins of the port */
    struct _PinInst    *assignedPinPtr; /* The pin assigned to the port     */
    PortInst           *nextCellPortPtr;/* The next port instance of 
                                        /* the cell                         */
    PortInst           *nextNetPortPtr; /* The next port instance of the net*/
    PortInst           *nextPtr;        /* The next port instance of the
                                         * port master
                                         */
    int                mark;            /* mark if routed port */
    ClientData         clientData;      /* This space for rent */
};


typedef struct _LinkedPort
{
    PortInst           *portInst;       /* A port instance                  */
    struct _LinkedPort *nextPtr;        /* Pointer to another linked
                                         * port instance
                                         */
} LinkedPort;

typedef struct   _LinkedLinkedPort
{
    LinkedPort                *portList; /* The list of ports               */
    struct _LinkedLinkedPort  *nextPtr;  /* Pointer to another linked       */
                                         /* list of linked portLists        */
} LinkedLinkedPort;

extern BearStatus Mac_InsertPortMaster(/* cellMaster, portMaster */);
/*
 * CellMaster *cellMaster;
 * PortMaster *portMaster;
 */

extern BearStatus Mac_DuplicatePort(/* cellInstPtr, origPortInst, instName, newPortInst */);
  
extern BearStatus Mac_DeletePortMaster(/*cellMaster, portMaster*/);
/*
CellMaster *cellMaster;
PortMaster *portMaster;
*/

extern BearStatus Mac_InsertPortInstOfCellInst(/*cellInst, portInst*/);
/*
CellInst  *cellInst;
PortInst  *portInst;
*/

extern BearStatus Mac_DeletePortInstOfCell(/*inst*/);
/*
PortInst   *inst;
*/

extern BearStatus Mac_InsertPortInstOfPortMaster(/*master, inst*/);
/*
PortMaster *master;
PortInst   *inst;
*/

extern BearStatus Mac_DeletePortInstOfPortMaster(/*masterPtr, instPtr*/);
/*
PortMaster  *masterPtr;
PortInst    *instPtr;
*/

extern BearStatus Mac_InsertPortInstOfNet(/*net, portInst*/);
/*
Net      *net;
PortInst *portInst;
*/

extern BearStatus Mac_DeletePortInstOfNet(/*inst*/);
/*
PortInst   *inst;
*/

#endif _MAC_PORT

