
/* macRCell.h --
 *
 *     Wei-ming Dai
 *     June Wang
 *     (Professor E. S. Kuh)
 *
 * It defines the data structures related to RouteCell and all the subroutines
 * to create, insert, and delete.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header: macRCell.h,v 1.2 87/02/07 18:51:12 weiming Exp $
 */

#ifndef _MAC_RCELL
#define _MAC_RCELL

typedef enum _RCellType
{
    HORZ_RCELL,     /* Horizontal straight channel    */
    VERT_RCELL,     /* Vertical straight channel      */
    TOP_LEFT_L_RCELL,   /* L-shaped channel which has the
             * top and left open sides.
             */
    TOP_RIGHT_L_RCELL,  /* L-shaped channel which has the
             * top and right open sides.
             */
    BOTTOM_LEFT_RCELL,  /* L-shaped channel which has the
             * bottom and left open sides.
             */
    BOTTOM_RIGHT_RCELL  /* L-shaped channel which has the
             * bottom and right open sides.
             */
} RCellType;

typedef enum _RCellBound
{
    BOTTOM_LEFT_BOUND,    /* Bottom or left boundary of the route cell */
    TOP_RIGHT_BOUND   /* Top or right boundary of the route cell   */
} RCellBound;

typedef struct _RouteCell
{
    BearId            id;               /* Routing cell ID                   */
    RCellType         type;             /* Routing cell type                 */
    int               mark;             /* Default value for this field
                                         * is BEAR_UNMARKED
                                         */
    LinkedPoint       *blBoundary;      /* Bottom or left boundary           */
    LinkedPoint       *trBoundary;      /* Top or right boundary             */
    Rect              bbox;             /* Routing cell bounding box         */
    LinkedRect        *horzRectList;    /* List of horizontally sliced
                                         * rectangles
                                         */
    LinkedRect        *vertRectList;    /* List of vertically sliced
                                         * rectangles.
                                         */
    Point             origin;           /* Boundary of the routing cell
                                         * and all of its pins are expressed
                                         * relative to this point.
                                         */
    Transform         transform;        /* Transformation suitable for
                                         * placing the routing cell on the 
                                         * horzizontal tile plane.
                                         */
    module            *routeModPtr;     /* Pointer to the corresponding 
                                         * BBL routing module (see macBBL.h)
                                         */
    root              *routeRootList;   /* List of the BBL routing root
                                         * of the route cell. (see macBBL.h)
                                         */
    struct _LinkedGPin *blBoundPinList; /* List of pin inst. on the bottom
                                         * or left boundary of the routing
                                         * cell.
                                         */
    struct _LinkedGPin *trBoundPinList; /* List of pin inst. on the top
                                         * or right boundary of the routing
                                         * cell.
                                         */
    struct _RoutePin  *tlSidePinList;   /* List of routing pins on the 
                                         * left or top open side of the 
                                         * routing cell. 
                                         */
    struct _RoutePin  *rSidePinList;    /* List of routing pins on the 
                                         * right open side of the 
                                         * routing cell. 
                                         */
    CellMaster        *cellMasterPtr;   /* The cell master which contains
                                         * the route cell.
                                         */
    CellGroup         *cellGroupPtr;    /* Pointer to cell group             */
    char              *cifFileNamePtr;
    struct _RouteCell *nextPtr;         /* Pointer to the next routing cell 
                                         * at the same hierarchy 
                                         */
    ClientData        clientData;       /* This space for rent               */
} RouteCell;

typedef struct _AdjPar
{
    int xAdj;
    int yAdj;
    int lExt;
    int rExt;
} AdjPar;

extern BearStatus Mac_CreateRCell(/* cellMasterPtr, rCellPtrPtr */);
/*
 * CellMaster      *cellMasterPtr;
 * RouteCell       **rCellPtrPtr;
 */

extern BearStatus Mac_DeleteRCell(/* rCellPtr */);
/*
 * RouteCell *rCellPtr;
 */

extern BearStatus Mac_InsertBoundPinOfRCell (/* rCellPtr, boundary, gPinPtr */);
/*
 * RouteCell   *rCellPtr;
 * RCellBound  boundary;
 * GenetricPin *gPinPtr;
 */

extern BearStatus Mac_DeleteBoundPinOfRCell (/* rCellPtr, boundary, gPinPtr */);
/*
 * RouteCell   *rCellPtr;
 * RCellBound  boundary;
 * GenetricPin *gPinPtr;
 */

extern BearStatus Mac_InsertSidePinOfRCell (/* rCellPtr, side, rPinPtr */);
/*
 * RouteCell     *rCellPtr;
 * SideDirection side;
 * RoutePin      *rPinPtr;
 */

extern BearStatus Mac_DeleteSidePinOfRCell (/* rCellPtr, side, rPinPtr */);
/*
 * RouteCell     *rCellPtr;
 * SideDirection side;
 * RoutePin      *rPinPtr;
 */

extern BearStatus Mac_InsertRRootOfRCell (/* rCellPtr, rRootPtr */);
/*
 * RouteCell *rCellPtr;
 * root      *rRootPtr;
 */

extern BearStatus Mac_DeleteRRootOfRCell(/* rCellPtr, rRootPtr */);
/*
 * RouteCell *rCellPtr;
 * root      *rRootPtr;
 */

extern BearStatus Rdb_UpdateBndOfRCell(/* adjParPtr, rCellPtr */);
/*
 * AdjPar    *adjParPtr;
 * RouteCell *rCellPtr;
 */

extern void Mac_MoveRouteCell(/* deltaX, deltaY, rcellPtr */);
/*
 * int          deltaX, deltaY;
 * RouteCell    *rcellPtr;
 */

#endif _MAC_RCELL

