/* hash.h --
 *
 * Copyright (C) 1983 Regents of the University of California
 * All rights reserved.
 *
 * This file contains definitions used by the hash module,
 * which maintains hash tables.
 */

/* sccsid "@(#)hash.h   4.2 MAGIC (Berkeley) 10/12/85" */

#ifndef _HASH
#define _HASH

/*---------------------------------------------------------
 *  This first stuff should only be used internally
 *  in the hash routines.
 *---------------------------------------------------------
 */

/* The following defines one entry in the hash table. */

typedef struct h1
{
    char *h_pointer;        /* Pointer to anything. */
    struct h1 *h_next;      /* Next element, zero for end. */
    union
    {
    char *h_ptr;        /* One-word key value to identify entry. */
    unsigned h_words[1];    /* N-word key value.  Note: the actual
                 * size may be longer if necessary to hold
                 * the entire key.
                 */
    char h_name[4];     /* Text name of this entry.  Note: the
                 * actual size may be longer if necessary
                 * to hold the whole string. This MUST be
                 * the last entry in the structure!!!
                 */
    } h_key;
} HashEntry;

/* A hash table consists of an array of pointers to hash
 * lists:
 */

typedef struct h3
{
    HashEntry **ht_table;   /* Pointer to array of pointers. */
    int ht_size;        /* Actual size of array. */
    int ht_nEntries;        /* Number of entries in the table. */
    int ht_downShift;       /* Shift count, used in hashing function. */
    int ht_mask;        /* Used to select bits for hashing. */
    int ht_ptrKeys;     /* 1 means that keys (h_names) are 1-word
                 * values (char *'s).  0 means keys are
                 * strings.  >1 means keys are ht_ptrKeys-word
                 * values.
                 */
} HashTable;

/* The following structure is used by the searching routines
 * to record where we are in the search.
 */

typedef struct h2
{
    int hs_nextIndex;       /* Next bucket to check (after current). */
    HashEntry * hs_h;       /* Next entry to check in current bucket. */
} HashSearch;

/*---------------------------------------------------------
 *  The following procedure declarations and macros
 *  are the only things that should be needed outside
 *  the implementation code.
 *---------------------------------------------------------
 */

extern void HashInit(), HashStats(), HashKill();
extern HashEntry *HashFind();
extern HashEntry *HashLookOnly();
extern void HashStartSearch();
extern HashEntry *HashNext();

/* char * HashGetValue(h); HashEntry *h; */

#define HashGetValue(h) ((h)->h_pointer)

/* HashSetValue(h, val); HashEntry *h; char *val; */

#define HashSetValue(h, val) ((h)->h_pointer = (char *) (val))

/* HashSize(n) returns the number of words in an object of n bytes */
#define HashSize(n) (((n) + sizeof (unsigned) - 1) / sizeof (unsigned))

#endif  _HASH

