
/* bguCifMain.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "bearGrUtils.h"

#include "../ezPlot/ezPlotInt.h"
#include "bguCifPlotInt.h"


/* ---------- Internal global variables for the bguCifPlot module ---------- */
Rect            curCifBnd;
Window          cifWin;
double          (*cifScrMatrix)[4];
unsigned int    curCifLayer;
int             cifOptions;
short           npoints;
char            *progNamePtr;

Pixel
    curCifColor,
    BND0Color,
    BND1Color,
    BND2Color,
    NCColor,
    NPColor,
    NMColor,
    TRMColor;


/* ---------- Local Procedures ---------- */
BearStatus  GetCifLine();
BearStatus  ProcessPolygon();
BearStatus  ProcessBox();
void        DrawCifPolygon();
void        CalcCifPlotBnd();
/*
void        PrintBnd();
*/


/* ---------- Local Global Variables ---------- */
int     wxCoord[MAXPOINTS], wyCoord[MAXPOINTS];
XPoint  vertex[MAXPOINTS];


static BearStatus
GetCifLine(filePtr, cifLine, lineLenPtr)
FILE *filePtr;
char *cifLine;
int  *lineLenPtr;
/*
 * Reads in up to (BGU_CIF_LINE_SIZE - 1) characters. The last 
 * character read into 'cifLine' is followed by a '\0'.  The
 * number of characters read is stored in 'lineLenPtr', where
 * 0 indicates end of file.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 */
{
    register int n = 0;
    register int c;        /* c is an int because getc() returns an int */

    /* skip the white space before the command */
    while (((c = getc(filePtr)) != EOF) &&
        ((c == ' ') || (c == '\t') || (c == '\n'))) {
        ;
    }
    cifLine[n++] = c;
    while (((c = getc(filePtr)) != EOF) && (c != ';') &&
        (n < (BGU_CIF_LINE_SIZE - 1))) {

        cifLine[n++] = c;
    }
    if ((n < (BGU_CIF_LINE_SIZE - 1)) == FALSE) {
        sprintf(bear_ErrorMsg,
            "cif file line length of %d characters exceeded\n",
            (BGU_CIF_LINE_SIZE - 1));
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR);
    }
    if (c == ';') {
        cifLine[n] = '\0';
        *lineLenPtr = n;
    }
    if (c == EOF) {
        *lineLenPtr = 0;
    }
    return (BEAR_OK);
}



BearStatus
Bgu_DrawCifPlot(drawCifPtr) 
DrawCif *drawCifPtr;
{
    WindowData  cifWinData;         /* IMPORTANT: Note that a local window
                                     * data structure must be maintained
                                     * because the cif file will probably
                                     * have a unique world coordinate system.
                                     */
    FILE        *cifFilePtr;
    int         lineLen;
    char        cifLine[BGU_CIF_LINE_SIZE];

    if (BguSetUpCifColors() != BEAR_OK) {
        return (BEAR_ERROR);
    }

    if (strcmp (drawCifPtr->fileNamePtr, "stdin") == 0) {
        cifFilePtr = stdin;
    } 
    else if ((cifFilePtr = fopen (drawCifPtr->fileNamePtr, "r")) ==
        NIL(FILE)) {
        (void) sprintf (bear_ErrorMsg, "%s: no such file or directory\n",
            drawCifPtr->fileNamePtr);
        Ez_Print(EZ_ITALIC_FONT, bear_ErrorMsg);
        return (BEAR_ERROR);
    }
    cifOptions = drawCifPtr->options;
    cifWin = drawCifPtr->win;
    cifScrMatrix = drawCifPtr->scrMatrix;
/*
 *  cifWin = cifWinData.wd_userWin = drawCifPtr->win;
 *  if (XGetWindowAttributes(ez_Display, cifWinData.wd_userWin,
 *      &cifWinData.wd_userWinInfo) == 0) {
 *      Bear_PrintErr("can not get window information");
 *      return (BEAR_ERROR);
 *  }
 */

    /*
     * Find the first P command so the boundaries of 
     * the window can be defined.
     *
     * note: no B (box) or 94 (label) commands are allowed yet.
     */
    while (TRUE) {
        if (GetCifLine(cifFilePtr, cifLine, &lineLen) == BEAR_ERROR) {
            fclose (cifFilePtr);
            return (BEAR_ERROR);
        }
        if (lineLen == 0) {
            /* end of file reached */
            break;
        }
/*
printf ("%s", cifLine); fflush (stdout);
*/

        switch (cifLine[0]) {
        case '(': /* start of a comment (ignor it) */
            break;
        case 'B': /* box command (fall through) */
        case '9': /* label command */
            Bear_PrintErr("boundary in cif file must be defined first\n");
            fclose (cifFilePtr);
            return (BEAR_ERROR);
            break;
        case 'L': /* layer command */
            if (BguProcessLayer(cifLine) == BEAR_ERROR) {
                fclose (cifFilePtr);
                return (BEAR_ERROR);
            }
            break;
        case 'P': /* polygon command */
            if (cifOptions & BGU_CALC_VIEW_TRANS) {
                if (ProcessPolygon(cifLine) == BEAR_ERROR) {
                    fclose (cifFilePtr);
                    return (BEAR_ERROR);
                }
                CalcCifPlotBnd();
/*
Mac_PrintRect(&curCifBnd);

                Ez_StretchRect(&curCifBnd, 1.10);

                Mac_CopyRect(&cifWinData.wd_imageBnd, &curCifBnd);
                Ez_CalcViewingTrans(&cifWinData);

                Mac_CopyRect(&curCifBnd, &cifWinData.wd_imageBnd);
                cifScrMatrix = cifWinData.wd_scrMatrix;
*/
            }
/*  PrintBnd(); */
            goto next;
            break;
        default:
            sprintf(bear_ErrorMsg, "%s\nis an unknown cif command\n", cifLine);
            Bear_PrintErr(bear_ErrorMsg);
            fclose (cifFilePtr);
            return (BEAR_ERROR);
            break;
        }
    }
next:
    /* Okay now process the rest of the file */
    while (TRUE) {
        if (GetCifLine(cifFilePtr, cifLine, &lineLen) == BEAR_ERROR) {
            fclose (cifFilePtr);
            return (BEAR_ERROR);
        }
        if (lineLen == 0) {
            /* end of file reached */
            break;
        }

        switch (cifLine[0]) {
        case '(': /* start of a comment (ignor it) */
            break;
        case 'B': /* box command */
            if (ProcessBox(cifLine) == BEAR_ERROR) {
                fclose (cifFilePtr);
                return (BEAR_ERROR);
            }
            break;
        case '9': /* label command (ignor it) */
            break;
        case 'L': /* layer command */
            if (BguProcessLayer(cifLine) == BEAR_ERROR) {
                fclose (cifFilePtr);
                return (BEAR_ERROR);
            }
            break;
        case 'P': /* polygon command */
            if (ProcessPolygon(cifLine) == BEAR_ERROR) {
                fclose (cifFilePtr);
                return (BEAR_ERROR);
            }
            DrawCifPolygon();
            break;
        default:
            sprintf(bear_ErrorMsg, "%s\nis an unknown cif command\n", cifLine);
            Bear_PrintErr(bear_ErrorMsg);
            fclose (cifFilePtr);
            return (BEAR_ERROR);
            break;
        }
    }
    Ez_Flush();
    fclose (cifFilePtr);
    return (BEAR_OK);
}



/*
static void
PrintBnd()
{
    Ez_Print(EZ_NORMAL_FONT, "The current views boundary is:\n");
    sprintf(bear_Msg, "w = %d  e = %d  s = %d  n = %d\n\n",
        curCifBnd.r_xbot, curCifBnd.r_xtop,
        curCifBnd.r_ybot, curCifBnd.r_ytop);
    Ez_Print(EZ_NORMAL_FONT, bear_Msg);
}
*/


static BearStatus
ProcessBox(cifLine)
char *cifLine;
{
    static  int numOfBoxes = 0;
    Rect    box, clippedBox;
    float   width, height, xCenter, yCenter;

    /*
     * B width height xcenter ycenter; (defines a box that is
     * to be filled in).
     */
    if (sscanf(cifLine, "B %f %f %f %f", &width, &height,
        &xCenter, &yCenter) != 4) {
        sprintf(bear_ErrorMsg, "%s is an invalid box command\n", cifLine);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR);
    }
    width /= 2.0;
    height /= 2.0;
    box.r_xbot = xCenter - width;
    box.r_xtop = xCenter + width;
    box.r_ybot = yCenter - height;
    box.r_ytop = yCenter + height;
    if ((cifOptions &  BGU_ENTIRE_CHIP) == FALSE) {
        /* clip box to window */
        if (Ez_FindOverlap(&box, &curCifBnd, &clippedBox) == TRUE) {
            Ez_FillRect(cifWin, cifScrMatrix, &clippedBox, curCifColor);
        }
        /*
        Ez_FillRect(cifWin, cifScrMatrix, &box, curCifColor);
        */
    } 
    else {
        Ez_FillRect(cifWin, cifScrMatrix, &box, curCifColor);
    }
    if (numOfBoxes++ == 25) {
        numOfBoxes = 0;
        Ez_Flush();
    }
    return (BEAR_OK);
}



static BearStatus
ProcessPolygon(cifLine)
char *cifLine;
{
    int          len, x, y;
    char         string[MAXDIGITS];
    register int i, j, k;

    cifLine++; /* skip over the P */
    j = k = x = y = 0;
    len = strlen(cifLine);
    for (i = 0; i < len ;i++) 
        /* skip white space */
        if ((cifLine[i] != ' ') && (cifLine[i] != '\t') &&
            (cifLine[i] != '\n')) {
            if (j < (MAXDIGITS - 1)) 
                string[j++] = cifLine[i];
            else {
                sprintf(bear_ErrorMsg,
                    "a number in the cif file has more then %d digits",
                    (MAXDIGITS - 1));
                Bear_PrintErr(bear_ErrorMsg);
                return (BEAR_ERROR);
            }
        }
        else if (j > 0) {
            /*
             * Each point will have two coordinates, and in addition there
             * must be room leftover to add the first point onto the end of
             * the list.
             */
            if (k < (MAXPOINTS * 2 - 2)) {
                string[j] = '\0'; 
                if (BEAR_EVEN(k++)) {
                    wxCoord[x++] = atoi(string);
                } else {
                    wyCoord[y++] = atoi(string);
                }
                j = 0;
            }
            else {
                sprintf(bear_ErrorMsg,
                    "a polygon in the cif file has more then %d vertices",
                    MAXPOINTS);
                Bear_PrintErr(bear_ErrorMsg);
                return (BEAR_ERROR);
            }
        }
    if (j != 0) {
        if (k < (MAXPOINTS * 2 - 2)) {
            string[j] = '\0'; 
            /*
             * Since the polygon description alternates x-y x-y etc.
             * a y coordinate will always be the last one.
             */
            wyCoord[y++] = atoi(string);
        }
        else {
            sprintf(bear_ErrorMsg,
                "a polygon in the cif file has more then %d vertices",
                MAXPOINTS);
            Bear_PrintErr(bear_ErrorMsg);
            return (BEAR_ERROR);
        }
    }
    npoints = x;

    return (BEAR_OK);
}



BearStatus
Bgu_GetCifPlotBnd(cifFileNamePtr, cifBndPtr) 
char            *cifFileNamePtr;
register Rect   *cifBndPtr;
/*
 * Open the named cif file and determine the boundary of the cif plot 
 * described there in.
 *
 * Side effects:
 *    None
 */
{
    FILE    *cifFilePtr;
    int     lineLen;
    char    cifLine[BGU_CIF_LINE_SIZE];
    bool    haveBnd;    /* have boundary */

    if ((cifFilePtr = fopen (cifFileNamePtr, "r")) == NIL(FILE)) {
        sprintf(bear_ErrorMsg, "%s: no such file or directory\n",
            cifFileNamePtr);
        Ez_Print(EZ_ITALIC_FONT, bear_ErrorMsg);
        return (BEAR_ERROR);
    }
                               
    /*
     * Find the first P command which is the boundary of the cif file.
     */
    haveBnd = FALSE;
    while (haveBnd == FALSE) {
        if (GetCifLine(cifFilePtr, cifLine, &lineLen) == BEAR_ERROR) {
            fclose (cifFilePtr);
            return (BEAR_ERROR);
        }
        if (lineLen == 0) {
            /* premature end of file reached */
            return (BEAR_ERROR);
        }

        switch (cifLine[0]) {
        case '(': /* start of a comment (ignor it) */
            break;
        case '9': /* label command */
        case 'B': /* box command (fall through) */
            Bear_PrintErr("boundary in cif file must be defined first\n");
            fclose (cifFilePtr);
            return (BEAR_ERROR);
            break;
        case 'L': /* layer command (ignor it) */
            break;
        case 'P': /* polygon command */
            if (ProcessPolygon(cifLine) == BEAR_ERROR) {
                fclose (cifFilePtr);
                return (BEAR_ERROR);
            }
            CalcCifPlotBnd();
            haveBnd = TRUE;
            break;
        default:
            sprintf(bear_ErrorMsg, "%s\nis an unknown cif command\n", cifLine);
            Bear_PrintErr(bear_ErrorMsg);
            fclose (cifFilePtr);
            return (BEAR_ERROR);
            break;
        }
    }
    Mac_CopyRect(cifBndPtr, &curCifBnd);
    fclose (cifFilePtr);

    return (BEAR_OK);
}



static void
CalcCifPlotBnd()
/*
 * Calculate the cif plot's boundary.
 *
 * Results:
 *    None
 *
 * Side effects:
 *    The global variable 'curCifBnd' is initialized.
 */
{
    float           x, y;
    register int    i;

    curCifBnd.r_xbot = curCifBnd.r_xtop = wxCoord[0];
    curCifBnd.r_ybot = curCifBnd.r_ytop = wyCoord[0];
    for (i = 1; i < npoints ;i++) {
        x = wxCoord[i];
        y = wyCoord[i];
        if (x < curCifBnd.r_xbot) {
            curCifBnd.r_xbot = x;
        } else if (x > curCifBnd.r_xtop) {
            curCifBnd.r_xtop = x;
        }
    
        if (y < curCifBnd.r_ybot) {
            curCifBnd.r_ybot = y;
        } else if (y > curCifBnd.r_ytop) {
            curCifBnd.r_ytop = y;
        }
    }
}



static void
DrawCifPolygon() 
{
/*
 * Points are in counter-clockwise order and the sides
 * of the polygon are either horizotal or vertical.
 */

    int             vertexCount;
    register int    i;

    for (i = 0; i < npoints ;i++) {
        vertex[i].x = Ez_XTrans(cifScrMatrix, wxCoord[i], wyCoord[i]);
        vertex[i].y = Ez_YTrans(cifScrMatrix, wxCoord[i], wyCoord[i]);
    }
    /* connect the last point to the first */
    vertex[npoints].x = vertex[0].x;
    vertex[npoints].y = vertex[0].y;
    vertexCount = npoints + 1;
    
    XSetForeground(ez_Display, ezFillGC, curCifColor);
    XFillPolygon(ez_Display, (Drawable) cifWin, ezFillGC, 
        vertex, vertexCount, Convex, CoordModeOrigin);
}


/*
ProcessLabel(line)
char *line;
{
    extern Font letter[];
    extern unsigned int curCifLayer;
    static short textcolor = TRM;
    float x, y;
    short nletters;
    char string[80];

    if (curCifLayer == BND2)
        zcsiz(&letter[1].width, &letter[1].height);
    zcolr(&textcolor);
    if (sscanf (line, "94 %s %f %f", string, &x, &y) != 3) {
        sprintf (bear_Msg, "%s is an invalid label command\n", line);
        Bear_PrintErr(bear_Msg);
        exit (1);
    }
    nletters = strlen(string);
    zmove(&x, &y);
    ztext(&nletters, string);
    if (curCifLayer == BND2)
        zcsiz(&letter[0].width, &letter[0].height); */ /* switch back */
/*
}
*/


