
#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "deque.h"


Deque *
MakeDeque()
{
    Deque *deque;

    deque = ALLOC(Deque, 1);
    deque->tail = NIL(Element);

    return (deque);
}


void
DeleteDeque(deque)
Deque *deque;
{
    FREE(deque);
}


int
IsEmptyDeque(deque)
Deque *deque;
{
#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("IsEmptyDeque() failed: no deque is given.");
    }
#endif DEBUG   

    return ((deque->tail == NIL(Element)) ? TRUE : FALSE);
}


void
PushDeque(deque, clientData)
Deque       *deque;
ClientData  clientData;
{
    Element *element;

#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("PushDeque() failed: no deque is given.");
    }
    else if (clientData == (ClientData) NULL) {
        Bear_PrintFatalErr("PushDeque() failed: no clientData is given.");
    }
#endif DEBUG   

    element = ALLOC(Element, 1);

    element->clientData = clientData;
    element->prevPtr    = NIL(Element);
    element->nextPtr    = NIL(Element);

    if (deque->tail == NIL(Element)) {
        deque->tail          = element;
        deque->tail->prevPtr = element;
        deque->tail->nextPtr = element;
    }
    else {
        deque->tail->nextPtr->prevPtr = element;
        element->prevPtr              = deque->tail;
        element->nextPtr              = deque->tail->nextPtr;
        deque->tail->nextPtr          = element;
    }
}


ClientData
PopDeque(deque)
Deque *deque;
{
    ClientData  clientData;
    Element     *headPtr;

#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("PopDeque() failed: no deque is given.");
    }
    else if (deque->tail == NIL(Element)) {
        Bear_PrintFatalErr("PopDeque() failed: deque is empty.");
    }
#endif DEBUG   
     
    if (deque->tail == deque->tail->nextPtr) {
        headPtr = deque->tail;
        deque->tail = NIL(Element);
    }
    else {
        headPtr = deque->tail->nextPtr;
        headPtr->nextPtr->prevPtr = deque->tail;
        deque->tail->nextPtr = headPtr->nextPtr;
    }

    clientData = headPtr->clientData;
    FREE(headPtr);

    return (clientData);
}


void
InjectDeque(deque, clientData)
Deque       *deque;
ClientData  clientData;
{
#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("InjectDeque() failed: no deque is given.");
    }
    else if (clientData == (ClientData) NULL) {
        Bear_PrintFatalErr("InjectDeque() failed: no clientData is given.");
    }
#endif DEBUG   

    PushDeque(deque, clientData);
    deque->tail = deque->tail->nextPtr;
}


ClientData
EjectDeque(deque)
Deque *deque;
{

#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("EjectDeque() failed: no deque is given.");
    }
    else if (deque->tail == NIL(Element)) {
        Bear_PrintFatalErr("EjectDeque() failed: deque is empty.");
    }
#endif DEBUG   

    deque->tail = deque->tail->prevPtr;

    return (PopDeque(deque));
}


ClientData
LookUpHead(deque)
Deque *deque;
{
#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("LookUpHead() failed: no deque is given.");
    }
    else if (deque->tail == NIL(Element)) {
        Bear_PrintFatalErr("LookUpHead() failed: deque is empty.");
    }
#endif DEBUG   

    return (deque->tail->nextPtr->clientData);
}



ClientData
LookUpTail(deque)
Deque *deque;
{
#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("LookUpTail() failed: no deque is given.");
    }
    else if (deque->tail == NIL(Element)) {
        Bear_PrintFatalErr("LookUpTail() failed: deque is empty.");
    }
#endif DEBUG   

    return (deque->tail->clientData);
}


BearStatus
ItrDeque(deque, func, clientData)
Deque       *deque;
BearStatus  (*func)();
ClientData  clientData;
{
    Element *elementPtr;

#ifdef DEBUG
    if (deque == NIL(Deque)) {
        Bear_PrintFatalErr("ItrDeque() failed: no deque is given");
    }
    else if (func == (BearStatus (*)()) NULL) {
        Bear_PrintFatalErr("ItrDeque() failed: no function is given");
    }
#endif DEBUG

    if (deque->tail == NIL(Element)) {
        return (BEAR_COMPLETE_SEARCH);
    }
    else {
        elementPtr = deque->tail;
        do {
            elementPtr = elementPtr->nextPtr;
            if (((*func)(elementPtr->clientData, clientData)) ==
                BEAR_STOP_SEARCH) {
                return (BEAR_STOP_SEARCH);
            }
        } while (elementPtr != deque->tail);
    }

    return (BEAR_COMPLETE_SEARCH);
}


