
/* ezDragBnd.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: ezDragBnd.c,v 1.1 87/09/24 10:25:49 winjamar Exp $";
#endif  not lint

#ifdef USE_OLD_X_CODE

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "ezPlot.h"
#include "ezPlotInt.h"

/*
 * ----------------------------------------------------------------------------
 *
 * Ez_DragBoundary() --
 *
 *  Allows you to interactively drag the boundary of some object
 *  around.
 *
 * Results:
 *  TRUE is returned and 'deltaPtr' filled in if everything goes
 *  down okay, FALSE otherwise.
 *
 * Side effects:
 *  None.
 * ----------------------------------------------------------------------------
 */

ezStatus
Ez_DragBoundary(winDataPtr, mousePtr, vertex, vertexCount, deltaPtr)
WindowData  *winDataPtr;
Point       *mousePtr;          /* Screen coordinates of where
                                 * the mouse should be when we
                                 * start dragging the boundary.
                                 */
Vertex      *vertex;            /* Screen coordinates of the boundary.*/
int         vertexCount;
Point       *deltaPtr;          /* The distance the boundary was
                                 * moved in world coordinates is
                                 * returned.
                                 */
{
    Point           wPoint1, wPoint2;
    Window          userWin;
    XEvent          report;
    XButtonEvent    *bev = (XButtonEvent *) &report.xbutton;
    double          (*invScrMatrix)[4];
    unsigned int    eventMask = ButtonPressMask | ButtonReleaseMask |
            LeaveWindowMask | Button1MotionMask;
    int             xMouse1, yMouse1;
    int             xMouse2, yMouse2;
    int         deltaX, deltaY;
    register XPointerMovedEvent *pev = (XPointerMovedEvent *) &report;
    register int    i;

    userWin = winDataPtr->wd_userWin;
    if (XGrabPointer(ez_Display, userWin, FALSE, eventMask, GrabModeAsync,
        GrabModeAsync, None, ezArrowCrossCur, CurrentTime) != GrabSuccess) {
        EzPrintErr("could not grab button");
        return EZ_ERROR;
    }
    invScrMatrix = winDataPtr->wd_invScrMatrix;
    xMouse1 = mousePtr->p_x;
    yMouse1 = mousePtr->p_y;
    wPoint1.p_x = Ez_XTrans(invScrMatrix, xMouse1, yMouse1);
    wPoint1.p_y = Ez_YTrans(invScrMatrix, xMouse1, yMouse1);
    XGrabServer(ez_Display);
    EZ_DRAW_VERTICES(userWin, vertex, vertexCount); /* draw box */
    while (TRUE) {
        XNextEvent(ez_Display, &report);
        if (report.xany.window != userWin) {
        XUngrabServer(ez_Display);
            Ez_XEventHandler(&report);
        XGrabServer(ez_Display);
            continue;
        }
        switch (report.type) {
        case ButtonPress:
                /*
             * Pressing a second button before releasing the
             * first cancels the command.
             */
        EZ_DRAW_VERTICES(userWin,
                 vertex, vertexCount); /* erase box */
        goto failure;
        break;
        case ButtonRelease:
        /* Thats a wrap ! */
        EZ_DRAW_VERTICES(userWin,
                 vertex, vertexCount); /* erase box */
        XUngrabServer(ez_Display);
        XUngrabPointer(ez_Display, CurrentTime);
        wPoint2.p_x = Ez_XTrans(invScrMatrix, bev->x, bev->y);
        wPoint2.p_y = Ez_YTrans(invScrMatrix, bev->x, bev->y);
        deltaPtr->p_x = wPoint2.p_x - wPoint1.p_x;
        deltaPtr->p_y = wPoint2.p_y - wPoint1.p_y;
        
        return EZ_OK;
        break;
        case LeaveNotify:
            /*
             * Leaving the window before releasing the left
             * button first cancels the command.
             */
        EZ_DRAW_VERTICES(userWin, vertex, vertexCount); /* erase box */
        XUngrabServer(ez_Display);
        Ez_RingBell();
        goto failure;
            break;
        case MotionNotify:
            /* erase box, reset, draw box */
            EZ_DRAW_VERTICES(userWin, vertex, vertexCount); 
            xMouse2 = pev->x;
            yMouse2 = pev->y;
            deltaX = xMouse2 - xMouse1;
            deltaY = yMouse2 - yMouse1;
            for (i = 0; i < vertexCount ;i++) {
                vertex[i].x += deltaX;
                vertex[i].y += deltaY;
            }
            EZ_DRAW_VERTICES(userWin, vertex, vertexCount);
            xMouse1 = xMouse2;
            yMouse1 = yMouse2;
            break;
        default:
        XUngrabServer(ez_Display);
            Ez_XEventHandler(&report);
        XGrabServer(ez_Display);
            break;
        }
    }

failure:
    XUngrabPointer(ez_Display, CurrentTime);
    return EZ_ERROR;
}

#endif /* USE_OLD_X_CODE */


