
/* ezDraw.c --
 *
 *     George Carvalho, Esq.
 *     Ben Chen
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: ezDraw.c,v 1.1 87/09/24 10:25:51 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "ezPlot.h"
#include "ezPlotInt.h"
#include "ezPsPlot.h"

/* ---------- Local Procedures ---------- */
static GC   SetUpTmpGC();
static void GetDimOfRect();
static int  DrawArrow();


static void
GetDimOfRect(scrMatrix, rectPtr, x, y, width, height)
register Matrix scrMatrix;
register Rect   *rectPtr;
int             *x, *y;
unsigned int    *width, *height;
{
    *x = Ez_XTrans(scrMatrix, rectPtr->r_xbot, rectPtr->r_ytop);
    *y = Ez_YTrans(scrMatrix, rectPtr->r_xbot, rectPtr->r_ytop);
    *width = ABS(Ez_XTrans(scrMatrix, rectPtr->r_xtop, 
        rectPtr->r_ybot) - *x);
    *height = ABS(Ez_YTrans(scrMatrix, rectPtr->r_xtop,
        rectPtr->r_ybot) - *y);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Ez_FillScrRect() --
 *
 * Fill a rect with fillColor in SCREEN coordinates
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_FillScrRect(win, rectPtr, fillColor)
Window          win;
register Rect   *rectPtr;
Pixel           fillColor;
{
    unsigned int    height, width;
    int             x, y;
    
    x = rectPtr->r_xbot;
    y = rectPtr->r_ytop;
    width = rectPtr->r_xtop-x;
    height = rectPtr->r_ybot-y;
    if (XSetForeground(ez_Display, ezFillGC, fillColor) == BadGC) {
        EzPrintWarn("Ez_FillScrRect() : Bad GC");
    }
    if ((width != 0) && (height != 0)) {
        XFillRectangle(ez_Display,(Drawable) win, ezFillGC, 
            x, y, width, height);

        if (ezPsPlotFlag) {
            PsDrawRectangle(ezPsFile, -x, y, -width, height, 1, 
                EzXPsGray(fillColor), EzXPsGray(fillColor));
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Ez_FillRect() --
 *
 * Transform the rectangle in 'rectPtr' from world coordinates to screen
 * coordinates and fill it in with 'fillColor'.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_FillRect(win, scrMatrix, rectPtr, fillColor)
Window          win;
register Matrix scrMatrix;
register Rect   *rectPtr;
Pixel           fillColor;
{
    unsigned int    height, width;
    int             x, y;
    
    GetDimOfRect(scrMatrix, rectPtr, &x, &y,  &width, &height);
    if (XSetForeground(ez_Display, ezFillGC, fillColor) == BadGC) {
        EzPrintWarn("Ez_FillRect() : Bad GC");
    }
    if ((width != 0) && (height != 0)) {
        XFillRectangle(ez_Display, (Drawable) win, ezFillGC, 
            x, y, width, height);

        if (ezPsPlotFlag) {
            PsDrawRectangle(ezPsFile, -x, y, -width, height, 1,
                EzXPsGray(fillColor), EzXPsGray(fillColor));
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Ez_OutlineRect() --
 *
 * Transform the rectangle in 'rectPtr' from world coordinates to screen
 * coordinates and draw its outline on the screen, using both the type of
 * line (see ezPlot.h) and the color of your choice.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_OutlineRect(win, scrMatrix, rectPtr, type, lineColor, backColor)
Window          win;
register Matrix scrMatrix;
register Rect   *rectPtr;
int             type;
Pixel           lineColor, backColor;
{
    unsigned int    height, width;
    int             lineType;
    int             x, y;
    
    lineType = type & EZ_LINETYPE_MASK;
    if (lineType) {
        GetDimOfRect(scrMatrix, rectPtr, &x, &y,  &width, &height);

        XDrawRectangle(ez_Display, win, 
            SetUpTmpGC(win, lineType, lineColor, backColor, GXcopy),
            x, y, width, height);

        if (ezPsPlotFlag) {
            PsDrawRectangle(ezPsFile, -x, y, -width, height, 1, 
                EzXPsGray(lineColor), PS_NOFILL);
        }
    }
}

    

/*
 * ----------------------------------------------------------------------------
 *
 * Ez_FillPolygon() --
 *
 * Transform the polygon from world coordinates to screen
 * coordinates and fill it in with 'fillColor'.
 *
 * IMPORTANT - The path that describes the polygon may not self-intesect
 *     and it may not be concave.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void 
Ez_FillPolygon(win, scrMatrix, points, numOfPoints, fillColor)
Window          win;
register Matrix scrMatrix;
register Point  *points;
int             numOfPoints;
Pixel           fillColor;
{
    static XPoint   fourScrPoints[4];
    register XPoint *scrPoints;
    register int    i;
    
    if (numOfPoints == 4) {
        /*
         * This is intended to speed up the filling of arbitrarily
         * oriented rectangles.
         */
        scrPoints = fourScrPoints;
    }
    else {
        scrPoints = ALLOC(XPoint, numOfPoints);
    }

    for (i = 0; i < numOfPoints; i++) {
        scrPoints[i].x = Ez_XTrans(scrMatrix, points[i].p_x, points[i].p_y);
        scrPoints[i].y = Ez_YTrans(scrMatrix, points[i].p_x, points[i].p_y);
    }

    if (XSetForeground(ez_Display, ezFillGC, fillColor) == BadGC) {
        EzPrintWarn("Ez_FillPolygon() : Bad GC");
    }
    XFillPolygon(ez_Display, (Drawable) win, ezFillGC,
        scrPoints, numOfPoints, Convex, CoordModeOrigin);

    if (ezPsPlotFlag) {
        for (i = 0; i < numOfPoints; i++) {
            scrPoints[i].x *= -1;
        }
        PsDrawPolygon(ezPsFile, scrPoints, numOfPoints, 1,
            EzXPsGray(fillColor), EzXPsGray(fillColor));
    }

    if (numOfPoints != 4) {
        FREE(scrPoints);
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Ez_TileRect() --
 *
 * Transform the rectangle in 'rectPtr' from world coordinates to screen
 * coordinates and potassium sorbate fill it in with the tile pattern
 * ('tile') using the given function ('function').
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_TileRect(win, scrMatrix, rectPtr, tile, function)
Window          win;
register Matrix scrMatrix;
register Rect   *rectPtr;
Pixmap          tile;
int             function;
{
    
    Ez_FillRect(win, scrMatrix, rectPtr, (unsigned long) tile);
/* Under development
    XGCValues values;
    static GC gc = (GC) 0;
    unsigned int height, width;
    int x, y;

    values.tile = tile;
    values.function = function;
    if (gc == (GC) 0) {
        values.fill_style = FillTiled;
        gc = (GC) XCreateGC(ez_Display, (Drawable) win,
            GCFillStyle | GCTile | GCFunction, &values);
    }
    else {
        XChangeGC(ez_Display, gc, GCTile | GCFunction, &values);
    }
    GetDimOfRect(scrMatrix, rectPtr, &x, &y,  &width, &height);
    if (width != 0 && height != 0) {
        XFillRectangle(ez_Display, (Drawable) win, gc, x, y, width, height);
    }
*/
}



#ifdef STIPPLE
void
Ez_StippleRect(win, scrMatrix, rectPtr, color, stipple, function)
Window          win;
register Matrix scrMatrix;
register Rect   *rectPtr;
Pixmap          stipple;
Pixel           color;
int             function;
{
    static GC       gc = (GC) 0;
    XGCValues       values;
    unsigned int    height, width;
    int             x, y;

    values.function = function;
    values.foreground = color;
    values.stipple = stipple;

    if (gc == (GC) 0) {
        values.fill_style = FillStippled;
        gc = (GC) XCreateGC(ez_Display, (Drawable) win,
            GCFillStyle | GCFunction | GCForeground | GCStipple,
            &values);
    }
    else {
        XChangeGC(ez_Display, gc, 
            GCFunction | GCForeground | GCStipple,
            &values);
    }

    GetDimOfRect(scrMatrix, rectPtr, &x, &y,  &width, &height);
    if (width != 0 && height != 0) {
        XFillRectangle(ez_Display, (Drawable) win, gc, x, y, width, height);

        if (ezPsPlotFlag) {
            EzPrintWarn("Ez_StippleRect() : postscript not implemented");
/*
 *          PsDrawRectangle(ezPsFile, -x, y, -width, height, 1, PS_BLACK,
 *              EzXPsGray(color));
 */
        }
    }
}
#endif STIPPLE



static GC
SetUpTmpGC(win, lineType, lineColor, backColor, function)
Window  win;
int     lineType;
Pixel   lineColor, backColor; 
int     function;
/*
 *
 */
{
    static char dashed[4] = { 3, 3, 3, 3 }; 
    static char dotted[4] = { 1, 1, 1, 1 }; 
    static GC   gc = (GC) 0;

    XGCValues   values;
    int         whichStyle; /* 0 for solid, 1 for dashed, 
                             * 2 for dotted. 
                             */

    values.function = function;
    values.foreground = lineColor;
    values.background = backColor;

    switch (lineType) {
    case EZ_SOLIDLINE1:
        values.line_style = LineSolid;
        values.line_width = 1;
        whichStyle = 0;
        break;
    case EZ_SOLIDLINE2:
        values.line_style = LineSolid;
        values.line_width = 2;
        whichStyle = 0;
        break;
    case EZ_DASHEDLINE1:
        values.line_style = LineOnOffDash;
        values.line_width = 1;
        whichStyle = 1;
        break;
    case EZ_DASHEDLINE2:
        values.line_style = LineOnOffDash;
        values.line_width = 2;
        whichStyle = 1;
        break;
    case EZ_DOTTEDLINE1:
        values.line_style = LineOnOffDash;
        values.line_width = 1;
        whichStyle = 2;
        break;
    case EZ_DOTTEDLINE2:
        values.line_style = LineOnOffDash;
        values.line_width = 2;
        whichStyle = 2;
        break;
    default:
        values.line_style = LineSolid;
        values.line_width = 1;
        whichStyle = 0;
        break;  
    }

    if (gc == (GC) 0) {
        gc = (GC) XCreateGC(ez_Display, (Drawable) win,
            GCFunction | GCForeground | GCBackground |
            GCLineStyle | GCLineWidth, 
            &values);
    }
    else {
        XChangeGC(ez_Display, gc, 
            GCFunction | GCForeground | GCBackground | 
            GCLineStyle | GCLineWidth, 
            &values);
    }

    if (whichStyle == 1)  {
        XSetDashes(ez_Display, gc, (int) 0, dashed, (int) 4);
    }
    else if (whichStyle == 2)  {
        XSetDashes(ez_Display, gc, (int) 0, dotted, (int) 4);
    }

    return gc;
}


/* ARGSUSED */
static int
DrawArrow(win, from_xs, from_ys, to_xs, to_ys, type, arrowColor, function)
Window          win;
int             from_xs, from_ys, to_xs, to_ys; /* screen coordinates */
int             type;
Pixel           arrowColor;
int             function;
/*
 * This procedure is based on code given to us courtesy of ..
 *
 * Copyright (c) 1988 Wayne A. Christopher, U. C. Berkeley CAD Group 
 *     faustus@cad.berkeley.edu, ucbvax!faustus
 * Permission is granted to modify and re-distribute this code in any manner
 * as long as this notice is preserved.  All standard disclaimers apply.
 *
 * Converted to X11 by Rick L. Spickelmier
 */
{
#define ARROW_ANGLE     (3.1415926 / 10)
#define ARROW_LENGTH    10

    static GC       arrowGC = (GC) 0;
    XGCValues       values;
    register XPoint scrPoints[3];
    double          angle;
    int             arrowType, lineType;

    angle = atan2((double) from_ys - to_ys, (double) from_xs - to_xs);

    scrPoints[0].x = ARROW_LENGTH * cos(angle - ARROW_ANGLE) + to_xs;
    scrPoints[0].y = ARROW_LENGTH * sin(angle - ARROW_ANGLE) + to_ys;
    scrPoints[1].x = to_xs;
    scrPoints[1].y = to_ys;
    scrPoints[2].x = ARROW_LENGTH * cos(angle + ARROW_ANGLE) + to_xs;
    scrPoints[2].y = ARROW_LENGTH * sin(angle + ARROW_ANGLE) + to_ys;

    values.function = function;
    values.foreground = arrowColor;

    if (arrowGC == (GC) 0) {
        arrowGC = (GC) XCreateGC(ez_Display, (Drawable) win,
            GCFunction | GCForeground, &values);
    }

    arrowType = type & EZ_ARROWTYPE_MASK;
    if (arrowType == EZ_OUTLINEDARROW) {

        lineType = type & EZ_LINETYPE_MASK;
        switch (lineType) {
        case EZ_SOLIDLINE1:
        case EZ_DASHEDLINE1:
        case EZ_DOTTEDLINE1:
            values.line_width = 1;
            break;
        case EZ_SOLIDLINE2:
        case EZ_DASHEDLINE2:
        case EZ_DOTTEDLINE2:
            values.line_width = 2;
            break;
        default:
            values.line_width = 1;
            break;  
        }
        XChangeGC(ez_Display, arrowGC, 
            GCFunction | GCForeground | GCLineWidth, &values);
        XDrawLines(ez_Display, win, arrowGC, scrPoints, 3, CoordModeOrigin);
    }
    else {
        XChangeGC(ez_Display, arrowGC, 
            GCFunction | GCForeground, &values);
        XFillPolygon(ez_Display, (Drawable) win, arrowGC,
            scrPoints, 3, Convex, CoordModeOrigin);
    }

    if (ezPsPlotFlag) {
        register int i;

        for (i = 0; i < 3; i++) {
            scrPoints[i].x *= -1;
        }
        PsDrawPolygon(ezPsFile, scrPoints, 3, 1,
            EzXPsGray(arrowColor), EzXPsGray(arrowColor));
    }

#undef ARROW_ANGLE
#undef ARROW_LENGTH
}


/*
 * ----------------------------------------------------------------------------
 *
 * Ez_DrawWorldLine() --
 *
 * Transform the line denoted by the world coordinates 'from_xw', 'from_yw',
 * 'to_xw', and 'to_yw' to screen coordinates and then draw it, using the 
 * type of line (see ezPlot.h), a function, and the color of your choice.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_DrawWorldLine(win, scrMatrix, from_xw, from_yw, to_xw, to_yw, 
    type, function, lineColor, backColor)

Window          win;
register Matrix scrMatrix;
int             from_xw, from_yw, to_xw, to_yw; /* world coordinates */
int             type, function;
Pixel           lineColor, backColor;
{
    int from_xs, from_ys, to_xs, to_ys; /* screen coordinates */
    int lineType;

    lineType = type & EZ_LINETYPE_MASK;
    if (lineType) {
        from_xs = Ez_XTrans(scrMatrix, from_xw, from_yw);
        from_ys = Ez_YTrans(scrMatrix, from_xw, from_yw);
        to_xs = Ez_XTrans(scrMatrix, to_xw, to_yw);
        to_ys = Ez_YTrans(scrMatrix, to_xw, to_yw);

        if (function == GXxor) {
            lineColor = 0xfd;
        }
        XDrawLine(ez_Display, win, 
            SetUpTmpGC(win, lineType, lineColor, backColor, function),
            from_xs, from_ys, to_xs, to_ys);

        if (type & EZ_ARROWTYPE_MASK) {
            DrawArrow(win, from_xs, from_ys, to_xs, to_ys, 
                type, lineColor, function);
        }

        if (ezPsPlotFlag) {
            PsDrawSolidLine(ezPsFile, -from_xs, from_ys, -to_xs, to_ys, 
                BUTT_CAP, 1, EzXPsGray(lineColor));
        }
    }
}

    

/*
 * ----------------------------------------------------------------------------
 *
 * Ez_DrawFatLine() --
 *
 * The line and it's width are both transformed from world coordinates to
 * screen coordinates, and then a solid line is drawn using the indicated
 * function and color.  Note that the line will always be at least 1 pixel 
 * wide and as such will always be drawn on the screen even if 'lineWidth'
 * is 0.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_DrawFatLine(win, scrMatrix, from_xw, from_yw, to_xw, to_yw, 
    lineWidth, function, lineColor)

Window          win;
register Matrix scrMatrix;
int             from_xw, from_yw, to_xw, to_yw; /* world coordinates */
int             lineWidth, function;
Pixel           lineColor;
{
    static GC   gc = (GC) 0;
    XGCValues   values;
    int         a, b;
    int         from_xs, from_ys, to_xs, to_ys; /* screen coordinates */

    values.function = function;
    values.foreground = lineColor;
    values.line_style = LineSolid;
    if (lineWidth == 0) {
        values.line_width = 0;
    }
    else {
        a = Ez_XTrans(scrMatrix, lineWidth, 0) - Ez_XTrans(scrMatrix, 0, 0);
        b = Ez_YTrans(scrMatrix, lineWidth, 0) - Ez_YTrans(scrMatrix, 0, 0);
        values.line_width = sqrt((double) (a*a + b*b));
    }

    if (gc == (GC) 0) {
        gc = (GC) XCreateGC(ez_Display, (Drawable) win,
            GCFunction | GCForeground | GCLineStyle | GCLineWidth,
            &values);
    }
    else {
        XChangeGC(ez_Display, gc,
            GCFunction | GCForeground | GCLineStyle | GCLineWidth,
            &values);
    }

    from_xs = Ez_XTrans(scrMatrix, from_xw, from_yw);
    from_ys = Ez_YTrans(scrMatrix, from_xw, from_yw);
    to_xs = Ez_XTrans(scrMatrix, to_xw, to_yw);
    to_ys = Ez_YTrans(scrMatrix, to_xw, to_yw);

    XDrawLine(ez_Display, win, gc, from_xs, from_ys, to_xs, to_ys);

    if (ezPsPlotFlag) {
        EzPrintWarn("Ez_DrawFatLine() : postscript not implemented");
    }
}


    
/*
 * ----------------------------------------------------------------------------
 *
 * Ez_WorldText() --
 *
 * Transform the world coordinates of the point denoted by 'wx', and 'wy', 
 * (which is the upper left corner of the first character), to screen
 * coordinates and then write the null terminated string
 * in 'textPtr', onto the screen.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_WorldText(win, scrMatrix, wx, wy, textPtr)
Window          win;
register Matrix scrMatrix;
int             wx, wy;             /* The upper left corner of the first
                                     * character in world coordinates.
                                     */
char            *textPtr;           /* The string must be null terminated. */
{
    int sx, sy;                     /* The upper left corner of the first
                                     * character in screen coordinates.
                                     */

    sx = Ez_XTrans(scrMatrix, wx, wy);
    sy = Ez_YTrans(scrMatrix, wx, wy);

    XDrawImageString(ez_Display, win, ezTextGC, sx, sy+ezPlanePtr->ascent,
        textPtr, strlen(textPtr));
    if (ezPsPlotFlag) {
        PsDrawRText(ezPsFile, textPtr, -sx, sy, "Helvetica", 12);
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Ez_ScreenText() --
 *
 * Take the screen coordinates of the point denoted by 'sx', and 'sy', 
 * (which is the upper left corner of the first character), and write
 * the null terminated string in 'textPtr', onto the screen.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Ez_ScreenText(win, sx, sy, textPtr)
Window   win;
int      sx, sy;                    /* The upper left corner of the first
                                     * character in screen coordinates.
                                     */
char     *textPtr;                  /* The string must be null terminated. */
{
    XDrawImageString(ez_Display, win, ezTextGC, sx, sy+ezPlanePtr->ascent,
        textPtr, strlen(textPtr));
    if (ezPsPlotFlag) {
        PsDrawRText(ezPsFile, textPtr, -sx, sy, "Helvetica", 12);
    }
}


