
/* ezPickRect.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: ezPickRect.c,v 1.1 87/09/24 10:26:05 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "ezPlot.h"
#include "ezPlotInt.h"


/* ---------- Local Procedures ---------- */
#ifdef USE_OLD_X_CODE
static ezStatus DragEdge();
#endif /* USE_OLD_X_CODE */

static ezStatus DragRect();
static bool     IsSameUserWin();


static ezStatus
DragRect(winDataPtr, llsPointPtr, ursPointPtr)
WindowData      *winDataPtr;    /* IN       */
register Point  *llsPointPtr;   /* IN / OUT : lower left-hand corner in
                                 * screen coordinates.
                                 */
register Point  *ursPointPtr;   /* IN / OUT : upper right-hand corner in 
                                 * screen coordinates.
                                 */
{
#define STORE_RECT()                                                        \
    if (sx1 < sx2) {                                                        \
        ulx = sx1;                                                          \
        width = sx2 - sx1;                                                  \
    }                                                                       \
    else {                                                                  \
        ulx = sx2;                                                          \
        width = sx1 - sx2;                                                  \
    }                                                                       \
    if (sy1 < sy2) {                                                        \
        uly = sy1;                                                          \
        height = sy2 - sy1;                                                 \
    }                                                                       \
    else {                                                                  \
        uly = sy2;                                                          \
        height = sy1 - sy2;                                                 \
    }                                                                       \

#define DRAW_RECT()                                                         \
    XDrawRectangle(ez_Display, userWin, ezDrawGC, ulx, uly, width, height)

    Window          userWin;
    XEvent          report;
    XButtonEvent    *bevPtr = (XButtonEvent *) &report.xbutton;
    XMotionEvent    *mevPtr = (XMotionEvent *) &report.xmotion;
    unsigned int    height, width;
    int             ulx, uly;   /* upper-left corner of rectangle */
    int             sx1, sy1, sx2, sy2;

    userWin = winDataPtr->wd_userWin;
        
    sx1 = llsPointPtr->p_x;
    sy1 = llsPointPtr->p_y;
    
    sx2 = ursPointPtr->p_x;
    sy2 = ursPointPtr->p_y;
    
    STORE_RECT();
    DRAW_RECT(); /* draw */
    XFlush(ez_Display);

    while (TRUE) {
        XNextEvent(ez_Display, &report);
        Ez_XEventHandler(&report);
        switch (report.type) {
        case ButtonPress:
            /*
             * Pressing a second button before releasing the
             * first cancels the command.
             */

            DRAW_RECT(); /* erase */
            XFlush(ez_Display);

            return EZ_ERROR;
            break;
        case ButtonRelease:
            DRAW_RECT(); /* erase */

            /*
             * The event queue is emptied to guarentee that the expose
             * window event generated when the digit window is destroyed
             * will be processed.
             */
            Ez_EmptyXEventQue();

            sx2 = bevPtr->x;
            sy2 = bevPtr->y;

            if (sx1 < sx2) {
                llsPointPtr->p_x = sx1;
                ursPointPtr->p_x = sx2;
            }
            else {
                llsPointPtr->p_x = sx2;
                ursPointPtr->p_x = sx1;
            }
            if (sy1 < sy2) {
                llsPointPtr->p_y = sy2;
                ursPointPtr->p_y = sy1;
            }
            else {
                llsPointPtr->p_y = sy1;
                ursPointPtr->p_y = sy2;
            }
            
            return EZ_OK;
            break;
        case LeaveNotify:
            /*
             * Leaving the window before releasing the first
             * button cancels the command.
             */

            DRAW_RECT(); /* erase */
            XFlush(ez_Display);
            Ez_RingBell();

            return EZ_ERROR;
            break;
        case MotionNotify:
            sx2 = mevPtr->x;
            sy2 = mevPtr->y;
        
            DRAW_RECT(); /* erase */
            STORE_RECT();
            DRAW_RECT(); /* draw */
            XFlush(ez_Display);
            break;
        default:
            Ez_XEventHandler(&report);
            break;
        }
    }
    /* NOTREACHED */

#undef STORE_RECT
#undef DRAW_RECT
}

    

/*
 * ----------------------------------------------------------------------------
 *
 * Ez_PickRect() --
 *
 *      Allows you to interactively pick the world coordinates of a
 * rectangle which is forced to align with an imaginary grid, that is
 * based on the windows boundary.
 *
 * Results:
 *   EZ_OK is returned if everything went swell, EZ_ERROR otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

ezStatus
Ez_PickRect(winDataPtr, llwPointPtr, rectPtr)
WindowData      *winDataPtr;
Point           *llwPointPtr;       /* RETURN lower left point in world 
                                     * coordinates.
                                     */
Rect            *rectPtr;           /* RETURN world coordinates of rectangle
                                     * whose lower left corner is (0,0).
                                     */
{
    Point           llsPoint, ursPoint;
    Point           wp1;    /* world point one */
    Point           wp2;    /* world point two */
    Window          userWin;
    double          (*invScrMatrix)[4];
    unsigned int    button = LeftButton;
    int             eventMask = ButtonPressMask | ButtonReleaseMask |
                        LeaveWindowMask | Button1MotionMask;
    bool            haveRectFlag;
    
    userWin = winDataPtr->wd_userWin;
    invScrMatrix = winDataPtr->wd_invScrMatrix;
    XGrabButton(ez_Display, button, AnyModifier, userWin, FALSE, 
        eventMask, GrabModeAsync, GrabModeAsync, None, 
        ezArrowCrossCur);
    Ez_Print(EZ_NORMAL_FONT, "pick a rectangle\n");
        
    if (Ez_FindUserWin(LeftButton, IsSameUserWin, (ClientData) userWin,
        &winDataPtr, &llsPoint) != EZ_OK) {
        Ez_RingBell();
        return EZ_ERROR;
    }
        
    ursPoint.p_x = llsPoint.p_x;
    ursPoint.p_y = llsPoint.p_y;
    
    haveRectFlag = 
        (DragRect(winDataPtr, &llsPoint, &ursPoint) == EZ_OK) ? TRUE : FALSE;
    XUngrabButton(ez_Display, button, AnyModifier, userWin);
    if (haveRectFlag == FALSE) {
        return EZ_ERROR;
    }
       
    wp1.p_x = Ez_XTrans(invScrMatrix, llsPoint.p_x, llsPoint.p_y);
    wp1.p_y = Ez_YTrans(invScrMatrix, llsPoint.p_x, llsPoint.p_y);
    
    wp2.p_x = Ez_XTrans(invScrMatrix, ursPoint.p_x, ursPoint.p_y);
    wp2.p_y = Ez_YTrans(invScrMatrix, ursPoint.p_x, ursPoint.p_y);

    rectPtr->r_xbot = rectPtr->r_ybot = 0;
    rectPtr->r_xtop = ABS(wp2.p_x - wp1.p_x);
    rectPtr->r_ytop = ABS(wp2.p_y - wp1.p_y);
    if ((rectPtr->r_xbot == rectPtr->r_xtop) ||
        (rectPtr->r_ybot == rectPtr->r_ytop)) {
        /*
         * They picked a line or a dot instead of a rectangle !!
         */
        Ez_Print(EZ_ITALIC_FONT, "illegal rectangle !!\n");
        return EZ_ERROR;
    }

    llwPointPtr->p_x = (wp1.p_x < wp2.p_x) ? wp1.p_x : wp2.p_x;
    llwPointPtr->p_y = (wp1.p_y < wp2.p_y) ? wp1.p_y : wp2.p_y;

    return EZ_OK;
}



/* 
 * Definitions for indicating which part of the rectangle is to modified. 
 */
#define URCORNER  0
#define LRCORNER  1
#define LLCORNER  2
#define ULCORNER  3
#define RIGHT     4
#define LEFT      5
#define TOP       6
#define BOTTOM    7

 
#ifdef USE_OLD_X_CODE
ezStatus
Ez_ResizeRect(winDataPtr, llPointPtr, urPointPtr, 
    mouseInitPtr, sizePtr, shiftPtr)

WindowData      *winDataPtr;
Point           *llPointPtr;        /* Defines the rectangle to be      */
Point           *urPointPtr;        /* resized in screen coordinates.   */
                                    /* RETURN in world coordinates      */

Point           *mouseInitPtr;      /* Place where mouse began resize.  */
                                      
Point           *sizePtr;           /* RETURN the resize amount.        */
Point           *shiftPtr;          /* RETURN the distance the lower
                                     * left corner moved.         
                                     */
{
    Point               tmpPoint;
    Window              userWin;
    double              (*invScrMatrix)[4];
    int                 eventMask = ButtonPressMask | ButtonReleaseMask |
                            LeaveWindowMask | PointerMotionMask;
    int                 urx, ury, llx, lly, urwx, urwy, llwx, llwy;
    int                 centerx, centery, mx, my; 
    int                 boxWidth, boxHeight, resHeight, resWidth, mode;
    int                 dragAreaXp, dragAreaXn, dragAreaYp, dragAreaYn;
    
    
    userWin = winDataPtr->wd_userWin;
    invScrMatrix = winDataPtr->wd_invScrMatrix;
    urx = urPointPtr->p_x;
    ury = urPointPtr->p_y;
    llx = llPointPtr->p_x;
    lly = llPointPtr->p_y;
    mx = mouseInitPtr->p_x;
    my = mouseInitPtr->p_y;
    urwx = Ez_XTrans(invScrMatrix, urx, ury);
    urwy = Ez_YTrans(invScrMatrix, urx, ury);
    llwx = Ez_XTrans(invScrMatrix, llx, lly);
    llwy = Ez_YTrans(invScrMatrix, llx, lly);
    boxWidth = urx - llx;
    boxHeight = lly - ury;

    /* Find the center of the rectangle in screen coordinates. */
    centerx = (boxWidth / 2) + llx;
    centery = (boxHeight / 2) + ury;
    
    /* dragArea used for deciding what mode to drag the rectangle. */
    dragAreaXp = centerx +(boxWidth / 6);
    dragAreaYp = centery - (boxHeight / 6);
    dragAreaXn = centerx - (boxWidth / 6);
    dragAreaYn = centery + (boxHeight / 6);

    if (mx < dragAreaXp) {
        if (mx > dragAreaXn) {
            if (my < dragAreaYp) {
                mode = TOP;
            }
            else if (my < centery) {
                if (mx < centerx) {
                    mode = ULCORNER;
                }
                else {
                    mode = URCORNER;
                }
            }
            else if (my < dragAreaYn) {
                if (mx > centerx) {
                    mode = LRCORNER;
                }
                else {
                    mode = LLCORNER;
                }
            }
            else {
                mode = BOTTOM;
            }
        }
        else {
            if (my < dragAreaYp) {
                mode = ULCORNER;
            }
            else if (my < dragAreaYn) {
                mode = LEFT; 
            }
            else {
                mode = LLCORNER;
            }
        }
    }
    else {
    if (my < dragAreaYp) {
        mode = URCORNER;
        }
        else if (my < dragAreaYn) {
        mode = RIGHT; 
        }
        else {
        mode = LRCORNER;
        }
    }
    
    sizePtr->p_x = sizePtr->p_y = 0;
    
    if (XGrabPointer(ez_Display, userWin, FALSE, eventMask, GrabModeAsync,
        GrabModeAsync, None, ezArrowCrossCur, CurrentTime ) != GrabSuccess) {
        EzPrintErr("could not grab button");
        return EZ_ERROR;
    }
    
    Ez_Print(EZ_NORMAL_FONT, "Resizing...");
    
    if (DragEdge(winDataPtr, llPointPtr,
        urPointPtr, (*mouseInitPtr), mode) != EZ_OK) {
        XUngrabPointer(ez_Display, CurrentTime);

        Ez_Print(EZ_ITALIC_FONT, "illegal resize !!\n");
        return EZ_ERROR;
    }
    else {
        XUngrabPointer(ez_Display, CurrentTime);

        /* Transform everything from screen to world coordinates.  */
        tmpPoint.p_x = urPointPtr->p_x;
        tmpPoint.p_y = urPointPtr->p_y;
        urPointPtr->p_x = Ez_XTrans(invScrMatrix, tmpPoint.p_x, tmpPoint.p_y);
        urPointPtr->p_y = Ez_YTrans(invScrMatrix, tmpPoint.p_x, tmpPoint.p_y);
        tmpPoint.p_x = llPointPtr->p_x;
        tmpPoint.p_y = llPointPtr->p_y;
        llPointPtr->p_x = Ez_XTrans(invScrMatrix, tmpPoint.p_x, tmpPoint.p_y);
        llPointPtr->p_y = Ez_YTrans(invScrMatrix, tmpPoint.p_x, tmpPoint.p_y);
        boxWidth = urwx - llwx;
        boxHeight = urwy - llwy;

        resWidth = urPointPtr->p_x - llPointPtr->p_x;
        resHeight = urPointPtr->p_y - llPointPtr->p_y;
        
        sizePtr->p_x = resWidth - boxWidth;
        sizePtr->p_y = resHeight - boxHeight;
            
        shiftPtr->p_x = llPointPtr->p_x - llwx;
        shiftPtr->p_y = llPointPtr->p_y - llwy;
        
        if (llPointPtr->p_x == urPointPtr->p_x ||
           llPointPtr->p_y == urPointPtr->p_y) {
            /*
             * They picked a line or a dot instead of a rectangle !!
             */
            Ez_Print(EZ_ITALIC_FONT, "illegal resize !!\n");
            Ez_RingBell();
            sizePtr->p_x = sizePtr->p_y = 0;
            return EZ_ERROR;
        }
        else {
            Ez_Print(EZ_NORMAL_FONT, "done.\n");
            return EZ_OK;
        }
    }
}
#endif /* USE_OLD_X_CODE */


    
#ifdef USE_OLD_X_CODE
static ezStatus
DragEdge(winDataPtr, llwPointPtr, urwPointPtr, mouseInit, mode)
WindowData      *winDataPtr;
register Point  *llwPointPtr;     /* RETURN lower left point in world 
                                   * coordinates.
                                   */
register Point  *urwPointPtr;     /* RETURN upper right point in world 
                                   * coordinates.
                                   */
Point           mouseInit;        /* Starting point of mouse.
                                   * Drag distance relative to this point.
                                   */
int             mode;             /* Which edge to drag. */ 
{
    Window              userWin, digitWin;
    XEvent              report;
    register XMotionEvent   *pev = (XMotionEvent *) &report.xmotion;
    int                 llx, lly, urx, ury;
    int                 deltaMouseX, deltaMouseY;

    userWin = winDataPtr->wd_userWin;
        
    llx = llwPointPtr->p_x;
    lly = llwPointPtr->p_y;
    
    urx = urwPointPtr->p_x;
    ury = urwPointPtr->p_y;

    Ez_StoreBox(ezVertex, ezVertexCount, llx, lly, urx, ury);
    EZ_DRAW_VERTICES(userWin, ezVertex, ezVertexCount); /* draw box */
    XFlush(ez_Display);

    while (TRUE) {
        XNextEvent(ez_Display, &report);
        switch (report.type) {
        case ButtonPress:
            /*
             * Pressing a second button before releasing the
             * first cancels the command.
             */

            EZ_DRAW_VERTICES(userWin, ezVertex, ezVertexCount); /* erase box */
            XFlush(ez_Display);

            return EZ_ERROR;
            break;
        case ButtonRelease:
            EZ_DRAW_VERTICES(userWin, ezVertex, ezVertexCount); /* erase box */

            /*
             * The event queue is emptied to guarentee that the expose
             * window event generated when the digit window is destroyed
             * will be processed.
             */
            Ez_EmptyXEventQue();
            
            urwPointPtr->p_x = urx;
            urwPointPtr->p_y = ury;
            llwPointPtr->p_x = llx;
            llwPointPtr->p_y = lly;
        
            return EZ_OK;
            break;
        case LeaveNotify:
            /*
             * Leaving the window before releasing the first
             * button first cancels the command.
             */

            EZ_DRAW_VERTICES(userWin, ezVertex, ezVertexCount); /* erase box */
            XFlush(ez_Display);
            Ez_RingBell();

            return EZ_ERROR;
            break;
        case MotionNotify:
            deltaMouseX = (pev->x - mouseInit.p_x);
            deltaMouseY = (pev->y - mouseInit.p_y);
            mouseInit.p_x = pev->x;
            mouseInit.p_y = pev->y;
            switch (mode) {
            case URCORNER:
                urx = urx + deltaMouseX;
                ury = ury + deltaMouseY;
                break;
            case LRCORNER:
                urx = urx + deltaMouseX;
                lly = lly + deltaMouseY;
                break;
            case LLCORNER:
                llx = llx + deltaMouseX;
                lly = lly + deltaMouseY;
                break;
            case ULCORNER:
                llx = llx + deltaMouseX;
                ury = ury + deltaMouseY;
                break;
            case RIGHT:
                urx = urx + deltaMouseX;
                break;
            case TOP:
                ury = ury + deltaMouseY;
                break;
            case LEFT:
                llx = llx + deltaMouseX;
                break;
            case BOTTOM:
                lly = lly + deltaMouseY;
                break;
            default:
                break;
            }
        
            /* erase box, reset, draw box */
            EZ_DRAW_VERTICES(userWin, ezVertex, ezVertexCount);
            Ez_StoreBox(ezVertex, ezVertexCount, llx, lly, urx, ury);
            EZ_DRAW_VERTICES(userWin, ezVertex, ezVertexCount);

            XFlush(ez_Display);
            break;
        default:
            Ez_XEventHandler(&report);
            break;
        }
    }
    /* NOTREACHED */
}
#endif /* USE_OLD_X_CODE */



/* ARGSUSED */
static bool
IsSameUserWin(winDataPtr, userWin)
WindowData  *winDataPtr;
Window      userWin;
{
    return (winDataPtr->wd_userWin == userWin);
}


#ifdef BETTY BOOP
/*
 * NumberOfDigits() is left over from when we used to have a digit window.
 */

static int
NumberOfDigits(x)
register int x;                 /* x is expected to be a positive integer */
/*
 * Local function that returns the number of digits in 'x'.
 */
{
    register int count = 1;

    x = ABS(x);
    while (TRUE) {
        if (x < 10) {
            break;
        } 
        else {
            x /= 10;
            count++;
        }
    }
    return (count);
}
#endif BETTY BOOP


