
/* ezPlot.h --
 *
 *     George Carvalho, Esq. -- wrote original X10 version
 *     Ben Chen              -- converted entire module to X11 and added
 *                              postscript stuff.
 *
 *     (Advising Professor - E. S. Kuh)
 *
 * This file contains definitions used by the ezPlot module,
 * which does graphics stuff using the X window system.
 *
 * Copyright (C) 1991 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header$
 */

#ifndef EZ_PLOT_EXPORT_H /* { */
#define EZ_PLOT_EXPORT_H /* once-only latch */

#define EZ_MOD_NAME "ezplot"

#include "bearCopyRight.h"

/*
 * The fonts that can be used when printing things in the console window.
 */
#define EZ_BOLD_FONT    0
#define EZ_ITALIC_FONT  1
#define EZ_NORMAL_FONT  2

#ifdef BEAR_BLACK_BOX /* { */

typedef struct _ezDummy ezDummy;
struct _ezDummy {
    int dummy;                      /* not used                             */
};

typedef ezDummy WindowData;

#else /*  } else { BEAR_BLACK_BOX */

#include "ansi.h"
#include "magic.h"
#include "geometry.h"

#include <X11/X.h>
#include <X11/Intrinsic.h>
#ifdef USE_OLD_X_CODE /* { */
#include <X11/X10.h>
#endif /* } USE_OLD_X_CODE */


#ifdef lint
/*
 * Include the "private" X11 header files when your using lint so that it
 * will complain a little bit less then usual.
 */
#include <X11/IntrinsicP.h>
#include <X11/CoreP.h>
#include <X11/ShellP.h>
#include <X11/Xaw/AsciiSrcP.h>
#include <X11/Xaw/AsciiSinkP.h>
#include <X11/Xaw/AsciiTextP.h>
#include <X11/Xaw/TextSrcP.h>
#endif /* lint */


#define DARK_BLUE       "#3d11fd"
#define DARK_PURPLE     "#7a04d4"
#define HOT_PINK        "#ff2af2"
#define ROSE            "#f0cbee"
#define LAVENDER        "#c07aed"
#define LIGHT_YELLOW    "#fdfe6e"
#define LIME            "#2cf9a6"
#define RED             "#f42d3f"
#define LIGHT_BLUE      "#9efef9"
#define MED_BLUE        "#00d8ff"
#define TEAL            "#0589a8"
#define YELLOW          "#fefc64"

#define LeftButton      Button1
#define MiddleButton    Button2
#define RightButton     Button3

typedef double Matrix[4][4];

typedef enum _ezItrStatus ezItrStatus;
enum _ezItrStatus {
    EZ_CONTINUE_SEARCH, EZ_COMPLETE_SEARCH, EZ_STOP_SEARCH
};

typedef enum _ezStatus ezStatus;
enum _ezStatus {
    EZ_OK, EZ_ERROR
};

typedef struct _ColorData ColorData;
struct _ColorData {
    char    *namePtr;                   /* Name of the color in .Xdefaults
                                         * file.
                                         */
    Pixel   *colorPtr;
    Pixel   *bwDef;                     /* black or white default       */
    char    *defColorSpecPtr;           /* default color specification  */
};

typedef struct _TitleBar TitleBar;
struct _TitleBar {
    bool        tb_visible;
    int         tb_x, tb_y;             /* Upper left corner of the first
                                         * character.
                                         */
    int         tb_winHeight;
    int         tb_pixLen;              /* length of string in pixels       */
    char        *tb_textPtr;
    XFontStruct *tb_fontInfoPtr;
};

typedef struct _CloseWinData CloseWinData;
struct _CloseWinData {
    bool        force;
};

typedef struct _WindowData WindowData;
struct _WindowData {
    Window      wd_baseWin;                 
    Window      wd_titleBarWin;         /* sub window of base window        */
    Window      wd_userWin;             /* sub window of base window        */

    XWindowAttributes  wd_userWinInfo;
    XWindowAttributes  wd_baseWinInfo;

    TitleBar    wd_titleBar;

    int         wd_scrGridSize;
    int         wd_worldGridSize;
    Rect        wd_imageBnd;            /* World coordinates of the rectangle
                                         * that encloses the area currently
                                         * in view.
                                         */
    Matrix      wd_scrMatrix;           /* Transformation from world
                                         * coordinates to screen coordinates.
                                         */
    Matrix      wd_invScrMatrix;        /* Transfromation from screen
                                         * coordinates to world coordinates.
                                         */
/*
 * Pointer to the procedure that is responsible for redrawing the window.
 * The procedure should have the following form:
 *
 * void
 * wd_redrawWinProc(winDataPtr, subWinBndPtr)
 * WindowData   *winDataPtr;
 * Rect         *subWinBndPtr;
 * {
 *     if (subWinBndPtr == NULL) then redraw the entire window
 * }
 */
    void        (*wd_redrawProc)(); 

/*
 * Pointer to the procedure that is responsible for closing the window.
 * The procedure should have the following form:
 *
 * void
 * wd_closeWinProc(winDataPtr, closeWinDataPtr)
 * WindowData   *winDataPtr;
 * CloseWinData *closeWinDataPtr;
 * {
 * }
 */
    void         (*wd_closeWinProc)();

/*
 * Pointer to the procedure that will be called when a mouse button
 * is pressed in the user window.  The procedure should have the
 * following form:
 *
 * void
 * wd_userWinButtonProc(winDataPtr, button,
 *     xRootWin, yRootWin, xUserWin, yUserWin)
 *
 * WindowData  *winDataPtr;
 * int         button:             (* one of LeftButton, MiddleButton,
 *                                  * or RightButton
 *                                  *)
 * int         xRootWin, yRootWin; (* relative to the 'RootWindow' *)
 * int         xUserWin, yUserWin; (* relative to 'winData->wd_userWin' *)
 * {
 * }
 */
    void        (*wd_userWinButtonProc)();

    WindowData  *wd_nextPtr;

    ClientData  wd_clientData;          /* This space for hire.             */
};

typedef struct _DrawData DrawData;
struct _DrawData {
    WindowData  *winDataPtr;
    int         displayFunc;            /* display function                 */
    int         color;
    float       width;
    ClientData  clientData;
};

/* Old X10 structure */
typedef struct _OpaqueFrame OpaqueFrame;
struct _OpaqueFrame {
    Window          self;           /* Window id of the window, filled in 
                                     * later.
                                     */
    int             x, y;           /* where to create the window */
    unsigned int    width, height;  /* width and height */
    unsigned int    bdrwidth;       /* border width */
    unsigned long   border;         /* border pixel */
    Pixmap          background;     /* background pixel */
};


#define EZ_SOLIDLINE1               01
#define EZ_SOLIDLINE2               02
#define EZ_DASHEDLINE1              04
#define EZ_DASHEDLINE2             010
#define EZ_DOTTEDLINE1             020
#define EZ_DOTTEDLINE2             040
#define EZ_LINETYPE_MASK           077

#define EZ_OUTLINEDARROW          0100
#define EZ_SOLIDARROW             0200
#define EZ_ARROWTYPE_MASK         0300

#define EZ_CROSS_CUR                01
#define EZ_ARROW_CROSS_CUR          02
#define EZ_NW_ARROW_CUR             04  /* north west pointing arrow */
#define EZ_HORZ_ARROWS_CUR         010
#define EZ_VERT_ARROWS_CUR         020
#define EZ_CIRCLE_OUTLINE_CUR      040
#define EZ_SQUARE_OUTLINE_CUR     0100
#define EZ_STOP_WATCH_CUR         0200

/* module options */
#define EZ_COLOR_MON_OPT            01
#define EZ_REVERSE_VIDEO_OPT        02


/* ---------- External procedures for the ezPlot module ---------- */

EXTERN ezStatus Ez_Begin
    ARGS((char *appNamePtr, Widget *topWidget, int argc, char **argv));

EXTERN void Ez_XEnd
    NULLARGS;

EXTERN ezStatus Ez_CreateConsoleWin
    ARGS((char *tbTextPtr, WindowData **winDataPtrPtr));

EXTERN void Ez_DestroyConsoleWin
    NULLARGS;

EXTERN bool Ez_IsConsoleWin
    ARGS((WindowData *winDataPtr));

EXTERN ezStatus Ez_CreateWin
    ARGS((char *promptPtr, char *geometryPtr, char *tbTextPtr, 
        void (*redrawProc)(), Rect *imageBndPtr, WindowData **winDataPtrPtr));

EXTERN ezStatus Ez_DestroyWin
    ARGS((WindowData *winDataPtr));

EXTERN void Ez_ToggleTitleBar
    ARGS((WindowData *winDataPtr));

EXTERN void Ez_FillScrRect
    ARGS((Window win, Rect *rectPtr, Pixel fillColor));

EXTERN void Ez_FillRect
    ARGS((Window win, Matrix scrMatrix, Rect *rectPtr, Pixel fillColor));

EXTERN void Ez_OutlineRect
    ARGS((Window win, Matrix scrMatrix, Rect *rectPtr, 
        int type, Pixel lineColor, Pixel backColor));

EXTERN void Ez_FillPolygon
    ARGS((Window win, Matrix scrMatrix, Point *points, int numOfPoints,
        Pixel fillColor));

EXTERN void Ez_TileRect
    ARGS((Window win, Matrix scrMatrix, Rect *rectPtr, Pixmap tile, 
        int function));

#ifdef STIPPLE /* { */
EXTERN void Ez_StippleRect
    ARGS((Window win, Matrix scrMatrix, Rect *rectPtr, Pixmap stipple,
        Pixel color, int function));
#endif /* } STIPPLE */

EXTERN void Ez_StretchRect
    ARGS((Rect *rectPtr, double percent));

EXTERN void Ez_DrawWorldLine
    ARGS((Window win, Matrix scrMatrix, 
        int from_xw, int from_yw, int to_xw, int to_yw, 
        int type, int function, 
        Pixel lineColor, Pixel backColor));

EXTERN void Ez_DrawFatLine
    ARGS((Window win, Matrix scrMatrix, 
        int from_xw, int from_yw, int to_xw, int to_yw, 
        int lineWidth, int function, 
        Pixel lineColor));

EXTERN void Ez_DrawWorldSmallDot
    ARGS((Window win, Matrix scrMatrix,
        int wx, int wy, /* world coordinates */
        int dotColor));

EXTERN void Ez_ScreenText
    ARGS((Window win, int sx, int sy, char *textPtr));

EXTERN void Ez_WorldText
    ARGS((Window win, Matrix scrMatrix, int wx, int wy, char *textPtr));

extern ezStatus Ez_FindUserWin(/* button, checkWinProc, clientData,
    winData, screenPtr */);
/*
 * int          button;
 * bool         (*checkWinProc)();
 * ClientData   clientData;
 * WindowData   **winData;
 * Point        *screenPtr;
 */

EXTERN ezStatus Ez_PickPointList
    ARGS((WindowData  *winDataPtr, LinkedPoint **pointListPtr));

EXTERN ezStatus Ez_PickRect
    ARGS((WindowData *winDataPtr, Point *llwPointPtr, Rect *rectPtr));

extern void Ez_Zoom(/* winDataPtr, percent */);
/*
 * WindowData *winDataPtr;
 * double percent:
 */

EXTERN ezItrStatus Ez_ItrWindows
    ARGS((ezItrStatus (*userProc)(), ClientData clientData));

/* 
 * The variable whichFont for Ez_Print() may be EZ_BOLD_FONT, EZ_ITALIC_FONT, 
 * or EZ_NORMAL_FONT 
 */
EXTERN void Ez_Print
    ARGS((int whichFont, char *stringPtr));

extern void Ez_DefineCursor(/* win, cursorType */);
/*
 * Window   win;
 * int      cursorType;
 */

EXTERN ezStatus Ez_SetUpColors
    ARGS((ColorData *colorData, int numOfColors));

EXTERN ezStatus Ez_GetLine
    ARGS((Window *winPtr, Point *screenPtr, char *buffer, int bufferSize, 
        bool *keepGoingPtr));

EXTERN ezStatus Ez_CalcViewingTrans
    ARGS((WindowData *winDataPtr));

EXTERN WindowData *Ez_GetWinDataOfWin
    ARGS((Window win));

EXTERN void Ez_GetPointerXY
    ARGS((Window win, int *xLocPtr, int *yLocPtr));

extern bool Ez_FindOverlap();

EXTERN bool Ez_Pause
    NULLARGS;

EXTERN void Ez_XEventHandler
    ARGS((XEvent *reportPtr));

EXTERN void Ez_EmptyXEventQue
    NULLARGS;

EXTERN void Ez_ClearConWin
    NULLARGS;

EXTERN void Ez_Prompt
    ARGS((char *promptPtr));

EXTERN ezStatus Ez_OpenPsFile
    ARGS((char  *fileNamePtr,
        int     width, int height, 
        double  translateX, double translateY,
        double  scaleX, double scaleY, 
        double  rotation));

EXTERN void Ez_ClosePsFile
    NULLARGS;

EXTERN void Ez_Flush
    NULLARGS;

EXTERN void Ez_RingBell
    NULLARGS;

#ifdef USE_OLD_X_CODE /* { */
EXTERN ezStatus Ez_DragBoundary
    ARGS((WindowData *winDataPtr, Point *mousePtr, Vertex *vertex, 
        int vertexCount, Point *deltaPtr));

EXTERN ezStatus Ez_ResizeRect
    ARGS((WindowData *winDataPtr, Point *llPointPtr, Point *urPointPtr, 
        Point *mouseInitPtr, Point *sizePtr, Point *shiftPtr));
#endif /* } USE_OLD_X_CODE */

#ifdef DEBUG /* { */
EXTERN void Ez_ToggleDebug
    NULLARGS;
#endif /* } DEBUG */


/* ---------- External global variables for the ezPlot module ---------- */

extern Display          
    *ez_Display;

extern unsigned int     
    ez_Options;

extern Pixel
    ez_WhitePixel,
    ez_BlackPixel,

    ez_BorderColor,                 /* border color of the windows           */
    ez_BackColor,                   /* background color of the windows       */
    ez_MouseColor,                  /* mouse color                           */
    ez_TextColor,                   /* text color                            */
    ez_HighLightColor,              /* high light color                      */

    ez_ConBackColor,                /* background color for the console      */
    ez_ConBorderColor,              /* border color for the console          */
    ez_ConTextColor;                /* normal text color for the console     */

extern Pixmap
    ez_CargoNetBitmap,
    ez_LeftSlashBitmap, 
    ez_RightSlashBitmap,
    ez_BackPixmap;                  /* background pixmap for the windows    */

extern FILE 
    *ez_LogFilePtr;


/* ---------- External macros for the ezPlot module ---------- */

#define Ez_XTrans(MAT, X, Y) ((X)*(MAT)[0][0] +  (Y)*(MAT)[1][0] + (MAT)[3][0])
#define Ez_YTrans(MAT, X, Y) ((X)*(MAT)[0][1] +  (Y)*(MAT)[1][1] + (MAT)[3][1])

#define EZ_ROOT_WINDOW  DefaultRootWindow(ez_Display)
#define EZ_CMAP         DefaultColormap(ez_Display, DefaultScreen(ez_Display))

#ifdef USE_OLD_X_CODE /* { */
#define Ez_StoreBox(vertex, vertexCount, llx, lly, urx, ury)            \
{                                                                       \
    vertex[0].x = vertex[3].x = vertex[4].x = (urx);                    \
    vertex[0].y = vertex[1].y = vertex[4].y = (ury);                    \
    vertex[1].x = vertex[2].x = (llx);                                  \
    vertex[2].y = vertex[3].y = (lly);                                  \
    vertexCount = 5;                                                    \
}
#endif /* } USE_OLD_X_CODE */

#endif /* } BEAR_BLACK_BOX */

#endif /* } EZ_PLOT_EXPORT_H */

