
/* ezXBitmaps.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: ezXBitmaps.c,v 1.1 87/09/24 10:26:17 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "ezPlot.h"
#include "ezPlotInt.h"
#include "ezPsPlot.h"


#define cargo_net_width 16
#define cargo_net_height 16
#define cargo_net_x_hot -1
#define cargo_net_y_hot -1
static char cargo_net_bits[] = {
   0x00, 0x00, 0x02, 0x40, 0x04, 0x20, 0x08, 0x10, 0x10, 0x08, 0x20, 0x04,
   0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x20, 0x04, 0x10, 0x08,
   0x08, 0x10, 0x04, 0x20, 0x02, 0x40, 0x00, 0x00};

#define half_weave_1_width 16
#define half_weave_1_height 16
#define half_weave_1_x_hot -1
#define half_weave_1_y_hot -1
static char half_weave_1_bits[] = {
   0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11, 0x88, 0x88, 0x00, 0x00,
   0x00, 0x00, 0x11, 0x11, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11,
   0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11};

#define half_weave_2_width 16
#define half_weave_2_height 16
#define half_weave_2_x_hot -1
#define half_weave_2_y_hot -1
static char half_weave_2_bits[] = {
   0x00, 0x00, 0x22, 0x22, 0x44, 0x44, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22,
   0x44, 0x44, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x44, 0x44, 0x00, 0x00,
   0x00, 0x00, 0x22, 0x22, 0x44, 0x44, 0x00, 0x00};

#define small_dot_width 16
#define small_dot_height 16
#define small_dot_x_hot -1
#define small_dot_y_hot -1
static char small_dot_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x03, 0xe0, 0x07,
   0xf0, 0x0f, 0xf0, 0x0f, 0xf0, 0x0f, 0xf0, 0x0f, 0xe0, 0x07, 0xc0, 0x03,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

#define gray_width 16
#define gray_height 16
#define gray_x_hot -1
#define gray_y_hot -1
static char gray_bits[] = {
   0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55,
   0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55,
   0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55};

#define light_gray_width 16
#define light_gray_height 16
#define light_gray_x_hot -1
#define light_gray_y_hot -1
static char light_gray_bits[] = {
   0x00, 0x00, 0x55, 0x55, 0x00, 0x00, 0x55, 0x55, 0x00, 0x00, 0x55, 0x55,
   0x00, 0x00, 0x55, 0x55, 0x00, 0x00, 0x55, 0x55, 0x00, 0x00, 0x55, 0x55,
   0x00, 0x00, 0x55, 0x55, 0x00, 0x00, 0x55, 0x55};


void
Ez_DrawWorldSmallDot(win, scrMatrix, wx, wy, dotColor)
Window          win;
register Matrix scrMatrix;
int             wx, wy;             /* world coordinates */
int             dotColor;
{
    static GC   gc = (GC) 0;

    XGCValues   values;
    int         sx, sy;             /* screen coordinates */

    sx = Ez_XTrans(scrMatrix, wx, wy) - small_dot_width / 2; 
    sy = Ez_YTrans(scrMatrix, wx, wy) - small_dot_height / 2;
    values.foreground = dotColor;
    values.clip_x_origin = sx;
    values.clip_y_origin = sy;
    if (gc) {
        XChangeGC(ez_Display, gc, GCForeground|GCClipXOrigin|GCClipYOrigin,
            &values);
    }
    else {
        values.clip_mask = ezSmallDotBitmap;
        gc = (GC) XCreateGC(ez_Display, (Drawable) win, GCClipMask| 
            GCForeground|GCClipXOrigin|GCClipYOrigin, &values);
    }
    
    XCopyPlane(ez_Display, (Drawable) ezSmallDotBitmap, (Drawable) win, gc,
        0, 0, (unsigned int) small_dot_width,
        (unsigned int) small_dot_height, sx, sy, 1);

    if (ezPsPlotFlag) {
        /* The '4' takes into account the padding of the bitmap. */
        PsDrawCircle(ezPsFile, -(sx+8), sy+8, 4, 1, PS_BLACK,
            EzXPsGray(dotColor));
    }
}



void
EzSetUpTilesAndBitmaps ()
{
    ezGrayBdrPixmap = XCreateBitmapFromData(ez_Display, 
        EZ_ROOT_WINDOW,
        gray_bits, 
        (unsigned int) gray_width, 
        (unsigned int) gray_height);    
    ez_CargoNetBitmap = XCreateBitmapFromData(ez_Display,
        EZ_ROOT_WINDOW,
        cargo_net_bits, 
        (unsigned int) cargo_net_width, 
        (unsigned int) cargo_net_height);
    ez_LeftSlashBitmap = XCreateBitmapFromData(ez_Display,
        EZ_ROOT_WINDOW,
        half_weave_1_bits,
        (unsigned int) half_weave_1_width, 
        (unsigned int) half_weave_1_height);
    ez_RightSlashBitmap = XCreateBitmapFromData(ez_Display,
        EZ_ROOT_WINDOW,
        half_weave_2_bits,
        (unsigned int) half_weave_2_width, 
        (unsigned int) half_weave_2_height);
    ezSmallDotBitmap = XCreateBitmapFromData(ez_Display,
        EZ_ROOT_WINDOW,
        small_dot_bits,
        (unsigned int) small_dot_width, 
        (unsigned int) small_dot_height);
    ez_BackPixmap = XCreateBitmapFromData(ez_Display,
        EZ_ROOT_WINDOW,
        light_gray_bits,
        (unsigned int) light_gray_width, 
        (unsigned int) light_gray_height);
}



