
/* ezXCursors.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: ezXCursors.c,v 1.1 87/09/24 10:26:20 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "ezPlot.h"
#include "ezPlotInt.h"
#include <X11/cursorfont.h>


void
EzSetUpCursors()
{
    XColor fg, bg;
    
    fg.pixel = ez_MouseColor;
    bg.pixel = ez_WhitePixel;
    XQueryColor(ez_Display, EZ_CMAP, &fg);
    XQueryColor(ez_Display, EZ_CMAP, &bg);

    ezCrossCur = XCreateFontCursor(ez_Display, XC_crosshair);
    XRecolorCursor(ez_Display, ezCrossCur, &fg, &bg);

    ezArrowCrossCur = XCreateFontCursor(ez_Display, XC_fleur);
    XRecolorCursor(ez_Display, ezArrowCrossCur, &fg, &bg);
    
    ezNWArrowCur = XCreateFontCursor(ez_Display, XC_left_ptr);
    XRecolorCursor(ez_Display, ezNWArrowCur, &fg, &bg);
    
    ezHorzArrowsCur = XCreateFontCursor(ez_Display, XC_sb_h_double_arrow);
    XRecolorCursor(ez_Display, ezHorzArrowsCur, &fg, &bg);

    ezVertArrowsCur = XCreateFontCursor(ez_Display, XC_sb_v_double_arrow);
    XRecolorCursor(ez_Display, ezVertArrowsCur, &fg, &bg);   

    ezCircleOutlineCur = XCreateFontCursor(ez_Display, XC_circle);
    XRecolorCursor(ez_Display, ezCircleOutlineCur, &fg, &bg);

    ezSquareOutlineCur = XCreateFontCursor(ez_Display, XC_dotbox);
    XRecolorCursor(ez_Display, ezSquareOutlineCur, &fg, &bg);

    ezStopWatchCur = XCreateFontCursor(ez_Display, XC_watch);
    XRecolorCursor(ez_Display, ezStopWatchCur, &fg, &bg);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Ez_DefineCursor() --
 *
 * The purpose of this function is to hide the memory containing
 * the cursors from the user so that he will not accidentally 
 * free it.
 *
 * Results:
 *   none
 *
 * Side effects:
 *   The cursor shape (visible when the mouse is in the window) is changed.
 * ----------------------------------------------------------------------------
 */

void
Ez_DefineCursor(win, cursorType)
Window  win;
int     cursorType;
{
    switch (cursorType) {
    case EZ_CROSS_CUR:
        XDefineCursor(ez_Display, win, ezCrossCur);
        break;
    case EZ_ARROW_CROSS_CUR:
        XDefineCursor(ez_Display, win, ezArrowCrossCur);
        break;

    case EZ_NW_ARROW_CUR:
        XDefineCursor(ez_Display, win, ezNWArrowCur);
        break;

    case EZ_HORZ_ARROWS_CUR:
        XDefineCursor(ez_Display, win, ezHorzArrowsCur);
        break;
    case EZ_VERT_ARROWS_CUR:
        XDefineCursor(ez_Display, win, ezVertArrowsCur);
        break;

    case EZ_CIRCLE_OUTLINE_CUR:
        XDefineCursor(ez_Display, win, ezCircleOutlineCur);
        break;
    case EZ_SQUARE_OUTLINE_CUR:
        XDefineCursor(ez_Display, win, ezSquareOutlineCur);
        break;

    case EZ_STOP_WATCH_CUR:
        XDefineCursor(ez_Display, win, ezStopWatchCur);
        break;
    default:
        EzPrintErr("Ez_DefineCursor() : unknown cursor type");
        break;
    }
}



