
/* ezXdefaults.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1991 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: ezXDefaults.c,v 1.1 87/09/24 10:26:22 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "ezPlot.h"
#include "ezPlotInt.h"


/* ---------- Local Procedures ---------- */
static void UpEachWord();
static char *GetDB();

/*
 * Below is a list of the currently defined X defaults used by the
 * ezPlot module, complete with a short description of each.
 * 
 * The application name field (writen appName) listed at the beginning of 
 * each string is passed to the ezPlot module with the Ez_Begin() procedure.
 *
 * Name                         Comment
 *  
 * appName.background.color:            Background color for all windows.
 * appName.blackAndWhite:               Use a black and white color scheme
 *                                      (on / off).
 * appName.border.color:                Border color for all windows.
 * appName.borderWidth:                 Border width for all windows.
 * appName.highLight.color:             High light color.
 * appName.mouse.color:                 Mouse color for all windows.
 * appName.reverseVideo:                Reverse black and white color scheme
 *                                      (on / off).
 * appName.text.color:                  Text color in all windows except the
 *                                      the console window.
 * appName.text.font                    Font for all windows except the console
 *                                      window.
 * appName.windowHeight:                Starting height for all windows except 
 *                                      the console window.
 * appName.windowWidth:                 Starting width for all windows except 
 *                                      the console window.
 *
 * appName.console.background.color:    Console background color.
 * appName.console.border.color:        Console border color.
 * appName.console.text.color:          Console text color.
 * appName.console.text.font:           Console font.
 */

/* ---------- Local Global Variables ---------- */
static ColorData colorData[] = {
    { "background.color", 
        &ez_BackColor, &ez_WhitePixel, ROSE },

    { "border.color", 
        &ez_BorderColor, &ez_BlackPixel, RED },

    { "highLight.color", 
        &ez_HighLightColor, &ez_WhitePixel, HOT_PINK },

    { "mouse.color", 
        &ez_MouseColor, &ez_BlackPixel, DARK_PURPLE },

    { "text.color", 
        &ez_TextColor, &ez_BlackPixel, "SteelBlue" },

    { "console.background.color",   
        &ez_ConBackColor, &ez_WhitePixel, ROSE },

    { "console.border.color", 
        &ez_ConBorderColor, &ez_BlackPixel, RED },

    { "console.text.color",   
        &ez_ConTextColor, &ez_BlackPixel, "#0684cb" },
};
static int numOfColors = sizeof (colorData) / sizeof (*colorData);


static void 
UpEachWord(spec)
char *spec;         /* Resource specification */
/*
 * Upcases each word in the specification.  Modifies the string in
 * place.
 *
 * This procedure is from the Octtools code (vem/main/defaults.c).
 */
{
    char *now, *next;

    now = spec;
    do {
        next = strchr(now, '.');
        if (*now != '.') {
            *now = (char) toupper(*now);
        }
        now = next;
        if (now) {
            now++;
        }
    } while (now && (*now != '\0'));
}


static char *
GetDB(disp, prog, name)
Display *disp;      /* Connection to X  */
char    *prog;      /* Program name     */
char    *name;      /* Name for default */
/*
 * This routine attempts to extract the default `name' from
 * the resource database constructed by the toolkit.  The
 * program name is concatenated onto the specification before
 * retrieval.  The class specification is derived directly 
 * from the original name by capitalizing each item down
 * the hierarchy.
 *
 * This procedure is from the Octtools code (vem/main/defaults.c).
 */
{
#define MAX_DB_SPEC 512

    static char full_spec[MAX_DB_SPEC], class_spec[MAX_DB_SPEC];

    XrmValue    value;
    char        *rep_str;

    strcpy(full_spec, prog);
    strcat(full_spec, ".");
    strcat(full_spec, name);

    strcpy(class_spec, prog);
    strcat(class_spec, ".");
    strcat(class_spec, name);
    UpEachWord(class_spec);

    if (XrmGetResource(XtDatabase(disp), full_spec, class_spec,
        &rep_str, &value)) {
        return (char *) value.addr;
    } 
    else {
        return (char *) 0;
    }

#undef MAX_DB_SPEC
}



ezStatus
EzGetXDefaults()
{
    char *defValuePtr;

    /* First, the non-color specific ones */
    ezBorderWidth = DEF_BDR_WIDTH;
    if ((defValuePtr = GetDB(ez_Display, ezAppNamePtr, "borderWidth")) != 
        NIL(char)) {
        ezBorderWidth = atoi(defValuePtr);
    }

    ezWinWidth = DEF_WIN_WIDTH;
    if ((defValuePtr = GetDB(ez_Display, ezAppNamePtr, "windowWidth")) != 
        NIL(char)) {
        ezWinWidth = atoi(defValuePtr);
    }

    ezWinHeight = DEF_WIN_HEIGHT;
    if ((defValuePtr = GetDB(ez_Display, ezAppNamePtr, "windowHeight")) != 
        NIL(char)) {
        ezWinHeight = atoi(defValuePtr);
    }

    if (((defValuePtr = GetDB(ez_Display, ezAppNamePtr, 
        "text.font")) == NIL(char)) ||
        ((ezPlanePtr = XLoadQueryFont(ez_Display, defValuePtr)) == NULL)) {
        ezPlanePtr = XLoadQueryFont(ez_Display, DEF_FONTNAME);
    }
    if (((defValuePtr = GetDB(ez_Display, ezAppNamePtr, 
        "console.text.font")) == NIL(char)) ||
        ((ezConFontPtr = XLoadQueryFont(ez_Display, defValuePtr)) == NULL)) {
        ezConFontPtr = XLoadQueryFont(ez_Display, DEF_CON_NORMAL_FONTNAME);
    }

    if ((DisplayCells(ez_Display, DefaultScreen(ez_Display)) <= 2) ||
        ((defValuePtr = GetDB(ez_Display, ezAppNamePtr, 
            "blackAndWhite")) != NIL(char) && 
        (strcmp(defValuePtr, "on") == 0 || strcmp(defValuePtr, "On") == 0))) {
        /* black and white display */

        if ((defValuePtr = GetDB(ez_Display, ezAppNamePtr,
            "reverseVideo")) != NIL(char) &&

            (strcmp(defValuePtr, "on") == 0 ||
                strcmp(defValuePtr, "On") == 0)) {
            Ez_SetOption(ez_Options, EZ_REVERSE_VIDEO_OPT);
        }
    }
    else {
        /* color display */
        Ez_SetOption(ez_Options, EZ_COLOR_MON_OPT);
    }

    if (Ez_SetUpColors(colorData, numOfColors) != EZ_OK) {
        return EZ_ERROR;
    }
        
    return EZ_OK;
}



ezStatus
Ez_SetUpColors(colorData, numOfColors)
ColorData   *colorData;
int         numOfColors;
{
    XColor      colorDef;
    Colormap    cmap;
    int         i;
    char        *colorSpecPtr;          /* color specification */
    bool        invert;

    ez_WhitePixel = WhitePixel(ez_Display, DefaultScreen(ez_Display));
    ez_BlackPixel = BlackPixel(ez_Display, DefaultScreen(ez_Display));

    if (ez_Options & EZ_COLOR_MON_OPT) {
        /* color display */
        cmap = DefaultColormap(ez_Display, DefaultScreen(ez_Display));
        for (i = 0; i < numOfColors ;i++) {
            if ((colorSpecPtr = GetDB(ez_Display, ezAppNamePtr, 
                colorData[i].namePtr)) == NIL(char)) {
                colorSpecPtr = colorData[i].defColorSpecPtr;
            }

            if (XParseColor(ez_Display, cmap, colorSpecPtr, &colorDef) &&
                XAllocColor(ez_Display, cmap, &colorDef)) {
                *colorData[i].colorPtr = colorDef.pixel;
            }
            else {
                colorData[i].colorPtr = colorData[i].bwDef;
            }
        }
    }
    else {
        /* black and white display */
        invert = (ez_Options & EZ_REVERSE_VIDEO_OPT) ? TRUE : FALSE;
        for (i = 0; i < numOfColors ;i++) {
            if (invert) {
                if (colorData[i].bwDef == &ez_WhitePixel) {
                    *(colorData[i].colorPtr) = ez_BlackPixel;
                }
                else {
                    *(colorData[i].colorPtr) = ez_WhitePixel;
                }
            }
            else {
                *(colorData[i].colorPtr) = *(colorData[i].bwDef);
            }
        }
    }

    return EZ_OK;
}


