
/* fpgMain.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header: fpgMain.c,v 1.2 88/06/27 14:57:31 bear Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "fpgInt.h"


typedef struct _MiscData MiscData;
struct _MiscData {
    Line    bnSearchLine;               /* bottleneck line search area      */
    BNLine  *bnLinePtr;
};

/* ---------- Local Procedures ---------- */
static void            AddWallJunctToList();
static void            BuildJunctionTable();
static void            FreeHorzWallList();
static BearStatus      FindHorzBnLine(), FindVertBnLine();
static BearStatus      HLineCrossesVLine();
static WallJunction    *MakeWallJunct();
static WallSegment     *MakeWallSeg();
static int             BuildHorzWall(), BuildVertWall();
static int             FindBnTile();
static int             FindHorzDomTile(), FindVertDomTile();
static int             IndexVertDomTile();

/* ---------- Local Global Variables ---------- */

/*
 * location of north-west, center and south-east wall junctions.
 */
static Point nwPoint, centerPoint, sePoint;

static FloorPlanGraph *globalFpgPtr;

static BNLine *horzBnLineList, *vertBnLineList;

static LinkedWallJunction *horzWallList; /* List of the first junction in
                                          * each horizontal wall.
                                          */

static Plane *horzPlanePtr, *vertPlanePtr;

static Rect domSearchArea;               /* dominant tile search area */

static Rect *horzPlaneBndPtr, *vertPlaneBndPtr;
static WallJunction *startJunctPtr, *NWJunctPtr, *SEJunctPtr;

static int domTileCount;                 /* dominant tile count */

static int horzDomTileIndex, vertDomTileIndex;


/* 
 * ----------------------------------------------------------------------------
 *
 * AddWallJunctToList() --
 *
 *    Allocates memory for a linked wall junction, assigns the given wall
 *    junction to it, and then adds it to the given list of linked wall
 *    junctions.
 *
 * Results:
 *    None.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static void
AddWallJunctToList(junctPtr, lnJunctListPtr)
register WallJunction         *junctPtr;
register LinkedWallJunction   **lnJunctListPtr;
{
    register LinkedWallJunction *lnJunctPtr;

    MALLOC(LinkedWallJunction *, lnJunctPtr, sizeof (LinkedWallJunction));
    lnJunctPtr->junctPtr = junctPtr;
    lnJunctPtr->nextPtr = (*lnJunctListPtr);
    (*lnJunctListPtr) = lnJunctPtr;
}



/* ARGSUSED */
static int
BuildHorzWall(vertTilePtr, clientData)
Tile        *vertTilePtr;
ClientData  clientData;
{
    MiscData    miscData;
    Rect        horzRect, vertRect;
    Rect        bnSearchArea;           /* bottleneck tile search area      */
    char        *s =                    "GEORGE CARVALHO WAS HERE 15 FEB 90";
    Tile        *bnTilePtr;             /* pointer to a bottleneck tile     */
    WallSegment *segPtr;
    int         oneFourth;

    if (Tp_IsDominantTile(vertTilePtr)) {
        TITORECT(vertTilePtr, &vertRect);
        GeoClip(&vertRect, vertPlaneBndPtr);
        Tp_VRectToHRect(&vertRect, &horzRect);
        oneFourth = (horzRect.r_xtop - horzRect.r_xbot) / 4;
        nwPoint.p_x = horzRect.r_xbot + oneFourth;
    /*
        centerPoint.p_x = nwPoint.p_x + oneFourth;
    */
        centerPoint.p_x = (horzRect.r_xtop + horzRect.r_xbot) / 2;
        sePoint.p_x = horzRect.r_xtop - oneFourth;

        SEJunctPtr = MakeWallJunct();
        globalFpgPtr->numJuncts++;

        FpgPutPointOfWallJunct(FPG_NW, &nwPoint, SEJunctPtr);
        FpgPutPointOfWallJunct(FPG_CENTER, &centerPoint, SEJunctPtr);
        FpgPutPointOfWallJunct(FPG_SE, &sePoint, SEJunctPtr);
        SEJunctPtr->wallSegPtr[FPG_NORTH] = (WallSegment *) horzDomTileIndex;
        SEJunctPtr->wallSegPtr[FPG_SOUTH] =
            (WallSegment *) Tp_GetJunctTableIndexOfDomTile(vertTilePtr);

        if (startJunctPtr == NIL(WallJunction)) {
            startJunctPtr = NWJunctPtr = SEJunctPtr;
        } else {

            /*
             * Right here we assume this segment will have a bottleneck tile,
             * and try to find it.  If it doesn't then it must have a
             * bottleneck line.
             */
            Tp_VRectToHRect(&domSearchArea, &horzRect);
            horzRect.r_xbot = (Fpg_GetPointOfWallJunct(FPG_CENTER,
                NWJunctPtr))->p_x;
            horzRect.r_xtop = (Fpg_GetPointOfWallJunct(FPG_CENTER,
                SEJunctPtr))->p_x;
            Tp_HRectToVRect(&horzRect, &bnSearchArea);
            if (TiSrArea(NIL(Tile), vertPlanePtr, &bnSearchArea,
                FindBnTile, (ClientData) &bnTilePtr) == 1 /* stop search */) {

                /* This segment has a bottleneck tile. */
                segPtr = MakeWallSeg();
                globalFpgPtr->numSegs++;
                FpgInsertWallSeg(NWJunctPtr, SEJunctPtr, FPG_HORIZONTAL,
                    segPtr);

                FpgPutBnTileOfWallSeg(bnTilePtr, segPtr);
                Tp_PutWallSegmentOfBnTile(segPtr, bnTilePtr);
            } else {
                miscData.bnSearchLine.point1.p_x = horzRect.r_xbot;
                miscData.bnSearchLine.point2.p_x = horzRect.r_xtop;
                miscData.bnSearchLine.point1.p_y = 
                    miscData.bnSearchLine.point2.p_y = 
                    (Fpg_GetPointOfWallJunct(FPG_CENTER, NWJunctPtr))->p_y;

                if (Tp_ItrBnLine(vertBnLineList, FindVertBnLine,
                    (ClientData) &miscData) == BEAR_STOP_SEARCH) {

                    /* This segment has a bottleneck line. */
                    segPtr = MakeWallSeg();
                    globalFpgPtr->numSegs++;
                    FpgInsertWallSeg(NWJunctPtr, SEJunctPtr, FPG_HORIZONTAL,
                        segPtr);

                    FpgPutBnLineOfWallSeg(miscData.bnLinePtr, segPtr);
                    Tp_PutWallSegmentOfBnLine(segPtr, miscData.bnLinePtr);
                } else {
/*
printf("westJunctionPoint:\n");
Mac_PrintPoint(Fpg_GetPointOfWallJunct(FPG_CENTER, NWJunctPtr));
printf("eastJunctionPoint:\n");
Mac_PrintPoint(Fpg_GetPointOfWallJunct(FPG_CENTER, SEJunctPtr));
*/
FpgPrintFatalErr("BuildHorzWall() failed: could not find a bn line, or bn tile on vertical plane");
                }
            }

            NWJunctPtr = SEJunctPtr;
        }
    }
    return (0); /* continue search */
}



static void
BuildJunctionTable()
{
    register LinkedWallJunction *lnJunctPtr;
    register WallJunction       *westJunctPtr;
    register WallSegment        *eastSegPtr;
    int                         r, c, rmax, cmax;

    rmax = globalFpgPtr->numHorzDomTiles;
    cmax = globalFpgPtr->numVertDomTiles;
    MALLOC(WallJunction ***, globalFpgPtr->junctTable, 
        rmax * sizeof (WallJunction **));
    for (r = 0; r < rmax ;r++) {
        MALLOC(WallJunction **, globalFpgPtr->junctTable[r], 
            cmax * sizeof (WallJunction *));
        for (c = 0; c < cmax ;c++) {
            globalFpgPtr->junctTable[r][c] = NIL(WallJunction);
        }
    }

    /*
     * Put all of the wall junctions into the table.
     */
    for (lnJunctPtr = horzWallList;
        lnJunctPtr != NIL(LinkedWallJunction) ;
        lnJunctPtr = lnJunctPtr->nextPtr) {

        westJunctPtr = lnJunctPtr->junctPtr;
        while ((eastSegPtr = Fpg_GetEastSegOfWallJunct(westJunctPtr)) !=
            NIL(WallSegment)) {

            horzDomTileIndex = (int) Fpg_GetNorthSegOfWallJunct(westJunctPtr);
            vertDomTileIndex = (int) Fpg_GetSouthSegOfWallJunct(westJunctPtr);
            westJunctPtr->wallSegPtr[FPG_NORTH] = NIL(WallSegment);
            westJunctPtr->wallSegPtr[FPG_SOUTH] = NIL(WallSegment);
            globalFpgPtr->junctTable[horzDomTileIndex][vertDomTileIndex] =
                westJunctPtr;

            westJunctPtr = Fpg_GetSEJunctOfWallSeg(eastSegPtr);
        }
        horzDomTileIndex = (int) Fpg_GetNorthSegOfWallJunct(westJunctPtr);
        vertDomTileIndex = (int) Fpg_GetSouthSegOfWallJunct(westJunctPtr);
        westJunctPtr->wallSegPtr[FPG_NORTH] = NIL(WallSegment);
        westJunctPtr->wallSegPtr[FPG_SOUTH] = NIL(WallSegment);
        globalFpgPtr->junctTable[horzDomTileIndex][vertDomTileIndex] =
            westJunctPtr;
    }
}



/* ARGSUSED */
static int
BuildVertWall(horzTilePtr, clientData)
Tile        *horzTilePtr;
ClientData  clientData;
{
    MiscData    miscData;
    Rect        bnSearchArea;           /* bottleneck tile search area      */
    Tile        *bnTilePtr;             /* pointer to a bottleneck tile     */
    WallSegment *segPtr;

    if (Tp_IsDominantTile(horzTilePtr)) {
        horzDomTileIndex = Tp_GetJunctTableIndexOfDomTile(horzTilePtr);
        SEJunctPtr =
            globalFpgPtr->junctTable[horzDomTileIndex][vertDomTileIndex];

        if (startJunctPtr == NIL(WallJunction)) {
            startJunctPtr = NWJunctPtr = SEJunctPtr;
        } else {
            /*
             * Right here we assume this segment will have a bottleneck tile,
             * and try to find it.  If it doesn't then it must have a
             * bottleneck line.
             */
            Mac_CopyRect(&bnSearchArea, &domSearchArea);
            bnSearchArea.r_ytop = (Fpg_GetPointOfWallJunct(FPG_CENTER,
                NWJunctPtr))->p_y;
            bnSearchArea.r_ybot = (Fpg_GetPointOfWallJunct(FPG_CENTER,
                SEJunctPtr))->p_y;
            if (TiSrArea(NIL(Tile), horzPlanePtr, &bnSearchArea,
                FindBnTile, (ClientData) &bnTilePtr) == 1 /* stop search */) {

                /* This segment has a bottleneck tile. */
                segPtr = MakeWallSeg();
                globalFpgPtr->numSegs++;
                FpgInsertWallSeg(NWJunctPtr, SEJunctPtr, FPG_VERTICAL, segPtr);

                FpgPutBnTileOfWallSeg(bnTilePtr, segPtr);
                Tp_PutWallSegmentOfBnTile(segPtr, bnTilePtr);
            } else {
                miscData.bnSearchLine.point1.p_x = 
                    miscData.bnSearchLine.point2.p_x = 
                    (Fpg_GetPointOfWallJunct(FPG_CENTER, NWJunctPtr))->p_x;
                miscData.bnSearchLine.point1.p_y = bnSearchArea.r_ybot;
                miscData.bnSearchLine.point2.p_y = bnSearchArea.r_ytop;

                if (Tp_ItrBnLine(horzBnLineList, FindHorzBnLine,
                    (ClientData) &miscData) == BEAR_STOP_SEARCH) {

                    /* This segment has a bottleneck line. */
                    segPtr = MakeWallSeg();
                    globalFpgPtr->numSegs++;
                    FpgInsertWallSeg(NWJunctPtr, SEJunctPtr, FPG_VERTICAL,
                        segPtr);

                    FpgPutBnLineOfWallSeg(miscData.bnLinePtr, segPtr);
                    Tp_PutWallSegmentOfBnLine(segPtr, miscData.bnLinePtr);
                } else {
/*
printf("northJunctionPoint:\n");
Mac_PrintPoint(Fpg_GetPointOfWallJunct(FPG_CENTER, NWJunctPtr));
printf("southJunctionPoint:\n");
Mac_PrintPoint(Fpg_GetPointOfWallJunct(FPG_CENTER, SEJunctPtr));
*/
FpgPrintFatalErr("BuildVertWall(): could not find a bn line, or bn tile on horizontal plane");
                }
            }
            NWJunctPtr = SEJunctPtr;
        }
    }
    return (0); /* continue search */
}



static int
FindBnTile(tilePtr, bnTilePtrPtr)
register Tile *tilePtr, **bnTilePtrPtr;
{
    if (Tp_IsBottleneckTile(tilePtr)) {
        *bnTilePtrPtr = tilePtr;
        return (1); /* terminate search */
    } else {
        return (0); /* continue search */
    }
    /* NOTREACHED */
}



static BearStatus
FindHorzBnLine(horzBnLinePtr, miscDataPtr)
BNLine      *horzBnLinePtr;
MiscData    *miscDataPtr;
{
    if (HLineCrossesVLine(Tp_GetLineOfBnLine(horzBnLinePtr),
        &miscDataPtr->bnSearchLine) == BEAR_TRUE) {
        miscDataPtr->bnLinePtr = horzBnLinePtr;
        return (BEAR_STOP_SEARCH);
    } else {
        return (BEAR_CONTINUE_SEARCH);
    }
    /* NOTREACHED */
}



static BearStatus
FindVertBnLine(vertBnLinePtr, miscDataPtr)
BNLine      *vertBnLinePtr;
MiscData    *miscDataPtr;
{
    if (HLineCrossesVLine(&miscDataPtr->bnSearchLine,
        Tp_GetLineOfBnLine(vertBnLinePtr)) == BEAR_TRUE) {
        miscDataPtr->bnLinePtr = vertBnLinePtr;
        return (BEAR_STOP_SEARCH);
    } else {
        return (BEAR_CONTINUE_SEARCH);
    }
    /* NOTREACHED */
}



/* ARGSUSED */
static int
FindHorzDomTile(horzTilePtr, clientData)
Tile        *horzTilePtr;
ClientData  clientData;
{
    Rect    horzRect;
    int     oneFourth;

    if (Tp_IsDominantTile(horzTilePtr)) {
        TITORECT(horzTilePtr, &horzRect);
        GeoClip(&horzRect, horzPlaneBndPtr);
        oneFourth = (horzRect.r_ytop - horzRect.r_ybot) / 4;
        nwPoint.p_y = horzRect.r_ytop - oneFourth;
    /*
        centerPoint.p_y = nwPoint.p_y - oneFourth;
    */
        centerPoint.p_y = (horzRect.r_ytop + horzRect.r_ybot) / 2;
        sePoint.p_y = horzRect.r_ybot + oneFourth;

        Tp_HRectToVRect(&horzRect, &domSearchArea);
        startJunctPtr = NIL(WallJunction);
        Tp_PutJunctTableIndexOfDomTile(domTileCount, horzTilePtr);
        horzDomTileIndex = domTileCount;
        if (TiSrArea(NIL(Tile), vertPlanePtr, &domSearchArea,
            BuildHorzWall, (ClientData) NULL) == 1 /* stop search */) {
            return (1); /* stop search */
        }
        /* Increment 'domTileCount' after you make the horizontal wall. */
        domTileCount++;

        AddWallJunctToList(startJunctPtr, &horzWallList);
    }
    return (0); /* continue search */
}



/* ARGSUSED */
static int
FindVertDomTile(vertTilePtr, clientData)
Tile        *vertTilePtr;
ClientData  clientData;
{
    Rect vertRect;

    if (Tp_IsDominantTile(vertTilePtr)) {
        TITORECT(vertTilePtr, &vertRect);
        GeoClip(&vertRect, vertPlaneBndPtr);
        Tp_VRectToHRect(&vertRect, &domSearchArea);
        startJunctPtr = NIL(WallJunction);
        vertDomTileIndex = Tp_GetJunctTableIndexOfDomTile(vertTilePtr);
        if (TiSrArea(NIL(Tile), horzPlanePtr, &domSearchArea,
            BuildVertWall, (ClientData) NULL) == 1 /* stop search */) {
            return (1); /* stop search */
        }
    }
    return (0); /* continue search */
}



/*
 * ----------------------------------------------------------------------------
 *
 * FreeHorzWallList() --
 *
 * For this to work the memory for the horizontal wall junction list
 * must have been allocated one element at a time !!
 *
 * Frees the memory used to make the linked list, but not the wall
 * junctions the list points to.
 *
 * Results:
 *   none
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

static void
FreeHorzWallList()
{
    register LinkedWallJunction *lnJunctPtr;

    while ((lnJunctPtr = horzWallList) != NIL(LinkedWallJunction)) {
        horzWallList = lnJunctPtr->nextPtr;
        FREE((char *) lnJunctPtr);
    }
}



static BearStatus
HLineCrossesVLine(horzLinePtr, vertLinePtr)
Line *horzLinePtr, *vertLinePtr;
/*
 * Note that the end points of each line may be in any order.
 *
 * Results:
 *   returns BEAR_TRUE if the two lines cross.
 *   returns BEAR_FALSE if the two lines do not cross.
 *
 *   returns BEAR_ERROR if 'horzLinePtr' is not a horizontal line or
 *   'vertLinePtr' is not a vertical line.
 */
{
    int horzX1, horzX2, horzY;
    int vertX, vertY1, vertY2;

    /*
     * if 'horzLinePtr' is not a horizontal line or 'vertLinePtr' is
     * not a vertical line return BEAR_ERROR.
     */
    if (((horzY = horzLinePtr->point1.p_y) != horzLinePtr->point2.p_y) ||
        ((vertX = vertLinePtr->point1.p_x) != vertLinePtr->point2.p_x)) {
        return (BEAR_ERROR);
    }

    horzX1 = horzLinePtr->point1.p_x;
    vertY1 = vertLinePtr->point1.p_y;
    horzX2 = horzLinePtr->point2.p_x;
    vertY2 = vertLinePtr->point2.p_y;
    return ((((horzX1 <= vertX && horzX2 >= vertX) ||
        (horzX2 <= vertX && horzX1 >= vertX)) &&

        ((vertY1 <= horzY && vertY2 >= horzY) ||
        (vertY2 <= horzY && vertY1 >= horzY))) ? BEAR_TRUE : BEAR_FALSE);
}



/* ARGSUSED */
static int
IndexVertDomTile(vertTilePtr, clientData)
Tile        *vertTilePtr;
ClientData  clientData;
{
    if (Tp_IsDominantTile(vertTilePtr)) {
        Tp_PutJunctTableIndexOfDomTile(domTileCount++, vertTilePtr);
    }
    return (0); /* continue search */
}



/* 
 * ----------------------------------------------------------------------------
 *
 * MakeFloorPlanGraph() --
 *
 *    Allocates memory for a new floor plan graph, and initializes its fields.
 *
 * Results:
 *    A pointer to a floor plan graph.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static FloorPlanGraph *
MakeFloorPlanGraph()
{
    register FloorPlanGraph *fpgPtr;

    MALLOC(FloorPlanGraph *, fpgPtr, sizeof (FloorPlanGraph));
    fpgPtr->id = 0;
    fpgPtr->junctTable = (WallJunction ***) NULL;
    fpgPtr->numHorzDomTiles = 0;
    fpgPtr->numVertDomTiles = 0;
    fpgPtr->numJuncts = 0;
    fpgPtr->numSegs = 0;
    fpgPtr->ulJunctPtr = NIL(WallJunction);
    fpgPtr->urJunctPtr = NIL(WallJunction);
    fpgPtr->lrJunctPtr = NIL(WallJunction);
    fpgPtr->llJunctPtr = NIL(WallJunction);
    fpgPtr->mainRoomPtr = NIL(Fpg_Room);
    fpgPtr->roomList = (lsList) NULL;
    fpgPtr->clientData = (ClientData) NULL;

    return (fpgPtr);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * MakeWallJunct() --
 *
 *    Allocates memory for a new wall junction and initializes its fields.
 *
 * Results:
 *    A pointer to a wall junction.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static WallJunction *
MakeWallJunct()
{
    register WallJunction *junctPtr;

    MALLOC(WallJunction *, junctPtr, sizeof (WallJunction));
    junctPtr->itrMark = BEAR_UNMARKED;
    junctPtr->mark = BEAR_UNMARKED;
    junctPtr->degree = 0;
    junctPtr->wallSegPtr[FPG_NORTH] = NIL(WallSegment);
    junctPtr->wallSegPtr[FPG_EAST] = NIL(WallSegment);
    junctPtr->wallSegPtr[FPG_SOUTH] = NIL(WallSegment);
    junctPtr->wallSegPtr[FPG_WEST] = NIL(WallSegment);
    junctPtr->roomPtr = NIL(Fpg_Room);
    junctPtr->greModulePtr = (BearGeneric) NULL;
    junctPtr->ngrModulePtr = (BearGeneric) NULL;
    junctPtr->nplModulePtr = (BearGeneric) NULL;
    junctPtr->clientData = (ClientData) NULL;

    return (junctPtr);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * MakeWallSeg() --
 *
 *    Allocates memory for a new wall segment, and initializes its fields.
 *
 * Results:
 *    A pointer to a wall segment.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static WallSegment *
MakeWallSeg()
{
    register WallSegment *segPtr;

    MALLOC(WallSegment *, segPtr, sizeof (WallSegment));
    segPtr->mark = BEAR_UNMARKED;
    MALLOC(BottleNeck *, segPtr->bnPtr, sizeof (BottleNeck));
    segPtr->NWJunctPtr = NIL(WallJunction);
    segPtr->SEJunctPtr = NIL(WallJunction);
    segPtr->ngrModulePtr = (BearGeneric) NULL;
    segPtr->clientData = (ClientData) NULL;

    return (segPtr);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Fpg_CreateFloorPlanGraph() --
 *
 * In the process of building the floor plan graph the dominant tiles
 * on each plane are assigned an index into 'globalFpgPtr->junctTable'.
 * 
 * August 7, 1989 : Code has been added so that all of the memory
 * allocated to build a floor plan graph is now freed before a new
 * floor plan graph is built.
 *
 * IMPORTANT:
 *   Tp_IdentifySpaceTiles() must be called before yippie yi aye this
 *   procedure or bad things will happen.
 *
 *   The algorithum used to build the floor plan graph relies entirely on
 *   the fact that TiSrArea() searches right to left, top to bottom.
 *
 * Results:
 *   BEAR_OK is returned if everything is spiffy, BEAR_ERROR otherwise.
 *   Also if 'llJunctPtrPtr' is non-zero the lower left wall junction of
 *   the floor plan graph will be returned.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Fpg_CreateFloorPlanGraph(planeDataPtr, llJunctPtrPtr)
PlaneData       *planeDataPtr;
WallJunction    **llJunctPtrPtr;        /* The lower left wall junction of the
                                         * floor plan graph.
                                         */
{
    if (llJunctPtrPtr != (WallJunction **) NULL) {
        *llJunctPtrPtr = NIL(WallJunction);
    }

    Fpg_DeleteFloorPlanGraph(planeDataPtr);
    globalFpgPtr = MakeFloorPlanGraph();
    globalFpgPtr->id = Mac_GenerateId();
    Tp_PutFpg(planeDataPtr, globalFpgPtr);

    horzPlanePtr = Tp_GetHorzPlane(planeDataPtr);
    vertPlanePtr = Tp_GetVertPlane(planeDataPtr);
    horzPlaneBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    vertPlaneBndPtr = Tp_GetVertPlaneBnd(planeDataPtr);
    horzBnLineList = Tp_GetHorzBnLineList(planeDataPtr);
    vertBnLineList = Tp_GetVertBnLineList(planeDataPtr);

    domTileCount = 0;
    (void) TiSrArea(NIL(Tile), vertPlanePtr, vertPlaneBndPtr,
        IndexVertDomTile, (ClientData) NULL);
    if (domTileCount == 0) {
        return (BEAR_OK);
    }
    globalFpgPtr->numVertDomTiles = domTileCount;

    /*
     * Locate each horizontal dominant tile, and then build up the
     * data structure cooresponding to the horizontal wall on
     * the floor plan graph which it represents.
     */
    domTileCount = 0;
    horzWallList = NIL(LinkedWallJunction);
    if (TiSrArea(NIL(Tile), horzPlanePtr, horzPlaneBndPtr, 
        FindHorzDomTile, (ClientData) NULL) == 1 /* stop search */) {
        return (BEAR_ERROR);
    }
    globalFpgPtr->numHorzDomTiles = domTileCount;

    BuildJunctionTable();

    /*
     * Locate each vertical dominant tile, and then build up the
     * data structure cooresponding to the vertical wall on
     * the floor plan graph which it represents, attaching 
     * the horziontal walls as you go along.
     */
    if (TiSrArea(NIL(Tile), vertPlanePtr, vertPlaneBndPtr,
        FindVertDomTile, (ClientData) NULL) == 1 /* stop search */) {
        return (BEAR_ERROR);
    }

    globalFpgPtr->llJunctPtr = horzWallList->junctPtr;
    if (llJunctPtrPtr != (WallJunction **) NULL) {
        *llJunctPtrPtr = globalFpgPtr->llJunctPtr;
    }

    FreeHorzWallList();
    
    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Fpg_DeleteFloorPlanGraph() --
 *
 * For this to work all the memory associated with the floor plan graph
 * must have been allocated one element at a time !!
 *
 * After this is called the pointer to the wall segment contained
 * in the bottleneck tiles (and lines) must never be referenced again.
 *
 * Results:
 *   none
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

void
Fpg_DeleteFloorPlanGraph(planeDataPtr)
PlaneData *planeDataPtr;
{
    FloorPlanGraph  *fpgPtr;
    WallJunction    ***junctTable, **junctArray, *junctPtr;
    WallSegment     *eastSegPtr, *southSegPtr;
    register int    r, c, rmax, cmax;


    if ((fpgPtr = Tp_GetFpg(planeDataPtr)) == NIL(FloorPlanGraph)) {
        return;
    }

    if (fpgPtr->junctTable != (WallJunction ***) NULL) {
        /*
         * Free all the memory associated with this floor plan's
         * junction table, including the junctions and segments.
         */

        rmax = fpgPtr->numHorzDomTiles;
        cmax = fpgPtr->numVertDomTiles;
        junctTable = fpgPtr->junctTable;
        for (r = 0; r < rmax ;r++) {
            junctArray = junctTable[r];
            for (c = 0; c < cmax ;c++) {
                junctPtr = junctArray[c];
                if (junctPtr != NIL(WallJunction)) {
                    if ((eastSegPtr = Fpg_GetEastSegOfWallJunct(junctPtr)) !=
                        NIL(WallSegment)) {
                        FREE((char *) eastSegPtr);
                    }
                    if ((southSegPtr = Fpg_GetSouthSegOfWallJunct(junctPtr)) !=
                        NIL(WallSegment)) {
                        FREE((char *) southSegPtr);
                    }
                    FREE((char *) junctPtr);
                }
            }
            FREE((char *) junctArray);
        }
        FREE((char *) junctTable);
    }

    FpgFreeRoom((lsGeneric) fpgPtr->mainRoomPtr);
    if (fpgPtr->roomList != (lsList) NULL) {
        lsDestroy(fpgPtr->roomList, FpgFreeRoom);
    }

    FREE((char *) fpgPtr);

    Tp_PutFpg(planeDataPtr, NIL(FloorPlanGraph));
}



BearStatus
Fpg_FindWallSeg(junct1Ptr, junct2Ptr, segPtrPtr)
register WallJunction   *junct1Ptr, *junct2Ptr;
WallSegment             **segPtrPtr;
/*
 * Looks for a wall segment between 'junct1Ptr' and 'junct2Ptr'.  If it
 * exists BEAR_OK is returned and '*segPtrPtr' is set equal to the wall
 * segment, otherwise BEAR_ERROR is returned and '*segPtrPtr' will be
 * NULL.
 */
{
    if (((*segPtrPtr = Fpg_GetNorthSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetNWJunctOfWallSeg(*segPtrPtr) == junct2Ptr)) {

        return (BEAR_OK);
    }
    else if (((*segPtrPtr = Fpg_GetEastSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetSEJunctOfWallSeg(*segPtrPtr) == junct2Ptr)) {

        return (BEAR_OK);
    }
    else if (((*segPtrPtr = Fpg_GetSouthSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetSEJunctOfWallSeg(*segPtrPtr) == junct2Ptr)) {
         
        return (BEAR_OK);
    }
    else if (((*segPtrPtr = Fpg_GetWestSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetNWJunctOfWallSeg(*segPtrPtr) == junct2Ptr)) {

        return (BEAR_OK);
    }

    *segPtrPtr = NIL(WallSegment);
    return (BEAR_ERROR);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * Fpg_GetAbsPosOfPinOrJunct() --
 *
 * Figure out the coordinates of the pin or junctions absolute position
 * on the chip.
 * 
 * IMPORTANT : The pointer to the point structure must point
 *     to a piece of memory.
 *
 *     PlaneData    *planeDataPtr;
 *     PinOrJunct   *pinOrJunctPtr;
 *     Point        absPos;
 *
 *     Fpg_GetAbsPosOfPinOrJunct(planeDataPtr, pinOrJunctPtr, &absPos);
 *
 * Results:
 *   BEAR_OK is returned if everything is spiffy, BEAR_ERROR otherwise.
 *   Also the absolute postion of the pin or junction is returned.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Fpg_GetAbsPosOfPinOrJunct(planeDataPtr, pinOrJunctPtr, absPosPtr)
PlaneData   *planeDataPtr;                      /* IN       */
PinOrJunct  *pinOrJunctPtr;                     /* IN       */
Point       *absPosPtr;                         /* OUT      */
{
    term        *bblTermPtr;
    Point       *junctPointPtr;
    PseudoJunct *pjunctPtr;

    switch (pinOrJunctPtr->unType) {
    case (int) FPG_PIN:
        if (Mac_IsPinPosDef(pinOrJunctPtr->unValue.pinPtr) == FALSE) {
            return (BEAR_ERROR);
        }
        Mac_GetAbsPosOfPin(pinOrJunctPtr->unValue.pinPtr, absPosPtr);
        break;
    case (int) FPG_PSEUDO_PIN:
        if (Tp_GetAbsPosOfPPin(planeDataPtr, pinOrJunctPtr->unValue.ppinPtr,
            absPosPtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
        break;
    case (int) FPG_ROUTE_PIN:
        Mac_GetAbsPosOfRPin(pinOrJunctPtr->unValue.rPinPtr, absPosPtr);
        break;
    case (int) FPG_WALL_JUNCT:
        junctPointPtr = Fpg_GetPointOfWallJunct((int) FPG_CENTER,
            pinOrJunctPtr->unValue.junctPtr);
        absPosPtr->p_x = junctPointPtr->p_x;
        absPosPtr->p_y = junctPointPtr->p_y;
        break;
    case (int) FPG_PSEUDO_JCT:
        pjunctPtr = pinOrJunctPtr->unValue.pjunctPtr;
        absPosPtr->p_x = pjunctPtr->absPos.p_x;
        absPosPtr->p_y = pjunctPtr->absPos.p_y;
        break;
    case (int) FPG_BBL_TERM:
        bblTermPtr = pinOrJunctPtr->unValue.bblTermPtr;
        absPosPtr->p_x = bblTermPtr->mp->geop->gtp[bblTermPtr->tnum].loc.xy[0];
        absPosPtr->p_y = bblTermPtr->mp->geop->gtp[bblTermPtr->tnum].loc.xy[1];
        break;
    case (int) FPG_BBL_JUNCT:
        Mac_GetAbsPosOfRJun(pinOrJunctPtr->unValue.bblJunctPtr, absPosPtr);
        break;
    default:
Bear_PrintErr("Fpg_GetAbsPosOfPinOrJunct() : unexpected pin or junction type");
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}



void
FpgPrintFatalErr(message)
BearString message;
{
    errRaise(FPG_MOD_NAME, ((int) -1), message);
}


