/*
 *      grAfterPull.c --
 *
 *  Malgorzata Marek-Sadowska
 *    (Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint




#include "grInt.h"


/*
*--------------------------------------------------------------------------
*   Gr_PullAssignment(kPtr)
*   This function pulls out nets from dead regions.
*   Nets are not pushed downward yet.
*   Results      : returns 1 if therewas any pull-action.
*                  returns 0 if nothing happened.
*   Side effect  : virtual nets
*              are created.
*--------------------------------------------------------------------------
*/

int 
Gr_PullAssignment(kPtr)
grRegion *kPtr;         /* pointer to the parent region  */
{
    grDeadRegion *current;
    grSubNet     *sNet;
    grVirtNet    *vNet;

    Gr_MkePullVirtualNets(kPtr);


    if (m==0)
    {
        return (0);
    }

    if (gr_Timing ==1)
    {
        sNet=kPtr->subNetsPtr;

        while (sNet!=NULL)
        {
        if ((sNet->active==5)||(sNet->active==9)||(sNet->active==10))
        {

            /*subtract approx length*/
            ((gr_HeadGrNets+sNet->netname)->currentLength) -=
            sNet->approxLength;
            ((gr_HeadGrNets+sNet->netname)->xapprox) -=
            sNet->xapprox;
            ((gr_HeadGrNets+sNet->netname)->yapprox) -=
            sNet->yapprox;
            /* end subtraction */
        }
            sNet=sNet->nextOnList;
        } /* end while*/
    }
    current=grHeadDeadRegion;
    while (current!=NULL)
    {

        /* PRINTOUT 
        if ((kPtr->xcutvirt == 33)&&(kPtr->ycutvirt == 44))
        {
        printf("\n AAA");
        Gr_PrintSubNets(kPtr);
        fflush(stdout);
        }
        */

        sNet=kPtr->subNetsPtr;
        while (sNet!=NULL)
        {
            vNet=sNet->children;
            if (vNet!=NULL)
            {
                if ((sNet->active==5)
                ||(sNet->active==9)||(sNet->active==10))
                {
                    vNet->hole=0;
                    Gr_PullVirtNetFromDeadRegion
                    (vNet,current,kPtr->dir);
                }
            }
            sNet=sNet->nextOnList;
        }
        current->pullControl=1;
        Gr_CollectDataInDeadRegion(kPtr,current);
        current=current->dr_inline;
    }
    return (1);
}



/*
*-------------------------------------------------------------------------
*   Results      : none   .                                            
*   Side effects :                                                    
*   If there was a hole in the cut line then this function             
*   puts all the data decided during the assignment process into the   
*   proper positions in the data structures. Action depends on net     
*   activity.                                                          
*-------------------------------------------------------------------------
*/

void Gr_CollectDataInDeadRegion(kPtr,deadRegion)

grRegion     *kPtr;         /* pointer to the parent region*/
grDeadRegion *deadRegion;       /* pointer to dead region */


{

    grSubNet *sNet ;
    int dir;
    int d1,d2,cnt,i;
    int pom1,pom2;
    grPrimit *xwhois,*ywhois,*xwhere,*ywhere;
    int holeNr;
    grVirtNet *vNet;
    grHole *gHole;
    float total;

    dir=kPtr->dir;
    /* get position of the cut */

    sNet=kPtr->subNetsPtr;

    while (sNet!=NULL)
    {
        if ((sNet->active==5)||(sNet->active==9)||(sNet->active==10))
        {

            if (gr_Timing ==1)
            {

                /* clear holes  */
                for (i=deadRegion->botLeftHoleNr;
                i<=deadRegion->topRightHoleNr;i++)
                {
                    gHole= (grHeadHolePtr+i);
                    gHole->leftEst=0;
                    gHole->rightEst=0;
                    gHole->lbpins=0;
                    gHole->rtpins=0;
                }
            }

            vNet=sNet->children;

            if (vNet!=NULL)
            {
                holeNr=vNet->hole;
                /* if (holeNr > 0) */
                {
                    Gr_UpdateBoundingBox(
                    vNet->leftNode->contentsPtr);
                    Gr_UpdateBoundingBox(
                    vNet->rightNode->contentsPtr);
                    d1=vNet->leftNode->contentsPtr->count;
                    d2=vNet->rightNode->contentsPtr->count;
                    cnt=d1+d2;
                    MALLOC(grPrimit*,xwhois,
                    sizeof (grPrimit)*(cnt+1));
                    MALLOC(grPrimit*,ywhois,
                    sizeof (grPrimit)*(cnt+1));
                    MALLOC(grPrimit*,xwhere,
                    sizeof (grPrimit)*(cnt+1));
                    MALLOC(grPrimit*,ywhere,
                    sizeof (grPrimit)*(cnt+1));
                    for (i=1;i<=cnt;i++)
                    {
                        (xwhois+i)->number=0;
                        (ywhois+i)->number=0;
                        (xwhere+i)->number=0;
                        (ywhere+i)->number=0;
                    }

                    Gr_DetermineWhichPinsAreInBtn(
                    vNet,xwhois,xwhere,0,dir,
                    deadRegion->botLeftHoleNr,
                    deadRegion->topRightHoleNr);
                    Gr_DetermineWhichPinsAreInBtn(
                    vNet,ywhois,ywhere,1,dir,
                    deadRegion->botLeftHoleNr,
                    deadRegion->topRightHoleNr);

                /*
                if (vNet->netName == 80)
                {
                    for (i=1;i<=cnt;i++)
                    {
        printf("\n d1=%d d2=%d",d1,d2);
    printf("\n xwhois(%4d)=%4d",i,(xwhois+i)->number);
    printf(" xwhere(%4d)=%4d",i,(xwhere+i)->number);
    printf(" ywhois(%4d)=%4d",i,(ywhois+i)->number);
    printf(" ywhere(%4d)=%4d",i,(ywhere+i)->number);
                    }
                fflush(stdout);
                }
                */
                    Gr_PullOutPinsFromBtnForAssignment(
                    kPtr,deadRegion,
                    sNet,vNet,dir,
                    xwhois,ywhois,xwhere,ywhere,d1,d2);

                    Gr_UpdateBoundingBox((vNet->
                    leftNode->contentsPtr));
                    Gr_UpdateBoundingBox((vNet->
                    rightNode->contentsPtr));

                    GR_FREE((char*) xwhere);
                    GR_FREE((char*) ywhere);
                    GR_FREE((char*) xwhois);
                    GR_FREE((char*) ywhois);

                    if (gr_Timing  > 0)
                    {
                        pom1=0;
                        pom2=0;
                        for (i=deadRegion->botLeftHoleNr;
                            i<=deadRegion->topRightHoleNr;i++) {

                            gHole = (grHeadHolePtr+i);
                            if (gHole->status ==0) {
                                pom1 += (gHole->leftEst);
                                pom1 += (gHole->rightEst);

                                if ((gHole->lbpins)* (gHole->rtpins)>0) {
                                    pom2 += (gHole->height);
                                }
                                else {
                                    if ((gHole->lbpins + gHole->rtpins)>0) {
                                        pom2 += (gHole->height)/2;
                                    }
                                }
                            }
                        }
                        if (dir == 1) {
                            ((gr_HeadGrNets+sNet->netname)->xfixed) += pom1;
                            ((gr_HeadGrNets+sNet->netname)->yfixed) += pom2;
                            total = pom1+pom2*gr_HorVertWire;
                            ((gr_HeadGrNets+sNet->netname)->currentLength) += 
                                total;
                        }
                        if (dir == 2) {
                            ((gr_HeadGrNets+sNet->netname)->xfixed) += pom2;
                            ((gr_HeadGrNets+sNet->netname)->yfixed) += pom1;
                            total=pom2+pom1*gr_HorVertWire;
                            ((gr_HeadGrNets+sNet->netname)->currentLength) += 
                                total;
                        }

                    }
                }
            }
        }
    sNet=sNet->nextOnList;
    }
}



/*
*---------------------------------------------------------------------------
*   Gr_PullVirtNetFromDeadRegion
*   For a virtual net it checks if it needs to be pulled
*   out from a dead region.
*   Results     : none;
*   Side effects: if a net needs to be pulled out, then hole
*   assignment is decided for the pull-out.
*---------------------------------------------------------------------------
*/


void Gr_PullVirtNetFromDeadRegion(vNet,deadRegion,dir)

grVirtNet    *vNet;             /* pointer to a virtual net */
grDeadRegion *deadRegion;       /* pointer to a dead region */
int           dir;          /* direction of a cut */



{

    int side;
    grPinRectangle *pRect,*sRect;
    int bLHole,rTHole,hNumber;

    side=deadRegion->side;

    /*
    if (vNet->netName == 80)
    {
        Gr_PrDeadRegion(deadRegion);
        fflush(stdout);
    }
    */

    if (side==0)
    {
        /* left side */
        pRect=vNet->leftNode->contentsPtr;
        sRect=vNet->rightNode->contentsPtr;
        if (pRect->firstxPin==NULL)
        {
            return;
        }
    }
    else
    {
        pRect=vNet->rightNode->contentsPtr;
        sRect=vNet->leftNode->contentsPtr;
        if (pRect->firstxPin==NULL)
        {
            return;
        }
    }

    if (Gr_ContinuePullFromDeadRegion(pRect,sRect,deadRegion,dir)==0)
    {
        return;
    }

    bLHole=deadRegion->botLeftHoleNr;
    rTHole=deadRegion->topRightHoleNr;

    if (bLHole==rTHole)
    {
        vNet->hole = bLHole;
        return;
    }
    hNumber=Gr_CanNetBeForcedToResidentHole(vNet,dir,bLHole,rTHole);

    if ((hNumber > 0)&&(hNumber >=bLHole)&&(hNumber<=rTHole))
    {
        vNet->hole=hNumber;
        return ;
    }

    if (dir==1)
    {
        if (sRect->firstxPin!=NULL)
        {
            if (sRect->maxy > pRect->maxy)
            {
                hNumber=Gr_GetLastHoleBelow(sRect->maxy);
            }
            else
            {
                hNumber=Gr_GetLastHoleBelow(pRect->maxy);
            }

        }
        else
        {
            hNumber=Gr_GetLastHoleBelow(pRect->maxy);
        }
    }
    else
    {
        if (sRect->firstxPin!=NULL)
        {
            if (sRect->maxx > pRect->maxx)
            {
                hNumber=Gr_GetLastHoleLeft(sRect->maxx);
            }
            else
            {
                hNumber=Gr_GetLastHoleLeft(pRect->maxx);
            }

        }
        else
    
        {
            hNumber=Gr_GetLastHoleLeft(pRect->maxx);
        }
    }

    if ((hNumber >= bLHole)&&(hNumber <=rTHole))
    {
        vNet->hole = hNumber;
        return;
    }
    if (hNumber > rTHole)
    {
        vNet->hole=rTHole;
        return;
    }
    if (hNumber < bLHole)
    {
        vNet->hole=bLHole;
        return;
    }

}


/*
*-----------------------------------------------------------------------------
*   Gr_ContinuePullFromDeadRegion
*   Side effects : none.
*   Results      :if pinRectangle still needs a pull,1 is returned;
*           otherwise 0.
*   Pull is needed when pinRectangle has at least one pin
*   inside deadRegion and at least one pin is in a region
*   from which there was no pull out yet.
*-----------------------------------------------------------------------------
*/



int Gr_ContinuePullFromDeadRegion(pinRectanglePtr,sRect,deadRegionPtr,dir)

grPinRectangle  *pinRectanglePtr,*sRect;
                        /*  pinRectanglePtr points
                         to the pins which sit on the
                             side "side" of deadRegion ,
                         sRect sits on the other side*/
grDeadRegion    *deadRegionPtr;         /* pointer to deadRegion
                           from where is the pull
                           out performed.*/
int              dir;               /* direction of a cut*/

{

    grDeadRegion *current;

    if (Gr_PinRectangleInDeadRegion(pinRectanglePtr,deadRegionPtr,dir)
    == 0 )
    {
        return (0);  /* no pull needed */
    }

    current=grHeadDeadRegion;

    while (current!=NULL)
    {
        if ((current!=deadRegionPtr)&&(current->pullControl==0))
        {
            if (Gr_PinRectangleInDeadRegion(pinRectanglePtr,
            current,dir)==1)
            {
                return (1);
            }
        }
        current=current->next;
    }
    current=grHeadDeadRegion;

    while (current!=NULL)
    {
        if ((current!=deadRegionPtr)&&(current->pullControl==0))
        {
            if (Gr_PinRectangleInDeadRegion(sRect,
            current,dir)==1)
            {
                return (1);
            }
        }
        current=current->next;
    }
    return (0);
}

