/*
 * grCreate.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *	  (Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint


/*
*--------------------------------------------------------------------------
*	This file contains functions allocating memory
*	for various structures. Memory is handed in an
*	initialized form.
*--------------------------------------------------------------------------
*/



#include "grInt.h"


/*
*-------------------------------------------------------------------------
*	Gr_CreateRegionForDeadRegion
*	Side effects: none.
*	Results     : allocated and filld-in memory fields of grRegion for
*	deadRegionPtr.
*-------------------------------------------------------------------------
*/


grRegion *Gr_CreateRegionForDeadRegion(deadRegionPtr)

grDeadRegion *deadRegionPtr;


{
	grRegion *new;

	MALLOC(grRegion*,new,sizeof (grRegion));

	new->xvirt=Gr_GetXVirtPosOfAbsPos(deadRegionPtr->xleft);
	new->yvirt=Gr_GetYVirtPosOfAbsPos(deadRegionPtr->ybot);
	new->whvirt=Gr_GetXVirtPosOfAbsPos(deadRegionPtr->xright)-
			new->xvirt;
	new->hvirt=Gr_GetYVirtPosOfAbsPos(deadRegionPtr->ytop)-
			new->yvirt;
	new->xcutvirt=0;
	new->ycutvirt=0;
	new->xpos=deadRegionPtr->xleft;
	new->ypos=deadRegionPtr->ybot;
	new->width=deadRegionPtr->xright-new->xpos;
	new->height=deadRegionPtr->ytop-new->ypos;
	new->dir=0;
	new->subNetsPtr=NULL;
	new->firstPinRectanglePtr=NULL;
	new->lastPinRectanglePtr=NULL;
	new->leftPtr=NULL;
	new->rightPtr=NULL;
	new->childrenPtr=NULL;
	new->siblingPtr=NULL;
	new->nextPtr=NULL;
	deadRegionPtr->region=new;

	return (new);
}


/*
*--------------------------------------------------------------------------
*	Gr_InitializeNewRegions
*	After the cut position and direction have been obtained, the sizes    
*	of the resulting new regions are determined.                          
*	As a side effect of this function children regions of the region      
*	pointed to by kPtr  are initialized                                   
*	Memory and geometrical dimensions are set                             
*	If par == 0 then left and right subregions are created.
*	If par == 1 then children subregions are created.
*-------------------------------------------------------------------------
*/



void Gr_InitializeNewRegions(kPtr,par)

grRegion *kPtr;
int       par ;


{

	int xv,yv,dir,lx,ly,lh,lw,rx,ry,rh,rw;
	grDeadRegion *dead;
	grRegion     *new;


	dir=kPtr->dir;
	xv= (kPtr->xcutvirt);
	yv= (kPtr->ycutvirt);


	if (dir==0)
	{
		(kPtr->leftPtr)=NULL;
		(kPtr->rightPtr)=NULL;
		(kPtr->childrenPtr)=NULL;
		return ;
	}


	if ( par == 0)
	{
	if (dir==1)
	{
		lx=(kPtr->xvirt);
		ly=(kPtr->yvirt);
		lh=(kPtr->hvirt);
		lw=xv-lx;
		rx=xv;
		ry=ly;
		rh=lh;
		rw=(kPtr->whvirt)-lw;
	}
	if (dir==2)
	{
		rx=(kPtr->xvirt);
		ry=(kPtr->yvirt);
		rh=yv-ry;
		rw=(kPtr->whvirt);
		lx=rx;
		ly=yv;
		lh=(kPtr->hvirt)-rh;
		lw=rw;
	}
	if (dir>0)
	{

		/* Initialize new children regions  */

		MALLOC(grRegion*,kPtr->leftPtr,sizeof (grRegion));
		MALLOC(grRegion*,kPtr->rightPtr,sizeof (grRegion));
		(kPtr->leftPtr->xvirt)=lx;
		(kPtr->rightPtr->xvirt)=rx;
		(kPtr->leftPtr->yvirt)=ly;
		(kPtr->rightPtr->yvirt)=ry;
		(kPtr->leftPtr->whvirt)=lw;
		(kPtr->rightPtr->whvirt)=rw;
		(kPtr->leftPtr->hvirt)=lh;
		(kPtr->rightPtr->hvirt)=rh;
		(kPtr->leftPtr->subNetsPtr)=NULL;
		(kPtr->rightPtr->subNetsPtr)=NULL;
                (kPtr->rightPtr->firstPinRectanglePtr) = NULL;
                (kPtr->leftPtr->firstPinRectanglePtr) = NULL;
                (kPtr->leftPtr->dir)=0;
                (kPtr->rightPtr->dir)=0;
                (kPtr->leftPtr->leftPtr)=NULL;
                (kPtr->leftPtr->childrenPtr)=NULL;
                (kPtr->leftPtr->siblingPtr)=NULL;
                (kPtr->leftPtr->rightPtr)=NULL;
                (kPtr->rightPtr->leftPtr)=NULL;
                (kPtr->rightPtr->childrenPtr)=NULL;
                (kPtr->rightPtr->siblingPtr)=NULL;
                (kPtr->rightPtr->rightPtr)=NULL;
		grTailRegionPtr=kPtr->rightPtr;
	}
	}
	else
	{
		dead=grHeadDeadRegion;
		
		while (dead!=NULL)
		{
			new=Gr_CreateRegionForDeadRegion(dead);
			if (kPtr->childrenPtr==NULL)
			{
				kPtr->childrenPtr=new;
				grTailRegionPtr->nextPtr=new;
				grTailRegionPtr=new;
			}
			else
			{
				grTailRegionPtr->siblingPtr=new;
				grTailRegionPtr->nextPtr=new;
				grTailRegionPtr=new;
			}
		dead=dead->next;
		}
	}

}


