/*
 * grCritical.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *	(Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint



/*
*------------------------------------------------------------------------
*	Functions in this file are called when critical nets are present
*	and we are running the program with in timing mode.
*-----------------------------------------------------------------------
*/


#include  "grInt.h"


/*
*-------------------------------------------------------------------------
*	Gr_CheckIfDecompositionImproves
*	Checks if decomposition imprives routing. Returns 0 if it does not
*	Returns  #of divisions  otherwise                                
*	NO PREROUTED NETS VERSION                                         
*-------------------------------------------------------------------------
*/
int Gr_CheckIfDecompositionImproves(sNet,dir)

grSubNet *sNet;				/* pointer to a subNet */
int dir;				/* direction of a cut  */

{

	grSuperNode *left,*right;
	int shadow,d;

	/* get left and right node*/

	left=sNet->NodeList;
	if (left->side==2) /* first sits on right side of cut line*/
	{
		right=left;
		left=right->nextOnList;
	}
	else
	{
		right=left->nextOnList;
	}

	/* end set left and right nodes*/

	if ((right->contentsPtr->count==1)&&(left->contentsPtr->count==1))
	{
		return (0);  /* single pins on both sides */
	}

	shadow=Gr_GetLengthOfCommonShadow(right->contentsPtr,
	left->contentsPtr,dir);
	if (shadow==0)
	{
		return (0);
	}
	else
	{
		d=Gr_GetDistanceBetweenBoxes(right->contentsPtr,
		left->contentsPtr);
	}

	if (shadow<=d)
	{
		return (0);
	}
	else
	{
		if (d>0)
		{
			return (shadow/d); /* number of divisions*/
		}
		else
		{
			return (2);
		}
	}

}

/*
*----------------------------------------------------------------------------
*	Gr_GetLengthOfCommonShadow
*	Side effects : none.
*	Results      : returns length of a common interval
*			of 2 rectangles projected on a line. 
*	If dir==1 it checks 2 boxes on vertical line. If dir==2 on horizontal 
*	line. 
*----------------------------------------------------------------------------
*/


int Gr_GetLengthOfCommonShadow(box1,box2,dir)

grPinRectangle *box1,*box2; 		/* Pointers to checked boxes */
int dir;				/* direction of a cut */

{


	int ytop,ybot,xleft,xright;


	if (dir==1)
	{
		if ((box1->miny>=box2->maxy)||(box2->miny>=box1->maxy))
		{
			return (0);
		}
		else
		{
			if (box1->maxy>=box2->maxy)
			{
				ytop=box2->maxy;
			}
			else
			{
				ytop=box1->maxy;
			}

			if (box1->miny>=box2->miny)
			{
				ybot=box1->miny;
			}
			else
			{
				ybot=box2->miny;
			}

			return (ytop-ybot);

		}
	}

	if (dir==2)
	{
		if ((box1->minx>=box2->maxx)||(box2->minx>=box1->maxx))
		{
			return (0);
		}
		else
		{
			if (box1->maxx>=box2->maxx)
			{
				xright=box2->maxx;
			}
			else
			{
				xright=box1->maxx;
			}
			if (box1->minx>=box2->minx)
			{
				xleft=box1->minx;
			}
			else
			{
				xleft=box2->minx;
			}

			return (xright-xleft);

		}
	}

	return (0);

}

/*
*------------------------------------------------------------------------
*	Gr_MkeSinglePinNodes(sNet)
*	Results      : none.
*	Side effects : for a net which is a decomposition candidate
*	it creates a set of single pin superNodes.
*	NO PARTIALLY PREROUTED NETS VERSION 
*-------------------------------------------------------------------------
*/


void Gr_MkeSinglePinNodes(sNet)

grSubNet *sNet; 		/* pointer to the subNet which will be
				   decomposed */

{

	grSuperNode *left,*right,*current;
	grPinRectangle *new,*lbox,*rbox;
	int flag;
	grSortedPin *gSPin;

	flag=0;
	left=sNet->NodeList;

	if (left->side==2)
	{
		right=left;
		left=right->nextOnList;
	}
	else
	{
		right=left->nextOnList;
	}

	sNet->NodeList=NULL;
	sNet->decomposed=NULL;
	lbox=left->contentsPtr;
	rbox=right->contentsPtr;

	gSPin=lbox->firstxPin;

lab1:;
	while (gSPin!=NULL)
	{
		new=Gr_AllocateMemoryForNewPinRectangle();
		Gr_AttachPinRectangleToDecomposed(sNet,new);
		current=Gr_AllocateMemoryForNewSuperNode(sNet,sNet->NodeList);
		sNet->NodeList = current; 
		current->contentsPtr=new;
		new->firstxPin=gSPin;
		new->firstyPin=gSPin;
		new->count=1;
		new->minx=gSPin->pinPtr->x;
		new->maxx=new->minx;
		new->miny=gSPin->pinPtr->y;
		new->maxy=new->miny;
		new->cutMark=0;
		gSPin=gSPin->nextxPin;

		if (flag==0)
		{
			current->kind=left->kind;
			current->nodeName=left->nodeName;
			current->side=left->side;
		}
		else
		{
			current->kind=right->kind;
			current->nodeName=right->nodeName;
			current->side=right->side;
		}
	}

	if (flag==0)
	{
		flag=1;
		gSPin=rbox->firstxPin;
		goto lab1;
	}

	new=sNet->decomposed;
	while (new!=NULL)
	{
		new->firstxPin->nextxPin=NULL;
		new->firstyPin->nextyPin=NULL;
		new=new->nextInNet;
	}
}

