
/* grCut.c --
 *
 *     Malgorzata Marek-Sadowska
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "grInt.h"


/*
*--------------------------------------------------------------------------
*   Functions in this file perform various tasks related to cutting
*   of the current region and creating new routing regions.
*   It is assumed everywhere, that dir=0 means no cut needed,
*   dir = 1 denotes vertical cut, dir = 2 denotes horizontal cut.
*--------------------------------------------------------------------------
*/



/*
*--------------------------------------------------------------------------
*   Gr_DetermineCut
*   This function calculates position of a cut
*   line and its direction for the currently considered region.      
*   Results      : none.
*   Side effects : fields containing cut positions are filled in
*   in the structure grRegion.
*--------------------------------------------------------------------------
*/

void
Gr_DetermineCut(kPtr)
grRegion *kPtr;     /* pointer to grRegion for which cutposition
                     * is to be determined
                     */
{
    int xv,yv,dir;
    int wv,hv;
    int cposh,cposv;

    wv=kPtr->whvirt;
    hv=kPtr->hvirt;
    if ((hv==1)&&(wv==1))
    {
        dir=0;
    }
    if ((hv>=wv)&&(hv>1))
    {
        cposh=Gr_TryHor(kPtr);

        if (cposh > 0)
        {
            yv=cposh;
            xv=kPtr->xvirt;
            dir=2;
        }
        else
        {
            cposv=Gr_TryVer(kPtr);

            if (cposv > 0)
            {
                xv=cposv;
                yv=kPtr->yvirt;
                dir=1;
            }
            else
            {
                if ((cposv == 0)&&(cposh == 0))
                {
                    dir = 0;
                }
                else
                {
                    if (cposh < 0)
                    {
                        yv= -cposh;
                        xv= kPtr->xvirt;
                        dir=2;
                    }
                    else
                    {
                        xv= -cposv;
                        yv= kPtr->yvirt;
                        dir=1;
                    }
                }
            }
        }
    }
    if (wv>hv)
    {
        cposv=Gr_TryVer(kPtr);

        if (cposv > 0)
        {
            xv=cposv;
            yv=kPtr->yvirt;
            dir=1;
        }
        else
        {
            cposh=Gr_TryHor(kPtr);

            if (cposh > 0)
            {
                xv=kPtr->xvirt;
                yv=cposh;
                dir=2;
            }
            else
            {
                if ((cposh == 0)&&(cposv == 0))
                {
                    dir = 0;
                }
                else
                {
                    if (cposv < 0)
                    {
                        xv= -cposv;
                        yv= kPtr->yvirt;
                        dir=1;
                    }
                    else
                    {
                        yv= -cposh;
                        xv= kPtr->xvirt;
                        dir=2;
                    }
                }
            }
        }
    }
    if (dir==0)
    {
        xv=0;
        yv=0;
    }
    if (dir>0)
    {
        /* DEBUGGING PRINTOUTS */  
        /*  if (debug=='y') */  
        if (grPrintFlag == TRUE) {
            (void) fprintf (bearout, "\n dir=%4d",dir);
            (void) fprintf (bearout, "\n xv=%4d yv=%4d",xv,yv);

            (void) fprintf (bearout, "\n xpos=%4d",(gr_xcoord+xv)->number);
            (void) fprintf (bearout, "   ypos=%4d",(gr_ycoord+yv)->number);

            (void) fflush (bearout);

            Gr_PrintCurrentLengthsOfNets();   
        }  
    }
    (kPtr->xcutvirt)=xv;
    (kPtr->ycutvirt)=yv;
    (kPtr->dir)=dir; 
}



/*
*-------------------------------------------------------------------------
*   Gr_CutRegion(kPtr)
*   Results      :none.
*   Side effects : for the current region all the structures con-
*   taining geometrical data related to cutting are filled in .
*   Number of holes n is determined here.
*-------------------------------------------------------------------------
*/



void Gr_CutRegion(kPtr)

grRegion *kPtr;         /* pointer to grRegion which needs to be
                   partitioned */

{
    int holeCount, dir ;

    Gr_DetermineCut(kPtr);

    dir=kPtr->dir;

    if (dir==1)
    {
        holeCount=Gr_VertHoleCount(kPtr);
                n=holeCount;
                Gr_SetMemoryForHoles();
                Gr_VertHoleAffiliation(kPtr);
    }

    if (dir==2)
    {
        holeCount=Gr_HorzHoleCount(kPtr);
                n=holeCount;
                Gr_SetMemoryForHoles();
                Gr_HorzHoleAffiliation(kPtr);
    }



}




/*
*---------------------------------------------------------------------------
*   Gr_MkeRegions()
*   This is a function which controlls the whole process of
*   routing.
*   Results      : BEAR_OK if routing has been completed,
*   otherwise it signals various forseeable problems causing
*   program termination.
*   Side effects : solution to global routing problem in
*   terms of pseudo pins inserted to appropriate lists in
*   database.
*---------------------------------------------------------------------------
*/

BearStatus
Gr_MkeRegions()
{
    grRegion       *kptr;
    grSubNet       *sNet;
    int             cntr;
    int             par,i;

    MALLOC(grRegion*,grHeadRegionPtr,sizeof (grRegion));
    grTailRegionPtr=grHeadRegionPtr;
    grTailRegionPtr->xvirt=0;
    grTailRegionPtr->yvirt=0;
    grTailRegionPtr->whvirt=gr_totxgrid;
    grTailRegionPtr->hvirt=gr_totygrid;
    grTailRegionPtr->subNetsPtr=NULL;
    grTailRegionPtr->firstPinRectanglePtr=NULL;
    grTailRegionPtr->childrenPtr=NULL;
    grTailRegionPtr->siblingPtr=NULL;
    kptr=grTailRegionPtr;
    if (Gr_InitializeSubNets(kptr) == BEAR_ERROR) 
    {
        return (BEAR_ERROR);
    }
    Gr_InitializeLayer(kptr);

    if (gr_Timing > 0)
    {
        Gr_UpdateCriticalNetStatus();
    }

    /*Gr_PrintCurrentLengthsOfNets();*/

    while (kptr!=NULL)
    {                 
        Gr_CutRegion(kptr);
        if ((kptr->xcutvirt == 8)&&(kptr->ycutvirt == 0))
        {
            for (i=1;i<=n;i++)
                Gr_PrintHole(i);
                if (grPrintFlag == TRUE) {
                    (void) fprintf (bearout, "\n nets before assignment");
                    (void) fprintf (bearout, "\n parent region");
                }
                Gr_PrintRegion(kptr);
                if (grPrintFlag == TRUE) {
                    (void) fprintf (bearout, "\n nets region");
                }
                Gr_PrintSubNets(kptr);  
                if (grPrintFlag == TRUE) {
                    (void) fflush (stdout);
                }
            }

        if (gr_Timing > 0)
        {
            Gr_SetBtnSizes(kptr);

            /*
            DEBUGGING PRINTOUTS 
            (void) fprintf (bearout, "\n");
            Gr_PrintEstimatesInBtn();
            (void) fprintf (bearout, "\n");
            */

            Gr_UpdateHoleBoundsForCriticalNets(kptr->dir);
        }

        if (kptr->dir > 0)
        {
            par = Gr_IsAssignmentPossible(kptr);
        }
        else
        {
            par = 0;
        }
/* CHANGE JULY 7th*/
        if ((kptr->dir) == 1)
        {
            Gr_MarkVerCut(kptr);
        }
        if ((kptr->dir) == 2)
        {
            Gr_MarkHorCut(kptr); 
        }
/* END */
        Gr_MarkCurrentBtnCut(kptr);
        Gr_InitializeNewRegions(kptr,par);
                Gr_CreateNewRegionsInLayer(kptr,par);

        if (par == 0)
        {
                    cntr = Gr_SetUpSceneForLinearAssignment(kptr);
                    if (cntr > 0)
                    {
                        /* make linear assignment */

                        Gr_LSAssign();

                        Gr_CollectDataAfterAssignment(kptr);

                        /* DEBUGGING PRINTOUTS */
                        /* if (debug=='y') */ 
                        if (grPrintFlag == TRUE) {
                            (void) fprintf (bearout,
                                "\n nets after assignment");
                            (void) fprintf (bearout, "\n left subregion");
                            Gr_PrintRegion(kptr->leftPtr);

                            (void) fprintf (bearout,
                                "\n nets in left subregion");
                            Gr_PrintSubNets((kptr->leftPtr));
                            (void) fprintf (bearout, "\n right subregion");
                            Gr_PrintRegion(kptr->rightPtr);
                            (void) fprintf (bearout,
                                "\n nets in right subregion");
                            Gr_PrintSubNets((kptr->rightPtr));
                        }
                    }
                    if (cntr < 0) {
                        Gr_CollectDataAfterAssignment(kptr);

                        /* DEBUGGING PRINTOUTS */
                        /*  if (debug=='y')  */
                        if (grPrintFlag == TRUE) {
                            (void) fprintf (bearout,
                                "\n assignment through bottleneck residents");
                            (void) fprintf (bearout, "\n left region");
                            if (kptr->leftPtr!=NULL) {
                                Gr_PrintRegion(kptr->leftPtr);
                                Gr_PrintSubNets(kptr->leftPtr);
                            }
                            (void) fprintf (bearout, "\n right region");
                            if (kptr->rightPtr!=NULL) {
                                Gr_PrintRegion(kptr->rightPtr);
                                (void) fprintf (bearout,
                                    "\n nets in right region");
                                Gr_PrintSubNets(kptr->rightPtr);
                                (void) fprintf (bearout, "\n");
                            }
                        }
                    }
                    if (cntr==0)
                    {
                /* DEBUGGING PRINTOUTS */
                /* if (debug=='y')  */
                if (grPrintFlag == TRUE) {
                    (void) fprintf (bearout, "\no assignment occured");
                    (void) fprintf (bearout, "\n left region");
                    if (kptr->leftPtr!=NULL) {
                        Gr_PrintRegion(kptr->leftPtr);
                        (void) fprintf (bearout, "\n nets in left region");
                        Gr_PrintSubNets(kptr->leftPtr);
                    }
                    (void) fprintf (bearout, "\n right region");
                    if (kptr->rightPtr!=NULL) {
                        Gr_PrintRegion(kptr->rightPtr);
                        (void) fprintf (bearout, "\n nets in right region");
                        Gr_PrintSubNets(kptr->rightPtr);
                        (void) fprintf (bearout, "\n\n");
                    }
                }
            }
            if (cntr!=0)
            {
                         Gr_CleanSceneAfterAssignment(); 
            }
        }
        else
        {
            /* DEBUGGING PRINTOUTS */
            if (grPrintFlag == TRUE) {
                (void) fprintf (bearout, "\n original region");
            }
            Gr_PrintRegion(kptr);
            Gr_PrDeadRegionLists();
            if (grPrintFlag == TRUE) {
                (void) fprintf (bearout, "\n subnets there");
            }
            Gr_PrintSubNets(kptr);
            if (grPrintFlag == TRUE) {
                (void) fflush (bearout);
            }

            Gr_PullAssignment(kptr);

            if (grPrintFlag == TRUE) {
                (void) fprintf (bearout, "\n  after Pull assignment");
            }
            Gr_PrintSubNets(kptr);
            (void) fflush (bearout);

            sNet=kptr->subNetsPtr;
            while (sNet!=NULL)
            {
                if (sNet->active == 5)
                {
                Gr_PushDownwardPartitionedNodes(kptr,sNet);
                }
                sNet=sNet->nextOnList;
            }
            Gr_FreeDeadRegions();
        }
                m=0;
                n=0;

        Gr_FreeRegionLists(kptr);
                kptr=Gr_NextInLayer(); /*breadth first search */
    } 

    return (BEAR_OK);
}


/*
*----------------------------------------------------------------------------
*   Gr_TryHor(kPtr)
*   This function checks if cut is possible in horizontal direction.
*   Results       : if impossible returns 0,
*           if possible returns best y position for it.
*   Side effects  : none.
*----------------------------------------------------------------------------
*/



int Gr_TryHor(kPtr)

grRegion *kPtr;         /* pointer to grRegion to be partitioned*/


{
    int j1,j2,dir;
    int cost;
    int posy1,posy2,posx1,posx2,by,mid;
    int by_nv,width_o,cbot,ctop;
    int bwidth_o;

    by=0;
    dir=0;
    posx1=kPtr->xvirt;
    posx2=posx1+(kPtr->whvirt)-1;
    posy1=(kPtr->yvirt)+1;
    posy2=(kPtr->yvirt)+(kPtr->hvirt)-1;

    if (posy2<posy1)
    {
         goto lab;
    }

    mid=(posy2-posy1)/2 +posy1;
    by_nv=0;
    bwidth_o=0;

    for (j1=posy1;j1<=posy2;j1++)
    {
        cost=0;
        width_o=0;
        for (j2=posx1;j2<=posx2;j2++)
        {
            ctop=Gr_UpRight(j2,j1);
            cbot=Gr_BotRight(j2,j1);

            if ((ctop==0)&&(cbot==0))
            {
                 width_o++;
            }
            if (cost==0)
            {
                if ((ctop>0)&&(Gr_IsHorzBottl(ctop)>0))
                {
                     cost=1;
                }
                if ((cbot>0)&&(Gr_IsHorzBottl(cbot)>0))
                {
                     cost=1;
                }
            }
        }
        if ((width_o>0)&&(cost>0))
        {
            if (by==0)
            {
                by=j1;
                bwidth_o=width_o;
            }
            else
            {
                if (bwidth_o>width_o)
                {
                    by=j1;
                    bwidth_o=width_o;
                }
                else
                {
                    if (bwidth_o==width_o)
                    {
                        if (((mid-by)*(mid-by)) >
                        ((mid-j1)*(mid-j1)))
                        {
                            by=j1;
                        }
                    }
                }
            }
        }
        if ((cost>0)&&(width_o==0))
        {
            if (by_nv==0)
            {
                by_nv=j1;
            }
            else
            {
                if (((mid-by_nv)*(mid-by_nv)) >
                ((mid-j1)*(mid-j1)))
                {
                    by_nv=j1;
                }
            }
        }
    }
    if (by_nv>0)
    {
        dir=by_nv;
    }
    else
    {
        if (by>0)
        {
            dir= -by;
        }
    }
    if ((by==0)&&(by_nv==0))
    {
         dir=0;
    }
lab: 
    return (dir);
}


/*
*----------------------------------------------------------------------------
*   Gr_TryVer
*   This function checks if cut is possible in vertical direction.
*   Results       : if impossible returns 0,
*           if possible returns best x position for it.
*   Side effects  : none.
*----------------------------------------------------------------------------
*/


int Gr_TryVer(kPtr)

grRegion *kPtr;         /* pointer to grRegion to be partitioned*/

{

    int j1,j2,dir;
    int posy1,posy2,posx1,posx2,bx,mid;
    int cost;
    int bheight_o;
    int bx_nh,height_o,cleft,cright;
    
    bx=0;
    dir=0;
    posx1=(kPtr->xvirt)+1;
    posx2=(kPtr->xvirt)+(kPtr->whvirt)-1;
    posy1=kPtr->yvirt;
    posy2=posy1+(kPtr->hvirt)-1;

    if (posx2<posx1)
    {
         goto lab;
    }
    mid=(posx2-posx1)/2 +posx1;
    bx_nh=0;
    bheight_o=0;

    for (j1=posx1;j1<=posx2;j1++)
    {
        cost=0;
        height_o=0;

        for (j2=posy1;j2<=posy2;j2++)
        {
            cleft=Gr_UpLeft(j1,j2);
            cright=Gr_UpRight(j1,j2);
            if ((cleft==0)&&(cright==0)) height_o++;
            if (cost==0)
            {
                if ((cleft>0)&&(Gr_IsVerBottl(cleft)>0))
                {
                    cost=1;
                }
                if ((cright>0)&&(Gr_IsVerBottl(cright)>0))
                {
                    cost=1;
                }
            }
        }
        if ((height_o>0)&&(cost>0))
        {
            if (bx==0)
            {
                bx=j1;
                bheight_o=height_o;
            }
            else
            {
                if (bheight_o>height_o)
                {
                    bx=j1;
                    bheight_o=height_o;
                }
                else
                {
                    if (bheight_o==height_o)
                    {
                        if (((mid-bx)*(mid-bx)) >
                        ((mid-j1)*(mid-j1)))
                        {
                            bx=j1;
                        }
                    }
                }
            }
        }
        if ((cost>0)&&(height_o==0))
        {
            if (bx_nh==0)
            {
                bx_nh=j1;
            }
            else
            {
                if (((mid-bx_nh)*(mid-bx_nh)) >
                ((mid-j1)*(mid-j1)))
                {
                    bx_nh=j1;
                }
            }
        }
    }
    if (bx_nh>0)
    {
        dir=bx_nh;
    }
    else
    {
        if (bx>0)
        {
            dir= -bx;
        }
    }
    if ((bx_nh==0)&&(bx==0)) 
    {
        dir=0;
    }
lab: 
    return (dir);
}


/*
*--------------------------------------------------------------------------
*   Gr_MarkVerCut
*   Traces vertical cut line and markes the touched blocks.
*   Results       : none.
*   Side effects  : all grBlocks adjacent to the cutline get
*   negative values in the bitmap.
*--------------------------------------------------------------------------
*/



void Gr_MarkVerCut(kPtr)

grRegion *kPtr;         /* pointer to grRegion which has been
                   partitioned */

{


    int posx,posy1,posy2,j1,cleft,cright,lleft,lright;

    posx=kPtr->xcutvirt;
    posy1=kPtr->ycutvirt;
    posy2=posy1+(kPtr->hvirt) - 1;

    for (j1=posy1;j1<=posy2;j1++)
    {
        lleft=Gr_UpLeft(posx,j1);
        cleft=posx;
        while (Gr_IsNotHorBottl(lleft)&&(cleft>0)&&(lleft>0))
        {
            /*mark left side of the cut*/
            Gr_ReverseUpLeft(cleft,j1);
            cleft--;
            if (cleft>0)
            {
                lleft=Gr_UpLeft(cleft,j1);
            }
            else
            {
                lleft=0;
            }
        }
        lright=Gr_UpRight(posx,j1);
        cright=posx;
        while (Gr_IsNotHorBottl(lright)&&
        (cright<gr_totxgrid)&&(lright>0))
        {
            /*mark right side*/
            Gr_ReverseUpRight(cright,j1);
            cright++;
            if (cright<gr_totxgrid)
            {
                lright=Gr_UpRight(cright,j1);
            }
            else
            {
                lright=0;
            }
        }
    }
}


/*
*--------------------------------------------------------------------------
*   Gr_MarkHorCut
*   Traces horizontal cut line and markes the touched blocks.
*   Results       : none.
*   Side effects  : all grBlocks adjacent to the cutline get
*   negative values in the bitmap.
*--------------------------------------------------------------------------
*/



void Gr_MarkHorCut(kPtr)

grRegion *kPtr;         /* pointer to grRegion which has been
                   cut  */

{

    int posx1,posx2,posy,j1,ctop,cbot,ltop,lbot;

    posx1=kPtr->xcutvirt;
    posx2=posx1+(kPtr->whvirt)-1;
    posy=kPtr->ycutvirt;

    for (j1=posx1;j1<=posx2;j1++)
    {
        ltop=Gr_UpRight(j1,posy);
        ctop=posy;
        while (Gr_IsNotVertBottl(ltop)&&(ctop<gr_totygrid)&&(ltop>0))
        {
            /*mark the top side*/
            Gr_ReverseUpRight(j1,ctop);
            ctop++;
            if (ctop<gr_totygrid)
            {
                ltop=Gr_UpRight(j1,ctop);
            }
            else
            {
                ltop=0;
            }
        }
        lbot=Gr_BotRight(j1,posy);
        cbot=posy;

        while (Gr_IsNotVertBottl(lbot)&&(cbot>0)&&(lbot>0))
        {
            /*mark bottom side*/
            Gr_ReverseBotRight(j1,cbot);
            cbot--;
            if (cbot>0)
            {
                 lbot=Gr_BotRight(j1,cbot);
            }
            else
            {
                lbot=0;
            }
        }
    }
}


/*
*----------------------------------------------------------------------------
*   Gr_IsNotHorBottl
*   This is a utility functon.
*   Results     : 1 if a cell in a bitmap is not a horizontal bottleneck,
*             0 if it is.
*   Side effects: none.
*----------------------------------------------------------------------------
*/


int Gr_IsNotHorBottl(number)

int number;         /* Position of a cell in gr_Bitmap array*/

{
    if (number<=0)
    {
         return (0);
    }
    if (number<=grSolidTileCount)
    {
         return (1);
    }
    if (number>(grSolidTileCount+grHorzBottlNumber))
    {
         return (1);
    }

    return (0);
}


/*
*----------------------------------------------------------------------------
*   Gr_IsNotVertBottl
*   This is a utility functon.
*   Results     : 1 if a cell in a bitmap is not a vertical bottleneck,
*             0 if it is.
*   Side effects: none.
*----------------------------------------------------------------------------
*/



int Gr_IsNotVertBottl(number)

int number;         /* Position of a cell in gr_Bitmap array*/

{

    if (number<=0)
    {
         return (0);
    }
    if (number<=(grSolidTileCount+grHorzBottlNumber))
    {
        return (1);
    }
    return (0);
}

/*
*----------------------------------------------------------------------------
*   Gr_IsHorzBottl
*   This is a utility functon.
*   Results     : 1 if a cell in a bitmap is a horizontal bottleneck,
*             0 if it is not.
*   Side effects: none.
*----------------------------------------------------------------------------
*/



int Gr_IsHorzBottl(number)

int number;         /* Position of a cell in gr_Bitmap array*/

{

    if (number<=grSolidTileCount)
    {
         return (0);
    }
    if (number>(grSolidTileCount+grHorzBottlNumber)) 
    {
        return 0;
    }
    return (1);

}


/*
*----------------------------------------------------------------------------
*   Gr_IsVerBottl
*   This is a utility functon.
*   Results     : 1 if a cell in a bitmap is a vertical bottleneck,
*             0 if it is not.
*   Side effects: none.
*----------------------------------------------------------------------------
*/



int Gr_IsVerBottl(number)

int number;         /* Position of a cell in gr_Bitmap array*/

{

    if (number>(grSolidTileCount+grHorzBottlNumber))
    {
         return (1);
    }
    return (0);
}

