
/* grDataAssign.c --
 *
 *     Malgorzata Marek-Sadowska
 *     (Professor E.S.Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint


#include "bearCopyRight.h"
#include "grInt.h"


/*
*---------------------------------------------------------------------------
*   This file contains functions which participate in calculating
*   weights for linear assignment , setting environment for the
*   assignment and collecting basic data after assignment has been
*   performed.
*---------------------------------------------------------------------------
*/

 
/*
*---------------------------------------------------------------------------
*   Gr_GetDistBetweenSortPins
*   Result      : length of a distance between two sorted pins.
*   Side effects: none.
*---------------------------------------------------------------------------
*/



int Gr_GetDistBetweenSortPins(lPin,rPin)

grSortedPin *lPin,*rPin;        /* pointers to sorted pins, distance
                       between which is to be calculated*/

{
    int x,y;

    x=lPin->pinPtr->x - rPin->pinPtr->x;
    if (x<0) x= -x;
    y=lPin->pinPtr->y - rPin->pinPtr->y;
    if (y<0) y= -y;
    return  (x+y);
}


/*
*----------------------------------------------------------------------------
*   Gr_CalculateGreedyWeightsForVirtualNets
*   Results       : none;
*   Side efects   : entries in matrix c of cost assignments
*   corresponding to virtual net are calculated.
*---------------------------------------------------------------------------
*/



void Gr_CalculateGreedyWeightsForVirtualNets(sNet,vNet,dir,count,mini)

grSubNet *sNet;         /* pointer to the parent subnet */
grVirtNet *vNet;        /* pointer to virtual net for which
                   costs of passing through various holes
                   are calculated */
int dir;            /* direction of cut line */
int count;          /* position of the vNet on an ordered
                   list if virtual nets  */
int mini;           /* value of the smallest cost */

{
    grNet *gNet;
    int i,xleft,xright,ytop,ybot;
    grSortedPin *lPin,*rPin,*blPin,*brPin;
    int dist,bestdist;
    int delta,thole,bhole;
    int v1,v2;
    int strt;

    strt=(count-1)*n;
    if ((vNet->leftNode->contentsPtr->count==1)&&
        (vNet->rightNode->contentsPtr->count==1))
    {
        lPin=vNet->leftNode->contentsPtr->firstxPin;
        blPin=lPin;
        rPin=vNet->rightNode->contentsPtr->firstxPin;
        brPin=rPin;
    }
    else
    {
        bestdist=0;
        blPin=NULL;
        brPin=NULL;

        lPin=vNet->leftNode->contentsPtr->firstxPin;
        while (lPin!=NULL)
        {
            rPin=vNet->rightNode->contentsPtr->firstxPin;
            while (rPin!=NULL)
            {
                dist=Gr_GetDistBetweenSortPins(lPin,rPin);
                if (blPin==NULL)
                {
                    blPin=lPin;
                    brPin=rPin;
                    bestdist=dist;
                }
                else
                {
                    if (bestdist>dist)
                    {
                        blPin=lPin;
                        brPin=rPin;
                    }
                }
                rPin=rPin->nextxPin;
            } /*end right while*/
            lPin=lPin->nextxPin;
        } /*end left while */
    }

    xleft=blPin->pinPtr->x;
    if (xleft<=brPin->pinPtr->x)
    {
        xright=brPin->pinPtr->x;
    }
    else
    {
        xleft=brPin->pinPtr->x;
        xright=blPin->pinPtr->x;
    }
    ybot=blPin->pinPtr->y;
    if (ybot<=brPin->pinPtr->y)
    {
        ytop=brPin->pinPtr->y;
    }
    else
    {
        ybot=brPin->pinPtr->y;
        ytop=blPin->pinPtr->y;
    }

    Gr_CalculateLinearLengthWeightsForPreferedBox(dir,count,
    xleft,xright,ytop,ybot,mini);

    if (sNet->critical > 0)
    {
    Gr_UpdateWeightsForResidentPins(vNet,dir,1,count);
        gNet=(gr_HeadGrNets+sNet->netname);
        thole=gNet->topHole;
        bhole=gNet->botHole;
            if (thole > 0)
            {
                delta=gNet->critical;
                    if (thole < n)
                    {
                        v1= (c+strt+thole+1)->number;
                    }
                    else
                    {
                        v1=0;
                    }
                    if (bhole>1)
                    {
                        v2= (c+strt+bhole-1)->number;
                    }
                    else
                    {
                        v2=0;
                    }
                    if (v1>v2)
                    {
                        delta += (v1-v2);
                    }
                    else
                    {
                        delta += (v2-v1);
                    }

                    if (bhole > 1)
                    {
                        for (i=1;i<bhole;i++)
                        if ((c+strt+i)->number > mini)
                        {
                        ((c+strt+i)->number) += delta;
                        }
                    }
                    if (thole < n)
                    {
                        for (i= (thole+1);i<=n;i++)
                        if ((c+strt+i)->number > mini)
                        {
                        ((c+strt+i)->number) += delta;
                        }
                    }
            }
    }
}


/*
*----------------------------------------------------------------------------
*   Gr_CalculateLinearLengthWeightsForPreferedBox
*   Results       : none;
*   Side effects  : weights based on real distance from the prefered 
*           box are calculated.
*    The interior of the prefered box is set to a value mini.
*----------------------------------------------------------------------------
*/


void Gr_CalculateLinearLengthWeightsForPreferedBox(
dir,count,xleft,xright,ytop,ybot,mini)

int dir;        /* direction of a cut */
int count;      /* number of virtual net */
int xleft;      /* left x coordinate of the prefered box */
int xright;         /* right x coordinate of the prefered box */
int ytop;       /* top y coordinate of the prefered box */
int ybot;       /* bot y coordinate of the prefered box */
int mini;       /* minimum cost value to be assigned to 
               holes covered by the prefered box */

{

    int i,curmax,curmin,curhol,hBot,hTop,flag,strt;
    int xcentr,ycentr;
    grPrimit *dist;
    int lFlag,rFlag;

    strt=(count-1)*n;
    MALLOC(grPrimit*,dist,sizeof (grPrimit)*(n+1));

    for (i=1;i<=n;i++)
    {
        (dist+i)->number = 0;
    }


    hBot=0;
    hTop=0;
    lFlag=0;
    rFlag=0;

    i=0;

    while (i<n)
    {
        i++;
        if (dir==1)
        {
            if (hBot==0)
            {
                if ((grHeadHolePtr+i)->topOrRight_p_y >ybot)
                {
                    hBot=i;
                }
            }
            if ((grHeadHolePtr+i)->botOrLeft_p_y < ytop)
                hTop=i;
        }

        if (dir==2)
        {
            if (hBot==0)
            {
                if ((grHeadHolePtr+i)->topOrRight_p_x>xleft)
                {
                    hBot=i;
                }
            }
            if ((grHeadHolePtr+i)->botOrLeft_p_x<xright)
                hTop=i;
        }
    }
    if ((hBot>0)&&(hTop>0))
    {
        i=0;
        while (i<n)
        {
            i++;
            if ((i<hBot)||(i>hTop))
            {
                (c+strt+i)->number=LARGE_NEGATIVE;
            }
            else
            {
                (c+strt+i)->number=mini;
            }
        }

        if (dir==1)
        {
            i=0;
            while (i<n)
            {
                i++;
                ycentr=((grHeadHolePtr+i)->topOrRight_p_y-
                (grHeadHolePtr+i)->botOrLeft_p_y)/2
                    + (grHeadHolePtr+i)->botOrLeft_p_y;
                if (ycentr<=ybot)
                {
                    (dist+i)->number=ybot-ycentr;
                }
                else
                {
                    if (ycentr>=ytop)
                    {
                        (dist+i)->number=ycentr-ytop;
                    }
                    else
                    {
                        (dist+i)->number=0;
                    }
                }
            }
        }
        if (dir==2)
        {
            i=0;
            while (i<n)
            {
                i++;
                xcentr=((grHeadHolePtr+i)->topOrRight_p_x-
                (grHeadHolePtr+i)->botOrLeft_p_x)/2
                    +(grHeadHolePtr+i)->botOrLeft_p_x;
                if (xcentr<=xleft)
                {
                    (dist+i)->number=xleft-xcentr;
                }
                else
                {
                    if (xcentr>=xright)
                    {
                        (dist+i)->number=xcentr-xright;
                    }
                    else
                    {
                        (dist+i)->number=0;
                    }
                }
            }
        }
        curmax=mini;
    lab1: 
        curmin=0;
        curhol=0;
        flag=0;

        for (i=1;i<=n;i++)
        {
            if ((c+strt+i)->number == LARGE_NEGATIVE)
            {
                flag=1;
                if (curmin==0)
                {
                    curmin=(dist+i)->number;
                    curhol=i;
                }
                else
                {
                    if ((dist+i)->number < curmin)
                    {
                        curmin=(dist+i)->number;
                        curhol=i;
                    }
                }
            }
        }
        curmax++;
        if (curhol>0)
        {
             (c+strt+curhol)->number=curmax;
        }
        if (hBot > hTop)
        {
                    if ((curhol <= hTop)&&(lFlag == 0))
            {
                if (curmax == (mini+1))
                {
                    (c+strt+curhol)->number = mini;
                    lFlag=1;
                }
                if (curmax == (mini+2))
                {
                    (c+strt+curhol)->number = mini;
                    lFlag=1;
                }
            }
                if ((curhol >= hBot)&&(rFlag==0))
                {
                if (curmax == (mini+1))
                {
                    (c+strt+curhol)->number = mini;
                    rFlag=1;
                }
                if (curmax == (mini+2))
                {
                    (c+strt+curhol)->number = mini;
                    rFlag=1;
                }
                }
        }
        if (flag==1)
        {
             goto lab1;
        }
    }
    else
    {
        if (hTop==0)
        {
            for (i=1;i<=n;i++)
            {
                (c+strt+i)->number = mini+i;
            }
        }
    
        if (hBot==0)
        {
            for (i=1;i<=n;i++)
            {
                (c+strt+i)->number = mini+n+1-i;
            }
        }
    }
    GR_FREE((char*) dist);
}



/*
*---------------------------------------------------------------------------
*   Gr_IsSortedPinInsideHole
*   Results     : 1 if sortedPin is inside a hole,0 otherwise.
*   Side effects: none.
*--------------------------------------------------------------------------
*/


int Gr_IsSortedPinInsideHole(holeNr,gSPin,dir)

int          holeNr;        /* Hole number on a list for the current cut*/
int          dir;       /* direction of a cut line */
grSortedPin *gSPin;     /* pointer to the sorted pin */

{
    grBlock *block;
    int x1,y1,h,w,x,y;

    if ((grHeadHolePtr+holeNr)->status > 0) return (0);

    x=gSPin->pinPtr->x;
    y=gSPin->pinPtr->y;
    block=(grHeadHolePtr+holeNr)->leftBlockPtr;
    if (block==NULL)
    {
        block=(grHeadHolePtr+holeNr)->rightBlockPtr;
    }

    x1=block->xpos;
    y1=block->ypos;
    h=block->height;
    w=block->width;

    if (dir==1)
    {
        y1=(grHeadHolePtr+holeNr)->botOrLeft_p_y;
        h=(grHeadHolePtr+holeNr)->topOrRight_p_y - y1;
    }

    if (dir==2)
    {
        x1=(grHeadHolePtr+holeNr)->botOrLeft_p_x;
        w =(grHeadHolePtr+holeNr)->topOrRight_p_x - x1;
    }

    if (dir==1)
    {
        if ((y!=y1)&&(y!=(y1+h))) return (0);
        if ((x<x1)||(x>(x1+w)))  return (0);
        return (1);
    }
    else
    {
        if ((x!=x1)&&(x!=(x1+w))) return (0);
        if ((y<y1)||(y>(y1+h))) return (0);
        return (1);
    }
}


/*
*-------------------------------------------------------------------------
*   Gr_UpdateWeightsForResidentPins
*   Results      : none .
*   Side effects : costs of virtual nets having pins covered
*   by holes are changed to mini for these holes.
*-------------------------------------------------------------------------
*/


void Gr_UpdateWeightsForResidentPins(vNet,dir,mini,count)

grVirtNet *vNet;        /* pointer to virtual net */
int        dir;         /* direction of the recent cut */
int        mini;        /* minimum cost value */
int        count;       /* position of the virtual net on an ordered
                   list of virtual nets */


{

    int i,strt;
    grSortedPin *gSPin;
    grPinRectangle *gPRect;

    gPRect = vNet->leftNode->contentsPtr;
    strt=(count-1)*n;

lab2: 
    gSPin=gPRect->firstxPin;

    while (gSPin!=NULL)
    {
        for (i=1;i<=n;i++)
        {
            if (Gr_IsSortedPinInsideHole(i,gSPin,dir)>0)
            {
                if ((c+strt+i)->number ==  (mini+1) )
                {
                    ((c+strt+i)->number) = mini;
                }
                else
                {
                    ((c+strt+i)->number) -= mini;
                }
                goto lab;
            }
        }
lab:;
        gSPin=gSPin->nextxPin;
    }

    if (gPRect==vNet->leftNode->contentsPtr)
    {
        gPRect=vNet->rightNode->contentsPtr;
        goto lab2;
    }


}



/*
*--------------------------------------------------------------------------
*   Gr_CanNetBeForcedToResidentHole
*   Results     : hole nr where the net has a resident pin.
*             0  if virtual net has no pin covered by
*             a hole for the current cut line.
*   Side effects: none.
*   In normal situations, when no internal blocades exist
*   bLHole=1 and tRHole=n.
*--------------------------------------------------------------------------
*/



int Gr_CanNetBeForcedToResidentHole(vNet,dir,bLHole,tRHole)

grVirtNet *vNet;        /* pointer to the virtual net        */
int        dir;         /* cut direction                     */
int    bLHole;      /* the number of bottom or left hole */
int    tRHole;      /* top or right Hole number      */

{
    int i;
    grSortedPin *gSPin;
    grPinRectangle *gPRect;

    gPRect=vNet->leftNode->contentsPtr;

lab2:
    gSPin=gPRect->firstxPin;

    while (gSPin!=NULL)
    {
        for (i=bLHole;i<=tRHole;i++)
        {
            if (Gr_IsSortedPinInsideHole(i,gSPin,dir) > 0)
            {
                if ((grHeadHolePtr+i)->signalCapacity >=1)
                {
                    ((grHeadHolePtr+i)->signalCapacity)--;
                }
                return  (i);
            }
        }
        gSPin=gSPin->nextxPin;
    }

    if (gPRect==vNet->leftNode->contentsPtr)
    {
        gPRect=vNet->rightNode->contentsPtr;
        goto lab2;
    }
    return (0);
}


/*
*-------------------------------------------------------------------------
*   Gr_SetUpSceneForLinearAssignment
*   Results     : 0 no assignment needed.
*             1 assignment needed.
*   Calls all necessary functions which assign memory,
*   calculate cut capacity, perform decompression when needed, 
*   and calculate weights for virtual nets. 
*-------------------------------------------------------------------------
*/

int
Gr_SetUpSceneForLinearAssignment(kPtr)
grRegion *kPtr;         /* Pointer to the parent region */
{
    int         i, cnt, dir, sum;
    grVirtNet   *current;
    grSubNet    *pending;

    Gr_MkeVirtualNets(kPtr);

    if (m==0) return (0);

    Gr_Setup1LAss();  /* get memory for the assignment */
    dir=kPtr->dir;

    sum=Gr_CalculateSignalCapacityForHoles(kPtr);

    if (sum < m)
    {
        if (Gr_DecompressProportionallyCutCapacity(sum) > 0)
        {
            for (i=1;i<=n;i++)
            {
                (grACap+i)->number = (grHeadHolePtr+i)->signalCapacity;
            }
        }
    }

    Gr_RelaxCapacities();
    neff=0;
    for (i=1;i<=n;i++)
    {
        neff += (grACap+i)->number;
    }
    Gr_Setup2LAss();  /* get memory for the assignment */
    cnt=0;
    pending=kPtr->subNetsPtr;

    while (pending!=NULL)
    {
        if (pending!=NULL)
        {
            current=pending->children;

            while (current!=NULL)
            {
                current->hole=
            Gr_CanNetBeForcedToResidentHole(current,dir,1,n);
                if (current->hole !=0)
                {
                    m--;
                                        if (m==0) 
                                        {
                                          return (-1);
                                        }
                }
                else
                {
                    cnt++;
                    (vNetHead+cnt)->vNetPtr=current;
                    current->assiName=cnt;


                    Gr_CalculateGreedyWeightsForVirtualNets(
                    pending,current,dir,cnt,1);
                }

                current=current->nextInNet;
            }

            pending=pending->nextOnList;
        }
    }

    return (1);
}


/*
 *----------------------------------------------------------------------------
 *   Gr_FillHolesInVirtualNetsAfterAssignment()
 *   Results       : none.
 *   Side effects  : after assignment is completed the hole number 
 *           is put into a proper field
 *           in virtual net description data. 
 *----------------------------------------------------------------------------
 */

void
Gr_FillHolesInVirtualNetsAfterAssignment()
{
    int i;

    for (i=1;i<=m;i++)
    {
        (vNetHead+i)->vNetPtr->hole
        =(map+(col+(vNetHead+i)->vNetPtr->assiName)->number)->number;
    }

}


/*
 *-----------------------------------------------------------------------------
 *   Gr_CleanSceneAfterAssignment()
 *
 *   Results     : none;
 *   Side effects: removes vector of holes and vector of virtual nets. 
 *             sets m,n to 0. 
 *-----------------------------------------------------------------------------
 */

void
Gr_CleanSceneAfterAssignment()
{
    if (grHeadHolePtr != NIL(grHole)) {
        GR_FREE((char *) grHeadHolePtr);
        grHeadHolePtr = NIL(grHole);
    }
    n = 0;

    if (vNetHead != NIL(grIndex)) {
        GR_FREE((char *) vNetHead);
        vNetHead = NIL(grIndex);
    }
    m = 0;

    GR_FREE((char *) c);
    GR_FREE((char *) grACap);
    GR_FREE((char *) cap);
    GR_FREE((char *) col);
    GR_FREE((char *) nexts);
    GR_FREE((char *) row);
    GR_FREE((char *) h1);
    GR_FREE((char *) h2);
    GR_FREE((char *) h3);
    GR_FREE((char *) h4);
    GR_FREE((char *) h5);
    GR_FREE((char *) h6);
    GR_FREE((char *) map);
    GR_FREE((char *) mback);

    mback = map = grACap = cap = c = row = col = nexts = NIL(grPrimit);
    h1 = h2 = h3 = h4 = h5 = h6 = NIL(grPrimit);
}


