/*
 * fileName.c --
 *
 *     Author
 *  Malgorzata Marek-Sadowska
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint




#include "grInt.h"

/*
*-----------------------------------------------------------------------
*   All functions in this file collect and put back memory which was
*   assigned by the program.
*------------------------------------------------------------------------
*/




/*
*-----------------------------------------------------------------------
*   Gr_FreeBlocks()
*   This function is called when we are exiting from the GlobalRouter.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   represent solid blocks,bottleneck tiles and bottleneck lines
*   is being freed.
*------------------------------------------------------------------------
*/


void Gr_FreeBlocks()

{
    grBlock *gBlock,*nextBlock;

    if (grHeadBlockPtr!=NULL)
    {
        gBlock    = grHeadBlockPtr;
        nextBlock = gBlock->nextBlockPtr;
        while (nextBlock!=NULL)
        {
            GR_FREE((char*) gBlock);
            gBlock=nextBlock;
            nextBlock=nextBlock->nextBlockPtr;
        }
        GR_FREE((char*) gBlock);
        grHeadBlockPtr=NULL;
    }
}


/*
*-----------------------------------------------------------------------
*   Gr_FreeBnLines()
*   This function is called when we are exiting from the GlobalRouter.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   store  bottleneck lines dimensions and proper pointers
*   is being freed.
*------------------------------------------------------------------------
*/

void Gr_FreeBnLines()

{
    grBnLine *gLine,*nextLine;

    if (grBnLineList!=NULL)
    {
        gLine    = grBnLineList;
        nextLine = gLine->nextBnLine;
        while (nextLine!=NULL)
        {
            GR_FREE((char*) gLine);
            gLine=nextLine;
            nextLine=nextLine->nextBnLine;
        }
        GR_FREE((char*) gLine);
        grBnLineList=NULL;
    }
}


/*
*-----------------------------------------------------------------------
*   Gr_UnSet1Memor()
*   This function is called when we are exiting from the GlobalRouter.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   store keys to Blocks, bitmap of placement and virtual coordinates
*   is being freed.
*------------------------------------------------------------------------
*/



void Gr_UnSet1Memor()

{

    GR_FREE((char*) grKeyToBlocksHeadPtr);
    GR_FREE((char*) gr_Bitmap);
    GR_FREE((char*) gr_xcoord);
    GR_FREE((char*) gr_ycoord);

}

/*
*-----------------------------------------------------------------------
*   Gr_FreeAppartments()
*   This function is called when we are exiting from the GlobalRouter.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   represent virtual cells for maze-type search is being freed.
*-----------------------------------------------------------------------
*/


void Gr_FreeAppartments()

{
    GR_FREE((char*) gr_RoomPlan);
}

/*
*-----------------------------------------------------------------------
*   Gr_FreeNets()
*   This function is called when we are exiting from the GlobalRouter.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   represent nets is being freed.
*------------------------------------------------------------------------
*/



void Gr_FreeNets()

{
    GR_FREE((char*) gr_HeadKeyToNets);
    GR_FREE((char*) gr_HeadGrNets);
}



/*
*-----------------------------------------------------------------------
*   Gr_FreePins()
*   This function is called when we are exiting from the GlobalRouter.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   represent pins is being freed.
*------------------------------------------------------------------------
*/


void Gr_FreePins()

{
    grPin *gPin, *nextPin;

    if (gr_PinHeadPtr!=NULL)
    {
        gPin=gr_PinHeadPtr;
        nextPin=gPin->nextPtr;
        while (nextPin!=NULL)
        {
            GR_FREE((char*) gPin);
            gPin=nextPin;
            nextPin=gPin->nextPtr;
        }
        GR_FREE((char*) gPin);
    }
}


/*
*-----------------------------------------------------------------------
*   Gr_FreeSortedPins()
*   This function is called when we are exiting from the GlobalRouter.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   represent sorted pins is being freed.
*------------------------------------------------------------------------
*/




void Gr_FreeSortedPins()

{
    grSortedPin *gSPin, *nextSPin;

    if (gr_HeadSortedPinPtr!=NULL)
    {
        gSPin=gr_HeadSortedPinPtr;
        nextSPin=gSPin->nextOnList;
        while (nextSPin!=NULL)
        {
            GR_FREE((char*) gSPin);
            gSPin=nextSPin;
            nextSPin=gSPin->nextOnList;
        }
        GR_FREE((char*) gSPin);
    }
}



/*
*-----------------------------------------------------------------------
*   Gr_FreeAttachementsOfSubNet()
*   This function is called when we are exiting from the current
*   region and moving to the next one in the layer of hierarchy.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   represent some structures related to subNet is being freed.
*   The ones freed here are SuperNodes, RootTrees and VirtNets.
*   These structures do not have global lists.
*------------------------------------------------------------------------
*/



void Gr_FreeAttachementsOfSubNet(sNet)

grSubNet *sNet;

{
    grSuperNode *sNode,*lastNode;
    grRootTree  *rTree,*lastTree;
    grVirtNet   *vNet ,*lastNet;

    if (sNet!=NULL)
    {
        lastNode=sNet->NodeList;
        lastTree=sNet->tree;
        lastNet =sNet->children;

        if (lastNet!=NULL)
        {
            vNet=lastNet->nextInNet;
            while (vNet!=NULL)
            {
                GR_FREE((char*) lastNet);
                lastNet=vNet;
                vNet=lastNet->nextInNet;
            }
            GR_FREE((char*) lastNet);
        }
        if (lastTree!=NULL)
        {
            rTree=lastTree->nextOnList;
            while (rTree!=NULL)
            {
                GR_FREE((char*) lastTree);
                lastTree=rTree;
                rTree=lastTree->nextOnList;
            }
            GR_FREE((char*) lastTree);
        }
        if (lastNode!=NULL)
        {
            sNode=lastNode->nextOnList;
            while (sNode!=NULL)
            {
                GR_FREE((char*) lastNode);
                lastNode=sNode;
                sNode=lastNode->nextOnList;
            }
            GR_FREE((char*) lastNode);
        }
    }
}




/*
*-----------------------------------------------------------------------
*   Gr_FreeSubNets()
*   This function is called when we are exiting from the current
*   region and moving to the next one in the layer of hierarchy.
*   Results     : none.
*   Side effects: memory used internally by global router to 
*   represent subNets is being freed.
*------------------------------------------------------------------------
*/

void Gr_FreeSubNets(kPtr)

grRegion *kPtr;

{
    grSubNet *sNet,*last;

    if (kPtr!=NULL)
    {
        last=kPtr->subNetsPtr;
        if (last!=NULL)
        {
            sNet=last->nextOnList;
            while (sNet!=NULL)
            {
                Gr_FreeAttachementsOfSubNet(last);
                GR_FREE((char*) last);
                last=sNet;
                sNet=sNet->nextOnList;
            }
            GR_FREE((char*) last);
        }
    }
}



/*
 *-----------------------------------------------------------------------
 *  Gr_CleanOldPinRectangles()
 *
 *  This function is called when we are exiting from the current
 *  region and moving to the next one in the layer of hierarchy.
 *
 *  Results     : none.
 *  Side effects: memory used internally by global router to 
 *  represent PinRectangles is being freed.
 *------------------------------------------------------------------------
 */

void
Gr_CleanOldPinRectangles(kPtr)
grRegion *kPtr;
{
    grPinRectangle *current, *next;

    current=kPtr->firstPinRectanglePtr;
    while (current!=NULL) {
        next=current->nextOnList;
        GR_FREE((char *) current);
        current=next;
    }
    kPtr->firstPinRectanglePtr=NULL;
    kPtr->lastPinRectanglePtr=NULL;
}


/*----------------------------------------------------------------------
*   Gr_FreeRegionLists(kPtr)
*   This function is called when we are leaving the region
*   kPtr and before entering the next region.
*   Results     : none.
*   Side effects: list of subNets and pinRectangles of the
*   region pointed to by kPtr are freed.
*   Note, that kPtr still has pointers to other regions.
*   Memory pointed to by kPtr is not freed at this moment.
*-----------------------------------------------------------------------
*/


void Gr_FreeRegionLists(kPtr)

grRegion *kPtr;

{
    Gr_FreeSubNets(kPtr);
    Gr_CleanOldPinRectangles(kPtr);
}


/*
*-------------------------------------------------------------------------
*   Gr_FreeDeadRegions()
*   Results     : none.
*   Side effects    : grHeadDeadRegion  list is freed.
*-------------------------------------------------------------------------
*/



void Gr_FreeDeadRegions()

{
    grDeadRegion *next;

    while (grHeadDeadRegion != NULL)
    {
        next=grHeadDeadRegion->next;
        GR_FREE((char*) grHeadDeadRegion);
        grHeadDeadRegion = next;
    }
}
