/*
 * grHCount.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *	(Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint





#include "grInt.h"

/*
*-------------------------------------------------------------------
*	This file contains functions needed to calculate number of
*	holes along a cut line.
*------------------------------------------------------------------
*/







/*
*---------------------------------------------------------------------
*	Gr_IsHorzHole
*	Results      : 1 if a grid line between two cells in a bitmap
*			is passing through a hole;
*		       0  otherwise.
*	Side effects :  none.
*---------------------------------------------------------------------
*/



int Gr_IsHorzHole(left,right)

int left;		/* Position of left cell in Bitmap */
int right;		/* Position of right cell in Bitmap */


{
	/*
	   CHANGE SEPT 29 1989 
	if (((left==0)&&(right==0))||(Gr_IsHorzBottl(left)
		&&Gr_IsHorzBottl(right))||
	    (Gr_IsVerBottl(left))||(Gr_IsVerBottl(right)))
         */
	    if ((Gr_IsVerBottl(left))||(Gr_IsVerBottl(right)))
	{
		return (1);
	}
	else
	{
		return (0);
	}
}

/*
*------------------------------------------------------------------------
*	Gr_IsVertHole
*	Results		: 1 if a grid line between two cells in a bitmap
*			  is passing through a vertival hole.
*			  0 otherwise.
*	Side effects	: None.
*------------------------------------------------------------------------
*/



int Gr_IsVertHole(top,bot)

int top,bot;		/* Cell values in the bitmap		*/


{
	/*
	   CHANGE SEPT 29 1989 
	if (((top==0)&&(bot==0))||
	    (Gr_IsVerBottl(top)&&Gr_IsVerBottl(bot))||
	    (Gr_IsHorzBottl(top))||(Gr_IsHorzBottl(bot)))
         */
	    if ((Gr_IsHorzBottl(top))||(Gr_IsHorzBottl(bot)))
	{
	    return (1);
	}
	else
	{
		return (0);
	}
}

/*
*------------------------------------------------------------------------
*	Gr_VertHoleCount
*	Results		: number of holes along a vertical cut.
*	Side effects	: None.
*------------------------------------------------------------------------
*/



int Gr_VertHoleCount(kPtr)

grRegion *kPtr;		/* Pointer to the parent region.		*/


{
	int posx,posy1,posy2,j1,cleft,cright,lleft,lright,sum;
	grBlock *gBlock;

	posx=kPtr->xcutvirt;
	posy1=kPtr->ycutvirt;
	posy2=posy1+(kPtr->hvirt)-1;
	sum=0;
	lleft=0;
	lright=0;
	for (j1=posy1;j1<=posy2;j1++)
	{
		cleft=Gr_UpLeft(posx,j1);
		cright=Gr_UpRight(posx,j1);
		if (cleft!=cright)
		{
			gBlock=Gr_GetBlockPtrFromBitValue(cleft);
			if (gBlock != NULL)
			{
				if (gBlock->persona == 6)
				{
					cleft=0;
				}
			}
			gBlock=Gr_GetBlockPtrFromBitValue(cright);
			if (gBlock != NULL)
			{
				if (gBlock->persona == 6)
				{
					cright=0;
				}
			}
		}
		if (sum==0) /* no hole yet*/
		    {
			sum=sum+Gr_IsHorzHole(cleft,cright);
		}
		else
		{
			if ((cleft!=lleft)||(cright!=lright))
			    {
				sum=sum+Gr_IsHorzHole(cleft,cright);
			}
		}
		lleft=cleft;
		lright=cright;
		/*
		if (lleft!=lright)
		{
			gBlock=Gr_GetBlockPtrFromBitValue(lleft);
			if (gBlock != NULL)
			{
				if (gBlock->persona == 6)
				{
					lleft=0;
				}
			}
			gBlock=Gr_GetBlockPtrFromBitValue(lright);
			if (gBlock != NULL)
			{
				if (gBlock->persona == 6)
				{
					lright=0;
				}
			}
		}
		*/
	}
	/*
	printf("\n sum = %4d",sum);
	fflush(stdout);
	*/
	return (sum);
}




/*
*------------------------------------------------------------------------
*	Gr_HorzHoleCount
*	Results		: number of holes along a horizontal cut.
*	Side effects	: None.
*------------------------------------------------------------------------
*/




int Gr_HorzHoleCount(kPtr)

grRegion *kPtr;		/* Pointer to the parent region			*/


{
	int posx1,posx2,posy,j1,cleft,cright,lleft,lright,sum;
	grBlock *gBlock;

	posx1=kPtr->xcutvirt;
	posx2=posx1+(kPtr->whvirt)-1;
	posy=kPtr->ycutvirt;
	sum=0;
	lleft=0;
	lright=0;
	for (j1=posx1;j1<=posx2;j1++)
	{
		cleft=Gr_UpRight(j1,posy);
		cright=Gr_BotRight(j1,posy);
		if (cleft!=cright)
		{
			gBlock=Gr_GetBlockPtrFromBitValue(cleft);
			if (gBlock!=NULL)
			{
				if (gBlock->persona == 6)
				{
					cleft=0;
				}
			}
			gBlock=Gr_GetBlockPtrFromBitValue(cright);
			if (gBlock!=NULL)
			{
				if (gBlock->persona == 6)
				{
					cright=0;
				}
			}
		}
		if (sum==0)  /* no hole yet*/
		    {
			sum=sum+Gr_IsVertHole(cleft,cright);
		}
		else
		{
			if ((cleft!=lleft)||(cright!=lright))
			    {
				sum=sum+Gr_IsVertHole(cleft,cright);
			}
		}
		lleft=cleft;
		lright=cright;
		/* if (lleft!=lright)
		{
			gBlock=Gr_GetBlockPtrFromBitValue(lleft);
			if (gBlock!=NULL)
			{
				if (gBlock->persona == 6)
				{
					lleft=0;
				}
			}
			gBlock=Gr_GetBlockPtrFromBitValue(lright);
			if (gBlock!=NULL)
			{
				if (gBlock->persona == 6)
				{
					lright=0;
				}
			}
		} */
	}
	/*
	printf("\n sum = %4d",sum);
	fflush(stdout);
	*/
	return (sum);
}
