/*
 * grLayer.c.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *	(Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint



/*
*-------------------------------------------------------------------------
*	Functions in this file are used for breadth first search of
*	routing regions.
*------------------------------------------------------------------------
*/




#include "grInt.h"

/*
*-------------------------------------------------------------------------
*	Gr_InitializeLayer
*	Results		: none.
*	Side effects	: for the first region which is equal
*	to the entire chip it sets a layer in the hierarchy.
*--------------------------------------------------------------------------
*/


void Gr_InitializeLayer(kptr)

grRegion *kptr;		/* Pointer to the parent region			*/

{

	MALLOC(grLayer*,gr_HeadCurLayer,sizeof (grLayer));

	gr_HeadCurLayer->gRegion=kptr;
	gr_HeadCurLayer->nextInLayer=NULL;
	gr_TailCurLayer=gr_HeadCurLayer;

	/* no cut occured yet, so pending is head*/

	gr_Pending=gr_HeadCurLayer;

	/* pointers of the new layer are set to null*/

	gr_HeadNewLayer=NULL;
	gr_TailNewLayer=NULL;

}


/*
*--------------------------------------------------------------------------
*	Gr_CreateNewRegionsInLayer(kptr,par)
*	Results		: none.
*	Side effects	:
*	For the current region kptr which was just bisected it adds new
*	regions to the new layer when dir > 0.                         
*	If dir =0 it does nothing because kptr needs no longer be divided
*--------------------------------------------------------------------------
*/


void Gr_CreateNewRegionsInLayer(kptr,par)

grRegion *kptr;		/* Pointer to the parent region.		*/
int       par;		/* 0 : left and right subregions are present,
			   otherwise : children induced by dead regions	*/

{

	int       dir;
	grRegion *region;

	dir=kptr->dir;

	if (dir == 0)
	{
		return;
	}

	if (gr_TailNewLayer==NULL)
	{
		MALLOC(grLayer*,gr_TailNewLayer,sizeof (grLayer));

		if (par==0)
		{	
			gr_TailNewLayer->gRegion=kptr->leftPtr;
		}
		else
		{	
			gr_TailNewLayer->gRegion=kptr->childrenPtr;
		}	

		gr_TailNewLayer->nextInLayer=NULL;
		gr_HeadNewLayer=gr_TailNewLayer;
	}
	else
	{
		if (par==0)
		{
			MALLOC(grLayer*,gr_TailNewLayer->nextInLayer,
			sizeof (grLayer));
			gr_TailNewLayer=gr_TailNewLayer->nextInLayer;
			gr_TailNewLayer->gRegion=kptr->leftPtr;
			gr_TailNewLayer->nextInLayer=NULL;

		}
		else
		{
			MALLOC(grLayer*,gr_TailNewLayer->nextInLayer,
			sizeof (grLayer));
			gr_TailNewLayer=gr_TailNewLayer->nextInLayer;
			gr_TailNewLayer->gRegion=kptr->childrenPtr;
			gr_TailNewLayer->nextInLayer=NULL;
		}
	}
	if (par==0)
	{
			
		MALLOC(grLayer*,gr_TailNewLayer->nextInLayer,sizeof (grLayer));

		gr_TailNewLayer=gr_TailNewLayer->nextInLayer;
		gr_TailNewLayer->gRegion=kptr->rightPtr;
		gr_TailNewLayer->nextInLayer=NULL;
	}
	else
	{
		region=kptr->childrenPtr;
		if (region==NULL)
		{
			return;
		}
		region=region->siblingPtr;
		while (region!=NULL)
		{
			MALLOC(grLayer*,gr_TailNewLayer->nextInLayer,
			sizeof (grLayer));
			gr_TailNewLayer=gr_TailNewLayer->nextInLayer;
			gr_TailNewLayer->gRegion=region;
			gr_TailNewLayer->nextInLayer=NULL;
			region=region->siblingPtr;
		}
	}
}


/*
*-------------------------------------------------------------------------
*	Gr_NextInLayer()
*	Results		: pointer to a new region for partitioning.
*	The new region comes from the current layer of hierarchy
*	Side effects	: gr_Pending which points to the current
*	region in hierarchy is updated.
*	Also new hierarchy may be started.
*-------------------------------------------------------------------------
*/



grRegion *Gr_NextInLayer()

{

	grRegion *kptr;
	grLayer *run;

	if (gr_Pending->nextInLayer!=NULL)
	{
		gr_Pending=gr_Pending->nextInLayer;
		kptr=gr_Pending->gRegion;
	}
	else
	{
		/* new hierarchy starts*/

		gr_HeadCurLayer=gr_HeadNewLayer;
		gr_TailCurLayer=gr_TailNewLayer;
		gr_Pending=gr_HeadCurLayer;
		gr_HeadNewLayer=NULL;
		gr_TailNewLayer=NULL;

		if (gr_Timing > 0)
		{
			Gr_UpdateCriticalNetStatus();
		}

		if (gr_Pending!=NULL)
		{
			kptr=gr_Pending->gRegion;
		}
		else
		{
			kptr=NULL;
		}
	}
	return (kptr);
}

