/*
 * fileName.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *	(Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint



/*
*----------------------------------------------------------------------
*	Functions in this file are used for calculating and updating
*	net lengths when program runs in timing mode.
*----------------------------------------------------------------------
*/



#include "grInt.h"

/*
*----------------------------------------------------------------------
*	Gr_CalculateHorizontalPortionOfTree(pRect)
*	Results		: value of horizontal estimate of Steiner Tree which
*	contains all the pins of pRect.                                   
*-----------------------------------------------------------------------
*/



int Gr_CalculateHorizontalPortionOfTree(pRect)


grPinRectangle *pRect; 		/* Pointer to pinRectangle		*/

{

	int hdist,vdist,xpos;
	grSortedPin *gSPin;
	int count,xl,xr;

	hdist=pRect->maxx - pRect->minx;
	vdist=pRect->maxy - pRect->miny;

	if (hdist > vdist)
	{
		return (hdist);
	}
	else
	{
		xpos=0;
		gSPin=pRect->firstxPin;
		count=0;

		while (gSPin!=NULL)
		{
			xpos=xpos+gSPin->pinPtr->x;
			count++;
			gSPin=gSPin->nextxPin;
		}

		xl=0;
		xr=0;
		xpos=xpos/count;
		gSPin=pRect->firstxPin;

		while (gSPin!=NULL)
		{
			if (gSPin->pinPtr->x > xpos)
			{
				xl=xl+(gSPin->pinPtr->x - xpos);
			}
			else
			{
				xl=xl+(xpos-gSPin->pinPtr->x);
			}
			if (gSPin->pinPtr->x > (xpos+1))
			{
				xr=xr+gSPin->pinPtr->x - (xpos + 1) ;
			}
			else
			{
				xr=xr+(xpos+1) - gSPin->pinPtr->x;
			}
			gSPin=gSPin->nextxPin;
		}
		if (xr > xl)
		{
			return (xl);
		}
		else
		{
			return (xr);
		}
	}
}


/*
*----------------------------------------------------------------------
*	Gr_CalculateVerticalPortionOfTree(pRect)
*	Results		: value of vertical estimate of Steiner Tree which
*	contains all the pins of pRect.                                   
*-----------------------------------------------------------------------
*/



int Gr_CalculateVerticalPortionOfTree(pRect)


grPinRectangle *pRect;

{

	int hdist,vdist,ypos;
	grSortedPin *gSPin;
	int count,yt,yb;

	hdist=pRect->maxx - pRect->minx;
	vdist=pRect->maxy - pRect->miny;

	if (vdist >= hdist)
	{
		return (vdist);
	}
	else
	{
		ypos=0;
		gSPin=pRect->firstxPin;
		count=0;

		while (gSPin!=NULL)
		{
			ypos=ypos+gSPin->pinPtr->y;
			count++;
			gSPin=gSPin->nextxPin;
		}

		yb=0;
		yt=0;

		ypos=ypos/count;
		gSPin=pRect->firstxPin;

		while (gSPin!=NULL)
		{
			if (gSPin->pinPtr->y > ypos)
			{
				yb=yb+ (gSPin->pinPtr->y - ypos);
			}
			else
			{
				yb=yb+ (ypos - gSPin->pinPtr->y);
			}
			if (gSPin->pinPtr->y > (ypos+1))
			{
				yt=yt+ gSPin->pinPtr->y - (ypos+1);
			}
			else
			{
				yt=yt+ ypos+1 - gSPin->pinPtr->y;
			}
			gSPin=gSPin->nextxPin;
		}

		if (yt > yb)
		{
			return (yb);
		}
		else
		{
			return (yt);
		}
	}
}


/*
*-----------------------------------------------------------------------
*	Gr_SetBtnSizes(kPtr)
*	Results : none                                                       
*	Side effects : fields height and width in hole structures for the    
*	current cut are filled in.
*-----------------------------------------------------------------------
*/



void Gr_SetBtnSizes(kPtr)

grRegion *kPtr; 		/* Pointer to the parent region.	*/


{

	int dir,i;
	int xpos,ypos;
	grHole *gHole;

	dir=kPtr->dir;
	xpos=kPtr->xcutvirt;
	xpos=(gr_xcoord+xpos)->number;
	ypos=kPtr->ycutvirt;
	ypos=(gr_ycoord+ypos)->number;

	for (i=1;i<=n;i++)
	{
		gHole=(grHeadHolePtr+i);
			if (dir==1)
			{
				gHole->height= gHole->topOrRight_p_y
				    - gHole->botOrLeft_p_y;

                                gHole->width = gHole->rTBound
				    - gHole->bLBound;

			}

			if (dir==2)
			{
				gHole->height= gHole->topOrRight_p_x
				    - gHole->botOrLeft_p_x;

				gHole->width = gHole->rTBound
				    - gHole->bLBound;

			}
	
	}
}



/*
*-----------------------------------------------------------------------
*	Gr_UpdateBtnEstimates
*	Results		: none.
*	Side effects  :  
*	For each virtual net it updates the information regarding
*	lengths of maximal horizontal segments 
*	of a subnet in bottleneck and counts
*	# pins pulled to the right and left sides of btn . 
*	If procedure is called
*	for left node : par =0, if for right node par =1; 
*	if a net has only one
*	side, par = 0. lcount = # pins in the left node. 
*	If one node is present,
*	lcount =0 
*----------------------------------------------------------------------
*/



void Gr_UpdateBtnEstimates(kPtr,dir,where,whois,gNode,par,
lcount,totcount,hNumber)

grRegion    *kPtr;	/* Pointer to the parent region.	*/
int          dir;	/* Direction of a cut.			*/
grPrimit    *where;	/* Hole numbers for covered pins.	*/
grPrimit    *whois;	/* Positions of pins covered by holes	*/
grSuperNode *gNode;	/* Pointer to super node .		*/
int          par;	/* 0 left node, 1 right node		*/
int          lcount;	/* # pins on left side 			*/
int	     totcount;	/* # pins on both sides			*/
int	     hNumber;	/* hole number				*/

{
	int xpos,ypos,i,cur,est;
	grSortedPin  *gSPin;
        int cntr,ct1,ct2,ct3;


	if (hNumber == 0 )  /* no principle btn */
	{
		cntr=0;
	}
	else
	{
		ct1=0;
		ct2=0;
		ct3=0;
		for (i=1;i<=totcount;i++)
		{
			if ((whois+i)->number == 2)
			{
				ct1=1;   /* pin in principal btn */
			}
			if ((( i<=lcount)&&((whois+i)->number ==0))||
			((whois+i)->number ==3)) /* pulled to the left */
			{
				ct2 =1;
			}
			if (((i>lcount)&&((whois+i)->number ==0))||
			((whois+i)->number == 4)) /* pulled to the right/bot */
			{
				ct3=1;
			}
		}
		cntr=0;
		if (ct2*ct3 == 1)
		{
			cntr = 1;  /* pull to both sides */
		}
		if ((ct1==1)&&(ct2==1)&&(ct3==0))
		{
			cntr = 2;  /* pull left/top */
		}
		if ((ct1==1)&&(ct2==0)&&(ct3==1))
		{
			cntr = 3;  /* pull right/bot */
		}
		if ((cntr == 2)&&(par==0)&&(totcount>lcount))
		{
		for (i=(lcount+1);i<=totcount;i++)
			{
			if ((whois+i)->number ==2)
				{
					cntr =0;
				}
			}
		}
		if ((cntr==3)&&(par==1)&&(lcount>0))
		{
			for (i=1;i<=lcount;i++)
			{
				if ((whois+i)->number ==2)
				{
					cntr=0;
				}
			}
		}
	}

	i=0;

        if ( par !=0 )
        i=lcount;

	if ( dir == 1)
	{
		gSPin=gNode->contentsPtr->firstxPin;
	}
	if ( dir == 2)
	{
		gSPin=gNode->contentsPtr->firstyPin;
	}
	while (gSPin != NULL )
	{
		i++;
		/* check where is the pin */
	 	xpos = gSPin->pinPtr->x;
		ypos = gSPin->pinPtr->y;

		cur=(where+i)->number;
		if (cur > 0)
		{
			if (dir==1)
			{
	   			if ((grHeadHolePtr+cur)->topOrRight_p_y==ypos)
	   			{
					(grHeadHolePtr+cur)->rtpins =1;
	   			}
	   			else
	   			{
					(grHeadHolePtr+cur)->lbpins =1;
	   			}
			}
			if (dir == 2)
			{
				if ((grHeadHolePtr+cur)->topOrRight_p_x == xpos)
				{
					(grHeadHolePtr+cur)->rtpins =1;
				}
				else
				{
					(grHeadHolePtr+cur)->lbpins =1;
				}
			}
			if ((whois+i)->number ==3)  /*pull to the left or top */
			{
				if (dir ==1)
				{
					est=xpos-(grHeadHolePtr+cur)->bLBound;
				}
				if (dir ==2)
				{
					est=(grHeadHolePtr+cur)->rTBound - ypos;
				}
	   			if ((grHeadHolePtr+cur)->leftEst < est)
	    			{
					(grHeadHolePtr+cur)->leftEst =est;
	    			}
			}
			if ((whois+i)->number == 4)   /*pull to right or bot */
			{
				if (dir ==1)
				{
					est=(grHeadHolePtr+cur)->rTBound - xpos;
				}
				if (dir ==2)
				{
					est=ypos-(grHeadHolePtr+cur)->bLBound;
				}
				if ((grHeadHolePtr+cur)->rightEst < est)
				{
					(grHeadHolePtr+cur)->rightEst  = est;
				}
			}
			if ((whois+i)->number ==2)   /* principal btn*/
			{
				if (dir == 1)
				{
					if (cntr == 2)  /* pull left/top only */
					{
						if ((grHeadHolePtr+cur)->leftEst<
					      xpos-(grHeadHolePtr+cur)->bLBound)
						{
						   (grHeadHolePtr+cur)->leftEst=
					 	      xpos-(grHeadHolePtr+cur)->
							bLBound;
						}
					}
					if (cntr == 3)  /* pull right/bot only */
					{
						if ((grHeadHolePtr+cur)->rightEst
						<(grHeadHolePtr+cur)->rTBound
						 - xpos )
						{
						   (grHeadHolePtr+cur)->rightEst
						   =(grHeadHolePtr+cur)->rTBound
							 - xpos; 
						}
					}
				}
				if (dir == 2)
				{
					if (cntr ==2 )
					{
						if ((grHeadHolePtr+cur)->leftEst 
						< (grHeadHolePtr+cur)->
						rTBound - ypos)
						{
							(grHeadHolePtr+cur)->
							leftEst =
					 		(grHeadHolePtr+cur)->
							rTBound - ypos;
						}

					}
					if (cntr == 3 ) /* pull right/bot only */
					{
						if ((grHeadHolePtr+cur)->rightEst
						< ypos-(grHeadHolePtr+cur)->
						bLBound)
						{
							(grHeadHolePtr+cur)->
							rightEst =
					  		ypos-(grHeadHolePtr+cur)
							->bLBound ;
						}
					}
				}
			}

		}
		if (dir==1)
		{
			gSPin=gSPin->nextxPin;
		}
		if (dir ==2)
		{
			gSPin=gSPin->nextyPin;
		}
	}
	if ((par ==0)&&(cntr == 1))
	{

		(grHeadHolePtr+hNumber)->leftEst =
		(grHeadHolePtr+hNumber)->width;
	}
}


/*
*--------------------------------------------------------------------------
*	Gr_UpdateCriticalNetStatus()
*	Results		: none.
*	Side effects	: based on current approximation of critical
*			  net length, their status is changed.
*---------------------------------------------------------------------------
*/



void Gr_UpdateCriticalNetStatus()

{

	int i;
	grNet *gNet;
	float clen,maxlen;

	for (i=1;i<=gr_NetCount;i++)
	{
		gNet=(gr_HeadGrNets+i);

		if (gNet->critical >0)
		{
			clen=gNet->currentLength;
			maxlen=gNet->maxlength;

			if (3*clen <=maxlen)
			{
				gNet->critical=1;
			}
			else
			{
				if (2*clen<=maxlen)
				{
					gNet->critical=2;
				}
				else
				{
					if (clen<=maxlen)
					{
						gNet->critical=3;
					}
					else
					{
						gNet->critical=4;
					}
				}
			}
		}
	}
}



/*
*--------------------------------------------------------------------------
*	Gr_UpdateHoleBoundsForCriticalNets(dir)
*	Results		: none.
*	Side effects	: bounding holes for critical nets are updated	
*--------------------------------------------------------------------------
*/


void Gr_UpdateHoleBoundsForCriticalNets(dir)

int dir;	/* Pointer to the parent region	.		*/

{
	grHole *gHole;
	int i,j,center,bot,top,flag,low,high;
	grNet *gNet;

	if (dir>0)
	{
		for (i=1;i<=gr_NetCount;i++)
		{
			low=0;
			high=0;
			flag=0;
			gNet=(gr_HeadGrNets+i);
			if (gNet->critical > 0)
			{
				if (dir==1)
				{
					bot=gNet->yBor;
					top=gNet->yTor;
				}
				else
				{
					bot=gNet->xLor;
					top=gNet->xRor;
				}


				for (j=1;j<=n;j++)
				{
					gHole=(grHeadHolePtr+j);

					if (dir==1)
					{
						center=(gHole->topOrRight_p_y
						 - gHole->botOrLeft_p_y)/2 
						+ gHole->botOrLeft_p_y ;
					}
					else
					{
						center=gHole->botOrLeft_p_x+
						(gHole->topOrRight_p_x 
						- gHole->botOrLeft_p_x)/2;
					}

					if (flag==0)  /*bottom checked*/
					{
						if (bot>=center)
						{
							low=j;
						}
						else
						{
							flag=1;
							if (top>=center)
							{
								high=j;
							}
						}
					}
					else
					{
						if (top>=center)
						{
							high=j;
						}
						else
						{
							goto lab20;
						}
					}
				}
lab20:;
				gNet->botHole=low+1;
				gNet->topHole=high;
			}
		}
	}
}

/*
*------------------------------------------------------------------------
*	Gr_AddFixedLengthsForBtnNets
*	Results : none .
*	Side effects	:for nets which are completely covered by one btn
*	it adds back their approximate lengths which are in fact their
*	fixed lengths.
*-------------------------------------------------------------------------
*/



void Gr_AddFixedLengthsForBtnNets(sNet)

grSubNet *sNet;		/* Pointer to a subnet.		*/

{



	(gr_HeadGrNets+sNet->netname)->xfixed = 
	Gr_CalculateHorizontalPortionOfTree(sNet->NodeList->contentsPtr);

	(gr_HeadGrNets+sNet->netname)->yfixed = 
	Gr_CalculateVerticalPortionOfTree(sNet->NodeList->contentsPtr);

	(gr_HeadGrNets+sNet->netname)->xapprox =0;
	(gr_HeadGrNets+sNet->netname)->yapprox =0;

	(gr_HeadGrNets+sNet->netname)->currentLength =
	(gr_HeadGrNets+sNet->netname)->xfixed +
	(gr_HeadGrNets+sNet->netname)->yfixed ;

}
