/*
 * grLook.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *	(Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint




#include "grInt.h"



/*
*--------------------------------------------------------------------------
*	Functions in this file are used when internal blockage has been
*	detected.
*-------------------------------------------------------------------------
*/




/*
*---------------------------------------------------------------------------
*	Gr_MkePullVirtualNets(kPtr)
*	Results		: none.
*	Side effects	:
*	For each net resident in kPtr it either builds virtual nets
*	for assignment
*	or pushes the net to the  child region of the region pointed
*	to by kPtr.
*	The total number of virtual nets : m is calculated here.
*--------------------------------------------------------------------------
*/



void Gr_MkePullVirtualNets(kPtr)

grRegion *kPtr;		/* Pointer to the current region.		*/


{

	int dir;
	grSubNet *sNet;
	grDeadRegion *dead;
	int bLHole,rTHole;



	m=0;  /* initially the number of virtual nets is 0*/
	dir=kPtr->dir;

	/*
	printf("\n before PullDecompose");
	Gr_PrintSubNets(kPtr);
	*/

	Gr_PullDecomposeNets(kPtr); /* nets are divided at cut line*/

	/*
	printf("\n  After PullDecompose");
	Gr_PrintSubNets(kPtr);
	*/

	if (n >= 1)  /* more than on hole */
	{
		sNet=kPtr->subNetsPtr;

		while (sNet!=NULL)
		{
			sNet->toBeConnected=NULL;
			sNet->areConnected=NULL;

			if ((sNet->active==1)||(sNet->active==9)
			||(sNet->active==10))  /* net needs a pull*/
			{
				/* no decomposition*/
				Gr_CreateSimpleVirtualNet(sNet);
				if (sNet->active==1)
				{
					sNet->active=5; 
				}
				m++;
			} /* end net needs assignment*/


			if (sNet->active==3) /*net sits on the left side*/
			{
				bLHole=0;
				rTHole=0;
				dead=grHeadDeadRegion;
				while (dead!=NULL)
				{
				if (Gr_IsPinRectangleFullyContainedInDeadRegion
				(sNet->NodeList->contentsPtr,dead,kPtr->dir)
				==1)
				{
					/*
					bLHole=dead->botLeftHoleNr;
					rTHole=dead->topRightHoleNr;
					*/
					if (kPtr->dir == 1)
					{
					bLHole=Gr_GetFirstHoleAbove(dead->
					ybot);
					rTHole=Gr_GetLastHoleBelow(dead->
					ytop);
					}
					else
					{
					bLHole=Gr_GetFirstHoleRight(dead->
					xleft);
					rTHole=Gr_GetLastHoleLeft(dead->
					xright);
					}
					goto lab10;
				}
				dead=dead->next;
				}
				lab10:;
				Gr_PrepareNetForDrop(kPtr,sNet,dir,0,bLHole,
				rTHole);
				if (sNet->NodeList->contentsPtr->count!=0)
				{
					Gr_DropNetToOneRegion(sNet,
					dead->region);
				}
				sNet->active=8;
			}
			if (sNet->active==4) /*net sits on the right side*/
			{
				bLHole=0;
				rTHole=0;
				dead=grHeadDeadRegion;
				while (dead!=NULL)
				{
				if (Gr_IsPinRectangleFullyContainedInDeadRegion
				(sNet->NodeList->contentsPtr,dead,kPtr->dir)
				==1)
				{
					/*
					bLHole=dead->botLeftHoleNr;
					rTHole=dead->topRightHoleNr;
					*/
					if (kPtr->dir == 1)
					{
					bLHole=Gr_GetFirstHoleAbove(dead->
					ybot);
					rTHole=Gr_GetLastHoleBelow(dead->
					ytop);
					}
					else
					{
					bLHole=Gr_GetFirstHoleRight(dead->
					xleft);
					rTHole=Gr_GetLastHoleLeft(dead->
					xright);
					}
					goto lab11;
				}
				dead=dead->next;
				}
				lab11:;
				Gr_PrepareNetForDrop(kPtr,sNet,dir,1,bLHole,
				rTHole);
				if ((sNet->NodeList->contentsPtr->count) !=0)
				{
					Gr_DropNetToOneRegion(sNet,
					dead->region);
				}
				sNet->active=8;
			}
		Gr_NameSuperNodesAfterVirtualNetsWereCreated(sNet);
                sNet=sNet->nextOnList;
                } /*end while*/
	} /* end  hole or holes occur */


}





/*
*------------------------------------------------------------------------
*	Gr_PullDecomposeNets(kPtr)
*	Side effects	: performes first cut of nets after cut position and 
*	direction have been determined.
*	The field "active" in grSubNet-s is filled out and if a net is
*	separated by the cut line the new grPinRectangles are attached to
*	the "decomposed" field and super nodes are created and attached to  
*	"NodeList".                     
*-------------------------------------------------------------------------
*/



void Gr_PullDecomposeNets(kPtr)

grRegion *kPtr;		/* Pointer to the parent region.		*/

{


	int dir,cutPos;
	grSubNet *pending;

	dir=kPtr->dir;
	if (dir!=0)
	{
		if (dir==1)
		{
			cutPos=kPtr->xcutvirt;
			cutPos=(gr_xcoord+cutPos)->number;
		}
		if (dir==2)
		{
			cutPos=kPtr->ycutvirt;
			cutPos=(gr_ycoord+cutPos)->number;
		}

		pending=kPtr->subNetsPtr;

		while (pending!=NULL)
		{
			Gr_PullCreateInitialSuperNodes(pending,cutPos,dir);

			pending=pending->nextOnList;
		}
	}
}




/*
*---------------------------------------------------------------------------
*	Gr_PullCreateSimpleVirtualNet
*	Results		: none.
*	Side effects	:
*	If there are only 2 superNodes on the NodeList then they induce a simple
*	virtual net
*----------------------------------------------------------------------------
*/



void Gr_PullCreateSimpleVirtualNet(sNet)

grSubNet *sNet;		/* Pointer to a sub net.		*/

{
	grSuperNode *current;
	grVirtNet *vNet;

        vNet= Gr_CreateNewVirtNet(sNet);

	current=sNet->NodeList;

	if (current->side == 1)
	{
		vNet->leftNode=current;
		vNet->rightNode=current->nextOnList;
	}
	else
	{
		vNet->rightNode = current;
		vNet->leftNode = current->nextOnList;
	}

	vNet->netName = sNet->netname;
	vNet->subNetName=sNet->subname;
	m++;
	vNet->leftNode->nodeName=sNet->subname; /*left side gets the old name*/
	vNet->rightNode->nodeName=Gr_GetNewSubNetName();
}


/*
*--------------------------------------------------------------------------
*	Gr_PullProcessAPinRectangle
*	Results		: none.
*	Side effects	:
*	the currentPinRectangle is cut along cutPos
*	in direction specified by dir.
*--------------------------------------------------------------------------
*/



void  Gr_PullProcessAPinRectangle(subNetPtr,currentPinRectangle,cutPos,dir)

grSubNet  	*subNetPtr;		/*Pointer to a subNet.		*/
grPinRectangle  *currentPinRectangle;	/*Pointer to pinRectangle.	*/
int 		 cutPos;		/*Position of a cut.		*/
int		 dir;			/*Direction of a cut.		*/



{
	grPinRectangle  *leftPinRectangle, *rightPinRectangle;
	grSuperNode *leftNodePtr, *rightNodePtr;


	if (dir==0) 
	{
		return ;
	}
	if (currentPinRectangle == NULL) 
	{
		return ;
	}

	if (currentPinRectangle->cutMark==1)  /*real intersection*/
	{
		leftPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
		Gr_AttachPinRectangleToDecomposed(subNetPtr,leftPinRectangle);
		rightPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
		Gr_AttachPinRectangleToDecomposed(subNetPtr,rightPinRectangle);


		/*vertical cut  */

		if (dir==1)
		{
			Gr_XHardSplitPinRectangle(cutPos,currentPinRectangle,
			leftPinRectangle,rightPinRectangle);
                	Gr_UpdateBoundingBox(leftPinRectangle);
                	Gr_UpdateBoundingBox(rightPinRectangle);
		}

		/*horizontal cut */

		if (dir==2)
		{
			Gr_YHardSplitPinRectangle(cutPos,currentPinRectangle,
			leftPinRectangle,rightPinRectangle);
                	Gr_UpdateBoundingBox(leftPinRectangle);
                	Gr_UpdateBoundingBox(rightPinRectangle);
		}

		leftNodePtr = Gr_AllocateMemoryForNewSuperNode(subNetPtr,
		subNetPtr->NodeList);
		subNetPtr->NodeList = leftNodePtr;
		leftNodePtr->contentsPtr = leftPinRectangle;
		leftNodePtr->kind = 1;
		leftNodePtr->nodeName = currentPinRectangle->name;
		leftPinRectangle->cutMark =0;
		leftNodePtr->side = 1;

		rightNodePtr = Gr_AllocateMemoryForNewSuperNode(subNetPtr,
		subNetPtr->NodeList);
		subNetPtr->NodeList = rightNodePtr;
		rightNodePtr->contentsPtr = rightPinRectangle;
		rightNodePtr->kind =1;
		rightNodePtr->nodeName = currentPinRectangle->name;
		rightNodePtr->side = 2;
		rightPinRectangle->cutMark = 0;

		return;
	}  /* end real intersection for a hierarchical cut line */

	if ((currentPinRectangle->cutMark ==4)
	||(currentPinRectangle->cutMark ==2)
	||(currentPinRectangle->cutMark ==6))
	{
		/*right list only*/
		rightPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
		Gr_AttachPinRectangleToDecomposed(subNetPtr,rightPinRectangle);
		Gr_CopyPinRectangle(currentPinRectangle,rightPinRectangle);
		rightNodePtr = Gr_AllocateMemoryForNewSuperNode(subNetPtr,
		subNetPtr->NodeList);
		subNetPtr->NodeList = rightNodePtr;
		rightNodePtr->contentsPtr = rightPinRectangle;
		rightNodePtr->kind = 1;
		rightNodePtr->nodeName = currentPinRectangle->name;
		rightPinRectangle->cutMark =0;
		rightNodePtr->side = 2;

		if (currentPinRectangle->cutMark==6)
		{
			leftPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
			Gr_AttachPinRectangleToDecomposed(subNetPtr,
			leftPinRectangle);
			leftNodePtr =
			Gr_AllocateMemoryForNewSuperNode(subNetPtr,
			subNetPtr->NodeList);
			subNetPtr->NodeList = leftNodePtr;
			leftNodePtr->contentsPtr = leftPinRectangle;
			leftNodePtr->kind = 1;
			leftNodePtr->nodeName = currentPinRectangle->name;
			leftPinRectangle->cutMark =0;
			leftNodePtr->side = 1;
		}

		return;
	}  /* end right list only */

	if ((currentPinRectangle->cutMark ==5)||
	(currentPinRectangle->cutMark==3)
	||(currentPinRectangle->cutMark==7))
	{
		/* left list only */
		leftPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
		Gr_AttachPinRectangleToDecomposed(subNetPtr,leftPinRectangle);
		Gr_CopyPinRectangle(currentPinRectangle,leftPinRectangle);
		leftNodePtr= Gr_AllocateMemoryForNewSuperNode(subNetPtr,
		subNetPtr->NodeList);
		subNetPtr->NodeList = leftNodePtr;
		leftNodePtr->contentsPtr = leftPinRectangle;
		leftNodePtr->kind = 1;
		leftNodePtr->nodeName = currentPinRectangle->name;
		leftPinRectangle->cutMark =0;
		leftNodePtr->side =1;

		if (currentPinRectangle->cutMark==7)
		{
			rightPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
   			Gr_AttachPinRectangleToDecomposed(subNetPtr,
			rightPinRectangle);
			rightNodePtr =
			Gr_AllocateMemoryForNewSuperNode(subNetPtr,
			subNetPtr->NodeList);
			subNetPtr->NodeList = rightNodePtr;
			rightNodePtr->contentsPtr = rightPinRectangle;
			rightNodePtr->kind =1;
			rightNodePtr->nodeName = currentPinRectangle->name;
			rightNodePtr->side = 2;
			rightPinRectangle->cutMark = 0;
		}
		return ;
	} /* end left side only */
}




/*
*----------------------------------------------------------------------------
*	Gr_PullCreateInitialSuperNodes(subNetPtr,cutPos,dir)
*	Results		: none.
*	Side effects 	: for a subnet, cutposition and direction it creates 
*	a list of initial super nodes.                  
*
*	REMARK : cut position must be in real dimensions, not virtual 
*	if dir=0 no cut; dir=1 vertical cut; dir=2 horizontal cut.   
*
*	Status of net activity is determined.
*---------------------------------------------------------------------------
*/


void Gr_PullCreateInitialSuperNodes(subNetPtr,cutPos,dir)

grSubNet *subNetPtr;
int cutPos,dir;

{
	grPinRectangle *currentPinRectangle;
	int cojest;

	cojest=0;
	currentPinRectangle=subNetPtr->toBeConnected;
	if (dir==1) 
	{
		Gr_MarkVertIntersectOfPinRectangle(currentPinRectangle,cutPos);
	}
	if (dir==2) 
	{
		Gr_MarkHorzIntersectOfPinRectangle(currentPinRectangle,cutPos);
	}

	if (currentPinRectangle->cutMark !=1)
	{
		Gr_CheckIfPullNeeded(currentPinRectangle,dir);
	}

	if (currentPinRectangle!=NULL)
	{
		if (currentPinRectangle->cutMark ==1) 
		{
			cojest = 1;
		}
		if (currentPinRectangle->cutMark ==4) 
		{
			cojest = 4;
		}
		if (currentPinRectangle->cutMark ==5) 
		{
			cojest = 3;
		}
		if (currentPinRectangle->cutMark ==6) 
		{
			cojest = 10;
		}
		if (currentPinRectangle->cutMark ==7) 
		{
			cojest = 9;
		}

		(void) Gr_PullProcessAPinRectangle(subNetPtr,
		currentPinRectangle,cutPos,dir);
	}

	subNetPtr->active = cojest;
}
