/*
 * grLookSupport.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *	(Advising Professor - E. S. Kuh)
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint



#include "grInt.h"


/*
*-------------------------------------------------------------------------
*	Functions in this file are utilities for pulling out nets from
*	dead regions.
*------------------------------------------------------------------------
*/

/*
*----------------------------------------------------------------------
*	Gr_CheckIfPullNeeded
*	Results		: none.
*	Side effects	:
*	If pinRectangle sits on one side of a cutline
*	and deadRegions are present, then it 
*	changes cutMarks if pinRectangle sits in
*	two or more deadRegions.
*	cutMark=6 if pinRectangle sits on right
*	side of a cutline in at least 2 deadRegions.
*	cutMark=7 if pinRectangle sits on left
*	in at least 2 deadRegions.                  
*----------------------------------------------------------------------
*/

void Gr_CheckIfPullNeeded(pinRectanglePtr,dir)

grPinRectangle *pinRectanglePtr;	/* Pointer to pinRectangle.	*/
int             dir;			/* Cut direction .		*/


{

	int           first,second,now;
	grDeadRegion *current;
	
	first=0;
	second=0;
	current=grHeadDeadRegion;
	
	if ((pinRectanglePtr->cutMark==4)||(pinRectanglePtr->
	cutMark==2))
	{
	while (current!=NULL)
	{
		if (current->side==1)
		{
			now=Gr_PinRectangleInDeadRegion
			(pinRectanglePtr,current,dir);
			if (now==1)
			{
				if (first==0)
				{
					first=1;
				}
				else
				{
					second=1;
				}
			}
			if ((first+second==2)) goto lab1;
		}
		current=current->next;
	}
	lab1: ;
	if ((first+second)==2)
	{
		pinRectanglePtr->cutMark=6;
	}
	}
	else
	{
	while (current!=NULL)
	{
		if (current->side==0)
		{
			now=Gr_PinRectangleInDeadRegion(pinRectanglePtr,
			    current,dir);
			if (now==1)
			{
				if (first==0)
				{
					first=1;
				}
				else
				{
					second=1;
				}
			}
			if ((first+second==2)) goto lab2;
		}
		current=current->next;
	}
	lab2: ;
	if ((first+second)==2)
	{
		pinRectanglePtr->cutMark=7;
	}
	}
}




/*
*-------------------------------------------------------------------------
*	Gr_IsPinInDeadRegion
*	Results		: if gSPin is  
*	in a deadRegion then 1 is returned, otherwise 0. 
*	Side effects	: none.
*-------------------------------------------------------------------------
*/



int Gr_IsPinInDeadRegion(gSPin,deadRegionPtr,dir)

grSortedPin      *gSPin;		/* Pointer to sorted pin.	*/
grDeadRegion     *deadRegionPtr;	/* Pointer to dead region.	*/
int               dir;			/* Direction.			*/



{

	int          xleft,xright,ytop,ybot;
	int          xpin,ypin;
	grPin       *pin;
 

	xleft=deadRegionPtr->xleft;
	xright=deadRegionPtr->xright;
	ytop=deadRegionPtr->ytop;
	ybot=deadRegionPtr->ybot;
	pin=gSPin->pinPtr;

		xpin=pin->x;
		ypin=pin->y;

		if (( xpin < xleft)||(xpin > xright))
		{
			return (0);
		}

		if (( ypin < ybot)||(ypin > ytop))
		{
			return (0);
		}

		if ((xpin < xright)&&(xpin > xleft)
		&&(ypin > ybot)&&(ypin < ytop))
		{
			return (1);
		}
		else
		{
			if (dir ==1)
			{
				if (ypin==ytop)
				{
					if (pin->shade!=NORTH_PIN)
					{
						return (1);
					}
				}
				if (ypin==ybot)
				{
					if (pin->shade!=SOUTH_PIN)
					{
						return (1);
					}
				}
				if ((xpin==xleft)||(xpin==xright))
				{
					return (1);
				}
			}
			if (dir==2)
			{
				if (xpin==xleft)
				{
					if (pin->shade!=WEST_PIN)
					{
						return (1);
					}
				}
				if (xpin==xright)
				{
					if (pin->shade!=EAST_PIN)
					{
						return (1);
					}
				}
				if ((ypin==ybot)||(ypin==ytop))
				{
					return (1);
				}
			}
		}
	return (0);
}
	

/*
*----------------------------------------------------------------------------
*	Gr_PinRectangleInDeadRegion
*	Results		: 1
*			    if PinRectangle has at least one pin in deadRegion;
*			  0 otherwise .
*	Side effects	: none.
*-----------------------------------------------------------------------------
*/



int Gr_PinRectangleInDeadRegion(pinRectanglePtr,deadRegionPtr,dir)

grPinRectangle    *pinRectanglePtr;	/* Pointer to pinRectangle.	*/
grDeadRegion      *deadRegionPtr;	/* Pointer to dead region.	*/
int                dir;			/* Cut direction.		*/



{
	grSortedPin  *gSPin;

	gSPin=pinRectanglePtr->firstxPin;
	while (gSPin!=NULL)
	{
		if (Gr_IsPinInDeadRegion(gSPin,deadRegionPtr,dir)==1)
		{
			return (1);
		}
		gSPin=gSPin->nextxPin;
	}
	return (0);

}




/*
*-------------------------------------------------------------------------
*	Gr_IsPinRectangleFullyContainedInDeadRegion
*	Results		: 1 if pinRectangle sits inside deadRegion 
*				and nowhere else.
*			  0  otherwise. 
*	Side effects	: none.
*--------------------------------------------------------------------------
*/


int Gr_IsPinRectangleFullyContainedInDeadRegion(pinRectanglePtr,
deadRegionPtr,dir)

grPinRectangle *pinRectanglePtr;	/* Pointer to pin rectangle.	*/
grDeadRegion   *deadRegionPtr;		/* Pointer to dead region.	*/
int             dir;			/* Cut direction.		*/


{
	grSortedPin  *gSPin;
	
	gSPin=pinRectanglePtr->firstxPin;
	while (gSPin!=NULL)
	{
		if (Gr_IsPinInDeadRegion(gSPin,deadRegionPtr,dir)==0)
		{
			return (0);
		}
		gSPin=gSPin->nextxPin;
	}
	return (1);
}
