
/* grMain.c --
 *
 *     Malgorzata Marek-Sadowska
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "grInt.h"


/* --------- Internal global variables for the globalRouter module --------- */

bool gr_FreeMemFlag;
bool grPrintFlag;

int grSolidTileCount; /*number of solid tiles seen by the global router*/
int grHorzBottlNumber; /*number of Horz bottlenecks*/
int grVertBottlNumber; /*number of Vert bottlenecks*/
int grBottleneckTileCount; /*the last name of bottleneck tile */
int bnLineCount; /* number of bottleneck lines*/
int hbnLCount;   /* number of horz bottleneck lines*/
int vbnLCount;   /* number of vert bottleneck lines*/
Rect *planeBndPtr; /* pointer to external boundary*/
int gr_xShift; /* x-shift so left bottom corner of boundary is at 0,0*/
int gr_yShift; /* y-shift*/
grBlock *grHeadBlockPtr; /*pointer to the first block on the list of */
                         /*solid blocks*/
grBlock *grLastBlockPtr; /*pointer to the last solid block*/
grBnLine *grBnLineList; /* pointer to the list of bottleneck lines*/
grKeyBlock  *grKeyToBlocksHeadPtr; /* pointer to key to blocks*/
grPrimit *gr_Bitmap; /* pointer to the bitmap*/
grPrimit *gr_xcoord; /* x virtual coordinates */
grPrimit *gr_ycoord; /* y virtual coordinates */
int gr_totxgrid;    /* total number of x coordinates induced by blocks*/
int gr_totygrid;   /* total number of y coordinates induced by blocks*/
grRoom *gr_RoomPlan; /* pointer to the list of rooms*/
grRegion *grHeadRegionPtr;/* ptr to the head of regions*/
grRegion *grTailRegionPtr;/* ptr to the tail of regions*/
int m;  /* number of virtual nets for assignment in each region*/
int n; /* number of holes for assignment in each region*/
int neff; /* masking number of holes*/
grHole *grHeadHolePtr; /*ptr to head of array of holes, n+1 of them*/
grIndex *vNetHead; /*ptr to array of pointers to virtual nets*/
int gr_horzDesRule; /* temporary horz des rule*/
int gr_vertDesRule; /* they are the same vert des rule*/
float gr_congestionFactor;
grPrimit *mback,*map,*grACap,*cap,*c,*row,*col,*nexts;
grPrimit *h1,*h2,*h3,*h4,*h5,*h6;
int maxint=1000000;

PlaneData   *grPlaneDataPtr;
CellMaster  *grCellMasterPtr;

char debug;  /* used for debugging printouts*/

/* Global variables for layers of hierachy. */

grLayer *gr_HeadCurLayer;
grLayer *gr_TailCurLayer;
grLayer *gr_Pending;
grLayer *gr_HeadNewLayer;
grLayer *gr_TailNewLayer;

/* Global variables for timing */

int gr_Timing; /*0 regular mode, 1 timing mode */
int gr_BearConnection; /* if = 1 connected to bear, 0 stand alone*/

float gr_HorVertWire; /* a constant to multiply vert wire length to normalize
                         it to horizontal wire */
float gr_CongestionFactor; /* a parameter controlling overflow of bottlenecks
             if = 0, no overflow, capacities are taken as seen,
             if = 1 - capacites are doubled*/


/*
 * Here are global variables for net initialization.
 */

int gr_TotalNetCount; /* number of all the nets present,also prerouted*/

int gr_NetCount; /*number of nets which need to be connected,set to 0 */
/*in GrInitializeNets()                             */

grPin *gr_PinHeadPtr; /*pointer to the list of grPin, set to NULL in  */
/*GrInitializeNets()                           */

grPin *gr_LastPinPtr; /*pointer to the last item on the list of grPin */

grKeyToNets *gr_HeadKeyToNets; /*pointer to head of array containing  */
/*old and new names of nets            */
/* initialized to size gr_TotalNetCount in Gr_Set2Memor()             */

grNet *gr_HeadGrNets; /*pointer to the head of grNet list:array of    */
/*gr_TotalNetCount initialized in Gr_Set2Memor()*/

grSortedPin *gr_HeadSortedPinPtr;/*pointer to the head of grSortedPin */
/*list initialized in Gr_Set2Memor() to NULL                          */

grSortedPin *gr_LastSortedPinPtr;/*pointer to the last item on        */
/*of grSortedPin list initialized in Gr_Set2Memor() to NULL           */


grPinRectangle *gr_HeadPinRectanglePtr;
/*pointer to the beginning of the list of grPinRectangle initialized  */
/*NULL in Gr_Set2Memor()                                              */

grPinRectangle *gr_LastPinRectanglePtr;
/*pointer to the last item on the list of grPinRectangle initialized to*/
/*NULL in Gr_Set2Memor()                                               */

grPin *gr_HeadLastToSort; /*pointer to the last segment of not yet
                             sorted grPins */

grDeadRegion  *grHeadDeadRegion; /* pointer to first deadRegion */

/* This variable added  to make the program run as stand alone */


static BearStatus
DoBottleNeck(bnPtr, clientData)
BottleNeck  *bnPtr;
ClientData  clientData;
{
    if (bnPtr->unType == TP_BN_TILE) {
        Tile    *bnTilePtr;
        int     tileCap, tileDen;
        int     delta;

        bnTilePtr = bnPtr->unValue.bnTilePtr;

/*
 *      tileCap = Tp_GetCapacityOfTile(bnTilePtr);
 *      tileDen = Tp_GetDensityOfTile(bnTilePtr);
 *      delta = tileCap - tileDen;
 *      if (delta <= 0) {
 *          (void) sprintf (bear_ErrorMsg,
 *              "%s    (bn tile id = %d) (cap = %d) (den = %d)\n",
 *              "Global Router : (cap - den) <= 0\n",
 *              Tp_GetIdOfTile(bnTilePtr), tileCap, tileDen);
 *          Bear_PrintErr(bear_ErrorMsg);
 *          return (BEAR_STOP_SEARCH);
 *      }
 */

        Tp_PutDensityOfTile(0, bnTilePtr);
        tileCap = Tp_GetCapacityOfTile(bnTilePtr);
        if (tileCap <= 0) {
            (void) sprintf (bear_ErrorMsg,
                "Global Router : bad data (bn tile id = %d) (cap = %d)\n",
                Tp_GetIdOfTile(bnTilePtr), tileCap);
            Bear_PrintErr(bear_ErrorMsg);
  
            return (BEAR_STOP_SEARCH);
        }
    }

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------
 *
 *   Gr_GlobalRouter() --
 *
 *   This is an implementation of an algorithm
 *   based on hierarchical partitions and linear assignments.
 *   The key ideas are descreibed in
 *   M.Marek-Sadowska,"Route Planner",Proc.ICCAD'86.
 *   This module is called when automatic topological routing
 *   of nets specified as groups of pins are sought.
 *   The current implementation does not take partially prerouted
 *   nets. The existing code is ready to accept them, but some
 *   procedures are missing. If a net is fully prerouted then
 *   it is ignored by the global router.
 *   Results     : BearStatus.
 *   Side effects: pseudoPins are created in Mac. External
 *   and internal names of these pseudoPins are such that
 *   matching names mean connected pins.
 *
 * ______________________________________________________________________
 *
 */

BearStatus
Gr_GlobalRouter(cellMasterPtr, timingMode, congestion)
CellMaster  *cellMasterPtr;
int         timingMode;             /* 0 : NO timing calculations
                                     * 1 : timing calculations
                                     */
float       congestion;
{
    extern          double ptime();

    PlaneData       *planeDataPtr;
    Rect            *horzPlaneBndPtr;
    char            string[10];
    /* int          netCount; */

        int             h_space,w_width,v_size,v_overlap;
#ifdef HP_PLOT
        int             netNumber;
#endif HP_PLOT

    grCellMasterPtr = cellMasterPtr;
    if (timingMode == 0 || timingMode == 1) {
        gr_Timing = timingMode;
    }
    else {
        Bear_PrintErr("Gr_GlobalRouter() : illegal timing mode");
        return (BEAR_ERROR);
    }

    grPlaneDataPtr = planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    horzPlaneBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    gr_xShift = (*horzPlaneBndPtr).r_ll.p_x;
    gr_yShift = (*horzPlaneBndPtr).r_ll.p_y;
    gr_xShift = -gr_xShift;
    gr_yShift = -gr_yShift;


    w_width = Mac_GetMinWidthOfLay(METAL2_LAYER) ;
    h_space = Mac_GetMinSpaceOfLayLay(METAL2_LAYER,METAL2_LAYER);
    v_size  = Mac_GetSizeOfHole(M1M2_VIA);
    v_overlap = Mac_GetMinOverlapOfLayHole(METAL2_LAYER,M1M2_VIA);

    if ((w_width+h_space) > (v_size+2*v_overlap+h_space)) {
        gr_horzDesRule = w_width + h_space;
    }
    else {
        gr_horzDesRule = v_size + 2*v_overlap+h_space;
    }
        
    gr_congestionFactor = congestion;
#ifdef HP_PLOT
    printf("\n horzDesRule=%4d",gr_horzDesRule);
    printf("\n Want to change design rule (y/n) ? ");
    scanf("%1s", string);
    if (string[0] == 'y') {
        printf("\n gr_horzDesRule=");
        scanf("%d", &gr_horzDesRule);
    }   
#endif HP_PLOT

    gr_vertDesRule = gr_horzDesRule;

    gr_FreeMemFlag = TRUE;
    grPrintFlag = FALSE;

#ifdef DEBUG
{
    char *printPtr;

    if ((printPtr = getenv("PRINT_GR_MSGS")) != NIL(char)) {
        if (strcmp (printPtr, "TRUE") == 0) {
            grPrintFlag = TRUE;
        }
    }
}
#endif DEBUG

    /* 
     * Set the capacities and densities of bottleneck tiles.
     */
    Gs_CalcCapDenOfTilePlane(grCellMasterPtr);

    if (Fpg_ItrBottleNeckOfFpg(Tp_GetFpg(planeDataPtr), DoBottleNeck,
        (ClientData) NULL) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    if (GrGeoRead(planeDataPtr) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    debug = 'n';
/*
#ifdef DEBUG
    printf("\n NEED DEBUGGING ? (y/n) ");
    scanf("%1s",string);
    debug = (strcmp (string, "y") == 0 ) ? 'y' : 'n';
#endif DEBUG
*/

#ifdef ASK_ABOUT_BEAR
    printf("\n DO YOU WANT CONNECTION TO BEAR ? (y/n) ");
    scanf("%1s",string);
    gr_BearConnection = (strcmp (string, "y") == 0) ? 1 : 0 ;
#else
    gr_BearConnection = 1;
#endif ASK_ABOUT_BEAR

    Gr_Set1Memor();
    Gr_MkeGrid();
    Gr_MkeBitMap();

    Gr_prgrid(); 
    Gr_pBitMap(1,gr_totxgrid,1,gr_totygrid);

    if (grPrintFlag == TRUE) {
        (void) fprintf (bearout, "\n totx=%d toty=%d ",
            gr_totxgrid, gr_totygrid);
        (void) fprintf (bearout, "\n\n");
    }
/*
    Gr_pBitMap(1,26,1,40);
    Gr_pBitMap(1,26,41,80);
    Gr_pBitMap(1,26,81,gr_totygrid);
    Gr_pBitMap(27,52,1,40);
    Gr_pBitMap(27,52,41,80);
    Gr_pBitMap(27,52,81,gr_totygrid);
    Gr_pBitMap(53,78,1,40);
    Gr_pBitMap(53,78,41,80);
    Gr_pBitMap(53,78,81,gr_totygrid);
    Gr_pBitMap(79,gr_totxgrid,1,40);
    Gr_pBitMap(79,gr_totxgrid,41,80);
    Gr_pBitMap(79,gr_totxgrid,81,gr_totygrid);
    

    */

    Gr_MkeRooms();

    if (GrInitializeNets(cellMasterPtr) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    grHeadDeadRegion = NULL;

    if (Gr_MkeRegions() == BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    if ((gr_BearConnection == 1) && (bnLineCount > 0)) {
        Gr_InsertIdToBtnPPins();
    }

    /*

    Gr_PrintEndResults();

    Gr_PrintFinalResult(172); 

    Gr_CrackLoops(); 

    Gr_PrintFinalResult(172); 
    */

    Gr_PrintEndResults();
    fflush(stdout);

#ifdef HP_PLOT
/*
 * This part of code is for drawing nets on HP terminal.
 */
        if (debug != 'y')
        {
        while (TRUE) 
        {
            printf("\n Need printout ? ");
            scanf("%1s", string);
            if (string[0] == 'n')
            {
                break;
            }
            else 
            {
                printf("\n Enter net number : ");
                scanf("%d", &netNumber);
                if (netNumber <= gr_NetCount)
                {
                    Gr_PloRealBlocks(netNumber);
                }
                else
                {
                    printf("\n TOO LARGE NET NUMBER \n");
                }
            }
        }
        }
#endif HP_PLOT



    /*
    Gr_PrintEndResults();
    Gr_PrintFinalResult(80); 
    */


    /* Memory clean-up */
    Gr_FreeSortedPins();
    Gr_FreePins();
    Gr_FreeNets();
    Gr_FreeAppartments();
    Gr_UnSet1Memor();
    Gr_FreeBnLines();
    Gr_FreeBlocks();
    /* End memory clean-up */

    /* Maintain status of GLOBAL_ROUTING_DONE */
    Mac_SetGlobalRoutingDone(cellMasterPtr);

    /* Maintain a count of the number of connected nets */
    /*
    Mac_NumOfNet(cellMasterPtr, &netCount);
    NumOfConnectedNets = netCount;
    */

    return (BEAR_OK);
}



