/*
 * grMaze.c --
 *
 *     Author
 *	MalgorzataMarek-Sadowska
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint




#include "grInt.h"


/*
*--------------------------------------------------------------------------
*	This file contains functions needed when an accureate net lengths
*	are needed. They use maze type search through virtual greed.
*-------------------------------------------------------------------------
*/



/*
*------------------------------------------------------------------------
*	Gr_CalculateEscapeCost
*	Results		: cost of leaving bottleneck through starting
*			  point in a specified direction.
*	Results		: none.
*------------------------------------------------------------------------
*/



int Gr_CalculateEscapeCost(xs,ys,vNet,kPtr,shade,side)

int 		 xs,ys;		/* Coordinates of the starting point.	*/
grVirtNet 	*vNet;		/* Pointer to a virtual net.		*/
grRegion 	*kPtr;		/* Pointer to the parent region.	*/
PinDirection 	 shade;		/* Direction of pin at xs,ys.		*/
int 		 side; 		/* 0 left, 1 right 			*/

{

  int i,bestdist,dist,x,y,ytop,ybot,xleft,xright;
  grRoom *new,*start,*result;
  grSortedPin *pending;
  grAddress *first,*last,*tail,*current;


  start=Gr_GetRoomForPPinPosition(xs,ys,shade);

  /* check if start is a target */

  if (start->status ==5)
  {
    if (side==0)
    {
      pending=vNet->leftNode->contentsPtr->firstxPin;
    }
    else
    {
      pending=vNet->rightNode->contentsPtr->firstxPin;
    }

    while (pending!=NULL)
    {
      if (pending->pinPtr->room == start)
      {
        x=pending->pinPtr->x;
	y=pending->pinPtr->y;

	bestdist=Gr_GetDistanceBetweenPoints(xs,ys,x,y);
	Gr_CleanUpRooms(kPtr,0);
	return (bestdist);
      }

      pending=pending->nextxPin;
    }
  }

  /* start is not a target */

  xleft=kPtr->xvirt+1;
  xright=xleft+kPtr->whvirt-1;
  ybot=kPtr->yvirt+1;
  ytop=kPtr->hvirt + ybot-1;


  bestdist=0;

  MALLOC(grAddress*,first,sizeof (grAddress));
  first->gRoom = start;
  last=first;
  tail=first;
  first->next=NULL;

lab1:;
  current=first;

  while (current!=NULL)
  {
    if ((bestdist==0)||(current->gRoom->dist < bestdist))
    {
      for (i=1;i<=4;i++)
      {
	if (i==1)
	{
	  new=Gr_GetTopRoom(current->gRoom,ytop);
	  if (new!=NULL)
	  {
	    if (current->gRoom==start)
	    {
	      dist=Gr_GetDistanceBetweenPoints(xs,ys,new->c_x,new->c_y);
	      current->gRoom->dist= dist/2 + 1;
	    }
	    else
	    {
	      dist=current->gRoom->dist + new->c_y - current->gRoom->c_y;

	    }
	  }
	}
	if (i==2)
	{
	  new=Gr_GetRightRoom(current->gRoom,xright);
	  if (new!=NULL)
	  {
	    if (current->gRoom == start)
	    {
	      dist=Gr_GetDistanceBetweenPoints(xs,ys,new->c_x,new->c_y);
	      current->gRoom->dist=dist/2 + 1;
	    }
	    else
	    {
	      dist=current->gRoom->dist + new->c_x - current->gRoom->c_x;
	    }
	  }
	}
	if (i==3)
	{
	  new=Gr_GetBotRoom(current->gRoom,ybot);
	  if (new!=NULL)
	  {
	    if (current->gRoom == start)
	    {
	      dist=Gr_GetDistanceBetweenPoints(xs,ys,new->c_x,new->c_y);
	      current->gRoom->dist=dist/2 +1;
	    }
	    else
	    {
	      dist=current->gRoom->dist + current->gRoom->c_y - new->c_y;
	    }
	  }
	}
	if (i==4)
	{
	  new=Gr_GetLeftRoom(current->gRoom,xleft);
	  if (new!=NULL)
	  {
	    if (current->gRoom == start)
	    {
	      dist=Gr_GetDistanceBetweenPoints(xs,ys,new->c_x,new->c_y);
	      current->gRoom->dist =dist/2 +1;
	    }
	    else
	    {
	      dist=current->gRoom->dist + current->gRoom->c_x - new->c_x;
	    }
	  }
	}
	if (new!=NULL)
	{
	  result=Gr_IsSuccessfulExpansion(current->gRoom,new,dist);
	  if (result!=NULL)
	  {
	    if ((result->status >= 3)&&(result->status <= 5))
	    {
	      /* target - no further expansion */
	      if (bestdist>dist)
	      {
		bestdist=dist;
	      }
	      if (bestdist==0)
	      {
		bestdist=dist;
	      }
	    }
	    else
	    {
	      /* not a target */
	      if ((bestdist==0)||(dist < bestdist))
	      {
		/* add to the expansion list */
		MALLOC(grAddress*,tail->next,sizeof (grAddress));
		tail=tail->next;
		tail->gRoom=result;
		tail->next=NULL;
	      }
	    }
	  }
	}
      }
    }
    if (current!=last)
    {
      current=current->next;
    }
    else
    {
      goto lab0;
    }
  }

lab0:;
  if (tail!=last)
  {
    first=last->next;
    last=tail;
    goto lab1;
  }

  Gr_CleanUpRooms(kPtr,0);
  while (first != NULL)
  {
    current=first->next;
    GR_FREE((char*) first);
    first=current;
  }
  return (bestdist);
}

/*
*---------------------------------------------------------------------------
*	Gr_DecideDirectionsForPullOut
*	Results 		: none.
*	Side effects		: decission, which way a pin should be
*				  pulled out from a bottleneck.
*--------------------------------------------------------------------------
*/



void Gr_DecideDirectionsForPullOut(kPtr,vNet,xwhois,ywhois,xwhere,ywhere,
lcount,rcount)

grRegion 	*kPtr;		/* Pointer to the parent region.	*/
grVirtNet 	*vNet;		/* Pointer to a virtual net.		*/
grPrimit 	*xwhois,*ywhois; /* Marked btn pins.			*/
grPrimit	*xwhere,*ywhere; /* Hole numbers.			*/
int 		 lcount,rcount;	/* #pins on left , #pins on right side.	*/

{

  int hNumber,dir,i,j,k,nopins,licz,xs,ys,pos;
  grPrimit *dl,*x,*y;
  grSortedPin *gSPin;

  hNumber=vNet->hole;
  dir=kPtr->dir;
  if (n > 1) /* more than one hole */
  {
    Gr_SetTargetsForVNet(vNet,hNumber,dir,xwhois);

    for (i=1;i<=n;i++)
    {
      if (i!=hNumber)
      {
         /* get hole number */
	for (j=1;j<=(lcount+rcount);j++)
	{
	  if ((xwhere+j)->number == i)
	  {
	    nopins=0;
	    for (k=1;k<=(lcount+rcount);k++)
	    {
	      if ((xwhere+k)->number==i)
	      {
		nopins++;
	      }
	    }

	    MALLOC(grPrimit*,x,(nopins+3)*sizeof (grPrimit));
	    MALLOC(grPrimit*,y,(nopins+3)*sizeof (grPrimit));
	    MALLOC(grPrimit*,dl,(nopins+3)*sizeof (grPrimit));
					
	    for (k=1;k<=(nopins+2);k++)
	    {
	      (x+k)->number=0;
	      (y+k)->number=0;
	      (dl+k)->number=0;
	    }

	    licz=0;
	    if (dir==1)
	    {
	      gSPin=vNet->leftNode->contentsPtr->firstxPin;
	      for (k=1;k<=lcount;k++)
	      {
		if ((xwhere+k)->number == i)
		{
		  licz++;
		  (x+licz)->number=gSPin->pinPtr->x;
		  (y+licz)->number=gSPin->pinPtr->y;
		  if (licz==1)
		  {
		    (dl+licz)->number = 0;
		  }
		  else
		  {
		    (dl+licz)->number=gSPin->pinPtr->x - (x+licz-1)->number;
	 	  }
		}
		gSPin=gSPin->nextxPin;
	      }

	      gSPin=vNet->rightNode->contentsPtr->firstxPin;
	      for (k=(lcount+1);k <=(lcount+rcount);k++)
	      {
	        if ((xwhere+k)->number == i)
		{
		  licz++;
		  (x+licz)->number=gSPin->pinPtr->x;
		  (y+licz)->number = gSPin->pinPtr->y;
		  if (licz==1)
		  {
		    (dl+licz)->number = 0;
		  }
		  else
		  {
		    (dl+licz)->number = gSPin->pinPtr->x - (x+licz-1)->number;
		  }
		}
		gSPin=gSPin->nextxPin;
	      }
	    }
	    if (dir==2)
	    {
	      gSPin=vNet->rightNode->contentsPtr->firstyPin;
	      for (k=(lcount+1);k<=(lcount+rcount);k++)
	      {
		if ((ywhere+k)->number ==i)
		{
		  licz++;
		  (x+licz)->number=gSPin->pinPtr->x;
		  (y+licz)->number=gSPin->pinPtr->y;
		  if (licz==1)
		  {
		    (dl+licz)->number =0;
		  }
		  else
		  {
		    (dl+licz)->number=gSPin->pinPtr->y - (y+licz-1)->number;
		  }
		}
		gSPin=gSPin->nextyPin;
	      }
	      gSPin=vNet->leftNode->contentsPtr->firstyPin;
	      for (k=1;k<=lcount;k++)
	      {
		if ((ywhere+k)->number==i)
		{
		  licz++;
		  (x+licz)->number = gSPin->pinPtr->x;
		  (y+licz)->number = gSPin->pinPtr->y;
		  if (licz==1)
		  {
		    (dl+licz)->number = 0;
		  }
		  else
		  {
		    (dl+licz)->number = gSPin->pinPtr->y - (y+licz-1)->number;
		  }
		}
		gSPin= gSPin->nextyPin;
	      }
	    }
	    if (dir==1)
	    {
	      /*left side */
	      xs=Gr_GetXPPinInVertHole(i,0);
	      ys=Gr_GetYPPinInVertHole(i);
	      (dl+1)->number=(x+1)->number - xs;
	     dl->number=Gr_CalculateEscapeCost(xs,ys,vNet,kPtr,EAST_WEST_PIN,0);
	      /*right side*/
	      xs=Gr_GetXPPinInVertHole(i,1);
	      (dl+nopins+1)->number = xs-(x+nopins)->number;
	      (dl+nopins+2)->number = Gr_CalculateEscapeCost(xs,ys,
	      vNet,kPtr,EAST_WEST_PIN,1);
	    }
	    if (dir==2)
	    {
	      /* bot side */
	      xs=Gr_GetXPPinInHorzHole(i);
	      ys=Gr_GetYPPinInHorzHole(i,1);
	      (dl+1)->number = (y+1)->number - ys;
	      dl->number=Gr_CalculateEscapeCost(xs,ys,
	      vNet,kPtr,NORTH_SOUTH_PIN,1);
	      /*top side */
	      ys=Gr_GetYPPinInHorzHole(i,0);
	      (dl+nopins+1)->number = ys - (y+nopins)->number;
	      (dl+nopins+2)->number=Gr_CalculateEscapeCost(xs,ys,
	      vNet,kPtr,NORTH_SOUTH_PIN,0);
	    }


	    if (dl->number==0)
	    {
	      if (dir==1)
	      {
		/* no escape to left possible, all pull out to the right */
		for (k=1;k<=(lcount+rcount);k++)
		{
		  if ((xwhere+k)->number == i)
		  {
		    (xwhois+k)->number =4;
		  }
		}
	      }
	      else
	      {
		/* no escape to the right possible */
		for (k=1;k<=(lcount+rcount);k++)
		{
		  if ((ywhere+k)->number == i)
		  {
		    (ywhois+k)->number = 3;
		  }
		}
	      }
	      GR_FREE((char*) x);
	      GR_FREE((char*) y);
	      GR_FREE((char*) dl);
	      goto lab11;
	    }

	    if ((dl+nopins+2)->number ==0)
	    {
	      if (dir==1)
	      {
		/*no escape to right possible,all pull out to the left */
		for (k=1;k<=(lcount+rcount);k++)
		{
		  if ((xwhere+k)->number == i)
		  {
		    (xwhois+k)->number =3;
		  }
		}
	      }
	      else
	      {
		/*no escape to left possible*/
		for (k=1;k<=(lcount+rcount);k++)
		{
		  if ((ywhere+k)->number ==i)
		  {
		    (ywhois+k)->number =4;
		  }
		}
	      }
	      GR_FREE((char*) x);
	      GR_FREE((char*) y);
	      GR_FREE((char*) dl);
	      goto lab11;
	    }
	    (dl+1)->number += dl->number;
	    (dl+nopins+1)->number += (dl+nopins+2)->number;

	    licz=1;
	    pos=(dl+1)->number;
	    for (k=2;k<=(nopins+1);k++)
	    {
	      if ((dl+k)->number>pos)
	      {
		licz=k;
		pos=(dl+k)->number;
	      }
	    }
	    pos=0;
	    if (dir==1)
	    {
	      for (k=1;k<=(lcount+rcount);k++)
	      {
		if ((xwhere+k)->number == i)
		{
		  pos++;
		  if (pos<licz)
		  {
		    (xwhois+k)->number = 3;
		  }
		  else
		  {
		    (xwhois+k)->number =4;
		  }
		}
	      }
	    }
	    else
	    {
	      for (k=(lcount+1);k<=(lcount+rcount);k++)
	      {
		if ((ywhere+k)->number == i)
		{
		  pos++;
		  if (pos<licz)
		  {
		    (ywhois+k)->number =4;
		  }
		  else
		  {
		    (ywhois+k)->number =3;
		  }
		}
	      }
	      for (k=1;k<=lcount;k++)
	      {
		if ((ywhere+k)->number == i)
		{
		  pos++;
		  if (pos<licz)
		  {
		    (ywhois+k)->number =4;
		  }
		  else
		  {
		    (ywhois+k)->number =3;
		  }
		}
	      }
	    }
	    GR_FREE((char*) x);
	    GR_FREE((char*) y);
	    GR_FREE((char*) dl);
	    goto lab11; /* take next i */
	  }
	}
      }
      lab11:;
    }
    Gr_CleanUpRooms(kPtr,2);
  }

}
