/*
 * grMkeVNets.c --
 *
 *     Author
 *	Malgorzata Marek-Sadowska
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint




#include "grInt.h"

/*
*----------------------------------------------------------------------------
*	This is a file  where virtual nets for assignment are created
*	and external pins near switchbox regions are assigned.
*---------------------------------------------------------------------------
*/


/*
*----------------------------------------------------------------------------
*	Gr_GiveNamesToPins
*	Results		: none.
*	Side effects	: in leaf regions all the resident pins get their   
*	final external names .            
*	HERE IS LINK TO BEAR 
*----------------------------------------------------------------------------
*/



void Gr_GiveNamesToPins(kPtr)

grRegion *kPtr;		/* pointer to the parent region.		*/

{

	grSubNet *headSubNet;
	grPinRectangle *pending;
	BearId name;
	grSortedPin *current;

	headSubNet=kPtr->subNetsPtr;

	while(headSubNet != NULL)
	{
		name=headSubNet->subname;
		pending=headSubNet->toBeConnected;
		if (pending!=NULL)
		{
			current=pending->firstxPin;

			while(current!=NULL)
			{
                                if (current->pinPtr->type!=3)
                                {
				current->pinPtr->extname = name;
				if (gr_BearConnection == 1)
				{
				/* link to bear */
				if (current->pinPtr->type == 1) /* real pin*/
					{
						Mac_PutExtNetIdOfPin(name,
					current->pinPtr->PinPtr.pinInst);
					}
				if ((current->pinPtr->type ==2 )||
                                  (current->pinPtr->type ==5))
                                  /* pseudo pin or ppin on the right of bnline*/
					{
						Tp_PutExtNetIdOfPPin(name,
					current->pinPtr->PinPtr.pseudoPin);
					}
				}

                                }
                                else
                                {
                                if ((current->side==3)||(current->side==4))
				{
                                	current->pinPtr->intname= name;
				}
                                
                                if ((current->side==1)||(current->side==2))
				{
                                	current->pinPtr->extname= name;
				}
                                }
				current=current->nextxPin;
			}

		}

		headSubNet=headSubNet->nextOnList;
	}

}

/*
*----------------------------------------------------------------------------
*	Gr_CreateMultipleVirtualNets
*	Results		: none.
*	Side effects	: virtual nets are created for decomposed subnets.
*-----------------------------------------------------------------------------
*/



void Gr_CreateMultipleVirtualNets(sNet)

grSubNet *sNet;

{

	grRootTree *branch,*father;
	int flag;
	grVirtNet *vNet;
	grSuperNode *current;

	sNet->active =6;  /* marks that at least one virtual net is created */
	flag=0;

	branch=sNet->tree;

        while(branch->nextInSubnet!=NULL)
        {
         	branch=branch->nextInSubnet;
        }

	while(branch!=NULL)
	{
		if (branch->mark==2) /*is removed*/
		{
			if (flag==1) sNet->active=7; /* more than one vNet*/
			flag=1;
			father=branch->fatherPtr;

                        vNet= Gr_CreateNewVirtNet(sNet);

			current=father->nodePtr;
			if (current->side==1)
			{
				vNet->leftNode=current;
				vNet->rightNode=branch->nodePtr;
			}
			else
			{
				vNet->leftNode=branch->nodePtr;
				vNet->rightNode=current;
			}
			vNet->netName=sNet->netname;
			vNet->subNetName=sNet->subname;
			m++;
			vNet->leftNode->nodeName=Gr_GetNewSubNetName();
			vNet->rightNode->nodeName=Gr_GetNewSubNetName();


		} /* end branch found*/

		branch=branch->nextOnList;
	}  /* end while*/
}




/*
*---------------------------------------------------------------------------
*	Gr_MkeVirtualNets
*	Side effects	: for each net resident in kPtr it either
*			  builds virtual nets for assignment
*			  or pushes the net to the  child
*			  region of the region pointed to by kPtr 
*			  The total number of virtual
*			  nets : m ( global variable) is calculated here.  
*---------------------------------------------------------------------------
*/



void Gr_MkeVirtualNets(kPtr)

grRegion *kPtr;		/* Pointer to the parent region.		*/


{

	int dir,cnt,num;
	grSubNet *sNet,*nextNet;
	grSuperNode *current;

	m=0;  /* initially the number of virtual nets is 0*/
	dir=kPtr->dir;

	if (dir==0)  /* no cut, pins need to be put into database */
	{
		Gr_GiveNamesToPins(kPtr);


	} /* end no cut line*/
	else 
	    /* cut occurs*/
	{
		Gr_DecomposeNets(kPtr); /* nets are divided at cut line*/

		Gr_CleanOldPinRectangles(kPtr); 
		/* removes the initial pinRectangles*/

		if (n>=1)  /* more than on hole */
		{
			sNet=kPtr->subNetsPtr;

			while(sNet!=NULL)
			{
				sNet->toBeConnected=NULL;
				sNet->areConnected=NULL;

				if (sNet->active==1)  /* net needs assignment*/
				{
                                        if (n>1)
                                        {
                                        num=Gr_CheckIfDecompositionImproves(
					sNet,dir);
                                        }
                                        else
                                        {
                                        num=0;
                                        }
					if ((sNet->decCandidate==1)&&(num>0))
					 /*decomposition needed*/
					{
          
                                		Gr_MkeSinglePinNodes(sNet);

						Gr_MkeRootTree(sNet,0);


						Gr_SplitRootTreeAtCutBranches(
						sNet);

						Gr_CreateMultipleVirtualNets(
						sNet);
						/*active determined in the
						 Gr_CreateMultipleVirtualNets*/
					}
					else
					{
						/* no decomposition*/
						cnt=0;
						current=sNet->NodeList;
						while(current!=NULL)
						{
							cnt++;
							current=current->
							nextOnList;
						}
						if (cnt==2)
						{
						Gr_CreateSimpleVirtualNet(sNet);
							sNet->active=5;
						}
						if (cnt>2)
						{
						Gr_CreateGreedyVirtualNet(sNet);
							sNet->active=6;
						}
					}  /* end no decomposition*/
				} /* end net needs assignment*/

				if (sNet->active==2) 
			/* no assignment is needed but intersection occurs*/
				{
					Gr_MkeRootTree(sNet,1);
					Gr_SplitRootTreeAtCutBranches(sNet);
					Gr_DropNetToBothRegions(kPtr,sNet);
					sNet->active=8; /*no virtual nets*/
				} 
			/* end no assignment needed but intersection occurs*/

				if (sNet->active==3) 
				/*net sits on the left side*/
				{
					Gr_PrepareNetForDrop(
					kPtr,sNet,dir,0,1,n);
					if (sNet->NodeList->contentsPtr->
					count!=0)
					{
						Gr_DropNetToOneRegion(sNet,
						kPtr->leftPtr);
					}
					sNet->active=8;
				}
				if (sNet->active==4) 
				/*net sits on the right side*/
				{
					Gr_PrepareNetForDrop(kPtr,sNet,dir,1,
					1,n);
					if ((sNet->NodeList->contentsPtr->
					count) !=0)
					{
						Gr_DropNetToOneRegion(sNet,
						kPtr->rightPtr);
					}
					sNet->active=8;
				}
			Gr_NameSuperNodesAfterVirtualNetsWereCreated(sNet);
                               sNet=sNet->nextOnList;
                        } /*end while*/
		} /* end  hole or holes occur */
	}  /* end cut occurs*/

}
