
/* grNet.c --
 *
 *     Malgorzata Marek-Sadowska
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "grInt.h"

/*
*------------------------------------------------------------------------
*   Functions in this file are used for readind in the routing
*   problem from BEAR.
*------------------------------------------------------------------------
*/


/* ---------- Local Procedures ---------- */
static BearStatus ReadPinsOfNet();


/*
 *---------------------------------------------------------------------------
 *   GrInitializeNets()
 *   Results     : BEAR_OK or BEAR_ERROR.
 *   Side effects    : sets memory and pointers to heads of lists of net     
 *             net related data.
 *             Reads in nets.
 *----------------------------------------------------------------------------
 */                                                        

BearStatus
GrInitializeNets(parentCellPtr)
CellMaster *parentCellPtr;
{

    if (Mac_NumOfNets(parentCellPtr, &gr_TotalNetCount)==BEAR_OK) {
        Gr_Set2Memor();
        gr_NetCount=0; /* counts nets which are uncomplete */
        gr_PinHeadPtr=NULL; /* header to the list of pins */
                gr_HeadLastToSort=NULL; /* pointer to the yet unsorted grPin*/

                /* read in data from the data base */

        if (Mac_ItrNet(Mac_GetNetListOfCell(parentCellPtr),
            ReadPinsOfNet, (ClientData) NULL) == BEAR_STOP_SEARCH) {
            return (BEAR_ERROR);
        }
    }
    Gr_EstablishHardLinksForSortedPins();

    return (BEAR_OK);
}



/*
 *---------------------------------------------------------------------------
 *   ReadPinsOfNet()
 *   Results     : BearStatus.
 *   Side effects    : pins of one net are read in.
 *   This function reads in only uncompleted signal or clock nets
 *---------------------------------------------------------------------------
 */

/* ARGSUSED */
static BearStatus
ReadPinsOfNet(netPtr, clientData)
Net         *netPtr;
ClientData  clientData;
{
#define RETURN(bearStatus)                                                  \
{                                                                           \
    BearStatus cos;                                                         \
                                                                            \
    cos = (bearStatus);                                                     \
    if ((netPtr != NIL(Net)) && (cos == BEAR_CONTINUE_SEARCH)) {            \
        Mac_PutStatusOfNetInTiPlane((bool) TRUE, netPtr);                   \
    }                                                                       \
    return (cos);                                                           \
}

    NetKind whoto;
    int     pinCount;
    bool    loopFlag, connFlag;

    if (Mac_GetUserGraphOfNet(netPtr) != (Gt_Graph) NULL) {
        if (Mac_CheckUserGraph(netPtr, &loopFlag, &connFlag) == BEAR_ERROR) {
            RETURN(BEAR_STOP_SEARCH);
        }

        if (connFlag == TRUE) {
            /* net is prerouted */
            RETURN(BEAR_CONTINUE_SEARCH);
        }
        else {
            sprintf(bear_ErrorMsg,
                "Error : net named \"%s\" is not connected.\n",
                Mac_GetNameOfNet(netPtr));
            Bear_PrintErr(bear_ErrorMsg);
            RETURN(BEAR_STOP_SEARCH);
        }
    }
    
    whoto = Mac_GetKindOfNet(netPtr);

    gr_NetCount++;
    /* set key to net */
    (gr_HeadKeyToNets+gr_NetCount)->key = gr_NetCount;
    (gr_HeadKeyToNets+gr_NetCount)->name = Mac_GetNameOfNet(netPtr);

    /* Initialize memory for the net */
    Gr_CreateNet(gr_NetCount);

    (gr_HeadGrNets+gr_NetCount)->noComponents= 0;
    (gr_HeadGrNets+gr_NetCount)->netBear = netPtr; /* link to bear*/

    /* net has no graph,iterate over ports*/
    (void) Mac_NumOfPortsOfNet(netPtr, &pinCount);
    (gr_HeadGrNets+gr_NetCount)->noLoosePins = pinCount;

    if (pinCount <= 1) {
        (void) sprintf (bear_ErrorMsg,
            "Net named \"%s\" has less then two pins.\n",
            Mac_GetNameOfNet(netPtr));
        Bear_PrintErr(bear_ErrorMsg);

        RETURN(BEAR_STOP_SEARCH);
    }

    (void) Mac_ItrPortOfNet(netPtr,Gr_GetPinOfPortInfo, (ClientData) NULL);

    RETURN(GrMkeSortedPins(pinCount,0));

#undef RETURN
}



/*
*---------------------------------------------------------------------------
*   This function is for reading in a net which contains only isolated 
*   terminals
*   Results     : BearStatus.
*   Side effects    : isolated pins are read in.
*---------------------------------------------------------------------------
*/




/* ARGSUSED */
BearStatus Gr_GetPinOfPortInfo(portPtr, clientData)

PortInst    *portPtr;
ClientData  clientData;

{
    PinInst *pinInstPtr;
    Point   point;
    grPin   *grPinPtr;

    pinInstPtr = Mac_GetAssignedPinOfPort(portPtr);

    Mac_GetAbsPosOfPin(pinInstPtr, &point);
        
    /* Set memory and link the new grPin  */

    grPinPtr = Gr_CreateGrPin();

    /* end memory assignment and linking */

    grPinPtr->x = point.p_x + gr_xShift;
    grPinPtr->y = point.p_y + gr_yShift;

    grPinPtr->type=1;
    grPinPtr->PinPtr.pinInst = pinInstPtr;
    grPinPtr->netname=gr_NetCount;
    grPinPtr->extname=0;
    grPinPtr->intname=0;
        

    grPinPtr->shade = Mac_GetDirectionOfPin(pinInstPtr);
    Gr_AssignRoomToPin(grPinPtr);

    return (BEAR_CONTINUE_SEARCH);
}


