/*
 * grNetMemor.c --
 *
 *     Author
 *  Malgorzata Marek-Sadowska
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint




#include "grInt.h"

/*
*----------------------------------------------------------------------------
*   This file contains functions which allocate memory and initialize
*   structures related to nets.
*   Also some utility routines for sorted pins.
*----------------------------------------------------------------------------
*/



/*
*--------------------------------------------------------------------------
*   Gr_CreateGrPin()
*   Results     : initialized memory for a new grPin.
*   Side effects    : new grPin is added to proper lists.
*             List of pins not sorted yet is being
*             updated.
*---------------------------------------------------------------------------
*/



grPin *Gr_CreateGrPin()

{
    grPin *gPin;

    MALLOC(grPin*,gPin,sizeof (grPin));


    if (gr_HeadLastToSort==NULL)
    {
        gr_HeadLastToSort=gPin;
    }

    if (gr_PinHeadPtr==NULL)
        {
        gr_PinHeadPtr=gPin;
        gr_LastPinPtr=gPin;
    }
    else
    {
        gr_LastPinPtr->nextPtr=gPin;
        gr_LastPinPtr=gPin;
    }
    gr_LastPinPtr->nextPtr=NULL;
    
    gPin->room = NULL;
    gPin->type = 0;
    gPin->x = 0;
    gPin->y = 0;



    return (gPin);
}

/*
*--------------------------------------------------------------------------
*   Gr_CreateSortedPin()
*   Results     : pointer to a new grSortedPin which is properly
*             initialized.
*   Side effects    : new grSortedPin is added to proper lists.
*--------------------------------------------------------------------------
*/



grSortedPin *
Gr_CreateSortedPin()
{
    grSortedPin *gSPin;

    MALLOC(grSortedPin *, gSPin, sizeof (grSortedPin));

    gSPin->pinPtr = NULL;
    gSPin->nextXPin = NULL;
    gSPin->nextYPin = NULL;
    gSPin->nextxPin = NULL;
    gSPin->nextyPin = NULL;
    gSPin->nextOnList = NULL;

    Gr_PutTogetherSortedPins(gSPin);

    return (gSPin);

}


/*
*---------------------------------------------------------------------------
*   Gr_CreateNet()
*   Results     : pointer to new initialized grNet whose internal
*             name is the paramater of the function.
*---------------------------------------------------------------------------
*/



void Gr_CreateNet(i)

int i;

{
    (gr_HeadGrNets+i)->name=i;
    (gr_HeadGrNets+i)->loosePins=NULL;
    (gr_HeadGrNets+i)->areConnected=NULL;
    (gr_HeadGrNets+i)->firstXPin=NULL;
    (gr_HeadGrNets+i)->firstYPin=NULL;
    (gr_HeadGrNets+i)->noLoosePins=0;
    (gr_HeadGrNets+i)->noComponents=0;
        (gr_HeadGrNets+i)->critical=0;
        (gr_HeadGrNets+i)->xLor=0;
        (gr_HeadGrNets+i)->xRor=0;
        (gr_HeadGrNets+i)->yTor=0;
        (gr_HeadGrNets+i)->yBor=0;
        (gr_HeadGrNets+i)->xfixed=0;
        (gr_HeadGrNets+i)->yfixed=0;
        (gr_HeadGrNets+i)->xapprox=0;
        (gr_HeadGrNets+i)->yapprox=0;
        (gr_HeadGrNets+i)->maxlength=0;
        (gr_HeadGrNets+i)->currentLength=0;
}

/*
*----------------------------------------------------------------------------
*   Gr_Set2Memor()
*   Results     : none.
*   Side effects    : memory is initialized for matrices storing
*   grNets and grKeyToNets. Heads and tails of grSortedPins and
*   grPinRectangles are set to NULL.
*---------------------------------------------------------------------------
*/


 
void Gr_Set2Memor()

{

    MALLOC(grKeyToNets*,gr_HeadKeyToNets,(gr_TotalNetCount+1)*sizeof (grKeyToNets));
    MALLOC(grNet*,gr_HeadGrNets,(gr_TotalNetCount+1)*sizeof (grNet));
    gr_LastSortedPinPtr=NULL;
    gr_HeadSortedPinPtr=NULL;
    gr_HeadPinRectanglePtr=NULL;
    gr_LastPinRectanglePtr=NULL;
}

/*
*---------------------------------------------------------------------------
*   Gr_CreateNewPinRectangle()
*   Results     : none.
*   Side effects    : allocates memory and initializes a new
*   grPinRectangle which is attached at the end of the list   
*   of grPinRectangles.Depending on parameter it may
*   be linked by  nextInNet link to previous grPinRectangle.
*--------------------------------------------------------------------------
*/



void Gr_CreateNewPinRectangle(treeControl)

int treeControl;

/* if treeControl=0 the new rectangle is not a member of the same net*/
/* as the previous rectangle , if treeControl >0 then it is and it is*/

{

    if (gr_HeadPinRectanglePtr==NULL)
        {
        gr_HeadPinRectanglePtr=Gr_AllocateMemoryForNewPinRectangle();
        gr_LastPinRectanglePtr=gr_HeadPinRectanglePtr;
    }
    else
    {
        gr_LastPinRectanglePtr->nextOnList
        =Gr_AllocateMemoryForNewPinRectangle();
        if (treeControl>0)
            {
            gr_LastPinRectanglePtr->nextInNet
            =gr_LastPinRectanglePtr->nextOnList;
        }
        gr_LastPinRectanglePtr=gr_LastPinRectanglePtr->nextOnList;
    }

}

/*
*---------------------------------------------------------------------------
*   Gr_PutTogetherSortedPins
*   Results     : none.
*   Side effects    : two lists of sortedPins are put together.
*---------------------------------------------------------------------------
*/



void Gr_PutTogetherSortedPins(sortedPinPtr)

grSortedPin *sortedPinPtr;

{
    if (gr_HeadSortedPinPtr==NULL) {
        gr_HeadSortedPinPtr=sortedPinPtr;
    }
    else {
        sortedPinPtr->nextOnList=gr_HeadSortedPinPtr;
        gr_HeadSortedPinPtr=sortedPinPtr;
    }
}

