
/*
 * grPlo.c --
 *
 *     Malgorzata Marek-Sadowska
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "grInt.h"


#ifdef HP_PLOT

#define ESC 033
#define CR 015



/*----------------------------------------------------------------------
*	Procedures in this file are for drawing on HP 2648a terminal.
*	Net drawing procedures do not make use of floorplan graph,
*	so it is easier to see if nets are connected.
*----------------------------------------------------------------------
*/


/*----------------------------------------------------------------------
*	Gr_Ramka
*	This function is called when picture on HP terminal is requested.
*	Drawing is in virtual grid equally spaced.
*	Results      : none.
*	Side effects : a frame of the picture is drawn.
*-----------------------------------------------------------------------
*/


void Gr_Ramka(dx,dy,clear)

int dx,dy,clear;

{

	int xo,yo,xt,yt;

	xo=clear;
	yo=clear;
	Gr_move(xo,yo);
	xt=xo+gr_totxgrid*dx;
	yt=yo+gr_totygrid*dy;
	Gr_draw(xt,yo);
	Gr_move(xt,yo);
	Gr_draw(xt,yt);
	Gr_move(xt,yt);
	Gr_draw(xo,yt);
	Gr_move(xo,yt);
	Gr_draw(xo,yo);
}



/*-------------------------------------------------------------------------
*	Gr_DrBlock
*	Results     : none.
*	Side effects: draws a rectangle representing i-th block.
*	Drawing is in virtal grid equally spaced.
*--------------------------------------------------------------------------
*/


void Gr_DrBlock(i,dx,dy,clear)

int i,dx,dy,clear;		/* i is an internal number of a block;
				   dx,dy are numbers of pixels per
				   grid in x and y direction;
				   clear is number of pixels from the
				   boundary of a screen.
				*/

{

	int xo,yo,xt,yt;

	xo=clear+dx*((grKeyToBlocksHeadPtr+i)->blockPtr->xvirt);
	yo=clear+dy*((grKeyToBlocksHeadPtr+i)->blockPtr->yvirt);
	yt=clear+dy*(((grKeyToBlocksHeadPtr+i)->blockPtr->yvirt)+
	((grKeyToBlocksHeadPtr+i)->blockPtr->hvirt));
	xt=clear+dx*(((grKeyToBlocksHeadPtr+i)->blockPtr->xvirt)+
	((grKeyToBlocksHeadPtr+i)->blockPtr->wvirt));
	Gr_move(xt,yo);
	Gr_draw(xt,yt);
	Gr_move(xt,yt);
	Gr_draw(xo,yt);
	Gr_move(xo,yt);
	Gr_draw(xo,yo);
	Gr_move(xo,yo);
	Gr_draw(xt,yo);

}
/*
*----------------------------------------------------------------------------
*	Gr_PloBlocks()
*	Results     : none.
*	Side effects: plots input blocks in equally spaced virtual
*	coordinates.
*---------------------------------------------------------------------------- 
*/

void Gr_PloBlocks()

{

	int dx,dy,clear,j;

	clear=10;
	dx=(719-2*clear)/(gr_totxgrid);
	dy=(359-2*clear)/(gr_totygrid);
	Gr_openpl();
	Gr_erase();
	Gr_sgtext();
	Gr_SelectOnePattern();
	Gr_Ramka(dx,dy,clear);

	for (j=1;j<=grSolidTileCount;j++)
		{
			{
				Gr_DrBlock(j,dx,dy,clear);
			}
		}

	/*This part is for drawing bottlenecks : 

	Gr_SelectSevenPattern();
	for (j=(grSolidTileCount+1);j<=grBottleneckTileCount;j++)
		Gr_DrBlock(j,dx,dy,clear);
	Gr_SelectOnePattern(); */

	Gr_closepl();
}



/*
*----------------------------------------------------------------------------
*	Gr_DrawBox(xo,yo,xt,yt)
*	This is a utility function which draws a box.
*	Results     : none.
*	Side effects: a rectangle is drawn. 
*----------------------------------------------------------------------------
*/


void Gr_DrawBox(xo,yo,xt,yt)

int xo,yo,xt,yt;		/* xo,yo are coordinates of left bottom
				         corner;
				   xt,yt are width and height, respectively.
				*/
{
	Gr_move(xo,yo);
	Gr_draw(xt,yo);
	Gr_draw(xt,yt);
	Gr_draw(xo,yt);
	Gr_draw(xo,yo);
}


/*----------------------------------------------------------------------
*	Gr_RealRamka
*	This function is called when picture on HP terminal is requested.
*	Drawing is in real distances.
*	Results      : none.
*	Side effects : a frame of the picture is drawn.
*-----------------------------------------------------------------------
*/
 

void Gr_RealRamka(dx,dy,clear,scalex,scaley)

int dx,dy,clear,scalex,scaley;		/* dx,dy are number of pixels
					   per unit of length in x
					   and y direction, respectively.
					   clear is number of pixels left
					   for frame,
					   scalex and scaley are scale
					   coefficients.
					*/

{
	int xo,yo,xt,yt;

	xo=clear;
	yo=clear;

        if (dx>=1)
        {
		xt=xo+((gr_xcoord+gr_totxgrid)->number)*dx;
        }
        else
        {
        	xt=xo+((gr_xcoord+gr_totxgrid)->number)/scalex;
        }
        if (dy>=1)
        {
		yt=yo+((gr_ycoord+gr_totygrid)->number)*dy;
        }
        else
        {
        	yt=yo+((gr_ycoord+gr_totygrid)->number)/scaley;
        }

	Gr_DrawBox(xo,yo,xt,yt);

}



/*
*------------------------------------------------------------------------------
*	Gr_DrRealBlock
*	Results     : none.
*	Side effects: draws a rectangle representing i-th block.
*	Drawing is in real dimensions.
*-----------------------------------------------------------------------------
*/


void Gr_DrRealBlock(i,dx,dy,clear,scalex,scaley)

int i,dx,dy,clear,scalex,scaley;	/* i is an internal number
					   of a block;
					   dx,dy are number of pixels
					   per unit of length in x
					   and y direction, respectively.
					   clear is number of pixels left
					   for frame,
					   scalex and scaley are scale
					   coefficients.
					*/

/*draws i-th block :cell or bottleneck in real dimensions*/
/*scalled by dx, dy                                      */

{
	int xo,yo,xt,yt;
	grBlock *gBlock;

	gBlock=(grKeyToBlocksHeadPtr+i)->blockPtr;

        if (dx>=1)
        {
		xo=clear+dx*(gBlock->xpos);
        }
        else
        {
        	xo=clear+(gBlock->xpos)/scalex;
        }
        if (dy>=1)
        {
		yo=clear+dy*(gBlock->ypos);
        }
        else
        {
        	yo=clear+(gBlock->ypos)/scaley;
        }
        if (dy>=1)
        {
		yt=clear+dy*((gBlock->ypos)+(gBlock->height));
        }
        else
        {
        	yt=clear+((gBlock->ypos)+(gBlock->height))/scaley;
        }
        if (dx>=1)
        {
		xt=clear+dx*((gBlock->xpos)+(gBlock->width));
        }
        else
        {
        	xt=clear+((gBlock->xpos)+(gBlock->width))/scalex;
        }

	Gr_DrawBox(xo,yo,xt,yt);

}


/*
*----------------------------------------------------------------------------
*	Gr_PloRealBlocks()
*	Results     : none.
*	Side effects: plots input blocks in real distances and one net.
*---------------------------------------------------------------------------- 
*/

void Gr_PloRealBlocks(i)

int i;				/* i is an internal number of a net to 
				   be drawn on the screen */

{

	int dx,dy,clear,j,scalex,scaley;

	clear=10;
	dx=(719-2*clear)/((gr_xcoord+gr_totxgrid)->number);
        if (dx<1)
        {
        	scalex=((gr_xcoord+gr_totxgrid)->number)/(719-2*clear);
        	if ((scalex*(719-2*clear))<(gr_xcoord+gr_totxgrid)->number)
		{
        		scalex=scalex+1;
		}
        }
        else
        {
        	scalex=1;
        }
	dy=(359-2*clear)/((gr_ycoord+gr_totygrid)->number);
        if (dy<1)
        {
        	scaley=((gr_ycoord+gr_totygrid)->number)/(359-2*clear);
        	if ((scaley*(359-2*clear))<(gr_ycoord+gr_totygrid)->number)
		{
        		scaley=scaley+1;
		}
        }
        else
        {
        	scaley=1;
        }

	Gr_openpl();
	Gr_erase();
	Gr_sgtext();
	Gr_SelectOnePattern();
	Gr_RealRamka(dx,dy,clear,scalex,scaley);

	for (j=1;j<=grSolidTileCount;j++)
	{
		/*if ((grKeyToBlocksHeadPtr+j)->blockPtr->persona !=6)*/
		{
			Gr_DrRealBlock(j,dx,dy,clear,scalex,scaley);
		}
	}

/*	Gr_SelectSevenPattern();
	for (j=(grSolidTileCount+1);j<=grBottleneckTileCount;j++)
		Gr_DrRealBlock(j,dx,dy,clear,scalex,scaley);
*/
        Gr_SelectFourPattern();
        Gr_DrawNet(i,clear,dx,dy,scalex,scaley);
	Gr_closepl();
}

/*
*--------------------------------------------------------------------------
*	Gr_CountTraces
*	This function is called when memory for some vectors needed
*	in the drawing process is to be allocated.
*	Results       : total number of traces ( pins and pseudopins)
*			left by the net on a plane.
*	Side effects  : none.
*---------------------------------------------------------------------------
*/


int Gr_CountTraces(i)

int i;


{
	int cnt;
	grSortedPin *gSPin;

	gSPin=(gr_HeadGrNets+i)->firstXPin;

	cnt=0;
	while(gSPin!=NULL)
	{
		cnt++;
		gSPin=gSPin->nextXPin;
	}

	return (cnt);
}

/*
*--------------------------------------------------------------------------
*	Gr_DrawNet
*	This is a function called to draw a net.
*	Results      : none;
*	Side effects : a picture of a net on a screen.
*-------------------------------------------------------------------------
*/



void Gr_DrawNet(i,clear,dx,dy,scalex,scaley)

int i,clear,dx,dy,scalex,scaley;	/* i - internal number of a net,
					   clear,dx,dy,scalex.scaley are
					   the same as in functions for
					   drawing real blocks and frame.
					*/

{
	int dim,j,current,j1,flag,cnt,xo,yo,xt,yt;
	grPrimit *x,*y,*iname,*ename,*li,*le,*mark;
	grSortedPin *gSPin;

	dim=Gr_CountTraces(i);

	MALLOC(grPrimit*,x,(dim+1)*sizeof (grPrimit));
	MALLOC(grPrimit*,y,(dim+1)*sizeof (grPrimit));
	MALLOC(grPrimit*,iname,(dim+1)*sizeof (grPrimit));
	MALLOC(grPrimit*,ename,(dim+1)*sizeof (grPrimit));
	MALLOC(grPrimit*,li,(dim+1)*sizeof (grPrimit));
	MALLOC(grPrimit*,le,(dim+1)*sizeof (grPrimit));
	MALLOC(grPrimit*,mark,(dim+1)*sizeof (grPrimit));

	for (j=1;j<=dim;j++)
	{
		(x+j)->number = 0;
		(y+j)->number = 0;
		(iname+j)->number = 0;
		(ename+j)->number = 0;
		(li+j)->number = 0;
		(le+j)->number = 0;
		(mark+j)->number = 0;
	}

	gSPin=(gr_HeadGrNets+i)->firstXPin;
	j=0;
	/*initialization*/
	while(gSPin!=NULL)
	{
		j++;
		(x+j)->number=gSPin->pinPtr->x;
		(y+j)->number=gSPin->pinPtr->y;
		(iname+j)->number=gSPin->pinPtr->intname;
		(ename+j)->number=gSPin->pinPtr->extname;
		(li+j)->number=0;
		(le+j)->number=0;
		(mark+j)->number=0;
		gSPin=gSPin->nextXPin;
	}
	/* end initialization*/

	j1=1;
	current=0;

	while(j1<=dim)
	{
		flag=0;

		if ((li+j1)->number>0)   /* internal name used up */
		{
			if ((le+j1)->number>0)  /*external name used up*/
			{
				goto labend;
		  		/* take next j1, this pin has been
				 routed from both ends*/
			}
			else
			{
				current=(ename+j1)->number; 
				/*current is the external name
				 of the j1-th pin*/
				(le+j1)->number=1;
				(mark+j1)->number=1;
			}
		}
		else
		{ /*internal name not used up*/
			if ((iname+j1)->number ==0)  /*a pin*/
			{
				flag=1;
				(li+j1)->number=1;
				if ((le+j1)->number>0)
				{
					/* just an x for pin needs to be drawn*/
					goto labend;
				}
				else
				{
					current=(ename+j1)->number;
					(le+j1)->number=1;
					(mark+j1)->number=1;
				}
			}  /* end of a pin*/
			else
			{
				current=(iname+j1)->number;
				(li+j1)->number=1;
				(mark+j1)->number=1;
			}
		}  /* end internal name not used up*/
		/*find other pins*/
		cnt=1;
		for (j=(j1+1);j<=dim;j++)
		{
			if ((iname+j)->number==current)
			{
				(mark+j)->number=1;
				(li+j)->number=1;
				cnt++;
			}
			if ((ename+j)->number == current)
			{
				(mark+j)->number=1;
				(le+j)->number=1;
				cnt++;
			}
		}

		xo=0;
		yo=0;

		for (j=j1;j<=dim;j++)
		{
			if ((mark+j)->number > 0)
			{
				xo=xo+(x+j)->number;
				yo=yo+(y+j)->number;
			}
		}
		xo=xo/cnt;
		yo=yo/cnt;

                if (dx>=1)
                {
			xo=clear+dx*xo;
                }
                else
                {
                	xo=clear+xo/scalex;
                }
                if (dy>=1)
                {
			yo=clear+dy*yo;
                }
                else
                {
                	yo=clear+yo/scaley;
                }

		for (j=j1;j<=dim;j++)
		{
			if ((mark+j)->number > 0)
			{
				(mark+j)->number =0;
                                if (dx>=1)
                                {
					xt=clear+dx*((x+j)->number);
                                } 
                                else
                                {
                                	xt=clear+((x+j)->number)/scalex;
                                }
                                if (dy>=1)
                                {
					yt=clear+dy*((y+j)->number);
                                }
                                else
                                {
                                	yt=clear+((y+j)->number)/scaley;
                                }
				Gr_move(xo,yo);
				Gr_draw(xt,yt);
			}
		}
labend:;
		if (flag>0)
		{
                        if (dx>=1)
                        {
				xo=clear+dx*((x+j1)->number);
                        }
                        else
                        {
                        	xo=clear+((x+j1)->number)/scalex;
                        }
                        if (dy>=1)
                        {
				yo=clear+dy*((y+j1)->number);
                        }
                        else
                        {
                        	yo=clear+((y+j1)->number)/scaley;
                        }
			Gr_cross(xo,yo);
			/*
			This lines are for debugging
			on HP terminal.
			Gr_move((xo+2),(yo-2));
			printf("%d",j1);
			Gr_move(xo,yo);
			*/
			flag=0;
		}
		if (((le+j1)->number)>0)
		{
			j1++;
		}
	}

	GR_FREE((char*) x);
	GR_FREE((char*) y);
	GR_FREE((char*) iname);
	GR_FREE((char*) ename);
	GR_FREE((char*) li);
	GR_FREE((char*) le);
	GR_FREE((char*) mark);
}

#endif HP_PLOT

