/*
 * grSubNet.c --
 *
 *     Malgorzata Marek-Sadowska
 *
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint



#include "grInt.h"
#define MAX_LENGTH 50



/*
*------------------------------------------------------------------------------
*	grPinRectangle *Gr_DuplicatePinRectangle(kPtr,pinRectanglePtr)
*
*	Results		: for a given pointer to pinRectangle 
*			it returns a pointer to a duplicate pinRectangle 
*	Side effects	: all siblings within the same subnet 
*			are also duplicated.                 
*-----------------------------------------------------------------------------
*/

grPinRectangle *Gr_DuplicatePinRectangle(kPtr,pinRectanglePtr)

grRegion *kPtr;
grPinRectangle *pinRectanglePtr;


{

	grPinRectangle *new,*current,*old,*save;

	old=NULL;
	current=pinRectanglePtr;
	save=NULL;

	while(current!=NULL)
	{
		new=Gr_AllocateMemoryForNewPinRectangle();

		if (save==NULL) save=new;
		new->firstxPin=current->firstxPin;
		new->firstyPin=current->firstyPin;
		new->count=current->count;
		new->cutMark=current->cutMark; 
		if (old!=NULL) 
		{
			old->nextInNet=new;
		}

		if (kPtr->firstPinRectanglePtr==NULL)
		{
			kPtr->firstPinRectanglePtr=new;
			kPtr->lastPinRectanglePtr=new;
		}
		else
		{
			kPtr->lastPinRectanglePtr->nextOnList=new;
			kPtr->lastPinRectanglePtr=new;
		}

		old=new;
		current=current->nextInNet;
	} /*end while loop*/

	return (save);

}

/*
*----------------------------------------------------------------------------
*	void Gr_ReadTimingData(fp,kPtr)
*	Results		: none.
*	Side effects	: data from timing file are read in.
*-----------------------------------------------------------------------------
*/


void Gr_ReadTimingData(fp,kPtr)

FILE *fp;
grRegion *kPtr;

{

	grSubNet *where;
	int noCrit,i,j;
	char buffer[MAX_LENGTH];
	grNet *curNetPtr;
	float trash;

	fscanf(fp,"%d",&noCrit);
	printf("\n noCrit=%4d",noCrit);
	fscanf(fp,"%f",&gr_HorVertWire);

	if (noCrit > 0)
	{

		for (i=1;i<=noCrit;i++)
		{
			fscanf (fp, "%s", buffer);

			for (j=1;j<=gr_NetCount;j++)
			{
				if (!strcmp(buffer,(gr_HeadKeyToNets+j)->name))
				{
				curNetPtr=(gr_HeadGrNets+j);
				fscanf(fp,"%f",&(curNetPtr->maxlength));
				/*
				printf("\n net number %4d",j);
				puts(buffer);
				printf("   length=%.2f",(curNetPtr->maxlength));
				*/
				curNetPtr->critical=1;

					where=kPtr->subNetsPtr;
					while(where!=NULL)
					    {
						if (where->subname == j)
						    {
							where->decCandidate = 1;
							where->critical = 1;
							goto lab0;
						}
						where=where->nextOnList;
					}
				}
			}
			printf("\n TIMING DATA FILE :");
			printf("\n net ");
			puts(buffer);
			printf(" does not exist in data base,");
			printf(" timing info ignored");
			fscanf(fp,"%f",&trash);
lab0:;
		}
	}
}


/*
*--------------------------------------------------------------------------
*	BearStatus Gr_InitializeSubNets(kPtr)
*	Results		: BearStatus.
*	Side effects	: for the initial region generate
*			the list of SubNets 
*			and attach them to that region.
*			Initial timing information is read in and calculated.
*----------------------------------------------------------------------------
*/

BearStatus Gr_InitializeSubNets(kPtr)

grRegion *kPtr;

{
	BearString name;
	grNet *curNetPtr;
	grSubNet *whereAssignPtr;
	FILE *fp,*fopen();
	float total;
	int j;
	int gRoutName;
	int length,width;
	char *tmFileNamePtr; /* name of timing file */
	char answer[80], pp[10], ap[10];

	kPtr->firstPinRectanglePtr=NULL;
	kPtr->lastPinRectanglePtr=NULL;
	kPtr->subNetsPtr=NULL;

	for (j=1;j<=gr_NetCount;j++)
	{
		curNetPtr=(gr_HeadGrNets+j);
		Gr_CreateNewSubnet(kPtr);
		whereAssignPtr=kPtr->subNetsPtr;   /*make assignments*/
		whereAssignPtr->netname = curNetPtr->name;
		gRoutName=curNetPtr->name;

		whereAssignPtr->subname=curNetPtr->name;
		whereAssignPtr->toBeConnected = 
		Gr_DuplicatePinRectangle(kPtr,(curNetPtr->loosePins));
		if (curNetPtr->loosePins!=NULL)
		{
			Gr_UpdateBoundingBox(whereAssignPtr->toBeConnected);
			if (gr_Timing == 1) 
			{
				/* all lengths in timing mode calculated*/

				Gr_PutInitialBoundBox(j,
					whereAssignPtr->toBeConnected);
				length=Gr_CalculateHorizontalPortionOfTree(
					whereAssignPtr->toBeConnected);
				curNetPtr->xapprox=length;
				whereAssignPtr->xapprox=length;
				width = Gr_CalculateVerticalPortionOfTree(
					whereAssignPtr->toBeConnected);
				curNetPtr->yapprox=width;
				whereAssignPtr->yapprox=width;
				total=length+width*gr_HorVertWire;
				whereAssignPtr->approxLength=total;
				curNetPtr->currentLength=total;
			}
		}
		whereAssignPtr->areConnected =
		Gr_DuplicatePinRectangle(kPtr,(curNetPtr->areConnected));
		whereAssignPtr->noLoosePins=curNetPtr->noLoosePins;
		if ((whereAssignPtr->noLoosePins > 2)
		&&(whereAssignPtr->noLoosePins < 15))
		{
			whereAssignPtr->decCandidate=1;
		}
		whereAssignPtr->noComponents=curNetPtr->noComponents;
		Gr_PutJointBox(whereAssignPtr);
	}
	if (gr_Timing == 1)
	{
	    tmFileNamePtr = Mac_GetTimingFileNameOfCellMaster(grCellMasterPtr);
	    if (tmFileNamePtr == NIL(char))
	    {
		printf ("Enter name of timing file : ");
		scanf ("%s", answer);

		if ((fp = fopen (answer, "r")) == NIL(FILE)) 
		{
	Bear_PrintErr("Gr_InitializeSubNets() failed : cannot open timing file");
		    return (BEAR_ERROR);
		}

	        MALLOC(char *, tmFileNamePtr,
		    (strlen (answer) + 1) * sizeof (char));
	        strcpy (tmFileNamePtr, answer);
	        Mac_PutTimingFileNameOfCellMaster(tmFileNamePtr,	
		    grCellMasterPtr);
	    }
	    else
	    {
		if ((fp = fopen (tmFileNamePtr, "r")) == NIL(FILE)) 
		{
	Bear_PrintErr("Gr_InitializeSubNets() failed : cannot open timing file");
		    return (BEAR_ERROR);
		}
	    }

	    printf ("\n reading in timing file \n"); fflush (stdout);
	    Gr_ReadTimingData(fp, kPtr);
	    fclose (fp); 
	}

	return (BEAR_OK);
}

/*
*-----------------------------------------------------------------------------
*	void Gr_MarkVertIntersectOfPinRectangle(pinRectangle,xpos)
*	Results		: none.
*	Side effects	: cutMarks are set .                             
*	Check if the given pinRectangle is vertically divided at xpos.
*	If formal touching occurs, pins are checked for visibility.        
*-----------------------------------------------------------------------------
*/

void Gr_MarkVertIntersectOfPinRectangle(pinRectangle,xpos)

grPinRectangle *pinRectangle;
int xpos;


{

	int cojest,lflag,rflag;
	grSortedPin *pendingPin;

	cojest=Gr_VertBoxCut(pinRectangle,xpos);

	if ((cojest==2)||(cojest==3))
	    {
		pendingPin=pinRectangle->firstxPin;
		lflag=0;
		rflag=0;
		if (pinRectangle->minx==pinRectangle->maxx) /*flat box*/
		{
			while(pendingPin!=NULL)
			{
				if ((pendingPin->pinPtr->shade)==WEST_PIN)
				{
					lflag=1;
				}
				if ((pendingPin->pinPtr->shade)==EAST_PIN)
				{
					rflag=1;
				}
				pendingPin=pendingPin->nextxPin;
			} /*end while*/
			if ((rflag+lflag)==2) cojest=1;
			if ((rflag==1)&&(lflag==0)) cojest=2;
			if ((rflag==0)&&(lflag==1)) cojest=3;
		} /*end flat box*/
		else
		{
			if (cojest==2)
			{

				/*formal touching at the left boundary */


				while((pendingPin!=NULL)&&(cojest!=1))
				{
					if ((pendingPin->pinPtr->x==xpos)
					&&((pendingPin->pinPtr->shade)==
					WEST_PIN))/*CHANGE*/
					{
					    cojest=1;
					}
					pendingPin=pendingPin->nextxPin;
				}
			}

			if (cojest==3)
			{
				/* formal touching at the right boundary */


				while((pendingPin!=NULL)&&(cojest!=1))
				{
					if (((pendingPin->pinPtr->x)==xpos)
					&&((pendingPin->pinPtr->shade)
					==EAST_PIN))/*CHANGE HERE*/
					{
					    cojest=1;
					}
					pendingPin=pendingPin->nextxPin;
				}
			}

		}
	}


	if (cojest==2) cojest=4;
	if (cojest==3) cojest=5;

	pinRectangle->cutMark=cojest;
}

/*
*-----------------------------------------------------------------------------
*	void Gr_MarkHorzIntersectOfPinRectangle(pinRectangle,ypos)
*	Side effect : marks cutMarks                    
*	Check if the given pinRectangle is horizontally 
*	divided at ypos. 
*-----------------------------------------------------------------------------
*/



void Gr_MarkHorzIntersectOfPinRectangle(pinRectangle,ypos)

grPinRectangle *pinRectangle;
int ypos;


{

	int cojest,lflag,rflag;
	grSortedPin *pendingPin;


	cojest=Gr_HorzBoxCut(pinRectangle,ypos);

	if ((cojest==2)||(cojest==3))
	{
		lflag=0;
		rflag=0;
		pendingPin=pinRectangle->firstyPin;
		if (pinRectangle->miny==pinRectangle->maxy)  /*flat*/
		{
			while(pendingPin!=NULL)
			{
				if ((pendingPin->pinPtr->shade)==NORTH_PIN)
				{
				    lflag=1;
				}
				if ((pendingPin->pinPtr->shade)==SOUTH_PIN)
				{
				    rflag=1;
				}
				pendingPin=pendingPin->nextxPin;
			}
			if ((rflag+lflag)==2) cojest=1;
			if ((rflag==1)&&(lflag==0)) cojest=2;
			if ((rflag==0)&&(lflag==1)) cojest=3;
		}/*end flat*/
		else
		{
			if (cojest==2)
			{
				/* formal touch at the top */

				while((pendingPin!=NULL)&&(cojest!=1))
				{
					if (((pendingPin->pinPtr->y)==ypos)
					&&((pendingPin->pinPtr->shade)
					==NORTH_PIN))
					{
					    cojest=1;
					}
					pendingPin=pendingPin->nextyPin;
				}
			}

			if (cojest==3)
			{
				/*formal touch at the bottom*/


				while((pendingPin!=NULL)&&(cojest!=1))
				{
					if (((pendingPin->pinPtr->y)==ypos)
					&&((pendingPin->pinPtr->shade)
					==SOUTH_PIN))
					{
					    cojest=1;
					}
					pendingPin=pendingPin->nextyPin;
				}
			}

		}
	}

	if (cojest==2) cojest=4;
	if (cojest==3) cojest=5;
	pinRectangle->cutMark=cojest;
}

/*
*----------------------------------------------------------------------------
*	void Gr_CheckSubNets(kPtr)
*	Results		: none.
*	Side effects	: for a region pointed to by kPtr
*			after cut selection marks all 
*			pinRectangles for resident nets if they intersect
*			with cutline.    
*------------------------------------------------------------------------------
*/


void Gr_CheckSubNets(kPtr)

grRegion *kPtr;


{

	int dir,xv,yv;
	grSubNet *pendingSubNet;
	grPinRectangle *pendingPinRectangle;

	m=0; /* clear old m*/
	pendingSubNet=kPtr->subNetsPtr; /*initialize pending subNet*/
	dir=kPtr->dir; /*get assigned direction of cut*/

	if (dir>0)
	{
		xv=kPtr->xcutvirt;
		yv=kPtr->ycutvirt;
		xv=(gr_xcoord+xv)->number;
		yv=(gr_ycoord+yv)->number;

		if (dir==1)  /* vertical cut/
		{
			while(pendingSubNet!=NULL)
			{
				pendingPinRectangle
				=pendingSubNet->toBeConnected;
				if (pendingPinRectangle!=NULL)
				{
					Gr_MarkVertIntersectOfPinRectangle(
					pendingPinRectangle,xv);
				}
				pendingPinRectangle=
				pendingSubNet->areConnected;
				while(pendingPinRectangle!=NULL)
				{
					Gr_MarkVertIntersectOfPinRectangle(
					pendingPinRectangle,xv);
					pendingPinRectangle=
					pendingPinRectangle->nextInNet;
				}
				pendingSubNet=pendingSubNet->nextOnList;
			}
		}
								
		    									if (dir==2) /*horizontal cut */
	{
		while(pendingSubNet!=NULL)
		{
			pendingPinRectangle=pendingSubNet->toBeConnected;
			if (pendingPinRectangle!=NULL)
			{
				Gr_MarkHorzIntersectOfPinRectangle(
				pendingPinRectangle,yv);
			}
			pendingPinRectangle=pendingSubNet->areConnected;

			while(pendingPinRectangle!=NULL)
			{
				Gr_MarkHorzIntersectOfPinRectangle(
				pendingPinRectangle,yv);
				pendingPinRectangle=
				pendingPinRectangle->nextInNet;
			}
			pendingSubNet=pendingSubNet->nextOnList;
		}
	}
   }
}

