
/* grSweep.c --
 *
 *  Malgorzata Marek-Sadowska
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "grInt.h"

/*
*----------------------------------------------------------------------------
*   Functions in this file perform various tasks related to 
*   sweeping of the parent region to determine if internal
*   blockages exist.
*----------------------------------------------------------------------------
*/



/*
*------------------------------------------------------------------------------
*   Gr_IsAssignmentPossible
*   Results     : if no deadRegions exist, return 0 : linear assignment;
*   if they do, return 1 : pull assignment              
*   Side effects    : none.
*------------------------------------------------------------------------------
*/


int Gr_IsAssignmentPossible(kPtr)

grRegion *kPtr;     /* Pointer to a parent region.          */


{
    int status;
    
    status = 0;
    if ((kPtr->dir == 1)&&(n > 1))
    {
        status = Gr_HorzSweepDeadRegions(kPtr);
        Gr_TerminateListOfDeadRegions();
    }
    if ((kPtr->dir == 2)&&(n > 1))
    {
        status = Gr_VertSweepDeadRegions(kPtr);
        Gr_TerminateListOfDeadRegions();
    }

    return (status);
}


/*
*---------------------------------------------------------------------------
*   Gr_CreateDeadRegion()
*   Results     : assign memory for new deadRegion and initialize.
*   Side effects    : none.
*---------------------------------------------------------------------------
*/



grDeadRegion *Gr_CreateDeadRegion()


{

    grDeadRegion *new;

    
    MALLOC(grDeadRegion*,new,sizeof (grDeadRegion));
    
    new->xleft=0;
    new->xright=0;
    new->ybot=0;
    new->ytop=0;
    new->side=0;
    new->botLeftHoleNr=0;
    new->topRightHoleNr=0;
    new->pullControl=0;
    new->region=NULL;
    new->next=NULL;
    new->partner=NULL;
    new->dr_inline=NULL;

    return (new);
}


/*
*-----------------------------------------------------------------------------
*   Gr_GetYVirtPosOfAbsPos
*   Results     :for a given absolute y-position returns corresponding
*            virtual coordinate.
*   Side effects    :none.
*-----------------------------------------------------------------------------
*/




int Gr_GetYVirtPosOfAbsPos(ypos)

int ypos;   /* Absolute y-coordinate.       */

{

    int i;
    
    for (i=0; i <= gr_totygrid; i++)
    {
        if ((gr_ycoord+i)->number == ypos)
        {
            return (i);
        }
    }
    return (-1);
}
            
/*
*--------------------------------------------------------------------------
*   Gr_GetFirstHoleAbove
*   Results     : number of the first hole above ypos.
*             if no hole exists, returns 0. 
*   Side effects    : none.
*--------------------------------------------------------------------------
*/


        
int Gr_GetFirstHoleAbove(ypos)

int ypos;   /* y-coordinate above which a hole is sought        */

{

    int i;
    
    for (i=1;i<= n; i++)
    {
        if ((grHeadHolePtr+i)->botOrLeft_p_y
        >= ypos)
        {
            return (i);
        }
    }
    return (0);
}


/*
*----------------------------------------------------------------------------
*   Gr_GetLastHoleBelow
*   Results     :  number of the first hole below ypos.
*              if no such a hole exists, 0.  
*   Side effects    : none.
*----------------------------------------------------------------------------
*/


int Gr_GetLastHoleBelow(ypos)

int ypos;       /* Absolute y-coordinate.       */


{
    int i,j;

    for (i=1;i<=n;i++)
    {
        j=n+1-i;
                              
        if ((grHeadHolePtr+j)->botOrLeft_p_y < ypos)
        {
            return (j);
        }
    }
    return (0);
}


/*
*---------------------------------------------------------------------------
*   Gr_IsHorzStripOpaque
*   Results     : checks if a strip in Bitmap from xlvirt to xrvirt
*   at horizontal position yvirt is completely opaque.
*   If yes - returns 1; otherwise 0.
*   Side effects    : none.
*---------------------------------------------------------------------------
*/


int Gr_IsHorzStripOpaque(yvirt,xlvirt,xrvirt)

int yvirt;      /* Virtual y-coordinate.            */
int xlvirt,xrvirt;  /* Virtual x coordinates bounding the strip.    */


{

    int i,pom;

    for (i = (xlvirt+1); i <= xrvirt; i++)
    {
        pom=(gr_Bitmap + Gr_WhoAmI((yvirt+1),i))->number;
        if ((pom == 0)||(pom > grSolidTileCount))
        {
            return (0); /* is not opaque */
        }
    }
    return (1);
}


/*
*----------------------------------------------------------------------------
*   Gr_HorzSweepDeadRegions
*   Results     : 0 is returned if no internal blockage exist or
*   if it does, but it is fatal. In the latter case
*   warning is printed. Fatal blockage exists if
*   in initial problem there existed isolated regions.
*   This is a case, when no solution can be found.
*             1 is returned when deadRegions are present.
*   Side effects    : during horizontal sweep of grRegion deadRegions
*   are created, if they exist. 
*----------------------------------------------------------------------------
*/

int 
Gr_HorzSweepDeadRegions(kPtr)
grRegion *kPtr;     /* Pointer to the parent region.    */
{
    grDeadRegion *current,*left,*right,*last,*lastended;
    int ybVirtL,yVirtHole;
    int i,xcutv,xlv,xrv;
    int botHole,checkHole;
    int yVtop;
    int boolL, boolR;

    grHeadDeadRegion= NULL;
    lastended = NULL;
    left = Gr_CreateDeadRegion();
    right = Gr_CreateDeadRegion();
    
    xcutv=kPtr->xcutvirt;
    xlv=kPtr->xvirt;
    xrv=xlv+kPtr->whvirt;
    ybVirtL=kPtr->yvirt;
    yVtop=ybVirtL+kPtr->hvirt;

    left->xleft=(gr_xcoord+xlv)->number;
    left->xright=(gr_xcoord+xcutv)->number;
    left->ybot=(gr_ycoord+ybVirtL)->number;
    left->side=0;
    left->botLeftHoleNr=1;
    
    right->xleft=(gr_xcoord+xcutv)->number;
    right->xright=(gr_xcoord+xrv)->number;
    right->ybot=left->ybot;
    right->side=1;
    right->botLeftHoleNr=1;


    last=left;

    botHole=Gr_GetFirstHoleAbove(last->ybot);
    if (botHole == 0)
    {
        botHole =1;
    }

    lab: ;
    yVirtHole=Gr_GetYVirtPosOfAbsPos((grHeadHolePtr+
    botHole)->botOrLeft_p_y);


    for (i=(yVirtHole+1);i < yVtop; i++) 
    {
        boolL=Gr_IsHorzStripOpaque(i,xlv,xcutv);
        boolR=Gr_IsHorzStripOpaque(i,xcutv,xrv);

        checkHole=Gr_GetFirstHoleAbove(
        (gr_ycoord+i)->number);

        if ((boolL==1)&&(boolR==1))
        {
            /* both strips are opaque, such a situation is 
            correct only if both regions are terminating*/
        
            if (checkHole !=0)
            {
                printf("\n TOTAL BLOCKAGE");
                printf(" SOME NETS MAY BE DISCONNECTED");

                if (grHeadDeadRegion==NULL)
                {
                    GR_FREE((char*)left);
                    GR_FREE((char*)right);
                }
                else
                {
                    while(grHeadDeadRegion!=NULL)
                    {
                        last=grHeadDeadRegion->
                        next;
                        GR_FREE((char*)grHeadDeadRegion);
                        grHeadDeadRegion=last;
                    }
                }
                return (0);
            }
            else
            {
                if (grHeadDeadRegion==NULL)
                {
                    GR_FREE((char*)left);
                    GR_FREE((char*)right);
                    return (0);
                }
                else
                {
                    left->topRightHoleNr=n;
                    right->topRightHoleNr=n;
                    left->ytop=((gr_ycoord
                    +yVtop)->number);
                    right->ytop=
                    left->ytop;
                    left->partner=right;
                    lastended->dr_inline=left;
                    lastended=left;
                    lastended->dr_inline=NULL;
                    if (right->ybot > left->ybot)
                    {
                        left->botLeftHoleNr=
                        Gr_GetFirstHoleAbove(right->
                        ybot);
                    }
                    else
                    {
                        left->botLeftHoleNr=
                        Gr_GetFirstHoleAbove(left->
                        ybot);
                    }
                        right->botLeftHoleNr=
                        left->botLeftHoleNr;
                    return (1);
                }
            }
        }


        if ((boolL==1)&&(boolR==0)&&(checkHole!=0))
        {
            /* left region terminates */

            if (grHeadDeadRegion==NULL)
            {
                grHeadDeadRegion=left;
                left->next=right;
                last=right;
            }

            left->partner=right;
            left->ytop=((gr_ycoord+i)->number);
            left->topRightHoleNr=Gr_GetLastHoleBelow
            (left->ytop);
            if (right->ybot > left->ybot)
            {
                left->botLeftHoleNr=Gr_GetFirstHoleAbove
                (right->ybot);
            }
            else
            {
                left->botLeftHoleNr=Gr_GetFirstHoleAbove
                (left->ybot);
            }

            if (lastended == NULL)
            {
                lastended=left;
            }
            else
            {
                lastended->dr_inline=left;
                lastended=left;
            }

            current=Gr_CreateDeadRegion();
            current->xleft=left->xleft;
            current->xright=left->xright;
            current->ybot=left->ytop;
            current->side=left->side;

            last->next=current;
            left=current;
            last=current;

            botHole=checkHole;
            goto lab ;
        }           
        if ((boolR==1)&&(boolL==0)&&(checkHole!=0))
        {
            /* right region terminates */

            if (grHeadDeadRegion==NULL)
            {
                grHeadDeadRegion=right;
                right->next=left;
                last=left;
            }

            right->partner=left;
            right->ytop=((gr_ycoord+i)->number);
            right->topRightHoleNr=Gr_GetLastHoleBelow
            (right->ytop);
            if (right->ybot > left->ybot)
            {
                right->botLeftHoleNr=Gr_GetFirstHoleAbove
                (right->ybot);
            }
            else
            {
                right->botLeftHoleNr=Gr_GetFirstHoleAbove
                (left->ybot);
            }

            if (lastended == NULL)
            {
                lastended=right;
            }
            else
            {
                lastended->dr_inline=right;
                lastended=right;
            }

            current=Gr_CreateDeadRegion();
            current->xleft=right->xleft;
            current->xright=right->xright;
            current->ybot=right->ytop;
            current->side=right->side;

            last->next=current;
            right=current;
            last=current;

            botHole=checkHole;
            goto lab ;
        }           
    }
    if (grHeadDeadRegion!=NULL)
    {
        lastended->dr_inline=left;
        lastended=left;
        lastended->dr_inline=NULL;

        left->topRightHoleNr=n;
        right->topRightHoleNr=n;
        if (right->ybot > left->ybot)
        {
            left->botLeftHoleNr=
            Gr_GetFirstHoleAbove(right->ybot);
        }
        else
        {
            left->botLeftHoleNr=
            Gr_GetFirstHoleAbove(left->ybot);
        }
        
        right->botLeftHoleNr = left->botLeftHoleNr ;

        left->ytop=((gr_ycoord+yVtop)->number);
        right->ytop=left->ytop;
        left->partner=right;
        right->partner=NULL;
        return (1);
    }
    return (0);
}



/*
*-------------------------------------------------------------------------
*   The procedures below do the same things for x-coordinate as
*   the procedures above do for y- coordinate.
*-------------------------------------------------------------------------
*/



/*
*-----------------------------------------------------------------------------
*   Gr_GetXVirtPosOfAbsPos
*   Results     :for a given absolute x-position returns corresponding
*            virtual coordinate.
*   Side effects    :none.
*-----------------------------------------------------------------------------
*/

int Gr_GetXVirtPosOfAbsPos(xpos)

int xpos;


{

    int i;
    
    for (i=0; i <= gr_totxgrid; i++)
    {
        if ((gr_xcoord+i)->number == xpos)
        {
            return (i);
        }
    }
    return (-1);
}
            
        
/*
*--------------------------------------------------------------------------
*   Gr_GetFirstHoleRight
*   Results     : number of the first hole right xpos.
*             if no hole exists, returns 0. 
*   Side effects    : none.
*--------------------------------------------------------------------------
*/


int Gr_GetFirstHoleRight(xpos)

int xpos;   /* Absolute x coordinate.       */

{

    int i;
    
    for (i=1;i<= n; i++)
    {
        if ((grHeadHolePtr+i)->botOrLeft_p_x
        >= xpos)
        {
            return (i);
        }
    }
    return (0);
}



/*
*----------------------------------------------------------------------------
*   Gr_GetLastHoleLeft
*   Results     :  number of the first hole left of xpos.
*              if no such a hole exists, 0.  
*   Side effects    : none.
*----------------------------------------------------------------------------
*/

int Gr_GetLastHoleLeft(xpos)

int xpos;       /* Absolute x coordinate.       */


{
    int i,j;

    for (i=1;i<=n;i++)
    {
        j=n+1-i;

        if ((grHeadHolePtr+j)->botOrLeft_p_x < xpos)
        {
            return (j);
        }
    }
    return (0);
}



/*
*---------------------------------------------------------------------------
*   Gr_IsVertStripOpaque
*   Results     : checks if a strip in Bitmap from ybvirt to ytvirt
*   at vertical position xvirt is completely opaque.
*   If yes - returns 1; otherwise 0.    
*   Side effects    : none.
*--------------------------------------------------------------------------
*/



int Gr_IsVertStripOpaque(xvirt,ybvirt,ytvirt)

int xvirt;      /* x-coordinate position.       */
int ybvirt,ytvirt;  /* bounding y positions.        */


{

    int i,pom;

    for (i = (ybvirt+1); i <= ytvirt; i++)
    {
        pom=(gr_Bitmap + Gr_WhoAmI(i,(xvirt+1)))->number;
        if ((pom == 0)||(pom > grSolidTileCount))
        {
            return (0); /* is not opaque */
        }
    }
    return (1);
}


/*
*-------------------------------------------------------------------------
*   Gr_VertSweepDeadRegions
*   Results     : 0 is returned if no internal blockage exist or
*   if it does, but it is fatal. In the latter case
*   warning is printed. Fatal blockage exists if
*   in initial problem there existed isolated regions.
*   This is a case, when no solution can be found.
*             1 is returned when deadRegions are present.
*   Side effects    : During vertical sweep of grRegion deadRegions
*   are created, if they exist. 
*----------------------------------------------------------------------------
*/

int Gr_VertSweepDeadRegions(kPtr)

grRegion *kPtr;     /* Pointer to the parrent region.       */


{

    grDeadRegion *current,*left,*right,*last, *lastended;

    int xbVirtL,xVirtHole;
    int i,ycutv,ylv,yrv;
    int botHole,checkHole;
    int xVtop;
    int boolL, boolR;

    grHeadDeadRegion= NULL;
    lastended = NULL;
    left = Gr_CreateDeadRegion();
    right = Gr_CreateDeadRegion();
    
    ycutv=kPtr->ycutvirt;
    ylv=kPtr->yvirt;
    yrv=ylv+kPtr->hvirt;
    xbVirtL=kPtr->xvirt;
    xVtop=xbVirtL+kPtr->whvirt;

    left->xleft=(gr_xcoord+xbVirtL)->number;
    left->ytop=(gr_ycoord+yrv)->number;
    left->ybot=(gr_ycoord+ycutv)->number;
    left->side=0;
    left->botLeftHoleNr=1;
    
    right->xleft=left->xleft;
    right->ytop=left->ybot;
    right->ybot=(gr_ycoord+ylv)->number;
    right->side=1;
    right->botLeftHoleNr=1;

    last=left;

    botHole=Gr_GetFirstHoleRight(last->xleft);

    if (botHole == 0)
    {
        botHole =1;
    }

    lab: ;
    xVirtHole=Gr_GetXVirtPosOfAbsPos((grHeadHolePtr+
    botHole)->botOrLeft_p_x);


    for (i= (xVirtHole+1);i < xVtop; i++)
    {
        boolR=Gr_IsVertStripOpaque(i,ylv,ycutv);
        boolL=Gr_IsVertStripOpaque(i,ycutv,yrv);

        checkHole=Gr_GetFirstHoleRight((gr_xcoord+i)->number);

        if ((boolL==1)&&(boolR==1))
        {
            /* both strips are opaque, such a situation is 
            correct only if both regions are terminating*/
        
            if (checkHole !=0)
            {
                printf("\n TOTAL BLOCKAGE");
                printf("  SOME NETS MAY BE DISCONNECTED");

                if (grHeadDeadRegion==NULL)
                {
                    GR_FREE((char*)left);
                    GR_FREE((char*)right);
                }
                else
                {
                    while(grHeadDeadRegion!=NULL)
                    {
                        last=grHeadDeadRegion->
                        next;
                        GR_FREE((char*)grHeadDeadRegion);
                        grHeadDeadRegion=last;
                    }
                }
                return (0);
            }
            else
            {
                if (grHeadDeadRegion==NULL)
                {
                    GR_FREE((char*)left);
                    GR_FREE((char*)right);
                    return (0);
                }
                else
                {
                    left->topRightHoleNr=n;
                    right->topRightHoleNr = n;
                    left->xright=((gr_xcoord
                            +xVtop)->number);
                    right->xright= left->xright;
                    left->partner=right;
                    lastended->dr_inline=left;
                    lastended=left;
                    lastended->dr_inline=NULL;
                    if (right->xleft > left->xleft)
                    {
                        left->botLeftHoleNr=
                         Gr_GetFirstHoleRight(right->xleft);
                    }
                    else
                    {
                        left->botLeftHoleNr=
                          Gr_GetFirstHoleRight(left->xleft);
                    }
                        right->botLeftHoleNr=
                        left->botLeftHoleNr;

                    return (1);
                }
            }
        }

    
        if ((boolL==1)&&(boolR==0)&&(checkHole!=0))
        {
            /* left region terminates, right continues */

            if (grHeadDeadRegion==NULL)
            {
                grHeadDeadRegion=left;
                left->next=right;
                last=right;
            }

            left->partner=right;
            left->xright=((gr_xcoord+i)->number);
            left->topRightHoleNr=Gr_GetLastHoleLeft
            (left->xright);
            if (right->xleft > left->xleft)
            {
                left->botLeftHoleNr=Gr_GetFirstHoleRight
                (right->xleft);
            }
            else
            {
                left->botLeftHoleNr=Gr_GetFirstHoleRight
                (left->xleft);
            }
            if (lastended == NULL)
            {
                lastended=left;
            }
            else
            {
                lastended->dr_inline=left;
                lastended=left;
            }


            current=Gr_CreateDeadRegion();
            current->ybot=left->ybot;
            current->ytop=left->ytop;
            current->xleft=left->xright;
            current->side=left->side; 

            last->next=current;
            last=current;
            left=current;
            botHole=checkHole;
            goto lab;
        }           
        if ((boolR==1)&&(boolL==0)&&(checkHole!=0))
        {
            /* right region terminates, left continues */

            if (grHeadDeadRegion==NULL)
            {
                grHeadDeadRegion=right;
                right->next=left;
                last=left;
            }

            right->partner=left;
            right->xright=((gr_xcoord+i)->number);
            right->topRightHoleNr=Gr_GetLastHoleLeft
            (right->xright);
            if (right->xleft > left->xleft)
            {
                right->botLeftHoleNr=Gr_GetFirstHoleRight
                (right->xleft);
            }
            else
            {
                right->botLeftHoleNr=Gr_GetFirstHoleRight
                (left->xleft);
            }
            if (lastended == NULL)
            {
                lastended=right;
            }
            else
            {
                lastended->dr_inline=right;
                lastended=right;
            }


            current=Gr_CreateDeadRegion();
            current->ybot=right->ybot;
            current->ytop=right->ytop;
            current->xleft=right->xright;
            current->side=right->side;

            last->next=current;
            last=current;
            right=current;
            botHole=checkHole;
            goto lab;
        }           
    }


    if (grHeadDeadRegion!=NULL)
    {
        left->topRightHoleNr=n;
        right->topRightHoleNr=n;
        right->partner = NULL;
        left->xright=((gr_xcoord+xVtop)->number);
        right->xright= left->xright;
        left->partner=right;
        lastended->dr_inline=left;
        lastended=left;
        lastended->dr_inline=NULL;
        if (right->xleft > left->xleft)
        {
            left->botLeftHoleNr=Gr_GetFirstHoleRight
            (right->xleft);
        }
        else
        {
            left->botLeftHoleNr=Gr_GetFirstHoleRight
            (left->xleft);
        }
         
        right->botLeftHoleNr = left->botLeftHoleNr;
        
        return (1);
    }
    return (0);
}


/*
*-----------------------------------------------------------------------------
*   Gr_TerminateListOfDeadRegions()
*   Results     : none.
*   Side effects    :
*             the last dead region is attached
*   to the list of dead regions. There is not assignment from
*   there, just drop down.
*-----------------------------------------------------------------------------
*/




void Gr_TerminateListOfDeadRegions()


{
    grDeadRegion *current;

        if (grHeadDeadRegion == NULL)
        {
            return;
        }

        current=grHeadDeadRegion;
    
        while(current->dr_inline != NULL)
        {
            current=current->dr_inline;
        }
        current->dr_inline = current->partner;

}
        

