
#include "bearCopyRight.h"
#include "grInt.h"

BearStatus
GrGeoRead(planeDataPtr)
PlaneData *planeDataPtr;
/*
 * This subroutine reads in informations about sizes of blocks and  
 * bottleneck tiles.                                                
 * Results : none                                               
 * Side effects : grSolidTileCount and grBottleneckTileCount are set
 * List of grBlocks pointed to by grHeadBlockPtr at the beginning   
 * and by grLastBlockPtr at the end of the list gets initialized   
 * and geometric informations about the blocks are put in.        
 */
{
    grSolidTileCount=0;
    Gr_FillSolidTiles();

    Gr_ReadAllBnLines(planeDataPtr);
    Gr_FillInPsBlocks();
    
    grBottleneckTileCount=grSolidTileCount;
    if (GrFillHBottlenecks() == BEAR_ERROR) {
        return (BEAR_ERROR);
    }
    Gr_FillPsHBottlenecks();

    if (GrFillVBottlenecks() == BEAR_ERROR) {
        return (BEAR_ERROR);
    }
    Gr_FillPsVBottlenecks();

    Gr_ConnectDummyCells();

    if( Gr_SqueezDCells() == 0 ) {
        Bear_PrintErr("GrGeoRead() failed : some bottlenecks are too thin");
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}


void
Gr_FillInPsBlocks()
/*
 * If bottleneck lines are present it puts in pseudo blocks which
 * eliminate all the bottleneck lines. 
 * If the bottleneck lines list is empty, it does nothing 
 */
{

    grBnLine *vList,*hList,*run;

    if (grBnLineList == NULL) 
        return ;

    vList = NULL;
    hList = NULL;

    if (grBnLineList->type == 0)
    {
        vList=grBnLineList;
    }
    else
    {
        hList=grBnLineList;
    }

    run=grBnLineList;

    while(run!=NULL)
    {
        if (run->type != grBnLineList->type)
        {
            if (vList==NULL)
            {
                vList=run;
            }
            else
            {
                hList=run;
            }
            goto lab1;
        }
        run=run->nextBnLine;
    }
lab1:;

    Gr_NECorner(vList,hList);
    Gr_NWCorner(vList,hList);
    Gr_SECorner(vList,hList);
    Gr_SWCorner(vList,hList);
    Gr_LeftHorz(hList,vList);
    Gr_LeftVert(hList,vList);
}

void Gr_NECorner(vList,hList)

grBnLine *vList,*hList;

/*
Puts pseudo block for North-East Corner of bottleneck lines.
*/

{

    int x,y,xa,yb;
    grBnLine *vrun,*hrun;

    vrun=vList;
    hrun=hList;

    if ((vrun==NULL)||(hrun==NULL))
        return;

    while((vrun!=NULL)&&(vrun->type!=1))
    {
        if (vrun->cover !=1)
        {
            x=vrun->base;
            y=vrun->lb;
            hrun=hList;
            while((hrun!=NULL)&&(hrun->type !=0))
            {
                if (hrun->cover != 1)
                {
                    xa=hrun->lb;
                    yb=hrun->base;
                    if ((xa==x)&&(yb==y))
                    {
                        vrun->substitute=5;
                        hrun->substitute=1;
                        vrun->cover=1;
                        hrun->cover=1;
                        grSolidTileCount++;

                        Gr_CreatePseudoBlock(
                    grSolidTileCount,hrun->substitute,x,y);


                        goto lab1;
                    }
                }
                hrun=hrun->nextBnLine;
            }
lab1:;
        }
        vrun=vrun->nextBnLine;
    }
}

void Gr_NWCorner(vList,hList)

grBnLine *vList,*hList;

/*
Puts pseudo block for North-West Corner of bottleneck lines.
*/

{

    int x,y,xa,yb;
    grBnLine *vrun,*hrun;

    vrun=vList;
    hrun=hList;

    if ((vrun==NULL)||(hrun==NULL))
        return;

    while((vrun!=NULL)&&(vrun->type!=1))
    {
        if (vrun->cover !=1)
        {
            x=vrun->base;
            y=vrun->lb;
            hrun=hList;
            while((hrun!=NULL)&&(hrun->type !=0))
            {
                if (hrun->cover != 1)
                {
                    xa=hrun->rt;
                    yb=hrun->base;
                    if ((xa==x)&&(yb==y))
                    {
                        vrun->substitute=7;
                        hrun->substitute=2;
                        vrun->cover=1;
                        hrun->cover=1;
                        grSolidTileCount++;

                        Gr_CreatePseudoBlock(
                    grSolidTileCount,hrun->substitute,x,y);

                        goto lab1;
                    }
                }
                hrun=hrun->nextBnLine;
            }
lab1:;
        }
        vrun=vrun->nextBnLine;
    }
}

void Gr_SECorner(vList,hList)

grBnLine *vList,*hList;

/*
Puts pseudo block for South-East Corner of bottleneck lines.
*/

{

    int x,y,xa,yb;
    grBnLine *vrun,*hrun;

    vrun=vList;
    hrun=hList;

    if ((vrun==NULL)||(hrun==NULL))
        return;

    while((vrun!=NULL)&&(vrun->type!=1))
    {
        if (vrun->cover !=1)
        {
            x=vrun->base;
            y=vrun->rt;
            hrun=hList;
            while((hrun!=NULL)&&(hrun->type !=0))
            {
                if (hrun->cover != 1)
                {
                    xa=hrun->rt;
                    yb=hrun->base;
                    if ((xa==x)&&(yb==y))
                    {
                        vrun->substitute=6;
                        hrun->substitute=3;
                        vrun->cover=1;
                        hrun->cover=1;
                        grSolidTileCount++;

                        Gr_CreatePseudoBlock(
                    grSolidTileCount,hrun->substitute,x,y);


                        goto lab1;
                    }
                }
                hrun=hrun->nextBnLine;
            }
lab1:;
        }
        vrun=vrun->nextBnLine;
    }
}


void Gr_SWCorner(vList,hList)

grBnLine *vList,*hList;

/*
Puts pseudo block for South-West Corner of bottleneck lines.
*/

{

    int x,y,xa,yb;
    grBnLine *vrun,*hrun;

    vrun=vList;
    hrun=hList;

    if ((vrun==NULL)||(hrun==NULL))
        return;

    while((vrun!=NULL)&&(vrun->type!=1))
    {
        if (vrun->cover !=1)
        {
            x=vrun->base;
            y=vrun->rt;
            hrun=hList;
            while((hrun!=NULL)&&(hrun->type !=0))
            {
                if (hrun->cover != 1)
                {
                    xa=hrun->lb;
                    yb=hrun->base;
                    if ((xa==x)&&(yb==y))
                    {
                        vrun->substitute=8;
                        hrun->substitute=4;
                        vrun->cover=1;
                        hrun->cover=1;
                        grSolidTileCount++;

                        Gr_CreatePseudoBlock(
                    grSolidTileCount,hrun->substitute,x,y);


                        goto lab1;
                    }
                }
                hrun=hrun->nextBnLine;
            }
lab1:;
        }
        vrun=vrun->nextBnLine;
    }
}

void Gr_LeftHorz(hList,vList)

grBnLine *hList,*vList;

{

/*
    Inserts pseudo blocks for left-over bottleneck lines after corner
    elimination 
*/

    int x,y,xv,yv1,yv2;
    grBnLine *hrun,*vrun;

    if (hList == NULL)
        return;

    hrun=hList;

    if (vList==NULL)
    {
        while((hrun!=NULL)&&(hrun->type != 0))
        {
            if (hrun->cover != 1)
            {
                x=hrun->lb;
                y=hrun->base;
                if (hrun->bntype==TP_HORZ_B_RIGHT)
                {
                    hrun->substitute=1;
                }
                else
                {
                    hrun->substitute=2;
                }
                hrun->cover=1;
                grSolidTileCount ++;

                Gr_CreatePseudoBlock(
                grSolidTileCount,hrun->substitute,x,y);


            }
            hrun=hrun->nextBnLine;
        }
    }
    else
    {
        while((hrun!=NULL)&&(hrun->type!=0))
        {
            if (hrun->cover!=1)
            {
                x=hrun->lb;
                y=hrun->base;
                vrun=vList;
                while((vrun!=NULL)&&(vrun->type!=1))
                {
                    xv=vrun->base;
                    if (xv==x)
                    {
                        yv1=vrun->lb;
                        yv2=vrun->rt;
                        if ((yv1==y)||(yv2==y))
                        {
                            if (hrun->bntype==
                            TP_HORZ_B_LEFT)
                            {
                                hrun->
                                substitute=2;
                            }
                            else
                            {
                                hrun->
                                substitute=3;
                            }
                            x=hrun->rt;
                            goto lab1;
                        }
                    }
                    vrun=vrun->nextBnLine;
                }
                if (hrun->bntype == TP_HORZ_B_LEFT)
                {
                    hrun->substitute=4;
                }
                else
                {
                    hrun->substitute=1;
                }
lab1:;
                hrun->cover=1;
                grSolidTileCount++;
                Gr_CreatePseudoBlock(
                grSolidTileCount,hrun->substitute,x,y);
            }
            hrun=hrun->nextBnLine;
        }
    }

}



void Gr_LeftVert(hList,vList)

grBnLine *hList,*vList;

{

/*
    Inserts pseudo blocks for left-over bottleneck lines after corner
    elimination 
*/

    int x,y,xv1,yv,xv2;
    grBnLine *hrun,*vrun;

    if (vList == NULL)
        return;

    vrun=vList;

    if (hList==NULL)
    {
        while((vrun!=NULL)&&(vrun->type != 1))
        {
            if (vrun->cover != 1)
            {
                x=vrun->base;
                y=vrun->lb;
                if (vrun->bntype==TP_VERT_R_DOWN)
                {
                    vrun->substitute=5;
                }
                else
                {
                    vrun->substitute=7;/* tu zmiana */
                }
                vrun->cover=1;
                grSolidTileCount ++;

                Gr_CreatePseudoBlock(
                grSolidTileCount,vrun->substitute,x,y);


            }
            vrun=vrun->nextBnLine;
        }
    }
    else
    {
        while((vrun!=NULL)&&(vrun->type!=1))
        {
            if (vrun->cover!=1)
            {
                x=vrun->base;
                y=vrun->lb;
                hrun=hList;
                while((hrun!=NULL)&&(hrun->type!=0))
                {
                    yv=hrun->base;
                    if (yv==y)
                    {
                        xv1=hrun->lb;
                        xv2=hrun->rt;
                        if ((xv1==x)||(xv2==x))
                        {
                            if (vrun->bntype==
                            TP_VERT_R_DOWN)
                            {
                            vrun->substitute=6;
                            }
                            else
                            {
                            vrun->substitute=8;
                            }
                            y=vrun->rt;
                            goto lab1;
                        }
                    }
                    hrun=hrun->nextBnLine;
                }
                if (vrun->bntype == TP_VERT_R_DOWN)
                {
                    vrun->substitute=5;
                }
                else
                {
                    vrun->substitute=7;
                }
lab1:;
                vrun->cover=1;
                grSolidTileCount++;
                Gr_CreatePseudoBlock(
                grSolidTileCount,vrun->substitute,x,y);
            }
            vrun=vrun->nextBnLine;
        }
    }

}

void Gr_FillPsHBottlenecks()

{

    grBnLine *hList,*run;

    if (grBnLineList == NULL)
        return;

    hList=NULL;

    if (grBnLineList->type==1)
    {
        hList=grBnLineList;
    }
    else
    {
        run=grBnLineList;
        while(run!=NULL)
        {
            if (run->type !=grBnLineList->type)
            {
                hList=run;
                goto lab1;
            }
            run=run->nextBnLine;
        }

lab1:;
    }

    run=hList;
    while((run!=NULL)&&(run->type!=0))
    {
        grHorzBottlNumber++;
        grBottleneckTileCount++;

        Gr_CreatePsHBn(grBottleneckTileCount,run->substitute,run);
        run=run->nextBnLine;
    }
    
}



void Gr_FillPsVBottlenecks()

{

    grBnLine *vList,*run;

    if (grBnLineList == NULL)
        return;

    vList=NULL;

    if (grBnLineList->type==0)
    {
        vList=grBnLineList;
    }
    else
    {
        run=grBnLineList;
        while(run!=NULL)
        {
            if (run->type !=grBnLineList->type)
            {
                vList=run;
                goto lab1;
            }
            run=run->nextBnLine;
        }

lab1:;
    }
    run=vList;

    while((run!=NULL)&&(run->type!=1))
    {
        grVertBottlNumber++;
        grBottleneckTileCount++;

        Gr_CreatePsVBn(grBottleneckTileCount,run->substitute,run);
        run=run->nextBnLine;
    }
    
}


