



#include "grInt.h"

int Gr_IsPinVisible(pinPtr,side)

grPin *pinPtr;
int side;

/* 
	Returns 0 if pin is not visible inside a rectangle
	side tells from what direction we are looking at pin
        1 from east, 2 from south, 3 from west, 4 from north
*/

{

	if ((side == 1)&&(pinPtr->shade==WEST_PIN)) return (0);
	if ((side == 2)&&(pinPtr->shade==NORTH_PIN)) return (0);
	if ((side == 3)&&(pinPtr->shade==EAST_PIN)) return (0);
	if ((side == 4)&&(pinPtr->shade==SOUTH_PIN)) return (0);

	return (1);

}

void Gr_VertSplitPinsOfPinRectangle(xpos,pinRectanglePtr)

int xpos;
grPinRectangle *pinRectanglePtr;

/*
	Determines side (visibility) for boundary pins.
	Boundary is xpos. Vertical version.
*/

{

	grSortedPin *current;

	current=pinRectanglePtr->firstxPin;

	while(current!=NULL)
	{
		if ((current->pinPtr->x == xpos)&&(current->side == 0))
		{
			if (Gr_IsPinVisible(current->pinPtr,1)
			&&Gr_IsPinVisible(current->pinPtr,3))
			    { 
				current->side = 1; 
			}
			else
			{
				if (Gr_IsPinVisible(current->pinPtr,1))
				    {
				  /*just visible on one side*/
					current->side = 1;
				}
				else
					if (Gr_IsPinVisible(current->pinPtr,3))
				    {
					  /*justvisible on the left */
					current->side = 3 ; 
				}
			}
		}
		current = current->nextxPin;
	}
}

void Gr_HorzSplitPinsOfPinRectangle(ypos,pinRectanglePtr)

int ypos;
grPinRectangle *pinRectanglePtr;

/*
	Determines side (visibility) for boundary pins.
	Boundary is ypos. Horizontal version.
*/

{

	grSortedPin *current;

	current=pinRectanglePtr->firstyPin;

	while(current!=NULL)
	    {
		if ((current->pinPtr->y == ypos)&&(current->side ==0))
		    {
			if (Gr_IsPinVisible(current->pinPtr,2)
			&&Gr_IsPinVisible(current->pinPtr,4))
			    {
				current->side = 4;
			}
			else
			{
				if (Gr_IsPinVisible(current->pinPtr,2)) 
				/* visible only from south */
				    {
					current->side = 2; 
				}
				else
					if (Gr_IsPinVisible(current->pinPtr,4))
					 /* visible from north */
				    {
					current->side =4; 
				}
			}
		}
		current = current->nextyPin;
	}
}


int Gr_IsSortedPinOnXLeftSide(sortedPinPtr,xpos)

grSortedPin *sortedPinPtr;
int xpos;

{
	if (sortedPinPtr->pinPtr->x < xpos )  
	{
		return  (1);
	}
	if (sortedPinPtr->pinPtr->x > xpos )  
	{
		return  (0);
	}
	if (sortedPinPtr->side == 3 ) 
	{
		return  (1);
	}
	return  (0);
}


int Gr_IsSortedPinOnYRightSide(sortedPinPtr,ypos)

grSortedPin *sortedPinPtr;
int ypos;

{
	if (sortedPinPtr->pinPtr->y > ypos )  
	{
		return  (0);
	}
	if (sortedPinPtr->pinPtr->y < ypos )  
	{
		return  (1);
	}
	if (sortedPinPtr->side == 2 )  
	{
		return  (1);
	}
	return  (0);
}


void Gr_XHardSplitPinRectangle(xpos,pinRectanglePtr,leftPinRectanglePtr,
rightPinRectanglePtr)

int xpos;
grPinRectangle *pinRectanglePtr,*leftPinRectanglePtr,*rightPinRectanglePtr;

/*
	Cuts the pinRectangle into a left and right sides.      
	The original rectangle will be eventually freed.    
	Left and Right rectangles are from different lists.   
	The resulting left and right rectangles are not to be merged back      
	Sorted pins at the boundaries  change sides.                               
*/

{

	grSortedPin  *current,*lastleft,*lastright;

	Gr_VertSplitPinsOfPinRectangle(xpos,pinRectanglePtr); 
	/*pins prepared for split*/

	/* basic update  */

	leftPinRectanglePtr->name = pinRectanglePtr->name;
	leftPinRectanglePtr->count = 0;

	leftPinRectanglePtr->minx = 0;
	leftPinRectanglePtr->maxx = 0;
	leftPinRectanglePtr->miny = 0;
	leftPinRectanglePtr->maxy = 0;

	leftPinRectanglePtr->firstxPin = NULL;
	leftPinRectanglePtr->firstyPin = NULL;

	rightPinRectanglePtr->name = pinRectanglePtr->name;
	rightPinRectanglePtr->count = 0;

	rightPinRectanglePtr->minx = 0;
	rightPinRectanglePtr->maxx = 0;
	rightPinRectanglePtr->miny = 0;
	rightPinRectanglePtr->maxy = 0;

	rightPinRectanglePtr->firstxPin = NULL;
	rightPinRectanglePtr->firstyPin = NULL;

	/* x links update  */

	current = pinRectanglePtr->firstxPin;
	lastleft=NULL;
	lastright=NULL;



	while(current!=NULL)
	    {
		if (Gr_IsSortedPinOnXLeftSide(current,xpos))
		{
		if (lastleft == NULL)
			{
				leftPinRectanglePtr->firstxPin=current;
				leftPinRectanglePtr->minx=current->pinPtr->x;
			}
		else
			{
				lastleft->nextxPin=current;
			}
			lastleft=current;
			leftPinRectanglePtr->count++;
		}
		else
		{
		if (lastright == NULL)
			{
			rightPinRectanglePtr->firstxPin=current;
			rightPinRectanglePtr->minx=current->pinPtr->x;
			}
			else
			{
				lastright->nextxPin=current;
			}
			lastright=current;
			rightPinRectanglePtr->count++;
		}
			current=current->nextxPin;
		} /* end while */


		if (lastleft != NULL)
		{
			leftPinRectanglePtr->maxx = lastleft->pinPtr->x;
			lastleft->nextxPin=NULL;
		}
		if (lastright != NULL)
		{
			rightPinRectanglePtr->maxx = lastright->pinPtr->x;
			lastright->nextxPin=NULL;
		}
	/* end x links update*/

	/*update y links */
	current=pinRectanglePtr->firstyPin;


	lastleft=NULL;
	lastright=NULL;

	while(current!=NULL)
	    {
		if (Gr_IsSortedPinOnXLeftSide(current,xpos))
		    {
			if (lastleft==NULL)
			    {
				leftPinRectanglePtr->firstyPin=current;
				leftPinRectanglePtr->miny=current->pinPtr->y;
				leftPinRectanglePtr->maxy=current->pinPtr->y;
				lastleft=current;
			}
			else
			{
				lastleft->nextyPin=current;
				leftPinRectanglePtr->maxy = current->pinPtr->y;
				lastleft=current;
			}
		}
		else
		{
			if (lastright==NULL)
			    {
				rightPinRectanglePtr->firstyPin = current;
				rightPinRectanglePtr->miny = current->pinPtr->y;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright = current ;

			}
			else
			{
				lastright->nextyPin = current;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright = current;
			}
		}
                current=current->nextyPin;
	}

     
     /*terminate y - rectangles */

     if (lastright!=NULL) lastright->nextyPin=NULL;
     if (lastleft!=NULL) lastleft->nextyPin=NULL;

}


void Gr_YHardSplitPinRectangle(ypos,pinRectanglePtr,leftPinRectanglePtr,
rightPinRectanglePtr)

int ypos;
grPinRectangle *pinRectanglePtr,*leftPinRectanglePtr,*rightPinRectanglePtr;

/*
	Cuts the PinRectangle into left and right part  along ypos. 
	The original rectangle will be eventually freed.               
	Left and right pin rectangles are from different lists.        
	The resulting rectangles are not to be merged back.  
	Sorted pins at the boundaries are split                       
*/

{

	grSortedPin *current,*lastleft,*lastright;


	Gr_HorzSplitPinsOfPinRectangle(ypos,pinRectanglePtr); 
	/* pins prepared */

	/* basic update */
	leftPinRectanglePtr->name = pinRectanglePtr->name;
	leftPinRectanglePtr->count = 0;

	leftPinRectanglePtr->minx = 0;
	leftPinRectanglePtr->maxx = 0;
	leftPinRectanglePtr->miny = 0;
	leftPinRectanglePtr->maxy = 0;

	leftPinRectanglePtr->firstxPin = NULL;
	leftPinRectanglePtr->firstyPin = NULL;

	rightPinRectanglePtr->name = pinRectanglePtr->name;
	rightPinRectanglePtr->count = 0;

	rightPinRectanglePtr->minx = 0;
	rightPinRectanglePtr->maxx = 0;
	rightPinRectanglePtr->miny = 0;
	rightPinRectanglePtr->maxy = 0;

	rightPinRectanglePtr->firstxPin = NULL;
	rightPinRectanglePtr->firstyPin = NULL;


	/* x links update */
	current = pinRectanglePtr->firstxPin;


	lastleft=NULL;
	lastright=NULL;

	while(current!=NULL)
	    {

		if (Gr_IsSortedPinOnYRightSide(current,ypos))
		    {
			if (lastright==NULL)
			    {
				rightPinRectanglePtr->firstxPin=current;
				rightPinRectanglePtr->minx = current->pinPtr->x;
				rightPinRectanglePtr->maxx = current->pinPtr->x;
				lastright=current;
			}
			else
			{
				lastright->nextxPin = current;
				rightPinRectanglePtr->maxx = current->pinPtr->x;
				lastright = current;
			}
		}
		else
		{
			if (lastleft==NULL)
			    {
				leftPinRectanglePtr->firstxPin = current;

				leftPinRectanglePtr->minx =
					 current->pinPtr->x;
				leftPinRectanglePtr->maxx =
					 current->pinPtr->x;
				lastleft=current;
			}
			else
			{
				lastleft->nextxPin=current;
				leftPinRectanglePtr->maxx =
					 current->pinPtr->x;
				lastleft=current;
			}
		}
            current=current->nextxPin;
	}  /* end while, end x update */

            /* terminate x - rectangles */

            if (lastleft!=NULL) lastleft->nextxPin=NULL;
            if (lastright!=NULL) lastright->nextxPin=NULL;

	/* update y links */
	current=pinRectanglePtr->firstyPin;
	lastleft=NULL;
	lastright=NULL;

	while(current!=NULL)
	    {

		if (Gr_IsSortedPinOnYRightSide(current,ypos))
		    {
			if (lastright==NULL)
			    {
				rightPinRectanglePtr->firstyPin=current;
				rightPinRectanglePtr->miny = current->pinPtr->y;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright=current;
			}
			else
			{
				lastright->nextyPin = current;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright = current;
			}
			rightPinRectanglePtr->count++;
		}
		else
		{
			if (lastleft==NULL)
			    {
				leftPinRectanglePtr->firstyPin = current;

				leftPinRectanglePtr->miny =
					 current->pinPtr->y;
				leftPinRectanglePtr->maxy =
					 current->pinPtr->y;
				lastleft=current;
			}
			else
			{
				lastleft->nextyPin=current;
				leftPinRectanglePtr->maxy =
					 current->pinPtr->y;
				lastleft=current;
			}
			leftPinRectanglePtr->count++;
		}
            current=current->nextyPin;
	}  /* end while, end y update */

            /* terminate y - rectangles */

            if (lastleft!=NULL) lastleft->nextyPin=NULL;
            if (lastright!=NULL) lastright->nextyPin=NULL;

}


void Gr_XSoftSplitPinRectangle(xpos,pinRectanglePtr,leftPinRectanglePtr,
rightPinRectanglePtr)

int xpos;
grPinRectangle *pinRectanglePtr, *leftPinRectanglePtr, *rightPinRectanglePtr;

/*
	Cuts the pin rectangle into left and right side     
	The original rectangle is eventually freed      
	Left and right pin rectangles are from the same node list             
	The resulting rectangles can be merged back         
	Sorted pins at the boundary are not split and are included on the left
	hand side                                                            
*/

{

	grSortedPin  *current,*lastleft,*lastright;


	/* basic update  */

	leftPinRectanglePtr->name = pinRectanglePtr->name;
	leftPinRectanglePtr->count = 0;

	leftPinRectanglePtr->minx = 0;
	leftPinRectanglePtr->maxx = 0;
	leftPinRectanglePtr->miny = 0;
	leftPinRectanglePtr->maxy = 0;

	leftPinRectanglePtr->firstxPin = NULL;
	leftPinRectanglePtr->firstyPin = NULL;

	rightPinRectanglePtr->name = pinRectanglePtr->name;
	rightPinRectanglePtr->count = 0;

	rightPinRectanglePtr->minx = 0;
	rightPinRectanglePtr->maxx = 0;
	rightPinRectanglePtr->miny = 0;
	rightPinRectanglePtr->maxy = 0;

	rightPinRectanglePtr->firstxPin = NULL;
	rightPinRectanglePtr->firstyPin = NULL;

	/* x links update  */

	current = pinRectanglePtr->firstxPin;
	lastleft=NULL;
	lastright=NULL;


	while(current!=NULL)
	    {
		if (current->pinPtr->x <= xpos)
		{
		if (lastleft == NULL)
			{
				leftPinRectanglePtr->firstxPin=current;
				leftPinRectanglePtr->minx=current->pinPtr->x;
			}
		else
			{
				lastleft->nextxPin=current;
			}
			lastleft=current;
			leftPinRectanglePtr->count++;
		}
		else
		{
		if (lastright == NULL)
			{
			rightPinRectanglePtr->firstxPin=current;
			rightPinRectanglePtr->minx=current->pinPtr->x;
			}
			else
			{
				lastright->nextxPin=current;
			}
			lastright=current;
			rightPinRectanglePtr->count++;
		}
			current=current->nextxPin;
		} /* end while */


		if (lastleft != NULL)
		{
			leftPinRectanglePtr->maxx = lastleft->pinPtr->x;
			lastleft->nextxPin=NULL;
		}
		if (lastright != NULL)
		{
			rightPinRectanglePtr->maxx = lastright->pinPtr->x;
			lastright->nextxPin=NULL;
		}
	/* end x links update*/

	/*update y links */
	current=pinRectanglePtr->firstyPin;


	lastleft=NULL;
	lastright=NULL;

	while(current!=NULL)
	    {
		if (current->pinPtr->x <= xpos)
		    {
			if (lastleft==NULL)
			    {
				leftPinRectanglePtr->firstyPin=current;
				leftPinRectanglePtr->miny=current->pinPtr->y;
				leftPinRectanglePtr->maxy=current->pinPtr->y;
				lastleft=current;
			}
			else
			{
				lastleft->nextyPin=current;
				leftPinRectanglePtr->maxy = current->pinPtr->y;
				lastleft=current;
			}
		}
		else
		{
			if (lastright==NULL)
			    {
				rightPinRectanglePtr->firstyPin = current;
				rightPinRectanglePtr->miny = current->pinPtr->y;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright = current ;

			}
			else
			{
				lastright->nextyPin = current;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright = current;
			}
		}
                current=current->nextyPin;
	}

     
     /*terminate y - rectangles */

     if (lastright!=NULL) lastright->nextyPin=NULL;
     if (lastleft!=NULL) lastleft->nextyPin=NULL;

}



void Gr_YSoftSplitPinRectangle(ypos,pinRectanglePtr,leftPinRectanglePtr,
rightPinRectanglePtr)

int ypos;
grPinRectangle *pinRectanglePtr, *leftPinRectanglePtr, *rightPinRectanglePtr;


/*
	Cuts the pin rectangle into a left and right side.  
	The original rectangle is eventually freed.   
	Left and right pin rectangles are from the same node lists.            
	The resulting rectangles can be merged back.                           
	Sorted pins at the boundary are not split and are included on the  
	left hand side.           
*/

{

	grSortedPin  *current, *lastleft, *lastright;

	/* basic update  */
	/* basic update */
	leftPinRectanglePtr->name = pinRectanglePtr->name;
	leftPinRectanglePtr->count = 0;

	leftPinRectanglePtr->minx = 0;
	leftPinRectanglePtr->maxx = 0;
	leftPinRectanglePtr->miny = 0;
	leftPinRectanglePtr->maxy = 0;

	leftPinRectanglePtr->firstxPin = NULL;
	leftPinRectanglePtr->firstyPin = NULL;

	rightPinRectanglePtr->name = pinRectanglePtr->name;
	rightPinRectanglePtr->count = 0;

	rightPinRectanglePtr->minx = 0;
	rightPinRectanglePtr->maxx = 0;
	rightPinRectanglePtr->miny = 0;
	rightPinRectanglePtr->maxy = 0;

	rightPinRectanglePtr->firstxPin = NULL;
	rightPinRectanglePtr->firstyPin = NULL;


	/* x links update */
	current = pinRectanglePtr->firstxPin;


	lastleft=NULL;
	lastright=NULL;

	while(current!=NULL)
	    {

		if (current->pinPtr->y < ypos)
		    {
			if (lastright==NULL)
			    {
				rightPinRectanglePtr->firstxPin=current;
				rightPinRectanglePtr->minx = current->pinPtr->x;
				rightPinRectanglePtr->maxx = current->pinPtr->x;
				lastright=current;
			}
			else
			{
				lastright->nextxPin = current;
				rightPinRectanglePtr->maxx = current->pinPtr->x;
				lastright = current;
			}
		}
		else
		{
			if (lastleft==NULL)
			    {
				leftPinRectanglePtr->firstxPin = current;

				leftPinRectanglePtr->minx =
					 current->pinPtr->x;
				leftPinRectanglePtr->maxx =
					 current->pinPtr->x;
				lastleft=current;
			}
			else
			{
				lastleft->nextxPin=current;
				leftPinRectanglePtr->maxx =
					 current->pinPtr->x;
				lastleft=current;
			}
		}
            current=current->nextxPin;
	}  /* end while, end x update */

            /* terminate x - rectangles */

            if (lastleft!=NULL) lastleft->nextxPin=NULL;
            if (lastright!=NULL) lastright->nextxPin=NULL;

	/* update y links */
	current=pinRectanglePtr->firstyPin;
	lastleft=NULL;
	lastright=NULL;

	while(current!=NULL)
	    {

		if (current->pinPtr->x < ypos)
		    {
			if (lastright==NULL)
			    {
				rightPinRectanglePtr->firstyPin=current;
				rightPinRectanglePtr->miny = current->pinPtr->y;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright=current;
			}
			else
			{
				lastright->nextyPin = current;
				rightPinRectanglePtr->maxy = current->pinPtr->y;
				lastright = current;
			}
			rightPinRectanglePtr->count++;
		}
		else
		{
			if (lastleft==NULL)
			    {
				leftPinRectanglePtr->firstyPin = current;

				leftPinRectanglePtr->miny =
					 current->pinPtr->y;
				leftPinRectanglePtr->maxy =
					 current->pinPtr->y;
				lastleft=current;
			}
			else
			{
				lastleft->nextyPin=current;
				leftPinRectanglePtr->maxy =
					 current->pinPtr->y;
				lastleft=current;
			}
			leftPinRectanglePtr->count++;
		}
            current=current->nextyPin;
	}  /* end while, end y update */

            /* terminate y - rectangles */

            if (lastleft!=NULL) lastleft->nextyPin=NULL;
            if (lastright!=NULL) lastright->nextyPin=NULL;

}
