
/* mmsSuperNode.c --
 *
 *     Malgorzata Marek-Sadowska
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "grInt.h"


grPinRectangle *Gr_AllocateMemoryForNewPinRectangle()


/*
   Creates a new pinRectangle which is not attached to any list.
   Sets default values.
*/

{
grPinRectangle *pinRectanglePtr;



	MALLOC(grPinRectangle*,pinRectanglePtr,sizeof (grPinRectangle));

	pinRectanglePtr->firstxPin=NULL;
	pinRectanglePtr->firstyPin=NULL;
	pinRectanglePtr->nextOnList=NULL;
	pinRectanglePtr->nextInNet=NULL;


	pinRectanglePtr->minx=0;
	pinRectanglePtr->maxx=0;
	pinRectanglePtr->maxy=0;
	pinRectanglePtr->miny=0;
	pinRectanglePtr->count=0;
	pinRectanglePtr->cutMark=0;
	pinRectanglePtr->name=0;

        return (pinRectanglePtr);

}



void Gr_AttachPinRectangleToDecomposed(sNet,pRect)

grSubNet       *sNet;
grPinRectangle *pRect;

/* Newly created pinRectangle is added to the list of decomposed
   pinRectangles of sNet.
*/ 

{
	pRect->nextInNet = sNet->decomposed;
	sNet->decomposed=pRect;
}


grSuperNode  *Gr_AllocateMemoryForNewSuperNode(sNet,nodeListPtr)

/*
	For a given nodeListPtr and a given sub net
	it inserts a new node at the head of the list
	and returns the pointer to the head
*/

grSubNet *sNet;
grSuperNode *nodeListPtr;

{
	grSuperNode *new;

	MALLOC(grSuperNode*,new,sizeof (grSuperNode));

	new->nextOnList = nodeListPtr;
        new->contentsPtr=NULL;

        new->kind=0;
        new->mark=0;
        new->side=0;

        sNet->NodeList=new;

	return (new);
}

void Gr_CopyPinRectangle(oldPtr,newPtr)

grPinRectangle *oldPtr, *newPtr;

{
	newPtr->firstxPin =oldPtr->firstxPin;
	newPtr->firstyPin =oldPtr->firstyPin;
	newPtr->count= oldPtr->count;
	newPtr->name =oldPtr->name;
	newPtr->minx =oldPtr->minx;
	newPtr->maxx =oldPtr->maxx;
	newPtr->miny =oldPtr->miny;
	newPtr->maxy =oldPtr->maxy;
}


grSuperNode  *Gr_ProcessAPinRectangle(subNetPtr,currentPinRectangle,cutPos,dir,routekind,cutway,side)

grSubNet  *subNetPtr;
grPinRectangle  *currentPinRectangle;
int cutPos,dir,routekind,cutway;
int side;

/*
	The currentPinRectangle is cut along cutPos in direction 
	specified by dir
	Routekind =0 : fragment of prerouted net, =1 : net to be connected    
	cutway =0 : hard split, null returned                               
	cutway=1  : soft split, right or top new rectangle returned for further
	split                                                                 
	side =1 for soft split the parent node was on the left,=2 on the right.
	For hard split side is determined                                   
*/


{
	grPinRectangle  *leftPinRectangle, *rightPinRectangle;
	grSuperNode *leftNodePtr, *rightNodePtr;


	if (dir==0) return (NULL);
	if (currentPinRectangle == NULL) return (NULL);
	if ((routekind==0)&&(cutway==1)) return (NULL);

	if (currentPinRectangle->cutMark==1)  /*real intersection*/
	{
	leftPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
	Gr_AttachPinRectangleToDecomposed(subNetPtr,leftPinRectangle);
	rightPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
	Gr_AttachPinRectangleToDecomposed(subNetPtr,rightPinRectangle);


		/*vertical cut  */

		if (dir==1)
		{
			if (cutway==0)    
			/* real split by hierarchical cut line */
			{
				Gr_XHardSplitPinRectangle(
				cutPos,currentPinRectangle,leftPinRectangle,
				rightPinRectangle);
                                Gr_UpdateBoundingBox(leftPinRectangle);
                                Gr_UpdateBoundingBox(rightPinRectangle);
			}
			if (cutway==1)   
			/*imaginary split induced by a hole boundary*/
			{
				Gr_XSoftSplitPinRectangle(cutPos,
				currentPinRectangle,leftPinRectangle,
				rightPinRectangle);
                                Gr_UpdateBoundingBox(leftPinRectangle);
                                Gr_UpdateBoundingBox(rightPinRectangle);
			}
		}

		/*horizontal cut */

		if (dir==2)
		{
			if (cutway==0)  /*real split by hierarchical cut line*/
			{
				Gr_YHardSplitPinRectangle(cutPos,
				currentPinRectangle,leftPinRectangle,
				rightPinRectangle);
                                Gr_UpdateBoundingBox(leftPinRectangle);
                                Gr_UpdateBoundingBox(rightPinRectangle);
			}
			if (cutway==1)  /*imaginary split*/
			{
				Gr_YSoftSplitPinRectangle(cutPos,
				currentPinRectangle,leftPinRectangle,
				rightPinRectangle);
                                Gr_UpdateBoundingBox(leftPinRectangle);
                                Gr_UpdateBoundingBox(rightPinRectangle);
			}
		}

		leftNodePtr = Gr_AllocateMemoryForNewSuperNode(
		subNetPtr,subNetPtr->NodeList);
		subNetPtr->NodeList = leftNodePtr;
		leftNodePtr->contentsPtr = leftPinRectangle;
		leftNodePtr->kind = routekind;
		leftNodePtr->nodeName = currentPinRectangle->name;
		leftPinRectangle->cutMark =0;
		if (cutway==0)
		{
			leftNodePtr->side = 1;
		}
		else
		{
			leftNodePtr->side = side;
		}

		rightNodePtr = Gr_AllocateMemoryForNewSuperNode(
		subNetPtr,subNetPtr->NodeList);
		subNetPtr->NodeList = rightNodePtr;
		rightNodePtr->contentsPtr = rightPinRectangle;
		rightNodePtr->kind =routekind;
		rightNodePtr->nodeName = currentPinRectangle->name;
		if (cutway==0)
		{
			rightNodePtr->side = 2;
		}
		else
		{
			rightNodePtr->side = side;
		}
		rightPinRectangle->cutMark = 0;

		if (cutway==0)  return (NULL);
		if (cutway==1)  return (rightNodePtr);

	}  /* end real intersection for a hierarchical cut line */

	if ((currentPinRectangle->cutMark ==4)||
	(currentPinRectangle->cutMark ==2))
	{
		/*right list only*/
		rightPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
		Gr_AttachPinRectangleToDecomposed(subNetPtr,rightPinRectangle);
		
		Gr_CopyPinRectangle(currentPinRectangle,rightPinRectangle);
		rightNodePtr = Gr_AllocateMemoryForNewSuperNode(
		subNetPtr,subNetPtr->NodeList);
		subNetPtr->NodeList = rightNodePtr;
		rightNodePtr->contentsPtr = rightPinRectangle;
		rightNodePtr->kind = routekind;
		rightNodePtr->nodeName = currentPinRectangle->name;
		rightPinRectangle->cutMark =0;
		if (cutway==0)
		{
			rightNodePtr->side = 2;
		}
		else
		{
			rightNodePtr->side = side;
		}

		if (cutway==0)  return (NULL);
		if (cutway==1)  return (rightNodePtr);

	}  /* end right list only */

	if ((currentPinRectangle->cutMark ==5)||
	(currentPinRectangle->cutMark==3))
	{
		/* left list only */
		leftPinRectangle=Gr_AllocateMemoryForNewPinRectangle();
		Gr_AttachPinRectangleToDecomposed(subNetPtr,leftPinRectangle);
		Gr_CopyPinRectangle(currentPinRectangle,leftPinRectangle);
		leftNodePtr=
		Gr_AllocateMemoryForNewSuperNode(subNetPtr,subNetPtr->NodeList);
		subNetPtr->NodeList = leftNodePtr;
		leftNodePtr->contentsPtr = leftPinRectangle;
		leftNodePtr->kind = routekind;
		leftNodePtr->nodeName = currentPinRectangle->name;
		leftPinRectangle->cutMark =0;
		if (cutway == 0)
		{
			leftNodePtr->side =1;
		}
		else
		{
			leftNodePtr->side = side;
		}

		return (NULL);
	} /* end left side only */
        return (NULL);
}


void Gr_CreateInitialSuperNodes(subNetPtr,cutPos,dir)

grSubNet *subNetPtr;
int cutPos,dir;

/*
	For a subnet, cutposition and direction it creates 
	a list of initial super nodes.       

	REMARK : cut position must be in real dimensions, not virtual. 
	If dir=0 no cut; dir=1 vertical cut; dir=2 horizontal cut.

	Status of net activity is determined
*/

{
	grPinRectangle *currentPinRectangle;
	int cojest,side;

	/* for initial supernodes side is evaluated during processing*/

	side=0;
	cojest=0;
	currentPinRectangle=subNetPtr->toBeConnected;
	if (dir==1) 
	{
		Gr_MarkVertIntersectOfPinRectangle(currentPinRectangle,cutPos);
	}
	if (dir==2) 
	{
		Gr_MarkHorzIntersectOfPinRectangle(currentPinRectangle,cutPos);
	}

	if (currentPinRectangle!=NULL)
	{
		if (currentPinRectangle->cutMark ==1) cojest = 1;
		if (currentPinRectangle->cutMark ==4) cojest = 4;
		if (currentPinRectangle->cutMark ==5) cojest = 3;

		(void) Gr_ProcessAPinRectangle(subNetPtr,
		currentPinRectangle,cutPos,dir,1,0,side);
	}

	currentPinRectangle = subNetPtr->areConnected;
	while(currentPinRectangle != NULL)
	{
		if (dir == 1) 
		{
			Gr_MarkVertIntersectOfPinRectangle(
			currentPinRectangle,cutPos);
		}
		if (dir == 2) 
		{
			Gr_MarkHorzIntersectOfPinRectangle(
			currentPinRectangle,cutPos);
		}

		if (cojest != 1)
		{
			/*update activity */
			if (cojest ==0)
			{
				if (currentPinRectangle->cutMark ==1) cojest=2;
				if (currentPinRectangle->cutMark ==4) cojest=4;
				if (currentPinRectangle->cutMark ==5) cojest=3;
			}
			else
			{
				if ((cojest==3)&&(currentPinRectangle->cutMark
				==4))
				{
					cojest=1;
				}
				else
				{
					if ((cojest==4)&&(
					currentPinRectangle->cutMark==5))
					{
						cojest=1;
					}
				}
			}
		}

		(void) Gr_ProcessAPinRectangle(subNetPtr,currentPinRectangle,
			cutPos,dir,0,0,side);

		currentPinRectangle = currentPinRectangle->nextInNet;
	}
	subNetPtr->active = cojest;
}


int Gr_GetLengthOfCommonXInterval(box1Ptr,box2Ptr)

grPinRectangle *box1Ptr, *box2Ptr;

{
	int interval;

	interval=0;
	if ((box1Ptr->minx <= box2Ptr->minx)&&(box1Ptr->maxx>=box2Ptr->minx))
	{
		if (box1Ptr->maxx <= box2Ptr->maxx)
		{
			interval=box1Ptr->maxx - box2Ptr->minx;
		}
		else
		{
			interval=box2Ptr->maxx - box2Ptr->minx;
		}
	}
	if ((box1Ptr->minx > box2Ptr->minx)&&(box2Ptr->maxx > box1Ptr->minx))
	{
		if (box2Ptr->maxx <= box1Ptr->maxx)
		{
			interval=box2Ptr->maxx - box1Ptr->minx;
		}
		else
		{
			interval = box1Ptr->maxx - box1Ptr->minx;
		}
	}
	return (interval);
}


int Gr_GetLengthOfCommonYInterval(box1Ptr,box2Ptr)

grPinRectangle *box1Ptr, *box2Ptr;

{
	int interval;

	interval=0;
	if ((box1Ptr->maxy >= box2Ptr->maxy)&&(box1Ptr->miny <= box2Ptr->maxy))
	{
		if (box2Ptr->miny <= box1Ptr->miny)
		{
			interval=box2Ptr->maxy - box1Ptr->miny;
		}
		if (box2Ptr->miny > box1Ptr->miny)
		{
			interval= box2Ptr->maxy - box2Ptr->miny ;
		}
	}
	if ((box2Ptr->maxy >= box1Ptr->maxy)&&(box2Ptr->miny <= box1Ptr->maxy))
	{
		if (box1Ptr->miny <= box2Ptr->miny)
		{
			interval = box1Ptr->maxy - box2Ptr->miny;
		}
		if (box1Ptr->miny > box2Ptr->miny)
		{
			interval = box1Ptr->maxy - box1Ptr->miny;
		}
	}
	return (interval);
}


int Gr_GetDistanceBetweenBoxes(box1,box2)


grPinRectangle  *box1, *box2;

{
	int distance,xcom,ycom;

	distance = 0;
	xcom= Gr_GetLengthOfCommonXInterval(box1,box2);
	ycom= Gr_GetLengthOfCommonYInterval(box1,box2);


	if ((xcom==0)&&(ycom==0))
	{
		if (box2->miny > box1->maxy)
		{
			distance = distance+(box2->miny - box1->maxy);
		}
		if (box1->miny > box2->maxy)
		{
			distance = distance + (box1->miny - box2->maxy);
		}
		if (box2->minx > box1->maxx)
		{
			distance = distance + (box2->minx - box1->maxx);
		}
		if (box1->minx > box2->maxx)
		{
			distance = distance +(box1->minx - box2->maxx);
		}
	}
	if ((xcom >0)&&(ycom ==0))
	{
		if (box1->miny > box2->maxy )
		{
			distance = distance +(box1->miny - box2->maxy);
		}
		if (box2->miny > box1->maxy)
		{
			distance = distance + (box2->miny - box1->maxy) ;
		}
	}

	if ((xcom ==0)&&(ycom >0))
	{
		if (box2->minx > box1->maxx)
		{
			distance = distance+(box2->minx - box1->maxx);
		}
		if (box1->minx > box2->maxx)
		{
			distance = distance+(box1->minx - box2->maxx);
		}
	}
	return (distance);
}


