
/* gsCalcCapDenOfBnLine.c --
 *
 *     M.Kobayashi
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "gtGraph.h"
#include "fpg.h"         
#include "tileProp.h"
#include "mac.h"

/* ---------- Local Procedures ---------- */
static BearStatus UpdateDensity();

/* ---------- Local Global Variables ---------- */
static int minWidth;
static int minSpace;
static int holeSize;
static int minOver;



/* ----------------------------------------------------------------------------
 *
 * Gs_CalcCapDenOfBnLine() --
 *
 *  <description>
 *
 * Results:
 *  None.
 *
 * Side effects:
 *  None.
 *
 * ----------------------------------------------------------------------------
 */
BearStatus
Gs_CalcCapDenOfBnLine(bnLinePtr)
BNLine     *bnLinePtr;
{

    int         capacity;
    int         density;
    BNLineType  bnLineType;
    Line        *linePtr;
    Tile        *tlTilePtr;
    Tile        *brTilePtr;

#ifdef DEBUG
    if (bnLinePtr == NIL(BNLine))
    {
        Bear_PrintFatalErr("Gs_CalcCapDenOfBnLine() failed: no bottleneck line is given .");
    }
#endif DEBUG

    minWidth = Mac_GetMinWidthOfLay(METAL2_LAYER);
    minSpace = Mac_GetMinSpaceOfLayLay(METAL2_LAYER,METAL2_LAYER);
    holeSize = Mac_GetSizeOfHole(M1M2_VIA);
    minOver  = Mac_GetMinOverlapOfLayHole(METAL1_LAYER,M1M2_VIA);

    bnLineType = Tp_GetBnLineTypeOfBnLine(bnLinePtr);
    linePtr = Tp_GetLineOfBnLine(bnLinePtr);
    if (bnLineType == TP_VERT_R_UP || bnLineType == TP_VERT_R_DOWN)
    {
    capacity = ABS(linePtr->point1.p_y - linePtr->point2.p_y);
    }
    else if (bnLineType == TP_HORZ_B_RIGHT || bnLineType == TP_HORZ_B_LEFT)
    {
    capacity = ABS(linePtr->point1.p_x - linePtr->point2.p_x);
    }
    Tp_PutCapacityOfBnLine(capacity,bnLinePtr);

    tlTilePtr = Tp_GetTLTileOfBnLine(bnLinePtr);
    brTilePtr = Tp_GetBRTileOfBnLine(bnLinePtr);
    density = minSpace;
    Tp_ItrPPinsOfBnLine(tlTilePtr,brTilePtr,UpdateDensity,(ClientData)&density);
    if (density == minSpace)
    {
    Tp_PutDensityOfBnLine(0, bnLinePtr);
    }
    else
    {
    Tp_PutDensityOfBnLine(density, bnLinePtr);
    }

    return (BEAR_OK);
}



static BearStatus
UpdateDensity(pPinPtr, densityPtr)
PseudoPin   *pPinPtr;
int         *densityPtr;
{
    int    width;

    if ((width=Tp_GetWidthOfPPin(pPinPtr)) == MAXWIDTH)
    {
    width = minWidth;
    }
    *densityPtr += (MAX(width, holeSize+2*minOver) + minSpace);

    return (BEAR_CONTINUE_SEARCH);
}
