
/* gsCalcCapDenOfBnTile.c --
 *
 *     M. Kobayashi
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "gtGraph.h"
#include "fpg.h"         
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"


#define  PINARRAYSIZE 10000
#define  START   0  /* start pininal of net */
#define  END     1  /* end pininal of net */

static BearStatus PutPPin();
static BearStatus PutPin();
static BearStatus PutRPin();
static int        CompNetId();
static int        CompPosition();

typedef struct _PinInfo
{
    BearId    netId;
    int       position;
    int       attr;
    float     width;
} PinInfo;

static PinInfo     pin[PINARRAYSIZE];
static int         maxPin;
static Transform   *transform;
static int         minWidth;
static int         minSpace;
static int         holeSize;
static int         minOver;



/* ----------------------------------------------------------------------------
 *
 * Gs_CalcCapDenOfBnTile() --
 *
 * Results:
 *  None.
 *
 * Side effects:
 *  None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gs_CalcCapDenOfBnTile(parentCell, bnTilePtr)
CellMaster *parentCell;
Tile       *bnTilePtr;
{
    float       curWidth;
    float       maxWidth;
    Tile        *curTilePtr;
    Line        interval;
    BearStatus  PutPin();
    BearStatus  PutPPin();
    BearStatus  PutRPin();
    int         CompNetId();
    int         CompPosition();
    int         CalcDensity();
    int         tileCapacity;
    Rect        tileRect;
    int         density;
    int         i, j;

#ifdef DEBUG
    if (Tp_IsBottleneckTile(bnTilePtr) == FALSE)
    {
        Bear_PrintFatalErr("Gs_CalcCapDenOfBntile() failed: Given tile is not a bottleneck tile.");
    }
#endif DEBUG

    /*
     * If the density is minus, it means the real density.
     * We don't need to estimate it.
     */
    density = Tp_GetDensityOfTile(bnTilePtr);
    if (density < 0)
    {
        Tp_PutDensityOfTile(-density, bnTilePtr);
        return (BEAR_OK);
    }

    minWidth = Mac_GetMinWidthOfLay(METAL2_LAYER);
    minSpace = Mac_GetMinSpaceOfLayLay(METAL2_LAYER,METAL2_LAYER);
    holeSize = Mac_GetSizeOfHole(M1M2_VIA);
    minOver  = Mac_GetMinOverlapOfLayHole(METAL2_LAYER,M1M2_VIA);

    maxPin = 0;

    if (Tp_IsHorzTile(bnTilePtr))
    {
        transform = &GeoIdentityTransform;
        Tp_HorzTileToRect(Mac_GetCellPlaneOfCell(parentCell),
                  bnTilePtr,&tileRect);
        tileCapacity = tileRect.r_xtop - tileRect.r_xbot;
    }
    else
    {
        transform = &Geo90Transform;
        Tp_VertTileToRect(Mac_GetCellPlaneOfCell(parentCell),
                  bnTilePtr,&tileRect);
        tileCapacity = tileRect.r_ytop - tileRect.r_ybot;
    }
    Tp_PutCapacityOfTile(tileCapacity,bnTilePtr);

    Tp_ItrPPinsOfSide(bnTilePtr,
              BOTTOM_LEFT_SIDE,
              PutPPin,
              (ClientData)&BOTTOM(bnTilePtr));
    Tp_ItrPPinsOfSide(bnTilePtr,
              TOP_RIGHT_SIDE,
              PutPPin,
              (ClientData)&TOP(bnTilePtr));

    interval.point1.p_x = LEFT(bnTilePtr);
    interval.point1.p_y = BOTTOM(bnTilePtr);
    interval.point2.p_x = LEFT(bnTilePtr);
    interval.point2.p_y = TOP(bnTilePtr);
    curTilePtr = BL(bnTilePtr);
    for (;;)
    {
        if (Tp_IsSolidTile(curTilePtr))
        {
            if (Tp_IsCellInstTile(curTilePtr))
            {
                Mac_ItrPinOfCellInst(Tp_GetCellInstOfTile(curTilePtr),
                         PutPin,
                         (ClientData)&interval);
            }
            else if (Tp_IsRCellTile(curTilePtr))
            {
                Mac_ItrSidePinOfRCell(Tp_GetRCellOfTile(curTilePtr),
                        PutRPin,
                        (ClientData)&interval);
            }
        }
        if (TOP(curTilePtr) >= TOP(bnTilePtr))
        {
            break;
        }
        curTilePtr = RT(curTilePtr);
    }
    
    interval.point1.p_x = RIGHT(bnTilePtr);
    interval.point1.p_y = BOTTOM(bnTilePtr);
    interval.point2.p_x = RIGHT(bnTilePtr);
    interval.point2.p_y = TOP(bnTilePtr);
    curTilePtr = TR(bnTilePtr);
    for (;;)
    {
        if (Tp_IsSolidTile(curTilePtr))
        {
            if (Tp_IsCellInstTile(curTilePtr))
            {
                Mac_ItrPinOfCellInst(Tp_GetCellInstOfTile(curTilePtr),
                        PutPin,
                        (ClientData)&interval);
            }
            else if (Tp_IsRCellTile(curTilePtr))
            {
                 Mac_ItrSidePinOfRCell(Tp_GetRCellOfTile(curTilePtr),
                        PutRPin,
                        (ClientData)&interval);
            }
        }
        if (BOTTOM(curTilePtr) <= BOTTOM(bnTilePtr))
        {
            break;
        }
        curTilePtr = LB(curTilePtr);
    }
    Mac_ItrPinOfCellInst(Mac_GetInstOfCoreMaster(parentCell), 
            PutPin,
            (ClientData)bnTilePtr);

    if (maxPin <= 0)
    {
        Tp_PutDensityOfTile(minSpace, bnTilePtr);
        return (BEAR_OK);
    }

    qsort(pin, maxPin, sizeof (PinInfo), CompNetId);

    for (i = 0 , j = 0 ; j < maxPin ; )
    {
        pin[i].netId    = pin[j].netId;
        pin[i].position = pin[j].position;
        pin[i].width    = pin[j].width;
        pin[i].attr     = START;

        pin[++i].netId  = pin[++j].netId;
        pin[i].position = pin[j].position;
        pin[i].width    = pin[j].width;
        pin[i].attr     = END;

        for (j++ ; (j < maxPin) && (pin[i].netId == pin[j].netId) ; j++)
        {
            pin[i].position = pin[j].position;
            pin[i].width    = pin[j].width;
        }
        ++i;
    }
    maxPin = i;

    qsort(pin, maxPin, sizeof (PinInfo), CompPosition);

    maxWidth = minSpace;
    curWidth = minSpace;
    for ( i = 0 ; i < maxPin ; i++)
    {
        if (pin[i].attr == START)
        {
            curWidth += (MAX(pin[i].width, holeSize+2*minOver) + minSpace);
            maxWidth = MAX(maxWidth, curWidth);
        }
        else
        {
            curWidth -= (MAX(pin[i].width, holeSize+2*minOver) + minSpace);
        }
    }
    Tp_PutDensityOfTile((int)maxWidth, bnTilePtr);

    return (BEAR_OK);
}



static BearStatus
PutPPin(pPinPtr, positionPtr)
PseudoPin   *pPinPtr;
int         *positionPtr;
{
    pin[maxPin].netId    = Tp_GetIntNetIdOfPPin(pPinPtr);
    pin[maxPin].position = *positionPtr;
    if ((pin[maxPin].width=Tp_GetWidthOfPPin(pPinPtr)) == MAXWIDTH)
    {
    pin[maxPin].width = minWidth;
    }
    ++maxPin;
    if (maxPin >= PINARRAYSIZE)
    {
    Bear_PrintFatalErr("Gs_CalcCapDenOfBnTile() failed: maxPin >= PINARRAYSIZE");
    }
    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
PutPin(pinPtr, interval)
PinInst     *pinPtr;
Line        *interval;
{
    int    netId;
    Point  absPos1;
    Point  absPos2;
    float  *widthVect;

    if (viewTypeFlag == H_DONE)
    {
        netId = Mac_GetHExtNetIdOfPin(pinPtr);
    }
    else if (viewTypeFlag == V_DONE)
    {
        netId = Mac_GetVExtNetIdOfPin(pinPtr);
    }
    else
    {
        netId = Mac_GetExtNetIdOfPin(pinPtr);
    }
    if (netId == 0)
    {
        return (BEAR_CONTINUE_SEARCH);
    }
    Mac_GetAbsPosOfPin(pinPtr, &absPos1);
    GeoTransPoint(transform, &absPos1, &absPos2);
    if ((absPos2.p_x >= interval->point1.p_x) &&
        (absPos2.p_x <= interval->point2.p_x) &&
        (absPos2.p_y >= interval->point1.p_y) &&
        (absPos2.p_y <= interval->point2.p_y))
    {
        pin[maxPin].netId = netId;
        pin[maxPin].position = absPos2.p_y;
        widthVect = Mac_GetWidthVectOfPin(pinPtr);
        if ((pin[maxPin].width=widthVect[METAL1_LAYER]) == MAXWIDTH)
        {
            pin[maxPin].width = minWidth;
        }
        ++maxPin;
        if (maxPin >= PINARRAYSIZE)
        {
            Bear_PrintFatalErr("Gs_CalcCapDenOfBnTile() failed: maxPin >= PINARRAYSIZE");
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
PutRPin(rPinPtr, interval)
RoutePin    *rPinPtr;
Line        *interval;
{
    int    netId;
    Point  absPos1;
    Point  absPos2;
    float  *widthVect;

    if (viewTypeFlag == H_DONE)
    {
    netId = Mac_GetHExtNetIdOfRPin(rPinPtr);
    }
    else if (viewTypeFlag == V_DONE)
    {
    netId = Mac_GetVExtNetIdOfRPin(rPinPtr);
    }
    else
    {
    netId = Mac_GetExtNetIdOfRPin(rPinPtr);
    }
    if (netId == 0)
    {
    return (BEAR_CONTINUE_SEARCH);
    }
    Mac_GetAbsPosOfRPin(rPinPtr, &absPos1);
    GeoTransPoint(transform, &absPos1, &absPos2);
    if ((absPos2.p_x >= interval->point1.p_x) &&
        (absPos2.p_x <= interval->point2.p_x) &&
        (absPos2.p_y >= interval->point1.p_y) &&
        (absPos2.p_y <= interval->point2.p_y))
    {
    pin[maxPin].netId = netId;
    pin[maxPin].position = absPos2.p_y; 
    widthVect = Mac_GetWidthVectOfRPin(rPinPtr);
    if ((pin[maxPin].width=widthVect[METAL1_LAYER]) == MAXWIDTH)
    {
        pin[maxPin].width = minWidth;
    }
    ++maxPin;
    if (maxPin >= PINARRAYSIZE)
    {
        Bear_PrintFatalErr("Gs_CalcCapDenOfBnTile() failed: maxPin >= PINARRAYSIZE");
    }
    }
    return (BEAR_CONTINUE_SEARCH);
}



static int
CompNetId(pin1,pin2)
PinInfo *pin1,*pin2;
{
    if (pin1->netId < pin2->netId)
    {
        return (-1);
    }
    else if (pin1->netId > pin2->netId)
    {
        return (1);
    }

    if (pin1->position < pin2->position)
    {
    return (-1);
    }
    else if (pin1->position > pin2->position)
    {
    return (1);
    }

    return (0);
}



static int
CompPosition(pin1,pin2)
PinInfo *pin1,*pin2;
{

    if (pin1->position < pin2->position)
    {
    return (-1);
    }
    else if (pin1->position > pin2->position)
    {
    return (1);
    }

    if (pin1->attr < pin2->attr)
    {
        return (-1);
    }
    else if (pin1->attr > pin2->attr)
    {
        return (1);
    }

    if (pin1->netId < pin2->netId)
    {
        return (-1);
    }
    else if (pin1->netId > pin2->netId)
    {
        return (1);
    }

    return (0);
}
