
/* gsCalcCapOfTilePlane.c --
 *
 *  M. Kobayashi
 *  (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "gtGraph.h"
#include "fpg.h"
#include "tileProp.h"
#include "mac.h"

static Transform *transform;

static int GsItrTiles();
static BearStatus GsItrBnLines();
BearStatus Gs_CalcCapOfBnTile();
BearStatus Gs_CalcCapOfBnLine();



/* ----------------------------------------------------------------------------
 *
 * Gs_CalcCapOfTilePlnae() --
 *
 *  <description>
 *
 * Results:
 *  None.
 *
 * Side effects:
 *  None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gs_CalcCapOfTilePlane(parentCellPtr)
CellMaster *parentCellPtr;
{

    PlaneData    *planeDataPtr;

    planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);

    TiSrArea((Tile *)NULL,
         Tp_GetHorzPlane(planeDataPtr),
         Tp_GetHorzPlaneBnd(planeDataPtr),
             GsItrTiles,
             (ClientData)parentCellPtr);
    TiSrArea((Tile *)NULL,
         Tp_GetVertPlane(planeDataPtr),
         Tp_GetVertPlaneBnd(planeDataPtr),
             GsItrTiles,
             (ClientData)parentCellPtr);

    Tp_ItrHorzBnLine(planeDataPtr,GsItrBnLines,(ClientData)NULL);
    Tp_ItrVertBnLine(planeDataPtr,GsItrBnLines,(ClientData)NULL);

    return (BEAR_OK);
}



static int
GsItrTiles(tilePtr,parentCellPtr)
Tile       *tilePtr;
CellMaster *parentCellPtr;
{

    if (Tp_IsBottleneckTile(tilePtr))
    {
    Gs_CalcCapOfBnTile(parentCellPtr, tilePtr);
    }
    return (0);
}



static BearStatus
GsItrBnLines(bnLinePtr,clientData)
BNLine     *bnLinePtr;
ClientData clientData;
{

    Gs_CalcCapOfBnLine(bnLinePtr);
    return (BEAR_CONTINUE_SEARCH);
}


BearStatus
Gs_CalcCapOfBnTile(parentCell, bnTilePtr)
CellMaster *parentCell;
Tile       *bnTilePtr;
{

    int         tileCapacity;
    Rect        tileRect;

#ifdef DEBUG
    if (Tp_IsBottleneckTile(bnTilePtr) == FALSE)
    {
        Bear_PrintFatalErr("Gs_CalcCapDenOfBntile() failed: Given tile is not a bottleneck tile.");
    }
#endif DEBUG
    if (Tp_IsHorzTile(bnTilePtr))
    {
    transform = &GeoIdentityTransform;
    Tp_HorzTileToRect(Mac_GetCellPlaneOfCell(parentCell),
              bnTilePtr,&tileRect);
    tileCapacity = tileRect.r_xtop - tileRect.r_xbot;
    }
    else
    {
    transform = &Geo90Transform;
    Tp_VertTileToRect(Mac_GetCellPlaneOfCell(parentCell),
              bnTilePtr,&tileRect);
    tileCapacity = tileRect.r_ytop - tileRect.r_ybot;
    }

    Tp_PutCapacityOfTile(tileCapacity,bnTilePtr);
   return (BEAR_OK);
}



BearStatus
Gs_CalcCapOfBnLine(bnLinePtr)
BNLine     *bnLinePtr;
{

    int         capacity;
    BNLineType  bnLineType;
    Line        *linePtr;

#ifdef DEBUG
    if (bnLinePtr == NIL(BNLine))
    {
        Bear_PrintFatalErr("Gs_CalcCapDenOfBnLine() failed: no bottleneck line is given .");
    }
#endif DEBUG

    bnLineType = Tp_GetBnLineTypeOfBnLine(bnLinePtr);
    linePtr = Tp_GetLineOfBnLine(bnLinePtr);
    if (bnLineType == TP_VERT_R_UP || bnLineType == TP_VERT_R_DOWN)
    {
    capacity = ABS(linePtr->point1.p_y - linePtr->point2.p_y);
    }
    else if (bnLineType == TP_HORZ_B_RIGHT || bnLineType == TP_HORZ_B_LEFT)
    {
    capacity = ABS(linePtr->point1.p_x - linePtr->point2.p_x);
    }
    Tp_PutCapacityOfBnLine(capacity,bnLinePtr);

    return (BEAR_OK);
}

