
/* gsCompact.c --
 *
 *     Wei-Ming Dai
 *     (Advising Professor - E. S. Kuh)
 *
 * This module performs global compaction on a given placement.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: gsSpacer.c,v 1.2 86/10/04 16:48:59 weiming Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "gtGraph.h"
#include "fpg.h"         /*temp*/
#include "tileProp.h"
#include "mac.h"
#include "globalSpacer.h"
#include "globalSpacerInt.h"

#ifndef BEAR_BLACK_BOX
#include "ezPlot.h"
#include "../polarBear/polarBear.h"

extern WindowData   *globalWinDataPtr;
#endif BEAR_BLACK_BOX


/* ---------- Local Procedures ---------- */
static int          UpdateHorzCompactRidge(), UpdateVertCompactRidge();
static int          FindCompactRidge();


/* ---------- Local Global Variables ---------- */
static Plane        *planePtr;
static PlaneData    *globalPlaneDataPtr;
static Rect         planeSearchRect;
static Rect         tailRect;
static Rect         updateTileRect;
static Tile         *leftmostTilePtr;
static Tile         *topmostTilePtr;


/*
 * ----------------------------------------------------------------------------
 *
 * UpdateHorzCompactRidge() --
 *
 *     Update horizontal compaction ridge. 
 *
 * Results:
 *    Return MAGIC_CONTINUE_SEARCH to keep TiSrArea() from 
 *    aborting the search.
 *    Update the best compaction ridge found so far.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static int
UpdateHorzCompactRidge(tilePtr, ridgeInfoPtr)

register Tile      *tilePtr;
register RidgeInfo *ridgeInfoPtr;
{
    int  curWidth;
    Rect tileRect;

    TiToRect (tilePtr, &tileRect);
    GeoClip (&tileRect, &planeSearchRect);

    if ((Tp_IsSpaceTile (tilePtr)== TRUE) 
     && (GEO_TOUCH (&tileRect, &updateTileRect) == TRUE))
    {
    curWidth = Tp_GetRidgeWidthOfTile (tilePtr);
    if (curWidth >= 0 && curWidth > ridgeInfoPtr->ridgeWidth)
    {
        ridgeInfoPtr->ridgeWidth = curWidth;
        ridgeInfoPtr->prevTilePtr = tilePtr;

/*
printf ("new ridge width = %d\n", ridgeInfoPtr->ridgeWidth);
fflush (stdout);
*/
    }
    if (ridgeInfoPtr->prevTilePtr == NIL(Tile))
    {
        if (Tp_GetPrevTileOfTile(tilePtr) != NIL(Tile) ||
        tilePtr == topmostTilePtr)
        {
        ridgeInfoPtr->ridgeWidth = curWidth;
        ridgeInfoPtr->prevTilePtr = tilePtr;
        }
    }
    }

    return (MAGIC_CONTINUE_SEARCH); 
}



/*
 * ----------------------------------------------------------------------------
 *
 * UpdateVertCompactRidge() --
 *
 *     Update vertical compaction ridge. 
 *
 * Results:
 *    Return MAGIC_CONTINUE_SEARCH to keep TiSrArea() from 
 *    aborting the search.
 *    Update the best compaction ridge found so far.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static int
UpdateVertCompactRidge(tilePtr, ridgeInfoPtr)

register Tile      *tilePtr;
register RidgeInfo *ridgeInfoPtr;
{
    int  curWidth;
    Rect tileRect;

    TiToRect (tilePtr, &tileRect);
    GeoClip (&tileRect, &planeSearchRect);

    if ((Tp_IsSpaceTile (tilePtr)== TRUE) 
     && (GEO_TOUCH (&tileRect, &updateTileRect) == TRUE))
    {
    curWidth = Tp_GetRidgeWidthOfTile (tilePtr);
    if (curWidth >= 0 && curWidth > ridgeInfoPtr->ridgeWidth)
    {
        ridgeInfoPtr->ridgeWidth = curWidth;
        ridgeInfoPtr->prevTilePtr = tilePtr;

/*
printf ("new ridge width = %d\n", ridgeInfoPtr->ridgeWidth);
fflush (stdout);
*/
    }
    if (ridgeInfoPtr->prevTilePtr == NIL(Tile))
    {
        if (Tp_GetPrevTileOfTile(tilePtr) != NIL(Tile) ||
        tilePtr == leftmostTilePtr)
        {
        ridgeInfoPtr->ridgeWidth = curWidth;
        ridgeInfoPtr->prevTilePtr = tilePtr;
        }
    }
    }

    return (MAGIC_CONTINUE_SEARCH); 
}



/* ----------------------------------------------------------------------------
 *
 * FindCompactRidge() --
 *
 *     Find the last tile on the compaction ridge 
 *
 * Results:
 *     Return MAGIC_CONTINUE_SEARCH to keep TiSrArea() from 
 *     aborting the search.
 *     Return the ridge width and the last tile of the ridge.
 *
 * Side effects:
 *     None.
 *
 * ----------------------------------------------------------------------------
 */

static int
FindCompactRidge(tilePtr, ridgeInfoPtr)

register Tile      *tilePtr;
register RidgeInfo *ridgeInfoPtr;
{
    int  curWidth;
    Rect tileRect;

/*
printf ("Enter FindCompactRidge - \n"); 
fflush (stdout);
*/

    TiToRect (tilePtr, &tileRect);
    GeoClip (&tileRect, &planeSearchRect);

    if (Tp_IsSpaceTile (tilePtr) && GEO_TOUCH (&tileRect, &planeSearchRect))
    {
    curWidth = Tp_GetRidgeWidthOfTile (tilePtr);
    if (curWidth < INFINITY && curWidth > ridgeInfoPtr->ridgeWidth)
    {
        ridgeInfoPtr->ridgeWidth = curWidth;
        ridgeInfoPtr->prevTilePtr = tilePtr;

/*
printf ("new ridge width = %d\n", ridgeInfoPtr->ridgeWidth);
fflush (stdout);
*/

    }
    }
    return (MAGIC_CONTINUE_SEARCH); 
}



/*
 * ----------------------------------------------------------------------------
 *
 * SearchHorzCompactRidge --
 *
 *    Search for a horizontal compaction ridge using dynamic 
 *    programming. 
 *
 * Results:
 *    Return BEAR_CONTINUE_SEARCH to keep TiSrArea() from 
 *    aborting the search.
 *
 * Side effects:
 *    Record the compaction ridge width so far and 
 *    correponding back tracing pointer of the tile.
 *
 * ----------------------------------------------------------------------------
 */

/* ARGSUSED */
static BearStatus
SearchHorzCompactRidge (horzTilePtr, clientData)

register Tile *horzTilePtr;
ClientData    clientData;
{
    RidgeInfo ridgeInfo;
    int       ridgeWidth;
    Rect      searchRect;

/*
printf ("Enter SearchHorzCompactRidge - \n"); 
fflush (stdout);
*/

    if (Tp_IsSpaceTile (horzTilePtr)) 
    {

    ridgeWidth = Tp_GetCapacityOfTile (horzTilePtr) 
             - Tp_GetDensityOfTile (horzTilePtr);
    Tp_PutRidgeWidthOfTile (ridgeWidth, horzTilePtr);

    if (ridgeWidth > 0)
    {
        TITORECT (horzTilePtr, &updateTileRect);
        GeoClip (&updateTileRect, &planeSearchRect);

/*
Mac_PrintRect (&updateTileRect);
*/

        Mac_CopyRect(&searchRect, &updateTileRect);
        searchRect.r_ybot = searchRect.r_ytop;
        searchRect.r_ytop += 1;

/*
Mac_PrintRect (&searchRect);
*/


        if (GEO_SURROUND(&planeSearchRect, &searchRect) == TRUE)
        {
            ridgeInfo.ridgeWidth = 0;
            ridgeInfo.prevTilePtr = NIL (Tile);
            (void) TiSrArea((Tile *) NULL, 
                    planePtr,
                    &searchRect,
                    UpdateHorzCompactRidge, 
                    (ClientData) &ridgeInfo);

            ridgeWidth = MIN(Tp_GetRidgeWidthOfTile (horzTilePtr), 
                     ridgeInfo.ridgeWidth);

/*
printf ("ridgeWidth = %d\n", ridgeWidth); 
fflush (stdout);
*/

            Tp_PutRidgeWidthOfTile (ridgeWidth, horzTilePtr);
            Tp_PutPrevTileOfTile (ridgeInfo.prevTilePtr, horzTilePtr);
        }
    }
    }
    return (BEAR_CONTINUE_SEARCH); 
}


/*
 * ----------------------------------------------------------------------------
 *
 * SearchVertCompactRidge --
 *
 *    Search for a vertical compaction ridge using dynamic 
 *    programming. 
 *
 * Results:
 *    Return BEAR_CONTINUE_SEARCH to keep TiSrArea() from 
 *    aborting the search.
 *
 * Side effects:
 *    Record the compaction ridge width so far and 
 *    correponding back tracing pointer of the tile.
 *
 * ----------------------------------------------------------------------------
 */

/* ARGSUSED */
static BearStatus 
SearchVertCompactRidge (vertTilePtr, clientData)

register Tile *vertTilePtr;
ClientData    clientData;
{
    RidgeInfo ridgeInfo;
    int       ridgeWidth;
    Rect      searchRect;

/*
printf ("Enter SearchVertCompactRidge - \n"); 
fflush (stdout);
*/

    if (Tp_IsSpaceTile (vertTilePtr)) 
    {

/*
printf ("Is space tile \n"); 
fflush (stdout);
*/

    ridgeWidth = Tp_GetCapacityOfTile (vertTilePtr) 
                     - Tp_GetDensityOfTile (vertTilePtr);
    Tp_PutRidgeWidthOfTile (ridgeWidth, vertTilePtr);

if (ridgeWidth > 0)
{
    TiToRect (vertTilePtr, &updateTileRect);
    GeoClip (&updateTileRect, &planeSearchRect);


/*
Mac_PrintRect (&updateTileRect);
*/

        Mac_CopyRect(&searchRect, &updateTileRect);
        searchRect.r_ybot = searchRect.r_ytop;
        searchRect.r_ytop += 1;

/*
Mac_PrintRect (&searchRect);
*/


        if (GEO_SURROUND(&planeSearchRect, &searchRect) == TRUE)
        {
        ridgeInfo.ridgeWidth = 0;
        ridgeInfo.prevTilePtr = NIL (Tile);
        (void) TiSrArea((Tile *) NULL, 
                planePtr,
                &searchRect,
                UpdateVertCompactRidge, 
                (ClientData) &ridgeInfo);

        ridgeWidth = MIN(Tp_GetRidgeWidthOfTile (vertTilePtr), ridgeInfo.ridgeWidth);

/*
printf ("ridgeWidth = %d\n", ridgeWidth); 
fflush (stdout);
*/

        Tp_PutRidgeWidthOfTile (ridgeWidth, vertTilePtr);
        Tp_PutPrevTileOfTile (ridgeInfo.prevTilePtr, vertTilePtr);
        }
    }
    }
    return (BEAR_CONTINUE_SEARCH); 
}



/* ----------------------------------------------------------------------------
 *
 * Gs_HorzCompact --
 *
 *     The main routine to perform horizontal compaction,
 *
 * Results:
 *     None.
 *
 * Side effects:
 *     Whatever the side effects resulted from the compaction.
 *
 * ----------------------------------------------------------------------------
 */

SpacingStatus
Gs_HorzCompact(planeDataPtr)
PlaneData *planeDataPtr;
{
#ifndef BEAR_BLACK_BOX
    ChipWin     *chipWinPtr;
    DrawRidge   drawRidge;
#endif BEAR_BLACK_BOX

    RidgeInfo   ridgeInfo;
    LinkedTile  *ridgeHead;
    Rect        *horzPlaneBndPtr;
    Rect        bbox;
    Point       topPoint;

/*
printf ("Enter Gs_HorzCompact - \n"); 
fflush (stdout);
*/

#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && gs_DrawRidge) {
        chipWinPtr = PbGetChipWin(globalWinDataPtr);

        drawRidge.displayFunc = GXcopy;
        drawRidge.planeDataPtr = planeDataPtr;
        drawRidge.ridgeColor = ez_HighLightColor;
    }
#endif BEAR_BLACK_BOX

    gs_Quit = FALSE;

    globalPlaneDataPtr = planeDataPtr;
    planePtr = Tp_GetHorzPlane(planeDataPtr);
    horzPlaneBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    Mac_CopyRect(&planeSearchRect, horzPlaneBndPtr);
    Tp_FindBBoxForAllCells(planeDataPtr, &bbox);
    planeSearchRect.r_xtop = bbox.r_xtop;
    Mac_CopyRect(&tailRect, &planeSearchRect);
    tailRect.r_ytop = tailRect.r_ybot + 1;

    topPoint.p_x = planeSearchRect.r_xbot;
    topPoint.p_y = planeSearchRect.r_ytop;
    topmostTilePtr = TiSrPointNoHint(planePtr, &topPoint);

/*
printf ("Horizontal Compaction ...\n ");
fflush (stdout);
 */

    /* Search the horizontal compaction ridge */
    (void) TiSrArea(NIL(Tile), 
             planePtr, 
             &planeSearchRect,
             SearchHorzCompactRidge, 
             NIL (ClientData));

    /* Find the compaction ridge */
    ridgeInfo.ridgeWidth = 0;
    ridgeInfo.prevTilePtr = NIL(Tile);
    (void) TiSrArea(NIL(Tile),
            planePtr, 
            &tailRect,
            FindCompactRidge, 
            (ClientData) &ridgeInfo);

/*
printf ("ridge width =  %d\n", ridgeInfo.ridgeWidth); 
fflush (stdout);
 */

    /* Trace the compaction ridge */
    if (ridgeInfo.ridgeWidth > 0 && ridgeInfo.ridgeWidth < INFINITY)
    {
        GsBuildRidgeTileList(ridgeInfo.prevTilePtr, &ridgeHead);
/*
printf ("Drawing the ridge ...\n");
fflush (stdout);
*/

#ifndef BEAR_BLACK_BOX
        if ((bear_BlackBoxFlag == FALSE) && gs_DrawRidge) {
            /* Draw the compaction ridge */
            drawRidge.tileList = ridgeHead;
            (void) PbItrChipWindows(PbDrawHorzRidge, (ClientData) &drawRidge);

            /*
             * Store the horizontal tile list with the window so that
             * the ridge can be redrawn automatically if the window
             * needs to be updated.
             */
            PbPutHorzTileListOfCHW(ridgeHead, chipWinPtr);

            Gs_Pause(); 
            PbPutHorzTileListOfCHW(NIL(LinkedTile), chipWinPtr);
        }
        else
#endif BEAR_BLACK_BOX
        {
            Gs_Pause(); 
        }

        if (gs_Quit == TRUE)
        {
            return (GS_QUIT);
        }
    
/* 
printf ("Spacing the horizontal compaction ridge ...\n ");
fflush (stdout);
*/

        if (Tp_PushHorz(globalPlaneDataPtr, ridgeHead, 
            -ridgeInfo.ridgeWidth) == FALSE) {
#ifdef DEBUG
            printf ("Tp_PushHorz failed.\n");
            fflush (stdout);
#endif DEBUG
        }
        planeSearchRect.r_xtop += -ridgeInfo.ridgeWidth;
        /*
        globalWinDataPtr->wd_imageBnd.r_xtop += (-ridgeInfo.ridgeWidth);
        (void) Ez_CalcViewingTrans(globalWinDataPtr);
        */

        /* Update tile capacity */
        GsUpdateCapOfTileList(ridgeHead, ridgeInfo.ridgeWidth);
        return (GS_HC_NOT_FINISHED);
    }
    else
    {
        return (GS_HC_FINISHED);
    }

/*
printf ("Exit Gs_HorzCompact - \n"); 
fflush (stdout);
*/
}



/* ----------------------------------------------------------------------------
 *
 * Gs_VertCompact --
 *
 *     The main routine to perform vertical compaction,
 *
 * Results:
 *     None.
 *
 * Side effects:
 *     Whatever the side effects resulted from the compaction.
 *
 * ----------------------------------------------------------------------------
 */

SpacingStatus
Gs_VertCompact(planeDataPtr)
PlaneData   *planeDataPtr;
{
#ifndef BEAR_BLACK_BOX
    ChipWin     *chipWinPtr;
    DrawRidge   drawRidge;
#endif BEAR_BLACK_BOX

    RidgeInfo   ridgeInfo;
    LinkedTile  *ridgeHead;
    Rect        *horzPlaneBndPtr;
    Rect        bbox;
    Rect        planeSearchRectH;
    Rect        tailRectH;
    Point       leftPoint;

/*
printf ("Enter Gs_VertCompact - \n"); 
fflush (stdout);
*/

#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && gs_DrawRidge) {
        chipWinPtr = PbGetChipWin(globalWinDataPtr);

        drawRidge.displayFunc = GXcopy;
        drawRidge.planeDataPtr = planeDataPtr;
        drawRidge.ridgeColor = ez_HighLightColor;
    }
#endif BEAR_BLACK_BOX

    gs_Quit = FALSE;

    globalPlaneDataPtr = planeDataPtr;
    planePtr = Tp_GetVertPlane (planeDataPtr);
    horzPlaneBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    Mac_CopyRect(&planeSearchRectH, horzPlaneBndPtr);
    Tp_FindBBoxForAllCells(planeDataPtr, &bbox);
    planeSearchRectH.r_ytop = bbox.r_ytop;
    Tp_HRectToVRect(&planeSearchRectH, &planeSearchRect);
    Mac_CopyRect(&tailRectH, &bbox);
    tailRectH.r_xbot = bbox.r_xtop - 1;
    Tp_HRectToVRect(&tailRectH, &tailRect);

    leftPoint.p_x = planeSearchRect.r_xbot;
    leftPoint.p_y = planeSearchRect.r_ytop;
    leftmostTilePtr = TiSrPointNoHint(planePtr, &leftPoint);

/*
 *  printf ("Vertical Compaction ...\n ");
 *  fflush (stdout);
 */

    /* Search the vertical compaction ridge */
    (void) TiSrArea((Tile *) NULL, 
             planePtr, 
             &planeSearchRect,
             SearchVertCompactRidge, 
             NIL (ClientData));

    /* Find the compaction ridge */
    ridgeInfo.ridgeWidth = 0;
    ridgeInfo.prevTilePtr = NIL (Tile);
    (void) TiSrArea((Tile *) NULL, 
            planePtr, 
            &tailRect,
            FindCompactRidge, 
            (ClientData) &ridgeInfo);


/*
 *  printf ("ridge width =  %d\n", ridgeInfo.ridgeWidth); 
 *  fflush (stdout);
 */

    /* Trace the compaction ridge */
    if (ridgeInfo.ridgeWidth > 0 && ridgeInfo.ridgeWidth < INFINITY)
    {
        GsBuildRidgeTileList (ridgeInfo.prevTilePtr, &ridgeHead);

/*
printf ("Drawing the ridge ...\n");
fflush (stdout);
*/

#ifndef BEAR_BLACK_BOX
        if ((bear_BlackBoxFlag == FALSE) && gs_DrawRidge) {
            /* Draw the compaction ridge */
            drawRidge.tileList = ridgeHead;
            (void) PbItrChipWindows(PbDrawVertRidge, (ClientData) &drawRidge);

            /*
             * Store the vertical tile list with the window so that
             * the ridge can be redrawn automatically if the window
             * needs to be updated.
             */
            PbPutVertTileListOfCHW(ridgeHead, chipWinPtr);

            Gs_Pause(); 
            PbPutVertTileListOfCHW(NIL(LinkedTile), chipWinPtr);
        }
        else
#endif BEAR_BLACK_BOX
        {
            Gs_Pause(); 
        }

        if (gs_Quit == TRUE)
        {
            return (GS_QUIT);
        }

/*
printf ("Spacing the vertical compaction ridge ...\n ");
fflush (stdout);
*/

        if (Tp_PushVert(globalPlaneDataPtr, ridgeHead, 
            -ridgeInfo.ridgeWidth) == FALSE) {
#ifdef DEBUG
            printf ("Tp_PushVert failed.\n");
            fflush (stdout);
#endif DEBUG
        }
        planeSearchRect.r_ytop += (-ridgeInfo.ridgeWidth);
        /*
        globalWinDataPtr->wd_imageBnd.r_ytop += (-ridgeInfo.ridgeWidth);
        (void) Ez_CalcViewingTrans (globalWinDataPtr);
        */
    
        /* Update tile capacity */
        GsUpdateCapOfTileList (ridgeHead, ridgeInfo.ridgeWidth);
        return (GS_VC_NOT_FINISHED);
    }
    else
    {
        return (GS_VC_FINISHED);
    }

/*
printf ("Exit Gs_VertCompact - \n"); 
fflush (stdout);
*/
}


