
/* grEditor.h --
 *
 *     George Carvalho, Esq.
 *     Malgorzata Marek-Sadowska
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright(C) 1988 Regents of the University of California
 * All rights reserved.
 *
 * needs: magic.h, geometry.h, tile.h
 *
 * rcsid $Header$
 */

#ifndef _GREDITOR
#define _GREDITOR /* once-only latch */

#ifndef _MAGIC
int err1 = Need_to_include_magic_header;
#endif _MAGIC

#ifndef _GEOMETRY
int err2 = Need_to_include_geometry_header;
#endif _GEOMETRY

#ifndef _TILES
int err3 = Need_to_include_tile_header;
#endif _TILES


#include "ansi.h"
#include "ezPlot.h"
#include "grDraw.h"


typedef BearDummy *Gre_BldGraphDataPtr;


extern BearStatus Gre_CheckConnForOneNet(/* cellMasterPtr, netPtr, warnFlag */);
/*
 * CellMaster   *cellMasterPtr;
 * Net          *netPtr;
 * bool         warnFlag;
 */

extern BearStatus Gre_CheckConnForAllNets(/* cellMasterPtr, warnFlag */);
/*
 * CellMaster   *cellMasterPtr;
 * bool         warnFlag;
 */

extern BearStatus Gre_FindPin(/* lineSegPtr, spanPtr, netPtr, pinInstPtrPtr */);
/*
 * LineSeg      *lineSegPtr;
 * Line         *spanPtr;
 * Net          *netPtr;
 * PinInst      **pinInstPtrPtr;
 */

EXTERN BearStatus Gre_GetPin
    ARGS((WindowData    *winDataPtr, 
          BearString    piNamePtr,     /* pin instance name     */
          BearString    ciNamePtr,     /* cell instance name    */
          int           pinColor, 
          PinInst       **pinPtrPtr));

extern BearStatus
Gre_PickJunct(/* horzDomTilePtrPtr, vertDomTilePtrPtr, junctPtrPtr */);
/*
 * Tile         **horzDomTilePtrPtr, **vertDomTilePtrPtr;
 * WallJunction **junctPtrPtr;
 */

extern BearStatus Gre_PickNet(/* winDataPtr, netPtrPtr, pinPtrPtr */);
/*
 * WindowData   *winDataPtr;
 * Net          **netPtrPtr;
 * PinInst      **pinPtrPtr;
 */

extern BearStatus
Gre_PickNetByName(/* winDataPtr, netNamePtr, whichNet, netPtrPtr */);
/*
 * WindowData   *winDataPtr;
 * char         *netNamePtr;
 * int          whichNet;       (* FPG_NW | FPG_CENTER | FPG_SE *)
 * Net          **netPtrPtr;
 */

extern BearStatus
Gre_PickPinArea(/* cellInst, lineSeg, wPoint, wSpan */);
/*
 * CellInst     **cellInst;
 * LineSeg      **lineSeg;
 * Point        *wPoint;
 * Line         *wSpan;
 */


/*
 * Here are a few procedures to manipulate a nets graph which contains the
 * topology hey dude check out that all time betty of the global routing.
 * ------------------------------------------------------------
 */

extern Gre_BldGraphDataPtr
Gre_BldGraphBegin(/* cellMasterPtr, warnFlag, placeFlag, pgFlag */);
/*
 * CellMaster           *cellMasterPtr;         (* IN       *)
 * bool                 warnFlag;               (* IN       *)
 * bool                 placeFlag;              (* IN       *)
 * bool                 pgFlag;                 (* IN       *)
 */

extern BearStatus Gre_BldGraph(/* bldDataPtr, netPtr, graphPtr */);
/*
 * Gre_BldGraphDataPtr bldDataPtr;              (* IN       *)
 * Net                 *netPtr;                 (* IN       *)
 * Gt_Graph            *graphPtr;               (* OUT      *)
 */

extern void Gre_BldGraphEnd(/* bldDataPtr */);
/*
 * Gre_BldGraphDataPtr bldDataPtr;              (* IN       *)
 */

extern BearStatus
Gre_BuildAutoGraphOfNet(/* cellMasterPtr, netPtr,
                         * warnFlag, placeFlag, pgFlag
                         */);
/*
 * CellMaster  *cellMasterPtr;                  (* IN       *)
 * Net         *netPtr;                         (* IN       *)
 * bool        warnFlag;                        (* IN       *)
 * bool        placeFlag;                       (* IN       *)
 * bool        pgFlag;                          (* IN       *)
 */

extern BearStatus Gre_AddEdgeToGraph(/* netPtr */);
/*
 * Net          *netPtr;
 */

extern BearStatus Gre_DeleteEdgeFromGraph(/* netPtr */);
/*
 * Net          *netPtr;
 */

extern void Gre_RemoveNet(/* netPtr */);
/*
 * Net          *netPtr;
 */

extern BearStatus Gre_StoreNetInTiPlane(/* netPtr, planeDataPtr */);
/*
 * Net          *netPtr;
 * PlaneData    *planeDataPtr;
 */

/*
 * Here are a few procedures for dealing with the power/ground
 * wiring package.
 * ------------------------------------------------------------
 */

extern BearStatus Gre_DoPgWiringForOneNet(/* cellMasterPtr, netPtr */);
/*
 * CellMaster   *cellMasterPtr;
 * Net          *netPtr;
 */

extern BearStatus Gre_DoPgWiringForAllNets(/* cellMasterPtr */);
/*
 * CellMaster   *cellMasterPtr;
 */

extern void Gre_InitPortsOfNet(/* netPtr, maxVoltDrop, maxCurrentReq */);
/*
 * Net          *netPtr;
 * double       maxVoltDrop, maxCurrentReq;
 */

#endif /* _GREDITOR */

