
/* greDrawNet.c --
 *
 *     George Carvalho
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifndef BEAR_BLACK_BOX

#include "bearCopyRight.h"
#include <stdio.h>
#include <signal.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "grEditor.h"
#include "grEditorInt.h"

#include "../polarBear/polarBear.h"

typedef struct _LabelNetInt {
    WindowData  *winDataPtr;
    PlaneData   *planeDataPtr;
} LabelNetInt;

typedef struct _DrawGraphInt {
    WindowData  *winDataPtr;
    PlaneData   *planeDataPtr;
    int         graphColor;
    bool        dotFlag;
    bool        widthFlag;
} DrawGraphInt;


/* ---------- Local Procedures ---------- */
static BearStatus   DrawGraphNodeInt();
static BearStatus   DrawGraphEdgeInt();
static BearStatus   LabelNode();
#ifdef USE_WIRE_CODE
static BearStatus   LabelPgNode();
#endif USE_WIRE_CODE


bool drawFlag = FALSE;


static BearStatus 
DrawGraphNodeInt(node, intDrawGraphPtr)
Gt_GraphNode    node;
DrawGraphInt    *intDrawGraphPtr;
{
    PinOrJunct  *pinOrJunctPtr;
    Point       dotPoint;

    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node);
    
    if (Fpg_GetAbsPosOfPinOrJunct(intDrawGraphPtr->planeDataPtr, pinOrJunctPtr,
        &dotPoint) == BEAR_ERROR) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if ((intDrawGraphPtr->dotFlag == FALSE) &&
        (pinOrJunctPtr->unType == FPG_PIN)) {
        DrawData drawData;

        drawData.winDataPtr = intDrawGraphPtr->winDataPtr;
        drawData.color = intDrawGraphPtr->graphColor;

        (void) Gre_DrawPinOne(pinOrJunctPtr->unValue.pinPtr, &drawData);
    }
    else {
        Ez_DrawWorldSmallDot(intDrawGraphPtr->winDataPtr->wd_userWin,
            intDrawGraphPtr->winDataPtr->wd_scrMatrix,
            dotPoint.p_x, dotPoint.p_y, intDrawGraphPtr->graphColor);
    }

    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus 
DrawGraphEdgeInt(edge, intDrawGraphPtr)
Gt_GraphEdge    edge;
DrawGraphInt    *intDrawGraphPtr;
/*
 * Draw a line between two nodes in the global routing graph.
 */
{
    Gt_GraphNode    node1, node2;
    Point           point1, point2;
    PinOrJunct      *pinOrJunct1Ptr, *pinOrJunct2Ptr;

    node1 = Gt_GetNodeOneOfEdge(edge);
    node2 = Gt_GetNodeTwoOfEdge(edge);

    pinOrJunct1Ptr = (PinOrJunct *) Gt_GetNodeDataOfNode(node1);
    pinOrJunct2Ptr = (PinOrJunct *) Gt_GetNodeDataOfNode(node2);
    
    if (Fpg_GetAbsPosOfPinOrJunct(intDrawGraphPtr->planeDataPtr, pinOrJunct1Ptr,
        &point1) == BEAR_ERROR) {
        return (BEAR_CONTINUE_SEARCH);
    }
    if (Fpg_GetAbsPosOfPinOrJunct(intDrawGraphPtr->planeDataPtr, pinOrJunct2Ptr,
        &point2) == BEAR_ERROR) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (intDrawGraphPtr->widthFlag == TRUE) {
        Mac_EdgeData *edgeDataPtr;

        edgeDataPtr = (Mac_EdgeData *) Gt_GetEdgeDataOfEdge(edge);
        Ez_DrawFatLine(intDrawGraphPtr->winDataPtr->wd_userWin, 
            intDrawGraphPtr->winDataPtr->wd_scrMatrix,
            point1.p_x, point1.p_y, point2.p_x, point2.p_y,
            (int) edgeDataPtr->width, GXcopy, intDrawGraphPtr->graphColor);
    } 
    else {
        Ez_DrawWorldLine(intDrawGraphPtr->winDataPtr->wd_userWin,
            intDrawGraphPtr->winDataPtr->wd_scrMatrix,
            point1.p_x, point1.p_y, point2.p_x, point2.p_y,
            EZ_SOLIDLINE2, GXcopy, intDrawGraphPtr->graphColor, pbChipColor);
    }

    return (BEAR_CONTINUE_SEARCH);
}



#ifdef BETTY_BOOP
/*
 * Left over code for DrawGraphEdgeInt procedure.
 */
    if (intDrawGraphPtr->widthFlag == TRUE) {
        term        *bblTermPtr;
        rjun        *bblJunctPtr;
        PinInst     *pinPtr;
        RoutePin    *rpinPtr;
        int         hwidth = 0;
        bool        rsegEnd1Flag = FALSE;
        bool        rsegEnd2Flag = FALSE;

        switch (pinOrJunct1Ptr->unType) {
        case (int) FPG_PIN:
            pinPtr = pinOrJunct1Ptr->unValue.pinPtr;
            if (pinPtr->BBLTermPtr != NIL(RdbPinInfo)) {
                if (pinPtr->BBLTermPtr->termPtr != NIL(term)) {
                    if (pinPtr->BBLTermPtr->termPtr->rsp != NIL(rseg)) {
                        rsegEnd1Flag = TRUE;
                        hwidth = pinPtr->BBLTermPtr->termPtr->rsp->hwidth;
                    }
                }
            }
            break;
        case (int) FPG_ROUTE_PIN:
            rpinPtr = pinOrJunct1Ptr->unValue.rPinPtr;
            if (rpinPtr->BBLTermPtr != NIL(RdbPinInfo)) {
                if (rpinPtr->BBLTermPtr->termPtr != NIL(term)) {
                    if (rpinPtr->BBLTermPtr->termPtr->rsp != NIL(rseg)) {
                        rsegEnd1Flag = TRUE;
                        hwidth = rpinPtr->BBLTermPtr->termPtr->rsp->hwidth;
                    }
                }
            }
            break;
        case (int) FPG_BBL_TERM:
            rsegEnd1Flag = TRUE;

            bblTermPtr = pinOrJunct1Ptr->unValue.bblTermPtr;
            hwidth = bblTermPtr->rsp->hwidth;
            break;
        case (int) FPG_BBL_JUNCT:
            rsegEnd1Flag = TRUE;

            bblJunctPtr = pinOrJunct1Ptr->unValue.bblJunctPtr;
            hwidth = bblJunctPtr->sljr->hwidth;
            break;
        case (int) FPG_PSEUDO_PIN:
        case (int) FPG_WALL_JUNCT:
            break;
        default:
         Bear_PrintErr("DrawGraphEdgeInt() : (pj 1) unexpected pin or junction type");
            return (BEAR_CONTINUE_SEARCH);
        }

        switch (pinOrJunct2Ptr->unType) {
        case (int) FPG_PIN:
            pinPtr = pinOrJunct2Ptr->unValue.pinPtr;
            if (pinPtr->BBLTermPtr != NIL(RdbPinInfo)) {
                if (pinPtr->BBLTermPtr->termPtr != NIL(term)) {
                    if (pinPtr->BBLTermPtr->termPtr->rsp != NIL(rseg)) {
                        rsegEnd2Flag = TRUE;
                        hwidth = pinPtr->BBLTermPtr->termPtr->rsp->hwidth;
                    }
                }
            }
            break;
        case (int) FPG_ROUTE_PIN:
            rpinPtr = pinOrJunct2Ptr->unValue.rPinPtr;
            if (rpinPtr->BBLTermPtr != NIL(RdbPinInfo)) {
                if (rpinPtr->BBLTermPtr->termPtr != NIL(term)) {
                    if (rpinPtr->BBLTermPtr->termPtr->rsp != NIL(rseg)) {
                        rsegEnd2Flag = TRUE;
                        hwidth = rpinPtr->BBLTermPtr->termPtr->rsp->hwidth;
                    }
                }
            }
            break;
        case (int) FPG_BBL_TERM:
            rsegEnd2Flag = TRUE;

            bblTermPtr = pinOrJunct2Ptr->unValue.bblTermPtr;
            hwidth = bblTermPtr->rsp->hwidth;
            break;
        case (int) FPG_BBL_JUNCT:
            rsegEnd2Flag = TRUE;

            bblJunctPtr = pinOrJunct2Ptr->unValue.bblJunctPtr;
            hwidth = bblJunctPtr->sljr->hwidth;
            break;
        case (int) FPG_PSEUDO_PIN:
        case (int) FPG_WALL_JUNCT:
            break;
        default:
         Bear_PrintErr("DrawGraphEdgeInt() : (pj 2) unexpected pin or junction type");
            return (BEAR_CONTINUE_SEARCH);
        }

        if ((rsegEnd1Flag == TRUE) && (rsegEnd2Flag == TRUE)) {
            /* draw the route segment */

            PinOrJunct  *tmpPinOrJunctPtr;
            Rect        rect;
            int         p1, p2;

            if (point1.p_x == point2.p_x) {
                rect.r_xbot = point1.p_x - hwidth;
                rect.r_xtop = point1.p_x + hwidth;
                if (point1.p_y > point2.p_y) {
                    p1 = point2.p_y;
                    p2 = point1.p_y;

                    tmpPinOrJunctPtr = pinOrJunct2Ptr;
                    pinOrJunct2Ptr = pinOrJunct1Ptr;
                    pinOrJunct1Ptr = tmpPinOrJunctPtr;
                }
                else {
                    p1 = point1.p_y;
                    p2 = point2.p_y;
                }

                rect.r_ybot = (pinOrJunct1Ptr->unType == (int) FPG_PIN) ? p1 :
                    p1 - hwidth;
                rect.r_ytop = (pinOrJunct2Ptr->unType == (int) FPG_PIN) ? p2 :
                    p2 + hwidth;
            }
            else {
                rect.r_ybot = point1.p_y - hwidth;
                rect.r_ytop = point1.p_y + hwidth;
                if (point1.p_x > point2.p_x) {
                    p1 = point2.p_x;
                    p2 = point1.p_x;

                    tmpPinOrJunctPtr = pinOrJunct2Ptr;
                    pinOrJunct2Ptr = pinOrJunct1Ptr;
                    pinOrJunct1Ptr = tmpPinOrJunctPtr;
                }
                else {
                    p1 = point1.p_x;
                    p2 = point2.p_x;
                }

                rect.r_xbot = (pinOrJunct1Ptr->unType == (int) FPG_PIN) ? p1 :
                    p1 - hwidth;
                rect.r_xtop = (pinOrJunct2Ptr->unType == (int) FPG_PIN) ? p2 :
                    p2 + hwidth;
            }

            Ez_FillRect(intDrawGraphPtr->winDataPtr->wd_userWin,
                intDrawGraphPtr->winDataPtr->wd_scrMatrix,
                &rect, intDrawGraphPtr->graphColor);
        }
    }
#endif BETTY_BOOP


#ifdef USE_WIRE_CODE
static BearStatus
LabelPGNode(node, intLabelNetPtr)
Gt_GraphNode    node;
LabelNetInt     *intLabelNetPtr;
/*
 * Label power ground graph node.
 */
{
    PinOrJunct  *pinOrJunctPtr;
    Point       *junctPointPtr, nodePoint;
    char        number[80];

    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node);

    if (pinOrJunctPtr->genericPtr != (BearGeneric) NULL) {
        sprintf (number, "%d",
            ((Pgr_NodeProp *) pinOrJunctPtr->genericPtr)->nodeNum);
    }
    else {
        Bear_PrintErr("LabelPGNode() : node number not present");
        return (BEAR_STOP_SEARCH);
    }

    if (Fpg_GetAbsPosOfPinOrJunct(intLabelNetPtr->planeDataPtr, pinOrJunctPtr,
        &nodePoint) == BEAR_ERROR) {
        return (BEAR_CONTINUE_SEARCH);
    }

    Ez_WorldText(intLabelNetPtr->winDataPtr->wd_userWin,
        intLabelNetPtr->winDataPtr->wd_scrMatrix,
        nodePoint.p_x, nodePoint.p_y,
        number);

    return (BEAR_CONTINUE_SEARCH);
}
#endif USE_WIRE_CODE



static BearStatus
LabelNode(node, intLabelNetPtr)
Gt_GraphNode    node;
LabelNetInt     *intLabelNetPtr;
{
    PinOrJunct  *pinOrJunctPtr;
    Point       *junctPointPtr, nodePoint;
    char        number[80];

    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node);
    if (Fpg_GetAbsPosOfPinOrJunct(intLabelNetPtr->planeDataPtr, pinOrJunctPtr,
        &nodePoint) == BEAR_ERROR) {
        return (BEAR_CONTINUE_SEARCH);
    }

    sprintf (number, "%d", Gt_GetIdOfNode(node));
    Ez_WorldText(intLabelNetPtr->winDataPtr->wd_userWin,
        intLabelNetPtr->winDataPtr->wd_scrMatrix,
        nodePoint.p_x, nodePoint.p_y,
        number);

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * GreDrawEdgeOfGraphOne() --
 *
 * This procedure is designed to be used by an "interator".
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, otherwise.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
GreDrawEdgeOfGraphOne(edge, drawDataPtr)
Gt_GraphEdge    edge;
DrawData        *drawDataPtr;
{
    PinOrJunct      *pinOrJunct1Ptr, *pinOrJunct2Ptr;
    Gt_GraphNode    node1, node2;

    node1 = Gt_GetNodeOneOfEdge(edge);
    node2 = Gt_GetNodeTwoOfEdge(edge);

    pinOrJunct1Ptr = (PinOrJunct *) Gt_GetNodeDataOfNode(node1);
    pinOrJunct2Ptr = (PinOrJunct *) Gt_GetNodeDataOfNode(node2);
    
    drawDataPtr->width = ((Mac_EdgeData *) Gt_GetEdgeDataOfEdge(edge))->width;

    if (pinOrJunct1Ptr->unType == FPG_PIN) {
        if (pinOrJunct2Ptr->unType == FPG_PIN) {
            /* pin to pin */
            Gre_DrawFromPinToPin(drawDataPtr,
                pinOrJunct1Ptr->unValue.pinPtr,
                pinOrJunct2Ptr->unValue.pinPtr);
        } 
        else if (pinOrJunct2Ptr->unType == FPG_WALL_JUNCT) {
            /* pin to junction */
            Gre_DrawFromPinToJunct(drawDataPtr,
                pinOrJunct1Ptr->unValue.pinPtr,
                pinOrJunct2Ptr->unValue.junctPtr);
        } 
        else  if (pinOrJunct2Ptr->unType == FPG_ROUTE_PIN){
            /* pin to route pin */ 
            Gre_DrawFromRPinToPin(drawDataPtr,
                pinOrJunct2Ptr->unValue.rPinPtr,
                pinOrJunct1Ptr->unValue.pinPtr);
        }
        else {
            Bear_PrintErr("GreDrawEdgeOfGraphOne(): illegal second node type.");
        }
    } 
    else if (pinOrJunct1Ptr->unType == FPG_WALL_JUNCT) {
        if (pinOrJunct2Ptr->unType == FPG_PIN) {
            /* junction to pin */
            Gre_DrawFromPinToJunct(drawDataPtr,
                pinOrJunct2Ptr->unValue.pinPtr,
                pinOrJunct1Ptr->unValue.junctPtr);
        } 
        else if (pinOrJunct2Ptr->unType == FPG_WALL_JUNCT) {
            /* junction to junction */
            Gre_DrawFromJunctToJunct(drawDataPtr,
                pinOrJunct1Ptr->unValue.junctPtr,
                pinOrJunct2Ptr->unValue.junctPtr);
        } 
        else  if (pinOrJunct2Ptr->unType == FPG_ROUTE_PIN) {
            /* junction to route pin */ 
            Gre_DrawFromRPinToJunct(drawDataPtr,
                pinOrJunct2Ptr->unValue.rPinPtr,
                pinOrJunct1Ptr->unValue.junctPtr);
        }
        else {
            Bear_PrintErr("GreDrawEdgeOfGraphOne(): illegal second node type.");
        }
    } 
    else if (pinOrJunct1Ptr->unType == FPG_ROUTE_PIN) {
        if (pinOrJunct2Ptr->unType == FPG_PIN) {
            /* route pin to pin */
            Gre_DrawFromRPinToPin(drawDataPtr,
                pinOrJunct1Ptr->unValue.rPinPtr,
                pinOrJunct2Ptr->unValue.pinPtr);
        } 
        else if (pinOrJunct2Ptr->unType == FPG_WALL_JUNCT) {
            /* route pin to junction */
            Gre_DrawFromRPinToJunct(drawDataPtr,
                pinOrJunct1Ptr->unValue.rPinPtr,
                pinOrJunct2Ptr->unValue.junctPtr);
        } 
        else  if (pinOrJunct2Ptr->unType == FPG_ROUTE_PIN){
            /* route pin to route pin */
            Gre_DrawFromRPinToRPin(drawDataPtr,
                pinOrJunct1Ptr->unValue.rPinPtr,
                pinOrJunct2Ptr->unValue.rPinPtr);
        } 
        else {
           Bear_PrintErr("GreDrawEdgeOfGraphOne(): illegal second node type.");
        }
    } 
    else {
        Bear_PrintErr("GreDrawEdgeOfGraphOne(): illegal first node type.");
    }

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * GreDrawEdgeOfGraphTwo() --
 *
 * This procedure is designed to be used by an "interator".
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion, even if theres
 *   an error because the interator should continue on to the next window.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
GreDrawEdgeOfGraphTwo(winDataPtr, edge)
WindowData      *winDataPtr;
Gt_GraphEdge    edge;
{
    DrawData drawData;

    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    drawData.winDataPtr = winDataPtr;
    drawData.color = PbGetColorOfNet(greWhichJunct);
    (void) GreDrawEdgeOfGraphOne(edge, &drawData);

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawNet() --
 *
 * This procedure is designed to be used by an "interator".
 * It draws the net and all of its pins in the chip window.
 *
 * All of the fields from the structure DrawNet must be filled in.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawNet(winDataPtr, drawNetPtr)
WindowData  *winDataPtr;
DrawNet     *drawNetPtr;
{
    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (drawNetPtr->netPtr != NIL(Net)) {
        DrawData    drawData;
        Gt_Graph    autoGraph, userGraph;

        drawData.winDataPtr = winDataPtr;
        drawData.color = drawNetPtr->netColor;

        (void) Gre_DrawPinsOfNet(drawNetPtr->netPtr, &drawData);

        Gre_ChooseWallJunct(drawNetPtr->whichNet);

        if ((userGraph = Mac_GetUserGraphOfNet(drawNetPtr->netPtr)) !=
            (Gt_Graph) NULL) {
            /*
             * Draw the net based on information in the user generated graph.
             */

            (void) Gt_ItrEdgesOfGraph(userGraph, GreDrawEdgeOfGraphOne,
                (ClientData) &drawData);
        }
        else if (Mac_IsGlobalRoutingDone(drawNetPtr->cellMasterPtr) == TRUE) {
            DrawGraph       drawGraph;
            NetKind         netKind;
            Mac_GraphData   *graphDataPtr;
            bool            pgFlag;
            bool            buildGraph = TRUE;

            netKind = Mac_GetKindOfNet(drawNetPtr->netPtr);
            pgFlag = ((netKind == POWER_NET) || (netKind == GROUND_NET));

            if ((autoGraph = Mac_GetAutoGraphOfNet(drawNetPtr->netPtr)) !=
                (Gt_Graph) NULL) {
                if (Mac_IsGraphValid(drawNetPtr->cellMasterPtr, autoGraph) ==
                    TRUE) {
                    graphDataPtr =
                        (Mac_GraphData *) Gt_GetGraphDataOfGraph(autoGraph);
                    if (pgFlag == graphDataPtr->pgFlag) {
                        buildGraph = FALSE;
                    }
                }
            }

            if (buildGraph == TRUE) {
                Mac_DeleteAutoGraphOfNet(drawNetPtr->netPtr);
                if (Gre_BuildAutoGraphOfNet(drawNetPtr->cellMasterPtr,
                    drawNetPtr->netPtr, FALSE, FALSE, pgFlag) == BEAR_ERROR) {
                    char *netNamePtr;

                    netNamePtr = Mac_GetNameOfNet(drawNetPtr->netPtr);
                    sprintf (bear_ErrorMsg,
                        "Cannot draw net named \"%s\".\n", netNamePtr);
                    PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);

                    return (BEAR_CONTINUE_SEARCH);
                }
            }

            drawGraph.graph = Mac_GetAutoGraphOfNet(drawNetPtr->netPtr);
            drawGraph.planeDataPtr =
                Mac_GetCellPlaneOfCell(drawNetPtr->cellMasterPtr);
            drawGraph.graphColor = drawNetPtr->netColor;
            drawGraph.dotFlag = FALSE;
#ifdef USE_WIRE_CODE
            drawGraph.widthFlag = ((pgFlag == TRUE) &&
                (drawNetPtr->netPtr->wirePtr != NIL(Pgr_Wire)));
#else
            drawGraph.widthFlag = FALSE;
#endif USE_WIRE_CODE
            (void) Gre_DrawGraph(winDataPtr, &drawGraph);
        }
        else {
            /*
             * (gyorgy 19 dec 90) This extra case was tacked on by me
             * to handle graphs generated by the PbSteinerTreeDialog()
             * procedure in polarBear.
             *
             * It should be extended to create them all be itself.
             */

            DrawGraph drawGraph;

            if ((autoGraph = Mac_GetAutoGraphOfNet(drawNetPtr->netPtr)) !=
                (Gt_Graph) NULL) {
                if (Mac_IsGraphValid(drawNetPtr->cellMasterPtr,
                    autoGraph) == TRUE) {

                    drawGraph.graph = autoGraph;
                    drawGraph.planeDataPtr =
                        Mac_GetCellPlaneOfCell(drawNetPtr->cellMasterPtr);
                    drawGraph.graphColor = drawNetPtr->netColor;
                    drawGraph.dotFlag = FALSE;
                    drawGraph.widthFlag = FALSE;
                    (void) Gre_DrawGraph(winDataPtr, &drawGraph);
                }
            }
        }
        Ez_Flush();
    }

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawGraph() --
 *
 * This procedure is designed to be used by an "interator".  It can draw
 * the given graph in a chip window or a placement window.
 *
 * All of the fields from the structure DrawGraph must be filled in.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon successful completion or
 *   if the given window is not a chip window, BEAR_STOP_SEARCH otherwise.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawGraph(winDataPtr, drawGraphPtr)
WindowData  *winDataPtr;
DrawGraph   *drawGraphPtr;
{
    DrawGraphInt intDrawGraph;

    /*
     * If this is not a chip or a placement window return BEAR_CONTINUE_SEARCH.
     */
    if ((PbIsChipWin(winDataPtr) == FALSE) &&
        (PbIsPlaceWin(winDataPtr) == FALSE)) {
        return (BEAR_CONTINUE_SEARCH);
    }

    intDrawGraph.winDataPtr = winDataPtr;
    intDrawGraph.planeDataPtr = drawGraphPtr->planeDataPtr;
    intDrawGraph.graphColor = drawGraphPtr->graphColor;
    intDrawGraph.dotFlag = drawGraphPtr->dotFlag;
    intDrawGraph.widthFlag = drawGraphPtr->widthFlag;

    (void) Gt_ItrEdgesOfGraph(drawGraphPtr->graph, DrawGraphEdgeInt,
        (ClientData) &intDrawGraph);
    (void) Gt_ItrNodesOfGraph(drawGraphPtr->graph, DrawGraphNodeInt,
        (ClientData) &intDrawGraph);

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawGraphNode() --
 *
 * This procedure is designed to be used by an "interator".  It can draw
 * the given node in a chip window or a placement window.
 *
 * All of the fields from the structure DrawGraphNode must be filled in.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon successful completion or
 *   if the given window is not a chip window, BEAR_STOP_SEARCH otherwise.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawGraphNode(winDataPtr, drawGraphNodePtr)
WindowData      *winDataPtr;
DrawGraphNode   *drawGraphNodePtr;
{
    DrawGraphInt intDrawGraph;

    /*
     * If this is not a chip or a placement window return BEAR_CONTINUE_SEARCH.
     */
    if ((PbIsChipWin(winDataPtr) == FALSE) &&
        (PbIsPlaceWin(winDataPtr) == FALSE)) {
        return (BEAR_CONTINUE_SEARCH);
    }

    intDrawGraph.winDataPtr = winDataPtr;
    intDrawGraph.planeDataPtr = drawGraphNodePtr->planeDataPtr;
    intDrawGraph.graphColor = drawGraphNodePtr->nodeColor;
    intDrawGraph.dotFlag = drawGraphNodePtr->dotFlag;
    intDrawGraph.widthFlag = FALSE;

    (void) DrawGraphNodeInt(drawGraphNodePtr->node, &intDrawGraph);

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawGraphEdge() --
 *
 * This procedure is designed to be used by an "interator".  It can draw
 * the given edge in a chip window or a placement window.
 *
 * All of the fields from the structure DrawGraphEdge must be filled in.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon successful completion or
 *   if the given window is not a chip window, BEAR_STOP_SEARCH otherwise.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawGraphEdge(winDataPtr, drawGraphEdgePtr)
WindowData      *winDataPtr;
DrawGraphEdge   *drawGraphEdgePtr;
{
    DrawGraphInt intDrawGraph;

    /*
     * If this is not a chip or a placement window return BEAR_CONTINUE_SEARCH.
     */
    if ((PbIsChipWin(winDataPtr) == FALSE) &&
        (PbIsPlaceWin(winDataPtr) == FALSE)) {
        return (BEAR_CONTINUE_SEARCH);
    }

    intDrawGraph.winDataPtr = winDataPtr;
    intDrawGraph.planeDataPtr = drawGraphEdgePtr->planeDataPtr;
    intDrawGraph.graphColor = drawGraphEdgePtr->edgeColor;
    intDrawGraph.dotFlag = drawGraphEdgePtr->dotFlag;
    intDrawGraph.widthFlag = drawGraphEdgePtr->widthFlag;

    (void) DrawGraphEdgeInt(drawGraphEdgePtr->edge, &intDrawGraph);

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Gre_LabelNet() --
 *
 * This procedure is designed to be used by an "interator".  It visits each
 * node in the auto-graph of the given net and prints a label next to
 * it.  The labels themselves are produced by a procedure.

 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_LabelNet(winDataPtr, labelNetPtr)
WindowData  *winDataPtr;
LabelNet    *labelNetPtr;
{
    BearStatus      (*labelProc)();
    Gt_Graph        autoGraph;
    LabelNetInt     intLabelNet;
    Mac_GraphData   *graphDataPtr;

    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (labelNetPtr->netPtr == NIL(Net)) {
        return (BEAR_STOP_SEARCH);
    } 

    if ((autoGraph = Mac_GetAutoGraphOfNet(labelNetPtr->netPtr)) ==
        (Gt_Graph) NULL) {

        Bear_PrintErr("Gre_LabelNet() failed : This net has no graph.");
        return (BEAR_STOP_SEARCH);
    }

#ifdef USE_WIRE_CODE
    graphDataPtr = (Mac_GraphData *) Gt_GetGraphDataOfGraph(autoGraph);
    if (graphDataPtr->pgFlag == TRUE) {
        if (labelNetPtr->netPtr->wirePtr == NIL(Pgr_Wire)) {
            Bear_PrintErr("Gre_LabelNet() failed : This net has no wire data.");
            return (BEAR_STOP_SEARCH);
        }
        labelProc = LabelPGNode;
    }
    else
#endif USE_WIRE_CODE
    {
        labelProc = LabelNode;
    }

    Gre_ChooseWallJunct(labelNetPtr->whichNet);

    intLabelNet.winDataPtr = winDataPtr;
    intLabelNet.planeDataPtr = labelNetPtr->planeDataPtr;

    if (Gt_ItrNodesOfGraph(autoGraph, labelProc,
        (ClientData) &intLabelNet) == BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }
    Ez_Flush();

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromRPinToRPin() --
 *
 * Draw each pin and a line between them.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromRPinToRPin(drawDataPtr, rpinPtr1, rpinPtr2)
DrawData    *drawDataPtr;
RoutePin    *rpinPtr1;
PinInst     *rpinPtr2;
{
    PinDirection    pinDir1, pinDir2;
    Point           pinPoint1, pinPoint2;
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, midPoint, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;
    pinDir1 = Mac_GetDirectionOfRPin(rpinPtr1);
    pinDir2 = Mac_GetDirectionOfRPin(rpinPtr2);

    Mac_GetAbsPosOfRPin(rpinPtr1, &pinPoint1);
    Mac_GetAbsPosOfRPin(rpinPtr2, &pinPoint2);

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }

            /*
             * Draw from pin number one to the midline.
             */
            rect.r_xbot = pinPoint1.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = midPoint - halfWidth;
                rect.r_ytop = pinPoint1.p_y;
            } else {
                rect.r_ybot = pinPoint1.p_y;
                rect.r_ytop = midPoint + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x - halfWidth;
                rect.r_xtop = pinPoint1.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint1.p_x - halfWidth;
                rect.r_xtop = pinPoint2.p_x + halfWidth;
            }
            rect.r_ybot = midPoint - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw from pin number two to the midline.
             */
            rect.r_xbot = pinPoint2.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y;
                rect.r_ytop = midPoint + halfWidth;
            } else {
                rect.r_ybot = midPoint - halfWidth;
                rect.r_ytop = pinPoint2.p_y;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            /*
             * Draw from pin number one to the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = midPoint - halfWidth;
                rect.r_xtop = pinPoint1.p_x;
            } else {
                rect.r_xbot = pinPoint1.p_x;
                rect.r_xtop = midPoint + halfWidth;
            }
            rect.r_ybot = pinPoint1.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw the midline.
             */
            rect.r_xbot = midPoint - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y - halfWidth;
                rect.r_ytop = pinPoint1.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint1.p_y - halfWidth;
                rect.r_ytop = pinPoint2.p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw from pin number two to the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x;
                rect.r_xtop = midPoint + halfWidth;
            } else {
                rect.r_xbot = midPoint - halfWidth;
                rect.r_xtop = pinPoint2.p_x;
            }
            rect.r_ybot = pinPoint2.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromRPinToRPin() : illegal choice of pins");
            return;
        }
    } else {
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }
            /* Draw from pin number one to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                pinPoint1.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, midPoint,
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw from pin number two to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, pinPoint2.p_y, 
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            /* Draw from pin number one to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                midPoint, pinPoint1.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                midPoint, pinPoint1.p_y, 
                midPoint, pinPoint2.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw from pin number two to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, pinPoint2.p_y, 
                midPoint, pinPoint2.p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromRPinToRPin() : illegal choice of pins");
            return;
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromRPinToPin() --
 *
 * Draw each pin and a line between them.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromRPinToPin(drawDataPtr, rpinPtr, pinInstPtr)
DrawData    *drawDataPtr;
RoutePin    *rpinPtr;
PinInst     *pinInstPtr;
{
    PinDirection    pinDir1, pinDir2;
    Point           pinPoint1, pinPoint2;
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, midPoint, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;
    pinDir1 = Mac_GetDirectionOfRPin(rpinPtr);
    pinDir2 = Mac_GetDirectionOfPin(pinInstPtr);

    Mac_GetAbsPosOfRPin(rpinPtr, &pinPoint1);

    Mac_GetAbsPosOfPin(pinInstPtr, &pinPoint2);
    (void) Gre_DrawPinOne(pinInstPtr, drawDataPtr);

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }

            /*
             * Draw from pin number one to the midline.
             */
            rect.r_xbot = pinPoint1.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = midPoint - halfWidth;
                rect.r_ytop = pinPoint1.p_y;
            } else {
                rect.r_ybot = pinPoint1.p_y;
                rect.r_ytop = midPoint + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x - halfWidth;
                rect.r_xtop = pinPoint1.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint1.p_x - halfWidth;
                rect.r_xtop = pinPoint2.p_x + halfWidth;
            }
            rect.r_ybot = midPoint - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw from pin number two to the midline.
             */
            rect.r_xbot = pinPoint2.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y;
                rect.r_ytop = midPoint + halfWidth;
            } else {
                rect.r_ybot = midPoint - halfWidth;
                rect.r_ytop = pinPoint2.p_y;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            /*
             * Draw from pin number one to the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = midPoint - halfWidth;
                rect.r_xtop = pinPoint1.p_x;
            } else {
                rect.r_xbot = pinPoint1.p_x;
                rect.r_xtop = midPoint + halfWidth;
            }
            rect.r_ybot = pinPoint1.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw the midline.
             */
            rect.r_xbot = midPoint - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y - halfWidth;
                rect.r_ytop = pinPoint1.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint1.p_y - halfWidth;
                rect.r_ytop = pinPoint2.p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw from pin number two to the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x;
                rect.r_xtop = midPoint + halfWidth;
            } else {
                rect.r_xbot = midPoint - halfWidth;
                rect.r_xtop = pinPoint2.p_x;
            }
            rect.r_ybot = pinPoint2.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromRPinToPin() : illegal choice of pins");
            return;
        }
    } else {
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }
            /* Draw from pin number one to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                pinPoint1.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, midPoint,
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw from pin number two to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, pinPoint2.p_y, 
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            /* Draw from pin number one to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                midPoint, pinPoint1.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                midPoint, pinPoint1.p_y, 
                midPoint, pinPoint2.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw from pin number two to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, pinPoint2.p_y, 
                midPoint, pinPoint2.p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromRPinToPin() : illegal choice of pins");
            return;
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromPinToPin() --
 *
 * Draw each pin and a line between them.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromPinToPin(drawDataPtr, pinInst1Ptr, pinInst2Ptr)
DrawData    *drawDataPtr;
PinInst     *pinInst1Ptr, *pinInst2Ptr;
{
    PinDirection    pinDir1, pinDir2;
    Point           pinPoint1, pinPoint2;
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, midPoint, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;
    pinDir1 = Mac_GetDirectionOfPin(pinInst1Ptr);
    pinDir2 = Mac_GetDirectionOfPin(pinInst2Ptr);

    Mac_GetAbsPosOfPin(pinInst1Ptr, &pinPoint1);
    (void) Gre_DrawPinOne(pinInst1Ptr, drawDataPtr);

    Mac_GetAbsPosOfPin(pinInst2Ptr, &pinPoint2);
    (void) Gre_DrawPinOne(pinInst2Ptr, drawDataPtr);

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }

            /*
             * Draw from pin number one to the midline.
             */
            rect.r_xbot = pinPoint1.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = midPoint - halfWidth;
                rect.r_ytop = pinPoint1.p_y;
            } else {
                rect.r_ybot = pinPoint1.p_y;
                rect.r_ytop = midPoint + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x - halfWidth;
                rect.r_xtop = pinPoint1.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint1.p_x - halfWidth;
                rect.r_xtop = pinPoint2.p_x + halfWidth;
            }
            rect.r_ybot = midPoint - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw from pin number two to the midline.
             */
            rect.r_xbot = pinPoint2.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y;
                rect.r_ytop = midPoint + halfWidth;
            } else {
                rect.r_ybot = midPoint - halfWidth;
                rect.r_ytop = pinPoint2.p_y;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            /*
             * Draw from pin number one to the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = midPoint - halfWidth;
                rect.r_xtop = pinPoint1.p_x;
            } else {
                rect.r_xbot = pinPoint1.p_x;
                rect.r_xtop = midPoint + halfWidth;
            }
            rect.r_ybot = pinPoint1.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw the midline.
             */
            rect.r_xbot = midPoint - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y - halfWidth;
                rect.r_ytop = pinPoint1.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint1.p_y - halfWidth;
                rect.r_ytop = pinPoint2.p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            /*
             * Draw from pin number two to the midline.
             */
            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x;
                rect.r_xtop = midPoint + halfWidth;
            } else {
                rect.r_xbot = midPoint - halfWidth;
                rect.r_xtop = pinPoint2.p_x;
            }
            rect.r_ybot = pinPoint2.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromPinToPin() : illegal choice of pins");
            return;
        }
    } else {
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }
            /* Draw from pin number one to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                pinPoint1.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, midPoint,
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw from pin number two to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, pinPoint2.p_y, 
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            /* Draw from pin number one to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                midPoint, pinPoint1.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                midPoint, pinPoint1.p_y, 
                midPoint, pinPoint2.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);

            /* Draw from pin number two to the midline. */
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, pinPoint2.p_y, 
                midPoint, pinPoint2.p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromPinToPin() : illegal choice of pins");
            return;
        }
    /* Draw a straight line between pin and junction.
        Ez_DrawWorldLine(chipWin, scrMatrix,
            pinPoint1.p_x, pinPoint1.p_y, 
            pinPoint2.p_x, pinPoint2.p_y, 
            EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        */
    }
}




/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromDCellToDCell() --
 *
 * Draw line between the center of dummy cells.
 *
 * If there are drawing bugs this piece of code should match
 * Gre_DrawFromPinToPin() which has been fixed.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromDCellToDCell(drawDataPtr, pinDir1, pinPoint1, pinDir2, pinPoint2)
DrawData     *drawDataPtr;
PinDirection pinDir1;
Point        pinPoint1;
PinDirection pinDir2;
Point        pinPoint2;
{
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, midPoint, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }

            rect.r_xbot = pinPoint1.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            rect.r_ybot = midPoint - halfWidth;
            rect.r_ytop = (pinPoint1.p_y > pinPoint2.p_y) ?
                pinPoint1.p_y : pinPoint2.p_y;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x - halfWidth;
                rect.r_xtop = pinPoint1.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint1.p_x - halfWidth;
                rect.r_xtop = pinPoint2.p_x + halfWidth;
            }
            rect.r_ybot = midPoint - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = pinPoint2.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            rect.r_ybot = (pinPoint1.p_y > pinPoint2.p_y) ?
                pinPoint2.p_y : pinPoint1.p_y;
            rect.r_ytop = midPoint + halfWidth;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            rect.r_xbot = midPoint - halfWidth;
            rect.r_xtop = (pinPoint1.p_x > pinPoint2.p_x) ?
                pinPoint1.p_x : pinPoint2.p_x;
            rect.r_ybot = pinPoint1.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = midPoint - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y - halfWidth;
                rect.r_ytop = pinPoint1.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint1.p_y - halfWidth;
                rect.r_ytop = pinPoint2.p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = (pinPoint1.p_x > pinPoint2.p_x) ?
                pinPoint2.p_x : pinPoint1.p_x;
            rect.r_xtop = midPoint + halfWidth;
            rect.r_ybot = pinPoint2.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromDCellToDCell() : illegal choice of pins");
            return;
        }
    } else {
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                pinPoint1.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, midPoint,
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, midPoint,
                pinPoint2.p_x, pinPoint2.p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                midPoint, pinPoint1.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                midPoint, pinPoint1.p_y, 
                midPoint, pinPoint2.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                midPoint, pinPoint2.p_y, 
                pinPoint2.p_x, pinPoint2.p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromDCellToDCell() : illegal choice of pins");
            return;
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromPinToDCell() --
 *
 * Draw pin and a line between it and the center of the dummy cell.
 *
 * If there are drawing bugs this piece of code should match
 * Gre_DrawFromPinToPin() which has been fixed.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromPinToDCell(drawDataPtr, pinInst1Ptr, pinDir2, pinPoint2)
DrawData     *drawDataPtr;
PinInst      *pinInst1Ptr;
PinDirection pinDir2;
Point        pinPoint2;
{
    PinDirection    pinDir1;
    Point           pinPoint1;
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, midPoint, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;
    pinDir1 = Mac_GetDirectionOfPin(pinInst1Ptr);

    Mac_GetAbsPosOfPin(pinInst1Ptr, &pinPoint1);
    (void) Gre_DrawPinOne(pinInst1Ptr, drawDataPtr);


    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }

            rect.r_xbot = pinPoint1.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            rect.r_ybot = midPoint - halfWidth;
            rect.r_ytop = (pinPoint1.p_y > pinPoint2.p_y) ?
                pinPoint1.p_y : pinPoint2.p_y;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            if (pinPoint1.p_x > pinPoint2.p_x) {
                rect.r_xbot = pinPoint2.p_x - halfWidth;
                rect.r_xtop = pinPoint1.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint1.p_x - halfWidth;
                rect.r_xtop = pinPoint2.p_x + halfWidth;
            }
            rect.r_ybot = midPoint - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = pinPoint2.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            rect.r_ybot = (pinPoint1.p_y > pinPoint2.p_y) ?
                pinPoint2.p_y : pinPoint1.p_y;
            rect.r_ytop = midPoint + halfWidth;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }

            rect.r_xbot = midPoint - halfWidth;
            rect.r_xtop = (pinPoint1.p_x > pinPoint2.p_x) ?
                pinPoint1.p_x : pinPoint2.p_x;
            rect.r_ybot = pinPoint1.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = midPoint - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint1.p_y > pinPoint2.p_y) {
                rect.r_ybot = pinPoint2.p_y - halfWidth;
                rect.r_ytop = pinPoint1.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint1.p_y - halfWidth;
                rect.r_ytop = pinPoint2.p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = (pinPoint1.p_x > pinPoint2.p_x) ?
                pinPoint2.p_x : pinPoint1.p_x;
            rect.r_xtop = midPoint + halfWidth;
            rect.r_ybot = pinPoint2.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromPinToDCell() : illegal choice of pins");
            return;
        }
    } else {
        if ((pinDir1 == (PinDirection) NORTH_PIN ||
            pinDir1 == (PinDirection) SOUTH_PIN) &&

            (pinDir2 == (PinDirection) NORTH_PIN ||
            pinDir2 == (PinDirection) SOUTH_PIN)) {

            if (pinPoint1.p_y > pinPoint2.p_y) {
                midPoint = pinPoint1.p_y -
                    ((pinPoint1.p_y - pinPoint2.p_y) / 2);
            } else {
                midPoint = pinPoint1.p_y +
                    ((pinPoint2.p_y - pinPoint1.p_y) / 2);
            }
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                pinPoint1.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, midPoint,
                pinPoint2.p_x, midPoint,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint2.p_x, midPoint,
                pinPoint2.p_x, pinPoint2.p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir1 == (PinDirection) WEST_PIN ||
            pinDir1 == (PinDirection) EAST_PIN) &&

            (pinDir2 == (PinDirection) WEST_PIN ||
            pinDir2 == (PinDirection) EAST_PIN)) {

            if (pinPoint1.p_x > pinPoint2.p_x) {
                midPoint = pinPoint1.p_x -
                    ((pinPoint1.p_x - pinPoint2.p_x) / 2);
            } else {
                midPoint = pinPoint1.p_x +
                    ((pinPoint2.p_x - pinPoint1.p_x) / 2);
            }
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint1.p_x, pinPoint1.p_y, 
                midPoint, pinPoint1.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                midPoint, pinPoint1.p_y, 
                midPoint, pinPoint2.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                midPoint, pinPoint2.p_y, 
                pinPoint2.p_x, pinPoint2.p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromPinToDCell() : illegal choice of pins");
            return;
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromRPinToJunct() --
 *
 * Draw the pin, a dot for the junction, and a line between the two.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *   width - use MAXWIDTH as a default value
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromRPinToJunct(drawDataPtr, rpinPtr, curJunctPtr)
DrawData        *drawDataPtr;
RoutePin        *rpinPtr;
WallJunction    *curJunctPtr;
{
    PinDirection    pinDir;
    Point           pinPoint, *dotPointPtr;
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;
    pinDir = Mac_GetDirectionOfRPin(rpinPtr);
    Mac_GetAbsPosOfRPin(rpinPtr, &pinPoint);
    dotPointPtr = Fpg_GetPointOfWallJunct(greWhichJunct, curJunctPtr);

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir == (PinDirection) NORTH_PIN) ||
            (pinDir == (PinDirection) SOUTH_PIN)) {

            rect.r_xbot = pinPoint.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint.p_y > dotPointPtr->p_y) {
                rect.r_ybot = dotPointPtr->p_y - halfWidth;
                rect.r_ytop = pinPoint.p_y;
            } else {
                rect.r_ybot = pinPoint.p_y;
                rect.r_ytop = dotPointPtr->p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            if (pinPoint.p_x > dotPointPtr->p_x) {
                rect.r_xbot = dotPointPtr->p_x;
                rect.r_xtop = pinPoint.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint.p_x - halfWidth;
                rect.r_xtop = dotPointPtr->p_x;
            }
            rect.r_ybot = dotPointPtr->p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir == (PinDirection) WEST_PIN) ||
            (pinDir == (PinDirection) EAST_PIN)) {

            if (pinPoint.p_x > dotPointPtr->p_x) {
                rect.r_xbot = dotPointPtr->p_x - halfWidth;
                rect.r_xtop = pinPoint.p_x;
            } else {
                rect.r_xbot = pinPoint.p_x;
                rect.r_xtop = dotPointPtr->p_x + halfWidth;
            }
            rect.r_ybot = pinPoint.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = dotPointPtr->p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint.p_y > dotPointPtr->p_y) {
                rect.r_ybot = dotPointPtr->p_y;
                rect.r_ytop = pinPoint.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint.p_y - halfWidth;
                rect.r_ytop = dotPointPtr->p_y;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromRPinToJunct() : illegal pin direction");
            return;
        }
    } else {
        Ez_DrawWorldSmallDot(chipWin, scrMatrix,
            dotPointPtr->p_x, dotPointPtr->p_y, color);

        if ((pinDir == (PinDirection) NORTH_PIN) ||
            (pinDir == (PinDirection) SOUTH_PIN)) {
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, pinPoint.p_y,
                pinPoint.p_x, dotPointPtr->p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, dotPointPtr->p_y, 
                dotPointPtr->p_x, dotPointPtr->p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir == (PinDirection) WEST_PIN) ||
            (pinDir == (PinDirection) EAST_PIN)) {
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, pinPoint.p_y, 
                dotPointPtr->p_x, pinPoint.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                dotPointPtr->p_x, pinPoint.p_y,
                dotPointPtr->p_x, dotPointPtr->p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromRPinToJunct() : illegal pin direction");
            return;
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromPinToJunct() --
 *
 * Draw the pin, a dot for the junction, and a line between the two.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *   width - use MAXWIDTH as a default value
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromPinToJunct(drawDataPtr, pinInstPtr, curJunctPtr)
DrawData        *drawDataPtr;
PinInst         *pinInstPtr;
WallJunction    *curJunctPtr;
{
    PinDirection    pinDir;
    Point           pinPoint, *dotPointPtr;
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;
    pinDir = Mac_GetDirectionOfPin(pinInstPtr);
    Mac_GetAbsPosOfPin(pinInstPtr, &pinPoint);
    (void) Gre_DrawPinOne(pinInstPtr, drawDataPtr);
    dotPointPtr = Fpg_GetPointOfWallJunct(greWhichJunct, curJunctPtr);

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir == (PinDirection) NORTH_PIN) ||
            (pinDir == (PinDirection) SOUTH_PIN)) {

            rect.r_xbot = pinPoint.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint.p_y > dotPointPtr->p_y) {
                rect.r_ybot = dotPointPtr->p_y - halfWidth;
                rect.r_ytop = pinPoint.p_y;
            } else {
                rect.r_ybot = pinPoint.p_y;
                rect.r_ytop = dotPointPtr->p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            if (pinPoint.p_x > dotPointPtr->p_x) {
                rect.r_xbot = dotPointPtr->p_x;
                rect.r_xtop = pinPoint.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint.p_x - halfWidth;
                rect.r_xtop = dotPointPtr->p_x;
            }
            rect.r_ybot = dotPointPtr->p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir == (PinDirection) WEST_PIN) ||
            (pinDir == (PinDirection) EAST_PIN)) {

            if (pinPoint.p_x > dotPointPtr->p_x) {
                rect.r_xbot = dotPointPtr->p_x - halfWidth;
                rect.r_xtop = pinPoint.p_x;
            } else {
                rect.r_xbot = pinPoint.p_x;
                rect.r_xtop = dotPointPtr->p_x + halfWidth;
            }
            rect.r_ybot = pinPoint.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = dotPointPtr->p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint.p_y > dotPointPtr->p_y) {
                rect.r_ybot = dotPointPtr->p_y;
                rect.r_ytop = pinPoint.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint.p_y - halfWidth;
                rect.r_ytop = dotPointPtr->p_y;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromPinToJunct() : illegal pin direction");
            return;
        }
    } else {
        Ez_DrawWorldSmallDot(chipWin, scrMatrix,
            dotPointPtr->p_x, dotPointPtr->p_y, color);
        if ((pinDir == (PinDirection) NORTH_PIN) ||
            (pinDir == (PinDirection) SOUTH_PIN)) {
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, pinPoint.p_y,
                pinPoint.p_x, dotPointPtr->p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, dotPointPtr->p_y, 
                dotPointPtr->p_x, dotPointPtr->p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir == (PinDirection) WEST_PIN) ||
            (pinDir == (PinDirection) EAST_PIN)) {
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, pinPoint.p_y, 
                dotPointPtr->p_x, pinPoint.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                dotPointPtr->p_x, pinPoint.p_y,
                dotPointPtr->p_x, dotPointPtr->p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromPinToJunct() : illegal pin direction");
            return;
        }
    /*
        Ez_DrawWorldLine(chipWin, scrMatrix,
            pinPoint.p_x, pinPoint.p_y,
            dotPointPtr->p_x, dotPointPtr->p_y,
            EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
         */
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromDCellToJunct() --
 *
 * Draw a dot for the junction, and a line between it and the center of the 
 * dcell.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *   width - use MAXWIDTH as a default value
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromDCellToJunct(drawDataPtr, pinDir, pinPoint, curJunctPtr)
DrawData        *drawDataPtr;
PinDirection    pinDir;
Point           pinPoint;
WallJunction    *curJunctPtr;
{
    Point           *dotPointPtr;
    Rect            rect;
    double          (*scrMatrix)[4];
    Window          chipWin;
    int             color, halfWidth, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;
    dotPointPtr = Fpg_GetPointOfWallJunct(greWhichJunct, curJunctPtr);

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if ((pinDir == (PinDirection) NORTH_PIN) ||
            (pinDir == (PinDirection) SOUTH_PIN)) {

            rect.r_xbot = pinPoint.p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint.p_y > dotPointPtr->p_y) {
                rect.r_ybot = dotPointPtr->p_y - halfWidth;
                rect.r_ytop = pinPoint.p_y;
            } else {
                rect.r_ybot = pinPoint.p_y;
                rect.r_ytop = dotPointPtr->p_y + halfWidth;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            if (pinPoint.p_x > dotPointPtr->p_x) {
                rect.r_xbot = dotPointPtr->p_x;
                rect.r_xtop = pinPoint.p_x + halfWidth;
            } else {
                rect.r_xbot = pinPoint.p_x - halfWidth;
                rect.r_xtop = dotPointPtr->p_x;
            }
            rect.r_ybot = dotPointPtr->p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else if ((pinDir == (PinDirection) WEST_PIN) ||
            (pinDir == (PinDirection) EAST_PIN)) {

            if (pinPoint.p_x > dotPointPtr->p_x) {
                rect.r_xbot = dotPointPtr->p_x - halfWidth;
                rect.r_xtop = pinPoint.p_x;
            } else {
                rect.r_xbot = pinPoint.p_x;
                rect.r_xtop = dotPointPtr->p_x + halfWidth;
            }
            rect.r_ybot = pinPoint.p_y - halfWidth;
            rect.r_ytop = rect.r_ybot + width;
            Ez_FillRect(chipWin, scrMatrix, &rect, color);

            rect.r_xbot = dotPointPtr->p_x - halfWidth;
            rect.r_xtop = rect.r_xbot + width;
            if (pinPoint.p_y > dotPointPtr->p_y) {
                rect.r_ybot = dotPointPtr->p_y;
                rect.r_ytop = pinPoint.p_y + halfWidth;
            } else {
                rect.r_ybot = pinPoint.p_y - halfWidth;
                rect.r_ytop = dotPointPtr->p_y;
            }
            Ez_FillRect(chipWin, scrMatrix, &rect, color);
        } else {
            Bear_PrintErr("Gre_DrawFromPinToJunct() : illegal pin direction");
            return;
        }
    } else {
        Ez_DrawWorldSmallDot(chipWin, scrMatrix,
            dotPointPtr->p_x, dotPointPtr->p_y, color);

        if ((pinDir == (PinDirection) NORTH_PIN) ||
            (pinDir == (PinDirection) SOUTH_PIN)) {
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, pinPoint.p_y,
                pinPoint.p_x, dotPointPtr->p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, dotPointPtr->p_y, 
                dotPointPtr->p_x, dotPointPtr->p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else if ((pinDir == (PinDirection) WEST_PIN) ||
            (pinDir == (PinDirection) EAST_PIN)) {
            Ez_DrawWorldLine(chipWin, scrMatrix,
                pinPoint.p_x, pinPoint.p_y, 
                dotPointPtr->p_x, pinPoint.p_y,
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                dotPointPtr->p_x, pinPoint.p_y,
                dotPointPtr->p_x, dotPointPtr->p_y, 
                EZ_SOLIDLINE2, GXcopy, color, pbChipColor);
        } else {
            Bear_PrintErr("Gre_DrawFromDCellToJunct() : illegal pin direction");
            return;
        }
    }
}




/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawFromJunctToJunct() --
 *
 * if (both 'junct1Ptr' and 'junct2Ptr' are not NULL) then
 *   Draw a dot for each junction, and a line between them.
 * else if ('junct1Ptr' is NULL) then
 *   Draw a dot for 'junct2Ptr' only.
 * end if
 * 
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *   width - use MAXWIDTH as a default value
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawFromJunctToJunct(drawDataPtr, junct1Ptr, junct2Ptr)
DrawData        *drawDataPtr;
WallJunction    *junct1Ptr, *junct2Ptr;
{
    Point   *dotPoint1Ptr, *dotPoint2Ptr;
    Rect    rect;
    double  (*scrMatrix)[4];
    Window  chipWin;
    int     color, halfWidth, width;

    chipWin = drawDataPtr->winDataPtr->wd_userWin;
    scrMatrix = drawDataPtr->winDataPtr->wd_scrMatrix;
    color = drawDataPtr->color;

    if (greUsePgWireData && ((width = drawDataPtr->width) < MAXWIDTH)) {
        halfWidth = width / 2;
        if (junct1Ptr == NIL(WallJunction)) {
            return;
        }
        dotPoint1Ptr = Fpg_GetPointOfWallJunct(greWhichJunct, junct1Ptr);
        dotPoint2Ptr = Fpg_GetPointOfWallJunct(greWhichJunct, junct2Ptr);
        if (dotPoint1Ptr->p_y == dotPoint2Ptr->p_y) {
            /* horizontal line */
            if (dotPoint1Ptr->p_x < dotPoint2Ptr->p_x) {
                rect.r_xbot = dotPoint1Ptr->p_x;
                rect.r_xtop = dotPoint2Ptr->p_x;
            } else {
                rect.r_xbot = dotPoint2Ptr->p_x;
                rect.r_xtop = dotPoint1Ptr->p_x;
            }
            rect.r_xbot -= halfWidth;
            rect.r_xtop += halfWidth;
            rect.r_ybot = dotPoint1Ptr->p_y - width / 2;
            rect.r_ytop = rect.r_ybot + width;
        } else {
            /* vertical line */
            rect.r_xbot = dotPoint1Ptr->p_x - width / 2;
            rect.r_xtop = rect.r_xbot + width;
            if (dotPoint1Ptr->p_y < dotPoint2Ptr->p_y) {
                rect.r_ybot = dotPoint1Ptr->p_y;
                rect.r_ytop = dotPoint2Ptr->p_y;
            } else {
                rect.r_ybot = dotPoint2Ptr->p_y;
                rect.r_ytop = dotPoint1Ptr->p_y;
            }
            rect.r_ybot -= halfWidth;
            rect.r_ytop += halfWidth;
        }
        Ez_FillRect (chipWin, scrMatrix, &rect, color);
    } else {
        dotPoint2Ptr = Fpg_GetPointOfWallJunct(greWhichJunct, junct2Ptr);
        Ez_DrawWorldSmallDot(chipWin, scrMatrix,
            dotPoint2Ptr->p_x, dotPoint2Ptr->p_y, color);

        if (junct1Ptr != NIL(WallJunction)) {
            dotPoint1Ptr = Fpg_GetPointOfWallJunct(greWhichJunct, junct1Ptr);
            Ez_DrawWorldLine(chipWin, scrMatrix,
                dotPoint1Ptr->p_x, dotPoint1Ptr->p_y,
                dotPoint2Ptr->p_x, dotPoint2Ptr->p_y,
                EZ_SOLIDLINE2, GXcopy, drawDataPtr->color, pbChipColor);
            Ez_DrawWorldSmallDot(chipWin, scrMatrix,
                dotPoint1Ptr->p_x, dotPoint1Ptr->p_y, color);
        }
    }
}

#endif BEAR_BLACK_BOX


