
/* greDrawPins.c --
 *
 *     George Carvalho
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifndef BEAR_BLACK_BOX

#include "bearCopyRight.h"
#include <stdio.h>
#include <signal.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "grEditor.h"
#include "grEditorInt.h"

#include "../polarBear/polarBear.h"

/* ---------- Local Procedures ---------- */
BearStatus CalcWhereToDrawPin();

/* ---------- Local Global Variables ---------- */
static int scrPinWidth = 6;  /* each pin is a square 6 pixels wide */

static BearStatus
CalcWhereToDrawPin(pinInstPtr, winDataPtr, pointPtr)
PinInst     *pinInstPtr;
WindowData  *winDataPtr;
Point       *pointPtr;          
/*
 * Calculate the upper left hand corner of the area on the screen that
 * will represent the pin.
 */
{
    Point           absPos;             /* World coordinates of the pin's
                                         * absolute position on the chip.
                                         */
    int             sx, sy;
    register double (*scrMatrix)[4];

    Mac_GetAbsPosOfPin(pinInstPtr, &absPos);
    scrMatrix = winDataPtr->wd_scrMatrix;
    sx = Ez_XTrans(scrMatrix, absPos.p_x, absPos.p_y);
    sy = Ez_YTrans(scrMatrix, absPos.p_x, absPos.p_y);

    if (Mac_GetKindOfPin(pinInstPtr) == (PinKind) MAC_BORDER_PIN) {
        switch (Mac_GetDirectionOfPin(pinInstPtr)) {
        case (PinDirection) NORTH_PIN:
            sx -= (scrPinWidth / 2);
            sy -= scrPinWidth;
            break;
        case (PinDirection) EAST_PIN:
            sy -= (scrPinWidth / 2);
            break;
        case (PinDirection) SOUTH_PIN:
            sx -= (scrPinWidth / 2);
            break;
        case (PinDirection) WEST_PIN:
            sx -= scrPinWidth;
            sy -= (scrPinWidth / 2);
            break;
        default:
            Bear_PrintErr("CalcWhereToDrawPin(): illegal pin direction");
            return (BEAR_ERROR);
            break;
        }
    } else {
        switch (Mac_GetDirectionOfPin(pinInstPtr)) {
        case (PinDirection) NORTH_PIN:
            sx -= (scrPinWidth / 2);
            break;
        case (PinDirection) EAST_PIN:
            sx -= scrPinWidth;
            sy -= (scrPinWidth / 2);
            break;
        case (PinDirection) SOUTH_PIN:
            sx -= (scrPinWidth / 2);
            sy -= scrPinWidth;
            break;
        case (PinDirection) WEST_PIN:
            sy -= (scrPinWidth / 2);
            break;
        default:
            Bear_PrintErr("CalcWhereToDrawPin(): illegal pin direction");
            return (BEAR_ERROR);
            break;
        }
    }

    pointPtr->p_x = sx;
    pointPtr->p_y = sy;

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawNode() --
 *
 * This procedure is designed to be used by an "iterater".
 * It draws the given node (which is either a pin or a wall junction) in
 * the window if its a chip window.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawNode(winDataPtr, drawNodePtr)
WindowData      *winDataPtr;
DrawNode        *drawNodePtr;
{
    DrawData drawData;

    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    drawData.winDataPtr = winDataPtr;
    drawData.width = MAXWIDTH;

    if (drawNodePtr->pinOrJunctPtr->unType == FPG_WALL_JUNCT) {
        drawData.color = drawNodePtr->junctColor;
        (void) Gre_DrawWallJunct(drawNodePtr->pinOrJunctPtr->unValue.junctPtr,
            &drawData);
    }
    else if (drawNodePtr->pinOrJunctPtr->unType == FPG_PIN) {
        drawData.color = drawNodePtr->pinColor;
        (void) Gre_DrawPinOne(drawNodePtr->pinOrJunctPtr->unValue.pinPtr,
            &drawData);
    }

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawAllPins() --
 *
 * For each cell instance it goes around the boundary and
 * draws each poop poop a doop pin instance with the user
 * specified pin color.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Gre_DrawAllPins(drawDataPtr)
DrawData *drawDataPtr;
{
    CellMaster *parentCellPtr;
    ChipWin    *chipWinPtr;

    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(drawDataPtr->winDataPtr) == FALSE) {
        return;
    }

    chipWinPtr = PbGetChipWin(drawDataPtr->winDataPtr);
    parentCellPtr = PbGetCellMasterOfCHW(chipWinPtr);

    /* Draw pins of the core cell. */
    (void) Gre_DrawPinsOfCell(Mac_GetInstOfCoreMaster(parentCellPtr),
        drawDataPtr);

    /* Go through each cell instance of the parent cell. */
    (void) Mac_ItrChildrenOfCell(parentCellPtr,
        Gre_DrawPinsOfCell, (ClientData) drawDataPtr);
    Ez_Flush();
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawPinOne() --
 *
 * This procedure is designed to be used by an "iterater".
 * It draws the given pin in the given chip window.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, otherwise.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawPinOne(pinInstPtr, drawDataPtr)
PinInst     *pinInstPtr;
DrawData    *drawDataPtr;
{
    Point   point;
    Rect    rect;

    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(drawDataPtr->winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (Mac_IsPinPosDef(pinInstPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (CalcWhereToDrawPin(pinInstPtr,
        drawDataPtr->winDataPtr, &point) == BEAR_ERROR) {
        return (BEAR_STOP_SEARCH);
    }
    rect.r_xbot = point.p_x;
    rect.r_ytop = point.p_y;
    rect.r_xtop = point.p_x + scrPinWidth;
    rect.r_ybot = point.p_y + scrPinWidth;
    Ez_FillScrRect(drawDataPtr->winDataPtr->wd_userWin, &rect,
        drawDataPtr->color);

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawPinTwo() --
 *
 * This procedure is designed to be used by an "iterater".
 * It draws the given pin in the given chip window.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion, even if theres an
 *   error because the iterater should continue on to the next window.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawPinTwo(winDataPtr, drawPinPtr)
WindowData  *winDataPtr;
DrawPin     *drawPinPtr;
{
    DrawData drawData;

    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    drawData.winDataPtr = winDataPtr;
    drawData.color = drawPinPtr->pinColor;
    (void) Gre_DrawPinOne(drawPinPtr->pinInstPtr, &drawData);

    return (BEAR_CONTINUE_SEARCH);
}



BearStatus
Gre_ErasePin(pinInstPtr, drawDataPtr)
PinInst     *pinInstPtr;
DrawData    *drawDataPtr;   /* The following fields must be filled in:
                             *     winDataPtr
                             */
/*
 * Erase an area that is three times the size of the pin's area when
 * it was drawn.
 */
{
    Rect            rect;       /* World coordinates of the area to be
                                 * erased.
                                 */
    Point           point;
    int             sx, sy, width;
    register double (*invScrMatrix)[4];

    if (CalcWhereToDrawPin(pinInstPtr, drawDataPtr->winDataPtr, &point) == 
        BEAR_ERROR) {
        return (BEAR_STOP_SEARCH);
    }
    invScrMatrix = drawDataPtr->winDataPtr->wd_invScrMatrix;

    sx = point.p_x - scrPinWidth;
    sy = point.p_y - scrPinWidth;
    rect.r_xbot = Ez_XTrans(invScrMatrix, sx, sy);
    rect.r_ytop = Ez_YTrans(invScrMatrix, sx, sy);

    width = 3 * scrPinWidth;
    sx += width;
    sy += width;
    rect.r_xtop = Ez_XTrans(invScrMatrix, sx, sy);
    rect.r_ybot = Ez_YTrans(invScrMatrix, sx, sy);

    (*drawDataPtr->winDataPtr->wd_redrawProc)(drawDataPtr->winDataPtr, &rect);

/*
#ifdef DEBUG
{
    DrawData drawData;

    drawData.winDataPtr = drawDataPtr->winDataPtr;
    Ez_FillRect(drawData.winDataPtr->wd_userWin,
        drawData.winDataPtr->wd_scrMatrix, &rect, WhitePixel);

    drawData.color = BlackPixel;
    (void) Gre_DrawPinOne(pinInstPtr, &drawData);
}
#endif DEBUG
*/

    return (BEAR_CONTINUE_SEARCH);
}



BearStatus
Gre_DrawPinsOfCell(cellInstPtr, drawDataPtr)
CellInst    *cellInstPtr;
DrawData    *drawDataPtr;
{
    GenericCell genCell;
    Rect        bbox;

    genCell.unType = CELL_INST;
    genCell.unValue.cellInstPtr = cellInstPtr;
    GeoTransRect(Mac_GetTransformOfGCell(&genCell),
        Mac_GetBBoxOfCell(cellInstPtr), &bbox);
    if (GEO_TOUCH(&bbox, &drawDataPtr->winDataPtr->wd_imageBnd)) {
        /* Go through each pin of the cell instance. */
        (void) Mac_ItrPinOfCellInst(cellInstPtr, Gre_DrawPinOne,
            (ClientData) drawDataPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}



BearStatus
Gre_DrawPinsOfNet(netPtr, drawDataPtr)
Net         *netPtr;
DrawData    *drawDataPtr;
{
    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(drawDataPtr->winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (netPtr != NIL(Net)) {
        (void) Mac_ItrPortOfNet(netPtr, Gre_DrawPinsOfPortOne, 
            (ClientData) drawDataPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawPinsOfPortOne() --
 *
 * This procedure is designed to be used by an "iterater".
 * It draws all the pins of the given port in the given chip window.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, otherwise.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawPinsOfPortOne(portInstPtr, drawDataPtr)
PortInst    *portInstPtr;
DrawData    *drawDataPtr;
{
    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(drawDataPtr->winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (Mac_ItrPinOfPortInst(portInstPtr, Gre_DrawPinOne,
        (ClientData) drawDataPtr) == BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawPinsOfPortTwo() --
 *
 * This procedure is designed to be used by an "iterater".
 * It draws all the pins of the given port in the given chip window.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion, even if theres an
 *   error because the iterater should continue on to the next window.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawPinsOfPortTwo(winDataPtr, drawPortPtr)
WindowData  *winDataPtr;
DrawPort    *drawPortPtr;
{
    DrawData drawData;

    /*
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    drawData.winDataPtr = winDataPtr;
    drawData.color = drawPortPtr->portColor;
    (void) Gre_DrawPinsOfPortOne(drawPortPtr->portInstPtr, &drawData);

    return (BEAR_CONTINUE_SEARCH);
}

#endif BEAR_BLACK_BOX


