
/* greEdgeToDataBase.c --
 *
 *     Malgorzata Marek-Sadowska
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "grEditor.h"
#include "grEditorInt.h"
#include "grStorage.h"


/*
 * Here is the function which puts the information about an edge
 * into the Mac database. All cases are considered here.
 */


/* ARGSUSED */
BearStatus
grEd_InsertEdgeOfGraph(edge, clientData)

Gt_GraphEdge    edge;
ClientData      clientData;

{

    BottleNeck      *bnPtr;
    PinOrJunct      *pOrJ1Ptr,*pOrJ2Ptr;
    WallSegment     *segPtr,*cover;
    Point           a,b;
    PinInst         *pin1Ptr,*pin2Ptr;
    WallJunction    *j1Ptr,*j2Ptr,*tmpJunctPtr;
    Tile            *bnTilePtr, *domTile1, *domTile2;
    BNLine          *bnLinePtr;
    grEd_Btn        *grebtn;
    grEd_BtnLine    *grebtnLine;
    grEd_Node       *greJunct1,*greJunct2;
    Rect            rect,rect2;
    Mac_EdgeData    *edgeDataPtr;
    Gt_GraphNode    node1, node2;

    node1 = Gt_GetNodeOneOfEdge(edge);
    node2 = Gt_GetNodeTwoOfEdge(edge);

    pOrJ1Ptr = (PinOrJunct *) Gt_GetNodeDataOfNode(node1);
    pOrJ2Ptr = (PinOrJunct *) Gt_GetNodeDataOfNode(node2);
    
    edgeDataPtr = (Mac_EdgeData *) Gt_GetEdgeDataOfEdge(edge);

    pin1Ptr = NIL(PinInst);
    pin2Ptr = NIL(PinInst);
    j1Ptr = NIL(WallJunction);
    j2Ptr = NIL(WallJunction);

    switch (pOrJ1Ptr->unType) {
    case (int) FPG_PIN:
        pin1Ptr = pOrJ1Ptr->unValue.pinPtr;
        break;
    case (int) FPG_WALL_JUNCT:
        j1Ptr = pOrJ1Ptr->unValue.junctPtr;
        break;
    default:
Bear_PrintErr("grEd_InsertEdgeOfGraph() : (pj 1) unexpected pin or junction type");
        return (BEAR_STOP_SEARCH);
    }

    switch (pOrJ2Ptr->unType) {
    case (int) FPG_PIN:
        pin2Ptr = pOrJ2Ptr->unValue.pinPtr;
        break;
    case (int) FPG_WALL_JUNCT:
        j2Ptr = pOrJ2Ptr->unValue.junctPtr;
        break;
    default:
Bear_PrintErr("grEd_InsertEdgeOfGraph() : (pj 2) unexpected pin or junction type");
        return (BEAR_STOP_SEARCH);
    }

    if ((j1Ptr != NIL(WallJunction)) && (j2Ptr != NIL(WallJunction))) {
    /* both nodes of the inserted edge are wall junctions */

    /*
    printf ("\n  both nodes of the inserted edge are wall junctions");
    */

    if (Fpg_FindWallSeg(j1Ptr, j2Ptr, &segPtr) == BEAR_ERROR) {
        Bear_PrintErr("\n junctions do not correspond to any wall segment");
        return (BEAR_STOP_SEARCH);
    }

    bnPtr = Fpg_GetBottleNeckOfWallSeg(segPtr);
    if (bnPtr == NIL(BottleNeck)) {
        Bear_PrintErr("\n WallSegment has no bottleNeck assigned");
        return  (BEAR_STOP_SEARCH);
    }
    edgeDataPtr->bnPtr = bnPtr;

    if (segPtr->direction == FPG_HORIZONTAL) {
        /* segment is horizontal */

    /*
        printf ("\n  segment is horizontal ");
    */

        tmpJunctPtr = Fpg_GetWestJunctOfWallJunct(j1Ptr);
        if (tmpJunctPtr == j2Ptr) {
            tmpJunctPtr = j1Ptr;
            j1Ptr = j2Ptr;
            j2Ptr = tmpJunctPtr;
        }
    /*
        printf ("\n end horz segment ");
    */
    } /* end horz segment */
    else {
        /* segment is vertical */

    /*
        printf ("\n segment is vertical");
    */

        tmpJunctPtr = Fpg_GetNorthJunctOfWallJunct(j1Ptr);
        if (tmpJunctPtr == j2Ptr) {
            tmpJunctPtr = j1Ptr;
            j1Ptr = j2Ptr;
            j2Ptr = tmpJunctPtr;
        }
    /*
        printf ("\n end vert segment");
    */
    } /* end vert segment */

    if (bnPtr->unType == TP_BN_TILE) {
        /* bottleneck intersects with the segment */

    /*
        printf ("\n bottleneck intersects with the segment");
    */

    bnTilePtr=bnPtr->unValue.bnTilePtr;
    /* check if the net already has ppins*/
    grebtn = grEd_FindEd_Btn(bnTilePtr);
    if (grebtn == NIL(grEd_Btn))
    {
        Bear_PrintErr("\n grebtn not found");
        return  (BEAR_STOP_SEARCH);
    }


    /* insert left or top pseudo pin */

    /*
    printf ("\n insert left or top pseudo pin");
    */

    greJunct1 = grEd_FindEd_Node(j1Ptr);

    edgeDataPtr->ppin1Ptr =
        grEd_InsertPPinToLeftTopSide(bnTilePtr,grebtn->name,greJunct1->name);

    /* insert right or bot pseudo pin */

    /*
    printf ("\n insert right or bot pseudo pin");
    */

    greJunct2 = grEd_FindEd_Node(j2Ptr);


    edgeDataPtr->ppin2Ptr =
        grEd_InsertPPinToRightBotSide(bnTilePtr,grebtn->name,greJunct2->name);

    /*
    printf ("\n end bottle neck");
    */

    } /* end bottle neck */
    else {
    /* btnLine intersects with segment */

    /*
    printf ("\n  btnLine intersects with segment");
    */

    bnLinePtr=bnPtr->unValue.bnLinePtr;
    grebtnLine=grEd_FindEd_BtnLine(bnLinePtr);
    if (grebtnLine == NIL(grEd_BtnLine))
    {
        Bear_PrintErr("\n grebtnLine not found");
        return (BEAR_STOP_SEARCH);
    }

    /*
        printf ("\n ppins need to be inerted");
    */
        greJunct1 =grEd_FindEd_Node(j1Ptr);
        greJunct2 =grEd_FindEd_Node(j2Ptr);

        edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToBtnLine(bnLinePtr,greJunct1->name,greJunct2->name,
            grebtnLine->tilePtr);

        edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
    /*
    printf ("\n end intersection with btn line"); 
    */

    } /* end intersection with btn line */

    /*
        printf ("\n end two junctions");
    */

        return (BEAR_CONTINUE_SEARCH);
    } /* end two junctions */

    if (((j1Ptr!=NIL(WallJunction))&&(pin2Ptr!= NIL(PinInst))) ||
        ((j2Ptr!= NIL(WallJunction))&&(pin1Ptr!=NIL(PinInst)))) {
        /* junction to pin edge */

    /*
        printf ("\n junction to pin edge");
    */

        if (j1Ptr == NIL(WallJunction)) {
            j1Ptr=j2Ptr;
        }
        if (pin1Ptr == NIL(PinInst)) {
            pin1Ptr=pin2Ptr;
        }

        greJunct1=grEd_FindEd_Node(j1Ptr);

        if (grEd_GetContainingPinDomTile(pin1Ptr)==NULL) {
            cover=grEd_GetCoveringWallSegment(pin1Ptr);
            if (cover == NULL) {
    /*
            printf ("\n cover not found");
    */
            }
    }
    else {

    /*
    printf ("\n pin is covered by a dominant tile");
    */

        domTile1= grEd_GetContainingPinDomTile(pin1Ptr);
        domTile2= grEd_GetPtrOfDomTileCoveringPin(pin1Ptr);
        if ((Tp_IsHorzTile(domTile1))&&(Tp_IsVertTile(domTile2)))
        {
            j2Ptr = Fpg_GetWallJunctWhereDomTilesCross(greFpgPtr, 
                domTile1, domTile2);
        }
        else
        {
            j2Ptr = Fpg_GetWallJunctWhereDomTilesCross(greFpgPtr, 
                domTile2, domTile1);
        }
        if (j1Ptr!=j2Ptr)
        {
        /*
            printf ("\n j1ptr!= j2Ptr");
        */
            Fpg_FindWallSeg(j1Ptr,j2Ptr,&cover);
            if (cover == NULL)
            {
        /*
            printf ("\n cover is NULL");
        */
            }
        }
        else
        {
        /* pin just needs external name*/

    /*
        printf ("\n pin just needs external name");
    */

        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        return (BEAR_CONTINUE_SEARCH);
        }
    }

    bnPtr=Fpg_GetBottleNeckOfWallSeg(cover);

    if (bnPtr == NIL(BottleNeck))
    {
        Bear_PrintErr("\n WallSegment has no BottleNeck assigned");
        return (BEAR_STOP_SEARCH);
    }

    edgeDataPtr->bnPtr = bnPtr;

    Mac_GetAbsPosOfPin(pin1Ptr,&a);

    if (bnPtr->unType == TP_BN_TILE)
    {
    /* wall segment intersects with a bottleneck tile */

    /*
    printf ("\n wall segment intersects with a bottleneck tile");
    */


    bnTilePtr=bnPtr->unValue.bnTilePtr;
    grebtn=grEd_FindEd_Btn(bnTilePtr);

    if (Tp_IsVertTile(bnTilePtr))
    {
    /* btn tile is vertical, pins are at horz walls */

    /*
    printf ("\n btn tile is vertical, pins are at horz walls");
    */

    Tp_VertTileToRect(grePlaneDataPtr,bnTilePtr,&rect);
    if ((a.p_x >= rect.r_xbot)&&(a.p_x <= rect.r_xtop))
    {
    /* pin is covered by a btn */

    /*
    printf ("\n pin is covered by a btn ");
    */

        j2Ptr = Fpg_GetNWJunctOfWallSeg(cover);
        if (j1Ptr==j2Ptr)
        {
            /* crossing through left side of btn tile */

        /*
            printf ("\n crossing through left side of btn tile");
        */

            edgeDataPtr->ppin1Ptr = grEd_InsertPPinToLeftTopSide(bnTilePtr,
                grebtn->name, greJunct1->name);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }
        else
        {
            /*crossing through right side of btn tile */

        /*
            printf ("\n crossing through right side of btn tile");
        */

            greJunct2=grEd_FindEd_Node(j2Ptr);

            /* below I made a change 1 was 2 , March 25 1988 */
            edgeDataPtr->ppin1Ptr = grEd_InsertPPinToRightBotSide(bnTilePtr, 
                grebtn->name, greJunct1->name);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }

        Mac_PutExtNetIdOfPin(grebtn->name,pin1Ptr);
    /*
        printf ("\n end pin covered by btn");
    */
        return (BEAR_CONTINUE_SEARCH);

    } /* end pin covered by btn */
    /*
    printf ("\n end vert tile"); 
    */
    } /* end vert tile */
    else
    {
    /* btn tile is horizontal */
    /*
    printf ("\n btn tile is horizontal");
    */
    Tp_HorzTileToRect(grePlaneDataPtr,bnTilePtr,&rect);

    if ((a.p_y >=rect.r_ybot)&&(a.p_y <= rect.r_ytop))
    {
    /* pin is covered by btn */
    /*
    printf ("\n pin is covered by btn");
    */
        j2Ptr=Fpg_GetNWJunctOfWallSeg(cover);
        if (j1Ptr==j2Ptr)
        {
            /* crossing through top side of btn tile */
        /*
            printf ("\n crossing through top side of btn tile");
        */

            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToLeftTopSide(bnTilePtr,grebtn->name,
            greJunct1->name);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }
        else
        {
            /*crossing through bot side of btn tile */
        /*
            printf ("\n crossing through bot side of btn tile");
        */
            greJunct2=grEd_FindEd_Node(j2Ptr);

            /* below I made a change 1 was 2 , March 25 1988 */
            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToRightBotSide(bnTilePtr,grebtn->name,
            greJunct1->name);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }

        Mac_PutExtNetIdOfPin(grebtn->name,pin1Ptr);
    /*
        printf ("\n end pin covered by btn");
    */
        return (BEAR_CONTINUE_SEARCH);
    } /* end pin covered by btn */
    /*
    printf ("\n  end horz tile");
    */
    } /* end horz tile */
    /* end pin inside a btn */
    /*
    printf ("\n end pin inside a btn");
    */

    /* 
     * If we are still in the game then the pin is not covered by the btn.
     */

    /*
    printf ("\n if we are still in the game then the pin is not covered by the btn");
    */

    if (Tp_IsVertTile(bnTilePtr))
    {

    /*
    printf ("\n vertical bottleneck");
    */
    Tp_VertTileToRect(grePlaneDataPtr,bnTilePtr,&rect);

    j2Ptr=Fpg_GetNWJunctOfWallSeg(cover);

    if (a.p_x < rect.r_xbot)
    {
    /* pin is on the left side of the btn */
    /*
    printf ("\n pin is on the left side of the btn");
    */
    if (j1Ptr==j2Ptr)
    {
    /* left junction */
    /*
    printf ("\n left junction");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
    }
    else
    {
    /* right junction and full intersection with btn */
    /*
    printf ("\n right junction and full intersection with btn");
    */

        edgeDataPtr->ppin1Ptr = grEd_InsertPPinToLeftTopSide(bnTilePtr,
            grebtn->name, greJunct1->name);
        greJunct2=grEd_FindEd_Node(j2Ptr);

        edgeDataPtr->ppin2Ptr = grEd_InsertPPinToRightBotSide(bnTilePtr,
            grebtn->name,greJunct2->name);

        Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
    }
    /*
    printf ("\n end pin on the left side of btn");
    */
    return ( BEAR_CONTINUE_SEARCH);
    } /* end pin on the left side of btn */
    else
    {
    /* pin is on the right side of the btn */
    /*
    printf ("\n pin is on the right side of the btn");
    */
        if (j1Ptr==j2Ptr)
        {
            /* left junction, full intersection with the bottleneck */
    /*
    printf ("\n left junction, full intersection with the bottleneck");
    */

            edgeDataPtr->ppin1Ptr = grEd_InsertPPinToLeftTopSide(bnTilePtr,
                grebtn->name, greJunct1->name);
            greJunct2=grEd_FindEd_Node(Fpg_GetSEJunctOfWallSeg(cover));
        
            edgeDataPtr->ppin2Ptr = grEd_InsertPPinToRightBotSide(bnTilePtr,
                grebtn->name,greJunct2->name);

            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
    /*
    printf ("\n end left junction");
    */
    } /* end left junction */
    else
    {
        greJunct2=grEd_FindEd_Node(j2Ptr);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
    }
    /*
    printf ("\n end right side of the btn");
    printf ("\n  end vert tile ");
    */
    return (BEAR_CONTINUE_SEARCH);
    } /* end right side of the btn */
    } /* end vert tile */
    else
    {
        /* is horizontal tile */
    /*
        printf ("\n is horizontal tile");
    */
        j2Ptr=Fpg_GetNWJunctOfWallSeg(cover);
        Tp_HorzTileToRect(grePlaneDataPtr,bnTilePtr,&rect);
        if (a.p_y > rect.r_ytop)
        {
        /* pin at the top of btn */
    /*
        printf ("\n  pin at the top of btn");
    */

        if (j1Ptr==j2Ptr)
        {
        /* top junction */
    /*
        printf ("\n top junction");
    */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        }
        else
        {
            /* bot junction and full intersection with btn */
        /*
            printf ("\n bot junction and full intersection with btn");
        */

            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToLeftTopSide(bnTilePtr,grebtn->name,
            greJunct1->name);

            greJunct2=grEd_FindEd_Node(j2Ptr);

            edgeDataPtr->ppin2Ptr = grEd_InsertPPinToRightBotSide(bnTilePtr,
                grebtn->name, greJunct2->name);

            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
        }
    /*
        printf ("\n  end pin on the top side of btn");
    */
        return ( BEAR_CONTINUE_SEARCH);
        } /* end pin on the top side of btn */
        else
        {
        /* pin is at the bottom of the junction */
    /*
        printf ("\n pin is at the bottom of the junction");
    */
        if (j1Ptr==j2Ptr)
        {
            /* top junction, full intersection with the bottleneck */
    /*
            printf ("\n top junction, full intersection with the bottleneck");
    */

            edgeDataPtr->ppin1Ptr = grEd_InsertPPinToLeftTopSide(bnTilePtr,
                grebtn->name, greJunct1->name);
            greJunct2=grEd_FindEd_Node(Fpg_GetSEJunctOfWallSeg(cover));

            edgeDataPtr->ppin2Ptr = grEd_InsertPPinToRightBotSide(bnTilePtr,
                grebtn->name,greJunct2->name);

            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
    /*
            printf ("\n end top junction");
    */
        } /* end top junction */
        else
        {
            /* bottom junction */
        /*
            printf ("\n  bottom junction");
        */
            greJunct2=grEd_FindEd_Node(j2Ptr);
            /*  HERE CHANGE 2 to 1 */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
    /*
        printf ("\n  end bottom junction");
    */
        }  /* end bottom junction */
    /*
        printf ("\n end bottom side of the btn");
    */
    /*
        printf ("\n end bottle neck tile");
    */
        return (BEAR_CONTINUE_SEARCH);
        } /* end bottom side of the btn */
    }/* end horz bottleneck */
    } /* end bottle neck tile */
    else
    {
    /* bottle neck line */
    /*
    printf ("\n bottle neck line ");
    */
    bnLinePtr=bnPtr->unValue.bnLinePtr;
    grebtnLine=grEd_FindEd_BtnLine(bnLinePtr);
    j2Ptr=Fpg_GetNWJunctOfWallSeg(cover);
    if (cover->direction == (SegmentDir) FPG_HORIZONTAL)
    { /* vert bnLine */
    /*
    printf ("\n vert bnLine");
    */
    if (a.p_x < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x)
    {
    /* pin sits on the left of btn line */
    /*
    printf ("\n  pin sits on the left of btn line");
    */
        if (j2Ptr==j1Ptr)
        {
        /* no intersection */
    /*
        printf ("\n no intersection");
    */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        }
        else
        {
        /*
            printf ("\n intersection");
        */
                greJunct2=grEd_FindEd_Node(j2Ptr);
            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);

                            edgeDataPtr->ppin1Ptr =
                grEd_InsertPPinToBtnLine(bnLinePtr,greJunct2->name,
                greJunct1->name,grebtnLine->tilePtr);
                            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }
    /*
    printf ("\n   end pin on the left side of btn line");
    */
        return (BEAR_CONTINUE_SEARCH);
    }  /* end pin on the left side of btn line */
    else
    {
    /* pin sits on the right of btn line */
    /*
    printf ("\n pin sits on the right of btn line");
    */
        if (j2Ptr != j1Ptr)
        {
        /* no intersection */
    /*
        printf ("\n no intersection");
    */
        greJunct2=grEd_FindEd_Node(j2Ptr);
        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        }
        else
        {
        /* intersection */
    /*
        printf ("\n intersection");
    */
        j2Ptr=Fpg_GetSEJunctOfWallSeg(cover);
        greJunct2=grEd_FindEd_Node(j2Ptr);
        /*
            printf ("\n insert");
        */

            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToBtnLine(bnLinePtr,greJunct1->name,
            greJunct2->name,grebtnLine->tilePtr);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
        }
    /*
    printf ("\n end pin sits on the right side of btn");
    */
    } /* end pin sits on the right side of btn */
    /*
    printf ("\n end vert btn line"); 
    */
    return (BEAR_CONTINUE_SEARCH);
    } /* end vert btn line */
    else
    {
    /* btn line is horizontal */
    /*
    printf ("\n btn line is horizontal");
    */
    if (a.p_y > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y)
    {
    /* pin is at the top of btn line */
    /*
    printf ("\n pin is at the top of btn line");
    */
        if (j2Ptr==j1Ptr)
        {
        /* no intersection */
    /*
        printf ("\n no intersection");
    */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        }
        else
        {
        /*
            printf ("\n intersection");
        */
                greJunct2=grEd_FindEd_Node(j2Ptr);
            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
        /*
                printf ("\n insertion");
        */

                edgeDataPtr->ppin1Ptr =
                grEd_InsertPPinToBtnLine(bnLinePtr,greJunct2->name,
                greJunct1->name,grebtnLine->tilePtr);
                edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }
    /*
        printf ("\n  end pin on the top side of btn line");
    */
        return (BEAR_CONTINUE_SEARCH);
    }  /* end pin on the top side of btn line */
    else
    {
    /* pin sits on the bot of btn line */
    /*
    printf ("\n pin sits on the bot of btn line");
    */
        if (j2Ptr != j1Ptr)
        {
        /* no intersection */
    /*
        printf ("\n no intersection"); 
    */
        greJunct2=grEd_FindEd_Node(j2Ptr);
        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        }
        else
        {
        /* intersection */
    /*
        printf ("\n intersection");
    */
        j2Ptr=Fpg_GetSEJunctOfWallSeg(cover);
        greJunct2=grEd_FindEd_Node(j2Ptr);
        /*
            printf ("\n insertion");
        */

            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToBtnLine(bnLinePtr,greJunct1->name,
            greJunct2->name,grebtnLine->tilePtr);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
        }
    /*
    printf ("\n end pin sits on the bot side of btn");
    */
    } /* end pin sits on the bot side of btn */
    /*
    printf ("\n end horz line");
    */
    return (BEAR_CONTINUE_SEARCH);
    } /* end horz line */
    } /* end btn line */
    } /* end junction and pin case */

    /* If we are here it means that we have two pins */
    /*
    printf ("\n If we are here it means that we have two pins");
    */

    if ((pin1Ptr!=NULL)&&(pin2Ptr!=NULL))
    {

    if (grEd_GetContainingPinDomTile(pin1Ptr)==NULL)
    {
    /*
        printf ("\n pin1Ptr is not NULL");
    */
        cover=grEd_GetCoveringWallSegment(pin1Ptr);
        if (cover == NULL)
        {
    /*
        printf ("\n pin1 not in dom tile");
    */
    /*
        printf ("\n search for cover failed");
    */
        }
        else
        {
    /*
        printf ("\n cover * found");
    */
        }
    }
    else
    {
    if (grEd_GetContainingPinDomTile(pin2Ptr)==NULL)
    {
    /*
        printf ("\n pin2Ptr is not NULL");
    */
        cover=grEd_GetCoveringWallSegment(pin2Ptr);
        if (cover == NULL)
        {
    /*
        printf ("\n pin2 not in dom tile");
        printf ("\n search for cover failed");
    */
        }
    }
    else
    {
        domTile1= grEd_GetContainingPinDomTile(pin1Ptr);
        domTile2= grEd_GetPtrOfDomTileCoveringPin(pin1Ptr);
    /*
        printf ("\n both pins inside dom tiles");
    */
        if ((Tp_IsHorzTile(domTile1))&&(Tp_IsVertTile(domTile2)))
        {
            j1Ptr = Fpg_GetWallJunctWhereDomTilesCross(greFpgPtr, 
                domTile1, domTile2);
        }
        else
        {
            j1Ptr = Fpg_GetWallJunctWhereDomTilesCross(greFpgPtr, 
                domTile2, domTile1);
        }
        domTile1= grEd_GetContainingPinDomTile(pin2Ptr);
        domTile2= grEd_GetPtrOfDomTileCoveringPin(pin2Ptr);
        if ((Tp_IsHorzTile(domTile1))&&(Tp_IsVertTile(domTile2)))
        {
            j2Ptr = Fpg_GetWallJunctWhereDomTilesCross(greFpgPtr, 
                domTile1, domTile2);
        }
        else
        {
            j2Ptr = Fpg_GetWallJunctWhereDomTilesCross(greFpgPtr, 
                domTile2, domTile1);
        }
        if (j1Ptr!=j2Ptr)
        {
        /*
            printf ("\n jp1Ptr!= j2Ptr");
        */
            Fpg_FindWallSeg(j1Ptr,j2Ptr,&cover);
            if (cover == NULL)
            {
        /*
            printf ("\n cover failed ");
        */
            }
        }
        else
        {
        /* pins just need external name*/
    /*
        printf ("\n pins just need external name");
    */

        greJunct1=grEd_FindEd_Node(j1Ptr);
        greJunct2=grEd_FindEd_Node(j2Ptr);

        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin2Ptr);
        
        return (BEAR_CONTINUE_SEARCH);
        }
    }
    }
    /*
    printf ("\n calculating data");
    */
    bnPtr=Fpg_GetBottleNeckOfWallSeg(cover);
    edgeDataPtr->bnPtr = bnPtr;
    Mac_GetAbsPosOfPin(pin1Ptr,&a);
    Mac_GetAbsPosOfPin(pin2Ptr,&b);
    /*
    printf ("\n a_pos: x= %4d y=%4d", a.p_x,a.p_y);
    printf ("\n b_pos: x= %4d y=%4d", b.p_x,b.p_y);
    */
    j1Ptr=Fpg_GetNWJunctOfWallSeg(cover);
    j2Ptr=Fpg_GetSEJunctOfWallSeg(cover);
    if (j1Ptr == NULL)
    {
    /*
    printf ("\n j1Ptr is NULL");
    */
    }
    else
    {
    /*
    printf ("\n j1Ptr found");
    */
    }
    if (j2Ptr == NULL)
    {
    /*
    printf ("\n j2Ptr is NULL");
    */
    }
    else
    {
    /*
    printf ("\n j2Ptr found");
    */
    }
    greJunct1=grEd_FindEd_Node(j1Ptr);
    greJunct2=grEd_FindEd_Node(j2Ptr);

    if (bnPtr->unType == TP_BN_TILE)
    {
        /* bottleneck tile case */
    /*
        printf ("\n bottleneck tile case"); 
    */
        bnTilePtr=bnPtr->unValue.bnTilePtr;
        grebtn=grEd_FindEd_Btn(bnTilePtr);

        if (Tp_IsVertTile(bnTilePtr))
        {
        Tp_VertTileToRect(grePlaneDataPtr,bnTilePtr,&rect);
        }
        else
        {
        Tp_HorzTileToRect(grePlaneDataPtr,bnTilePtr,&rect2);
        }

        if (((Tp_IsVertTile(bnTilePtr))&&(a.p_x >= rect.r_xbot)&&(a.p_x
        <= rect.r_xtop)&&(b.p_x>=rect.r_xbot)&&(b.p_x <= rect.r_xtop))||
        ((Tp_IsHorzTile(bnTilePtr))&&(a.p_y >=rect2.r_ybot)&&(a.p_y <=
        rect2.r_ytop)&&(b.p_y >=rect2.r_ybot)&&(b.p_y <= rect2.r_ytop)))
        {
        /* both pins are covered by btn */
    /*
        printf ("\n both pins are covered by btn ");
    */
        Mac_PutExtNetIdOfPin(grebtn->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(grebtn->name,pin2Ptr);
        return (BEAR_CONTINUE_SEARCH);
        } /* both pins inside a btn */

        if (((Tp_IsVertTile(bnTilePtr))&&(a.p_x >=rect.r_xbot)&&(a.p_x <=
        rect.r_xtop))||((Tp_IsHorzTile(bnTilePtr))&&(a.p_y<=rect2.r_ytop)&&
        (a.p_y >= rect2.r_ybot)))
        {
        /* point a is inside a btn ( pin1 ) */
    /*
        printf ("\n point a is inside a btn ( pin1 )");
    */
        if (((Tp_IsVertTile(bnTilePtr))&&(b.p_x < rect.r_xbot))||
        ((Tp_IsHorzTile(bnTilePtr))&&(b.p_y > rect2.r_ytop)))
        {
        /* point b is at the left or top of btn */
    /*
        printf ("\n point b is at the left or top of btn");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
        Mac_PutExtNetIdOfPin(grebtn->name,pin1Ptr);

        edgeDataPtr->ppin1Ptr =
        grEd_InsertPPinToLeftTopSide(bnTilePtr,grebtn->name,greJunct1->name);
        edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        return (BEAR_CONTINUE_SEARCH);
        }
        else
        {
        /* point b is at the right or bot of bottleneck */
    /*
        printf ("\n point b is at the right or bot of bottleneck");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
        Mac_PutExtNetIdOfPin(grebtn->name,pin1Ptr);

    /* change made on March 25 i988 ;was 2, changed to 1*/
        edgeDataPtr->ppin1Ptr =
        grEd_InsertPPinToRightBotSide(bnTilePtr,grebtn->name,greJunct2->name);
        edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }
        return (BEAR_CONTINUE_SEARCH);
        } /* end pin a inside btn*/


        if (((Tp_IsVertTile(bnTilePtr))&&(b.p_x >=rect.r_xbot)&&(b.p_x <=
        rect.r_xtop))||((Tp_IsHorzTile(bnTilePtr))&&(b.p_y<=rect2.r_ytop)&&
        (b.p_y >= rect2.r_ybot)))
        {
        /* point b is inside a btn ( pin2 ) */
    /*
        printf ("\n point b is inside a btn ( pin2 )");
    */
        if (((Tp_IsVertTile(bnTilePtr))&&(a.p_x < rect.r_xbot))||
        ((Tp_IsHorzTile(bnTilePtr))&&(a.p_y > rect2.r_ytop)))
        {
        /* point a is at the left or top of btn */
    /*
        printf ("\n point a is at the left or top of btn");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(grebtn->name,pin2Ptr);
        edgeDataPtr->ppin1Ptr =
        grEd_InsertPPinToLeftTopSide(bnTilePtr,grebtn->name,greJunct1->name);
        edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        return (BEAR_CONTINUE_SEARCH);
        }
        else
        {
    /*
        printf ("\n point a is at the right or bot of bottleneck");
    */
        /* point a is at the right or bot of bottleneck */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(grebtn->name,pin2Ptr);

        /* change made on March 25, 1988 ; was 2 changed to 1 */
        edgeDataPtr->ppin1Ptr =
        grEd_InsertPPinToRightBotSide(bnTilePtr,grebtn->name,greJunct2->name);
        edgeDataPtr->ppin2Ptr = NIL(PseudoPin);
        }
        return (BEAR_CONTINUE_SEARCH);
        } /* end pin b inside btn*/

        /* here both pins are outside btn */
    /*
        printf ("\n here both pins are outside btn");
    */

        if (((Tp_IsVertTile(bnTilePtr))&&(a.p_x < rect.r_xbot)&&
        (b.p_x < rect.r_xbot))||((Tp_IsHorzTile(bnTilePtr))&&(a.p_y >
        rect2.r_ytop)&&(b.p_y > rect2.r_ytop)))
        {
        /* no intersection, both pins are at left or top side */
    /* 
        printf ("\n no intersection, both pins are at left or top side");
    */

        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
    /*
        printf ("\n end both pins on the left or top side of btn");
    */
        return (BEAR_CONTINUE_SEARCH);
        } /* end both pins on the left or top side of btn */

        if (((Tp_IsVertTile(bnTilePtr))&&(a.p_x > rect.r_xtop)&&
        (b.p_x > rect.r_xtop))||((Tp_IsHorzTile(bnTilePtr))&&(a.p_y < 
        rect2.r_ytop)&&(b.p_y < rect2.r_ybot)))
        {
        /* no intersection, both pins are at bottom or right side */
    /*
        printf ("\n no intersection, both pins are at bottom or right side");
    */

        Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin2Ptr);
    /*
        printf ("\n end both pins on the left or top side of btn");
    */
        return (BEAR_CONTINUE_SEARCH);
        } /* end both pins on the left or top side of btn */

        /* here we know that a full intersection occurs */
    /*
        printf ("\n here we know that a full intersection occurs");
    */

            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToLeftTopSide(bnTilePtr,grebtn->name,
            greJunct1->name);

    /* change made on March 25 th 1988   was :
            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToLeftTopSide(bnTilePtr,grebtn->name,
            greJunct2->name);
            */

    /* is : */
            edgeDataPtr->ppin2Ptr =
            grEd_InsertPPinToRightBotSide(bnTilePtr,grebtn->name,
            greJunct2->name);

        if (((Tp_IsVertTile(bnTilePtr))&&(a.p_x < rect.r_xbot))||
        ((Tp_IsHorzTile(bnTilePtr))&&(a.p_y > rect2.r_ytop)))
        {
        /* a is on the left or top */
    /*
        printf ("\n a is on the left or top");
    */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
            Mac_PutExtNetIdOfPin(greJunct2->name,pin2Ptr);
        }
        else
        {
            /* a is on the right */
        /*
            printf ("\n a is on the right");
        */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
        }
    /*
        printf ("\n end bottleneck tile"); 
    */
        return (BEAR_CONTINUE_SEARCH);
    } /* end bottleneck tile*/
    else
    {
    /* bottle neck line */
    /*
    printf ("\n bottle neck line");
    */
    bnLinePtr=bnPtr->unValue.bnLinePtr;
    grebtnLine=grEd_FindEd_BtnLine(bnLinePtr);
    if (cover->direction == (SegmentDir) FPG_HORIZONTAL)
    { /* vert bnLine */
    /*
    printf ("\n vert bnLine");
    */
    if ((a.p_x < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x)
    &&(b.p_x < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x))
    {
    /* pins sit on the left of btn line */
    /*
    printf ("\n pins sit on the left of btn line ");
    */
        /* no intersection */
    /*
        printf ("\n no intersection ");
    */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
            Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
    /*
        printf ("\n end pin on the left side of btn line");
    */
        return (BEAR_CONTINUE_SEARCH);
    }  /* end pin on the left side of btn line */

    if ((a.p_x > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x)
    &&(b.p_x > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x))
    {
    /* pins sit on the right of btn line */
    /*
    printf ("\n pins sit on the right of btn line");
    */
        /* no intersection */
    /*
        printf ("\n no intersection");
    */
            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
            Mac_PutExtNetIdOfPin(greJunct2->name,pin2Ptr);
    /*
        printf ("\n  end pin on the right side of btn line");
    */
        return (BEAR_CONTINUE_SEARCH);
    }  /* end pin on the right side of btn line */

    /* pin sits on both sides of btn line */
    /*
    printf ("\n pin sits on both sides of btn line");
    */
        /* intersection */
    /*
        printf ("\n intersection");
    */

            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToBtnLine(bnLinePtr,greJunct1->name,
            greJunct2->name,grebtnLine->tilePtr);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);

    if ((a.p_x > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x)
    &&(b.p_x < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x))
    {
    /* a on th right, b on th left */
    /*
    printf ("\n  a on th right, b on th left");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
            
    }
    if ((a.p_x < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x)
    &&(b.p_x > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x))
    {
    /* b on th right,  on th left */
    /*
    printf ("\n b on th right,  on th left");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin2Ptr);
            
    }
    return (BEAR_CONTINUE_SEARCH);
    } /* end vert btn line */
    else
    { /* horz bnLine */
    /*
    printf ("\n horz bnLine"); 
    */
    if ((a.p_y  > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y)
    &&(b.p_y > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y))
    {
    /* pins sit on the top of btn line */
    /*
    printf ("\n pins sit on the top of btn line");
    */
        /* no intersection */
    /*
        printf ("\n no intersection");
    */
            Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
            Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
    /*
        printf ("\n end pins on the top side of btn line");
    */
        return (BEAR_CONTINUE_SEARCH);
    }  /* end pins on the top side of btn line */

    if ((a.p_y < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y)
    &&(b.p_x < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y))
    {
    /* pins sit on the bot of btn line */
    /*
    printf ("\n pins sit on the bot of btn line");
    */
        /* no intersection */
    /*
        printf ("\n no intersection");
    */
            Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
            Mac_PutExtNetIdOfPin(greJunct2->name,pin2Ptr);
        return (BEAR_CONTINUE_SEARCH);
    }  /* end pins on the bot side of btn line */

    /* pin sits on both sides of btn line */
    /*
    printf ("\n pin sit on both sides of btn line");
    */
        /* intersection */
    /*
        printf ("\n intersection");
    */

            edgeDataPtr->ppin1Ptr =
            grEd_InsertPPinToBtnLine(bnLinePtr,greJunct1->name,
            greJunct2->name,grebtnLine->tilePtr);
            edgeDataPtr->ppin2Ptr = NIL(PseudoPin);

    if ((a.p_y < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y)
    &&(b.p_y > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_x))
    {
    /* a on the bot, b on th top */
    /*
    printf ("\n a on the bot, b on th top");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin2Ptr);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin1Ptr);
            
    }
    if ((a.p_y  > (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y)
    &&(b.p_y < (Tp_GetLineOfBnLine(bnLinePtr))->point1.p_y))
    {
    /* b on th bot,  on th top */
    /*
    printf ("\n b on th bot,  on th top");
    */
        Mac_PutExtNetIdOfPin(greJunct1->name,pin1Ptr);
        Mac_PutExtNetIdOfPin(greJunct2->name,pin2Ptr);
            
    }
    return (BEAR_CONTINUE_SEARCH);
    } /* end horz btn line */
    }/* end bottle neck line */
    } /* end two pins */
    return (BEAR_CONTINUE_SEARCH);
}



