
/* greMain.c --
 *
 *     George Carvalho
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifndef BEAR_BLACK_BOX

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "bearGrUtils.h"
#include "grEditor.h"
#include "grEditorInt.h"

#include "../polarBear/polarBear.h"


/* ---------- Internal global variables for the grEditor module ---------- */
int     greWhichJunct;
bool    greUsePgWireData;


/* ---------- Local Procedures ---------- */
static BearStatus   PickNode();
static bool         IsPointInChipBnd();


static bool
IsScreenPointInChipBnd(winDataPtr, screenPtr)
WindowData  *winDataPtr;
Point       *screenPtr;             /* screen coordinates */
{
    CellMaster  *cellMasterPtr;
    ChipWin     *chipWinPtr;
    PlaneData   *planeDataPtr;
    Point       wPoint;             /* world coordinates */
    Rect        *chipBndPtr;

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    chipBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);

    wPoint.p_x = Ez_XTrans(winDataPtr->wd_invScrMatrix, screenPtr->p_x,
        screenPtr->p_y);
    wPoint.p_y = Ez_YTrans(winDataPtr->wd_invScrMatrix, screenPtr->p_x,
        screenPtr->p_y);

    return (GEO_ENCLOSE(&wPoint, chipBndPtr));
}



/*
 * ----------------------------------------------------------------------------
 *
 * PickNode() --
 *
 * Three different types of nodes may be picked as follows :
 *
 *   1.) To pick a pin on the chips border click the mouse outside of the 
 *       chips border (which will cause the cursor to change to a square)
 *       and then click the mouse again to indicate the appropiate pin.
 *
 *   2.) To pick a pin on a cell instance click the mouse inside of that
 *       cell instance (which will cause the cursor to change to a square)
 *       and then click the mouse again to indicate the appropiate pin.
 *
 *   3.) To pick a wall junction click the mouse near one.
 *
 * Results:
 *   BEAR_OK is returned, the structure pinOrJunct is filled in, and
 *   the node is high-lighted on the screen if everything went well,
 *   BEAR_ERROR otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

static BearStatus
PickNode(pinOrJunctPtr, netPtr)
PinOrJunct  *pinOrJunctPtr;         /* Fields of this structure will be
                                     * filled in based on what they pick.
                                     */
Net         *netPtr;                /* If they pick a pin it must belong to
                                     * this net.
                                     */

{
    CellInst        *cellInstPtr;
    ezStatus        findStatus;
    DrawNode        drawNode;
    Line            span;
    LineSeg         *lineSegPtr;
    Pb_DefCurData   defCurData;
    Point           absPos, screen;
    PinInst         *pinInstPtr;
    Tile            *horzDomTilePtr, *vertDomTilePtr;
    Tile            *horzTilePtr;
    WallJunction    *junctPtr;
    WindowData      *winDataPtr;

    /*
     * Change the cursor to a circle in all of the chip windows.
     */
    defCurData.idWinProc = PbIsChipWin;
    defCurData.cursorType = (int) EZ_CIRCLE_OUTLINE_CUR;
    (void) PbItrChipWindows(PbDefCurInUserSubWin, (ClientData) &defCurData);
    Ez_Flush();

    /*
     * Wait for the user to click the mouse in a chip window, which
     * will indicate the location of the node.
     */
    findStatus = Ez_FindUserWin(LeftButton, PbIsChipWin, (ClientData) NULL,
        &winDataPtr, &screen);

    /*
     * Change the cursor back to a cross in all of the chip windows.
     */
    defCurData.cursorType = (int) EZ_CROSS_CUR;
    (void) PbItrChipWindows(PbDefCurInUserSubWin, (ClientData) &defCurData);
    Ez_Flush();

    if (findStatus != EZ_OK) {
        PbPrint(EZ_ITALIC_FONT, "Must pick a point in a chip window.");
        return (BEAR_ERROR);
    }

    if (IsScreenPointInChipBnd(winDataPtr, &screen) == FALSE) {
        if ((Gre_PickPinArea(&cellInstPtr, &lineSegPtr, 
            &absPos, &span) == BEAR_ERROR) ||

            (Gre_FindPin(lineSegPtr, &span, netPtr, &pinInstPtr) ==
                BEAR_ERROR)) {
            return (BEAR_ERROR);
        }

        if (Mac_GetNetOfPin(pinInstPtr) != netPtr) {
            sprintf (bear_ErrorMsg,
                "Pin must belong to net whose name is %s.\n",
                Mac_GetNameOfNet(netPtr));
            PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
            return (BEAR_ERROR);
        }

        pinOrJunctPtr->unType = FPG_PIN;
        pinOrJunctPtr->unValue.pinPtr = pinInstPtr;
    }
    else if ((PbFindTile(winDataPtr, &screen,
        &horzTilePtr, HORZ_TILE_PLANE) == TRUE) &&

        Tp_IsCellInstTile(horzTilePtr)) {

        if ((Gre_PickPinArea(&cellInstPtr, &lineSegPtr, 
            &absPos, &span) == BEAR_ERROR) ||

            (cellInstPtr != Tp_GetCellInstOfTile(horzTilePtr)) ||

            (Gre_FindPin(lineSegPtr, &span, netPtr, &pinInstPtr) ==
                BEAR_ERROR)) {
            return (BEAR_ERROR);
        }

        if (Mac_GetNetOfPin(pinInstPtr) != netPtr) {
            sprintf (bear_ErrorMsg,
                "Pin must belong to net whose name is %s.\n",
                Mac_GetNameOfNet(netPtr));
            PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
            return (BEAR_ERROR);
        }
    
        pinOrJunctPtr->unType = FPG_PIN;
        pinOrJunctPtr->unValue.pinPtr = pinInstPtr;
    }
    else if (GreFindJunct(winDataPtr, &screen,
        &horzDomTilePtr, &vertDomTilePtr, &junctPtr) == BEAR_OK) {

        pinOrJunctPtr->unType = FPG_WALL_JUNCT;
        pinOrJunctPtr->unValue.junctPtr = junctPtr;
    }
    else {
        return (BEAR_ERROR);
    }

    /*
     * High-light the node in all of the chip windows.
     */
    drawNode.pinOrJunctPtr = pinOrJunctPtr;
    drawNode.pinColor = drawNode.junctColor = ez_HighLightColor;
    (void) PbItrChipWindows(Gre_DrawNode, (ClientData) &drawNode);
    Ez_Flush();

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_UsePgWireDataToDrawNet() --
 *
 * Enables you to choose whether or not the power ground wire width
 * information for a net will be used when drawing the net on the
 * screen.
 *
 * Results:
 *   none
 *
 * Side effects:
 *   Alters the grEditor module's internal global variable 'greUsePgWireData'
 * ----------------------------------------------------------------------------
 */

void
Gre_UsePgWireDataToDrawNet(usePgWireData)
bool usePgWireData;
{
    greUsePgWireData = usePgWireData;
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_ChooseWallJunct() --
 *
 * Enables you to choose the floor plan that is to be used for
 * drawing nets and wall junctions.
 *
 * Results:
 *   none
 *
 * Side effects:
 *   Alters the grEditor module's internal global variable 'greWhichJunct'
 * ----------------------------------------------------------------------------
 */

void
Gre_ChooseWallJunct(whichJunct)
int whichJunct; /* FPG_NW | FPG_CENTER | FPG_SE */
{
    greWhichJunct = whichJunct;
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_PickNet() --
 *
 * Enables the user to pick a net, by having him select a pin that
 * belongs to the net with the mouse.
 *
 * Results:
 *   If everything goes well, the net is high lighted in every chip window,
 *   pointers to the pin and the net are filled in, and BEAR_OK is
 *   returned.  Otherwise BEAR_ERROR is returned and all bets are off.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_PickNet(winDataPtr, netPtrPtr, pinPtrPtr)
WindowData  *winDataPtr;
Net         **netPtrPtr;
PinInst     **pinPtrPtr;
{
    CellMaster  *cellMasterPtr;
    ChipWin     *chipWinPtr;
    DrawNet     drawNet;
    Point       screen;
    int         whichNet = FPG_CENTER;

    *netPtrPtr = NIL(Net);
    *pinPtrPtr = NIL(PinInst);

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    if (PbGetNetOfCHW(whichNet, chipWinPtr) != NIL(Net)) {
        PbPutNetOfCHW(whichNet, NIL(Net), chipWinPtr);
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    }

    /*
     * If the pins are not currently being displayed in this
     * window draw them in.
     */
    if ((chipWinPtr->chw_options & PB_SHOW_PINS_OPT) == FALSE) {
        Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_PINS_OPT);
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
    }

    PbPrint(EZ_NORMAL_FONT, "Pick a net.\n");
    if (Ez_FindUserWin(LeftButton, PbIsChipWin, (ClientData) NULL,
        &winDataPtr, &screen) != EZ_OK) {
        PbPrint(EZ_ITALIC_FONT, "must pick a point in a chip window\n");
        return (BEAR_ERROR);
    }
    if (Gre_GetPin(winDataPtr, (BearString) NULL, (BearString) NULL,
        ez_HighLightColor, pinPtrPtr) != BEAR_OK) {
        return (BEAR_ERROR);
    }
    *netPtrPtr = Mac_GetNetOfPin(*pinPtrPtr);
    PbPutNetOfCHW(whichNet, *netPtrPtr, chipWinPtr);
    PbPutPinOfCHW(NIL(PinInst), chipWinPtr);

    /*
     * Draw the net that the user picked using the high light color in
     * all of the chip windows.
     */
    drawNet.netPtr = *netPtrPtr;
    drawNet.cellMasterPtr = cellMasterPtr;
    drawNet.whichNet = whichNet;
    drawNet.netColor = PbGetColorOfNet(whichNet);
    (void) PbItrChipWindows(Gre_DrawNet, (ClientData) &drawNet);
    Ez_Flush();

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_PickNetByName() --
 *
 * Enables the user to pick a net, by specifying the net's name.
 *
 * Results:
 *   If everything goes well, the net is high lighted on the screen,
 *   pointers to the net are filled in, and BEAR_OK is returned.
 *   Otherwise BEAR_ERROR is returned and all bets are off.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_PickNetByName(winDataPtr, netNamePtr, whichNet, netPtrPtr)
WindowData  *winDataPtr;
char        *netNamePtr;
int         whichNet;       /* FPG_NW | FPG_CENTER | FPG_SE */
Net         **netPtrPtr;
{
    ChipWin     *chipWinPtr;
    DrawNet     drawNet;
    Net         *oldNetPtr, *newNetPtr;
    CellMaster  *cellMasterPtr;

    *netPtrPtr = NIL(Net);

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (Mac_IsMember(NET_HT, netNamePtr)) {
        newNetPtr = (Net *) Mac_LookUp(NET_HT, netNamePtr);
    }
    else {
        sprintf (bear_ErrorMsg, "Cannot find a net named %s.\n", netNamePtr);
        PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
        return (BEAR_ERROR);
    }

    oldNetPtr = PbGetNetOfCHW(whichNet, chipWinPtr);
    if (oldNetPtr == NIL(Net)) {
        PbPutNetOfCHW(whichNet, newNetPtr, chipWinPtr);

        /*
         * Draw only the net that the user picked by name using the
         * high light color in all of the chip windows.
         */
        drawNet.netPtr = newNetPtr;
        drawNet.cellMasterPtr = cellMasterPtr;
        drawNet.whichNet = whichNet;
        drawNet.netColor = PbGetColorOfNet(whichNet);
        (void) PbItrChipWindows(Gre_DrawNet, (ClientData) &drawNet);
    }
    else if (oldNetPtr != newNetPtr) {
        PbPutNetOfCHW(whichNet, newNetPtr, chipWinPtr);

        /*
         * Redraw all of the chip windows with the new net.
         */
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    }

    *netPtrPtr = newNetPtr;
    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_AddEdgeToGraph() --
 *
 * Add an edge to the given net's user generated graph.
 *
 * To do this the user needs to pick two nodes (a node is a pin or a wall
 * junction) that share a common edge.  The nodes may be already belong to
 * the graph of the given net or be brand new.  A new edge will then be
 * added.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   The graph which is part of the data base is altered accordingly.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_AddEdgeToGraph(netPtr)
Net *netPtr;
{
    BearGeneric     bearGen;
    BearStatus      bearStatus;
    CellMaster      *cellMasterPtr;
    DrawNode        drawNode1, drawNode2;
    FloorPlanGraph  *fpgPtr;
    Mac_EdgeData    *edgeDataPtr;
    Gt_Graph        userGraph;
    Gt_GraphNode    node1, node2;
    Gt_GraphNode    newNode1 = (Gt_GraphNode) NULL;
    Gt_GraphNode    newNode2 = (Gt_GraphNode) NULL;
    Gt_GraphEdge    edge;
    PlaneData       *planeDataPtr;
    PinOrJunct      pinOrJunct1, pinOrJunct2, *pinOrJunctPtr;
    WallSegment     *wallSegPtr;

    Gre_ChooseWallJunct(FPG_CENTER);
    Gre_UsePgWireDataToDrawNet(FALSE);

    userGraph = Mac_GetUserGraphOfNet(netPtr);
    if (userGraph == (Gt_Graph) NULL) {
        Mac_GraphData *graphDataPtr;

        graphDataPtr = Mac_MakeGraphData();
/*
 * proposed change : by george : on 6 mar 90
 *
        graphDataPtr->fpgId = Fpg_GetIdOfFpg(fpgPtr);
 */
        graphDataPtr->pgFlag = TRUE;
        graphDataPtr->edgeDataFlag = TRUE;

        userGraph = Gt_CreateGraph(Mac_GenerateId(), 
            (BearGeneric) graphDataPtr);
        Mac_PutUserGraphOfNet(userGraph, netPtr);
    }

    PbPrint(EZ_NORMAL_FONT, "Pick node one.\n");
    if (PickNode(&pinOrJunct1, netPtr) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }
    drawNode1.pinColor = PbGetColorOfNet(greWhichJunct);
    drawNode1.pinOrJunctPtr = &pinOrJunct1;
    if (GreFindNodeOfGraph(userGraph, &pinOrJunct1, &node1) == BEAR_TRUE) {
        /* Node one is part of the graph. */

        drawNode1.junctColor = PbGetColorOfNet(greWhichJunct);
    }
    else {
        /*
         * Node one is brand new, and as such does not belong to the
         * graph.
         */

        drawNode1.junctColor = pbCellBorderColor;
    }

    PbPrint(EZ_NORMAL_FONT, "Pick node two.\n");
    if (PickNode(&pinOrJunct2, netPtr) == BEAR_ERROR) {
        goto error1;
    }
    drawNode2.pinColor = PbGetColorOfNet(greWhichJunct);
    drawNode2.pinOrJunctPtr = &pinOrJunct2;
    if (GreFindNodeOfGraph(userGraph, &pinOrJunct2, &node2) == BEAR_TRUE) {
        /* Node two is part of the graph. */

        drawNode2.junctColor = PbGetColorOfNet(greWhichJunct);
    }
    else {
        /*
         * Node two is brand new, and as such does not belong to the
         * graph.
         */

        drawNode2.junctColor = pbCellBorderColor;
    }

    if (GreAreTheseNodesLegal(&pinOrJunct1, &pinOrJunct2) == BEAR_ERROR) {
        goto error2;
    }

    if (node1 == (Gt_GraphNode) NULL) {
        pinOrJunctPtr = Fpg_MakePinOrJunct();
        pinOrJunctPtr->unType = pinOrJunct1.unType;
        if (pinOrJunctPtr->unType == FPG_PIN) {
            pinOrJunctPtr->unValue.pinPtr = pinOrJunct1.unValue.pinPtr;
        }
        else {
            pinOrJunctPtr->unValue.junctPtr = pinOrJunct1.unValue.junctPtr;
        }
        newNode1 = node1 = Gt_AddGraphNode(userGraph, Mac_GenerateId(),
            (BearGeneric) pinOrJunctPtr);
    }
    if (node2 == (Gt_GraphNode) NULL) {
        pinOrJunctPtr = Fpg_MakePinOrJunct();
        pinOrJunctPtr->unType = pinOrJunct2.unType;
        if (pinOrJunctPtr->unType == FPG_PIN) {
            pinOrJunctPtr->unValue.pinPtr = pinOrJunct2.unValue.pinPtr;
        }
        else {
            pinOrJunctPtr->unValue.junctPtr = pinOrJunct2.unValue.junctPtr;
        }
        newNode2 = node2 = Gt_AddGraphNode(userGraph, Mac_GenerateId(),
            (BearGeneric) pinOrJunctPtr);
    }

    edgeDataPtr = Mac_MakeEdgeData();
    if (pinOrJunct1.unType == FPG_WALL_JUNCT &&
        pinOrJunct2.unType == FPG_WALL_JUNCT) {

        if (Fpg_FindWallSeg(pinOrJunct1.unValue.junctPtr,
            pinOrJunct2.unValue.junctPtr, &wallSegPtr) == BEAR_OK) {

            edgeDataPtr->bnPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
        }
        else {
            goto error3;
        }
    }

    bearStatus = Gt_AddGraphEdge(node1, node2, Mac_GenerateId(),
        (BearGeneric) edgeDataPtr, &edge);
    switch (bearStatus) {
    case BEAR_OK:
        break;
    case BEAR_ERROR:
        PbPrint(EZ_ITALIC_FONT, "Nodes must belong to the same graph.\n");
        goto error3;
    case BEAR_ERROR_2:
        PbPrint(EZ_ITALIC_FONT, "Self-loops are not allowed.\n");
        goto error3;
    case BEAR_ERROR_3:
        PbPrint(EZ_ITALIC_FONT,
            "Only one edge between two nodes is allowed.\n");
        goto error3;
    default:
        PbPrint(EZ_ITALIC_FONT,
            "Unknown error (must be really bad -- take an early lunch).\n");
        goto error3;
    }

    /* 
     * High-light the new edge in all of the chip windows.
     */
    (void) PbItrChipWindows(GreDrawEdgeOfGraphTwo, (ClientData) edge);
 
    return (BEAR_OK);

error3:
    if (newNode1 != (Gt_GraphNode) NULL) {
        (void) Gt_DeleteGraphNode(newNode1, &bearGen);
        FREE((char *) bearGen);
    }
    if (newNode2 != (Gt_GraphNode) NULL) {
        (void) Gt_DeleteGraphNode(newNode2, &bearGen);
        FREE((char *) bearGen);
    }
    FREE((char *) edgeDataPtr);

error2:
    /*
     * Draw node two again in all of the chip windows so that
     * it won't be high-lighted.
     */
    (void) PbItrChipWindows(Gre_DrawNode, (ClientData) &drawNode2);

error1:
    /*
     * Draw node one again in all of the chip windows so that
     * it won't be high-lighted.
     */
    (void) PbItrChipWindows(Gre_DrawNode, (ClientData) &drawNode1);
    Ez_Flush();

    return (BEAR_ERROR);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DeleteEdgeFromGraph() --
 *
 * Delete an edge from the given net's user generated graph.
 *
 * To do this the user needs to pick two nodes (a node is a pin or a wall
 * junction) that belong to the given net's graph.  The common edge between
 * the two nodes will then be deleted.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   The graph which is part of the data base is altered accordingly.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DeleteEdgeFromGraph(netPtr)
Net *netPtr;
{
    BearGeneric     nodeData, edgeData;
    BearStatus      bearStatus;
    DrawNode        drawNode1, drawNode2;
    Gt_Graph        userGraph;
    Gt_GraphEdge    edge;
    Gt_GraphNode    node1, node2;
    PinOrJunct      pinOrJunct1, pinOrJunct2;

    Gre_ChooseWallJunct(FPG_CENTER);
    Gre_UsePgWireDataToDrawNet(FALSE);

    if ((userGraph = Mac_GetUserGraphOfNet(netPtr)) == (Gt_Graph) NULL) {
        PbPrint(EZ_ITALIC_FONT, "This net has no graph.\n");
        return (BEAR_ERROR);
    }

    /* Node one must belong to an existing graph. */
    PbPrint(EZ_NORMAL_FONT, "Pick node one.\n");
    if (PickNode(&pinOrJunct1, netPtr) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }
    drawNode1.pinOrJunctPtr = &pinOrJunct1;
    if (GreFindNodeOfGraph(userGraph, &pinOrJunct1, &node1) == BEAR_TRUE) {
        drawNode1.pinColor = drawNode1.junctColor = 
            PbGetColorOfNet(greWhichJunct);
    }
    else {
        drawNode1.pinColor = pbPinColor;
        drawNode1.junctColor = pbCellBorderColor;
        PbPrint(EZ_ITALIC_FONT, "The node must belong to a graph.\n");
        goto error1;
    }

    /* Node two must belong to an existing graph. */
    PbPrint(EZ_NORMAL_FONT, "Pick node two.\n");
    if (PickNode(&pinOrJunct2, netPtr) == BEAR_ERROR) {
        goto error1;
    }
    drawNode2.pinOrJunctPtr = &pinOrJunct2;
    if (GreFindNodeOfGraph(userGraph, &pinOrJunct2, &node2) == BEAR_TRUE) {
        drawNode2.pinColor = drawNode2.junctColor = 
            PbGetColorOfNet(greWhichJunct);
    }
    else {
        drawNode2.pinColor = pbPinColor;
        drawNode2.junctColor = pbCellBorderColor;
        PbPrint(EZ_ITALIC_FONT, "The node must belong to a graph.\n");
        goto error2;
    }

    if (Gt_FindEdgeOfNodes(node1, node2, &edge) == BEAR_ERROR) {
        PbPrint(EZ_ITALIC_FONT, "Nodes must belong to the same graph.\n");
        goto error2;
    }
    if (edge == (Gt_GraphEdge) NULL) {
        PbPrint(EZ_ITALIC_FONT, "These two nodes are not connected.\n");
        goto error2;
    }
    Gt_DeleteGraphEdge(edge, &edgeData);
    FREE((char *) edgeData);

    if (Gt_IsNodeUnconnected(node1) == TRUE) {
        (void) Gt_DeleteGraphNode(node1, &nodeData);
        FREE((char *) nodeData);
    }

    if (Gt_IsNodeUnconnected(node2) == TRUE) {
        (void) Gt_DeleteGraphNode(node2, &nodeData);
        FREE((char *) nodeData);
    }

    (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    return (BEAR_OK);

error2:
    /*
     * Draw node two again in all of the chip windows again so that
     * it won't be high-lighted.
     */
    (void) PbItrChipWindows(Gre_DrawNode, (ClientData) &drawNode2);

error1:
    /*
     * Draw node one again in all of the chip windows again so that
     * it won't be high-lighted.
     */
    (void) PbItrChipWindows(Gre_DrawNode, (ClientData) &drawNode1);
    Ez_Flush();

    return (BEAR_ERROR);
}



#ifdef BETTY_BOOP
/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DeleteTreeFromNet() --
 *
 * Delete a tree from the given net's global routing tree.
 *
 * To do this the user needs to pick one node (a node is a pin or a wall
 * junction) that belongs to the given net's global routing tree.  The
 * cooresponding sub-tree will then be deleted.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   The global routing tree which is part of the data base is
 *   altered accordingly.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DeleteTreeFromNet(netPtr)
Net *netPtr;
{
    DrawNode    drawNode;
    PinOrJunct  pinOrJunct;
    TreeNode    *nodePtr;
    TreeNode    *oldRootPtr;

    Gre_ChooseWallJunct(FPG_CENTER);
    Gre_UsePgWireDataToDrawNet(FALSE);

    /* Node one must belong to an existing sub-tree. */
    PbPrint(EZ_NORMAL_FONT, "Pick node one.\n");
    if (PickNode(&pinOrJunct, netPtr) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }
    if (GreFindNodeOfGraph(userGraph, &pinOrJunct, &nodePtr) == BEAR_TRUE) {
        oldRootPtr = Gt_FindRootOfTree(nodePtr);
    }
    else {
        PbPrint(EZ_ITALIC_FONT, "The node must belong to a sub-tree.\n");

        /*
         * Draw node one again in all of the chip windows again so that
         * it won't be high-lighted.
         */
        drawNode.pinOrJunctPtr = &pinOrJunct;
        drawNode.pinColor = pbPinColor;
        drawNode.junctColor = pbCellBorderColor;
        (void) PbItrChipWindows(Gre_DrawNode, (ClientData) &drawNode);
        Ez_Flush();

        return (BEAR_ERROR);
    }

    (void) Mac_DeleteGRTreeOfNet(netPtr, oldRootPtr);

    (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    return (BEAR_OK);
}

#endif BETTY_BOOP


/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawWallJunct() --
 *
 * This procedure is designed to be used by an "interator".  It
 * draws the wall junction it's given.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *   width - use MAXWIDTH as a default value
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawWallJunct(junctPtr, drawDataPtr)
WallJunction    *junctPtr;
DrawData        *drawDataPtr;
{
    Gre_DrawFromJunctToJunct(drawDataPtr, NIL(WallJunction), junctPtr);

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_DrawWallSeg() --
 *
 * This procedure is designed to be used by an "interator".  It
 * draws the wall segment it's given.
 *
 * The following fields from the structure DrawData must be filled in:
 *   winDataPtr
 *   color
 *   width - use MAXWIDTH as a default value
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_DrawWallSeg(segPtr, drawDataPtr)
WallSegment *segPtr;
DrawData    *drawDataPtr;
{
    Gre_DrawFromJunctToJunct(drawDataPtr, 
        Fpg_GetSEJunctOfWallSeg(segPtr),
        Fpg_GetNWJunctOfWallSeg(segPtr));

    return (BEAR_CONTINUE_SEARCH);
}


#endif BEAR_BLACK_BOX


