
/* greMisc.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include <signal.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "grEditor.h"
#include "grEditorInt.h"

#include "../polarBear/polarBear.h"

typedef struct _CheckPPinData {
    Net *netPtr;
    int count;       
    int bnLinePPinCount;       
    int bnTilePPinCount;       
    int dcellPPinCount;       
} CheckPPinData;

typedef struct _FindData {
    PinOrJunct      *pinOrJunctPtr;
    Gt_GraphNode    node;
} FindData;


/* ---------- Local Procedures ---------- */
BearStatus  CheckPPin();
BearStatus  CheckPPinsOfBottleNeck();
BearStatus  CheckPPinsOfDCell();
BearStatus  CheckConnForOneNet();
BearStatus  CompareNode();
int         FindDomTile();



static BearStatus
CheckPPin(ppinPtr, checkPPinDataPtr)
PseudoPin       *ppinPtr;
CheckPPinData   *checkPPinDataPtr;
{
    Net *netPtr;
 
#ifdef DEBUG
    checkPPinDataPtr->count++;
#endif DEBUG

    if ((checkPPinDataPtr->netPtr != NIL(Net)) &&
        (checkPPinDataPtr->netPtr != Tp_GetNetOfPPin(ppinPtr))) {
            return (BEAR_CONTINUE_SEARCH);
    }
    if (Tp_GetIntNetIdOfPPin(ppinPtr) == Tp_GetExtNetIdOfPPin(ppinPtr)) {

        netPtr = (checkPPinDataPtr->netPtr) ?
            checkPPinDataPtr->netPtr : Tp_GetNetOfPPin(ppinPtr);

        sprintf(bear_ErrorMsg,
            "Pseudo pin in net named \"%s\" has matching net ids.\n",
            Mac_GetNameOfNet(netPtr));
        PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);

        return (BEAR_STOP_SEARCH);
    }

    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
CheckPPinsOfBottleNeck(bnPtr, checkPPinDataPtr)
BottleNeck      *bnPtr;
CheckPPinData   *checkPPinDataPtr;
{
    BNLine *bnLinePtr;

    if (bnPtr->unType == TP_BN_TILE) {
#ifdef DEBUG
        checkPPinDataPtr->count = 0;
#endif DEBUG

        if (Tp_ItrPPinsOfBnTile(bnPtr->unValue.bnTilePtr,
            CheckPPin, checkPPinDataPtr) == BEAR_STOP_SEARCH) {

            return (BEAR_STOP_SEARCH);
        }

#ifdef DEBUG
        checkPPinDataPtr->bnTilePPinCount += checkPPinDataPtr->count;
#endif DEBUG
    } else {
#ifdef DEBUG
        checkPPinDataPtr->count = 0;
#endif DEBUG

        bnLinePtr = bnPtr->unValue.bnLinePtr;
        if (Tp_ItrPPinsOfBnLine(bnLinePtr->tlTilePtr, bnLinePtr->brTilePtr,
            CheckPPin, checkPPinDataPtr) == BEAR_STOP_SEARCH) {

            return (BEAR_STOP_SEARCH);
        }

#ifdef DEBUG
        checkPPinDataPtr->bnLinePPinCount += checkPPinDataPtr->count;
#endif DEBUG
    }

    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
CheckPPinsOfDCell(dcellPtr, checkPPinDataPtr)
DummyCell       *dcellPtr;
CheckPPinData   *checkPPinDataPtr;
{
#ifdef DEBUG
    checkPPinDataPtr->count = 0;
#endif DEBUG

    if (Mac_ItrPPinOfDCell(dcellPtr, (SideDirection) TOP_SIDE,
        CheckPPin, checkPPinDataPtr) == BEAR_STOP_SEARCH)
    {
        return (BEAR_STOP_SEARCH);
    }

    if (Mac_ItrPPinOfDCell(dcellPtr, (SideDirection) BOTTOM_SIDE,
        CheckPPin, checkPPinDataPtr) == BEAR_STOP_SEARCH)
    {
        return (BEAR_STOP_SEARCH);
    }

    if (Mac_ItrPPinOfDCell(dcellPtr, (SideDirection) LEFT_SIDE,
        CheckPPin, checkPPinDataPtr) == BEAR_STOP_SEARCH)
    {
        return (BEAR_STOP_SEARCH);
    }

    if (Mac_ItrPPinOfDCell(dcellPtr, (SideDirection) RIGHT_SIDE,
        CheckPPin, checkPPinDataPtr) == BEAR_STOP_SEARCH)
    {
        return (BEAR_STOP_SEARCH);
    }

#ifdef DEBUG
    checkPPinDataPtr->dcellPPinCount += checkPPinDataPtr->count;
#endif DEBUG

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_CheckPins() --
 *
 * if (netPtr == NIL(Net)) then
 *    check all the nets
 * else
 *    only check netPtr
 * 
 * Checks the pseudo pins in bottlenecks and dummy cells for
 * matching id's.
 *
 * Results:
 *   BEAR_OK is returned, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_CheckPins(cellMasterPtr, netPtr)
CellMaster  *cellMasterPtr;
Net         *netPtr;
{
    CheckPPinData checkPPinData;
    PlaneData     *planeDataPtr;

#ifdef DEBUG
    checkPPinData.bnLinePPinCount = 0;
    checkPPinData.bnTilePPinCount = 0;
    checkPPinData.dcellPPinCount = 0;
#endif DEBUG

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    checkPPinData.netPtr = netPtr;

    /* 
     * Check all of the pseudo pins in the bottle neck tiles
     * and the bottle neck lines.
     */
    if (Fpg_ItrBottleNeckOfFpg(Tp_GetFpg(planeDataPtr),
        CheckPPinsOfBottleNeck, &checkPPinData) == BEAR_STOP_SEARCH) {

        return (BEAR_ERROR);
    }

    /* 
     * Check all of the pseudo pins in the dummy cells.
     */
    if (Mac_ItrDCell(Tp_GetDCellList(planeDataPtr),
        CheckPPinsOfDCell, &checkPPinData) == BEAR_STOP_SEARCH) {

        return (BEAR_ERROR);
    }

#ifdef DEBUG
    {
        int total;

        printf ("---\n");
        if (netPtr == NIL(Net)) {
            printf ("combined ppin statistics for all the nets\n\n");
        } else {
            printf ("ppin statistics for the net named \"%s\".\n\n",
                Mac_GetNameOfNet(netPtr));
        }

        printf ("bn line ppin count : %d\n",
            checkPPinData.bnLinePPinCount);
        printf ("bn tile ppin count : %d\n",
            checkPPinData.bnTilePPinCount);
        printf ("dcell ppin count : %d\n",
            checkPPinData.dcellPPinCount);

        total = checkPPinData.bnLinePPinCount +
            checkPPinData.bnTilePPinCount +
            checkPPinData.dcellPPinCount;
        printf ("total : %d\n", total);
        printf ("---\n");
        fflush (stdout);
    }
#endif DEBUG

    return (BEAR_OK);
}



bool
GreJunctTouchesJunct(junct1Ptr, junct2Ptr)
WallJunction *junct1Ptr, *junct2Ptr;
/*
 * Determines if 'junct1Ptr' is next to 'junct2Ptr'.
 */
{
    WallSegment *segmentPtr;

    if (((segmentPtr = Fpg_GetNorthSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetNWJunctOfWallSeg(segmentPtr) == junct2Ptr)) {

        return (TRUE);
    } 
    else if (((segmentPtr = Fpg_GetEastSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetSEJunctOfWallSeg(segmentPtr) == junct2Ptr)) {

        return (TRUE);
    } 
    else if (((segmentPtr = Fpg_GetSouthSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetSEJunctOfWallSeg(segmentPtr) == junct2Ptr)) {
         
        return (TRUE);
    } 
    else if (((segmentPtr = Fpg_GetWestSegOfWallJunct(junct1Ptr)) !=
        NIL(WallSegment)) &&
        (Fpg_GetNWJunctOfWallSeg(segmentPtr) == junct2Ptr)) {

        return (TRUE);
    } 
    else {
        return (FALSE);
    }
}



BearStatus
Gre_PickJunct(horzDomTilePtrPtr, vertDomTilePtrPtr, junctPtrPtr)
Tile            **horzDomTilePtrPtr; /* Pointer to a horizontal dominant
                                      * tile pointer.
                                      */
Tile            **vertDomTilePtrPtr; /* Pointer to a vertical dominant
                                      * tile pointer.
                                      */
WallJunction    **junctPtrPtr;
{
#ifndef BEAR_BLACK_BOX

    DrawNode        drawNode;
    ezStatus        findStatus;
    Pb_DefCurData   defCurData;
    PinOrJunct      pinOrJunct;
    Point           screen;
    Tile            *horzDomTilePtr, *vertDomTilePtr;
    WallJunction    *junctPtr;
    WindowData      *winDataPtr;

    *horzDomTilePtrPtr = NIL(Tile);
    *vertDomTilePtrPtr = NIL(Tile);
    *junctPtrPtr = NIL(WallJunction);

    /*
     * Change the cursor to a circle in all of the chip windows.
     */
    defCurData.idWinProc = PbIsChipWin;
    defCurData.cursorType = (int) EZ_CIRCLE_OUTLINE_CUR;
    (void) PbItrChipWindows(PbDefCurInUserSubWin, (ClientData) &defCurData);
    Ez_Flush();

    /*
     * Wait for the user to click the mouse in a chip window, which
     * will indicate the location of the node.
     */
    findStatus = Ez_FindUserWin(LeftButton, PbIsChipWin, (ClientData) NULL,
        &winDataPtr, &screen);

    /*
     * Change the cursor back to a cross in all of the chip windows.
     */
    defCurData.cursorType = (int) EZ_CROSS_CUR;
    (void) PbItrChipWindows(PbDefCurInUserSubWin, (ClientData) &defCurData);
    Ez_Flush();

    if (findStatus != EZ_OK) {
        PbPrint(EZ_ITALIC_FONT, "Must pick a point in a chip window.");
        return (BEAR_ERROR);
    }

    if (GreFindJunct(winDataPtr, &screen,
        &horzDomTilePtr, &vertDomTilePtr, &junctPtr) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    /*
     * Draw the junction in all of the chip windows.
     */
    pinOrJunct.unType = FPG_WALL_JUNCT;
    pinOrJunct.unValue.junctPtr = junctPtr;
    drawNode.pinOrJunctPtr = &pinOrJunct;
    drawNode.pinColor = drawNode.junctColor = ez_HighLightColor;
    (void) PbItrChipWindows(Gre_DrawNode, (ClientData) &drawNode);
    Ez_Flush();

    *horzDomTilePtrPtr = horzDomTilePtr;
    *vertDomTilePtrPtr = vertDomTilePtr;
    *junctPtrPtr = junctPtr;
#endif BEAR_BLACK_BOX

    return (BEAR_OK);
}



BearStatus
GreFindJunct(winDataPtr, screenPtr,
    horzDomTilePtrPtr, vertDomTilePtrPtr, junctPtrPtr)

WindowData      *winDataPtr;
Point           *screenPtr;
Tile            **horzDomTilePtrPtr; /* Pointer to a horizontal dominant
                                      * tile pointer.
                                      */
Tile            **vertDomTilePtrPtr; /* Pointer to a vertical dominant
                                      * tile pointer.
                                      */
WallJunction    **junctPtrPtr;
{
#ifndef BEAR_BLACK_BOX

    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    FloorPlanGraph  *fpgPtr;
    PlaneData       *planeDataPtr;
    Rect            horzSquare, vertSquare;
    Tile            *horzDomTilePtr, *vertDomTilePtr;

    *horzDomTilePtrPtr = NIL(Tile);
    *vertDomTilePtrPtr = NIL(Tile);
    *junctPtrPtr = NIL(WallJunction);

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    PbCalcSquare(winDataPtr, screenPtr, &horzSquare);
    GeoClip(&horzSquare, Tp_GetHorzPlaneBnd(planeDataPtr));

    horzDomTilePtr = NIL(Tile);
    if ((TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
        &horzSquare, FindDomTile, (ClientData) &horzDomTilePtr) == 1) ||

        (horzDomTilePtr == NIL(Tile))) {
        return (BEAR_ERROR);
    }

    Tp_HRectToVRect(&horzSquare, &vertSquare);
    vertDomTilePtr = NIL(Tile);
    if ((TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr),
        &vertSquare, FindDomTile, (ClientData) &vertDomTilePtr) == 1) ||

        (vertDomTilePtr == NIL(Tile))) {
        return (BEAR_ERROR);
    }

    fpgPtr = Tp_GetFpg(planeDataPtr);
    *horzDomTilePtrPtr = horzDomTilePtr;
    *vertDomTilePtrPtr = vertDomTilePtr;
    *junctPtrPtr = Fpg_GetWallJunctWhereDomTilesCross(fpgPtr,
        horzDomTilePtr, vertDomTilePtr);
#endif BEAR_BLACK_BOX

    return (BEAR_OK);
}



static int
FindDomTile(tilePtr, domTilePtrPtr)
Tile    *tilePtr;
Tile    **domTilePtrPtr;
{
    if (Tp_IsDominantTile(tilePtr)) {
        if (*domTilePtrPtr == NIL(Tile)) {
            *domTilePtrPtr = tilePtr;
            return (0); /* countinue search */
        } else {
            return (1); /* stop search */
        }
    }
    return (0); /* continue search */
}



BearStatus
GreFindNodeOfGraph(graph, pinOrJunctPtr, nodePtr)
Gt_Graph        graph;                              /* IN       */
PinOrJunct      *pinOrJunctPtr;                     /* IN       */
Gt_GraphNode    *nodePtr;                           /* OUT      */
{
    FindData findData;

    findData.pinOrJunctPtr = pinOrJunctPtr;
    findData.node = (Gt_GraphNode) NULL;
    (void) Gt_ItrNodesOfGraph(graph, CompareNode, (ClientData) &findData);
    *nodePtr = findData.node;
    return ((*nodePtr == (Gt_GraphNode) NULL) ? BEAR_FALSE : BEAR_TRUE);
}



static BearStatus
CompareNode(node, findDataPtr)
Gt_GraphNode    node;
FindData        *findDataPtr;
{
    register PinOrJunct *pinOrJunctPtr;

    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node);
    switch (pinOrJunctPtr->unType) {
    case FPG_PIN:
        if ((findDataPtr->pinOrJunctPtr->unType == FPG_PIN) &&
            (pinOrJunctPtr->unValue.pinPtr ==
            findDataPtr->pinOrJunctPtr->unValue.pinPtr)) {

            findDataPtr->node = node;
            return (BEAR_STOP_SEARCH);
        }
        break;
    case FPG_WALL_JUNCT:
        if ((findDataPtr->pinOrJunctPtr->unType == FPG_WALL_JUNCT) &&
            (pinOrJunctPtr->unValue.junctPtr ==
            findDataPtr->pinOrJunctPtr->unValue.junctPtr)) {

            findDataPtr->node = node;
            return (BEAR_STOP_SEARCH);
        }
        break;
    default:
        Bear_PrintErr("CompareNode() : unexpected pin or junction type");
        return (BEAR_ERROR);
    }

    return (BEAR_CONTINUE_SEARCH);
}



