
/* grePickPinArea.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifndef BEAR_BLACK_BOX

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "bearGrUtils.h"
#include "grEditor.h"
#include "grEditorInt.h"

#include "../polarBear/polarBear.h"

typedef struct _PinData {
    Net         *netPtr;                /* If this is not NULL then the pin
                                         * must belong to this net.
                                         */
    PinInst     *pinInstPtr;
    int         endPoint1, endPoint2;   /* 'endPoint1' is expected to be
                                         * SMALLER then 'endPoint2'.
                                         */
    bool        horzLineSeg;
} PinData;

/* ---------- Local Procedures ---------- */
static BearStatus   FindPinInt();
static BearStatus   BoxLineOverLap();
static int          FindCell();

/* ---------- Local Global Variables ---------- */
static CellInst     *cellInstPtr;
static LineSeg      *lineSegPtr;
static Rect         searchArea;


/* ARGSUSED */
static BearStatus
BoxLineOverLap(curLineSegPtr, clientData)
LineSeg     *curLineSegPtr;
ClientData  *clientData;
{
    register Line *linePtr;

    linePtr = &curLineSegPtr->line;
    if ((linePtr->point1.p_x == linePtr->point2.p_x) &&
        (searchArea.r_xtop >= linePtr->point1.p_x) &&
        (searchArea.r_xbot <= linePtr->point1.p_x)) {

        if ((linePtr->point1.p_y > linePtr->point2.p_y) &&
            (searchArea.r_ytop <= linePtr->point1.p_y) &&
            (searchArea.r_ybot >= linePtr->point2.p_y)) {
            goto onLineSeg;
        } else if ((searchArea.r_ytop <= linePtr->point2.p_y) &&
            (searchArea.r_ybot >= linePtr->point1.p_y)) {
            goto onLineSeg;
        }
        goto notOnLineSeg;
    } else if ((searchArea.r_ytop >= linePtr->point1.p_y) &&
        (searchArea.r_ybot <= linePtr->point1.p_y)) {

        if ((linePtr->point1.p_x > linePtr->point2.p_x) &&
            (searchArea.r_xtop <= linePtr->point1.p_x) &&
            (searchArea.r_xbot >= linePtr->point2.p_x)) {
            goto onLineSeg;
        } else if ((searchArea.r_xtop <= linePtr->point2.p_x) &&
            (searchArea.r_xbot >= linePtr->point1.p_x)) {
            goto onLineSeg;
        }
        goto notOnLineSeg;
    }

notOnLineSeg:
    return (BEAR_CONTINUE_SEARCH);

onLineSeg:
    if (lineSegPtr == NIL(LineSeg)) {
        lineSegPtr = curLineSegPtr;
        return (BEAR_CONTINUE_SEARCH);
    }
    return (BEAR_STOP_SEARCH);
}



/* ARGSUSED */
static int
FindCell(tilePtr, clientData)
register Tile   *tilePtr;
ClientData      clientData;
{
    CellInst *curCellInstPtr;

    if (Tp_IsCellInstTile(tilePtr)) {
        curCellInstPtr = Tp_GetCellInstOfTile(tilePtr);
        if (cellInstPtr == NIL(CellInst)) {
            cellInstPtr = curCellInstPtr;
            return (0); /* continue search */
        } else if (cellInstPtr != curCellInstPtr) {
            return (1); /* stop search */
        }
    }
    return (0); /* continue search */
}



static BearStatus
FindPinInt(pinInstPtr, pinDataPtr)
PinInst             *pinInstPtr;
register PinData    *pinDataPtr;
{
    Point absPos;

    Mac_GetAbsPosOfPin(pinInstPtr, &absPos);
    if (pinDataPtr->horzLineSeg ? 
        ((absPos.p_x >= pinDataPtr->endPoint1) &&
        (absPos.p_x <= pinDataPtr->endPoint2)) :

        ((absPos.p_y >= pinDataPtr->endPoint1) &&
        (absPos.p_y <= pinDataPtr->endPoint2))) {

        if (pinDataPtr->pinInstPtr == NIL(PinInst)) {
            if (pinDataPtr->netPtr == NIL(Net)) {
                pinDataPtr->pinInstPtr = pinInstPtr;
            } else {
                if (pinDataPtr->netPtr == Mac_GetNetOfPin(pinInstPtr)) {
                    pinDataPtr->pinInstPtr = pinInstPtr;
                }
            }
            return (BEAR_CONTINUE_SEARCH);
        } else {
            /*
             * Another pin is on the line segment within the search area,
             * so stop the search.
             */
            return (BEAR_STOP_SEARCH);
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_FindPin() --
 *
 * Find a pin within the given span on the given line segment.  If a net
 * is given the pin must also belong to this net.
 *
 * Note that since there will often be many pins within the span, specifying
 * the pin's net (if that's posible) will greatly increase the chances of
 * a successful search.
 *
 * Results:
 *   BEAR_OK is returned and the pin pointer is filled in if everything
 *   went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_FindPin(lineSegPtr, spanPtr, netPtr, pinInstPtrPtr)
LineSeg *lineSegPtr;
Line    *spanPtr;
Net     *netPtr;
PinInst **pinInstPtrPtr;
{
    PinData pinData;
    int     a;

    if (spanPtr->point1.p_y == lineSegPtr->line.point2.p_y) {
        pinData.horzLineSeg = TRUE;
        pinData.endPoint1 = spanPtr->point1.p_x;
        pinData.endPoint2 = spanPtr->point2.p_x;
    } else {
        pinData.horzLineSeg = FALSE;
        pinData.endPoint1 = spanPtr->point1.p_y;
        pinData.endPoint2 = spanPtr->point2.p_y;
    }
    if (pinData.endPoint1 > pinData.endPoint2) {
        /* switch them */
        a = pinData.endPoint1;
        pinData.endPoint1 = pinData.endPoint2;
        pinData.endPoint2 = a;
    }

    pinData.netPtr = netPtr;
    pinData.pinInstPtr = NIL(PinInst);
    if (Mac_ItrPinOfLineSeg(lineSegPtr, FindPinInt, (ClientData) &pinData) ==
        BEAR_COMPLETE_SEARCH) {

        if ((*pinInstPtrPtr = pinData.pinInstPtr) == NIL(PinInst)) {
            PbPrint(EZ_ITALIC_FONT, "Cannot find a pin here.\n");
            return (BEAR_ERROR);
        }
    } else {
        PbPrint(EZ_ITALIC_FONT, "Cannot uniquely determine pin.\n");
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}



/*
 *----------------------------------------------------------------------------
 *
 * Gre_PickPinArea() --
 *
 * Results:
 *   BEAR_OK is returned if everything is spiffy, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   The world coordinates of the one dimensional neighborhood along the
 *   appropiate line segment within which the user is looking for a pin
 *   will be put in 'wSpanPtr' in such a way that the first point
 *   is always smaller then the second.
 * 
 *----------------------------------------------------------------------------
 */

BearStatus
Gre_PickPinArea(cellInstPtrPtr, lineSegPtrPtr, wPointPtr, wSpanPtr)
CellInst        **cellInstPtrPtr;
LineSeg         **lineSegPtrPtr;
Point           *wPointPtr;
Line            *wSpanPtr;          
{
    CellMaster  *parentCellPtr;
    ChipWin     *chipWinPtr;
    Line        *linePtr;
    PlaneData   *planeDataPtr;
    Point       screen;
    WindowData  *winDataPtr;
    double      (*invScrMatrix)[4];

    *cellInstPtrPtr = NIL(CellInst);
    *lineSegPtrPtr = NIL(LineSeg);

    if (PbGetSquare(&winDataPtr, &screen, &searchArea) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    parentCellPtr = PbGetCellMasterOfCHW(chipWinPtr);
    planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);

    /*
     * Search the horizontal tile plane and locate the cell
     * they have in mind.
     */
    cellInstPtr = NIL(CellInst);
    GeoClip(&searchArea, Tp_GetHorzPlaneBnd(planeDataPtr));
    if (TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
        &searchArea, FindCell, (ClientData) NULL) == 1 /* stop search */) {

        PbPrint(EZ_ITALIC_FONT, "Cannot uniquely determine cell instance.\n");
        return (BEAR_ERROR);
    } 
    if (cellInstPtr == NIL(CellInst)) {
        /* 
         * Pin maybe on the parent cell -- I/O pin
         */
        cellInstPtr = parentCellPtr->instList;
    }

    /* Search each line segment of the cell to locate the appropiate one. */
    lineSegPtr = NIL(LineSeg);
    if (Mac_ItrLineSeg(Mac_GetBoundaryOfCell(cellInstPtr),
        BoxLineOverLap, (ClientData) NULL) == BEAR_STOP_SEARCH) {
        PbPrint(EZ_ITALIC_FONT, "Cannot uniquely determine line segment.\n");
        return (BEAR_ERROR);
    } else if (lineSegPtr == NIL(LineSeg)) {
        PbPrint(EZ_ITALIC_FONT,
            "Area picked must include a line segment.\n");
        return (BEAR_ERROR);
    }

    /*
     * Convert the point they picked to world coordinates and shift
     * it so that it falls on the line segment.  Also compute the
     * region of the line segment that 'searchArea' covers, in such
     * a way that the first end of 'wSpanPtr' precedes the second
     * as you clockwise around the cell.
     */
    linePtr = &lineSegPtr->line;
    invScrMatrix = winDataPtr->wd_invScrMatrix;
    if (linePtr->point1.p_x == linePtr->point2.p_x) {
        /* Line segment is vertical. */
        wPointPtr->p_x = linePtr->point1.p_x;
        wPointPtr->p_y = Ez_YTrans(invScrMatrix, screen.p_x, screen.p_y);

        wSpanPtr->point1.p_x = wSpanPtr->point2.p_x = wPointPtr->p_x;
        wSpanPtr->point1.p_y = searchArea.r_ybot;
        wSpanPtr->point2.p_y = searchArea.r_ytop;
        /*
        if (linePtr->point1.p_y > linePtr->point2.p_y) {
            wSpanPtr->point1.p_y = searchArea.r_ytop;
            wSpanPtr->point2.p_y = searchArea.r_ybot;
        } else {
            wSpanPtr->point1.p_y = searchArea.r_ybot;
            wSpanPtr->point2.p_y = searchArea.r_ytop;
        }
        */
    } else {
        /* Line segment is horizontal. */
        wPointPtr->p_x = Ez_XTrans(invScrMatrix, screen.p_x, screen.p_y);
        wPointPtr->p_y = linePtr->point1.p_y;

        wSpanPtr->point1.p_y = wSpanPtr->point2.p_y = wPointPtr->p_y;
        wSpanPtr->point1.p_x = searchArea.r_xbot;
        wSpanPtr->point2.p_x = searchArea.r_xtop;
        /*
        if (linePtr->point1.p_x > linePtr->point2.p_x) {
            wSpanPtr->point1.p_x = searchArea.r_xtop;
            wSpanPtr->point2.p_x = searchArea.r_xbot;
        } else {
            wSpanPtr->point1.p_x = searchArea.r_xbot;
            wSpanPtr->point2.p_x = searchArea.r_xtop;
        }
        */
    }
/*
printf("line x1 = %d y1 = %d x2 = %d y2 = %d\n",
linePtr->point1.p_x,
linePtr->point1.p_y,
linePtr->point2.p_x,
linePtr->point2.p_y);

printf("span x1 = %d y1 = %d x2 = %d y2 = %d\n",
wSpanPtr->point1.p_x,
wSpanPtr->point1.p_y,
wSpanPtr->point2.p_x,
wSpanPtr->point2.p_y);
*/

    *cellInstPtrPtr = cellInstPtr;
    *lineSegPtrPtr = lineSegPtr;
    
    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Gre_GetPin() --
 *
 * if ((piNamePtr != (BearString) NULL) and
 *     (ciNamePtr != (BearString) NULL)) then
 *     Use the pin instance and cell instance names provided to locate
 *     the pin instance.
 * else
 *     Use the mouse to locate the pin instance.  Note that the user
 *     is given three chances to select a pin instance with the mouse.
 * endif
 *
 * Results:
 *   If everything goes well, the pin instance is drawn in the given color
 *   in every chip window, a pointer to the pin instance is filled in, and
 *   BEAR_OK is returned.  Otherwise BEAR_ERROR is returned and all bets
 *   are off.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Gre_GetPin(winDataPtr, piNamePtr, ciNamePtr, pinColor, pinPtrPtr)
WindowData  *winDataPtr;
BearString  piNamePtr;      /* pin instance name    */
BearString  ciNamePtr;      /* cell instance name   */
int         pinColor;
PinInst     **pinPtrPtr;
{
    CellInst    *cellInstPtr;
    ChipWin     *chipWinPtr;
    DrawPin     drawPin;
    Line        span;
    LineSeg     *lineSegPtr;
    PinInst     *oldPinPtr, *newPinPtr;
    Point       absPos;
    int         badPick = 0;

    *pinPtrPtr = NIL(PinInst);

    if ((piNamePtr != (BearString) NULL) &&
        (ciNamePtr != (BearString) NULL)) {
        /* 
         * Use the pin instance and cell instance names provided to locate 
         * the pin instance.
         */
        if (Mac_LookUpPinInst(piNamePtr, ciNamePtr, &newPinPtr) == FALSE) {
            return (BEAR_ERROR);
        }
    }
    else {
        /* 
         * Use the mouse to locate the pin instance.
         */
        while (TRUE) {
            if ((Gre_PickPinArea(&cellInstPtr, &lineSegPtr, &absPos,
                &span) == BEAR_OK) &&
                (Gre_FindPin(lineSegPtr, &span, NIL(Net), &newPinPtr) ==
                BEAR_OK)) {
    
                break;
            } 
            else {
                Ez_RingBell();
                if (++badPick >= 3) {
                    return (BEAR_ERROR);
                }
            }
        }
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    oldPinPtr = PbGetPinOfCHW(chipWinPtr);
    if (oldPinPtr == NIL(PinInst)) {
        PbPutPinOfCHW(newPinPtr, chipWinPtr);

        /*
         * Draw only the pin instance that the user picked by name
         * using the given color in all of the chip windows.
         */
        drawPin.pinColor = pinColor;
        drawPin.pinInstPtr = newPinPtr;
        (void) PbItrChipWindows(Gre_DrawPinTwo, (ClientData) &drawPin);
        Ez_Flush();
    } 
    else if (oldPinPtr != newPinPtr) {
        PbPutPinOfCHW(newPinPtr, chipWinPtr);

        /*
         * Redraw all of the chip windows with the new pin instance.
         */
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        Ez_Flush();
    }

    *pinPtrPtr = newPinPtr;

    return (BEAR_OK);
}

#endif /* BEAR_BLACK_BOX */


