
/* gtGraph.h --
 *
 *     George Carvalho, Esq.
 *     Wei-ming Dai
 *     June Wang
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1990 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header: gtGraph.h$
 *
 */

#ifndef _GT_GRAPH
#define _GT_GRAPH /* once-only latch */

#ifndef _MAGIC
int err1 = Need_to_include_magic_header;
#endif _MAGIC

#ifndef _BEAR
int err2 = Need_to_include_bear_header;
#endif _BEAR


typedef BearDummy *Gt_Graph;
typedef BearDummy *Gt_GraphNode;
typedef BearDummy *Gt_GraphEdge;
typedef BearDummy *Gt_AlgData;

typedef struct _Gt_PathEdge {
    Gt_GraphNode    node1;
    Gt_GraphNode    node2;
    Gt_GraphEdge    edge;
} Gt_PathEdge;

typedef struct _Gt_Path {
    lsList         pathList;
    int            pathLength;
} Gt_Path;


/* ---------- External procedures for the gtGraph module ---------- */

/* -- Main procedures -- */

extern Gt_Graph Gt_CreateGraph(/* graphId, graphData */);
/*
 * BearId       graphId;                        (* IN       *)
 * BearGeneric  graphData;                      (* IN       *)
 */

extern void
Gt_DeleteGraph(/* graph,
                * freeGraphDataProc, freeNodeDataProc, freeEdgeDataProc
                */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * void         (*freeGraphDataProc)();         (* IN       *)
 * void         (*freeNodeDataProc)();          (* IN       *)
 * void         (*freeEdgeDataProc)();          (* IN       *)
 */

extern Gt_GraphNode Gt_AddGraphNode(/* graph, nodeId, nodeData */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * BearId       nodeId;                         (* IN       *)
 * BearGeneric  nodeData;                       (* IN       *)
 */

extern BearStatus Gt_DeleteGraphNode(/* node, nodeDataPtr */);
/*
 * Gt_GraphNode node;                           (* IN       *)
 * BearGeneric  *nodeDataPtr;                   (* OUT      *)
 */

extern BearStatus
Gt_AddGraphEdge(/* node1, node2, edgeId, edgeData, edgePtr */);
/*
 * Gt_GraphNode node1, node2;                   (* IN       *)
 * BearId       edgeId;                         (* IN       *)
 * BearGeneric  edgeData;                       (* IN       *)
 * Gt_GraphEdge *edgePtr;                       (* OUT      *)
 */

extern void Gt_DeleteGraphEdge(/* edge, edgeDataPtr */);
/*
 * Gt_GraphEdge edge;                           (* IN       *)
 * BearGeneric  *edgeDataPtr;                   (* OUT      *)
 */

extern BearStatus Gt_FindEdgeOfNodes(/* node1, node2, edgePtr */);
/*
 * Gt_GraphNode node1;                          (* IN       *)
 * Gt_GraphNode node2;                          (* IN       *)
 * Gt_GraphEdge *edgePtr;                       (* OUT      *)
 */

extern void Gt_CheckGraph(/* graph, loopFlagPtr, connFlagPtr */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * bool         *loopFlagPtr;                   (* OUT      *)
 * bool         *connFlagPtr;                   (* OUT      *)
 */

extern Gt_AlgData Gt_GenAlgData(/* graph, weightProc */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * int          (*weightProc)();                (* IN       *)
 */

extern void Gt_FreeAlgData(/* graph, algDataPtr */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * Gt_AlgData   *algDataPtr;                    (* IN / OUT *)
 */

extern BearStatus
Gt_ShortestPath(/* graph, algData, node1, node2, listFlag, pathPtrPtr */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * Gt_AlgData   algData;                        (* IN       *)
 * Gt_GraphNode node1;                          (* IN       *)
 * Gt_GraphNode node2;                          (* IN       *)
 * bool         listFlag;                       (* IN       *)
 * Gt_Path      **pathPtrPtr;                   (* OUT      *)
 */

extern void Gt_FreePath(/* pathPtrPtr */);
/*
 * Gt_Path      **pathPtrPtr;                   (* IN / OUT *)
 */

extern BearStatus
Gt_MinSpanningTree(/* graph, weightProc, freeNodeDataProc, freeEdgeDataProc */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * int          (*weightProc)();                (* IN       *)
 * void         (*freeNodeDataProc)();          (* IN       *)
 * void         (*freeEdgeDataProc)();          (* IN       *)
 */

extern bool Gt_IsNodeUnconnected(/* node */);
/*
 * Gt_GraphNode node;                           (* IN       *)
 */


/* -- Get Primitives -- */

extern BearGeneric Gt_GetGraphDataOfGraph(/* graph */);
/*
 * Gt_Graph graph;                              (* IN       *)
 */

extern BearGeneric Gt_GetNodeDataOfNode(/* node */);
/*
 * Gt_GraphNode node;                           (* IN       *)
 */

extern BearGeneric Gt_GetEdgeDataOfEdge(/* edge */);
/*
 * Gt_GraphEdge edge;                           (* IN       *)
 */

extern BearId Gt_GetIdOfGraph(/* graph */);
/*
 * Gt_Graph graph;                              (* IN       *)
 */

extern BearId Gt_GetIdOfNode(/* node */);
/*
 * Gt_GraphNode node;                           (* IN       *)
 */

extern BearId Gt_GetIdOfEdge(/* edge */);
/*
 * Gt_GraphEdge edge;                           (* IN       *)
 */

extern int Gt_GetNumOfGraphNodes(/* graph */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 */

extern int Gt_GetNumOfGraphEdges(/* graph */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 */

extern Gt_GraphNode Gt_GetNodeOneOfEdge(/* edge */);
/*
 * Gt_GraphEdge edge;                           (* IN       *)
 */

extern Gt_GraphNode Gt_GetNodeTwoOfEdge(/* edge */);
/*
 * Gt_GraphEdge edge;                           (* IN       *)
 */

extern int Gt_GetDegreeOfNode(/* node */);
/*
 * Gt_GraphNode node;                           (* IN       *)
 */


/* -- Put Primitives -- */

extern BearGeneric Gt_PutNodeDataOfNode(/* nodeData, node */);
/*
 * BearGeneric     nodeData;                    (* IN       *)
 * Gt_GraphNode    node;                        (* IN       *)
 */

extern BearGeneric Gt_PutEdgeDataOfEdge(/* edgeData, edge */);
/*
 * BearGeneric     edgeData;                    (* IN       *)
 * Gt_GraphEdge    edge;                        (* IN       *)
 */


/* -- Iterators -- */

extern BearStatus Gt_ItrNodesOfGraph(/* graph, func, clientData */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * BearStatus   (*func)();                      (* IN       *)
 * ClientData   clientData;                     (* IN       *)
 */

extern BearStatus Gt_ItrEdgesOfGraph(/* graph, func, clientData */);
/*
 * Gt_Graph     graph;                          (* IN       *)
 * BearStatus   (*func)();                      (* IN       *)
 * ClientData   clientData;                     (* IN       *)
 */

extern BearStatus Gt_ItrEdgesOfNode(/* node, func, clientData */);
/*
 * Gt_GraphNode node;                           (* IN       *)
 * BearStatus   (*func)();                      (* IN       *)
 * ClientData   clientData;                     (* IN       *)
 */


/* ---------- External global variables for the gtGraph module ---------- */

extern bool gt_DebugFlag;

#endif _GT_GRAPH

