
/* gtGraphTest.c --
 *
 *     George Carvalho
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "port.h"
#include "gtBearStuff.h"
#include "gtGraph.h"
#include "list.h"
#include "tile.h"

#include "gtBearStuff.h"


/* ---------- Local Procedures ---------- */
static int GetWeight();
static int Test_MinSpanTree();


main()
{
    Test_MinSpanTree();
}


void
PrintList(list, topDownFlag)
lsList  list;
bool    topDownFlag;
{
    lsGen   gen;
    int     *c;

    if (topDownFlag) {
        lsForeachItem(list, gen, c) {
            printf ("%d", *c);
        }
    }
    else {
        lsBackeachItem(list, gen, c) {
            printf ("%d", *c);
        }
    }
    printf ("\n");
}


int
listTest1()
{
    lsGen       gen;
    lsList      list;
    int         a[10], *b, *c, i;

    list = lsCreate();
    a[0] = 1;
    a[1] = 2;
    a[2] = 3;
    a[3] = 1;
    a[4] = 2;
    a[5] = 3;
    a[6] = 1;
    a[7] = 2;
    a[8] = 3;
    a[9] = 1;
    for (i = 0; i < 10 ;i++) {
        lsNewEnd(list, (lsGeneric) &a[i], LS_NH);
    }

    PrintList(list, TRUE);

    while (lsDelBegin(list, (lsGeneric *) &b) == LS_OK) {
        gen = lsStart(list);
        while (lsNext(gen, (lsGeneric *) &c, LS_NH) == LS_OK) {
            if (*b == *c) {
                lsDelBefore(gen, (lsGeneric *) NULL);
            }
        }
        lsFinish(gen);
    
        PrintList(list, TRUE);
    }

    lsDestroy(list, (void (*)()) NULL);
}


int
CompareItems(item1, item2)
lsGeneric item1, item2;
{
    if (((int) *item1) == ((int) *item2)) {
        return (0);
    } else if (((int) *item1) < ((int) *item2)) {
        return (-1);
    }

    return (1);
}


int
listTest2()
{
    lsGen       gen;
    lsList      list;
    int         a[10], *b, *c, i;

    list = lsCreate();
    a[0] = 1;
    a[1] = 2;
    a[2] = 3;
    a[3] = 1;
    a[4] = 2;
    a[5] = 3;
    a[6] = 1;
    a[7] = 2;
    a[8] = 3;
    a[9] = 1;
    for (i = 0; i < 10 ;i++) {
        lsNewEnd(list, (lsGeneric) &a[i], LS_NH);
    }

    PrintList(list, TRUE);
    lsSort(list, CompareItems);
    lsUniq(list, CompareItems, (void (*)()) NULL);
    PrintList(list, TRUE);

    lsDestroy(list, (void (*)()) NULL);
}


int
listTest3()
{
    lsGen       gen;
    lsList      list;
    int         a[4], *b, *c, i;

    list = lsCreate();
    a[0] = 1;
    a[1] = 2;
    a[2] = 3;
    a[3] = 4;
    for (i = 0; i < 4 ;i++) {
        lsNewEnd(list, (lsGeneric) &a[i], LS_NH);
    }

    PrintList(list, TRUE);
    PrintList(list, FALSE);

    lsDestroy(list, (void (*)()) NULL);
}


int
GtTest()
{
    Gt_Graph        graph;
    Gt_GraphNode    node[10];
    Gt_GraphEdge    edge[10];
    int             i, numOfNodes, numOfEdges;
    bool            loopFlag;
    bool            connFlag;

    gt_DebugFlag = TRUE;

    graph = Gt_CreateGraph((BearId) 0, (BearGeneric) NULL);
    for (i = 0; i < 4; i++) {
        node[i] = Gt_AddGraphNode(graph, (BearId) i, (BearGeneric) NULL);
    }
    (void) Gt_AddGraphEdge(node[0], node[1], (BearId) 0,
        (BearGeneric) NULL, &edge[0]);
    (void) Gt_AddGraphEdge(node[1], node[2], (BearId) 1,
        (BearGeneric) NULL, &edge[1]);
    (void) Gt_AddGraphEdge(node[2], node[0], (BearId) 2,
        (BearGeneric) NULL, &edge[2]);
    (void) Gt_AddGraphEdge(node[2], node[3], (BearId) 3,
        (BearGeneric) NULL, &edge[3]);

    /* Try adding some illegal cases. */
    (void) Gt_AddGraphEdge(node[0], node[0], (BearId) 0,
        (BearGeneric) NULL, &edge[0]);
    (void) Gt_AddGraphEdge(node[0], node[1], (BearId) 0,
        (BearGeneric) NULL, &edge[0]);

    numOfNodes = Gt_GetNumOfGraphNodes(graph);
    numOfEdges = Gt_GetNumOfGraphEdges(graph);
    printf ("Gt_GetNumOfGraphNodes() reports that numOfNodes = %d\n",
        numOfNodes);
    printf ("Gt_GetNumOfGraphEdges() reports that numOfEdges = %d\n",
        numOfEdges);
    Gt_CheckGraph(graph, &loopFlag, &connFlag);
    Gt_DeleteGraph(graph,
        (void (*)()) NULL, (void (*)()) NULL, (void (*)()) NULL);
    printf ("\n---\n\n");

    graph = Gt_CreateGraph((BearId) 1, (BearGeneric) NULL);
    for (i = 0; i < 4; i++) {
        node[i] = Gt_AddGraphNode(graph, (BearId) i, (BearGeneric) NULL);
    }
    (void) Gt_AddGraphEdge(node[0], node[1], (BearId) 0,
        (BearGeneric) NULL, &edge[0]);
    (void) Gt_AddGraphEdge(node[0], node[2], (BearId) 1,
        (BearGeneric) NULL, &edge[1]);
    (void) Gt_AddGraphEdge(node[1], node[2], (BearId) 2,
        (BearGeneric) NULL, &edge[2]);
    (void) Gt_AddGraphEdge(node[2], node[3], (BearId) 3,
        (BearGeneric) NULL, &edge[3]);
    Gt_CheckGraph(graph, &loopFlag, &connFlag);
    Gt_DeleteGraph(graph,
        (void (*)()) NULL, (void (*)()) NULL, (void (*)()) NULL);
    printf ("\n---\n\n");

    graph = Gt_CreateGraph((BearId) 2, (BearGeneric) NULL);
    for (i = 0; i < 5; i++) {
        node[i] = Gt_AddGraphNode(graph, (BearId) i, (BearGeneric) NULL);
    }
    (void) Gt_AddGraphEdge(node[0], node[1], (BearId) 0,
        (BearGeneric) NULL, &edge[0]);
    (void) Gt_AddGraphEdge(node[0], node[2], (BearId) 1,
        (BearGeneric) NULL, &edge[1]);
    (void) Gt_AddGraphEdge(node[2], node[3], (BearId) 2,
        (BearGeneric) NULL, &edge[2]);
    (void) Gt_AddGraphEdge(node[4], node[2], (BearId) 3,
        (BearGeneric) NULL, &edge[3]);
    (void) Gt_AddGraphEdge(node[2], node[1], (BearId) 4,
        (BearGeneric) NULL, &edge[4]);
    Gt_CheckGraph(graph, &loopFlag, &connFlag);
    Gt_DeleteGraph(graph,
        (void (*)()) NULL, (void (*)()) NULL, (void (*)()) NULL);
    printf ("\n---\n\n");

    graph = Gt_CreateGraph((BearId) 3, (BearGeneric) NULL);
    for (i = 0; i < 5; i++) {
        node[i] = Gt_AddGraphNode(graph, (BearId) i, (BearGeneric) NULL);
    }
    (void) Gt_AddGraphEdge(node[0], node[1], (BearId) 0,
        (BearGeneric) NULL, &edge[0]);
    (void) Gt_AddGraphEdge(node[0], node[2], (BearId) 1,
        (BearGeneric) NULL, &edge[1]);
    (void) Gt_AddGraphEdge(node[2], node[3], (BearId) 2,
        (BearGeneric) NULL, &edge[2]);
    Gt_CheckGraph(graph, &loopFlag, &connFlag);
    Gt_DeleteGraph(graph,
        (void (*)()) NULL, (void (*)()) NULL, (void (*)()) NULL);
    printf ("\n---\n\n");
}


static int
GetWeight(edge)
Gt_GraphEdge edge;
{
    return ((int) Gt_GetEdgeDataOfEdge(edge));
}


static int
Test_MinSpanTree()
/*
 * Test minimum spanning tree code.
 */
{
    Gt_Graph        graph;
    Gt_GraphNode    node[10];
    Gt_GraphEdge    edge[10];
    int             i;

    graph = Gt_CreateGraph((BearId) 0, (BearGeneric) NULL);
    for (i = 0; i < 4; i++) {
        node[i] = Gt_AddGraphNode(graph, (BearId) i, (BearGeneric) NULL);
    }
    (void) Gt_AddGraphEdge(node[0], node[1], (BearId) 0,
        (BearGeneric) 1, &edge[0]);
    (void) Gt_AddGraphEdge(node[1], node[2], (BearId) 1,
        (BearGeneric) 2, &edge[1]);
    (void) Gt_AddGraphEdge(node[2], node[0], (BearId) 2,
        (BearGeneric) 4, &edge[2]);
    (void) Gt_AddGraphEdge(node[2], node[3], (BearId) 3,
        (BearGeneric) 1, &edge[3]);
    (void) Gt_AddGraphEdge(node[0], node[3], (BearId) 4,
        (BearGeneric) 1, &edge[4]);

    (void) fprintf (stderr,
        "Gt_GetNumOfGraphNodes() reports that numOfNodes = %d\n",
        Gt_GetNumOfGraphNodes(graph));
    (void) fprintf (stderr,
        "Gt_GetNumOfGraphEdges() reports that numOfEdges = %d\n\n",
        Gt_GetNumOfGraphEdges(graph));

    if (Gt_MinSpanningTree(graph, GetWeight, (void (*)()) NULL) ==
        BEAR_ERROR) {
        (void) fprintf (stderr, "minimum spanning tree failed\n");
        exit (-1);
    }
}


