
/* cpaTest.c --
 *
 *     Benjamin Chen
 *     (Advising Professor - E. S. Kuh)
 *
 *     Electronics Research Laboratory
 *     University of California, Berkeley
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifdef CPA_BEAR
#include "bearCopyRight.h"
#endif CPA_BEAR
#include <stdio.h>
#include <string.h>
#include "cpa.h"
#include "cpaInt.h"
#include "cpaList.h"
#include "sort.h"


/* ---------- Local Procedures ---------- */
static void PrintProgUsage();


/* ---------- Local Global Variables ---------- */
static char *progName = (char *) NULL;


main (argc, argv)
int     argc;
char    **argv;
/*
 * Results:
 *    A zero exit status means everything went brilliantly.
 *    A non-zero exit status means the program failed.
 */
{    
    int i;
    
    /*
     * Determine the program name.
     */
    if ((progName = strrchr (argv[0], (int) '/')) == (char *) NULL) {
        progName = argv[0];
    }
    else {
        progName++;
    }

    if (argc == 1) {
        PrintProgUsage();
    }

    cpaInFiName = (char *) NULL;
    cpaOutFiName = (char *) NULL;

    for (i = 1; i < argc ;i++) {
        if (strcmp(argv[i], "-help") == 0) {
            PrintProgUsage();
        }

        if (strcmp(argv[i], "-in") == 0) {
            if (++i >= argc) {
                PrintProgUsage();
            }
            else {
                MALLOC(char *, cpaInFiName,
                    (strlen(argv[i]) + 1) * sizeof(char));
                strcpy(cpaInFiName, argv[i]);
            }
            continue;
        }

        if (strcmp(argv[i], "-out") == 0) {
            if (++i >= argc) {
                PrintProgUsage();
            }
            else {
                MALLOC(char *, cpaOutFiName,
                    (strlen(argv[i]) + 1) * sizeof(char));
                strcpy(cpaOutFiName, argv[i]);
            }
            continue;
        }

        PrintProgUsage();
    }

    if (cpaInFiName == (char *) NULL) {
        (void) fprintf (stderr, "Error : Input file name is missing.\n");
        KB_EXIT(-1);
    }
    if (cpaOutFiName == (char *) NULL) {
        (void) fprintf (stderr, "Error : Output file name is missing.\n");
        KB_EXIT(-1);
    }

#ifdef USE_OCT_CODE
    (void) VOVbegin(argc, argv);
#endif /* USE_OCT_CODE */

    initscanner();

#ifdef USE_OCT_CODE
    (void) VOVinput(VOV_UNIX_FILE, cpaInFiName);
#endif /* USE_OCT_CODE */

    if ((yyin = fopen (cpaInFiName, "r")) == (FILE *) NULL) {
        (void) fprintf (stderr,
            "Could not open file named \"%s\" for reading.\n",
            cpaInFiName);
        KB_EXIT(-1);
    }

    if (yyparse()) {
        fprintf(stderr, "Compilation halted because of syntax errors.\n");
        KB_EXIT(-1);
    }

    SortNetList(topNetList);
    SortNetList(bottomNetList);
    ArrangePins(theChannel, topNetList, bottomNetList);
    PrintChannel(theChannel);

    KB_EXIT(0);
}


static void
PrintProgUsage()
{
    fprintf (stderr, "Usage: %s [options ...]\n\n", progName);
    fprintf (stderr, "where options include:\n");
    fprintf (stderr,
        "    -help                        print out this message\n");
    fprintf (stderr,
        "    -in filename                 input file name\n");
    fprintf (stderr,
        "    -out filename                output file name\n");
    fprintf (stderr, "\n");
    KB_EXIT(-1);
}


