#include <stdio.h>
#include <ctype.h>
#include "y.tab.h"
#include "cpa.h"
#include "cpaInt.h"

/*
 * This file contains routines to build the reserved word table
 * and perform searches into the reserved word table.
 */

#define LOWER(x)    (isupper(x) ? tolower(x) : x)

static char *ResWords[] = {"channel", "height", "spacing", "top",
               "bottom", "net", "pin", "left", "right", 
               "both", "none", "no_fixed", "top_fixed",
               "bottom_fixed", "both_fixed", ""};

static int RWTokens[] = {CHANNEL, HEIGHT, SPACING, TOP_L, BOTTOM_L,
             NET, PIN, LEFT, RIGHT, BOTH_EX, NO_EX, NO_FIXED,
             TOP_FIXED, BOTTOM_FIXED, BOTH_FIXED, 0};

struct RW {
    char *word;
    int token;
    struct RW *next;
} *RWTable[27];     /* one entry for each letter and for '_' */


/* Get string corresponding to token */
/* slow, but won't be used in final version */
char *
GetTString(token)
     int token;
{
    register int i;

    if (token < 258 || token > 276)  {
        yyerror("Illegal token");
        KB_EXIT(1);
    }

    for (i = 0; RWTokens[i] != token && RWTokens[i] != 0; i++) ;

    if (RWTokens[i] == token)
        return(ResWords[i]);

    yyerror("Illegal token");
    KB_EXIT(1);
}

void
PutInRWTable(word, token)
     char *word;
     int token;
{
    struct RW *new;
    struct RW **row;

    new = (struct RW *)malloc(sizeof(struct RW));
    new->word = word;
    new->token = token;
    if (word[0] == '_')
        row = &(RWTable[26]);
    else
        row = &(RWTable[LOWER(word[0]) - 'a']);
    new->next = *row;
    *row = new;
}

/*
 * Allocate the reserved word table and associate semantic values
 * with their corresponding tokens.
 */
void
InitRWTable()
{
    register int i;

    for (i = 0; i < 27; i++)
        RWTable[i] = NULL;

    for (i = 0; ResWords[i][0] != '\0' && RWTokens[i] != 0; i++)
        PutInRWTable(ResWords[i], RWTokens[i]);
}

/*
 * This routine checks to see if 'word' is in the reserved word table.
 * If it is, then return its token, else return 0
 */
InRWTable(word)
     char *word;
{
    struct RW *row;

    if (!isalpha(word[0]) && word[0] != '_')  {
        yyerror("Illegal identifier");
        KB_EXIT(1);
    }
    row = RWTable[LOWER(word[0]) - 'a'];
    while (row != NULL)  {
        if (!strcmp(row->word, word))  {
            return(row->token);
        }
        row = row->next;
    }
    return(0);
}
