/*
 * Scanner utility routines.
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "y.tab.h"
#include "cpa.h"
#include "cpaInt.h"


extern int      yylineno;


/*
 * Open the file to be scanned and initialize the reserved word and
 * string table.
 */
void
initscanner()
{
    InitRWTable();      /* Initialize reserved word table */
}

ScanIntConst(cp)
     char *cp;
{
    yylval.i = atoi(cp);
    return(INTCONST);
}

/* 
 * Returns the token associated with a given identifier and sets yylval
 * to the appropriate semantic value.
 */
ScanStringConst(cp)
char *cp;
{
    int token;

    switch (token = InRWTable(cp))  {
    case 0:     /* identifier */
        MALLOC(char *, yylval.s, (strlen(cp) + 1) * sizeof(char));
        strcpy(yylval.s, cp);
        return(IDENT);
    case NO_FIXED:
        yylval.i = (int) NO_FIX;
        return(token);
    case TOP_FIXED:
        yylval.i = (int) TOP;
        return(token);
    case BOTTOM_FIXED:
        yylval.i = (int) BOTTOM;
        return(token);
    case BOTH_FIXED:
        yylval.i = (int) BOTH_FIX;
        return(token);
    case LEFT:
        yylval.i = (int) LEFT_EXIT;
        return(token);
    case RIGHT:
        yylval.i = (int) RIGHT_EXIT;
        return(token);
    case BOTH_EX:
        yylval.i = (int) BOTH_EXITS;
        return(token);
    case NO_EX:
        yylval.i = (int) NO_EXITS;
        return(token);
    default:
        return(token);
    }
}

/* Warning printing function */
void
yywarning(s)
     char *s;
{
    fprintf(stderr, "WARNING:  %s in line %d.\n", s, yylineno);
}

void
yyerror(s)
    char *s;
{
    if (s != (char *) NULL) {
    printf("%s", s);
    } else {
    printf("Syntax error");
    }
    /* printf(" near line %d. \n", yylloc.first_line); */
    printf(" near line %d. \n", yylineno);
}
