
/* localSpacerInt.h --
 *
 *     Xiao-Ming Xiong
 *     (Professor E. S. Kuh)
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 * needs: localRouter.h
 * 
 * rcsid $Header$
 */

/* head file for channel spacer -- Nutcracker */

#ifndef LOCAL_SPACER_INT
#define LOCAL_SPACER_INT

#define signal Signal

#define MAX_NODES	2500	/*maximum number of nodes allowed*/

#if MAX_NODES <= 255		/*NODE must be able to hold MAX_NODES*/
    typedef unsigned char NODE;	/*type for node numbers*/
#else
    typedef unsigned int NODE;
#endif

#define DIE	22
#define FALSE	0
#define TRUE	1
#define HEAD	1
#define TAIL	2
#define LEFT	1
#define RIGHT	2
#define TOP	3
#define BOTTOM	4
#define MID	5
#define VIR	6
#define UP 	1
#define DOWN	2
#define BOUND   1
#define WIRE	2
#define	VIA	3
#define TERM	4
#define JOG	5
#define UNTOUCH 0
#define ACTIVE	1
#define FINISH	2
#define INF	100000000

#define SUCCESS 1
#define FAILED	 -1
#define ODD(X)	((X)&1)		/*returns TRUE if X odd*/
#define MAX(x, y) ( (x > y) ? x : y )
#define MIN(x, y) ( (x > y) ? y : x )
#define COMP(x, y) ( (x = y) ? 1 : 0 )	/* return 1 if x equals y */

typedef	char * pointer;
typedef struct nut_node nut_node;
typedef struct nut_jog nut_jog;
typedef struct glocxy glocxy;

int 	hfirst, tsecond;
int	channel_height, sp2, v_clear, v_wid;
int	l_term, r_term, t_bound, b_bound;
int	nut_amount;
struct  glocxy *bound;

struct glocxy {
	int 		dflag;
	locxy		*head;
	locxy		*tail;
	struct glocxy	*next;
};

struct nut_jog {
	int		flag;		/* flag of the jog:
					   LEFT
					   RIGHT
					   MID;
					*/
	int 		type;		/* type of the jog:
					   UP
					   DOWN
					*/
	int 		status;		/* used in scannig:
					   UNTOUCH -- not sacn yet
					   ACTIVE  -- under scanning
					   FINISH  -- scanned
					*/
	locxy		coord;		/* coordinates of the jog */
	nut_node	*left_node;	/* pointer to the node in the
					   left side */
	nut_node	*right_node;	/* pointer to the node in the
					   right side */
	nut_jog *scan_next;
	nut_jog *scan;	/* pointer to the next jog
					   on the same scan line */
	nut_jog *next;			/* pointer to the next jog
					   in jog linked list */
};

struct nut_jog_list {
	int		num_jogs;
	nut_jog *jog_root;
} nut_jog_list;


extern nut_jog *nut_add_jog( /* lp, rp, flag, coord */ );
/*
nut_node	*lp, *rp;	pointer to the points defining it
int	flag;	type of point
locxy	coord;		coordinate of the point
*/

extern nut_node *nut_sort( /*list*/ );
/*
nut_node *list;		list to sorted
*/

extern nut_node *nut_node_sort( /*list*/ );
/*
nut_node *list;
*/

struct nut_node {
	int		flag;		/* type of the node:
					   BOUND
					   WIRE
					   VIA
					   TERM
					   JOG
					*/
	int		status;		/* status of the node:
					   UNTOUCH
					   FINISH
					*/
	int		dflag;		/* direction flag for boundary
					   and float terminals */
	int		weight;
	int		length;
	locxy 	key1, key2;
	locxy 	coord1, coord2;
	root		*hsr;
	nut_node	*tailnode;
	nut_node	*headnode;
	nut_node	*above, *below;
	nut_node 	*nextnode;	/* pointer to the next
	node in the linked list */
	pointer 		obj;		/* pointer to actual
	routing objects in database */
};

struct nut_node_list {
	int		num_nodes;
	nut_node	*node_root;
} nut_node_list;

extern nut_node *nut_add_node( /* pnt, node_flag */ );
/*
pointer	pnt;		pointer to a wire, a via or a jog which is
			the actual routing object in the node
int	node_flag;	type of routing object in the node
*/

#endif LOCAL_SPACER_INT

