/* macDCell.h --
 *
 *      June Wang
 *      Wei-ming Dai
 *      (Professor E. S. Kuh)
 *
 * It defines the structure of a dummy cell and a dummy cell list.
 * Also, it contains the declarations of the subroutines doing inserting,
 * deleting, and creating.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header: macDCell.h,v 1.1 87/02/04 14:05:30 weiming Exp $
 */

#ifndef _MAC_DCELL
#define _MAC_DCELL

typedef enum _DCellClass                /* dummy cell class                 */
{
    HBNTI_AND_VBNTI,                    /* horizontal bottleneck tile and
                                         * vertical bottleneck tile.
                                         */
    HBNTI_AND_VBNLN,                    /* horizontal bottleneck tile and
                                         * vertical bottleneck line.
                                         */
    HBNLN_AND_VBNTI,                    /* horizontal bottleneck line and
                                         * vertical bottleneck tile.
                                         */
    HBNLN_AND_VBNLN                     /* horizontal bottleneck line and
                                         * vertical bottleneck line.
                                         */
} DCellClass; 

typedef struct _DummyCell
{
    BearId            id;              /* Dummy cell id                     */
    DCellClass        class;
    Rect              bbox;            /* Dummy cell bounding box and shape */
    LinkedRect        *horzRectList;   /* List of horizontally sliced
                                        * rectangles
                                        */
    LinkedRect        *vertRectList;   /* List of vertically sliced
                                        * rectangles.
                                        */
    Transform         transform;       /* Transformation suitable for
                                        * placing the dummy cell on the 
                                        * horzizontal tile plane.
                                        */
    struct _PseudoPin *topPPinList;    /* Pseudo pin list on the top side     */
    struct _PseudoPin *bottomPPinList; /* Pseudo pin list on the bottom side  */
    struct _PseudoPin *leftPPinList;   /* Pseudo pin list on the left side    */
    struct _PseudoPin *rightPPinList;  /* Pseudo pin list on the right side   */
    Tile              *tilePtr;        /* The solid tile for the dummy cell   */
    Fpg_Room          *roomPtr;
    struct _DummyCell *nextPtr;        /* Pointer to another linked dummy cell*/
} DummyCell;

/* ------------ Dummy Cell Primitives -------------- */

extern BearStatus Mac_CreateDCell(/* dcellList, dcell */);
/*
 *  DummyCell **dcellList;
 *  DummyCell **dcell;
 */

extern BearStatus Mac_DeleteDCell(/* dcellListPtr, dcellPtr */);
/*
 * DummyCell *dcellPtr;
 * DummyCell **dcellListPtr;
 */

extern void Mac_DeleteDCellList(/* dcellListPtr */);
/*
 * DummyCell **dcellListPtr;
 */

extern BearStatus Mac_InsertPPinOfDCell(/* dcellPtr, side, ppinPtr */);
/*
 *  DummyCell       *dcellPtr;
 *  SideDirection   side;
 *  PseudoPin       *ppinPtr;
 */

extern BearStatus Mac_DeletePPinOfDCell(/* dcellPtr, side, ppinPtr */);
/*
 *  DummyCell       *dcellPtr;
 *  SideDirection   side;
 *  PseudoPin       *ppinPtr;
 */

extern void Mac_RemovePPinListOfDCell(/* dcellPtr, side, ppinListPtr */);
/*
 *  DummyCell       *dcellPtr;
 *  SideDirection   side;
 *  PseudoPin       **ppinListPtr;
 */

extern void Mac_AppendPPinListOfDCell(/* ppinList, dcellPtr, side */);
/*
 *  PseudoPin       *ppinList;
 *  DummyCell       *dcellPtr;
 *  SideDirection   side;
 */

#endif _MAC_DCELL

