
/* macDCellItr.c --
 *
 *      June Wang
 *      (Advising Professor - E. S. Kuh)
 *
 * It contains all the iterators related to a dummy cell.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: macDCellItr.c,v 1.1 87/02/04 15:33:19 weiming Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "macInt.h"


/*
 * ----------------------------------------------------------------------------
 * Mac_ItrDCell() --
 *  Given a list dummy cells, it iterates all of the dummy cells
 *  in that list.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, when successful.
 *   BEAR_STOP_SEARCH, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrDCell(dCellList, func, clientData)
DummyCell  *dCellList;
BearStatus (*func)();
ClientData clientData;

{
    DummyCell  *curDCellPtr;

#ifdef DEBUG
    if (func == (BearStatus (*)()) NULL)
    {
        MacPrintFatalErr("Mac_ItrMCell() failed: no function is given");
    }
#endif DEBUG

    if (dCellList == NIL(DummyCell))
    {
        return (BEAR_COMPLETE_SEARCH);
    }
    else
    {
        curDCellPtr = dCellList;
        while (curDCellPtr != NIL(DummyCell))
        {
            if (((*func)(curDCellPtr, clientData)) == BEAR_STOP_SEARCH)
            {
                return (BEAR_STOP_SEARCH);
            }
            curDCellPtr = curDCellPtr->nextPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 * Mac_ItrPPinOfDCell() --
 *  Given a dummy cell, it fetches the list of pseudo pins from the
 *  designated side of the dummy cell.
 *  Then, it iterates the all of the pseudo pins in that list.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, when successful.
 *   BEAR_STOP_SEARCH, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrPPinOfDCell(dCellPtr, side, func, clientData)
DummyCell  *dCellPtr;
SideDirection side;
BearStatus (*func)();
ClientData clientData;

{
    PseudoPin *curPPinPtr; 

#ifdef DEBUG
    if (dCellPtr == NIL(DummyCell))
    {
        MacPrintFatalErr("Mac_ItrMCell() failed: no dummy cell is given");
    }
    else if (func == (BearStatus (*)()) NULL)
    {
        MacPrintFatalErr("Mac_ItrMCell() failed: no function is given");
    }
#endif DEBUG

    switch (side)
    {
        case LEFT_SIDE:
            curPPinPtr = dCellPtr->leftPPinList;
            break;
        case RIGHT_SIDE:
            curPPinPtr = dCellPtr->rightPPinList;
            break;
        case TOP_SIDE:
            curPPinPtr = dCellPtr->topPPinList;
            break;
        case BOTTOM_SIDE:
            curPPinPtr = dCellPtr->bottomPPinList;
            break;
    }
    if (curPPinPtr == NIL(PseudoPin))
    {
        return (BEAR_COMPLETE_SEARCH);
    }
    else
    {
        while (curPPinPtr != NIL(PseudoPin))
        {
            if (((*func)(curPPinPtr, clientData)) == BEAR_STOP_SEARCH)
            {
                return (BEAR_STOP_SEARCH);
            }
            curPPinPtr = curPPinPtr->nextPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}



