
/* macGPin.c --
 *
 *     Wei-Ming Dai
 *     (Advising Professor - E. S. Kuh)
 *
 *    Basic operations related to generic pins.
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "macInt.h"

/* ---------- Local Procedures ---------- */

GenericPin      *MakeGPin();

/* 
 * ----------------------------------------------------------------------------
 *
 * MakeGPin() --
 *
 *    Allocates memory for a new generic pin, and initializes
 *    the fields.
 *
 * Results:
 *    a generic pin.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static GenericPin *
MakeGPin()
{
    GenericPin *gPinPtr;

    MALLOC(GenericPin *, gPinPtr, sizeof (GenericPin));
    gPinPtr->pinType = PIN_INST;
    gPinPtr->pinPtrUn.pinInstPtr = NIL(PinInst);

    return (gPinPtr);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_CreateGPin() --
 *
 *    Create a generic pin of specified pin type. 
 *
 * Results:
 *    Newly created generic pin.
 *    BEAR_OK, if successful; BEAR_ERROR, otherwise.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus 
Mac_CreateGPin(pinType, pinPtr, gpinPtrPtr)
PinType     pinType;
ClientData  pinPtr;
GenericPin  **gpinPtrPtr;
{
    GenericPin *gpinPtr;

#ifdef DEBUG
    if (pinPtr == (ClientData) NULL) {
        MacPrintFatalErr("Mac_CreateGPin() failed: no pin is given");
    }
#endif DEBUG

    switch (pinType) {
    case PIN_INST:
        gpinPtr = MakeGPin();
        gpinPtr->pinType = PIN_INST;
        gpinPtr->pinPtrUn.pinInstPtr = (PinInst *) pinPtr;
        break;

    case ROUTE_PIN:
        gpinPtr = MakeGPin();
        gpinPtr->pinType = ROUTE_PIN;
        gpinPtr->pinPtrUn.routePinPtr = (RoutePin *) pinPtr;
        break;

    default:
        MacPrintFatalErr("Mac_CreateGPin() failed: illegal pin type");
        return (BEAR_ERROR);
    }

    *gpinPtrPtr = gpinPtr;

    return (BEAR_OK);
}


