
/* macGeometry.c --
 *
 *     Wei-Ming Dai
 *     (Advising Professor - E. S. Kuh)
 *
 * This is the extension of geometry.c. 
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "macInt.h"

/*
 * ----------------------------------------------------------------------------
 *
 * MakeLinkedPoint() --
 *
 * Allocates a memory space for a pointer to a linked point. 
 *
 * Results:
 *   A pointer to a linked point.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

static LinkedPoint *
MakeLinkedPoint()
{
    LinkedPoint *lnPointPtr;

    MALLOC(LinkedPoint *, lnPointPtr, sizeof (LinkedPoint));

    return (lnPointPtr);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_CreateLinkedPoint() --
 *
 * Create a linked point and insert it to a given linked point list.
 * If 'pointPtrPtr' is non-zero, it will be filled in with a pointer
 * to the newly created point.
 *
 * Note that the linked point should be freed by the user when they
 * are done with it, but the pointer to the point itself must not be
 * freed.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   A new linked point is inserted to the given linked point list. 
 * ----------------------------------------------------------------------------
 */

void
Mac_CreateLinkedPoint(xCoord, yCoord, lnPointListPtr, pointPtrPtr)
int         xCoord;                             /* IN       */
int         yCoord;                             /* IN       */
LinkedPoint **lnPointListPtr;                   /* IN / OUT */
Point       **pointPtrPtr;                      /* OUT      */
{
    LinkedPoint *lnPointPtr;
    
#ifdef DEBUG
    if (lnPointListPtr == (LinkedPoint **) NULL) {
        MacPrintFatalErr("Mac_CreateLinkedPoint() : failed");
    }
#endif DEBUG

    lnPointPtr = MakeLinkedPoint();

    lnPointPtr->point.p_x = xCoord;
    lnPointPtr->point.p_y = yCoord;
    lnPointPtr->nextPtr = *lnPointListPtr;
    *lnPointListPtr = lnPointPtr;

    if (pointPtrPtr != (Point **) NULL) {
        *pointPtrPtr = &lnPointPtr->point;
    }
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_CreatePointListFromRect() --
 *
 * Create a linked point list consisting of each of the four
 * corners in the given rectangle.  A pointer to a pointer to
 * the newly created linked point list is returned.
 *
 * Note that the memory for each point is allocated separately
 * on purpose so that the point list may be freed one element
 * at a time.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Mac_CreatePointListFromRect(rectPtr, pointListPtr)
register Rect   *rectPtr;
LinkedPoint     **pointListPtr;
{
    register LinkedPoint *lpPtr;

    MALLOC(LinkedPoint *, lpPtr, sizeof (LinkedPoint));
    lpPtr->point.p_x = rectPtr->r_xtop;
    lpPtr->point.p_y = rectPtr->r_ybot;
    lpPtr->nextPtr = NIL(LinkedPoint);
    *pointListPtr = lpPtr;

    MALLOC(LinkedPoint *, lpPtr, sizeof (LinkedPoint));
    lpPtr->point.p_x = rectPtr->r_xtop;
    lpPtr->point.p_y = rectPtr->r_ytop;
    lpPtr->nextPtr = *pointListPtr;
    *pointListPtr = lpPtr;

    MALLOC(LinkedPoint *, lpPtr, sizeof (LinkedPoint));
    lpPtr->point.p_x = rectPtr->r_xbot;
    lpPtr->point.p_y = rectPtr->r_ytop;
    lpPtr->nextPtr = *pointListPtr;
    *pointListPtr = lpPtr;
    
    MALLOC(LinkedPoint *, lpPtr, sizeof (LinkedPoint));
    lpPtr->point.p_x = rectPtr->r_xbot;
    lpPtr->point.p_y = rectPtr->r_ybot;
    lpPtr->nextPtr = *pointListPtr;
    *pointListPtr = lpPtr;
}



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_IsPointListCW() --
 *
 * Check to see if the point list is in clockwise order.
 *
 * Results:
 *   BEAR_TRUE is returned if the point list is in clockwise order,
 *   BEAR_FALSE otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_IsPointListCW(pointList)
LinkedPoint *pointList;

{
/*
    LinkedPoint *pointPtr;
    Point       point1, point2;
    bool        havePointTwo = FALSE;

*/
#ifdef DEBUG
    if (pointList == NIL(LinkedPoint)) 
    {
        MacPrintFatalErr("Mac_IsPointListCW() failed: no point list given");
    }
#endif DEBUG

    MacPrintFatalErr("Mac_IsPointListCW() failed: Don't call me I have a bug");

/*
    point1.p_x = pointList->point.p_x;
    point1.p_y = pointList->point.p_y;
    for (pointPtr = pointList;
        pointPtr != NIL(LinkedPoint);
        pointPtr = pointPtr->nextPtr)
    {
        if (pointPtr->point.p_y > point1.p_y) {
            point1.p_x = pointPtr->point.p_x;
            point1.p_y = pointPtr->point.p_y;
            if (pointPtr->point.p_y == point1.p_y) {
        }
    }   
    if (havePointTwo == FALSE) {
        point2.p_x = pointPtr->point.p_x;
        point2.p_y = pointPtr->point.p_y;
    }

    return ((point1.p_x < point2.p_x) ? BEAR_TRUE : BEAR_FALSE);
*/
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_IsBoundaryCW () --
 *
 * Check to see if the boundary is in clockwise order.
 *
 * Results:
 *   BEAR_TRUE is returned if the boundary is in clockwise order,
 *   BEAR_FALSE otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_IsBoundaryCW(boundaryList)
LinkedLineSeg *boundaryList;

{
    register LinkedLineSeg  *llsPtr;
    register Line           *linePtr;
    Line                    *topLinePtr;

#ifdef DEBUG
    if (boundaryList == NIL(LinkedLineSeg)) 
    {
        MacPrintFatalErr("Mac_IsBoundaryCW() failed: no boundary given");
    }
#endif DEBUG

    topLinePtr = &boundaryList->lineSeg->line;
    for (llsPtr = boundaryList;
        llsPtr != NIL(LinkedLineSeg);
        llsPtr = llsPtr->nextPtr)
    {
        linePtr = &llsPtr->lineSeg->line;
        if (linePtr->point1.p_y == linePtr->point2.p_y)
        {
            if ((linePtr->point1.p_y >= topLinePtr->point1.p_y)
                || (topLinePtr->point1.p_y != topLinePtr->point2.p_y))
            {
                topLinePtr = linePtr;
            }
        }
    }

    return ((topLinePtr->point1.p_x < topLinePtr->point2.p_x)
      ? BEAR_TRUE : BEAR_FALSE);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ReversePointList() --
 *
 * The linked list of points is reversed in place.
 * In case the list is NIL, do nothing.
 *
 * Results:
 *   BEAR_OK, when done.
 *
 * Side effects:
 *   The list is in reverse order.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ReversePointList(pointListPtr)

LinkedPoint **pointListPtr;
{
    LinkedPoint *curPointPtr;
    LinkedPoint *nextPointPtr;

    if (*pointListPtr != NIL(LinkedPoint)) 
    {
        curPointPtr = *pointListPtr;
        *pointListPtr = NIL(LinkedPoint);
        while (curPointPtr != NIL(LinkedPoint))
        {
            nextPointPtr = curPointPtr->nextPtr;
            curPointPtr->nextPtr = *pointListPtr;
            *pointListPtr = curPointPtr;
            curPointPtr = nextPointPtr;
        }   
    }
    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_CopyPointList() --
 *
 * A copy of the linked list is created and returned.
 *
 * Results:
 *   BEAR_OK, when done.
 *
 * Side effects:
 *   A copy of the old list is made.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_CopyPointList(oldList, newListPtr)

LinkedPoint *oldList;
LinkedPoint **newListPtr;
{
    register LinkedPoint *oldPointPtr;
    register LinkedPoint *newPointPtr;

#ifdef DEBUG
    if (oldList == NIL(LinkedPoint)) 
    {
        MacPrintFatalErr("Mac_CopyPointList() failed: no point list is given");
    }
#endif DEBUG
    
    oldPointPtr = oldList;
    newPointPtr = *newListPtr = MakeLinkedPoint();
    while (oldPointPtr->nextPtr != NIL(LinkedPoint))
    {
        newPointPtr->point.p_x = oldPointPtr->point.p_x;
        newPointPtr->point.p_y = oldPointPtr->point.p_y;
        newPointPtr = newPointPtr->nextPtr = MakeLinkedPoint();
        oldPointPtr = oldPointPtr->nextPtr;
    }
    newPointPtr->point.p_x = oldPointPtr->point.p_x;
    newPointPtr->point.p_y = oldPointPtr->point.p_y;
    newPointPtr->nextPtr = NIL(LinkedPoint);
    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_MergePointLists () --
 *
 * Two lists are merged and a new pointer to the merged list is returned.
 *
 * Results:
 *   BEAR_OK, when done.
 *
 * Side effects:
 *   The two lists are merged.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_MergePointLists (pointList1, pointList2, newPointListPtr)

LinkedPoint *pointList1;
LinkedPoint *pointList2;
LinkedPoint **newPointListPtr;
{
    LinkedPoint *pointPtr;

    if (pointList1 == NIL(LinkedPoint))
    {
    *newPointListPtr = pointList2;
    }
    else if (pointList2 == NIL(LinkedPoint))
    {
    *newPointListPtr = pointList1;
    }
    else
    {
    pointPtr = *newPointListPtr = pointList1;
    while (pointPtr->nextPtr != NIL(LinkedPoint))
    {
        pointPtr = pointPtr->nextPtr;
    }
    pointPtr->nextPtr = pointList2;
    }
    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_DeleteLinkedPoint() --
 *
 * Delete a linked point from a given point list.
 *
 * Results:
 *   BEAR_OK, when done; BEAR_ERROR, if the point is not in the list.
 *
 * Side effects:
 *   The linked point gets deleted from the point list.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_DeleteLinkedPoint(pointPtr, pointListPtr)

LinkedPoint *pointPtr;
LinkedPoint **pointListPtr;
{
    LinkedPoint *curLinkedPointPtr;

#ifdef DEBUG
    if (pointPtr == NIL(LinkedPoint)) 
    {
        MacPrintFatalErr("Mac_DeleteLinkedPoint() failed: no point is given");
    }
    else if (*pointListPtr == NIL(LinkedPoint)) 
    {
        MacPrintFatalErr("Mac_DeleteLinkedPoint() failed: no point list is given");
    }
#endif DEBUG

    if (pointPtr == *pointListPtr)
    {
    *pointListPtr = (*pointListPtr)->nextPtr;
    }
    else
    {
    curLinkedPointPtr = *pointListPtr;
    while (curLinkedPointPtr->nextPtr != NIL(LinkedPoint) &&
           curLinkedPointPtr->nextPtr != pointPtr)
    {
         curLinkedPointPtr = curLinkedPointPtr->nextPtr;
    }
    if (curLinkedPointPtr->nextPtr == pointPtr)
    {
        curLinkedPointPtr->nextPtr = curLinkedPointPtr->nextPtr->nextPtr;
    }
    else
    {
        return (BEAR_ERROR);
    }
    }
    FREE((char *) pointPtr);
    return (BEAR_OK);
}

/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrLinkedPoint() --
 *
 * Given a list of points, iterates the points on the list.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   Whatever the side effects of the user function if any.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrLinkedPoint(pointList, func, clientData)

LinkedPoint *pointList;
BearStatus  (*func)();
ClientData  clientData;
{
    LinkedPoint  *curPointPtr;

#ifdef DEBUG
    if (pointList == NIL(LinkedPoint))
    {
    MacPrintFatalErr("Mac_ItrLinkedPoint() failed: no point list is given");
    }
    else if (func == (BearStatus (*)()) NULL)
    {
    MacPrintFatalErr("Mac_ItrLinkedPoint() failed: no function is given");
    }
#endif DEBUG
    
    if (pointList == NIL(LinkedPoint))
    {
    return (BEAR_COMPLETE_SEARCH);
    }

    curPointPtr = pointList;
    while (curPointPtr != NIL(LinkedPoint))
    {
    if (((*func)(curPointPtr, clientData)) == BEAR_STOP_SEARCH)
    {
        return (BEAR_STOP_SEARCH);
    }
        curPointPtr = curPointPtr->nextPtr;
    }
    return (BEAR_COMPLETE_SEARCH);
}



void
Mac_FreePointList(headPtr)
register LinkedPoint *headPtr;
/*
 * For this to work the memory for the list must have been allocated
 * one element at a time !!
 */
{
    register LinkedPoint *tmpPtr;

    while ((tmpPtr = headPtr) != NIL(LinkedPoint)) 
    {
        headPtr = tmpPtr->nextPtr;
        FREE((char *) tmpPtr);
    }
}


void
Mac_FreeRectList(headPtr)
register LinkedRect *headPtr;
/*
 * For this to work the memory for the list must have been allocated
 * one element at a time !!
 */
{
    register LinkedRect *tmpPtr;

    while ((tmpPtr = headPtr) != NIL(LinkedRect)) 
    {
        headPtr = tmpPtr->r_next;
        FREE((char *) tmpPtr);
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_FindBBoxForPointList() --
 *
 * Find the bounding box of the given point list.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_FindBBoxForPointList(pointList, bboxPtr)
LinkedPoint     *pointList;
register Rect   *bboxPtr;
{
    register LinkedPoint *lnPointPtr;
    
#ifdef DEBUG
    if (pointList == NIL(LinkedPoint) || bboxPtr == NIL(Rect)) {
        MacPrintFatalErr("Mac_FindBBoxForPointList() failed");
    }
#endif DEBUG

    lnPointPtr = pointList;
    bboxPtr->r_xbot = bboxPtr->r_xtop = lnPointPtr->point.p_x;
    bboxPtr->r_ybot = bboxPtr->r_ytop = lnPointPtr->point.p_y;

    for (lnPointPtr = lnPointPtr->nextPtr;
        lnPointPtr != NIL(LinkedPoint);
        lnPointPtr = lnPointPtr->nextPtr) {

        if (lnPointPtr->point.p_x > bboxPtr->r_xtop) {
            bboxPtr->r_xtop = lnPointPtr->point.p_x;
        } else if (lnPointPtr->point.p_x < bboxPtr->r_xbot) {
            bboxPtr->r_xbot = lnPointPtr->point.p_x;
        }
        if (lnPointPtr->point.p_y > bboxPtr->r_ytop) {
            bboxPtr->r_ytop = lnPointPtr->point.p_y;
        } else if (lnPointPtr->point.p_y < bboxPtr->r_ybot) {
            bboxPtr->r_ybot = lnPointPtr->point.p_y;
        }
    }
    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_MakeLineSegList() --
 *
 * Make a line segment list from the given point list.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_MakeLineSegList(pointList, lineSegListPtr)
LinkedPoint     *pointList;
LinkedLineSeg   **lineSegListPtr;
{
    int                     i;
    register LineSeg        *lineSegPtr;
    register LinkedPoint    *lpPtr;

#ifdef DEBUG
    if (pointList == NIL(LinkedPoint) ||
        lineSegListPtr == (LinkedLineSeg **) NULL) {
        MacPrintFatalErr("Mac_MakeLineSegList() failed");
    }
#endif DEBUG

    *lineSegListPtr = NIL(LinkedLineSeg);
    for (lpPtr = pointList, i = 0;
        lpPtr != NIL(LinkedPoint);
        lpPtr = lpPtr->nextPtr, i++) {

        lineSegPtr = Mac_MakeLineSeg();
        lineSegPtr->segID = i;
        lineSegPtr->pinList = NIL(PinInst);

        Mac_CopyPoint(&lineSegPtr->line.point1, &lpPtr->point);
        if (lpPtr->nextPtr != NIL(LinkedPoint)) {
            Mac_CopyPoint(&lineSegPtr->line.point2, &lpPtr->nextPtr->point);
        }
        else {
            Mac_CopyPoint(&lineSegPtr->line.point2, &pointList->point);
        }

        (void) Mac_AppendLineSeg(lineSegListPtr, lineSegPtr);
    }
    return (BEAR_OK);
}



