
/* macHelper.c --
 *
 *    George Carvalho, Esq.
 *    (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1990 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "macInt.h"


BearStatus
Mac_SetUpTiPlane(cellMasterPtr, dummyCellFlag)
CellMaster  *cellMasterPtr;
bool        dummyCellFlag;
/*
 * Set up the tile plane for the given cell master.
 */
{
    PlaneData *planeDataPtr;

    if ((planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr)) !=
        NIL(PlaneData)) {
        Tp_FreePlaneData(planeDataPtr);
        Mac_PutCellPlaneOfCell(NIL(PlaneData), cellMasterPtr);
    }

    if (Tp_CreateTiPlaneForCellMaster(cellMasterPtr) != BEAR_OK) {
      Bear_PrintErr("Mac_SetUpTiPlane() failed : could not create tile plane.");
        return (BEAR_ERROR);
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    Tp_IdentifySpaceTiles(planeDataPtr);

    if (Mac_IsNumOfCellsZero(cellMasterPtr) == TRUE) {
        /*
         * The chip master which is the parent of the core master
         * is an example of a cell master that will never have any
         * cell instances.
         */
        return (BEAR_OK);
    }

    if (dummyCellFlag == TRUE) {
        if (Tp_CreateDCells(planeDataPtr) == BEAR_ERROR) {
   Bear_PrintErr("Mac_SetUpTiPlane() failed : could not create dummy cells.");
            return (BEAR_ERROR);
        } 

        /*
         * Paint all dummy cells onto the tile planes.
         */
        (void) Mac_ItrDCell(Tp_GetDCellList(planeDataPtr), Tp_PaintDCell,
            (ClientData) planeDataPtr);

        /*
         * Painting the dummy cells onto the tile plane disrupts
         * most of the space tiles, so memory must be re-allocated
         * for each one of them and then they must all be
         * re-identified.
         */
        Tp_IdentifySpaceTiles(planeDataPtr);
    }

    if (Fpg_CreateFloorPlanGraph(planeDataPtr, (WallJunction **) NULL) ==
        BEAR_ERROR) {
Bear_PrintErr("Mac_SetUpTiPlane() failed : couldn't create floor plan graph.");
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}



