
/* macMisc.h --
 *
 *     George Carvalho, Esq.
 *     June Wang
 *     (Advising Professor - E. S. Kuh)
 *
 * It contains the data structures for miscellaneous purposes and
 * the declarations of the subroutines defined in macMain.c.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 *
 * Needs:
 *        magic.h
 *        geometry.h
 *        bear.h
 *
 * rcsid $Header: macMain.h,v 1.2 87/02/11 01:41:07 weiming Exp $
 */

#ifndef _MAC_MISC
#define _MAC_MISC

/* ---------- Basic ---------- */

#define CLOCKWISE           1

typedef int LayerVector; 

typedef struct _RdbPinInfo {
    term   *termPtr;
    gterm  *gtermPtr;
} RdbPinInfo;

typedef enum _Flexibility {
    FIXED =                 0,

    FLEXIBLE =              1,

    FLEXIBLE_NORTH =        2,
    FLEXIBLE_EAST =         3,
    FLEXIBLE_SOUTH =        4,
    FLEXIBLE_WEST =         5
} Flexibility;

typedef struct _LineSeg {
    BearId          segID;          /* Line segment identification.         */
    Line            line;           /* The line of the segment              */
    int             dir;            /* The direction is one GEO_NORTH,
                                     * GEO_EAST, GEO_SOUTH, or GEO_WEST
                                     * and has a default value of GEO_CENTER.
                                     */
    int             routCap;        /* Capacity of the line segment for
                                     * over-the-cell routing.
                                     */
    struct _PinInst *pinList;       /* head of the pin list on the segment  */ 
} LineSeg;

typedef struct _LinkedLineSeg {     /* A linked line segment                */
    LineSeg               *lineSeg; /* A line segment                       */
    struct _LinkedLineSeg *nextPtr; /* Pointer to another linked 
                                     * line segment 
                                     */
} LinkedLineSeg;

typedef struct _LinkedSegID {       /* A linked segment identifier          */
    BearId              segID;      /* A segment identifier                 */
    struct _LinkedSegID *nextPtr;   /* Pointer to another linked 
                                     * segment identifier 
                                     */
} LinkedSegID;


/* ---------- Basic Primitives ---------- */

extern BearStatus Mac_Begin();

extern void Mac_InitId();
extern BearId Mac_GenerateId();

extern LineSeg *Mac_MakeLineSeg();

extern BearStatus Mac_AppendLineSeg(/* lineSegList, lineSeg*/);
/*
 * LinkedLineSeg *lineSegList;
 * LineSeg       *lineSeg;
 */

extern LineSeg *Mac_GetSegOfCell(/* cellInst, segID */);
/*
 * CellInst *cellInst;
 * BearId   segID;
 */

extern int GetRoutCapOfSeg(/* lineSeg */);
/* 
 * LineSeg *lineSeg;
 */

extern BearStatus Mac_InsertPinOfLineSeg(/*linkedLineSegPtr, pinPtr*/);
/*
 * LinkedLineSeg  *linkedLineSegPtr;
 * PinInst    *pinPtr;
 */

extern BearStatus Mac_AssignDirOfLineSegments(/* lineSegList */);
/*
 * LineSeg *lineSegList;
 */

extern BearStatus Mac_ItrLineSeg(/* lineSegList, func, clientData */);
/*
 * LinkedLineSeg    *lineSegList;
 * BearStatus       (*func)();
 * ClientData       clientData;
 */

extern BearStatus Mac_ItrPinOfLineSeg(/* lineSeg, func, clientData */);
/*
 * LineSeg    *lineSeg;
 * BearStatus (*func)();
 * ClientData clientData;
 */

extern BearStatus
Mac_ItrPinOfInterval(/* lineSegPtr, interval, func, clientData */);
/*
 * LineSeg    *lineSegPtr;
 * Line       interval;
 * BearStatus (*func)();
 * ClientData clientData;
 */

extern void Mac_GenCellName(/* bblFileName, cellNamePtr */);
/*
 * BearString bblFileName;      (* IN  *)
 * BearString *cellNamePtr;     (* OUT *)
 */

extern void
Mac_GenFileName(/* oldFileNamePtr, oldPostFixPtr, newPostFixPtr,
    newFileNamePtrPtr */);
/*
 * char *oldFileNamePtr;        (* IN  *)
 * char *oldPostFixPtr;         (* IN  *)
 * char *newPostFixPtr;         (* IN  *)
 * char **newFileNamePtrPtr;    (* OUT *)
 */

extern BearStatus Mac_WriteStats(/* type, cellMasterPtr, fileName */);
/*
 * unsigned int type;             (either 1 or 2)
 * CellMaster   *cellMasterPtr;
 * BearString   fileName;
 */

extern term *Mac_LinkTermToTerm(/* termPtr, modPtr */);
/*
 * term     *termPtr;
 * module   *modPtr;
 */

extern void Mac_GetAbsPosOfRJun(/* rjunPtr, absPosPtr */);
/*
 * rjun    *rjunPtr;                               (* IN       *)
 * Point   *absPosPtr;                             (* OUT      *)
 */

extern void Mac_CheckAllBottlenecks(/* planeDataPtr, thinBnList */);
/*
 * PlaneData    *planeDataPtr;
 * lsList       thinBnList;
 */

extern BearStatus Mac_CheckPins(/* cellMasterPtr, pinSpacingFlagPtr */);
/*
 * CellMaster   *cellMasterPtr;         (* IN  *)
 * bool         *pinSpacingFlagPtr;     (* OUT *)
 */

extern BearStatus Mac_CheckCellMaster(/* cellMasterPtr, errFileNamePtr */);
/*
 * CellMaster  *cellMasterPtr;
 * char        *errFileNamePtr
 */

#endif _MAC_MISC

