
/* macPin.c --
 *
 *     Sangjin Hong
 *     June Wang
 *     (Advising Professor - E. S. Kuh)
 *
 * It contains the operations to create, insert, and delete pins.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: macPin.c,v 1.2 87/02/07 18:52:59 weiming Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "macInt.h"
#include "list.h"


/* ---------- Local Procedures ---------- */
static PinMaster    *MakePinMaster();
static BearStatus   RemovePin();

PinInst             *MakePinInst();
extern PortMaster   *Mac_MakePortMaster();
extern PortInst     *Mac_MakePortInst();


/* ---------- Pin Primitives ---------- */


/*
 * ----------------------------------------------------------------------------
 *
 * MakePinMaster() --
 *
 * Allocates a piece of memory for a pointer to PinMaster.
 *
 * Results:
 *   Pointer to PinMaster is returned.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

static PinMaster *
MakePinMaster()
{
    PinMaster *pinMasterPtr;

    MALLOC(PinMaster *, pinMasterPtr, sizeof (PinMaster));
    pinMasterPtr->name = (BearString) NULL;
    pinMasterPtr->posDefFlag = FALSE;
    pinMasterPtr->flexType = FIXED;
    pinMasterPtr->layer = 12;
    pinMasterPtr->portPtr = NIL(PortMaster);
    pinMasterPtr->instList = NIL(PinInst);
    pinMasterPtr->nextPtr = NIL(PinMaster);
    pinMasterPtr->clientData = (ClientData) NULL;

    return (pinMasterPtr);
}


/*
 * ----------------------------------------------------------------------------
 *
 * MakePinInst() --
 *
 * Allocates a piece of memory for a pointer to PinInst.
 *
 * Results:
 *   Pointer to PinInst is returned.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

/*
Channel interface testing need MakePinInst()

static PinInst *
*/
PinInst *
MakePinInst()
{
    PinInst *pinPtr;
    int     i;

    MALLOC(PinInst *, pinPtr, sizeof (PinInst));

    pinPtr->pi_name = (BearString) NULL;

    pinPtr->pi_id = Mac_GenerateId();
    Mac_Insert(PIN_INSTANCE_HT, (char *) pinPtr->pi_id, (char *) pinPtr);

    pinPtr->intNetId = (BearId) 0;
    pinPtr->extNetId = (BearId) 0;
    pinPtr->horzExtNetId = (BearId) 0;
    pinPtr->vertExtNetId = (BearId) 0;
    pinPtr->pinKind = (PinKind) MAC_INSIDE_PIN;
    pinPtr->masterPtr = NIL(PinMaster);
    pinPtr->portPtr = NIL(PortInst);
    pinPtr->nextInMListPtr = NIL(PinInst);
    pinPtr->nextInCellPtr = NIL(PinInst);
    for (i = 1; i < 8; i++) {
        pinPtr->widthVect[i] = MAXWIDTH;
    }
    pinPtr->pinWidth = MAXWIDTH;
    pinPtr->segmWidth = MAXWIDTH;
    pinPtr->BBLTermPtr = NIL(RdbPinInfo);
    pinPtr->nextPtr = NIL(PinInst);
    pinPtr->nextInMListPtr = NIL(PinInst);
    pinPtr->nextInCellPtr = NIL(PinInst);
    pinPtr->ngrModulePtr = (BearGeneric) NULL;
    pinPtr->clientData = (ClientData) NULL;

    return (pinPtr);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_CreatePinOfMaster() --
 *
 * Makes port instance, and pin instance with names given in the parameter 
 * list.  Links them all up by calling necessary insertion subroutines.  
 * Updates the related hash tables.
 *
 * Notice that pin instance is passed back to the calling subroutine
 * as a pointer to a pointer to PinInst.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_CreatePinOfMaster(cellInstPtr, portMasterPtr, portInstName,
    pinMasterPtr, pinInstName, pinInstPtrPtr)

CellInst    *cellInstPtr;
PortMaster  *portMasterPtr;
PinMaster   *pinMasterPtr;
BearString  portInstName;
BearString  pinInstName;
PinInst     **pinInstPtrPtr;
{
    PortInst    *portInstPtr;
    char        *string1Ptr, *string2Ptr;

#ifdef DEBUG
    if (cellInstPtr == NIL(CellInst)) {
  MacPrintFatalErr("Mac_CreatePinOfMaster() failed: no cell pointer is given");
    }
#endif DEBUG

    MALLOC(char *, string1Ptr, (strlen(portInstName) + 1) * sizeof (char));
    strcpy(string1Ptr, portInstName);

    MALLOC(char *, string2Ptr, (strlen(pinInstName) + 1) * sizeof (char));
    strcpy(string2Ptr, pinInstName);

    portInstPtr = Mac_MakePortInst();
    if (Mac_InsertPortInstOfCellInst(cellInstPtr, portInstPtr) == BEAR_ERROR)
    {
        Bear_PrintErr("Mac_InsertPortInstOfCellInst() failed in Mac_CreatePinOfMaster()");
        return (BEAR_ERROR);
    }
    else if (Mac_InsertPortInstOfPortMaster(portMasterPtr, portInstPtr) ==
        BEAR_ERROR) {
        Bear_PrintErr("Mac_InsertPortInstOfPortMaster() failed in Mac_CreatePinOfMaster()");
        return (BEAR_ERROR);
    }
    else if (Mac_PutNameOfPortInst(string1Ptr, portInstPtr) == BEAR_ERROR) {
   Bear_PrintErr("Mac_PutNameOfPortInst() failed in Mac_CreatePinOfMaster()");
        return (BEAR_ERROR);
    }
    else {
        *pinInstPtrPtr = MakePinInst();
        if (Mac_InsertPinInstOfPort(portInstPtr, *pinInstPtrPtr) != BEAR_OK) {
  Bear_PrintErr("Mac_InsertPinInstOfPort() failed in Mac_CreatePinOfMaster()");
            return (BEAR_ERROR);
        }
        else if (Mac_InsertPinInstOfPinMaster(*pinInstPtrPtr, pinMasterPtr) !=
            BEAR_OK) {
Bear_PrintErr("Mac_InsertPinInstOfPinMaster() failed in Mac_CreatePinOfMaster()");
            return (BEAR_ERROR);
        }
        else if (Mac_PutNameOfPinInst(string2Ptr, *pinInstPtrPtr) != BEAR_OK) {
     Bear_PrintErr("Mac_PutNameOfPinInst() failed in Mac_CreatePinOfMaster()");
            return (BEAR_ERROR);
        }
    }

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_CreatePinMaster() --
 *
 * Makes pin master, and port master  with names given in the parameter list.  
 * Links them by calling necessary insertion subroutines.  
 * Updates the related hash tables.
 *
 * Notice that pin master and port master is passed back to the 
 * calling subroutine as a pointer to a pointer to PinMaster and port master.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_CreatePinMaster(cellMasterPtr, portMasterPtrPtr,portMasterName,
    pinMasterName, pinMasterPtrPtr)

CellMaster  *cellMasterPtr;
PortMaster  **portMasterPtrPtr;
BearString  portMasterName;
BearString  pinMasterName;
PinMaster   **pinMasterPtrPtr;
{
    char *string1Ptr, *string2Ptr;

    MALLOC(char *, string1Ptr, (strlen(portMasterName) + 1) * sizeof (char));
    strcpy(string1Ptr, portMasterName);

    MALLOC(char *, string2Ptr, (strlen(pinMasterName) + 1) * sizeof (char));
    strcpy(string2Ptr, pinMasterName);

    *portMasterPtrPtr = Mac_MakePortMaster();
    if (Mac_InsertPortMaster(cellMasterPtr, *portMasterPtrPtr) == BEAR_ERROR) {
        Bear_PrintErr("Mac_InsertPortMaster() failed in Mac_CreatePinMaster()");
        return (BEAR_ERROR);
    }
    else if (Mac_PutNameOfPortMaster(string1Ptr, *portMasterPtrPtr) 
        == BEAR_ERROR) {
        Bear_PrintErr("Mac_PutNameOfPortMaster() failed in Mac_CreatePinMaster()");
        return (BEAR_ERROR);
    }
    else {
        *pinMasterPtrPtr = MakePinMaster();

        if (Mac_InsertPinMaster(*portMasterPtrPtr, *pinMasterPtrPtr) ==
            BEAR_ERROR) {
            Bear_PrintErr("Mac_InsertPinMaster() failed in Mac_CreatePinMaster()");
            return (BEAR_ERROR);
        }
        else if (Mac_PutNameOfPinMaster(string2Ptr, *pinMasterPtrPtr) ==
            BEAR_ERROR)
        {
            Bear_PrintErr("Mac_PutNameOfPinMaster() failed in Mac_CreatePinMaster()");
            return (BEAR_ERROR);
        }
    }
    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_CreatePin() --
 *
 * Makes pin master, port instance, pin master, and port master  with 
 * names given in the parameter list.  
 * Links them by calling necessary insertion subroutines.  
 * Updates the related hash tables.
 *
 * Notice that pin instance is passed back to the 
 * calling subroutine as a pointer to a pointer to PinInst.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_CreatePin(cellInstPtr,
    portMasterName, portInstName,
    pinMasterName, pinInstName,
    pinInstPtrPtr)

CellInst    *cellInstPtr;
BearString  portMasterName;
BearString  portInstName;
BearString  pinMasterName;
BearString  pinInstName;
PinInst     **pinInstPtrPtr;
{
    CellMaster  *cellMasterPtr;
    PortMaster  *portMasterPtr;
    PortInst    *portInstPtr;
    PinMaster   *pinMasterPtr;
    char        *string1Ptr, *string2Ptr, *string3Ptr, *string4Ptr;

#ifdef DEBUG
    if (cellInstPtr == NIL(CellInst)) {
        MacPrintFatalErr("Mac_CreatePin() failed: no cell pointer is given");
    }
#endif DEBUG

    MALLOC(char *, string1Ptr, (strlen(portMasterName) + 1) * sizeof (char));
    strcpy(string1Ptr, portMasterName);

    MALLOC(char *, string2Ptr, (strlen(portInstName) + 1) * sizeof (char));
    strcpy(string2Ptr, portInstName);

    MALLOC(char *, string3Ptr, (strlen(pinMasterName) + 1) * sizeof (char));
    strcpy(string3Ptr, pinMasterName);

    MALLOC(char *, string4Ptr, (strlen(pinInstName) + 1) * sizeof (char));
    strcpy(string4Ptr, pinInstName);

    portMasterPtr = Mac_MakePortMaster();
    portInstPtr = Mac_MakePortInst();
    cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);

    if (Mac_InsertPortMaster(cellMasterPtr, portMasterPtr) != BEAR_OK) {
        Bear_PrintErr("Mac_InsertPortMaster() failed in Mac_CreatePin()");
        return (BEAR_ERROR);
    }
    else if (Mac_InsertPortInstOfCellInst(cellInstPtr, portInstPtr) != 
        BEAR_OK) {
   Bear_PrintErr("Mac_InsertPortInstOfCellInst() failed in Mac_CreatePin()");
        return (BEAR_ERROR);
    }
    else if (Mac_InsertPortInstOfPortMaster(portMasterPtr, portInstPtr) != 
        BEAR_OK) {
   Bear_PrintErr("Mac_InsertPortInstOfPortMaster() failed in Mac_CreatePin()");
        return (BEAR_ERROR);
    }

    else if (Mac_PutNameOfPortMaster(string1Ptr, portMasterPtr) != BEAR_OK) {
   Bear_PrintErr("Mac_PutNameOfPortMaster() failed in Mac_CreatePinMaster()");
        return (BEAR_ERROR);
    }
    else if (Mac_PutNameOfPortInst(string2Ptr, portInstPtr) != BEAR_OK) {
         Bear_PrintErr("Mac_PutNameOfPortInst() failed in Mac_CreatePin()");
         return (BEAR_ERROR);
    }

    else {
        pinMasterPtr = MakePinMaster();
        *pinInstPtrPtr = MakePinInst();

        if (Mac_InsertPinInstOfPort(portInstPtr, *pinInstPtrPtr) != BEAR_OK) {
   Bear_PrintErr("Mac_InsertPinInstOfPort() failed in Mac_CreatePin()");
            return (BEAR_ERROR);
        }
        else if (Mac_InsertPinInstOfPinMaster(*pinInstPtrPtr, pinMasterPtr) !=
            BEAR_OK) {
   Bear_PrintErr("Mac_InsertPinInstPinMaster() failed in Mac_CreatePin()");
            return (BEAR_ERROR);
        }
        else if (Mac_InsertPinMaster(portMasterPtr, pinMasterPtr) != BEAR_OK) {
            Bear_PrintErr("Mac_InsertPinMaster() failed in Mac_CreatePin()");
            return (BEAR_ERROR);
        }

        else if (Mac_PutNameOfPinMaster(string3Ptr, pinMasterPtr) != BEAR_OK) {
   Bear_PrintErr("Mac_PutNameOfPinMaster() failed in Mac_CreatePin()");
            return (BEAR_ERROR);
        }
        else if (Mac_PutNameOfPinInst(string4Ptr, *pinInstPtrPtr) != BEAR_OK) {
            Bear_PrintErr("Mac_PutNameOfPinInst() failed in Mac_CreatePin()");
            return (BEAR_ERROR);
        }
    }

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_DuplicatePinMaster() --
 *
 * Makes a new pin master with the given name, and links it with
 * the port master of the original pin master.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_DuplicatePinMaster(oldPinMasterPtr, copyPinMasterName, copyPinMasterPtrPtr)
PinMaster     *oldPinMasterPtr;
BearString    copyPinMasterName;
PinMaster     **copyPinMasterPtrPtr;
{
    char *stringPtr;

/* Make new pin instance and a pin master */

    *copyPinMasterPtrPtr = MakePinMaster();

/* Insert new pin master into its port master */

    if (Mac_InsertPinMaster(oldPinMasterPtr->portPtr, *copyPinMasterPtrPtr) == 
        BEAR_ERROR) {
        Bear_PrintErr("Mac_DuplicatePinMaster() failed \n");
        return (BEAR_ERROR);
    }


/* Put new pin master name */

    MALLOC(char *, stringPtr, (strlen(copyPinMasterName) + 1) * sizeof (char));
    strcpy(stringPtr, copyPinMasterName);

    if (Mac_PutNameOfPinMaster(stringPtr,*copyPinMasterPtrPtr)
        == BEAR_ERROR) {
        Bear_PrintErr("Mac_DuplicatePinMaster() failed in Mac_PutNameOfPinMaster()\n");
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}




/*
 * ----------------------------------------------------------------------------
 *
 * Mac_DuplicatePin() --
 *
 * Makes a new pin instance with the given name, and links it with
 * the port instance and the pin master of the original pin instance.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_DuplicatePin(copyPortInstPtr, pinMasterPtr, copyPinInstName, 
    copyPinInstPtrPtr)

PortInst    *copyPortInstPtr;
PinMaster   *pinMasterPtr;
BearString  copyPinInstName;
PinInst     **copyPinInstPtrPtr;
{
    char *stringPtr;

    /* Make new pin instance */

    *copyPinInstPtrPtr = MakePinInst();

    /* Insert new pin instance into its pin master */

   if (Mac_InsertPinInstOfPinMaster(*copyPinInstPtrPtr,pinMasterPtr) 
       == BEAR_ERROR) {
       Bear_PrintErr("Mac_DuplicatePin() failed in Mac_InsertPinInstOfPinMaster()\n");
       return (BEAR_ERROR);
   }


/* Attach pin instance to its port instances */

    if (Mac_InsertPinInstOfPort(copyPortInstPtr, *copyPinInstPtrPtr)
        == BEAR_ERROR)
    {
        Bear_PrintErr("Mac_DuplicatePin() failed in Mac_InsertPinInstOfPort\n");
        return (BEAR_ERROR);
    }

    /* Put new pin instance name and new pin master name */

    MALLOC(char *, stringPtr, (strlen(copyPinInstName) + 1) * sizeof (char));
    strcpy(stringPtr, copyPinInstName);

    if (Mac_PutNameOfPinInst(stringPtr, *copyPinInstPtrPtr) != BEAR_OK) {
       Bear_PrintErr("Mac_DuplicatePin() failed in Mac_PutNameOfPinInst()\n");
        return (BEAR_ERROR);
    }

    /* Copy pin instance contents */

    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_InsertPinMaster() --
 *
 * Takes the list of pin masters from the given port master, and inserts
 * the pin master at the beginning of the list.  
 *
 * Results:
 *   BEAR_OK, when successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus 
Mac_InsertPinMaster(portMasterPtr, pinMasterPtr)
PortMaster *portMasterPtr;
PinMaster  *pinMasterPtr;

{
#ifdef DEBUG
    if (portMasterPtr == NIL(PortMaster))
    {
    MacPrintFatalErr("Mac_InsertPinMaster() failed: no portMasterPtr is given");
    }
    else if (pinMasterPtr == NIL(PinMaster))
    {
    MacPrintFatalErr("Mac_InsertPinMaster() failed: no pinMasterPtr is given");
    }
#endif DEBUG

    if (portMasterPtr->pinMasterList != NIL(PinMaster)) 
    {
        pinMasterPtr->nextPtr = portMasterPtr->pinMasterList;
        portMasterPtr->pinMasterList = pinMasterPtr;
        pinMasterPtr->portPtr = portMasterPtr;
    }
    else
    {
        pinMasterPtr->portPtr = portMasterPtr;
        portMasterPtr->pinMasterList = pinMasterPtr;
    }

    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_DeletePinMaster() --
 *
 * From the given pin master, it finds the related port master, and
 * retrieves the list of pin masters from the port master.
 * Then, searches the list for the pin master to be deleted.
 * After the pin master is deleted from the list, it deletes all the
 * pin instances attached to the pin master.  Finally, the hashtable
 * for pin master is updated and the pointer to pin master is freed.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_DeletePinMaster(pin)
PinMaster *pin;
{
    PortMaster *portPtr; 
    PinMaster  *currentPinPtr;
    PinInst    *currentInstPtr;

#ifdef DEBUG
    if (pin == NIL(PinMaster))
    {
    MacPrintFatalErr("Mac_DeletePinMaster() failed: no pin master is given");
    }
    else if (pin->portPtr == NIL(PortMaster))
    {
    MacPrintFatalErr("Mac_DeletePinMaster() failed: the given pin is not connected to a port");
    }
#endif DEBUG

/* Deletes the given pin master from the port master */

    portPtr = pin->portPtr;
    if (pin == portPtr->pinMasterList)
    {
        portPtr->pinMasterList = pin->nextPtr;
    }
    else 
    {
        currentPinPtr = portPtr->pinMasterList;
        while (currentPinPtr->nextPtr != NIL(PinMaster) && 
               currentPinPtr->nextPtr != pin)
        {
            currentPinPtr = currentPinPtr->nextPtr;
        }
        if (currentPinPtr->nextPtr == pin)
        {
            currentPinPtr->nextPtr = pin->nextPtr;
        }
        else
        {
            Bear_PrintErr("Mac_DeletePinMaster() failed: the pin master does not exists");
            return (BEAR_ERROR);
        }
    }

/* Deletes all the pin instances attached to the pin master */

    currentInstPtr = pin->instList;
    while (currentInstPtr != NIL(PinInst))
    {
        if (Mac_DeletePinInstOfPinMaster(currentInstPtr, pin) == BEAR_ERROR)
        {
            Bear_PrintErr("Mac_DeletePinInstOfPinMaster() failed in Mac_DeletePinMaster()");
            return (BEAR_ERROR);
        }
        currentInstPtr = currentInstPtr->nextInMListPtr;
    }
    Mac_Delete(PIN_MASTER_HT, pin->name);
    FREE((char *) pin);
    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_InsertPinInstOfPinMaster() --
 *
 * Takes the list of pin intances from the given pin master, and inserts
 * the given pin instance to the beginning of the list.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_InsertPinInstOfPinMaster(pinInst, pinMaster)
PinInst    *pinInst;
PinMaster  *pinMaster;

{
#ifdef DEBUG
    if (pinMaster == NIL(PinMaster))
    {
    MacPrintFatalErr("Mac_InsertPinInstOfPinMaster() failed: pinMaster is NULL");
    }
    else if (pinInst == NIL(PinInst))
    {
    MacPrintFatalErr("Mac_InsertPinInstOfPinMaster() failed: pinInst is NULL");
    }
#endif DEBUG

    if (pinMaster->instList != NIL(PinInst))
    {
        pinInst->nextInMListPtr = pinMaster->instList;
        pinMaster->instList = pinInst;
        pinInst->masterPtr = pinMaster;
    }
    else
    {
        pinMaster->instList = pinInst;
        pinInst->masterPtr = pinMaster;
    }
    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_DeletePinInstOfPinMaster() --
 *
 * Takes the list of pin instances from the given pin master, and
 * searches for the given pin instance.
 * When the pin instance is found, it deletes it from the list, updates
 * the hash table, and frees the pointer.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_DeletePinInstOfPinMaster(pinInstPtr, pinMasterPtr)
PinInst     *pinInstPtr;
PinMaster   *pinMasterPtr;
{
    PinInst *curPinPtr;

#ifdef DEBUG
    if (pinMasterPtr == NIL(PinMaster)) {
  MacPrintFatalErr("Mac_DeletePinInstOfPinMaster() failed: pinMaster is NULL");
    }
    else if (pinInstPtr == NIL(PinInst)) {
    MacPrintFatalErr("Mac_DeletePinInstOfPinMaster() failed: pinInst is NULL");
    }
#endif DEBUG

    if (pinInstPtr == pinMasterPtr->instList) {
        pinMasterPtr->instList = pinInstPtr->nextInMListPtr;
    }
    else {
        curPinPtr = pinMasterPtr->instList;
        while (curPinPtr->nextInMListPtr != NIL(PinInst) && 
               curPinPtr->nextInMListPtr != pinInstPtr)
        {
            curPinPtr = curPinPtr->nextInMListPtr;
        }
        if (curPinPtr->nextInMListPtr == pinInstPtr) {
            curPinPtr->nextInMListPtr = pinInstPtr->nextInMListPtr;
        }
        else {
            Bear_PrintErr("Mac_DeletePinInstOfPinMaster() failed: the given pin instance can not be found");
            return (BEAR_ERROR);
        }
    }

    /*
    Mac_Delete(PIN_INSTANCE_HT, (char *) pinInstPtr->id);
    */

    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_DeletePin() --
 *
 * This procedure take a pin instance and deletes it.  The pin master is 
 * deleted, the port instance is changed, and the net is changed.  If it is
 * needed, the port master and net are removed.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_DeletePin(pinPtr)
PinInst *pinPtr;
{
    PinMaster     *pinMasterPtr;
    PortInst      *portInstPtr;
    PortMaster    *portMasterPtr;
    CellMaster    *cellMasterPtr;
    CellInst      *cellInstPtr;
    Net           *netPtr;
    LinkedLineSeg *boundary;

#ifdef DEBUG
    if (pinPtr == NIL(PinInst)) {
    MacPrintFatalErr("Mac_InsertPinInstOfPinMaster() failed: pinInst is NULL");
    }
#endif DEBUG

    pinMasterPtr = pinPtr->masterPtr;
    portInstPtr = pinPtr->portPtr;
    cellInstPtr = portInstPtr->cellInstPtr;
    netPtr = portInstPtr->netPtr;
    portMasterPtr = portInstPtr->masterPtr;
    cellMasterPtr = portMasterPtr->cellMasterPtr;

    /* Deletes pin from the LinkedLineSegment list */

    boundary = Mac_GetBoundaryOfCell(cellInstPtr);
    if (Mac_ItrLineSeg(boundary, RemovePin, (ClientData) pinPtr) == 
        BEAR_STOP_SEARCH) {
        Bear_PrintErr("Mac_ItrLineSeg() failed in Mac_DeletePin()\n");
        return (BEAR_ERROR);
    }

/* Deletes the given pin from its pin master and
   if necessary, pin master will be deleted       */

    if (pinMasterPtr->instList->nextInMListPtr == NIL(PinInst)) {
        if (Mac_DeletePinMaster(pinMasterPtr) == BEAR_ERROR) {
            Bear_PrintErr("Mac_DeletePinMaster() failed in Mac_DeletePin()\n");
            return (BEAR_ERROR);
        }
    }
    else {
        if (Mac_DeletePinInstOfPinMaster(pinPtr, pinMasterPtr) == BEAR_ERROR) {
            Bear_PrintErr("Mac_DeletePinInstOfPinMaster() failed in Mac_DeletePin()\n");
            return (BEAR_ERROR);
        }
    }


/* Updates the net */

    if (portInstPtr->netPtr != NIL(Net))
    {
        if (portInstPtr->pinList->nextPtr == NIL(PinInst))
        {
            if (Mac_DeletePortInstOfNet(portInstPtr) == BEAR_ERROR)
            {
                Bear_PrintErr("Mac_DeletePortInstOfNet() failed in Mac_DeletePin()\n");
                return (BEAR_ERROR);
            }
        }

        if (netPtr->portList == NIL(PortInst))
        {
            if (Mac_DeleteNet(netPtr) == BEAR_ERROR)
            {
                Bear_PrintErr("Mac_DeleteNet() failed in Mac_DeletePin()\n");
                return (BEAR_ERROR);
            }
        }
    }


/* Deletes the given pin from its port instance and
   if necessary, port master will be deleted         */

    if (Mac_DeletePinInstOfPort(portInstPtr, pinPtr) == BEAR_ERROR)
    {
        Bear_PrintErr("Mac_DeletePinInstOfPort() failed in Mac_DeletePin()\n");
        return (BEAR_ERROR);
    }

    if (portMasterPtr->pinMasterList == NIL(PinMaster) &&
        portMasterPtr->instList->nextPtr == NIL(PortInst) &&
        portMasterPtr->instList->pinList->nextPtr == NIL(PinInst))
    {
        if (Mac_DeletePortMaster(cellMasterPtr,portMasterPtr) == BEAR_ERROR)
        {
            Bear_PrintErr("Mac_DeletePortMaster() failed in Mac_deletePin()\n");
            return (BEAR_ERROR);
        }
    }

    return (BEAR_OK);
}


static BearStatus
RemovePin(lineSegPtr, pinInstPtr)
LineSeg *lineSegPtr;
PinInst *pinInstPtr;
{
    PinInst *pinList, *currentPinPtr;

    pinList = lineSegPtr->pinList;
    if (pinList == NIL(PinInst))
    {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (pinInstPtr == pinList) {
        lineSegPtr->pinList = pinInstPtr->nextInCellPtr;
        return (BEAR_STOP_SEARCH);
    }
    else {
        currentPinPtr = pinList;
        while (currentPinPtr->nextInCellPtr != NIL(PinInst) && 
            currentPinPtr->nextInCellPtr != pinInstPtr) {
            currentPinPtr = currentPinPtr->nextInCellPtr;
        }

        if (currentPinPtr->nextInCellPtr == pinInstPtr) {
            currentPinPtr->nextInCellPtr = pinInstPtr->nextInCellPtr;
            return (BEAR_STOP_SEARCH);
        }
        else {
            return (BEAR_CONTINUE_SEARCH);
        }
    }
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_InsertPinInstOfPort() --
 *
 * Takes the list of pin instances from the given port instance, and
 * inserts the pin instance at the beginning of the list.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_InsertPinInstOfPort(port, pin)
PortInst *port;
PinInst  *pin;

{
#ifdef DEBUG
    if (port == NIL(PortInst))
    {
    MacPrintFatalErr("Mac_InsertPinInstOfPort() failed: no port is given");
    }
    else if (pin == NIL(PinInst)) 
    {
    MacPrintFatalErr("Mac_InsertPinInstOfPort() failed: no pin is given");
    }
#endif DEBUG
    if (port->pinList != NIL(PinInst))
    {
        pin->nextPtr = port->pinList;
        port->pinList= pin;  
        pin->portPtr = port;
    }
    else
    {
        pin->portPtr = port;
        port->pinList = pin;
    }
    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_DeletePinInstofPort() --
 *
 * Takes the list of pin instance from the given port instance, and 
 * searches for the given pin instance.  Deletes the pin instance, and
 * updates the hash table.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_DeletePinInstOfPort(port, pin)
PortInst *port;
PinInst  *pin;
{
    PinInst *currentPinPtr;

#ifdef DEBUG
    if (port == NIL(PortInst)) {
        MacPrintFatalErr("Mac_DeletePinInstOfPort() failed: no port is given");
    }
    else if (pin == NIL(PinInst)) {
        MacPrintFatalErr("Mac_DeletePinInstOfPort() failed: no pin is given");
    }
    else if (port->pinList == NIL(PinInst)) {
        Bear_PrintErr(
      "Mac_DeletePinInstOfPort() failed: the given port has no pins attached");
        return (BEAR_ERROR);
    }
#endif DEBUG

    if (pin == port->pinList) {
        port->pinList = pin->nextPtr;
    }
    else {
        currentPinPtr = port->pinList;
        while ((currentPinPtr->nextPtr != NIL(PinInst)) &&
            (currentPinPtr->nextPtr != pin)) { 

            currentPinPtr = currentPinPtr->nextPtr;
        }
        if (currentPinPtr->nextPtr == pin) {
            currentPinPtr->nextPtr = pin->nextPtr;
        }
        else {
            return (BEAR_ERROR);
        }
    }

    Mac_Delete(PIN_INSTANCE_HT, (char *) pin->pi_id);
    /*
     * Should also delete the pin from the cells hash table.
     */
    FREE((char *) pin); 

    return (BEAR_OK);
}

  

/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ConvertToBorderPin() --
 *
 * This procedure is designed to be used by an "interator".
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

/* ARGSUSED */
BearStatus
Mac_ConvertToBorderPin(pinInstPtr, clientData)
PinInst     *pinInstPtr;
ClientData  clientData;
{
    Mac_PutKindOfPin(MAC_BORDER_PIN, pinInstPtr);
    switch (Mac_GetDirectionOfPin(pinInstPtr)) {
        case NORTH_PIN:
            Mac_PutDirectionOfPin(SOUTH_PIN, pinInstPtr);
            break;
        case SOUTH_PIN:
            Mac_PutDirectionOfPin(NORTH_PIN, pinInstPtr);
            break;
        case WEST_PIN:
            Mac_PutDirectionOfPin(EAST_PIN, pinInstPtr);
            break;
        case EAST_PIN:
            Mac_PutDirectionOfPin(WEST_PIN, pinInstPtr);
            break;
        default:
            MacPrintFatalErr("ConvertToBorderPin: illegal pin direction.");
            break;
    }
    return (BEAR_CONTINUE_SEARCH);
}


