
/* macPortItr.c --
 *
 *      June Wang
 *      (Advising Professor - E. S. Kuh)
 *
 * It contains the iterators related to ports.
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: macPortItr.c,v 1.1 87/02/04 15:33:54 weiming Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "macInt.h"



/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrEleEqvOfCell() --
 *
 * Takes the eleEqvList from the given cell instance.
 * For every eleEqv in the eleEqvList, it iterates all the port instances.

 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, when successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrEleEqvOfCell(cellInst, func, clientData) 
CellInst   *cellInst;
BearStatus (*func)();
ClientData clientData;

{
    LinkedLinkedPort  *eleEqvList;
    LinkedPort  *portList;

#ifdef DEBUG
    if (cellInst == NIL(CellInst))
    {
        MacPrintFatalErr("Mac_ItrEleEqvOfCell() failed: no cell instance is given");
    }
    else if ((func == (BearStatus (*)()) NULL))
    {
        MacPrintFatalErr("Mac_ItrEleEqvOfCell() failed: no function is given");
    }
#endif DEBUG

    if (cellInst->eleEqvList == NIL(LinkedLinkedPort) ||   
        cellInst->eleEqvList->portList == NIL(LinkedPort))        
    {
        return (BEAR_COMPLETE_SEARCH);
    }
    else
    {
        eleEqvList = cellInst->eleEqvList;
        while (eleEqvList != NIL(LinkedLinkedPort))
        {
            portList = eleEqvList->portList;
            while (portList != NIL(LinkedPort))
            {
                if (((*func)(portList->portInst, clientData)) == BEAR_STOP_SEARCH)
                {
                    return (BEAR_STOP_SEARCH);
                }
                portList = portList->nextPtr;
            }
            eleEqvList = eleEqvList->nextPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrPortOfEleEqv() --
 *
 * Iterates the port instance in the list of linked port instances.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus 
Mac_ItrPortOfEleEqv(eleEqv, func, clientData)
LinkedPort *eleEqv;
BearStatus (*func)();
ClientData clientData;

{
#ifdef DEBUG
    if (eleEqv == NIL(LinkedPort))  
    {
        MacPrintFatalErr("Mac_ItrPortOfEleEqv() failed: no Ele Eqv is given");
    }
    else if (func == (BearStatus (*)()) NULL)
    {
        MacPrintFatalErr("Mac_ItrPortOfEleEqv() failed: no function is given");
    }
#endif DEBUG
    if (eleEqv->portInst == NIL(PortInst))
    {
        return (BEAR_COMPLETE_SEARCH);
    }
    else
    {
        while (eleEqv != NIL(LinkedPort) && eleEqv->portInst != NIL(PortInst))
        {
            if (((*func)(eleEqv->portInst, clientData)) == BEAR_STOP_SEARCH)
            {
                return (BEAR_STOP_SEARCH);
            }
            eleEqv = eleEqv->nextPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrFeedThrOfCell() --
 *
 * Fetches the feedThrList from the cell instance.  Iterates all the
 * port instances in that list.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrFeedThrOfCell(cellInst, func, clientData)
CellInst   *cellInst;
BearStatus (*func)();
ClientData clientData;

{
    LinkedLinkedPort  *feedThrList;
    LinkedPort  *portList;

#ifdef DEBUG
    if (cellInst == NIL(CellInst))
    {
        MacPrintFatalErr("Mac_ItrFeedThrOfCell() failed: no cell instance is given");
    }
    else if (func == (BearStatus (*)()) NULL)
    {
        MacPrintFatalErr("Mac_ItrFeedThrOfCell() failed: no function is given");
    }
#endif DEBUG
    if (cellInst->feedThrList == NIL(LinkedLinkedPort) ||
        cellInst->feedThrList->portList == NIL(LinkedPort))
    {
        return (BEAR_COMPLETE_SEARCH);
    }
    else 
    {
        feedThrList = cellInst->feedThrList;
        while (feedThrList != NIL(LinkedLinkedPort))
        {
            portList = feedThrList->portList;
            while (portList != NIL(LinkedPort))
            {
                if (((*func)(portList->portInst, clientData)) == BEAR_STOP_SEARCH)
                {
                    return (BEAR_STOP_SEARCH);
                }
                portList = portList->nextPtr;
            }
            feedThrList = feedThrList->nextPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrPortOfFeedThr() --
 *
 * Given a list of linked port instance, feedThr, it iterates 
 * every single port instance.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus 
Mac_ItrPortOfFeedThr(feedThr, func, clientData)
LinkedPort *feedThr;
BearStatus (*func)();
ClientData clientData;

{
#ifdef DEBUG
    if (feedThr == NIL(LinkedPort))
    {
        MacPrintFatalErr("Mac_ItrPortOfFeedThr() failed: no FeedThr is given");
    }
    else if (func == (BearStatus (*)()) NULL)
    {
        MacPrintFatalErr("Mac_ItrPortOfFeedThr() failed: no function is given");
    }
#endif DEBUG

    if (feedThr->portInst != NIL(PortInst))
    {
        while (feedThr != NIL(LinkedPort) && feedThr->portInst != NIL(PortInst))
        {
            if (((*func)(feedThr->portInst, clientData)) == BEAR_STOP_SEARCH)
            {
                return (BEAR_STOP_SEARCH);
            }
            feedThr = feedThr->nextPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrInstOfPortOfCell() --
 *
 * Given a port master, it iterates all the port instances, which belong
 * to the cell link, under the master.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrInstOfPortOfCell(master, func, clientData)
PortMaster *master;
BearStatus (*func)();
ClientData clientData;

{
    PortInst *inst;

#ifdef DEBUG
    if (master == NIL(PortMaster))
    {
        MacPrintFatalErr("Mac_ItrInstOfPortOfCell() failed: no master is given");
    }
    else if (func == (BearStatus (*)()) NULL)
    {
        MacPrintFatalErr("Mac_ItrInstOfPortOfCell() failed: no function is given");
    }
#endif DEBUG
    if (master->instList == NIL(PortInst))
    {
        return (BEAR_COMPLETE_SEARCH);
    }
    else
    {
        inst = master->instList;
        while (inst != NIL(PortInst))
        {
            if (((*func)(inst, clientData)) == BEAR_STOP_SEARCH) 
            {
                return (BEAR_STOP_SEARCH);
            }
            inst = inst->nextCellPortPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrInstOfPortOfNet() --
 *
 * Fetches the list of port instances from the port master, and iterates
 * the list of instances through the net link.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrInstOfPortOfNet(master, func, clientData)
PortMaster *master;
BearStatus (*func)();
ClientData clientData;

{
    PortInst *inst;

#ifdef DEBUG
    if (master == NIL(PortMaster))
    {
        MacPrintFatalErr("Mac_ItrInstOfPortOfNet() failed: no master is given");
    }
    else if ((func == (BearStatus (*)()) NULL))
    {
        MacPrintFatalErr("Mac_ItrInstOfPortOfNet() failed: no function is given");
    }
#endif DEBUG
    if (master->instList == NIL(PortInst))
    {
        return (BEAR_COMPLETE_SEARCH);
    }
    else
    {
        inst = master->instList;
        while (inst != NIL(PortInst))
        {
            if (((*func)(inst, clientData)) == BEAR_STOP_SEARCH) 
            {
                return (BEAR_STOP_SEARCH);
            }
            inst = inst->nextNetPortPtr;
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Mac_ItrPinOfPortInst() --
 *
 * Fetches the list of pin instances from th given port instance.
 * And iterates the list of pins.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, if the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Mac_ItrPinOfPortInst(inst, func, clientData)
PortInst   *inst;
BearStatus (*func)();
ClientData clientData;
{
    PinInst  *pinPtr;

#ifdef DEBUG
    if (inst == NIL(PortInst)) {
  MacPrintFatalErr("Mac_ItrPinOfPortInst() failed: no port instance is given");
    }
    else if (func == (BearStatus (*)()) NULL) {
       MacPrintFatalErr("Mac_ItrPinOfPortInst() failed: no function is given");
    }
#endif DEBUG

    if (inst->pinList == NIL(PinInst)) {
        return (BEAR_COMPLETE_SEARCH);
    }
    else {
        for (pinPtr = inst->pinList;
            pinPtr != NIL(PinInst);
            pinPtr = pinPtr->nextPtr) {

            if (((*func)(pinPtr, clientData)) == BEAR_STOP_SEARCH) {
                return (BEAR_STOP_SEARCH);
            }
        }
    }

    return (BEAR_COMPLETE_SEARCH);
}



