
/* macPrint.c --
 *
 *     Wei-Ming Dai
 *     (Advising Professor - E. S. Kuh)
 *
 * It contains all the printing functions for mac.
 *
 * Copyright (C) 1987 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "macInt.h"


typedef struct _PrintPPinData {
    Net         *netPtr;
    int         count;
    bool        oneNetFlag;
    bool        countFlag;
} PrintPPinData;


/* ---------- Local Procedures ---------- */
static BearStatus PrintGPin();
static BearStatus PrintRPin();


static BearStatus
PrintGPin(gpinPtr, prtDataPtr)
GenericPin      *gpinPtr;
PrintPPinData   *prtDataPtr;
{
    if (gpinPtr->pinType == PIN_INST) {
        (void) Mac_PrintPinInst(gpinPtr->pinPtrUn.pinInstPtr,
            (ClientData) NULL);
        prtDataPtr->count++;
    }
    else {
        return (PrintRPin(gpinPtr->pinPtrUn.routePinPtr, prtDataPtr));
    }

    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
PrintRPin(rpinPtr, prtDataPtr)
RoutePin        *rpinPtr;
PrintPPinData   *prtDataPtr;
{
    Net     *netPtr;
    Point   absPos;

    netPtr = Mac_GetNetPtrOfRPin(rpinPtr);
    if ((prtDataPtr->oneNetFlag == TRUE) && (netPtr != prtDataPtr->netPtr)) {
        return (BEAR_CONTINUE_SEARCH);
    }

    /*
     * Begin line 1 of the output.
     */
    if (prtDataPtr->countFlag == TRUE) {
        printf ("%4d) ", prtDataPtr->count);
    }
    printf ("[rpin id = %d] ", Mac_GetIdOfRPin(rpinPtr));
    if (netPtr == NIL(Net)) {
        printf ("[net name = UNDEFINED] ");
        printf ("[net id = UNDEFINED]\n");
    }
    else {
        printf ("[net name = %s] ", Mac_GetNameOfNet(netPtr));
        printf ("[net id = %d]\n", Mac_GetIdOfNet(netPtr));
    }

    /*
     * Begin line 2 of the output.
     */
    printf ("      (mark = %s) (layer = %d) (order = %d) ",
        (rpinPtr->mark == BEAR_UNMARKED) ? "U" : "M",
        Mac_GetLayerOfRPin(rpinPtr),
        Mac_GetOrderOfRPin(rpinPtr));

    printf ("(dir = ");
    switch (Mac_GetDirectionOfRPin(rpinPtr)) {
    case NORTH_PIN:
        printf ("NORTH)\n");
        break;
    case SOUTH_PIN:
        printf ("SOUTH)\n");
        break;
    case EAST_PIN:
        printf ("EAST)\n");
        break;
    case WEST_PIN:
        printf ("WEST)\n");
        break;
    case NORTH_SOUTH_PIN:
        printf ("NORTH/SOUTH)\n");
        break;
    case EAST_WEST_PIN:
        printf ("EAST/WEST)\n");
        break;
    default:
        printf ("Illegal)\n");
        break;
    }

    /*
     * Begin line 3 of the output.
     */
    printf ("      (int net id = %d) (ext net id = %d)\n",
        Mac_GetIntNetIdOfRPin(rpinPtr), 
        Mac_GetExtNetIdOfRPin(rpinPtr));

    /*
     * Begin line 4 of the output.
     */
    printf ("      (vert ext net id = %d) (horz ext net id = %d)\n",
        Mac_GetVExtNetIdOfRPin(rpinPtr), 
        Mac_GetHExtNetIdOfRPin(rpinPtr));

    prtDataPtr->count++;

    return (BEAR_CONTINUE_SEARCH);
}



void
Mac_PrintTransform(transPtr)

Transform *transPtr;
{
#ifdef DEBUG
    if (transPtr == NIL(Transform)) {
        Bear_PrintErr("Mac_PrintTransform() failed");
    }
#endif DEBUG

    printf ("a = %d d = %d\n", transPtr->t_a, transPtr->t_d);
    printf ("b = %d e = %d\n", transPtr->t_b, transPtr->t_e);
    printf ("c = %d f = %d\n", transPtr->t_c, transPtr->t_f);
    fflush(stdout);
}


void
Mac_PrintPoint(pointPtr)

Point *pointPtr;
{
    if (pointPtr != NIL(Point)) 
    {
        printf ("x = %d  y = %d\n", pointPtr->p_x, pointPtr->p_y);
        fflush(stdout);
    }
}


void
Mac_PrintPointList(transPtr, pointList)

Transform *transPtr;
LinkedPoint *pointList;
{
    Point printPoint;
    register LinkedPoint *lpPtr;

    if (pointList != NIL(LinkedPoint))
    {
    if (transPtr == NIL(Transform) || pointList == NIL(LinkedPoint)) {
        MacPrintFatalErr("Mac_PrintPointList() failed");
    }

    for (lpPtr = pointList;
        lpPtr != NIL(LinkedPoint);
        lpPtr = lpPtr->nextPtr) {

        GeoTransPoint(transPtr, &lpPtr->point, &printPoint);
        Mac_PrintPoint(&printPoint);
    }
    }
}


void
Mac_PrintRect(rectPtr)
Rect *rectPtr;
{
#ifdef DEBUG
    if (rectPtr == NIL(Rect)) {
        MacPrintFatalErr("Mac_PrintRect() failed");
    }
#endif DEBUG

    printf ("xb = %d  xt = %d  yb = %d  yt = %d\n",
        rectPtr->r_xbot, rectPtr->r_xtop,
        rectPtr->r_ybot, rectPtr->r_ytop);
    fflush(stdout);
}


void
Mac_PrintRectList(transPtr, rectList)

Transform   *transPtr;
LinkedRect  *rectList;
{
    Rect printRect;
    register LinkedRect *lrPtr;

#ifdef DEBUG
    if (transPtr == NIL(Transform) || rectList == NIL(LinkedRect)) {
        MacPrintFatalErr("Mac_PrintRectList() failed");
    }
#endif DEBUG

    for (lrPtr = rectList;
        lrPtr != NIL(LinkedRect);
        lrPtr = lrPtr->r_next) {

        GeoTransRect(transPtr, &lrPtr->r_r, &printRect);
        Mac_PrintRect(&printRect);
    }
}



/* ARGSUSED */
BearStatus
Mac_PrintPinInst(pinInstPtr, clientData)
PinInst     *pinInstPtr;
ClientData  clientData;
{
    Point   absPos;
    Net     *netPtr;

    if (pinInstPtr == NIL(PinInst)) {
        return (BEAR_COMPLETE_SEARCH);
    }

    printf ("[pin inst. name = %s] ", pinInstPtr->pi_name);
    (void) Mac_GetAbsPosOfPin(pinInstPtr, &absPos);
    printf ("(abs pos = (%d, %d)) (width = %f)\n", 
        absPos.p_x, absPos.p_y, Mac_GetPinWidthOfPin(pinInstPtr));

    if ((netPtr = Mac_GetNetOfPin(pinInstPtr)) == NIL(Net)) {
        printf ("No net attached to the pin.\n");
    }
    else {
        printf ("[net name = %s] ", Mac_GetNameOfNet(netPtr));
        printf ("[net id = %d] ", Mac_GetIdOfNet(netPtr));
        printf ("(int net id = %d) (ext net id = %d)\n",
            Mac_GetIntNetIdOfPin(pinInstPtr), 
            Mac_GetExtNetIdOfPin(pinInstPtr));

        printf ("(vert ext net id = %d) (horz ext net id = %d) (layer = %d)\n",
            Mac_GetVExtNetIdOfPin(pinInstPtr), 
            Mac_GetHExtNetIdOfPin(pinInstPtr),
            Mac_GetLayerOfPin(pinInstPtr));
    }

    return (BEAR_CONTINUE_SEARCH);
}



/* ARGSUSED */
BearStatus
Mac_PrintRPin(rpinPtr, clientData)
RoutePin    *rpinPtr;
ClientData  clientData;
{
    PrintPPinData prtData;

    prtData.oneNetFlag = FALSE;
    prtData.netPtr = NIL(Net);
    prtData.countFlag = FALSE;

    return (PrintRPin(rpinPtr, &prtData));
}



/* ARGSUSED */
BearStatus
Mac_PrintGPin(gpinPtr, clientData)
GenericPin  *gpinPtr;
ClientData  clientData;
{
    PrintPPinData prtData;

    prtData.oneNetFlag = FALSE;
    prtData.netPtr = NIL(Net);
    prtData.countFlag = FALSE;

    return (PrintGPin(gpinPtr, &prtData));
}



void
Mac_PrintPPinListsOfRouteCell(rcellPtr, netPtr)
RouteCell   *rcellPtr;
Net         *netPtr;
/*
 * if netPtr != NIL(Net) then
 *     Print out information about only those ppin's that belong
 *     to this net.
 */
{
    PrintPPinData prtData;

    prtData.countFlag = TRUE;

    if (netPtr != NIL(Net)) {
        prtData.oneNetFlag = TRUE;
        prtData.netPtr = netPtr;

        printf ("\nOnly ppin's belonging to the net named %s are shown.\n\n",
            Mac_GetNameOfNet(prtData.netPtr));
    }
    else {
        prtData.oneNetFlag = FALSE;
        prtData.netPtr = NIL(Net);
    }

    printf ("\nBL bound pin list :\n");
    prtData.count = 1;
    (void) Mac_ItrBLBoundPinOfRCell(rcellPtr, PrintGPin, (ClientData) &prtData);

    printf ("\nTR bound pin list :\n");
    prtData.count = 1;
    (void) Mac_ItrTRBoundPinOfRCell(rcellPtr, PrintGPin, (ClientData) &prtData);
    
    printf ("\nTL side pin list :\n");
    prtData.count = 1;
    (void) Mac_ItrTLSidePinOfRCell(rcellPtr, PrintRPin, (ClientData) &prtData);

    printf ("\nR side pin list :\n");
    prtData.count = 1;
    (void) Mac_ItrRSidePinOfRCell(rcellPtr, PrintRPin, (ClientData) &prtData);

    printf ("\n"); fflush(stdout);
}



void
MacPrintPinError(pinInstPtr, msgPtr)
PinInst *pinInstPtr;
char    *msgPtr;
{
    CellInst    *cellInstPtr;
    Net         *netPtr;
    PortInst    *portInstPtr;

    netPtr = Mac_GetNetOfPin(pinInstPtr);
    portInstPtr = Mac_GetPortOfPin(pinInstPtr);
    cellInstPtr = Mac_GetCellInstOfPort(portInstPtr);

    Bear_PrintErr(msgPtr);
    (void) fprintf (bearerr, "Pin belongs to :\n");
    (void) fprintf (bearerr, "    net named  : \"%s\"\n",
        Mac_GetNameOfNet(netPtr));
    (void) fprintf (bearerr, "    cell named : \"%s\"\n",
        Mac_GetNameOfCellInst(cellInstPtr));
    (void) fprintf (bearerr, "\n");
}


