/*
 * macRing.h --
 *
 *      Wei-ming Dai
 *
 * It defines the data structures related to Ring and all the subroutines
 * to create, insert, and delete.
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header$
 */

#ifndef _MAC_RING

#define _MAC_RING

typedef enum _RingBound
{
    INNER_BOUND,    /* Inner boundary of the ring    */
    OUTER_BOUND     /* Outer boundary of the ring    */
} RingBound;

typedef struct _Ring
{
    BearId            id;               /* Routing ring ID                    */
    LinkedPoint       *inBoundary;      /* Inner boundary                 */
    LinkedPoint       *outBoundary;     /* Outer boundary                     */
    Rect              inBBox;           /* Inner bounding box                 */
    Rect              outBBox;          /* Outer bounding box                 */
    LinkedRect        *horzRectList;    /* List of horizontally sliced
                                         * rectangles
                                         */
    LinkedRect        *vertRectList;    /* List of vertically sliced
                                         * rectangles.
                                         */
    Point             origin;           /* Boundary of the routing ring
                                         * and all of its pins are expressed
                                         * relative to this point.
                                         */
    Transform         transform;        /* Transformation suitable for
                                         * placing the routing cell on the 
                                         * horzizontal tile plane.
                                         */
    module            *routeModPtr;     /* Pointer to the corresponding 
                                         * BBL routing module (see macBBL.h)
                                         */
    root              *routeRootList;   /* List of the BBL routing root
                                         * of the route cell. (see macBBL.h)
                                         */
    struct _LinkedGPin *inBoundPinList; /* List of pin inst. on the inner
                                         * boundary of the routing ring.
                                         */
    struct _LinkedGPin *outBoundPinList;/* List of pin inst. on the outer
                                         * boundary of the routing ring.
                                         */
    CellMaster        *cellMasterPtr;   /* The cell master which contains
                                         * the routing ring.
                                         */
    char              *cifFileName;     /* The cif file for the ring.         */
    ClientData        clientData;       /* This space for rent                */
} Ring;


extern BearStatus Mac_CreateRing(/* cellMasterPtr, ringPtrPtr */);
/*
 * CellMaster      *cellMasterPtr;
 * Ring            **ringPtrPtr;
 */

extern BearStatus Mac_DeleteRing(/* ringPtr */);
/*
 * Ring    *ringPtr;
 */

extern BearStatus Mac_InsertBoundPinOfRing (/* ringPtr, boundary, gPinPtr */);
/*
 * Ring        *ringPtr;
 * RingBound   boundary;
 * GenetricPin *gPinPtr;
 */

extern BearStatus Mac_DeleteBoundPinOfRing (/* ringPtr, boundary, gPinPtr */);
/*
 * Ring        *ringPtr;
 * RingBound   boundary;
 * GenetricPin *gPinPtr;
 */

extern BearStatus Mac_InsertRRootOfRing (/* ringPtr, rRootPtr */);
/*
 * Ring     *ringPtr;
 * root     *rRootPtr;
 */

extern BearStatus Mac_DeleteRRootOfRing (/* ringPtr, rRootPtr */);
/*
 * Ring    *ringPtr;
 * root    *rRootPtr;
 */

extern BearStatus Rdb_UpdateBndOfRing (/* adjParPtr, ringPtr */);
/*
 * AdjPar  *adjParPtr;
 * Ring    *ringPtr;
 */

#endif _MAC_RING
