/*
 * databaseInt.h --
 *
 * Definitions internal to the database module.
 *
 * Copyright (C) 1983 Regents of the University of California
 * All rights reserved.
 *
 * Needs to include: magic.h, tiles.h, database.h
 *
 * sccsid "@(#)databaseInt.h	4.1 MAGIC (Berkeley) 7/4/85"
 */

#ifndef _DATABASE_INT
#define _DATABASE_INT

#ifndef	_MAGIC
int err0 = Need_to_include_magic_header;
#endif	_MAGIC

#ifndef	_TILES
int err0 = Need_to_include_tiles_header;
#endif	_TILES

#ifndef	_DATABASE
int err0 = Need_to_include_database_header;
#endif	_DATABASE

/* ------------------------ Tunable constants ------------------------- */

    /*
     * The maximum number of uses of a given CellDef that may be
     * subcells of a given parent and have automatically generated
     * cu_id's is MAXDEFUSES.
     */
#define	MAXDEFUSES	1024

/* ----------- Argument to area search when writing out cell ---------- */

struct writeArg
{
    FILE	*wa_file;	/* File to which to output */
    TileType	 wa_type;	/* Type of tile being searched for */
    bool	 wa_found;	/* Have any tiles been found yet? */
};

/* --------------------- Undo info for painting ----------------------- */

/* The following is the structure of the undo info saved for each tile */
typedef struct
{
    Rect	 pue_rect;	/* Rectangle painted/erased */
    char	 pue_oldtype;	/* Material erased */
    char	 pue_newtype;	/* Material painted */
    char	 pue_plane;	/* Plane index affected */
} paintUE;

/* -------------- Codes for undo of cell use operations --------------- */

#define UNDO_CELL_CLRID		0	/* Clear use id */
#define UNDO_CELL_SETID		1	/* Set use id */
#define UNDO_CELL_PLACE		2	/* Create and place cell use */
#define UNDO_CELL_DELETE	3	/* Delete and destroy cell use */

/* -------------------- Internal procedure headers -------------------- */

extern void DBUndoPutLabel();
extern void DBUndoEraseLabel();
extern void DBUndoCellUse();
extern void DBStampMismatch();

extern void dbComputeBbox();
extern void dbFreeCellPlane();
extern void dbFreePaintPlane();
extern bool dbTechAddPaint();
extern bool dbTechAddErase();

/* --------------- Internal database technology variables ------------- */

/*
 * Macros to set the paint result tables.
 * The argument order is different from the index order in
 * the tables, for historical reasons.
 *
 * Usage:
 *	dbSetPaintEntry(oldType, paintType, planeNum, resultType)
 *	dbSetEraseEntry(oldType, paintType, planeNum, resultType)
 *	dbSetWriteEntry(oldType, paintType, resultType)
 */
#define	dbSetPaintEntry(h,t,p,r) 	(DBPaintResultTbl[p][t][h] = r)
#define	dbSetEraseEntry(h,t,p,r)	(DBEraseResultTbl[p][t][h] = r)
#define	dbSetWriteEntry(h,t,r)		(DBWriteResultTbl[t][h] = r)

#endif _DATABASE_INT

