/*
 * strdup.c --
 *
 * Return a malloc'd copy of a string.
 *
 * Copyright (C) 1983 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)strdup.c    4.1 MAGIC (Berkeley) 7/4/85";
#endif  not lint

#include <stdio.h>
#include <ctype.h>
#include "magic.h"
#include "malloc.h"


/*
 * ----------------------------------------------------------------------------
 * StrDup --
 *
 * Return a malloc'd copy of a string.
 *
 * Results:
 *  Returns a pointer to a newly malloc'd character array just
 *  large enough to hold the supplied string and its trailing
 *  null byte, which contains a copy of the supplied string.
 *
 * Side effects:
 *  MallocMagic's a character array large enough to hold str, and
 *  copies str into it, unless str is NULL.  If str is NULL, no
 *  malloc is done.  Also, if oldstr is non-NULL, then a) if
 *  *oldstr is not NULL, frees the storage allocated to oldstr,
 *  and b) sets *oldstr to the new string allocated, or to
 *  NULL if str is NULL.
 * ----------------------------------------------------------------------------
 */

char *
StrDup(oldstr, str)
    char **oldstr;
    char *str;
{
    char *newstr;

    if (str != NULL)
    {
    newstr = (char *) malloc((unsigned) (strlen(str) + 1));
    strcpy(newstr, str);
    }
    else newstr = NULL;
    if (oldstr != (char **) NULL)
    {
    if (*oldstr != NULL)
        free(*oldstr);
    *oldstr = newstr;
    }

    return (newstr);
}


/*
 * ----------------------------------------------------------------------------
 * StrIsWhite:
 *
 *  Check to see if a string is all white space or is a comment.
 *
 * Results:
 *  True if it is all white, false otherwise.
 *
 * Side effects:
 *  none.
 * ----------------------------------------------------------------------------
 */

bool
StrIsWhite(line, commentok)
    char *line;
    bool commentok; /* TRUE means # comments are considered all-white */
{
    if ( (*line == '#') && commentok)
    return TRUE;
    while(*line)
    {
    if ( !isspace(*line) && (*line != '\n') )
        return FALSE;
    line++;
    }
    return TRUE;
}
