#include "mis.h"
#include "graph_int.h"

int g_unique_id;

graph_t
g_alloc()
{
    graph_t_int *graph = ALLOC(graph_t_int,1);

    graph->user_data = (gGeneric) NULL;
    graph->v_list = lsCreate();
    graph->e_list = lsCreate();

    return((graph_t) graph);
}

void
g_free(g,f_free_g,f_free_v,f_free_e)
graph_t g;
void (*f_free_g)();
void (*f_free_v)();
void (*f_free_e)();
{
    lsGen gen;
    vertex_t v;
    edge_t e;

    if (g == (graph_t) NULL) {
        return;
    }
    if (f_free_g != (void (*)()) NULL) {
        (*f_free_g)(g->user_data);
    }
    foreach_vertex (g,gen,v) {
        if (f_free_v != (void (*)()) NULL) {
        (*f_free_v)(v->user_data);
    }
    (void) lsDestroy(((vertex_t_int *) v)->in_list,(void (*)()) NULL);
    (void) lsDestroy(((vertex_t_int *) v)->out_list,(void (*)()) NULL);
    FREE(v);
    }
    foreach_edge (g,gen,e) {
        if (f_free_e != (void (*)()) NULL) {
        (*f_free_e)(e->user_data);
    }
    FREE(e);
    }
    (void) lsDestroy(((graph_t_int *) g)->v_list,(void (*)()) NULL);
    (void) lsDestroy(((graph_t_int *) g)->e_list,(void (*)()) NULL);
    FREE(g);
}

GraStatus
g_check(g)
graph_t g;
{
    vertex_t_int *vertex;
    vertex_t v;
    edge_t_int *edge;
    edge_t e,test;
    lsGen gen,gen2;
    int found;
    int isGraphConnFlag = TRUE;     /* Is graph connected */

    if (g == (graph_t) NULL) {
        return (GRA_ERROR);
    }
    foreach_edge (g,gen,e) {
        edge = (edge_t_int *) e;
        if (edge->from == NIL(vertex_t_int)) {
            fail("In g_check: Edge has no source");
        }
        if (edge->to == NIL(vertex_t_int)) {
            fail("In g_check: Edge has no destination");
        }
        if (edge->from->g != edge->to->g) {
            fail("In g_check: Edge connects different graphs");
        }
        found = FALSE;
        foreach_out_edge ((vertex_t) edge->from,gen2,test) {
            if (test == e) {
                found = TRUE;
                (void) lsFinish(gen2);
                break;
            }
        }
        if (found == FALSE) {
            fail("In g_check: Vertex does not point back to edge");
        }
        found = FALSE;
        foreach_in_edge ((vertex_t) edge->to,gen2,test) {
            if (test == e) {
                found = TRUE;
                (void) lsFinish(gen2);
                break;
            }
        }
        if (found = FALSE) {
            fail("In g_check: Vertex does not point back to edge");
        }
    }
    foreach_vertex (g,gen,v) {
        vertex = (vertex_t_int *) v;
        if ((graph_t) vertex->g != g) {
            fail("In g_check: Vertex not a member of graph");
        }
        if (!lsLength(vertex->in_list) && !lsLength(vertex->out_list)) {
            isGraphConnFlag = FALSE;
            continue;
        }
        foreach_in_edge(v,gen2,test) {
            if (((edge_t_int *) test)->to != vertex) {
                fail("In g_check: Edge does not point back to vertex");
            }
        }
        foreach_out_edge(v,gen2,test) {
            if (((edge_t_int *) test)->from != vertex) {
                fail("In g_check: Edge does not point back to vertex");
            }
        }
    }
    if (isGraphConnFlag == FALSE) {
        return (GRA_DISCONN);
    }
    return (GRA_OK);
}

graph_t
g_dup(g,f_copy_g,f_copy_v,f_copy_e)
graph_t g;
gGeneric (*f_copy_g)();
gGeneric (*f_copy_v)();
gGeneric (*f_copy_e)();
{
    graph_t new;
    vertex_t v,new_v,from,to;
    edge_t_int *edge,*new_edge;
    edge_t e;
    st_table *ptrtable = st_init_table(st_ptrcmp,st_ptrhash);
    lsGen gen;

    new = g_alloc();
    if (g == (graph_t) NULL) {
        return(new);
    }

    if (f_copy_g == (gGeneric (*)()) NULL) {
        new->user_data = g->user_data;
    }
    else {
        new->user_data = (*f_copy_g)(g->user_data);
    }
    foreach_vertex (g,gen,v) {
        new_v = g_add_vertex(new);
    if (f_copy_v == (gGeneric (*)()) NULL) {
        new_v->user_data = v->user_data;
    }
    else {
        new_v->user_data = (*f_copy_v)(v->user_data);
    }
    (void) st_insert(ptrtable,(char *) v,(char *) new_v);
    }
    foreach_edge (g,gen,e) {
        edge = (edge_t_int *) e;
        (void) st_lookup(ptrtable,(char *) edge->from,(char **) &from);
    (void) st_lookup(ptrtable,(char *) edge->to,(char **) &to);
    new_edge = (edge_t_int *) g_add_edge(from,to);
    if (f_copy_e == (gGeneric (*)()) NULL) {
        new_edge->user_data = edge->user_data;
    }
    else {
        new_edge->user_data = (*f_copy_e)(edge->user_data);
    }
    }
    st_free_table(ptrtable);
    return(new);
}

array_t *
g_graph_sort(g,cmp)
graph_t g;
int (*cmp)();
{
    int i;
    lsGen gen;
    vertex_t v;
    array_t *v_array;

    i = 0;
    v_array = array_alloc(vertex_t,0);

    foreach_vertex (g,gen,v) {
        array_insert(vertex_t,v_array,i++,v);
    }
    array_sort(v_array,cmp);
    return(v_array);
}
