
#ifndef GRAPH_H
#define GRAPH_H

typedef enum _GraStatus {
    GRA_OK      = 0,
    GRA_ERROR   = 1,
    GRA_DISCONN = 2,    /* Disconnected graph */
    GRA_CORRUPT = 3     /* Corrupt graph      */
} GraStatus;

typedef char *gGeneric;

typedef struct graph_struct {
    gGeneric user_data;
} *graph_t;

typedef struct vertex_struct {
    gGeneric user_data;
} *vertex_t;

typedef struct edge_struct {
    gGeneric user_data;
} *edge_t;

extern graph_t g_alloc();
extern void g_free();
extern GraStatus g_check();
extern graph_t g_dup();

extern lsList g_get_vertices();

#define foreach_vertex(g, lgen, v)              \
    for (lgen = lsStart(g_get_vertices(g));         \
        lsNext(lgen, (lsGeneric *) &v, LS_NH) == LS_OK  \
           || ((void) lsFinish(lgen), 0); )

#define foreach_edge(g, lgen, e)                \
    for (lgen = lsStart(g_get_edges(g));            \
        lsNext(lgen, (lsGeneric *) &e, LS_NH) == LS_OK  \
           || ((void) lsFinish(lgen), 0); )

extern vertex_t g_add_vertex();
extern void g_delete_vertex();
extern graph_t g_vertex_graph();

extern lsList g_get_edges();
extern lsList g_get_in_edges();
extern lsList g_get_out_edges();

#define foreach_in_edge(v, lgen, e)             \
    for (lgen = lsStart(g_get_in_edges(v));         \
        lsNext(lgen, (lsGeneric *) &e, LS_NH) == LS_OK  \
           || ((void) lsFinish(lgen), 0); )

#define foreach_out_edge(v, lgen, e)                \
    for (lgen = lsStart(g_get_out_edges(v));        \
        lsNext(lgen, (lsGeneric *) &e, LS_NH) == LS_OK  \
           ||  ((void) lsFinish(lgen), 0); )

extern edge_t g_add_edge();
extern void g_delete_edge();
extern graph_t g_edge_graph();
extern vertex_t g_e_source();
extern vertex_t g_e_dest();

extern array_t *g_dfs();
extern int g_is_acyclic();
extern array_t *g_graph_sort();

#endif GRAPH_H

