#include "mis.h"
#include "graph_int.h"

static vertex_t_int *
find_an_end_vertex(vert,visit_list)
vertex_t_int *vert;
st_table *visit_list;
{
    edge_t e;
    edge_t_int *edge;
    lsGen gen;

    if (lsLength(vert->out_list) == 0) {
        return(vert);
    }
    foreach_out_edge ((vertex_t) vert,gen,e) {
        edge = (edge_t_int *) e;
	    (void) lsFinish(gen);
        if (st_insert(visit_list,(char *) edge->to,(char *) 0) == 1) {
	        return(NULL);
	    }
	    return(find_an_end_vertex(edge->to,visit_list));
    }
    /* no free out_edges */
    return(NULL);
}

static int
dfs_recurr(vert,dfs_list,dfs_array) 
vertex_t_int *vert;
st_table *dfs_list;
array_t *dfs_array;
{
    edge_t e;
    lsGen gen;
    int val;

    if (st_lookup(dfs_list,(char *) vert,(char **) &val)) {
        return(val == 0);
    }
    (void) st_insert(dfs_list,(char *) vert,(char *) 1);

    foreach_in_edge ((vertex_t) vert,gen,e) {
        if (!dfs_recurr(((edge_t_int *) e)->from,dfs_list,dfs_array)) {
	    return(0);
	}
    }
    (void) st_insert(dfs_list,(char *) vert,(char *) 0);
    array_insert_last(vertex_t,dfs_array,(vertex_t) vert);
    return(1);
}

static array_t *
g_dfs_int(g)
graph_t g;
{
    vertex_t_int *vert;
    vertex_t v;
    graph_t_int *graph = (graph_t_int *) g;
    lsGen gen;
    array_t *dfs_array;
    st_table *visit_list,*dfs_list;
    int cycle = FALSE;

    dfs_array = array_alloc(vertex_t,0);
    visit_list = st_init_table(st_ptrcmp,st_ptrhash);
    dfs_list = st_init_table(st_ptrcmp,st_ptrhash);

    foreach_vertex ((graph_t) graph,gen,v) {
        if (!st_is_member(dfs_list,(char *) v)) {
	    (void) st_insert(visit_list,(char *) v,(char *) 0);
	    vert = find_an_end_vertex((vertex_t_int *) v,visit_list);
	    if (vert == NULL || !dfs_recurr(vert,dfs_list,dfs_array)) {
	        cycle = TRUE;
		(void) lsFinish(gen);
		goto end;
	    }
	}
    }
end:
    st_free_table(visit_list);
    st_free_table(dfs_list);
    if (cycle == TRUE) {
        array_free(dfs_array);
        return(NULL);
    }
    return(dfs_array);
}

array_t *
g_dfs(g)
graph_t g;
{
    array_t *x;

    x = g_dfs_int(g);
    if (x == NULL) {
        fail("in g_dfs: Graph has cycle");
    }
    return(x);
}

int
g_is_acyclic(g)
graph_t g;
{
    array_t *x;

    x = g_dfs_int(g);
    if (x) {
        array_free(x);
	return(TRUE);
    }
    return(FALSE);
}
