#include "mis.h"
#include "graph_int.h"
#include "graph_static_int.h"

graph_t
g_alloc_static(ng, nv, ne)
int ng, nv, ne;
{
    graph_t g;

    g = g_alloc();
    g->user_data = (gGeneric) ALLOC(g_field_t, 1);
    ((g_field_t *) g->user_data)->num_g_slots = ng;
    ((g_field_t *) g->user_data)->num_v_slots = nv;
    ((g_field_t *) g->user_data)->num_e_slots = ne;
    ((g_field_t *) g->user_data)->user_data = (gGeneric) ALLOC(gGeneric, ng);
    return g;
}


void
g_free_static(g, f_free_g, f_free_v, f_free_e)
graph_t g;
void (*f_free_g)();
void (*f_free_v)();
void (*f_free_e)();
{
    vertex_t v;
    edge_t e;
    lsGen gen;
    lsGeneric junk;

    if (g == (graph_t) NULL) {
        return;
    }
    if (f_free_g != (void (*)()) NULL) {
        (*f_free_g)(((g_field_t *) g->user_data)->user_data);
    }
    FREE(((g_field_t *) g->user_data)->user_data);
    FREE(g->user_data);

    foreach_vertex(g,gen,v) {
        if (f_free_v != (void (*)()) NULL) {
	    (*f_free_v)(v->user_data);
	}
	FREE(v->user_data);
	(void) lsDestroy(((vertex_t_int *) v)->in_list,(void (*)()) NULL);
	(void) lsDestroy(((vertex_t_int *) v)->out_list,(void (*)()) NULL);
	(void) lsRemoveItem(((vertex_t_int *) v)->handle,&junk);
	FREE(v);
    }
    foreach_edge(g,gen,e) {
        if (f_free_e != (void (*)()) NULL) {
	    (*f_free_e)(e->user_data);
	}
	FREE(e->user_data);
	(void) lsRemoveItem(((edge_t_int *) e)->handle,&junk);
	FREE(e);
    }
    g_free(g,(void (*)()) NULL,(void (*)()) NULL,(void (*)()) NULL);
}

static graph_t theGraph;

static gGeneric
copy_v_slots(user_data)
gGeneric user_data;
{
    int i;
    int num_v_slots = ((g_field_t *) (theGraph->user_data))->num_v_slots;
    gGeneric *new = ALLOC(gGeneric,num_v_slots);

    for (i = 0; i < num_v_slots; i++) {
        new[i] = ((gGeneric *) user_data)[i];
    }
    return((gGeneric) new);
}    

static gGeneric
copy_e_slots(user_data)
gGeneric user_data;
{
    int i;
    int num_e_slots = ((g_field_t *) (theGraph->user_data))->num_e_slots;
    gGeneric *new = ALLOC(gGeneric,num_e_slots);

    for (i = 0; i < num_e_slots; i++) {
        new[i] = ((gGeneric *) user_data)[i];
    }
    return((gGeneric) new);
}    

graph_t
g_dup_static(g, f_copy_g, f_copy_v, f_copy_e)
graph_t g;
gGeneric (*f_copy_g)();
gGeneric (*f_copy_v)();
gGeneric (*f_copy_e)();
{
    g_field_t *gf, *gf2;
    graph_t g2;
    gGeneric *new;
    int i;

    if (f_copy_v == (gGeneric (*)()) NULL) {
        theGraph = g;        
        f_copy_v = copy_v_slots;
    }
    if (f_copy_e == (gGeneric (*)()) NULL) {
        theGraph = g;
	f_copy_e = copy_e_slots;
    }
    g2 = g_dup(g,(gGeneric (*)()) NULL,f_copy_v,f_copy_e);
    if (g == (graph_t) NULL) {
        return(g2);
    }

    gf = (g_field_t *) g->user_data;
    gf2 = ALLOC(g_field_t,1);
    gf2->num_g_slots = gf->num_g_slots;
    gf2->num_v_slots = gf->num_v_slots;
    gf2->num_e_slots = gf->num_e_slots;
    if (f_copy_g == (gGeneric (*)()) NULL) {
        new = ALLOC(gGeneric,gf->num_g_slots);
	for (i = gf->num_g_slots - 1; i >= 0; i--) {
	    new[i] = ((gGeneric *) gf->user_data)[i];
	}
	gf2->user_data = (gGeneric) new;
    }
    else {
        gf2->user_data = (*f_copy_g)(gf->user_data);
    }
    g2->user_data = (gGeneric) gf2;        
    return(g2);
}


void
g_set_g_slot_static(g, i, val)
graph_t g;
int i;
gGeneric val;
{
    if (g == (graph_t) NULL) {
        fail("In g_set_g_slot_static: Null graph");
    }
    ((gGeneric *) ((g_field_t *) g->user_data)->user_data)[i] = val;
    return;
}


gGeneric
g_get_g_slot_static(g, i)
graph_t g;
int i;
{
    if (g == (graph_t) NULL) {
        fail("In g_get_g_slot_static: Null graph");
    }
    return ((gGeneric *) ((g_field_t *) g->user_data)->user_data)[i];
}

void
g_copy_g_slots_static(g1,g2,f_copy_g)
graph_t g1,g2;
gGeneric (*f_copy_g)();
{
    g_field_t *gf1,*gf2;
    gGeneric slots1,*slots2;
    int n;

    if (g1 == (graph_t) NULL || g2 == (graph_t) NULL) {
        fail("In g_copy_g_slots_static: Null graph");
    }
    gf1 = (g_field_t *) g1->user_data;
    gf2 = (g_field_t *) g2->user_data;
    n = gf1->num_g_slots;

    if (n != gf2->num_g_slots) {
        fail("In g_copy_g_slots_static: Graphs have different numbers of slots");
    }
    slots1 = gf1->user_data;
    slots2 = (gGeneric *) gf2->user_data;
    if (f_copy_g == (gGeneric (*)()) NULL) {
        for (n-- ; n >= 0; n--) {
	    slots2[n] = ((gGeneric *) slots1)[n];
	}
    }
    else {
        FREE(slots2);
        gf2->user_data = (*f_copy_g)(slots1);
    }
}
