#include "mis.h"
#include "graph_int.h"
#include "graph_static_int.h"

vertex_t
g_add_vertex_static(g)
graph_t g;
{
    g_field_t *gf;
    vertex_t v;

    if (g == (graph_t) NULL) {
        fail("In g_add_vertex_static: Null graph");
    }
    gf = (g_field_t *) g->user_data;
    v = g_add_vertex(g);
    v->user_data = (gGeneric) ALLOC(gGeneric, gf->num_v_slots);
    return(v);
}
    

void
g_delete_vertex_static(v,f_free_v,f_free_e)
vertex_t v;
void (*f_free_v)();
void (*f_free_e)();
{
    edge_t e;
    lsGen gen;

    if (v == (vertex_t) NULL) {
        fail("In g_delete_vertex_static: Null vertex");
    }
    foreach_in_edge(v, gen, e) {
        if (f_free_e != (void (*)()) NULL) {
	    (*f_free_e)(e->user_data);
	}
	FREE(e->user_data);
    }
    foreach_out_edge(v, gen, e) {
        if (f_free_e != (void (*)()) NULL) {
	    (*f_free_e)(e->user_data);
	}
	FREE(e->user_data);
    }
    if (f_free_v != (void (*)()) NULL) {
        (*f_free_v)(v->user_data);
    }
    FREE(v->user_data);
    g_delete_vertex(v, (void (*)()) NULL, (void (*)()) NULL);
}


void
g_set_v_slot_static(v, i, val)
vertex_t v;
int i;
gGeneric val;
{
    if (v == (vertex_t) NULL) {
        fail("In g_set_v_slot_static: Null vertex");
    }
    ((gGeneric *) v->user_data)[i] = val;
}


gGeneric
g_get_v_slot_static(v, i)
vertex_t v;
int i;
{
    if (v == (vertex_t) NULL) {
        fail("In g_get_v_slot_static: Null vertex");
    }
    return ((gGeneric *) v->user_data)[i]; 
}

void
g_copy_v_slots_static(v1,v2,f_copy_v)
vertex_t v1, v2;
gGeneric (*f_copy_v)();
{
    int n;
    gGeneric slots1,*slots2;
    
    if (v1 == (vertex_t) NULL || v2 == (vertex_t) NULL) {
        fail("In g_copy_v_slots_static: Null vertex");
    }
    n = ((g_field_t *) ((vertex_t_int *) v1)->g->user_data)->num_v_slots;

    if (n != ((g_field_t *) ((vertex_t_int *) v2)->g->user_data)->num_v_slots) {
        fail("In g_copy_v_slots_static: Vertices have differing numbers of slots");
    }
    slots1 = v1->user_data;
    slots2 = (gGeneric *) v2->user_data;
    if (f_copy_v == (gGeneric (*)()) NULL) {
        for (n--; n >= 0; n--) {
	    slots2[n] = ((gGeneric *) slots1)[n];
	}
    }
    else {
        FREE(slots2);
        v2->user_data = (*f_copy_v)(slots1);
    }
}

