
/* nclAnneal.c --
 *
 *    (Advising Professor - E. S. Kuh)
 *
 */

#include "bearCopyRight.h"
#include "port.h"

#define DEBUG_LEVEL 0

#define SRAND(n) srand(n)
#define IRAND()  ((rand() & 0x0ff0) >> 4)
#define FRAND()  (rand()/(256.0*256.0*256.0*128.0))

/* ---------- Local Procedures ---------- */
static void     init_params();
static void     init_cluster();
static void     add_to_cluster();
static float    iterate_cluster();
static float    get_weight1();
static float    calc_new_cost();
static float    get_curr_cost();
static float    get_max_cost();
static void     make_new_cluster();
static void     init_weight_matrix();
static int      *translate_cluster_list();
static int      node_order();
static int      end_cond();
static int      compatible();
static int      update_clusters();

static void     print_clusters();
static void     print_weight_matrix();
static void     save_input_to_file();
static void     save_state_to_file();
static void     save_output_to_file();

/***************************************************************************/
/*                                                                         */
/*  Array to hold the current sizes of the clusters.                       */ 
/*  Storage is allocated when the top level function is called for the     */
/*  maximum possible number of clusters:  the number of nodes.             */
/*                                                                         */
/***************************************************************************/

int *cluster_size;
 
float curr_cluster_cost;
int num_curr_clusters;

static int num_edges;

/***************************************************************************/
/*                                                                         */
/*   Array of all the edges of the graph.  The array is taken from the     */
/*   argument of the top level function.                                   */
/*                                                                         */
/***************************************************************************/

static int *edge_list;

#define EDGE(i, n)  edge_list[2*(i) + (n)]

/***************************************************************************/ 
/*                                                                         */
/*  Array of weights of the edges of the graph.  The array is taken from   */
/*  an argument of the top level function.                                 */
/*                                                                         */
/***************************************************************************/

static double *edge_weight;

/***************************************************************************/
/*                                                                         */
/*  Adjacency matrix for the weights.                                      */
/*                                                                         */
/***************************************************************************/

static float *weight_matrix;

#define get_weight(i, j)   weight_matrix[(i)*num_nodes + (j)]

/***************************************************************************/
/*                                                                         */
/*  Array to hold information about the nodes of the graph.                */
/*                                                                         */
/***************************************************************************/

static int *curr_cluster;

static int num_nodes;

/***************************************************************************/
/*                                                                         */
/*  macro to return the value of the i'th node of the n'th cluster         */
/*                                                                         */
/***************************************************************************/

#define ELEMENT(n, i) curr_solution[(n)*max_cluster_size + (i)]  

/***************************************************************************/
/*                                                                         */
/*  The currently chosen clusters of nodes, and the number of current      */
/*  clusters.                                                              */
/*  The array in this form can be returned by the top level function.      */
/*                                                                         */
/***************************************************************************/

static int *curr_solution;

static int *dim;

#define NCL_NAME(n)   dim[3*(n)]
#define NCL_DIM(n)    dim[3*(n) + 1]

static double *dimArea;

#define NCL_AREA(n)   dimArea[(n)]

/***************************************************************************/
/*                                                                         */
/*   The constraints which must be adhered to.                             */ 
/*                                                                         */
/***************************************************************************/

static double max_area_ratio, min_area_ratio;
static double max_dim_ratio, min_dim_ratio;

static int max_cluster_size;

#define SIM_ANNEAL   0
#define BOLTZ_MACH   1
#define LOCAL_OPT    2

char *method_string[] = {
   "Standard Simulated Annealing",
   "Simulated Annealing with Boltzman Machine Acceptance Function",
   "Local Optimization" 
  };

FILE *debug;

float Temp;
float cooling_fctr;
unsigned max_iter;
int method;
int max_tries;

int num_moves;
int num_tries;


/***************************************************************************/
/*                                                                         */
/*  function:  DoSimAnnealing()                                            */
/*  arguments:                                                             */
/*                                                                         */
/*  top level function which interfaces with BEAR                          */
/*                                                                         */
/*  bug fix (George Carvalho 16 April 91)                                  */
/*      Added the array of doubles named Dim_area so that the area portion */
/*      of the input data will be valid for a larger range of values.      */
/*      The "area" portion of the Dim array is no longer used but it is    */
/*      present and must consequentally be skipped over.                   */
/*                                                                         */
/***************************************************************************/

void 
DoSimAnnealing(Dim, Dim_area, Edge_list, Edge_weight, Curr_solution,
    Num_nodes, Num_edges, Max_cluster_size,
    Num_clusters_ptr,
    Max_area_ratio, Max_dim_ratio)

int     *Dim;
double  *Dim_area;
int     *Edge_list;  
double  *Edge_weight;  
int     *Curr_solution;
int     Num_nodes;
int     Num_edges;
int     Max_cluster_size;
int     *Num_clusters_ptr;
int     Max_area_ratio;
int     Max_dim_ratio;
{
    float   calc_curr_cost;
    float   delta, last_cost;
    int     i, *temp_list;

    dim = Dim;
    dimArea = Dim_area;
    num_nodes = Num_nodes;

    curr_cluster = (int *) malloc(num_nodes * sizeof (int));
    for (i = 0; i < num_nodes; i++) { 
        curr_cluster[i] = -1; 
    }

    edge_list = Edge_list;
    num_edges = Num_edges;

    edge_weight = Edge_weight;

    curr_solution = Curr_solution;
    max_cluster_size = Max_cluster_size;

    max_area_ratio = Max_area_ratio;
    max_dim_ratio = Max_dim_ratio;

    min_area_ratio = 1.0 / max_area_ratio;
    min_dim_ratio = 1.0 / max_dim_ratio;

#ifdef DEBUG
    if (DEBUG_LEVEL) {
        debug = fopen("cluster.debug", "a");
    }
    else 
#endif /* DEBUG */
    {
        debug = (FILE *) NULL;
    }

#ifdef DEBUG
    if (DEBUG_LEVEL) {
        save_input_to_file();
    }
#endif /* DEBUG */

    temp_list = (int *)malloc(num_nodes*max_cluster_size*sizeof (int));
    cluster_size = (int *)malloc(num_nodes*sizeof (int));
    for (i = 0; i < num_nodes; ++i) {
        cluster_size[i] = 0;
    }
    init_weight_matrix();
    if (num_nodes <= 4) {
        print_weight_matrix();
    }
    SRAND(getpid());

#ifdef DEBUG
    if (DEBUG_LEVEL) {
        printf("Begin Clustering:\n");
        printf("# nodes: %d   # edges: %d  max cluster size: %d\n",
            num_nodes, num_edges, max_cluster_size);
        printf("max area ratio: %1.2lf  max dim ratio: %1.2lf\n", 
            max_area_ratio, max_dim_ratio);
    }
#endif /* DEBUG */

    curr_cluster_cost = get_max_cost();

#ifdef DEBUG
    if (DEBUG_LEVEL) {
        (void) printf ("Maximum cost = %f\n", (float)curr_cluster_cost); 
    }
#endif /* DEBUG */

    init_cluster();
    if (num_nodes <= 4) {
        print_weight_matrix();
    }
    init_params(DEBUG_LEVEL);

    translate_cluster_list(curr_solution, temp_list);

#ifdef DEBUG
    if (DEBUG_LEVEL) {
        save_state_to_file(temp_list);
    }
    if (DEBUG_LEVEL >= 2) {
        print_clusters(temp_list, num_curr_clusters, max_cluster_size); 
        (void) printf ("Cost = %f from decrementing\n", 
            (float)curr_cluster_cost); 
        (void) printf ("Cost = %f from get_curr_cost\n", 
            last_cost = get_curr_cost());
    }
#endif /* DEBUG */

    num_moves = 0;
    if (num_curr_clusters > 1) {
        do  {
            delta = iterate_cluster();  
/*
            translate_cluster_list(curr_solution, temp_list);
*/
#ifdef DEBUG
/*
            if (DEBUG_LEVEL) {
                printf("temperature: %6.4f\n", Temp);
                printf("cost:  %f   delta:  %f\n", curr_cluster_cost, delta);
                calc_curr_cost = get_curr_cost();
                if (fabs(calc_curr_cost - curr_cluster_cost) > 0.01) {
                    printf("WARNING:  cost from get_curr_cost is now %f\n", 
                        calc_curr_cost);
                }
            }
*/
            if ((DEBUG_LEVEL >= 2) && (delta != 0.0)) {
                print_clusters(temp_list, num_curr_clusters, max_cluster_size); 
                (void) printf ("cost is now                    %f\n", 
                    curr_cluster_cost);
                (void) printf ("previous cost minus delta is   %f\n", 
                    last_cost + delta);
                (void) printf ("cost from get_curr_cost is now %f\n", 
                    last_cost = (float)get_curr_cost());
            }
#endif /* DEBUG */
        } while (!end_cond()) ;
    }
    translate_cluster_list(curr_solution, curr_solution);

#ifdef DEBUG
    if (DEBUG_LEVEL)  {
        save_output_to_file();
        printf("Total number of iterations: %d\n", num_moves);
        printf("Final Temperature: %8.4f\n", Temp);
    }
#endif /* DEBUG */

    *Num_clusters_ptr = num_curr_clusters;
}


static void init_params(debug_level)
int debug_level;
{
#ifdef DEBUG
    FILE *params_file;

    params_file = fopen("cluster.prm", "r");
    if (params_file != NULL) {
        (void) fscanf (params_file, "%f%f%d%d%d\n",
            &Temp, &cooling_fctr, &max_iter, &max_tries, &method);
        fclose(params_file);     
    }
    else 
#endif /* DEBUG */
    {
        Temp = 8.0;
        cooling_fctr = 0.998;
        max_iter = 2500;
        method = SIM_ANNEAL;
        max_tries = 100;
    }

#ifdef DEBUG
   if (debug_level) {
      printf("Initializing parameters for clustering:\n");
      printf("Temp = %1.4f  alpha = %1.4f  max iter = %d  max_tries = %d\n",
             Temp, cooling_fctr, max_iter, max_tries);
      printf("Method: %s\n", method_string[method]); 
     }
#endif /* DEBUG */
}


/***************************************************************************/
/*                                                                         */
/*  function:  init_cluster()                                              */
/*  Sets up the global variables and data structures.  Chooses a first     */
/*  clustering using a greedy algorithm.                                   */
/*                                                                         */
/***************************************************************************/

static void init_cluster()
{
   int i, j;
   num_curr_clusters = 0;

   /*  should check to see if all incompatible size nodes here */
   if (num_nodes <= 3) {    /* hack to handle the crash with 2 or 3 nodes   */
      max_area_ratio = 100000.0;
      max_dim_ratio = 100000.0;
      min_area_ratio = 1.0/max_area_ratio;
      min_dim_ratio = 1.0/max_dim_ratio;
     }
   for (i = 0; i < num_nodes; ++i) {
      if (curr_cluster[i] != -1)  continue;
      for (j = 0; j < num_curr_clusters; ++j) 
         if (compatible(i, j, 1))  { add_to_cluster(i, j);  break; }
      if (curr_cluster[i] == -1)  make_new_cluster(i);
     }
}


/***************************************************************************/
/*                                                                         */
/*  function name:  end condition                                          */
/*  Checks to see if the terminate condition is reached.                   */
/*  Currently this condition occurs if 100 or more moves were required in  */
/*  the last iteration before one was accepted.                            */
/*                                                                         */
/***************************************************************************/

static int end_cond()
  {
   return (++num_moves >= max_iter);
  }
 

/***************************************************************************/
/*                                                                         */
/*  function name:  iterate cluster                                        */
/*  Performs one iteration of the simulated annealing algorithm.           */
/*  Trial moved are generated until one is accepted.                       */
/*                                                                         */
/***************************************************************************/

static float iterate_cluster()
  {
   int accepted = 0, time_out = 0;
   int cluster_cand1, cluster_cand2;  int node_cand1, node_cand2;
   int node_num1, node_num2;
   float Boltz_fctr, delta;
   delta = 0.0;
   num_tries = 0;
   while (!(accepted || time_out)) {
      cluster_cand1 = IRAND() % num_curr_clusters;
      while ((cluster_cand2 = IRAND() % num_curr_clusters) == cluster_cand1) ;
      node_num1 = IRAND() % cluster_size[cluster_cand1];
      node_num2 = IRAND() % cluster_size[cluster_cand2];
      node_cand1 = ELEMENT(cluster_cand1, node_num1);
      node_cand2 = ELEMENT(cluster_cand2, node_num2);
      if (++num_tries == max_tries)  time_out = 1;
      if (compatible(node_cand1, cluster_cand2, 0)
          && compatible(node_cand2, cluster_cand1, 0)) {
         delta = calc_new_cost(node_cand1, node_cand2, 
                               cluster_cand1, cluster_cand2);
/*
             printf("iterate: n1 %d  n2 %d  c1 %d  c2 %d  delta %f  Temp %f\n",
                    node_cand1, node_cand2, cluster_cand1, cluster_cand2, 
                    delta, Temp);
*/
  
         switch (method) {
            case SIM_ANNEAL:
               if (delta < 0.0)  accepted = 1;
               else {
                  Boltz_fctr = exp((double)(-delta/Temp));
                  if (FRAND() < Boltz_fctr)  accepted = 1;
                 }
               break;
            case BOLTZ_MACH:
               Boltz_fctr = exp((double)(-delta/Temp));
               if (FRAND() < Boltz_fctr/(1.0 + Boltz_fctr))  accepted = 1;
               break;
            case LOCAL_OPT:
               if (delta < 0.0)  accepted = 1;
               break;
           }
        }  /*  if chosen nodes and clusters are compatible  */
     }  /*  while (!(accepted || time_out))  */
   if (DEBUG_LEVEL >= 2)
      if (time_out)  printf("Iteration timed out\n");
      else  printf("Accepted change after %d tries\n", num_tries);
   Temp *= cooling_fctr;
   if (!time_out)
      update_clusters(node_cand1, node_cand2, cluster_cand1, cluster_cand2);
   return (delta);
  }


/*  Assumes c1 does not equal c2.                                       */ 
static float calc_new_cost(n1, n2, c1, c2)
int n1, n2;  int c1, c2;
  {
   int i, node;  float delta = 0.0;
   for (i = 0; i < cluster_size[c1]; ++i)  {
      node = ELEMENT(c1, i);
      if (node != n1)  {
         delta += get_weight(node, n1);
         delta -= get_weight(node, n2);
        }
     }
   for (i = 0; i < cluster_size[c2]; ++i)  {
      node = ELEMENT(c2, i);
      if (node != n2)  {
         delta += get_weight(node, n2);
         delta -= get_weight(node, n1);
        }
     }
   return (delta);
  }


/***************************************************************************/
/*                                                                         */
/*  function name:  get_curr_cost                                          */
/*  This function returns the total cost associated with all existing      */ 
/*  clusters, ignoring any nodes which have not yet been assigned to one.  */
/*                                                                         */
/***************************************************************************/

static float get_curr_cost()
  {
   int i, j, size1, size2, n1, n2;  float cost = 0.0;
   for (i = 0; i < num_curr_clusters; ++i) 
      for (j = i + 1; j < num_curr_clusters; ++j) {
      size1 = cluster_size[i];  size2 = cluster_size[j];
      for (n1 = 0; n1 < size1; ++n1)
         for (n2 = 0; n2 < size2; ++n2)
            cost += get_weight(ELEMENT(i, n1), ELEMENT(j, n2));
     }
   return (cost);
  }


static float get_max_cost()
  {
   int i;
   float cost = 0.0;
   for (i = 0; i < num_edges; ++i)  cost += edge_weight[i];
   return (cost);
  }


/***************************************************************************/
/*                                                                         */
/*  function name:  compatible                                             */
/*  arguments:  node_num, the number of the node to be checked             */
/*              cluster_num, the cluster it will be checked against        */
/*              check_size, determines whether to check againt max_size    */
/*  The function determines if any of the constraints will be violated if  */
/*  the node "node_num" is added to the cluster "cluster_num".  The three  */
/*  constraints checked are maximum cluster size, maximum area ratio, and  */
/*  maximum dimension ratio.                                               */
/*                                                                         */
/***************************************************************************/

static int compatible(node_num, cluster_num, check_size)
int node_num, cluster_num, check_size;
{
    double  ratio;
    int     i, size;

    size = cluster_size[cluster_num];
    if (check_size && size == max_cluster_size) {
        return (0);
    }

    for (i = 0; i < size; ++i) {
        ratio = NCL_AREA(node_num) / 
            NCL_AREA(ELEMENT(cluster_num, i));
        if ((ratio > max_area_ratio) || (ratio < min_area_ratio)) {
            return (0);
        }

        ratio = ((double) NCL_DIM(node_num)) / 
            ((double) NCL_DIM(ELEMENT(cluster_num, i)));
        if ((ratio > max_dim_ratio) || (ratio < min_dim_ratio)) {
            return (0);
        }
    }

    return (1);
}


/***************************************************************************/
/*                                                                         */
/*  function name:  make_new_cluster                                       */
/*  Sets up a cluster of maximum size (from the "max_cluster_size"         */
/*  parameter of "cluster", sets its size to 1, puts the argument as       */
/*  the only node in the cluster.                                          */ 
/*                                                                         */
/***************************************************************************/

static void make_new_cluster(node)
int node;
  {
   int cluster_num, i;
   cluster_num = num_curr_clusters++;
   ELEMENT(cluster_num, 0) = node;
   cluster_size[cluster_num] = 1;
   for (i = 1; i < max_cluster_size; ++i)   ELEMENT(cluster_num, i) = -1;
  }


/***************************************************************************/
/*                                                                         */
/*  function name:  get_weight1                                            */
/*  Returns the weight of the edge between the two nodes whose numbers are */
/*  its arguments (returns 0.0 if there is no edge).                       */
/*  Uses a linear search through the list.                                 */
/*                                                                         */
/***************************************************************************/

static float get_weight1(n1, n2)
int n1, n2;
  {
   int i;
   for (i = 0; i < num_edges; ++i) {
      if ((EDGE(i, 0) == NCL_NAME(n1) && EDGE(i, 1) == NCL_NAME(n2))
          || (EDGE(i, 1) == NCL_NAME(n1) && EDGE(i, 0) == NCL_NAME(n2)))
      return ((float)edge_weight[i]);
     }
   return (0.0);
  }


/***************************************************************************/
/*                                                                         */
/*  function name:  add_to_cluster                                         */
/*  Adds the node "node_num" to the cluster "cluster_num".                 */
/*  If the node is already in the cluster, no action is taken.             */
/*  Updates "curr_cost"                                                    */ 
/*                                                                         */
/***************************************************************************/

static void add_to_cluster(node_num, cluster_num)
int node_num, cluster_num;
  {
   int i;
   for (i = 0; i < cluster_size[cluster_num]; ++i) 
      if (ELEMENT(cluster_num, i) == node_num)  return;
   for (i = 0; i < cluster_size[cluster_num]; ++i) 
      curr_cluster_cost -= get_weight(node_num, ELEMENT(cluster_num, i));
   ELEMENT(cluster_num, cluster_size[cluster_num]) = node_num;
   curr_cluster[node_num] = cluster_num;
   ++cluster_size[cluster_num];
  }


/***************************************************************************/
/*                                                                         */
/*  function name:  remove_from_cluster                                    */
/*  Removes the node "node_num" from the cluster "cluster_num".            */
/*  If the node is not in the cluster, no action is taken.                 */
/*  Updates "curr_cost"                                                    */ 
/*                                                                         */
/***************************************************************************/

static void remove_from_cluster(node_num, cluster_num)
int node_num, cluster_num;
  {
   int i;
   for (i = 0; 
       (i < cluster_size[cluster_num]) && (ELEMENT(cluster_num, i) != node_num);
       ++i) ;
   if (i < cluster_size[cluster_num]) {
      --cluster_size[cluster_num];
      for ( ; i < cluster_size[cluster_num]; ++i)  
          ELEMENT(cluster_num, i) = ELEMENT(cluster_num, i + 1);
      for (i = 0; i < cluster_size[cluster_num]; ++i)  
         curr_cluster_cost += get_weight(ELEMENT(cluster_num, i), node_num);
     }
  }


/***************************************************************************/
/*                                                                         */
/*  function name:  update_clusters                                        */
/*  Removes "node1" from "cluster1", removes "node2" from "cluster2",      */
/*  adds "node1" to "cluster2", adds "node2" to "cluster1".                */
/*  Updates "curr_cost" (via calls to "remove_from_cluster" and            */
/*  "add_to_cluster".                                                      */ 
/*                                                                         */
/***************************************************************************/

static int update_clusters(node1, node2, cluster1, cluster2)
int node1, node2;
int cluster1, cluster2;
  {
   remove_from_cluster(node1, cluster1);
   remove_from_cluster(node2, cluster2);
   add_to_cluster(node1, cluster2);
   add_to_cluster(node2, cluster1);
  }


/* should sort first */
static int node_order(name)
int name;
  {
   int i;
   for (i = 0; i < num_nodes; ++i)  if (NCL_NAME(i) == name)  return (i); 
   return (0);
  }


static void init_weight_matrix()
{
    int i, j, order1, order2;

    weight_matrix = (float *)malloc(num_nodes*num_nodes*sizeof (float));
    for (i = 0; i < num_nodes; ++i) 
        for (j = 0; j < num_nodes; ++j) 
            weight_matrix[i*num_nodes + j] = 0.0;

    for (i = 0; i < num_edges; ++i)  {
        order1 = node_order(EDGE(i, 0));
        order2 = node_order(EDGE(i, 1));
        weight_matrix[order1*num_nodes + order2] = 
            weight_matrix[order2*num_nodes + order1] = edge_weight[i]; 

#ifdef DEBUG
        if (DEBUG_LEVEL && (num_nodes == 2)) {
            (void) printf ("init_weight_matrix() :  number of nodes = 2\n");
            (void) printf ("i = %d, order1 = %d, order2 = %d\n",
                i, order1, order2); 
        }
#endif /* DEBUG */
    }
}


static int *translate_cluster_list(in_list, out_list)
int *in_list, *out_list;
  {
   int i;
   for (i = 0; i < max_cluster_size*num_curr_clusters; ++i) 
      if (in_list[i] == -1)  out_list[i] = 0;
      else  out_list[i] = NCL_NAME(in_list[i]);
   return (out_list);
  }


static void print_clusters(list, num, max_size)
int *list, num, max_size;  
{
    int i, j, node;

    if (debug == (FILE *) NULL) {
        return;
    }

    (void) fprintf(debug, "Number of clusters: %d\n", num);
    for (i = 0; i < num; ++i) {
        fprintf(debug, "(");
        for (j = 0; j < max_size && (node = list[i*max_size + j]); ++j)
            fprintf(debug, !j ? "%3d" : ",%3d", node);
        fprintf(debug, ")\n");
    }
    fflush(debug);
}


static void save_input_to_file(file_name)
char *file_name;
  {
   int i;

    if (debug == (FILE *) NULL) {
        return;
    }

   fprintf(debug, "\nInput to DoSimAnnealing():\n");
   fprintf(debug, "NODES:\n# nodes: %d\n", num_nodes);
   for (i = 0; i < num_nodes; ++i)
      fprintf(debug, "     NAME: %d   AREA: %f   DIM: %d\n", 
              NCL_NAME(i), NCL_AREA(i), NCL_DIM(i));

   fprintf(debug, "EDGES:\n# edges: %d\n", num_edges);
   for (i = 0; i < num_edges; ++i)
      fprintf(debug, "     (%d,%d)  WEIGHT: %lf\n", 
              EDGE(i, 0), EDGE(i, 1), edge_weight[i]);

   fprintf(debug, "max cluster size: %d\n", max_cluster_size);
   fprintf(debug, "max area ratio: %lf\n", max_area_ratio);
   fprintf(debug, "max dim ratio: %lf\n", max_dim_ratio);
   fprintf(debug, "\n");
   fflush(debug);
  }


static void save_state_to_file(temp_list)
int *temp_list;
  {
   int i, j;

    if (debug == (FILE *) NULL) {
        return;
    }

   fprintf(debug, "\nCurrent state of DoSimAnnealing():\n");
   if (num_nodes <= 4)  print_weight_matrix();
   fprintf(debug, "CLUSTERS:\n# clusters: %d\n", num_curr_clusters);
   fprintf(debug, "Cost is %f\n", curr_cluster_cost);
   print_clusters(temp_list, num_curr_clusters, max_cluster_size);
   fprintf(debug, "\n");
   fflush(debug);
  }


static void save_output_to_file()
  {
   int i;

    if (debug == (FILE *) NULL) {
        return;
    }

   fprintf(debug, "\nOutput from DoSimAnnealing():\n");
   fprintf(debug, "CLUSTERS:\n# clusters: %d\n", num_curr_clusters);
   fprintf(debug, "Cost is %f\n", curr_cluster_cost);
   print_clusters(curr_solution, num_curr_clusters, max_cluster_size);
   fprintf(debug, "\n");
   fflush(debug);
  }


static void print_weight_matrix()
  {
   int i, j;  

    if (debug == (FILE *) NULL) {
        return;
    }

   fprintf(debug, "WEIGHT MATRIX:\n");
   for (i = 0; i < num_nodes; ++i) {
      for (j = 0; j < num_nodes; ++j)
         fprintf(debug, "%8.2f", get_weight(i, j));
      fprintf(debug, "\n");
     }
   fflush(debug);
  }


