
/* nclClster.c --
 *
 *    (Advising Professor - E. S. Kuh)
 *
 *    This subroutine is the heart of the program
 *    It takes a hypergraph as an input, covert it to
 *    a graph according to one of the rules and  performs
 *    a clustering of the nodes of the graph into groups
 *    of size at least kinf and at most ksup using matching
 *    as a subroutine.                                         
 */

#define DEBUG_COND 0

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint
    
#include "bearCopyRight.h"
#include "newCInt.h"

#ifdef USE_FORTRAN_CODE
extern void matchi_();
#endif /* USE_FORTRAN_CODE */

extern void c_matchi();

#define ARC(i, j)     arc[3*(i) + (j)]
#define GROUP(i, j)   group[2*(i) + (j)]
#define KLASS(i, j)   klass[4*(i) + (j)]


/* ---------- Local Procedures ---------- */
static float Cnvr1(); 


FILE *tmp_debug;


void
DoMatching(dimen, dimArea, netlst, wght, clustr, rnode, nnet, ksup,
    ilvl, method, cnvr, nclust, error, inner, areart, dimrt)

int     dimen[];               /* in:  node ptr, maxDim, "not used" for nodes */
double  dimArea[];             /* in:  area for nodes */

int     netlst[];              /* in:  node, connected node for nets */
double  wght[];                /* in:  weight for nets */
int     clustr[];              /* out: next level clustering results */ 
int     *rnode, *nnet, *ksup;  /* in:  numOfNodes, numOfNets, MAX_CHILDREN=4 */
int     *ilvl, *method, *cnvr; /* in:  cl_clusterTreeDepth, 1 (or 2), 
                                *      1 (or else) 
                                */
int     *nclust;               /* out: numOfClusters */ 
double  *error, *inner;        /* out:     */ 
int     *areart, *dimrt;       /* in:  clAreaRatioPtr, clDimRatioPtr */
/*
 *  bug fix (George Carvalho 16 April 91)
 *      Added the array of doubles named dimArea so that the area portion
 *      of the input data will be valid for a larger range of values.
 *      The "area" portion of the dimen array is no longer used but it is
 *      present and must consequentally be skipped over.
 */
{
    void  Dmatch(); 
    void  Solval(); 
    bool  Postp2(); 
    bool  Postp1();

/*  int  arc[20000][3], net[20000], point[2000];    */
/*  int  nodper[200], maxxy[200], area[200];        */
/*  int  assi1[200], assi2[200];  */
/*  int  ka[200], kb[200];        */
/*  float weight[20000];          */

    int     *arc, *net, *point, *nodper, *maxxy, *assi1, *assi2, *ka, *kb;
    double  *area;
    float   *weight;

    int     i, j, l, index, netid, node1, node2, node4, nodeid;
    bool    ichang;
    int     kinf, ibound, nnoimp, kaid;
    int     measur, maxl, clustr_add, cl_add,cl_id;
    int     node, narc;
    float   realma, totcon;

    node = (*rnode%4 == 0) ? *rnode : (*rnode/4+1)*4;

    if (DEBUG_COND) {
        tmp_debug = fopen("debug", "a");
    }

/**/
    if (DEBUG_COND) {
       fprintf(tmp_debug, "*rnode = %d  node = %d\n",*rnode, node);
       for (i = 0; i < *rnode; ++i) {
          fprintf (tmp_debug, "dimen[%d]=%d ",i, dimen[3 * i]);
       }

       fflush(tmp_debug);
    }
/**/

    /*  initializing the vector clustr */
    if (*rnode <= 4) {
        for (i = 0; i < *rnode; i++) {
            clustr[i] = dimen[3 * i];
            /* printf ("clustr[%d] = %d ",i,clustr[i]); */
        }
        *nclust = 1;
        *inner = (100.0);
        /* printf ("\n"); */
        /* printf ("*nclust = %d\n",*nclust); */
        return;
    } 
    else {
        arc = (int *)malloc(3*(node*(node - 1)/2)*sizeof (int));
        net = (int *)malloc(4*(*nnet)*sizeof (int));
        point = (int *)malloc((*nnet)*sizeof (int));
        nodper = (int *)malloc(node*sizeof (int));
        maxxy = (int *)malloc(node*sizeof (int));
        area = (double *)malloc(node*sizeof (double));
        assi1 = (int *)malloc(node*sizeof (int));
        assi2 = (int *)malloc(node*sizeof (int));
        ka = (int *)malloc(node*sizeof (int));
        kb = (int *)malloc(node*sizeof (int));
        weight = (float *)malloc((node*(node - 1)/2)*sizeof (float));

        /*  Performing permutation of nodes */
        for (i = 0; i < *rnode; i++) {
            j = 3 * i;
            nodper[i] = dimen[j];
            maxxy[i] =  dimen[j + 1];

            area[i] = dimArea[i];
        } 

        /*  storing netlist in vector net and point */
        index = 0;
        for (netid = 0; netid < *nnet; netid++) {
            point[netid] = index;
            net[index++] = 2;
            net[index++] = wght[netid]*(10000.0);
            node1 = netlst[netid*2];
            node2 = netlst[netid*2+1];
            for (nodeid = 0; nodeid < *rnode; nodeid++) {
                if (node1 == nodper[nodeid]) {
                    net[index++] = nodeid;
                    break;
                }
            }
            for (nodeid = 0; nodeid < *rnode; nodeid++) {
                if (node2 == nodper[nodeid]) {
                    net[index++] = nodeid;
                    break;
                }
            }
        }
/**/
     if (DEBUG_COND) {
        for (netid = 0; netid < *nnet; netid++) 
           fprintf(tmp_debug, "point[%d]: %d\n", netid, point[netid]);
        for (index = 0; index < 4*node*(node - 1)/2; ++index)
           fprintf(tmp_debug, "net[%d]: %d\n", index, net[index]);
        fflush(tmp_debug);
       }
/**/

        if (*cnvr == 1) {
            totcon = Cnvr1(net, point, nnet, node, arc, weight,
                maxxy, area, *areart, *dimrt);
        }

        /*  Calling the double matching procedure Dmatch */
        narc = node*(node-1)/2;
        node2 = node/2;
        node4 = node/4;
        kinf=0;
/**/
        if (DEBUG_COND) {
           for (i = 0; i < node*(node - 1)/2; ++i)
              fprintf(tmp_debug, "ARC(%d .):  %d %d %d\n",
                            i, ARC(i, 0), ARC(i, 1), ARC(i, 2));
           fflush(tmp_debug);
          }
/**/
        Dmatch(ilvl, node, node2, node4, narc, arc, 
            method, ka, kb, assi1, assi2);
        /* printf ("Dmatch was called\n"); */

        /*  Evaluating the solution */ 
        realma = 0.0;
        Solval(narc, arc, assi2, weight, &realma);
        /* printf ("Solval was called\n"); */

        /*  Submiting the solution to postprocessors */

        /*  Initializing the number of elements per cluster */
        for (kaid = 0; kaid < node4; kaid++) ka[kaid] = 4;
        if ((ibound = node/10) == 0) ibound++;
        ibound = (6 > ibound) ? 6 : ibound;
        ichang = TRUE;
        do {
            do {
                if (ichang) nnoimp = 0;

                /*  Calling the first post processor Postp1 */
                ichang = Postp1 (node, node4, narc, assi2, assi1,
                                  ka, *ksup, kinf, weight, &realma);

                /* printf("Postp1 was called\n"); */
                /*  Calling the second post processor Postp2 */
                ichang = Postp2 (rnode, node, narc, assi2, assi1,
                                  weight, &realma);                  
                /* printf("Postp2 was called\n"); */
            }
            while (ichang);
            /* printf ("%d,%d,\n",nnoimp,ibound);  */
        }
        while (++nnoimp < ibound);                    
        *inner = (100.0)*realma/totcon;
        /* printf ("done loop:  *inner = %f\n",*inner); */

        /*  Setting the vector kb to 0 for storage */
        for (i = 0; i < node; i++) kb[i] = 0;

        /*  counting the number of nodes for each cluster */
        for (i = 0; i < *rnode; i++) kb[assi2[i]]++;

        /*  filling the vector cluster */
        maxl = 0;
        for (i=0; i < *rnode; i++) {
            clustr_add = assi2[i]*(*ksup);
            while (clustr[clustr_add] != 0) clustr_add++;
            clustr[clustr_add] = nodper[i];
            if (maxl < clustr_add) maxl = clustr_add;
            /* printf ("clustr[],add,assi2[],i,maxl %d,%d,%d,%d,%d,\n", */
            /*          clustr[clustr_add],clustr_add,assi2[i],i,maxl); */
        }

        /*  counting the effective number of clusters */
        *nclust = 0;
        for (i = 0; i < node; ++i) {
            if (kb[i] != 0) (*nclust)++;
        }

        /*  making sure that children information is stored in vector 
         *  clustr in a contiguos fashion */
        /* printf ("clustr   [1]      [2]      [3]      [4],\n");
         * for (i = 0; i < maxl; ++i) {
         *     printf ("   %d",clustr[i]);
         *     printf ("   %d",clustr[++i]);
         *     printf ("   %d",clustr[++i]);
         *     printf ("   %d,\n",clustr[++i]);
         * }                                         
         * printf ("*nclust = %d\n",*nclust);
         */
        for (clustr_add = 0; clustr_add <= maxl; clustr_add += (*ksup)) {
            /* printf ("clustr[],add,cl_id,%d,%d,%d,\n",
             *          clustr[clustr_add],clustr_add,cl_id); */
            if (clustr[clustr_add] == 0) {
                for (cl_add = clustr_add; cl_add <= maxl-(*ksup); cl_add++) {
                    clustr[cl_add] = clustr[cl_add + (*ksup)];
                }
                clustr_add -= (*ksup);
            }
        }
    }
/*
    printf ("clustr [1]      [2]      [3]      [4]\n");
    for (i = 0; i < *nclust*(*ksup); ++i) {
        printf ("%9d",clustr[i]);
        printf ("%9d",clustr[++i]);
        printf ("%9d",clustr[++i]);
        printf ("%9d\n",clustr[++i]);
    }
    printf ("clster was terminated\n");
*/
    free(arc);
    free(net);
    free(point);
    free(nodper);
    free(maxxy);
    free(area);
    free(assi1);
    free(assi2);
    free(ka);
    free(kb);
    free(weight);
}


/*   subroutine Cnvr1 used to transform netlist format to edge list
 *   format according to rule 1.
 *   rule1: nets made of 2 nodes are transformed into arcs with
 *   same weight. net made of v nodes (v>=3) are transformed into
 *   complete subgraph with weight =(2/v(v-1))
 */

static float
Cnvr1(net, point, nnet, node, arc, weight, maxxy, area, areart, dimrt)
int     net[];          /* in   */
int     point[];        /* in   */ 
int     *nnet, node;    /* in   */
int     arc[];          /* out  */
float   weight[];       /* out  */
int     maxxy[];        /* in   */
double  area[];         /* in   */
int     areart;         /* in:  area ratio */
int     dimrt;          /* in:  dimension ratio */
{
    double  amax, amin;
    float   rcost, totcon;
    int     index1, index2, max_index, min_index, arc_add, net_id;
    int     ip, n1, kost, iarc, j, k, i1, i2, l1, l2, ma, mi;
    int     dmax, dmin; 

    totcon = 0.0;
    for (index2 = 1; index2 < node; index2++) {
        for (index1 = 0; index1 < index2; index1++) {
            max_index = (index1 > index2) ? index1 : index2;
            min_index = (index1 < index2) ? index1 : index2;
            arc_add = max_index*(max_index-1)/2+min_index;
            ARC(arc_add, 0) = min_index;
            ARC(arc_add, 1) = max_index;
            ARC(arc_add, 2) = 0;
            weight[arc_add] = 0.0;
           }
    }
/**/
        if (DEBUG_COND) {
           fprintf(tmp_debug, "ARC(%d .):  %d %d %d\n",
                            170, ARC(170, 0), ARC(170, 1), ARC(170, 2));
           fprintf(tmp_debug, "ARC(%d .):  %d %d %d\n",
                            189, ARC(189, 0), ARC(189, 1), ARC(189, 2));
           fflush(tmp_debug);
          }
/**/

    for (net_id = 0; net_id < *nnet; net_id++) {
        ip = point[net_id];
        n1 = net[ip];
        if (n1 <= 2) {
            kost = net[ip+1];
            rcost = net[ip+1];
        } else
        {
            kost = net[ip+1]*2/(n1*(n1-1));
            rcost = ((float) net[ip+1])*(2.0)/((float) (n1*(n1-1)));
        }
        for (k = 0; k < n1-1; k++) {
            i1 = net[ip+2+k];
            l1 = ip+3+k;
            l2 = ip+n1+1;
            if (DEBUG_COND) {
               fprintf(tmp_debug, "k %d  i1 %d  l1 %d  l2 %d\n", k, i1, l1, l2);
               fflush(tmp_debug);
              }
            do {
                j = l1;
                i2 = net[j];
                ma = (i1 > i2) ? i1 : i2;
                mi = (i1 < i2) ? i1 : i2;
                iarc = ma*(ma-1)/2+mi;
                ARC(iarc, 0) = mi;
                ARC(iarc, 1) = ma;
                if (DEBUG_COND) {
                    fprintf(tmp_debug, "iarc %d  mi %d  ma %d\n", iarc, mi, ma);
                    fflush(tmp_debug);
                }
                dmax = (maxxy[mi] > maxxy[ma]) ? maxxy[mi] : maxxy[ma];
                dmin = (maxxy[mi] < maxxy[ma]) ? maxxy[mi] : maxxy[ma];

                amax = (area[mi] > area[ma]) ? area[mi] : area[ma];
                amin = (area[mi] < area[ma]) ? area[mi] : area[ma];

                if ((dmax / dmin) > dimrt) {
                    kost = kost/2;
                    rcost = 0.1;
                }
                if (((int) (amax / amin)) > areart) {
                    kost = 1;
                    rcost = 0.1;
                }
                ARC(iarc, 2) = kost+ARC(iarc, 2);
                weight[iarc] = rcost+weight[iarc];
                l1++;
                totcon = totcon+rcost;
            }
            while (j != l2);
        }
    }

    return (totcon);
}
     

/*  subroutine to update cost  after i and j have been exchanged  */

void
Cosup1( node, weight, assi1, node1, node2, x4)
int  node;               /*in  node  */
float  weight[];         /*in  weight of a node  */
int  assi1[];            /*in  cluster id including a node  */
int  node1, node2;       /*in  i,j  */
float  *x4;              /*inout  cost  */
{
    int  node_id, max1, min1, max2, min2, add1, add2;

    for (node_id = 0; node_id < node; node_id++) {
        if (node_id != node1) {
            if (assi1[node_id] == assi1[node1]) {
                max1 = (node_id > node1) ? node_id : node1;
                min1 = (node_id < node1) ? node_id : node1;
                max2 = (node_id > node2) ? node_id : node2;
                min2 = (node_id < node2) ? node_id : node2;
                add1 = max1*(max1-1)/2+min1;
                add2 = max2*(max2-1)/2+min2;
                *x4 = *x4-weight[add1]+weight[add2];
            }
            if (node_id != node2) {
                if (assi1[node_id] == assi1[node2]) {
                    max1 = (node_id > node1) ? node_id : node1;
                    min1 = (node_id < node1) ? node_id : node1;
                    max2 = (node_id > node2) ? node_id : node2;
                    min2 = (node_id < node2) ? node_id : node2;
                    add1 = max1*(max1-1)/2+min1;
                    add2 = max2*(max2-1)/2+min2;
                    *x4 = *x4-weight[add2]+weight[add1];
                }
            }
        }
    }
 
    return;
}


/*
 *  This subroutine is used to update cost after i  
 *  has been inserted to cluster j  
 */

void
Cosup2 (node, weight, assi1, node1, klust, x4)

    int  node;               /*in  node  */
    float  weight[];         /*in  weight of a node  */
    int  assi1[];            /*in  cluster id including a node  */
    int  node1, klust;       /*in  i, new cluster id of node i  */
    float  *x4;              /*in  cost  */

{
    int  node_id, max1, min1, add;

    for (node_id = 0; node_id < node; node_id++) {
        if (node_id != node1) {
            if (assi1[node_id] != assi1[node1]) {
                if (assi1[node_id] == klust) {  
                    max1 = (node_id > node1) ? node_id : node1;
                    min1 = (node_id < node1) ? node_id : node1;
                    add = max1*(max1-1)/2+min1;
                    *x4 = *x4+weight[add];
                }
            } else
            {
                max1 = (node_id > node1) ? node_id : node1;
                min1 = (node_id < node1) ? node_id : node1;
                add = max1*(max1-1)/2+min1;
                *x4 = *x4-weight[add];
            }
        }
    }    
    return;
}


/*   this subroutine takes as an input a graph                        
 *   and cluster the nodes into sets of four                         
 *   the procedure uses matching successively                        
 *
 *   node =  number of nodes in the original graph                   
 *           this number should be a multiple of 4                   
 *   node2=  node/2                                                  
 *   node4 = node/4                                                  
 *   narc = number of arcs in the original graphs                   
 *   arc(narc,3) = integer where                                    
 *                 arc(i,1)=starting node of arc i                 
 *                 arc(i,2)=end node of arc i                       
 *                 arc(i,3)=weight of arc i                        
 *   in this subroutine the vector assi1 is used as an indicator    
 *   the vector assi2 is used as a pointer from the nodes to the   
 *   clusters  
 */

void
Dmatch(ilvl, node, node2, node4, narc, arc, 
        method,ka,kb,assi1,assi2)
                  
    int  *ilvl;
    int  node, node2, node4; /*in    */
    int  narc, *method;      /*in    */
    int  arc[];           /*in 1:starting node,2:end node,3:weight  */
    int  assi1[], assi2[];   /*out  */
    int  *ka, *kb;   /*out  */

{    
    void  Mathe1();
/*
    int  group[100][2], klass[50][4];
    int  basis[200], mem[200], p[200], cc[20000], sm[200];
    int  tma[200], tmb[200];
    int  nmatch[200], m1[200];
    float  y1[200], y2[200], dplus[200], dminus[200];
*/
    int *group, *klass;
    int *basis, *mem, *p, *cc, *sm;
    int *tma, *tmb;
    int *nmatch, *m1;
    float *y1, *y2, *dplus, *dminus;

    int  big, zfw, kk, i1, max, min, maxii;
    int  i, i2, nk, j;
    float  sup, eps;

    group = (int *)malloc(node*sizeof (int));
    klass = (int *)malloc(node*sizeof (int));
    basis = (int *)malloc(node*sizeof (int));
    mem = (int *)malloc(node*sizeof (int));
    p = (int *)malloc(node*sizeof (int));
    cc = (int *)malloc((node*(node - 1)/2)*sizeof (int));
    sm = (int *)malloc(node*sizeof (int));
    tma = (int *)malloc(node*sizeof (int));
    tmb = (int *)malloc(node*sizeof (int));
    nmatch = (int *)malloc(node*sizeof (int));
    m1 = (int *)malloc(node*sizeof (int));

    y1 = (float *)malloc(node*sizeof (float));
    y2 = (float *)malloc(node*sizeof (float));
    dplus = (float *)malloc(node*sizeof (float));
    dminus = (float *)malloc(node*sizeof (float));

    /*  Constructing the cost matrix for the first matching */
    big = 0;
    maxii=0;
    for (kk = 0; kk < narc; kk++) {
        if (ARC(kk, 0) != ARC(kk, 1)) {
            min = (ARC(kk, 0) < ARC(kk, 1)) ? ARC(kk, 0) : ARC(kk, 1);
            max = (ARC(kk, 0) > ARC(kk, 1)) ? ARC(kk, 0) : ARC(kk, 1);
            i1 = max*(max-1)/2+min;
            cc[i1] = ARC(kk, 2);
            if (cc[i1] > maxii) maxii = cc[i1];
            big = big+cc[i1];
            /* printf ("cc[%d] = %d",i1,cc[i1]); */
        }
    }
    /* printf("\n"); */
    for (i1 = 0; i1 < narc; i1++) {
        cc[i1] = (maxii-cc[i1])+1; 
        /* printf ("cc[%d] = %d",i1,cc[i1]); */
    }
    /* printf("\n"); */

    /*  Calling the first matching  */
    sup = big*100.0;
    eps = 0.000000001;
    for (i = 0; i< node; ++i) nmatch[i] = i+1;
    if (*method == 1) {
#ifdef USE_FORTRAN_CODE
        matchi_(&node,&zfw,nmatch,cc,p,basis,mem,
            ka,kb,sm,tma,tmb,m1,
            y1,y2,dplus,dminus,&sup,&eps);
#else
        c_matchi(node,&zfw,nmatch,cc,p,basis,mem,
            ka,kb,sm,tma,tmb,m1,
            y1,y2,dplus,dminus,sup,eps);
#endif USE_FORTRAN_CODE
        for (i = 0; i< node; ++i) --nmatch[i];
        /* printf ("mathci arg. = %d %d %f %f \n",node,zfw,sup,eps); */ 
    }  
    if (*method == 2) {
        Mathe1(node, nmatch, cc, tma);
    }
    /* fprintf(stderr, "matchi (1) was called\n");  */

    /* for (i = 0; i < node; ++i) {
     *     printf ("nmatch[%d] = %d ",i,nmatch[i]);
     * }
     * printf("\n"); */

    /*  Forming the group of two elements  */
    for (i = 0; i < node; i++) {
        assi2[i] = 0;
        assi1[i] = 0;
    }
    i1 = 0;
    for (i2 = 0; i2 < node; i2++) {
        if (assi1[i2] != 1) {
            GROUP(i1, 0) = i2;
            GROUP(i1, 1) = nmatch[i2];
            assi1[i2] = 1;
            assi1[nmatch[i2]] = 1;
            assi2[i2] = i1;
            assi2[nmatch[i2]] = i1;
            i1++;
        }
    }
    /* for (i = 0; i < i1; ++i) {
     *     printf ("GROUP(%d, 0,1) = %d  %d ",i,GROUP(i, 0),GROUP(i, 1));
     * }
     * printf("\n"); */
    /* for (i = 0; i < node; ++i) 
     *    printf ("assi1[%d], assi2[%d] =  %d,%d\n",i,i,assi1[i],assi2[i]); */

    /*  Constructing the cost matrix for the second matching  */
    nk = node2*(node2-1)/2;
    for (kk = 0; kk < nk; kk++) {
        cc[kk] = 0;
    }
    big = 0;
    for (kk = 0; kk < narc; kk++) {
        if (assi2[ARC(kk, 0)] != assi2[ARC(kk, 1)]) {
            min = (assi2[ARC(kk, 0)] < assi2[ARC(kk, 1)]) ?
                   assi2[ARC(kk, 0)] : assi2[ARC(kk, 1)];
            max = (assi2[ARC(kk, 0)] > assi2[ARC(kk, 1)]) ?
                   assi2[ARC(kk, 0)] : assi2[ARC(kk, 1)];
            i1 = max*(max-1)/2+min;
            cc[i1] = cc[i1]+ARC(kk, 2);
            big = big+ARC(kk, 2);
        }
    }
    maxii = 0;
    for (kk = 0; kk < nk; kk++) {
        if (cc[kk] > maxii) maxii = cc[kk];
    }   
    for (i = 0; i < node2; i++) {
        nmatch[i] = i+1;
    }
    for (kk = 0; kk < nk; kk++) {
        cc[kk] = maxii-cc[kk]+1;
    }
    
    /*  Calling the second matching  */
    sup = big*100.0;
    eps = 0.000000001;
    if (*method = 1) {
#ifdef USE_FORTRAN_CODE
        matchi_(&node2,&zfw,nmatch,cc,p,basis,mem,
                ka,kb,sm,tma,tmb,m1,
                y1,y2,dplus,dminus,&sup,&eps);
#else
        c_matchi(node2,&zfw,nmatch,cc,p,basis,mem,
                  ka,kb,sm,tma,tmb,m1,
                  y1,y2,dplus,dminus,sup,eps);
#endif USE_FORTRAN_CODE
        for (i = 0; i< node; ++i) --nmatch[i];
        /* printf ("mathci arg. = %d %d %f %f \n",node,zfw,sup,eps); */ 
    }
    if (*method = 2) {
        Mathe1 ( node2, nmatch, cc, tma);
    }
    /* fprintf(stderr, "matchi (2) was called\n");  */
    /* for (i = 0; i < node2; ++i) {
     *     printf ("nmatch[%d] = %d ",i,nmatch[i]);
     * } 
     * printf("\n"); */
    
    /*  Forming the clusters  */
    for (i = 0; i < node2; i++) {
        assi2[i] = 0;
        assi1[i] = 0;
    }
    i1 = 0;
    for (i2 = 0; i2 < node2; i2++) {
        if (assi1[i2] != 1) {
            KLASS(i1, 0) = GROUP(i2, 0); 
            KLASS(i1, 1) = GROUP(i2, 1);
            KLASS(i1, 2) = GROUP(nmatch[i2], 0);
            KLASS(i1, 3) = GROUP(nmatch[i2], 1);
            assi1[i2] = 1;
            assi1[nmatch[i2]] = 1;
        /* printf ("KLASS(%d, 4) = %d %d %d %d\n",i1,KLASS(i1, 0),KLASS(i1, 1),
         *                                      KLASS(i1, 2),KLASS(i1, 3)); */
            i1++;
         }
    }
    for (i = 0; i < node4; i++) {
        for (j = 0; j < 4; j++) {
            assi2[KLASS(i, j)] = i;
        }
    }
    /* for (i = 0; i < node; ++i) {
     *     printf ("assi1[%d], assi2[%d] =  %d,%d\n",i,i,assi1[i],assi2[i]);
    }*/
   free(group);
   free(klass);
   free(basis);
   free(mem);
   free(p);
   free(cc);
   free(sm);
   free(tma);
   free(tmb);
   free(nmatch);
   free(m1);
   free(y1);
   free(y2);
   free(dplus);
   free(dminus);
}



/*   subroutine to compute a weight perfect matching of a
 *   graph. the graph must have a even number of vertices and the
 *   cost can be real. the routine is written in fortran vi
 *   and is inspired by the heuristic vertex-scan in the paper by
 *   david avis: a survey of heuristics for the weighted matching
 *   problem, networks, vol. 13, 1983.
 *   parameters
 *   input:
 *          n    number of nodes (even). also dimension of the cost
 *               matrix
 *          eps  machine accuracy
 *          sup  large real number
 *          cc(k)upper triangular part of the integer cost matrix
 *                k=1,...,n*(n-1)/2
 *   output:
 *          zfw  cost of the optimal matching
 *          nmatch(n) optimal matching
 *   integer array of lenght n:
 *          basis(n),mem(n),ka(n),kb(n),sm(n),tma(n),tmb(n),m1(n),p(n)
 *   real arrays of lenght n:
 *          y1(n),y2(n),dplus(n),dminus(n)
 *   call to other subroutine:
 *          subroutine Ranper
 *          fuction    ggubfs  
 */

void
Mathe1 ( n, nmatch, cc, tma)
                  
    int  n;
    int  nmatch[], cc[], tma[];

{
    int  Ranper();
    int  node_i, node_j, index, nodmat, arcand, nocand;
    int  max, min, cc_id;

    for (node_i = 0; node_i < n; node_i++)
        nmatch[node_i] = 0;
    nodmat = 0;
    Ranper ( n, tma);
    for (index = 0; index < n; index++) {
        if (nmatch[node_i = tma[index]] == 0) {
            arcand = 10000000;
            nocand = 0;
            for (node_j = 0; node_j < n; node_j++) {
                if (node_i != node_j) {
                    if (nmatch[node_j] == 0) {
                        max = (node_i > node_j) ? node_i : node_j;
                        min = (node_i < node_j) ? node_i : node_j;
                        cc_id = (max-1)*(max-2)/2 + min;
                        if (cc[cc_id] < arcand) {
                            arcand = cc[cc_id];
                            nocand = node_j;
                        }
                    }
                }
            }
            nmatch[nocand] = node_i;
            nmatch[node_i] = nocand;
            nodmat = nodmat+2;
        } 
    }
    return;
}



/*
 *    subroutine Postp1. this subrouitine is a post processor
 *    to any feasible solution. it allows the size of the clusters
 *    to vary between the given bounds. it proceeds by considering
 *    each node
 *    at a time and move to another set without violating the
 *    size constraint. the search is resumed from the incumbent
 *    solution
 */

bool
Postp1 (node, node4, narc, assi2, assi1, ka, ksup, kinf, weight, realma)

    int   node, node4, narc, ksup, kinf;  /*in   */
    int   ka[];            /*in     */
    float weight[];        /*in     */
    int   assi1[];         /*out    */
    int   assi2[];         /*inout  */
    float *realma;         /*inout  */

{
    void Cosup2();
    bool  ichang;
    int   node_i, node_j;
    float x4;

    x4 = *realma;
    ichang = FALSE;

/*  copy assi2 to assi1  */
    for (node_i = 0; node_i < node; ++node_i) {
        assi1[node_i] = assi2[node_i];
    }
    for (node_i = 0; node_i < node; ++node_i) {
        for (node_j = 0; node_j < node4; ++node_j) {
            if ((assi1[node_i] != node_j) && (ka[node_j] < ksup)
                                          && (ka[assi1[node_i]] != kinf)) {
                Cosup2 (node, weight, assi1, node_i, node_j, &x4);
                if (x4 < *realma) {
                    x4 = *realma;
                } else
                {
                    if (x4 > *realma) ichang = TRUE;
                    *realma = x4;
                    ka[assi1[node_i]] = ka[assi1[node_i]]-1;
                    ka[node_j] = ka[node_j]+1;
                    assi1[node_i] = node_j;
                    assi2[node_i] = node_j;
                }
            }
        }
    }
    return (ichang);
}



/*   subroutine Postp2. this subrouitine is a post processor
 *   to the double matching procedure. it takes as input an
 *   assignement of nodes to cluster and performs all possible
 *   pair exchanges an return a new assignement (if better).
 *   the serch is resumed from the incumbent solution.
 *   the size of the cluster is preserved 
 */

bool
Postp2 (rnode, node, narc, assi2, assi1, weight, realma)

    int  *rnode, node;       /*in  */
    int  assi2[];            /*inout   */
    int  assi1[];            /*out   */
    float *realma;          /*inout  */
    float weight[];         /*in   */

{
    void  Cosup1();
    bool  ichang;
    int  node_i, node_j, deposit;
    float  x4;

    ichang = FALSE;
    x4 = *realma;

    /*  copy assi2 to assi1 and inintialize costkl and dumcos */
    for (node_i = 0; node_i < node; node_i++) {
        assi1[node_i] = assi2[node_i];
    }
    for (node_i = 0; node_i < node-1; node_i++) {
        for (node_j = node_i+1; node_j < node; node_j++) {
            if ((node_i <= *rnode || node_j <= *rnode) &&
                          assi1[node_i] != assi1[node_j]) {
                Cosup1 (node, weight, assi1, node_i, node_j, &x4);
                if (x4 < *realma) {
                    x4 = *realma;
                } else
                {
                    if (x4 > *realma) ichang = TRUE;
                    *realma = x4;
                    deposit = assi1[node_i];
                    assi1[node_i] = assi1[node_j];
                    assi1[node_j] = deposit;
                    assi2[node_i] = assi1[node_i];
                    assi2[node_j] = assi1[node_j];
                }
            }
        }
    }
    return (ichang);
}



int
Ranper ( n, a)

    int  a[];
{
    int i;
   
    for (i = 0; i < n; i++)
        a[i] = i;
    return;
}


/*
 *   this subroutine takes an assignement of nodes to   
 *   clusters and returns its value in realma        
 */

void 
Solval (narc, arc, assi2, weight, realma)

    int   narc;               /*in     */
    int   arc[], assi2[];   /*in     */
    float weight[];           /*in     */
    float *realma;            /*inout  */

{
    int   i;

    *realma = 0.0;
    for ( i = 0; i < narc; ++i) {
        if (assi2[ARC(i, 0)] == assi2[ARC(i, 1)]) {
            /* printf (" %d and %d in same cluster %d\n",ARC(i, 0),ARC(i, 1),
                                                      assi2[ARC(i, 0)]);
             * printf (" add %f to fct\n",weight[i]);   */
            *realma += weight[i];
        }
    }
    return;
}

