
#include "bearCopyRight.h"
#include "port.h"

/***********************************************************************
      SUBROUTINE MATCHI (N,ZFW,NMATCH,CC,P,BASIS,MEM,KA,KB,SM,TMA,TMB,M1
                         ,Y1,Y2,DPLUS,DMINUS,SUP,EPS)                   

      SUBROUTINE TO COMPUTE THE MINIMUM WEIGHT PERFECT MATCHING OF A    
       GRAPH. THE GRAPH MUST HAVE A EVEN NUMBER OF VERTICES AND THE     
      COST CAN BE REAL. THE ROUTINE IS WRITTEN IN FORTRAN VI BY         
      G.KAZAKIDIS AND CAN BE FOUND IN R.E. BURKARD AND U. DERIGS,       
      ASSIGNEMENT AND MATCHING PROBLEMS: SOLUITON METHODS WITH          
      FORTRAN PROGRAMS, LECTURE NOTES IN ECONOMICS AND MATHEMATICAL     
       SYSTEMS, SPRINGER VERLAG 1980.                                   
                                                                        
      PARAMETERS                                                        
      INPUT:                                                          
             N    NUMBER OF NODES (EVEN). ALSO DIMENSION OF THE COST    
                  MATRIX                                                
             EPS  MACHINE ACCURACY                                      
             SUP  LARGE REAL NUMBER                                     
             CC(K)UPPER TRIANGULAR PART OF THE INTEGER COST MATRIX      
                   K = 1,...,N*(N-1)/2                                    
      OUTPUT:                                                         
             ZFW  COST OF THE OPTIMAL MATCHING                          
             NMATCH(N) OPTIMAL MATCHING                                 
      INTEGER ARRAY OF LENGHT N:                                      
             BASIS(N),MEM(N),KA(N),KB(N),SM(N),TMA(N),TMB(N),M1(N),P(N) 
      REAL ARRAYS OF LENGHT N:                                        
             Y1(N),Y2(N),DPLUS(N),DMINUS(N)                             
      CALL TO OTHER SUBROUTINE:                                       
             SUBROUTINE SCAN1                                           
             SUBROUTINE SCAN2                                           

      INTEGER  BASIS(N),MEM(N),KA(N),KB(N)                              
      INTEGER  CC(N),P(N),SM(N),TMA(N),TMB(N)                           
      INTEGER  TOP,ZFW,NMATCH(N),M1(N)                                  
      REAL     Y1(N),Y2(N),DMINUS(N),DPLUS(N)                           

 ***********************************************************************/

#define INT         int
#define FLOAT       float

#define P(n)        p[(n) - 1]
#define BASIS(n)    basis[(n) - 1]
#define MEM(n)      mem[(n) - 1]
#define CC(n)       cc[(n) - 1]
#define M1(n)       m1[(n) - 1]
#define Y1(n)       y1[(n) - 1]
#define Y2(n)       y2[(n) - 1]
#define SM(n)       sm[(n) - 1]
#define TMA(n)      tma[(n) - 1]
#define TMB(n)      tmb[(n) - 1]
#define NMATCH(n)   nmatch[(n) - 1]
#define DPLUS(n)    dplus[(n) - 1]
#define DMINUS(n)   dminus[(n) - 1]
#define KA(n)       ka[(n) - 1]
#define KB(n)       kb[(n) - 1]

#define INT_BRANCH(expr, label1, label2, label3)  \
     if ((expr) < 0)  goto label1;                \
     else  if (!(expr))  goto label2;             \
     else  goto label3

#define FLOAT_BRANCH(expr, label1, label2, label3) \
     if ((expr) < 0.0)  goto label1;               \
     else  if ((expr) == 0.0)  goto label2;        \
     else  goto label3

void SCAN1();                                           
void SCAN2();                                           

void c_matchi(N, ZFW, nmatch, cc, p, basis, mem, ka, kb, sm, tma, tmb, m1,
              y1, y2, dplus, dminus, SUP, EPS)                   
INT N;
INT *ZFW;
INT nmatch[];
INT cc[];
INT p[]; 
INT basis[];
INT mem[];
INT ka[];
INT kb[];
INT sm[];
INT tma[];
INT tmb[];
INT m1[];
FLOAT y1[];
FLOAT y2[];
FLOAT dplus[];
FLOAT dminus[];
FLOAT SUP;
FLOAT EPS;
     {
      INT TOP;
      INT I2I2;
      INT MIN, MAX;
      INT I, I1, IND;
      INT N1, N2, N3;
      INT NN, NI, NC;
      INT NB, NBEST, NB1, NB2, NB3;
      INT NK, NK1, NK2, NK3, NK4, NKA, NKB;
      INT NM, NMB;

      FLOAT D, DBEST;
      FLOAT CO, YB, Y1B, Y2B;

      I2I2 = 2;
      P(I2I2) = 0;

      for (I = 3; I <= N; ++I) {
         I1 = I - 1;
         P(I) = P(I1) + I - 2;
        }

      TOP = N + 2;
      for (N1 = 1; N1 <= N; ++N1) {
         BASIS(N1) = N1;
         MEM(N1) = N1;
         Y1(N1) = 0.0;
         Y2(N1) = 0.0;
         SM(N1) = TOP;
         TMA(N1) = TOP;
         TMB(N1) = TOP;
         NMATCH(N1) = TOP;
         DPLUS(N1) = SUP;
         DMINUS(N1) = SUP;
         KA(N1) = 0;
         KB(N1) = N1;
        }

/*************************************************************************
***   START PROCEDURE
*************************************************************************/

       for (N1 = 1; N1 <= N; ++N1) {
          if (NMATCH(N1) != TOP)  goto L106;
          NN = 0;
          D = SUP;
          for (N2 = 1; N2 <= N; ++N2) { 
             MIN = N1;
             MAX = N2;
             INT_BRANCH(N1 - N2, L91, L104, L90);
 L90:        MAX = N1;
             MIN = N2;
 L91:        IND = P(MAX) + MIN;
             NC = CC(IND);
             CO = (FLOAT)NC - Y1(N2);
             FLOAT_BRANCH(CO - D, L101, L103, L104);
 L101:       D = CO;
             NN = 0;
 L102:       if (NMATCH(N2) != TOP)  goto L104;
             NN = N2;
             goto L104;
 L103:       if (NN == 0)  goto L102;
 L104:       ;
            }
          if (NN == 0)  goto L106;
          Y1(N1) = D;
          NMATCH(N1) = NN;
          NMATCH(NN) = N1;
L106:     ;
         }

/************************************************************************
***    INITIAL LABELING
************************************************************************/

       NN = 0;
       for (NI = 1; NI <= N; ++NI) { 
          if (NMATCH(NI) != TOP)  goto L110;
          NN = NN + 1;
          SM(NI) = 0;
          DPLUS(NI) = 0.0;
          Y1B = Y1(NI);
          for (NK = 1; NK <= N; ++NK) { 
             MIN = NI;
             MAX = NK;
             INT_BRANCH(NI - NK, L108, L105, L107);
 L107:       MAX = NI;
             MIN = NK;
 L108:       IND = P(MAX) + MIN;
             NC = CC(IND);
             CO = (FLOAT)NC - Y1B - Y1(NK);
             if (CO >= DMINUS(NK))  goto L105;
             DMINUS(NK) = CO;
             KA(NK) = NI;
 L105:       ;
            }
 L110:    ;
         }
       if (NN <= 1)  goto L700;

/***********************************************************************
**** EXAMINATION OF THE LABELING AND DECISION FOR THE NEXT STEP
************************************************************************/

 L200: 
       DBEST = SUP;
       for (NB = 1; NB <= N; ++NB) {
          if (BASIS(NB) != NB)  goto L215;
          D = DMINUS(NB);
          if (SM(NB) >=  TOP)  goto L205;
          D = 0.5*(D + DPLUS(NB));
          if (D > DBEST)  goto L215;
          NBEST = NB;
          DBEST = D;
          goto L215;
 L205:    if (TMA(NB) >= TOP)  goto L210;
          if (MEM(NB) == NB)  goto L215;
          D = D + Y1(NB);
          if (D >= DBEST)  goto L215;
          NBEST = NB;
          DBEST = D;
          goto L215;
 L210:  
          if (D >= DBEST)  goto L215;
          NBEST = NB;
          DBEST = D;
 L215:    ;
         }
  
        if (TMA(NBEST) < TOP)  goto L500;
        if (SM(NBEST) >= TOP)  goto L300;
        NKA = KA(NBEST);
        NKB = KB(NBEST);
        N1 = NBEST;
        NB1 = N1;
        N2 = BASIS(NKA);
        NB2 = N2;
 L220:  TMA(NB1) = NB2;
        NK = SM(NB1);
        if (NK == 0)  goto L225;
        NB2 = BASIS(NK);
        NB1 = TMA(NB2);
        NB1 = BASIS(NB1);
        goto L220;
 L225:  NB = NB1;
        NB1 = N2;
        NB2 = N1;
 L230:  if (TMA(NB1) < TOP)  goto L235;
        TMA(NB1) = NB2;
        NK = SM(NB1);
        if (NK == 0)  goto L600;
        NB2 = BASIS(NK);
        NB1 = TMA(NB2);
        NB1 = BASIS(NB1);
        goto L230;
 L235:  if (NB1 == NB)  goto L400;
        NK = TMA(NB);
        TMA(NB) = TOP;
        NM = NMATCH(NK);
        NB = BASIS(NM);
        goto L235;

/***********************************************************************
**** GROWING AN ALTERNATING TREE BY ADDING TWO EDGES
***********************************************************************/
  
 L300:  
        TMA(NBEST) = KA(NBEST);
        TMB(NBEST) = KB(NBEST);
        NM = NMATCH(NBEST);
        NMB = BASIS(NM);
        DPLUS(NMB) = DBEST;
        SM(NMB) = NMATCH(NMB);

        SCAN1(NMB, N, SUP, cc, p, basis, mem, ka, kb, sm, tma, tmb,
              y1, y2, dplus, dminus, m1);

        goto L200;

/***********************************************************************
****   SHINKING A BLOSSOM
***********************************************************************/
  
 L400: 
       YB = Y1(NB) + DBEST - DPLUS(NB);
       Y1(NB) = 0.0;
       NK1 = NB;
 L430: Y2(NK1) = Y2(NK1) + YB;
       NK1 = MEM(NK1);
       if (NK1 != NB)  goto L430;
       NK = MEM(NB);
       if (NB != N2)  goto L436;
 L435: N2 = N1;
       NB2 = TMA(NB);
 L436: MEM(NK1) = NB2;
       NM = NMATCH(NB2);
       SM(NB2) = NM;
       Y1B = Y1(NB2) + DMINUS(NB2) - DBEST;
       NK1 = NB2;
 L440: NK2 = NK1;
       Y2(NK2) = Y2(NK2) + Y1B;
       BASIS(NK2) = NB;
       NK1 = MEM(NK2);
       if (NK1 != NB2)  goto L440;
       KB(NB2) = NK2;
       Y1(NB2) = Y1B;
       NB1 = BASIS(NM);
       MEM(NK2) = NB1;
       Y1B = Y1(NB1) + DBEST - DPLUS(NB1);
       NK2 = NB1;
 L445: NK1 = NK2;
       Y2(NK1) = Y2(NK1) + Y1B;
       BASIS(NK1) = NB;
       NK2 = MEM(NK1);
       if (NK2 != NB1)  goto L445;
       KB(NB1) = NK1;
       Y1(NB1) = Y1B;
       if (N2 == NB1)  goto L450;
       NB2 = TMA(NB1);
       TMA(NB1) = TMB(NB2);
       TMB(NB1) = TMA(NB2);
       goto L436;
  
 L450: if (N2 == NBEST)  goto L455;
       TMA(N2) = NKB;
       TMB(N2) = NKA;
       if (NB != NBEST)  goto L435;
       goto L460;
 L455: TMA(NBEST) = NKA;
       TMB(NBEST) = NKB;
 L460: MEM(NK1) = NK;
       N1 = MEM(NB);
       KA(N1) = NK;
       DPLUS(N1) = YB;
       TMA(NB) = TOP;
       DPLUS(NB) = DBEST;

       SCAN1(NB, N, SUP, cc, p, basis, mem, ka, kb, sm, tma, tmb,
             y1, y2, dplus, dminus, m1);

       goto L200;

/***********************************************************************
**** EXPANSION OF A T LABELED BLOSSOM
***********************************************************************/
  
 L500:  ;  
        N1 = MEM(NBEST);
        NB3 = N1;
        NKA = KA(N1);
        NK2 = N1;
 L505:  NK1 = NK2;
        NKB = KB(NK1);
        Y1B = Y1(NK1);
  
 L510:  BASIS(NK2) = NK1;
        Y2(NK2) = Y2(NK2) - Y1B;
        if (NK2 == NKB)  goto L515;
        NK2 = MEM(NK2);
        goto L510;
 L515:  NK2 = MEM(NKB);
        MEM(NKB) = NK1;
        if (NK2 != NKA)  goto L505;
        Y1B = DPLUS(N1);
        Y1(NBEST) = Y1B;
        MEM(NBEST) = NKA;
        NK2 = NKA;
  
 L520:  Y2(NK2) = Y2(NK2) - Y1B;
        if (NK2 == NBEST)  goto L525;
        NK2 = MEM(NK2);
        goto L520;
  
 L525:   
        NK1 = NMATCH(NBEST);
        NB1 = BASIS(NK1);
        NK2 = SM(NB1);
        NB = BASIS(NK2);
        if (NB == NBEST)  goto L545;
        NB2 = NB;
 L530:  NK = TMA(NB2);
        NB1 = BASIS(NK);
        if (NB1 == NBEST)  goto L535;
        NB2 = SM(NB1);
        NB2 = BASIS(NB2);
        goto L530;
  
 L535:  TMA(NB) = TMA(NBEST);
        TMA(NBEST) = TMB(NB2);
        TMB(NB) = TMB(NBEST);
        TMB(NBEST) = NK;
        NK3 = SM(NB);
        NB3 = BASIS(NK3);
        NK4 = SM(NB3);
        SM(NB) = TOP;
        NMATCH(NB) = NK1;
        NB1 = NB3;
 L540:  NK1 = TMA(NB1);
        NK2 = TMB(NB1);
        TMA(NB1) = NK4;
        TMB(NB1) = NK3;
        SM(NB1) = NK1;
        NMATCH(NB1) = NK1;
        NB2 = BASIS(NK1);
        NMATCH(NB2) = NK2;
        NK3 = SM(NB2);
        SM(NB2) = NK2;
        if (NB2 == NBEST)  goto L545;
        NB1 = BASIS(NK3);
        NK4 = SM(NB1);
        TMA(NB2) = NK3;
        TMB(NB2) = NK4;
        goto L540;
  
 L545:   
        NK2 = TMB(NB);
        NB1 = BASIS(NK2);
        DMINUS(NB1) = DBEST;
        N1 = 0;
        if (NB1 == NB)  goto L555;
        NK1 = TMA(NB1);
        NB3 = BASIS(NK1);
        TMA(NB1) = TMA(NB);
        TMB(NB1) = NK2;
  
 L550:  NK = SM(NB1);
        SM(NB1) = TOP;
        NB2 = BASIS(NK);
        NK = TMA(NB2);
        TMA(NB2) = TOP;
        N2 = TMB(NB2);
        TMB(NB2) = N1;
        N1 = NB2;
        DPLUS(NB2) = DBEST;
        NB1 = BASIS(NK);
        DMINUS(NB1) = DBEST;
        if (NB1 != NB)  goto L550;
        TMA(NB) = N2;
        TMB(NB) = NK;
        SM(NB) = TOP;
  
        if (NB3 == NB)  goto L570;
  
 L555:  NB1 = 0;
        NB2 = NB3;
  
 L560:  NK = SM(NB2);
        SM(NB2) = TOP;
        TMA(NB2) = TOP;
        TMB(NB2) = NB1;
        NB1 = BASIS(NK);
        NK = TMA(NB1);
        SM(NB1) = TOP;
        TMA(NB1) = TOP;
        TMB(NB1) = NB2;
        NB2 = BASIS(NK);
        if (NB2 != NB)  goto L560;

        SCAN2(NB1, N, SUP, cc, p, basis, mem, ka, kb, sm, tma, tmb,
                   y1, y2, dplus, dminus);
 L570:   
 L575:  if (N1 == 0)  goto L200;
        NB = N1;

        SCAN1(NB, N, SUP, cc, p, basis, mem, ka, kb, sm, tma, tmb,
              y1, y2, dplus, dminus, m1);

        N1 = TMB(NB);
        TMB(NB) = TOP;
        goto L575;

/***********************************************************************
**** AUGMENTATION OF MATCHING
**** EXCHANGE OF THE MATCHING AND NON MATCHING EDGES ALONG THE
**** AUGMENTING PATH
***********************************************************************/
  
 L600:   
         NB = N1;
         NK = NKA;
 L605:   NB1 = NB;
 L606:   NMATCH(NB1) = NK;
         NK = SM(NB1);
         TMA(NB1) = TOP;
         if (NK == 0)  goto L607;
         NB2 = BASIS(NK);
         NK1 = TMA(NB2);
         NK = TMB(NB2);
         NB1 = BASIS(NK1);
         NMATCH(NB2) = NK1;
         goto L606;
  
 L607:   if (NB != N1)  goto L608;
         NB = N2;
         NK = NKB;
         goto L605;

/***********************************************************************
****    REMOVING ALL LABELS ON NO EXPOSED BASE NODES
***********************************************************************/
  
 L608:   
         for (NB = 1; NB <= N; ++NB) { 
            if (BASIS(NB) != NB)  goto L620;
            if (SM(NB) >= TOP)  goto L610;
            D = DBEST - DPLUS(NB);
            Y1(NB) = Y1(NB) + D;
            SM(NB) = TOP;
            if (NMATCH(NB) != TOP)  goto L615;
            SM(NB) = 0;
            DPLUS(NB) = 0.0;
            goto L616;
 L610:      if (TMA(NB) >= TOP)  goto L615;
            D = DMINUS(NB) - DBEST;
            Y1(NB) = Y1(NB) + D;
            TMA(NB) = TOP;
            TMB(NB) = TOP;
 L615:      DPLUS(NB) = SUP;
 L616:      DMINUS(NB) = SUP;
 L620:      ;
           }
         NN = NN - 2;
         if (NN <= 1)  goto L700;

/***********************************************************************
**** DETERMINATION OF THE NEW DMINUS VALUES
***********************************************************************/
  
          for (N1 = 1; N1 <= N; ++N1) {
             NB1 = BASIS(N1);
             if (SM(NB1) != 0)  goto L635;
             Y1B = Y1(NB1);
             Y2B = Y2(N1);
     
             for (N2 = 1; N2 <= N; ++N2) {
                NB2 = BASIS(N2);
                if (NB1 == NB2)  goto L630;
                MIN = N1;
                MAX = N2;
                INT_BRANCH(N1 - N2, L625, L630, L624);
 L624:          MAX = N1;
                MIN = N2;
 L625:          IND = P(MAX) + MIN;
                NC = CC(IND);
                CO = (FLOAT)NC - Y1B - Y2B;
                CO = CO - Y1(NB2) - Y2(N2);
                if (CO >= DMINUS(NB2))  goto L630;
                KA(NB2) = N1;
                KB(NB2) = N2;
                DMINUS(NB2) = CO;
 L630:          ;
               }
 L635:       ;
            }
         goto L200;

/***********************************************************************
**** GENERATION OF THE ORIGINAL GRAPH BY EXPANSION OF ALL
**** SHRUNK BLOSSOMS
***********************************************************************/
  
 L700:   
  
         *ZFW = 0;
         for (NB1 = 1; NB1 <= N; ++NB1) {
            if (BASIS(NB1) != NB1)  goto L702;
            if (SM(NB1) < 0)  goto L702;
            N2 = NMATCH(NB1);
            NB2 = BASIS(N2);
            N1 = NMATCH(NB2);
            SM(NB1) = -1;
            SM(NB2) = -1;
            MIN = N1;
            MAX = N2;
            INT_BRANCH(N1 - N2, L704, L702, L703);
 L703:      MAX = N1;
            MIN = N2;
 L704:      IND = P(MAX) + MIN;
            NC = CC(IND);
            D = (FLOAT)NC - Y1(NB1) - Y1(NB2);
            D = D - Y2(N1) - Y2(N2);
            *ZFW = *ZFW + NC;
 L702:      ;
           }
         for (N1 = 1; N1 <= N; ++N1) {
 L705:      NB = BASIS(N1);
            if (NB == N1)  goto L750; 
            NK2 = MEM(NB);
            NKA = KA(NK2);
            NB3 = NK2;
            YB = DPLUS(NK2);
  
 L710:      NK1 = NK2;
            NKB = KB(NK1);
            Y1B = Y1(NK1);
  
 L715:      BASIS(NK2) = NK1;
            Y2(NK2) = Y2(NK2) - Y1B;
            if (NK2 == NKB)  goto L720;
            NK2 = MEM(NK2);
            goto L715;
  
 L720:      NK2 = MEM(NKB);
            MEM(NKB) = NK1;
            if (NK2 != NKA)  goto L710;
            Y1(NB) = YB;
            MEM(NB) = NKA;
            NK2 = NKA;
  
 L725:      Y2(NK2) = Y2(NK2) - YB;
            if (NK2 == NB)  goto L730;
            NK2 = MEM(NK2);
            goto L725;
  
 L730:      NK = NMATCH(NB);
            NK1 = BASIS(NK);
            NK1 = NMATCH(NK1);
            NB1 = BASIS(NK1);
            if (NB == NB1)  goto L745;
            NMATCH(NB1) = NK;
            NB3 = TMA(NB1);
            NB3 = BASIS(NB3);
 L735:      NK3 = SM(NB1);
            NB2 = BASIS(NK3);
            NK1 = TMA(NB2);
            NK2 = TMB(NB2);
            NB1 = BASIS(NK1);
            NMATCH(NB1) = NK2;
            NMATCH(NB2) = NK1;
            MIN = NK1;
            MAX = NK2;
            INT_BRANCH(NK1 - NK2, L737, L705, L736);
 L736:      MAX = NK1;
            MIN = NK2;
 L737:      IND = P(MAX) + MIN;
            NC = CC(IND);
            D = (FLOAT)NC - Y1(NB1) - Y1(NB2);
            D = D - Y2(NK1) - Y2(NK2);
/*
            if (fabs((double)D) > EPS)
               fprintf(stderr, 
                  "Optimality conditions are violated at edge %3d <--> %3d)\n",
                  NK1 - 1, NK2 - 1);
*/
            *ZFW = *ZFW + NC;
            if (NB1 != NB)  goto L735;
 L740:      if (NB3 == NB)  goto L705;
 L745:      N2 = SM(NB3);
            NB2 = BASIS(N2);
            N3 = SM(NB2);
            MIN = N2;
            MAX = N3;
            INT_BRANCH(N2 - N3, L747, L705, L746);
 L746:      MAX = N2;
            MIN = N3;
 L747:      IND = P(MAX) + MIN;
            NC = CC(IND);
            D = (FLOAT)NC - Y1(NB2) - Y1(NB3);
            D = D - Y2(N2) - Y2(N3);

/*
            if (fabs((double)D) > EPS)  
               fprintf(stderr, 
                  "Optimality conditions are violated at edge %3d <--> %3d)\n",
                  N2 - 1, N3 - 1);
*/

            *ZFW = *ZFW + NC;
            N3 = TMA(NB2);
            NB3 = BASIS(N3);
            goto L740;
 L750:      ;
           }
        }

/*********************************************************************  

       SUBROUTINE SCAN1(NB1,N,SUP,CC,P,BASIS,MEM,KA,KB,SM,TMA,TMB,      
                        Y1,Y2,DPLUS,DMINUS,M1)                          
       INTEGER  BASIS(1),MEM(1),KA(1),KB(1)                             
       INTEGER  CC(1),P(1),SM(1),TMA(1),TMB(1)                          
       INTEGER  N,TOP,M1(1)                                             
       REAL Y1(1),Y2(1),DPLUS(1),DMINUS(1)                              

*********************************************************************/

void SCAN1(NB1, N, SUP, cc, p, basis, mem, ka, kb, sm, tma, tmb,
           y1, y2, dplus, dminus, m1)

INT NB1, N;
FLOAT SUP;
INT cc[], p[], basis[];
INT mem[], ka[], kb[], sm[], tma[], tmb[];
FLOAT y1[], y2[], dplus[], dminus[];
INT m1[];
                                                                       
/*********************************************************************      
**** SCANNING OF NODE NB1                                               
*********************************************************************/

      {
       INT I1, I2;
       INT N1, N2;
       INT NC;
       INT MIN, MAX;
       INT IND;
       INT TOP;
       INT NB2;

       FLOAT CO;
       FLOAT D1, D2;

       TOP = N + 2;
       D1 = DPLUS(NB1) - Y1(NB1);
       DMINUS(NB1) = SUP;
       D2 = D1 - Y2(NB1);
       TMA(NB1) = 0;
       I1 = 0;
       for (N2 = 1; N2 <= N; ++N2) {
          NB2 = BASIS(N2);
          if (TMA(NB2) < TOP)  goto L1300;
          I1 = I1 + 1;
          M1(I1) = N2;
          MIN = NB1;
          MAX = N2;
          INT_BRANCH(NB1 - N2, L1291, L1300, L1290);
L1290:    MAX = NB1;
          MIN = N2;
L1291:    IND = P(MAX) + MIN;
          NC = CC(IND);
          CO = (FLOAT)NC + D2;
          CO = CO - Y1(NB2) - Y2(N2);
          if (CO >= DMINUS(NB2))  goto L1300;
          KA(NB2) = NB1;
          KB(NB2) = N2;
          DMINUS(NB2) = CO;
L1300:    ;
         }
       TMA(NB1) = TOP;
       N1 = NB1;
       goto L1315;
L1305: D2 = D1 - Y2(N1);
       for (I2 = 1; I2 <= I1; ++I2) { 
          N2 = M1(I2);
          NB2 = BASIS(N2);
          MIN = N1;
          MAX = N2;
          INT_BRANCH(N1 - N2, L1308, L1310, L1307);
L1307:    MAX = N1;
          MIN = N2;
L1308:    IND = P(MAX) + MIN;
          NC = CC(IND);
          CO = (FLOAT)NC + D2;
          CO = CO - Y1(NB2) - Y2(N2);
          if (CO >= DMINUS(NB2))  goto L1310;
          KA(NB2) = N1;
          KB(NB2) = N2;
          DMINUS(NB2) = CO;
L1310:    ;
         }
L1315: N1 = MEM(N1);
       if (N1 != NB1)  goto L1305;
      }

/***********************************************************************
 
         SUBROUTINE SCAN2(NB,N,SUP,CC,P,BASIS,MEM,KA,KB,SM,TMA,TMB, 
                           Y1,Y2,DPLUS,DMINUS)
         INTEGER BASIS(1),MEM(1),KA(1),KB(1)
         INTEGER CC(1),P(1),SM(1),TMA(1),TMB(1)
         INTEGER N,TOP
         REAL Y1(1),Y2(1),DMINUS(1),DPLUS(1)
 
***********************************************************************/

void SCAN2(NB, N, SUP, cc, p, basis, mem, ka, kb, sm, tma, tmb,
           y1, y2, dplus, dminus)

INT NB, N;
FLOAT SUP;
INT cc[], p[], basis[], mem[], ka[], kb[], sm[], tma[], tmb[];
FLOAT y1[], y2[], dplus[], dminus[];

        {
         INT N1, N2;
         INT NNK;
         INT TOP;
         INT NKA, NKB;
         INT NC;
         INT MAX, MIN;
         INT NB1, NB2;
         INT IND;
         FLOAT D;
         FLOAT Y1B, Y2B;
         FLOAT CO;

/**********************************************************************
***** SCANNING OF NODE NB
**********************************************************************/
        
         TOP = N + 2;
L2300:   NB1 = NB;
         NB = TMB(NB1);
         TMB(NB1) = TOP;
         D = SUP;
         NKA = 0;
         NKB = 0;
         N1 = NB1;
         Y1B = Y1(NB1);
L2315:     
         Y2B = Y2(N1);
         for (N2 = 1; N2 <= N; ++N2) {
            NB2 = BASIS(N2);
            if (SM(NB2) >= TOP)  goto L2320;
            MIN = N1;
            MAX = N2;
            INT_BRANCH(N1 - N2, L2318, L2320, L2317);
L2317:      MAX = N1;
            MIN = N2;
L2318:      IND = P(MAX) + MIN;
            NC = CC(IND);
            CO = (FLOAT)NC - Y1B - Y2B;
            CO = CO - Y1(NB2) - Y2(N2);
            CO = CO + DPLUS(NB2);
            if (CO >= D)  goto L2320;
            NKA = N2;
            NKB = N1;
            D = CO;
L2320:      ; 
           }
         N1 = MEM(N1);
         if (N1 != NB1)  goto L2315;
         KA(NB1) = NKA;
         KB(NB1) = NKB;
         DMINUS(NB1) = D;
     if (NB != 0)  goto L2300;
    }
