
/* nclReadData.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "gtGraph.h"
#include "fpg.h"
#include "tileProp.h"
#include "mac.h"
#include "newC.h"
#include "nclMacros.h"

#ifdef USE_OCT_CODE
#include "vov.h"
#endif /* USE_OCT_CODE */


#define MAX_LINE_SIZE 1024

#define RETURN_ERR \
{ \
    fclose (filePtr); \
    return (BEAR_ERROR); \
}


/* ---------- Local Procedures ---------- */
static void         FreeShape();
static BearStatus   ResetChildCell();
static BearStatus   InitCriticalNet();
#ifdef DEBUG
static void         PrintShapeList();
#endif DEBUG


/* ---------- Local Global Variables ---------- */
#ifdef DEBUG
static bool         echoShapesFlag = FALSE;
#endif DEBUG


static void
FreeShape(item)
lsGeneric item;
{
    FREE((char *) item);
}


#ifdef DEBUG
static void
PrintShapeList(shapeList)
lsList shapeList;
{
    lsGen   gen;
    Shape   *shapePtr;
    int     i;

    i = 1;
    lsForeachItem(shapeList, gen, shapePtr) {
        (void) fprintf (bearout, "%3d) %8d %8d\n", i,
            shapePtr->horzDim, shapePtr->vertDim);
        i++;
    }
    (void) fprintf (stdout, "\n");
    fflush (stdout);
}
#endif DEBUG


/* ARGSUSED */
static BearStatus
ResetChildCell(cellInstPtr, clientData)
CellInst    *cellInstPtr;
ClientData  clientData;
{
    CellMaster  *cellMasterPtr;
    GenericCell gcell;
    LinkedRect  *horzRectList;
    Shape       curShape, *shapePtr;

    cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);

    cellMasterPtr->shapeFlag = MAC_UNDEF_SHAPE;
    if (cellMasterPtr->shapeList != (lsList) NULL) {
        lsDestroy(cellMasterPtr->shapeList, FreeShape);
        cellMasterPtr->shapeList = (lsList) NULL;
    }

    gcell.unType = CELL_INST;
    gcell.unValue.cellInstPtr = cellInstPtr;

    horzRectList = Mac_GetHorzRectListOfGCell(&gcell);
    if (horzRectList->r_next != NIL(LinkedRect)) {
        return (BEAR_CONTINUE_SEARCH);
    }

    (void) Mac_GetCurrentShape(cellInstPtr, &curShape);
    cellMasterPtr->shapeList = lsCreate();

    MALLOC(Shape *, shapePtr, sizeof (Shape));
    shapePtr->horzDim = curShape.horzDim;
    shapePtr->vertDim = curShape.vertDim;
    lsNewEnd(cellMasterPtr->shapeList, (lsGeneric) shapePtr, LS_NH);

    MALLOC(Shape *, shapePtr, sizeof (Shape));
    shapePtr->horzDim = curShape.vertDim;
    shapePtr->vertDim = curShape.horzDim;
    lsNewEnd(cellMasterPtr->shapeList, (lsGeneric) shapePtr, LS_NH);

    Mac_TidyUpShapeList(cellMasterPtr->shapeList);

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * InitCriticalNet() --
 *
 *    Critical net names and maximum or minimum bounds on their wire 
 *    length are stored in the Net structure in "macNet.h".
 *
 * Results:
 *    BEAR_CONTINUE_SEARCH.
 *
 * Side effects:
 *    None. 
 *
 * ----------------------------------------------------------------------------
 */
 
/* ARGSUSED */
static BearStatus 
InitCriticalNet(netPtr, clientData)
Net         *netPtr;        /* IN */
ClientData  clientData;
{
    Mac_PutCriticalityOfNet(0, netPtr);

    netPtr->minLength = 0;
    netPtr->maxLength = 0;

    return (BEAR_CONTINUE_SEARCH);
}


void
Ncl_ResetShapeLists(coreMasterPtr)
CellMaster *coreMasterPtr;
{
    (void) Mac_ItrChildrenOfCell(coreMasterPtr, ResetChildCell,
        (ClientData) NULL);
}


BearStatus
Ncl_ReadShapeData(coreMasterPtr, shapeFileNamePtr)
CellMaster  *coreMasterPtr;
char        *shapeFileNamePtr;
{
    FILE        *filePtr;
    lsGen       gen;
    lsList      mirrorList;
    lsList      shapeList = (lsList) NULL;
    CellMaster  *cellMasterPtr;
    CellInst    *cellInstPtr;
    Net         *netPtr;
    Shape       *mirrorShapePtr, *shapePtr;
    int         count, horzDim, vertDim;
    bool        eofFlag;
    bool        readShapeFlag;
    char        line[MAX_LINE_SIZE];
    char        *ciNamePtr; /* cell instance name */
    char        *tokenPtr;

    Ncl_ResetShapeLists(coreMasterPtr);

#ifdef USE_OCT_CODE
    (void) VOVinput(VOV_UNIX_FILE, shapeFileNamePtr);
#endif /* USE_OCT_CODE */

    if ((filePtr = fopen (shapeFileNamePtr, "r")) == NIL(FILE)) {
        (void) sprintf(bear_ErrorMsg,
            "Could not open shape file named \"%s\" for reading.",
            shapeFileNamePtr);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR_2);
    }

    eofFlag = FALSE;
    readShapeFlag = FALSE;
    while (TRUE) {
        if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) ==
            BEAR_ERROR) {
            RETURN_ERR;
        }
        if (eofFlag == TRUE) {
            fclose (filePtr);
            break;
        }

        tokenPtr = strtok(line, " ");
        if (strcmp(tokenPtr, "Version") == 0) {
            /* Someday we may want to verify the version number. */
            continue;
        }
        else if (strcmp(tokenPtr, "ShapesForCell") == 0) {
            if ((ciNamePtr = strtok(NIL(char), " ")) == NIL(char)) {
           Bear_PrintErr("Ncl_ReadShapeData() failed : cell name is missing.");
                RETURN_ERR;
            }
            if (Mac_IsMember(CELL_INSTANCE_HT, ciNamePtr) == TRUE) {
                cellInstPtr = 
                    (CellInst *) Mac_LookUp(CELL_INSTANCE_HT, ciNamePtr);
            }
            else { 
                Bear_PrintErr("Ncl_ReadShapeData() failed : cannot find cell.");
                RETURN_ERR;
            }

            cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);
            if (cellMasterPtr->shapeList != (lsList) NULL) {
                shapeList = cellMasterPtr->shapeList;
            }
            else {
                (void) sprintf (bear_ErrorMsg,
                 "Ncl_ReadShapeData() : cell instance %s is not a rectangle.\n",
                    ciNamePtr);
                Bear_PrintErr(bear_ErrorMsg);

                RETURN_ERR;
            }

            while (TRUE) {
                if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) ==
                    BEAR_ERROR) {
                    lsDestroy(shapeList, FreeShape);
                    RETURN_ERR;
                }
                if (eofFlag == TRUE) {
           Bear_PrintErr("Ncl_ReadShapeData() failed : eof detected to soon.");
                    lsDestroy(shapeList, FreeShape);
                    RETURN_ERR;
                }

                if (line[0] == ';') {
                    break;
                }
                count = sscanf (line, "%d %d", &horzDim, &vertDim);
                if (count != 2) {
              Bear_PrintErr("Ncl_ReadShapeData() failed : missing shape data.");
                    lsDestroy(shapeList, FreeShape);
                    RETURN_ERR;
                }

                MALLOC(Shape *, shapePtr, sizeof (Shape));
                shapePtr->horzDim = horzDim;
                shapePtr->vertDim = vertDim;
                lsNewEnd(shapeList, (lsGeneric) shapePtr, LS_NH);
                readShapeFlag = TRUE;
            }

            /*
             * Make a new list that contains the mirror image of each shape.
             */
            mirrorList = lsCreate();
            lsForeachItem(shapeList, gen, shapePtr) {
                MALLOC(Shape *, mirrorShapePtr, sizeof (Shape));
                mirrorShapePtr->horzDim = shapePtr->vertDim;
                mirrorShapePtr->vertDim = shapePtr->horzDim;
                lsNewEnd(mirrorList, (lsGeneric) mirrorShapePtr, LS_NH);
            }

            /*
             * Add the new list to 'shapeList'.
             */
            lsForeachItem(mirrorList, gen, mirrorShapePtr) {
                lsNewEnd(shapeList, (lsGeneric) mirrorShapePtr, LS_NH);
            }
            lsDestroy(mirrorList, (void (*)()) NULL);

            Mac_TidyUpShapeList(shapeList);

            cellMasterPtr->shapeFlag = MAC_FLEX_SHAPE;
            cellMasterPtr->shapeList = shapeList;

#ifdef DEBUG
            if (echoShapesFlag == TRUE) {
                fprintf (stdout, "Shape list for cell named \"%s\"\n",
                    ciNamePtr);
                PrintShapeList(shapeList);
            }
#endif DEBUG
        }
        else {
            Bear_PrintErr("Ncl_ReadShapeData() failed : unknown keyword.");
            RETURN_ERR;
        }
    }

    if (readShapeFlag == FALSE) {
        (void) sprintf (bear_Msg,
            "No shape specifications in file named \"%s\"", shapeFileNamePtr);
        Bear_Print(bear_Msg);
    }

    return (BEAR_OK);
}


/*
 * ----------------------------------------------------------------------------
 *
 * NclReadTimingData() --
 *
 *    Read a timing specification file named "*****.timing". It specifies 
 *    critical nets and maximum bounds or minimum bounds on their wire 
 *    length. Such info is stored in the Net structure in "macNet.h".
 *
 * Results:
 *    BEAR_OK is returned if everything is spiffy.
 *    BEAR_ERROR_2 if the file could not be opened for reading.
 *    BEAR_ERROR otherwise.
 *
 * Side effects:
 *    None.
 * 
 * ----------------------------------------------------------------------------
 */

BearStatus 
NclReadTimingData(coreMasterPtr, timingFileNamePtr)
CellMaster  *coreMasterPtr;
char        *timingFileNamePtr;
{
    FILE        *filePtr;
    Net         *netPtr;
    int         count, critNetCount, critNetLen;
    bool        eofFlag;
    char        netName[80], minMax[80];
    char        line[MAX_LINE_SIZE];
    char        *tokenPtr;

#ifdef USE_OCT_CODE
    (void) VOVinput(VOV_UNIX_FILE, timingFileNamePtr);
#endif /* USE_OCT_CODE */

    if ((filePtr = fopen (timingFileNamePtr, "r")) == NIL(FILE)) {
        (void) sprintf(bear_ErrorMsg,
            "Could not open timing file named \"%s\" for reading.",
            timingFileNamePtr);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR_2);
    }

    (void) Mac_ItrNetOfCell(coreMasterPtr, InitCriticalNet, (ClientData) NULL);

    critNetCount = 0;
    eofFlag = FALSE;
    while (TRUE) {
        if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) ==
            BEAR_ERROR) {
            RETURN_ERR;
        }
        if (eofFlag == TRUE) {
            fclose (filePtr);
            break;
        }

        tokenPtr = strtok(line, " ");
        if (strcmp(tokenPtr, "Version") == 0) {
            /* Someday we may want to verify the version number. */
            continue;
        }
        else if (strcmp(tokenPtr, "CriticalNetList") == 0) {
            while (TRUE) {
                if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) ==
                    BEAR_ERROR) {
                    RETURN_ERR;
                }
                if (eofFlag == TRUE) {
           Bear_PrintErr("NclReadTimingData() failed : eof detected to soon.");
                    RETURN_ERR;
                }

                if (line[0] == ';') {
                    break;
                }
                count = sscanf (line, "%s %s %d", netName, minMax, &critNetLen);
                if (count != 3) {
          Bear_PrintErr("NclReadTimingData() failed : missing crit net data.");
                    RETURN_ERR;
                }
                if (Mac_IsMember(NET_HT, netName) == TRUE) {
                    netPtr = (Net *) Mac_LookUp(NET_HT, netName);
                }
                else { 
           Bear_PrintErr("NclReadTimingData() failed : cannot find net.");
                    RETURN_ERR;
                }

                Mac_PutCriticalityOfNet(1, netPtr);
                if (strcmp(minMax, "MIN") == 0) {
                    netPtr->minLength = critNetLen;
                }
                else if (strcmp(minMax, "MAX") == 0) {
                    netPtr->maxLength = critNetLen;
                }
                else {
          Bear_PrintErr("NclReadTimingData() failed : illegal crit net data.");
                    RETURN_ERR;
                }

                critNetCount++;
            }
        }
        else {
            Bear_PrintErr("NclReadTimingData() failed : unknown keyword.");
            RETURN_ERR;
        }
    }

    if (critNetCount > 0) {
        (void) fprintf (stdout,
            "Read timing specifications from file named \"%s\"\n\n",
            timingFileNamePtr);
    }
    else {
        (void) fprintf (stdout,
            "No timing specifications in file named \"%s\"\n\n",
            timingFileNamePtr);
    }
    fflush (stdout);

    return (BEAR_OK);
}



BearStatus
Ncl_ReadControlData(fileNamePtr, parmsPtr)
char            *fileNamePtr;
Ncl_UserParms   *parmsPtr;
{
#define GET_VALUE \
    if ((valuePtr = strtok(NIL(char), " ")) == NIL(char)) { \
        (void) sprintf (bear_ErrorMsg, \
            "Ncl_ReadControlData() failed : %s value is missing.", \
            tokenPtr); \
        Bear_PrintErr(bear_ErrorMsg); \
        printf ("---\n\n"); \
        fflush (stdout); \
        fclose (filePtr); \
        return (BEAR_ERROR); \
    }

    FILE    *filePtr;
    int     count;
    bool    eofFlag;
    char    line[MAX_LINE_SIZE];
    char    *tokenPtr, *valuePtr;

#ifdef USE_OCT_CODE
    (void) VOVinput(VOV_UNIX_FILE, fileNamePtr);
#endif /* USE_OCT_CODE */

    if ((filePtr = fopen (fileNamePtr, "r")) == NIL(FILE)) {
        (void) sprintf(bear_ErrorMsg,
            "Could not open control file named \"%s\" for reading.",
            fileNamePtr);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR);
    }

#ifdef DEBUG
    (void) fprintf (bearout, "---\n");
    (void) fprintf (bearout, "Data from file named \"%s\"\n\n", fileNamePtr);
#endif /* DEBUG */

    eofFlag = FALSE;
    while (TRUE) {
        if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) == 
            BEAR_ERROR) {
            RETURN_ERR;
        }
        if (eofFlag == TRUE) {
            fclose (filePtr);
            break;
        }

        tokenPtr = strtok(line, " ");
        if (strcmp(tokenPtr, "newC.TimingFlag") == 0) {
            GET_VALUE;
            parmsPtr->timingFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr, 
                Bear_GetBooleanString(parmsPtr->timingFlag));
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newC.ChannelFreeFlag") == 0) {
            GET_VALUE;
            parmsPtr->chanFreeFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr, 
                Bear_GetBooleanString(parmsPtr->chanFreeFlag));
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newC.QuickRouteEstFlag") == 0) {
            GET_VALUE;
            parmsPtr->quickRtEstFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr, 
                Bear_GetBooleanString(parmsPtr->quickRtEstFlag));
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newC.RouteAdjust") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->routAdjust);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %lf\n",
                tokenPtr, parmsPtr->routAdjust);
#endif /* DEBUG */
        }

#ifdef DEBUG
        else if (strcmp(tokenPtr, "newC.Debug.MaxNumOfShapes") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%d", &parmsPtr->maxNumOfShapes);

            (void) fprintf (bearout, "%-33s : %u\n", 
                tokenPtr, parmsPtr->maxNumOfShapes);
        }
#endif /* DEBUG */
    }

#ifdef DEBUG
    (void) fprintf (bearout, "---\n\n");
    fflush (bearout);

    Ncl_EchoUserParms("Cluster parameters values from control data file.", 
        parmsPtr);
#endif /* DEBUG */

    return (BEAR_OK);

#undef GET_VALUE
}


#undef MAX_LINE_SIZE
#undef RETURN_ERR


