
/* nclWriteData.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "gtGraph.h"
#include "fpg.h"
#include "tileProp.h"
#include "mac.h"
#include "newG.h"
#include "newC.h"
#include "newCInt.h"

#ifdef USE_OCT_CODE
#include "vov.h"
#endif /* USE_OCT_CODE */


/* ---------- Local Procedures ---------- */
static BearStatus   AddNetToList();

#ifdef DEBUG
static BearStatus   WriteCellShapes();
#endif DEBUG


BearStatus
NclWriteCTree(rootPtr, ctreeFileName)
Ncl_Node    *rootPtr;
char        *ctreeFileName;
/*
 * The nd_id (identification) field of each node is filled in at 
 * the beginning of the placement process so we can play with the
 * field here without goofing anything up.
 */
{
    FILE        *filePtr;
    Ncl_Node    *nodePtr, *childPtr;
    lsGen       gen;
    lsList      bfsList;    /* List of nodes in bread first search order. */
    lsList      levelList;
    bool        rootFlag;
    int         i, id;
    char        *keyWordPtr;

    if (rootPtr == NIL(Ncl_Node)) {
        Bear_PrintErr("NclWriteCTree() : Cluster tree is empty.");
        return (BEAR_ERROR);
    }

    levelList = lsCreate();
    lsNewEnd(levelList, (lsGeneric) rootPtr, LS_NH);
    bfsList = lsCreate();
    while (TRUE) {
        if (lsDelBegin(levelList, (lsGeneric *) &nodePtr) != LS_OK) {
            break;
        }
        lsNewEnd(bfsList, (lsGeneric) nodePtr, LS_NH);

        for (i = 0; i < nodePtr->nd_numOfChildren; i++) {
            childPtr = nodePtr->nd_children[i];

            if (childPtr->nd_numOfChildren != 0) {
                lsNewEnd(levelList, (lsGeneric) childPtr, LS_NH);
            }
        }
    }
    lsDestroy(levelList, (void (*)()) NULL);

    id = 0;
    lsForeachItem(bfsList, gen, nodePtr) {
        nodePtr->nd_id = id;
        id++;
    }

#ifdef USE_OCT_CODE
    (void) VOVoutput(VOV_UNIX_FILE, ctreeFileName);
#endif /* USE_OCT_CODE */

    if ((filePtr = fopen (ctreeFileName, "w")) == NIL(FILE)) {
        (void) sprintf (bear_ErrorMsg,
            "Could not open cluster tree file \"%s\" for writing.\n",   
            ctreeFileName);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR);
    }

    (void) fprintf (filePtr, "Version 1.0\n\n");

    rootFlag = TRUE;
    lsForeachItem(bfsList, gen, nodePtr) {
        if (rootFlag == TRUE) {
            keyWordPtr = "RootCluster";
            rootFlag = FALSE;
        }
        else {
            keyWordPtr = "Cluster";
        }

        (void) fprintf (filePtr, "%s C%d\n", keyWordPtr, nodePtr->nd_id);
        for (i = 0; i < nodePtr->nd_numOfChildren; i++) {
            childPtr = nodePtr->nd_children[i];

            if (childPtr->nd_numOfChildren == 0) {
                (void) fprintf (filePtr, "%s\n",
                    Mac_GetNameOfCellInst(childPtr->nd_cellInstPtr));
            }
            else {
                (void) fprintf (filePtr, "C%d\n", childPtr->nd_id);
            }
        }
        (void) fprintf (filePtr, ";\n\n");
    }

    lsForeachItem(bfsList, gen, nodePtr) {
        nodePtr->nd_id = 0;
    }

    lsDestroy(bfsList, (void (*)()) NULL);
    fclose (filePtr);

    return (BEAR_OK);
}


#ifdef DEBUG
static BearStatus
WriteCellShapes(cellInstPtr, filePtr)
CellInst    *cellInstPtr;
FILE        *filePtr;
{
    GenericCell gcell;
    LinkedRect  *horzRectList;
    double      area, startAR, ar, delta;
    int         i, x, y;
    char        *ciNamePtr; /* cell instance name */

    ciNamePtr = Mac_GetNameOfCellInst(cellInstPtr);
    (void) fprintf (filePtr, "ShapesForCell %s\n", ciNamePtr);

    gcell.unType = CELL_INST;
    gcell.unValue.cellInstPtr = cellInstPtr;

    horzRectList = Mac_GetHorzRectListOfGCell(&gcell);
    if (horzRectList->r_next != NIL(LinkedRect)) {
        (void) sprintf (bear_ErrorMsg,
            "WriteCellShapes() : cell instance %s is not a rectangle.\n",
            ciNamePtr);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR);
    }
    x = horzRectList->r_r.r_xtop - horzRectList->r_r.r_xbot;
    y = horzRectList->r_r.r_ytop - horzRectList->r_r.r_ybot;
    (void) fprintf (filePtr, "%d %d\n", x, y);

    startAR = y / x;
    if (startAR < 1.0) {
        startAR = x / y;
    }
    startAR = MAX(startAR, 2.0);
    delta = (startAR - 1.0) / 5.0;

    area = x * y;
    for (i = 0, ar = startAR; i < 5 ;i++, ar -= delta) {
        x = sqrt (area / ar);
        y = sqrt (area * ar);
        (void) fprintf (filePtr, "%d %d\n", x, y);
    }

    x = y = sqrt (area); /* aspect ratio = 1 */
    (void) fprintf (filePtr, "%d %d\n", x, y);

    (void) fprintf (filePtr, ";\n\n");

    return (BEAR_OK);
}
#endif DEBUG



/*
 * ----------------------------------------------------------------------------
 *
 * Ncl_WriteBBLShapes() --
 *
 * Generates a shape file which is intended to be used as test input
 * for the floor planner.
 *
 * Results:
 *   BEAR_OK is returned if everything is spiffy.
 *   BEAR_ERROR is returned if the output file could not be opened
 *       for writing.
 *   BEAR_ERROR_2 is returned if one of the cells is not a rectangle.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

#ifdef DEBUG
BearStatus
Ncl_WriteBBLShapes(cmFileNamePtr, ciList, shapeFileNamePtr)
char        *cmFileNamePtr;         /* Cell master file name. */
lsList      ciList;                 /* List of cell instances. */
char        *shapeFileNamePtr;
{
#define SHAPE_FILE_VERSION "1.0"

    CellInst    *cellInstPtr;
    lsGen       gen;
    FILE        *filePtr;

#ifdef USE_OCT_CODE
    (void) VOVoutput(VOV_UNIX_FILE, shapeFileNamePtr);
#endif /* USE_OCT_CODE */

    if ((filePtr = fopen (shapeFileNamePtr, "w")) == NIL(FILE)) {
        (void) sprintf (bear_ErrorMsg,
            "Ncl_WriteShapes() : could not open \"%s\" for writing\n",   
            shapeFileNamePtr);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR);
    }

    (void) fprintf (filePtr, "Version %s\n\n", SHAPE_FILE_VERSION);

    (void) fprintf (filePtr, "#\n");
    (void) fprintf (filePtr, "# Shapes for %s\n", cmFileNamePtr);
    (void) fprintf (filePtr, "#\n\n");

    lsForeachItem(ciList, gen, cellInstPtr) {
        if (WriteCellShapes(cellInstPtr, filePtr) == BEAR_ERROR) {
            lsFinish(gen);
            fclose (filePtr);
            return (BEAR_ERROR_2);
        }
    }

    (void) fclose (filePtr);

    return (BEAR_OK);
}
#endif DEBUG



static BearStatus
AddNetToList(netPtr, netList)
Net     *netPtr;
lsList  netList;
/*
 * Add the net to the list.
 */
{
    int numOfPins;

    (void) Mac_NumOfPinsInPortListOfNet(netPtr, &numOfPins);
    if (numOfPins <= 4) {
        lsNewEnd(netList, (lsGeneric) netPtr, LS_NH);
    }

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Ncl_WriteTiming() --
 *
 * Generates a timing file that can then be used as test data.
 *
 * Results:
 *   BEAR_OK is returned if everything is spiffy.
 *   BEAR_ERROR is returned if the output file could not be opened
 *       for writing or the newG module could not be started.
 *   BEAR_ERROR_2 is returned if there are no nets with 4 pins or less.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Ncl_WriteTiming(cellMasterPtr, timingFileNamePtr)
CellMaster  *cellMasterPtr;
char        *timingFileNamePtr;
{
#define TIMING_FILE_VERSION "1.0"

    BearStatus          bearStatus = BEAR_OK;
    FILE                *filePtr = NIL(FILE);
    lsList              netList = (lsList) NULL; /* List of nets. */

    Gt_Graph            baseGraph, compGraph;
    lsGen               gen;
    Net                 *netPtr;
    Ngr_BldTreeDataPtr  bldTreePtr;      
    PlaneData           *planeDataPtr;
    int                 critNetLen, dx, dy;
    int                 a, i, num;
    char                *cmFileNamePtr;          /* Cell master file name. */

    netList = lsCreate();
    (void) Mac_ItrNetOfCell(cellMasterPtr, AddNetToList, (ClientData) netList);

    if (lsLength(netList) == 0) {
        Bear_PrintErr("Ncl_WriteTiming() : no nets with 4 pins or less");

        bearStatus = BEAR_ERROR_2;
        goto fin;
    }

#ifdef USE_OCT_CODE
    (void) VOVoutput(VOV_UNIX_FILE, timingFileNamePtr);
#endif /* USE_OCT_CODE */

    if ((filePtr = fopen (timingFileNamePtr, "w")) == NIL(FILE)) {
        (void) sprintf (bear_ErrorMsg,
            "Ncl_WriteTiming() : could not open \"%s\" for writing\n",   
            timingFileNamePtr);
        Bear_PrintErr(bear_ErrorMsg);

        bearStatus = BEAR_ERROR;
        goto fin;
    }

    if (Ngr_BldTreeBegin(cellMasterPtr, NIL(Ngr_BldOptions),
        &bldTreePtr) != BEAR_OK) {

        bearStatus = BEAR_ERROR;
        goto fin;
    }

    (void) fprintf (filePtr, "Version %s\n\n", TIMING_FILE_VERSION);

    cmFileNamePtr = Mac_GetFileNameOfCellMaster(cellMasterPtr);
    (void) fprintf (filePtr, "#\n");
    (void) fprintf (filePtr, "# Critical nets for %s\n", cmFileNamePtr);
    (void) fprintf (filePtr, "#\n\n");

    (void) fprintf (filePtr, "CriticalNetList\n");

    a = lsLength(netList);
    num = a * 0.1;
    num = MAX(3, num);

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    for (gen = lsStart(netList), i = 0;
        (lsNext(gen, (lsGeneric *) &netPtr, LS_NH) == LS_OK) && (i < num);
        i++) {

        if (Ngr_BuildBaseGraph(bldTreePtr, netPtr, &baseGraph) != BEAR_OK) {
            bearStatus = BEAR_ERROR;
            break;
        }
        if (Ngr_BuildCompGraph(bldTreePtr, &compGraph) != BEAR_OK) {
            bearStatus = BEAR_ERROR;
            break;
        }
        if (Ngr_FindMST(bldTreePtr) != BEAR_OK) {
            bearStatus = BEAR_ERROR;
            break;
        }
        if (Ngr_FindSteinerTree(bldTreePtr) != BEAR_OK) {
            bearStatus = BEAR_ERROR;
            break;
        }
        Ngr_DeleteCompGraph(&compGraph);

        dx = 0;
        dy = 0;
        if (Ngr_CalcEdgeLenOfGraph(baseGraph, planeDataPtr,
            &dx, &dy) != BEAR_OK) {
            bearStatus = BEAR_ERROR;
            break;
        }
        Ngr_DeleteBaseGraph(&baseGraph);

        critNetLen = (dx + dy) * 0.8;
        (void) fprintf (filePtr, "%s MAX %d\n", Mac_GetNameOfNet(netPtr),
            critNetLen);
    }
    lsFinish(gen);
    (void) Ngr_BldTreeEnd(bldTreePtr);

    (void) fprintf (filePtr, ";\n\n");

fin:
    if (netList != (lsList) NULL) {
        lsDestroy(netList, (void (*)()) NULL);
    }
    if (filePtr != NIL(FILE)) {
        (void) fclose (filePtr);
    }

    if (bearStatus == BEAR_OK) {
        char *stringPtr;

        MALLOC(char *, stringPtr,
            (strlen (timingFileNamePtr) + 1) * sizeof (char));
        strcpy (stringPtr, timingFileNamePtr);
        Mac_PutFileNameOfCellMaster(stringPtr, cellMasterPtr);
    }

    return (bearStatus);
}



