
/* ngrMain.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "newG.h"
#include "newGInt.h"


typedef struct _SegmentData SegmentData;
struct _SegmentData {
    lsList pjunctList;
};


/* ---------- External global variables for the newG module ---------- */
bool ngr_DebugFlag = FALSE;


/* ---------- Miscellaneous local procedures. ---------- */
static BearStatus       HandleNet();
static BearStatus       StorePinData(), FreePinData();
static BearStatus       StoreSegData(), FreeSegData();

/* ---------- Local procedures used to build base graph. ---------- */
static BearStatus       AddJunctToBaseGraph();
static BearStatus       AddSegToBaseGraph();
static BearStatus       AddPinToBaseGraph();
static Ngr_BaseEdgeData *CreateBaseEdge();
static int              CmpHorzPJuncts(), CmpVertPJuncts();

/* ---------- Local procedures used to build steiner tree. ---------- */
static BearStatus AddJunctToList();

/* ----- Local procedures used to do the actual global routing. ----- */
static BearStatus GlobalRouteNet();


static int
CmpHorzPJuncts(item1, item2)
lsGeneric item1, item2;
/*
 * Note that we want pseudo junctions on horizontal wall segments
 * to be sorted in such a way that the smallest one is at the
 * begining of the list.
 */
{
    PseudoJunct    *pjunctPtr;
    register int   a, b;

    pjunctPtr = (PseudoJunct *) item1;
    a = pjunctPtr->absPos.p_x;

    pjunctPtr = (PseudoJunct *) item2;
    b = pjunctPtr->absPos.p_x;

    return ((a == b) ? 0 : ((a < b) ? -1 : 1));
}


static int
CmpVertPJuncts(item1, item2)
lsGeneric item1, item2;
/*
 * Note that we want pseudo junctions on vertical wall segments
 * to be sorted in such a way that the biggest one is at the
 * begining of the list.
 */
{
    PseudoJunct    *pjunctPtr;
    register int   a, b;

    pjunctPtr = (PseudoJunct *) item1;
    a = pjunctPtr->absPos.p_y;

    pjunctPtr = (PseudoJunct *) item2;
    b = pjunctPtr->absPos.p_y;

    return ((a == b) ? 0 : ((a > b) ? -1 : 1));
}


static Ngr_BaseEdgeData *
CreateBaseEdge(node1, node2, planeDataPtr)
Gt_GraphNode    node1, node2;
PlaneData       *planeDataPtr;
{
    PinOrJunct          *pinOrJunctPtr;
    Ngr_BaseEdgeData    *baseEdgePtr;
    Point               absPos1, absPos2;
    int                 distance;

    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node1);
    Fpg_GetAbsPosOfPinOrJunct(planeDataPtr, pinOrJunctPtr, &absPos1);

    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node2);
    Fpg_GetAbsPosOfPinOrJunct(planeDataPtr, pinOrJunctPtr, &absPos2);

    if (absPos1.p_x == absPos2.p_x) {
        distance = abs(absPos2.p_y - absPos1.p_y);
    }
    else if (absPos1.p_y == absPos2.p_y) {
        distance = abs(absPos2.p_x - absPos1.p_x);
    }
    else {
        int dx, dy;

        dx = absPos2.p_x - absPos1.p_x;
        dy = absPos2.p_y - absPos1.p_y;
        distance = sqrt((double) (dx*dx + dy*dy));
    }

    baseEdgePtr = ALLOC(Ngr_BaseEdgeData, 1);
    baseEdgePtr->distance = distance;

    return (baseEdgePtr);
}


static BearStatus
HandleNet(netPtr, pinProc)
Net         *netPtr;
BearStatus  (*pinProc)();
{
    (void) Mac_ItrAllPinOfNet(netPtr, pinProc, (ClientData) NULL);

    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
AddJunctToBaseGraph(junctPtr, intBldTreePtr)
WallJunction    *junctPtr;
BldTreeDataInt  *intBldTreePtr;
{
    Gt_GraphNode    node;
    PinOrJunct      *pinOrJunctPtr;

    pinOrJunctPtr = Fpg_MakePinOrJunct();
    pinOrJunctPtr->unType = FPG_WALL_JUNCT;
    pinOrJunctPtr->unValue.junctPtr = junctPtr;
    node = Gt_AddGraphNode(intBldTreePtr->baseGraph,
        (BearId) intBldTreePtr->nodeNum,
        (BearGeneric) pinOrJunctPtr);
    (intBldTreePtr->nodeNum)++;

    junctPtr->ngrModulePtr = (BearGeneric) node;

    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
AddPinToBaseGraph(pinPtr, intBldTreePtr)
PinInst         *pinPtr;
BldTreeDataInt  *intBldTreePtr;
{
    lsGen               pjunctGen, segGen;
    lsList              pjunctList, wallList;
    CellInst            *cellInstPtr;
    Fpg_Room            *roomPtr;
    Gt_GraphNode        node1, node2;
    Ngr_BaseEdgeData    *baseEdgePtr;
    PinDirection        pinDir;
    PinOrJunct          *pinOrJunctPtr;
    PinKind             pinKind;
    Point               pinAbsPos, pjunctAbsPos;
    Point               *nwAbsPosPtr, *seAbsPosPtr;
    PortInst            *portInstPtr;
    PseudoJunct         *pjunctPtr;
    WallJunction        *nwJunctPtr, *seJunctPtr;
    WallSegment         *segPtr;
    int                 segDir;
    bool                foundSegFlag, haveNode2Flag;

    pinOrJunctPtr = Fpg_MakePinOrJunct();
    pinOrJunctPtr->unType = FPG_PIN;
    pinOrJunctPtr->unValue.pinPtr = pinPtr;
    node1 = Gt_AddGraphNode(intBldTreePtr->baseGraph,
        (BearId) intBldTreePtr->nodeNum,
        (BearGeneric) pinOrJunctPtr);
    BASE_NODE_OF_PIN(pinPtr) = (BearGeneric) node1;
    (intBldTreePtr->pinCount)++;
    (intBldTreePtr->nodeNum)++;

    if (Mac_IsPinPosDef(pinPtr) == FALSE) {
        Bear_PrintErr("AddPinToBaseGraph() : pin's position is undefined.");
        return (BEAR_STOP_SEARCH);
    }
    Mac_GetAbsPosOfPin(pinPtr, &pinAbsPos);
    portInstPtr = Mac_GetPortOfPin(pinPtr);
    cellInstPtr = Mac_GetCellInstOfPort(portInstPtr);
    roomPtr = cellInstPtr->roomPtr;

    pinDir = Mac_GetDirectionOfPin(pinPtr);
    segDir = Fpg_MacPinDirToFpgDir(pinDir);
    haveNode2Flag = FALSE;
    if (Mac_GetKindOfPin(pinPtr) == MAC_BORDER_PIN) {
        Rect *rectPtr;

        rectPtr = &(intBldTreePtr->fpgPtr->mainRoomPtr->rect);
        if ((pinAbsPos.p_x < rectPtr->r_xbot) &&
            (pinAbsPos.p_y > rectPtr->r_ytop)) {
            node2 = 
                (Gt_GraphNode) intBldTreePtr->fpgPtr->ulJunctPtr->ngrModulePtr;
            haveNode2Flag = TRUE;
        } 
        else if ((pinAbsPos.p_x > rectPtr->r_xtop) &&
            (pinAbsPos.p_y > rectPtr->r_ytop)) {
            node2 =
                (Gt_GraphNode) intBldTreePtr->fpgPtr->urJunctPtr->ngrModulePtr;
            haveNode2Flag = TRUE;
        } 
        else if ((pinAbsPos.p_x > rectPtr->r_xtop) &&
            (pinAbsPos.p_y < rectPtr->r_ybot)) {
            node2 =
                (Gt_GraphNode) intBldTreePtr->fpgPtr->lrJunctPtr->ngrModulePtr;
            haveNode2Flag = TRUE;
        } 
        else if ((pinAbsPos.p_x < rectPtr->r_xbot) &&
            (pinAbsPos.p_y < rectPtr->r_ybot)) {
            node2 =
                (Gt_GraphNode) intBldTreePtr->fpgPtr->llJunctPtr->ngrModulePtr;
            haveNode2Flag = TRUE;
        } 

        segDir = Fpg_OppositeDir(segDir);
    }

    if (haveNode2Flag == FALSE) {
        switch (segDir) {
        case FPG_NORTH:
            pjunctAbsPos.p_x = pinAbsPos.p_x;
            pjunctAbsPos.p_y = roomPtr->rect.r_ytop;
            break;
        case FPG_EAST:
            pjunctAbsPos.p_x = roomPtr->rect.r_xtop;
            pjunctAbsPos.p_y = pinAbsPos.p_y;
            break;
        case FPG_SOUTH:
            pjunctAbsPos.p_x = pinAbsPos.p_x;
            pjunctAbsPos.p_y = roomPtr->rect.r_ybot;
            break;
        case FPG_WEST:
            pjunctAbsPos.p_x = roomPtr->rect.r_xbot;
            pjunctAbsPos.p_y = pinAbsPos.p_y;
            break;
        default:
        NgrPrintFatalErr("AddPinToBaseGraph() failed : illegal pin direction");
            break;
        }
      
        foundSegFlag = FALSE;
        wallList = roomPtr->wallList[segDir];
        lsForeachItem(wallList, segGen, segPtr) {
            nwJunctPtr = Fpg_GetNWJunctOfWallSeg(segPtr);
            nwAbsPosPtr = Fpg_GetPointOfWallJunct((int) FPG_CENTER, nwJunctPtr);

            seJunctPtr = Fpg_GetSEJunctOfWallSeg(segPtr);
            seAbsPosPtr = Fpg_GetPointOfWallJunct((int) FPG_CENTER, seJunctPtr);

            if (segPtr->direction == FPG_HORIZONTAL) {
                foundSegFlag = ((pjunctAbsPos.p_x >= nwAbsPosPtr->p_x) &&
                    (pjunctAbsPos.p_x <= seAbsPosPtr->p_x)) ? TRUE : FALSE;
            }
            else {
                foundSegFlag = ((pjunctAbsPos.p_y >= seAbsPosPtr->p_y) &&
                    (pjunctAbsPos.p_y <= nwAbsPosPtr->p_y)) ? TRUE : FALSE;
            }

            if (foundSegFlag == TRUE) {
                node2 = (Gt_GraphNode) NULL;

                if ((pjunctAbsPos.p_x == nwAbsPosPtr->p_x) &&
                    (pjunctAbsPos.p_y == nwAbsPosPtr->p_y)) {
                    node2 = (Gt_GraphNode) nwJunctPtr->ngrModulePtr;
                }
                else if ((pjunctAbsPos.p_x == seAbsPosPtr->p_x) &&
                    (pjunctAbsPos.p_y == seAbsPosPtr->p_y)) {
                    node2 = (Gt_GraphNode) seJunctPtr->ngrModulePtr;
                }
                else {
                    pjunctList =
                        ((SegmentData *) segPtr->ngrModulePtr)->pjunctList;
                    lsForeachItem(pjunctList, pjunctGen, pjunctPtr) {
                        if ((pjunctAbsPos.p_x == pjunctPtr->absPos.p_x) &&
                            (pjunctAbsPos.p_y == pjunctPtr->absPos.p_y)) {
                            node2 = (Gt_GraphNode) pjunctPtr->clientData;
                            lsFinish(pjunctGen);
                            break;
                        }
                    }
                }
                lsFinish(segGen);
                break;
            }
        }
        if (foundSegFlag == FALSE) {
    Bear_PrintErr("AddPinToBaseGraph() : cannot place pin within a segment.");
            return (BEAR_STOP_SEARCH);
        }

        if (node2 == (Gt_GraphNode) NULL) {
            pjunctPtr = Fpg_MakePseudoJunct();
            pjunctPtr->absPos = pjunctAbsPos;

            pinOrJunctPtr = Fpg_MakePinOrJunct();
            pinOrJunctPtr->unType = FPG_PSEUDO_JCT;
            pinOrJunctPtr->unValue.pjunctPtr = pjunctPtr;
            node2 = Gt_AddGraphNode(intBldTreePtr->baseGraph,
                (BearId) intBldTreePtr->nodeNum,
                (BearGeneric) pinOrJunctPtr);
            (intBldTreePtr->nodeNum)++;

            pjunctPtr->clientData = (ClientData) node2;

            lsNewEnd(pjunctList, (lsGeneric) pjunctPtr, LS_NH);
        }
    }

    baseEdgePtr = CreateBaseEdge(node1, node2, intBldTreePtr->planeDataPtr);
    if (Gt_AddGraphEdge(node1, node2, (BearId) intBldTreePtr->edgeNum,
        (BearGeneric) baseEdgePtr, (Gt_GraphEdge *) NULL) != BEAR_OK) {
        return (BEAR_STOP_SEARCH);
    }
    (intBldTreePtr->edgeNum)++;

    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
AddSegToBaseGraph(segPtr, intBldTreePtr)
WallSegment     *segPtr;
BldTreeDataInt  *intBldTreePtr;
{
    lsGen               gen;
    lsList              pjunctList;
    Gt_GraphNode        node1, node2;
    Ngr_BaseEdgeData    *baseEdgePtr;
    PseudoJunct         *pjunctPtr;
    SegmentData         *segDataPtr;

    segDataPtr = (SegmentData *) segPtr->ngrModulePtr;
    pjunctList = segDataPtr->pjunctList;
    if (lsLength(pjunctList) == 0) {
        node1 = (Gt_GraphNode) (Fpg_GetNWJunctOfWallSeg(segPtr)->ngrModulePtr);
        node2 = (Gt_GraphNode) (Fpg_GetSEJunctOfWallSeg(segPtr)->ngrModulePtr);
        baseEdgePtr = CreateBaseEdge(node1, node2, intBldTreePtr->planeDataPtr);
        if (Gt_AddGraphEdge(node1, node2, (BearId) intBldTreePtr->edgeNum,
            (BearGeneric) baseEdgePtr, (Gt_GraphEdge *) NULL) != BEAR_OK) {

            return (BEAR_STOP_SEARCH);
        }
        (intBldTreePtr->edgeNum)++;
    }
    else {
        lsSort(pjunctList, (segPtr->direction == FPG_HORIZONTAL) ?
            CmpHorzPJuncts : CmpVertPJuncts);

        node1 = (Gt_GraphNode) (Fpg_GetNWJunctOfWallSeg(segPtr)->ngrModulePtr);
        lsForeachItem(pjunctList, gen, pjunctPtr) {
            node2 = (Gt_GraphNode) pjunctPtr->clientData;
            baseEdgePtr =
                CreateBaseEdge(node1, node2, intBldTreePtr->planeDataPtr);
            if (Gt_AddGraphEdge(node1, node2, (BearId) intBldTreePtr->edgeNum,
                (BearGeneric) baseEdgePtr, (Gt_GraphEdge *) NULL) != BEAR_OK) {

                return (BEAR_STOP_SEARCH);
            }
            (intBldTreePtr->edgeNum)++;

            node1 = node2;
        }

        node2 = (Gt_GraphNode) (Fpg_GetSEJunctOfWallSeg(segPtr)->ngrModulePtr);
        baseEdgePtr = CreateBaseEdge(node1, node2, intBldTreePtr->planeDataPtr);
        if (Gt_AddGraphEdge(node1, node2, (BearId) intBldTreePtr->edgeNum,
            (BearGeneric) baseEdgePtr, (Gt_GraphEdge *) NULL) != BEAR_OK) {

            return (BEAR_STOP_SEARCH);
        }
        (intBldTreePtr->edgeNum)++;

        lsDestroy(pjunctList, (void (*)()) NULL);
        segDataPtr->pjunctList = lsCreate();
    }

    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
static BearStatus
FreePinData(pinPtr, clientData)
PinInst     *pinPtr;
ClientData  clientData;
{
    FREE(pinPtr->ngrModulePtr);
    pinPtr->ngrModulePtr = (BearGeneric) NULL;

    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
static BearStatus
FreeSegData(segPtr, clientData)
WallSegment    *segPtr;
ClientData     clientData;
{
    SegmentData *segDataPtr = (SegmentData *) segPtr->ngrModulePtr;

    lsDestroy(segDataPtr->pjunctList, (void (*)()) NULL);
    FREE(segDataPtr);
    segPtr->ngrModulePtr = (BearGeneric) NULL;

    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
static BearStatus
StorePinData(pinPtr, clientData)
PinInst     *pinPtr;
ClientData  clientData;
{
    PinData *pinDataPtr;

    pinDataPtr = ALLOC(PinData, 1);
    pinDataPtr->baseNode = (Gt_GraphNode) NULL;
    pinDataPtr->compNode = (Gt_GraphNode) NULL;

    pinPtr->ngrModulePtr = (BearGeneric) pinDataPtr;

    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
static BearStatus
StoreSegData(segPtr, clientData)
WallSegment    *segPtr;
ClientData     clientData;
{
    SegmentData *segDataPtr;

    segDataPtr = ALLOC(SegmentData, 1);
    segDataPtr->pjunctList = lsCreate();

    segPtr->ngrModulePtr = (BearGeneric) segDataPtr;

    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
BearStatus
Ngr_BldTreeBegin(cellMasterPtr, bldOptsPtr, bldTreePtrPtr)
CellMaster          *cellMasterPtr;         /* IN       */
Ngr_BldOptions      *bldOptsPtr;            /* IN       */
Ngr_BldTreeDataPtr  *bldTreePtrPtr;         /* OUT      */
{
    BldTreeDataInt  *intBldTreePtr;
    FloorPlanGraph  *fpgPtr;
    PlaneData       *planeDataPtr;

    *bldTreePtrPtr = (Ngr_BldTreeDataPtr) NULL;

    if ((planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr)) == 
        NIL(PlaneData)) {
        Bear_PrintErr("Ngr_BldTreeBegin() : tile plane is NULL.");
        return (BEAR_ERROR);
    }

    if (Mac_GetPlacedFlagOfCell(cellMasterPtr) == FALSE) {
        Bear_PrintErr("Ngr_BldTreeBegin() : cells aren't placed.");
        return (BEAR_ERROR);
    }

    if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE) {
        if (Fpg_CreateFloorPlanGraph(planeDataPtr, (WallJunction **) NULL) !=
            BEAR_OK) {
            return (BEAR_ERROR);
        }
    }

    fpgPtr = Tp_GetFpg(planeDataPtr);
    if ((fpgPtr->mainRoomPtr == NIL(Fpg_Room)) ||
        (fpgPtr->roomList == (lsList) NULL)) {
        if (Fpg_GenRooms(cellMasterPtr) != BEAR_OK) {
            return (BEAR_ERROR);
        }
    }

    intBldTreePtr = ALLOC(BldTreeDataInt, 1);
    intBldTreePtr->cellMasterPtr = cellMasterPtr;
    intBldTreePtr->planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    intBldTreePtr->fpgPtr = Tp_GetFpg(intBldTreePtr->planeDataPtr);

    (void) Fpg_ItrWallSegSeq(intBldTreePtr->fpgPtr, StoreSegData,
        (ClientData) NULL);

    (void) Mac_ItrNetOfCell(intBldTreePtr->cellMasterPtr, HandleNet,
        (ClientData) StorePinData);

    *bldTreePtrPtr = (Ngr_BldTreeDataPtr) intBldTreePtr;

    return (BEAR_OK);
}


BearStatus
Ngr_BldTreeEnd(bldTreePtr)
Ngr_BldTreeDataPtr bldTreePtr;             /* IN       */
{
    BldTreeDataInt *intBldTreePtr = (BldTreeDataInt *) bldTreePtr;

    (void) Fpg_ItrWallSegSeq(intBldTreePtr->fpgPtr, FreeSegData,
        (ClientData) NULL);

    (void) Mac_ItrNetOfCell(intBldTreePtr->cellMasterPtr, HandleNet,
        (ClientData) FreePinData);

    FREE(intBldTreePtr);

    return (BEAR_OK);
}


BearStatus
Ngr_BuildBaseGraph(bldTreePtr, netPtr, baseGraphPtr)
Ngr_BldTreeDataPtr  bldTreePtr;             /* IN       */
Net             *netPtr;                    /* IN       */
Gt_Graph        *baseGraphPtr;              /* OUT      */
/*
 * The base graph is the floor plan graph augmented to also
 * include all of the pins of the given net.
 */
{
    BearStatus      bearStatus = BEAR_OK;
    BldTreeDataInt  *intBldTreePtr = (BldTreeDataInt *) bldTreePtr;
    Mac_GraphData   *graphDataPtr;

    intBldTreePtr->netPtr = netPtr;
    intBldTreePtr->haveTreeFlag = FALSE;
    intBldTreePtr->edgeNum = 0;

    graphDataPtr = Mac_MakeGraphData();
    graphDataPtr->fpgId = Fpg_GetIdOfFpg(intBldTreePtr->fpgPtr);
    graphDataPtr->pgFlag = FALSE;
    graphDataPtr->edgeDataFlag = FALSE;

    intBldTreePtr->baseGraph = Gt_CreateGraph(Mac_GenerateId(),
        (BearGeneric) graphDataPtr);
    intBldTreePtr->compGraph = (Gt_Graph) NULL;

    intBldTreePtr->nodeNum = 0;
    (void) Fpg_ItrWallJunctSeq(intBldTreePtr->fpgPtr, AddJunctToBaseGraph,
        (ClientData) intBldTreePtr);

    intBldTreePtr->pinCount = 0;
    if (Mac_ItrAllPinOfNet(intBldTreePtr->netPtr, AddPinToBaseGraph, 
        (ClientData) intBldTreePtr) == BEAR_STOP_SEARCH) {
        bearStatus = BEAR_ERROR;
        goto fin;
    }

    if (Fpg_ItrWallSegSeq(intBldTreePtr->fpgPtr, AddSegToBaseGraph,
        (ClientData) intBldTreePtr) == BEAR_STOP_SEARCH) {
        bearStatus = BEAR_ERROR;
        goto fin;
    }

fin:
    if (bearStatus != BEAR_OK) {
        intBldTreePtr->netPtr = NIL(Net);

        Mac_DeleteGraph(intBldTreePtr->baseGraph);
        intBldTreePtr->baseGraph = (Gt_Graph) NULL;

        *baseGraphPtr = (Gt_Graph) NULL;
    }
    else {
        *baseGraphPtr = intBldTreePtr->baseGraph;
    }

    return (bearStatus);
}


static BearStatus
AddJunctToList(node, junctList)
Gt_GraphNode    node;
lsList          junctList;
{
    PinOrJunct *pinOrJunctPtr;

    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node);
    if (pinOrJunctPtr->unType == FPG_WALL_JUNCT) {
        lsNewEnd(junctList, (lsGeneric) pinOrJunctPtr->unValue.junctPtr, LS_NH);
    }

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Ngr_FindSteinerTree() --
 *
 * This procedure is the last of the four steps required to build a
 * steiner tree for the net.  After completing the tree it builds the
 * wall junction list and stores it in the Mac_GraphData structure.
 *
 * Note that this procedure does not use the complete graph, and that
 * a list of bottle neck tiles is not built.
 * 
 * Results:
 *   BEAR_OK is returned if everything is spiffy, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Ngr_FindSteinerTree(bldTreePtr)
Ngr_BldTreeDataPtr bldTreePtr;              /* IN       */
{
    BldTreeDataInt  *intBldTreePtr = (BldTreeDataInt *) bldTreePtr;
    lsList          junctList;
    Mac_GraphData   *graphDataPtr;

    if (intBldTreePtr->haveTreeFlag == FALSE) {
        (void) Gt_MinSpanningTree(intBldTreePtr->baseGraph, 
            Ngr_GetBaseEdgeWeight, Mac_FreeNodeData, NgrFreeBaseEdgeData);
    }

    junctList = lsCreate();
    (void) Gt_ItrNodesOfGraph(intBldTreePtr->baseGraph, AddJunctToList,
        (ClientData) junctList);

    graphDataPtr =
        (Mac_GraphData *) Gt_GetGraphDataOfGraph(intBldTreePtr->baseGraph);
    graphDataPtr->junctList = junctList;

    return (BEAR_OK);
}


static BearStatus
GlobalRouteNet(netPtr, bldTreePtr)
Net                 *netPtr;
Ngr_BldTreeDataPtr  bldTreePtr;
{
    Gt_Graph baseGraph, compGraph;

    if (Ngr_BuildBaseGraph(bldTreePtr, netPtr, &baseGraph) != BEAR_OK) {
        return (BEAR_STOP_SEARCH);
    }
    if (Ngr_BuildCompGraph(bldTreePtr, &compGraph) != BEAR_OK) {
        return (BEAR_STOP_SEARCH);
    }
    if (Ngr_FindMST(bldTreePtr) != BEAR_OK) {
        return (BEAR_STOP_SEARCH);
    }
    if (Ngr_FindSteinerTree(bldTreePtr) != BEAR_OK) {
        return (BEAR_STOP_SEARCH);
    }

    Ngr_DeleteBaseGraph(&baseGraph);
    Ngr_DeleteCompGraph(&compGraph);

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * Ngr_NewGlobalRouter() --
 *
 * This procedure builds a steiner tree for each net.  Currently it
 * frees the trees as fast as it makes them and as such is here mostly
 * for testing purposes, but some day it could be used to put pseudo
 * pins in the tile plane for each net.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Ngr_NewGlobalRouter(cellMasterPtr)
CellMaster *cellMasterPtr;
{
    Ngr_BldTreeDataPtr bldTreePtr;

    if (Ngr_BldTreeBegin(cellMasterPtr, NIL(Ngr_BldOptions), &bldTreePtr) !=
        BEAR_OK) {
        return (BEAR_ERROR);
    }

    (void) Mac_ItrNetOfCell(cellMasterPtr, GlobalRouteNet,
        (ClientData) bldTreePtr);

    (void) Ngr_BldTreeEnd(bldTreePtr);

    return (BEAR_OK);
}



