
/* ngrUtils.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "newG.h"
#include "newGInt.h"


typedef struct _LengthData LengthData;
struct _LengthData {
    PlaneData   *planeDataPtr;
    int         dx, dy;
};


/* ---------- Local Procedures ---------- */
static BearStatus AddLenOfEdgeToTotal();


void
NgrPrintFatalErr(message)
BearString message;
{
    errRaise(NGR_MOD_NAME, ((int) -1), message);
}


void
NgrFreeBaseEdgeData(generic)
BearGeneric generic;
{
    if (generic != (BearGeneric) NULL) {
        FREE(generic); /* Free Ngr_BaseEdgeData */
    }
}


void
NgrFreeCompEdgeData(generic)
BearGeneric generic;
{
    if (generic != (BearGeneric) NULL) {
        Ngr_CompEdgeData *compEdgePtr = (Ngr_CompEdgeData *) generic;

        Gt_FreePath(&(compEdgePtr->pathPtr));

        FREE(compEdgePtr); /* Free Ngr_CompEdgeData */
    }
}


int
Ngr_GetBaseEdgeWeight(baseEdge)
Gt_GraphEdge baseEdge;
{
    return (((Ngr_BaseEdgeData *) 
        Gt_GetEdgeDataOfEdge(baseEdge))->distance);
}


int
Ngr_GetCompEdgeWeight(compEdge)
Gt_GraphEdge compEdge;
{
    return (((Ngr_CompEdgeData *)
        Gt_GetEdgeDataOfEdge(compEdge))->pathPtr->pathLength);
}


void
Ngr_DeleteBaseGraph(baseGraphPtr)
Gt_Graph *baseGraphPtr;
{
    Gt_DeleteGraph(*baseGraphPtr,
        Mac_FreeGraphData, Mac_FreeNodeData, NgrFreeBaseEdgeData);
    *baseGraphPtr = (Gt_Graph) NULL;
}


void
Ngr_DeleteCompGraph(compGraphPtr)
Gt_Graph *compGraphPtr;
{
    Gt_DeleteGraph(*compGraphPtr,
        Mac_FreeGraphData, Mac_FreeNodeData, NgrFreeCompEdgeData);
    *compGraphPtr = (Gt_Graph) NULL;
}


static BearStatus
AddLenOfEdgeToTotal(edge, lenDataPtr)
Gt_GraphEdge    edge;
LengthData      *lenDataPtr;
{
    Gt_GraphNode    node;
    PinOrJunct      *pinOrJunctPtr;
    Point           absPos1, absPos2;

    node = Gt_GetNodeOneOfEdge(edge);
    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node);
    if (Fpg_GetAbsPosOfPinOrJunct(lenDataPtr->planeDataPtr, 
        pinOrJunctPtr, &absPos1) != BEAR_OK) {
        return (BEAR_STOP_SEARCH);
    }

    node = Gt_GetNodeTwoOfEdge(edge);
    pinOrJunctPtr = (PinOrJunct *) Gt_GetNodeDataOfNode(node);
    if (Fpg_GetAbsPosOfPinOrJunct(lenDataPtr->planeDataPtr,   
        pinOrJunctPtr, &absPos2) != BEAR_OK) {
        return (BEAR_STOP_SEARCH);
    }

    lenDataPtr->dx += abs(absPos2.p_x - absPos1.p_x);
    lenDataPtr->dy += abs(absPos2.p_y - absPos1.p_y);

    return (BEAR_CONTINUE_SEARCH);
}


BearStatus
Ngr_CalcEdgeLenOfGraph(graph, planeDataPtr, dxPtr, dyPtr)
Gt_Graph    graph;                          /* IN       */
PlaneData   *planeDataPtr;                  /* IN       */
int         *dxPtr;                         /* OUT      */
int         *dyPtr;                         /* OUT      */
{
    LengthData lenData;

    lenData.planeDataPtr = planeDataPtr;
    lenData.dx = 0;
    lenData.dy = 0;
    if (Gt_ItrEdgesOfGraph(graph, AddLenOfEdgeToTotal,
        (ClientData) &lenData) == BEAR_STOP_SEARCH) {
        return (BEAR_ERROR);
    }

    *dxPtr = lenData.dx;
    *dyPtr = lenData.dy;

    return (BEAR_OK);
}


