
/* nplIOGoals.c --
 *
 *     Massoud Pedram
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1989 Regents of the University of Calif or nia
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif not lint

#include "bearCopyRight.h"
#include "newPInt.h"

typedef struct _LeafConnInfo LeafConnInfo;
struct _LeafConnInfo {
    Ncl_Node        *nodePtr;
    int             nodeIndex;
    Ncl_SideDir     pinDir;
    int             self;
    double          conn[MAX_CHILDREN+4];
};



BearStatus
CalcConnectivityOfNet(netDataPtr, ioGoal) 
Npl_NetData *netDataPtr;
double      ioGoal[][4];
{
    lsGen       bdrPinGen, pinGen;
    Npl_PinData *pinDataPtr, *bdrPinDataPtr;
    Ncl_Node    *childPtr;
    lsList      bdrPinList;
    double      mstWeight;
    int         i, j, numOfPins, ioGoalIndex;
    
    bdrPinList = lsCreate();
    
    numOfPins = lsLength(netDataPtr->pinList);
    if (numOfPins == 0) {
        Bear_PrintErr("CalcConnectivityOfNet() failed: net has no pins.");
        return (BEAR_ERROR);
    }
    mstWeight = 100.0 / numOfPins;
    
    lsForeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
        if (pinDataPtr->pinKind == NPL_BORDER_PIN) {
            lsNewEnd(bdrPinList, (lsGeneric) pinDataPtr, LS_NH);
        }
    }
    
    lsForeachItem(bdrPinList, bdrPinGen, bdrPinDataPtr) {
        ioGoalIndex = (bdrPinDataPtr->tileDir + 2) % 4;
        
        lsForeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
            if (pinDataPtr->pinKind == NPL_INSIDE_PIN) {
                childPtr = pinDataPtr->nodePtr;
                ioGoal[childPtr->nd_index][ioGoalIndex] += mstWeight;
            }
        }
    }
    lsDestroy(bdrPinList, (void (*)()) NULL);

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 * NplCalcIOConnectivities() --
 *
 *      This propcedure assumes exactly one border pin per any net within
 *      the node.  If feedthrus are introduced, we will have to rewrite this.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
NplCalcIOConnectivities(nodePtr)
Ncl_Node *nodePtr;
{
    lsGen       netGen;
    Npl_NetData *netDataPtr;
    double      ioGoal[MAX_CHILDREN][4];
    int         i, j, numOfChildren = nodePtr->nd_numOfChildren;

    if (nodePtr == ncl_ClusterTree) {
        return (BEAR_OK);
    }

    for (i = 0; i < numOfChildren; i++) {
        for (j = 0; j < 4; j++) {
            ioGoal[i][j] = 0;
        }
    }

    lsForeachItem(nodePtr->nd_totalNetList, netGen, netDataPtr) {
        if (CalcConnectivityOfNet(netDataPtr, ioGoal) == BEAR_ERROR) {
            lsFinish(netGen);
            return (BEAR_ERROR);
        }
    }

    for (i = 0; i < numOfChildren; i++) {
        for (j = 0; j < 4; j++) {
            nodePtr->nd_connectivity[i][j + MAX_CHILDREN] = (int) ioGoal[i][j];
        }
    }

    return (BEAR_OK);
}



BearStatus
NplCalcCritNetIOConnectivities(nodePtr)
Ncl_Node *nodePtr;
{
    Ncl_LinkedCritNetInNode  *critInNodePtr;
    Npl_NetData              *netDataPtr;
    double                  ioGoal[MAX_CHILDREN][4];
    int                     i, j, numOfChildren = nodePtr->nd_numOfChildren;

    if (nodePtr == ncl_ClusterTree) {
        return (BEAR_OK);
    }

    for (critInNodePtr = nodePtr->nd_lnkCritNetList;
            critInNodePtr != NIL(Ncl_LinkedCritNetInNode); 
            NPLNEXT(critInNodePtr)) {
    
        for (i=0; i < numOfChildren; i++) {
            for (j=0; j < 4; j++) {
                ioGoal[i][j] = 0;
            }
        }
    
        netDataPtr = (Npl_NetData *) 
                critInNodePtr->critNetPtr->netPtr->clientData;
        if (CalcConnectivityOfNet(netDataPtr, ioGoal) == BEAR_ERROR) {
            Bear_PrintErr("NplCalcCritNetIOConnectivities() failed.");
            return (BEAR_ERROR);
        }
    
        for (i=0; i < numOfChildren; i++) {
                for (j=0; j < 4; j++) {
                    critInNodePtr->connectivity[i][j+MAX_CHILDREN] = 
                            (int) ioGoal[i][j];
                }
            }
        }

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * GetConnectionsToPin --
 *
 *    This function goes through the list of connections of a pin and
 *    determines the IO goals that these connections establish.
 *
 * Results:
 *    BEAR_CONTINUE_SEARCH.
 *
 * Side effects:
 *    The elements of connInfoPtr->conn are set.
 *
 * ----------------------------------------------------------------------------
 */

static BearStatus
GetConnectionsToPin(nextPinDataPtr, connInfoPtr)
Npl_PinData     *nextPinDataPtr;
LeafConnInfo    *connInfoPtr;
{
    Ncl_Node    *connNodePtr;
    lsGen       pinGen;
    int         connIndex;

    if (nextPinDataPtr->pinKind == NPL_BORDER_PIN) {
        connIndex = (nextPinDataPtr->tileDir + 2) % 4 + MAX_CHILDREN;
        /* 
         * The pins on the IO boundary point inside. The direction of
         * a NCL_SOUTH_DIR pin, therefore, is actually NCL_NORTH_DIR.
         */
        connInfoPtr->conn[connIndex]++;

        return (BEAR_OK);
    }

    connNodePtr = nextPinDataPtr->nodePtr;
    if (connNodePtr == NIL(Ncl_Node)) {
        Bear_PrintErr("GetConnectionsToPin() failed: connNodePtr is NIL.");
        return (BEAR_ERROR);
    }

    if (connNodePtr->nd_parentPtr != connInfoPtr->nodePtr->nd_parentPtr) {
    Bear_PrintErr("GetConnectionsToPin() failed: netData has pins outside node.");
        return (BEAR_ERROR);
    }

    connInfoPtr->conn[connNodePtr->nd_index]++;

    if (connNodePtr->nd_index == connInfoPtr->nodeIndex &&
        nextPinDataPtr->initDir == connInfoPtr->pinDir) {
        /* 
         * connection between 2 pins on the same side of one cell. 
         */
        connInfoPtr->self++;
        nextPinDataPtr->clientData = (ClientData) TRUE;
    }

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * CalcIOConnections --
 *
 *    This is the function that does the equivalent thing as NplCalcIOGoals
 *    for the different sides of the blocks on the leaf level.
 *
 * Results:
 *    BEAR_OK.
 *
 * Side effects:
 *    Sets the nd_connectivity array of the leaf level node (referenced by
 *    pinDataPtr->nodePtr) to the number of connections from the 4 sides
 *    of the block to the IO goals.
 *
 * ----------------------------------------------------------------------------
 */

static BearStatus
CalcIOConnections(pinDataPtr, connInfoPtr)
Npl_PinData     *pinDataPtr;
LeafConnInfo    *connInfoPtr;
{
    Npl_NetData  *netDataPtr = pinDataPtr->netDataPtr;
    Npl_PinData  *nextPinDataPtr;
    int         i, dirIndex, numOfPins;
    lsGen       pinGen;
    double      mstWeight;

    if (netDataPtr == NIL(Npl_NetData)) {
        Bear_PrintErr("CalcIOConnections() failed: netData is NIL.");
        return (BEAR_ERROR);
    }

    if (((bool) pinDataPtr->clientData) == TRUE) {
        /* This pin has already been considered. */
        return (BEAR_OK);
    }

    numOfPins = lsLength(netDataPtr->pinList);

    for (i = 0; i < MAX_CHILDREN+4; i++) {
        connInfoPtr->conn[i] = 0.0;
    }
    connInfoPtr->self = 0;
    connInfoPtr->pinDir = pinDataPtr->initDir;

    lsForeachItem(netDataPtr->pinList, pinGen, nextPinDataPtr) {
        if (GetConnectionsToPin(nextPinDataPtr, connInfoPtr) == BEAR_ERROR) {
            lsFinish(pinGen);
            return (BEAR_ERROR);
        }
    }

    dirIndex = pinDataPtr->initDir;
    mstWeight = 100.0 / numOfPins;
    for (i = 0; i < MAX_CHILDREN+4; i++) {
        /*
         * We multiply by 1/n since each edge is counted twice. 
         */
        connInfoPtr->nodePtr->nd_connectivity[dirIndex][i] +=
                connInfoPtr->conn[i] * mstWeight;
    }

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 * NplCalcLeafConnectivities() --
 *
 * The procedure fill in the connectivity matrix of the fixed-shape child
 * nodes.  For flexible child nodes (which have floating pins), the matrix 
 * cannot be calculated in a meaningful way and is not used later.
 * We need relDir on inside pinData and tileDir on border pinData for 
 * this procedure to return the correct leaf connInfo.
 *
 * Results:
 *    BEAR_OK.
 *
 * Side effects:
 *    Sets the nd_connectivity array of the leaf nodes.
 * ----------------------------------------------------------------------------
 */

BearStatus
NplCalcLeafConnectivities(nodePtr)
Ncl_Node    *nodePtr;
{
    LeafConnInfo    connInfo;
    Npl_PinData     *pinDataPtr;
    lsGen           pinGen;
    Ncl_Node        *leafNodePtr;
    int             i, j, k;

    if (nodePtr->nd_type != P_LEAF) {
        Bear_PrintErr("NplCalcLeafConnectivities() failed: nodePtr is not a p_leaf.");
        return (BEAR_ERROR);
    }

    for (i = 0; i < nodePtr->nd_numOfChildren; i++) {
        leafNodePtr = nodePtr->nd_children[i];

        if (leafNodePtr->nd_cellInstPtr->masterPtr->shapeFlag == MAC_FLEX_SHAPE) {
            continue;
        }

        connInfo.nodePtr = leafNodePtr;
        connInfo.nodeIndex = i;

        for (j = 0; j < MAX_CHILDREN; j++) {
            for (k = 0; k < MAX_CHILDREN+4; k++) {
                leafNodePtr->nd_connectivity[j][k] = 0;
            }
        }

        NclForeachSide(j) {
            lsForeachItem(leafNodePtr->nd_initPinList[j], pinGen, pinDataPtr) {
                pinDataPtr->clientData = (ClientData) FALSE;
            }
        }

        NclForeachSide(j) {
            lsForeachItem(leafNodePtr->nd_initPinList[j], pinGen, pinDataPtr) {
                if (CalcIOConnections(pinDataPtr, &connInfo) == BEAR_ERROR) {
                    lsFinish(pinGen);
                    return (BEAR_ERROR);
                }
            }
        }

        NclForeachSide(j) {
            lsForeachItem(leafNodePtr->nd_initPinList[j], pinGen, pinDataPtr) {
                pinDataPtr->clientData = (ClientData) NULL;
            }
        }

    }

    return BEAR_OK;
}

