
/* nplMST.c --
 *
 *      Ben Chen / Massoud Pedram
 *     (Advising Professor E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <math.h>
/*
 * values.h is here so that a sun4 can find the definition of MAXFLOAT.
 */
#include <values.h>
#include "newPInt.h"
#include "mst.h"

#define WEIGHT(mst, v1, v2) mst[v1][v2]
#define MST_EDGE(mst, v1,v2) mst[v2][v1]

#define EUCL_DIST(p1, p2) \
    sqrt((double) (((p1.p_x - p2.p_x)*(p1.p_x - p2.p_x)) + \
         ((p1.p_y - p2.p_y)*(p1.p_y - p2.p_y))))


/*
 * NplFindMST() --
 *
 *      This procedure finds a minimum spanning tree for a *complete* graph
 *      whose vertices are represented by a list of points.  The list of
 *      points are arbitrarily labeled from 0 to n-1 (number of vertices - 1.)
 *      (i.e. the first vertex is labeled vertex 0, second vertex 1, etc.)
 *      Information in the resulting MST are kept in a two-dimensional array:
 *
 *      mst[i,j] = weight of edge from vertex i to vertex j for  i < j
 *               = 1.0 or 0.0 if edge is in the mst or not, respectively
 *                 for i > j
 *      Entries along the main diagonal are undefined.
 *
 * Results:
 *      The total weight of the MST is returned.
 *
 * Side effects:
 *      None.
 *
 */
float
NplFindMST(vertices)
     lsList vertices;  /* List of points in the graph     */
{
    lsGen pinGen;
    lsHandle pinHandle;
    Npl_PinData *pinDataPtr;
    Point p1, p2;
    static float mst[MAX_PINS_ON_NET][MAX_PINS_ON_NET]; /* weights and MST edges */
    static short fringe[MAX_PINS_ON_NET];  /* List of candidate vertices */
    short inTree[MAX_PINS_ON_NET];         /* List of vertices in the MST */
    float  edgeWeight, minWeight, totalWeight = 0;
    register int  i, j;
    int  vfringe, vtmp, v1mst, v2mst, v1, v2;
    int inTreeCount = 0;  /* Number of vertices in the MST   */
    int n;                /* Number of vertices in the graph */

    i = 0; n = lsLength(vertices);
    /* Initialize edge matrix */
    for (pinGen = lsStart(vertices);
    (lsNext(pinGen, (lsGeneric *) &pinDataPtr, &pinHandle) == LS_OK)
    || ((void) lsFinish(pinGen), 0); ) {
        p1 = pinDataPtr->tilePos;
        j = i + 1;
        while (lsNext(pinGen, (lsGeneric *) &pinDataPtr, LS_NH) == LS_OK) {
            p2 = pinDataPtr->tilePos;
            WEIGHT(mst, i, j) =  (float) EUCL_DIST(p1, p2);
            MST_EDGE(mst, i, j) = 0.0;
            j++;
        }
        fringe[i] = 0;  /* Initially, all vertices are in the fringe */
        i++;
    lsFinish(pinGen);
    pinGen = lsGenHandle(pinHandle, (lsGeneric *) 0, LS_AFTER);
    }
    
    /* Start with an arbitrary vertex, in this case vertex 0 */
    v1 = 0;
    fringe[0] = 1;
    inTree[inTreeCount++] = 0;

    /* MST calculation */
    while (inTreeCount != n) {
        minWeight = MAXFLOAT;
        for (i = 0; i < inTreeCount; i++) {
            for (j = 0; j < n; j++) {  /* Search the fringe list */
                v1 = inTree[i];
                if (fringe[j]) continue;
                v2 = j;
                if (v1 > v2) {  /* Make sure using upper triangular */
                    vtmp = v1;    
                    v1 = v2;
                    v2 = vtmp;
                }
                if ((edgeWeight = WEIGHT(mst, v1, v2)) < minWeight) {
                    minWeight = edgeWeight;
                    v1mst = v1;
                    v2mst = v2;
                    vfringe = j;
                }
            }
        }
        MST_EDGE(mst, v1mst, v2mst) = 1.0;
        inTree[inTreeCount++] = vfringe;
        fringe[vfringe] = 1;
/*
        (void) printf("minWeight = %.3f\n", minWeight);
*/
        totalWeight += minWeight;
    }
/*
    (void) printf("Edges: ");
    for (i = 0; i < n; i++) {
        for (j = i + 1; j < n; j++) {
            if (MST_EDGE(mst, i, j) == 1.0) {
                (void) printf("%d-%d ", i, j);
            }
        }
    }
    (void) printf("\n");
*/
    return totalWeight;
}

