
/* nplReadData.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "gtGraph.h"
#include "fpg.h"
#include "tileProp.h"
#include "mac.h"
#include "list.h"
#include "newP.h"
#include "newPInt.h"

#ifdef USE_OCT_CODE
#include "vov.h"
#endif /* USE_OCT_CODE */


BearStatus
Npl_ReadControlData(fileNamePtr, parmsPtr)
char            *fileNamePtr;
Npl_UserParms   *parmsPtr;
{
#define MAX_LINE_SIZE 1024

#define RETURN_ERR \
{ \
    fclose (filePtr); \
    return (BEAR_ERROR); \
}

#define GET_VALUE \
    if ((valuePtr = strtok(NIL(char), " ")) == NIL(char)) { \
        (void) sprintf (bear_ErrorMsg, \
            "NplReadControlData() failed : %s value is missing.", \
            tokenPtr); \
        Bear_PrintErr(bear_ErrorMsg); \
        (void) fprintf (bearout, "---\n\n"); \
        (void) fflush (bearout); \
        (void) fclose (filePtr); \
        return (BEAR_ERROR); \
    }

    FILE    *filePtr;
    bool    eofFlag;
    char    line[MAX_LINE_SIZE];
    char    *tokenPtr, *valuePtr;

#ifdef USE_OCT_CODE
    (void) VOVinput(VOV_UNIX_FILE, fileNamePtr);
#endif /* USE_OCT_CODE */

    if ((filePtr = fopen (fileNamePtr, "r")) == NIL(FILE)) {
        (void) sprintf (bear_ErrorMsg,
            "Could not open control file named \"%s\" for reading.", 
            fileNamePtr);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR_2);
    }

#ifdef DEBUG
    (void) fprintf (bearout, "---\n");
    (void) fprintf (bearout, "Data from file named \"%s\"\n\n", fileNamePtr);
#endif /* DEBUG */

    eofFlag = FALSE;
    while (TRUE) {
        if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) == 
            BEAR_ERROR) {
            RETURN_ERR;
        }
        if (eofFlag == TRUE) {
            fclose (filePtr);
            break;
        }

        tokenPtr = strtok(line, " ");
        if (strcmp(tokenPtr, "newP.BorderSize") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->p_borderSize);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %lf\n",
                tokenPtr, parmsPtr->p_borderSize);
#endif /* DEBUG */
        }

        else if (strcmp(tokenPtr, "newP.QuickPlanFlag") == 0) {
            GET_VALUE;
            parmsPtr->p_quickPlanFlag = 
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr,
                Bear_GetBooleanString(parmsPtr->p_quickPlanFlag));
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.MaxLoopCnt") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%d", &parmsPtr->p_maxLoopCnt);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %d\n", tokenPtr,
                parmsPtr->p_maxLoopCnt);
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.FinalOptFlag") == 0) {
            GET_VALUE;
            parmsPtr->p_finalOptFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr,
                Bear_GetBooleanString(parmsPtr->p_finalOptFlag));
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.FullEnumFlag") == 0) {
            GET_VALUE;
            parmsPtr->p_fullEnumFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr,
                Bear_GetBooleanString(parmsPtr->p_fullEnumFlag));
#endif /* DEBUG */
        }

        else if (strcmp(tokenPtr, "newP.FullEnum.Tolerance") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->p_fullEnum.tolerance);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %lf\n", 
                tokenPtr, parmsPtr->p_fullEnum.tolerance);
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.FullEnum.AreaWeight") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->p_fullEnum.areaWeight);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %lf\n", 
                tokenPtr, parmsPtr->p_fullEnum.areaWeight);
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.FullEnum.WireWeight") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->p_fullEnum.wireWeight);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %lf\n", 
                tokenPtr, parmsPtr->p_fullEnum.wireWeight);
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.FullEnum.CritWeight") == 0) {
            GET_VALUE;
            if (strcmp (valuePtr, "undef") != 0) {
                (void) sscanf (valuePtr, "%lf", 
                    &parmsPtr->p_fullEnum.critWeight);

#ifdef DEBUG
                (void) fprintf (bearout, "%-33s : %lf\n", 
                    tokenPtr, parmsPtr->p_fullEnum.critWeight);
            }
            else {
                (void) fprintf (bearout, "%-33s : %s\n", tokenPtr, valuePtr);
#endif /* DEBUG */
            }
        }

        else if (strcmp(tokenPtr, "newP.PartialEnum.Tolerance") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->p_partEnum.tolerance);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %lf\n", 
                tokenPtr, parmsPtr->p_partEnum.tolerance);
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.PartialEnum.WireWeight") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->p_partEnum.wireWeight);

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %lf\n", 
                tokenPtr, parmsPtr->p_partEnum.wireWeight);
#endif /* DEBUG */
        }
        else if (strcmp(tokenPtr, "newP.PartialEnum.CritWeight") == 0) {
            GET_VALUE;
            if (strcmp (valuePtr, "undef") != 0) {
                (void) sscanf (valuePtr, "%lf",
                    &parmsPtr->p_partEnum.critWeight);

#ifdef DEBUG
                (void) fprintf (bearout, "%-33s : %lf\n", 
                    tokenPtr, parmsPtr->p_partEnum.critWeight);
            }
            else {
                (void) fprintf (bearout, "%-33s : %s\n", tokenPtr, valuePtr);
#endif /* DEBUG */
            }
        }

        else if (strcmp(tokenPtr, "newP.TmpDir") == 0) {
            GET_VALUE;

            if (parmsPtr->p_tmpDir != NIL(char)) {
                FREE(parmsPtr->p_tmpDir);
            }
            parmsPtr->p_tmpDir = util_strsav(util_tilde_expand(valuePtr));

#ifdef DEBUG
            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr,
                parmsPtr->p_tmpDir);
#endif /* DEBUG */
        }

#ifdef DEBUG
        else if (strcmp(tokenPtr, "newP.Debug.SaveDataFlag") == 0) {
            GET_VALUE;
            parmsPtr->p_saveDataFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr,
                Bear_GetBooleanString(parmsPtr->p_saveDataFlag));
        }
        else if (strcmp(tokenPtr, "newP.Debug.AdjustSlots") == 0) {
            GET_VALUE;
            (void) sscanf (valuePtr, "%lf", &parmsPtr->p_adjustSlots);

            (void) fprintf (bearout, "%-33s : %lf\n",
                tokenPtr, parmsPtr->p_adjustSlots);
        }
        else if (strcmp(tokenPtr, "newP.Debug.ArrPinsFlag") == 0) {
            GET_VALUE;
            parmsPtr->p_arrPinsFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr,
                Bear_GetBooleanString(parmsPtr->p_arrPinsFlag));
        }
        else if (strcmp(tokenPtr, "newP.Debug.InteractiveFlag") == 0) {
            GET_VALUE;
            parmsPtr->p_interactiveFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr, 
                Bear_GetBooleanString(parmsPtr->p_interactiveFlag));
        }
        else if (strcmp(tokenPtr, "newP.Debug.CreateWinFlag") == 0) {
            GET_VALUE;
            parmsPtr->p_createWinFlag =
                (strcmp (valuePtr, "TRUE") == 0) ? TRUE : FALSE;

            (void) fprintf (bearout, "%-33s : %s\n", tokenPtr,
                Bear_GetBooleanString(parmsPtr->p_createWinFlag));
        }
#endif /* DEBUG */
    }

#ifdef DEBUG
    (void) fprintf (bearout, "---\n\n");
    (void) fflush (bearout);
#endif /* DEBUG */

    return (BEAR_OK);

#undef GET_VALUE
#undef MAX_LINE_SIZE
#undef RETURN_ERR
}


