
/* nplUtilities.c --
 *
 *     Massoud Pedram / George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif not lint

#include "bearCopyRight.h"
#include "newPInt.h"

#include "ezPlot.h"
#include "../polarBear/polarBear.h"


/* ---------- Local Procedures ---------- */
static BearStatus   IncludePin();
static BearStatus   TouchPin();
static void         SetInitPinListOfNode();
static void         SetTilePinListOfNode();

#ifdef DEBUG
static BearStatus   HandlePort();
static BearStatus   CheckMacPin();
#endif /* DEBUG */



void
NplSwapDimOfNode(nodePtr)
Ncl_Node *nodePtr;
{
    int tempStor;

    tempStor = nodePtr->nd_dim.p_x;
    nodePtr->nd_dim.p_x = nodePtr->nd_dim.p_y;
    nodePtr->nd_dim.p_y = tempStor;
}


void
NplSwapDimOfChildren(nodePtr)
Ncl_Node *nodePtr;
{
    int   i, tempStor;
    int   numOfChildren = nodePtr->nd_numOfChildren;

    for (i = 0; i < numOfChildren; i++) {
        tempStor = nodePtr->nd_children[i]->nd_dim.p_x;
        nodePtr->nd_children[i]->nd_dim.p_x = 
                nodePtr->nd_children[i]->nd_dim.p_y;
        nodePtr->nd_children[i]->nd_dim.p_y = tempStor;
    }
}


/*
 * ----------------------------------------------------------------------------
 *
 * NplNodeDepth() --
 *
 *    Computes the level of the hierarchy in which the current node is.
 *
 * Results:
 *    Hierarchical level of nodePtr (root = level 0).
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

int
NplNodeDepth(nodePtr)
Ncl_Node *nodePtr;
{
    if (nodePtr == ncl_ClusterTree) {
        return (0);
    }
    else {
        return (NplNodeDepth(nodePtr->nd_parentPtr) + 1);
    }
}



BearStatus
NplCalcTileDataOfNode(nodePtr)
Ncl_Node *nodePtr;
{
    Rect initRect;

    if (NPLISLEAF(nodePtr) == TRUE) {
        Bear_PrintErr("NplCalcTileDataOfNode() failed: nodePtr is a leaf.");
        return (BEAR_ERROR);
    }

    GeoTranslateTrans(&nodePtr->nd_nodeTrans,
        nplDelta.p_x, nplDelta.p_y, &nodePtr->nd_tileTrans);

    initRect.r_xbot = initRect.r_ybot = 0;
    initRect.r_xtop = nodePtr->nd_dim.p_x;
    initRect.r_ytop = nodePtr->nd_dim.p_y;
    GeoTransRect(&nodePtr->nd_tileTrans, &initRect, &nodePtr->nd_tileRect);

    NplCalcTilePinLists(nodePtr);

    return (BEAR_OK);
}


/*
 * NplCalcTileDataOfLeafNode()
 *
 * Requires correct setting of nd_dim and nd_nodeTrans of the leaf node.
 */

BearStatus
NplCalcTileDataOfLeafNode(leafNodePtr)
Ncl_Node *leafNodePtr;
{
    Rect        initRect;
    Transform   tmp1, tmp2, tmp3, tmp4;
    int         halfX, halfY;

    if (NPLISLEAF(leafNodePtr) == FALSE) {
   Bear_PrintErr("NplCalcTileDataOfLeafNode() failed: nodePtr is not a leaf.");
        return (BEAR_ERROR);
    }

    halfX = leafNodePtr->nd_dim.p_x / 2;
    halfY = leafNodePtr->nd_dim.p_y / 2;
    GeoTranslateTrans(&GeoIdentityTransform, -halfX, -halfY, &tmp1);
    GeoTransTrans(&tmp1, &nplTrans[leafNodePtr->nd_orient%4], &tmp2);
    GeoTranslateTrans(&tmp2, halfX, halfY, &tmp3);
    GeoTransTrans(&tmp3, &leafNodePtr->nd_nodeTrans, &tmp4);
    GeoTranslateTrans(&tmp4, nplDelta.p_x, nplDelta.p_y,
        &leafNodePtr->nd_tileTrans);

    initRect.r_xbot = initRect.r_ybot = 0;
    initRect.r_xtop = leafNodePtr->nd_dim.p_x;
    initRect.r_ytop = leafNodePtr->nd_dim.p_y;
    GeoTransRect(&leafNodePtr->nd_tileTrans, &initRect, 
        &leafNodePtr->nd_tileRect);

    NplCalcTilePinLists(leafNodePtr);

    return (BEAR_OK);
}



static BearStatus
EnclosePin(pinDataPtr, enclosingRectPtr)
Npl_PinData     *pinDataPtr;
register Rect   *enclosingRectPtr;
{
    Point       tilePos;
    Ncl_Node    *tmpNodePtr, *nodePtr;
    Point       centerPoint;
    Transform   tileTrans;

    if (pinDataPtr->pinKind == NPL_BORDER_PIN) {
        tilePos = pinDataPtr->tilePos;
        GeoIncludePoint(&tilePos, enclosingRectPtr);
        return (BEAR_OK);
    }

    nodePtr = pinDataPtr->nodePtr;
    if (nodePtr == NIL(Ncl_Node)) {
        Bear_PrintErr("EnclosePin() failed: nodePtr is NIL.");
        return (BEAR_ERROR);
    }

    if (nodePtr->nd_status == PLACED || nodePtr->nd_status == ACTIVE) {
        switch (pinDataPtr->pinFlexType) {
        case (Flexibility) FIXED:
            GeoTransPoint(&nodePtr->nd_tileTrans, &pinDataPtr->initPos, 
                    &tilePos);
            break;
        case (Flexibility) FLEXIBLE:
            tilePos.p_x = nodePtr->nd_tileRect.r_xbot + 
                    nodePtr->nd_tileRect.r_xtop;
            tilePos.p_y = nodePtr->nd_tileRect.r_ybot + 
                    nodePtr->nd_tileRect.r_ytop;
            break;
        case (Flexibility) FLEXIBLE_NORTH:
            tilePos.p_x = nodePtr->nd_tileRect.r_xbot + 
                    nodePtr->nd_tileRect.r_xtop;
            tilePos.p_y = nodePtr->nd_tileRect.r_ytop;
            break;
        case (Flexibility) FLEXIBLE_EAST:
            tilePos.p_x = nodePtr->nd_tileRect.r_xtop;
            tilePos.p_y = nodePtr->nd_tileRect.r_ybot + 
                    nodePtr->nd_tileRect.r_ytop;
            break;
        case (Flexibility) FLEXIBLE_SOUTH:
            tilePos.p_x = nodePtr->nd_tileRect.r_xbot + 
                    nodePtr->nd_tileRect.r_xtop;
            tilePos.p_y = nodePtr->nd_tileRect.r_ybot;
            break;
        case (Flexibility) FLEXIBLE_WEST:
            tilePos.p_x = nodePtr->nd_tileRect.r_xbot;
            tilePos.p_y = nodePtr->nd_tileRect.r_ybot + 
                    nodePtr->nd_tileRect.r_ytop;
            break;
        default:
            Bear_PrintErr("EnclosePin() failed : illegal flexibility type");
            break;
        }
/*
    (void) printf("->(%d, %d)", 
              pinDataPtr->tilePos.p_x, pinDataPtr->tilePos.p_y);
*/
        GeoIncludePoint(&tilePos, enclosingRectPtr);
        return (BEAR_OK);
    }

    for (tmpNodePtr = nodePtr; tmpNodePtr != ncl_ClusterTree;
            tmpNodePtr = tmpNodePtr->nd_parentPtr) {

        if (tmpNodePtr->nd_status == PLACED) {
            nodePtr = tmpNodePtr;
            break;
        }
    }

    GeoTranslateTrans(&nodePtr->nd_nodeTrans,
            nplDelta.p_x, nplDelta.p_y, &tileTrans);
    centerPoint.p_x = nodePtr->nd_dim.p_x / 2;
    centerPoint.p_y = nodePtr->nd_dim.p_y / 2;
    GeoTransPoint(&tileTrans, &centerPoint, &tilePos);

    return (BEAR_OK);
}



/* 
 * ---------------------------------------------------------------------------
 * NplCalcEncloseRect
 *
 *     This function calculates the minimum rect enclosing all cells (including
 *     the I/O pads) connected by the given netDataPtr.
 *
 * Results:
 *      rectPtr is assigned fields.
 *
 * Side Effects: 
 *      None.
 *
 * ---------------------------------------------------------------------------
 */

BearStatus
NplCalcEncloseRect(netDataPtr, enclosingRectPtr)
Npl_NetData *netDataPtr;
Rect        *enclosingRectPtr;
{
    Rect        canonicalRect;
    lsGen       pinGen;
    Npl_PinData *pinDataPtr;

    *enclosingRectPtr = nplInsideOutRect;
/*
    (void) printf("net %s:", netDataPtr->netPtr->name);
*/
    lsForeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
/*
    (void) printf(" (%d, %d)", 
              pinDataPtr->tilePos.p_x, pinDataPtr->tilePos.p_y);
*/
        if (EnclosePin(pinDataPtr, enclosingRectPtr) == BEAR_ERROR) {
            lsFinish(pinGen);
            return (BEAR_ERROR);
        }
    }
/*
    (void) printf("\n");
*/
    if (GEO_RECTNULL(enclosingRectPtr) == TRUE) {
        GeoCanonicalRect(enclosingRectPtr, &canonicalRect);
        *enclosingRectPtr = canonicalRect;
    }

    return (BEAR_OK);
}



/* 
 * ---------------------------------------------------------------------------
 * NplTileDirOfRect
 *
 *     This function determines the tileDir of an enclosing rect.  Note that
 *     the net goes through no junction region, i.e., it has all its pins
 *     either in the horz tile plane or vert tile plane.  Otherwise, the
 *     function returns BEAR_ERROR.
 *
 * Results:
 *      horzFlagPtr is set.
 *
 * Side Effects: 
 *      None.
 *
 * ---------------------------------------------------------------------------
 */
BearStatus
NplTileDirOfRect(netDataPtr, horzFlagPtr)
Npl_NetData *netDataPtr;
bool        *horzFlagPtr;
{
    lsGen       pinGen;
    Npl_PinData *pinDataPtr;

    lsFirstItem(netDataPtr->pinList, (lsGeneric *) &pinDataPtr, LS_NH);
    if (pinDataPtr->tileDir == NCL_EAST_DIR || 
            pinDataPtr->tileDir == NCL_WEST_DIR) {
        *horzFlagPtr = TRUE;
    } else {
        *horzFlagPtr = FALSE;
    }

    lsForeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
        switch (pinDataPtr->tileDir) {
        case NCL_NORTH_DIR:
            if (*horzFlagPtr == TRUE) {
        Bear_PrintErr("NplTileDirOfRect() failed: pin tile direction is bad.");
                lsFinish(pinGen);
                return (BEAR_ERROR);
            }
            break;
        case NCL_EAST_DIR:
            if (*horzFlagPtr == FALSE) {
        Bear_PrintErr("NplTileDirOfRect() failed: pin tile direction is bad.");
                lsFinish(pinGen);
                return (BEAR_ERROR);
            }
            break;
        case NCL_SOUTH_DIR:
            if (*horzFlagPtr == TRUE) {
        Bear_PrintErr("NplTileDirOfRect() failed: pin tile direction is bad.");
                lsFinish(pinGen);
                return (BEAR_ERROR);
            }
            break;
        case NCL_WEST_DIR:
            if (*horzFlagPtr == FALSE) {
        Bear_PrintErr("NplTileDirOfRect() failed: pin tile direction is bad.");
                lsFinish(pinGen);
                return (BEAR_ERROR);
            }
            break;
        default:
       Bear_PrintErr("NplTileDirOfRect() failed: unknown pin tile direction.");
            lsFinish(pinGen);
            break;
        }
    }

    return (BEAR_OK);
}



static BearStatus
TouchPin(pinDataPtr, touchingRectPtr)
Npl_PinData      *pinDataPtr;
register Rect   *touchingRectPtr;
{
    Ncl_Node *nodePtr = pinDataPtr->nodePtr;

/*
    if (nodePtr == NIL(Ncl_Node)) {
        Bear_PrintErr("TouchPin() failed: nodePtr is NIL.");
        return (BEAR_ERROR);
    }
*/

    if (pinDataPtr->pinKind == NPL_BORDER_PIN) {

        Point   tilePos;

    if (nodePtr != NIL(Ncl_Node)) {
        GeoTransPoint(&nodePtr->nd_tileTrans, &pinDataPtr->initPos,
              &tilePos);
        if (tilePos.p_x != pinDataPtr->tilePos.p_x ||
        tilePos.p_y != pinDataPtr->tilePos.p_y) {
        Bear_PrintErr("TouchPin() failed: tilePos not same.");
        return (BEAR_ERROR);
        }
    }

    tilePos = pinDataPtr->tilePos;
        touchingRectPtr->r_xbot = NPLMIN(touchingRectPtr->r_xbot, tilePos.p_x);
        touchingRectPtr->r_xtop = NPLMAX(touchingRectPtr->r_xtop, tilePos.p_x);
        touchingRectPtr->r_ybot = NPLMIN(touchingRectPtr->r_ybot, tilePos.p_y);
        touchingRectPtr->r_ytop = NPLMAX(touchingRectPtr->r_ytop, tilePos.p_y);

    } else {

        Rect    *cellRectPtr = &(nodePtr->nd_tileRect);

        touchingRectPtr->r_xbot = 
                NPLMIN(touchingRectPtr->r_xbot, cellRectPtr->r_xtop);
        touchingRectPtr->r_xtop = 
                NPLMAX(touchingRectPtr->r_xtop, cellRectPtr->r_xbot);
        touchingRectPtr->r_ybot = 
                NPLMIN(touchingRectPtr->r_ybot, cellRectPtr->r_ytop);
        touchingRectPtr->r_ytop = 
                NPLMAX(touchingRectPtr->r_ytop, cellRectPtr->r_ybot);

    }

    return (BEAR_OK);
}



/* 
 * ---------------------------------------------------------------------------
 * NplCalcTouchRect
 *
 *      This function calculates the minimum rect touching all cells (including
 *      the I/O pads) connected by the given netPtr.
 *
 * Results:
 *      rectPtr is assigned fields.
 *
 * Side Effects: 
 *      None.
 *
 * ---------------------------------------------------------------------------
 */

BearStatus
NplCalcTouchRect(netDataPtr, touchingRectPtr)
Npl_NetData  *netDataPtr;
Rect        *touchingRectPtr;
{
    Rect        canonicalRect;
    lsGen       pinGen;
    Npl_PinData  *pinDataPtr;

    *touchingRectPtr = nplInsideOutRect;

    lsForeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
        if (TouchPin(pinDataPtr, touchingRectPtr) == BEAR_ERROR) {
            lsFinish(pinGen);
            return (BEAR_ERROR);
        }
    }

    if (GEO_RECTNULL(touchingRectPtr) == TRUE) {
        /*
         * We turn touchingRectPtr into a canonical rect in which the lower left 
         * is really below and to the left of the upper right.
         */
        GeoCanonicalRect(touchingRectPtr, &canonicalRect);
        *touchingRectPtr = canonicalRect;
    }

    return (BEAR_OK);
}



static void
SetInitPinListOfNode(dir, tilePinList, initTransPtr, initPinList)
int         dir;                /* IN */
lsList      tilePinList;        /* IN */
Transform   *initTransPtr;      /* IN */
lsList      initPinList[];      /* OUT */
{
    lsGen       pinGen;
    lsList      pinList;
    Npl_PinData  *pinDataPtr;
    Ncl_SideDir  initDir;

    switch (dir) {
    case GEO_NORTH:
        initDir = NCL_NORTH_DIR;
        pinList = initPinList[NCL_NORTH_DIR] = tilePinList;
        break;
    case GEO_EAST:
        initDir = NCL_EAST_DIR;
        pinList = initPinList[NCL_EAST_DIR] = tilePinList;
        break;
    case GEO_SOUTH:
        initDir = NCL_SOUTH_DIR;
        pinList = initPinList[NCL_SOUTH_DIR] = tilePinList;
        break;
    case GEO_WEST:
        initDir = NCL_WEST_DIR;
        pinList = initPinList[NCL_WEST_DIR] = tilePinList;
        break;
    default:
        Bear_PrintFatalErr("PutPinListOfNode() failed : bad direction");
        break;
    }

    if (lsLength(pinList) != 0) {
        lsFirstItem(pinList, (lsGeneric *) &pinDataPtr, LS_NH);
        if (pinDataPtr->pinKind == NPL_BORDER_PIN) {
            initDir = Ncl_GetOppositeSideDir(initDir);
        }

        lsForeachItem(pinList, pinGen, pinDataPtr) {
            pinDataPtr->initDir = initDir;

            GeoTransPoint(initTransPtr,
                &pinDataPtr->tilePos, &pinDataPtr->initPos);
        }
    }
}



static void
SetTilePinListOfNode(dir, initPinList, tileTransPtr, tilePinList)
int         dir;                /* IN */
lsList      initPinList;        /* IN */
Transform   *tileTransPtr;      /* IN */
lsList      tilePinList[];      /* OUT */
{
    lsGen       pinGen;
    lsList      pinList;
    Npl_PinData  *pinDataPtr;
    Ncl_SideDir  tileDir;

    switch (dir) {
    case GEO_NORTH:
        tileDir = NCL_NORTH_DIR;
        pinList = tilePinList[NCL_NORTH_DIR] = initPinList;
        break;
    case GEO_EAST:
        tileDir = NCL_EAST_DIR;
        pinList = tilePinList[NCL_EAST_DIR] = initPinList;
        break;
    case GEO_SOUTH:
        tileDir = NCL_SOUTH_DIR;
        pinList = tilePinList[NCL_SOUTH_DIR] = initPinList;
        break;
    case GEO_WEST:
        tileDir = NCL_WEST_DIR;
        pinList = tilePinList[NCL_WEST_DIR] = initPinList;
        break;
    default:
        Bear_PrintFatalErr("PutPinListOfNode() failed : bad direction");
        break;
    }

    if (lsLength(pinList) != 0) {
        lsFirstItem(pinList, (lsGeneric *) &pinDataPtr, LS_NH);
        if (pinDataPtr->pinKind == NPL_BORDER_PIN) {
            tileDir = Ncl_GetOppositeSideDir(tileDir);
        }

        lsForeachItem(pinList, pinGen, pinDataPtr) {
            pinDataPtr->tileDir = tileDir;

            GeoTransPoint(tileTransPtr,
                &pinDataPtr->initPos, &pinDataPtr->tilePos);
        }
    }
}



void
NplCalcInitPinLists(nodePtr)
Ncl_Node *nodePtr;
/*
 * This procedure assumes that the tile data for the node has
 * already been filled in.
 */
{
    Transform invTrans;

    GeoInvertTrans(&nodePtr->nd_tileTrans, &invTrans);

    SetInitPinListOfNode(GeoTransPos(&invTrans, GEO_NORTH),
        nodePtr->nd_tilePinList[NCL_NORTH_DIR],
        &invTrans,
        nodePtr->nd_initPinList);
    SetInitPinListOfNode(GeoTransPos(&invTrans, GEO_EAST),
        nodePtr->nd_tilePinList[NCL_EAST_DIR],
        &invTrans,
        nodePtr->nd_initPinList);
    SetInitPinListOfNode(GeoTransPos(&invTrans, GEO_SOUTH),
        nodePtr->nd_tilePinList[NCL_SOUTH_DIR],
        &invTrans,
        nodePtr->nd_initPinList);
    SetInitPinListOfNode(GeoTransPos(&invTrans, GEO_WEST),
        nodePtr->nd_tilePinList[NCL_WEST_DIR],
        &invTrans,
        nodePtr->nd_initPinList);
}


void
NplCalcTilePinLists(nodePtr)
Ncl_Node *nodePtr;
/*
 * This procedure assumes that the initial data for the node has
 * already been filled in.
 */
{
    Transform *tileTransPtr;

    tileTransPtr = &nodePtr->nd_tileTrans;

    SetTilePinListOfNode(GeoTransPos(tileTransPtr, GEO_NORTH),
        nodePtr->nd_initPinList[NCL_NORTH_DIR],
        tileTransPtr,
        nodePtr->nd_tilePinList);
    SetTilePinListOfNode(GeoTransPos(tileTransPtr, GEO_EAST),
        nodePtr->nd_initPinList[NCL_EAST_DIR],
        tileTransPtr,
        nodePtr->nd_tilePinList);
    SetTilePinListOfNode(GeoTransPos(tileTransPtr, GEO_SOUTH),
        nodePtr->nd_initPinList[NCL_SOUTH_DIR],
        tileTransPtr,
        nodePtr->nd_tilePinList);
    SetTilePinListOfNode(GeoTransPos(tileTransPtr, GEO_WEST),
        nodePtr->nd_initPinList[NCL_WEST_DIR],
        tileTransPtr,
        nodePtr->nd_tilePinList);
}



int
NplDistanceFromRect(p, r)
register Point *p;
register Rect  *r;
{
    if (p->p_x >= r->r_xbot && p->p_x <= r->r_xtop && p->p_y >= r->r_ytop) {
        return (p->p_y - r->r_ytop);
    } 
    else if (p->p_x > r->r_xtop && p->p_y > r->r_ytop) {
        return (p->p_x - r->r_xtop + p->p_y - r->r_ytop);
    } 
    else if (p->p_x >= r->r_xtop && p->p_y >= r->r_ybot && 
        p->p_y <= r->r_ytop) {
        return (p->p_x - r->r_xtop);
    } 
    else if (p->p_x > r->r_xtop && p->p_y < r->r_ybot) {
        return (p->p_x - r->r_xtop + r->r_ybot - p->p_y);
    } 
    else if (p->p_x >= r->r_xbot && p->p_x <= r->r_xtop && 
        p->p_y <= r->r_ybot) {
        return (r->r_ybot - p->p_y);
    } 
    else if (p->p_x < r->r_xbot && p->p_y < r->r_ybot) {
        return (r->r_xbot - p->p_x + r->r_ybot - p->p_y);
    } 
    else if (p->p_x <= r->r_xbot && p->p_y >= r->r_ybot && 
        p->p_y <= r->r_ytop) {
        return (r->r_xbot - p->p_x);
    } 
    else if (p->p_x < r->r_xbot && p->p_y > r->r_ytop) {
        return (r->r_xbot - p->p_x + p->p_y - r->r_ybot);
    } 
    else {
        return (0);
    }
}



Npl_UserParms *
Npl_MakeUserParms()
{
    register Npl_UserParms *parmsPtr;

    MALLOC(Npl_UserParms *, parmsPtr, sizeof (Npl_UserParms));
    parmsPtr->p_mode = NPL_UNDEF_MODE;

    parmsPtr->p_quickPlanFlag = FALSE;
    parmsPtr->p_finalOptFlag = TRUE;
    parmsPtr->p_maxLoopCnt = 2;
    parmsPtr->p_borderSize = 1.5;
    parmsPtr->p_tmpDir = util_strsav(bear_tmpDir);

    parmsPtr->p_fullEnumFlag = TRUE;

    parmsPtr->p_fullEnum.tolerance = 0.15;
    parmsPtr->p_fullEnum.areaWeight = 0.6;
    parmsPtr->p_fullEnum.wireWeight = 0.4;
    parmsPtr->p_fullEnum.critWeight = 1.0;

    parmsPtr->p_partEnum.tolerance = 0.15;
    parmsPtr->p_partEnum.wireWeight = 1.0;
    parmsPtr->p_partEnum.critWeight = 1.0;

    /* These parameters are for debugging only. */
    parmsPtr->p_adjustSlots = 1.0;
    parmsPtr->p_arrPinsFlag = FALSE;
    parmsPtr->p_saveDataFlag = FALSE;
    parmsPtr->p_interactiveFlag = FALSE;
    parmsPtr->p_createWinFlag = FALSE;

    return (parmsPtr);
}



void
Npl_FreeUserParms(parmsPtrPtr)
Npl_UserParms **parmsPtrPtr;
{
    if (*parmsPtrPtr == NIL(Npl_UserParms)) {
        return;
    }

    if ((*parmsPtrPtr)->p_tmpDir != NIL(char)) {
        FREE((*parmsPtrPtr)->p_tmpDir);
    }

    FREE((char *) *parmsPtrPtr);
    *parmsPtrPtr = NIL(Npl_UserParms);
}



/*
 * ----------------------------------------------------------------------------
 *
 * NplMakePinData() --
 *
 * Allocates memory for the structure Npl_PinData, and initializes
 * the fields.
 *
 * Results:
 *   A pointer to the structure Npl_PinData.
 *
 * Side effects:
 *   None.
 *
 * ----------------------------------------------------------------------------
 */

Npl_PinData *
NplMakePinData()
{
    Npl_PinData *pinDataPtr;

    MALLOC(Npl_PinData *, pinDataPtr, sizeof (Npl_PinData));
    pinDataPtr->pinPtr = NIL(PinInst);
    pinDataPtr->pinKind = NPL_INSIDE_PIN;
    pinDataPtr->pinFlexType = FIXED;
    pinDataPtr->netDataPtr = NIL(Npl_NetData);
    pinDataPtr->nodePtr = NIL(Ncl_Node);
    pinDataPtr->initDir = NCL_NORTH_DIR;
    pinDataPtr->initPos.p_x = 0;
    pinDataPtr->initPos.p_y = 0;
    pinDataPtr->tileDir = NCL_NORTH_DIR;
    pinDataPtr->tilePos.p_x = 0;
    pinDataPtr->tilePos.p_y = 0;
    pinDataPtr->row = 0;
    pinDataPtr->clientData = (ClientData) NULL;

    return (pinDataPtr);
}


void
NplFreePinData(item)
lsGeneric item;
{
    FREE((char *) item); /* Free Npl_PinData */
}


#ifdef BETTY_BOOP
BearStatus
NplDoCheck(cellMasterPtr)
CellMaster *cellMasterPtr;
{
    bool pinSpacingFlag;

    if (Mac_CheckPins(cellMasterPtr, &pinSpacingFlag) == BEAR_ERROR) {
        Bear_PrintErr("NplDoCheck() failed : Bad pin data.");
        return (BEAR_ERROR);
    }
    if (pinSpacingFlag == FALSE) {
        Bear_PrintErr("NplDoCheck() failed : some pins are to close.");
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}
#endif BETTY_BOOP


#ifdef DEBUG
/* ARGSUSED */
static BearStatus
CheckMacPin(pinPtr, clientData)
PinInst     *pinPtr;
ClientData  *clientData;
{
    if (pinPtr == NIL(PinInst)) {
        (void) fprintf (bearerr, "NIL ");
    }
    else {
        (void) fprintf (bearerr, "%s ", Mac_GetNameOfPinInst(pinPtr));
    }

    return (BEAR_CONTINUE_SEARCH);
}
#endif /* DEBUG */


#ifdef DEBUG
static BearStatus
HandlePort(portPtr, clientData)
PortInst    *portPtr;
ClientData  *clientData;
{
    (void) Mac_ItrPinOfPortInst(portPtr, CheckMacPin, (ClientData) NULL);

    return (BEAR_CONTINUE_SEARCH);
}
#endif /* DEBUG */


#ifdef DEBUG
void 
NplCheckNetList(netList, cmpWithMacFlag)
lsList  netList;
bool    cmpWithMacFlag; /* compare with MAC flag */
{
    lsGen       netGen, pinGen;
    Npl_NetData *netDataPtr;
    Npl_PinData *pinDataPtr;

    lsForeachItem(netList, netGen, netDataPtr) {
        if (netDataPtr->pinList == NULL) {
            /* (void) fprintf (bearerr, " no pins.\n"); */
            continue;
        }

        (void) fprintf (bearerr, "Net name = %s\n", netDataPtr->netPtr->name);

        if (cmpWithMacFlag) {
            /*
             * Note that the MAC pin list is only equivalent to the internal
             * pin list immediately after you have written the internal data
             * out to MAC.
             */

            PortInst *portPtr, *headPtr;

            (void) fprintf (bearerr, "Internal pin list:\n");
            (void) fprintf (bearerr, "Length = %d\n", 
                lsLength(netDataPtr->pinList));

            (void) fprintf (bearerr, "    ");
            lsForeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
                if (pinDataPtr->pinPtr == NIL(PinInst)) {
                    (void) fprintf (bearerr, "NIL ");
                }
                else {
                    (void) fprintf (bearerr, "%s ", 
                        Mac_GetNameOfPinInst(pinDataPtr->pinPtr));
                }
            }
            (void) fprintf (bearerr, "\n");

            (void) fprintf (bearerr, "MAC pin list:\n");
            (void) fprintf (bearerr, "    ");
/*
 *           (void) Mac_ItrPortOfNet(netDataPtr->netPtr, HandlePort, 
 *              (ClientData) NULL);
 */

            portPtr = headPtr = netDataPtr->netPtr->portList;
            while (portPtr != NIL(PortInst)) {

                if (portPtr->mark == BEAR_UNMARKED) {
                    (void) HandlePort(portPtr, (ClientData) NULL);
                }

                portPtr = portPtr->nextNetPortPtr;

                if (portPtr == headPtr) {
                    (void) fprintf (bearerr, "LOOP !!\n");
                    abort();
                }
            }

            (void) fprintf (bearerr, "\n");
        } 
        else {
            lsForeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
                (void) fprintf (bearerr, " %s ", pinDataPtr->pinName);
                if (pinDataPtr->pinPtr != NIL(PinInst)) {
                    (void) fprintf ("(%s)", 
                        Mac_GetNameOfPinInst(pinDataPtr->pinPtr));
                }
            }
        }
        (void) fprintf (bearerr, "\n");
    }
}
#endif /* DEBUG */


#ifdef DEBUG
void
NplSaveCellMaster(cellMasterPtr)
CellMaster *cellMasterPtr;
/*
 *  MAJOR HACK
 */
{
#define FILE_PREFIX "bear"

    if (nplSaveDataFlag) {
        static int  count = 0;
        char        command[BEAR_MSG_SIZE];
        bool        oldFlag;

        count++;

        oldFlag = bear_BlackBoxFlag;

        bear_BlackBoxFlag = TRUE;
        pbCellMasterPtr = cellMasterPtr;

        (void) sprintf (command,
            "w -bfp %s/%s_%d.bfp -err %s/%s_%d.errlist",
            nplTmpDirPtr, FILE_PREFIX, count,
            nplTmpDirPtr, FILE_PREFIX, count);
        (void) PbExecCommand(NIL(WindowData), NIL(Point), command);

        (void) sprintf (command, "w -tech %s/%s_%d.tech",
            nplTmpDirPtr, FILE_PREFIX, count);
        (void) PbExecCommand(NIL(WindowData), NIL(Point), command);

        bear_BlackBoxFlag = oldFlag;
    }

#undef FILE_PREFIX
}
#endif /* DEBUG */


