
/* nplWriteToMac.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "gtGraph.h"
#include "fpg.h"
#include "tileProp.h"
#include "mac.h"
#include "rectSlice.h"
#include "list.h"
#include "newP.h"
#include "newPInt.h"
#include "nplWriteToMac.h"


/* ---------- Local Procedures ---------- */
static void         RewritePinList();
static void         AttachCoreBdrPins();
static void         ResetAllNets();
static void         FillInCellInst();
static void         ResizePinsOfNode();
static PinDirection SideDirToMacPinDir();
static PinKind      PinKindToMacPinKind();
static void         MoveSlots();
static void         StretchRect();
static void         ResizeChip();


static PinDirection
GetOppositeMacPinDir(macPinDir)
PinDirection macPinDir;
{
    switch (macPinDir) {
    case NORTH_PIN:
        return (SOUTH_PIN);
    case EAST_PIN:
        return (WEST_PIN);
    case SOUTH_PIN:
        return (NORTH_PIN);
    case WEST_PIN:
        return (EAST_PIN);
    default:
    Bear_PrintFatalErr("GetOppositeMacPinDir() failed : illegal pin direction");
        break;
    }
    /* NOTREACHED */
}


static PinDirection
SideDirToMacPinDir(sideDir)
Ncl_SideDir sideDir;
{
    switch (sideDir) {
    case NCL_NORTH_DIR:
        return (NORTH_PIN);
    case NCL_EAST_DIR:
        return (EAST_PIN);
    case NCL_SOUTH_DIR:
        return (SOUTH_PIN);
    case NCL_WEST_DIR:
        return (WEST_PIN);
    default:
    Bear_PrintFatalErr("SideDirToMacPinDir() failed : illegal side direction");
        break;
    }
    /* NOTREACHED */
}


static PinKind
PinKindToMacPinKind(pinKind)
Npl_PinKind pinKind;
{
    switch (pinKind) {
    case NPL_BORDER_PIN:
        return (MAC_BORDER_PIN);
    case NPL_INSIDE_PIN:
        return (MAC_INSIDE_PIN);
    default:
    Bear_PrintFatalErr("PinKindToMacPinKind() failed : illegal pin kind");
        break;
    }
    /* NOTREACHED */
}


BearStatus
NplAttachPinListOne(lineSegPtr, nodePtr)
LineSeg     *lineSegPtr;
Ncl_Node    *nodePtr;
{
    lsGen           gen;
    lsList          pinList;
    Npl_PinData     *pinDataPtr;
    PinDirection    macPinDir;
    PinKind         macPinKind;
    PinInst         *pinPtr;

    switch (lineSegPtr->dir) {
    case GEO_NORTH:
        macPinDir = NORTH_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_NORTH_DIR];
        break;
    case GEO_SOUTH:
        macPinDir = SOUTH_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_SOUTH_DIR];
        break;
    case GEO_WEST:
        macPinDir = WEST_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_WEST_DIR];
        break;
    case GEO_EAST:
        macPinDir = EAST_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_EAST_DIR];
        break;
    default:
Bear_PrintFatalErr("NplAttachPinListOne() failed : line seg has bad direction");
        break;
    }

    lineSegPtr->pinList = NIL(PinInst);
    if (lsLength(pinList) == 0) {
        return (BEAR_OK);
    }

    lsFirstItem(pinList, (lsGeneric *) &pinDataPtr, LS_NH);
    if (pinDataPtr->pinKind == NPL_BORDER_PIN) {
        macPinKind = MAC_BORDER_PIN;
        macPinDir = GetOppositeMacPinDir(macPinDir);
    }
    else {
        macPinKind = MAC_INSIDE_PIN;
    }

    Bear_BackeachItem(pinList, gen, pinDataPtr) {
        pinPtr = pinDataPtr->pinPtr;

        if (pinPtr == NIL(PinInst)) {
            char pinMasterName[80];
            char portMasterName[80]; 
            char portInstName[80];

            (void) sprintf(pinMasterName,  "%d", Mac_GenerateId());
            (void) sprintf(portMasterName, "%d", Mac_GenerateId());
            (void) sprintf(portInstName,   "%d", Mac_GenerateId());
            if (Mac_CreatePin(pinDataPtr->nodePtr->nd_cellInstPtr,
                portMasterName, portInstName,
                pinMasterName, pinDataPtr->pinName,
                &pinPtr) == BEAR_ERROR) {
          Bear_PrintErr("NplAttachPinListOne() failed : couldn't create pin.");

                lsFinish(gen);
                return (BEAR_ERROR);
            }
            pinDataPtr->pinPtr = pinPtr;
        }

        Mac_PutRelPosOfPin(&pinDataPtr->tilePos, pinPtr);
        Mac_PutPinPosDefFlag(TRUE, pinPtr);

        Mac_PutDirectionOfPin(macPinDir, pinPtr);
        Mac_PutKindOfPin(macPinKind, pinPtr);
        Mac_PutFlexOfPin(pinDataPtr->pinFlexType, pinPtr);

        pinPtr->nextInCellPtr = lineSegPtr->pinList;
        lineSegPtr->pinList = pinPtr;
    }

    return (BEAR_OK);
}


void
NplAttachPinListFour(lineSegPtr, nodePtr)
LineSeg     *lineSegPtr;
Ncl_Node    *nodePtr;
{
    lsGen           gen;
    lsList          pinList;
    Npl_PinData     *pinDataPtr;
    PinDirection    macPinDir;
    PinKind         macPinKind;
    PinInst         *pinPtr;

    switch (lineSegPtr->dir) {
    case GEO_NORTH:
        macPinDir = NORTH_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_NORTH_DIR];
        break;
    case GEO_SOUTH:
        macPinDir = SOUTH_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_SOUTH_DIR];
        break;
    case GEO_WEST:
        macPinDir = WEST_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_WEST_DIR];
        break;
    case GEO_EAST:
        macPinDir = EAST_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_EAST_DIR];
        break;
    default:
Bear_PrintFatalErr("NplAttachPinListFour() failed : line seg has bad direction");
        break;
    }

    lineSegPtr->pinList = NIL(PinInst);
    if (lsLength(pinList) == 0) {
        return;
    }

    lsFirstItem(pinList, (lsGeneric *) &pinDataPtr, LS_NH);
    if (pinDataPtr->pinKind == NPL_BORDER_PIN) {
        macPinKind = MAC_BORDER_PIN;
        macPinDir = GetOppositeMacPinDir(macPinDir);
    }
    else {
        macPinKind = MAC_INSIDE_PIN;
    }

    Bear_BackeachItem(pinList, gen, pinDataPtr) {
        pinPtr = pinDataPtr->pinPtr;

        Mac_PutRelPosOfPin(&pinDataPtr->tilePos, pinPtr);
        Mac_PutPinPosDefFlag(TRUE, pinPtr);

        Mac_PutDirectionOfPin(macPinDir, pinPtr);
        Mac_PutKindOfPin(macPinKind, pinPtr);
        Mac_PutFlexOfPin(pinDataPtr->pinFlexType, pinPtr);

        pinPtr->nextInCellPtr = lineSegPtr->pinList;
        lineSegPtr->pinList = pinPtr;
    }
}


static void
RewritePinList(lineSegPtr, nodePtr)
LineSeg     *lineSegPtr;
Ncl_Node    *nodePtr;
/*
 * Rewrite the given line segments pin list.
 */
{
    lsGen           gen;
    lsList          pinList;
    Npl_PinData     *pinDataPtr;
    PinDirection    macPinDir;
    PinKind         macPinKind;
    PinInst         *pinPtr;
    Point           relPos;
    Transform       *tileTransPtr;

    tileTransPtr = &nodePtr->nd_tileTrans;

    switch (lineSegPtr->dir) {
    case GEO_NORTH:
        macPinDir = NORTH_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_NORTH_DIR];
        break;
    case GEO_SOUTH:
        macPinDir = SOUTH_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_SOUTH_DIR];
        break;
    case GEO_WEST:
        macPinDir = WEST_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_WEST_DIR];
        break;
    case GEO_EAST:
        macPinDir = EAST_PIN;
        pinList = nodePtr->nd_tilePinList[NCL_EAST_DIR];
        break;
    default:
   Bear_PrintFatalErr("RewritePinList() failed : line seg has bad direction");
        break;
    }

    lineSegPtr->pinList = NIL(PinInst);
    if (lsLength(pinList) == 0) {
        return;
    }

    lsFirstItem(pinList, (lsGeneric *) &pinDataPtr, LS_NH);
    if (pinDataPtr->pinKind == NPL_BORDER_PIN) {
        macPinKind = MAC_BORDER_PIN;
        macPinDir = GetOppositeMacPinDir(macPinDir);
    }
    else {
        macPinKind = MAC_INSIDE_PIN;
    }

    Bear_BackeachItem(pinList, gen, pinDataPtr) {
        if (pinDataPtr->pinFlexType == FIXED) {
            pinPtr = pinDataPtr->pinPtr;

            GeoTransPoint(tileTransPtr, &pinDataPtr->initPos, &relPos);
            Mac_PutRelPosOfPin(&relPos, pinPtr);
            Mac_PutPinPosDefFlag(TRUE, pinPtr);

            Mac_PutDirectionOfPin(macPinDir, pinPtr);
            Mac_PutKindOfPin(macPinKind, pinPtr);
            Mac_PutFlexOfPin(FIXED, pinPtr);

            pinPtr->nextInCellPtr = lineSegPtr->pinList;
            lineSegPtr->pinList = pinPtr;
        }
    }
}


static void
AttachCoreBdrPins(lineSegPtr)
LineSeg *lineSegPtr;
{
    lsGen           gen;
    lsList          pinList;
    Npl_PinData     *pinDataPtr;
    PinDirection    macPinDir;
    PinInst         *pinPtr;

    switch (lineSegPtr->dir) {
    case GEO_NORTH:
        macPinDir = SOUTH_PIN;
        pinList = nplBdrPinList[NCL_NORTH_DIR];
        break;
    case GEO_EAST:
        macPinDir = WEST_PIN;
        pinList = nplBdrPinList[NCL_EAST_DIR];
        break;
    case GEO_SOUTH:
        macPinDir = NORTH_PIN;
        pinList = nplBdrPinList[NCL_SOUTH_DIR];
        break;
    case GEO_WEST:
        macPinDir = EAST_PIN;
        pinList = nplBdrPinList[NCL_WEST_DIR];
        break;
    default:
Bear_PrintFatalErr("NplAttachCoreBdrPins() failed : line seg has bad direction");
        break;
    }

    lineSegPtr->pinList = NIL(PinInst);
    if (lsLength(pinList) == 0) {
        return;
    }

    Bear_BackeachItem(pinList, gen, pinDataPtr) {
        pinPtr = pinDataPtr->pinPtr;

        Mac_PutRelPosOfPin(&pinDataPtr->tilePos, pinPtr);
        Mac_PutPinPosDefFlag(TRUE, pinPtr);

        Mac_PutDirectionOfPin(macPinDir, pinPtr);
        Mac_PutKindOfPin(MAC_BORDER_PIN, pinPtr);
        Mac_PutFlexOfPin(pinDataPtr->pinFlexType, pinPtr);

        pinPtr->nextInCellPtr = lineSegPtr->pinList;
        lineSegPtr->pinList = pinPtr;
    }
}


static void
FillInCellInst(cellInstPtr, cellRectPtr, parentCellPtr)
CellInst    *cellInstPtr;
Rect        *cellRectPtr;
CellMaster  *parentCellPtr;
{
    GenericCell     gcell;
    LineSeg         *lineSegPtr;
    LinkedRect      *horzRectList, *vertRectList;
    LinkedLineSeg   *lnLineSeg0Ptr, *lnLineSeg1Ptr;
    LinkedLineSeg   *lnLineSeg2Ptr, *lnLineSeg3Ptr;
    Point           origin;

    cellInstPtr->parentPtr = parentCellPtr;
    cellInstPtr->siblingPtr = NIL(CellInst);

    Mac_PutBBoxOfCell(cellRectPtr, cellInstPtr);

    origin.p_x = origin.p_y = 0;
    Mac_PutOriginOfCell(&origin, cellInstPtr);

    gcell.unType = CELL_INST;
    gcell.unValue.cellInstPtr = cellInstPtr;

    Mac_PutTransformOfGCell(&GeoIdentityTransform, &gcell);

    if ((horzRectList = Mac_GetHorzRectListOfGCell(&gcell)) !=
        NIL(LinkedRect)) {
        horzRectList->r_r = *cellRectPtr;
    }

    if ((vertRectList = Mac_GetVertRectListOfGCell(&gcell)) !=
        NIL(LinkedRect)) {
        vertRectList->r_r = *cellRectPtr;
    }

    if (cellInstPtr->boundary != NIL(LinkedLineSeg)) {
        lnLineSeg0Ptr = cellInstPtr->boundary;
        lineSegPtr = lnLineSeg0Ptr->lineSeg;
        lineSegPtr->segID = 0;
        lineSegPtr->dir = GEO_NORTH;
        lineSegPtr->line.point1.p_x = cellRectPtr->r_xbot;
        lineSegPtr->line.point1.p_y = cellRectPtr->r_ytop;
        lineSegPtr->line.point2.p_x = cellRectPtr->r_xtop;
        lineSegPtr->line.point2.p_y = cellRectPtr->r_ytop;
        lineSegPtr->pinList = NIL(PinInst);

        lnLineSeg1Ptr = lnLineSeg0Ptr->nextPtr;
        lineSegPtr = lnLineSeg1Ptr->lineSeg;
        lineSegPtr->segID = 1;
        lineSegPtr->dir = GEO_EAST;
        lineSegPtr->line.point1.p_x = cellRectPtr->r_xtop;
        lineSegPtr->line.point1.p_y = cellRectPtr->r_ytop;
        lineSegPtr->line.point2.p_x = cellRectPtr->r_xtop;
        lineSegPtr->line.point2.p_y = cellRectPtr->r_ybot;
        lineSegPtr->pinList = NIL(PinInst);

        lnLineSeg2Ptr = lnLineSeg1Ptr->nextPtr;
        lineSegPtr = lnLineSeg2Ptr->lineSeg;
        lineSegPtr->segID = 2;
        lineSegPtr->dir = GEO_SOUTH;
        lineSegPtr->line.point1.p_x = cellRectPtr->r_xtop;
        lineSegPtr->line.point1.p_y = cellRectPtr->r_ybot;
        lineSegPtr->line.point2.p_x = cellRectPtr->r_xbot;
        lineSegPtr->line.point2.p_y = cellRectPtr->r_ybot;
        lineSegPtr->pinList = NIL(PinInst);

        lnLineSeg3Ptr = lnLineSeg2Ptr->nextPtr;
        lineSegPtr = lnLineSeg3Ptr->lineSeg;
        lineSegPtr->segID = 3;
        lineSegPtr->dir = GEO_WEST;
        lineSegPtr->line.point1.p_x = cellRectPtr->r_xbot;
        lineSegPtr->line.point1.p_y = cellRectPtr->r_ybot;
        lineSegPtr->line.point2.p_x = cellRectPtr->r_xbot;
        lineSegPtr->line.point2.p_y = cellRectPtr->r_ytop;
        lineSegPtr->pinList = NIL(PinInst);
    }
}


static void
ResizePinsOfNode(nodePtr, xScale, yScale)
Ncl_Node    *nodePtr;
float       xScale, yScale;
{
    lsGen       pinGen;
    Npl_PinData *pinDataPtr;
    int         i, xtop, ytop;

    if ((xScale == 1.0) && (yScale == 1.0)) {
        return;
    }

    xtop = nodePtr->nd_dim.p_x;
    ytop = nodePtr->nd_dim.p_y;

    lsForeachItem(nodePtr->nd_initPinList[NCL_NORTH_DIR], pinGen, pinDataPtr) {
        if (xScale != 1.0) {
            pinDataPtr->initPos.p_x *= xScale;
        }
        if (yScale != 1.0) {
            pinDataPtr->initPos.p_y = ytop;
        }
    }
    lsForeachItem(nodePtr->nd_initPinList[NCL_EAST_DIR], pinGen, pinDataPtr) {
        if (xScale != 1.0) {
            pinDataPtr->initPos.p_x = xtop;
        }
        if (yScale != 1.0) {
            pinDataPtr->initPos.p_y *= yScale;
        }
    }
    if (xScale != 1.0) {
        lsForeachItem(nodePtr->nd_initPinList[NCL_SOUTH_DIR],
            pinGen, pinDataPtr) {
            pinDataPtr->initPos.p_x *= xScale;
        }
    }
    if (yScale != 1.0) {
        lsForeachItem(nodePtr->nd_initPinList[NCL_WEST_DIR],
            pinGen, pinDataPtr) {
            pinDataPtr->initPos.p_y *= yScale;
        }
    }
}


static void
MoveSlots(nodePtr, dx, dy)
Ncl_Node    *nodePtr;
int         dx, dy;
{
    int i;

    if (nodePtr->nd_northSlot != NIL(Ncl_SlotData)) {
        for (i = 0; i < nodePtr->nd_numOfNorthSlots ;i++) {
            nodePtr->nd_northSlot[i].tilePos.p_x += dx;
            nodePtr->nd_northSlot[i].tilePos.p_y += dy;
        }
    }
    if (nodePtr->nd_eastSlot != NIL(Ncl_SlotData)) {
        for (i = 0; i < nodePtr->nd_numOfEastSlots ;i++) {
            nodePtr->nd_eastSlot[i].tilePos.p_x += dx;
            nodePtr->nd_eastSlot[i].tilePos.p_y += dy;
        }
    }
    if (nodePtr->nd_southSlot != NIL(Ncl_SlotData)) {
        for (i = 0; i < nodePtr->nd_numOfSouthSlots ;i++) {
            nodePtr->nd_southSlot[i].tilePos.p_x += dx;
            nodePtr->nd_southSlot[i].tilePos.p_y += dy;
        }
    }
    if (nodePtr->nd_westSlot != NIL(Ncl_SlotData)) {
        for (i = 0; i < nodePtr->nd_numOfWestSlots ;i++) {
            nodePtr->nd_westSlot[i].tilePos.p_x += dx;
            nodePtr->nd_westSlot[i].tilePos.p_y += dy;
        }
    }
}


BearStatus
NplReformatCell(nodePtr)
Ncl_Node *nodePtr;
{
    int             dx, dy;
    float           xScale, yScale;
    CellInst        *cellInstPtr;
    LinkedLineSeg   *lnLineSegPtr;

    dx = nodePtr->nd_nodeTrans.t_c - nodePtr->nd_storeTrans.t_c;
    dy = nodePtr->nd_nodeTrans.t_f - nodePtr->nd_storeTrans.t_f;

    MoveSlots(nodePtr, dx, dy);

    if ((cellInstPtr = nodePtr->nd_cellInstPtr) == NIL(CellInst)) {
        if (NplCalcTileDataOfNode(nodePtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
        return (BEAR_OK);
    }

    xScale = ((float) nodePtr->nd_dim.p_x) / ((float) nodePtr->nd_storeDim.p_x);
    yScale = ((float) nodePtr->nd_dim.p_y) / ((float) nodePtr->nd_storeDim.p_y); 

    if (nodePtr->nd_type == LEAF) {
        if (dx != 0 || dy != 0) {
            Mac_MoveCell(dx, dy, cellInstPtr);
        }
        if (NplCalcTileDataOfLeafNode(nodePtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
    }
    else {
        if ((xScale == 1.0) && (yScale == 1.0)) {
            if (dx != 0 || dy != 0) {
                Mac_MoveCell(dx, dy, cellInstPtr);
            }
            if (NplCalcTileDataOfNode(nodePtr) == BEAR_ERROR) {
                return (BEAR_ERROR);
            }
        }
        else {
            ResizePinsOfNode(nodePtr, xScale, yScale);

            if (NplCalcTileDataOfNode(nodePtr) == BEAR_ERROR) {
                return (BEAR_ERROR);
            }

            cellInstPtr = nodePtr->nd_cellInstPtr;
            FillInCellInst(cellInstPtr, &nodePtr->nd_tileRect, NIL(CellMaster));

            for (lnLineSegPtr = cellInstPtr->boundary;
                lnLineSegPtr != NIL(LinkedLineSeg);
                lnLineSegPtr = lnLineSegPtr->nextPtr) {

                NplAttachPinListFour(lnLineSegPtr->lineSeg, nodePtr);
            }
        }
    }

    return (BEAR_OK);
}


BearStatus
NplWriteNodeToMac(nodePtr)
Ncl_Node *nodePtr;
{
    CellInst        *cellInstPtr;
    CellMaster      *cellMasterPtr;
    GenericCell     gcell;
    LinkedPoint     *pointList;
    LinkedLineSeg   *boundaryPtr, *lnLineSegPtr;
    LinkedRect      *rectList;
    Point           origin;
    Rect            tileRect;
    Shape           *shapePtr;
    char            blockName[80];
    char            blockMasterName[80];
    
    (void) sprintf (blockName, "B%d", Mac_GenerateId());
    (void) sprintf (blockMasterName, "%s_master", blockName);

    if (Mac_CreateCell(NIL(CellMaster),
        blockMasterName, blockName, &cellInstPtr) == BEAR_ERROR) {

        Bear_PrintErr("NplWriteNodeToMac() : cannot create cell instance.");
        return (BEAR_ERROR);
    }
    cellInstPtr->clientData = (ClientData) nodePtr;
    nodePtr->nd_cellInstPtr = cellInstPtr;

    if (NplCalcTileDataOfNode(nodePtr) == BEAR_ERROR) {
        return (BEAR_ERROR);
    }
    tileRect = nodePtr->nd_tileRect;

    gcell.unType = CELL_INST;
    gcell.unValue.cellInstPtr = cellInstPtr;

    Mac_PutBBoxOfCell(&tileRect, cellInstPtr);

    origin.p_x = origin.p_y = 0;
    Mac_PutOriginOfCell(&origin, cellInstPtr);

    Mac_PutTransformOfGCell(&GeoIdentityTransform, &gcell);

    Mac_CreatePointListFromRect(&tileRect, &pointList);

    if (Mac_MakeLineSegList(pointList, &boundaryPtr) == BEAR_OK) {
        (void) Mac_AssignDirOfLineSegments(boundaryPtr);
        Mac_PutBoundaryOfCell(boundaryPtr, cellInstPtr);
    }
    else {
        Bear_PrintErr("NplWriteNodeToMac() : cannot make line segment list.");
        goto error;
    }

    if (Rs_HorzRectSlice(pointList, &rectList) == BEAR_OK) {
        Mac_PutHorzRectListOfGCell(rectList, &gcell);
    }
    else {
 Bear_PrintErr("NplWriteNodeToMac() : cannot make horizontal rectangle list.");
        goto error;
    }
    if (Rs_VertRectSlice(pointList, &rectList) == BEAR_OK) {
        Mac_PutVertRectListOfGCell(rectList, &gcell);
    }
    else {
   Bear_PrintErr("NplWriteNodeToMac() : cannot make vertical rectangle list.");
        goto error;
    }

    Mac_FreePointList(pointList);

    for (lnLineSegPtr = boundaryPtr;
        lnLineSegPtr != NIL(LinkedLineSeg);
        lnLineSegPtr = lnLineSegPtr->nextPtr) {

        lnLineSegPtr->lineSeg->pinList = NIL(PinInst);
    }

    cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);
    cellMasterPtr->shapeFlag = MAC_FLEX_SHAPE;
    cellMasterPtr->shapeList = lsCreate();
    MALLOC(Shape *, shapePtr, sizeof (Shape));
    shapePtr->horzDim = tileRect.r_xtop - tileRect.r_xbot;
    shapePtr->vertDim = tileRect.r_ytop - tileRect.r_ybot;
    lsNewEnd(cellMasterPtr->shapeList, (lsGeneric) shapePtr, LS_NH);

    return (BEAR_OK);

error:
    nodePtr->nd_cellInstPtr = NIL(CellInst);

    return (BEAR_ERROR);
}



BearStatus
NplRewriteNodeToMac(nodePtr)
Ncl_Node *nodePtr;
{
    CellInst        *cellInstPtr;
    LinkedLineSeg   *lnLineSegPtr;
    
    if (NPLISLEAF(nodePtr) == TRUE) {
        if (NplCalcTileDataOfLeafNode(nodePtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
    }
    else {
        if (NplCalcTileDataOfNode(nodePtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
    }

    if ((cellInstPtr = nodePtr->nd_cellInstPtr) == NIL(CellInst)) {
        Bear_PrintErr("NplRewriteNodeToMac() : cell instance is missing.");
        return (BEAR_ERROR);
    }
    FillInCellInst(cellInstPtr, &nodePtr->nd_tileRect, NIL(CellMaster));

    for (lnLineSegPtr = Mac_GetBoundaryOfCell(cellInstPtr);
        lnLineSegPtr != NIL(LinkedLineSeg);
        lnLineSegPtr = lnLineSegPtr->nextPtr) {

        RewritePinList(lnLineSegPtr->lineSeg, nodePtr);
    }

    return (BEAR_OK);
}



static void
ResetAllNets()
{
    lsGen       netGen;
    Net         *netPtr;
    Npl_NetData *netDataPtr;

    lsForeachItem(nplNetList, netGen, netDataPtr) {
        netPtr = netDataPtr->netPtr;

        netPtr->cellMasterPtr = NIL(CellMaster);
        netPtr->portList = NIL(PortInst);
        netPtr->nextPtr = NIL(Net);
    }
}


static void
StretchRect(rectPtr, percent)
register Rect   *rectPtr;
double          percent;
{
    register int deltaX, deltaY;

    deltaX = (rectPtr->r_xtop - rectPtr->r_xbot) * (percent - 1.0) / 2.0;
    deltaY = (rectPtr->r_ytop - rectPtr->r_ybot) * (percent - 1.0) / 2.0;
    rectPtr->r_xbot -= deltaX;
    rectPtr->r_ybot -= deltaY;
    rectPtr->r_xtop += deltaX;
    rectPtr->r_ytop += deltaY;
}


static void
ResizeChip(coreMasterPtr)
CellMaster *coreMasterPtr;
{
    lsGen           pinGen;
    CellInst        *chipInstPtr, *coreInstPtr;
    CellMaster      *chipMasterPtr;
    LinkedLineSeg   *lnLineSegPtr;
    Point           tmpPos;
    Npl_PinData     *pinDataPtr;
    Rect            newCmBBox, *oldCmBBoxPtr;
    Transform       trans;
    double          oldCmWidth, oldCmHeight;
    double          newCmWidth, newCmHeight;
    double          xScale, yScale;
    int             xbot, ybot, xtop, ytop;

    oldCmBBoxPtr = Mac_GetBBoxOfCellMaster(coreMasterPtr);
    oldCmWidth = oldCmBBoxPtr->r_xtop - oldCmBBoxPtr->r_xbot;
    oldCmHeight = oldCmBBoxPtr->r_ytop - oldCmBBoxPtr->r_ybot;

    Mac_FindBBoxForAllCells(coreMasterPtr, &newCmBBox);
    StretchRect(&newCmBBox, nplBorderSize);
    newCmWidth = newCmBBox.r_xtop - newCmBBox.r_xbot;
    newCmHeight = newCmBBox.r_ytop - newCmBBox.r_ybot;

    xScale = newCmWidth / oldCmWidth;
    yScale = newCmHeight / oldCmHeight;

    GeoTranslateTrans(&GeoIdentityTransform,
        -oldCmBBoxPtr->r_xbot, -oldCmBBoxPtr->r_ybot, &trans);
    xbot = newCmBBox.r_xbot;
    ybot = newCmBBox.r_ybot;
    lsForeachItem(nplBdrPinList[NCL_WEST_DIR], pinGen, pinDataPtr) {
        GeoTransPoint(&trans, &pinDataPtr->tilePos, &tmpPos);
        pinDataPtr->tilePos.p_x = xbot;
        pinDataPtr->tilePos.p_y = ybot + tmpPos.p_y * yScale;
    }
    lsForeachItem(nplBdrPinList[NCL_SOUTH_DIR], pinGen, pinDataPtr) {
        GeoTransPoint(&trans, &pinDataPtr->tilePos, &tmpPos);
        pinDataPtr->tilePos.p_x = xbot + tmpPos.p_x * xScale;
        pinDataPtr->tilePos.p_y = ybot;
    }

    GeoTranslateTrans(&GeoIdentityTransform,
        -oldCmBBoxPtr->r_xbot, -oldCmBBoxPtr->r_ytop, &trans);
    xbot = newCmBBox.r_xbot;
    ytop = newCmBBox.r_ytop;
    lsForeachItem(nplBdrPinList[NCL_NORTH_DIR], pinGen, pinDataPtr) {
        GeoTransPoint(&trans, &pinDataPtr->tilePos, &tmpPos);
        pinDataPtr->tilePos.p_x = xbot + tmpPos.p_x * xScale;
        pinDataPtr->tilePos.p_y = ytop;
    }

    GeoTranslateTrans(&GeoIdentityTransform,
        -oldCmBBoxPtr->r_xtop, -oldCmBBoxPtr->r_ybot, &trans);
    xtop = newCmBBox.r_xtop;
    ybot = newCmBBox.r_ybot;
    lsForeachItem(nplBdrPinList[NCL_EAST_DIR], pinGen, pinDataPtr) {
        GeoTransPoint(&trans, &pinDataPtr->tilePos, &tmpPos);
        pinDataPtr->tilePos.p_x = xtop;
        pinDataPtr->tilePos.p_y = ybot + tmpPos.p_y * yScale;
    }

    coreInstPtr = Mac_GetInstOfCoreMaster(coreMasterPtr);
    chipMasterPtr = Mac_GetParentOfCell(coreInstPtr);
    chipInstPtr = Mac_GetInstOfChipMaster(chipMasterPtr);

    FillInCellInst(chipInstPtr, &newCmBBox, NIL(CellMaster));
    FillInCellInst(coreInstPtr, &newCmBBox, chipMasterPtr);

    for (lnLineSegPtr = Mac_GetBoundaryOfCell(coreInstPtr);
        lnLineSegPtr != NIL(LinkedLineSeg);
        lnLineSegPtr = lnLineSegPtr->nextPtr) {

        AttachCoreBdrPins(lnLineSegPtr->lineSeg);
    }
}



BearStatus
NplSetUpCoreMaster(coreMasterPtr, nodeList)
CellMaster  *coreMasterPtr;
lsList      nodeList;
{
    lsGen       nodeGen, netGen, pinGen;
    lsList      netList;
    CellInst    *chipInstPtr, *coreInstPtr, *cellInstPtr;
    CellMaster  *chipMasterPtr;
    Net         *netPtr;
    Ncl_Node    *nodePtr;
    PinInst     *pinPtr;
    Npl_NetData *netDataPtr;
    Npl_PinData *pinDataPtr;
    PlaneData   *planeDataPtr;
    int         i;

    coreMasterPtr->contents.cellInstList = NIL(CellInst);
    lsForeachItem(nodeList, nodeGen, nodePtr) {
        cellInstPtr = nodePtr->nd_cellInstPtr;

        cellInstPtr->parentPtr = coreMasterPtr;

        cellInstPtr->siblingPtr = coreMasterPtr->contents.cellInstList;
        coreMasterPtr->contents.cellInstList = cellInstPtr;
    }

    ResizeChip(coreMasterPtr);

    coreInstPtr = Mac_GetInstOfCoreMaster(coreMasterPtr);
    chipMasterPtr = Mac_GetParentOfCell(coreInstPtr);
    Mac_PutPlacedFlagOfCell(TRUE, chipMasterPtr);
    if (Mac_SetUpTiPlane(chipMasterPtr, FALSE) != BEAR_OK) {
        return (BEAR_ERROR);
    }

    Mac_PutPlacedFlagOfCell(TRUE, coreMasterPtr);
    if (Mac_SetUpTiPlane(coreMasterPtr, FALSE) != BEAR_OK) {
        return (BEAR_ERROR);
    }
    if (Fpg_GenRooms(coreMasterPtr) != BEAR_OK) {
        Bear_PrintErr("NplSetUpCoreMaster() : could not generate rooms.");
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}



BearStatus
NplSetUpNetList(coreMasterPtr, nodeList)
CellMaster  *coreMasterPtr;
lsList      nodeList;
{
    lsGen       nodeGen, netGen, pinGen;
    lsList      netList;
    CellInst    *chipInstPtr, *coreInstPtr, *cellInstPtr;
    CellMaster  *chipMasterPtr;
    Net         *netPtr;
    Ncl_Node    *nodePtr;
    PinInst     *pinPtr;
    Npl_NetData *netDataPtr;
    Npl_PinData *pinDataPtr;
    PlaneData   *planeDataPtr;
    int         i;

    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(coreMasterPtr));
    ResetAllNets();

    lsForeachItem(nplNetList, netGen, netDataPtr) {
        if (netDataPtr->pinList != (lsList) NULL) {
            lsDestroy(netDataPtr->pinList, (void (*)()) NULL);
            netDataPtr->pinList = (lsList) NULL;
        }
    }

    netList = lsCreate();

    NclForeachSide(i) {
        lsForeachItem(nplBdrPinList[i], pinGen, pinDataPtr) {
            netDataPtr = pinDataPtr->netDataPtr;
            if (netDataPtr->pinList == (lsList) NULL) {
                lsNewEnd(netList, (lsGeneric) netDataPtr, LS_NH);
                netDataPtr->pinList = lsCreate();
            }
            lsNewEnd(netDataPtr->pinList, (lsGeneric) pinDataPtr, LS_NH);
        }
    }

    lsForeachItem(nodeList, nodeGen, nodePtr) {
        NclForeachSide(i) {
            lsForeachItem(nodePtr->nd_initPinList[i], pinGen, pinDataPtr) {
                netDataPtr = pinDataPtr->netDataPtr;
                if (netDataPtr->pinList == (lsList) NULL) {
                    lsNewEnd(netList, (lsGeneric) netDataPtr, LS_NH);
                    netDataPtr->pinList = lsCreate();
                }
                lsNewEnd(netDataPtr->pinList, (lsGeneric) pinDataPtr, LS_NH);
            }
        }
    }

    coreMasterPtr->contents.netList = NIL(Net);
    Bear_BackeachItem(netList, netGen, netDataPtr) {
        netPtr = netDataPtr->netPtr;

        Bear_BackeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
            pinPtr = pinDataPtr->pinPtr;

            Mac_PutDirectionOfPin(SideDirToMacPinDir(pinDataPtr->tileDir),
                pinPtr);
            Mac_PutKindOfPin(PinKindToMacPinKind(pinDataPtr->pinKind),
                pinPtr);

            (void) Mac_InsertPortInstOfNet(netPtr, pinPtr->portPtr);
        }

        netPtr->cellMasterPtr = coreMasterPtr;
        netPtr->nextPtr = coreMasterPtr->contents.netList;
        coreMasterPtr->contents.netList = netPtr;
    }

    return (BEAR_OK);
}


BearStatus
NplSetUpMacNetList(coreMasterPtr, netList)
CellMaster  *coreMasterPtr;
lsList      netList;
{
    lsGen       nodeGen, netGen, pinGen;
    CellInst    *chipInstPtr, *coreInstPtr, *cellInstPtr;
    CellMaster  *chipMasterPtr;
    Net         *netPtr;
    Ncl_Node    *nodePtr;
    PinInst     *pinPtr;
    Npl_NetData *netDataPtr;
    Npl_PinData *pinDataPtr;
    PlaneData   *planeDataPtr;
    int         i;
    
    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(coreMasterPtr));
    ResetAllNets();
    
    coreMasterPtr->contents.netList = NIL(Net);
    Bear_BackeachItem(netList, netGen, netDataPtr) {
        netPtr = netDataPtr->netPtr;
    
        if (netDataPtr->pinList != (lsList) NULL) {
            Bear_BackeachItem(netDataPtr->pinList, pinGen, pinDataPtr) {
                pinPtr = pinDataPtr->pinPtr;
                if (pinPtr == NIL(PinInst)) {
                   Bear_PrintErr("NplSetUpMacNetList() failed: pinPtr is NIL.");
                    lsFinish(pinGen);
                    lsFinish(netGen);
                    return (BEAR_ERROR);
                }
                
                Mac_PutDirectionOfPin(SideDirToMacPinDir(pinDataPtr->tileDir),
                    pinPtr);
                Mac_PutKindOfPin(PinKindToMacPinKind(pinDataPtr->pinKind),
                    pinPtr);
                
                (void) Mac_InsertPortInstOfNet(netPtr, pinPtr->portPtr);
            }
            
            netPtr->cellMasterPtr = coreMasterPtr;
            netPtr->nextPtr = coreMasterPtr->contents.netList;
            coreMasterPtr->contents.netList = netPtr;
        }
    }
    
    return (BEAR_OK);
}


