
/* buildMacCell.c --
 *      
 *     Denis Lee
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 */

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "oct.h"
#include "octInterface.h"
#include "octInterfaceInt.h"


/*
 * ----------------------------------------------------------------------------
 *
 * BuildMacCell() --
 *
 * Stores the name of the cell. Then creates the cell and its origin.
 * Also, calls BuildMacCellBoundary() to create and store the boundary
 * of the cell.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
BuildMacCell(parentCellPtr, cellInstPtrPtr, cellInstName, xl, yb, xr, yt)
CellMaster  *parentCellPtr;
CellInst    **cellInstPtrPtr;
char        *cellInstName;
int         xl, yb, xr, yt;
{
    char        *cellInstMasterName;
    CellInst    *cellInstPtr;
    Point       origin;

    /* store name of cell */
    MALLOC(char *, cellInstMasterName, (strlen(cellInstName) + 8)
        * sizeof (char));
    sprintf(cellInstMasterName, "%s_master", cellInstName);
    /* create cell */
    if (Mac_CreateCell(parentCellPtr, cellInstMasterName, cellInstName,
        &cellInstPtr) == BEAR_ERROR) {
        Bear_PrintErr("BuildMacCell: Mac_CreateCell() failed.");
        return (BEAR_ERROR);
    }

    /* store origin and transform */
    origin.p_x = xl;
    origin.p_y = yb;
    GeoTranslateTrans(&GeoIdentityTransform, origin.p_x, origin.p_y,
        &cellInstPtr->transform);
    Mac_PutOriginOfCell(&origin, cellInstPtr);

    /* create and store boundary */
    if (BuildMacCellBoundary(0, 0, xr - xl, yt - yb, &cellInstPtr)
        == BEAR_ERROR) {
        Bear_PrintErr("BuildMacCell: BuildMacCellBoundary() failed.");
        return (BEAR_ERROR);
    }

    *cellInstPtrPtr = cellInstPtr;
    return (BEAR_OK);
}
