
/* buildMacCellBoudary.c --
 *          
 *     Denis Lee
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 */

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "rectSlice.h"
#include "octInterface.h"
#include "octInterfaceInt.h"


/*
 * ----------------------------------------------------------------------------
 *
 * BuildMacCellBoundary() --
 *
 * Takes up a list of edges and stores them as line segments in 
 * boundary of the cell.  Also the direction of the line segments
 * is determined.  Dissection of rectangles is done here, too.
 * Notice that the memory allocation of line segments is done here.
 *
 * Results:
 *   BEAR_OK, if successful.
 *   BEAR_ERROR, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
BuildMacCellBoundary(xl, yb, xr, yt, cellInst)
int xl, yb, xr, yt;
CellInst  **cellInst;
{
    int             x[3], y[3];
    int             edgei;
    GenericCell     gcell;
    LineSeg         *lineSeg, *firstLineSeg;
    LinkedLineSeg   *linkedLineSeg, *firstLinked;
    int             counter;
    LinkedPoint     *pointList, *newPointPtr;
    LinkedRect      *horzRectList = NIL(LinkedRect);
    LinkedRect      *vertRectList = NIL(LinkedRect);
    Rect            bbox;
    Transform       *transPtr;

/*
    x[0] = xl;
    y[0] = yt;
    x[1] = xr;
    y[1] = yt;
    x[2] = xr;
    y[2] = yb;
    x[3] = xl;
    y[3] = yb;
*/
    x[0] = xr;
    y[0] = yb;
    x[1] = xr;
    y[1] = yt;
    x[2] = xl;
    y[2] = yt;
/*
    x[3] = xl;
    y[3] = yb;
*/
    firstLineSeg = Mac_MakeLineSeg();
    MALLOC(LinkedLineSeg *, firstLinked, sizeof (LinkedLineSeg));
    MALLOC(LinkedPoint *, pointList, sizeof (LinkedPoint));
    pointList->nextPtr = NIL(LinkedPoint);  
    firstLineSeg->line.point1.p_x = 
        pointList->point.p_x = xl;
    firstLineSeg->line.point1.p_y = 
        pointList->point.p_y = yb;
    firstLinked->lineSeg = firstLineSeg;
    (*cellInst)->boundary = firstLinked;
    counter = 1;
    for (edgei = 0; edgei <= 2; edgei++) { 
        lineSeg = Mac_MakeLineSeg();
        MALLOC(LinkedLineSeg *, linkedLineSeg, sizeof (LinkedLineSeg));
        MALLOC(LinkedPoint *, newPointPtr, sizeof (LinkedPoint));
        newPointPtr->nextPtr = NIL(LinkedPoint);
        lineSeg->line.point1.p_x =
            newPointPtr->point.p_x = x[edgei];
        lineSeg->line.point1.p_y =
            newPointPtr->point.p_y = y[edgei];
        
        if (counter >= 2) {
            lineSeg->line.point2.p_x = 
                (*cellInst)->boundary->nextPtr->lineSeg->line.point1.p_x;
            lineSeg->line.point2.p_y = 
                (*cellInst)->boundary->nextPtr->lineSeg->line.point1.p_y;
        } else {
            lineSeg->line.point2.p_x = 
                firstLinked->lineSeg->line.point1.p_x;
            lineSeg->line.point2.p_y = 
                firstLinked->lineSeg->line.point1.p_y;
        }
        if (pointList->nextPtr != NIL(LinkedPoint)) {
            newPointPtr->nextPtr = pointList->nextPtr;
        }
        pointList->nextPtr = newPointPtr;
        linkedLineSeg->lineSeg = lineSeg;
        counter = counter +1;
        if (counter == 2) {
            (*cellInst)->boundary->nextPtr = linkedLineSeg;
        } else {
            linkedLineSeg->nextPtr = (*cellInst)->boundary->nextPtr;
            (*cellInst)->boundary->nextPtr = linkedLineSeg;
        }
    }
    (*cellInst)->boundary->lineSeg->line.point2.p_x = 
        (*cellInst)->boundary->nextPtr->lineSeg->line.point1.p_x;
    (*cellInst)->boundary->lineSeg->line.point2.p_y = 
        (*cellInst)->boundary->nextPtr->lineSeg->line.point1.p_y;
    gcell.unType = CELL_INST;
    gcell.unValue.cellInstPtr = *cellInst;
    transPtr = Mac_GetTransformOfGCell(&gcell);
    (void) Mac_MoveBoundaryOfCell(transPtr, *cellInst);
    if (Mac_AssignDirOfLineSegments(Mac_GetBoundaryOfCell(*cellInst)) ==
        BEAR_ERROR) {
        return (BEAR_ERROR);
    }

{
/*
 * Look for circular lists and if you find one convert it to
 * a linked list.
 */
LinkedPoint *lpPtr;

for (lpPtr = pointList; lpPtr != NIL(LinkedPoint); lpPtr = lpPtr->nextPtr) {
    if (lpPtr->nextPtr == pointList) {
        lpPtr->nextPtr = NIL(LinkedPoint);
        break;
    }
}
}

/*
Mac_PrintPointList(&GeoIdentityTransform,pointList);
*/
    if (Mac_FindBBoxForPointList(pointList, &bbox) == BEAR_ERROR) {
        return (BEAR_ERROR);
    } else {
        Mac_PutBBoxOfCell(&bbox, *cellInst);
    }

    if (Rs_HorzRectSlice(pointList, &horzRectList) == BEAR_ERROR) {
        return (BEAR_ERROR);
    } else {
        Mac_PutHorzRectListOfGCell(horzRectList, &gcell);
    }
    if (Rs_VertRectSlice(pointList, &vertRectList) == BEAR_ERROR) {
        return (BEAR_ERROR);
    } else {
        Mac_PutVertRectListOfGCell(vertRectList, &gcell);
    }

    Mac_FreePointList(pointList);
    return (BEAR_OK);
}
