
/* readFromOct.c --
 *      
 *     Denis Lee
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "oct.h"
#include "region.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "octInterface.h"
#include "octInterfaceInt.h"

/*
#define IMIN -9999999;
#define IMAX 9999999;
*/

octObject odi_outchip;
int chip_xl, chip_yb, chip_xr, chip_yt;
/*
    octBegin();
        out_chip.contents.facet.cell = outchip_cell;
    octEnd();
*/


/* ---------- Local Procedures ---------- */
static BearStatus CopyFacets(/* cell1, view1, cell2, view2 */);
static BearStatus IOPadsFindChipBox(/* iopad, chip_xl, chip_yb, chip_xr,
                                    chip_yt */);
static BearStatus CoreCellsFindChipBox(/* instance, chip_xl, chip_yb, chip_xr,
                                       chip_yt */);
static BearStatus FindPinPosition(/* inst_bbox, term_bbox, pinX, pinY */);


/*
 * CopyFacets() - copies a facet of a cell and view to
 *                another facet of another cell and view.
 */
static BearStatus
CopyFacets(cell1, view1, cell2, view2)
char *cell1, *view1, *cell2, *view2;

{
    octObject chip1, chip2;

    /* copy contents facet */
    chip1.type = OCT_FACET;
    chip1.contents.facet.cell = cell1;
    chip1.contents.facet.view = view1;
    chip1.contents.facet.version = OCT_CURRENT_VERSION;
    chip1.contents.facet.facet = "contents";
    chip1.contents.facet.mode = "r";
    if (octOpenFacet(&chip1) < OCT_OK) {
        octError("CopyFacets: cannot open chip 1 contents.");
        return(BEAR_ERROR);
    }
    chip2.type = OCT_FACET;
    chip2.contents.facet.cell = cell2;
    chip2.contents.facet.view = view2;
    chip2.contents.facet.version = OCT_CURRENT_VERSION;
    chip2.contents.facet.facet = "contents";
    if (octCopyFacet(&chip2,&chip1) != OCT_OK) {
        octError("CopyFacets: cannot copy facet contents.");
        return(BEAR_ERROR);
    }

    /* copy interface facet */
    chip1.contents.facet.facet = "interface";
    chip2.contents.facet.facet = "interface";
    if (octOpenFacet(&chip1) < OCT_OK) {
        octError("CopyFacets: cannot open chip 1 interface.");
        return(BEAR_ERROR);
    }
    if (octCopyFacet(&chip2,&chip1) != OCT_OK) {
        octError("CopyFacets: cannot copy facet interface.");
        return(BEAR_ERROR);
    }
    return(BEAR_OK);
}


/*
 * IOPadsFindChipBox() - determines the bounding box of the chip
 *                       from the I/O pads' coordinates
 */
static BearStatus
IOPadsFindChipBox(iopad, chip_xl, chip_yb, chip_xr, chip_yt)
octObject iopad;
int *chip_xl, *chip_yb, *chip_xr, *chip_yt;

{
    octGenerator genTerm;
    octStatus genTermStatus;
    octObject terminal;
    struct octBox term_bbox;
    int x, y;

    /* initialize terminal generator */
    if (octInitGenContents(&iopad, OCT_TERM_MASK, &genTerm) != OCT_OK) {
        octError("IOPadsFindChipBox: cannot initialize io pad terminal generator.");
        return(BEAR_ERROR);
    }
    /* go through (generate) all I/O pad terminals */
    while ((genTermStatus = octGenerate(&genTerm, &terminal)) == OCT_OK) {
        /* get bounding box of terminal */
        if (regFindImpl(&terminal, &term_bbox) != REG_OK) {
            octError("IOPadsFindChipBox: cannot create terminal bounding box.");
            return(BEAR_ERROR);
        }
        /* determine center position of terminal */
        x = (term_bbox.lowerLeft.x + term_bbox.upperRight.x) / 2;
        y = (term_bbox.lowerLeft.y + term_bbox.upperRight.y) / 2;
/*
fprintf(stderr,"%d, %d: %d, %d %d, %d\n",x,y,*chip_xl,*chip_yb,*chip_xr,*chip_yt);
*/
        /* grow the bounding box of the chip if needed
           according to the terminal position */
        if (x < *chip_xl) {
            *chip_xl = x;
        }
        if (x > *chip_xr) {
            *chip_xr = x;
        }
        if (y < *chip_yb) {
            *chip_yb = y;
        }
        if (y > *chip_yt) {
            *chip_yt = y;
        }
    }
    /* check if an error has occured while generating */
    if (genTermStatus != OCT_GEN_DONE) {
        octError("IOPadsFindChipBox: io pad terminal generator failed.");
        return(BEAR_ERROR);
    }
    return(BEAR_OK);
}


/*
 * CoreCellsFindChipBox() - determines the bounding box of the chip
 *                          from the instances' bounding boxes
 */
static BearStatus
CoreCellsFindChipBox(instance, chip_xl, chip_yb, chip_xr, chip_yt)
octObject instance;
int *chip_xl, *chip_yb, *chip_xr, *chip_yt;

{
    struct octBox inst_bbox;

    /* get bounding box of instance */
    if (octBB(&instance, &inst_bbox) != OCT_OK) {
        octError("CoreCellsFindChipBox: cannot create instance bounding box.");
        return(BEAR_ERROR);
    }
/*
fprintf(stderr,"%d, %d %d, %d: %d, %d %d, %d\n",inst_bbox.lowerLeft.x,inst_bbox.lowerLeft.y,inst_bbox.upperRight.x,inst_bbox.upperRight.y,*chip_xl,*chip_yb,*chip_xr,*chip_yt);
*/
    /* grow the bounding box of the chip if needed
       according to the instance bounding box edges */
    if (inst_bbox.lowerLeft.x < *chip_xl) {
        *chip_xl = inst_bbox.lowerLeft.x;
    }
    if (inst_bbox.upperRight.x > *chip_xr) {
        *chip_xr = inst_bbox.upperRight.x;
    }
    if (inst_bbox.lowerLeft.y < *chip_yb) {
        *chip_yb = inst_bbox.lowerLeft.y;
    }
    if (inst_bbox.upperRight.y > *chip_yt) {
        *chip_yt = inst_bbox.upperRight.y;
    }
    return(BEAR_OK);
}


/*
 * FindPinPosition() - determines the position of a pin
 *                     along the boundary of the instance
 */
static BearStatus
FindPinPosition(inst_bbox, term_bbox, pinX, pinY)
struct octBox inst_bbox, term_bbox;
int *pinX, *pinY;

{
/*
fprintf(stderr,"inst: %d, %d, %d, %d\n",inst_bbox.lowerLeft.x,inst_bbox.lowerLeft.y,inst_bbox.upperRight.x,inst_bbox.upperRight.y);
fprintf(stderr,"term: %d, %d, %d, %d\n",term_bbox.lowerLeft.x,term_bbox.lowerLeft.y,term_bbox.upperRight.x,term_bbox.upperRight.y);
*/
    /* finds bounding box edge the pin is on within a 100 OCT units
       and determines the pin coordinates for MAC */
    if ((term_bbox.lowerLeft.x <= inst_bbox.lowerLeft.x + 50)
        && (term_bbox.lowerLeft.x >= inst_bbox.lowerLeft.x - 50)) {
        *pinX = inst_bbox.lowerLeft.x;
        *pinY = (term_bbox.lowerLeft.y + term_bbox.upperRight.y) / 2;
    } else if ((term_bbox.lowerLeft.y <= inst_bbox.lowerLeft.y + 50)
        && (term_bbox.lowerLeft.y >= inst_bbox.lowerLeft.y - 50)) {
        *pinX = (term_bbox.lowerLeft.x + term_bbox.upperRight.x) / 2;
        *pinY = inst_bbox.lowerLeft.y;
    } else if ((term_bbox.upperRight.x <= inst_bbox.upperRight.x + 50)
        && (term_bbox.upperRight.x >= inst_bbox.upperRight.x - 50)) {
        *pinX = inst_bbox.upperRight.x;
        *pinY = (term_bbox.lowerLeft.y + term_bbox.upperRight.y) / 2;
    } else if ((term_bbox.upperRight.y <= inst_bbox.upperRight.y + 50)
        && (term_bbox.upperRight.y >= inst_bbox.upperRight.y - 50)) {
        *pinX = (term_bbox.lowerLeft.x + term_bbox.upperRight.x) / 2;
        *pinY = inst_bbox.upperRight.y;
    } else {
        octError("FindPinPosition: terminal not along the edge of instance.");
        return(BEAR_ERROR);
    }
    return(BEAR_OK);
}


/*
 * Odi_ReadFromOct() - copies the facet so that the original data
 *                     will not get destroyed during manipulation,
 *                     produces two bags, CoreBag and IOPadBag,
 *                     generates MAC chip core cell and chidren cells
 *                     from the bags, and returns the core master pointer
 */
BearStatus
Odi_ReadFromOct(chip_cell, chip_view, cellMasterPtrPtr)
char        *chip_cell, *chip_view;
CellMaster  **cellMasterPtrPtr;
{
    char *outchip_cell;
    BearString chipName, chipMasterName;
    CellMaster *chipMasterPtr, *coreMasterPtr;
    CellInst *chipInstPtr, *coreInstPtr, *cellInstPtr;
    octObject connectorBag, instanceBag;
    octObject instance;
    octObject coreBag, iopadBag;
    int coreFlag, iopadFlag;
    octGenerator genInst, genNet, genTerm;
    octStatus genInstStatus, genNetStatus, genTermStatus;
    octObject instanceClass, placementClass;
    octObject instTerm, formalTerm;
    octObject instMaster;
    octObject  net, terminal;
    struct octBox inst_bbox, term_bbox;
    int pinX, pinY;
    octObject termMaster;
    octObject netType, peakCurrent;

    /* check if name for output chip cell does not exist */
    if (strlen(odi_outchip.contents.facet.cell) == 0) {
        /* create output chip cell name using chip cell
           name with ".macout" attached at the end */
        MALLOC(char *, outchip_cell, (strlen(chip_cell) + 8) * sizeof(char));
        sprintf(outchip_cell, "%s.macout", chip_cell);
        /* put name into odi_outchip */
        odi_outchip.contents.facet.cell = outchip_cell;
    } else {
        /* create output chip cell name using the name from odi_outchip */
        MALLOC(char *, outchip_cell, (strlen(odi_outchip.contents.facet.cell)
            + 1) * sizeof(char));
        sprintf(outchip_cell, "%s", odi_outchip.contents.facet.cell);
    }
    /* make a copy of the chip facet as the output chip facet */
    CopyFacets(chip_cell, chip_view, outchip_cell, chip_view);

    /* open output chip facet */
    odi_outchip.type = OCT_FACET;
    odi_outchip.contents.facet.view = chip_view;
    odi_outchip.contents.facet.version = OCT_CURRENT_VERSION;
    odi_outchip.contents.facet.facet = "contents";
    odi_outchip.contents.facet.mode = "a";
    if (octOpenFacet(&odi_outchip) < OCT_OK) {
        octError("Odi_ReadFromOct: cannot open odi_outchip.");
        return(BEAR_ERROR);
    }

/*
    chip_xl = IMAX;
    chip_yb = IMAX;
    chip_xr = IMIN;
    chip_yt = IMIN;
*/
    /* get bounding box of the output chip */
    if (octBB(&odi_outchip,&inst_bbox) != OCT_OK) {
        octError("Odi_ReadFromOct: cannot create facet bounding box.");
        return(BEAR_ERROR);
    }
    /* initialize chip bounding box */
    chip_xl = inst_bbox.lowerLeft.x;
    chip_yb = inst_bbox.lowerLeft.y;
    chip_xr = inst_bbox.upperRight.x;
    chip_yt = inst_bbox.upperRight.y;
/*
fprintf(stderr,"%d, %d %d, %d\n",chip_xl,chip_yb,chip_xr,chip_yt);
*/

    /* initialize bags flags */
    coreFlag = 0;
    iopadFlag = 0;

    /* get or create CONNECTORS bag */
    connectorBag.type = OCT_BAG;
    connectorBag.contents.bag.name = "CONNECTORS";
    if (octGetOrCreate(&odi_outchip, &connectorBag) != OCT_OK) {
        octError("Odi_ReadFromOct: connector bag error.");
        return(BEAR_ERROR);
    }
    /* get INSTANCES bag */
    instanceBag.type = OCT_BAG;
    instanceBag.contents.bag.name = "INSTANCES";
    if (octGetByName(&odi_outchip, &instanceBag) == OCT_NOT_FOUND) {
        octError("Odi_ReadFromOct: no instance bag.");
        return(BEAR_ERROR);
    }
    /* get CORECELLS bag */
    coreBag.type = OCT_BAG;
    coreBag.contents.bag.name = "CORECELLS";
    if (octGetByName(&odi_outchip, &coreBag) == OCT_NOT_FOUND) {
        /* CORECELLS bag does not exist */
        /* create CORECELLS bag */
        if (octCreate(&odi_outchip, &coreBag) != OCT_OK) {
            octError("Odi_ReadFromOct: core bag error.");
            return(BEAR_ERROR);
        }
    } else {
        /* CORECELLS bag exists */
        /* set core flag */
        coreFlag = 1;
    }
    /* get IOPADS bag */
    iopadBag.type = OCT_BAG;
    iopadBag.contents.bag.name = "IOPADS";
    if (octGetByName(&odi_outchip, &iopadBag) == OCT_NOT_FOUND) {
        /* IOPADS bag does not exist */
        /* create IOPADS bag */
        if (octCreate(&odi_outchip, &iopadBag) != OCT_OK) {
            octError("Odi_ReadFromOct: io pad bag error.");
            return(BEAR_ERROR);
        }
    } else {
        /* IOPADS bag exist */
        /* set I/O pad flag */
        iopadFlag = 1;
    }

    /* check if any of the core and I/O pad flags are not set */
    if (!(coreFlag && iopadFlag)) {
        /* setup INSTANCECLASS and PLACEMENT_CLASS properties */
        instanceClass.type = OCT_PROP;
        instanceClass.contents.prop.name = "INSTANCECLASS";
        instanceClass.contents.prop.type = OCT_STRING;
        placementClass.type = OCT_PROP;
        placementClass.contents.prop.name = "PLACEMENT_CLASS";
        placementClass.contents.prop.type = OCT_STRING;

        /* initialize instance generator */
        if (octInitGenContents(&instanceBag, OCT_INSTANCE_MASK, &genInst)
            != OCT_OK) {
            octError("Odi_ReadFromOct: cannot initialize instance bag instance generator.");
            return(BEAR_ERROR);
        }
        /* go through (generate) all instances in the instance bag */
        while ((genInstStatus = octGenerate(&genInst, &instance)) == OCT_OK) {
            /* open instance master facet */
            instMaster.type = OCT_FACET;
            instMaster.contents.facet.cell
             = instance.contents.instance.master;
            instMaster.contents.facet.view
             = instance.contents.instance.view;
            instMaster.contents.facet.version = OCT_CURRENT_VERSION;
            instMaster.contents.facet.facet = "interface";
            instMaster.contents.facet.mode = "r";
            if (octOpenFacet(&instMaster) < OCT_OK) {
                octError("Odi_ReadFromOct: cannot open instance master.");
                return(BEAR_ERROR);
            }

            /* check if instance class of instance is PAD */
            if (octGetByName(&instance, &instanceClass) == OCT_OK &&
                strcmp(instanceClass.contents.prop.value.string, "PAD") == 0) {
                /* instance is an I/O pad */
                /* put instance in I/O pad bag */
                octAttachOnce(&iopadBag, &instance);
                /* use instance in finding the chip bounding box */
                IOPadsFindChipBox(instance, &chip_xl, &chip_yb, &chip_xr,
                    &chip_yt);
            /* check if placement class of instance master is PAD */
            } else if (octGetByName(&instMaster, &placementClass) == OCT_OK &&
                strcmp(placementClass.contents.prop.value.string, "PAD") == 0) {
                /* instance is an I/O pad */
                /* put instance in I/O pad bag */
                octAttachOnce(&iopadBag, &instance);
                /* use instance in finding the chip bounding box */
                IOPadsFindChipBox(instance, &chip_xl, &chip_yb, &chip_xr,
                    &chip_yt);
            /* check if placement class of instance is PAD */
            } else if (octGetByName(&instance, &placementClass) == OCT_OK &&
                strcmp(placementClass.contents.prop.value.string, "PAD") == 0) {
                /* instance is an I/O pad */
                /* put instance in I/O pad bag */
                octAttachOnce(&iopadBag, &instance);
                /* use instance in finding the chip bounding box */
                IOPadsFindChipBox(instance, &chip_xl, &chip_yb, &chip_xr,
                    &chip_yt);
            /* check if instance class of instance master is PAD */
            } else if (octGetByName(&instMaster, &instanceClass) == OCT_OK &&
                /* instance is an I/O pad */
                /* put instance in I/O pad bag */
                strcmp(instanceClass.contents.prop.value.string, "PAD") == 0) {
                octAttachOnce(&iopadBag, &instance);
                /* use instance in finding the chip bounding box */
                IOPadsFindChipBox(instance, &chip_xl, &chip_yb, &chip_xr,
                    &chip_yt);
            } else {
                /* otherwise, instance is a (core) cell */
                /* put instance in core bag */
                octAttachOnce(&coreBag, &instance);
                /* use instance in finding the chip bounding box */
                CoreCellsFindChipBox(instance, &chip_xl, &chip_yb, &chip_xr,
                    &chip_yt);
            }
/*
fprintf(stderr,"%s, %s\n",instanceClass.contents.prop.value.string,placementClass.contents.prop.value.string);
*/
        }
        /* check if an error occured while generating instances */
        if (genInstStatus != OCT_GEN_DONE) {
            octError("Odi_ReadFromOct: instance bag instance generator failed.");
            return(BEAR_ERROR);
        }
    
        /* initialize instance generator */
        if (octInitGenContents(&connectorBag, OCT_INSTANCE_MASK, &genInst)
            != OCT_OK) {
            octError("Odi_ReadFromOct: cannot initialize connector bag instance generator.");
            return(BEAR_ERROR);
        }
        /* go through (generate) all instances in the connector bag */
        while ((genInstStatus = octGenerate(&genInst, &instance)) == OCT_OK) {
            /* get instance terminal from instance */
            if (octGenFirstContent(&instance, OCT_TERM_MASK, &instTerm)
                == OCT_OK) {
                /* get formal terminal containing instance terminal */
                if (octGenFirstContainer(&instTerm, OCT_TERM_MASK, &formalTerm)
                    == OCT_OK) {
                    /* check if formal terminal */
                    if (formalTerm.contents.term.instanceId == 0) {
                        /* put instance in I/O pad bag */
                        octAttachOnce(&iopadBag, &instance);
                        /* use instance in finding the chip bounding box */
                        IOPadsFindChipBox(instance,
                            &chip_xl, &chip_yb, &chip_xr, &chip_yt);
                    } else {
                        octError("Odi_ReadFromOct: connector terminal is not a formal terminal.");
                        return(BEAR_ERROR);
                    }
                }
            } else {
                octError("Odi_ReadFromOct: connector has no actual terminal.");
                return(BEAR_ERROR);
            }
        }
        /* check if an error occured while generating instances */
        if (genInstStatus != OCT_GEN_DONE) {
            octError("Odi_ReadFromOct: connector bag instance generator failed.");
            return(BEAR_ERROR);
        }
    } else {
        /* otherwise, both the core bag and I/O pad bag exist */
        /* initialize instance generator */
        if (octInitGenContents(&iopadBag, OCT_INSTANCE_MASK, &genInst)
            != OCT_OK) {
            octError("Odi_ReadFromOct: cannot initialize io pad bag instance generator for find chip box.");
            return(BEAR_ERROR);
        }
        /* go through (generate) all instances in I/O pad bag */
        while ((genInstStatus = octGenerate(&genInst, &instance)) == OCT_OK) {
            /* use instance (pad) in finding the chip bounding box */
            IOPadsFindChipBox(instance, &chip_xl, &chip_yb, &chip_xr, &chip_yt);
        }
        /* check if an error occured while generating instances */
        if (genInstStatus != OCT_GEN_DONE) {
            octError("Odi_ReadFromOct: io pad bag instance generator for find chip box failed.");
            return(BEAR_ERROR);
        }
        /* initialize instance generator */
        if (octInitGenContents(&coreBag, OCT_INSTANCE_MASK, &genInst)
            != OCT_OK) {
            octError("Odi_ReadFromOct: cannot initialize core bag instance generator for find chip box.");
            return(BEAR_ERROR);
        }
        /* go through (generate) all instances in core bag */
        while ((genInstStatus = octGenerate(&genInst, &instance)) == OCT_OK) {
            /* use instance (cell) in finding the chip bounding box */
            CoreCellsFindChipBox(instance, &chip_xl, &chip_yb, &chip_xr, &chip_yt);
        }
        /* check if an error occured while generating instances */
        if (genInstStatus != OCT_GEN_DONE) {
            octError("Odi_ReadFromOct: core bag instance generator for find chip box failed.");
            return(BEAR_ERROR);
        }
    }

    /* setup net type and peak current properties */
    netType.type = OCT_PROP;
    netType.contents.prop.name = "NETTYPE";
    netType.contents.prop.type = OCT_STRING;
    peakCurrent.type = OCT_PROP;
    peakCurrent.contents.prop.name = "PEAKCURRENT";
    peakCurrent.contents.prop.type = OCT_REAL;

    /*
     * Create the top level cell, the chip which contains 
     * a core cell and I/O pads. 
     */
    MALLOC(char *, chipName, (strlen(chip_cell) + 1) * sizeof(char));
    strcpy(chipName, chip_cell);
    MALLOC(char *, chipMasterName, (strlen(chip_cell) + 8) * sizeof(char));
    sprintf(chipMasterName, "%s_master", chipName);
    (void) Mac_CreateCell(NIL(CellMaster),
        chipMasterName, chipName, &chipInstPtr);

    /* Create the core cell */
    chipMasterPtr = Mac_GetMasterOfCell(chipInstPtr);
    BuildMacCell(chipMasterPtr, &coreInstPtr, outchip_cell, 0, 0,
        (chip_xr - chip_xl) / OCT_PER_LAMBDA,
        (chip_yt - chip_yb) / OCT_PER_LAMBDA);
    /* set bounding box of core cell */
    inst_bbox.lowerLeft.x = chip_xl;
    inst_bbox.lowerLeft.y = chip_yb;
    inst_bbox.upperRight.x = chip_xr;
    inst_bbox.upperRight.y = chip_yt;
    /* initialize instance generator */
    if (octInitGenContents(&iopadBag, OCT_INSTANCE_MASK, &genInst)
        != OCT_OK) {
        octError("Odi_ReadFromOct: cannot initialize io pad bag instance generator.");
        return(BEAR_ERROR);
    }
    /* go through (generate) all instances in I/O pads bag */
    while ((genInstStatus = octGenerate(&genInst, &instance)) == OCT_OK) {
        /* get bounding box of instance */
        if (octBB(&instance, &term_bbox) != OCT_OK) {
            octError("Odi_ReadFromOct: cannot create instance bounding box.");
            return(BEAR_ERROR);
        }
/*
        instMaster.type = OCT_FACET;
        instMaster.contents.facet.version = OCT_CURRENT_VERSION;
        instMaster.contents.facet.facet = "interface";
        instMaster.contents.facet.mode = "r";
        instMaster.contents.facet.cell = instance.contents.instance.master;
        instMaster.contents.facet.view = instance.contents.instance.view;
*/
/*
        if (octOpenMaster(&instance, &instMaster) < OCT_OK) {
            octError("Odi_ReadFromOct: cannot open instance master.");
            return(BEAR_ERROR);
        }
*/
/*
        if (octOpenFacet(&instMaster) < OCT_OK) {
            octError("Odi_ReadFromOct: cannot open instance master.");
            return(BEAR_ERROR);
        }
*/
        /* find the I/O pad position on the core cell boundary */
        FindPinPosition(inst_bbox, term_bbox, &pinX, &pinY);
/*
        if (octInitGenContents(&instance, OCT_TERM_MASK, &genTerm) != OCT_OK) {
            octError("Odi_ReadFromOct: cannot initialize io pad terminal generator.");
            return(BEAR_ERROR);
        }
        while ((genTermStatus = octGenerate(&genTerm, &terminal)) == OCT_OK) {
*/
/*
            if (regFindImpl(&terminal, &term_bbox) != REG_OK) {
                octError("Odi_ReadFromOct: cannot create terminal bounding box.");
                return(BEAR_ERROR);
            }
            pinX = (term_bbox.lowerLeft.x + term_bbox.upperRight.x) / 2;
            pinY = (term_bbox.lowerLeft.y + term_bbox.upperRight.y) / 2;
*/
        /* get instance terminal from instance */
        if (octGenFirstContent(&instance, OCT_TERM_MASK, &instTerm) == OCT_OK) {
            /* get formal terminal containing instance terminal */
            if (octGenFirstContainer(&instTerm, OCT_TERM_MASK, &formalTerm)
                == OCT_OK) {
                /* initialize net generator */
                if (octInitGenContainers(&formalTerm, OCT_NET_MASK, &genNet)
                    != OCT_OK) {
                    octError("Odi_ReadFromOct: cannot initialize terminal net generator.");
                    return(BEAR_ERROR);
                }
            } else {
                /* initialize net generator */
                if (octInitGenContainers(&instTerm, OCT_NET_MASK, &genNet)
                    != OCT_OK) {
                    octError("Odi_ReadFromOct: cannot initialize terminal net generator.");
                    return(BEAR_ERROR);
                }
            }
        }
/*
fprintf(stderr,"IOP: %d, %d.\n",pinX,pinY);
fprintf(stderr,"Term %s\n",instance.contents.instance.name);
*/
        /* go through (generate) all nets connected to the terminal */
        while ((genNetStatus = octGenerate(&genNet, &net)) == OCT_OK) {
/*
fprintf(stderr,"Net %s\n",net.contents.net.name);
*/
            /* get net type and check if it is SIGNAL */
            if ((octGetByName(&net, &netType) != OCT_OK) ||
                (strcmp(netType.contents.prop.value.string, "SIGNAL")
                == 0)) {
                /* create (signal) I/O pad */
                BuildMacPin(chipMasterPtr, coreInstPtr,
                    instance.contents.instance.name, net.contents.net.name,
                    (pinX - chip_xl) / OCT_PER_LAMBDA,
                    (pinY - chip_yb) / OCT_PER_LAMBDA,
                    "SIGNAL", (double) 0);
            } else {
/*
if (octInitGenContents(&net, OCT_TERM_MASK, &genTerm) != OCT_OK) {
    octError("Odi_ReadFromOct: cannot initialize io pad terminal generator.");
    return(BEAR_ERROR);
}
if ((genTermStatus = octGenerate(&genTerm, &terminal)) == OCT_OK) {
                    termMaster.type = OCT_TERM;
                    termMaster.contents.term.name = instTerm.contents.term.name;
                    if (octGetByName(&instMaster, &termMaster) != OCT_OK) {
                        octError("Odi_ReadFromOct: cannot get terminal from instance master.");
                        return(BEAR_ERROR);
                    } else if (octGetByName(&termMaster, &peakCurrent) != OCT_OK) {
                        octError("Odi_ReadFromOct: cannot get peak current.");
                        return(BEAR_ERROR);
                    }
*/
                /* create (others) I/O pad */
                BuildMacPin(chipMasterPtr, coreInstPtr,
                    instance.contents.instance.name, net.contents.net.name,
                    (pinX - chip_xl) / OCT_PER_LAMBDA,
                    (pinY - chip_yb) / OCT_PER_LAMBDA,
                    netType.contents.prop.value.string,
                    (double) 0);
/*
                    peakCurrent.contents.prop.value.real);
}
*/
            }
        }
        /* check if an error occured while generating nets */
        if (genNetStatus != OCT_GEN_DONE) {
            octError("Odi_ReadFromOct: terminal net generator failed.");
            return(BEAR_ERROR);
        }
/*
        }
        if (genTermStatus != OCT_GEN_DONE) {
            octError("Odi_ReadFromOct: io pad terminal generator failed.");
            return(BEAR_ERROR);
        }
*/
    }
    /* check if an error occured while generating instances */
    if (genInstStatus != OCT_GEN_DONE) {
        octError("Odi_ReadFromOct: io pad bag instance generator failed.");
        return(BEAR_ERROR);
    }

    /* Create the child cells of the core cell */
    /* setup core master */
    coreMasterPtr = Mac_GetMasterOfCell(coreInstPtr);
    coreMasterPtr->contents.netList = chipMasterPtr->contents.netList;
    /* initialize instance generator */
    if (octInitGenContents(&coreBag, OCT_INSTANCE_MASK, &genInst)
        != OCT_OK) {
        octError("Odi_ReadFromOct: cannot initialize core bag instance generator.");
        return(BEAR_ERROR);
    }
    /* go through (generate) all instances in core bag */
    while ((genInstStatus = octGenerate(&genInst, &instance)) == OCT_OK) {
        /* open instance master facet */
        instMaster.type = OCT_FACET;
        instMaster.contents.facet.version = OCT_CURRENT_VERSION;
        instMaster.contents.facet.facet = "interface";
        instMaster.contents.facet.mode = "r";
        instMaster.contents.facet.cell = instance.contents.instance.master;
        instMaster.contents.facet.view = instance.contents.instance.view;
/*
        if (octOpenMaster(&instance, &instMaster) < OCT_OK) {
            octError("Odi_ReadFromOct: cannot open instance master.");
            return(BEAR_ERROR);
        }
*/
        if (octOpenFacet(&instMaster) < OCT_OK) {
            octError("Odi_ReadFromOct: cannot open instance master.");
            return(BEAR_ERROR);
        }
        /* get bounding box of instance */
        if (octBB(&instance, &inst_bbox) != OCT_OK) {
            octError("Odi_ReadFromOct: cannot create instance bounding box.");
            return(BEAR_ERROR);
        }
        /* create (child) cells */
        BuildMacCell(coreMasterPtr, &cellInstPtr,
            instance.contents.instance.name,
            (inst_bbox.lowerLeft.x - chip_xl) / OCT_PER_LAMBDA,
            (inst_bbox.lowerLeft.y - chip_yb) / OCT_PER_LAMBDA,
            (inst_bbox.upperRight.x - chip_xl) / OCT_PER_LAMBDA,
            (inst_bbox.upperRight.y - chip_yb) / OCT_PER_LAMBDA);
        /* initialize terminal generator */
        if (octInitGenContents(&instance, OCT_TERM_MASK, &genTerm) != OCT_OK) {
            octError("Odi_ReadFromOct: cannot initialize io pad terminal generator.");
            return(BEAR_ERROR);
        }
        /* go through (generate) all terminals of the instance */
        while ((genTermStatus = octGenerate(&genTerm, &terminal)) == OCT_OK) {
            /* get bounding box of terminal */
            if (regFindImpl(&terminal, &term_bbox) != REG_OK) {
                octError("Odi_ReadFromOct: cannot create terminal bounding box.");
                return(BEAR_ERROR);
            }
            /* find the pin position on the cell boundary */
            FindPinPosition(inst_bbox, term_bbox, &pinX, &pinY);
            /* initialize net generator */
            if (octInitGenContainers(&terminal, OCT_NET_MASK, &genNet)
                != OCT_OK) {
                octError("Odi_ReadFromOct: cannot initialize terminal net generator.");
                return(BEAR_ERROR);
            }
/*
fprintf(stderr,"Term %s\n",terminal.contents.term.name);
*/
            /* go through (generate) all nets connected to the terminal */
            while ((genNetStatus = octGenerate(&genNet, &net)) == OCT_OK) {
/*
fprintf(stderr,"Net %s\n",net.contents.net.name);
*/
                /* get net type and check if it is SIGNAL */
                if ((octGetByName(&net, &netType) != OCT_OK) ||
                    (strcmp(netType.contents.prop.value.string, "SIGNAL")
                    == 0)) {
                    /* create (signal) pin */
                    BuildMacPin(coreMasterPtr, cellInstPtr,
                        terminal.contents.term.name, net.contents.net.name,
                        (pinX - inst_bbox.lowerLeft.x) / OCT_PER_LAMBDA,
                        (pinY - inst_bbox.lowerLeft.y) / OCT_PER_LAMBDA,
                        "SIGNAL", (double) 0);
                } else {
                    /* get term master */
                    termMaster.type = OCT_TERM;
                    termMaster.contents.term.name = terminal.contents.term.name;
                    if (octGetByName(&instMaster, &termMaster) != OCT_OK) {
                        octError("Odi_ReadFromOct: cannot get terminal from instance master.");
                        return(BEAR_ERROR);
                    }
                    /* get peak current */
                    if (octGetByName(&termMaster, &peakCurrent) != OCT_OK) {
                        octError("Odi_ReadFromOct: cannot get peak current.");
                        return(BEAR_ERROR);
                    }
                    /* create (others) pin */
                    BuildMacPin(coreMasterPtr, cellInstPtr,
                        terminal.contents.term.name, net.contents.net.name,
                        (pinX - inst_bbox.lowerLeft.x) / OCT_PER_LAMBDA,
                        (pinY - inst_bbox.lowerLeft.y) / OCT_PER_LAMBDA,
                        netType.contents.prop.value.string,
                        peakCurrent.contents.prop.value.real);
                }
            }
            /* check if an error occured while generating nets */
            if (genNetStatus != OCT_GEN_DONE) {
                octError("Odi_ReadFromOct: terminal net generator failed.");
                return(BEAR_ERROR);
            }
        }
        /* check if an error occured while generating terminals */
        if (genTermStatus != OCT_GEN_DONE) {
            octError("Odi_ReadFromOct: io pad terminal generator failed.");
            return(BEAR_ERROR);
        }
    }
    /* check if an error occured while generating instances */
    if (genInstStatus != OCT_GEN_DONE) {
        octError("Odi_ReadFromOct: core bag instance generator failed.");
        return(BEAR_ERROR);
    }

    /* store bounding box of the cell into MAC */
    (void) Mac_PutBBoxOfCell(Mac_GetBBoxOfCell(coreInstPtr),chipInstPtr);

    /* set cell master */
    *cellMasterPtrPtr = coreMasterPtr;

    /* free unneeded memory */
    FREE((char *) chipName);
    FREE((char *) chipMasterName);
    FREE((char *) outchip_cell);

    /*
     * Convert all of the pins on the boundary of the chip to
     * border pins.
     */
    if (Mac_ItrPinOfCellInst(coreInstPtr, Mac_ConvertToBorderPin,
        (ClientData) NULL) == BEAR_STOP_SEARCH) {
        octError("Odi_ReadFromOct: cannot convert border pins.");
    }

    return(BEAR_OK);
}


/* function moved
static BearStatus ConvertToBorderPin();


static BearStatus
ConvertToBorderPin(pinInstPtr, clientData)
PinInst     *pinInstPtr;
ClientData  clientData;
{
    Mac_PutKindOfPin((PinKind) MAC_BORDER_PIN, pinInstPtr);
    switch (Mac_GetDirectionOfPin(pinInstPtr)) {
        case NORTH_PIN:
            Mac_PutDirectionOfPin(SOUTH_PIN,pinInstPtr);
            break;
        case SOUTH_PIN:
            Mac_PutDirectionOfPin(NORTH_PIN,pinInstPtr);
            break;
        case WEST_PIN:
            Mac_PutDirectionOfPin(EAST_PIN,pinInstPtr);
            break;
        case EAST_PIN:
            Mac_PutDirectionOfPin(WEST_PIN,pinInstPtr);
            break;
        default:
            Bear_PrintFatalErr("ConvertToBorderPin: illegal pin direction.");
            break;
    }
    return(BEAR_CONTINUE_SEARCH);
}
*/
