
/* writeBackToOct.c --
 *         
 *     Denis Lee
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "oct.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "octInterface.h"
#include "octInterfaceInt.h"


/* ---------- Local Procedures ---------- */
static BearStatus WriteBackCellsToOct();


/*
 * TransMagicToOct() - convert Magic transforms to Oct tranforms
 */
octTransformType
TransMagicToOct(trans)
Transform trans;
{
    if (trans.t_a == 0) {
        if (trans.t_d < 0) {
            if (trans.t_b < 0) {
                return(OCT_MX_ROT90);
            } else {
                return(OCT_ROT270);
            }
        } else {
            if (trans.t_b < 0) {
                return(OCT_ROT90);
            } else {
                return(OCT_MY_ROT90);
            }
        }
    } else if (trans.t_a < 0) {
        if (trans.t_e < 0) {
            return(OCT_ROT180);
        } else {
            return(OCT_MIRROR_X);
        }
    } else {
        if (trans.t_e < 0) {
            return(OCT_MIRROR_Y);
        }
    }

    return(OCT_NO_TRANSFORM);
}


/*
 * WriteCellsToOct() - write back each cell from MAC to OCT
 */
static BearStatus
WriteBackCellsToOct(cellInstPtr, clientData)
CellInst    *cellInstPtr;
ClientData  *clientData;
{
    octObject   instanceBag, instance;
    struct      octBox inst_bbox;
    int         deltaX, deltaY;
/*
    Transform trans;
*/
    struct octTransform octTrans;

    /* get INSTANCES bag */
    instanceBag.type = OCT_BAG;
    instanceBag.contents.bag.name = "INSTANCES";
    if (octGetByName(&odi_outchip, &instanceBag) == OCT_NOT_FOUND) {
        octError("WriteCellsToOct: no instance bag.");
        return(BEAR_ERROR);
    }
    /* get instance of the cell and its bounding box */
    instance.type = OCT_INSTANCE;
    instance.contents.instance.name = cellInstPtr->name;
    if (octGetByName(&instanceBag, &instance) == OCT_NOT_FOUND) {
        octError("WriteCellsToOct: no instance by that name.");
        return(BEAR_ERROR);
    }
    if (octBB(&instance, &inst_bbox) != OCT_OK) {
        octError("WriteCellsToOct: cannot get instance bounding box.");
        return(BEAR_ERROR);
    }
    /* determine positional and rotational changes of the instance
       and adjust the instance appropriately */
    /* determine center of instance */
    deltaX = (inst_bbox.lowerLeft.x + inst_bbox.upperRight.x) / (double) 2;
    deltaY = (inst_bbox.lowerLeft.y + inst_bbox.upperRight.y) / (double) 2;
    /* move center of instance to the origin */
    instance.contents.instance.transform.translation.x -= deltaX;
    instance.contents.instance.transform.translation.y -= deltaY;
/*
    instance.contents.instance.transform.translation.x -= deltaX;
    instance.contents.instance.transform.translation.y -= deltaY;
    switch (instance.contents.instance.transform.transformType) {
        case OCT_NO_TRANSFORM:
        GeoTransTrans(&GeoIdentityTransform,&cellInstPtr->transform,&trans);
    break;
        case OCT_MIRROR_X:
        GeoTransTrans(&GeoSidewaysTransform,&cellInstPtr->transform,&trans);
    break;
        case OCT_MIRROR_Y:
        GeoTransTrans(&GeoUpsideDownTransform,&cellInstPtr->transform,&trans);
    break;
        case OCT_ROT90:
        GeoTransTrans(&Geo90Transform,&cellInstPtr->transform,&trans);
    break;
        case OCT_ROT180:
        GeoTransTrans(&Geo180Transform,&cellInstPtr->transform,&trans);
    break;
        case OCT_ROT270:
        GeoTransTrans(&Geo270Transform,&cellInstPtr->transform,&trans);
        break;
        case OCT_MX_ROT90:
            GeoTransTrans(&GeoRef135Transform,&cellInstPtr->transform,&trans);
    break;
        case OCT_MY_ROT90:
        GeoTransTrans(&GeoRef45Transform,&cellInstPtr->transform,&trans);
    break;
        default:
    break;
    }
    instance.contents.instance.transform.transformType = TransMagicToOct(trans);
    deltaX += cellInstPtr->transform.t_c - (inst_bbox.lowerLeft.x - chip_xl);
    deltaY += cellInstPtr->transform.t_f - (inst_bbox.lowerLeft.y - chip_yb);
    instance.contents.instance.transform.translation.x += deltaX;
    instance.contents.instance.transform.translation.y += deltaY;
*/
    /* add rotational changes to instance */
    octTrans.transformType = TransMagicToOct(cellInstPtr->transform);
    octTrans.translation.x = octTrans.translation.y = 0;
    octTransformGeo(&instance,&octTrans);

    /* add positional changes to instance
       including the movement to the origin */
    deltaX += cellInstPtr->origin.p_x * OCT_PER_LAMBDA - (inst_bbox.lowerLeft.x
        - chip_xl);
    deltaY += cellInstPtr->origin.p_y * OCT_PER_LAMBDA - (inst_bbox.lowerLeft.y
        - chip_yb);
    instance.contents.instance.transform.translation.x += deltaX;
    instance.contents.instance.transform.translation.y += deltaY;
    /* update instance with these changes */
    if (octModify(&instance) != OCT_OK) {
        octError("WriteCellsToOct: cannot change instance.");
        return(BEAR_ERROR);
    }
    return(BEAR_CONTINUE_SEARCH);
}


/*
 * Odi_WriteBackToOct() - iterate through all the children of the
 *                        chip core master and call WriteCellsToOct()
 *                        for each child to write back all the cells
 */
BearStatus
Odi_WriteBackToOct(cellMasterPtr)
CellMaster *cellMasterPtr;
{
    /* go through all the cells of the chip (top level cell) */
    (void) Mac_ItrChildrenOfCell(cellMasterPtr, WriteBackCellsToOct,
        (ClientData) NULL);
    /* save modifications and close facet */
/*
    if (octFlushFacet(&odi_outchip) < OCT_OK) {
*/
    if (octCloseFacet(&odi_outchip) < OCT_OK) {
        octError("Odi_WriteBackToOct: cannot close odi_outchip.");
        return(BEAR_ERROR);
    }
    /* reinitialize chip so only can write back once for every read */
    odi_outchip.contents.facet.cell = NIL(char);
    return(BEAR_OK);
}


