
/* writeRouting.c --
 *      
 *     Denis Lee
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifdef USE_DR_CODE

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#undef ALLOC
#undef REALLOC
#undef FREE
#include <sys/stat.h>
#include "malloc.h"
#include "oct.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "routeDB.h"
#include "octInterface.h"
#include "oh.h"
#include "octInterfaceInt.h"


/* names of layers for vias */
static char *vialayersname[2][2] = {
    {"m1m1","com2"},
    {"com2","m2m2"}
};
/* distinct part of via names for each chip (or top level cell) */
static unsigned long int vianumber = 0;


/* ---------- Local Procedures ---------- */
static BearStatus   WriteRoutingOfNetsToOct();

static BearStatus   InitializeRRootsSegmentsJunctions();
static BearStatus   InitializeSegmentsJunctionsClientData();
static BearStatus   WriteRoutingOfRRootsToOct();
static BearStatus   WriteSegmentsJunctionsConnectionsToOct();
static BearStatus   FreeRRootsSegmentsJunctions();
static BearStatus   FreeSegmentsJunctionsClientData();

static int          viaDirDoesNotExist();


/* ---------- Local Global Variables ---------- */
static char *odiOctHomePtr;


/*
 * viaDirDoesNotExist() - checks if via directory with
 *                        that pathname exists or not
 */
static int
viaDirDoesNotExist(viaDirName)
char *viaDirName;
{
/*
    octObject facet;
    struct octFacetInfo info;

    facet.type = OCT_FACET;
    facet.contents.facet.cell = vianame;
    facet.contents.facet.view = "physical";
    facet.contents.facet.facet = "contents";
    facet.contents.facet.version = OCT_CURRENT_VERSION;
    facet.contents.facet.mode = "r";
    return ((octGetFacetInfo(&facet,&info) >= OCT_OK));
*/
    struct stat buf;

    return (stat(viaDirName,&buf));
}


/*
 * WriteSegmentsJunctionsConnectionsToOct() - writes segments, junctions, and
 *                                            their connections from MAC to OCT
 * Results:
 *   BEAR_CONTINUE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, otherwise.
 */

static BearStatus
WriteSegmentsJunctionsConnectionsToOct(rSegPtr, net)
rseg        *rSegPtr;
octObject   *net;
{
    static mkdirvias = TRUE /*1*/;
    static int32 number2 = 2;

    octObject *segment;
    struct octPoint points[2];
    octObject chip;
    octObject bag;
    octObject layer;
    Point coordinate;
    int end0layer1, end0layer2, end1layer1, end1layer2;
    term *termPtr;
    octObject terminal;
    term *termPtr2;
    struct octPoint points2[2];
    octObject *segmentPtr2;
    int end0width, end0height, end1width, end1height;
    rseg *rSegPtr2;
    octObject *end0viaPtr, *end1viaPtr;
    Point coordinate2;
    char vianame[20], fullvianame[80], command[40];
/*
wire
*/
/*
octObject *via;
contact, connector
*/
/*
name or mark segment
clientData
mask level {1,2,12(either?)}- layer
*/

/*
if (vianumber == 80) {
fprintf(stderr,"VIA %ld.\n",vianumber);
}
*/
    /* get chip facet using net information */
    if (octGenFirstContainer(net, OCT_FACET_MASK, &chip) != OCT_OK) {
        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot generate first FACET container of net.");
/*
, %s.", net->contents.net.name);
*/
        return (BEAR_STOP_SEARCH);
    }

    /* get CONNECTORS bag from chip */
    bag.type = OCT_BAG;
    bag.contents.bag.name = "CONNECTORS";
    if (octGetByName(&chip, &bag) == OCT_NOT_FOUND) {
        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get CONNECTORS bag.");
        return (BEAR_STOP_SEARCH);
    }

    /* initialize layer and via at both ends, 0 and 1, of segment */
    end0viaPtr = end1viaPtr = NULL;
    if (rSegPtr->msklvl == 12) {
        end0layer1 = end1layer1 = end0layer2 = end1layer2 = 1;
    } else {
        end0layer1 = end1layer1 = end0layer2 = end1layer2 = rSegPtr->msklvl - 1;
    }
    layer.type = OCT_LAYER;
    /* check if segment does not exist */
    if (rSegPtr->clientData == NULL) {
/*
fprintf(stderr,"create segment\n");
*/
        /* create segment */
        MALLOC(octObject *, segment, sizeof(octObject));
        segment->type = OCT_PATH;
        segment->contents.path.width = 
            (rSegPtr->hwidth + rSegPtr->hwidth) * OCT_PER_LAMBDA;
        if (octCreate(net, segment) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create segment.");
            return (BEAR_STOP_SEARCH);
        }

        /* put segement on the proper layer */
        if (rSegPtr->msklvl == 1) {
            /* metal 1 layer */
            layer.contents.layer.name = "MET1";
        } else if ((rSegPtr->msklvl == 2) || (rSegPtr->msklvl == 12)) {
            /* metal 2 layer */
            layer.contents.layer.name = "MET2";
        } else {
            /* otherwise error; unknown layer */
            Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : unknown layer.");
            return (BEAR_STOP_SEARCH);
        }
        if (octGetByName(&chip,&layer) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() :: cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
            return (BEAR_STOP_SEARCH);
        }
        if (octAttach(&layer, segment) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
            return (BEAR_STOP_SEARCH);
        }
     } else {
        /* get and use segment */
        segment = (octObject *) rSegPtr->clientData;
     }

/*
create other segments? mark segments, terms?
*/
    /* check if end 0 of segment is a (MAC/BBL) terminal */
    if (rSegPtr->t0fsr) {
/*
fprintf(stderr,"terminal at end 0\n");
term, pos. rel.
which if first? have same problem with term to pin in l channels?
*/
        /* check if module pointer has an ancestor or parent */
        if (rSegPtr->j0sr.rt->mp->ansmp == NULL) {
            /* regular channel */
            termPtr = rSegPtr->j0sr.rt;
        } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
            termPtr = Mac_LinkTermToTerm(rSegPtr->j0sr.rt,
                                         rSegPtr->j0sr.rt->mp->ansmp);
/*
fprintf(stderr,"    find other terminal\n");
*/
        }
        /* check if the terminal has a generic pin pointer
           and if so, check its pin type */
        if ((GenericPin *)termPtr->genericPinPtr != NULL
            && ((GenericPin *)termPtr->genericPinPtr)->pinType == PIN_INST) {
/*
fprintf(stderr,"    pin terminal\n");
*/
            /* the segment is connected to a pin instance */
            /* get and convert coordinates using OCT_PER_LAMBDA */
            Mac_GetAbsPosOfPin(((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
            points[0].x = coordinate.p_x * OCT_PER_LAMBDA;
            points[0].y = coordinate.p_y * OCT_PER_LAMBDA;
            
            /* get and attach the terminal (pin) to the segment */
            terminal.type = OCT_TERM;
            terminal.contents.term.name = (char *) Mac_GetNameOfPinInst(((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.pinInstPtr);
/*
            terminal.contents.term.instanceId = oct_null_id;
fprintf(stderr,"pin instance %s, net %s\n", terminal.contents.term.name,(char *) Mac_GetNetNameOfPin(((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.pinInstPtr));
*/
            if (octGetByName(net, &terminal) == OCT_NOT_FOUND) {
        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term");
/*
, %s.", terminal.contents.term.name);
*/
                return (BEAR_STOP_SEARCH);
            }
            if (octAttach(segment, &terminal) != OCT_OK) {
                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach term.");
/*
, %s.", terminal.contents.term.name);
*/
                return (BEAR_STOP_SEARCH);
            }
/*
get pin
attach pin
*/
        } 
        else {
            /* the segment is connected to a route pin */
/*
fprintf(stderr,"    route pin\n");
*/
            /* check if route pin does not exist */
            if (rSegPtr->j0sr.rt->clientData == NULL) {
/*
fprintf(stderr,"        route pin null\n");
*/
                /* check if terminal has no generic pin pointer */
                if (((GenericPin *)termPtr->genericPinPtr) == NULL) {
                    /* ring route pin */
                    /* get and convert coordinates of
                       the route pin using OCT_PER_LAMBDA */
                    points[0].x
                        = termPtr->mp->geop->gtp[termPtr->tnum].loc.xy[0]
                          * OCT_PER_LAMBDA;
                    points[0].y
                        = termPtr->mp->geop->gtp[termPtr->tnum].loc.xy[1]
                          * OCT_PER_LAMBDA;
                } else {
                    /* otherwise; regular route pin  */
                    /* get and convert coordinates of
                       the route pin using OCT_PER_LAMBDA */
                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                    points[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                    points[0].y = coordinate.p_y * OCT_PER_LAMBDA;

                    /* get the other terminal pointer */ 
                    if (termPtr == ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[0].termPtr) {
                        termPtr2 = ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[1].termPtr;
                    } else if (termPtr == ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[1].termPtr) {
                        termPtr2 = ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[0].termPtr;
                    } else {
                        /* otherwise error; cannot match terminal pointer */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : can not find term pointer on route pin.");
                        return (BEAR_STOP_SEARCH);
                    }
/*
                if (termPtr2->mp->desmp == NULL)
                    termPtr = (term *) Mac_LinkTermToTerm(termPtr2,termPtr2->mp->desmp);
*/
                    /* check if l-channel module */
                    if (termPtr2->mp->type == 11) {
                        /* l-channel module */
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                        /* get duplicate terminal using top module */
                        if (termPtr2->mp->ansmp == NULL) {
                            termPtr = (term *) Mac_LinkTermToTerm(termPtr2,termPtr2->mp);
                        } else {
                            termPtr = (term *) Mac_LinkTermToTerm(termPtr2,termPtr2->mp->ansmp);
                        }
                    } else {
                        /* regular module */
                        /* get terminal */
                        termPtr = termPtr2;
                    }
                }
                /* check layer of the segment */
                if (termPtr->rsp->msklvl == 12) {
                    /* special mask level 12 = layer 1 */
                    /* check if layer 1 is not already used */
                    if (end0layer1 != 1) {
                        /* put as second layer */
                        end0layer2 = 1;
                    }
                } else {
                    /* otherwise; regular mask levels
                       and their corresponding layers */
                    /* check if layer is not already used */
                    if (end0layer1 != (termPtr->rsp->msklvl - 1)) {
                        /* put as second layer */
                        end0layer2 = termPtr->rsp->msklvl - 1;
                    }
                }
                /* check if other segment does not exist */
                if (termPtr->rsp->clientData == NULL ) {
/*
fprintf(stderr,"            term ptr to other end of seg null\n");
*/
                    /* find the end of the other segment
                       with the same terminal */
                    if (termPtr == termPtr->rsp->j0sr.rt) {
                        /* check if the other end of the
                           other segment is a terminal */
                        if (termPtr->rsp->t1fsr) {
                            /* check if the terminal does not exist */
                            if (termPtr->rsp->j1sr.rt->clientData == NULL) {
                                /* check if module pointer has
                                   an ancestor or parent */
                                if (termPtr->rsp->j1sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = termPtr->rsp->j1sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(termPtr->rsp->j1sr.rt,termPtr->rsp->j1sr.rt->mp->ansmp);
                                }
                                /* check if the terminal has
                                   no generic pin pointer */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                    == NULL) {
                                    /* ring route pin */
                                    /* get coordinates of the ring route pin */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                    
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* pin instance */
                                    /* get absolute coordinates
                                       of the pin instance */
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; regular route pin */
                                    /* get absolute coordinates
                                       of the regular route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
/*
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                                /* convert coordinates using OCT_PER_LAMBDA */
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
                            } else {
                                /* terminal exists; get the coordinates */
                                points2[0].x = ((octObject *)termPtr->rsp->j1sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[0].y = ((octObject *)termPtr->rsp->j1sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* the other end of the other
                               segment is a routing junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (termPtr->rsp->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates 
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->ansmp->rCellPtr
                                    == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates 
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates 
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                    } else if (termPtr == termPtr->rsp->j1sr.rt) {
                        /* check if the other end of the
                           other segment is a terminal */
                        if (termPtr->rsp->t0fsr) {
                            /* check if the terminal does not exist */
                            if (termPtr->rsp->j0sr.rt->clientData == NULL) {
                                /* check if module pointer has
                                   an ancestor or parent */
                                if (termPtr->rsp->j0sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = termPtr->rsp->j0sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(termPtr->rsp->j0sr.rt,termPtr->rsp->j0sr.rt->mp->ansmp);
                                }
                                /* check if the terminal has
                                   no generic pin pointer */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                    == NULL) {
                                    /* ring route pin */
                                    /* get coordinates of the ring route pin */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* pin instance */
                                    /* get absolute coordinates
                                       of the pin instance */
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; regular route pin */
                                    /* get absolute coordinates
                                       of the regular route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
                                /* convert coordinates using OCT_PER_LAMBDA */
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
/*
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                            } else {
                                /* terminal exists; get the coordinates */
                                points2[0].x = ((octObject *)termPtr->rsp->j0sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[0].y = ((octObject *)termPtr->rsp->j0sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* the other end of the other
                               segment is a routing junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (termPtr->rsp->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates 
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->ansmp->rCellPtr
                                    == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates 
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates 
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                    } else {
                        /* otherwise, error */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : can not match terminal on segment.");
                        return (BEAR_STOP_SEARCH);
                    }
/*
fprintf(stderr,"            make seg\n");
*/
                    /* create segment */
                    MALLOC(octObject *, segmentPtr2, sizeof(octObject));
                    segmentPtr2->type = OCT_PATH;
                    segmentPtr2->contents.path.width = 
                        (termPtr->rsp->hwidth + termPtr->rsp->hwidth) * 
                            OCT_PER_LAMBDA;
                    if (octCreate(net, segmentPtr2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create segment.");
                        return (BEAR_STOP_SEARCH);
                    }

                    /* put segment on the proper layer */
                    if (termPtr->rsp->msklvl == 1) {
                        /* if metal 1 layer was not used previously */
                        if (layer.contents.layer.name != "MET1") {
                            /* get metal 1 layer */
                            layer.contents.layer.name = "MET1";
                            if (octGetByName(&chip,&layer) != OCT_OK) {
                                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                return (BEAR_STOP_SEARCH);
                            }
                        }
                    } else if ((termPtr->rsp->msklvl == 2)
                               || (termPtr->rsp->msklvl == 12)) {
                        /* if metal 2 layer was not used previously */
                        if (layer.contents.layer.name != "MET2") {
                            /* get metal 2 layer */
                            layer.contents.layer.name = "MET2";
                            if (octGetByName(&chip,&layer) != OCT_OK) {
                                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                return (BEAR_STOP_SEARCH);
                            }
                        }
                    } else {
                        /* error, no such layer */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : unknown layer.");
                        return (BEAR_STOP_SEARCH);
                    }
                    if (octAttach(&layer, segmentPtr2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* put end points of segment */
                    points2[1].x = points[0].x;
                    points2[1].y = points[0].y;
                    if (octPutPoints(segmentPtr2, 2, points2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot put segment points.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* store segment pointer in client data */
                    termPtr->rsp->clientData = (ClientData) segmentPtr2;
                } else {
/*
fprintf(stderr,"            get old points\n");
*/
                    /* get end points already stored in
                       client data containing segment pointer */
                    if (octGetPoints((octObject *) termPtr->rsp->clientData,
                        &number2, points2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get segment points.");
                        return (BEAR_STOP_SEARCH);
                    }
                }
                /* set via (at end 0) size */
                if (points2[0].x == points2[1].x) {
                    /* vertical segment */
                    /* set appropriate via (at end 0) width and height */
                    end0width = MAX((termPtr->rsp->hwidth
                        + termPtr->rsp->hwidth), 4);
                    end0height = 4;
                } else if (points2[0].y == points2[1].y) {
                    /* horizontal segment */
                    /* set appropriate via (at end 0) height and width */
                    end0height = MAX((termPtr->rsp->hwidth
                        + termPtr->rsp->hwidth), 4);
                    end0width = 4;
                } else {
                    /* error, non-manhattan segment */
                    Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
                    return (BEAR_STOP_SEARCH);
                }
/*
flag, make at end?
*/
                /* create via at end 0 */
                MALLOC(octObject *, end0viaPtr, sizeof(octObject));
                termPtr->clientData = rSegPtr->j0sr.rt->clientData =
                    (ClientData) end0viaPtr;
            
/*
width
s?lsr work on route pins?
assume only one in same direction
other seg?
make via
attach via
vertical or horizontal seg?
*/
            } else {
/*
fprintf(stderr,"     attach old route term to seg\n");
*/
                /* get location of previously saved route terminal */
                points[0].x = ((octObject *)rSegPtr->j0sr.rt->clientData)->contents.instance.transform.translation.x;
                points[0].y = ((octObject *)rSegPtr->j0sr.rt->clientData)->contents.instance.transform.translation.y;
                /* get previously saved route terminal and attach to segment */
                terminal.type = OCT_TERM;
                terminal.contents.term.name = "t";
                if (octGetByName((octObject *) rSegPtr->j0sr.rt->clientData,
                                 &terminal) == OCT_NOT_FOUND) {
                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term, t, from via instance.");
                    return (BEAR_STOP_SEARCH);
                }
                if (octAttach(segment, &terminal) != OCT_OK) {
                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via.");
/*
, %s.", ((octObject *)rSegPtr->j0sr.rt->clientData)->contents.instance.name);
*/
                    return (BEAR_STOP_SEARCH);
                }
            }
        }
    } else {
/*
fprintf(stderr,"junction at end 0\n");
junction location relative or absolute?
junct.
*/
        /* route junction at end 0 */
        /* check if route junction does not exist */
        if (rSegPtr->j0sr.rj->clientData == NULL) {
            /* check if module pointer has an ancestor or parent */
            if (rSegPtr->hsr->mp->ansmp == NULL) {
                /* regular channel */
                /* check if route cell pointer does not exist */
                if (rSegPtr->hsr->mp->rCellPtr == NULL) {
                    /* ring route junction */
                    /* get and convert coordinates using OCT_PER_LAMBDA */
                    points[0].x = rSegPtr->j0sr.rj->locjr.xy[0]
                                  * OCT_PER_LAMBDA;
                    points[0].y = rSegPtr->j0sr.rj->locjr.xy[1]
                                  * OCT_PER_LAMBDA;
                } else {
                    /* otherwise; regular inner route junction */
                    /* get and convert coordinates using OCT_PER_LAMBDA */
                    coordinate2.p_x = rSegPtr->j0sr.rj->locjr.xy[0];
                    coordinate2.p_y = rSegPtr->j0sr.rj->locjr.xy[1];
                    GeoTransPoint(&(((RouteCell *)rSegPtr->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                    points[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                    points[0].y = coordinate.p_y * OCT_PER_LAMBDA;
                }
            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                /* check if route cell pointer does not exist */
                if (rSegPtr->hsr->mp->ansmp->rCellPtr == NULL) {
                    /* ring route junction */
                    /* get and convert coordinates using OCT_PER_LAMBDA */
                    points[0].x = rSegPtr->j0sr.rj->locjr.xy[0]
                                  * OCT_PER_LAMBDA;
                    points[0].y = rSegPtr->j0sr.rj->locjr.xy[1]
                                  * OCT_PER_LAMBDA;
                } else {
                    /* otherwise; regular inner route junction */
                    /* get and convert coordinates using OCT_PER_LAMBDA */
                    coordinate2.p_x = rSegPtr->j0sr.rj->locjr.xy[0];
                    coordinate2.p_y = rSegPtr->j0sr.rj->locjr.xy[1];
                    GeoTransPoint(&(((RouteCell *)rSegPtr->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                    points[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                    points[0].y = coordinate.p_y * OCT_PER_LAMBDA;
                }
            }

            /* create via at end 0 */
            MALLOC(octObject *, end0viaPtr, sizeof(octObject));
            rSegPtr->j0sr.rj->clientData = (ClientData) end0viaPtr;
/*
termPtr->clientData = 
*/
            /* initialize via (at end 0) height and width */
            end0height = 4;
            end0width = 4;
            /* store initial segment */
            rSegPtr2 = rSegPtr->j0sr.rj->sljr;
            /* loop through all segments having the via as an end */
            do {
                /* initialize one end point with the via (at end 0) location */
                points2[1].x = points[0].x;
                points2[1].y = points[0].y;
                /* check if segment is the same as the initial segment */
                if (rSegPtr2 == rSegPtr) {
                    /* same as initial segment, go to next segment */
                    rSegPtr2 = rSegPtr->s0lsr;
                } else if (rSegPtr2->j0sr.rj == rSegPtr->j0sr.rj) {
                    /* other segment has same route junction at end 0 */
/*
rSegPtr2->j0sr.rj->clientData = (ClientData) end0viaPtr;
*/
                    /* check layer of the segment */
                    if (rSegPtr2->msklvl == 12) {
                        /* special mask level 12 = layer 1 */
                        /* check if layer 1 is not already used */
                        if (end0layer1 != 1) {
                            /* put as second layer */
                            end0layer2 = 1;
                        }
                    } else {
                        /* otherwise; regular mask levels
                           and their corresponding layers */
                        if (end0layer1 != (rSegPtr2->msklvl - 1)) {
                            /* put as second layer */
                            end0layer2 = rSegPtr2->msklvl - 1;
                        }
                    }
                    /* check if segment pointer does not exist */
                    if (rSegPtr2->clientData == NULL) {
                        /* check if end 1 of segment is a (MAC/BBL) terminal */
                        if (rSegPtr2->t1fsr) {
                            /* check if terminal exist */
                            if (rSegPtr2->j1sr.rt->clientData == NULL) {
                                /* check if module pointer
                                   has an ancestor or parent */
                                if (rSegPtr2->j1sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = rSegPtr2->j1sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(rSegPtr2->j1sr.rt,rSegPtr2->j1sr.rt->mp->ansmp);
                                }
                                /* check if ring route terminal */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                    == NULL) {
                                    /* get location of ring route terminal */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                /* check if pin instance */
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* get location of pin instance */
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; get location of route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
                                /* store and convert coordinates
                                   using OCT_PER_LAMBDA */
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
/*
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                            } else {
                                /* use coordinates of previously
                                   stored route terminal */
                                points2[0].x = ((octObject *)rSegPtr2->j1sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[0].y = ((octObject *)rSegPtr2->j1sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* end 1 of segment is a route junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (rSegPtr2->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (rSegPtr2->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                if (rSegPtr2->hsr->mp->ansmp->rCellPtr
                                    == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                        /* create segment 2 */
                        MALLOC(octObject *, segmentPtr2, sizeof(octObject));
                        segmentPtr2->type = OCT_PATH;
                        segmentPtr2->contents.path.width = 
                            (rSegPtr2->hwidth + rSegPtr2->hwidth) * 
                                OCT_PER_LAMBDA;
                        if (octCreate(net, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put segment on the proper layer */
                        if (rSegPtr2->msklvl == 1) {
                            /* if metal 1 layer was not used previously */
                            if (layer.contents.layer.name != "MET1") {
                                /* get metal 1 layer */
                                layer.contents.layer.name = "MET1";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() :: cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else if ((rSegPtr2->msklvl == 2)
                                   || (rSegPtr2->msklvl == 12)) {
                            /* if metal 2 layer was not used previously */
                            if (layer.contents.layer.name != "MET2") {
                                /* get metal 2 layer */
                                layer.contents.layer.name = "MET2";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() :: cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else {
                            /* error, no such layer */
                            Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : unknown layer.");
                            return (BEAR_STOP_SEARCH);
                        }
                        if (octAttach(&layer, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put end points of segment */
                        if (octPutPoints(segmentPtr2, 2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot put segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* store segment pointer in client data */
                        rSegPtr2->clientData = (ClientData) segmentPtr2;
                    } else {
                        /* get end points already stored in
                           client data containing segment pointer */
                        if (octGetPoints((octObject *) rSegPtr2->clientData,
                            &number2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                    }
                    /* set via (at end 0) size */
                    if (points2[0].x == points2[1].x) {
                        /* vertical segment */
                        /* set appropriate via (at end 0) width and height */
                        end0width = MAX((rSegPtr2->hwidth + rSegPtr2->hwidth),
                                        end0width);
                    } else if (points2[0].y == points2[1].y) {
                        /* horizontal segment */
                        /* set appropriate via (at end 0) height and width */
                        end0height = MAX((rSegPtr2->hwidth + rSegPtr2->hwidth),
                                         end0height);
                    } else {
                        /* otherwise; error, non-manhattan segment */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* go to next segment */
                    rSegPtr2 = rSegPtr2->s0lsr;
                } else if (rSegPtr2->j1sr.rj == rSegPtr->j0sr.rj) {
                    /* other segment has same route junction at end 1 */
/*
rSegPtr2->j1sr.rj->clientData = (ClientData) end0viaPtr;
*/
                    /* check layer of the segment */
                    if (rSegPtr2->msklvl == 12) {
                        /* special mask level 12 = layer 1 */
                        /* check if layer 1 is not already used */
                        if (end0layer1 != 1) {
                            /* put as second layer */
                            end0layer2 = 1;
                        }
                    } else {
                        /* otherwise; regular mask levels
                           and their corresponding layers */
                        if (end0layer1 != (rSegPtr2->msklvl - 1)) {
                            /* put as second layer */
                            end0layer2 = rSegPtr2->msklvl - 1;
                        }
                    }
                    /* check if segment pointer does not exist */
                    if (rSegPtr2->clientData == NULL) {
                        /* check if end 0 of segment is a (MAC/BBL) terminal */
                        if (rSegPtr2->t0fsr) {
                            /* check if terminal exist */
                            if (rSegPtr2->j0sr.rt->clientData == NULL) {
                                /* check if module pointer
                                   has an ancestor or parent */
                                if (rSegPtr2->j0sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = rSegPtr2->j0sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(rSegPtr2->j0sr.rt,rSegPtr2->j0sr.rt->mp->ansmp);
                                }
                                /* check if ring route terminal */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                     == NULL) {
                                    /* get location of ring route terminal */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                /* check if pin instance */
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* get location of pin instance */
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; get location of route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
                                /* store and convert coordinates
                                   using OCT_PER_LAMBDA */
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
/*
                                points2[0].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[0].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                            } else {
                                /* use coordinates of previously
                                   stored route terminal */
                                points2[0].x = ((octObject *)rSegPtr2->j0sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[0].y = ((octObject *)rSegPtr2->j0sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* end 0 of segment is a route junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (rSegPtr2->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (rSegPtr2->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                if (rSegPtr2->hsr->mp->ansmp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[0].x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[0].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[0].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                        /* create segment 2 */
                        MALLOC(octObject *, segmentPtr2, sizeof(octObject));
                        segmentPtr2->type = OCT_PATH;
                        segmentPtr2->contents.path.width = 
                            (rSegPtr2->hwidth + rSegPtr2->hwidth) * 
                                OCT_PER_LAMBDA;
                        if (octCreate(net, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put segment on the proper layer */
                        if (rSegPtr2->msklvl == 1) {
                            /* if metal 1 layer was not used previously */
                            if (layer.contents.layer.name != "MET1") {
                                /* get metal 1 layer */
                                layer.contents.layer.name = "MET1";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else if ((rSegPtr2->msklvl == 2)
                                   || (rSegPtr2->msklvl == 12)) {
                            /* if metal 2 layer was not used previously */
                            if (layer.contents.layer.name != "MET2") {
                                /* get metal 2 layer */
                                layer.contents.layer.name = "MET2";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else {
                            /* otherwise; error, no such layer */
                            Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : unknown layer.");
                            return (BEAR_STOP_SEARCH);
                        }
                        if (octAttach(&layer, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put end points of segment */
                        if (octPutPoints(segmentPtr2, 2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot put segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* store segment pointer in client data */
                        rSegPtr2->clientData = (ClientData) segmentPtr2;
                    } else {
                        /* get end points already stored in
                           client data containing segment pointer */
                        if (octGetPoints((octObject *) rSegPtr2->clientData,
                            &number2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                    }
                    /* set via (at end 0) size */
                    if (points2[0].x == points2[1].x) {
                        /* vertical segment */
                        /* set appropriate via (at end 0) width and height */
                        end0width = MAX((rSegPtr2->hwidth
                                         + rSegPtr2->hwidth), end0width);
                    } else if (points2[0].y == points2[1].y) {
                        /* horizontal segment */
                        /* set appropriate via (at end 0) height and width */
                        end0height = MAX((rSegPtr2->hwidth
                                          + rSegPtr2->hwidth), end0height);
                    } else {
                        /* otherwise; error, non-manhattan segment */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* go to next segment */
                    rSegPtr2 = rSegPtr2->s1lsr;
                } else {
                    /* otherwise; error, cannot find matching junction */
                    Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : junction not found of a connected segment.");
                    return (BEAR_STOP_SEARCH);
                }
/*
start at junction
(rSegPtr2 != rSegPtr) && 
*/
            /* loop until no more segments */
            } while (rSegPtr2 != NULL);
/*
flag, make at end?
assume s?lsr circular, must check which at each seg
make via, other segs?
attach via
*/
        } else {
            /* use coordinates of previously stored route junction */
            points[0].x = ((octObject *)rSegPtr->j0sr.rj->clientData)->contents.instance.transform.translation.x;
            points[0].y = ((octObject *)rSegPtr->j0sr.rj->clientData)->contents.instance.transform.translation.y;
            /* get terminal from client data and attach to segment */
            terminal.type = OCT_TERM;
            terminal.contents.term.name = "t";
            if (octGetByName((octObject *) rSegPtr->j0sr.rj->clientData,
                &terminal) == OCT_NOT_FOUND) {
                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term, t, from via instance.");
                return (BEAR_STOP_SEARCH);
            }
            if (octAttach(segment, &terminal) != OCT_OK) {
                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via.");
/*
, %s.", ((octObject *)rSegPtr->j0sr.rj->clientData)->contents.instance.name);
*/
                return (BEAR_STOP_SEARCH);
            }
/*
attach via
*/
        }
    }

    /* check if end 1 of segment is a (MAC/BBL) terminal */
    if (rSegPtr->t1fsr) {
/*
fprintf(stderr,"terminal at end 1\n");
term, pos. rel.
which if first? have same problem with term to pin in l channels?
*/
        /* check if module pointer has an ancestor or parent */
        if (rSegPtr->j1sr.rt->mp->ansmp == NULL) {
            /* regular channel */
            termPtr = rSegPtr->j1sr.rt;
        } 
        else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
            termPtr = Mac_LinkTermToTerm(rSegPtr->j1sr.rt,
                                         rSegPtr->j1sr.rt->mp->ansmp);
/*
fprintf(stderr,"find other terminal\n");
*/
        }
        /* check if the terminal has a generic pin pointer
           and if so, check its pin type */
        if ((GenericPin *)termPtr->genericPinPtr != NULL
            && ((GenericPin *)termPtr->genericPinPtr)->pinType == PIN_INST) {
/*
fprintf(stderr,"     pin instance\n");
*/
            /* the segment is connected to a pin instance */
            /* get and convert coordinates using OCT_PER_LAMBDA */
            Mac_GetAbsPosOfPin(((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
            points[1].x = coordinate.p_x * OCT_PER_LAMBDA;
            points[1].y = coordinate.p_y * OCT_PER_LAMBDA;

            /* get and attach the terminal (pin) to the segment */
            terminal.type = OCT_TERM;
            terminal.contents.term.name = (char *) Mac_GetNameOfPinInst(((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.pinInstPtr);
/*
            terminal.contents.term.instanceId = oct_null_id;
*/
            if (octGetByName(net, &terminal) == OCT_NOT_FOUND) {
                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term.");
/*
, %s.", terminal.contents.term.name);
*/
                return (BEAR_STOP_SEARCH);
            }
            if (octAttach(segment, &terminal) != OCT_OK) {
                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach term.");
/*
, %s.", terminal.contents.term.name);
*/
                return (BEAR_STOP_SEARCH);
            }
/*
get pin
attach pin
*/
        } 
        else {
            /* the segment is connected to a route pin */
/*
fprintf(stderr,"     route pin\n");
*/
            /* check if route pin does not exist */
            if (rSegPtr->j1sr.rt->clientData == NULL) {
/*
fprintf(stderr,"     route pin null\n");
*/
                /* check if terminal has no generic pin pointer */
                if (((GenericPin *)termPtr->genericPinPtr) == NULL) {
                    /* ring route pin */
                    /* get and convert coordinates of
                       the route pin using OCT_PER_LAMBDA */
                    points[0].x
                        = termPtr->mp->geop->gtp[termPtr->tnum].loc.xy[0]
                          * OCT_PER_LAMBDA;
                    points[0].y
                        = termPtr->mp->geop->gtp[termPtr->tnum].loc.xy[1]
                          * OCT_PER_LAMBDA;
                } else {
                    /* otherwise; regular route pin  */
                    /* get and convert coordinates of
                       the route pin using OCT_PER_LAMBDA */
                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                    points[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                    points[1].y = coordinate.p_y * OCT_PER_LAMBDA;

                    /* get the other terminal pointer */
                    if (termPtr == ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[0].termPtr) {
                        termPtr2 = ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[1].termPtr;
                    } else if (termPtr == ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[1].termPtr) {
                        termPtr2 = ((GenericPin *)termPtr->genericPinPtr)->pinPtrUn.routePinPtr->BBLTermPtr[0].termPtr;
                    } else {
                        /* otherwise error; cannot match terminal pointer */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : can not find term pointer on route pin.");
                        return (BEAR_STOP_SEARCH);
                    }
/*
                if (termPtr2->mp->desmp == NULL)
                    termPtr = (term *) Mac_LinkTermToTerm(termPtr2,termPtr2->mp->desmp);
*/
                    /* check if l-channel module */
                    if (termPtr2->mp->type == 11) {
                        /* l-channel module */
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                        /* get duplicate terminal using top module */
                        if (termPtr2->mp->ansmp == NULL) {
                            termPtr = (term *) Mac_LinkTermToTerm(termPtr2,termPtr2->mp);
                        } else {
                            termPtr = (term *) Mac_LinkTermToTerm(termPtr2,termPtr2->mp->ansmp);
                        }
                    } 
                    else {
                        /* regular module */
                        /* get terminal */
                        termPtr = termPtr2;
                    }
                }
                /* check layer of the segment */
                if (termPtr->rsp->msklvl == 12) {
                    /* special mask level 12 = layer 1 */
                    /* check if layer 1 is not already used */
                    if (end1layer1 != 1) {
                        /* put as second layer */
                        end1layer2 = 1;
                    }
                } else {
                    /* otherwise; regular mask levels
                       and their corresponding layers */
                    /* check if layer is not already used */
                    if (end1layer1 != (termPtr->rsp->msklvl - 1)) {
                        /* put as second layer */
                        end1layer2 = termPtr->rsp->msklvl - 1;
                    }
                }
                /* check if other segment does not exist */
                if (termPtr->rsp->clientData == NULL) {
/*
fprintf(stderr,"         terminal at other end null\n");
*/
                    /* find the end of the other segment
                       with the same terminal */
                    if (termPtr == termPtr->rsp->j0sr.rt) {
                        /* check if the other end of the
                           other segment is a terminal */
                        if (termPtr->rsp->t1fsr) {
                            /* check if the terminal does not exist */
                            if (termPtr->rsp->j1sr.rt->clientData == NULL) {
                                /* check if module pointer has
                                   an ancestor or parent */
                                if (termPtr->rsp->j1sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = termPtr->rsp->j1sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(termPtr->rsp->j1sr.rt,termPtr->rsp->j1sr.rt->mp->ansmp);
                                }
                                /* check if the terminal has
                                   no generic pin pointer */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                     == NULL) {
                                    /* ring route pin */
                                    /* get coordinates of the ring route pin */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* pin instance */
                                    /* get absolute coordinates
                                       of the pin instance */
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; regular route pin */
                                    /* get absolute coordinates
                                       of the regular route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
                                /* convert coordinates using OCT_PER_LAMBDA */
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
/*
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                            } else {
                                /* terminal exists; get the coordinates */
                                points2[1].x = ((octObject *)termPtr->rsp->j1sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[1].y = ((octObject *)termPtr->rsp->j1sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* the other end of the other
                               segment is a routing junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (termPtr->rsp->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[1].x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->ansmp->rCellPtr
                                    == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[1].x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                    } else if (termPtr == termPtr->rsp->j1sr.rt) {
                        /* check if the other end of the
                           other segment is a terminal */
                        if (termPtr->rsp->t0fsr) {
                            /* check if the terminal does not exist */
                            if (termPtr->rsp->j0sr.rt->clientData == NULL) {
                                /* check if module pointer has
                                   an ancestor or parent */
                                if (termPtr->rsp->j0sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = termPtr->rsp->j0sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(termPtr->rsp->j0sr.rt,termPtr->rsp->j0sr.rt->mp->ansmp);
                                }
                                /* check if the terminal has
                                   no generic pin pointer */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                    == NULL) {
                                    /* ring route pin */
                                    /* get coordinates of the ring route pin */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* pin instance */
                                    /* get absolute coordinates
                                       of the pin instance */
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; regular route pin */
                                    /* get absolute coordinates
                                       of the regular route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
                                /* convert coordinates using OCT_PER_LAMBDA */
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
/*
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                            } else {
                                /* terminal exists; get the coordinates */
                                points2[1].x = ((octObject *)termPtr->rsp->j0sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[1].y = ((octObject *)termPtr->rsp->j0sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* the other end of the other
                               segment is a routing junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (termPtr->rsp->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[1].x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                /* check if route cell pointer does not exist */
                                if (termPtr->rsp->hsr->mp->ansmp->rCellPtr
                                    == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[1].x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                    /* othrewise; regular route junction */
                                    /* get and convert corrdinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = termPtr->rsp->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = termPtr->rsp->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)termPtr->rsp->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                    } else {
                        /* otherwise, error */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : can not match terminal on segment.");
                        return (BEAR_STOP_SEARCH);
                    }
/*
fprintf(stderr,"     make seg\n");
*/
                    /* create segment */
                    MALLOC(octObject *, segmentPtr2, sizeof(octObject));
                    segmentPtr2->type = OCT_PATH;
                    segmentPtr2->contents.path.width = 
                        (termPtr->rsp->hwidth + termPtr->rsp->hwidth) * 
                            OCT_PER_LAMBDA;
                    if (octCreate(net, segmentPtr2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create segment.");
                        return (BEAR_STOP_SEARCH);
                    }

                    /* put segment on the proper layer */
                    if (termPtr->rsp->msklvl == 1) {
                        /* if metal 1 layer was not used previously */
                        if (layer.contents.layer.name != "MET1") {
                            /* get metal 1 layer */
                            layer.contents.layer.name = "MET1";
                            if (octGetByName(&chip,&layer) != OCT_OK) {
                                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                return (BEAR_STOP_SEARCH);
                            }
                        }
                    } else if ((termPtr->rsp->msklvl == 2)
                               || (termPtr->rsp->msklvl == 12)) {
                        /* if metal 2 layer was not used previously */
                        if (layer.contents.layer.name != "MET2") {
                            /* get metal 2 layer */
                            layer.contents.layer.name = "MET2";
                            if (octGetByName(&chip,&layer) != OCT_OK) {
                                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                return (BEAR_STOP_SEARCH);
                            }
                        }
                    } else {
                        /* error, no such layer */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : unknown layer.");
                        return (BEAR_STOP_SEARCH);
                    }
                    if (octAttach(&layer, segmentPtr2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* put end points of segment */
                    points2[0].x = points[1].x;
                    points2[0].y = points[1].y;
                    if (octPutPoints(segmentPtr2, 2, points2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot put segment points.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* store segment pointer in client data */
                    termPtr->rsp->clientData = (ClientData) segmentPtr2;
                } else {
                    /* get end points already stored in
                       client data containing segment pointer */
                    if (octGetPoints((octObject *) termPtr->rsp->clientData,
                        &number2, points2) != OCT_OK) {
                        octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get segment points.");
                        return (BEAR_STOP_SEARCH);
                    }
                }
                /* set via (at end 1) size */
                if (points2[0].x == points2[1].x) {
                    /* vertical segment */
                    /* set appropriate via (at end 1) width and height */
                    end1width = MAX((termPtr->rsp->hwidth
                                     + termPtr->rsp->hwidth), 4);
                    end1height = 4;
                } else if (points2[0].y == points2[1].y) {
                    /* horizontal segment */
                    /* set appropriate via (at end 1) height and width */
                    end1height = MAX((termPtr->rsp->hwidth
                                      + termPtr->rsp->hwidth), 4);
                    end1width = 4;
                } else {
                    /* error, non-manhattan segment */
                    Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
                    return (BEAR_STOP_SEARCH);
                }
/*
flag, make at end?
*/
                /* create via at end 1 */
                MALLOC(octObject *, end1viaPtr, sizeof(octObject));
                termPtr->clientData = rSegPtr->j1sr.rt->clientData = (ClientData) end1viaPtr;
            
/*
s?lsr work on route pins?
assume only one in same direction
width
other seg?
make via
attach via
vertical or horizontal seg?
*/
            } else {
                /* get location of previously saved route terminal */
                points[1].x = ((octObject *)rSegPtr->j1sr.rt->clientData)->contents.instance.transform.translation.x;
                points[1].y = ((octObject *)rSegPtr->j1sr.rt->clientData)->contents.instance.transform.translation.y;
                /* get previously saved route terminal and attach to segment */
                terminal.type = OCT_TERM;
                terminal.contents.term.name = "t";
                if (octGetByName((octObject *) rSegPtr->j1sr.rt->clientData,
                                 &terminal) == OCT_NOT_FOUND) {
                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term, t, from via instance.");
                    return (BEAR_STOP_SEARCH);
                }
                if (octAttach(segment, &terminal) != OCT_OK) {
                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via.");
/*
, %s.", ((octObject *)rSegPtr->j1sr.rt->clientData)->contents.instance.name);
*/
                    return (BEAR_STOP_SEARCH);
                }
            }
        }
    } 
    else {
/*
fprintf(stderr,"junction at end 1\n");
junct.
*/
        /* route junction at end 1 */
        /* check if route junction does not exist */
        if (rSegPtr->j1sr.rj->clientData == NULL) {
            /* check if module pointer has an ancestor or parent */
            if (rSegPtr->hsr->mp->ansmp == NULL) {
                /* regular channel */
                /* check if route cell pointer does not exist */
                if (rSegPtr->hsr->mp->rCellPtr == NULL) {
                    points[1].x = rSegPtr->j1sr.rj->locjr.xy[0]
                                  * OCT_PER_LAMBDA;
                    points[1].y = rSegPtr->j1sr.rj->locjr.xy[1]
                                  * OCT_PER_LAMBDA;
                } else {
                    /* otherwise; regular inner route junction */
                    /* get and convert coordinates using OCT_PER_LAMBDA */
                    coordinate2.p_x = rSegPtr->j1sr.rj->locjr.xy[0];
                    coordinate2.p_y = rSegPtr->j1sr.rj->locjr.xy[1];
                    GeoTransPoint(&(((RouteCell *)rSegPtr->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                    points[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                    points[1].y = coordinate.p_y * OCT_PER_LAMBDA;
                }
            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                /* check if route cell pointer does not exist */
                if (rSegPtr->hsr->mp->ansmp->rCellPtr == NULL) {
                    /* ring route junction */
                    /* get and convert coordinates using OCT_PER_LAMBDA */
                    points[1].x = rSegPtr->j1sr.rj->locjr.xy[0]
                                  * OCT_PER_LAMBDA;
                    points[1].y = rSegPtr->j1sr.rj->locjr.xy[1]
                                  * OCT_PER_LAMBDA;
                } else {
                    /* otherwise; regular inner route junction */
                    /* get and convert coordinates using OCT_PER_LAMBDA */
                    coordinate2.p_x = rSegPtr->j1sr.rj->locjr.xy[0];
                    coordinate2.p_y = rSegPtr->j1sr.rj->locjr.xy[1];
                    GeoTransPoint(&(((RouteCell *)rSegPtr->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                    points[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                    points[1].y = coordinate.p_y * OCT_PER_LAMBDA;
                }
            }

            /* create via at end 1 */
            MALLOC(octObject *, end1viaPtr, sizeof(octObject));
            rSegPtr->j1sr.rj->clientData = (ClientData) end1viaPtr;
/*
termPtr->clientData = 
*/
            /* initialize via (at end 1) height and width */
            end1height = 4;
            end1width = 4;
            /* store initial segment */
            rSegPtr2 = rSegPtr->j1sr.rj->sljr;
            /* loop through all segments having the via as an end */
            do {
                /* initialize one end point with the via (at end 1) location */
                points2[0].x = points[1].x;
                points2[0].y = points[1].y;
                /* check if segment is the same as the initial segment */
                if (rSegPtr2 == rSegPtr) {
                    /* same as initial segment, go to next segment */
                    rSegPtr2 = rSegPtr->s1lsr;
                } else if (rSegPtr2->j0sr.rj == rSegPtr->j1sr.rj) {
                    /* other segment has same route junction at end 0 */
/*
rSegPtr2->j0sr.rj->clientData = (ClientData) end1viaPtr;
*/
                    /* check layer of the segment */
                    if (rSegPtr2->msklvl == 12) {
                        /* special mask level 12 = layer 1 */
                        /* check if layer 1 is not already used */
                        if (end1layer1 != 1) {
                            /* put as second layer */
                            end1layer2 = 1;
                        }
                    } else {
                        /* otherwise; regular mask levels
                           and their corresponding layers */
                        if (end1layer1 != (rSegPtr2->msklvl - 1)) {
                            /* put as second layer */
                            end1layer2 = rSegPtr2->msklvl - 1;
                        }
                    }
                    /* check if segment pointer does not exist */
                    if (rSegPtr2->clientData == NULL) {
                        /* check if end 1 of segment is a (MAC/BBL) terminal */
                        if (rSegPtr2->t1fsr) {
                            /* check if terminal exist */
                            if (rSegPtr2->j1sr.rt->clientData == NULL) {
                                /* check if module pointer
                                   has an ancestor or parent */
                                if (rSegPtr2->j1sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = rSegPtr2->j1sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(rSegPtr2->j1sr.rt,rSegPtr2->j1sr.rt->mp->ansmp);
                                }
                                /* check if ring route terminal */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                    == NULL) {
                                    /* get location of ring route terminal */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                /* check if pin instance */
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* get location of pin instance */
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; get location of route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
/*
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                                /* store and convert coordinates
                                   using OCT_PER_LAMBDA */
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
                            } else {
                                /* use coordinates of previously
                                   stored route terminal */
                                points2[1].x = ((octObject *)rSegPtr2->j1sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[1].y = ((octObject *)rSegPtr2->j1sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* end 1 of segment is a route junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (rSegPtr2->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (rSegPtr2->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[1].x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                if (rSegPtr2->hsr->mp->ansmp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[1].x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA*/
                                    coordinate2.p_x
                                        = rSegPtr2->j1sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j1sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                        /* create segment 2 */
                        MALLOC(octObject *, segmentPtr2, sizeof(octObject));
                        segmentPtr2->type = OCT_PATH;
                        segmentPtr2->contents.path.width = 
                            (rSegPtr2->hwidth + rSegPtr2->hwidth) * 
                                OCT_PER_LAMBDA;
                        if (octCreate(net, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put segment on the proper layer */
                        if (rSegPtr2->msklvl == 1) {
                            /* if metal 1 layer was not used previously */
                            if (layer.contents.layer.name != "MET1") {
                                /* get metal 1 layer */
                                layer.contents.layer.name = "MET1";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else if ((rSegPtr2->msklvl == 2)
                                   || (rSegPtr2->msklvl == 12)) {
                            /* if metal 2 layer was not used previously */
                            if (layer.contents.layer.name != "MET2") {
                                /* get metal 2 layer */
                                layer.contents.layer.name = "MET2";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else {
                            /* error, no such layer */
                            Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : unknown layer.");
                            return (BEAR_STOP_SEARCH);
                        }
                        if (octAttach(&layer, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put end points of segment */
                        if (octPutPoints(segmentPtr2, 2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot put segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* store segment pointer in client data */
                        rSegPtr2->clientData = (ClientData) segmentPtr2;
                    } else {
                        /* get end points already stored in
                           client data containing segment pointer */
                        if (octGetPoints((octObject *) rSegPtr2->clientData,
                            &number2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                    }
                    /* set via (at end 1) size */
                    if (points2[0].x == points2[1].x) {
                        /* vertical segment */
                        /* set appropriate via (at end 1) width and height */
                        end1width = MAX((rSegPtr2->hwidth
                                         + rSegPtr2->hwidth), end1width);
                    } else if (points2[0].y == points2[1].y) {
                        /* horizontal segment */
                        /* set appropriate via (at end 1) height and width */
                        end1height = MAX((rSegPtr2->hwidth
                                         + rSegPtr2->hwidth), end1height);
                    } else {
                        /* otherwise; error, non-manhattan segment */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* go to next segment */
                    rSegPtr2 = rSegPtr2->s0lsr;
                } else if (rSegPtr2->j1sr.rj == rSegPtr->j1sr.rj) {
                    /* other segment has same route junction at end 1 */
/*
rSegPtr2->j1sr.rj->clientData = (ClientData) end1viaPtr;
*/
                    /* check layer of the segment */
                    if (rSegPtr2->msklvl == 12) {
                        /* special mask level 12 = layer 1 */
                        /* check if layer 1 is not already used */
                        if (end1layer1 != 1) {
                            /* put as second layer */
                            end1layer2 = 1;
                        }
                    } else {
                        /* otherwise; regular mask levels
                           and their corresponding layers */
                        if (end1layer1 != (rSegPtr2->msklvl - 1)) {
                            /* put as second layer */
                            end1layer2 = rSegPtr2->msklvl - 1;
                        }
                    }
                    /* check if segment pointer does not exist */
                    if (rSegPtr2->clientData == NULL) {
                        /* check if end 0 of segment is a (MAC/BBL) terminal */
                        if (rSegPtr2->t0fsr) {
                            /* check if terminal exist */
                            if (rSegPtr2->j0sr.rt->clientData == NULL) {
                                /* check if module pointer
                                   has an ancestor or parent */
                                if (rSegPtr2->j0sr.rt->mp->ansmp == NULL) {
                                    /* regular channel */
                                    termPtr2 = rSegPtr2->j0sr.rt;
                                } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */
                                    termPtr2 = (term *) Mac_LinkTermToTerm(rSegPtr2->j0sr.rt,rSegPtr2->j0sr.rt->mp->ansmp);
                                }
                                /* check if ring route terminal */
                                if (((GenericPin *)termPtr2->genericPinPtr)
                                    == NULL) {
                                    /* get location of ring route terminal */
                                    coordinate.p_x = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[0];
                                    coordinate.p_y = termPtr2->mp->geop->gtp[termPtr2->tnum].loc.xy[1];
                                /* check if pin instance */
                                } else if (((GenericPin *)termPtr2->genericPinPtr)->pinType == PIN_INST) {
                                    /* get location of pin instance *
                                    Mac_GetAbsPosOfPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.pinInstPtr, &coordinate);
                                } else {
                                    /* otherwise; get location of route pin */
                                    Mac_GetAbsPosOfRPin(((GenericPin *)termPtr2->genericPinPtr)->pinPtrUn.routePinPtr, &coordinate);
                                }
/*
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
*/
                                /* store and convert coordinates
                                   using OCT_PER_LAMBDA */
                                points2[1].x = coordinate.p_x * OCT_PER_LAMBDA;
                                points2[1].y = coordinate.p_y * OCT_PER_LAMBDA;
                            } else {
                               /* use coordinates of previously
                                   stored route terminal */
                                points2[1].x = ((octObject *)rSegPtr2->j0sr.rt->clientData)->contents.instance.transform.translation.x;
                                points2[1].y = ((octObject *)rSegPtr2->j0sr.rt->clientData)->contents.instance.transform.translation.y;
                            }
                        } else {
                            /* end 0 of segment is a route junction */
                            /* check if module pointer has
                               an ancestor or parent */
                            if (rSegPtr2->hsr->mp->ansmp == NULL) {
                                /* regular channel */
                                /* check if route cell pointer does not exist */
                                if (rSegPtr2->hsr->mp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA*/
                                    points2[1].x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            } else {
            /* l-channel, must search for the same terminal at top level
               because there are no connections between the duplicated terminals
               (don't ask me why, ask someone else like Howard or Debra) */

                                if (rSegPtr2->hsr->mp->ansmp->rCellPtr == NULL) {
                                    /* ring route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    points2[1].x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0]
                                          * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1]
                                          * OCT_PER_LAMBDA;
                                } else {
                                   /* otherwise; regular inner route junction */
                                    /* get and convert coordinates
                                       using OCT_PER_LAMBDA */
                                    coordinate2.p_x
                                        = rSegPtr2->j0sr.rj->locjr.xy[0];
                                    coordinate2.p_y
                                        = rSegPtr2->j0sr.rj->locjr.xy[1];
                                    GeoTransPoint(&(((RouteCell *)rSegPtr2->hsr->mp->ansmp->rCellPtr)->transform), &coordinate2, &coordinate);
                                    points2[1].x
                                        = coordinate.p_x * OCT_PER_LAMBDA;
                                    points2[1].y
                                        = coordinate.p_y * OCT_PER_LAMBDA;
                                }
                            }
                        }
                        /* create segment 2 */
                        MALLOC(octObject *, segmentPtr2, sizeof(octObject));
                        segmentPtr2->type = OCT_PATH;
                        segmentPtr2->contents.path.width = 
                            (rSegPtr2->hwidth + rSegPtr2->hwidth) * 
                                OCT_PER_LAMBDA;
                        if (octCreate(net, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put segment on the proper layer */
                        if (rSegPtr2->msklvl == 1) {
                            /* if metal 1 layer was not used previously */
                            if (layer.contents.layer.name != "MET1") {
                                /* get metal 1 layer */
                                layer.contents.layer.name = "MET1";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else if ((rSegPtr2->msklvl == 2)
                                   || (rSegPtr2->msklvl == 12)) {
                            /* if metal 2 layer was not used previously */
                            if (layer.contents.layer.name != "MET2") {
                                /* get metal 2 layer */
                                layer.contents.layer.name = "MET2";
                                if (octGetByName(&chip,&layer) != OCT_OK) {
                                    octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get layer.");
/*
%s layer.", layer.contents.layer.name);
*/
                                    return (BEAR_STOP_SEARCH);
                                }
                            }
                        } else {
                            /* otherwise; error, no such layer */
                            Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : unknown layer.");
                            return (BEAR_STOP_SEARCH);
                        }
                        if (octAttach(&layer, segmentPtr2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* put end points of segment */
                        if (octPutPoints(segmentPtr2, 2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot put segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                        /* store segment pointer in client data */
                        rSegPtr2->clientData = (ClientData) segmentPtr2;
                    } else {
                        /* get end points already stored in
                           client data containing segment pointer */
                        if (octGetPoints((octObject *) rSegPtr2->clientData,
                            &number2, points2) != OCT_OK) {
                            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get segment points.");
                            return (BEAR_STOP_SEARCH);
                        }
                    }
                    /* set via (at end 1) size */
                    if (points2[0].x == points2[1].x) {
                        /* vertical segment */
                        /* set appropriate via (at end 1) width and height */
                        end1width = MAX((rSegPtr2->hwidth
                                         + rSegPtr2->hwidth), end1width);
                    } else if (points2[0].y == points2[1].y) {
                        /* horizontal segment */
                        /* set appropriate via (at end 1) height and width */
                        end1height = MAX((rSegPtr2->hwidth
                                          + rSegPtr2->hwidth), end1height);
                    } else {
                        /* otherwise; error, non-manhattan segment */
                        Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
                        return (BEAR_STOP_SEARCH);
                    }
                    /* go to next segment */
                    rSegPtr2 = rSegPtr2->s1lsr;
                } else {
                    /* otherwise; error, cannot find matching junction */
                    Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : junction not found of a connected segment.");
                    return (BEAR_STOP_SEARCH);
                }
/*
start at junction
(rSegPtr2 != rSegPtr) && 
*/
            /* loop until no more segments */
            } while (rSegPtr2 != NULL);
/*
make via, other seg?
attach via
*/
        } else {
            /* use coordinates of previously stored route junction */
            points[1].x = ((octObject *)rSegPtr->j1sr.rj->clientData)->contents.instance.transform.translation.x;
            points[1].y = ((octObject *)rSegPtr->j1sr.rj->clientData)->contents.instance.transform.translation.y;
            /* get terminal from client data and attach to segment */
            terminal.type = OCT_TERM;
            terminal.contents.term.name = "t";
            if (octGetByName((octObject *) rSegPtr->j1sr.rj->clientData,
                             &terminal) == OCT_NOT_FOUND) {
                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term, t, from via instance.");
                return (BEAR_STOP_SEARCH);
            }
            if (octAttach(segment, &terminal) != OCT_OK) {
                octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via.");
/*
, %s.", ((octObject *)rSegPtr->j1sr.rj->clientData)->contents.term.name);
*/
                return (BEAR_STOP_SEARCH);
            }
/*
attach via
*/
        }
    }

    /* check if segment does not exist */
    if (rSegPtr->clientData == NULL) {
        /* put end points of segment */
        if (octPutPoints(segment, 2, points) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot put segment points.");
            return (BEAR_STOP_SEARCH);
        }
        /* store segment pointer in client data */
        rSegPtr->clientData = (ClientData) segment;
    }
/*
else {
        if (octGetPoints(segment, &number2, points) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get segment points.");
            return (BEAR_STOP_SEARCH);
        }
}
*/

    /* check if there is a via at end 0 */
    if (end0viaPtr != NULL) {
        /* make via at end 0 */
/*
fprintf("make end 0 via\n");
*/
        /* check if vertical segment */
        if (points[0].x == points[1].x) {
            /* adjust via width at end 0 */
            end0width = MAX((rSegPtr->hwidth + rSegPtr->hwidth), end0width);
        /* check if horizontal segment */
        } else if (points[0].y == points[1].y) {
            /* adjust via height at end 0 */
            end0height = MAX((rSegPtr->hwidth + rSegPtr->hwidth), end0height);
        } else {
            /* otherwise; error, non-manhattan segment */
            Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
            return (BEAR_STOP_SEARCH);
        }
/*
seg determine via dimensions
make, attach via
*/
        /* setup end 0 via instance variable in OCT */
        end0viaPtr->type = OCT_INSTANCE;
        /* setup via name and orientation */
        if (end0height < end0width) {
            sprintf(vianame,"%s.%dx%d",vialayersname[end0layer1][end0layer2],
                end0width,end0height);
            end0viaPtr->contents.instance.transform.transformType 
                = OCT_NO_TRANSFORM;
        } 
        else {
            (void) sprintf (vianame, "%s.%dx%d", 
                vialayersname[end0layer1][end0layer2], end0height, end0width);
            end0viaPtr->contents.instance.transform.transformType = OCT_ROT90;
        }

        /* setup full path via name */
        (void) sprintf (fullvianame, "$OCTTOOLS/tech/scmos/contacts/%s", 
            vianame);

        /* check if via doesn't exist */
        if (viaDirDoesNotExist(fullvianame)) {
            /* setup via path with vias directory */
            sprintf(fullvianame,"vias/%s",vianame);
            /* check if that via doesn't exist */
            if (viaDirDoesNotExist(fullvianame)) {
                /* check if need to make vias directory */
                if (mkdirvias) {
                    /* check if vias directory does not exist */
                    if (viaDirDoesNotExist("vias")) {
                        /* make vias directory */
                        system("mkdir vias");
                    }
                    /* no more need to make directory */
                    mkdirvias = FALSE/*0*/;
                }

                /* 
                 * Create via using prim (OCT tools command) with the 
                 * proper dimension ordering.
                 */
                if (end0height < end0width) {
                    (void) sprintf (command, "cd vias; %s/bin/prim %s %d %d",
                        odiOctHomePtr, vialayersname[end0layer1][end0layer2],
                        end0width, end0height);
                } 
                else {
                    (void) sprintf (command, "cd vias; %s/bin/prim %s %d %d",
                        odiOctHomePtr, vialayersname[end0layer1][end0layer2],
                        end0height, end0width);
                }
                system(command);
            }
        }
/*
create via - tap or dir and prim
*/
        /* setup, create, and attach to CONNECTORS bag
          the via instance in OCT */
        (void) sprintf (vianame,"via%d",++vianumber);
end0viaPtr->contents.instance.facet = "contents";
        end0viaPtr->contents.instance.name = vianame;
/*
cell.contents.facet.cell;
*/
        end0viaPtr->contents.instance.master = fullvianame;
        end0viaPtr->contents.instance.view = "physical";
        end0viaPtr->contents.instance.version = OCT_CURRENT_VERSION;
        end0viaPtr->contents.instance.transform.translation.x 
            = points[0].x;
        end0viaPtr->contents.instance.transform.translation.y 
            = points[0].y;
        if (octCreate(&chip, end0viaPtr) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create via instance.");
/*
, %s.", end0viaPtr->contents.instance.name);
*/
            return (BEAR_STOP_SEARCH);
        }
        if (octAttach(&bag, end0viaPtr) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via instance to CONNECTORS bag.");
            return (BEAR_STOP_SEARCH);
        }
        /* get and attach to segment the terminal from the instance */
        terminal.type = OCT_TERM;
        terminal.contents.term.name = "t";
/*
    terminalt.type = OCT_TERM;
    terminalt.contents.term.name = "t";
           Create
    if (octGetByName(&terminst, &terminalt) == OCT_NOT_FOUND) {
        octError("WriteNetPinConnectionsToOct: cannot get term, t.");
    }
    if (octAttach(&terminal, &terminalt) != OCT_OK) {
        octError("WriteIOPadsToOct() : cannot attach term, t, of terminst.");
    }
        if (octAttach(end0viaPtr, &terminal) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via terminal instance, %s.", terminal.contents.instance.name);
        }
*/
        if (octGetByName(end0viaPtr, &terminal) == OCT_NOT_FOUND) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term, t, from via instance.");
            return (BEAR_STOP_SEARCH);
        }
        if (octAttach(segment, &terminal) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
            return (BEAR_STOP_SEARCH);
        }
    }
    /* check if there is a via at end 1 */
    if (end1viaPtr != NULL) {
        /* make via at end 1 */
/*
fprintf("make end 1 via\n");
*/
        /* check if vertical segment */
        if (points[0].x == points[1].x) {
            /* adjust via width at end 1 */
            end1width = MAX((rSegPtr->hwidth + rSegPtr->hwidth), end1width);
        /* check if horizontal segment */
        } else if (points[0].y == points[1].y) {
            /* adjust via height at end 1 */
            end1height = MAX((rSegPtr->hwidth + rSegPtr->hwidth), end1height);
        } else {
            /* otherwise; error, non-manhattan segment */
            Bear_PrintErr("WriteSegmentsJunctionsConnectionsToOct() : segment not straight.");
            return (BEAR_STOP_SEARCH);
        }
/*
seg determine via dimensions
make, attach via
*/
        /* setup end 1 via instance variable in OCT */
        end1viaPtr->type = OCT_INSTANCE;
        /* setup via name and orientation */
        if (end1height < end1width) {
            sprintf(vianame,"%s.%dx%d",vialayersname[end1layer1][end1layer2],
                end1width,end1height);
            end1viaPtr->contents.instance.transform.transformType 
                = OCT_NO_TRANSFORM;
        } else {
            sprintf(vianame,"%s.%dx%d",vialayersname[end1layer1][end1layer2],
                end1height,end1width);
            end1viaPtr->contents.instance.transform.transformType 
                = OCT_ROT90;
        }

        /* setup full path via name */
        (void) sprintf (fullvianame, "$OCTTOOLS/tech/scmos/contacts/%s", 
            vianame);

        /* check if via doesn't exist */
        if (viaDirDoesNotExist(fullvianame)) {
            /* setup via path with vias directory */
            sprintf(fullvianame,"vias/%s",vianame);
            /* check if that via doesn't exist */
            if (viaDirDoesNotExist(fullvianame)) {
                /* check if need to make vias directory */
                if (mkdirvias) {
                    /* check if vias directory does not exist */
                    if (viaDirDoesNotExist("vias")) {
                        /* make vias directory */
                        system("mkdir vias");
                    }
                    /* no more need to make directory */
                    mkdirvias = FALSE/*0*/;
                }

                /* 
                 * Create via using prim (OCT tools command) with the
                 * proper dimension ordering.
                 */
                if (end1height < end1width) {
                    (void) sprintf (command, "cd vias; %s/bin/prim %s %d %d",
                        odiOctHomePtr, vialayersname[end1layer1][end1layer2],
                        end1width, end1height);
                } 
                else {
                    (void) sprintf (command, "cd vias; %s/bin/prim %s %d %d",
                        odiOctHomePtr, vialayersname[end1layer1][end1layer2],
                        end1height, end1width);
                }
                system(command);
            }
        }
/*
create via - tap or dir and prim
*/
        /* give unique name to via instance */
        (void) sprintf (vianame,"via%d",++vianumber);
        /* setup, create, and attach to CONNECTORS bag
          the via instance in OCT */
end1viaPtr->contents.instance.facet = "contents";
        end1viaPtr->contents.instance.name = vianame;
/*
cell.contents.facet.cell;
*/
        end1viaPtr->contents.instance.master = fullvianame;
        end1viaPtr->contents.instance.view = "physical";
        end1viaPtr->contents.instance.version = OCT_CURRENT_VERSION;
        end1viaPtr->contents.instance.transform.translation.x 
            = points[1].x;
        end1viaPtr->contents.instance.transform.translation.y 
            = points[1].y;
        if (octCreate(&chip, end1viaPtr) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create via instance.");
/*
, %s.", end1viaPtr->contents.instance.name);
*/
            return (BEAR_STOP_SEARCH);
        }
        if (octAttach(&bag, end1viaPtr) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via instance to CONNECTORS bag.");
            return (BEAR_STOP_SEARCH);
        }
        /* get and attach to segment the terminal from the instance */
        terminal.type = OCT_TERM;
        terminal.contents.term.name = "t";
/*
        end1viaPtr->contents.term.instanceId = terminal.objectId;
        if (octCreate(&chip, end1viaPtr) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot create via term, %s.", end1viaPtr->contents.term.name);
        }
        if (octAttach(end1viaPtr, &terminal) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach via terminal instance, %s.", terminal.contents.instance.name);
        }
        if (octGetByName(&terminal, end0viaPtr) == OCT_NOT_FOUND) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term, t, from via instance.");
        }
*/
        if (octGetByName(end1viaPtr, &terminal) == OCT_NOT_FOUND) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot get term, t, from via instance.");
            return (BEAR_STOP_SEARCH);
        }
        if (octAttach(segment, &terminal) != OCT_OK) {
            octError("WriteSegmentsJunctionsConnectionsToOct() : cannot attach segment.");
            return (BEAR_STOP_SEARCH);
        }
    }

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * WriteRoutingOfRRootsToOct() - writes routing of routing roots from MAC to OCT
 */

static BearStatus
WriteRoutingOfRRootsToOct(rRootPtr, net)
root        *rRootPtr;
octObject   *net;
{
    /* go through all segment of the routing root */
    if (Rdb_ItrRSegmOfRRoot(rRootPtr, WriteSegmentsJunctionsConnectionsToOct,
        (ClientData) net) == BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }
    return (BEAR_CONTINUE_SEARCH);
}


/*
 * FreeSegmentsJunctionsClientData() - frees segments', junctions',
 *                                     and terminals' client data
 */
static BearStatus
FreeSegmentsJunctionsClientData(rSegPtr, clientData)
rseg        *rSegPtr;
ClientData  *clientData;
{
    /* 
     * Enable freeing more the once in the memory manager.
     */
    bear_mm_FreeTwiceFlag = TRUE;

    /* free routing segment's client data if exists */
    if (rSegPtr->clientData != NULL) {
        FREE(rSegPtr->clientData);
        rSegPtr->clientData = (ClientData) NULL;
    }
    /* free terminal's or junction's at one end of routing segment if exists */
    if (rSegPtr->t0fsr) {
        if (rSegPtr->j0sr.rt->clientData != NULL) {
            FREE(rSegPtr->j0sr.rt->clientData);
            rSegPtr->j0sr.rt->clientData = (ClientData) NULL;
        }
    } 
    else {
        if (rSegPtr->j0sr.rj->clientData != NULL) {
            FREE(rSegPtr->j0sr.rj->clientData);
            rSegPtr->j0sr.rj->clientData = (ClientData) NULL;
        }
    }
    /* free terminal's or junction's at
       other end of routing segment if exists */
    if (rSegPtr->t1fsr) {
        if (rSegPtr->j1sr.rt->clientData != NULL) {
            FREE(rSegPtr->j1sr.rt->clientData);
            rSegPtr->j1sr.rt->clientData = (ClientData) NULL;
        }
    } 
    else {
        if (rSegPtr->j1sr.rj->clientData != NULL) {
            FREE(rSegPtr->j1sr.rj->clientData);
            rSegPtr->j1sr.rj->clientData = (ClientData) NULL;
        }
    }

    /* 
     * Disable freeing more the once in the memory manager.
     */
    bear_mm_FreeTwiceFlag = FALSE;

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * FreeRRootsSegmentsJunctions() - frees client data of routing roots'
 *                                 segments, junctions, and terminals
 */
static BearStatus
FreeRRootsSegmentsJunctions(rRootPtr, clientData)
root        *rRootPtr;
ClientData  *clientData;
{
    /* go through all the segment ofthe routing root and free clienta data */
    (void) Rdb_ItrRSegmOfRRoot(rRootPtr, FreeSegmentsJunctionsClientData,
        (ClientData) NULL);
    return (BEAR_CONTINUE_SEARCH);
}


/*
 * InitializeSegmentsJunctionsClientData() - initializes segments', junctions',
 *                                           and terminals' client data
 */
static BearStatus
InitializeSegmentsJunctionsClientData(rSegPtr, clientData)
rseg        *rSegPtr;
ClientData  *clientData;
{
    /* initialize routing segment's cleint data */
    rSegPtr->clientData = (ClientData) NULL;
    /* initialize terminal's or junction's cleint data
       at one end of routing segment */
    if (rSegPtr->t0fsr) {
        rSegPtr->j0sr.rt->clientData = (ClientData) NULL;
    } 
    else {
        rSegPtr->j0sr.rj->clientData = (ClientData) NULL;
    }
    /* initialize terminal's or junction's cleint data
       at other end of routing segment */
    if (rSegPtr->t1fsr) {
        rSegPtr->j1sr.rt->clientData = (ClientData) NULL;
    } 
    else {
        rSegPtr->j1sr.rj->clientData = (ClientData) NULL;
    }

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * InitializeRRootsSegmentsJunctions() - initializes cleint data of routing
 *                                       roots' segments, junctions,
 *                                       and terminals
 */
static BearStatus
InitializeRRootsSegmentsJunctions(rRootPtr, clientData)
root        *rRootPtr;
ClientData  *clientData;
{
    /* go through all routing segments of
       routing root and initialize client data */
    (void) Rdb_ItrRSegmOfRRoot(rRootPtr, InitializeSegmentsJunctionsClientData,
        (ClientData) NULL);
    return (BEAR_CONTINUE_SEARCH);
}


/*
 * WriteRoutingOfNetsToOct() - writes routing of nets from MAC to OCT
 */

static BearStatus
WriteRoutingOfNetsToOct(netPtr, chip)
Net         *netPtr;
octObject   *chip;
{
    octObject net;

/*
netPtr->name
layer
*/
    /* go through all routing roots of net and initialize
       client data of segments, junctions, and terminals */
    if (Mac_ItrRRootOfNet(netPtr, InitializeRRootsSegmentsJunctions,
        (ClientData) NULL) == BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }

    /* Get the net. */
    if (ohGetByNetName(chip, &net, 
        (char *) Mac_GetNameOfNet(netPtr)) == OCT_NOT_FOUND) {
        octError("WriteRoutingOfNetsToOct(): cannot get net");
/*
, %s.", net.contents.net.name);
*/
        return (BEAR_STOP_SEARCH);
    }
    (void) fprintf (bearout,
        "Writing routing of net : %s\n", net.contents.net.name);

    /* go through all routing roots of net and write the routing to OCT */
    if (Mac_ItrRRootOfNet(netPtr, WriteRoutingOfRRootsToOct,
        (ClientData) &net) == BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }

    /* go through all routing roots of net and free
       client data of segments, junctions, and terminals */
    if (Mac_ItrRRootOfNet(netPtr, FreeRRootsSegmentsJunctions,
        (ClientData) NULL) == BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * WriteRoutingToOct() - writes the routing from MAC to OCT
 */
BearStatus
OdiWriteRoutingToOct(cellMasterPtr, chip)
CellMaster  *cellMasterPtr;
octObject   *chip;
{
    char *stringPtr;

    stringPtr = util_logical_expand("$OCTTOOLS");
    odiOctHomePtr = util_strsav(stringPtr);

    /* initialize distinct part of via names for each chip */
    vianumber = 0;
    /* go through all the nets and write out their routing information to OCT */
    if (Mac_ItrNet(Mac_GetNetListOfCell(cellMasterPtr),
        WriteRoutingOfNetsToOct, (ClientData) chip) == BEAR_STOP_SEARCH) {
        return (BEAR_ERROR);
    }

    FREE(odiOctHomePtr);

    return (BEAR_OK);
}

#endif /* USE_DR_CODE */

