
/* pbARUtils.c --
 *
 *    Sherry Solden
 *    (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifdef USE_DR_CODE

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "deque.h"
#include "grEditor.h"
#include "geoChDecomposer.h"
#include "topChDecomposer.h"
#include "globalSpacer.h"
#include "routeDB.h"
#include "tileMapping.h"
#include "polarBear.h"
#include "stack.h"
#include "pbDetour.h"


typedef struct _LocalData
{
    Point        *pointPtr;
    WallJunction *junctPtr;
} LocalData;

/*  inserted by Sherry for fixBadNets during unification */
typedef struct _LocalData2
{
   BearId intId;
   BearId extId;
} LocalData2;

typedef struct _AdjacentData
{
   PlaneData *planeDataPtr;
   Tile      *tilePtr;
   RouteCell *rCellPtr;
} AdjacentData;

/* Status bits for type of channel */
#define alive 1
#define halfDead 2
#define allDead 3
#define leftDead 4
#define rightDead 5
#define topDead 6
#define botDead 7
#define bothDead 8


extern BearStatus      MarkGPin();
extern BearStatus      InsertRPin();
extern BearStatus      GenNewId();
extern BearStatus      ReverseDir();
extern PinDirection    OppositeDir();
extern void            CreateNewCellGroup();
extern void            LCreateNewCellGroup();
extern BearStatus      DefineAdjacentTile();
extern int             FindAdjacentTile();
extern char            *GetReply();
extern WallJunction    *GetJunction();
extern BearStatus      CheckPoint();
extern int             GetStatusOfChannel();
extern int             GetStatusOfWallSeg();
extern void            GetLAndRTiles();
extern void            GetBAndTTiles();
extern void            PinsAroundVWallSect();
extern void            PinsAroundHWallSect();
extern void            SearchTAndBJunct();
extern void            SearchRAndLJunct();
extern void            FixAllBadNets();
extern BearStatus      ChangeBadPinIds();
extern void            CleanPinList();
extern BearStatus      FindPPin();
extern bool            PPinOnTRSideOfTile();
extern BearStatus      CheckTopLeftSidePins();
extern BearStatus      CheckBottomRightSidePins();
extern void            RemoveSingleSidePins();
extern void            AbortRouteChannel();
extern void            AbortLChannel();


/* ---------- Local Procedures ---------- */
static bool FindRPinMatch();
static bool FindGPinMatch();

/* New set of procedures to match by extId instead of netPtr */
static bool FindRPinMatchById();
static bool FindGPinMatchById();


void
AbortRouteChannel(cellMasterPtr, planeDataPtr,
    rCellPtr, point1, point2, adjParPtr, dCellListsPtr, status)

CellMaster        *cellMasterPtr;
PlaneData         *planeDataPtr;
RouteCell         *rCellPtr;
Point             point1;
Point             point2;
AdjPar            *adjParPtr;
DCellsAroundRCell *dCellListsPtr;
int               status;
{
    Rect         dmntRect;
    Rect         vdmntRect;
    WallJunction *llJunctPtr;
    WallJunction *junct1Ptr;
    WallJunction *junct2Ptr;
    WallSegment  *wallSegPtr;
    BottleNeck   *bottleNeckPtr; 
    BNLine       *bnLinePtr;
    Tile         *bnTilePtr;
    int          density;

    if (Mac_GetTypeOfRCell(rCellPtr) == HORZ_RCELL) {
        /* Vertical Split */
        if (status == halfDead) {
            /* Need to split the circle - Tm_VCircleSplit */
            if (dCellListsPtr->BRptr) {
                dmntRect.r_xbot = dCellListsPtr->BRpoint->p_x;
                dmntRect.r_xtop = point2.p_x;
                dmntRect.r_ytop = dCellListsPtr->BRpoint->p_y;
                dmntRect.r_ybot = dmntRect.r_ytop - 1;
                Tp_HRectToVRect(&dmntRect, &vdmntRect);
                (void) Tm_VCSplit(cellMasterPtr, planeDataPtr, vdmntRect);
            }
            if (dCellListsPtr->BLptr) {
                dmntRect.r_xtop = dCellListsPtr->BLpoint->p_x;
                dmntRect.r_xbot = point1.p_x;
                dmntRect.r_ytop = dCellListsPtr->BLpoint->p_y;
                dmntRect.r_ybot = dmntRect.r_ytop - 1;
                Tp_HRectToVRect(&dmntRect, &vdmntRect);
                (void) Tm_VCSplit(cellMasterPtr, planeDataPtr, vdmntRect);
            }
            if (dCellListsPtr->TRptr) {
                dmntRect.r_xbot = dCellListsPtr->TRpoint->p_x;
                dmntRect.r_xtop = point2.p_x;
                dmntRect.r_ybot = dCellListsPtr->TRpoint->p_y;
                dmntRect.r_ytop = dmntRect.r_ybot + 1;
                Tp_HRectToVRect(&dmntRect, &vdmntRect);
                (void) Tm_VCSplit(cellMasterPtr, planeDataPtr, vdmntRect);
            }
            if (dCellListsPtr->TLptr) {
                dmntRect.r_xtop = dCellListsPtr->TLpoint->p_x;
                dmntRect.r_xbot = point1.p_x;
                dmntRect.r_ybot = dCellListsPtr->TLpoint->p_y;
                dmntRect.r_ytop = dmntRect.r_ybot + 1;
                Tp_HRectToVRect(&dmntRect, &vdmntRect);
                (void) Tm_VCSplit(cellMasterPtr, planeDataPtr, vdmntRect);
            }
        }
        Tm_InsDCellOnVTP(cellMasterPtr);
        Tm_VFSplit(cellMasterPtr);

#ifdef USE_WIRE_CODE
        if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
            Bear_PrintErr("AbortRouteChannel() : failed 1");
        }
#endif USE_WIRE_CODE
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
        llJunctPtr = Mac_GetFpgOfCell(cellMasterPtr);
        junct1Ptr = GetJunction(llJunctPtr, &point1);
        junct2Ptr = GetJunction(llJunctPtr, &point2);
        while (junct1Ptr != junct2Ptr) {
            wallSegPtr = Fpg_GetEastSegOfWallJunct(junct1Ptr);
            bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
            if (bottleNeckPtr->unType == TP_BN_TILE) {
                bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
                /*
                 * We put -density to the density field for the
                 * next density estimation.
                 */
                density = Tp_GetCapacityOfTile(bnTilePtr) + adjParPtr->yAdj;
                Tp_PutDensityOfTile(-density, bnTilePtr);
            }
            else {
                bnLinePtr = bottleNeckPtr->unValue.bnLinePtr;
                density = Tp_GetCapacityOfBnLine(bnLinePtr) + adjParPtr->yAdj;
                Tp_PutDensityOfBnLine(-density, bnLinePtr);
            }
            junct1Ptr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
        }
    }
    else {
        /* Horizontal Split */
        if (status == halfDead) {
            /* Need to split the circle  Tm_HCircleSplit */
            if (dCellListsPtr->BRptr) {
                dmntRect.r_xbot = dCellListsPtr->BRpoint->p_x;
                dmntRect.r_xtop = dmntRect.r_xbot + 1;
                dmntRect.r_ytop = point1.p_y;
                dmntRect.r_ybot = dCellListsPtr->BRpoint->p_y;
                (void) Tm_HCSplit(cellMasterPtr, planeDataPtr, dmntRect);
            }
            if (dCellListsPtr->BLptr) {
                dmntRect.r_xbot = dCellListsPtr->BLpoint->p_x;
                dmntRect.r_xtop = dmntRect.r_xbot + 1;
                dmntRect.r_ybot = point2.p_y;
                dmntRect.r_ytop = dCellListsPtr->BLpoint->p_y;
                (void) Tm_HCSplit(cellMasterPtr, planeDataPtr, dmntRect);
            }
            if (dCellListsPtr->TRptr) {
                dmntRect.r_xtop = dCellListsPtr->TRpoint->p_x;
                dmntRect.r_xbot = dmntRect.r_xtop - 1;
                dmntRect.r_ytop = point1.p_y;
                dmntRect.r_ybot = dCellListsPtr->TRpoint->p_y;
                (void) Tm_HCSplit(cellMasterPtr, planeDataPtr, dmntRect);
            }
            if (dCellListsPtr->TLptr) {
                dmntRect.r_xtop = dCellListsPtr->TLpoint->p_x;
                dmntRect.r_xbot = dmntRect.r_xtop - 1;
                dmntRect.r_ybot = point2.p_y;
                dmntRect.r_ytop = dCellListsPtr->TLpoint->p_y;
                (void) Tm_HCSplit(cellMasterPtr, planeDataPtr, dmntRect);
            }
        }
        Tm_InsDCellOnHTP(cellMasterPtr);
        Tm_HFSplit(cellMasterPtr);

#ifdef USE_WIRE_CODE
        if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
            Bear_PrintErr("AbortRouteChannel() : failed 2");
        }
#endif USE_WIRE_CODE
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
        llJunctPtr = Mac_GetFpgOfCell(cellMasterPtr);
        junct1Ptr = GetJunction(llJunctPtr, &point1);
        junct2Ptr = GetJunction(llJunctPtr, &point2);
        while (junct1Ptr != junct2Ptr) {
            wallSegPtr = Fpg_GetSouthSegOfWallJunct(junct1Ptr);
            bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
            if (bottleNeckPtr->unType == TP_BN_TILE) {
                bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
                /*
                 * We put -density to the density field for the
                 * next density estimation.
                 */
                density = Tp_GetCapacityOfTile(bnTilePtr) + adjParPtr->xAdj;
                Tp_PutDensityOfTile(-density, bnTilePtr);
            }
            else {
                bnLinePtr = bottleNeckPtr->unValue.bnLinePtr;
                density = Tp_GetCapacityOfBnLine(bnLinePtr) + adjParPtr->xAdj;
                Tp_PutDensityOfBnLine(-density, bnLinePtr);
            }
            junct1Ptr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
        }
    }

    Mac_ItrBLBoundPinOfRCell(rCellPtr,ReverseDir,(ClientData)NULL);
    Mac_ItrTRBoundPinOfRCell(rCellPtr,ReverseDir,(ClientData)NULL);
    if (Mac_DeleteRCell(rCellPtr) == BEAR_ERROR) {
        Bear_PrintErr("AbortRouteChannel() : failed 3");
    }
    Rdb_FreeChip();
}



void
CreateNewCellGroup(planeDataPtr, rCellPtr)
PlaneData    *planeDataPtr;
RouteCell    *rCellPtr;
{
    AdjacentData    adjacentData;
    ClientData      blCellPtr;
    ClientData      trCellPtr;
    CellClass       blCellClass;
    CellClass       trCellClass;
    CellGroup       *rCellGroupPtr;
    CellGroup       *blCellGroupPtr;
    CellGroup       *trCellGroupPtr;
    CellGroup       *tempCellGroupPtr;
    CellGroup       *newCellGroupPtr;
    Tile            *blTilePtr;
    Tile            *trTilePtr;

    adjacentData.planeDataPtr = planeDataPtr;
    adjacentData.rCellPtr = rCellPtr;
    adjacentData.tilePtr = NIL(Tile);
    Mac_ItrLinkedPoint(Mac_GetBLBoundOfRCell(rCellPtr),
               DefineAdjacentTile,
               (ClientData) &adjacentData);
    blTilePtr = adjacentData.tilePtr;
    adjacentData.tilePtr = NIL(Tile);
    Mac_ItrLinkedPoint(Mac_GetTRBoundOfRCell(rCellPtr),
               DefineAdjacentTile,
               (ClientData) &adjacentData);
    trTilePtr = adjacentData.tilePtr;

    if (Tp_IsCellInstTile(blTilePtr))
    {
    blCellPtr   = (ClientData) Tp_GetCellInstOfTile(blTilePtr);
    blCellClass = CELL_INST;
    }
    else if (Tp_IsRCellTile(blTilePtr))
    {
    blCellPtr   = (ClientData) Tp_GetRCellOfTile(blTilePtr);
    blCellClass = ROUTE_CELL;
    }
    else
    {
    Bear_PrintFatalErr("CreateNewCellGroup() : blCellPtr is not Generic Cell");
    }
    if (Tp_IsCellInstTile(trTilePtr))
    {
    trCellPtr   = (ClientData) Tp_GetCellInstOfTile(trTilePtr);
    trCellClass = CELL_INST;
    }
    else if (Tp_IsRCellTile(trTilePtr))
    {
    trCellPtr   = (ClientData) Tp_GetRCellOfTile(trTilePtr);
    trCellClass = ROUTE_CELL;
    }
    else
    {
    Bear_PrintFatalErr("CreateNewCellGroup() : trCellPtr is not Generic Cell");
    }
    (void) Mac_CreateCellGroup(ROUTE_CELL, (ClientData)rCellPtr,
                   &rCellGroupPtr);
    if (Mac_IsGroupedCell(blCellClass, blCellPtr))
    {
    blCellGroupPtr = blCellClass == CELL_INST ?
             Mac_GetGroupOfCell((CellInst *)blCellPtr) :
             Mac_GetGroupOfRCell((RouteCell *)blCellPtr);
    }
    else
    {
    (void) Mac_CreateCellGroup(blCellClass, blCellPtr, &blCellGroupPtr);
    }
    if (Mac_IsGroupedCell(trCellClass, trCellPtr))
    {
    trCellGroupPtr = trCellClass == CELL_INST ?
             Mac_GetGroupOfCell((CellInst *)trCellPtr) :
             Mac_GetGroupOfRCell((RouteCell *)trCellPtr);
    }
    else
    {
    (void) Mac_CreateCellGroup(trCellClass, trCellPtr, &trCellGroupPtr);
    }
    (void) Mac_MergeCellGroups(blCellGroupPtr, trCellGroupPtr,
                   &tempCellGroupPtr);
    (void) Mac_MergeCellGroups(tempCellGroupPtr, rCellGroupPtr,
                   &newCellGroupPtr);
}



BearStatus
DefineAdjacentTile(linkedPointPtr, adjacentDataPtr)
LinkedPoint *linkedPointPtr;
AdjacentData *adjacentDataPtr;
{
    Point point1;
    Point point2;
    Point minPoint;
    Point maxPoint;
    Rect  searchRect;
    Rect  vsearchRect;
    Transform *transPtr;

    if (linkedPointPtr->nextPtr == NIL(LinkedPoint))
    { 
    return (BEAR_STOP_SEARCH);
    }

    transPtr = Mac_GetTransformOfRCell(adjacentDataPtr->rCellPtr);
    GeoTransPoint(transPtr, &linkedPointPtr->point, &point1);
    GeoTransPoint(transPtr, &linkedPointPtr->nextPtr->point, &point2);
    minPoint.p_x = MIN(point1.p_x, point2.p_x);
    minPoint.p_y = MIN(point1.p_y, point2.p_y);
    maxPoint.p_x = MAX(point1.p_x, point2.p_x);
    maxPoint.p_y = MAX(point1.p_y, point2.p_y);

    if (point1.p_y == point2.p_y)
    {
    searchRect.r_xbot = point1.p_x;
    searchRect.r_ybot = point1.p_y - 1;
    searchRect.r_xtop = point2.p_x;
    searchRect.r_ytop = point2.p_y + 1;
    TiSrArea(NIL(Tile), Tp_GetHorzPlane(adjacentDataPtr->planeDataPtr),
         &searchRect, FindAdjacentTile, (ClientData)adjacentDataPtr);
    if (adjacentDataPtr->tilePtr != NIL(Tile))
    {
        return (BEAR_STOP_SEARCH);
    }
    }
    else
    {
        searchRect.r_xbot = point1.p_x - 1;
        searchRect.r_ybot = point1.p_y;
        searchRect.r_xtop = point2.p_x + 1;
        searchRect.r_ytop = point2.p_y;
        Tp_HRectToVRect(&searchRect, &vsearchRect);
        TiSrArea(NIL(Tile), Tp_GetVertPlane(adjacentDataPtr->planeDataPtr),
             &vsearchRect, FindAdjacentTile, (ClientData)adjacentDataPtr);
        if (adjacentDataPtr->tilePtr != NIL(Tile))
        {
            return (BEAR_STOP_SEARCH);
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



int
FindAdjacentTile(tilePtr, adjacentDataPtr)
Tile *tilePtr;
AdjacentData *adjacentDataPtr;
{
    if (Tp_IsCellInstTile(tilePtr) || Tp_IsRCellTile(tilePtr))
    {
        adjacentDataPtr->tilePtr = tilePtr;
        return (1);
    }
    return (0);
}



WallJunction *
GetJunction(llJunctPtr, pointPtr)
WallJunction  *llJunctPtr;
Point         *pointPtr;
{
    LocalData  localData;

    localData.pointPtr = pointPtr;
    if (Fpg_ItrWallJunct(llJunctPtr, CheckPoint, (ClientData)&localData) ==
        BEAR_CONTINUE_SEARCH)
    {
        Bear_PrintFatalErr("GetJunction() failed: can't find junction");
    }
    return (localData.junctPtr);
}



BearStatus
CheckPoint(junctPtr, localDataPtr)
WallJunction  *junctPtr;
LocalData     *localDataPtr;
{
    Point  *pointPtr;

    /* Attempt to find junction within 1-unit of specified point */
    /* in case the junction point shifts during unify/split of bnLine/dCell */
    pointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, junctPtr);
    if ((abs(pointPtr->p_x - localDataPtr->pointPtr->p_x) <= 1) &&
    (abs(pointPtr->p_y - localDataPtr->pointPtr->p_y) <= 1))
    {
    localDataPtr->junctPtr = junctPtr;
    return (BEAR_STOP_SEARCH);
    }
    return (BEAR_CONTINUE_SEARCH);
}

/* Procedure checks the status of the channel.
 * Definitions of Status:
 *    alive channel
 *       all segments of the channel are alive.
 *    halfDead channel -
 *       at least one segment of the channel is HalfDead, 
 *       but not all segments can be HalfDead (on the same side)
 *    allDead channel
 *       all segments of the channel are HalfDead.
 *    halfDead segment - wallSegment is adjacent to a dummyCell.
 * 
 * Assumption: Empty cells can only exist at the "edges" of the channel
*/
int
GetStatusOfChannel(planeDataPtr, tlJunctPtr, brJunctPtr, type)
PlaneData    *planeDataPtr;
WallJunction *tlJunctPtr;
WallJunction *brJunctPtr;
ChannelType  type;
{
    WallSegment  *segPtr;
    WallJunction *tmpJunctPtr;
    int          status;
    int          seg1Status;

    if (type == VERT_CHANNEL)
    {
       segPtr = Fpg_GetSouthSegOfWallJunct(tlJunctPtr);
       status = GetStatusOfWallSeg(planeDataPtr, segPtr);
       if (status == alive)
       {
           segPtr = Fpg_GetNorthSegOfWallJunct(brJunctPtr);
           status = GetStatusOfWallSeg(planeDataPtr, segPtr);
           if (status != alive)
           return (halfDead);
           else return (alive);
       }
       else
       {
          seg1Status = status;
          while ((tmpJunctPtr = Fpg_GetSEJunctOfWallSeg(segPtr)) != brJunctPtr)
          {
          segPtr = Fpg_GetSouthSegOfWallJunct(tmpJunctPtr);
          status = GetStatusOfWallSeg(planeDataPtr, segPtr);
          if (status != seg1Status)
              return (halfDead);
          }
          return (allDead);
       }
    }
    else
    {
       segPtr = Fpg_GetEastSegOfWallJunct(tlJunctPtr);
       status = GetStatusOfWallSeg(planeDataPtr, segPtr);
       if (status == alive)
       {
           segPtr = Fpg_GetWestSegOfWallJunct(brJunctPtr);
           status = GetStatusOfWallSeg(planeDataPtr, segPtr);
           if (status != alive)
           return (halfDead);
           else return (alive);
       }
       else
       {
          seg1Status = status;
          while ((tmpJunctPtr = Fpg_GetSEJunctOfWallSeg(segPtr)) != brJunctPtr)
          {
          segPtr = Fpg_GetEastSegOfWallJunct(tmpJunctPtr);
          status = GetStatusOfWallSeg(planeDataPtr, segPtr);
          if (status != seg1Status)
              return (halfDead);
          }
          return (allDead);
       }
       }
}


int
GetStatusOfWallSeg(planeDataPtr, segPtr)
PlaneData   *planeDataPtr;
WallSegment *segPtr;
{
   BottleNeck *bottleNeckPtr;
   Tile *rightTilePtr;
   Tile *topTilePtr;
   Tile *botTilePtr;
   Tile *leftTilePtr;

   bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(segPtr);
   if (segPtr->direction == FPG_VERTICAL)
   {
       GetLAndRTiles(planeDataPtr, bottleNeckPtr, &leftTilePtr, &rightTilePtr);
       if (Tp_IsDCellTile(leftTilePtr) && Tp_IsDCellTile(rightTilePtr))
       return (bothDead);
       if (Tp_IsDCellTile(leftTilePtr))
       return (leftDead);
       if (Tp_IsDCellTile(rightTilePtr))
       return (rightDead);
       return (alive);
   }
   else
   {
       GetBAndTTiles(planeDataPtr, bottleNeckPtr, &botTilePtr, &topTilePtr);
       if (Tp_IsDCellTile(botTilePtr) && Tp_IsDCellTile(topTilePtr))
       return (bothDead);
       if (Tp_IsDCellTile(botTilePtr))
       return (botDead);
       if (Tp_IsDCellTile(topTilePtr))
       return (topDead);
       return (alive);
   }
}



void
GetLAndRTiles(planeDataPtr, bottleNeckPtr, leftTilePtrPtr, rightTilePtrPtr)
PlaneData *planeDataPtr;
BottleNeck *bottleNeckPtr;
Tile      **leftTilePtrPtr;
Tile      **rightTilePtrPtr;
{
    Point        searchPnt;
    Rect         rect;
    Plane        *planePtr;
    Tile         *bnTilePtr;
    BNLine       *bnLinePtr;

    planePtr = Tp_GetHorzPlane(planeDataPtr);
    if (bottleNeckPtr->unType == TP_BN_TILE)
    {
       bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
       (void) Tp_HorzTileToRect(planeDataPtr, bnTilePtr, &rect);
       searchPnt.p_x = rect.r_ll.p_x - 1;
       searchPnt.p_y = rect.r_ll.p_y;
    }
    else
    {
       bnLinePtr = bottleNeckPtr->unValue.bnLinePtr;
       searchPnt.p_x = (bnLinePtr->line.point1.p_x) - 1;
       searchPnt.p_y = bnLinePtr->line.point1.p_y;
    }
    *leftTilePtrPtr = TiSrPointNoHint(planePtr, &searchPnt);

    if (bottleNeckPtr->unType == TP_BN_TILE)
    {
        searchPnt.p_x = rect.r_ur.p_x;
        searchPnt.p_y = rect.r_ll.p_y;
    }
    else
    {
       searchPnt.p_x = bnLinePtr->line.point2.p_x;
       searchPnt.p_y = bnLinePtr->line.point2.p_y;
    }
    *rightTilePtrPtr = TiSrPointNoHint(planePtr, &searchPnt);
}



void
GetBAndTTiles(planeDataPtr, bottleNeckPtr, bottomTilePtrPtr, topTilePtrPtr)
PlaneData *planeDataPtr;
BottleNeck *bottleNeckPtr;
Tile      **bottomTilePtrPtr;
Tile      **topTilePtrPtr;
{
    Point        searchPnt;
    Point        vSearchPnt;
    Rect         rect;
    Plane        *planePtr;
    Tile         *bnTilePtr;
    BNLine       *bnLinePtr;

    planePtr = Tp_GetVertPlane(planeDataPtr);
    if (bottleNeckPtr->unType == TP_BN_TILE)
    {
       bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
       (void) Tp_VertTileToRect(planeDataPtr, bnTilePtr, &rect);
       searchPnt.p_x = rect.r_ur.p_x;
       searchPnt.p_y = rect.r_ll.p_y - 1;
    }
    else
    {
       bnLinePtr = bottleNeckPtr->unValue.bnLinePtr;
       searchPnt.p_x = bnLinePtr->line.point2.p_x;
       searchPnt.p_y = (bnLinePtr->line.point2.p_y) - 1;
    }
    Tp_HPointToVPoint(&searchPnt, &vSearchPnt);
    *bottomTilePtrPtr = TiSrPointNoHint(planePtr, &vSearchPnt);

    if (bottleNeckPtr->unType == TP_BN_TILE)
    {
       searchPnt.p_x = rect.r_ur.p_x;
       searchPnt.p_y = rect.r_ur.p_y;
    }
    else
    {
       searchPnt.p_x = bnLinePtr->line.point1.p_x;
       searchPnt.p_y = bnLinePtr->line.point1.p_y;
    }
    Tp_HPointToVPoint(&searchPnt, &vSearchPnt);
    *topTilePtrPtr = TiSrPointNoHint(planePtr, &vSearchPnt);
}



void
AbortLChannel(rCellPtr, junct1Ptr, junct2Ptr, junct3Ptr, adjParPtr)
RouteCell    *rCellPtr;
WallJunction *junct1Ptr;
WallJunction *junct2Ptr;
WallJunction *junct3Ptr;
AdjPar       *adjParPtr;
{
    RCellType    rCellType;
    WallJunction *junctPtr;
    WallSegment  *wallSegPtr;
    BottleNeck   *bottleNeckPtr;
    Tile         *bnTilePtr;
    BNLine       *bnLinePtr;
    int          density;

    rCellType = Mac_GetTypeOfRCell(rCellPtr);
    junctPtr = junct1Ptr;
    while (junctPtr != junct2Ptr)
    {
    if (rCellType==TOP_RIGHT_L_RCELL || rCellType==BOTTOM_RIGHT_RCELL)
    {
        wallSegPtr = Fpg_GetWestSegOfWallJunct(junctPtr);
    }
    else 
    {
        wallSegPtr = Fpg_GetEastSegOfWallJunct(junctPtr);
    }
    bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
    if (bottleNeckPtr->unType == TP_BN_TILE)
    {
        bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
        /*
         * We put -adjPar to the density field for the
         * next density estimation.
         */
        density = Tp_GetCapacityOfTile(bnTilePtr) + adjParPtr->yAdj;
        Tp_PutDensityOfTile(-density, bnTilePtr);
    }
    else
    {
        bnLinePtr = bottleNeckPtr->unValue.bnLinePtr;
        density = Tp_GetCapacityOfBnLine(bnLinePtr) + adjParPtr->yAdj;
        Tp_PutDensityOfBnLine(-density, bnLinePtr);
    }
    if (rCellType==TOP_RIGHT_L_RCELL || rCellType==BOTTOM_RIGHT_RCELL)
    {
        junctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
    }
    else 
    {
        junctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
    }
    }
    junctPtr = junct2Ptr;
    while (junctPtr != junct3Ptr)
    {
    if (rCellType==TOP_RIGHT_L_RCELL || rCellType==TOP_LEFT_L_RCELL)
    {
        wallSegPtr = Fpg_GetNorthSegOfWallJunct(junctPtr);
    }
    else 
    {
        wallSegPtr = Fpg_GetSouthSegOfWallJunct(junctPtr);
        }
    bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
    if (bottleNeckPtr->unType == TP_BN_TILE)
    {
        bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
        /*
         * We put -adjPar to the density field for the
         * next density estimation.
         */
        density = Tp_GetCapacityOfTile(bnTilePtr) + adjParPtr->xAdj;
        Tp_PutDensityOfTile(-density, bnTilePtr);
    }
    else
    {
        bnLinePtr = bottleNeckPtr->unValue.bnLinePtr;
        density = Tp_GetCapacityOfBnLine(bnLinePtr) + adjParPtr->xAdj;
        Tp_PutDensityOfBnLine(-density, bnLinePtr);
    }
    if (rCellType==TOP_RIGHT_L_RCELL || rCellType==TOP_LEFT_L_RCELL)
    {
        junctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
    }
    else 
    {
        junctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
    }
    }
    Mac_ItrBLBoundPinOfRCell(rCellPtr,ReverseDir,(ClientData)NULL);
    Mac_ItrTRBoundPinOfRCell(rCellPtr,ReverseDir,(ClientData)NULL);
    if (Mac_DeleteRCell(rCellPtr) == BEAR_ERROR) {
        Bear_PrintErr("AbortLChannel() : failed");
    }
    Rdb_FreeChip();
}



void 
FixAllBadNets(PPinList, faceListPtr)
LinkedPPin *PPinList;
PinsAroundJunction *faceListPtr;
{
    /* For each pin' in PPinList if intId != extId then need to fix net ids. */
    /*  Specifically fix the pins in faceListPtr that belong to same net */
    /*  as pin'. */

    BearId    intId;
    BearId    extId;
    LocalData2 localData;

    while (PPinList != NIL(LinkedPPin))
    {
        intId = Tp_GetIntNetIdOfPPin(PPinList->ppinPtr);
        extId = Tp_GetExtNetIdOfPPin(PPinList->ppinPtr);
        if (intId != extId)
        {
            localData.intId = intId;
            localData.extId = extId;
            ChangeBadPinIds(faceListPtr, &localData);
            (void) Tp_PutIntNetIdOfPPin(extId, PPinList->ppinPtr);
        }
        PPinList = PPinList->nextPtr;
    }
}



BearStatus
ChangeBadPinIds(faceListPtr, localDataPtr)
PinsAroundJunction *faceListPtr;
LocalData2 *localDataPtr;
{
    BearId extId;
    LinkedPPin *ppinList;
    LinkedPinInst *pinInstList;
    LinkedRPin    *rPinList;

    ppinList = faceListPtr->horzPPinList;
    while (ppinList != NIL(LinkedPPin))
    {
       extId = Tp_GetExtNetIdOfPPin(ppinList->ppinPtr);
       if (extId == localDataPtr->intId)
       {
         (void) Tp_PutIntNetIdOfPPin(localDataPtr->extId, ppinList->ppinPtr);
         (void) Tp_PutExtNetIdOfPPin(localDataPtr->extId, ppinList->ppinPtr);
       }
       ppinList = ppinList->nextPtr;
    }
    ppinList = faceListPtr->vertPPinList;
    while (ppinList != NIL(LinkedPPin))
    {
       extId = Tp_GetExtNetIdOfPPin(ppinList->ppinPtr);
       if (extId == localDataPtr->intId)
       {
         (void) Tp_PutExtNetIdOfPPin(localDataPtr->extId, ppinList->ppinPtr);
       }
       ppinList = ppinList->nextPtr;
    }
    pinInstList = faceListPtr->pinInstList;
    while (pinInstList != NIL(LinkedPinInst))
    {
       extId = Mac_GetExtNetIdOfPin(pinInstList->pinInstPtr);
       if (extId == localDataPtr->intId)
       {
         (void) Mac_PutExtNetIdOfPin(localDataPtr->extId, 
                                     pinInstList->pinInstPtr);
       }
       pinInstList = pinInstList->nextPtr;
    }
    rPinList = faceListPtr->rPinList;
    while (rPinList != NIL(LinkedRPin))
    {
       extId = Mac_GetExtNetIdOfRPin(rPinList->rPinPtr);
       if (extId == localDataPtr->intId)
       {
         (void) Mac_PutExtNetIdOfRPin(localDataPtr->extId, 
                                      rPinList->rPinPtr);
       }
       rPinList = rPinList->nextPtr;
    }
}



/* ARGSUSED */
BearStatus
MarkGPin(gpinPtr, clientData)
GenericPin   *gpinPtr;
ClientData   clientData;
{
    if (gpinPtr->pinType == PIN_INST) {
        Mac_MarkPort(Mac_GetPortOfPin(gpinPtr->pinPtrUn.pinInstPtr));
    }
    else {
        RoutePin *rpinPtr;

        rpinPtr = gpinPtr->pinPtrUn.routePinPtr;
        Mac_MarkRPin(rpinPtr);
        switch (Mac_GetDirectionOfRPin(rpinPtr)) {
        case NORTH_PIN:
        case SOUTH_PIN:
            Mac_PutDirectionOfRPin(NORTH_SOUTH_PIN, rpinPtr);
            break;
        case EAST_PIN:
        case WEST_PIN:
            Mac_PutDirectionOfRPin(EAST_WEST_PIN, rpinPtr);
            break;
        default:
            Bear_PrintFatalErr("MarkGPin() : Illegal pin direction\n");
            break;
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



/* ARGSUSED */
BearStatus
InsertRPin(rPinPtr,clientData)
RoutePin    *rPinPtr;
ClientData  clientData;
{
    Mac_InsertRPinOfNet(Mac_GetNetPtrOfRPin(rPinPtr),rPinPtr);

    return (BEAR_CONTINUE_SEARCH);
}



BearStatus
GenNewId(rPinPtr, pinListPtr)
RoutePin           *rPinPtr;
PinsAroundJunction *pinListPtr;
{
    BearId        id;
    BearId        newId;
    BearId        oldId;
    LinkedPPin    *curLinkedPPinPtr;
    LinkedPinInst *curLinkedPinInstPtr;
    LinkedRPin    *curLinkedRPinPtr;
    Point         point;
    Tile          *tilePtr;
    bool          useExtId;

    newId = Mac_GenerateId();
    oldId = Mac_GetExtNetIdOfRPin(rPinPtr);
    Mac_PutExtNetIdOfRPin(newId,rPinPtr);
    for (curLinkedPPinPtr  = pinListPtr->horzPPinList;
        curLinkedPPinPtr != NIL(LinkedPPin);
        curLinkedPPinPtr  = curLinkedPPinPtr->nextPtr)
    {
        tilePtr = Tp_GetTileOfPPin(curLinkedPPinPtr->ppinPtr);
        if (Tp_IsTileFilled(tilePtr)) 
        {
            Mac_GetAbsPosOfRPin(rPinPtr, &point);
            if (Tp_IsSBNTile(tilePtr))
            {
                if (TiBottom(tilePtr) > point.p_y)
                {
                    id = Tp_GetIntNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = FALSE;
                }
                else if (TiTop(tilePtr) < point.p_y)
                {
                    id = Tp_GetExtNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = TRUE;
                }
                else if (PPinOnTRSideOfTile(curLinkedPPinPtr->ppinPtr, tilePtr))
                {
                    id = Tp_GetIntNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = FALSE;
                }
                else
                {
                    id = Tp_GetExtNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = TRUE;
                }
            }
            else
            {
                id = Tp_GetExtNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                useExtId = TRUE;
            }
            
            if (id == oldId)
            {
                if (useExtId == TRUE)
                {
                    Tp_PutExtNetIdOfPPin(newId,curLinkedPPinPtr->ppinPtr);
                }
                else
                {
                    Tp_PutIntNetIdOfPPin(newId,curLinkedPPinPtr->ppinPtr);
                }
            }
        }
    }
    for (curLinkedPPinPtr  = pinListPtr->vertPPinList;
        curLinkedPPinPtr != NIL(LinkedPPin);
        curLinkedPPinPtr  = curLinkedPPinPtr->nextPtr)
    {
        tilePtr = Tp_GetTileOfPPin(curLinkedPPinPtr->ppinPtr);
        if (Tp_IsTileFilled(tilePtr)) 
        {
            Mac_GetAbsPosOfRPin(rPinPtr, &point);
            if (Tp_IsSBNTile(tilePtr))
            {
                if (TiLeft(tilePtr) > point.p_x)
                {
                    id = Tp_GetIntNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = FALSE;
                }
                else if (TiRight(tilePtr) < point.p_x)
                {
                    id = Tp_GetExtNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = TRUE;
                }
                else if (PPinOnTRSideOfTile(curLinkedPPinPtr->ppinPtr, tilePtr))
                {
                    id = Tp_GetIntNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = FALSE;
                }
                else
                {
                    id = Tp_GetExtNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                    useExtId = TRUE;
                }
            }
            else
            {
                id = Tp_GetExtNetIdOfPPin(curLinkedPPinPtr->ppinPtr);
                useExtId = TRUE;
            }
            
            if (id == oldId)
            {
                if (useExtId == TRUE)
                {
                    Tp_PutExtNetIdOfPPin(newId,curLinkedPPinPtr->ppinPtr);
                }
                else
                {
                    Tp_PutIntNetIdOfPPin(newId,curLinkedPPinPtr->ppinPtr);
                }
            }
        }
    }
    for (curLinkedPinInstPtr  = pinListPtr->pinInstList;
        curLinkedPinInstPtr != NIL(LinkedPinInst);
        curLinkedPinInstPtr  = curLinkedPinInstPtr->nextPtr)
    {
        if (Mac_GetExtNetIdOfPin(curLinkedPinInstPtr->pinInstPtr) == oldId)
        {
            Mac_PutExtNetIdOfPin(newId,curLinkedPinInstPtr->pinInstPtr);
            Mac_PutHExtNetIdOfPin(newId,curLinkedPinInstPtr->pinInstPtr);
            Mac_PutVExtNetIdOfPin(newId,curLinkedPinInstPtr->pinInstPtr);
        }
    }
    for (curLinkedRPinPtr  = pinListPtr->rPinList;
        curLinkedRPinPtr != NIL(LinkedRPin);
        curLinkedRPinPtr  = curLinkedRPinPtr->nextPtr)
    {
        if (Mac_GetExtNetIdOfRPin(curLinkedRPinPtr->rPinPtr) == oldId)
        {
            Mac_PutExtNetIdOfRPin(newId,curLinkedRPinPtr->rPinPtr);
            Mac_PutHExtNetIdOfRPin(newId,curLinkedRPinPtr->rPinPtr);
            Mac_PutVExtNetIdOfRPin(newId,curLinkedRPinPtr->rPinPtr);
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



void
PinsAroundVWallSect(cellMasterPtr,rCellPtr,endJunctPtr,sideDir,pinListPtr)
CellMaster      *cellMasterPtr;
RouteCell       *rCellPtr;
WallJunction        *endJunctPtr;
SideDirection       sideDir;
PinsAroundJunction  *pinListPtr;
{
    PlaneData       *planeDataPtr;
    PinsAroundJunction  *faceListPtr;
    WallJunction        *topJunctPtr;
    WallJunction        *bottomJunctPtr;
    Tile                *dmntTilePtr;
    Rect                dmntRect;
    Point               *topPointPtr;
    Point               *bottomPointPtr;
    Point               vPoint;

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    /*
     * Search top and bottom junctions which belong to the interval of 
     * the left or right most vertical rect of the route cell.
     */
    SearchTAndBJunct(rCellPtr, endJunctPtr, sideDir, &topJunctPtr, &bottomJunctPtr);
    if (topJunctPtr == bottomJunctPtr)
    {
    /*
     * No wall section is found.
     */
    Tm_PinsAroundJunction(cellMasterPtr, planeDataPtr, endJunctPtr,
                                   pinListPtr);
    return;
    }
    /*
     * Apply Vertical Rect Unification to the wall section.
     */
    bottomPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, bottomJunctPtr);
    topPointPtr    = Fpg_GetPointOfWallJunct(FPG_CENTER, topJunctPtr);
    Tp_HPointToVPoint(bottomPointPtr, &vPoint);
    dmntTilePtr = TiSrPointNoHint(Tp_GetVertPlane(planeDataPtr),&vPoint);
    (void) Tp_VertTileToRect(planeDataPtr, dmntTilePtr, &dmntRect);
    dmntRect.r_ybot = bottomPointPtr->p_y;
    dmntRect.r_ytop = topPointPtr->p_y;
    Tm_VRectUnify(cellMasterPtr, planeDataPtr, dmntRect);
    /*
     * Apply Tm_PinsAroundVFace to the wall section
     */
    faceListPtr = Tm_PinsAroundVFace(cellMasterPtr, planeDataPtr, 
                                     dmntRect, FALSE);
    pinListPtr->pinInstList  = faceListPtr->pinInstList;
    pinListPtr->rPinList     = faceListPtr->rPinList;
    pinListPtr->vertPPinList = faceListPtr->vertPPinList;
    pinListPtr->horzPPinList = faceListPtr->horzPPinList;
    FREE((char *)faceListPtr);
    return;
}



void
PinsAroundHWallSect(cellMasterPtr,rCellPtr,endJunctPtr,sideDir,pinListPtr)
CellMaster      *cellMasterPtr;
RouteCell       *rCellPtr;
WallJunction        *endJunctPtr;
SideDirection       sideDir;
PinsAroundJunction  *pinListPtr;
{
    PlaneData       *planeDataPtr;
    PinsAroundJunction  *faceListPtr;
    WallJunction        *rightJunctPtr;
    WallJunction        *leftJunctPtr;
    Tile                *dmntTilePtr;
    Rect                dmntRect;
    Point               *rightPointPtr;
    Point               *leftPointPtr;

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    /*
     * Search right and left junctions which belong to the interval of 
     * the bottom or top vertical rect of the route cell.
     */
    SearchRAndLJunct(rCellPtr, endJunctPtr, sideDir, &rightJunctPtr, &leftJunctPtr);
    if (rightJunctPtr == leftJunctPtr)
    {
    /*
     * No wall section is found.
     */
    Tm_PinsAroundJunction(cellMasterPtr, planeDataPtr, endJunctPtr,
                                  pinListPtr);
    return;
    }
    /*
     * Apply Horizontal Rect Unification to the wall section.
     */
    leftPointPtr  = Fpg_GetPointOfWallJunct(FPG_CENTER, leftJunctPtr);
    rightPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, rightJunctPtr);
    dmntTilePtr = TiSrPointNoHint(Tp_GetHorzPlane(planeDataPtr),leftPointPtr);
    (void) Tp_HorzTileToRect(planeDataPtr, dmntTilePtr, &dmntRect);
    dmntRect.r_xbot = leftPointPtr->p_x;
    dmntRect.r_xtop = rightPointPtr->p_x;
    Tm_HRectUnify(cellMasterPtr, planeDataPtr, dmntRect);
    /*
     * Apply Tm_PinsAroundHFace to the wall section
     */
    faceListPtr = Tm_PinsAroundHFace(cellMasterPtr, planeDataPtr, dmntRect,
                                     FALSE);
    pinListPtr->pinInstList  = faceListPtr->pinInstList;
    pinListPtr->rPinList     = faceListPtr->rPinList;
    pinListPtr->horzPPinList = faceListPtr->horzPPinList;
    pinListPtr->vertPPinList = faceListPtr->vertPPinList;
    FREE((char *)faceListPtr);
    return;
}



void
SearchTAndBJunct(rCellPtr, junctPtr, sideDir, topJunctPtrPtr, bottomJunctPtrPtr)
RouteCell       *rCellPtr;
WallJunction    *junctPtr;
SideDirection   sideDir;
WallJunction    **topJunctPtrPtr;
WallJunction    **bottomJunctPtrPtr;
{
    Rect        *endRectPtr;
    LinkedRect      *linkedRectPtr;
    Point           *pointPtr;
    WallJunction    *curJunctPtr;

    endRectPtr = &rCellPtr->vertRectList->r_r;
    for (linkedRectPtr  = rCellPtr->vertRectList->r_next;
     linkedRectPtr != NIL(LinkedRect);
     linkedRectPtr  = linkedRectPtr->r_next)
    {
    if (sideDir == TOP_RIGHT_SIDE)
    {
        /*
         * Try to find right most rect.
         */
        if (linkedRectPtr->r_r.r_xbot > endRectPtr->r_xbot)
        {
        endRectPtr = &linkedRectPtr->r_r;
        }
    }
    else
    {
        /*
         * Try to find left most rect.
         */
        if (linkedRectPtr->r_r.r_xbot < endRectPtr->r_xbot)
        {
        endRectPtr = &linkedRectPtr->r_r;
        }
    }
    }
    /*
     * Go north in order to find the top end junction of the wall section
     */
    *topJunctPtrPtr = junctPtr;
    curJunctPtr = Fpg_GetNorthJunctOfWallJunct(junctPtr);
    while (curJunctPtr != NIL(WallJunction))
    {
        pointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, curJunctPtr);
    if (pointPtr->p_y >= endRectPtr->r_ytop)
    {
        break;
    }
    *topJunctPtrPtr = curJunctPtr;
    curJunctPtr = Fpg_GetNorthJunctOfWallJunct(curJunctPtr);
    }
    /*
     * Go south in order to find the bottom end junction of the wall section
     */
    *bottomJunctPtrPtr = junctPtr;
    curJunctPtr = Fpg_GetSouthJunctOfWallJunct(junctPtr);
    while (curJunctPtr != NIL(WallJunction))
    {
        pointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, curJunctPtr);
    if (pointPtr->p_y <= endRectPtr->r_ybot)
    {
        break;
    }
    *bottomJunctPtrPtr = curJunctPtr;
    curJunctPtr = Fpg_GetSouthJunctOfWallJunct(curJunctPtr);
    }
}



void
SearchRAndLJunct(rCellPtr, junctPtr, sideDir, rightJunctPtrPtr, leftJunctPtrPtr)
RouteCell       *rCellPtr;
WallJunction    *junctPtr;
SideDirection   sideDir;
WallJunction    **rightJunctPtrPtr;
WallJunction    **leftJunctPtrPtr;
{
    Rect        *endRectPtr;
    LinkedRect      *linkedRectPtr;
    Point           *pointPtr;
    WallJunction    *curJunctPtr;

    endRectPtr = &rCellPtr->horzRectList->r_r;
    for (linkedRectPtr  = rCellPtr->horzRectList->r_next;
     linkedRectPtr != NIL(LinkedRect);
     linkedRectPtr  = linkedRectPtr->r_next)
    {
    if (sideDir == TOP_RIGHT_SIDE)
    {
        /*
         * Try to find top rect.
         */
        if (linkedRectPtr->r_r.r_ybot > endRectPtr->r_ybot)
        {
        endRectPtr = &linkedRectPtr->r_r;
        }
    }
    else
    {
        /*
         * Try to find bottom rect.
         */
        if (linkedRectPtr->r_r.r_ybot < endRectPtr->r_ybot)
        {
        endRectPtr = &linkedRectPtr->r_r;
        }
    }
    }
    /*
     * Go east in order to find the right end junction of the wall section
     */
    *rightJunctPtrPtr = junctPtr;
    curJunctPtr = Fpg_GetEastJunctOfWallJunct(junctPtr);
    while (curJunctPtr != NIL(WallJunction))
    {
        pointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, curJunctPtr);
    if (pointPtr->p_x >= endRectPtr->r_xtop)
    {
        break;
    }
    *rightJunctPtrPtr = curJunctPtr;
    curJunctPtr = Fpg_GetEastJunctOfWallJunct(curJunctPtr);
    }
    /*
     * Go west in order to find the left end junction of the wall section
     */
    *leftJunctPtrPtr = junctPtr;
    curJunctPtr = Fpg_GetWestJunctOfWallJunct(junctPtr);
    while (curJunctPtr != NIL(WallJunction))
    {
        pointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, curJunctPtr);
    if (pointPtr->p_x <= endRectPtr->r_xbot)
    {
        break;
    }
    *leftJunctPtrPtr = curJunctPtr;
    curJunctPtr = Fpg_GetWestJunctOfWallJunct(curJunctPtr);
    }
}



/* ARGSUSED */
BearStatus
ReverseDir(gPinPtr, clientData)
GenericPin  *gPinPtr;
ClientData  clientData;
{
    RoutePin     *rPinPtr;
    PinDirection direction;

    if (Mac_GetTypeOfGPin(gPinPtr) == ROUTE_PIN )
    {
    rPinPtr = Mac_GetRoutePinOfGPin(gPinPtr);
    direction = OppositeDir(Mac_GetDirectionOfRPin(rPinPtr));
    Mac_PutDirectionOfRPin(direction, rPinPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}



PinDirection
OppositeDir(direction)
PinDirection  direction;
{
    if (direction == NORTH_PIN)
    {
    return (SOUTH_PIN);
    }
    else if (direction == SOUTH_PIN)
    {
    return (NORTH_PIN);
    }
    else if (direction == EAST_PIN)
    {
    return (WEST_PIN);
    }
    else if (direction == WEST_PIN)
    {
    return (EAST_PIN);
    }
    else
    {
    return (direction);
    }
}



void
LCreateNewCellGroup(planeDataPtr, rCellPtr, wallSegPtr)
PlaneData    *planeDataPtr;
RouteCell    *rCellPtr;
WallSegment  *wallSegPtr;
{
    BottleNeck      *bottleNeckPtr;
    Tile            *bnTilePtr;
    Tile            *blTilePtr;
    Tile            *trTilePtr;
    Rect            bnRect;
    BNLine          *bnLinePtr;
    BNLineType      bnLineType;
    Line            *linePtr;
    Point           blSearchPoint;
    Point           trSearchPoint;
    ClientData      blCellPtr;
    ClientData      trCellPtr;
    CellClass       blCellClass;
    CellClass       trCellClass;
    CellGroup       *rCellGroupPtr;
    CellGroup       *blCellGroupPtr;
    CellGroup       *trCellGroupPtr;
    CellGroup       *tempCellGroupPtr;
    CellGroup       *newCellGroupPtr;

    bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
    if (bottleNeckPtr->unType == TP_BN_TILE)
    {
    bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
    if (Tp_IsHorzTile(bnTilePtr))
    {
        Tp_HorzTileToRect(planeDataPtr,bnTilePtr,&bnRect);
        blSearchPoint.p_x = bnRect.r_xbot - 1;
        blSearchPoint.p_y = bnRect.r_ybot + 1;
        trSearchPoint.p_x = bnRect.r_xtop + 1;
        trSearchPoint.p_y = bnRect.r_ytop - 1;
    }
    else
    {
        Tp_VertTileToRect(planeDataPtr,bnTilePtr,&bnRect);
        blSearchPoint.p_x = bnRect.r_xbot + 1;
        blSearchPoint.p_y = bnRect.r_ybot - 1;
        trSearchPoint.p_x = bnRect.r_xtop - 1;
        trSearchPoint.p_y = bnRect.r_ytop + 1;
    }
    }
    else
    {
    bnLinePtr  = bottleNeckPtr->unValue.bnLinePtr;
    bnLineType = Tp_GetBnLineTypeOfBnLine(bnLinePtr);
    linePtr    = Tp_GetLineOfBnLine(bnLinePtr);
    if (bnLineType == TP_HORZ_B_RIGHT)
    {
        blSearchPoint.p_x = linePtr->point1.p_x - 1;
        blSearchPoint.p_y = linePtr->point1.p_y - 1;
        trSearchPoint.p_x = linePtr->point2.p_x + 1;
        trSearchPoint.p_y = linePtr->point2.p_y + 1;
    }
    else if (bnLineType == TP_HORZ_B_LEFT)
    {
        blSearchPoint.p_x = linePtr->point1.p_x - 1;
        blSearchPoint.p_y = linePtr->point1.p_y + 1;
        trSearchPoint.p_x = linePtr->point2.p_x + 1;
        trSearchPoint.p_y = linePtr->point2.p_y - 1;
    }
    else if (bnLineType == TP_VERT_R_UP)
    {
        Tp_VPointToHPoint(&linePtr->point1, &blSearchPoint);
        blSearchPoint.p_x += 1;
        blSearchPoint.p_y -= 1;
        Tp_VPointToHPoint(&linePtr->point2, &trSearchPoint);
        trSearchPoint.p_x -= 1;
        trSearchPoint.p_y += 1;
    }
    else if (bnLineType == TP_VERT_R_DOWN)
    {
        Tp_VPointToHPoint(&linePtr->point1, &blSearchPoint);
        blSearchPoint.p_x -= 1;
        blSearchPoint.p_y -= 1;
        Tp_VPointToHPoint(&linePtr->point2, &trSearchPoint);
        trSearchPoint.p_x += 1;
        trSearchPoint.p_y += 1;
    }
    else 
    {
        Bear_PrintFatalErr("LCreateNewCellGroup() : Illegal BNLineType is found");
    }
    }
    blTilePtr = TiSrPointNoHint(Tp_GetHorzPlane(planeDataPtr), &blSearchPoint);
    trTilePtr = TiSrPointNoHint(Tp_GetHorzPlane(planeDataPtr), &trSearchPoint);
    if (Tp_IsCellInstTile(blTilePtr))
    {
    blCellPtr   = (ClientData) Tp_GetCellInstOfTile(blTilePtr);
    blCellClass = CELL_INST;
    }
    else if (Tp_IsRCellTile(blTilePtr))
    {
    blCellPtr   = (ClientData) Tp_GetRCellOfTile(blTilePtr);
    blCellClass = ROUTE_CELL;
    }
    else
    {
    Bear_PrintFatalErr("LCreateNewCellGroup() : blCellPtr is not Generic Cell");
    }
    if (Tp_IsCellInstTile(trTilePtr))
    {
    trCellPtr   = (ClientData) Tp_GetCellInstOfTile(trTilePtr);
    trCellClass = CELL_INST;
    }
    else if (Tp_IsRCellTile(trTilePtr))
    {
    trCellPtr   = (ClientData) Tp_GetRCellOfTile(trTilePtr);
    trCellClass = ROUTE_CELL;
    }
    else
    {
    Bear_PrintFatalErr("LCreateNewCellGroup() : trCellPtr is not Generic Cell");
    }
    (void) Mac_CreateCellGroup(ROUTE_CELL, (ClientData)rCellPtr,
                              &rCellGroupPtr);
    if (Mac_IsGroupedCell(blCellClass, blCellPtr))
    {
    blCellGroupPtr = blCellClass == CELL_INST ?
             Mac_GetGroupOfCell((CellInst *)blCellPtr) :
             Mac_GetGroupOfRCell((RouteCell *)blCellPtr);
    }
    else
    {
    (void) Mac_CreateCellGroup(blCellClass, blCellPtr, &blCellGroupPtr);
    }
    if (Mac_IsGroupedCell(trCellClass, trCellPtr))
    {
    trCellGroupPtr = trCellClass == CELL_INST ?
             Mac_GetGroupOfCell((CellInst *)trCellPtr) :
             Mac_GetGroupOfRCell((RouteCell *)trCellPtr);
    }
    else
    {
    (void) Mac_CreateCellGroup(trCellClass, trCellPtr, &trCellGroupPtr);
    }
    (void) Mac_MergeCellGroups(blCellGroupPtr, trCellGroupPtr,
                              &tempCellGroupPtr);
    (void) Mac_MergeCellGroups(tempCellGroupPtr, rCellGroupPtr,
                              &newCellGroupPtr);
    return;
}



char *
GetReply(msg)
char   *msg;
{
    static char reply[10];

    printf(msg);
    scanf("%s",reply);
    return (reply);
}


void
CleanPinList(pinList)
PinsAroundJunction pinList;
{
    LinkedPPin *pPinList;
    LinkedPPin *prevPinList;
    
    /* Remove ppins from the list that belong to a tile with no space prop */
    if (pinList.horzPPinList != NIL(LinkedPPin))
    {
    prevPinList = NIL(LinkedPPin);
    for (pPinList = pinList.horzPPinList;
         pPinList->nextPtr != NIL(LinkedPPin); 
         pPinList = pPinList->nextPtr)
    {
        if ((pPinList->ppinPtr->tilePtr == NIL(Tile)) ||
            (Tp_IsTileFilled(pPinList->ppinPtr->tilePtr) == FALSE))
        {
        if (prevPinList == NIL(LinkedPPin))
        {
            pinList.horzPPinList = pPinList->nextPtr;
        }
        else
        {
            prevPinList->nextPtr = pPinList->nextPtr;
        }
        }
        else 
        {
        prevPinList = pPinList;
        }
    }
    }
    if (pinList.vertPPinList != NIL(LinkedPPin))
    {
    prevPinList = NIL(LinkedPPin);
    for (pPinList = pinList.vertPPinList;
         pPinList->nextPtr != NIL(LinkedPPin); 
         pPinList = pPinList->nextPtr)
    {
        if ((pPinList->ppinPtr->tilePtr == NIL(Tile)) ||
            (Tp_IsTileFilled(pPinList->ppinPtr->tilePtr) == FALSE))
        {
        if (prevPinList == NIL(LinkedPPin))
        {
            pinList.vertPPinList = pPinList->nextPtr;
        }
        else
        {
            prevPinList->nextPtr = pPinList->nextPtr;
        }
        }
        else 
        {
        prevPinList = pPinList;
        }
    }
    }
   
}



bool
PPinOnTRSideOfTile(pPinPtr, tilePtr)
PseudoPin *pPinPtr;
Tile *tilePtr;
{
    return (Tp_ItrPPinsOfSide(tilePtr, TOP_RIGHT_SIDE, FindPPin, 
        (ClientData) pPinPtr) == BEAR_STOP_SEARCH);
}



BearStatus
FindPPin(tmpPinPtr, pPinPtr)
PseudoPin *tmpPinPtr;
PseudoPin *pPinPtr;
{
     if (tmpPinPtr == pPinPtr)
     {
     return (BEAR_STOP_SEARCH);
     }
     else 
     return (BEAR_CONTINUE_SEARCH);
     {
     }
}



void
RemoveSingleSidePins(rCellPtr)
RouteCell *rCellPtr;
{
    Mac_ItrTLSidePinOfRCell(rCellPtr, CheckTopLeftSidePins,
                (ClientData)rCellPtr);
    Mac_ItrRSidePinOfRCell(rCellPtr, CheckBottomRightSidePins,
               (ClientData)rCellPtr);
}



BearStatus
CheckTopLeftSidePins(rPinPtr, rCellPtr)
RoutePin *rPinPtr;
RouteCell *rCellPtr;
{
   if ((FindRPinMatchById(rPinPtr, rCellPtr->rSidePinList) == FALSE) &&
       (FindGPinMatchById(rPinPtr, rCellPtr->blBoundPinList) == FALSE) &&
       (FindGPinMatchById(rPinPtr, rCellPtr->trBoundPinList) == FALSE))

       /* Delete Side Pin From List */
       Mac_DeleteSidePinOfRCell(rCellPtr, BOTTOM_LEFT_SIDE, rPinPtr);
   
   return (BEAR_CONTINUE_SEARCH);
}



BearStatus
CheckBottomRightSidePins(rPinPtr, rCellPtr)
RoutePin *rPinPtr;
RouteCell *rCellPtr;
{
   if ((FindRPinMatchById(rPinPtr, rCellPtr->tlSidePinList) == FALSE) &&
       (FindGPinMatchById(rPinPtr, rCellPtr->blBoundPinList) == FALSE) &&
       (FindGPinMatchById(rPinPtr, rCellPtr->trBoundPinList) == FALSE))

       /* Delete Side Pin From List */
       Mac_DeleteSidePinOfRCell(rCellPtr, RIGHT_SIDE, rPinPtr);
   
   return (BEAR_CONTINUE_SEARCH);
}



static bool
FindRPinMatchById(rPinPtr, linkedRPinPtr)
RoutePin *rPinPtr;
RoutePin *linkedRPinPtr;
{
    BearId searchId;

    searchId = Mac_GetExtNetIdOfRPin(rPinPtr);
    while (linkedRPinPtr != NIL(RoutePin))
    { 
       if (Mac_GetExtNetIdOfRPin(linkedRPinPtr) == searchId)
          return (TRUE);
           linkedRPinPtr = linkedRPinPtr->nextPtr;
    }
    return (FALSE);
}



static bool
FindRPinMatch(rPinPtr, linkedRPinPtr)
RoutePin *rPinPtr;
RoutePin *linkedRPinPtr;
{
    Net *searchNetPtr;

    searchNetPtr = Mac_GetNetPtrOfRPin(rPinPtr);
    while (linkedRPinPtr != NIL(RoutePin))
    { 
       if (Mac_GetNetPtrOfRPin(linkedRPinPtr) == searchNetPtr)
          return (TRUE);
       linkedRPinPtr = linkedRPinPtr->nextPtr;
    }
    return (FALSE);
}



static bool
FindGPinMatchById(rPinPtr, linkedGPinPtr)
RoutePin *rPinPtr;
LinkedGPin *linkedGPinPtr;
{
        BearId searchId;
    RoutePin *rPinPtr2;
    PinInst  *pinInstPtr;

    searchId = Mac_GetExtNetIdOfRPin(rPinPtr);
    while (linkedGPinPtr != NIL(LinkedGPin))
    { 
       if (Mac_GetTypeOfGPin(linkedGPinPtr->gPinPtr) == ROUTE_PIN)
       {
          rPinPtr2 = Mac_GetRoutePinOfGPin(linkedGPinPtr->gPinPtr);
          if (Mac_GetExtNetIdOfRPin(rPinPtr2) == searchId)
             return (TRUE);
       }
       else
       {
          pinInstPtr = Mac_GetPinInstOfGPin(linkedGPinPtr->gPinPtr);
              if (Mac_GetExtNetIdOfPin(pinInstPtr) == searchId)
         return (TRUE);
       }
           linkedGPinPtr = linkedGPinPtr->nextPtr;
    }
    return (FALSE);
}


static bool
FindGPinMatch(rPinPtr, linkedGPinPtr)
RoutePin *rPinPtr;
LinkedGPin *linkedGPinPtr;
{
    Net *searchNetPtr;
    RoutePin *rPinPtr2;
    PinInst  *pinInstPtr;

    searchNetPtr = Mac_GetNetPtrOfRPin(rPinPtr);
    while (linkedGPinPtr != NIL(LinkedGPin))
    { 
       if (Mac_GetTypeOfGPin(linkedGPinPtr->gPinPtr) == ROUTE_PIN)
       {
          rPinPtr2 = Mac_GetRoutePinOfGPin(linkedGPinPtr->gPinPtr);
          if (Mac_GetNetPtrOfRPin(rPinPtr2) == searchNetPtr)
             return (TRUE);
       }
       else
       {
          pinInstPtr = Mac_GetPinInstOfGPin(linkedGPinPtr->gPinPtr);
          if (Mac_GetNetOfPin(pinInstPtr) == searchNetPtr)
             return (TRUE);
       }
       linkedGPinPtr = linkedGPinPtr->nextPtr;
    }
    return (FALSE);
}

#endif /* USE_DR_CODE */


