
/* pbCellGroup.c --
 *
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifdef USE_OLD_X_CODE

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "polarBear.h"


typedef struct _LinkedVertex LinkedVertex;
struct _LinkedVertex {
    Vertex          theVertex;
    LinkedVertex    *nextPtr;
};

typedef enum _RECT_CORNER RectCorner;
enum _RECT_CORNER {
    UL,    /*  Upper left  */
    UR,    /*  Upper right */
    LL,    /*  Lower left  */
    LR,    /*  Lower right */
    NONE,  /*  No corner   */
    NIL_CORNER,   /* No value */
};

typedef struct _POINT_NODE PointNode;
struct _POINT_NODE {
    RectCorner          corner;
    Point               *thePointPtr;
    struct _POINT_NODE  *nextPtr, *prevPtr;
};
    
typedef struct _TREE_NODE TreePointNode;
struct _TREE_NODE {
    struct _POINT_NODE *thePointPtr; /* The list of associated points */
    struct _TREE_NODE *leftPtr, *rightPtr;
};

#define NEXT_PTR(ptr) ((ptr)->nextPtr)
#define PREV_PTR(ptr) ((ptr)->prevPtr)
#define LEFT_PTR(ptr) ((ptr)->leftPtr)
#define RIGHT_PTR(ptr) ((ptr)->rightPtr)
#define POINT_PTR(ptr) ((ptr)->thePointPtr)
#define POINT(ptr) ((*((ptr)->thePointPtr)->thePointPtr))
#define POINTX(ptr) ((ptr)->thePointPtr->thePointPtr->p_x)
#define POINTY(ptr) ((ptr)->thePointPtr->thePointPtr->p_y)
#define POINT_CMP(p1, p2) ((p1.p_x == p2.p_x) && (p1.p_y == p2.p_y))
#define CORNER(ptr) ((ptr)->corner)

#define GetCellListOfCell(cellInstPtr) \
    ((cellInstPtr)->cellGroupPtr->cellList)

#define NEW_POINT(pointPtr, x, y, corner) \
    MALLOC(PointNode *, pointPtr, sizeof (PointNode)); \
    POINT_PTR(pointPtr) = NIL(Point); \
    CORNER(pointPtr) = corner; \
    MALLOC(Point *, pointPtr->thePointPtr, sizeof (Point)); \
    pointPtr->thePointPtr->p_x = x; \
    pointPtr->thePointPtr->p_y = y;
   

/* ---------- Local Procedures ---------- */
static TreePointNode    *SearchNode();
static BearStatus       InsertNode();
static BearStatus       BuildPointTrees();
static int              CompareX(), CompareY();
static void             Store();
static void             InsertRect();
static void             PrintNode();
static void             StoreVertices();


/* ---------- Local Global Variables ---------- */
static double   (*scrMatrix)[4];
static int      vertexCount;


static int 
CompareX(node1Ptr, node2Ptr)
TreePointNode *node1Ptr, *node2Ptr;
{
    if (POINTX(node1Ptr) < POINTX(node2Ptr))
    return (-1);
    else if (POINTX(node1Ptr) > POINTX(node2Ptr))
    return (1);
    else
    return (0);
}


static int 
CompareY(node1Ptr, node2Ptr)
TreePointNode *node1Ptr, *node2Ptr;
{
    if (POINTY(node1Ptr) < POINTY(node2Ptr))
        return (-1);
    else if (POINTY(node1Ptr) > POINTY(node2Ptr))
        return (1);
    else
        return (0);
}


static TreePointNode *
SearchNode(rootPtr, keyPtr, cmpFunc)
TreePointNode *rootPtr, *keyPtr;
int (*cmpFunc)();
{
    bool done = FALSE;
    TreePointNode *currentPtr, *matchPtr;

    currentPtr = rootPtr;
    matchPtr = NIL(TreePointNode);
    while (!done) {
    if (currentPtr == NIL(TreePointNode)) {
        done = TRUE;
    }
    else if (cmpFunc(currentPtr, keyPtr) == 0) {
        matchPtr = currentPtr;
        done = TRUE;
    }
    else {
        if (cmpFunc(currentPtr, keyPtr) > 0) {
        currentPtr = LEFT_PTR(currentPtr);
        }
        else {
        currentPtr = RIGHT_PTR(currentPtr);
        }
    }
    }
    return (matchPtr);
}

void InsertList(newPointPtr, pointPtr, cmpFunc)
     PointNode *newPointPtr, *pointPtr;
     int (*cmpFunc)();
{
    PointNode *currentPtr = pointPtr, *oldPtr = NIL(PointNode);
    
    if (currentPtr != NIL(PointNode)) {
    for (;;) {
        if (PREV_PTR(currentPtr) != NIL(PointNode)) {
        currentPtr = PREV_PTR(currentPtr);
        }
        else {
        break;
        }
    }
    }
    if (cmpFunc == CompareX) {
    while (currentPtr != NIL(PointNode)) {
        if (POINT_PTR(newPointPtr)->p_y 
        < POINT_PTR(currentPtr)->p_y) {
        if (PREV_PTR(currentPtr) == NIL(PointNode)) {
            PREV_PTR(currentPtr) = newPointPtr;
            NEXT_PTR(newPointPtr) = currentPtr;
            return;
        }
        else {
            NEXT_PTR(PREV_PTR(currentPtr)) = newPointPtr;
            PREV_PTR(currentPtr) = newPointPtr;
            NEXT_PTR(newPointPtr) = currentPtr;
            PREV_PTR(newPointPtr) = oldPtr;
            return;
        }
        }
        else if (POINT_PTR(newPointPtr)->p_y 
             == POINT_PTR(currentPtr)->p_y) {
        currentPtr->corner = NONE;
        return;
        }
        oldPtr = currentPtr;
        currentPtr = NEXT_PTR(currentPtr);
    }
    NEXT_PTR(oldPtr) = newPointPtr;
    PREV_PTR(newPointPtr) = oldPtr;
    return;
    }
    else {
    while (currentPtr != NIL(PointNode)) {
        if (POINT_PTR(newPointPtr)->p_x
        < POINT_PTR(currentPtr)->p_x) {
        if (PREV_PTR(currentPtr) == NIL(PointNode)) {
            PREV_PTR(currentPtr) = newPointPtr;
            NEXT_PTR(newPointPtr) = currentPtr;
            return;
        }
        else {
            NEXT_PTR(PREV_PTR(currentPtr)) = newPointPtr;
            PREV_PTR(currentPtr) = newPointPtr;
            NEXT_PTR(newPointPtr) = currentPtr;
            PREV_PTR(newPointPtr) = oldPtr;
            return;
        }
        }
        else if (POINT_PTR(newPointPtr)->p_x 
             == POINT_PTR(currentPtr)->p_x) {
        currentPtr->corner = NONE;
        return;
        }
        oldPtr = currentPtr;
        currentPtr = NEXT_PTR(currentPtr);
    }
    NEXT_PTR(oldPtr) = newPointPtr;
    PREV_PTR(newPointPtr) = oldPtr;
    return;
    }
}


static BearStatus 
InsertNode(rootPtr, newPtr, cmpFunc)
TreePointNode **rootPtr;
PointNode *newPtr;
int (*cmpFunc)();
{
    TreePointNode *currentPtr, *newNodePtr;
    
    MALLOC(TreePointNode *, newNodePtr, sizeof (TreePointNode));
    LEFT_PTR(newNodePtr) = NIL(TreePointNode);
    RIGHT_PTR(newNodePtr) = NIL(TreePointNode);
    POINT_PTR(newNodePtr) = newPtr;
    currentPtr = *rootPtr;
    while (currentPtr != NIL(TreePointNode)) {
    if (cmpFunc(currentPtr, newNodePtr) > 0) {
        if (LEFT_PTR(currentPtr) != NIL(TreePointNode)) {
        currentPtr = LEFT_PTR(currentPtr);
        }
        else {
        LEFT_PTR(currentPtr) = newNodePtr;
        currentPtr = NIL(TreePointNode);
        }
    }
    else if (cmpFunc(currentPtr, newNodePtr) < 0) {
        if (RIGHT_PTR(currentPtr) != NIL(TreePointNode)) {
        currentPtr = RIGHT_PTR(currentPtr);
        }
        else {
        RIGHT_PTR(currentPtr) = newNodePtr;
        currentPtr = NIL(TreePointNode);
        }
    }
    else {
        InsertList(POINT_PTR(newNodePtr), 
               currentPtr->thePointPtr, cmpFunc);
        FREE((char *) newNodePtr);
        return (BEAR_OK);
    }
    }
    if (*rootPtr == NIL(TreePointNode)) {
    *rootPtr = newNodePtr;
    return (BEAR_OK);
    }
    return (BEAR_ERROR);
}


void
FreeTree(rootPtr)
TreePointNode *rootPtr;
{
    if (rootPtr == NIL(TreePointNode)) {
    return;
    }
    FreeTree(LEFT_PTR(rootPtr));
    FreeTree(RIGHT_PTR(rootPtr));
    FREE((char *) rootPtr);
}


static void
InsertRect(theRect, rootXPtrPtr, rootYPtrPtr)
Rect            *theRect;
TreePointNode   **rootXPtrPtr, **rootYPtrPtr;
{
/*
 * DRAW_VERTICES() is out of date. See EZ_DRAW_VERTICES() in ezPlotInt.h
 *
#define DRAW_VERTICES(win, vertex, vertexCount)                      \
    XDraw((win), (vertex), (vertexCount), 2, 2, WhitePixel, GXcopy, AllPlanes)
*/

    PointNode *newPointPtr;
    int llx, lly, urx, ury;
/*
 *  Vertex  vertex[25];
 *  int vertexCount;
 */
    
    llx = Ez_XTrans(scrMatrix, theRect->r_xbot, theRect->r_ybot);
    lly = Ez_YTrans(scrMatrix, theRect->r_xbot, theRect->r_ybot);
    urx = Ez_XTrans(scrMatrix, theRect->r_xtop, theRect->r_ytop);
    ury = Ez_YTrans(scrMatrix, theRect->r_xtop, theRect->r_ytop);
    
    /*
      bzero((char *) vertex, sizeof (*vertex) * 25);
      vertex[0].flags = VertexDontDraw;
      vertexCount = 0;
      Ez_StoreBox(vertex, vertexCount, llx, lly, urx, ury);
      DRAW_VERTICES(TestWin, vertex, vertexCount);
      Ez_Flush();
      (void) printf("Drawing (%d, %d), (%d,%d)\n", llx, lly, urx, ury);
      */
    
    NEW_POINT(newPointPtr, llx, lly, LL);
    (void) InsertNode(rootXPtrPtr, newPointPtr, CompareX);
    NEW_POINT(newPointPtr, llx, lly, LL);
    (void) InsertNode(rootYPtrPtr, newPointPtr, CompareY);
    NEW_POINT(newPointPtr, llx, ury, UL);
    (void) InsertNode(rootXPtrPtr, newPointPtr, CompareX);
    NEW_POINT(newPointPtr, llx, ury, UL);
    (void) InsertNode(rootYPtrPtr, newPointPtr, CompareY);
    NEW_POINT(newPointPtr, urx, lly, LR);
    (void) InsertNode(rootXPtrPtr, newPointPtr, CompareX);
    NEW_POINT(newPointPtr, urx, lly, LR);
    (void) InsertNode(rootYPtrPtr, newPointPtr, CompareY);
    NEW_POINT(newPointPtr, urx, ury, UR);
    (void) InsertNode(rootXPtrPtr, newPointPtr, CompareX);
    NEW_POINT(newPointPtr, urx, ury, UR);
    (void) InsertNode(rootYPtrPtr, newPointPtr, CompareY);
}


        
static BearStatus 
BuildPointTrees(cellInstPtr, rootXPtrPtr, rootYPtrPtr)
CellInst *cellInstPtr;
TreePointNode    **rootXPtrPtr, **rootYPtrPtr;
{
    GenericCell genCell;
    LinkedCell  *cellListPtr;
    LinkedRect  *tmpLinkedRectPtr;   
    Rect        currentRect;

    *rootXPtrPtr = NIL(TreePointNode);
    *rootYPtrPtr = NIL(TreePointNode);
    genCell.unType = CELL_INST;
    genCell.unValue.cellInstPtr = cellInstPtr;
    cellListPtr = GetCellListOfCell(cellInstPtr);
    while (cellListPtr != NIL(LinkedCell)) {
        tmpLinkedRectPtr = Mac_GetHorzRectListOfGCell(cellListPtr->cellPtr);
        while (tmpLinkedRectPtr != NIL(LinkedRect)) {
            currentRect = tmpLinkedRectPtr->r_r;
            GeoTransRect(Mac_GetTransformOfGCell(cellListPtr->cellPtr),
                 &currentRect, &currentRect);
            InsertRect(&currentRect, rootXPtrPtr, rootYPtrPtr);
            tmpLinkedRectPtr = tmpLinkedRectPtr->r_next;
        }
        cellListPtr = NEXT_PTR(cellListPtr);
    }
    return (BEAR_OK);
}


static void
Store(nodePtr, vertexList)
PointNode       *nodePtr;
LinkedVertex    **vertexList;
{
    LinkedVertex *newVertex;

    CORNER(nodePtr) = NONE;
    MALLOC(LinkedVertex *, newVertex, sizeof (LinkedVertex));
    NEXT_PTR(*vertexList) = newVertex;
    *vertexList = NEXT_PTR(*vertexList);
    (*vertexList)->theVertex.x = POINT_PTR(nodePtr)->p_x;
    (*vertexList)->theVertex.y = POINT_PTR(nodePtr)->p_y;
    NEXT_PTR(*vertexList) = NIL(LinkedVertex);
    vertexCount++;
}


PointNode *SearchList(nodePtr, keyPtr)
     PointNode *nodePtr, *keyPtr;
{
    PointNode *currentPtr;
    currentPtr = nodePtr;
    if (currentPtr != NIL(PointNode)) {
    for (;;) {
        if (PREV_PTR(currentPtr) != NIL(PointNode)) {
        currentPtr = PREV_PTR(currentPtr);
        }
        else {
        break;
        }
    }
    }
    nodePtr = currentPtr;
    while (nodePtr != NIL(PointNode)) {
    if (POINT_CMP((*(POINT_PTR(nodePtr))), ((*(POINT_PTR(keyPtr)))))
        == TRUE && CORNER(nodePtr) != NIL_CORNER) {
        break;
    }
    nodePtr = NEXT_PTR(nodePtr);
    }
    return nodePtr;
}


void 
AddPoint(nodePtr, keyPtr, vertexList)
TreePointNode   *nodePtr, *keyPtr;
LinkedVertex    **vertexList;
{
    PointNode *matchPtr, *nextPtr, *prevPtr;
    RectCorner nextCorner, prevCorner;

    if ((matchPtr = SearchList(POINT_PTR(nodePtr), POINT_PTR(keyPtr)))
    == NIL(PointNode)) {
    return;
    }
    if (POINT_CMP(POINT(keyPtr), (*(matchPtr->thePointPtr))) == 0) {
    CORNER(matchPtr) = NONE;
    }
    /* Speed things up a bit */
    nextPtr = NEXT_PTR(matchPtr);
    prevPtr = PREV_PTR(matchPtr);
    if (nextPtr != NIL(PointNode)) {
    nextCorner = CORNER(nextPtr);
    }
    else {
    nextCorner = NIL_CORNER;
    }
    if (prevPtr != NIL(PointNode)) {
    prevCorner = CORNER(prevPtr); 
    }
    else {
    prevCorner = NIL_CORNER;
    }
    switch (CORNER(matchPtr)) {
    case UL:
    case LR:
    for (;;) {
        switch (nextCorner) {
        case LL:
        case UR:
        Store(nextPtr, vertexList);
        POINTX(keyPtr) = POINT_PTR(nextPtr)->p_x;
        POINTY(keyPtr) = POINT_PTR(nextPtr)->p_y;
        break;
        case NONE:
        nextPtr = NEXT_PTR(nextPtr);
        if (nextPtr != NIL(PointNode)) {  
            nextCorner = CORNER(nextPtr);
            continue;
        }
        default:
        for (;;) {
            switch (prevCorner) {
            case LL:
            case UR:
            Store(prevPtr, vertexList);
            POINTX(keyPtr) = POINT_PTR(prevPtr)->p_x;
            POINTY(keyPtr) = POINT_PTR(prevPtr)->p_y;
            break;
            case NONE:
            prevPtr = PREV_PTR(prevPtr);
            if (prevPtr != NIL(PointNode)) {
                prevCorner = CORNER(prevPtr);
                continue;
            }
            default:
            break;
            }
            break;
        }
        break;
        }
        break;
    }
    break;
    case UR:
    case LL:
    for (;;) {
        switch (nextCorner) {
        case UL:
        case LR:
        Store(nextPtr, vertexList);
        POINTX(keyPtr) = POINT_PTR(nextPtr)->p_x;
        POINTY(keyPtr) = POINT_PTR(nextPtr)->p_y;
        break;
        case NONE:
        nextPtr = NEXT_PTR(nextPtr);
        if (nextPtr != NIL(PointNode)) {
            nextCorner = CORNER(nextPtr);
            continue;
        }
        default:
        for (;;) {
            switch (prevCorner) {
            case UL:
            case LR:
            Store(prevPtr, vertexList);
            POINTX(keyPtr) = POINT_PTR(prevPtr)->p_x;
            POINTY(keyPtr) = POINT_PTR(prevPtr)->p_y;
            break;
            case NONE:
            prevPtr = PREV_PTR(prevPtr);
            if (prevPtr != NIL(PointNode)) {
                prevCorner = CORNER(prevPtr);
                continue;
            }
            default:
            break;
            }
            break;
        }
        break;
        }
        break;
    }
    break;
    }
}


static void
StoreVertices(vertexList, vertexArray)
LinkedVertex    *vertexList;
Vertex          **vertexArray;
{
    LinkedVertex    *freePtr;
    int             i;

    /* MALLOC(Vertex *, (*vertexArray), (vertexCount * sizeof (Vertex))); */
    *vertexArray = (Vertex *) calloc (vertexCount, sizeof (Vertex));

    i = 0;
    while (vertexList != NIL(LinkedVertex)) {
        (*vertexArray)[i].x = vertexList->theVertex.x;
        (*vertexArray)[i++].y = vertexList->theVertex.y;

        freePtr = vertexList;
        vertexList = NEXT_PTR(vertexList);
        FREE((char *) freePtr);
    }
}


static void 
PrintNode(theNodePtr, mode)
TreePointNode *theNodePtr;
bool mode;
{
    PointNode *currentPtr;

    int i = 0;
    if (theNodePtr == NIL(TreePointNode)) {
    (void) printf("NIL\n");
    }
    else {
    if (mode) {
        currentPtr = POINT_PTR(theNodePtr);
        if (currentPtr != NIL(PointNode)) {
        for (;;) {
            if (PREV_PTR(currentPtr) != NIL(PointNode)) {
            currentPtr = PREV_PTR(currentPtr);
            }
            else {
            break;
            }
        }
        }
        (void) printf("TreeNode:\n");
        while (currentPtr != NIL(PointNode)) {
        (void) printf("%d) x=%d, ", i, currentPtr->thePointPtr->p_x);
        (void) printf("%d) y=%d\n", i, currentPtr->thePointPtr->p_y);
        i++;
        currentPtr = NEXT_PTR(currentPtr);
        }
    }
    else {
        (void) printf("x=%d, ", POINTX(theNodePtr));
        (void) printf("y=%d\n", POINTY(theNodePtr));
    }
    }
}


void PrintTree(rootPtr)
     TreePointNode *rootPtr;
{
    static int i = 0;
    
    if (rootPtr == NIL(TreePointNode)) {
    i--;
    return;
    }
    (void) printf("Left\n");
    PrintTree(LEFT_PTR(rootPtr));
    (void) printf("Right\n");
    PrintTree(RIGHT_PTR(rootPtr));
    (void) printf("Level %d - ", i);
    PrintNode(rootPtr, 1);
    i++;
}


BearStatus
PbGetOutlineVert(cellInstPtr, theMatrix, vertexArray, vertCount)
CellInst    *cellInstPtr;
double      (*theMatrix)[4];
Vertex      **vertexArray;
int         *vertCount;
{
    TreePointNode   *searchPtr, *keyPtr, *currentPtr, *rootXPtr, *rootYPtr;
    LinkedVertex    *vertexListPtr, *vertexListEndPtr;
    Point           origin;
    
    if (Mac_GetGroupOfCell(cellInstPtr) == NIL(CellGroup)) {
        return BEAR_ERROR;
    }
    scrMatrix = theMatrix;

    if (BuildPointTrees(cellInstPtr, &rootXPtr, &rootYPtr) == BEAR_ERROR) {
        Bear_PrintErr("PbGetOutlineVert(): Could not build point tree.");
    }
    /*
      PrintTree(rootXPtr);
      PrintTree(rootYPtr);
    */
    searchPtr = rootYPtr;
    while (LEFT_PTR(searchPtr) != NIL(TreePointNode)) {
        /* Find lowest set of Y points */
        searchPtr = LEFT_PTR(searchPtr);
    }
    origin.p_x = POINTX(searchPtr);
    origin.p_y = POINTY(searchPtr);
    MALLOC(LinkedVertex *, vertexListPtr, sizeof (LinkedVertex));
    vertexListPtr->theVertex.x = origin.p_x;
    vertexListPtr->theVertex.y = origin.p_y;
    vertexCount = 1;
    NEXT_PTR(vertexListPtr) = NIL(LinkedVertex);
    keyPtr = searchPtr;
    MALLOC(TreePointNode *, searchPtr, sizeof (TreePointNode));
    NEW_POINT(POINT_PTR(searchPtr), POINTX(keyPtr), POINTY(keyPtr),
          CORNER(POINT_PTR(keyPtr)));
    /* printf("Origin: ");
       PrintNode(searchPtr, 0);
     */
    vertexListEndPtr = vertexListPtr;
    for (;;) {
        currentPtr = SearchNode(rootYPtr, searchPtr, CompareY);
        AddPoint(currentPtr, searchPtr, &vertexListEndPtr);
        /* printf("%d) add: ", vertexCount);
           PrintNode(searchPtr, 0);
             */
        if (POINT_CMP(origin, POINT(searchPtr)) == TRUE) {
            break;
        }
        currentPtr = SearchNode(rootXPtr, searchPtr, CompareX);
        AddPoint(currentPtr, searchPtr, &vertexListEndPtr);
        /* printf("%d) add: ", vertexCount);
           PrintNode(searchPtr, 0);
             */
        if (POINT_CMP(origin, POINT(searchPtr)) == TRUE) {
            break;
        }
    } 
    StoreVertices(vertexListPtr, vertexArray);
    *vertCount = vertexCount;
    /*  Weiming 5/4/90
    FREE((char *) vertexListPtr);
    */
    FREE((char *) searchPtr);
    FreeTree(rootXPtr);
    FreeTree(rootYPtr);

    return BEAR_OK;
}

#endif /* USE_OLD_X_CODE */


