
/* pbCommands.h --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * This file contains information regarding the commands that can
 * be called from the console window of the polarBear program.
 *
 * Copyright (C) 1991 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header$
 */

#ifndef _PB_COMMANDS_H
#define _PB_COMMANDS_H /* once-only latch */

/*
 * Markers to indicate a commands properties.
 */
#define PB_NULL_CP              0
#define PB_BLACK_BOX_CP        01       /* This command can be run in the
                                         * black box.
                                         */
#define PB_SCRIPT_CP           02       /* This command can be run in a
                                         * script.
                                         */
#define PB_MOUSE_LOC_CP        04       /* This command needs the location
                                         * of the mouse.
                                         */

typedef struct _Command Command;
struct _Command {
    char            *co_shortName;      /* (2 or 3 letter) abbreviation for
                                         * the command.
                                         */
    char            *co_longName;       

    void            (*co_voidProc)();
/*
 * This procedure should have the following form:
 *
 * void
 * co_voidProc(winDataPtr, screenPtr, argCount, argList)
 * WindowData   *winDataPtr;
 * Point        *screenPtr;
 * int          argCount;
 * char         **argList;
 * {
 * }
 *
 * If a command accepts arguements it should explain them
 * when given the -help option.
 */

    BearStatus      (*co_statusProc)();
/*
 * This procedure should have the following form:
 *
 * BearStatus
 * co_statusProc(winDataPtr, screenPtr, argCount, argList)
 * WindowData   *winDataPtr;
 * Point        *screenPtr;
 * int          argCount;
 * char         **argList;
 * {
 *     return (BEAR_OK or BEAR_ERROR);
 * }
 *
 * If a command accepts arguements it should explain them
 * when given the -help option.
 */

    unsigned int    co_validWins;       /* Which windows the command is valid
                                         * in, e.g. either PB_ANY_WIN or some
                                         * combination of the following:
                                         *
                                         * PB_CIF_WIN
                                         * PB_CONSOLE_WIN
                                         * PB_CHIP_WIN
                                         * PB_TREE_WIN
                                         * PB_FLOOR_WIN
                                         */

    unsigned int    co_propList;        /* Property list.                    */

    unsigned int    co_usageCount;      /* Used to keep track of the number
                                         * of times this command is used.
                                         */
};


extern BearStatus   PbSource();
extern BearStatus   PbRunBlackBox();
extern BearStatus   PbRunCommandLoop();

extern Command      *PbFindCommand();
extern BearStatus   PbStoreCommand();
extern BearStatus   PbExecCommand();
extern int          PbCompareCommands();


EXTERN BearStatus PbHelpCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbVersionCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbLogConsoleCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbSourceCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbMailCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbReadChipCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbWriteChipCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbClearConsoleCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbOpenWindowCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbCloseWindowCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbGenericCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/* 
 * Viewing commands.
 */
EXTERN void PbCenterCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbMagAreaCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbPanCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbZoomInCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbZoomOutCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/* 
 * Cell commands.
 */
EXTERN void PbCreateCellCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDeleteCellCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbCreateDCellsCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDeleteDCellsCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

#ifdef USE_OLD_X_CODE
EXTERN void PbDuplicateCellCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbResizeParentCellCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));
#endif /* USE_OLD_X_CODE */

EXTERN void PbTransformCellCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/* 
 * Pin commands.
 */
EXTERN void PbAddPinToNetCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbCreatePinCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDeletePinCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDuplicatePinCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/* 
 * Net commands.
 */
EXTERN BearStatus PbBuildGraphCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbCreateNetCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDeleteNetCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

#ifdef USE_WIRE_CODE
EXTERN void PbRouteNetCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbWireCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));
#endif USE_WIRE_CODE

/* 
 * Information commands.
 */
EXTERN void PbShowCellPropCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbShowJunctPropCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbShowNetPropCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbShowPinPropCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbShowTilePropCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/*
 * Compaction and Decompaction commands.
 */
EXTERN BearStatus PbCompactCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbHorzCompactCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbVertCompactCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbDecompactCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbHorzDecompactCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbVertDecompactCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/*
 * Cluster and Placement commands.
 */
#ifdef USE_OLDP_CODE
EXTERN BearStatus PbClusterCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbPlacerCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbWritePlacementCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbReminderCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));
#endif USE_OLDP_CODE

#ifdef USE_NEWP_CODE
EXTERN BearStatus PbNewClusterCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbNewPlacerCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbReminderCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));
#endif USE_NEWP_CODE

EXTERN BearStatus PbShapeOptCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/*
 * Routing commands.
 */
EXTERN BearStatus PbAutoRouteCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDefChannelCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDefLChannelCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbGlobalRouterCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbNewGlobalRouterCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbRingRouterCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/*
 * Miscellaneous commands.
 */
EXTERN BearStatus PbChanPinArrCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbPushCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

/*
 * Debugging commands.
 */
#ifdef DEBUG
EXTERN BearStatus PbCrashProgramCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN BearStatus PbDebugCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbTestCmd
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));
#endif DEBUG

/*
 * Dialog creation commands.
 */
#ifdef USE_XDM_CODE
EXTERN void PbCreateDialog
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDeleteDialog
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));

EXTERN void PbDuplicateDialog
    ARGS((WindowData *winDataPtr, Point *screenPtr, int argCount, 
        char **argList));
#endif USE_XDM_CODE

#endif /* _PB_COMMANDS_H */

