
/* pbDebugCmds.c --
 *
 *     George Carvalho
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include <signal.h>
#include <strings.h>
#include <sys/time.h>
#include <ctype.h>
#include <math.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "geoChDecomposer.h"
#include "deque.h"
#include "topChDecomposer.h"
#include "globalRouter.h"
#include "globalSpacer.h"
#include "grEditor.h"
#include "newG.h"
#include "tileMapping.h"
#include "ringRouter.h"
#include "pbDialogs.h"
#include "polarBear.h"
#include "stack.h"


static PlaneData *gPlaneDataPtr;


#ifdef DEBUG
/* ARGSUSED */
BearStatus
PbCrashProgramCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    unsigned int    totalMem = 0;
    int             i;
    char            *memPtr;
    bool            memFlag = FALSE;

    /* parse arguements */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-m") == 0) {
            memFlag = TRUE;
            continue;
        }
        goto usage;
    }

    if (memFlag) {
        while (TRUE) {
            /*
             * Allocate 100K at a time.
             */
            MALLOC(char *, memPtr, 102400 * sizeof (char));
            totalMem += 102400;
            fprintf (stderr, "total memory allocated = %d K\n",
                totalMem / 1024);
        }
    }
    else {
        abort();
    }

usage:
    PbPrint(EZ_NORMAL_FONT, "crash\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: crash [-m]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT,
        "    -m keep allocating memory until you run out\n");

    return (BEAR_ERROR);
}

#endif /* DEBUG */


#ifdef DEBUG
/* ARGSUSED */
BearStatus
PbDebugCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;     /* not used */
int         argCount;
char        **argList;
{
    int     i;
    bool    dialogWinFlag = TRUE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        /* 
         * Don't pop up any dialog windows.  Used when you want to run
         * this command from a script.
         */
        if (strcmp (argList[i], "-NDW") == 0) {
            dialogWinFlag = FALSE;
            continue;
        }

        if (strcmp (argList[i], "-ngr") == 0) {
            ngr_DebugFlag = TRUE;
            continue;
        }

        (void) sprintf (bear_Msg, "unknown option %s\n", argList[i]);
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
        goto usage;
    }

#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && (dialogWinFlag == TRUE)) {
        PbSetDebugFlags();
    }
#endif BEAR_BLACK_BOX

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT, "Debug (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: Debug [-options ...]\n");

    PbPrint(EZ_NORMAL_FONT, "\nwhere options include:\n");
    PbPrint(EZ_NORMAL_FONT, "    -ngr set ngr_DebugFlag to TRUE\n");
    PbPrint(EZ_NORMAL_FONT, "    -NDW no dialog windows\n");

    return (BEAR_ERROR);
}

#endif /* DEBUG */


#ifndef BEAR_BLACK_BOX
#ifdef DEBUG
/* ARGSUSED */
static int
PrintRect(tilePtr, clientData)
Tile *tilePtr;
ClientData clientData;
{
    Rect rect;

    if (Tp_IsSolidTile(tilePtr)) {
        Tp_HorzTileToRect(gPlaneDataPtr, tilePtr, &rect);
        Mac_PrintRect(&rect);
    }

    return (MAGIC_CONTINUE_SEARCH);
}
#endif /* DEBUG */


#ifdef DEBUG
/* ARGSUSED */
void
PbTestCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    CellMaster  *parentCellPtr;
    ChipWin     *chipWinPtr;
    PlaneData   *planeDataPtr;
    DrawData    drawData;
    Window      win;
    Net         *netPtr;
    PinInst     *pinPtr;
    char        *netNamePtr;
    int         i;
    char        answer[80], menu[1000];
    double      aspectR;
    char        mode;

    chipWinPtr = PbGetChipWin(winDataPtr);
    parentCellPtr = PbGetCellMasterOfCHW(chipWinPtr);
    gPlaneDataPtr = planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);

    i = 0;
    sprintf (&menu[i], "\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "o) hilite indep. channel\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "a) rrdo\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "w) tile density\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "y) test rect slice\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "t) define side & bound pin\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "v) Rect Unify\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "j) itrJunct\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "s) itrSeg\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "k) draw pins around junction\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "d) h-face unify\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "r) v-face unify\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "i) hv-face unify\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "g) H bottleneck unify\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "z) V bottleneck unify\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "h) horizontal bn split\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "e) vertical bn split\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "l) h-f split\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "m) v-f split\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "f) floorplanning\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "n) show net connectivity\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "9) resize ring cell\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "8) start black box mode using bear_script\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "6) do global routing 50 times\n");
    i += strlen (&menu[i]);
    sprintf (&menu[i], "q) quit\n\nenter choice : ");
    i += strlen (&menu[i]);

    while (TRUE) {
start:
        PbPrint(EZ_NORMAL_FONT, menu);
        (void) Ez_GetLine(&win, screenPtr, answer, 80, &pbCmdLoop);
        if (pbCmdLoop == FALSE) {
            return;
        }
        switch (answer[0]) {
        case '6':
            {
                char    command[80];
                int     i;

                for (i = 1; i <= 50 ;i++) {
                    (void) printf ("i = %d\n", i);
                    planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);
                    Tp_FreePlaneData(planeDataPtr);
                    Mac_PutCellPlaneOfCell(NIL(PlaneData), parentCellPtr);

                    if (Tp_CreateTiPlaneForCellMaster(parentCellPtr) !=
                        BEAR_OK) {
    Bear_PrintErr("PbTestCmd() (case 6) failed : could not create tile plane.");
                        return;
                    }

                    BuildFloorPlan(parentCellPtr, TRUE);
                    strcpy(command, "gr -NDW");
                    (void) PbExecCommand(winDataPtr, screenPtr, command);
                    strcpy(command, "bg -ac");
                    (void) PbExecCommand(winDataPtr, screenPtr, command);
                }
            }
            break;
        case '5':
            {
                LinkedRPin  *lnRPinPtr;
                Net         *netPtr;

                if ((netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr)) !=
                    NIL(Net)) {
                    for (lnRPinPtr = netPtr->routePinList;
                        lnRPinPtr != NIL(LinkedRPin);
                        lnRPinPtr = lnRPinPtr->nextPtr) {

                        Mac_PrintRPin(lnRPinPtr->rPinPtr, (ClientData) NULL);
                    }
                }
            }
            break;
        case '7':
            fprintf (stdout, "=== start ===\n");
            (void) TiSrArea(NIL(Tile),
                Tp_GetHorzPlane(planeDataPtr),
                Tp_GetHorzPlaneBnd(planeDataPtr),
                PrintRect, (ClientData) NULL);
            fprintf (stdout, "=== end ===\n");
            fflush (stdout);
            break;
        case 'o':
            PbHighlightAllChannels(winDataPtr);
        break;
        case 'a':
{
    FloorPlanGraph  *fpgPtr;
    PlaneData       *planeDataPtr;
    WallJunction    *llJunctPtr;
    DrawData        *drawDataPtr;
    Stack           *channelStackPtr;
    Channel         *channelPtr;
    Deque           *channelQue;
    WallJunction    *endJunct1Ptr;
    WallJunction    *endJunct2Ptr;
    WallJunction    *junctPtr;
    int             a;

    if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE)
    {
    BuildFloorPlan(parentCellPtr, FALSE);
    (*winDataPtr->wd_redrawProc)(winDataPtr, NIL(Rect));
    }
    drawData.winDataPtr = winDataPtr;
    drawData.color = ez_HighLightColor;
    drawData.width = MAXWIDTH;
    drawDataPtr = &drawData;

    fpgPtr = Tp_GetFpg(planeDataPtr);
    channelStackPtr = StackNew(1);
    (void) Tcd_RRDO(fpgPtr, channelStackPtr);

    channelQue = MakeDeque();
    while (StackEmpty(channelStackPtr) == FALSE)
    {
    channelPtr = (Channel *)StackPop(channelStackPtr);
    PushDeque(channelQue, channelPtr);
    }
    StackFree(channelStackPtr);
    while (IsEmptyDeque(channelQue) == FALSE)
    {
    printf("press any character key to continue : ");
    scanf("%s",&a);
    channelPtr = (Channel *)PopDeque(channelQue);
    if (channelPtr->type == VERT_CHANNEL)
    {
        drawDataPtr->color = pbVChannelColor;
        endJunct1Ptr = channelPtr->channelUn.straightChannel.tlEndJunct;
        endJunct2Ptr = channelPtr->channelUn.straightChannel.brEndJunct;
        do
        {
        junctPtr  = Fpg_GetSouthJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
    }
    else if (channelPtr->type == HORZ_CHANNEL)
    {
        drawDataPtr->color = pbHChannelColor;
        endJunct1Ptr = channelPtr->channelUn.straightChannel.tlEndJunct;
        endJunct2Ptr = channelPtr->channelUn.straightChannel.brEndJunct;
        do
        {
        junctPtr  = Fpg_GetEastJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
    }
    else if (channelPtr->type == TOP_LEFT_L_CHANNEL)
    {
        drawDataPtr->color = pbLChannelColor;
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.vEndJunct;
        do
        {
        junctPtr  = Fpg_GetNorthJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.hEndJunct;
        do
        {
        junctPtr  = Fpg_GetWestJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
    }
    else if (channelPtr->type == TOP_RIGHT_L_CHANNEL)
    {
        drawDataPtr->color = pbLChannelColor;
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.vEndJunct;
        do
        {
        junctPtr  = Fpg_GetNorthJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.hEndJunct;
        do
        {
        junctPtr  = Fpg_GetEastJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
    }
    else if (channelPtr->type == BOTTOM_RIGHT_L_CHANNEL)
    {
        drawDataPtr->color = pbLChannelColor;
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.vEndJunct;
        do
        {
        junctPtr  = Fpg_GetSouthJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.hEndJunct;
        do
        {
        junctPtr  = Fpg_GetEastJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
    }
    else if (channelPtr->type == BOTTOM_LEFT_L_CHANNEL)
    {
        drawDataPtr->color = pbLChannelColor;
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.vEndJunct;
        do
        {
        junctPtr  = Fpg_GetSouthJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
        endJunct1Ptr = channelPtr->channelUn.lShapedChannel.cornerJunct;
        endJunct2Ptr = channelPtr->channelUn.lShapedChannel.hEndJunct;
        do
        {
        junctPtr  = Fpg_GetWestJunctOfWallJunct(endJunct1Ptr);
        Gre_DrawFromJunctToJunct(drawDataPtr, endJunct1Ptr, junctPtr);
        endJunct1Ptr = junctPtr;
        } while (endJunct1Ptr != endJunct2Ptr);
    }
    Ez_Flush();
    FREE((char *)channelPtr);
    }
}
        break;
        case 'w':
    {
        int       cap;
        int       den;
        PlaneData *planeDataPtr;
        BNLine    *curBnLinePtr;

#ifdef USE_WIRE_CODE
        (void) Gre_DoPgWiringForAllNets(parentCellPtr);
#endif USE_WIRE_CODE
        Gs_CalcCapDenOfTilePlane(parentCellPtr);

        planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);
        for (curBnLinePtr = Tp_GetHorzBnLineList(planeDataPtr);
         curBnLinePtr != NIL(BNLine);
         curBnLinePtr = curBnLinePtr->nextPtr)
        {
        cap = Tp_GetCapacityOfBnLine(curBnLinePtr);
        den = Tp_GetDensityOfBnLine(curBnLinePtr);
        printf("Horz: cap= %d, den= %d\n",cap,den);
        }
        for (curBnLinePtr = Tp_GetVertBnLineList(planeDataPtr);
         curBnLinePtr != NIL(BNLine);
         curBnLinePtr = curBnLinePtr->nextPtr)
        {
        cap = Tp_GetCapacityOfBnLine(curBnLinePtr);
        den = Tp_GetDensityOfBnLine(curBnLinePtr);
        printf("Vert: cap= %d, den= %d\n",cap,den);
        }
    }
    break;
    
    case '9':
    {
        CellInst        *coreInstPtr;
        CellMaster      *chipMasterPtr, *coreMasterPtr;
        PlaneData       *planeDataPtr;
        WallJunction    *llJunctPtr;
        int             adjust[4];

        chipWinPtr = PbGetChipWin(winDataPtr);
        coreMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
        planeDataPtr = Mac_GetCellPlaneOfCell(coreMasterPtr);

        coreInstPtr = Mac_GetInstOfCoreMaster(coreMasterPtr);
        chipMasterPtr = Mac_GetParentOfCell(coreInstPtr);

        adjust[0] = 0;
        adjust[1] = 0;
        adjust[2] = 100;
        adjust[3] = 100;

        (void) Rr_ResizeRing(coreMasterPtr, adjust);
        (void) Tp_IdentifySpaceTiles(planeDataPtr);
        (void) Fpg_CreateFloorPlanGraph(planeDataPtr, &llJunctPtr);

#ifdef USE_WIRE_CODE
        (void) Gre_DoPgWiringForAllNets(coreMasterPtr);
#endif USE_WIRE_CODE
        (void) Gs_CalcCapDenOfTilePlane(coreMasterPtr);
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    }
        break;
        case 'y':
    {
    FILE *fp;
    int  numOfPoints;
    LinkedPoint  *linkedPointList;
    LinkedPoint  *linkedPointPtr;
    LinkedRect   *linkedRectList;

    fp = fopen("koba.points","r");
    fscanf(fp, "%d", &numOfPoints);
    printf("numOfPoints : %d\n", numOfPoints);
    MALLOC(LinkedPoint *, linkedPointList, sizeof (LinkedPoint));
    linkedPointList->nextPtr = NIL(LinkedPoint);
    fscanf(fp, "%d %d",&linkedPointList->point.p_x,
               &linkedPointList->point.p_y);
    linkedPointPtr = linkedPointList;
    while (--numOfPoints > 0)
    {
        MALLOC(LinkedPoint *, linkedPointPtr->nextPtr, sizeof (LinkedPoint));
        linkedPointPtr = linkedPointPtr->nextPtr;
        linkedPointPtr->nextPtr = NIL(LinkedPoint);
        fscanf(fp, "%d %d",&linkedPointPtr->point.p_x,
                   &linkedPointPtr->point.p_y);
    }
    fclose(fp);
    Mac_PrintPointList(&GeoIdentityTransform, linkedPointList);

    Rs_HorzRectSlice(linkedPointList, &linkedRectList);
    printf("Horizontal Rect List\n");
    Mac_PrintRectList(&GeoIdentityTransform, linkedRectList);

    Rs_VertRectSlice(linkedPointList, &linkedRectList);
    printf("Vertical Rect List\n");
    Mac_PrintRectList(&GeoIdentityTransform, linkedRectList);
    }
    break;
        case 'v':
        {
#define BAD_RETURN \
{ \
    Ez_RingBell(); \
    Ez_DefineCursor(winDataPtr->wd_userWin, EZ_CROSS_CUR); \
    return; \
}

        Point           *point1Ptr; 
        Point           *point2Ptr;
        Tile            *horzDomTile1Ptr; 
        Tile            *vertDomTile1Ptr;
        Tile            *horzDomTile2Ptr; 
        Tile            *vertDomTile2Ptr;
        WallJunction    *junct1Ptr; 
        WallJunction    *junct2Ptr;
        Rect            dmntRect;
        WallSegment     *wallSegPtr;
        Tile            *bnTilePtr;

        if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE)
        {
        BuildFloorPlan(parentCellPtr, FALSE);
        Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
        } 
        else if ((chipWinPtr->chw_options & PB_SHOW_FPG_OPT) == FALSE) 
        {
        Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
        }
        Ez_DefineCursor(winDataPtr->wd_userWin, EZ_CIRCLE_OUTLINE_CUR);

        PbPrint(EZ_NORMAL_FONT, "Pick junction number one.\n");
        if (Gre_PickJunct(&horzDomTile1Ptr, &vertDomTile1Ptr, &junct1Ptr) ==
            BEAR_ERROR) 
        {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
        }

        point1Ptr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct1Ptr);

        PbPrint(EZ_NORMAL_FONT, "Pick junction number two.\n");
        if (Gre_PickJunct(&horzDomTile2Ptr, &vertDomTile2Ptr, &junct2Ptr) ==
        BEAR_ERROR) 
        {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
        }

        point2Ptr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct2Ptr);

        /*
         * Make sure that the wall junctions they picked are along the same
         * line and that they do not have any cells between them !!
         */
        if (point1Ptr->p_x == point2Ptr->p_x) 
        {
        /* The channel is vertical. */
        if (vertDomTile1Ptr != vertDomTile2Ptr) 
        {
            PbPrint(EZ_ITALIC_FONT,
            "There is a cell crossing the vertical channel.\n");
            BAD_RETURN;
        }
        else
        {
            Tp_VertTileToRect(planeDataPtr,vertDomTile1Ptr,&dmntRect);
            dmntRect.r_ytop = point1Ptr->p_y;
            dmntRect.r_ybot = point2Ptr->p_y;
            Tm_VRectUnify(parentCellPtr, planeDataPtr,dmntRect);
        }
        } 
        else if (point1Ptr->p_y == point2Ptr->p_y) 
        {
        /* The channel is horizontal. */
        if (horzDomTile1Ptr != horzDomTile2Ptr) 
        {
            PbPrint(EZ_ITALIC_FONT,
            "There is a cell crossing the horizontal channel.\n");
            BAD_RETURN;
        }
        else
        {
            Tp_HorzTileToRect(planeDataPtr,horzDomTile1Ptr,&dmntRect);
            dmntRect.r_xbot = point1Ptr->p_x;
            dmntRect.r_xtop = point2Ptr->p_x;
            Tm_HRectUnify(parentCellPtr, planeDataPtr,dmntRect);
        }
        } 
        else 
        {
        PbPrint(EZ_ITALIC_FONT, "It is not a valid definition for a straight channel.\n");
        BAD_RETURN;
        }

        Ez_DefineCursor(winDataPtr->wd_userWin, EZ_CROSS_CUR);
    }

    break;
    case 't':
    break;
    case 'j':
            if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE)
            {
                BuildFloorPlan(parentCellPtr, FALSE);
                (*winDataPtr->wd_redrawProc)(winDataPtr, NIL(Rect));
            }
            drawData.winDataPtr = winDataPtr;
            drawData.color = ez_HighLightColor;
            drawData.width = MAXWIDTH;
            (void) Fpg_ItrWallJunct(Mac_GetFpgOfCell(parentCellPtr),
                Gre_DrawWallJunct, (ClientData) &drawData);
            break;
        case 's':
            if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE)
            {
                BuildFloorPlan(parentCellPtr, FALSE);
                (*winDataPtr->wd_redrawProc)(winDataPtr, NIL(Rect));
            }
            drawData.winDataPtr = winDataPtr;
            drawData.color = ez_HighLightColor;
            drawData.width = MAXWIDTH;
            (void) Fpg_ItrWallSeg(Mac_GetFpgOfCell(parentCellPtr),
                Gre_DrawWallSeg, (ClientData) &drawData);
            break;
        case 'k':
            {
                Point               *pointPtr;
                Tile                *horzDomTilePtr, vertDomTilePtr;
                WallJunction        *junctPtr;
                PinsAroundJunction  pinLists;
                LinkedPinInst       *pinPtr;
                LinkedPPin          *ppinPtr;
                LinkedRPin          *rpinPtr;

                if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE) {
                    BuildFloorPlan(parentCellPtr, FALSE);
                    (*winDataPtr->wd_redrawProc)(winDataPtr, NIL(Rect));
                }

                Ez_DefineCursor(winDataPtr->wd_userWin, EZ_CIRCLE_OUTLINE_CUR);
                PbPrint(EZ_NORMAL_FONT, "pick a wall junction\n");
                Gre_PickJunct(&horzDomTilePtr, &vertDomTilePtr, &junctPtr);
                pointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, junctPtr);
                Ez_Flush();

                Tm_PinsAroundJunction(parentCellPtr, planeDataPtr, junctPtr, &pinLists);
    
                pinPtr = pinLists.pinInstList;
                while (pinPtr != NIL(LinkedPinInst))
                {
                    (void) Mac_PrintPinInst(pinPtr->pinInstPtr, 
                      (ClientData) NULL);
                    pinPtr = pinPtr->nextPtr;
                }
        printf("\n");

                ppinPtr = pinLists.horzPPinList;
                while (ppinPtr != NIL(LinkedPPin))
                {
                    (void) Tp_PrintPPin(ppinPtr->ppinPtr,
                        (ClientData) NULL);
                    ppinPtr = ppinPtr->nextPtr;
                }
        printf("\n");
                ppinPtr = pinLists.vertPPinList;
                while (ppinPtr != NIL(LinkedPPin))
                {
                    (void) Tp_PrintPPin(ppinPtr->ppinPtr,
                        (ClientData) NULL);
                    ppinPtr = ppinPtr->nextPtr;
                }
        printf("\n");
                rpinPtr = pinLists.rPinList;
                while (rpinPtr != NIL(LinkedRPin))
                {
                    (void) Mac_PrintRPin(rpinPtr->rPinPtr, (ClientData) NULL);
                    rpinPtr = rpinPtr->nextPtr;
                }
                Ez_DefineCursor(winDataPtr->wd_userWin, EZ_CROSS_CUR);
                break;
            }
        case 'd':
            Tm_HFOnlyUnify(parentCellPtr, planeDataPtr);
            Tp_FreeAllPPinsOnVertPlane(planeDataPtr);
            if (PbHorzTilesOn(winDataPtr)) 
            {
                (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
            }
            break;
        case 'r':
            Tm_VFOnlyUnify(parentCellPtr, planeDataPtr);
            Tp_FreeAllPPinsOnHorzPlane(planeDataPtr);
            if (PbVertTilesOn(winDataPtr)) 
        {
            (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
        }
            break;
        case 'i':
            Tm_HVFUnify(parentCellPtr, planeDataPtr);
            if (PbBothTilesOn(winDataPtr)) 
        {
            (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
        }
            break;
        case 'g':
            Tm_HBnUnify(parentCellPtr);
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
            break;
        case 'z':
            Tm_VBnUnify(parentCellPtr);
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
            break;
        case 'h':
    {
        WallJunction    *llJunctPtr;
            Tm_InsDCellOnHTP(parentCellPtr);
        (void) Fpg_CreateFloorPlanGraph(planeDataPtr, &llJunctPtr);
            (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
            break;
    }
        case 'e':
    {
        WallJunction    *llJunctPtr;
            Tm_InsDCellOnVTP(parentCellPtr);
        (void) Fpg_CreateFloorPlanGraph(planeDataPtr, &llJunctPtr);
            (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
            break;
    }
        case 'l':
            Tm_HFSplit(parentCellPtr);
            if (PbBothTilesOn(winDataPtr)) 
        {
            (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
        }
            break;
        case 'm':
            Tm_VFSplit(parentCellPtr);
            if (PbBothTilesOn(winDataPtr)) 
        {
            (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
        }
            break;

        case 'n':
/*
            netPtr = Mac_GetNetListOfCell(parentCellPtr);
 */
            PbShowConn(winDataPtr,parentCellPtr);
            break;

        case 'q':
            return;
        default:
            Ez_RingBell();
            break;
        }
    }
}
#endif /* DEBUG */
#endif /* BEAR_BLACK_BOX */



