
/* pbDrawChip.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#if !defined(BEAR_BLACK_BOX)
/*
 * This file does not need to be compiled when the black box version of
 * the BEAR-FP system is built.
 */

#include "bearCopyRight.h"
#include "port.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "routeDB.h"
#include "ezPlot.h"
#include "bearGrUtils.h"
#include "grEditor.h"
#include "polarBear.h"


typedef struct _TiDrawData TiDrawData;
struct _TiDrawData {
    Window      chipWin;
    double      (*scrMatrix)[4];    /* The screen matrix contains the 
                                     * transformation from world 
                                     * coordinates to screen coordinates.
                                     */
    Rect        horzPlaneBnd;
    Rect        horzPlaneClipBnd;
    Rect        vertPlaneClipBnd;
    bool        overLap, showBnTiles, showDomTiles;

    CellMaster  *cellMasterPtr;
    DummyCell   *dcellList;         /* dummy cell list                      */
    RouteCell   *rcellList;         /* route cell list                      */
    int         color;
    WindowData  *winDataPtr;
};


/* ---------- Local Procedures ---------- */
static BearStatus   DrawLineSeg();
static BearStatus   OutlineCellInst(), OutlineDCell();
static BearStatus   LabelCellInst();
static void         Outline();
static int          DrawHorzTile(), DrawVertTile();

#ifdef USE_DR_CODE
static BearStatus   OutlineRCell();
static BearStatus   DrawRCell(), DrawRCell2();
static BearStatus   DrawRing(), DrawRing2();
#endif /* USE_DR_CODE */


static BearStatus 
DrawLineSeg(lineSegPtr, tiDrawDataPtr)
LineSeg     *lineSegPtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Draw line segment.
 */
{
    Ez_DrawWorldLine(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
        lineSegPtr->line.point1.p_x, lineSegPtr->line.point1.p_y,
        lineSegPtr->line.point2.p_x, lineSegPtr->line.point2.p_y,
        EZ_SOLIDLINE1, GXcopy, tiDrawDataPtr->color, pbChipColor);

    return (BEAR_CONTINUE_SEARCH);
}



#ifdef USE_DR_CODE
static BearStatus
DrawRCell(rcellPtr, tiDrawDataPtr)
RouteCell   *rcellPtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Draw a route cell.
 */
{
    DrawCif             drawCif;
    GenericCell         gcell;
    LinkedRect          *horzRectList;
    Rect                drawRect;
    Transform           *transPtr;
    register LinkedRect *rectPtr;

    if (rcellPtr->cifFileNamePtr == NIL(char)) {
        if (rcellPtr->routeModPtr != NIL(module)) {
            gcell.unType = ROUTE_CELL;
            gcell.unValue.routeCellPtr = rcellPtr;
            horzRectList = Mac_GetHorzRectListOfGCell(&gcell);
            transPtr = Mac_GetTransformOfGCell(&gcell);
            for (rectPtr = horzRectList;
                rectPtr != NIL(LinkedRect);
                rectPtr = rectPtr->r_next) {

                GeoTransRect(transPtr, &rectPtr->r_r, &drawRect);
                Ez_FillRect(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                    &drawRect, tiDrawDataPtr->color);
            }
        }
    }
    else {
        drawCif.fileNamePtr = rcellPtr->cifFileNamePtr;
        drawCif.win = tiDrawDataPtr->chipWin;
        drawCif.scrMatrix = tiDrawDataPtr->scrMatrix;
        drawCif.options = BGU_ENTIRE_CHIP | BGU_CALC_VIEW_TRANS;
        (void) Bgu_DrawCifPlot(&drawCif);
    }

    return (BEAR_CONTINUE_SEARCH);
}
#endif /* USE_DR_CODE */



#ifdef USE_DR_CODE
static BearStatus
DrawRCell2(rcellPtr, tiDrawDataPtr)
RouteCell   *rcellPtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Draw the routing segments and junctions of the route cell.
 */
{
    /*
    if (rcellPtr->routeModPtr != NIL(module)
    && Mac_IsRCellMarked(rcellPtr) == BEAR_TRUE)
    */
    if (rcellPtr->routeModPtr != NIL(module)) {
        (void) Rdb_DrawRCell(rcellPtr, (ClientData) tiDrawDataPtr->winDataPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}
#endif /* USE_DR_CODE */



#ifdef USE_DR_CODE
static BearStatus
DrawRing(ringPtr, tiDrawDataPtr)
Ring        *ringPtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Draw the background of the routing ring.
 */
{
    DrawCif             drawCif;
    LinkedRect          *horzRectList;
    Rect                drawRect;
    Transform           *transPtr;
    char                *cifFileName;
    register LinkedRect *rectPtr;

    if ((cifFileName = Mac_GetCifFileNameOfRing(ringPtr)) == NIL(char)) {
        horzRectList = ringPtr->horzRectList;
        transPtr = Mac_GetTransformOfRing(ringPtr);
        for (rectPtr = horzRectList;
             rectPtr != NIL(LinkedRect);
             rectPtr = rectPtr->r_next) {

            GeoTransRect(transPtr, &rectPtr->r_r, &drawRect);
            Ez_FillRect(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                &drawRect, tiDrawDataPtr->color);
        }
    } 
    else {
        drawCif.fileNamePtr = cifFileName;
        drawCif.win = tiDrawDataPtr->chipWin;
        drawCif.scrMatrix = tiDrawDataPtr->scrMatrix;
        drawCif.options = BGU_ENTIRE_CHIP | BGU_CALC_VIEW_TRANS;
        (void) Bgu_DrawCifPlot(&drawCif);
    }

    return (BEAR_CONTINUE_SEARCH);
}
#endif /* USE_DR_CODE */



#ifdef USE_DR_CODE
static BearStatus
DrawRing2(ringPtr, tiDrawDataPtr)
Ring          *ringPtr;
TiDrawData    *tiDrawDataPtr;
/*
 * Draw the routing segments and junctions of the routing ring.
 */
{
    module *rModPtr;

    if ((rModPtr = Mac_GetRModOfRing(ringPtr)) != NIL(module)) {
    /*
        Rdb_DrawRMod(ringPtr->routeModPtr, Mac_GetTransformOfRing(ringPtr),
            tiDrawDataPtr->winDataPtr);
    */
        (void) Rdb_DrawRing(ringPtr, Mac_GetTransformOfRing(ringPtr),
            (ClientData) tiDrawDataPtr->winDataPtr);
    }

    return (BEAR_CONTINUE_SEARCH);
}
#endif /* USE_DR_CODE */



/* ARGSUSED */
static int
DrawSolidTile(horzTilePtr, tiDrawDataPtr)
Tile        *horzTilePtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Note: you cannot draw route cells tile by tile because once
 *   the actual routing has been generated the cif file containing
 *   the routing information must be drawn all at once.
 */
{
    Rect horzRect;

    TITORECT(horzTilePtr, &horzRect);
    GeoClip(&horzRect, &tiDrawDataPtr->horzPlaneClipBnd);
    if (horzRect.r_xtop > horzRect.r_xbot) {
        if (Tp_IsCellInstTile(horzTilePtr)) {
            Ez_FillRect(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                &horzRect, pbCellColor);
        } 
        else if (Tp_IsDCellTile(horzTilePtr)) {
            Ez_FillRect(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                &horzRect, pbDCellColor);
        }
    }
    return (0); /* continue search */
}



static void
Outline(tiDrawDataPtr)
TiDrawData *tiDrawDataPtr;
/* 
 * Draw the outline of all the dummy cells, route cells, cell instances,
 * as well as the outline of the planes boundary.  Note that you should
 * outline the cell instances last so that it will not be obscured by
 * anything.
 */
{
    XSync(ez_Display, 0);

    (void) Mac_ItrDCell(tiDrawDataPtr->dcellList,
        OutlineDCell, (ClientData) tiDrawDataPtr);

#ifdef USE_DR_CODE
    tiDrawDataPtr->color = pbCellColor;
    (void) Mac_ItrRCell(tiDrawDataPtr->rcellList,
        OutlineRCell, (ClientData) tiDrawDataPtr);
#endif /* USE_DR_CODE */

    tiDrawDataPtr->color = pbCellBorderColor;
    (void) Mac_ItrChildrenOfCell(tiDrawDataPtr->cellMasterPtr,
        OutlineCellInst, (ClientData) tiDrawDataPtr);

    Ez_OutlineRect(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
        &tiDrawDataPtr->horzPlaneBnd, EZ_SOLIDLINE1,
        pbCellBorderColor, pbChipColor);
}



static BearStatus
OutlineCellInst(cellInstPtr, tiDrawDataPtr)
CellInst    *cellInstPtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Draw the outline of a cell instance.
 */
{
    (void) Mac_ItrLineSeg(Mac_GetBoundaryOfCell(cellInstPtr), DrawLineSeg,
        (ClientData) tiDrawDataPtr);

    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
OutlineDCell(dcellPtr, tiDrawDataPtr)
DummyCell   *dcellPtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Draw the outline of a dummy cell.
 */
{
    GenericCell gcell;
    Rect        *rectPtr;

    gcell.unType = DUMMY_CELL;
    gcell.unValue.dummyCellPtr = dcellPtr;
    rectPtr = &(Mac_GetHorzRectListOfGCell(&gcell))->r_r;
    Ez_OutlineRect(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
        rectPtr, EZ_SOLIDLINE1, tiDrawDataPtr->color, pbChipColor);

    return (BEAR_CONTINUE_SEARCH);
}



#ifdef USE_DR_CODE
static BearStatus
OutlineRCell(rcellPtr, tiDrawDataPtr)
RouteCell   *rcellPtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Draw the outline of a route cell.
 */
{
    Point   *firstBLPointPtr; 
    Point   *lastBLPointPtr;
    Point   *firstTRPointPtr; 
    Point   *lastTRPointPtr;
    Point   point1;
    Point   point2;
    LinkedPoint *curPointPtr;
    Transform   *transPtr;

    if (rcellPtr->routeModPtr == NIL(module))
    {
       return (BEAR_CONTINUE_SEARCH);
    }

    /* Draw bottom or left boundary */
    transPtr = Mac_GetTransformOfRCell(rcellPtr);
    curPointPtr = rcellPtr->blBoundary;
    firstBLPointPtr = &(curPointPtr->point);
    while (curPointPtr->nextPtr != NIL(LinkedPoint))
    {
    GeoTransPoint(transPtr, &curPointPtr->point, &point1);
    GeoTransPoint(transPtr, &curPointPtr->nextPtr->point, &point2);
        Ez_DrawWorldLine(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                     point1.p_x, point1.p_y,
                     point2.p_x, point2.p_y,
            EZ_SOLIDLINE1, GXcopy, tiDrawDataPtr->color, pbChipColor);
        curPointPtr = curPointPtr->nextPtr;
    }
    lastBLPointPtr = &(curPointPtr->point);

    /* Draw top or right boundary */
    curPointPtr = rcellPtr->trBoundary;
    firstTRPointPtr = &(curPointPtr->point);
    while (curPointPtr->nextPtr != NIL(LinkedPoint))
    {
    GeoTransPoint(transPtr, &curPointPtr->point, &point1);
    GeoTransPoint(transPtr, &curPointPtr->nextPtr->point, &point2);
        Ez_DrawWorldLine(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                     point1.p_x, point1.p_y,
                     point2.p_x, point2.p_y,
            EZ_SOLIDLINE1, GXcopy, tiDrawDataPtr->color, pbChipColor);
        curPointPtr = curPointPtr->nextPtr;
    }
    lastTRPointPtr = &(curPointPtr->point);

    /* Draw two ends */
    GeoTransPoint(transPtr, firstBLPointPtr, &point1);
    GeoTransPoint(transPtr, firstTRPointPtr, &point2);
    Ez_DrawWorldLine(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                     point1.p_x, point1.p_y,
                     point2.p_x, point2.p_y,
    EZ_SOLIDLINE1, GXcopy, tiDrawDataPtr->color, pbChipColor);

    GeoTransPoint(transPtr, lastBLPointPtr, &point1);
    GeoTransPoint(transPtr, lastTRPointPtr, &point2);
    Ez_DrawWorldLine(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
                     point1.p_x, point1.p_y,
                     point2.p_x, point2.p_y,
    EZ_SOLIDLINE1, GXcopy, tiDrawDataPtr->color, pbChipColor);

    return (BEAR_CONTINUE_SEARCH);
}
#endif /* USE_DR_CODE */



static int
DrawHorzTile(horzTilePtr, tiDrawDataPtr)
Tile        *horzTilePtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Note: you cannot draw route cells tile by tile because once
 *   the actual routing has been generated the cif file containing
 *   the routing information must be drawn all at once.
 */
{
    Rect horzRect, horzClipRect;
    Window chipWin;
    double (*scrMatrix)[4];

    chipWin = tiDrawDataPtr->chipWin;
    scrMatrix = tiDrawDataPtr->scrMatrix;

    TITORECT(horzTilePtr, &horzRect);
    Mac_CopyRect(&horzClipRect, &horzRect);
    GeoClip(&horzClipRect, &tiDrawDataPtr->horzPlaneClipBnd);
    if (horzClipRect.r_xtop > horzClipRect.r_xbot) {
        if (tiDrawDataPtr->overLap) {
            if (Tp_IsSpaceTile(horzTilePtr)) {
                if (horzClipRect.r_ybot == horzRect.r_ybot) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xbot, horzClipRect.r_ybot,
                        horzClipRect.r_xtop, horzClipRect.r_ybot,
                        EZ_DOTTEDLINE1, GXcopy, pbCellColor, pbChipColor);
                }
                if (horzClipRect.r_ytop == horzRect.r_ytop) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xbot, horzClipRect.r_ytop,
                        horzClipRect.r_xtop, horzClipRect.r_ytop,
                        EZ_DOTTEDLINE1, GXcopy, pbCellColor, pbChipColor);
                }
            }
        } else {
            if (Tp_IsSpaceTile(horzTilePtr)) {
                if (horzClipRect.r_ybot == horzRect.r_ybot) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xbot, horzClipRect.r_ybot,
                        horzClipRect.r_xtop, horzClipRect.r_ybot,
                        EZ_SOLIDLINE1, GXcopy, pbCellColor, pbChipColor);
                }
                if (horzClipRect.r_ytop == horzRect.r_ytop) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xbot, horzClipRect.r_ytop,
                        horzClipRect.r_xtop, horzClipRect.r_ytop,
                        EZ_SOLIDLINE1, GXcopy, pbCellColor, pbChipColor);
                }
            } else if (Tp_IsCellInstTile(horzTilePtr)) {
                Ez_StippleRect(chipWin, scrMatrix, &horzClipRect, 
                    pbCellColor, pbCellPixmap, GXcopy);
            } else if (Tp_IsDCellTile(horzTilePtr)) {
                Ez_StippleRect(chipWin, scrMatrix, &horzClipRect,
                    pbDCellColor, pbCellPixmap, GXcopy);
            }
        }
        if (tiDrawDataPtr->showBnTiles && Tp_IsBottleneckTile(horzTilePtr)) {
            Ez_StippleRect(chipWin, scrMatrix, &horzClipRect, 
                pbHorzBnColor, ez_RightSlashBitmap, GXcopy);
        }
        if (tiDrawDataPtr->showDomTiles && Tp_IsDominantTile(horzTilePtr)) {
            Ez_StippleRect(chipWin, scrMatrix, &horzClipRect, 
                pbHorzDomColor, ez_RightSlashBitmap, GXcopy);
        }
    }
    return (0); /* continue search */
}

    

static int
DrawVertTile(vertTilePtr, tiDrawDataPtr)
Tile        *vertTilePtr;
TiDrawData  *tiDrawDataPtr;
/*
 * Note: you cannot draw route cells tile by tile because once
 *   the actual routing has been generated the cif file containing
 *   the routing information must be drawn all at once.
 */
{
    Rect horzRect, horzClipRect, vertRect, vertClipRect;
    Window chipWin;
    double (*scrMatrix)[4];

    chipWin = tiDrawDataPtr->chipWin;
    scrMatrix = tiDrawDataPtr->scrMatrix;

    TITORECT(vertTilePtr, &vertRect);
    Tp_VRectToHRect(&vertRect, &horzRect);

    Mac_CopyRect(&vertClipRect, &vertRect);
    GeoClip(&vertClipRect, &tiDrawDataPtr->vertPlaneClipBnd);
    Tp_VRectToHRect(&vertClipRect, &horzClipRect);
    if (horzClipRect.r_xtop > horzClipRect.r_xbot) {
        if (tiDrawDataPtr->overLap) {
            if (Tp_IsSpaceTile(vertTilePtr)) {
                if (horzClipRect.r_xbot == horzRect.r_xbot) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xbot, horzClipRect.r_ybot,
                        horzClipRect.r_xbot, horzClipRect.r_ytop,
                        EZ_DOTTEDLINE1, GXcopy,
                        pbCellBorderColor, pbChipColor);
                }
                if (horzClipRect.r_xtop == horzRect.r_xtop) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xtop, horzClipRect.r_ybot,
                        horzClipRect.r_xtop, horzClipRect.r_ytop,
                        EZ_DOTTEDLINE1, GXcopy,
                        pbCellBorderColor, pbChipColor);
                }
            }
        } else {
            if (Tp_IsSpaceTile(vertTilePtr)) {
                if (horzClipRect.r_xbot == horzRect.r_xbot) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xbot, horzClipRect.r_ybot,
                        horzClipRect.r_xbot, horzClipRect.r_ytop,
                        EZ_SOLIDLINE1, GXcopy,
                        pbCellBorderColor, pbChipColor);
                }
                if (horzClipRect.r_xtop == horzRect.r_xtop) {
                    Ez_DrawWorldLine(chipWin, scrMatrix,
                        horzClipRect.r_xtop, horzClipRect.r_ybot,
                        horzClipRect.r_xtop, horzClipRect.r_ytop,
                        EZ_SOLIDLINE1, GXcopy,
                        pbCellBorderColor, pbChipColor);
                }
            } else if (Tp_IsCellInstTile(vertTilePtr)) {
                Ez_StippleRect(chipWin, scrMatrix, &horzClipRect, pbCellColor,
                    pbCellPixmap, GXcopy);
            } else if (Tp_IsDCellTile(vertTilePtr)) {
                Ez_FillRect(chipWin, scrMatrix, &horzClipRect,
                    pbDCellColor);
            }
        }
        if (tiDrawDataPtr->showBnTiles && Tp_IsBottleneckTile(vertTilePtr)) {
            Ez_StippleRect(chipWin, scrMatrix, &horzClipRect, 
                pbVertBnColor, ez_LeftSlashBitmap, GXcopy);
        }
        if (tiDrawDataPtr->showDomTiles && Tp_IsDominantTile(vertTilePtr)) {
            Ez_StippleRect(chipWin, scrMatrix, &horzClipRect, 
                pbVertDomColor, ez_LeftSlashBitmap, GXcopy);
        }
    }
    return (0); /* continue search */
}

    

static BearStatus
LabelCellInst(cellInstPtr, tiDrawDataPtr)
CellInst    *cellInstPtr;
TiDrawData  *tiDrawDataPtr;
{
    Rect    absRect;
    int     x, y;
    char    *namePtr;

    Mac_GetAbsRectOfCellInst(cellInstPtr, &absRect);
    x = (absRect.r_xbot + absRect.r_xtop) / 2;
    y = (absRect.r_ybot + absRect.r_ytop) / 2;

    namePtr = Mac_GetNameOfCellInst(cellInstPtr);
    Ez_WorldText(tiDrawDataPtr->chipWin, tiDrawDataPtr->scrMatrix,
        x, y, (namePtr != NIL(char)) ? namePtr : "unknown");

    return (BEAR_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * PbDrawChipRegion() --
 *
 * This procedure is designed to be used by an "interator".  It
 * clears and redraws a chip window.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion, even if theres an
 *   error because the interator should continue on to the next window.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
PbDrawChipRegion(winDataPtr, subWinBndPtr)
WindowData  *winDataPtr;
Rect        *subWinBndPtr;              /* subWinBndPtr contains the world
                                         * coordinates of a portion of the
                                         * window that is to be drawn, if it
                                         * is NULL the whole window is drawn.
                                         */
{
    CellMaster  *cellMasterPtr;
    ChipWin     *chipWinPtr;
    DrawBnLine  drawBnLine;
    DrawData    drawData;
    DrawGCell   drawGCell;
    DrawNet     drawNet;
    DrawPin     drawPin;
    DrawRidge   drawRidge;
    LabelNet    labelNet;
    Net         *netPtr;
    PinInst     *pinInstPtr;
    PlaneData   *planeDataPtr;
    TiDrawData  tiDrawData;
    int         winOptions;

#ifdef DEBUG
    if (winDataPtr == NIL(WindowData)) {
        Bear_PrintErr("PbDrawChipRegion() failed : winDataPtr is NULL");
        return (BEAR_CONTINUE_SEARCH);
    }
#endif /* DEBUG */

    /* 
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    if ((cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr)) ==
        NIL(CellMaster)) {
        XClearWindow(ez_Display, winDataPtr->wd_userWin);
        return (BEAR_CONTINUE_SEARCH);
    }
    if ((planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr)) ==
        NIL(PlaneData)) {
        XClearWindow(ez_Display, winDataPtr->wd_userWin);
        return (BEAR_CONTINUE_SEARCH);
    }

    winOptions = PbGetOptionsOfCHW(chipWinPtr);
    
    drawData.winDataPtr = winDataPtr;

    tiDrawData.winDataPtr = winDataPtr;
    tiDrawData.chipWin = winDataPtr->wd_userWin;
    tiDrawData.scrMatrix = winDataPtr->wd_scrMatrix;
    Mac_CopyRect(&tiDrawData.horzPlaneBnd, Tp_GetHorzPlaneBnd(planeDataPtr));
    tiDrawData.showBnTiles = winOptions & PB_SHOW_BN_TILES_OPT;
    tiDrawData.showDomTiles = winOptions & PB_SHOW_DOM_TILES_OPT;
    tiDrawData.dcellList = Tp_GetDCellList(planeDataPtr);
    tiDrawData.rcellList = Mac_GetRCellListOfCell(cellMasterPtr);
    tiDrawData.cellMasterPtr = cellMasterPtr;
    tiDrawData.color = pbCellBorderColor;

    if ((subWinBndPtr == NIL(Rect)) || pbUserIsPickingRidge) {
        /* 
         * Redraw the whole window.
         */
        if (Ez_FindOverlap(&tiDrawData.horzPlaneBnd,
            &winDataPtr->wd_imageBnd,
            &tiDrawData.horzPlaneClipBnd) == FALSE) {
            return (BEAR_CONTINUE_SEARCH);
        }
        XClearWindow(ez_Display, winDataPtr->wd_userWin);
    }
    else {
        /*
         * Redraw part of the window.
         */
        if (Ez_FindOverlap(&tiDrawData.horzPlaneBnd,
            subWinBndPtr,
            &tiDrawData.horzPlaneClipBnd) == FALSE) {
            return (BEAR_CONTINUE_SEARCH);
        }

        /* Adjust redraw area to be sure everything is redrawn. */
        tiDrawData.horzPlaneClipBnd.r_ll.p_x--;
        tiDrawData.horzPlaneClipBnd.r_ll.p_y--;
        tiDrawData.horzPlaneClipBnd.r_ur.p_x++;
        tiDrawData.horzPlaneClipBnd.r_ur.p_y++;
    }

    if (Mac_GetPlacedFlagOfCell(cellMasterPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (Mac_IsRingRoutingDone(cellMasterPtr) == FALSE) {
        Ez_StippleRect(tiDrawData.chipWin, tiDrawData.scrMatrix,
            &tiDrawData.horzPlaneClipBnd, pbChipColor, pbChipPixmap, GXcopy);
        XSync(ez_Display,0);
    }

    if (Mac_IsNumOfCellsZero(cellMasterPtr) == TRUE) {
        Outline(&tiDrawData);
        return (BEAR_CONTINUE_SEARCH);
    }

    if (((winOptions & PB_SHOW_HORZ_PLANE_OPT) == FALSE) &&
        ((winOptions & PB_SHOW_VERT_PLANE_OPT) == FALSE)) {
        /*
         * Only draw the cells.
         */
        (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr), 
            &tiDrawData.horzPlaneClipBnd, DrawSolidTile,
            (ClientData) &tiDrawData);
    }
    else {
        if (winOptions & PB_SHOW_HORZ_PLANE_OPT) {
            /*
             * Draw the cells and the horizontal tile plane.
             */
            tiDrawData.overLap = FALSE;
            (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr), 
                &tiDrawData.horzPlaneClipBnd, DrawHorzTile,
                (ClientData) &tiDrawData);

            if (winOptions & PB_SHOW_VERT_PLANE_OPT) {
                /*
                 * Draw the vertical tile plane.
                 */
                Tp_HRectToVRect(&tiDrawData.horzPlaneClipBnd,
                    &tiDrawData.vertPlaneClipBnd);
                tiDrawData.overLap = TRUE;
                (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr), 
                    &tiDrawData.vertPlaneClipBnd, DrawVertTile,
                    (ClientData) &tiDrawData);
                if (tiDrawData.showBnTiles) {
                    /* Draw the vertical bottle neck lines. */
                    drawData.color = pbVertBnColor;
                    Tp_ItrBnLine(Tp_GetVertBnLineList(planeDataPtr),
                        PbDrawBnLineOne, (ClientData) &drawData);
                }
            }
            if (tiDrawData.showBnTiles) {
                /* Draw the horizontal bottle neck lines. */
                drawData.color = pbHorzBnColor;
                Tp_ItrBnLine(Tp_GetHorzBnLineList(planeDataPtr),
                    PbDrawBnLineOne, (ClientData) &drawData);
            }
        }
        else if (winOptions & PB_SHOW_VERT_PLANE_OPT) {
            /*
             * Draw the cells and the vertical tile plane.
             */
            Tp_HRectToVRect(&tiDrawData.horzPlaneClipBnd,
                &tiDrawData.vertPlaneClipBnd);
            tiDrawData.overLap = FALSE;
            (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr), 
                &tiDrawData.vertPlaneClipBnd, DrawVertTile,
                (ClientData) &tiDrawData);
            if (tiDrawData.showBnTiles) {
                /* Draw the vertical bottle neck lines. */
                drawData.color = pbVertBnColor;
                Tp_ItrBnLine(Tp_GetVertBnLineList(planeDataPtr),
                    PbDrawBnLineOne, (ClientData) &drawData);
            }
        }
    }

    /*
     * Draw the route cells and the routing ring.
     */
#ifdef USE_DR_CODE
    tiDrawData.color = pbRCellColor;
    Mac_ItrRCell(Mac_GetRCellListOfCell(cellMasterPtr),
        DrawRCell, (ClientData) &tiDrawData);
    if (cellMasterPtr->contents.ringPtr != NIL(Ring)) {
        (void) DrawRing(cellMasterPtr->contents.ringPtr, &tiDrawData);
    }
    Mac_ItrRCell(Mac_GetRCellListOfCell(cellMasterPtr),
        DrawRCell2, (ClientData) &tiDrawData);
    if (cellMasterPtr->contents.ringPtr != NIL(Ring)) {
        (void) DrawRing2(cellMasterPtr->contents.ringPtr, &tiDrawData);
    }
#endif /* USE_DR_CODE */

    Outline(&tiDrawData);


    if ((drawBnLine.bnLinePtr = PbGetBnLineOfCHW(chipWinPtr)) !=
        NIL(BNLine)) {
        /*
         * High-light the indicated bottleneck line.
         */
        drawBnLine.bnLineColor = ez_HighLightColor;
        (void) PbDrawBnLineTwo(winDataPtr, &drawBnLine);
    }

    if (winOptions & PB_SHOW_FPG_OPT) {
        if (Mac_GetFpgOfCell(cellMasterPtr) != NIL(WallJunction)) {
            /*
             * Draw the floor plan graph.
             */
            Gre_ChooseWallJunct(FPG_CENTER);
            (void) PbDrawFloorPlanGraph(winDataPtr);
        }
    }

    if (winOptions & PB_SHOW_PINS_OPT) {
        /*
         * Draw all of the pins.
         */
        drawData.color = pbPinColor;
        Gre_DrawAllPins(&drawData);
    }

    drawNet.cellMasterPtr = cellMasterPtr;
    if ((netPtr = PbGetNetOfCHW(FPG_NW, chipWinPtr)) != NIL(Net)) {
        drawNet.netPtr = netPtr;
        drawNet.whichNet = FPG_NW;
        drawNet.netColor = PbGetColorOfNet(FPG_NW);
        Gre_DrawNet(winDataPtr, &drawNet);
    } 
    if ((netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr)) != NIL(Net)) {
        drawNet.netPtr = netPtr;
        drawNet.whichNet = FPG_CENTER;
        drawNet.netColor = PbGetColorOfNet(FPG_CENTER);
        Gre_DrawNet(winDataPtr, &drawNet);
    } 
    if ((netPtr = PbGetNetOfCHW(FPG_SE, chipWinPtr)) != NIL(Net)) {
        drawNet.netPtr = netPtr;
        drawNet.whichNet = FPG_SE;
        drawNet.netColor = PbGetColorOfNet(FPG_SE);
        Gre_DrawNet(winDataPtr, &drawNet);
    } 

    if (winOptions & PB_SHOW_PINS_OPT) {
        if ((pinInstPtr = PbGetPinOfCHW(chipWinPtr)) != NIL(PinInst)) {
            /*
             * If the pins are currently being displayed then
             * high-light the indicated pin as well.
             */
            drawPin.pinInstPtr = pinInstPtr;
            drawPin.pinColor = ez_HighLightColor;
            Gre_DrawPinTwo(winDataPtr, (ClientData) &drawPin);
        }
    }

    if ((drawGCell.gcellPtr = PbGetGCellOfCHW(chipWinPtr)) !=
        NIL(GenericCell)) {
        /*
         * High-light the indicated generic cell.
         */
        drawGCell.gcellColor = ez_HighLightColor;
        (void) PbHiLightGCell(winDataPtr, &drawGCell);
    }

    if (winOptions & PB_LABEL_CENTER_NET_OPT) {
        netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr);
        if ((netPtr != NIL(Net)) &&
            (netPtr->wirePtr != NIL(Pgr_Wire)) ) { 
            /*
             * Show the node numbers (of the edges in the power/ground
             * routing data structure) of the center net.
             */
            labelNet.netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr);
            labelNet.planeDataPtr = planeDataPtr;
            labelNet.whichNet = FPG_CENTER;
            (void) Gre_LabelNet(winDataPtr, (ClientData) &labelNet);
        }
    }

    if (winOptions & PB_LABEL_CELLS_OPT) {
        /*
         * Print the name of each cell instance on the screen.
         */
        (void) Mac_ItrChildrenOfCell(cellMasterPtr, LabelCellInst,
            (ClientData) &tiDrawData);
    }

    /*
     * Note that you cannot draw only part of a ridge (you must
     * always draw the entire ridge) and that you cannot display
     * both ridges at the same time.
     */
    drawRidge.displayFunc = pbUserIsPickingRidge ? GXxor : GXcopy;
    drawRidge.planeDataPtr = planeDataPtr;
    drawRidge.ridgeColor = ez_HighLightColor;
    if ((drawRidge.tileList = PbGetHorzTileListOfCHW(chipWinPtr)) !=
        NIL(LinkedTile)) {
        /*
         * High-light the indicated horizontal ridge.
         */
        (void) PbDrawHorzRidge(winDataPtr, (ClientData) &drawRidge);
    }
    else if ((drawRidge.tileList = PbGetVertTileListOfCHW(chipWinPtr)) !=
        NIL(LinkedTile)) {
        /*
         * High-light the indicated vertical ridge.
         */
        (void) PbDrawVertRidge(winDataPtr, (ClientData) &drawRidge);
    }

    Ez_Flush();

    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * PbHiLightGCell() --
 *
 * This procedure is designed to be used by an "interator".  The given generic
 * cell is filled in with a cargo net pattern using the given color.
 *
 * Results:
 *   BEAR_CONTINUE_SEARCH is returned upon completion.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
PbHiLightGCell(winDataPtr, drawGCellPtr)
WindowData  *winDataPtr;
DrawGCell   *drawGCellPtr;
{
    LinkedRect  *horzRectList, *rectPtr;
    Rect        drawRect;
    TiDrawData  tiDrawData;
    Transform   *transPtr;

    /* 
     * If this is not a chip window return BEAR_CONTINUE_SEARCH.
     */
    if (PbIsChipWin(winDataPtr) == FALSE) {
        return (BEAR_CONTINUE_SEARCH);
    }

    horzRectList = Mac_GetHorzRectListOfGCell(drawGCellPtr->gcellPtr);
    transPtr = Mac_GetTransformOfGCell(drawGCellPtr->gcellPtr);
    for (rectPtr = horzRectList;
        rectPtr != NIL(LinkedRect);
        rectPtr = rectPtr->r_next) {

        GeoTransRect(transPtr, &rectPtr->r_r, &drawRect);
        Ez_StippleRect(winDataPtr->wd_userWin, winDataPtr->wd_scrMatrix,
            &drawRect, drawGCellPtr->gcellColor, ez_CargoNetBitmap, GXcopy);
    }

    tiDrawData.chipWin = winDataPtr->wd_userWin;
    tiDrawData.scrMatrix = winDataPtr->wd_scrMatrix;
    tiDrawData.color = drawGCellPtr->gcellColor;
    switch (drawGCellPtr->gcellPtr->unType) {
    case CELL_INST:
        (void) OutlineCellInst(drawGCellPtr->gcellPtr->unValue.cellInstPtr, 
            &tiDrawData);
        break;
    case DUMMY_CELL:
        (void) OutlineDCell(drawGCellPtr->gcellPtr->unValue.dummyCellPtr,   
            &tiDrawData);
        break;
#ifdef USE_DR_CODE
    case ROUTE_CELL:
        (void) OutlineRCell(drawGCellPtr->gcellPtr->unValue.routeCellPtr,   
            &tiDrawData);
        break;
#endif /* USE_DR_CODE */
    default:
        Bear_PrintFatalErr("PbHiLightGCell() failed: unknown cell class");
        break;
    }

    return (BEAR_CONTINUE_SEARCH);
}

#endif /* BEAR_BLACK_BOX */


