
/* pbFpCmds.c --
 *
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "globalSpacer.h"
#include "ezPlot.h"
#include "grEditor.h"
#include "shapeOptimizer.h"
#include "pbDialogs.h"
#include "polarBear.h"


/* ARGSUSED */
BearStatus
PbShapeOptCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
    DrawGCell       drawGCell;
#endif /* BEAR_BLACK_BOX */

    BearString      flexFileName;
    CellMaster      *cellMasterPtr;
    GenericCell     *gCellPtr;
    ResizeData      resizeData;
    So_UserParms    parms;
    SoStatus        soStatus = SO_NOT_FINISHED;
    int             i;
    char            ch;
    bool            dialogWinFlag = TRUE;
    bool            haveAlgFlag = FALSE;

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    }
    else
#endif BEAR_BLACK_BOX
    {
        if (pbCellMasterPtr == NIL(CellMaster)) {
            PbPrint(EZ_ITALIC_FONT, "Must read in a cell master first.\n");
            return (BEAR_ERROR);
        }
        else {
            cellMasterPtr = pbCellMasterPtr;
        }
    }

    if (Mac_IsGlobalRoutingDone(cellMasterPtr) == FALSE) {
        PbPrint(EZ_ITALIC_FONT,
            "Cannot do shape optimization before globalRouting.\n");
        return (BEAR_ERROR);
    }

    parms.algorithm = TWO_D;
    parms.prefDirect = SO_NONE;
    parms.option = BEST_SLACK;
    parms.style = BBL;
    parms.gridSize = 8;
    parms.maxCnt = 12;
    parms.stopParm = 0.5;
    if ((flexFileName = Mac_GetFlexFileNameOfCellMaster(cellMasterPtr)) !=
        (BearString) NULL) {
        parms.flexFileName = flexFileName;
    }
    else {
        parms.flexFileName = NIL(char);
    }

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        /* 
         * Don't pop up any dialog windows.  Used when you want to run
         * this command from a script.
         */
        if (strcmp (argList[i], "-NDW") == 0) {
            dialogWinFlag = FALSE;
            continue;
        }

        if (strcmp (argList[i], "-alg") == 0) {
            if (++i >= argCount) {
                goto usage;
            }

            if (strcmp (argList[i], "1d") == 0) {
                parms.algorithm = ONE_D;
            }
            else if (strcmp (argList[i], "2d") == 0) {
                parms.algorithm = TWO_D;
            }
            else {
                PbPrint(EZ_ITALIC_FONT,
                    "Illegal choice of algorithum parameter.\n");
                return (BEAR_ERROR);
            }
            haveAlgFlag = TRUE;
            continue;
        }

        if (strcmp (argList[i], "-pd") == 0) {
            if (++i >= argCount) {
                goto usage;
            }

            switch (*argList[i]) {
            case 'H':
            case 'h': 
                parms.prefDirect = SO_HORZ;
                break;

            case 'V':
            case 'v':
                parms.prefDirect = SO_VERT;
                break;

            case 'N':
            case 'n': 
                parms.prefDirect = SO_NONE;
                break;

            default:
                PbPrint(EZ_ITALIC_FONT,
                    "Illegal choice of prefered direction parameter.\n");
                return (BEAR_ERROR);
            }
            continue;
        }

        (void) sprintf (bear_Msg, "unknown option %s\n", argList[i]);
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
        goto usage;
    }

    if (haveAlgFlag == FALSE) {
#ifndef BEAR_BLACK_BOX
        if ((bear_BlackBoxFlag == FALSE) && (dialogWinFlag == TRUE)) {
            if (PbShapeOptDialog(&parms) != BEAR_OK) {
                return (BEAR_ERROR);
            }
        }
        else 
#endif BEAR_BLACK_BOX
        {
            PbPrint(EZ_ITALIC_FONT, "Must specify an algorithum.\n");
            return (BEAR_ERROR);
        }
    }

    if (parms.prefDirect == SO_NONE) {
        parms.prefDirect = SO_HORZ;
    }

#ifdef DEBUG
    if (parms.algorithm == ONE_D) {
        (void) printf("\nAlgorithum:              ONE_D\n");
    }
    else if (parms.algorithm == TWO_D) {
        (void) printf("\nAlgorithum:              TWO_D\n");
    }
    else {
        (void) printf("\nAlgorithum:              %d\n", parms.algorithm);
    }

    (void) printf("Pref Direction:          %d\n", parms.prefDirect);
    (void) printf("Options:                 %d\n", parms.option);
    (void) printf("Design Style:            %d\n", parms.style);
    (void) printf("Minimum Slack Size:      %d\n", parms.gridSize);
    (void) printf("Maximum Iteration Count: %d\n", parms.maxCnt);
    /* (void) printf("Stop Parm:            %lf\n", parms.stopParm); */
    (void) printf("Flex file name:          %s\n\n", 
        (parms.flexFileName != NIL(char)) ?  
            parms.flexFileName : "No file given");

    fflush (stdout);
#endif DEBUG

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        drawGCell.gcellColor = ez_HighLightColor;
    }
#endif BEAR_BLACK_BOX
    
    So_Init();
    if (So_DefineShapes(cellMasterPtr, parms.flexFileName) != BEAR_OK) {
        return (BEAR_ERROR);
    }
    
    while (soStatus != SO_FINISHED) {
        resizeData.cellInstPtr = NIL(CellInst);
#ifdef USE_WIRE_CODE
        if (Gre_DoPgWiringForAllNets(cellMasterPtr) != BEAR_OK) {
            return (BEAR_ERROR);
        }
#endif USE_WIRE_CODE
        Gs_CalcCapDenOfTilePlane(cellMasterPtr);
        soStatus = So_ShapeOptimize(cellMasterPtr, &resizeData, &parms);
    
        if (resizeData.cellInstPtr != NIL(CellInst)) {
            Mac_CreateGCell(CELL_INST, 
                (ClientData) resizeData.cellInstPtr, &gCellPtr);
#ifndef BEAR_BLACK_BOX
            if (bear_BlackBoxFlag == FALSE) {
                if ((drawGCell.gcellPtr = gCellPtr) != NIL(GenericCell)) {
                    (void) PbHiLightGCell(winDataPtr, &drawGCell);
                    (void) PbItrChipWindows(PbHiLightGCell, 
                        (ClientData) &drawGCell);
                     Ez_Flush();
                }
                /* printf("continue  ?\n"); scanf("%s", &ch); */
            }
#endif BEAR_BLACK_BOX
        }
        
        So_ResizeCell(&resizeData, soStatus); 
#ifndef BEAR_BLACK_BOX
        if (bear_BlackBoxFlag == FALSE) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            Ez_Flush();
        }
#endif BEAR_BLACK_BOX
        /* printf("continue  ?\n"); scanf("%s", &ch); */
    }
    
    PbPrint(EZ_NORMAL_FONT, "Shape optimization done.\n");

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT,
        "shape optimization (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT,
        "Usage: so -alg < 1d || 2d > [-pd < h || v || n >] [-NDW]\n");

    PbPrint(EZ_NORMAL_FONT, "\noptions:\n");
    PbPrint(EZ_NORMAL_FONT, "    -alg algorithum\n");
    PbPrint(EZ_NORMAL_FONT, "    -pd prefered direction\n");

    PbPrint(EZ_NORMAL_FONT, "    -NDW no dialog windows\n");
    PbPrint(EZ_NORMAL_FONT, "\n");

    return (BEAR_ERROR);
}


