
/* pbGSCmds.c --
 *
 *     Wei-Ming Dai
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "grEditor.h"
#include "globalSpacer.h"
#include "tileMapping.h"
#include "ezPlot.h"
#include "polarBear.h"

#ifdef USE_XDM_CODE
#include "vemDM.h"
#endif /* USE_XDM_CODE */


extern WindowData *globalWinDataPtr;


/* ---------- Local Procedures ---------- */
static char *GetReply();


/* ---------- Local Global Variables ---------- */
#ifdef USE_XDM_CODE
static dmWhichItem optionsPtr[] = {{"Debug Yes", 0, 0},
                                   {"Debug No", 0, 0}};
#endif USE_XDM_CODE


static char *
GetReply(msg)
char *msg;
{
    static char reply[10];

    printf(msg);
    scanf("%s",reply);
    return (reply);
}



BearStatus
PbDoHorzCompact(winDataPtr, ridgeCountPtr,
    showSpacingFlag, printMsgFlag, debugFlag)
WindowData  *winDataPtr;
int         *ridgeCountPtr;
bool        showSpacingFlag, printMsgFlag, debugFlag;
/*
 * Currently if the showSpacingFlag is FALSE and you quit in the
 * middle the screen will not be automatically updated and
 * you'll have to type control-l.
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    CellMaster      *cellMasterPtr;
    PlaneData       *planeDataPtr;
    SpacingStatus   hCompactStatus;
    int             ridgeCount;
    char            *reply;

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        gs_DrawRidge = showSpacingFlag;

        globalWinDataPtr = winDataPtr;
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    } 
    else
#endif BEAR_BLACK_BOX
    {
        gs_DrawRidge = showSpacingFlag = FALSE;

        if (pbCellMasterPtr == NIL(CellMaster)) {
    Bear_PrintFatalErr("PbDoHorzCompact() failed : pbCellMasterPtr is NULL.\n");
        }
        else {
            cellMasterPtr = pbCellMasterPtr;
        }
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

    /*
     * Horizontal unify
     */
#ifdef USE_WIRE_CODE
    if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
        Bear_PrintErr("PbDoHorzCompact() failed : unable to wire nets.\n");
        return (BEAR_ERROR);
    }
#endif USE_WIRE_CODE
    reply = debugFlag ? GetReply("H-Unify? (y/n): ") : "y";
    if (strcmp (reply,"y") == 0) {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
            Tm_HUnify(cellMasterPtr);
        }
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
        if (showSpacingFlag && PbHorzTilesOn(winDataPtr)) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
#endif BEAR_BLACK_BOX
    }
    else {
        *ridgeCountPtr = 0;
        return (BEAR_OK);
    }

    gs_Quit = FALSE;
    hCompactStatus = GS_HC_NOT_FINISHED;
    ridgeCount = 0;
    while (hCompactStatus == GS_HC_NOT_FINISHED) {
        if (gs_Quit == TRUE) {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
        }

        /*
         * Horizontal compaction
         */
        reply = debugFlag ? GetReply("H-Compact? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0) {
            Gs_InitSpaceTile(planeDataPtr,cellMasterPtr);
            hCompactStatus = Gs_HorzCompact(planeDataPtr);
            ridgeCount++;
            Tp_IdentifySpaceTiles(planeDataPtr);
            (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                (WallJunction **) NULL);
            /* Only need to recalculate capacity of tiles */
        /* Don't recalculate density, because will lose info */
        /* stored by the channel router. */
            (void) Gs_CalcCapOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
            if (showSpacingFlag) {
                (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            }
#endif BEAR_BLACK_BOX
        }
        else {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
        }
    }
    
    /*
     * Horizontal split
     */
    reply = debugFlag ? GetReply("H-Split? (y/n): ") : "y";
    if (strcmp (reply,"y") == 0) {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
            Tm_HSplit(cellMasterPtr);
        }
#ifdef USE_WIRE_CODE
        if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
            Bear_PrintErr("PbDoHorzCompact() failed : unable to wire nets.\n");
            return (BEAR_ERROR);
        }
#endif USE_WIRE_CODE
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
        if (showSpacingFlag && PbBothTilesOn(winDataPtr)) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
#endif BEAR_BLACK_BOX
    }

    if (printMsgFlag) {
        PbPrint(EZ_NORMAL_FONT, "Horizontal compaction is done.\n");
    }

    *ridgeCountPtr = ridgeCount;

    return (BEAR_OK);
}



BearStatus
PbDoVertCompact(winDataPtr, ridgeCountPtr,
    showSpacingFlag, printMsgFlag, debugFlag)

WindowData  *winDataPtr;
int         *ridgeCountPtr;
bool        showSpacingFlag, printMsgFlag, debugFlag;
/*
 * Currently if the showSpacingFlag is FALSE and you quit in the
 * middle the screen will not be automatically updated and
 * you'll have to type control-l.
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    CellMaster      *cellMasterPtr;
    PlaneData       *planeDataPtr;
    SpacingStatus   vCompactStatus;
    int             ridgeCount;
    char            *reply;

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        gs_DrawRidge = showSpacingFlag;

        globalWinDataPtr = winDataPtr;
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    } 
    else
#endif BEAR_BLACK_BOX
    {
        gs_DrawRidge = showSpacingFlag = FALSE;

        if (pbCellMasterPtr == NIL(CellMaster)) {
    Bear_PrintFatalErr("PbDoVertCompact() failed : pbCellMasterPtr is NULL.\n");
        }
        else {
            cellMasterPtr = pbCellMasterPtr;
        }
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

    /*
     * Vertical unify
     */
#ifdef USE_WIRE_CODE
    if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
        Bear_PrintErr("PbDoVertCompact() failed : unable to wire nets.\n");
        return (BEAR_ERROR);
    }
#endif USE_WIRE_CODE
    reply = debugFlag ? GetReply("V-Unify? (y/n): ") : "y";
    if (strcmp (reply,"y") == 0)
    {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
            Tm_VUnify(cellMasterPtr);
        }
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
        if (showSpacingFlag && PbVertTilesOn(winDataPtr)) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
#endif BEAR_BLACK_BOX
    }
    else {
        *ridgeCountPtr = 0;
        return (BEAR_OK);
    }

   gs_Quit = FALSE;
   vCompactStatus = GS_VC_NOT_FINISHED;
   ridgeCount = 0;
   while (vCompactStatus == GS_VC_NOT_FINISHED) {
        if (gs_Quit == TRUE) {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
        }

       /* 
        * Vertical compaction
        */
       reply = debugFlag ? GetReply("V-Compact? (y/n): ") : "y";
       if (strcmp (reply,"y") == 0)
       {
            Gs_InitSpaceTile(planeDataPtr, cellMasterPtr);
            vCompactStatus = Gs_VertCompact(planeDataPtr);
            ridgeCount++;
            Tp_IdentifySpaceTiles(planeDataPtr);
        (void) Fpg_CreateFloorPlanGraph(planeDataPtr, 
                        (WallJunction **) NULL);
            /* Only need to recalculate capacity of tiles */
        /* Don't recalculate density, because will lose info */
        /* stored by the channel router. */
            (void) Gs_CalcCapOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
            if (showSpacingFlag) {
                (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            }
#endif BEAR_BLACK_BOX
       }    
       else {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
       }
   }
   reply = debugFlag ? GetReply("V-Split? (y/n): ") : "y";
   if (strcmp (reply,"y") == 0)
   {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
            Tm_VSplit(cellMasterPtr);
        }
#ifdef USE_WIRE_CODE
        if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
            Bear_PrintErr("PbDoVertCompact() failed : unable to wire nets.\n");
            return (BEAR_ERROR);
        }
#endif USE_WIRE_CODE
       (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
       if (showSpacingFlag && PbBothTilesOn(winDataPtr)) {
           (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
       }
#endif BEAR_BLACK_BOX
   }

    if (printMsgFlag) {
        PbPrint(EZ_NORMAL_FONT, "Vertical compaction is done.\n");
    }

    *ridgeCountPtr = ridgeCount;

    return (BEAR_OK);
}



BearStatus
PbDoHorzDecompact(winDataPtr, ridgeCountPtr,
showSpacingFlag, printMsgFlag, debugFlag)
WindowData  *winDataPtr;
int         *ridgeCountPtr;
bool        showSpacingFlag, printMsgFlag, debugFlag;
/*
 * Currently if the showSpacingFlag is FALSE and you quit in the
 * middle the screen will not be automatically updated and
 * you'll have to type control-l.
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    CellMaster      *cellMasterPtr;
    PlaneData       *planeDataPtr;
    SpacingStatus   hDecompactStatus;
    int             ridgeCount;
    char            *reply;

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        gs_DrawRidge = showSpacingFlag;

        globalWinDataPtr = winDataPtr;
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    } 
    else
#endif BEAR_BLACK_BOX
    {
        gs_DrawRidge = showSpacingFlag = FALSE;

        cellMasterPtr = pbCellMasterPtr;
        if (cellMasterPtr == NIL(CellMaster)) {
  Bear_PrintFatalErr("PbDoHorzDecompact() failed : pbCellMasterPtr is NULL.\n");
        }
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

    /*
     * Horizontal unify
     */
#ifdef USE_WIRE_CODE
    if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
        Bear_PrintErr("PbDoHorzDecompact() failed : unable to wire nets.\n");
        return (BEAR_ERROR);
    }
#endif USE_WIRE_CODE
    reply = debugFlag ? GetReply("H-Unify? (y/n): ") : "y";
    if (strcmp (reply,"y") == 0) {
       if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
          Tm_HUnify(cellMasterPtr);
       }
       (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
       if (showSpacingFlag && PbHorzTilesOn(winDataPtr)) {
           (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
       }
#endif BEAR_BLACK_BOX
    }
    else {
        *ridgeCountPtr = 0;
        return (BEAR_OK);
    }

    gs_Quit = FALSE;
    hDecompactStatus = GS_HDC_NOT_FINISHED;
    ridgeCount = 0;
    while (hDecompactStatus == GS_HDC_NOT_FINISHED) {
        if (gs_Quit == TRUE) {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
        }

        /*
         * Horizontal decompaction
         */
        reply = debugFlag ? GetReply("H-Decompact? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0)
        {
            Gs_InitSpaceTile(planeDataPtr,cellMasterPtr);
            hDecompactStatus = Gs_HorzDecompact(planeDataPtr);
            ridgeCount++;
            Tp_IdentifySpaceTiles(planeDataPtr);
            (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                (WallJunction **) NULL);
            /* Only need to recalculate capacity of tiles */
        /* Don't recalculate density, because will lose info */
        /* stored by the channel router. */
            (void) Gs_CalcCapOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
            if (showSpacingFlag) {
                PbViewEntireObject(winDataPtr,
                    Tp_GetHorzPlaneBnd(planeDataPtr), FALSE);
                (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            }
#endif BEAR_BLACK_BOX
        }
        else {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
        }
    }

    /*
     * Horizontal split
     */
    reply = debugFlag ? GetReply("H-Split? (y/n): ") : "y";
    if (strcmp (reply,"y") == 0)
    {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr))
        {
           Tm_HSplit(cellMasterPtr);
        }
#ifdef USE_WIRE_CODE
        if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
          Bear_PrintErr("PbDoHorzDecompact() failed : unable to wire nets.\n");
            return (BEAR_ERROR);
        }
#endif USE_WIRE_CODE
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
        if (showSpacingFlag && PbBothTilesOn(winDataPtr)) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
#endif BEAR_BLACK_BOX
    }

    if (printMsgFlag) {
        PbPrint(EZ_NORMAL_FONT, "Horizontal decompaction is done.\n");
    }

    *ridgeCountPtr = ridgeCount;

    return (BEAR_OK);
}



BearStatus
PbDoVertDecompact(winDataPtr, ridgeCountPtr,
    showSpacingFlag, printMsgFlag, debugFlag)

WindowData  *winDataPtr;
int         *ridgeCountPtr;
bool        showSpacingFlag, printMsgFlag, debugFlag;
/*
 * Currently if the showSpacingFlag is FALSE and you quit in the
 * middle the screen will not be automatically updated and
 * you'll have to type control-l.
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    CellMaster      *cellMasterPtr;
    PlaneData       *planeDataPtr;
    SpacingStatus   vDecompactStatus;
    int             ridgeCount;
    char            *reply;

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        gs_DrawRidge = showSpacingFlag;

        globalWinDataPtr = winDataPtr;
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    } 
    else
#endif BEAR_BLACK_BOX
    {
        gs_DrawRidge = showSpacingFlag = FALSE;

        cellMasterPtr = pbCellMasterPtr;
        if (cellMasterPtr == NIL(CellMaster)) {
  Bear_PrintFatalErr("PbDoVertDecompact() failed : pbCellMasterPtr is NULL.\n");
        }
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

    /*
     * Vertical unify
     */
#ifdef USE_WIRE_CODE
    if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
        Bear_PrintErr("PbDoVertDecompact() failed : unable to wire nets.\n");
        return (BEAR_ERROR);
    }
#endif USE_WIRE_CODE
    reply = debugFlag ? GetReply("V-Unify? (y/n): ") : "y";
    if (strcmp (reply,"y") == 0)
    {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr))
        {
           Tm_VUnify(cellMasterPtr);
        }
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
        if (showSpacingFlag && PbVertTilesOn(winDataPtr)) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
#endif BEAR_BLACK_BOX
    }
    else {
        *ridgeCountPtr = 0;
        return (BEAR_OK);
    }

    gs_Quit = FALSE;
    vDecompactStatus = GS_VDC_NOT_FINISHED;
    ridgeCount = 0;
    while (vDecompactStatus == GS_VDC_NOT_FINISHED)
    {
        if (gs_Quit == TRUE)
        {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
        }

        /*
         * Vertical decompaction
         */
        reply = debugFlag ? GetReply("V-Decompact? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0)
        {
            Gs_InitSpaceTile(planeDataPtr, cellMasterPtr);
            vDecompactStatus = Gs_VertDecompact(planeDataPtr);
            ridgeCount++;
            Tp_IdentifySpaceTiles(planeDataPtr);
            (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                (WallJunction **) NULL);
            /* Only need to recalculate capacity of tiles */
        /* Don't recalculate density, because will lose info */
        /* stored by the channel router. */
            (void) Gs_CalcCapOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
            if (showSpacingFlag) {
                PbViewEntireObject(winDataPtr,
                    Tp_GetHorzPlaneBnd(planeDataPtr), FALSE);
                (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            }
#endif BEAR_BLACK_BOX
        }
        else {
            *ridgeCountPtr = ridgeCount;
            return (BEAR_OK);
        }
    }

    /*
     * Vertical split
     */
    reply = debugFlag ? GetReply("V-Split? (y/n): ") : "y";
    if (strcmp (reply,"y") == 0) {
       if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
          Tm_VSplit(cellMasterPtr);
       }
#ifdef USE_WIRE_CODE
        if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
           Bear_PrintErr("PbDoVertDecompact() failed : unable to wire nets.\n");
            return (BEAR_ERROR);
        }
#endif USE_WIRE_CODE
       (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
#ifndef BEAR_BLACK_BOX
       if (showSpacingFlag && PbBothTilesOn(winDataPtr)) {
           (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
       }
#endif BEAR_BLACK_BOX
    }

    if (printMsgFlag) {
        PbPrint(EZ_NORMAL_FONT, "Vertical decompaction is done.\n");
    }

    *ridgeCountPtr = ridgeCount;

    return (BEAR_OK);
}



#ifndef BEAR_BLACK_BOX
/* ARGSUSED */
void
PbHorzCompactCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    int     i, selected, ridgeCount;
    bool    showSpacingFlag = TRUE;
    bool    printMsgFlag = TRUE;
    bool    debugFlag = FALSE;

    Gs_SetPause(FALSE); /* No prompting during compactions. */

    /* parse arguments */
    for (i = 1; i < argCount ;i++) 
    {
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) 
        {
            debugFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-i") == 0) ||
            (strcmp (argList[i], "i") == 0)) 
        {
            Gs_SetPause(TRUE);
            continue;
        }

#ifdef USE_XDM_CODE
        if ((strcmp (argList[i], "-o") == 0) ||
            (strcmp (argList[i], "o") == 0)) 
        {
            if (dmWhichOne("Options",
                sizeof (optionsPtr)/sizeof (dmWhichItem),
                optionsPtr, &selected, 0, "Choose one, dude!")) {
                debugFlag = (selected == 0);
            }
            else {
                return; /* Aborted dialog. */
            }
            continue;
        }
#endif USE_XDM_CODE

        goto usage;
    }
    (void) PbDoHorzCompact(winDataPtr, &ridgeCount,
        showSpacingFlag, printMsgFlag, debugFlag);
    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "Horizontal compaction\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: hcm [-id]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -i    interactive mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    debug mode\n");
}
#endif BEAR_BLACK_BOX



#ifndef BEAR_BLACK_BOX
/* ARGSUSED */
void
PbVertCompactCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    int     i, selected, ridgeCount;
    bool    showSpacingFlag = TRUE;
    bool    printMsgFlag = TRUE;
    bool    debugFlag = FALSE;

    Gs_SetPause(FALSE); /* No prompting during compactions. */

    /* parse arguments */
    for (i = 1; i < argCount ;i++) 
    {
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0))  {
            debugFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-i") == 0) ||
            (strcmp (argList[i], "i") == 0)) {
            Gs_SetPause(TRUE);
            continue;
        }

#ifdef USE_XDM_CODE
        if ((strcmp (argList[i], "-o") == 0) ||
            (strcmp (argList[i], "o") == 0)) {
            if (dmWhichOne("Options", sizeof (optionsPtr)/sizeof (dmWhichItem),
                optionsPtr, &selected, 0, "Choose one, dude!")) {
                debugFlag = (selected == 0);
            }
            else {
                return; /* Aborted dialog. */
            }
            continue;
        }
#endif USE_XDM_CODE

        goto usage;
    }
    (void) PbDoVertCompact(winDataPtr, &ridgeCount,
        showSpacingFlag, printMsgFlag, debugFlag);
    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "Vertical compaction\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: vcm [-id]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -i    interactive mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    debug mode\n");
}
#endif BEAR_BLACK_BOX



/* ARGSUSED */
BearStatus
PbCompactCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;     /* not used */
int         argCount;
char        **argList;
/*
 * BEAR_OK is returned only if the compaction is completed successfully.
 *
 * if showSpacingFlag then
 *     update the windows after each step
 * else
 *     update the windows once at the end
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    BearStatus      bearStatus;
    CellMaster      *cellMasterPtr;
    PlaneData       *planeDataPtr;
    SpacingStatus   hCompactStatus;
    SpacingStatus   vCompactStatus;
    int             i, selected;
    char            *reply;
    bool            debugFlag = FALSE;
    bool            showSpacingFlag = FALSE;
    bool            checkConnFlag = FALSE;
    bool            tileRedrawFlag = FALSE;
    bool            redrawFlag = FALSE;
    bool            warnFlag;
    
    Gs_SetPause(FALSE); /* No prompting during compactions. */

    /* parse arguments */
    for (i = 1; i < argCount ;i++) 
    {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-C") == 0) {
            checkConnFlag = TRUE;
            warnFlag = FALSE;
            continue;
        }
        if (strcmp (argList[i], "-CW") == 0) {
            checkConnFlag = TRUE;
            warnFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) {
            debugFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-i") == 0) ||
            (strcmp (argList[i], "i") == 0)) {

            if (bear_BlackBoxFlag == TRUE) {
                PbPrint(EZ_ITALIC_FONT,
                    "-i option not used in black box mode.\n");
                return (BEAR_ERROR);
            }
            Gs_SetPause(TRUE);
            showSpacingFlag = TRUE;
            continue;
        }

#ifdef USE_XDM_CODE
        if ((strcmp (argList[i], "-o") == 0) ||
            (strcmp (argList[i], "o") == 0)) {

            if (bear_BlackBoxFlag == TRUE) {
                PbPrint(EZ_ITALIC_FONT,
                    "-o option not used in black box mode.\n");
                return (BEAR_ERROR);
            }
            if (dmWhichOne("Options",
                sizeof (optionsPtr)/sizeof (dmWhichItem),
                optionsPtr, &selected, 0, "Choose one, dude!")) {
                debugFlag = (selected == 0);
            }
            else {
                return (BEAR_ERROR); /* Aborted dialog. */
            }
            continue;
        }
#endif USE_XDM_CODE

        goto usage;
    }

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        gs_DrawRidge = showSpacingFlag;

        globalWinDataPtr = winDataPtr;
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    }
    else
#endif BEAR_BLACK_BOX
    {
        gs_DrawRidge = showSpacingFlag = FALSE;

        cellMasterPtr = pbCellMasterPtr;
        if (cellMasterPtr == NIL(CellMaster)) {
            PbPrint(EZ_ITALIC_FONT, "Must read in a cell master first.\n");
            return (BEAR_ERROR);
        }
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

    gs_Quit = FALSE;
    hCompactStatus = GS_HC_NOT_FINISHED;
    vCompactStatus = GS_VC_NOT_FINISHED;
    while (hCompactStatus == GS_HC_NOT_FINISHED || 
        vCompactStatus == GS_VC_NOT_FINISHED) {
        if (hCompactStatus == GS_HC_NOT_FINISHED) {
            if (gs_Quit == TRUE) {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }

            reply = debugFlag ? GetReply("H-Unify? (y/n): ") : "y";
            if (strcmp (reply,"y") == 0) {
                if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
                   Tm_HUnify(cellMasterPtr);
                }
                (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
                if (bear_BlackBoxFlag == FALSE) {
                    tileRedrawFlag = PbHorzTilesOn(winDataPtr);
                    if (showSpacingFlag && tileRedrawFlag) {
                        (void) PbItrChipWindows(PbDrawChipRegion,
                            (ClientData) NULL);
                    }
                    else if (tileRedrawFlag) {
                        redrawFlag = TRUE;
                    }
                }
#endif BEAR_BLACK_BOX
            }
            else {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
    
            reply = debugFlag ? GetReply("H-Compact? (y/n): ") : "y";
            if (strcmp (reply, "y") == 0)
            {
                Gs_InitSpaceTile(planeDataPtr, cellMasterPtr);
                hCompactStatus = Gs_HorzCompact(planeDataPtr);
                Tp_IdentifySpaceTiles(planeDataPtr);
                (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                    (WallJunction **) NULL);
                /* Only need to recalculate capacity of tiles */
            /* Don't recalculate density, because will lose info */
            /* stored by the channel router. */
                (void) Gs_CalcCapOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
                if (bear_BlackBoxFlag == FALSE) {
                    if (showSpacingFlag) {
                        (void) PbItrChipWindows(PbDrawChipRegion,   
                            (ClientData) NULL);
                    }
                    else {
                        redrawFlag = TRUE;
                    }
                }
#endif BEAR_BLACK_BOX
            }
            else {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
    
            reply = debugFlag ? GetReply("H-Split? (y/n): ") : "y";
            if (strcmp (reply,"y") == 0) {
                if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
                   Tm_HSplit(cellMasterPtr);
                }
                if (checkConnFlag == TRUE) {
                    if (Gre_CheckConnForAllNets(cellMasterPtr, warnFlag) ==
                        BEAR_ERROR) {
                        PbPrint(EZ_NORMAL_FONT,
                            "Compaction aborted (disconnected nets).\n");
                        bearStatus = BEAR_ERROR;
                        goto redraw;
                    }
                }
#ifdef USE_WIRE_CODE
                if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
                   Bear_PrintErr("Compaction aborted (unable to wire nets).\n");
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }
#endif USE_WIRE_CODE
                (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
    
#ifndef BEAR_BLACK_BOX
                if (bear_BlackBoxFlag == FALSE) {
                    tileRedrawFlag = PbBothTilesOn(winDataPtr);
                    if (showSpacingFlag && tileRedrawFlag) {
                        (void) PbItrChipWindows(PbDrawChipRegion,
                            (ClientData) NULL);
                    }
                    else if (tileRedrawFlag) {
                        redrawFlag = TRUE;
                    }
                }
#endif BEAR_BLACK_BOX
                if (hCompactStatus == GS_QUIT) {
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }
            }
            else {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
            }
    
            if (vCompactStatus == GS_VC_NOT_FINISHED)
            {
                if (gs_Quit == TRUE) 
                {
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }

                reply = debugFlag ? GetReply("V-Unify? (y/n): ") : "y";
                if (strcmp (reply,"y") == 0)
                {
                    if (Mac_IsGlobalRoutingDone(cellMasterPtr))
                    {
                       Tm_VUnify(cellMasterPtr);
                    }
                    (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
                    if (bear_BlackBoxFlag == FALSE) {
                        tileRedrawFlag = PbVertTilesOn(winDataPtr);
                        if (showSpacingFlag && tileRedrawFlag) {
                            (void) PbItrChipWindows(PbDrawChipRegion,
                                (ClientData) NULL);
                        }
                        else if (tileRedrawFlag) {
                            redrawFlag = TRUE;
                        }
                    }
#endif BEAR_BLACK_BOX
                }
                else {
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }

                reply = debugFlag ? GetReply("V-Compact? (y/n): ") : "y";
                if (strcmp (reply,"y") == 0) {
                    Gs_InitSpaceTile(planeDataPtr, cellMasterPtr);
                    vCompactStatus = Gs_VertCompact(planeDataPtr);
                    Tp_IdentifySpaceTiles(planeDataPtr);
                    (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                        (WallJunction **) NULL);
                    /* Only need to recalculate capacity of tiles */
                /* Don't recalculate density, because will lose info */
                /* stored by the channel router. */
                    (void) Gs_CalcCapOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
                    if (bear_BlackBoxFlag == FALSE) {
                        if (showSpacingFlag) {
                            (void) PbItrChipWindows(PbDrawChipRegion,   
                                (ClientData) NULL);
                        }
                        else {
                            redrawFlag = TRUE;
                        }
                    }
#endif BEAR_BLACK_BOX
                }
                else {
                    PbPrint(EZ_NORMAL_FONT, "Compaction aborted.\n");
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }

            reply = debugFlag ? GetReply("V-Split? (y/n): ") : "y";
            if (strcmp (reply,"y") == 0) {
                if (Mac_IsGlobalRoutingDone(cellMasterPtr))
                {
                   Tm_VSplit(cellMasterPtr);
                }
                if (checkConnFlag) {
                    if (Gre_CheckConnForAllNets(cellMasterPtr, warnFlag) ==
                        BEAR_ERROR) {
                        PbPrint(EZ_NORMAL_FONT,
                            "Compaction aborted (disconnected nets).\n");
                        bearStatus = BEAR_ERROR;
                        goto redraw;
                    }
                }
#ifdef USE_WIRE_CODE
                if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
                   Bear_PrintErr("Compaction aborted (unable to wire nets).\n");
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }
#endif USE_WIRE_CODE
                (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
                if (bear_BlackBoxFlag == FALSE) {
                    tileRedrawFlag = PbBothTilesOn(winDataPtr);
                    if (showSpacingFlag && tileRedrawFlag) {
                        (void) PbItrChipWindows(PbDrawChipRegion,
                            (ClientData) NULL);
                    }
                    else if (tileRedrawFlag) {
                        redrawFlag = TRUE;
                    }
                }
#endif BEAR_BLACK_BOX
                if (vCompactStatus == GS_QUIT) 
                {
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }
            }
            else {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
        }
    }
    bearStatus = BEAR_OK;

redraw:
#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && (redrawFlag == TRUE)) {
        if (bearStatus == BEAR_OK) {
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_HORZ_PLANE_OPT);
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_VERT_PLANE_OPT);
        }
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    }
#endif BEAR_BLACK_BOX
    if (bearStatus == BEAR_OK) {
        PbPrint(EZ_NORMAL_FONT, "Compaction is done.\n");
    }

    return (bearStatus);

usage:
    PbPrint(EZ_NORMAL_FONT, "Compaction (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: cm [-C] [-CW] [-i] [-d] [-o]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -C   check connectivity after each step\n");
    PbPrint(EZ_NORMAL_FONT, "    -CW  check connectivity after each step\n");
    PbPrint(EZ_NORMAL_FONT, "         and print warnings\n");
    PbPrint(EZ_NORMAL_FONT, "    -d   debug mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -i   interactive mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -o   show option window\n");

    return (BEAR_ERROR);
}



#ifndef BEAR_BLACK_BOX
/* ARGSUSED */
void
PbHorzDecompactCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    int     i, selected, ridgeCount;
    bool    showSpacingFlag = TRUE;
    bool    printMsgFlag = TRUE;
    bool    debugFlag = FALSE;
    
    Gs_SetPause(FALSE); /* No prompting during compactions. */

    /* parse arguments */
    for (i = 1; i < argCount ;i++) 
    {
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) 
        {
            debugFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-i") == 0) ||
            (strcmp (argList[i], "i") == 0)) {
            Gs_SetPause(TRUE);
            continue;
        }

#ifdef USE_XDM_CODE
        if ((strcmp (argList[i], "-o") == 0) ||
            (strcmp (argList[i], "o") == 0)) {
            if (dmWhichOne("Options",
                sizeof (optionsPtr)/sizeof (dmWhichItem),
                optionsPtr, &selected, 0, "Choose one, dude!")) {
                debugFlag = (selected == 0);
            }
            else {
                return; /* Aborted dialog. */
            }
            continue;
        }
        else
#endif USE_XDM_CODE
        {
            goto usage;
        }
    }
    (void) PbDoHorzDecompact(winDataPtr, &ridgeCount,
        showSpacingFlag, printMsgFlag, debugFlag);
    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "Horizontal decompaction\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: hdcm [-id]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -i    interactive mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    debug mode\n");
}
#endif BEAR_BLACK_BOX



#ifndef BEAR_BLACK_BOX
/* ARGSUSED */
void
PbVertDecompactCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    int     i, selected, ridgeCount;
    bool    showSpacingFlag = TRUE;
    bool    printMsgFlag = TRUE;
    bool    debugFlag = FALSE;

    Gs_SetPause(FALSE); /* No prompting during compactions. */

    /* parse arguments */
    for (i = 1; i < argCount ;i++) 
    {
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) 
        {
            debugFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-i") == 0) ||
            (strcmp (argList[i], "i") == 0)) {
            Gs_SetPause(TRUE);
            continue;
        }

#ifdef USE_XDM_CODE
        if ((strcmp (argList[i], "-o") == 0) ||
            (strcmp (argList[i], "o") == 0)) {
            if (dmWhichOne("Options",
                sizeof (optionsPtr)/sizeof (dmWhichItem),
                optionsPtr, &selected, 0, "Choose one, dude!")) {
                debugFlag = (selected == 0);
            }
            else {
                return; /* Aborted dialog. */
            }
            continue;
        }
        else
#endif USE_XDM_CODE
        {
            goto usage;
        }
    }
    (void) PbDoVertDecompact(winDataPtr, &ridgeCount,
        showSpacingFlag, printMsgFlag, debugFlag);
    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "Vertical decompaction\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: vdcm [-id]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -i    interactive mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    debug mode\n");
}
#endif BEAR_BLACK_BOX



/* ARGSUSED */
BearStatus
PbDecompactCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;     /* not used */
int         argCount;
char        **argList;
/*
 * BEAR_OK is returned only if the decompaction is completed.
 *
 * if showSpacingFlag then
 *     update the windows after each step
 * else
 *     update the windows once at the end
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    BearStatus      bearStatus;
    CellMaster      *cellMasterPtr;
    PlaneData       *planeDataPtr;
    SpacingStatus   hDecompactStatus;
    SpacingStatus   vDecompactStatus;
    int             i, selected;
    char            *reply;
    bool            debugFlag = FALSE;
    bool            showSpacingFlag = FALSE;
    bool            checkConnFlag = FALSE;
    bool            redrawFlag = FALSE;
    bool            tileRedrawFlag = FALSE;
    bool            warnFlag;
    
    Gs_SetPause(FALSE); /* No prompting during compactions. */

    /* parse arguments */
    for (i = 1; i < argCount ;i++) 
    {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-C") == 0) {
            checkConnFlag = TRUE;
            warnFlag = FALSE;
            continue;
        }
        if (strcmp (argList[i], "-CW") == 0) {
            checkConnFlag = TRUE;
            warnFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) 
        {
            debugFlag = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-i") == 0) ||
            (strcmp (argList[i], "i") == 0)) {

            if (bear_BlackBoxFlag == TRUE) {
                PbPrint(EZ_ITALIC_FONT,
                    "-i option not used in black box mode.\n");
                return (BEAR_ERROR);
            }
            Gs_SetPause(TRUE);
            showSpacingFlag = TRUE;
            continue;
        }
#ifdef USE_XDM_CODE
        if ((strcmp (argList[i], "-o") == 0) ||
            (strcmp (argList[i], "o") == 0)) {

            if (bear_BlackBoxFlag == TRUE) {
                PbPrint(EZ_ITALIC_FONT,
                    "-o option not used in black box mode.\n");
                return (BEAR_ERROR);
            }
            if (dmWhichOne("Options", sizeof (optionsPtr)/sizeof (dmWhichItem),
                optionsPtr, &selected, 0, "Choose one, dude!")) {
                debugFlag = (selected == 0);
            }
            else {
                return (BEAR_ERROR); /* Aborted dialog. */
            }
            continue;
        }
        else
#endif USE_XDM_CODE
        {
            goto usage;
        }
    }

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        gs_DrawRidge = showSpacingFlag;

        globalWinDataPtr = winDataPtr;
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    }
    else
#endif BEAR_BLACK_BOX
    {
        gs_DrawRidge = showSpacingFlag = FALSE;

        cellMasterPtr = pbCellMasterPtr;
        if (cellMasterPtr == NIL(CellMaster)) {
            PbPrint(EZ_ITALIC_FONT, "Must read in a cell master first.\n");
            return (BEAR_ERROR);
        }
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

    gs_Quit = FALSE;
    hDecompactStatus = GS_HDC_NOT_FINISHED;
    vDecompactStatus = GS_VDC_NOT_FINISHED;
    while (hDecompactStatus == GS_HDC_NOT_FINISHED || 
        vDecompactStatus == GS_VDC_NOT_FINISHED)
    {
        if (hDecompactStatus == GS_HDC_NOT_FINISHED)
        {
            if (gs_Quit == TRUE)
            {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }

        reply = debugFlag ? GetReply("H-Unify? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0)
        {
            if (Mac_IsGlobalRoutingDone(cellMasterPtr))
            {
                Tm_HUnify(cellMasterPtr);
            }
            (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
            if (bear_BlackBoxFlag == FALSE) {
                tileRedrawFlag = PbHorzTilesOn(winDataPtr);
                if (showSpacingFlag && tileRedrawFlag) {
                    (void) PbItrChipWindows(PbDrawChipRegion, 
                        (ClientData) NULL);
                }
                else if (tileRedrawFlag) {
                    redrawFlag = TRUE;
                }
            }
#endif BEAR_BLACK_BOX
        }
        else {
            bearStatus = BEAR_ERROR;
            goto redraw;
        }

        reply = debugFlag ? GetReply("H-Decompact? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0)
        {
            Gs_InitSpaceTile(planeDataPtr,cellMasterPtr);
            hDecompactStatus = Gs_HorzDecompact(planeDataPtr);
            Tp_IdentifySpaceTiles(planeDataPtr);
            (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                (WallJunction **) NULL);
            (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
            if (bear_BlackBoxFlag == FALSE) {
                if (showSpacingFlag) {
                    (void) PbItrChipWindows(PbDrawChipRegion,
                        (ClientData) NULL);
                }
                else {
                    redrawFlag = TRUE;
                }
            }
#endif BEAR_BLACK_BOX
        }
        else {
            bearStatus = BEAR_ERROR;
            goto redraw;
        }

        reply = debugFlag ? GetReply("H-Split? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0) {
            if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
                Tm_HSplit(cellMasterPtr);
            }
    
            if (checkConnFlag) {
                if (Gre_CheckConnForAllNets(cellMasterPtr, warnFlag) ==
                    BEAR_ERROR) {
                    PbPrint(EZ_NORMAL_FONT,
                        "Decompaction aborted (disconnected nets).\n");
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }
            }
#ifdef USE_WIRE_CODE
            if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
                Bear_PrintErr("Decompaction aborted (unable to wire nets).\n");
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
#endif USE_WIRE_CODE
            (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
            if (bear_BlackBoxFlag == FALSE) {
                tileRedrawFlag = PbBothTilesOn(winDataPtr);
                if (showSpacingFlag && tileRedrawFlag) {
                    (void) PbItrChipWindows(PbDrawChipRegion,
                        (ClientData) NULL);
                } 
                else if (tileRedrawFlag) {
                    redrawFlag = TRUE;
                }
            }
#endif BEAR_BLACK_BOX

            if (hDecompactStatus == GS_QUIT) {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
        }
        else {
            bearStatus = BEAR_ERROR;
            goto redraw;
        }
        }

        if (vDecompactStatus == GS_VDC_NOT_FINISHED)
        {
        if (gs_Quit == TRUE)
        {
            bearStatus = BEAR_ERROR;
            goto redraw;
        }

        reply = debugFlag ? GetReply("V-Unify? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0)
        {
            if (Mac_IsGlobalRoutingDone(cellMasterPtr))
            {
              Tm_VUnify(cellMasterPtr);
            }
            (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
            if (bear_BlackBoxFlag == FALSE) {
                tileRedrawFlag = PbVertTilesOn(winDataPtr);
                if (showSpacingFlag && tileRedrawFlag) {
                    (void) PbItrChipWindows(PbDrawChipRegion,
                        (ClientData) NULL);
                }
                else if (tileRedrawFlag) {
                    redrawFlag = TRUE;
                }
            }
#endif BEAR_BLACK_BOX
         }
         else {
             bearStatus = BEAR_ERROR;
             goto redraw;
         }

        reply = debugFlag ? GetReply("V-Decompact? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0)
        {
        Gs_InitSpaceTile(planeDataPtr, cellMasterPtr);
        vDecompactStatus = Gs_VertDecompact(planeDataPtr);
        Tp_IdentifySpaceTiles(planeDataPtr);
        (void) Fpg_CreateFloorPlanGraph(planeDataPtr, (WallJunction **) NULL);
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
        if (bear_BlackBoxFlag == FALSE) {
            if (showSpacingFlag) {
                (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            }
            else {
                redrawFlag = TRUE;
            }
        }
#endif BEAR_BLACK_BOX
        }
        else {
            bearStatus = BEAR_ERROR;
            goto redraw;
        }

        reply = debugFlag ? GetReply("V-Split? (y/n): ") : "y";
        if (strcmp (reply,"y") == 0) {
            if (Mac_IsGlobalRoutingDone(cellMasterPtr)) {
                Tm_VSplit(cellMasterPtr);
            }
            if (checkConnFlag) {
                if (Gre_CheckConnForAllNets(cellMasterPtr, warnFlag) ==
                    BEAR_ERROR) {
                    PbPrint(EZ_NORMAL_FONT,
                        "Decompaction aborted (disconnected nets).\n");
                    bearStatus = BEAR_ERROR;
                    goto redraw;
                }
            }
#ifdef USE_WIRE_CODE
            if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
                Bear_PrintErr("Decompaction aborted (unable to wire nets).\n");
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
#endif USE_WIRE_CODE
            (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);

#ifndef BEAR_BLACK_BOX
            if (bear_BlackBoxFlag == FALSE) {
                tileRedrawFlag = PbBothTilesOn(winDataPtr);
                if (showSpacingFlag && tileRedrawFlag) {
                    (void) PbItrChipWindows(PbDrawChipRegion,
                        (ClientData) NULL);
                }
                else if (tileRedrawFlag) {
                    redrawFlag = TRUE;
                }
            }
#endif BEAR_BLACK_BOX

            if (vDecompactStatus == GS_QUIT) 
            {
                bearStatus = BEAR_ERROR;
                goto redraw;
            }
        }
        else {
            bearStatus = BEAR_ERROR;
            goto redraw;
        }
    }
    }
    bearStatus = BEAR_OK;

redraw:
#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && (redrawFlag == TRUE)) {
        if (bearStatus == BEAR_OK) {
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_HORZ_PLANE_OPT);
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_VERT_PLANE_OPT);
        }
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    }
#endif BEAR_BLACK_BOX
    if (bearStatus == BEAR_OK) {
        PbPrint(EZ_NORMAL_FONT, "Decompaction is done.\n");
    }

    return (bearStatus);

usage:
    PbPrint(EZ_NORMAL_FONT, "Decompaction (ok inscript) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: dcm [-C] [-CW] [-i] [-d] [-o]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -C  check connectivity after each step\n");
    PbPrint(EZ_NORMAL_FONT, "    -CW check connectivity after each step\n");
    PbPrint(EZ_NORMAL_FONT, "        and print warnings\n");
    PbPrint(EZ_NORMAL_FONT, "    -d  debug mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -i  interactive mode\n");
    PbPrint(EZ_NORMAL_FONT, "    -o  show option window\n");

    return (BEAR_ERROR);
}



