
/* pbGraphics.h --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Copyright (C) 1986 Regents of the University of California
 * All rights reserved.
 *
 * rcsid $Header$
 */

#ifndef _PB_GRAPHICS_H
#define _PB_GRAPHICS_H /* once-only latch */

/*
 * Markers to indicate window type.
 */
#define PB_NULL_WIN               0
#define PB_CONSOLE_WIN           01
#define PB_CHIP_WIN              02
#define PB_CIF_WIN               04
#define PB_PLACEMENT_WIN        010
#ifdef USE_OLDP_CODE
#define PB_FLOOR_WIN            020
#define PB_TREE_WIN             040
#define PB_ANY_WIN              077
#else
#define PB_ANY_WIN              017
#endif /* USE_OLDP_CODE */

#ifndef BEAR_BLACK_BOX

#define PB_DEF_ZOOM_MAG     1.25    /* default zoom magnification       */

typedef struct _CifWin CifWin;
struct _CifWin {
    Rect cw_cifBnd;
    char *cw_fileNamePtr;
};

typedef struct _SharedChipWinMem SharedChipWinMem;
struct _SharedChipWinMem {
    /*
     * This structure contains the attributes that all the windows
     * of a given chip share.
     */

    CellMaster  *cellMasterPtr;         /* Cell master that is being 
                                         * displayed.
                                         */
    bool        upDate;                 /* TRUE if cell master has been
                                         * updated and needs to be saved.
                                         */
    Net         *nwNetPtr;
    Net         *centerNetPtr;
    Net         *seNetPtr;
    BNLine      *bnLinePtr;             /* Bottleneck line that is being
                                         * high-lighted.
                                         */
    GenericCell *gcellPtr;              /* Generic cell that is being
                                         * high-lighted.
                                         */
    PinInst     *pinInstPtr;            /* Pin that is being high-lighted.  */

    /*
     * Note that you cannot draw only part of a ridge (you must
     * always draw the entire ridge) and that you cannot display
     * both ridges at the same time.
     */
    LinkedTile  *horzTileList;          /* List of tiles that form a 
                                         * horizontal ridge.
                                         */
    LinkedTile  *vertTileList;          /* List of tiles that form a 
                                         * vertical ridge.
                                         */
};

typedef struct _ChipWin ChipWin;
struct _ChipWin {
    /* 
     * Each chip window has its own options.
     */
    int              chw_options;

    /* 
     * Each chip window also has a number of attributes that
     * are shared.
     */
    SharedChipWinMem *chw_sharedMemPtr;
};

/* 
 * Markers to indicate chip window options.
 */
#define PB_NULL_OPT                 00
#define PB_SHOW_HORZ_PLANE_OPT      01  /* Outline space tiles on the
                                         * horizontal tile plane.
                                         */
#define PB_SHOW_VERT_PLANE_OPT      02  /* Outline space tiles on the vertical
                                         * tile plane.
                                         */
#define PB_SHOW_BN_TILES_OPT        04  /* fill in bottleneck tiles         */
#define PB_SHOW_DOM_TILES_OPT      010  /* fill in dominant tiles           */
#define PB_SHOW_FPG_OPT            020  /* draw the floor plan graph        */
#define PB_SHOW_PINS_OPT           040  /* draw all of the pins             */
#define PB_LABEL_CELLS_OPT        0100  /* print all of the cells' names    */
#define PB_LABEL_CENTER_NET_OPT   0200  /* label the center net             */


#ifdef USE_OLDP_CODE
typedef struct _FloorWin FloorWin;
struct _FloorWin {
    int         level;
    Node        *tree;
    int         depth;
};
#endif /* USE_OLDP_CODE */

#ifdef USE_OLDP_CODE
typedef struct _TreeWin TreeWin;
struct _TreeWin {
    int         level;
    Node        *tree;
    int         depth;
};
#endif /* USE_OLDP_CODE */

typedef struct _PbWindowType PbWindowType;
struct _PbWindowType {
    int unType;
    union {
        ChipWin     *chipWinPtr;        /* unType = PB_CHIP_WIN            */
        CifWin      *cifWinPtr;         /* unType = PB_CIF_WIN             */

#ifdef USE_OLDP_CODE
        FloorWin    *floorWinPtr;       /* unType = PB_FLOOR_WIN           */
        TreeWin     *treeWinPtr;        /* unType = PB_TREE_WIN            */
#endif /* USE_OLDP_CODE */

        BearGeneric placeWinPtr;        /* unType = PB_PLACEMENT_WIN       */
    } unValue;
};

typedef struct _DefCurData Pb_DefCurData;
struct _DefCurData {
    bool    (*idWinProc)();             /* Procedure that identifies the
                                         * correct window.
                                         */
    int     cursorType;
};

typedef struct _DrawBnLine DrawBnLine;
struct _DrawBnLine {
    BNLine  *bnLinePtr;
    Pixel   bnLineColor;
};

typedef struct _DrawGCell DrawGCell;
struct _DrawGCell {
    GenericCell *gcellPtr;
    Pixel       gcellColor;
};

typedef struct _DrawRidge DrawRidge;
struct _DrawRidge {
    LinkedTile  *tileList;
    PlaneData   *planeDataPtr;
    int         displayFunc;            /* Display function should be either
                                         * GXcopy or GXxor.
                                         */
    Pixel       ridgeColor;             /* ridge color                      */
};

/* ---------- Internal procedures for the polarBear program ---------- */

extern bool PbHorzTilesOn();
extern bool PbVertTilesOn();
extern bool PbBothTilesOn();

/*
 * window utilities
 */
extern void         PbCenterView();
extern void         PbViewEntireObject();
extern void         PbHandleUserWinButton();

extern ChipWin      *PbMakeChipWin();
extern CifWin       *PbMakeCifWin();
#ifdef USE_OLDP_CODE
extern FloorWin     *PbMakeFloorWin();
extern TreeWin      *PbMakeTreeWin();
#endif /* USE_OLDP_CODE */

extern bool         PbIsChipWin();
extern bool         PbIsCifWin();
extern bool         PbIsPlaceWin();
#ifdef USE_OLDP_CODE
extern bool         PbIsFloorWin();
extern bool         PbIsTreeWin();
#endif /* USE_OLDP_CODE */

extern BearStatus   PbCreateChipWinOne();
extern BearStatus   PbCreateCifWinOne();
#ifdef USE_OLDP_CODE
extern BearStatus   PbCreateFloorWinOne();
extern BearStatus   PbCreateTreeWinOne();
#endif /* USE_OLDP_CODE */

extern BearStatus   PbDupChipWin();
extern BearStatus   PbDupCifWin();

extern void         PbCloseChipWin();
extern void         PbCloseCifWin();
extern void         PbCloseConsoleWin();
#ifdef USE_OLDP_CODE
extern void         PbCloseFloorWin();
extern void         PbCloseTreeWin();
#endif /* USE_OLDP_CODE */

extern BearStatus PbNumOfChipWindows();

EXTERN BearStatus PbItrChipWindows
    ARGS((BearStatus (*userProc)(), ClientData clientData));

extern BearStatus PbDefCurInUserSubWin(/* winDataPtr, defCurDataPtr */);
/*
 * WindowData       *winDataPtr;
 * Pb_DefCurData    *defCurDataPtr;
 */

extern BearStatus   PbGetXDefaults();
extern void         PbSetUpTilesAndBitmaps();
extern bool         PbFindTile();

extern void         PbPrintBnLineProp();
extern void         PbPrintGCellProp();
extern void         PbPrintJunctProp();
extern void         PbPrintNetProp();
extern void         PbPrintPinProp();
extern void         PbPrintRoomProp();
extern void         PbPrintTileProp();

extern BearStatus PbGetSquare(/* winDataPtrPtr, screenPtr, squarePtr */);
/*
 * WinowData    **winDataPtrPtr;
 * Point        *screenPtr;
 * Rect         *squarePtr;
 */

extern void PbCalcSquare(/* winDataPtr, screenPtr, squarePtr */);
/*
 * WinowData    *winDataPtr;
 * Point        *screenPtr;
 * Rect         *squarePtr;
 */

extern BearStatus PbPickBnLine(/* bnLinePtr, drawBnLinePtr */);
/*
 * BNLine       *bnLinePtr;
 * DrawBnLine   *drawBnLinePtr;
 */

extern BearStatus PbDrawBnLineOne(/* bnLinePtr, drawDataPtr */);
/*
 * BNLine       *bnLinePtr;
 * DrawData     *drawDataPtr; 
 */

extern BearStatus PbDrawBnLineTwo(/* winDataPtr, drawBnLinePtr */);
/*
 * WindowData   *winDataPtr;
 * DrawBnLine   *drawBnLinePtr;
 */

extern BearStatus PbHiLightGCell(/* gcellPtr, drawGCellPtr */);
/*
 * WindowData   *winDataPtr;
 * DrawGCell    *drawGCellPtr;
 */

extern void PbHighlightChannel(/* cellMasterPtr, winDataPtr, channelPtr */);
/*
 * CellMaster   *cellMasterPtr;
 * WindowData   *winDataPtr;
 * Channel      *channelPtr;
 */

extern void PbHighlightAllChannels(/* winDataPtr */);
/*
 * WindowData *winDataPtr;
 */

extern BearStatus PbPickRidge(/* spaceTileListPtr, planeType */);
/*
 * LinkedTile   **spaceTileListPtr;
 * int          planeType;
 */

extern BearStatus PbDrawHorzRidge(/* winDataPtr, drawRidgePtr */);
/*
 * WindowData   *winDataPtr;
 * DrawRidge    *drawRidgePtr;
 */

extern BearStatus PbDrawVertRidge(/* winDataPtr, drawRidgePtr */);
/*
 * WindowData   *winDataPtr;
 * DrawRidge    *drawRidgePtr;
 */

extern BearStatus PbDrawChipRegion(/* winDataPtr, rect */);
/*
 * WindowData   *winDataPtr;
 * Rect         *rect;
 */

extern BearStatus PbDrawCifRegion(/* winDataPtr, rect */);
/*
 * WindowData   *winDataPtr;
 * Rect         *rect;
 */

extern BearStatus PbDrawFloorRegion(/* winDataPtr, rect */);
/*
 * WindowData   *winDataPtr;
 * Rect         *rect;
 */

extern BearStatus PbDrawTreeRegion(/* winDataPtr, rect */);
/*
 * WindowData   *winDataPtr;
 * Rect         *rect;
 */

extern void PbDrawFloorPlanGraph(/* winDataPtr */);
/*
 * WindowData   *winDataPtr;
 */

extern void PbDrawRoom(/* winDataPtr, junctPtr */);
/*
 * WindowData   *winDataPtr;
 * WallJunction *junctPtr;
 */

extern BearStatus PbGetOutlineVert();


/* ---------- Internal global variables for the polarBear program ---------- */

extern bool             pbUserIsPickingRidge;

extern Widget           pbTopWidget;
 
extern Pixmap           pbCellPixmap;
extern Pixmap           pbChipPixmap;

extern Pixel            pbCellColor;
extern Pixel            pbDCellColor;
extern Pixel            pbRCellColor;
extern Pixel            pbCellBorderColor;  /* The cell border color is also
                                             * used as the floor plan graph 
                                             * color.
                                             */
extern Pixel            pbPinColor;
extern Pixel            pbHorzBnColor;
extern Pixel            pbHorzDomColor;
extern Pixel            pbNet1Color;
extern Pixel            pbNet2Color;
extern Pixel            pbNet3Color;
extern Pixel            pbTreeColor;
extern Pixel            pbChipColor;
extern Pixel            pbVertBnColor;
extern Pixel            pbVertDomColor;

extern Pixel            pbHChannelColor;
extern Pixel            pbVChannelColor;
extern Pixel            pbLChannelColor;

#ifdef USE_OLDP_CODE
extern Pixel            pbTFLeafColor[5];
extern Pixel            pbTFNodeColor;
extern Pixel            pbTFEdgeColor;
#endif /* USE_OLDP_CODE */

/*
extern Pixel            color[5], colorNode, colorEdge;
*/

/* ---------- cif window macros and procedures ---------- */

#define PbGetCifWin(winDataPtr) \
    (((PbWindowType *) (winDataPtr)->wd_clientData)->unValue.cifWinPtr)

#define PbPutFileNameOfCW(X, cifWinPtr) \
    ((cifWinPtr)->cw_fileNamePtr = (X))

#define PbGetFileNameOfCW(cifWinPtr) \
    ((cifWinPtr)->cw_fileNamePtr)

extern void PbPutCifBndOfCW(/* cifBndPtr, cifWinPtr */);
/*
 * Rect     *cifBndPtr;
 * CifWin   *cifWinPtr;
 */

extern Rect *PbGetCifBndOfCW(/* cifWinPtr */);
/*
 * CifWin   *cifWinPtr;
 */


/* ---------- chip window macros and procedures ---------- */

#define PbGetChipWin(winDataPtr) \
    (((PbWindowType *) (winDataPtr)->wd_clientData)->unValue.chipWinPtr)

#define PbPutOptionsOfCHW(X, chipWinPtr) \
    ((chipWinPtr)->chw_options = (X))

#define PbGetOptionsOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_options)

#define PbPutCellMasterOfCHW(X, chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->cellMasterPtr = (X))

#define PbGetCellMasterOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->cellMasterPtr)

#define PbPutUpdateStatusOfCHW(X, chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->upDate = (bool) (X))

#define PbGetUpdateStatusOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->upDate)

#define PbPutBnLineOfCHW(X, chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->bnLinePtr = (X))

#define PbGetBnLineOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->bnLinePtr)

extern void PbPutGCellOfCHW(/* gcellPtr, chipWinPtr */);
/*
 * GenericCell  *gcellPtr;
 * ChipWin      *chipWinPtr;
 */

#define PbGetGCellOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->gcellPtr)

extern void PbPutNetOfCHW(/* whichNet, netPtr, chipWinPtr */);
/*
 * int          whichNet;    (* FPG_NW | FPG_CENTER | FPG_SE *)
 * Net          *netPtr;
 * ChipWin      *chipWinPtr;
 */

extern Net *PbGetNetOfCHW(/* whichNet, chipWinPtr */);
/*
 * int          whichNet;    (* FPG_NW | FPG_CENTER | FPG_SE *)
 * ChipWin      *chipWinPtr;
 */

#define PbPutPinOfCHW(X, chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->pinInstPtr = (X))

#define PbGetPinOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->pinInstPtr)

#define PbPutHorzTileListOfCHW(X, chipWinPtr) \
{ \
    (chipWinPtr)->chw_sharedMemPtr->horzTileList = (X); \
    (chipWinPtr)->chw_sharedMemPtr->vertTileList = NIL(LinkedTile); \
}

#define PbGetHorzTileListOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->horzTileList)

#define PbPutVertTileListOfCHW(X, chipWinPtr) \
{ \
    (chipWinPtr)->chw_sharedMemPtr->horzTileList = NIL(LinkedTile); \
    (chipWinPtr)->chw_sharedMemPtr->vertTileList = (X); \
}

#define PbGetVertTileListOfCHW(chipWinPtr) \
    ((chipWinPtr)->chw_sharedMemPtr->vertTileList)

EXTERN Pixel PbGetColorOfNet
    ARGS((int whichNet /* FPG_NW | FPG_CENTER | FPG_SE */ ));

/* ---------- floor window macros and procedures ---------- */
#define PbGetTreeWin(winDataPtr) \
    (((PbWindowType *) (winDataPtr)->wd_clientData)->unValue.treeWinPtr)

/* ---------- tree window macros and procedures ---------- */
#define PbGetFloorWin(winDataPtr) \
    (((PbWindowType *) (winDataPtr)->wd_clientData)->unValue.floorWinPtr)

/* ---------- placement window macros and procedures ---------- */
#define PbGetPlaceWin(winDataPtr) \
    (((PbWindowType *) (winDataPtr)->wd_clientData)->unValue.placeWinPtr)


extern void PbShowConn(/* winDataPtr, netListPtr */);
/*
 * WindowData *winDataPtr;
 * Net *netListPtr;
 */


/* ---------- other macros and procedures ---------- */

#ifdef USE_XDM_CODE
extern int PbCallBack();
#endif /* USE_XDM_CODE */

#endif /* BEAR_BLACK_BOX */

#endif /* _PB_GRAPHICS_H */

