
/* pbLRCellCmds.c --
 *
 *     Wei-Ming Dai
 *     M. Kobayashi
 *     (Advising Professor - E. S. Kuh)
 *
 * Commands that let you choose the routing cells.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifdef USE_DR_CODE

#include "bearCopyRight.h"
#include <stdio.h>
#include <math.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "geoChDecomposer.h"
#include "topChDecomposer.h"
#include "globalSpacer.h"
#include "grEditor.h"
#include "polarBear.h"


#define BAD_RETURN \
{ \
    Ez_RingBell(); \
    return; \
}


/* ARGSUSED */
void
PbDefLChannelCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    WallJunction    *junctPtr; 
    WallJunction    *junct1Ptr; 
    WallJunction    *junct2Ptr; 
    WallJunction    *junct3Ptr; 
    WallJunction    *leftJunctPtr; 
    WallJunction    *botJunctPtr; 
    WallJunction    *rightJunctPtr; 
    WallJunction    *topJunctPtr; 
    Tile            *horzDomTile1Ptr;
    Tile            *vertDomTile1Ptr;
    Tile            *horzDomTile2Ptr;
    Tile            *vertDomTile2Ptr;
    Tile            *horzDomTile3Ptr;
    Tile            *vertDomTile3Ptr;
    Point           *point1Ptr; 
    Point           *point2Ptr; 
    Point           *point3Ptr;
    int             i;
    bool            debug=FALSE;
    BearStatus      bearStatus;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) {
            debug = TRUE;
            continue;
        } else {
            goto usage;
        }
    }
    
    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (!Mac_IsGlobalRoutingDone(cellMasterPtr)) {
        PbPrint(EZ_ITALIC_FONT, "Complete global routing and try again.\n");
        return;
    }
    PbHighlightAllChannels(winDataPtr);
    PbPrint(EZ_NORMAL_FONT, "Pick horizontal end junction.\n");
    if (Gre_PickJunct(&horzDomTile1Ptr, &vertDomTile1Ptr, &junct1Ptr) ==
        BEAR_ERROR) 
    {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
    }
    point1Ptr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct1Ptr);

    PbPrint(EZ_NORMAL_FONT, "Pick corner junction.\n");
    if (Gre_PickJunct(&horzDomTile2Ptr, &vertDomTile2Ptr, &junct2Ptr) ==
        BEAR_ERROR) 
    {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
    }
    point2Ptr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct2Ptr);

    if (point1Ptr->p_y == point2Ptr->p_y) 
    {
        /* The channel is horizontal. */
        if (horzDomTile1Ptr != horzDomTile2Ptr) 
        {
            PbPrint(EZ_ITALIC_FONT,
                "There is a cell crossing the horizontal channel.\n");
            BAD_RETURN;
        }
    } 
    else 
    {
        PbPrint(EZ_ITALIC_FONT, "Pick the horizontal wall first.\n");
        BAD_RETURN;
    }

    PbPrint(EZ_NORMAL_FONT, "Pick vertical end junction.\n");
    if (Gre_PickJunct(&horzDomTile3Ptr, &vertDomTile3Ptr, &junct3Ptr) ==
        BEAR_ERROR) 
    {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
    }
    point3Ptr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct3Ptr);

    if (point2Ptr->p_x == point3Ptr->p_x) 
    {
        /* The channel is vertical. */
        if (vertDomTile3Ptr != vertDomTile2Ptr) 
        {
            PbPrint(EZ_ITALIC_FONT,
                "There is a cell crossing the vertical channel.\n");
            BAD_RETURN;
        }
    } 
    else 
    {
        PbPrint(EZ_ITALIC_FONT,
         "It is not a valid L-shaped channel definition.\n");
        BAD_RETURN;
    }

    /* Make sure that the L-channel is not adjacent to any empty cells */
    if (Fpg_GetDegreeOfWallJunct(junct2Ptr) > 2)
    {
        PbPrint(EZ_ITALIC_FONT,
            "Can't define L-shaped channel near empty cell.\n");
        BAD_RETURN;
    }

   if (junct2Ptr->centerPoint.p_y < junct3Ptr->centerPoint.p_y)
   {
      topJunctPtr = junct3Ptr;
      botJunctPtr = junct2Ptr;
   }
   else
   {
      topJunctPtr = junct2Ptr;
      botJunctPtr = junct3Ptr;
   }
   for (junctPtr  = Fpg_GetSouthJunctOfWallJunct(topJunctPtr);
        junctPtr != botJunctPtr;
        junctPtr  = Fpg_GetSouthJunctOfWallJunct(junctPtr))
   {
       if (Fpg_GetDegreeOfWallJunct(junctPtr) > 2)
       {
       PbPrint(EZ_ITALIC_FONT,
            "Can't define L-shaped channel near empty cell.\n");
       BAD_RETURN;
       }
   }

   if (junct2Ptr->centerPoint.p_x < junct1Ptr->centerPoint.p_x)
   {
       leftJunctPtr = junct2Ptr;
       rightJunctPtr = junct1Ptr;
   }
   else
   {
       leftJunctPtr = junct1Ptr;
       rightJunctPtr = junct2Ptr;
   }
   for (junctPtr  = Fpg_GetEastJunctOfWallJunct(leftJunctPtr);
        junctPtr != rightJunctPtr;
        junctPtr  = Fpg_GetEastJunctOfWallJunct(junctPtr))
   {
       if (Fpg_GetDegreeOfWallJunct(junctPtr) > 2)
       {
       PbPrint(EZ_ITALIC_FONT,
            "Can't define L-shaped channel near empty cell.\n");
       BAD_RETURN;
       }
   }

    /* Route CHANNEL */
    bearStatus = PbRouteLChannel(cellMasterPtr,
        junct1Ptr, junct2Ptr, junct3Ptr, TRUE, FALSE, debug);

    if (bearStatus == BEAR_OK)
    {
        PbPrint(EZ_NORMAL_FONT, "L-Channel successfully routed.\n");
    }
    else
    {
        PbPrint(EZ_NORMAL_FONT, "L-Channel route aborted.\n");
    }

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "define l shaped channel \n");
    PbPrint(EZ_NORMAL_FONT, "Usage: dlch [-d]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    debug mode\n");
}

#endif /* USE_DR_CODE */


