
/* pbMain.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  /* lint */

#include "bearCopyRight.h"
#include "port.h"
#include "utility.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "grEditor.h"
#include "tileMapping.h"
#include "polarBear.h"

#ifdef USE_OCT_CODE /* { */
#include "options.h"
#include "vov.h"
#define BEAR_EXIT(_X) VOVend(_X);
#else /* } else { USE_OCT_CODE */
#define BEAR_EXIT(_X) exit(_X);
#endif /* } USE_OCT_CODE */

/* ---------- Internal global variables for the polarBear program ---------- */

FILE         *pbLogFilePtr;
HashTable    pbCmdHashTable;
MailData     pbMailData;

char pbConsPrompt[80];
char pbHelpString[PB_HELP_STRING_LEN];
char pbHostName[PB_HOST_NAME_LEN];  /* Name of the host computer            */
bool pbBusyFlag;
bool pbCmdLoop;
bool pbDebugFlag;
bool pbScriptFlag;                  /* TRUE if PbSourceCmd() is in
                                     * progress.
                                     */
bool pbUserIsPickingRidge;

/*
 * Contains the pointer to the cell master when you run polarBear
 * in the black box mode.  Note that in the black box mode you are
 * limited to a single cell master.
 */
CellMaster *pbCellMasterPtr;

ViewType viewTypeFlag = NONE_DONE;


#ifndef BEAR_BLACK_BOX /* { */
Widget          pbTopWidget;

Pixmap          pbCellPixmap;       /* Pixmap for drawing cells.            */
Pixmap          pbChipPixmap;       /* Pixmap for drawing the chip.         */

Pixel           pbCellColor;        /* cell color                           */
Pixel           pbCellBorderColor;  /* cell border color                    */
Pixel           pbDCellColor;       /* dummy cell color                     */
Pixel           pbRCellColor;       /* route cell color                     */
Pixel           pbPinColor;         /* pin color                            */
Pixel           pbHorzBnColor;      /* horizontal bottleneck tile color     */
Pixel           pbHorzDomColor;     /* horizontal dominant tile color       */
Pixel           pbNet1Color;        /* net 1 color                          */
Pixel           pbNet2Color;        /* net 2 color                          */
Pixel           pbNet3Color;        /* net 3 color                          */
Pixel           pbChipColor;        /* chip color                           */
Pixel           pbVertDomColor;     /* vertical dominant tile color         */
Pixel           pbVertBnColor;      /* vertical bottleneck tile color       */
Pixel           pbHChannelColor;    /* horizontal channel color             */
Pixel           pbVChannelColor;    /* vertical channel color               */
Pixel           pbLChannelColor;    /* l-shaped channel color               */

#ifdef USE_OLDP_CODE /* { */
Pixel           pbTFLeafColor[5];
Pixel           pbTFNodeColor;
Pixel           pbTFEdgeColor;
#endif /* } USE_OLDP_CODE */

#endif /* } BEAR_BLACK_BOX */

#ifdef USE_OCT_CODE /* { */
/*
 * Below is the argument specifications for polarBear (argument parsing
 * is done by the options package).
 */

optionStruct optionList[] = {
    { "v",      "",             "print out version information"             },
    { "b",      "scriptFile",   "batch mode"                                },

#ifndef BEAR_BLACK_BOX /* { */
    { OPT_DESC, 0, "\n"                                                     },
    { OPT_CONT, 0,
  "X server to contact is indicated with the environment variable $DISPLAY" },
#endif /* } BEAR_BLACK_BOX */

    { 0,        0,              0                                           }
};
#endif /* } USE_OCT_CODE */


/* ---------- Local Procedures ---------- */
static BearStatus   Begin();
static BearStatus   End();
static void         PrintProgUsage();
static void         PrintProgVersion();


main (argc, argv)
int     argc;
char    **argv;
/*
 * Results:
 *    A zero exit status means everything went brilliantly.
 *    A non-zero exit status means the program failed.
 */
{
    char    *scriptFileNamePtr = NIL(char);

    if (Bear_Begin() != BEAR_OK) {
        BEAR_EXIT(1);
    }

#ifdef USE_OCT_CODE /* { */
{
    int option;

    while ((option = optGetOption(argc, argv)) != EOF) {
        switch (option) {
        case 'v':
            PrintProgVersion();
            BEAR_EXIT(0);
            break;
        case 'b':
            scriptFileNamePtr = optarg;
            bear_BlackBoxFlag = TRUE;
            break;
        default:
            PrintProgUsage(argv[0]);
            break;
        }
    }

    if (optind < argc) {
        PrintProgUsage(argv[0]);
    }
}
#else /* } else { USE_OCT_CODE */
{
    int i;

    for (i = 1; i < argc ;i++) {
        if (strcmp (argv[i], "-v") == 0) {  
            PrintProgVersion();
            BEAR_EXIT(0);
        }

        if (strcmp (argv[i], "-b") == 0) {     
            if (++i >= argc) {
                (void) fprintf (bearerr,
                    "-b option requires an argument\n");
                PrintProgUsage(argv[0]);
            }
            scriptFileNamePtr = argv[i];
            bear_BlackBoxFlag = TRUE;
            continue;
        }

        PrintProgUsage(argv[0]);
    }
}
#endif /* } USE_OCT_CODE */

#ifdef BEAR_BLACK_BOX
    if (scriptFileNamePtr == NIL(char)) {
        Bear_PrintErr("Must provide a script.\n");
        PrintProgUsage(argv[0]);
    }
#endif /* BEAR_BLACK_BOX */

    if (Begin(argc, argv) != BEAR_OK) {
        BEAR_EXIT(1);
    }

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        if (PbRunCommandLoop() != BEAR_OK) {
            BEAR_EXIT(2);
        }
    }
    else
#endif /* BEAR_BLACK_BOX */
    {
        if (PbRunBlackBox(scriptFileNamePtr) != BEAR_OK) {
            BEAR_EXIT(2);
        }
    }

    if (End() != BEAR_OK) {
        BEAR_EXIT(1);
    }

    BEAR_EXIT(0);
}



static BearStatus
Begin(argc, argv)
int     argc;
char    **argv;
{
    int     i;
    char    *dotPtr;
    
#ifdef USE_OCT_CODE
    (void) VOVbegin(argc, argv);
#endif /* USE_OCT_CODE */

    pbBusyFlag = TRUE;
    pbDebugFlag = FALSE;
    pbScriptFlag = FALSE;
    
    pbLogFilePtr = NIL(FILE);

    pbMailData.userPtr = NIL(char);
    pbMailData.subjectPtr = NIL(char);
    pbMailData.fileNamePtr = NIL(char);

#ifndef BEAR_BLACK_BOX /* { */
    if (bear_BlackBoxFlag == FALSE) {
        pbUserIsPickingRidge = FALSE;
        if (Ez_Begin(BEAR_PROG_NAME, &pbTopWidget, argc, argv) != EZ_OK) {
            return (BEAR_ERROR);
        }
        if (PbGetXDefaults() != BEAR_OK) {
            return (BEAR_ERROR);
        }
        PbSetUpTilesAndBitmaps();
        Gre_UsePgWireDataToDrawNet(TRUE);
        Gre_ChooseWallJunct(FPG_CENTER);
#ifdef USE_XDM_CODE /* { */
        dmCompatInit(PbCallBack);
#endif /* } USE_XDM_CODE */
    }
    else
#endif /* } BEAR_BLACK_BOX */
    {
        pbCellMasterPtr = NIL(CellMaster);
    }

    if (gethostname(pbHostName, PB_HOST_NAME_LEN) == -1) {
        (void) fprintf (bearerr, "%s: gethostname() failed\n", BEAR_PROG_NAME);
        perror (BEAR_PROG_NAME);
        return (BEAR_ERROR);
    }
    if ((dotPtr = strchr (pbHostName, '.')) != NIL(char)) {
        *dotPtr = '\0';
    }

    (void) sprintf (pbConsPrompt, "%s > ", BEAR_PROG_NAME);

    HashInit(&pbCmdHashTable, 10, 0);

    /*
     * Sort the commands into alphabetical order by menu name.
     */
    qsort ((char *) pbCommand, pbNumOfCmds, sizeof (Command),
        PbCompareCommands);

    for (i = 0; i < pbNumOfCmds ;i++) {
        /*
         * Skip over commands that may only be used in the black box mode.
         */
        if ((bear_BlackBoxFlag == FALSE) &&
            (pbCommand[i].co_validWins == PB_NULL_WIN)) {
            continue;
        }

        pbCommand[i].co_usageCount = 0;

        if (PbStoreCommand(&pbCmdHashTable, &pbCommand[i]) != BEAR_OK) {
            return (BEAR_ERROR);
        }
    }

    errProgramName(BEAR_PROG_NAME);
#ifdef DEBUG
    /*
     * Have the default error handler produce a core dump.
     */
    errCore(TRUE);
#endif /* DEBUG */

    return (BEAR_OK);
}



static BearStatus
End()
{
#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        Ez_XEnd();
    }
#endif /* BEAR_BLACK_BOX */

    return (BEAR_OK);
}


static void
PrintProgVersion()
{
    bool bbValue, drValue, octValue;

    (void) fprintf (bearout, "%s\n", PbGetVersion());

#ifdef BEAR_BLACK_BOX
    bbValue = TRUE;
#else
    bbValue = FALSE;
#endif /* BEAR_BLACK_BOX */
    (void) fprintf (bearout, "Has X11 R4 graphics interface : %s\n",
        Bear_GetBooleanString( ! bbValue ));

#ifdef USE_OCT_CODE
    octValue = TRUE;
#else
    octValue = FALSE;
#endif /* USE_OCT_CODE */
    (void) fprintf (bearout, "Has Octtools 5.1 interface    : %s\n",
        Bear_GetBooleanString(octValue));

#ifdef USE_DR_CODE
    drValue = TRUE;
#else
    drValue = FALSE;
#endif /* USE_DR_CODE */
    (void) fprintf (bearout, "Has detailed routing package  : %s\n",
        Bear_GetBooleanString(drValue));

    (void) fprintf (bearout, "\n");
    (void) fflush (bearout);
}


static void
PrintProgUsage(progNamePtr)
char *progNamePtr;
/*
 * Print a list of this programs command line options.
 */
{
    PrintProgVersion();

#ifdef USE_OCT_CODE /* { */
    (void) fprintf (bearerr, "%s", optUsageString());
#else /* } else { USE_OCT_CODE */
    (void) fprintf (bearerr, "Usage: %s [-options ...]\n\n", progNamePtr);

    (void) fprintf (bearerr, "where options include:\n");
    (void) fprintf (bearerr,
        "    -v                           print out version information\n");

    (void) fprintf (bearerr,
        "    -b scriptFile                batch mode\n");

#ifndef BEAR_BLACK_BOX /* { */
    (void) fprintf (bearerr, "\n");
    (void) fprintf (bearerr,
  "X server to contact is indicated with the environment variable $DISPLAY\n");
#endif /* } BEAR_BLACK_BOX */
#endif /* } USE_OCT_CODE */

    BEAR_EXIT(1);
}



/*
 * Major hack to able to use the Dave Harrison's dm package.
 * the package needs to know what the top level widget is.
 */
#ifdef USE_XDM_CODE
Widget
xv_topw()
{
    return pbTopWidget;
}
#endif /* USE_XDM_CODE */


/*ARGSUSED*/
#ifdef USE_XDM_CODE
int
PbCallBack(eventPtr)
XEvent *eventPtr;
{
    return 0;
}
#endif /* USE_XDM_CODE */


