
/* pbMiscCmds.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * Miscellaneous commands
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"

#ifdef USE_OCT_CODE
#include "octInterface.h"
#include "vov.h"
#endif USE_OCT_CODE

#include "ezPlot.h"
#include "grEditor.h"
#include "bearGrUtils.h"
#include "deque.h"
#include "tileMapping.h"
#include "globalSpacer.h"
#include "pbDialogs.h"
#include "polarBear.h"


/* ---------- Local Procedures ---------- */
static BearStatus   MakeHelpString();
static char         *GetReply();


static char *
GetReply(msg)
char   *msg;
{
    static char reply[10];

    printf(msg);
    scanf("%s",reply);
    return (reply);
}



static BearStatus
MakeHelpString(numOfCmds, command)
int     numOfCmds;
Command command[];  
{
    static  bool done = FALSE;
    static  bool success = FALSE;
    int     i, totalLength;
    char    string[80], *srPtr, *validWinsPtr;

    if (done) {
        return (success ? BEAR_OK : BEAR_ERROR);
    } else {
        done = TRUE;
    }

    bzero(pbHelpString, sizeof (*pbHelpString) * PB_HELP_STRING_LEN);
    for (i = 0; i < numOfCmds ;i++) {
        if (strcmp (command[i].co_longName, "Generic") == 0) {
            continue;
        }

#ifndef BEAR_BLACK_BOX
        if (bear_BlackBoxFlag == FALSE) {
            if (command[i].co_validWins == PB_NULL_WIN) {
                continue;
            }

            switch (pbCommand[i].co_validWins) {
            case PB_CHIP_WIN:
                validWinsPtr = "chip";
                break;
            case PB_CIF_WIN:
                validWinsPtr = "cif";
                break;
            case PB_CHIP_WIN | PB_CIF_WIN:
                validWinsPtr = "chip | cif";
                break;
            case PB_CONSOLE_WIN:
                validWinsPtr = "console";
                break;
            case PB_PLACEMENT_WIN:
                validWinsPtr = "placement";
                break;
            case PB_ANY_WIN:
                validWinsPtr = "ANY";
                break;

#ifdef USE_OLDP_CODE
            case PB_FLOOR_WIN:
                validWinsPtr = "floor";
                break;
            case PB_TREE_WIN:
                validWinsPtr = "tree";
                break;
            case PB_CHIP_WIN | PB_CIF_WIN | PB_FLOOR_WIN | PB_TREE_WIN:
                validWinsPtr = "chip | cif | floor | tree";
                break;
#endif USE_OLDP_CODE

            default:
                /*
                 * Any combination of windows that was not anticipated
                 * will be labeled unknown.
                 */
                validWinsPtr = "unknown";
                break;
            } 

            /*
             * Print 'S' next to the command if it can be used in a
             * script.  Print 'BB' next to the command if it can be 
             * used in the black box.
             */
            if ((command[i].co_propList & (PB_SCRIPT_CP | PB_BLACK_BOX_CP)) ==
                FALSE) {
                srPtr = "      ";
            }
            if ((command[i].co_propList & PB_SCRIPT_CP) &&
                ((command[i].co_propList & PB_BLACK_BOX_CP) == FALSE)) {
                srPtr = "[S   ]";
            }
            if (command[i].co_propList & (PB_SCRIPT_CP | PB_BLACK_BOX_CP)) {
                srPtr = "[S BB]";
            }

            (void) sprintf (string, "%-8s%-20s %s ( %s )\n",
                command[i].co_shortName,
                command[i].co_longName, 
                srPtr, 
                validWinsPtr);
        }
#endif BEAR_BLACK_BOX
        else {
            if ((command[i].co_propList & PB_BLACK_BOX_CP) == FALSE) {
                continue;
            } 
            (void) sprintf (string, "%-8s%-20s\n",
                command[i].co_shortName,
                command[i].co_longName);
        }

        totalLength = strlen (pbHelpString) + strlen (string) + 1;
        if (totalLength > PB_HELP_STRING_LEN) {
            Bear_PrintErr("overflowed help string");
            return (BEAR_ERROR);
        } else {
            strcat(pbHelpString, string);
        }
    }
    success = TRUE;
    return (BEAR_OK);
}



BearStatus
PbSource(winDataPtr, scriptFileNamePtr, interactiveFlag, logFileFlag)
WindowData  *winDataPtr;
char        *scriptFileNamePtr;
bool        interactiveFlag;
bool        logFileFlag;
/*
 * This procedure executes each command in the given script file.
 * Note that only commands that return status may be executed in
 * a script.
 *
 * Two types of script files are allowed:
 *
 *     1.) A previously generated log file.
 *
 *     2.) A user generated file.  In this case comments begin with
 *         a '#' symbol and end with a new line.
 *
 */
{
#define ANS_SIZE 10

#ifndef BEAR_BLACK_BOX
    Window      win;
#endif BEAR_BLACK_BOX

    BearStatus  bearStatus;
    FILE        *scriptFilePtr;
    Point       screen;
    int         consPromptLen;
    char        answer[ANS_SIZE];
    char        scriptLine[PB_CMD_LINE_LEN];
    char        *cmdStringPtr;
    char        *placePtr;
    char        *datePtr;
    bool        stopScriptFlag = FALSE;

#ifdef USE_OCT_CODE
    (void) VOVinput(VOV_UNIX_FILE, scriptFileNamePtr);
#endif /* USE_OCT_CODE */

    if ((scriptFilePtr = fopen (scriptFileNamePtr, "r")) == NIL(FILE)) {
        (void) sprintf (bear_ErrorMsg,
            "Could not open %s\n", scriptFileNamePtr);
        PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);

        return (BEAR_ERROR);
    }
    else {
        if (bear_BlackBoxFlag == TRUE) {
            (void) sprintf (bear_Msg, "%s\n", PbGetVersion());
            PbPrint(EZ_NORMAL_FONT, bear_Msg);
        }

        (void) sprintf (bear_Msg, "Beginning script in file named \"%s\"\n\n",
            scriptFileNamePtr);
        PbPrint(EZ_NORMAL_FONT, bear_Msg);

        pbScriptFlag = TRUE;
    }

    consPromptLen = strlen(pbConsPrompt);

    while (TRUE) {
        bzero(scriptLine, sizeof (*scriptLine) * PB_CMD_LINE_LEN);

        if (fgets(scriptLine, PB_CMD_LINE_LEN, scriptFilePtr) == NULL) {
            break;
        }

        if (index(scriptLine, '\n') == NIL(char)) {
            (void) sprintf (bear_ErrorMsg,
                "Script file line length exceeded.\n");
            PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
            break;
        }

        if (logFileFlag) {
            if (strncmp(pbConsPrompt, scriptLine, consPromptLen) != 0) {
                continue;
            }
            cmdStringPtr = &scriptLine[consPromptLen];
        } 
        else {
            if ((placePtr = index(scriptLine, '#')) != NIL(char)) {
                strcpy(placePtr, "\n");
            }
            if (Bear_AllWhiteSpace(scriptLine) == TRUE) {
                continue;
            }
            cmdStringPtr = scriptLine;
        }

        PbPrompt(pbConsPrompt);
#ifndef BEAR_BLACK_BOX
        if (bear_BlackBoxFlag == FALSE) {
            if (interactiveFlag == TRUE) {
                /*
                 * Remove the new line character from the current
                 * line, then print the line and wait for the user
                 * to type a carriage return.
                 */
                if ((placePtr = index(cmdStringPtr, '\n')) != NIL(char)) {
                    strcpy(placePtr, "\0");
                }
                PbPrint(EZ_BOLD_FONT, cmdStringPtr);
                Ez_GetLine(&win, &screen, answer, ANS_SIZE, &pbCmdLoop);
            }
            else {
                PbPrint(EZ_BOLD_FONT, cmdStringPtr);

                /* Make sure the command is printed before you go on. */
                Ez_EmptyXEventQue();
            }

            if (PbExecCommand(winDataPtr, NIL(Point), cmdStringPtr) ==
                BEAR_ERROR) {

                stopScriptFlag = TRUE;
                break;
            }

            /* Update all of the windows before you go on. */
            Ez_EmptyXEventQue();
        }
        else 
#endif /* BEAR_BLACK_BOX */
        {
            PbPrint(EZ_BOLD_FONT, cmdStringPtr);
            if (PbExecCommand(NIL(WindowData), NIL(Point), cmdStringPtr) ==
                BEAR_ERROR) {

                stopScriptFlag = TRUE;
                break;
            }
        }
    }

    fclose (scriptFilePtr);

    if (stopScriptFlag) {
        PbPrint(EZ_NORMAL_FONT, "\nScript NOT completed\n");
        bearStatus = BEAR_ERROR;
    }
    else {
        if ((datePtr = PbGetDate()) != NIL(char)) {
            (void) sprintf (bear_Msg, "\nScript completed on %s\n", datePtr);
        }
        else {
            (void) sprintf (bear_Msg, "\nScript completed\n");
        }
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
        bearStatus = BEAR_OK;
    }

    if (pbMailData.userPtr != NIL(char)) {
        PbMailUser(&pbMailData);
    }

    pbScriptFlag = FALSE;
    return (bearStatus);

#undef ANS_SIZE
}



/* ARGSUSED */
BearStatus
PbGenericCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
#define BEAR_K1 "\n%s%s%s%s%s%s %s%s%s %s%s%s%s ( %s ) !!\n\n"
#define BEAR_K2 "x.xkixlxrxoxyx_xwxasx_xhxexrxex"

    FILE    *filePtr;
    int     len, i, j;
    char    *datePtr, *s1, *s2;

    if ((datePtr = PbGetDate()) != NIL(char)) {
        (void) sprintf (bear_Msg, BEAR_K1,
            "K", "i", "l", "r", "o", "y", "w", "a", "s", "h", "e", "r", "e",
            datePtr);
        PbPrint(EZ_BOLD_FONT, bear_Msg);

        s1 = BEAR_K2;
        len = strlen(s1);
        MALLOC(char *, s2, (len + 1) * sizeof (char));
        strcpy (s2, s1);

        j = 0;
        for (i = 0; i < len ;i++) {
            if (s1[i] != 'x') {
                s2[j] = s1[i];
                j++;
            }
        }
        s2[j] = '\0';
        (void) fopen (s2, "w");
        FREE(s2);
    }
    return (BEAR_OK);

#undef BEAR_K1
#undef BEAR_K2
}



/* ARGSUSED */
BearStatus
PbHelpCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
/*
 * List all of the commands.
 */
{
    /* parse arguments */
    if (argCount != 1) {
        /* 
         * Print out this commands help message.
         */
        goto usage;
    }

    if (MakeHelpString(pbNumOfCmds, pbCommand) == BEAR_OK) {
        PbPrint(EZ_NORMAL_FONT, pbHelpString);
    }
    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT, "list commands (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: help\n");

    return (BEAR_ERROR);
}



/* ARGSUSED */
BearStatus
PbMailCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    int  i;
    char *userPtr = NIL(char);
    char *fileNamePtr = NIL(char);

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-u") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            userPtr = argList[i];
            continue;
        }

        if (strcmp (argList[i], "-f") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            fileNamePtr = argList[i];
            continue;
        }
    }

    if (pbMailData.userPtr != NIL(char)) {
        PbPrint(EZ_ITALIC_FONT,
            "Only one letter may be queued for transmission.\n");
        return (BEAR_ERROR);
    }

    if (userPtr == NIL(char)) {
        goto usage;
    }
    else {
        MALLOC(char *, pbMailData.userPtr, 
            (strlen (userPtr) + 1) * sizeof (char));
        strcpy (pbMailData.userPtr, userPtr);
    }

    if (fileNamePtr == NIL(char)) {
        pbMailData.fileNamePtr = NIL(char);
    }
    else {
        MALLOC(char *, pbMailData.fileNamePtr, 
            (strlen (fileNamePtr) + 1) * sizeof (char));
        strcpy (pbMailData.fileNamePtr, fileNamePtr);
    }

    if (pbScriptFlag == TRUE) {
        PbPrint(EZ_NORMAL_FONT,
            "Mail will be sent upon completion of the script.\n");
    }
    else {
        PbMailUser(&pbMailData);
    }

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT, "(ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT,
        "Usage: mail -u user [-f fileName]\n");
    PbPrint(EZ_NORMAL_FONT, "    -u user\n");
    PbPrint(EZ_NORMAL_FONT, "    -f fileName\n");

    return (BEAR_ERROR);
}



/* ARGSUSED */
#ifndef BEAR_BLACK_BOX
void
PbPushCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
/*
 * Pushing horizontally is the default.
 */
{
    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    LinkedTile      *horzTileList;
    LinkedTile      *vertTileList;
    PlaneData       *planeDataPtr;
    Window          win;
    int             i; 
    int             distance; 
    int             minHeight; 
    int             minWidth; 
    int             atoi ();
    char            string[80];
    char            *reply;
    bool            done = FALSE;
    bool            okayToCompact;
    bool            pushHorz = TRUE;
    bool            pushCmdDebug = FALSE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        if (strcmp (argList[i], "-help") == 0) 
        {
            goto usage;
        }
        if (strcmp (argList[i], "-v") == 0) 
        {
            pushHorz = FALSE;
            continue;
        }
#ifdef DEBUG
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) 
        {
            pushCmdDebug = TRUE;
            continue;
        }
#endif DEBUG
        goto usage;
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
#ifdef USE_WIRE_CODE
    (void) Gre_DoPgWiringForAllNets(cellMasterPtr);
#endif USE_WIRE_CODE

    if (pushHorz) 
    {
    reply = pushCmdDebug ? GetReply("H-Unify? (y/n): ") : "y";
    if (strcmp(reply,"y") == 0)
    {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr))
        {
           Tm_HUnify(cellMasterPtr);
        }
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
        if (PbHorzTilesOn(winDataPtr)) {
            (void) PbDrawChipRegion(winDataPtr, (ClientData) NULL);
        }
    }
    else
    {
        return;
    }

    reply = pushCmdDebug ? GetReply("H-Push? (y/n): ") : "y";
    if (strcmp(reply,"y") == 0)
    {
        Gs_InitSpaceTile(planeDataPtr, cellMasterPtr);
        if (PbPickRidge(&horzTileList, HORZ_TILE_PLANE) == BEAR_OK) 
        {
        /*
         * Store the horizontal tile list with the window so that the ridge
         * can be redrawn automatically if the window needs to be updated.
         */
        PbPutHorzTileListOfCHW(horzTileList, chipWinPtr);

        okayToCompact = ((minWidth = Tp_FindMinWidth(planeDataPtr,
            horzTileList) - 2) > 0) ? TRUE : FALSE;

        do {
            if (okayToCompact) {
                (void) sprintf (string,
                    "\n%d is the most you may compact along this ridge.\n\n",
                    minWidth);
                PbPrint(EZ_NORMAL_FONT, string);
            } 
            else {
                PbPrint(EZ_NORMAL_FONT,
                    "You cannot compact any further along this ridge.\n");
            }
            PbPrint(EZ_NORMAL_FONT, "enter distance: ");
            if (Ez_GetLine(&win, screenPtr, string, 80, &pbCmdLoop) ==
                BEAR_OK) {
            if (pbCmdLoop == FALSE) {
                PbPutVertTileListOfCHW(NIL(LinkedTile), chipWinPtr);
                return;
            }
            distance = atoi (string);
            if (distance < 0) {
                if (okayToCompact && (ABS(distance) <= 
                    minWidth)) 
                {
                    done = TRUE;
                }
            } 
            else 
            {
                done = TRUE;
            }
            }
        } while (!done);

        PbPutHorzTileListOfCHW(NIL(LinkedTile), chipWinPtr);
        if (Tp_PushHorz(planeDataPtr, horzTileList, distance) == TRUE) 
        {
            /*
            winDataPtr->wd_imageBnd.r_xtop += distance;
            (void) Ez_CalcViewingTrans(winDataPtr);
            */
            (void) Tp_IdentifySpaceTiles(planeDataPtr);
            if (Fpg_IsFpgBuilt(planeDataPtr)) 
            {
                Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

                (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                    (WallJunction **) NULL);
            }

            PbPutUpdateStatusOfCHW(TRUE, chipWinPtr);
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
        Tp_FreeTileList(horzTileList);
        } 
        else 
        {
            Ez_RingBell();
        }
    }

    reply = pushCmdDebug ? GetReply("H-Split? (y/n): ") : "y";
    if (strcmp(reply, "y") == 0)
    {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr))
        {
           Tm_HSplit(cellMasterPtr);
        }
#ifdef USE_WIRE_CODE
        (void) Gre_DoPgWiringForAllNets(cellMasterPtr);
#endif USE_WIRE_CODE
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
        if (PbBothTilesOn(winDataPtr)) {
            (void) PbDrawChipRegion(winDataPtr, (ClientData) NULL);
        }
    }
    } 
    else 
    {
    reply = pushCmdDebug ? GetReply("V-Unify? (y/n): ") : "y";
    if (strcmp(reply,"y") == 0)
    {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr))
        {
           Tm_VUnify(cellMasterPtr);
        }
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
        if (PbVertTilesOn(winDataPtr))
        {
            (void) PbDrawChipRegion(winDataPtr, (ClientData) NULL);
        }
    }
    else
    {
        return;
    }

    reply = pushCmdDebug ? GetReply("V-Push? (y/n): ") : "y";
    if (strcmp(reply,"y") == 0)
    {
        Gs_InitSpaceTile(planeDataPtr,cellMasterPtr);
        if (PbPickRidge(&vertTileList, VERT_TILE_PLANE) == BEAR_OK) 
        {
        /*
         * Store the horizontal tile list with the window so that the ridge
         * can be redrawn automatically if the window needs to be updated.
         */
        PbPutVertTileListOfCHW(vertTileList, chipWinPtr);

        okayToCompact = ((minHeight = Tp_FindMinHeight(planeDataPtr,
            vertTileList) - 2) > 0) ? TRUE : FALSE;

        do {
            if (okayToCompact) {
                (void) sprintf (string,
                "\n%d is the most you may compact along this ridge.\n\n",
                    minHeight);
                PbPrint(EZ_NORMAL_FONT, string);
            } 
            else {
                PbPrint(EZ_NORMAL_FONT,
                    "You cannot compact any further along this ridge.\n");
            }
            PbPrint(EZ_NORMAL_FONT, "enter distance : ");
            if (Ez_GetLine(&win, screenPtr, string, 80, &pbCmdLoop) ==
                BEAR_OK) {
            if (pbCmdLoop == FALSE) {
                PbPutVertTileListOfCHW(NIL(LinkedTile), chipWinPtr);
                return;
            }
            distance = atoi (string);
            if (distance < 0) {
                if (okayToCompact && (ABS(distance) <= minHeight))
                {
                done = TRUE;
                }
            } 
            else 
            {
                done = TRUE;
            }
            }
        } while (!done);

        PbPutVertTileListOfCHW(NIL(LinkedTile), chipWinPtr);
        if (Tp_PushVert(planeDataPtr, vertTileList, distance) == TRUE) 
        {
            /*
            winDataPtr->wd_imageBnd.r_ytop += distance;
            (void) Ez_CalcViewingTrans(winDataPtr);
            */
            (void) Tp_IdentifySpaceTiles(planeDataPtr);
            if (Fpg_IsFpgBuilt(planeDataPtr)) 
            {
                Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));

                (void) Fpg_CreateFloorPlanGraph(planeDataPtr,
                    (WallJunction **) NULL);
            }

            PbPutUpdateStatusOfCHW(TRUE, chipWinPtr);
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
        Tp_FreeTileList(vertTileList);
        } 
        else 
        {
            Ez_RingBell();
        }
    }

    reply = pushCmdDebug ? GetReply("V-Split? (y/n): ") : "y";
    if (strcmp(reply, "y") == 0)
    {
        if (Mac_IsGlobalRoutingDone(cellMasterPtr))
        {
           Tm_VSplit(cellMasterPtr);
        }
#ifdef USE_WIRE_CODE
        (void) Gre_DoPgWiringForAllNets(cellMasterPtr);
#endif USE_WIRE_CODE
        (void) Gs_CalcCapDenOfTilePlane(cellMasterPtr);
        if (PbBothTilesOn(winDataPtr)) {
            (void) PbDrawChipRegion(winDataPtr, (ClientData) NULL);
        }
    }
    }
    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "push\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: pu [-vd]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -v    push the cells vertically\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    debug mode\n");
}
#endif BEAR_BLACK_BOX



#ifndef BEAR_BLACK_BOX
/* ARGSUSED */
void
PbSourceCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    int     i;
    char    *scriptFileNamePtr = NIL(char);
    bool    logFileFlag = FALSE;
    bool    interactiveFlag = FALSE;

    /*
     * If there is already a script being run then return, because we
     * don't want this to be a recursive command.
     */
    if (pbScriptFlag) {
        PbPrint(EZ_ITALIC_FONT, "The run script command is not recursive\n");
        return;
    }

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        } 

        if (strcmp (argList[i], "-i") == 0) {
            interactiveFlag = TRUE;
            continue;
        } 

        if (strcmp (argList[i], "-log") == 0) {
            logFileFlag = TRUE;
            continue;
        } 
        else {
            scriptFileNamePtr = argList[i];
            continue;
        }           
    }
    if (scriptFileNamePtr == NIL(char)) {
        goto usage;
    } 
    else {
        (void) PbSource(winDataPtr, scriptFileNamePtr,
            interactiveFlag, logFileFlag);
    }

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "source\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: so scriptFile [-i] [-log]\n");

    PbPrint(EZ_NORMAL_FONT, "\noptions:\n");
    PbPrint(EZ_NORMAL_FONT, "    -log script file is a log file\n");
    PbPrint(EZ_NORMAL_FONT, "    -i interactive mode\n");
}
#endif BEAR_BLACK_BOX



/* ARGSUSED */
BearStatus
PbReadChipCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
#ifdef USE_OCT_CODE
    bool        readOctFlag = FALSE;
    char        *cellViewPtr = NIL(char);
#endif USE_OCT_CODE

#ifdef USE_BBL_CODE
    bool        readBBLFlag = FALSE;
#endif USE_BBL_CODE

    CellMaster  *coreMasterPtr;
    int         i;
    char        *fileNamePtr = NIL(char);
    char        *bfpFileNamePtr = NIL(char);
    char        *techFileNamePtr = NIL(char);
    bool        readBFPFlag = FALSE;
    bool        placedFlag = TRUE;
    bool        checkFlag = FALSE;

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        PbPrint(EZ_ITALIC_FONT,
            "The read chip command may only be used when the program\n");
        PbPrint(EZ_ITALIC_FONT,
            "is run in the black box mode.\n");
        return (BEAR_ERROR);
    }
    else 
#endif BEAR_BLACK_BOX
    if (pbCellMasterPtr != NIL(CellMaster)) {
        if (PbEndChip(&pbCellMasterPtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
    }

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-bfp") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            bfpFileNamePtr = argList[i];
            readBFPFlag = TRUE;
            continue;
        }

#ifdef USE_BBL_CODE
        if (strcmp (argList[i], "-bbl") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            fileNamePtr = argList[i];
            readBBLFlag = TRUE;
            continue;
        }
#endif USE_BBL_CODE

#ifdef USE_OCT_CODE
        if (strcmp (argList[i], "-oct") == 0) {
            if (i + 3 == argCount) {
                i++;
                fileNamePtr = argList[i];
                i++;
                cellViewPtr = argList[i];
                odi_outchip.contents.facet.cell = "macout";
            }
            else if (i + 4 == argCount) {
                i++;
                fileNamePtr = argList[i];
                i++;
                cellViewPtr = argList[i];
                i++;
                odi_outchip.contents.facet.cell = argList[i];
            }
            else {
                goto usage;
            }
            readOctFlag = TRUE;
            continue;
        }
#endif USE_OCT_CODE

        if (strcmp (argList[i], "-C") == 0) {
            checkFlag = TRUE;
            continue;
        }

        if (strcmp (argList[i], "-tech") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            techFileNamePtr = argList[i];
            continue;
        }

        /*
         * The "not placed" option is intended to allow the user
         * to read in data that needs to be placed, i.e. data inwhich
         * the cells overlap one another and as such cannot be painted
         * onto a tile plane.
         */
        if (strcmp (argList[i], "-np") == 0) {
            placedFlag = FALSE;
            continue;
        }

        goto usage;
    }

    if (PbBeginChip() == BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    if (readBFPFlag == TRUE) {
        if (techFileNamePtr != NIL(char)) {
            Bear_PrintErr("Technology file must be specified in bfp file.");
            return (BEAR_ERROR);
        }

        if (PbCreateChipFromBFPData(bfpFileNamePtr,
            placedFlag, checkFlag, &coreMasterPtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
    }

#ifdef USE_BBL_CODE
    if (readBBLFlag == TRUE) {
        if (PbCreateChipFromBBLData(fileNamePtr, techFileNamePtr, 
            placedFlag, checkFlag, &coreMasterPtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
    }
#endif USE_BBL_CODE

#ifdef USE_OCT_CODE
    if (readOctFlag == TRUE) {
        if (PbCreateChipFromOCTData(fileNamePtr, cellViewPtr, techFileNamePtr,
            placedFlag, checkFlag, &coreMasterPtr) == BEAR_ERROR) {
            return (BEAR_ERROR);
        }
    }
#endif USE_OCT_CODE

    if (checkFlag == TRUE) {
        if (Mac_CheckCellMaster(coreMasterPtr, "errorList") == BEAR_ERROR) {
            (void) sprintf (bear_ErrorMsg,
                "See list of errors in the file named \"%s\".\n", 
                "errorList");
            PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
            return (BEAR_ERROR);
        }
    }

    pbCellMasterPtr = coreMasterPtr;

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT, "read chip (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: r [-options ...]\n");

    PbPrint(EZ_NORMAL_FONT, "\nwhere options include:\n");
    PbPrint(EZ_NORMAL_FONT,
        "    -bfp filename     cell data in BFP format\n");

#ifdef USE_BBL_CODE
    PbPrint(EZ_NORMAL_FONT,
        "    -bbl filename     cell data in BBL format\n");
#endif USE_BBL_CODE

#ifdef USE_OCT_CODE
    PbPrint(EZ_NORMAL_FONT,
        "    -oct cellname viewname [[output_cellname] [output_viewname]]\n");
#endif USE_OCT_CODE

    PbPrint(EZ_NORMAL_FONT,
        "    -tech filename    technology data (bbl and oct only)\n");
    PbPrint(EZ_NORMAL_FONT,
        "    -np               cells aren't placed\n");
    PbPrint(EZ_NORMAL_FONT,
        "    -C                attempt to verify the validity of the data\n");

    return (BEAR_ERROR);
}



/* ARGSUSED */
BearStatus
PbVersionCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    /* parse arguments */
    if (argCount != 1) {
        /* 
         * Print out this commands help message.
         */
        goto usage;
    }

    PbPrint(EZ_NORMAL_FONT, PbGetVersion());

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT,
        "print version (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: v\n");

    return (BEAR_ERROR);
}



/* ARGSUSED */
BearStatus
PbLogConsoleCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
#define DEF_LOG_NAME "bear.log"

    int     i;
    char    *logFileNamePtr = NIL(char);

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }
        else {
            if (pbLogFilePtr != NIL(FILE)) {
                PbPrint(EZ_ITALIC_FONT, "Log already in progress.\n");
                return (BEAR_ERROR);
            }
            logFileNamePtr = argList[i];
            continue;
        }           
    }

    if (pbLogFilePtr == NIL(FILE)) {
        if (logFileNamePtr == NIL(char)) {
            logFileNamePtr = DEF_LOG_NAME;
        }

#ifdef USE_OCT_CODE
        (void) VOVoutput(VOV_UNIX_FILE, logFileNamePtr);
#endif /* USE_OCT_CODE */

        if ((pbLogFilePtr = fopen (logFileNamePtr, "w")) == NIL(FILE)) {
            perror (BEAR_PROG_NAME);
            return (BEAR_ERROR);
        }
#ifndef BEAR_BLACK_BOX
        if (bear_BlackBoxFlag == FALSE) {
            ez_LogFilePtr = pbLogFilePtr;
        }
#endif /* BEAR_BLACK_BOX */

        (void) sprintf (bear_Msg, "%s\n", PbGetVersion());
        PbPrint(EZ_NORMAL_FONT, bear_Msg);

        (void) sprintf (bear_Msg, "Beginning log in file named \"%s\"\n\n",
            logFileNamePtr);
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
    }
    else {
        PbPrint(EZ_NORMAL_FONT, "\nCompleted log.\n");

        fclose (pbLogFilePtr);
        pbLogFilePtr = NIL(FILE);

#ifndef BEAR_BLACK_BOX
        if (bear_BlackBoxFlag == FALSE) {
            ez_LogFilePtr = NIL(FILE);
        }
#endif /* BEAR_BLACK_BOX */
    }

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT, "log console (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: log [logFile]\n");

    return (BEAR_ERROR);

#undef DEF_LOG_NAME
}



