
/* pbNetCmds.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * The code within the USE_WIRE_CODE markers is stil under development.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"
#include "ezPlot.h"
#include "bearGrUtils.h"
#include "geoChDecomposer.h"
#include "deque.h"
#include "topChDecomposer.h"
#include "globalRouter.h"
#include "newG.h"
#include "grEditor.h"
#include "cpaInterface.h"
#include "polarBear.h"
#include "list.h"

#ifndef BEAR_BLACK_BOX
#include "bearIV.h"
#include "pbDialogs.h"
#endif BEAR_BLACK_BOX


/* ---------- Local Procedures ---------- */
static void         AlterPortData();
static BearStatus   EnterPortData();
static BearStatus   PickOneSrcPort();
static BearStatus   CountPin();


/* ---------- Local Global Variables ---------- */
#ifdef DEBUG
bool wireDebug;
#endif DEBUG


#ifdef USE_WIRE_CODE
static void
AlterPortData(winDataPtr)
WindowData *winDataPtr;
{
    PinInst     *pinPtr;
    PortInst    *portInstPtr = NIL(PortInst);
    PortKind    portKind;

    PbPrint(EZ_NORMAL_FONT, "Pick a port.\n\n");
    if (Gre_GetPin(winDataPtr, (BearString) NULL, (BearString) NULL,
        ez_HighLightColor, &pinPtr) != BEAR_OK) {
        PbPrint(EZ_ITALIC_FONT, "Picking port failed..\n\n");
        return;
    }
    portInstPtr = Mac_GetPortOfPin(pinPtr);
    portKind = Mac_GetKindOfPort(portInstPtr);
    if (portKind == (PortKind) PG_PORT ||
        portKind == (PortKind) PG_SRC_PORT) {
        if (EnterPortData(portInstPtr, (ClientData) NULL) ==
            BEAR_STOP_SEARCH) {
            return;
        }
    }
    else {
        PbPrint(EZ_ITALIC_FONT, "Must be a power/ground port.\n\n");
    }
}
#endif USE_WIRE_CODE



#ifndef BEAR_BLACK_BOX
#ifdef USE_WIRE_CODE
/* ARGSUSED */
static BearStatus
EnterPortData(portInstPtr, clientData)
PortInst    *portInstPtr;
ClientData  clientData;
{
#define RETURN(status) \
    drawPort.portInstPtr = portInstPtr;\
    drawPort.portColor = PbGetColorOfNet(FPG_CENTER);\
    (void) PbItrChipWindows(Gre_DrawPinsOfPortTwo, (ClientData) &drawPort); \
    Ez_Flush();\
    return (status);\

    CellInst    *cellInstPtr;
    DrawPort    drawPort;
    Net         *netPtr;
    PortKind    portKind;
    double      maxVoltDrop, maxCurrentReq;
    char        *portNamePtr, *cellNamePtr, *netNamePtr;
    /* Used for IV */
    XEvent      theEvent;
    IVWindow    *IVwin;
    bool        pgSrcFlag = FALSE;
    bool        acceptFlag = FALSE;
    bool        exitFlag = FALSE;
   
    drawPort.portInstPtr = portInstPtr;
    drawPort.portColor = ez_HighLightColor;
    (void) PbItrChipWindows(Gre_DrawPinsOfPortTwo, (ClientData) &drawPort);
    Ez_Flush();

    portKind = Mac_GetKindOfPort(portInstPtr);
    if ((portKind != PG_PORT) && (portKind != PG_SRC_PORT)) {
        PbPrint(EZ_ITALIC_FONT, "Must be a power/ground port.\n\n");
        RETURN(BEAR_STOP_SEARCH);
    }

    IVwin = iv_InitWindow(ez_Display, BEAR_PROG_NAME,"Enter Port Data", 13);
    if ((portNamePtr = Mac_GetNameOfPortInst(portInstPtr)) == NIL(char)) {
        MALLOC(char *, portNamePtr, 10);
        strcpy(portNamePtr, "NONE");
    }
    (void) iv_AddVar(IVwin, "Port Name",
        (int *) &portNamePtr, IV_STRING, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_SetEnableFlag(IVwin, (int *) &portNamePtr, 0);
    cellInstPtr = Mac_GetCellInstOfPort(portInstPtr);
    if ((cellNamePtr = Mac_GetNameOfCellInst(cellInstPtr)) == NIL(char)) {
        MALLOC(char *, cellNamePtr, 10);
        strcpy(cellNamePtr, "NONE");
    }
    netPtr = Mac_GetNetOfPort(portInstPtr);
    if ((netNamePtr = Mac_GetNameOfNet(netPtr)) == NIL(char)) {
        MALLOC(char *, netNamePtr, 10);
        strcpy(netNamePtr, "NONE");
    }
    (void) iv_AddVar(IVwin, "Net Name",
        (int *) &netNamePtr, IV_STRING, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_SetEnableFlag(IVwin, (int *) &netNamePtr, 0);

    maxVoltDrop = Mac_GetMaxVoltDropOfPort(portInstPtr);
    (void) iv_AddVar(IVwin, "Max Voltage Drop",
        (int *) &maxVoltDrop, IV_DOUBLE, 3, IV_NULL_FUNCTION);

    maxCurrentReq = Mac_GetMaxCurrentReqOfPort(portInstPtr);
    (void) iv_AddVar(IVwin, "Default Current Requirement",
        (int *) &maxCurrentReq, IV_DOUBLE, 3, IV_NULL_FUNCTION);

    pgSrcFlag = (portKind == PG_SRC_PORT);
    (void) iv_AddVar(IVwin, "P/G source", &pgSrcFlag,
        IV_BOOLEAN, IV_NO_OPT, IV_NULL_FUNCTION);

    (void) iv_AddVar(IVwin, "Accept port data", &acceptFlag,
        IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

    (void) iv_AddVar(IVwin, "Exit", &exitFlag,
        IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_MapWindow(IVwin, IV_NO_OPT, IV_NO_OPT, IV_MOUSE);

    while (!acceptFlag) {
        XNextEvent(ez_Display, &theEvent);
        (void) iv_HandleEvent(&theEvent);
        Ez_XEventHandler(&theEvent);
        if (exitFlag) {
           iv_FreeWindow(IVwin);
           RETURN(BEAR_STOP_SEARCH);
        }
    }
    iv_FreeWindow(IVwin);
    if (pgSrcFlag) {
        portKind = PG_SRC_PORT;
    } else {
        portKind = PG_PORT;
    }
    Mac_PutKindOfPort(portKind, portInstPtr);
    Mac_PutMaxVoltDropOfPort(maxVoltDrop, portInstPtr);
    Mac_PutMaxCurrentReqOfPort(maxCurrentReq, portInstPtr);
    RETURN(BEAR_CONTINUE_SEARCH);
    /* NOTREACHED */
    
#undef RETURN
}
#endif USE_WIRE_CODE
#endif BEAR_BLACK_BOX



#ifdef USE_WIRE_CODE
#ifdef DEBUG
/* ARGSUSED */
static BearStatus
PickOneSrcPort(portInstPtr, clientData)
PortInst    *portInstPtr;
ClientData  clientData;
{
    Mac_PutKindOfPort(PG_SRC_PORT, portInstPtr);

    return (BEAR_STOP_SEARCH);
}
#endif DEBUG
#endif USE_WIRE_CODE


#ifdef USE_WIRE_CODE
#ifdef DEBUG
static void
PrintPgWireDataOfNet(winDataPtr, netPtr, labelFlag)
WindowData  *winDataPtr;
Net         *netPtr;
bool        labelFlag;
{
    BearString  chipInstName;
    CellInst    *chipInstPtr, *coreInstPtr;
    CellMaster  *chipMasterPtr, *coreMasterPtr;
    ChipWin     *chipWinPtr;
    LabelNet    labelNet;
    char        fileName[80];
    
    chipWinPtr = PbGetChipWin(winDataPtr);
    coreMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    coreInstPtr = Mac_GetInstOfCoreMaster(coreMasterPtr);
    chipMasterPtr = Mac_GetParentOfCell(coreInstPtr);
    chipInstPtr = Mac_GetInstOfChipMaster(chipMasterPtr);
    
    if ((chipInstName = Mac_GetNameOfCellInst(chipInstPtr)) ==
        (BearString) NULL) {
    
        sprintf (fileName, "w_%s", Mac_GetNameOfNet(netPtr));
    } else {
        sprintf (fileName, "w_%s_%s", chipInstName, 
            Mac_GetNameOfNet(netPtr));
    }
    printf ("See file named \"%s\"\n", fileName);
    fflush (stdout);
    Pgr_PrintPgWireData(fileName, netPtr->wirePtr);
    
    if (labelFlag == TRUE) {
        Bear_SetOption(chipWinPtr->chw_options, PB_LABEL_CENTER_NET_OPT);
        
        labelNet.netPtr = netPtr;
        labelNet.planeDataPtr = Mac_GetCellPlaneOfCell(coreMasterPtr);
        labelNet.whichNet = FPG_CENTER;
        Gre_LabelNet(winDataPtr, (ClientData) &labelNet);
    }
}
#endif DEBUG
#endif USE_WIRE_CODE



/* ARGSUSED */
#ifndef BEAR_BLACK_BOX
void
PbDeleteNetCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    Net     *netPtr;
    PinInst *pinPtr;

    /*
     * Have the user select the net he wants to delete 
     * with the mouse.
     */
    if (Gre_PickNet(winDataPtr, &netPtr, &pinPtr) == BEAR_ERROR) {
        return;
    }

    if (netPtr == NIL(Net)) {
        PbPrint(EZ_ITALIC_FONT, "no net is given. can not delete net.\n");
        return;
    }
    Mac_DeleteNet(netPtr);

    /*
     * You should remove it from the tile plane too.
     */
}
#endif BEAR_BLACK_BOX



#ifndef BEAR_BLACK_BOX
#ifdef USE_WIRE_CODE
/* ARGSUSED */
void
PbRouteNetCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
#define RETURN                                                              \
{                                                                           \
    Gre_UsePgWireDataToDrawNet(TRUE);                                       \
    return;                                                                 \
}

    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    Net             *netPtr;
    PinInst         *pinPtr;
    PlaneData       *planeDataPtr;
    WallJunction    *llJunctPtr;
    Window          win;
    IVWindow        *IVwin;
    XEvent          theEvent;
    int             i;
    int             showMenu = 3;
    char            answer[80], *netName;
    char            *netNamePtr = NIL(char);
    bool            redraw = FALSE;
    bool            storeNetInTiPlane = FALSE;
    bool            exitFlag = FALSE;
    bool            addFlag = FALSE, deleteFlag = FALSE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-n") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            if (netNamePtr == NIL(char)) {
                netNamePtr = argList[i];
            }
            else {
                PbPrint(EZ_ITALIC_FONT,
                    "Only one net name may be specified.\n");
                goto usage;
            }
            continue;
        }

        goto usage;
    }

    Gre_UsePgWireDataToDrawNet(FALSE);

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE) {
        BuildFloorPlan(cellMasterPtr, TRUE);
        Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        redraw = TRUE;
    }
    else if (! (chipWinPtr->chw_options & PB_SHOW_FPG_OPT)) {
        Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        redraw = TRUE;
    }

    /*
     * Set floor plan graph pointer after you are sure the floor plan
     * has been built.
     */
    llJunctPtr = Mac_GetFpgOfCell(cellMasterPtr);

    if (redraw) {
        (*winDataPtr->wd_redrawProc)(winDataPtr, NIL(Rect));
    }
                
    if (netNamePtr != NIL(char)) {
        /* 
         * Use the net name they provided to locate the net, and
         * then display it.
         */
        if (Gre_PickNetByName(winDataPtr, netNamePtr, FPG_CENTER, &netPtr) ==
            BEAR_ERROR) {
            RETURN;
        }
    }
    else {
        if ((netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr)) == NIL(Net)) {
            /* 
             * Use the mouse to locate the net, and then display it.
             */
            if (Gre_PickNet(winDataPtr, &netPtr, &pinPtr) == BEAR_ERROR) {
                RETURN;
            }
        }
    }

    if ((Mac_GetUserGraphOfNet(netPtr) == (Gt_Graph) NULL) &&
        (Mac_GetStatusOfNetInTiPlane(netPtr) == TRUE)) {
        PbPrint(EZ_ITALIC_FONT, "This net has already been routed.\n");
        RETURN;
    }

    IVwin = iv_InitWindow(ez_Display, BEAR_PROG_NAME, "Route Net", 13);
    netName = Mac_GetNameOfNet(netPtr);
    (void) iv_AddVar(IVwin, "Net Name",
        (int *) &netName, IV_STRING, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_SetEnableFlag(IVwin, (int *) &netName, 0);
    (void) iv_AddVar(IVwin, "Add graph edge",
        &addFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Delete graph edge",
        &deleteFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Exit Routing",
        &exitFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

    (void) iv_MapWindow(IVwin, IV_NO_OPT, IV_NO_OPT, IV_MOUSE);

    /* Maintain the number of completely routed nets */
    /*
    Mac_NumOfGRTreesOfNet(netPtr, &netCount);
    Mac_NumOfPinsInGRForestOfNet(netPtr, &pinTreeCount);
    Mac_NumOfPinsInPortListOfNet(netPtr, &pinNetCount);
    if ((netCount == 1) && (pinTreeCount == pinNetCount))
    {
       NumOfConnectedNets--;
    }
    */

    while (!exitFlag) {
        XNextEvent(ez_Display, &theEvent);
        (void) iv_HandleEvent(&theEvent);
        Ez_XEventHandler(&theEvent);
        
        if (addFlag == TRUE) {
            if (Mac_GetStatusOfNetInTiPlane(netPtr) == TRUE) {
                Gre_RemoveNet(netPtr);
            }
            (void) Gre_AddEdgeToGraph(netPtr);
            storeNetInTiPlane = TRUE;
            addFlag = FALSE;
        }

        if (deleteFlag == TRUE) {
            if (Mac_GetStatusOfNetInTiPlane(netPtr) == TRUE) {
                Gre_RemoveNet(netPtr);
            }
            (void) Gre_DeleteEdgeFromGraph(netPtr);
            storeNetInTiPlane = TRUE;
            deleteFlag = FALSE;
        }
    }
    iv_FreeWindow(IVwin);
    if (storeNetInTiPlane) {
        (void) Gre_StoreNetInTiPlane(netPtr, planeDataPtr);
    
    /* Maintain number of completely connected nets */
    /*
    Mac_NumOfGRTreesOfNet(netPtr, &netCount);
    Mac_NumOfPinsInGRForestOfNet(netPtr, &pinTreeCount);
    Mac_NumOfPinsInPortListOfNet(netPtr, &pinNetCount);
    if ((netCount == 1) && (pinTreeCount == pinNetCount)) {
       NumOfConnectedNets++;
    }
    */

    /* If all nets in the chip are connected - the set GLOBAL_ROUTING_DONE */
    /*
    Mac_NumOfNet(cellMasterPtr, &netCount);
    if (netCount == NumOfConnectedNets) {
       Mac_SetGlobalRoutingDone(cellMasterPtr);
    }
    else {
       Mac_UnSetGlobalRoutingDone(cellMasterPtr);
    }
    */
    }   
    RETURN;
            
usage:
    PbPrint(EZ_NORMAL_FONT, "interactively route a net\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: rn [-n netName]\n");

#undef RETURN
}
#endif USE_WIRE_CODE
#endif BEAR_BLACK_BOX



/* ARGSUSED */
BearStatus
PbGlobalRouterCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;     /* not used */
int         argCount;
char        **argList;
/*
 * BEAR_OK is returned only if the global routing is successfully completed.
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    lsList          thinBnList;
    double          atof ();
    int             atoi ();
    CellMaster      *cellMasterPtr;
    FILE            *filePtr;
    Net             *netPtr;
    PlaneData       *planeDataPtr;
    WallJunction    *llJunctPtr;
    double          congestion = 0.0;   /* default congestion factor    */
    int             timingMode = 0;     /* default timing mode          */
    int             numOfPin;
    int             i;
    bool            dialogWinFlag = TRUE;
    bool            warnFlag = FALSE;
    bool            checkConnFlag = FALSE;
    char            *tmFileNamePtr = NIL(char);
    char            *tmpStringPtr = NIL(char);

    bool            exitFlag = FALSE;
    bool            routeFlag = FALSE;
    bool            timingModeFlag = FALSE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-C") == 0) {
            checkConnFlag = TRUE;
            warnFlag = FALSE;
            continue;
        }
        if (strcmp (argList[i], "-CW") == 0) {
            checkConnFlag = TRUE;
            warnFlag = TRUE;
            continue;
        }

        /* 
         * Don't pop up any dialog windows.  Used when you want to run
         * this command from a script.
         */
        if (strcmp (argList[i], "-NDW") == 0) {
            dialogWinFlag = FALSE;
            continue;
        }

        /* 
         * The congestion factor.
         */
        if (strcmp (argList[i], "-cf") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            congestion = atof (argList[i]);
            continue;
        }

        /* 
         * The name of the timing file.
         */
        if (strcmp (argList[i], "-tf") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            tmFileNamePtr = argList[i];
            continue;
        }

        /* 
         * The timing mode.
         */
        if (strcmp (argList[i], "-tm") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            timingMode = atoi (argList[i]);
            continue;
        }

        (void) sprintf (bear_Msg, "unknown option %s\n", argList[i]);
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
        goto usage;
    }

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    }
    else
#endif BEAR_BLACK_BOX
    {
        if (pbCellMasterPtr == NIL(CellMaster)) {
            PbPrint(EZ_ITALIC_FONT, "Must read in a cell master first.\n");
            return (BEAR_ERROR);
        }
        else {
            cellMasterPtr = pbCellMasterPtr;
        }
    }

    if (Mac_IsNumOfCellsZero(cellMasterPtr) == TRUE) {
        PbPrint(EZ_ITALIC_FONT,
            "Current cell master has no cell instances or nets.\n");
        return (BEAR_ERROR);
    }
    if (Mac_IsRingRoutingDone(cellMasterPtr) == TRUE) {
        PbPrint(EZ_ITALIC_FONT,
            "Cannot do global routing after ring routing.\n");
        return (BEAR_ERROR);
    }

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE) {
        /*
         * In this case we don't have a floor plan graph, and
         * so we build one with dummy cells.
         */
        BuildFloorPlan(cellMasterPtr, TRUE);
    }
    else {
        if (Tp_GetDCellList(planeDataPtr) == NIL(DummyCell)) {
            if (Tp_CreateDCells(planeDataPtr) == BEAR_OK) {
                if (Tp_GetDCellList(planeDataPtr) != NIL(DummyCell)) {
                    Tp_FreeDCellList(planeDataPtr);
                    /*
                     * In this case we had a floor plan graph, but it was
                     * build without dummy cells so we need to build a new
                     * one with dummy cells.
                     */
                    BuildFloorPlan(cellMasterPtr, TRUE);
                }
            } 
            else {
                PbPrint(EZ_ITALIC_FONT, "Cannot create dummy cells.");
                return (BEAR_ERROR);
            }
        }
    }

/*
 *  THIN BOTTLENECKS COMMENTED OUT
 *
 *  thinBnList = lsCreate();
 *  Mac_CheckAllBottlenecks(planeDataPtr, thinBnList);
 *  if (lsLength(thinBnList) > 0) {
 *      if (bear_BlackBoxFlag == FALSE) {
 *          lsGen       gen;
 *          DrawData    drawData;
 *          Tile        *bnTilePtr;
 *
 *          drawData.winDataPtr = winDataPtr;
 *          drawData.width = MAXWIDTH;
 *
 *          lsForeachItem(thinBnList, gen, bnTilePtr) {
 *              drawData.color = (Tp_IsHorzTile(bnTilePtr) == TRUE) ?
 *                  pbHChannelColor : pbVChannelColor;
 *
 *              Gre_DrawWallSeg(Tp_GetWallSegmentOfBnTile(bnTilePtr),
 *                  &drawData);
 *          }
 *      }
 *      lsDestroy(thinBnList, (void (*)()) NULL);
 *
 *      PbPrint(EZ_ITALIC_FONT, "Bottlenecks too thin.\n");
 *      return (BEAR_ERROR);
 *  }
 *  else {
 *      lsDestroy(thinBnList, (void (*)()) NULL);
 *  }
 *
 * 
 * END
 */

    if (tmFileNamePtr != NIL(char)) {
        if ((filePtr = fopen (tmFileNamePtr, "r")) == NIL(FILE)) {
            sprintf (bear_ErrorMsg, "Cannot open timing file named %s\n", 
                tmFileNamePtr);
            PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
            return (BEAR_ERROR);
        }
        else {
            fclose (filePtr);
        }
        MALLOC(char *, tmpStringPtr,
            (strlen (tmFileNamePtr) + 1) * sizeof (char));
        strcpy (tmpStringPtr, tmFileNamePtr);
        Mac_PutTimingFileNameOfCellMaster(tmpStringPtr, cellMasterPtr);

        if (timingMode == 0) {
            timingMode = 1;
        }
    }
    else if (tmFileNamePtr == NIL(char) && timingMode != 0) {
        if (Mac_GetTimingFileNameOfCellMaster(cellMasterPtr) == NIL(char)) {
            PbPrint(EZ_ITALIC_FONT, "You must specify a timing file\n");
            goto usage;
        }
    }

#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && dialogWinFlag) {
        IVWindow    *IVwin;
        XEvent      theEvent;

        IVwin = iv_InitWindow(ez_Display, BEAR_PROG_NAME, "Global Router", 13);

        (void) iv_AddVar(IVwin, "Congestion Factor",
            (int *) &congestion, IV_DOUBLE, 2, IV_NULL_FUNCTION);
        (void) iv_AddVar(IVwin, "Timing mode flag",
            &timingModeFlag, IV_BOOLEAN, IV_NO_OPT, IV_NULL_FUNCTION);
        (void) iv_AddVar(IVwin, "Okay",
            &routeFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

        (void) iv_AddVar(IVwin, "Exit",
            &exitFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

        (void) iv_MapWindow(IVwin, IV_NO_OPT, IV_NO_OPT, IV_MOUSE);

        while (!routeFlag) {
            XNextEvent(ez_Display, &theEvent);
            (void) iv_HandleEvent(&theEvent);
            Ez_XEventHandler(&theEvent);
            if (exitFlag) {
                iv_FreeWindow(IVwin);
                return (BEAR_ERROR);
            }
        }
        iv_FreeWindow(IVwin);
        Ez_EmptyXEventQue();

        timingMode = timingModeFlag ? 1 : 0;
    }
#endif BEAR_BLACK_BOX

    if (Gr_GlobalRouter(cellMasterPtr, timingMode, congestion) == BEAR_ERROR) {
        PbPrint(EZ_ITALIC_FONT, "Global router failed.\n");
        return (BEAR_ERROR);
    }
    Mac_DeleteAllGraphs(Mac_GetNetListOfCell(cellMasterPtr));
  
    if (checkConnFlag == TRUE) {
        if (Gre_CheckConnForAllNets(cellMasterPtr, warnFlag) == BEAR_ERROR) {
            PbPrint(EZ_ITALIC_FONT,
                "Global router error (disconnected nets).\n");
            return (BEAR_ERROR);
        }
    }

#ifdef USE_WIRE_CODE
    if (Gre_DoPgWiringForAllNets(cellMasterPtr) == BEAR_ERROR) {
        PbPrint(EZ_ITALIC_FONT, "Unable to do p/g wiring for all the nets.\n");
        return (BEAR_ERROR);
    }
#endif USE_WIRE_CODE

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        DrawNet drawNet;

        if ((netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr)) != NIL(Net)) {
            drawNet.cellMasterPtr = cellMasterPtr;
            drawNet.netPtr = netPtr;
            drawNet.whichNet = FPG_CENTER;
            drawNet.netColor = PbGetColorOfNet(FPG_CENTER);
            Gre_DrawNet(winDataPtr, &drawNet);
        } 
    }
#endif BEAR_BLACK_BOX

    for (netPtr = cellMasterPtr->contents.netList;
        netPtr != NIL(Net);
        netPtr = netPtr->nextPtr) {
        numOfPin = 0;
        (void) Mac_ItrPortOfNet(netPtr, CountPin, (ClientData) &numOfPin);
        netPtr->numOfPin = numOfPin;
    }
    viewTypeFlag = BOTH_DONE;

    PbPrint(EZ_NORMAL_FONT, "Global routing is done.\n");
    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT, "global router (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: gr [-cf #] [-tf fileName]\n");
    PbPrint(EZ_NORMAL_FONT, "    [-tm timingMode] [-C] [-CW] [-NDW]\n\n");

    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -cf congestion factor\n");
    PbPrint(EZ_NORMAL_FONT, "    -tf fileName is a timing file\n");
    PbPrint(EZ_NORMAL_FONT, "    -tm timingMode is 0 or 1\n");
    PbPrint(EZ_NORMAL_FONT, "    -C check connectivity of routed nets\n");
    PbPrint(EZ_NORMAL_FONT, "    -CW  check connectivity of routed nets\n");
    PbPrint(EZ_NORMAL_FONT, "         and print warnings\n");
    PbPrint(EZ_NORMAL_FONT, "    -NDW no dialog windows\n");

    return (BEAR_ERROR);
}



/* ARGSUSED */
#ifndef BEAR_BLACK_BOX
void
PbNewGlobalRouterCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;     /* not used */
int         argCount;
char        **argList;
{
    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    FloorPlanGraph  *fpgPtr;
    Net             *netPtr;
    PinInst         *pinPtr;
    PlaneData       *planeDataPtr;
    bool            allFlag = FALSE;
    int             i;
    char            *netNamePtr = NIL(char);

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-all") == 0) {
            allFlag = TRUE;
            continue;
        }
        else {
            if (netNamePtr == NIL(char)) {
                netNamePtr = argList[i];
            }
            else {
                PbPrint(EZ_ITALIC_FONT,
                    "Only one net name may be specified.\n");
                goto usage;
            }
            continue;
        }
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (Mac_IsNumOfCellsZero(cellMasterPtr) == TRUE) {
        PbPrint(EZ_ITALIC_FONT,
            "Current cell master has no cell instances or nets.\n");
        return;
    }
    if (Mac_IsRingRoutingDone(cellMasterPtr) == TRUE) {
        PbPrint(EZ_ITALIC_FONT,
            "Cannot do global routing after ring routing.\n");
        return;
    }

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE) {
        /*
         * In this case we don't have a floor plan graph, and
         * so we build one with dummy cells.
         */
        BuildFloorPlan(cellMasterPtr, TRUE);
    }
    else {
        if (Tp_GetDCellList(planeDataPtr) == NIL(DummyCell)) {
            if (Tp_CreateDCells(planeDataPtr) == BEAR_OK) {
                if (Tp_GetDCellList(planeDataPtr) != NIL(DummyCell)) {
                    Tp_FreeDCellList(planeDataPtr);
                    /*
                     * In this case we had a floor plan graph, but it was
                     * build without dummy cells so we need to build a new
                     * one with dummy cells.
                     */
                    BuildFloorPlan(cellMasterPtr, TRUE);
                }
            } 
            else {
                PbPrint(EZ_ITALIC_FONT, "Cannot create dummy cells.");
                return;
            }
        }
    }

    fpgPtr = Tp_GetFpg(planeDataPtr);

    if ((fpgPtr->mainRoomPtr == NIL(Fpg_Room)) ||
        (fpgPtr->roomList == (lsList) NULL)) {
        if (Fpg_GenRooms(cellMasterPtr) == BEAR_ERROR) {
            PbPrint(EZ_ITALIC_FONT, "Failed to generate rooms.\n");
            return;
        }
    }

    if (! (chipWinPtr->chw_options & PB_SHOW_FPG_OPT)) {
        Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
    }

    if (allFlag == TRUE) {
        if (Ngr_NewGlobalRouter(cellMasterPtr) == BEAR_ERROR) {
            PbPrint(EZ_NORMAL_FONT,
                "Global routing (based on steiner trees) is done.\n");
        }
        return;
    }

    if (netNamePtr != NIL(char)) {
        /* 
         * Use the net name provided to locate the net, and
         * then display it.
         */
        if (Gre_PickNetByName(winDataPtr, netNamePtr, FPG_CENTER,
            &netPtr) == BEAR_ERROR) {
            return;
        }
    }
    else {
        if ((netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr)) == NIL(Net)) {
            /* 
             * Use the mouse to locate the net, and then display it.
             */
            if (Gre_PickNet(winDataPtr, &netPtr, &pinPtr) == BEAR_ERROR) {
                return;
            }
        }
    }

    (void) PbSteinerTreeDialog(winDataPtr, netPtr);

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "new global router\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: ngr [netName] -all\n");

    return;
}
#endif BEAR_BLACK_BOX



/* ARGSUSED */
BearStatus
PbChanPinArrCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;     /* not used */
int         argCount;
char        **argList;
/*
 * BEAR_OK is returned only if the channel pin arrangement is
 * successfully completed.
 */
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    CellMaster      *cellMasterPtr;
    bool            dialogWinFlag = TRUE;
    int             i;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        /* 
         * Don't pop up any dialog windows.  Used when you want to run
         * this command from a script.
         */
        if (strcmp (argList[i], "-NDW") == 0) {
            dialogWinFlag = FALSE;
            continue;
        }

        (void) sprintf (bear_Msg, "unknown option %s\n", argList[i]);
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
        goto usage;
    }

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    }
    else
#endif BEAR_BLACK_BOX
    {
        if (pbCellMasterPtr == NIL(CellMaster)) {
            PbPrint(EZ_ITALIC_FONT, "Must read in a cell master first.\n");
            return (BEAR_ERROR);
        }
        else {
            cellMasterPtr = pbCellMasterPtr;
        }
    }

    if (Mac_IsGlobalRoutingDone(cellMasterPtr) == FALSE) {
        PbPrint(EZ_ITALIC_FONT,
            "This command requires global routing.\n");
        return (BEAR_ERROR);
    }

#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && (dialogWinFlag == TRUE)) {
        (void) PbCpaDialog(winDataPtr);
    }
    else 
#endif BEAR_BLACK_BOX
    {
        if (Cpa_ArrPinsForAllBnTiles(cellMasterPtr, NIL(Cpa_ArrPinData)) !=
            BEAR_OK) {
            PbPrint(EZ_ITALIC_FONT,
                "Unable to do pin arrangement for all the bn tiles.\n");
            return (BEAR_ERROR);
        }
#ifndef BEAR_BLACK_BOX
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
#endif BEAR_BLACK_BOX

        PbPrint(EZ_NORMAL_FONT, "Channel pin arrangement is done.\n");
    }

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT,
        "channel pin arrangement (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: cpa [-options ...]\n");
    PbPrint(EZ_NORMAL_FONT, "\nwhere options include:\n");
    PbPrint(EZ_NORMAL_FONT,
        "    -NDW              no dialog windows\n");

    return (BEAR_ERROR);
}



static BearStatus
CountPin(portPtr,numOfPin)
PortInst    *portPtr;
int         *numOfPin;
{
    int count;

    (void) Mac_NumOfPinsOfPort(portPtr, &count);
    *numOfPin = *numOfPin + count;

    return (BEAR_CONTINUE_SEARCH);
}


bool hackCalled = FALSE;

#ifndef BEAR_BLACK_BOX
int setDefHackFunc()
{
    hackCalled = TRUE;
    return IV_OK;
}
#endif BEAR_BLACK_BOX



#ifndef BEAR_BLACK_BOX
#ifdef USE_WIRE_CODE
/* ARGSUSED */
void
PbWireCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    FILE        *filePtr;
    CellMaster  *cellMasterPtr;
    ChipWin     *chipWinPtr;
    Net         *netPtr;
    Net         *curNetPtr;
    PinInst     *pinPtr;
    Window      win;
    double      maxVoltDrop, maxCurrentReq;
    int         i;
    char        *netNamePtr = NIL(char);
    
    /* Used for IV */
    IVWindow    *IVwin;
    XEvent      theEvent;
    bool        singlePortFlag = FALSE;
    bool        allPortsFlag = FALSE;
    bool        calcFlag = FALSE;
    bool        draw1Flag = FALSE;
    bool        draw2Flag = FALSE;
    bool        draw3Flag = FALSE;
#ifdef DEBUG
    bool        ivLabelFlag, labelFlag;
    bool        printFlag = FALSE;
    bool        randomPortFlag = FALSE;
#endif DEBUG
    bool        exitFlag = FALSE;
    bool        loopFlag, connFlag;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-n") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            if (netNamePtr == NIL(char)) {
                netNamePtr = argList[i];
            }
            else {
                PbPrint(EZ_ITALIC_FONT,
                    "Only one net name may be specified.\n");
                goto usage;
            }
            continue;
        }
        goto usage;
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (Mac_IsRingRoutingDone(cellMasterPtr) == TRUE) {
        PbPrint(EZ_ITALIC_FONT,
            "You may not wire a net after ring routing is completed.\n");
        return;
    }

    if (netNamePtr != NIL(char)) {
        /* 
         * Use the net name they provided to locate the net, and
         * then display it.
         */
        if (Gre_PickNetByName(winDataPtr, netNamePtr, FPG_CENTER, &netPtr) ==
            BEAR_ERROR) {
            return;
        }
    }
    else {
        if ((netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr)) == NIL(Net)) {
            /* 
             * Use the mouse to locate the net, and then display it.
             */
            if (Gre_PickNet(winDataPtr, &netPtr, &pinPtr) == BEAR_ERROR) {
                return;
            }
        }
    }

/*
    if (Mac_GetUserGraphOfNet(netPtr) == (Gt_Graph) NULL) {
        PbPrint(EZ_ITALIC_FONT, "This net has no graph.\n");
        return;
    }
    else {
        if (Mac_CheckUserGraph(netPtr, &loopFlag, &connFlag) == BEAR_ERROR) {
            return;
        }
        if (connFlag == FALSE) {
            PbPrint(EZ_ITALIC_FONT, "This net is not connected.\n");
            return;
        }
    }
 */

#ifdef DEBUG
    wireDebug = FALSE;

    maxVoltDrop = 2.2;
    maxCurrentReq = 0.1;
    Gre_InitPortsOfNet(netPtr, maxVoltDrop, maxCurrentReq);

    ivLabelFlag = labelFlag = FALSE;
    Bear_UnsetOption(chipWinPtr->chw_options, PB_LABEL_CENTER_NET_OPT);
#else
    maxVoltDrop = 0.0;
    maxCurrentReq = 0.0;
    Gre_InitPortsOfNet(netPtr, maxVoltDrop, maxCurrentReq);
#endif DEBUG

ivstart: /* Ugly, I know but the quickest solution for the time being 
          * and in all probability the far future as well.
          */

    netNamePtr = Mac_GetNameOfNet(netPtr);
    IVwin = iv_InitWindow(ez_Display, BEAR_PROG_NAME, "Wire Net", 13);

    (void) iv_AddVar(IVwin, "Net Name",
        (int *) &netNamePtr, IV_STRING, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_SetEnableFlag(IVwin, (int *) &netNamePtr, 0);

    (void) iv_AddVar(IVwin, "Default Max Voltage Drop",
        (int *) &maxVoltDrop, IV_DOUBLE, 3, setDefHackFunc);
    (void) iv_AddVar(IVwin, "Default Current Requirement",
        (int *) &maxCurrentReq, IV_DOUBLE, 3, setDefHackFunc);
    (void) iv_AddVar(IVwin, "Specify Data for Single Port",
        &singlePortFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Specify Data for All Ports",
        &allPortsFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Calculate Wire Widths",
        &calcFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

#ifdef DEBUG
    (void) iv_AddVar(IVwin, "Draw check net (debug)",
        &draw1Flag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Draw place net (debug)",
        &draw2Flag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Draw pg net (debug)",
        &draw3Flag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

    (void) iv_AddVar(IVwin, "Pick src port at random (debug)",
        &randomPortFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Show Numbers (debug)",
        &ivLabelFlag, IV_BOOLEAN, IV_NO_OPT, IV_NULL_FUNCTION);
    (void) iv_AddVar(IVwin, "Print Wire Data (debug)",
        &printFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
#endif DEBUG

    (void) iv_AddVar(IVwin, "Exit Wire Net",
        &exitFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

    (void) iv_MapWindow(IVwin, IV_NO_OPT, IV_NO_OPT, IV_MOUSE);

    while (!exitFlag) {
        XNextEvent(ez_Display, &theEvent);
        (void) iv_HandleEvent(&theEvent);
        Ez_XEventHandler(&theEvent);
        if (hackCalled) {
            Gre_InitPortsOfNet(netPtr, maxVoltDrop, maxCurrentReq);
            hackCalled = FALSE;
        }

        if (singlePortFlag == TRUE) {
            iv_FreeWindow(IVwin);
            AlterPortData(winDataPtr);
            singlePortFlag = FALSE;
            goto ivstart;
        }
        else if (allPortsFlag == TRUE) {
            iv_FreeWindow(IVwin);
            (void) Mac_ItrPortOfNet(netPtr, EnterPortData, (ClientData) NULL);
            allPortsFlag = FALSE;
            goto ivstart;
        }
        else if (calcFlag == TRUE) {
            (void) Gre_DoPgWiringForOneNet(cellMasterPtr, netPtr);
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            calcFlag = FALSE;
        }
#ifdef DEBUG
        else if (draw1Flag == TRUE) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            if (Gre_BuildAutoGraphOfNet(cellMasterPtr, netPtr,
                FALSE, FALSE, FALSE) == BEAR_ERROR) {
                PbPrint(EZ_ITALIC_FONT, "couldn't build check graph\n");
            }
            else {
                DrawGraph drawGraph;

                drawGraph.graph = Mac_GetAutoGraphOfNet(netPtr);
                drawGraph.planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
                drawGraph.graphColor = ez_HighLightColor;
                drawGraph.dotFlag = FALSE;
                (void) Gre_DrawGraph(winDataPtr, &drawGraph);
            }
            draw1Flag = FALSE;
        }
        else if (draw2Flag == TRUE) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            if (Gre_BuildAutoGraphOfNet(cellMasterPtr, netPtr,
                FALSE, TRUE, FALSE) == BEAR_ERROR) {
                PbPrint(EZ_ITALIC_FONT, "couldn't build place graph\n");
            }
            else {
                DrawGraph drawGraph;

                drawGraph.graph = Mac_GetAutoGraphOfNet(netPtr);
                drawGraph.planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
                drawGraph.graphColor = ez_HighLightColor;
                drawGraph.dotFlag = FALSE;
                (void) Gre_DrawGraph(winDataPtr, &drawGraph);
            }
            draw2Flag = FALSE;
        }
        else if (draw3Flag == TRUE) {
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            if (Gre_BuildAutoGraphOfNet(cellMasterPtr, netPtr,
                FALSE, FALSE, TRUE) == BEAR_ERROR) {
                PbPrint(EZ_ITALIC_FONT, "couldn't build pg graph\n");
            }
            else {
                DrawGraph drawGraph;

                drawGraph.graph = Mac_GetAutoGraphOfNet(netPtr);
                drawGraph.planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
                drawGraph.graphColor = ez_HighLightColor;
                drawGraph.dotFlag = FALSE;
                (void) Gre_DrawGraph(winDataPtr, &drawGraph);
            }
            draw3Flag = FALSE;
        }
        else if (randomPortFlag == TRUE) {
            (void) Mac_ItrPortOfNet(netPtr, PickOneSrcPort, (ClientData) NULL);
            randomPortFlag = FALSE;
        }
        else if (ivLabelFlag != labelFlag) {
            if (ivLabelFlag == TRUE) {
                if (netPtr->wirePtr == NIL(Pgr_Wire)) {
                    PbPrint(EZ_ITALIC_FONT,
                        "Do the wire calculations first.\n");
                    ivLabelFlag = FALSE;
                    (void) iv_UpdateVar(IVwin, &ivLabelFlag);
                }
                else {
                    PrintPgWireDataOfNet(winDataPtr, netPtr, TRUE);
                }
            }
            else {
                Bear_UnsetOption(chipWinPtr->chw_options,
                    PB_LABEL_CENTER_NET_OPT);
                (void) PbDrawChipRegion(winDataPtr, (ClientData) NULL);
            }
            labelFlag = ivLabelFlag;
        }
        else if (printFlag == TRUE) {
            if (netPtr->wirePtr == NIL(Pgr_Wire)) {
                PbPrint(EZ_ITALIC_FONT, "Do the wire calculations first.\n");
            }
            else {
                PrintPgWireDataOfNet(winDataPtr, netPtr, FALSE);
            }
            printFlag = FALSE;
        }
#endif DEBUG
/*
        case 'T': 
            if (wireDebug) {
                wireDebug = FALSE;
                PbPrint(EZ_BOLD_FONT, "wire debugging is off\n");
            } else {
                wireDebug = TRUE;
                PbPrint(EZ_BOLD_FONT, "wire debugging is on\n");
            }
            break;
*/
    }
    iv_FreeWindow(IVwin);
    return;

/* NOTREACHED */
usage:
    PbPrint(EZ_NORMAL_FONT, "do power/ground wiring calculations\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: wn [-n netName]\n");
}
#endif USE_WIRE_CODE
#endif BEAR_BLACK_BOX



#ifndef BEAR_BLACK_BOX
/* ARGSUSED */
void
PbCreateNetCmd(winDataPtr, screenPtr, argCount, argList)
WindowData      *winDataPtr;
Point           *screenPtr;
int             argCount;
char            **argList;
{
    CellMaster       *parentCellPtr;
    ChipWin          *chipWinPtr;
    Net              *netPtr;
    PortInst         *portInstPtr;
    PinInst          *pinInstPtr;
    Window           win;
    char             netName[80];
    char             answer[80];   
    bool             done;

    sprintf(netName, "%d", Mac_GenerateId());
    chipWinPtr = PbGetChipWin(winDataPtr);
    parentCellPtr = PbGetCellMasterOfCHW(chipWinPtr);

    /* Create new net */

    if (Mac_CreateNet(parentCellPtr, netName, &netPtr) == BEAR_ERROR) {
        PbPrint(EZ_ITALIC_FONT, "can not create net.\n");
        return;
    }

    /* Insert a port to a newly created net until done == TRUE */

    do {
    PbPrint(EZ_NORMAL_FONT, "insert a port to a net?(y,n) ");
    } while (Ez_GetLine(&win, screenPtr, answer, 80, &pbCmdLoop) != EZ_OK);
    if (answer[0] == 'y')
    {
        done = FALSE;
        while (done == FALSE) {
            PbPrint(EZ_NORMAL_FONT, "Pick a port.\n");
        if (Gre_GetPin(winDataPtr, (BearString) NULL, (BearString) NULL,
            ez_HighLightColor, &pinInstPtr) != BEAR_OK) {
            PbPrint(EZ_ITALIC_FONT, "can not insert port to a net.\n");
            return;
        }

        portInstPtr = Mac_GetPortOfPin(pinInstPtr);
        if (Mac_GetNetOfPin(pinInstPtr) != NIL(Net)) {
            PbPrint(EZ_ITALIC_FONT, "this port already belongs to a net.\n");
            do {
                PbPrint(EZ_NORMAL_FONT,
                    "delete this port from the previous net?(y,n) ");
            } while (Ez_GetLine(&win, screenPtr, answer, 80, &pbCmdLoop) != 
                EZ_OK);
            if (answer[0] == 'y')
            {
                if (Mac_DeletePortInstOfNet(portInstPtr) == BEAR_ERROR)
                {
                    PbPrint(EZ_ITALIC_FONT, "can not delete this port.\n");
                    }
                }
        }
        if (Mac_InsertPortInstOfNet(netPtr, portInstPtr) == BEAR_ERROR)
        {
            PbPrint(EZ_ITALIC_FONT, "can not insert port to a net\n");
            return;
        }
            do {
        PbPrint(EZ_NORMAL_FONT, "insert more port to a net? (y,n) ");
            } while (Ez_GetLine(&win, screenPtr, answer, 80, &pbCmdLoop) != 
            EZ_OK);
            if (answer[0] == 'n')
        {
        done = TRUE;
            }
        else
        {
        done = FALSE;
            }
        }
    }
}
#endif BEAR_BLACK_BOX



/* ARGSUSED */
BearStatus
PbBuildGraphCmd(winDataPtr, screenPtr, argCount, argList)
WindowData    *winDataPtr;
Point         *screenPtr;
int           argCount;
char          **argList;
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    CellMaster      *cellMasterPtr;
    Net             *netPtr;
    PinInst         *pinPtr;
    int             i;
    char            *netNamePtr = NIL(char);
    bool            oneConnFlag = FALSE;
    bool            allConnFlag = FALSE;
    bool            warnFlag;

    /*
     * parse arguements
     *
     * Note that only one of options -C, -CW, and -n may be
     * specified.
     */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-C") == 0) {
            allConnFlag = TRUE;
            warnFlag = FALSE;
            break;
        }

        if (strcmp (argList[i], "-CW") == 0) {
            allConnFlag = TRUE;
            warnFlag = TRUE;
            break;
        }

        if (strcmp (argList[i], "-n") == 0) {
            if (bear_BlackBoxFlag == TRUE) {
                PbPrint(EZ_ITALIC_FONT,
                    "-n option not used in black box mode.\n");
                return (BEAR_ERROR);
            }
            oneConnFlag = TRUE;
            continue;
        }
        else {
            if (netNamePtr == NIL(char)) {
                netNamePtr = argList[i];
            }
            else {
                PbPrint(EZ_ITALIC_FONT,
                    "Only one net name may be specified.\n");
                goto usage;
            }
            continue;
        }
    }

    if ((allConnFlag == FALSE) && (oneConnFlag == FALSE)) {
        PbPrint(EZ_ITALIC_FONT, "Choose an option.\n\n");
        goto usage;
    }

#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
        chipWinPtr = PbGetChipWin(winDataPtr);
        cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    }
    else
#endif BEAR_BLACK_BOX
    {
        if (pbCellMasterPtr == NIL(CellMaster)) {
            PbPrint(EZ_ITALIC_FONT, "Must read in a cell master first.\n");
            return (BEAR_ERROR);
        }
        else {
            cellMasterPtr = pbCellMasterPtr;
        }
    }

    if (allConnFlag) {
        if (Gre_CheckConnForAllNets(cellMasterPtr, warnFlag) == BEAR_OK) {
            PbPrint(EZ_NORMAL_FONT, "Every net is connected.\n");
            return (BEAR_OK);
        }
        else {
            return (BEAR_ERROR);
        }
    }

#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && (oneConnFlag == TRUE)) {
        if (netNamePtr != NIL(char)) {
            /* 
             * Use the net name provided to locate the net, and
             * then display it.
             */
            if (Gre_PickNetByName(winDataPtr, netNamePtr, FPG_CENTER,
                &netPtr) == BEAR_ERROR) {
                return (BEAR_ERROR);
            }
        }
        else {
            if ((netPtr = PbGetNetOfCHW(FPG_CENTER, chipWinPtr)) == NIL(Net)) {
                /* 
                 * Use the mouse to locate the net, and then display it.
                 */
                if (Gre_PickNet(winDataPtr, &netPtr, &pinPtr) == BEAR_ERROR) {
                    return (BEAR_ERROR);
                }
            }
        }

        return (Gre_CheckConnForOneNet(cellMasterPtr, netPtr, TRUE));
    }
#endif BEAR_BLACK_BOX

    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT,
        "build graph of net (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: bg [-C] [-CW] [-n [netName]]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -C  : check connectivity of all the nets\n");
    PbPrint(EZ_NORMAL_FONT, "    -CW  : check connectivity of all the nets\n");
    PbPrint(EZ_NORMAL_FONT, "           and print warnings as you go\n");
    PbPrint(EZ_NORMAL_FONT,
        "    -n [netName] : check connectivity of one net\n");

    return (BEAR_ERROR);
}



