
/* pbRCellCmds.c --
 *
 *     Wei-Ming Dai
 *     M. Kobayashi
 *     Sherry Solden - empty cells, bnLines
 *     (Advising Professor - E. S. Kuh)
 *
 * Commands that let you choose the routing cells.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifdef USE_DR_CODE

#include "bearCopyRight.h"
#include <stdio.h>
#include <math.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "grEditor.h"
#include "tileMapping.h"
#include "routeDB.h"
#include "pbDetour.h"
#include "polarBear.h"


#define OK         1
#define SWITCHED   2
#define ERROR     -1

#define BAD_RETURN \
{ \
    Ez_RingBell(); \
    return; \
}

/* ---------- Local Procedures ---------- */
static int SortPoints();


static int
SortPoints(point1Ptr, point2Ptr)
Point *point1Ptr;
Point *point2Ptr;
/*
 * Sort point from top to bottom, left to right.
 */
{
    register int x1, y1, x2, y2;
    
    x1 = point1Ptr->p_x;
    y1 = point1Ptr->p_y;
    x2 = point2Ptr->p_x;
    y2 = point2Ptr->p_y;
    if ((x2 < x1) || (y2 > y1)) {
        return (SWITCHED);
    }
    else if ((x1 == x2) && (y1 == y2)) {
        return (ERROR);
    }
    return (OK);
}



/* ARGSUSED */
void
PbDefChannelCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    int             isSwitched;
    BearStatus      bearStatus;
    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    Point           *point1Ptr; 
    Point           *point2Ptr;
    Point           *tmpPointPtr;
    Tile            *horzDomTile1Ptr; 
    Tile            *vertDomTile1Ptr;
    Tile            *horzDomTile2Ptr; 
    Tile            *vertDomTile2Ptr;
    Tile            *tmpTilePtr;
    WallJunction    *junct1Ptr; 
    WallJunction    *junct2Ptr;
    WallJunction    *tmpJunctPtr;
    int             i;
    int             delta;
    bool            debug=FALSE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) {
            debug = TRUE;
            continue;
        }
        else {
            goto usage;
        }
    }
    
    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (!Mac_IsGlobalRoutingDone(cellMasterPtr)) 
    {
        PbPrint(EZ_ITALIC_FONT, "Complete global routing and try again.\n");
        return;
    }
    PbHighlightAllChannels(winDataPtr);

    PbPrint(EZ_NORMAL_FONT, "Pick junction number one.\n");
    if (Gre_PickJunct(&horzDomTile1Ptr, &vertDomTile1Ptr, &junct1Ptr) ==
        BEAR_ERROR) 
    {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
    }
    point1Ptr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct1Ptr);

    PbPrint(EZ_NORMAL_FONT, "Pick junction number two.\n");
    if (Gre_PickJunct(&horzDomTile2Ptr, &vertDomTile2Ptr, &junct2Ptr) ==
        BEAR_ERROR) 
    {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
    }
    point2Ptr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct2Ptr);

    /* Guarantee that point1 is TOP or LEFT Junction */
    isSwitched = SortPoints(point1Ptr, point2Ptr);
    if (isSwitched == ERROR) 
    {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        BAD_RETURN;
    }
    if (isSwitched == SWITCHED)
    {
       tmpPointPtr = point1Ptr;
       point1Ptr = point2Ptr;
       point2Ptr = tmpPointPtr;

       tmpJunctPtr = junct1Ptr;
       junct1Ptr = junct2Ptr;
       junct2Ptr = tmpJunctPtr;

       tmpTilePtr = horzDomTile1Ptr;
       horzDomTile1Ptr = horzDomTile2Ptr;
       horzDomTile2Ptr = tmpTilePtr;

       tmpTilePtr = vertDomTile1Ptr;
       vertDomTile1Ptr = vertDomTile2Ptr;
       vertDomTile2Ptr = tmpTilePtr;
    }

    /*
     * Make sure that the wall junctions they picked are along the same
     * line and that they do not have any cells between them !!
     */
    if (point1Ptr->p_x == point2Ptr->p_x) 
    {
        /* The channel is vertical. */
        if (vertDomTile1Ptr != vertDomTile2Ptr) 
        {
            PbPrint(EZ_ITALIC_FONT,
                "There is a cell crossing the vertical channel.\n");
            BAD_RETURN;
        }
    } 
    else if (point1Ptr->p_y == point2Ptr->p_y) 
    {
        /* The channel is horizontal. */
        if (horzDomTile1Ptr != horzDomTile2Ptr) 
        {
            PbPrint(EZ_ITALIC_FONT,
                "There is a cell crossing the horizontal channel.\n");
            BAD_RETURN;
        }
    } 
    else 
    {
        PbPrint(EZ_ITALIC_FONT,    
            "It is not a valid definition for a straight channel.\n");
        BAD_RETURN;
    }

    /* ROUTE CHANNEL */
    bearStatus = PbRouteChannel(cellMasterPtr, junct1Ptr, junct2Ptr, 
        TRUE, TRUE, TRUE, TRUE, &delta, debug);

    if (bearStatus == BEAR_OK)
    {
        PbPrint(EZ_NORMAL_FONT, "Channel successfully routed.\n");
    }
    else
    {
        PbPrint(EZ_NORMAL_FONT, "Channel route aborted.\n");
    }

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "define straight channel \n");
    PbPrint(EZ_NORMAL_FONT, "Usage: dch [-d]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    debug mode\n");
}

#endif /* USE_DR_CODE */


