
/* pbRingCmds.c --
 *
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#ifdef USE_DR_CODE

#include "bearCopyRight.h"
#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "grEditor.h"
#include "globalSpacer.h"
#include "geoChDecomposer.h"
#include "tileMapping.h"
#include "ringRouter.h"
#include "polarBear.h"
#ifndef BEAR_BLACK_BOX
#include "bearIV.h"
#endif BEAR_BLACK_BOX


/* ---------- Local Procedures ---------- */
static BearStatus   OutputGPinOnOutBnd(), OutputGPinOnInBnd();
static int          InsertHorzRectList(), InsertVertRectList();
static void         FindData();


/* ---------- Local Global Variables ---------- */
static PlaneData *gPlaneDataPtr;


static void
FindData(cellMasterPtr, hRect1Ptr, vRect1Ptr, hRect2Ptr, vRect2Ptr,
    urJunctPtrPtr)

CellMaster      *cellMasterPtr;
Rect            *hRect1Ptr, *vRect1Ptr, *hRect2Ptr, *vRect2Ptr;
WallJunction    **urJunctPtrPtr;    /* upper right wall junction */
{
    FloorPlanGraph   *fpgPtr;
    Plane            *hPlanePtr;
    Plane            *vPlanePtr;
    PlaneData        *planeDataPtr;
    Point            *hPointPtr;
    Point            vPoint;
    Tile             *hDomTilePtr;
    Tile             *vDomTilePtr;
    WallJunction     *junctPtr;
    WallJunction     *nextJunctPtr;
    WallJunction     *llJunctPtr;      /* lower left wall junction */

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
    fpgPtr = Tp_GetFpg(planeDataPtr);
    llJunctPtr = Fpg_GetLLJunctOfFpg(fpgPtr);

    hPlanePtr = Tp_GetHorzPlane(planeDataPtr);
    vPlanePtr = Tp_GetVertPlane(planeDataPtr);
    
    hPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, llJunctPtr);
    hDomTilePtr = TiSrPointNoHint(hPlanePtr, hPointPtr);
    Tp_HorzTileToRect(planeDataPtr, hDomTilePtr, hRect1Ptr);

    Tp_HPointToVPoint(hPointPtr, &vPoint);
    vDomTilePtr = TiSrPointNoHint(vPlanePtr, &vPoint);
    Tp_VertTileToRect(planeDataPtr, vDomTilePtr, vRect1Ptr);
    
    nextJunctPtr = Fpg_GetEastJunctOfWallJunct(llJunctPtr);
    while (nextJunctPtr != NIL(WallJunction))
    {
        junctPtr = nextJunctPtr;
        nextJunctPtr = Fpg_GetEastJunctOfWallJunct(junctPtr);
    }
    nextJunctPtr = Fpg_GetNorthJunctOfWallJunct(junctPtr);
    while (nextJunctPtr != NIL(WallJunction))
    {
        junctPtr = nextJunctPtr;
        nextJunctPtr = Fpg_GetNorthJunctOfWallJunct(junctPtr);
    }
    (*urJunctPtrPtr) = junctPtr;

    hPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, (*urJunctPtrPtr));
    hDomTilePtr = TiSrPointNoHint(hPlanePtr, hPointPtr);
    Tp_HorzTileToRect(planeDataPtr, hDomTilePtr, hRect2Ptr);

    Tp_HPointToVPoint(hPointPtr, &vPoint);
    vDomTilePtr = TiSrPointNoHint(vPlanePtr, &vPoint);
    Tp_VertTileToRect(planeDataPtr, vDomTilePtr, vRect2Ptr);
}



static int
InsertHorzRectList(tilePtr,rectListPtr)
Tile       *tilePtr;
LinkedRect **rectListPtr;
{
    LinkedRect   *linkedRectPtr;

    if (Tp_IsSpaceTile(tilePtr))
    {
        MALLOC(LinkedRect *,linkedRectPtr,sizeof (LinkedRect));
        Tp_HorzTileToRect(gPlaneDataPtr,tilePtr,&linkedRectPtr->r_r);
        linkedRectPtr->r_next = *rectListPtr;
        *rectListPtr = linkedRectPtr;
    }
    return (MAGIC_CONTINUE_SEARCH);
}



static int
InsertVertRectList(tilePtr,rectListPtr)
Tile       *tilePtr;
LinkedRect **rectListPtr;
{
    LinkedRect   *linkedRectPtr;

    if (Tp_IsSpaceTile(tilePtr))
    {
        MALLOC(LinkedRect *,linkedRectPtr,sizeof (LinkedRect));
        Tp_VertTileToRect(gPlaneDataPtr,tilePtr,&linkedRectPtr->r_r);
        linkedRectPtr->r_next = *rectListPtr;
        *rectListPtr = linkedRectPtr;
    }
    return (MAGIC_CONTINUE_SEARCH);
}



static BearStatus
OutputGPinOnOutBnd(gPinPtr,fp)
GenericPin  *gPinPtr;
FILE        *fp;
{
    PinInst      *pinInstPtr;
    RoutePin     *rPinPtr;
    Point        absPos;
    BearId       extNetId;
    BearId       netId;
    BearString   netName;
    NetKind      netKind;
    PinDirection pinDirection;
    int          sigType;
    int          routDirection;
    float        *widthVect;
    int          layer;

    if (Mac_GetTypeOfGPin(gPinPtr) == PIN_INST)
    {
    pinInstPtr   = Mac_GetPinInstOfGPin(gPinPtr);
    Mac_GetAbsPosOfPin(pinInstPtr,&absPos);
    extNetId     = Mac_GetExtNetIdOfPin(pinInstPtr);
    netId        = Mac_GetIdOfNet(Mac_GetNetOfPin(pinInstPtr));
    netName      = Mac_GetNameOfNet(Mac_GetNetOfPin(pinInstPtr));
    netKind      = Mac_GetKindOfNet(Mac_GetNetOfPin(pinInstPtr));
    pinDirection = Mac_GetDirectionOfPin(pinInstPtr);
    widthVect    = Mac_GetWidthVectOfPin(pinInstPtr);
    /*
    layer        = METAL1_LAYER;
    */
    layer        = 12;
    }
    else
    {
    rPinPtr      = Mac_GetRoutePinOfGPin(gPinPtr);
    Mac_GetAbsPosOfRPin(rPinPtr,&absPos);
    extNetId     = Mac_GetExtNetIdOfRPin(rPinPtr);
    netId        = Mac_GetIdOfNet(Mac_GetNetPtrOfRPin(rPinPtr));
    netName      = Mac_GetNameOfNet(Mac_GetNetPtrOfRPin(rPinPtr));
    netKind      = Mac_GetKindOfNet(Mac_GetNetPtrOfRPin(rPinPtr));
    pinDirection = Mac_GetDirectionOfRPin(rPinPtr);
    widthVect    = Mac_GetWidthVectOfRPin(rPinPtr);
    layer        = Mac_GetLayerOfRPin(rPinPtr);
    }
    switch (netKind)
    {
        case POWER_NET:
        sigType = 2;
        extNetId = netId;
        break;
        case GROUND_NET:
        sigType = 1;
        extNetId = netId;
        break;
    default:
        sigType = 0;
    }
    switch (pinDirection)
    {
    case NORTH_PIN:
        routDirection = 3;
        break;
    case EAST_PIN :
        routDirection = 2;
        break;
    case SOUTH_PIN:
        routDirection = 1;
        break;
    case WEST_PIN :
        routDirection = 0;
    }
    if (sigType == 0)
    {
    if (widthVect[METAL1_LAYER] >= MAXWIDTH)
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer);
    }
    else
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d %d %d\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer,
        (int)widthVect[METAL2_LAYER]/2,
        (int)widthVect[METAL1_LAYER]/2);
    }
    }
    else
    {
    if (widthVect[METAL1_LAYER] >= MAXWIDTH)
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d 5 5\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer);
    }
    else
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d %d %d\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer,
        (int)widthVect[METAL2_LAYER]/2,
        (int)widthVect[METAL1_LAYER]/2);
    }
    }
    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
OutputGPinOnInBnd(gPinPtr,fp)
GenericPin  *gPinPtr;
FILE        *fp;
{
    PinInst      *pinInstPtr;
    RoutePin     *rPinPtr;
    Point        absPos;
    BearId       extNetId;
    BearId       netId;
    BearString   netName;
    NetKind      netKind;
    PinDirection pinDirection;
    int          sigType;
    int          routDirection;
    float        *widthVect;
    int          layer;

    if (Mac_GetTypeOfGPin(gPinPtr) == PIN_INST)
    {
    pinInstPtr   = Mac_GetPinInstOfGPin(gPinPtr);
    Mac_GetAbsPosOfPin(pinInstPtr,&absPos);
    extNetId     = Mac_GetExtNetIdOfPin(pinInstPtr);
    netId        = Mac_GetIdOfNet(Mac_GetNetOfPin(pinInstPtr));
    netKind      = Mac_GetKindOfNet(Mac_GetNetOfPin(pinInstPtr));
    netName      = Mac_GetNameOfNet(Mac_GetNetOfPin(pinInstPtr));
    pinDirection = Mac_GetDirectionOfPin(pinInstPtr);
    widthVect    = Mac_GetWidthVectOfPin(pinInstPtr);
    layer        = Mac_GetLayerOfPin(pinInstPtr);
    }
    else
    {
    rPinPtr      = Mac_GetRoutePinOfGPin(gPinPtr);
    Mac_GetAbsPosOfRPin(rPinPtr,&absPos);
        extNetId     = Mac_GetExtNetIdOfRPin(rPinPtr);
    netId        = Mac_GetIdOfNet(Mac_GetNetPtrOfRPin(rPinPtr));
    netName      = Mac_GetNameOfNet(Mac_GetNetPtrOfRPin(rPinPtr));
    netKind      = Mac_GetKindOfNet(Mac_GetNetPtrOfRPin(rPinPtr));
    pinDirection = Mac_GetDirectionOfRPin(rPinPtr);
    widthVect    = Mac_GetWidthVectOfRPin(rPinPtr);
    layer        = Mac_GetLayerOfRPin(rPinPtr);
    if (layer > 50 || layer <= 0)
    {
        layer = 2;
    }
    }
    switch (netKind)
    {
        case POWER_NET:
        sigType = 2;
        extNetId = netId;
        break;
        case GROUND_NET:
        sigType = 1;
        extNetId = netId;
        break;
    default:
        sigType = 0;
    }
    switch (pinDirection)
    {
    case NORTH_PIN:
        routDirection = 3;
            break;
    case EAST_PIN :
        routDirection = 2;
            break;
    case SOUTH_PIN:
        routDirection = 1;
            break;
    case WEST_PIN :
        routDirection = 0;
    }
    if (sigType == 0)
    {
    if (widthVect[METAL1_LAYER] >= MAXWIDTH)
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer);
    }
    else
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d %d %d\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer,
        (int)widthVect[METAL2_LAYER]/2,
        (int)widthVect[METAL1_LAYER]/2);
    }
    }
    else
    {
    if (widthVect[METAL1_LAYER] >= MAXWIDTH)
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d 5 5\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer);
    }
    else
    {
        fprintf(fp,"%6d %6d %6d %s %d %d %d %d %d\n",
        absPos.p_x,absPos.p_y,extNetId,netName,routDirection,
        sigType,layer,
        (int)widthVect[METAL2_LAYER]/2,
        (int)widthVect[METAL1_LAYER]/2);
    }
    }
    return (BEAR_CONTINUE_SEARCH);
}



/* ARGSUSED */
BearStatus
PbRingRouterCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;     /* not used */
int         argCount;
char        **argList;
{
#ifndef BEAR_BLACK_BOX
    ChipWin         *chipWinPtr;
#endif BEAR_BLACK_BOX

    CellMaster   *coreMasterPtr;
    PlaneData    *planeDataPtr;
    
    FILE          *filePtr;
    BearString    cellNamePtr;
    Point         *originPtr;
    Rect          *bBoxPtr;
    Rect          hRect1;
    Rect          hRect2;
    Rect          vRect1;
    Rect          vRect2;
    Ring          *ringPtr;
    Tile          *bnTilePtr;
    WallJunction  *llJunctPtr;
    WallJunction  *junctPtr;
    WallJunction  *urJunctPtr;
    WallSegment   *wallSegPtr;
    int           adjust[NumSide];
    int           newDensity;
    int           i;
    char          *outputFileName;
    char          *fileNamePtr;

#ifndef BEAR_BLACK_BOX
    IVWindow      *ivWinPtr;
    XEvent        theEvent;
#endif BEAR_BLACK_BOX
    bool          doRoute = FALSE, exitFlag = FALSE;
    bool          viaReduceFlag = TRUE;
    bool          dialogWinFlag = TRUE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * Print out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        /*
         * Set reduce via's mode.
         */
        if (strcmp (argList[i], "-via") == 0)
        {
            viaReduceFlag = TRUE;
            continue;
        }

        /* 
         * Don't pop up any dialog windows.  Used when you want to run
         * this command from a script.
         */
        if (strcmp (argList[i], "-NDW") == 0) {
            dialogWinFlag = FALSE;
            continue;
        }
        goto usage;
    }

    MALLOC(char *, outputFileName, 25 * sizeof (char));    
    strcpy(outputFileName, "bear.ring.cif");
    
#ifndef BEAR_BLACK_BOX
    if ((bear_BlackBoxFlag == FALSE) && dialogWinFlag) {
        ivWinPtr = iv_InitWindow(ez_Display, BEAR_PROG_NAME, "Ring Router", 20);
        (void) iv_AddVar(ivWinPtr, "Via Reduction", 
            &viaReduceFlag, IV_BOOLEAN, IV_NO_OPT, IV_NULL_FUNCTION);
        (void) iv_AddVar(ivWinPtr, "Output Filename",
            (int *) &outputFileName, IV_STRING, IV_NO_OPT, IV_NULL_FUNCTION);
        (void) iv_AddVar(ivWinPtr, "Okay",
            &doRoute, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);
        (void) iv_AddVar(ivWinPtr, "Exit",
            &exitFlag, IV_TOGGLE, IV_NO_OPT, IV_NULL_FUNCTION);

        (void) iv_MapWindow(ivWinPtr, IV_NO_OPT, IV_NO_OPT, IV_MOUSE);

        while (!doRoute) {
            XNextEvent(ez_Display, &theEvent);
            (void) iv_HandleEvent(&theEvent);
            Ez_XEventHandler(&theEvent);
            if (exitFlag) {
                FREE((char *) outputFileName);
                iv_FreeWindow(ivWinPtr);
                return (BEAR_ERROR);
            }
        }
        iv_FreeWindow(ivWinPtr);
        Ez_EmptyXEventQue();
    }
    
    if (bear_BlackBoxFlag == FALSE) {
        chipWinPtr = PbGetChipWin(winDataPtr);
        coreMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    }
    else 
#endif BEAR_BLACK_BOX
    {
        if (pbCellMasterPtr == NIL(CellMaster)) {
            PbPrint(EZ_ITALIC_FONT, "Must read in a cell master first.\n");
            return (BEAR_ERROR);
        }
        else {
            coreMasterPtr = pbCellMasterPtr;
        }
    }

    if (Mac_IsGlobalRoutingDone(coreMasterPtr) == FALSE) {
        PbPrint(EZ_ITALIC_FONT, "Complete routing and try again.\n");
        return (BEAR_ERROR);
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(coreMasterPtr);

    FindData(coreMasterPtr, &hRect1, &vRect1, &hRect2, &vRect2, &urJunctPtr);
    Tm_RingUnify(coreMasterPtr, hRect1, vRect1, hRect2, vRect2, urJunctPtr);
    Mac_CreateRing(coreMasterPtr,&ringPtr);
    gPlaneDataPtr = planeDataPtr;
    TiSrArea(NIL(Tile),
        Tp_GetHorzPlane(planeDataPtr),
        Tp_GetHorzPlaneBnd(planeDataPtr),
        InsertHorzRectList,
        (ClientData)&ringPtr->horzRectList);

    TiSrArea(NIL(Tile),
        Tp_GetVertPlane(planeDataPtr),
        Tp_GetVertPlaneBnd(planeDataPtr),
        InsertVertRectList,
        (ClientData)&ringPtr->vertRectList);

    cellNamePtr = Mac_GetNameOfCellMaster(coreMasterPtr);
    MALLOC(char *, fileNamePtr, (strlen(cellNamePtr) + 6) * sizeof (char));
    strcpy (fileNamePtr, cellNamePtr);
    strcat (fileNamePtr, ".ring");
    Gcd_DefineRing(coreMasterPtr, ringPtr);
    if ((filePtr = fopen(fileNamePtr, "w")) == NIL(FILE)) {
        sprintf(bear_ErrorMsg, "could not open %s\n", fileNamePtr);
        PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
        return (BEAR_ERROR);
    }

    fprintf (filePtr,"SN %d\n",Mac_GetNumOfNetsOfRing(ringPtr));
    fprintf (filePtr,"MOD\n");
    originPtr = Mac_GetOriginOfRing(ringPtr);
    fprintf (filePtr,"%d %d\n",originPtr->p_x,originPtr->p_y);
    fprintf (filePtr,"%s\n",cellNamePtr);
    fprintf (filePtr,"10\n");
    bBoxPtr = Mac_GetOutBBoxOfRing(ringPtr);
    fprintf (filePtr,"%d %d\n",bBoxPtr->r_xbot,bBoxPtr->r_ytop);
    fprintf (filePtr,"%d %d\n",bBoxPtr->r_xtop,bBoxPtr->r_ybot);
    bBoxPtr = Mac_GetInBBoxOfRing(ringPtr);
    fprintf (filePtr,"%d %d\n",bBoxPtr->r_xbot,bBoxPtr->r_ytop);
    fprintf (filePtr,"%d %d\n",bBoxPtr->r_xtop,bBoxPtr->r_ybot);
    fprintf (filePtr,"$\n");
    fprintf (filePtr,"termout\n");
    Mac_ItrOutBoundPinOfRing(ringPtr,OutputGPinOnOutBnd,(ClientData)filePtr);
    fprintf (filePtr,"$\n");
    fprintf (filePtr,"termin\n");
    Mac_ItrInBoundPinOfRing(ringPtr,OutputGPinOnInBnd,(ClientData)filePtr);
    fprintf (filePtr,"$\n");
    fprintf (filePtr,"DES\n");
    fprintf (filePtr,"hwidth1 %d\n",
        (Mac_GetMinWidthOfLay(METAL1_LAYER))/2);
    fprintf (filePtr,"hwidth2 %d\n",
        (Mac_GetMinWidthOfLay(METAL2_LAYER))/2);
    fprintf (filePtr,"sp1 %d\n",
        Mac_GetMinSpaceOfLayLay(METAL1_LAYER,METAL1_LAYER));
    fprintf (filePtr,"sp2 %d\n",
        (Mac_GetMinWidthOfLay(METAL2_LAYER))/2);
    fprintf (filePtr,"hvia %d\n",
        (Mac_GetSizeOfHole(M1M2_VIA))/2);
    fprintf (filePtr,"overlap %d\n",
        Mac_GetMinOverlapOfLayHole(METAL1_LAYER, M1M2_VIA));
    fprintf (filePtr,"$\n");
    fclose (filePtr);

#ifdef USE_WIRE_CODE
    (void) Gre_DoPgWiringForAllNets(coreMasterPtr);
#endif USE_WIRE_CODE
    (void) Gs_CalcCapDenOfTilePlane(coreMasterPtr);
    if (Rr_RouteRing(fileNamePtr, ringPtr, adjust, 0,
        outputFileName, viaReduceFlag) == BEAR_ERROR) {
        PbPrint(EZ_NORMAL_FONT, "Ring routing aborted (error).\n");
        return (BEAR_ERROR);
    }

    (void) Rr_ResizeRing(coreMasterPtr, adjust);
    Tp_IdentifySpaceTiles(planeDataPtr);
    (void) Fpg_CreateFloorPlanGraph(planeDataPtr, &llJunctPtr);
#ifdef USE_WIRE_CODE
    (void) Gre_DoPgWiringForAllNets(coreMasterPtr);
#endif USE_WIRE_CODE
    (void) Gs_CalcCapDenOfTilePlane(coreMasterPtr);

    Mac_FreeRectList(ringPtr->horzRectList);
    ringPtr->horzRectList = NIL(LinkedRect);
    TiSrArea(NIL(Tile),
        Tp_GetHorzPlane(planeDataPtr),
        Tp_GetHorzPlaneBnd(planeDataPtr),
        InsertHorzRectList,
        (ClientData) &ringPtr->horzRectList);

    Mac_FreeRectList(ringPtr->vertRectList);
    ringPtr->vertRectList = NIL(LinkedRect);
    TiSrArea(NIL(Tile),
        Tp_GetVertPlane(planeDataPtr),
        Tp_GetVertPlaneBnd(planeDataPtr),
        InsertVertRectList,
        (ClientData) &ringPtr->vertRectList);
    
    FindData(coreMasterPtr, &hRect1, &vRect1, &hRect2, &vRect2, &urJunctPtr);
    Tm_RingUnify(coreMasterPtr, hRect1, vRect1, hRect2, vRect2, urJunctPtr);
    if (Rr_RouteRing(fileNamePtr, ringPtr, adjust, 1, outputFileName,
        viaReduceFlag) == BEAR_ERROR) {
        PbPrint(EZ_NORMAL_FONT, "Ring routing aborted (error).\n");
        return (BEAR_ERROR);
    }
    Tp_PaintRing(ringPtr, planeDataPtr);
    Tp_IdentifySpaceTiles(planeDataPtr);
    Fpg_CreateFloorPlanGraph(planeDataPtr, &llJunctPtr);

    coreMasterPtr->contents.ringPtr = ringPtr;

    Mac_SetRingRoutingDone(coreMasterPtr);
#ifndef BEAR_BLACK_BOX
    if (bear_BlackBoxFlag == FALSE) {
         PbCenterView(winDataPtr, Tp_GetHorzPlaneBnd(planeDataPtr));
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
    }
#endif BEAR_BLACK_BOX
    FREE((char *) outputFileName);

    /* This portion deletes the route ring, but is currently not used

       Mac_DeleteRing(ringPtr);
       junctPtr = llJunctPtr;
       while ((wallSegPtr=Fpg_GetEastSegOfWallJunct(junctPtr)) !=
       NIL(WallSegment))
       {
       bnTilePtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr)->
       unValue.bnTilePtr;
       newDensity = Tp_GetCapacityOfTile(bnTilePtr)
       + adjust[Rr_BOTTOM_SIDE];
       Tp_PutDensityOfTile(-newDensity, bnTilePtr);
       junctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
       }
       while ((wallSegPtr=Fpg_GetNorthSegOfWallJunct(junctPtr)) !=
       NIL(WallSegment))
       {
       bnTilePtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr)->
       unValue.bnTilePtr;
       newDensity = Tp_GetCapacityOfTile(bnTilePtr)
       + adjust[Rr_RIGHT_SIDE];
       Tp_PutDensityOfTile(-newDensity, bnTilePtr);
       junctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
       }
       while ((wallSegPtr=Fpg_GetWestSegOfWallJunct(junctPtr)) !=
       NIL(WallSegment))
       {
       bnTilePtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr)->
       unValue.bnTilePtr;
       newDensity = Tp_GetCapacityOfTile(bnTilePtr)
       + adjust[Rr_TOP_SIDE];
       Tp_PutDensityOfTile(-newDensity, bnTilePtr);
       junctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
       }
       while ((wallSegPtr=Fpg_GetSouthSegOfWallJunct(junctPtr)) !=
       NIL(WallSegment))
       {
       bnTilePtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr)->
       unValue.bnTilePtr;
       newDensity = Tp_GetCapacityOfTile(bnTilePtr)
       + adjust[Rr_LEFT_SIDE];
       Tp_PutDensityOfTile(-newDensity, bnTilePtr);
       junctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
       }
       */
    PbPrint(EZ_NORMAL_FONT, "Ring routing is done.\n");
    return (BEAR_OK);

usage:
    PbPrint(EZ_NORMAL_FONT, "ring router (ok in script) (ok in black box)\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: rr [-via] [-NDW]\n\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -via reduce via's\n");
    PbPrint(EZ_NORMAL_FONT, "    -NDW no dialog windows\n");

    return (BEAR_ERROR);
}


#endif /* USE_DR_CODE */


