
/* pbShowCmds.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#if !defined(BEAR_BLACK_BOX)
/*
 * This file does not need to be compiled when the black box version of
 * the BEAR-FP system is built.
 */

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "deque.h"
#include "grEditor.h"
#include "polarBear.h"


typedef struct _PlaneProp PlaneProp;
struct _PlaneProp {
    bool    bothPlanes;
    bool    cellsOnly;
    bool    horzTilesOnly;
    bool    vertTilesOnly;
    
    bool    toggleBnTiles;
    bool    bnTilesVisible;

    bool    toggleCellNames;
    bool    cellNamesVisible;

    bool    toggleDomTiles;
    bool    domTilesVisible;

    bool    toggleFloorPlan;
    bool    floorPlanVisible;

    bool    togglePins;
    bool    pinsVisible;
};


/* ---------- Local Procedures ---------- */
void        InitPlaneProp();
BearStatus  UpdateChipWin();



void
InitPlaneProp(planePropPtr)
PlaneProp *planePropPtr;
{
    planePropPtr->bothPlanes = FALSE;
    planePropPtr->cellsOnly = FALSE;
    planePropPtr->horzTilesOnly = FALSE;
    planePropPtr->vertTilesOnly = FALSE;

    planePropPtr->toggleBnTiles = FALSE;
    planePropPtr->toggleCellNames = FALSE;
    planePropPtr->toggleDomTiles = FALSE;
    planePropPtr->toggleFloorPlan = FALSE;
    planePropPtr->togglePins = FALSE;
}



static BearStatus
UpdateChipWin(winDataPtr, planePropPtr)
WindowData  *winDataPtr;
PlaneProp   *planePropPtr;
{
    CellMaster  *cellMasterPtr;
    ChipWin     *chipWinPtr;
    bool        redrawFlag = FALSE;

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (PbIsChipWin(winDataPtr) == FALSE ||
        Mac_IsNumOfCellsZero(cellMasterPtr)) {
        return (BEAR_CONTINUE_SEARCH);
    }

    if (planePropPtr->bothPlanes) {
        if (((chipWinPtr->chw_options & PB_SHOW_HORZ_PLANE_OPT) == FALSE) ||
            ((chipWinPtr->chw_options & PB_SHOW_VERT_PLANE_OPT) == FALSE)) {
            /*
             * Redraw this window and show both tile planes.
             */
            Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_HORZ_PLANE_OPT);
            Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_VERT_PLANE_OPT);
            redrawFlag = TRUE;
        }
    }

    if (planePropPtr->cellsOnly) {
        if ((chipWinPtr->chw_options & PB_SHOW_HORZ_PLANE_OPT) ||
            (chipWinPtr->chw_options & PB_SHOW_VERT_PLANE_OPT)) {
            /*
             * Redraw this window so that it shows only cells, i.e. don't show
             * any of the tiles.
             */
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_HORZ_PLANE_OPT);
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_VERT_PLANE_OPT);
            redrawFlag = TRUE;
        }
    }

    if (planePropPtr->horzTilesOnly) {
        if (PbHorzTilesOn(winDataPtr)) {
            redrawFlag = TRUE;
        }
    }

    if (planePropPtr->vertTilesOnly) {
        if (PbVertTilesOn(winDataPtr)) {
            redrawFlag = TRUE;
        }
    }

    if (planePropPtr->toggleBnTiles) {
        if (planePropPtr->bnTilesVisible) {
            Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_BN_TILES_OPT);
            if ((chipWinPtr->chw_options & PB_SHOW_HORZ_PLANE_OPT) == FALSE &&
                (chipWinPtr->chw_options & PB_SHOW_VERT_PLANE_OPT) == FALSE) {
                Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_HORZ_PLANE_OPT);
                Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_VERT_PLANE_OPT);
            }
        } 
        else {
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_BN_TILES_OPT);
        }
        redrawFlag = TRUE;
    }

    if (planePropPtr->toggleCellNames) {
        if (planePropPtr->cellNamesVisible) {
            Bear_SetOption(chipWinPtr->chw_options, PB_LABEL_CELLS_OPT);
        } 
        else {
            Bear_UnsetOption(chipWinPtr->chw_options, PB_LABEL_CELLS_OPT);
        }
        redrawFlag = TRUE;
    }

    if (planePropPtr->toggleDomTiles) {
        if (planePropPtr->domTilesVisible) {
            Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_DOM_TILES_OPT);
            if ((chipWinPtr->chw_options & PB_SHOW_HORZ_PLANE_OPT) == FALSE &&
                (chipWinPtr->chw_options & PB_SHOW_VERT_PLANE_OPT) == FALSE) {
                Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_HORZ_PLANE_OPT);
                Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_VERT_PLANE_OPT);
            }
        } 
        else {
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_DOM_TILES_OPT);
        }
        redrawFlag = TRUE;
    }

    if (planePropPtr->toggleFloorPlan) {
        if (planePropPtr->floorPlanVisible) {
            Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        } 
        else {
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        }
        redrawFlag = TRUE;
    }

    if (planePropPtr->togglePins) {
        if (planePropPtr->pinsVisible) {
            Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_PINS_OPT);
        } 
        else {
            Bear_UnsetOption(chipWinPtr->chw_options, PB_SHOW_PINS_OPT);
        }
        redrawFlag = TRUE;
    }

    if (redrawFlag) {
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
    }

    return (BEAR_CONTINUE_SEARCH);
}



/* ARGSUSED */
void
PbShowCellPropCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    CellInst    *cellInstPtr;
    CellMaster  *cellMasterPtr;
    ChipWin     *chipWinPtr;
    DrawGCell   drawGCell;
    GenericCell gcell;
    PlaneData   *planeDataPtr;
    Tile        *tilePtr;
    int         i, planeType;
    char        *cellNamePtr = NIL(char);
    bool        stopHiLightingFlag = FALSE;
    bool        toggleNamesFlag = FALSE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-names") == 0) {
            toggleNamesFlag = TRUE;
            continue;
        }

        if (strcmp (argList[i], "-off") == 0) {
            /* 
             * Stop cell high-lighting.
             */
            stopHiLightingFlag = TRUE;
            continue;
        }
        else {
            cellNamePtr = argList[i];
            continue;
        }
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (stopHiLightingFlag == TRUE) {
        if (PbGetGCellOfCHW(chipWinPtr) != NIL(GenericCell)) {
            PbPutGCellOfCHW(NIL(GenericCell), chipWinPtr);
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
        return;
    }

    if (toggleNamesFlag == TRUE) {
        PlaneProp planeProp;

        /*
         * Toggle cell names, i.e. if the cell names are currently being
         * displayed in the window the mouse is in stop displaying them 
         * and vica versa.
         */
        InitPlaneProp(&planeProp);
        planeProp.toggleCellNames = TRUE;
        planeProp.cellNamesVisible =
            ! (PbGetOptionsOfCHW(chipWinPtr) & PB_LABEL_CELLS_OPT);
        /* Update the chip window the mouse is in. */
        (void) UpdateChipWin(winDataPtr, &planeProp);
        return;
    }

    if (cellNamePtr != NIL(char)) {
        /* 
         * Use the cell instance name they provided to locate the pointer
         * to the cell instance.
         */
        if (Mac_IsMember(CELL_INSTANCE_HT, cellNamePtr)) {
            cellInstPtr = (CellInst *) Mac_LookUp(CELL_INSTANCE_HT,
                cellNamePtr);
            gcell.unType = CELL_INST;
            gcell.unValue.cellInstPtr = cellInstPtr;
        }
        else {
            sprintf (bear_ErrorMsg, "Cannot find a cell instance named %s\n",
                cellNamePtr);
            PbPrint(EZ_ITALIC_FONT, bear_ErrorMsg);
            return;
        }
    }
    else {
        if (Mac_GetPlacedFlagOfCell(cellMasterPtr) == FALSE) {
            PbPrint(EZ_ITALIC_FONT, 
                "Cells aren't placed so they may only be selected by name.\n");
            return;
        }

        planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);
        if (Tp_AreSpaceTilesReady(planeDataPtr) == FALSE) {
            Tp_IdentifySpaceTiles(planeDataPtr);
        }
        if (chipWinPtr->chw_options & PB_SHOW_HORZ_PLANE_OPT) {
            planeType = HORZ_TILE_PLANE;
        }
        else if (chipWinPtr->chw_options & PB_SHOW_VERT_PLANE_OPT) {
            planeType = VERT_TILE_PLANE;
        }
        else {
            planeType = NULL_TILE_PLANE;
        }
        if ((PbFindTile(winDataPtr, screenPtr, &tilePtr, planeType) == TRUE) &&
            Tp_IsSolidTile(tilePtr)) {

            if (Tp_IsCellInstTile(tilePtr)) {
                gcell.unType = CELL_INST;
                gcell.unValue.cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
            }
            else if (Tp_IsDCellTile(tilePtr)) {
                gcell.unType = DUMMY_CELL;
                gcell.unValue.dummyCellPtr = Tp_GetDCellOfTile(tilePtr);
            }
            else if (Tp_IsRCellTile(tilePtr)) {
                gcell.unType = ROUTE_CELL;
                gcell.unValue.routeCellPtr = Tp_GetRCellOfTile(tilePtr);
            }
        }
        else {
            Ez_RingBell();
            PbPrint(EZ_ITALIC_FONT, 
           "Mouse must be in a cell instance a dummy cell or a route cell.\n");
            return;
        }
    }

    if (Mac_GetPlacedFlagOfCell(cellMasterPtr) == TRUE) {
        if (PbGetGCellOfCHW(chipWinPtr) == NIL(GenericCell)) {
            PbPutGCellOfCHW(&gcell, chipWinPtr);

            drawGCell.gcellPtr = &gcell;
            drawGCell.gcellColor = ez_HighLightColor;
            (void) PbItrChipWindows(PbHiLightGCell, (ClientData) &drawGCell);
            Ez_Flush();
        }
        else {
            PbPutGCellOfCHW(&gcell, chipWinPtr);
            (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
        }
    }
    PbPrintGCellProp(&gcell);

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "show cell properties\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: scp [cellName] [-off]\n");
    PbPrint(EZ_NORMAL_FONT, "    -off turn off cell high-lighting (only)\n");
    PbPrint(EZ_NORMAL_FONT, "    -names toggle cell names (only)\n");
}



/* ARGSUSED */
void
PbShowJunctPropCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    FloorPlanGraph  *fpgPtr;
    PlaneData       *planeDataPtr;
    Tile            *horzDomTilePtr, *vertDomTilePtr;
    WallJunction    *junctPtr; 
    int             i;
    bool            toggleFloorPlanFlag = FALSE;
    bool            mainRoomFlag = FALSE;

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if ((strcmp (argList[i], "-help") == 0) ||
            (strcmp (argList[i], "help") == 0)) {
            goto usage;
        }

        if ((strcmp (argList[i], "-fp") == 0) ||
            (strcmp (argList[i], "fp") == 0)) {
            toggleFloorPlanFlag = TRUE;
            continue;
        }

        if ((strcmp (argList[i], "-mr") == 0) ||
            (strcmp (argList[i], "mr") == 0)) {
            mainRoomFlag = TRUE;
            continue;
        }

        (void) sprintf (bear_Msg, "unknown option %s\n", argList[i]);
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
        goto usage;
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    if (Mac_IsNumOfCellsZero(cellMasterPtr) == TRUE) {
        Ez_RingBell();
        return;
    }

    planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr);

    if (Fpg_IsFpgBuilt(planeDataPtr) == FALSE) {
        /*
         * In this case we don't have a floor plan graph, and
         * so we build one with dummy cells.
         */
        BuildFloorPlan(cellMasterPtr, FALSE);
    }

    fpgPtr = Tp_GetFpg(planeDataPtr);

    if ((fpgPtr->mainRoomPtr == NIL(Fpg_Room)) ||
        (fpgPtr->roomList == (lsList) NULL)) {
        if (Fpg_GenRooms(cellMasterPtr) == BEAR_ERROR) {
            PbPrint(EZ_ITALIC_FONT, "Failed to generate rooms.\n");
            return;
        }
    }

    if (toggleFloorPlanFlag == TRUE) {
        PlaneProp planeProp;

        InitPlaneProp(&planeProp);
        planeProp.toggleFloorPlan = TRUE;
        planeProp.floorPlanVisible =
            !(chipWinPtr->chw_options & PB_SHOW_FPG_OPT);

        /* Update the chip window the mouse is in. */
        (void) UpdateChipWin(winDataPtr, &planeProp);

        return;
    }

    if (! (chipWinPtr->chw_options & PB_SHOW_FPG_OPT)) {
        Bear_SetOption(chipWinPtr->chw_options, PB_SHOW_FPG_OPT);
        (void) PbDrawChipRegion(winDataPtr, NIL(Rect));
    }

    if (mainRoomFlag == TRUE) {
        PbDrawRoom(winDataPtr, fpgPtr->mainRoomPtr);
        PbPrintRoomProp(fpgPtr->mainRoomPtr);
        return;
    }

    PbPrint(EZ_NORMAL_FONT, "Pick a junction.\n");
    if (Gre_PickJunct(&horzDomTilePtr, &vertDomTilePtr, &junctPtr) ==
        BEAR_ERROR) {
        PbPrint(EZ_ITALIC_FONT, "Picking junction failed.\n");
        return;
    }
    PbDrawRoom(winDataPtr, junctPtr->roomPtr);

    /* 
     * Print out information about the wall junction.
     */
    PbPrintJunctProp(junctPtr);
    PbPrint(EZ_NORMAL_FONT, "-----\n");
    PbPrintRoomProp(junctPtr->roomPtr);

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "show junct properties\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: sjp [-fp] [-mr]\n");
    PbPrint(EZ_NORMAL_FONT, "    -fp toggle floor plan graph (only)\n");
    PbPrint(EZ_NORMAL_FONT, "    -mr main room               (only)\n");
}



/* ARGSUSED */
void
PbShowNetPropCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    CellMaster      *cellMasterPtr;
    ChipWin         *chipWinPtr;
    Net             *netPtr;
    PinInst         *pinInstPtr;
    int             atoi ();
    int             i;
    int             whichNet = FPG_CENTER;
    bool            eraseFlag = FALSE;
    char            *netNamePtr = NIL(char);

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        /* 
         * The -off option is used to erase all of the nets that are
         * currently being displayed in all of the chip windows.
         */
        if (strcmp (argList[i], "-off") == 0) {
            eraseFlag = TRUE;
            break;
        }

/*
 * code change : by george : on 5 mar 90
 *
 * See note below
 *
 *      if (strcmp (argList[i], "-c") == 0) {
 *          if (netNamePtr == NIL(char)) {
 *              PbPrint(EZ_ITALIC_FONT,
 *                  "Must specify a net name before picking a color.\n");
 *              goto usage;
 *          }
 *          if (++i >= argCount) {
 *              goto usage;
 *          }
 *          switch (atoi (argList[i])) {
 *          case 1:
 *              whichNet = FPG_NW;
 *              break;
 *          case 2:
 *              whichNet = FPG_CENTER;
 *              break;
 *          case 3:
 *              whichNet = FPG_SE;
 *              break;
 *          default:
 *              goto usage;
 *          }
 *      }
 */
        else {
            if (netNamePtr == NIL(char)) {
                netNamePtr = argList[i];
            }
            else {
                PbPrint(EZ_ITALIC_FONT,
                    "Only one net name may be specified.\n");
                goto usage;
            }
            continue;
        }
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);

    if (eraseFlag == TRUE) {
        PbPutNetOfCHW(FPG_NW, NIL(Net), chipWinPtr);
        PbPutNetOfCHW(FPG_CENTER, NIL(Net), chipWinPtr);
        PbPutNetOfCHW(FPG_SE, NIL(Net), chipWinPtr);
        (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);

        return;
    }

    if (netNamePtr != NIL(char)) {
        /* 
         * Use the net name they provided to locate the net, and
         * then display it.
         */
        if (Gre_PickNetByName(winDataPtr, netNamePtr, whichNet, &netPtr) ==
            BEAR_ERROR) {
            return;
        }
    }
    else {
        /* 
         * Use the mouse to locate the net, and then display it.
         */
        if (Gre_PickNet(winDataPtr, &netPtr, &pinInstPtr) == BEAR_ERROR) {
            Ez_RingBell();
            return;
        }
    }

#ifdef USE_WIRE_CODE
    (void) Gre_DoPgWiringForAllNets(cellMasterPtr);
#endif USE_WIRE_CODE

    /* 
     * Print out information about the net.
     */
    PbPrintNetProp(netPtr);

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "show net properties\n");

    PbPrint(EZ_NORMAL_FONT, "Usage: snp [netName] [-off]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -off erase all of the nets\n");

/*
 * code change : by george : on 5 mar 90
 *
 * Since wall junctions are no longer used in the graph of the net we
 * can no longer show 3 nets at a time.
 *
 *  PbPrint(EZ_NORMAL_FONT, "Usage: snp [netName] [netName -c (1,2,or 3)]\n");
 *  PbPrint(EZ_NORMAL_FONT, "           [-off]\n");
 *  PbPrint(EZ_NORMAL_FONT, "options:\n");
 *  PbPrint(EZ_NORMAL_FONT, "    -c (1,2,or 3) number of the color\n");
 *  PbPrint(EZ_NORMAL_FONT, "    -off erase all of the nets\n");
 */
}



/* ARGSUSED */
void
PbShowPinPropCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    BearString  piNamePtr = (BearString) NULL;
    BearString  ciNamePtr = (BearString) NULL;
    ChipWin     *chipWinPtr;
    DrawPin     drawPin;
    PinInst     *pinInstPtr;
    PlaneProp   planeProp;
    int         i;
    bool        getPinByNameFlag = FALSE;
    bool        stopHiLightingFlag = FALSE;
    bool        togglePins = FALSE;
    bool        mpsFlag = FALSE;

    InitPlaneProp(&planeProp);

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if (strcmp (argList[i], "-p") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            piNamePtr = argList[i];
            continue;
        }

        if (strcmp (argList[i], "-c") == 0) {
            if (++i >= argCount) {
                goto usage;
            }
            ciNamePtr = argList[i];
            continue;
        }

        if (strcmp (argList[i], "-s") == 0) {
            togglePins = TRUE;
            continue;
        }

        if (strcmp (argList[i], "-off") == 0) {
            /* 
             * Stop pin high-lighting.
             */
            stopHiLightingFlag = TRUE;
            continue;
        }

        if (strcmp (argList[i], "-mps") == 0) {
            /* 
             * Print minimum pin spacing.
             */
            mpsFlag = TRUE;
            continue;
        }

        goto usage;
    }

    chipWinPtr = PbGetChipWin(winDataPtr);

    if ((piNamePtr != (BearString) NULL) && (ciNamePtr != (BearString) NULL)) {
        getPinByNameFlag = TRUE;
    }

    if ((argCount == 1) || getPinByNameFlag) {
        if (Gre_GetPin(winDataPtr, piNamePtr, ciNamePtr,
            ez_HighLightColor, &pinInstPtr) != BEAR_OK) {
            return;
        }

        /* 
         * Print out information about the pin.
         */
        PbPrintPinProp(pinInstPtr);
        return;
    }

    if (stopHiLightingFlag) {
        /*
         * If there is currently a pin being high-lighted redraw it
         * in all of the chip windows in the regular pin color.
         */
        if ((pinInstPtr = PbGetPinOfCHW(chipWinPtr)) != NIL(PinInst)) {
            PbPutPinOfCHW(NIL(PinInst), chipWinPtr);

            drawPin.pinInstPtr = pinInstPtr;
            drawPin.pinColor = pbPinColor;
            (void) PbItrChipWindows(Gre_DrawPinTwo, (ClientData) &drawPin);
            Ez_Flush();
        }
        return;
    }
    if (mpsFlag == TRUE) {
        sprintf (bear_Msg, "Minimum pin spacing = %d\n", 
            Mac_CalcMinPinSpacing());
        PbPrint(EZ_NORMAL_FONT, bear_Msg);
        return;
    }
    if (togglePins) {
        /*
         * Toggle pins, i.e. if the pins are currently being displayed
         * in the window the mouse is in stop displaying them and vica
         * versa.
         */
        planeProp.togglePins = TRUE;
        planeProp.pinsVisible =
            ! (PbGetOptionsOfCHW(chipWinPtr) & PB_SHOW_PINS_OPT);
        /* Update the chip window the mouse is in. */
        (void) UpdateChipWin(winDataPtr, &planeProp);
    }

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "show pin properties\n");
    PbPrint(EZ_NORMAL_FONT, 
        "Usage: spp [-p pinName -c cellName] [-s] [-off] [-mps]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -s   show pins (on/off)\n");
    PbPrint(EZ_NORMAL_FONT, "    -off turn off pin high-lighting\n");
    PbPrint(EZ_NORMAL_FONT, "    -mps print minimum pin spacing\n");
}



/* ARGSUSED */
void
PbShowTilePropCmd(winDataPtr, screenPtr, argCount, argList)
WindowData  *winDataPtr;
Point       *screenPtr;
int         argCount;
char        **argList;
{
    BNLine      *bnLinePtr;
    CellMaster  *cellMasterPtr;
    ChipWin     *chipWinPtr;
    PlaneData   *planeDataPtr;
    PlaneProp   planeProp;
    Tile        *tilePtr;
    int         i, planeType;
    bool        listBnTilePPins = FALSE;
    bool        pickBnLine = FALSE;
    bool        stopHiLightingFlag = FALSE;

    InitPlaneProp(&planeProp);

    /* parse arguments */
    for (i = 1; i < argCount ;i++) {
        /* 
         * The -help option prints out this commands help message.
         */
        if (strcmp (argList[i], "-help") == 0) {
            goto usage;
        }

        if ((strcmp (argList[i], "-bp") == 0) ||
            (strcmp (argList[i], "bp") == 0)) {
            /* 
             * Outline the tiles on both the horizontal and the vertical
             * tile planes.
             */
            planeProp.bothPlanes = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-co") == 0) ||
            (strcmp (argList[i], "co") == 0)) {
            /* 
             * Show only the cells.
             */
            planeProp.cellsOnly = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-h") == 0) ||
            (strcmp (argList[i], "h") == 0)) {
            /* 
             * Outline only the horizontal tiles.
             */
            planeProp.horzTilesOnly = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-v") == 0) ||
            (strcmp (argList[i], "v") == 0)) {
            /* 
             * Outline only the vertical tiles.
             */
            planeProp.vertTilesOnly = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-b") == 0) ||
            (strcmp (argList[i], "b") == 0)) {
            /* 
             * Toggle the bottleneck lines and tiles.
             */
            planeProp.toggleBnTiles = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-d") == 0) ||
            (strcmp (argList[i], "d") == 0)) {
            /* 
             * Toggle the dominant tiles.
             */
            planeProp.toggleDomTiles = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-lp") == 0) ||
            (strcmp (argList[i], "lp") == 0)) {
            /* 
             * List the pseudo pins of the tile the mouse is in.
             */
            listBnTilePPins = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-bnl") == 0) ||
            (strcmp (argList[i], "bnl") == 0)) {
            /* 
             * Pick a bottleneck line and list its pseudo pins.
             */
            pickBnLine = TRUE;
            continue;
        }
        if ((strcmp (argList[i], "-off") == 0) ||
            (strcmp (argList[i], "off") == 0)) {
            /* 
             * Stop bottleneck line high-lighting.
             */
            stopHiLightingFlag = TRUE;
            continue;
        }

        goto usage;
    }

    chipWinPtr = PbGetChipWin(winDataPtr);
    cellMasterPtr = PbGetCellMasterOfCHW(chipWinPtr);
    if ((planeDataPtr = Mac_GetCellPlaneOfCell(cellMasterPtr)) ==
        NIL(PlaneData)) {
        PbPrint(EZ_ITALIC_FONT, "No tile plane given.\n");
        return;
    }

    if ((argCount == 1) || (listBnTilePPins == TRUE)) {
        /*
         * Give them information about the tile that the mouse is in.
         */

        if (Tp_AreSpaceTilesReady(planeDataPtr) == FALSE) {
            Tp_IdentifySpaceTiles(planeDataPtr);
        }
        if (chipWinPtr->chw_options & PB_SHOW_HORZ_PLANE_OPT) {
            planeType = HORZ_TILE_PLANE;
        } 
        else if (chipWinPtr->chw_options & PB_SHOW_VERT_PLANE_OPT) {
            planeType = VERT_TILE_PLANE;
        } 
        else {
            planeType = NULL_TILE_PLANE;
        }
        if (PbFindTile(winDataPtr, screenPtr, &tilePtr, planeType) == TRUE) {
            PbPrintTileProp(planeDataPtr, tilePtr);

            if ((Tp_IsBottleneckTile(tilePtr) ||
                Tp_IsSBNTile(tilePtr) ||
                Tp_IsRCellTile(tilePtr) ||
                Tp_IsDCellTile(tilePtr) ||
                Tp_IsRingTile(tilePtr)) && 
                listBnTilePPins) {

                Tp_PrintPPinListsOfTile(tilePtr,    
                    PbGetNetOfCHW(FPG_CENTER, chipWinPtr));
            }
        }
        else {
            Ez_RingBell();
            return;
        }
    }
    else {
        if (planeProp.toggleBnTiles) {
            if (Tp_AreSpaceTilesReady(planeDataPtr) == FALSE) {
                Tp_IdentifySpaceTiles(planeDataPtr);
            }
            planeProp.bnTilesVisible =
                !(chipWinPtr->chw_options & PB_SHOW_BN_TILES_OPT);
        }
        if (planeProp.toggleDomTiles) {
            if (Tp_AreSpaceTilesReady(planeDataPtr) == FALSE) {
                Tp_IdentifySpaceTiles(planeDataPtr);
            }
            planeProp.domTilesVisible =
                !(chipWinPtr->chw_options & PB_SHOW_DOM_TILES_OPT);
        }
        /* Update the chip window the mouse is in. */
        (void) UpdateChipWin(winDataPtr, &planeProp);

        if (stopHiLightingFlag) {
            if (PbGetBnLineOfCHW(chipWinPtr) != NIL(BNLine)) {
                PbPutBnLineOfCHW(NIL(BNLine), chipWinPtr);
                (void) PbItrChipWindows(PbDrawChipRegion, (ClientData) NULL);
            }
            return;
        }
        if (pickBnLine) {
            /*
             * Let them choose a bottle-neck line with the mouse.
             */
            if (PbPickBnLine(winDataPtr, &bnLinePtr) == BEAR_ERROR) {
                return;
            }

            /* 
             * Print out information about the bottle-neck line.
             */
            PbPrintBnLineProp(bnLinePtr);
            Tp_PrintPPinListOfBnLine(bnLinePtr,
                PbGetNetOfCHW(FPG_CENTER, chipWinPtr));
        }
    }

    return;

usage:
    PbPrint(EZ_NORMAL_FONT, "show tile properties\n");
    PbPrint(EZ_NORMAL_FONT, "Usage: stp [-co] [[-bp -h -v] [-b] [-d]]\n");
    PbPrint(EZ_NORMAL_FONT, "           [-lp] [-lbn] [-off]\n");
    PbPrint(EZ_NORMAL_FONT, "options:\n");
    PbPrint(EZ_NORMAL_FONT, "    -co   show cells only\n");
    PbPrint(EZ_NORMAL_FONT, "    -bp   show both tile planes\n");
    PbPrint(EZ_NORMAL_FONT, "    -h    show horizontal tile plane only\n");
    PbPrint(EZ_NORMAL_FONT, "    -v    show vertical tile plane only\n");
    PbPrint(EZ_NORMAL_FONT,
        "    -b    show bottleneck lines and tiles (on/off)\n");
    PbPrint(EZ_NORMAL_FONT, "    -d    show dominant tiles (on/off)\n");
    PbPrint(EZ_NORMAL_FONT, "    -lp   list pseudo pins of tile\n");
    PbPrint(EZ_NORMAL_FONT, "    -bnl  list pseudo pins of bottleneck line\n");
    PbPrint(EZ_NORMAL_FONT,
    "    -off  turn off bottleneck line high-lighting\n");
}

#endif /* BEAR_BLACK_BOX */


