
/* pbShowConn.c --
 *
 *     Denis Lee
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#if !defined(BEAR_BLACK_BOX) && defined(DEBUG)
/*
 * This file does not need to be compiled when the black box version of
 * the BEAR-FP system is built.
 */

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "polarBear.h"


typedef struct _ShowConnInfo {
    WindowData  *winDataPtr;
    CellMaster *cellMasterPtr;
    Net *netPtr;
    Point pinPos;
    int xMin;
    int yMin;
    int xMax;
    int yMax;
    int netotal;
    int marked;
    int netmax;
    double netcutoff;
} ShowConnInfo;


/* ---------- Local Procedures ---------- */
static BearStatus DrawNetConn(/* netPtr, showConnInfoPtr */);
static BearStatus DrawEndPin(/* pinPtr, showConnInfoPtr */);
static BearStatus DrawStartPin(/* pinPtr, showConnInfoPtr */);
static BearStatus FreePinMarker(/* pinPtr, clientData */);


void
PbShowConn(winDataPtr, cellMasterPtr)
WindowData *winDataPtr;
CellMaster *cellMasterPtr;
{
    ShowConnInfo showConnInfo;
    Net *netListPtr;

    showConnInfo.winDataPtr = winDataPtr;
    showConnInfo.cellMasterPtr = cellMasterPtr;
    showConnInfo.netotal = 0;
    showConnInfo.marked = 1;
    showConnInfo.netmax = 0;

    fprintf(stderr,
        "Please enter in the percentage of cell coverage for net cutoff: ");
    fscanf(stdin,"%f",&showConnInfo.netcutoff);
    showConnInfo.netcutoff = showConnInfo.netcutoff / 100;
    netListPtr = Mac_GetNetListOfCell(cellMasterPtr);
    Mac_ItrNet(netListPtr,DrawNetConn,(ClientData) &showConnInfo);

    fprintf(stderr,"\n\nEstimated sum of wire lengths = %d.\n",
    showConnInfo.netotal);
    fprintf(stderr,"Estimated maximum wire length = %d.\n\n",
    showConnInfo.netmax);
}


static BearStatus
DrawNetConn(netPtr, showConnInfoPtr)
Net *netPtr;
ShowConnInfo *showConnInfoPtr;
{
    int cellnum, netports;
    int length;

    showConnInfoPtr->netPtr = netPtr;
    showConnInfoPtr->xMin = 0;
    showConnInfoPtr->yMin = 0;
    showConnInfoPtr->xMax = 0;
    showConnInfoPtr->yMax = 0;

    Mac_NumOfCells(showConnInfoPtr->cellMasterPtr,&cellnum);
    Mac_NumOfPortsOfNet(netPtr,&netports);
    if (netports < (cellnum * showConnInfoPtr->netcutoff)) {
        Mac_ItrPinOfNet(netPtr,DrawStartPin,(ClientData) showConnInfoPtr);
        Mac_ItrPinOfNet(netPtr,FreePinMarker,(ClientData)NULL);

        length = (showConnInfoPtr->xMax - showConnInfoPtr->xMin)
         + (showConnInfoPtr->yMax - showConnInfoPtr->yMin);
        showConnInfoPtr->netotal += length;
        if (length > showConnInfoPtr->netmax) showConnInfoPtr->netmax = length;
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
DrawStartPin(pinPtr, showConnInfoPtr)
PinInst *pinPtr;
ShowConnInfo *showConnInfoPtr;
{
    pinPtr->clientData = (ClientData) &showConnInfoPtr->marked;
    Mac_GetAbsPosOfPin(pinPtr,&showConnInfoPtr->pinPos);
    if (showConnInfoPtr->xMin == 0 && showConnInfoPtr->xMax == 0
        && showConnInfoPtr->yMin == 0 && showConnInfoPtr->yMax == 0) {
        showConnInfoPtr->xMin = showConnInfoPtr->xMax = 
            showConnInfoPtr->pinPos.p_x;
        showConnInfoPtr->yMin = showConnInfoPtr->yMax = 
            showConnInfoPtr->pinPos.p_y;
    }
    Mac_ItrPinOfNet(showConnInfoPtr->netPtr,DrawEndPin,
        (ClientData) showConnInfoPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
DrawEndPin(pinPtr, showConnInfoPtr)
PinInst *pinPtr;
ShowConnInfo *showConnInfoPtr;
{
    Point pinPos;

    if (pinPtr->clientData != (ClientData) &showConnInfoPtr->marked) {
        Mac_GetAbsPosOfPin(pinPtr,&pinPos);
        Ez_DrawWorldLine(showConnInfoPtr->winDataPtr->wd_userWin,
            showConnInfoPtr->winDataPtr->wd_scrMatrix,
            showConnInfoPtr->pinPos.p_x, showConnInfoPtr->pinPos.p_y,
            pinPos.p_x, pinPos.p_y, EZ_SOLIDLINE1, GXcopy,
#ifdef USE_OLDP_CODE
            pbTFEdgeColor, 
#else
            ez_HighLightColor, 
#endif /* USE_OLDP_CODE */
            ez_BackColor);
        if (pinPos.p_x < showConnInfoPtr->xMin) {
            showConnInfoPtr->xMin = pinPos.p_x;
        }
        if (pinPos.p_x > showConnInfoPtr->xMax) {
            showConnInfoPtr->xMax = pinPos.p_x;
        }
        if (pinPos.p_y < showConnInfoPtr->yMin) {
            showConnInfoPtr->yMin = pinPos.p_y;
        }
        if (pinPos.p_y > showConnInfoPtr->yMax) {
            showConnInfoPtr->yMax = pinPos.p_y;
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



/* ARGSUSED */
static BearStatus
FreePinMarker(pinPtr, clientData)
PinInst     *pinPtr;
ClientData  clientData;
{
    pinPtr->clientData = (ClientData) NULL;

    return (BEAR_CONTINUE_SEARCH);
}

#endif /* BEAR_BLACK_BOX */


