
/* pbXdefaults.c --
 *
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#if !defined(BEAR_BLACK_BOX)
/*
 * This file does not need to be compiled when the black box version of
 * the BEAR-FP system is built.
 */

#include "bearCopyRight.h"
#include "port.h"
#include "malloc.h"
#include "magic.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "ezPlot.h"
#include "polarBear.h"

/*
 * Below is a list of the currently defined X defaults, used by the
 * polarBear program complete with a short description of each.
 * 
 * Name                         Comment
 *  
 * chip.background.color:       Chip's background color
 * chip.cell.color:             Cell color
 * chip.cell.dummy.color:       Dummy Cell color
 * chip.cell.route.color:       Route Cell color
 * chip.cell.border.color:      Cell border and floor plan graph color
 * chip.channel.horz.color:     Horzontal channel color
 * chip.channel.vert.color:     Vertical  channel color
 * chip.channel.L.color:        L-Shaped  channel color
 * chip.pin.color:              Pin color
 * chip.net1.color:             Net 1 color
 * chip.net2.color:             Net 2 color
 * chip.net3.color:             Net 3 color
 * chip.tile.horzBN.color:      Horizontal bottleneck tile color
 * chip.tile.vertBN.color:      Vertical bottleneck tile color
 * chip.tile.horzDom.color:     Horizontal dominant tile color
 * chip.tile.vertDom.color:     Vertical dominant tile color
 *
 * TF.Leaf1.color:              Tree and Floorplan leaf color 1
 * TF.Leaf2.color:              Tree and Floorplan leaf color 2
 * TF.Leaf3.color:              Tree and Floorplan leaf color 3
 * TF.Leaf4.color:              Tree and Floorplan leaf color 4
 * TF.Leaf5.color:              Tree and Floorplan leaf color 5
 * TF.Node.color:               Tree and FloorPlan node color
 * TF.Edge.color:               Tree and Floorplan edge color
 */

/* ---------- Local Global Variables ---------- */
static ColorData colorData[] = {
#ifdef USE_OLDP_CODE
    { "TF.Leaf1.color", &pbTFLeafColor[0], &ez_WhitePixel, "cyan" },
    { "TF.Leaf2.color", &pbTFLeafColor[1], &ez_WhitePixel, RED },
    { "TF.Leaf3.color", &pbTFLeafColor[2], &ez_WhitePixel, "green" },
    { "TF.Leaf4.color", &pbTFLeafColor[3], &ez_WhitePixel, DARK_BLUE },
    { "TF.Leaf5.color", &pbTFLeafColor[4], &ez_WhitePixel, YELLOW },
    { "TF.Node.color", &pbTFNodeColor, &ez_WhitePixel, MED_BLUE },
    { "TF.Edge.color", &pbTFEdgeColor, &ez_BlackPixel, "white" },
#endif /* USE_OLDP_CODE */

    { "chip.background.color", &pbChipColor, &ez_BlackPixel, "SkyBlue" },
    { "chip.cell.color", &pbCellColor, &ez_WhitePixel, "cyan"},
    { "chip.cell.dummy.color", &pbDCellColor, &ez_WhitePixel, "#0684cb" },
    { "chip.cell.route.color", &pbRCellColor, &ez_WhitePixel, LAVENDER },
    { "chip.cell.border.color",&pbCellBorderColor, &ez_BlackPixel, DARK_BLUE },

    { "chip.channel.horz.color", 
        &pbHChannelColor,  &ez_BlackPixel, "#fefefe"   },

    { "chip.channel.vert.color", 
        &pbVChannelColor,  &ez_BlackPixel, "#87eafe"   },

    { "chip.channel.L.color", &pbLChannelColor,  &ez_BlackPixel, "#fdc4f8"   },
    { "chip.net1.color", &pbNet1Color, &ez_WhitePixel, LIGHT_BLUE },
    { "chip.net2.color", &pbNet2Color, &ez_WhitePixel, MED_BLUE },
    { "chip.net3.color", &pbNet3Color, &ez_WhitePixel, DARK_BLUE },
    { "chip.pin.color", &pbPinColor, &ez_BlackPixel, DARK_BLUE },
    { "chip.tile.horzBN.color", &pbHorzBnColor, &ez_BlackPixel, "#0589a8" },
    { "chip.tile.horzDom.color", &pbHorzDomColor, &ez_BlackPixel, "#3d11fd" },
    { "chip.tile.vertBN.color", &pbVertBnColor, &ez_BlackPixel, "#3d11fd" },
    { "chip.tile.vertDom.color", &pbVertDomColor, &ez_BlackPixel, "#0589a8" }
};
static int numOfColors = sizeof (colorData) / sizeof (*colorData);


BearStatus
PbGetXDefaults()
{
    if (Ez_SetUpColors(colorData, numOfColors) != EZ_OK) {
        return (BEAR_ERROR);
    }
        
    return (BEAR_OK);
}



#define gray_width 16
#define gray_height 16
static char gray_bits[] = {
   0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55,
   0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55,
   0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55};

#define solid_width 16
#define solid_height 16
static char solid_bits[] = {
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};


void
PbSetUpTilesAndBitmaps()
{
    if (ez_Options & EZ_COLOR_MON_OPT) {
        /* color display */
        pbCellPixmap = XCreateBitmapFromData(ez_Display, 
            EZ_ROOT_WINDOW,
            solid_bits, 
            (unsigned int) solid_width, 
            (unsigned int) solid_height);
        pbChipPixmap = pbCellPixmap;
    }
    else {
        /* black and white display */
        pbCellPixmap = XCreateBitmapFromData(ez_Display,
            EZ_ROOT_WINDOW,
            gray_bits, 
            (unsigned int) gray_width, 
            (unsigned int) gray_height);;
        pbChipPixmap = ez_BackPixmap;
    }
}

#endif /* BEAR_BLACK_BOX */


