
#include <stdio.h>
#include <strings.h>

#define TEST_LIST_FILE  "testList"
#define DATA_DIR        "testBearFP"

#define MAX_CF          4


typedef struct _CtlFileData CtlFileData;
struct _CtlFileData {
    char    *fileNamePtr;
    double  routeAdjust;
};


/* ---------- Local Procedures ---------- */
static void GenCtlFiles();
static void WritePlaceTest();
static void WriteCtlFile();


/* ---------- Local Global Variables ---------- */
static CtlFileData  cfData[MAX_CF];


main() 
{
    char        *alphabet     = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    int         randomFlag    = 1;
    int         greedyFlag    = 1;
    int         matchingFlag  = 1;
    int         annealingFlag = 1;

    FILE        *filePtr;
    double      aspectRatio[7];
    int         i, n, max;
    char        scriptName[80];

    if ((filePtr = fopen (TEST_LIST_FILE, "w")) == (FILE *) NULL) {
        (void) fprintf (stderr,
            "Could not open file named \"%s\" for writing.\n", TEST_LIST_FILE);
        exit (-1);
    }

    aspectRatio[0] = 0.5;
    aspectRatio[1] = 1.0;
    aspectRatio[2] = 2.0;
    aspectRatio[3] = 4.0;
    max = 4;

/*
 *  aspectRatio[0] = 0.7;
 *  aspectRatio[1] = 0.8;
 *  aspectRatio[2] = 0.9;
 *  aspectRatio[3] = 1.0;
 *  aspectRatio[4] = 1.1;
 *  aspectRatio[5] = 1.2;
 *  aspectRatio[6] = 1.3;
 *  max = 7;
 */

    GenCtlFiles();

    if (randomFlag) {
        for (n = 0; n < MAX_CF; n++) {
            (void) sprintf (scriptName, ":TEST_R_%d", n + 1);

            WritePlaceTest(scriptName, "r", aspectRatio[n],
                cfData[2].fileNamePtr);

            (void) fprintf (filePtr, "%s/%s\n", DATA_DIR, scriptName);
        }
    }
    if (greedyFlag) {
        for (n = 0; n < MAX_CF; n++) {
            (void) sprintf (scriptName, ":TEST_G_%d", n + 1);

            WritePlaceTest(scriptName, "g", aspectRatio[n],
                cfData[2].fileNamePtr);

            (void) fprintf (filePtr, "%s/%s\n", DATA_DIR, scriptName);
        }
    }
    if (matchingFlag) {
        for (n = 0; n < MAX_CF; n++) {
            (void) sprintf (scriptName, ":TEST_M_%d", n + 1);

            WritePlaceTest(scriptName, "m", aspectRatio[n],
                cfData[2].fileNamePtr);

            (void) fprintf (filePtr, "%s/%s\n", DATA_DIR, scriptName);
        }
    }
    if (annealingFlag) {
        for (n = 0; n < MAX_CF; n++) {
            (void) sprintf (scriptName, ":TEST_A_%d", n + 1);

            WritePlaceTest(scriptName, "a", aspectRatio[n],
                cfData[2].fileNamePtr);

            (void) fprintf (filePtr, "%s/%s\n", DATA_DIR, scriptName);
        }
    }
    (void) fclose (filePtr);

    if (i > 26) {
        (void) fprintf (stderr, "error : i > 26\n");
    }
}


static void
WritePlaceTest(scriptNamePtr, algPtr, aspectRatio, ctlFileNamePtr)
char    *scriptNamePtr;
char    *algPtr;
double  aspectRatio;
char    *ctlFileNamePtr;
{
    FILE    *filePtr;
    char    command[80], pathPtr[256], saveDir[256], *startPtr;

    (void) sprintf (pathPtr, "%s/%s", DATA_DIR, scriptNamePtr);
    if ((filePtr = fopen (pathPtr, "w")) == (FILE *) NULL) {
        (void) fprintf (stderr,
            "Could not open file named \"%s\" for writing.\n", pathPtr);
        exit (-1);
    }

    (void) fprintf (filePtr, "#! /bin/csh -f\n\n");

    (void) fprintf (filePtr, "set checkOpt    = \"\"\n");
    (void) fprintf (filePtr, "set nutOpt      = \"\"\n\n");

    (void) fprintf (filePtr, "set polarBear   = $1\n");
    (void) fprintf (filePtr, "set bfpDataFile = $2\n");
    (void) fprintf (filePtr, "set logFile     = $3\n\n");

    (void) fprintf (filePtr, "set checkFlag   = $4\n");
    (void) fprintf (filePtr, "set nutFlag     = $5\n\n");

    (void) fprintf (filePtr, "if (${checkFlag} == TRUE) then\n");
    (void) fprintf (filePtr, "    set checkOpt = -C\n");
    (void) fprintf (filePtr, "endif\n\n");

    (void) fprintf (filePtr, "set saveFlag = TRUE\n");
    (void) fprintf (filePtr,
   "set bblDataFile = `echo ${bfpDataFile} | sed -e \"s/\\.bfp/\\.r/g\"`\n\n");
    (void) fprintf (filePtr,
        "set cellName = `basename ${bfpDataFile} .bfp`\n\n");

    (void) fprintf (filePtr,
        "# ----- End of variable initialization -----\n\n\n");


    (void) fprintf (filePtr, "echo \\# >! bboxScript\n");
    (void) fprintf (filePtr, "echo \\# File name = %s >> bboxScript\n",
        scriptNamePtr);
    (void) fprintf (filePtr, "echo \\# >> bboxScript\n");
    (void) fprintf (filePtr, "echo \" \" >> bboxScript\n\n");

    (void) fprintf (filePtr, "echo log ${logFile} >> bboxScript\n\n");

    (void) fprintf (filePtr, "# ----- Test One -----\n\n");
    (void) fprintf (filePtr,
        "echo r ${checkOpt} -np -bfp ${bfpDataFile} >> bboxScript\n");

    (void) fprintf (filePtr,
        "echo ncl -alg %s -cf ./%s/%s >> bboxScript\n", algPtr, 
            DATA_DIR, ctlFileNamePtr);
    (void) fprintf (filePtr,
        "echo npl -m r %f -cf ./%s/%s >> bboxScript\n\n", aspectRatio, 
            DATA_DIR, ctlFileNamePtr);

    if ((startPtr = strchr(scriptNamePtr, ':')) == (char *) NULL) {
        startPtr = scriptNamePtr;
    }
    else {
        startPtr++;
    }
    (void) sprintf (saveDir, "./%s/%s", DATA_DIR, startPtr);

    (void) fprintf (filePtr, "if (${saveFlag} == TRUE) then\n");
    (void) fprintf (filePtr,
        "    echo w -bfp  %s/${cellName}_1.bfp >> bboxScript\n", saveDir);
/*
 *  (void) fprintf (filePtr,
 *      "    echo w -cif  %s/${cellName}_1.cif >> bboxScript\n", saveDir);
 *  (void) fprintf (filePtr,
 *      "    echo w -rst2 %s/${cellName}_1.rst2 >> bboxScript\n", saveDir);
 */
    (void) fprintf (filePtr, "endif\n\n");

/*
    (void) fprintf (filePtr, "# ----- Test Two -----\n\n");
    (void) fprintf (filePtr,
        "echo r ${checkOpt} -np -bfp ${bfpDataFile} >> bboxScript\n");

    (void) fprintf (filePtr,
        "echo ncl -alg %s -cf ./%s/%s >> bboxScript\n", algPtr, 
            DATA_DIR, "cf_2");
    (void) fprintf (filePtr,
        "echo npl -m r %f -cf ./%s/%s >> bboxScript\n\n", aspectRatio, 
            DATA_DIR, "cf_2");

    (void) fprintf (filePtr, "if (${saveFlag} == TRUE) then\n");
    (void) fprintf (filePtr,
        "    echo w -bfp  %s/${cellName}_2.bfp >> bboxScript\n", saveDir);
    (void) fprintf (filePtr,
        "    echo w -cif  %s/${cellName}_2.cif >> bboxScript\n", saveDir);
    (void) fprintf (filePtr,
        "    echo w -rst2 %s/${cellName}_2.rst2 >> bboxScript\n", saveDir);
    (void) fprintf (filePtr, "endif\n\n");
*/

    (void) fprintf (filePtr,
        "# ----- End of bear black box script generation -----\n\n\n");


    (void) fprintf (filePtr, "if (${polarBear} == dontRun) then\n");
    (void) fprintf (filePtr, "    cat bboxScript\n");
    (void) fprintf (filePtr, "    exit\n");
    (void) fprintf (filePtr, "endif\n\n");

    (void) fprintf (filePtr, "if (${saveFlag} == TRUE) then\n");
    (void) fprintf (filePtr, "    if (! (-e %s)) then\n", saveDir);
    (void) fprintf (filePtr, "        mkdir %s\n", saveDir);
    (void) fprintf (filePtr, "    endif\n");
    (void) fprintf (filePtr, "endif\n\n");

    (void) fprintf (filePtr, "${polarBear} -b bboxScript >&! pb.out\n");
    (void) fprintf (filePtr, "set pbStatus = ${status}\n\n");

    (void) fprintf (filePtr, "if ($pbStatus == 0) then\n");
    (void) fprintf (filePtr, "    head -2 %s/${cellName}_1.rst2 >! p1\n",
        saveDir);
    (void) fprintf (filePtr, "    tail -13 %s/${cellName}_1.rst2 >! p2\n",
        saveDir);
    (void) fprintf (filePtr, "    cat p1 p2 >! %s/${cellName}_1.rst2\n",
        saveDir);
    (void) fprintf (filePtr, "    \\rm -f p1 p2\n\n");

/*
    (void) fprintf (filePtr, "    head -2 %s/${cellName}_2.rst2 >! p1\n",
        saveDir);
    (void) fprintf (filePtr, "    tail -13 %s/${cellName}_2.rst2 >! p2\n",
        saveDir);
    (void) fprintf (filePtr, "    cat p1 p2 >! %s/${cellName}_2.rst2\n",
        saveDir);
    (void) fprintf (filePtr, "    \\rm -f p1 p2\n");
*/
    (void) fprintf (filePtr, "endif\n\n");

    (void) fprintf (filePtr, "exit ${pbStatus}\n\n");

    (void) fclose (filePtr);

    (void) sprintf (command, "chmod 770 %s", pathPtr);
    system (command);
}


static void
WriteCtlFile(ctlFileName, routeAdjust)
char    *ctlFileName;
double  routeAdjust;
{
    FILE    *filePtr;
    char    pathPtr[256];

    (void) sprintf (pathPtr, "%s/%s", DATA_DIR, ctlFileName);
    if ((filePtr = fopen (pathPtr, "w")) == (FILE *) NULL) {
        (void) fprintf (stderr,
            "Could not open control file named \"%s\" for writing.\n", 
                pathPtr);
        exit (-1);
    }

    (void) fprintf (filePtr, "newC.QuickRouteEstFlag          FALSE\n");
    (void) fprintf (filePtr, "newC.ChannelFreeFlag            FALSE\n");
    (void) fprintf (filePtr, "newC.TimingFlag                 FALSE\n\n");

    (void) fprintf (filePtr, "newC.RouteAdjust                %f\n\n",
        routeAdjust);

    (void) fprintf (filePtr, "newP.BorderSize                 1.5\n\n");

    (void) fprintf (filePtr, "newP.QuickPlanFlag              FALSE\n\n");

    (void) fprintf (filePtr, "newP.FinalOptFlag               TRUE\n");
    (void) fprintf (filePtr, "newP.FullEnumFlag               TRUE\n");
    (void) fprintf (filePtr, "newP.MaxLoopCnt                 2\n\n");

    (void) fprintf (filePtr, "newP.FullEnum.Tolerance         0.25\n");
    (void) fprintf (filePtr, "newP.FullEnum.AreaWeight        0.7\n");
    (void) fprintf (filePtr, "newP.FullEnum.WireWeight        0.3\n");
    (void) fprintf (filePtr, "newP.FullEnum.CritWeight        1.0\n\n");

    (void) fprintf (filePtr, "newP.PartialEnum.Tolerance      0.15\n");
    (void) fprintf (filePtr, "newP.PartialEnum.WireWeight     1.0\n");
    (void) fprintf (filePtr, "newP.PartialEnum.CritWeight     1.0\n");

    (void) fclose (filePtr);
}


static void
GenCtlFiles()
{
    int n;

    cfData[0].fileNamePtr = "cf_1";
    cfData[0].routeAdjust = 0.25;

    cfData[1].fileNamePtr = "cf_2";
    cfData[1].routeAdjust = 0.5;

    cfData[2].fileNamePtr = "cf_3";
    cfData[2].routeAdjust = 1.0;

    cfData[3].fileNamePtr = "cf_4";
    cfData[3].routeAdjust = 4.0;

    for (n = 0; n < MAX_CF; n++) {
        WriteCtlFile(cfData[n].fileNamePtr, cfData[n].routeAdjust);
    }
}


