
/* soIO.c --
 *
 *     Massoud Pedram
 *     George Carvalho, Esq.
 *     (Advising Professor - E. S. Kuh)
 *
 * This module contains input output routine for shapeOptimizer module.
 *
 * Copyright(C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header: soIO.c,v 07/06/88 massoud Exp $";
#endif not lint

#include "soInt.h"
#include "soUtil.h"
#include <math.h>
#include <string.h>


/* ARGSUSED */
static BearStatus
InitShapeConstr(cellInstPtr, clientData)
CellInst    *cellInstPtr;
ClientData  clientData;
{
    CellMaster  *cellMasterPtr;
    Rect        bBox;
    int         dimX, dimY;
    ShapeConstr *shapeConstrPtr;

    cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);
    cellMasterPtr->shapeFlag = MAC_FIXED_SHAPE;
    bBox = *Mac_GetBBoxOfCell(cellInstPtr);
    dimX = bBox.r_ur.p_x - bBox.r_ll.p_x;
    dimY = bBox.r_ur.p_y - bBox.r_ll.p_y;
    MALLOC(ShapeConstr *, shapeConstrPtr, sizeof(ShapeConstr));
    shapeConstrPtr->minArea = 
    shapeConstrPtr->tarArea =
    shapeConstrPtr->maxArea = dimX * dimY;
    shapeConstrPtr->minAspectRatio.numerator = 
    shapeConstrPtr->tarAspectRatio.numerator =
    shapeConstrPtr->maxAspectRatio.numerator = dimX;
    shapeConstrPtr->minAspectRatio.denominator = 
    shapeConstrPtr->tarAspectRatio.denominator =
    shapeConstrPtr->maxAspectRatio.denominator = dimY;
    cellMasterPtr->shapeConstr = shapeConstrPtr;

    return(BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * So_ReadShapeConstr() --
 *
 * Results:
 *    BEAR_OK is returned if everything is spiffy.
 *    BEAR_ERROR_2 if the file could not be opened for reading.
 *    BEAR_ERROR otherwise.
 *
 * Side effects:
 *    None.
 * 
 * ----------------------------------------------------------------------------
 */

BearStatus
So_ReadShapeConstr(flexFileNamePtr)
char *flexFileNamePtr;
{
#define MAX_LINE_SIZE 1024

#define RETURN_ERR \
{ \
    fclose (filePtr); \
    return (BEAR_ERROR); \
}

    FILE        *filePtr;
    CellMaster  *cellMasterPtr;
    CellInst    *cellInstPtr;
    Rect        bBox;
    ShapeConstr *shapeConstrPtr;
    double      area, minAR, maxAR;
    int         flexCellCount;
    int         dimX, dimY;
    int         count;
    bool        eofFlag;
    char        cellName[MAX_LINE_SIZE];
    char        line[MAX_LINE_SIZE];
    char        *tokenPtr;

    if ((filePtr = fopen (flexFileNamePtr, "r")) == NIL(FILE)) {
        (void) sprintf (bear_ErrorMsg,
            "Could not open flex file named \"%s\" for reading.",
            flexFileNamePtr);
        Bear_PrintErr(bear_ErrorMsg);
        return (BEAR_ERROR_2);
    }

    flexCellCount = 0;
    eofFlag = FALSE;
    while (TRUE) {
        if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) ==
            BEAR_ERROR) {
            RETURN_ERR;
        }
        if (eofFlag == TRUE) {
            fclose (filePtr);
            break;
        }

        tokenPtr = strtok(line, " ");
        if (strcmp(tokenPtr, "Version") == 0) {
            /* Someday we may want to verify the version number. */
            continue;
        }
        else if (strcmp(tokenPtr, "FlexibleCellList") == 0) {
            while (TRUE) {
                if (Bear_ReadLine(line, MAX_LINE_SIZE, filePtr, &eofFlag) ==
                    BEAR_ERROR) {
                    RETURN_ERR;
                }
                if (eofFlag == TRUE) {
           Bear_PrintErr("So_ReadShapeConstr() failed : eof detected to soon.");
                    RETURN_ERR;
                }

                if (line[0] == ';') {
                    break;
                }

                count = sscanf (line, "%s %lf %lf", cellName, &minAR, &maxAR);
                if (count != 3) {
    Bear_PrintErr("So_ReadShapeConstr() failed : missing flexible cell data.");
                    RETURN_ERR;
                }

                if (Mac_IsMember(CELL_INSTANCE_HT, cellName) == TRUE) {
                    cellInstPtr = 
                        (CellInst *) Mac_LookUp(CELL_INSTANCE_HT, cellName);
                }
                else { 
                    (void) sprintf (bear_ErrorMsg, 
                        "So_ReadShapeConstr() : Cannot find cell named \"%s\"", 
                        cellName);
                    Bear_PrintErr(bear_ErrorMsg);
                    RETURN_ERR;
                }

                bBox = *Mac_GetBBoxOfCell(cellInstPtr);
                dimX = bBox.r_ur.p_x - bBox.r_ll.p_x; 
                dimY = bBox.r_ur.p_y - bBox.r_ll.p_y;
                if (dimX < 0) {
                    dimX *= -1;
                }
                if (dimY < 0) {
                    dimY *= -1;
                }

                cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);
                if ((shapeConstrPtr = cellMasterPtr->shapeConstr) ==
                    NIL(ShapeConstr)) {
                    MALLOC(ShapeConstr * ,
                        shapeConstrPtr, sizeof (ShapeConstr));
                }
                cellMasterPtr->shapeFlag = MAC_FLEX_SHAPE;

                if ((minAR <= 0.0) || (maxAR <= 0.0)) {
                    (void) sprintf (bear_ErrorMsg,
                   "Non-positive aspect constraint for cell %s in file %s!", 
                        cellName, flexFileNamePtr);
                    Bear_PrintErr(bear_ErrorMsg);
                    RETURN_ERR;
                }
                if (minAR > maxAR) {
                    (void) sprintf (bear_ErrorMsg,
                     "minAspectRatio > maxAspectRatio for cell %s in file %s!", 
                        cellName, flexFileNamePtr);
                    Bear_PrintErr(bear_ErrorMsg);
                    RETURN_ERR;
                }

                area = ((double) dimX) * ((double) dimY);
                /* 
                 * minX = minAspectRatio.numerator
                 * maxY = minAspectRatio.denominato
                 * maxX = maxAspectRatio.numerator
                 * minY = maxAspectRatio.denominator
                 */
                shapeConstrPtr->minAspectRatio.numerator = 
                    (int) sqrt(area / maxAR);
                shapeConstrPtr->minAspectRatio.denominator =
                    (int) sqrt(area * maxAR);
                shapeConstrPtr->maxAspectRatio.numerator =
                    (int) sqrt(area / minAR);
                shapeConstrPtr->maxAspectRatio.denominator =
                    (int) sqrt(area * minAR);

                if ((dimX < shapeConstrPtr->minAspectRatio.numerator) ||
                    (dimX > shapeConstrPtr->maxAspectRatio.numerator) ||
                    (dimY < shapeConstrPtr->maxAspectRatio.denominator) ||
                    (dimY > shapeConstrPtr->minAspectRatio.denominator)) {

                   (void) sprintf (bear_ErrorMsg,
                "Current shape of modeule %s does NOT satisfy aspect constraints given for cell in file %s!", 
                        cellName, flexFileNamePtr);
                    Bear_PrintErr(bear_ErrorMsg);
                    RETURN_ERR;
                }
                shapeConstrPtr->minArea =
                    shapeConstrPtr->minAspectRatio.numerator *
                        shapeConstrPtr->minAspectRatio.denominator;
                shapeConstrPtr->maxArea =
                    shapeConstrPtr->maxAspectRatio.numerator *
                        shapeConstrPtr->maxAspectRatio.denominator;

                cellMasterPtr->shapeConstr = shapeConstrPtr;

                flexCellCount++;
            }
        }
        else {
            Bear_PrintErr("So_ReadShapeConstr() failed : unknown keyword.");
            RETURN_ERR;
        }
    }

    if (flexCellCount > 0) {
        (void) fprintf (stdout,
            "Read flexible cell data from file named \"%s\"\n\n",
            flexFileNamePtr);
    }
    else {
        (void) fprintf (stdout,
            "No flexible cell data in file named \"%s\"\n\n",
            flexFileNamePtr);
    }
    fflush (stdout);

    return(BEAR_OK);

#undef RETURN_ERR
#undef MAX_LINE_SIZE
}



/*
 * ----------------------------------------------------------------------------
 *
 * So_DefineShapes() --
 *
 *   Main procedure which initializes shapes of individual cells
 *   
 * Results:
 *  
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
So_DefineShapes(coreMasterPtr, flexFileNamePtr)
CellMaster  *coreMasterPtr;
char        *flexFileNamePtr;
{
    (void) Mac_ItrChildrenOfCell(coreMasterPtr, InitShapeConstr,
        (ClientData) NULL);

    return (So_ReadShapeConstr(flexFileNamePtr));
}



/* ARGSUSED */
static int
PrintHorzSlackInfoOfTile(tilePtr, clientData)
Tile        *tilePtr;
ClientData  clientData;
{
    if(Tp_IsCellInstTile(tilePtr) || Tp_IsBottleneckTile(tilePtr) ||
        Tp_IsDCellTile(tilePtr))
    {
        TileProp    *tilePropPtr;
        SlackInfo    *slackInfoPtr;
        CellInst    *cellInstPtr;
        DummyCell    *dummyCellPtr;
        Rect        *bBoxPtr;
        int            dimX, dimY;

        if(tilePtr->ti_client ==(ClientData) MINFINITY) {
            Bear_PrintFatalErr("PrintSlackInfoOfTile() failed: no tile prop is given");
        }

        tilePropPtr =(TileProp *) tilePtr->ti_client;
        slackInfoPtr =(SlackInfo *) tilePropPtr->clientData;

        if(slackInfoPtr == NIL(SlackInfo)) {
            Bear_PrintFatalErr("PrintSlackInfoOfTile() failed: no tile prop is given");
        }

        printf("\n");

        if(Tp_IsCellInstTile(tilePtr)) {
            cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
            printf("cell instance name: %s \n", 
                Mac_GetNameOfCellInst(cellInstPtr));
        } else if(Tp_IsBottleneckTile(tilePtr)) {
            printf("bearId: %d \n", Tp_GetIdOfTile(tilePtr));
        } else {
            dummyCellPtr = Tp_GetDCellOfTile(tilePtr);
            printf("bearId: %d \n", 
                Mac_GetIdOfDCell(dummyCellPtr));
        }

        puts(Tp_GetTypeNameOfTile(tilePtr));
        printf("horzPosSlack: %d, horzNegSlack: %d \n", 
                slackInfoPtr->horzSlack.pos, slackInfoPtr->horzSlack.neg);

        if(Tp_IsCellInstTile(tilePtr)) {
            bBoxPtr = Mac_GetBBoxOfCell(cellInstPtr);
            dimX = bBoxPtr->r_xtop - bBoxPtr->r_xbot;
            dimY = bBoxPtr->r_ytop - bBoxPtr->r_ybot;
            printf("cell X dimension: %d, cell Y dimension: %d \n", dimX, dimY);
        } else if(Tp_IsBottleneckTile(tilePtr)) {
            printf("capacity: %d, density: %d \n",
                Tp_GetCapacityOfTile(tilePtr), Tp_GetDensityOfTile(tilePtr));
        } else {
            bBoxPtr = Mac_GetBBoxOfDCell(dummyCellPtr);
            dimX = bBoxPtr->r_xtop - bBoxPtr->r_xbot;
            dimY = bBoxPtr->r_ytop - bBoxPtr->r_ybot;
            printf("cell X dimension: %d, cell Y dimension: %d \n", dimX, dimY);
        }

        fflush(stdout);
    }

    return(MAGIC_CONTINUE_SEARCH);
}



/* ARGSUSED */
static int
PrintVertSlackInfoOfTile(tilePtr, clientData)
Tile        *tilePtr;
ClientData  clientData;
{
    if(Tp_IsCellInstTile(tilePtr) || Tp_IsBottleneckTile(tilePtr) ||
        Tp_IsDCellTile(tilePtr)) {
        TileProp    *tilePropPtr;
        SlackInfo    *slackInfoPtr;
        CellInst    *cellInstPtr;
        DummyCell   *dummyCellPtr;
        Rect        *bBoxPtr;
        int            dimX, dimY;

        if(tilePtr->ti_client ==(ClientData) MINFINITY) {
            Bear_PrintFatalErr("PrintSlackInfoOfTile() failed: no tile prop is given");
        }

        tilePropPtr =(TileProp *) tilePtr->ti_client;
        slackInfoPtr =(SlackInfo *) tilePropPtr->clientData;

        if(slackInfoPtr == NIL(SlackInfo)) {
            Bear_PrintFatalErr("PrintSlackInfoOfTile() failed: no tile prop is given");
        }

        printf("\n");

        if(Tp_IsCellInstTile(tilePtr)) {
            cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
            printf("cell instance name: %s \n", 
                Mac_GetNameOfCellInst(cellInstPtr));
        } else if(Tp_IsBottleneckTile(tilePtr)) {
            printf("bearId: %d \n", Tp_GetIdOfTile(tilePtr));
        } else {
            dummyCellPtr = Tp_GetDCellOfTile(tilePtr);
            printf("bearId: %d \n", 
                Mac_GetIdOfDCell(dummyCellPtr));
        }

        puts(Tp_GetTypeNameOfTile(tilePtr));
        printf("vertPosSlack: %d, vertNegSlack: %d \n", 
            slackInfoPtr->vertSlack.pos, slackInfoPtr->vertSlack.neg);

        if(Tp_IsCellInstTile(tilePtr)) {
            bBoxPtr = Mac_GetBBoxOfCell(cellInstPtr);
            dimX = bBoxPtr->r_xtop - bBoxPtr->r_xbot;
            dimY = bBoxPtr->r_ytop - bBoxPtr->r_ybot;
            printf("cell X dimension: %d, cell Y dimension: %d \n", dimX, dimY);
        } else if(Tp_IsBottleneckTile(tilePtr)) {
            printf("capacity: %d, density: %d \n",
                Tp_GetCapacityOfTile(tilePtr), Tp_GetDensityOfTile(tilePtr));
        } else {
            bBoxPtr = Mac_GetBBoxOfDCell(dummyCellPtr);
            dimX = bBoxPtr->r_xtop - bBoxPtr->r_xbot;
            dimY = bBoxPtr->r_ytop - bBoxPtr->r_ybot;
            printf("cell X dimension: %d, cell Y dimension: %d \n", dimX, dimY);
        }

        fflush(stdout);
    }

    return(MAGIC_CONTINUE_SEARCH);
}


/*
 * ----------------------------------------------------------------------------
 *
 * SoPrintHorzSlackInfo --
 *
 * Results:
 *
 * Side Effects:
 *
 * ----------------------------------------------------------------------------
 */

void
SoPrintHorzSlackInfo()
{

    printf("\n");
    printf("Printing horizental solid and bottleneck tiles slacks ... \n");
    (void) TiSrArea((Tile *) NULL,
        Tp_GetHorzPlane(soGlobalPlaneDataPtr),
        Tp_GetHorzPlaneBnd(soGlobalPlaneDataPtr),
        PrintHorzSlackInfoOfTile,
        NIL(ClientData));
}


/*
 * ----------------------------------------------------------------------------
 *
 * SoPrintVertSlackInfo --
 *
 * Results:
 *
 * Side Effects:
 *
 * ----------------------------------------------------------------------------
 */

void
SoPrintVertSlackInfo()
{

    Rect        vertSearchRect, *horzPlaneBndPtr;

    horzPlaneBndPtr = Tp_GetHorzPlaneBnd(soGlobalPlaneDataPtr);
    Tp_HRectToVRect(horzPlaneBndPtr, &vertSearchRect);

    printf("\n");
    printf("Printing vertical solid and bottleneck tiles slacks ... \n");
    (void) TiSrArea((Tile *) NULL,
        Tp_GetVertPlane(soGlobalPlaneDataPtr),
        &vertSearchRect,
        PrintVertSlackInfoOfTile,
        NIL(ClientData));

}


/*
 * ----------------------------------------------------------------------------
 *
 * SoPrintBNTile() --
 *
 *   Outputs slackInfo about this cellInst tile.
 *   
 * Results:
 *  
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

void
SoPrintBNTile(tilePtr, planePtr, printSlack)
Tile    *tilePtr;
Plane    *planePtr;
bool    printSlack;
{
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;

    if(Tp_IsBottleneckTile(tilePtr)) {
        printf("\n");
        tilePropPtr =(TileProp *) tilePtr->ti_client;
        slackInfoPtr =(SlackInfo *) tilePropPtr->clientData;
        if(planePtr == Tp_GetHorzPlane(soGlobalPlaneDataPtr)) {
            printf("Bottleneck plane tile: SO_HORZ \n");
            if(printSlack) {
                printf("HorzPosSlack: %d, HorzNegSlack: %d \n", 
                  slackInfoPtr->horzSlack.pos, slackInfoPtr->horzSlack.neg);
            }
        } else {
            printf("Bottleneck plane tile: SO_VERT \n");
            if(printSlack) {
                printf("VertPosSlack: %d, VertNegSlack: %d \n", 
                  slackInfoPtr->vertSlack.pos, slackInfoPtr->vertSlack.neg);
            }
        }
        printf("Capacity: %d, Density: %d \n",
            Tp_GetCapacityOfTile(tilePtr), Tp_GetDensityOfTile(tilePtr));
    } else {
        Bear_PrintErr("SoPrintBNTile() failed, tile is not a bottleneck. ");
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * SoPrintBNLine() --
 *
 * Results:
 *  
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

void
SoPrintBNLine(linePtr, planePtr)
BNLine   *linePtr;
Plane    *planePtr;
{

    printf("\n");
    if(planePtr == Tp_GetHorzPlane(soGlobalPlaneDataPtr)) {
        printf("Bottleneck plane tile: SO_HORZ \n");
    } else {
        printf("Bottleneck plane tile: SO_VERT \n");
    }
    printf("Capacity: %d, Density: %d \n",
        Tp_GetCapacityOfBnLine(linePtr), Tp_GetDensityOfBnLine(linePtr));
}


/*
 * ----------------------------------------------------------------------------
 *
 * SoPrintCellInstTile() --
 *
 *   Outputs slackInfo about this cellInst tile.
 *   
 * Results:
 *  
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

void
SoPrintCellInstTile(tilePtr, planePtr)
Tile    *tilePtr;
Plane    *planePtr;

{
    Tile        *vertTilePtr, *horzTilePtr;
    TileProp    *tilePropPtr;
    SlackInfo   *slackInfoPtr;
    int            horzSlk, vertSlk;
    TileInfo    *tileInfoPtr;
    CellInst    *cellInstPtr;

    if(Tp_IsCellInstTile(tilePtr)) {
        cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
        tileInfoPtr =(TileInfo *) cellInstPtr->clientData;

        if(tileInfoPtr == NIL(TileInfo)) {
            Bear_PrintFatalErr("SoPrintCellInstTile() failed, NIL tileInfo. \n");
        }

        if(planePtr == Tp_GetHorzPlane(soGlobalPlaneDataPtr)) {
            printf("cell instance name: %s, plane type: SO_HORZ \n",
            Mac_GetNameOfCellInst(Tp_GetCellInstOfTile(tilePtr)));
            tilePropPtr =(TileProp *) tilePtr->ti_client;
            slackInfoPtr =(SlackInfo *) tilePropPtr->clientData;
            horzSlk = slackInfoPtr->horzSlack.neg - slackInfoPtr->horzSlack.pos;
            vertTilePtr = tileInfoPtr->vPtr;

            tilePropPtr =(TileProp *) vertTilePtr->ti_client;
            slackInfoPtr =(SlackInfo *) tilePropPtr->clientData;
            vertSlk = slackInfoPtr->vertSlack.neg - slackInfoPtr->vertSlack.pos;
            printf("horz slack: %d, vert slack: %d \n", horzSlk, vertSlk);
        } else if(planePtr == Tp_GetVertPlane(soGlobalPlaneDataPtr)) {
            printf("cell instance name: %s, plane type: SO_VERT \n",
            Mac_GetNameOfCellInst(Tp_GetCellInstOfTile(tilePtr)));
            tilePropPtr =(TileProp *) tilePtr->ti_client;
            slackInfoPtr =(SlackInfo *) tilePropPtr->clientData;
            vertSlk = slackInfoPtr->vertSlack.neg - slackInfoPtr->vertSlack.pos;
            horzTilePtr = tileInfoPtr->hPtr;

            tilePropPtr =(TileProp *) horzTilePtr->ti_client;
            slackInfoPtr =(SlackInfo *) tilePropPtr->clientData;
            horzSlk = slackInfoPtr->horzSlack.neg - slackInfoPtr->horzSlack.pos;
            printf("horz slack: %d, vert slack: %d \n", horzSlk, vertSlk);
        }
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * SoPrintTileList() --
 *
 *   Outputs information about cellInst tiles in the list.
 *   
 * Results:
 *  
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

void
SoPrintTileList(lnkTilePtr, planePtr)
LinkedTile    *lnkTilePtr;
Plane        *planePtr;
{
    LinkedTile    *ltPtr;

    for(ltPtr = lnkTilePtr; ltPtr != NIL(LinkedTile); ltPtr = ltPtr->nextPtr) {
        SoPrintCellInstTile(ltPtr->tilePtr, planePtr);
    }
}


