
/* soShapeFunc.c --
 *
 *    Massoud Pedram
 *
 * Copyright(C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "soInt.h"
#define LARGE_DENSITY 9999


/* ---------- Local Procedures ---------- */
static BearStatus SoItrBnLines();
static BearStatus GetBottleneckLineInfo();


/* ---------- so ShapeFunction ---------- */


static int
SoItrTiles(tilePtr, clientData)
Tile       *tilePtr;
ClientData clientData;
{
    if (Tp_IsBottleneckTile(tilePtr)) {
        Tp_PutCapacityOfTile(0, tilePtr);
        Tp_PutDensityOfTile(- LARGE_DENSITY, tilePtr);
    }
    return(MAGIC_CONTINUE_SEARCH);
}

static BearStatus
SoItrBnLines(bnLinePtr, clientData)
BNLine     *bnLinePtr;
ClientData clientData;
{
    Tp_PutCapacityOfBnLine(0, bnLinePtr);
    Tp_PutDensityOfBnLine(- LARGE_DENSITY, bnLinePtr);
    return(BEAR_CONTINUE_SEARCH);
} 

void
ForceDecomp(density, bnTilePtr)
int        density;
Tile       *bnTilePtr;
{
    WallJunction    *fpgPtr;

    TiSrArea((Tile *)NULL,
        Tp_GetHorzPlane(soGlobalPlaneDataPtr),
        Tp_GetHorzPlaneBnd(soGlobalPlaneDataPtr),
        SoItrTiles, (ClientData) NULL);
    TiSrArea((Tile *)NULL,
        Tp_GetVertPlane(soGlobalPlaneDataPtr),
        Tp_GetVertPlaneBnd(soGlobalPlaneDataPtr),
        SoItrTiles, (ClientData) NULL);

    Tp_ItrHorzBnLine(soGlobalPlaneDataPtr,SoItrBnLines,(ClientData)NULL);
    Tp_ItrVertBnLine(soGlobalPlaneDataPtr,SoItrBnLines,(ClientData)NULL);

    if(density < 0) {
        Tp_PutDensityOfTile(density, bnTilePtr);
    } else {
        Tp_PutDensityOfTile(0, bnTilePtr);
        Bear_PrintErr("ForceDecomp() failed, density is non-negative.");
    }
    /* calculate cap and den of bn tiles */
    (void) Gs_CalcCapDenOfTilePlane(soGlobalParentCellPtr);
    /* initialize cap and den of non-bn tileSpace tiles */
    Gs_InitSpaceTile(soGlobalPlaneDataPtr,soGlobalParentCellPtr);
    if(Tp_IsHorzTile(bnTilePtr)) {
        (void) Gs_HorzDecompact(soGlobalPlaneDataPtr);
    } else {
        (void) Gs_VertDecompact(soGlobalPlaneDataPtr);
    }
    /* determines the type of each tileSpace tile */
    Tp_IdentifySpaceTiles(soGlobalPlaneDataPtr);
    Fpg_CreateFloorPlanGraph(soGlobalPlaneDataPtr, &fpgPtr);
    (void) Gs_CalcCapDenOfTilePlane(soGlobalParentCellPtr);
}


static int
GetBottleneckTileInfo(tilePtr, bnInfoPtr)
Tile        *tilePtr;
BNInfo        *bnInfoPtr;
{
    int        bnSpace;

    if(Tp_IsBottleneckTile(tilePtr)) {
        bnSpace = Tp_GetCapacityOfTile(tilePtr) - Tp_GetDensityOfTile(tilePtr);
        if(bnSpace < bnInfoPtr->tileSpace) {
            bnInfoPtr->tileSpace = bnSpace;
            bnInfoPtr->tilePtr = tilePtr;
        }
    }

    return(MAGIC_CONTINUE_SEARCH);
}


static bool
PointInsideRect(pt, rect)
Point  *pt;
Rect   *rect;
{
    if(pt->p_x >= rect->r_xbot && pt->p_x <= rect->r_xtop &&
       pt->p_y >= rect->r_ybot && pt->p_y <= rect->r_ytop) return(TRUE);
    else return(FALSE);
}

static BearStatus
GetBottleneckLineInfo(bnLinePtr, bnInfoPtr)
BNLine        *bnLinePtr;
BNInfo        *bnInfoPtr;
{
    int       bnSpace;
    Line      *linePtr; 
    Point     p1, p2;

    linePtr = Tp_GetLineOfBnLine(bnLinePtr);
    switch(bnInfoPtr->transformType) {
    case HtoV:
        Tp_HPointToVPoint(&(linePtr->point1), &p1);
        Tp_HPointToVPoint(&(linePtr->point2), &p2);
        break;
    case VtoH:
        Tp_VPointToHPoint(&(linePtr->point1), &p1);
        Tp_VPointToHPoint(&(linePtr->point1), &p2);
        break;    
    case IDENTITY:
        Mac_CopyPoint(&p1, &(linePtr->point1));
        Mac_CopyPoint(&p2, &(linePtr->point2));
        break;    
    default:
        Bear_PrintFatalErr("GetBottleneckLineInfo() failed, programming error.");
        break;
    }
    if(PointInsideRect(&p1, bnInfoPtr->searchRectPtr) ||
       PointInsideRect(&p2, bnInfoPtr->searchRectPtr)) {
        bnSpace = Tp_GetCapacityOfBnLine(bnLinePtr) - 
            Tp_GetDensityOfBnLine(bnLinePtr);
        if(bnSpace < bnInfoPtr->bnLineSpace) {
            bnInfoPtr->bnLineSpace = bnSpace;
            bnInfoPtr->bnLinePtr = bnLinePtr;
        }
    }
    return(BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * SoShapeFunc --
 *
 * Results: resizes cellMaster of the cell by increasing its dimension along
 *    slkDir(opposite direction of critTileInfo.planeDir) by 
 *    critTileInfo.slk amount, keeping area fixed.
 *
 * Side Effects:
 *
 * ----------------------------------------------------------------------------
 */


SoStatus
SoShapeFunc(critTileInfoPtr, dataPtr ,parmsPtr)
CritTileInfo    *critTileInfoPtr;
ResizeData      *dataPtr;
So_UserParms    *parmsPtr;
{
    Rect            hPlaneSearchRect, vPlaneSearchRect;
    BNInfo          bnInfo;
    int             hFsblSpace=0, vFsblSpace=0;
    int             bottomBNSpace, topBNSpace, leftBNSpace, rightBNSpace;
    Rect            newBox, bBox;
    int             dimX, dimY;
    Tile            *bottmBNTilePtr, *topBNTilePtr;
    Tile            *leftBNTilePtr, *rightBNTilePtr;

    if(! Tp_IsCellInstTile(critTileInfoPtr->tilePtr)) {
        Bear_PrintFatalErr("SoShapeFunc() failed, tilePtr is not a CellInst. ");
        return(SO_NOT_FINISHED);
    }

    switch(parmsPtr->style) {
    case BBL:
        if(critTileInfoPtr->planeDir == (PlaneDirection) SO_HORZ) {
            Tm_HUnify(soGlobalParentCellPtr);
            (void) Gs_CalcCapDenOfTilePlane(soGlobalParentCellPtr);

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            hPlaneSearchRect.r_xbot = RIGHT(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_ybot = BOTTOM(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_xtop = RIGHT(critTileInfoPtr->tilePtr) + 1;
            hPlaneSearchRect.r_ytop = TOP(critTileInfoPtr->tilePtr) + 1;

            (void) TiSrArea(NIL(Tile),
                    Tp_GetHorzPlane(soGlobalPlaneDataPtr),
                    &hPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &hPlaneSearchRect;
            bnInfo.transformType = IDENTITY;;
            (void) Tp_ItrHorzBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine)) {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on right of hTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Right bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr));
                }
            }

            rightBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            hPlaneSearchRect.r_xbot = LEFT(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_ybot = BOTTOM(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_xtop = LEFT(critTileInfoPtr->tilePtr) + 1;
            hPlaneSearchRect.r_ytop = TOP(critTileInfoPtr->tilePtr) + 1;

            (void) TiSrArea(NIL(Tile),
                    Tp_GetHorzPlane(soGlobalPlaneDataPtr),
                    &hPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &hPlaneSearchRect;
            bnInfo.transformType = IDENTITY;;
            (void) Tp_ItrHorzBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine))
            {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on left of hTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Left bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr));
                }
            }

            leftBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);

            Tm_HSplit(soGlobalParentCellPtr);

            Tm_VUnify(soGlobalParentCellPtr);
            (void) Gs_CalcCapDenOfTilePlane(soGlobalParentCellPtr);

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            hPlaneSearchRect.r_xbot = LEFT(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_ybot = TOP(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_xtop = RIGHT(critTileInfoPtr->tilePtr) + 1;
            hPlaneSearchRect.r_ytop = TOP(critTileInfoPtr->tilePtr) + 1;
            Tp_HRectToVRect(&hPlaneSearchRect, &vPlaneSearchRect);

            (void) TiSrArea(NIL(Tile),
                    Tp_GetVertPlane(soGlobalPlaneDataPtr),
                    &vPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &vPlaneSearchRect;
            bnInfo.transformType = HtoV;
            (void) Tp_ItrVertBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine)) {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on top of hTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Top bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr));
                }
            }

            topBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);
            topBNTilePtr = bnInfo.tilePtr;

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            hPlaneSearchRect.r_xbot = LEFT(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_ybot = BOTTOM(critTileInfoPtr->tilePtr) - 1;
            hPlaneSearchRect.r_xtop = RIGHT(critTileInfoPtr->tilePtr) + 1;
            hPlaneSearchRect.r_ytop = BOTTOM(critTileInfoPtr->tilePtr) + 1;
            Tp_HRectToVRect(&hPlaneSearchRect, &vPlaneSearchRect);

            (void) TiSrArea(NIL(Tile),
                    Tp_GetVertPlane(soGlobalPlaneDataPtr),
                    &vPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &vPlaneSearchRect;
            bnInfo.transformType = HtoV;
            (void) Tp_ItrVertBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine)) {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on bottom of hTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Bottom bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr));
                }
                printf("\n");
            }

            bottomBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);

            vFsblSpace = topBNSpace + bottomBNSpace;
            bottmBNTilePtr = bnInfo.tilePtr;

            if(vFsblSpace >= critTileInfoPtr->slk) {
                dataPtr->resize.p_y = critTileInfoPtr->slk;
            } else {
                dataPtr->bnDensity = critTileInfoPtr->slk - vFsblSpace;
                dataPtr->bnTilePtr = topBNTilePtr;
                if(topBNTilePtr != NIL(Tile) && bottmBNTilePtr != NIL(Tile))
                {
                    ForceDecomp(- dataPtr->bnDensity, dataPtr->bnTilePtr);
                    dataPtr->resize.p_y = critTileInfoPtr->slk;
                } else {
                    dataPtr->resize.p_x = 0;
                    dataPtr->resize.p_y = 0;
                    dataPtr->move.p_x = 3;
                    dataPtr->move.p_y = 0;
                    Tm_VSplit(soGlobalParentCellPtr);
                    return(SO_FINISHED);
                }
            }

            bBox = *Mac_GetBBoxOfCell(dataPtr->cellInstPtr);
            GeoTransRect(&(dataPtr->cellInstPtr->transform), &bBox, &newBox);
            dimX = newBox.r_ur.p_x - newBox.r_ll.p_x;
            dimY = newBox.r_ur.p_y - newBox.r_ll.p_y;
            if(dimX < 0) dimX *= -1; if(dimY < 0) dimY *= -1;
            dataPtr->resize.p_x = (int)
                ((double) -(dimX * dataPtr->resize.p_y) /
                (double)(dimY + dataPtr->resize.p_y));
            if(-dataPtr->resize.p_x >= dimX)
                Bear_PrintFatalErr("SoShapeFunc() failed,programming error.");
            dataPtr->move.p_y = - bottomBNSpace;
            dataPtr->move.p_x = 0;

            Tm_VSplit(soGlobalParentCellPtr);
            return(SO_NOT_FINISHED);
        } else {
            Tm_VUnify(soGlobalParentCellPtr);
            (void) Gs_CalcCapDenOfTilePlane(soGlobalParentCellPtr);

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            vPlaneSearchRect.r_xbot = RIGHT(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_ybot = BOTTOM(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_xtop = RIGHT(critTileInfoPtr->tilePtr) + 1;
            vPlaneSearchRect.r_ytop = TOP(critTileInfoPtr->tilePtr) + 1;

            (void) TiSrArea(NIL(Tile),
                    Tp_GetVertPlane(soGlobalPlaneDataPtr),
                    &vPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &vPlaneSearchRect;
            bnInfo.transformType = IDENTITY;;
            (void) Tp_ItrVertBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine)) {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on top of vTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Top bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr));
                }
            }

            topBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            vPlaneSearchRect.r_xbot = LEFT(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_ybot = BOTTOM(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_xtop = LEFT(critTileInfoPtr->tilePtr) + 1;
            vPlaneSearchRect.r_ytop = TOP(critTileInfoPtr->tilePtr) + 1;

            (void) TiSrArea(NIL(Tile),
                    Tp_GetVertPlane(soGlobalPlaneDataPtr),
                    &vPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &vPlaneSearchRect;
            bnInfo.transformType = IDENTITY;;
            (void) Tp_ItrVertBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine)) {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on bottom of vTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Bottom bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetVertPlane(soGlobalPlaneDataPtr));
                }
            }

            bottomBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);

            Tm_VSplit(soGlobalParentCellPtr);

            Tm_HUnify(soGlobalParentCellPtr);
            (void) Gs_CalcCapDenOfTilePlane(soGlobalParentCellPtr);

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            vPlaneSearchRect.r_xbot = LEFT(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_ybot = TOP(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_xtop = RIGHT(critTileInfoPtr->tilePtr) + 1;
            vPlaneSearchRect.r_ytop = TOP(critTileInfoPtr->tilePtr) + 1;
            Tp_VRectToHRect(&vPlaneSearchRect, &hPlaneSearchRect);

            (void) TiSrArea(NIL(Tile),
                    Tp_GetHorzPlane(soGlobalPlaneDataPtr),
                    &hPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &hPlaneSearchRect;
            bnInfo.transformType = VtoH;
            (void) Tp_ItrHorzBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine)) {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on left of vTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Left bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr));
                }
            }

            leftBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);
            leftBNTilePtr = bnInfo.tilePtr;

            bnInfo.tilePtr = NIL(Tile);
            bnInfo.bnLinePtr = NIL(BNLine);
            bnInfo.tileSpace = INFINITY;
            bnInfo.bnLineSpace = INFINITY;

            vPlaneSearchRect.r_xbot = LEFT(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_ybot = BOTTOM(critTileInfoPtr->tilePtr) - 1;
            vPlaneSearchRect.r_xtop = RIGHT(critTileInfoPtr->tilePtr) + 1;
            vPlaneSearchRect.r_ytop = BOTTOM(critTileInfoPtr->tilePtr) + 1;
            Tp_VRectToHRect(&vPlaneSearchRect, &hPlaneSearchRect);

            (void) TiSrArea(NIL(Tile),
                    Tp_GetHorzPlane(soGlobalPlaneDataPtr),
                    &hPlaneSearchRect,
                    GetBottleneckTileInfo,
                    (ClientData) &bnInfo);

            bnInfo.searchRectPtr = &hPlaneSearchRect;
            bnInfo.transformType = VtoH;
            (void) Tp_ItrHorzBnLine(soGlobalPlaneDataPtr,
                    GetBottleneckLineInfo,
                    (ClientData) &bnInfo);

            if(bnInfo.tilePtr == NIL(Tile) && bnInfo.bnLinePtr == NIL(BNLine)) {
                Bear_PrintFatalErr("SoShapeFunc() failed, could not find bottleneck tile/line on right of vTile. ");
            }

            if(soDebug == 'y') {
                printf("\n");
                printf("Right bottleneck tile: ");
                if(bnInfo.tilePtr != NIL(Tile) && 
                  bnInfo.tileSpace <= bnInfo.bnLineSpace) {
                    SoPrintBNTile(bnInfo.tilePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr), FALSE);
                } else {
                    SoPrintBNLine(bnInfo.bnLinePtr, 
                        Tp_GetHorzPlane(soGlobalPlaneDataPtr));
                }
                printf("\n");
            }

            rightBNSpace = MIN2(bnInfo.tileSpace, bnInfo.bnLineSpace);
            rightBNTilePtr = bnInfo.tilePtr;

            hFsblSpace = (leftBNSpace + rightBNSpace);

            if(hFsblSpace >= critTileInfoPtr->slk) {
                dataPtr->resize.p_x = critTileInfoPtr->slk;
            } else {
                dataPtr->bnDensity = critTileInfoPtr->slk - hFsblSpace;
                dataPtr->bnTilePtr = rightBNTilePtr;
                if(rightBNTilePtr != NIL(Tile) && leftBNTilePtr != NIL(Tile))
                {
                    ForceDecomp(- dataPtr->bnDensity, dataPtr->bnTilePtr);
                    dataPtr->resize.p_x = critTileInfoPtr->slk;
                } else {
                    dataPtr->resize.p_x = 0;
                    dataPtr->resize.p_y = 0;
                    dataPtr->move.p_x = 0;
                    dataPtr->move.p_y = 3;
                    Tm_HSplit(soGlobalParentCellPtr);
                    return(SO_FINISHED);
                }
            }

            bBox = *Mac_GetBBoxOfCell(dataPtr->cellInstPtr);
            GeoTransRect(&(dataPtr->cellInstPtr->transform), &bBox, &newBox);
            dimX = newBox.r_ur.p_x - newBox.r_ll.p_x;
            dimY = newBox.r_ur.p_y - newBox.r_ll.p_y;
            if(dimX < 0) dimX *= -1; if(dimY < 0) dimY *= -1;
            dataPtr->resize.p_y = (int)
                ((double) -(dimY * dataPtr->resize.p_x) /
                (double)(dimX + dataPtr->resize.p_x));
            if(-dataPtr->resize.p_y >= dimY)
                Bear_PrintFatalErr("SoShapeFunc() failed,programming error.");
            dataPtr->move.p_x = - leftBNSpace;
            dataPtr->move.p_y = 0;

            Tm_HSplit(soGlobalParentCellPtr);

            return(SO_NOT_FINISHED);
        }
        break;
    case SC:
    case GA:
        Bear_PrintErr("Layout Style not implemented in current BEAR release. ");
        return(SO_FINISHED);
        break;
    default:
        Bear_PrintErr("soShapeFunction() failed, no style given. ");
        return(SO_FINISHED);
        break;
    }

    return(SO_NOT_FINISHED);
}

 
