
/* soUtil.c --
 *
 *    Massoud Pedram
 *
 * Copyright(C) 1988 Regents of the University of California
 * All rights reserved.
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "soInt.h"



/* ---------- so Utilities ---------- */

int
SoGetHorzPosSlack(tilePtr)
Tile    *tilePtr;
{
    int            posSlack;
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;

    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoGetHorzPosSlack() failed : no tile is given");
    }
    if(tilePtr->ti_client == (ClientData) MINFINITY) {
        Bear_PrintFatalErr("SoGetHorzPosSlack() failed : no tile prop is given");
    }
    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    posSlack = slackInfoPtr->horzSlack.pos;

    return(posSlack);
}

void
SoPutHorzPosSlack(posSlack, tilePtr)
int        posSlack;
Tile    *tilePtr;
{
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;

    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoPutHorzPosSlack() failed : no tile is given");
    }

    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    slackInfoPtr->horzSlack.pos = posSlack;
}


int
SoGetHorzNegSlack(tilePtr)
Tile    *tilePtr;
{
    int        negSlack;
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;


    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoGetHorzNegSlack() failed : no tile is given");
    }
    if(tilePtr->ti_client == (ClientData) MINFINITY) {
        Bear_PrintFatalErr("SoGetHorzNegSlack() failed : no tile prop is given");
    }
    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    negSlack = slackInfoPtr->horzSlack.neg;

    return(negSlack);
}

void
SoPutHorzNegSlack(negSlack, tilePtr)
int        negSlack;
Tile    *tilePtr;
{
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;

    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoPutHorzNegSlack() failed : no tile is given");
    }

    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    slackInfoPtr->horzSlack.neg = negSlack;
}

int
SoGetVertPosSlack(tilePtr)
Tile    *tilePtr;
{
    int        posSlack;
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;


    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoGetVertPosSlack() failed : no tile is given");
    }
    if(tilePtr->ti_client == (ClientData) MINFINITY) {
        Bear_PrintFatalErr("SoGetVertPosSlack() failed : no tile prop is given");
    }
    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    posSlack = slackInfoPtr->vertSlack.pos;

    return(posSlack);
}

void
SoPutVertPosSlack(posSlack, tilePtr)
int        posSlack;
Tile    *tilePtr;
{
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;

    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoPutVertPosSlack() failed : no tile is given");
    }

    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    slackInfoPtr->vertSlack.pos = posSlack;
}

int
SoGetVertNegSlack(tilePtr)
Tile    *tilePtr;
{
    int        negSlack;
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;


    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoGetVertNegSlack() failed : no tile is given");
    }
    if(tilePtr->ti_client == (ClientData) MINFINITY) {
        Bear_PrintFatalErr("SoGetVertNegSlack() failed : no tile prop is given");
    }
    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    negSlack = slackInfoPtr->vertSlack.neg;

    return(negSlack);
}

void
SoPutVertNegSlack(negSlack, tilePtr)
int        negSlack;
Tile    *tilePtr;
{
    TileProp    *tilePropPtr;
    SlackInfo    *slackInfoPtr;

    if(tilePtr == NIL(Tile)) {
        Bear_PrintFatalErr("SoPutVertNegSlack() failed : no tile is given");
    }

    tilePropPtr = (TileProp *) tilePtr->ti_client;
    slackInfoPtr = (SlackInfo *) tilePropPtr->clientData;
    slackInfoPtr->vertSlack.neg = negSlack;
}


/*
 * ----------------------------------------------------------------------------
 *
 * SoGetCritTileInfo --
 *
 * Results: returns CritTileInfo for the tile most desired to shape optimize.
 *    Currently, tile on any critical path with maximum slack is considered 
 *    to be best. If two tiles in SO_VERT and SO_HORZ planes have same slacks, 
 *    prefDirect is used to decide which one to pick. 
 *
 * Side Effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

CritTileInfo
SoGetCritTileInfo(parmsPtr)
So_UserParms *parmsPtr;
{
    int             curSlack, vSlack, hSlack, v2Slack=0, h2Slack=0;
    CritTileInfo    critTileInfo;
    LinkedTile      *ltPtr;
    Tile            *hTilePtr = NIL(Tile);
    Tile            *vTilePtr = NIL(Tile);
    Tile            *horzTilePtr, *vertTilePtr;
    CellInst        *cellInstPtr;
    TileInfo        *tileInfoPtr;
    CellMaster      *cellMasterPtr;
    ShapeConstr     *shapeConstrPtr;
    int             x, y, newX, newY;
    Rect            bBox, newBox;
    int             slk1, slk2, slk3;

    critTileInfo.tilePtr = NIL(Tile);
    critTileInfo.planeDir = SO_HORZ;
    critTileInfo.slk = 0;

    if(parmsPtr->option == BEST_SLACK) {
        h2Slack = (soH2TileList == NIL(LinkedTile)) ? 0 :
            SoGetHorzNegSlack(soH2TileList->tilePtr) -
            SoGetHorzPosSlack(soH2TileList->tilePtr);
        v2Slack = (soV2TileList == NIL(LinkedTile)) ? 0 :
            SoGetVertNegSlack(soV2TileList->tilePtr) -
            SoGetVertPosSlack(soV2TileList->tilePtr);
    }

    vSlack = 0;
    /* Find cellInst tile with largest vert slack in soH1TileList. */
    for(ltPtr = soH1TileList; ltPtr != NIL(LinkedTile); ltPtr = ltPtr->nextPtr) {
        cellInstPtr = Tp_GetCellInstOfTile(ltPtr->tilePtr);
        cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);
        if(cellMasterPtr->shapeFlag != MAC_FIXED_SHAPE) {
            shapeConstrPtr = cellMasterPtr->shapeConstr;
            bBox = *Mac_GetBBoxOfCell(cellInstPtr);
            x = bBox.r_ur.p_x - bBox.r_ll.p_x;
            y = bBox.r_ur.p_y - bBox.r_ll.p_y;
            GeoTransRect(&(cellInstPtr->transform), &bBox, &newBox);
            newX = newBox.r_ur.p_x - newBox.r_ll.p_x;
            newY = newBox.r_ur.p_y - newBox.r_ll.p_y;
            if(newX < 0) newX *= -1; if(newY < 0) newY *= -1;
            tileInfoPtr = (TileInfo *) cellInstPtr->clientData;
            vertTilePtr = tileInfoPtr->vPtr; 
            slk1 = SoGetVertNegSlack(vertTilePtr)-SoGetVertPosSlack(vertTilePtr);
            if(x == newX && y == newY) {
                slk2 = shapeConstrPtr->minAspectRatio.denominator - newY;
                slk3 = (newX*newY)/shapeConstrPtr->minAspectRatio.numerator-newY;
            } else if(x== newY && y == newX) {
                slk2 = shapeConstrPtr->maxAspectRatio.numerator-newY;
                slk3 = (newX*newY)/shapeConstrPtr->maxAspectRatio.denominator-newY;
            } else {
                Bear_PrintErr("SoGetCritTileInfo() failed, programming error.");
            }
            curSlack = MIN3(slk1, slk2, slk3);

            if(curSlack > vSlack) {
                vSlack = curSlack;
                hTilePtr = ltPtr->tilePtr;
            }
        }
    }

    hSlack = 0;
    /* Find cellInst tile with largest horz slack in soV1TileList. */
    for(ltPtr = soV1TileList; ltPtr != NIL(LinkedTile); ltPtr = ltPtr->nextPtr) {
        cellInstPtr = Tp_GetCellInstOfTile(ltPtr->tilePtr);
        cellMasterPtr = Mac_GetMasterOfCell(cellInstPtr);
        if(cellMasterPtr->shapeFlag != MAC_FIXED_SHAPE) {
            shapeConstrPtr = cellMasterPtr->shapeConstr;
            bBox = *Mac_GetBBoxOfCell(cellInstPtr);
            x = bBox.r_ur.p_x - bBox.r_ll.p_x;
            y = bBox.r_ur.p_y - bBox.r_ll.p_y;
            GeoTransRect(&(cellInstPtr->transform), &bBox, &newBox);
            newX = newBox.r_ur.p_x - newBox.r_ll.p_x;
            newY = newBox.r_ur.p_y - newBox.r_ll.p_y;
            if(newX < 0) newX *= -1; if(newY < 0) newY *= -1;
            tileInfoPtr = (TileInfo *) cellInstPtr->clientData;
            horzTilePtr = tileInfoPtr->hPtr; 
            slk1 = SoGetHorzNegSlack(horzTilePtr)-SoGetHorzPosSlack(horzTilePtr);
            if(x == newX && y == newY) {
                slk2 = shapeConstrPtr->maxAspectRatio.numerator-newX;
                slk3 = (newX*newY)/shapeConstrPtr->maxAspectRatio.denominator-newX;
            } else if(x== newY && y == newX) {
                slk2 = shapeConstrPtr->minAspectRatio.denominator-newX;
                slk3 = (newX*newY)/shapeConstrPtr->minAspectRatio.numerator-newX;
            } else {
                Bear_PrintErr("SoGetCritTileInfo() failed, programming error.");
            }
            curSlack = MIN3(slk1, slk2, slk3);

            if(curSlack > hSlack) {
                hSlack = curSlack;
                vTilePtr = ltPtr->tilePtr;
            }
        }
    }

    if(parmsPtr->algorithm == TWO_D) {
        if(hTilePtr == NIL(Tile) && vTilePtr == NIL(Tile)) 
            return(critTileInfo);
        if(parmsPtr->option == BEST_SLACK) {
            if(vSlack-v2Slack > hSlack-h2Slack) parmsPtr->prefDirect = SO_HORZ;
            else parmsPtr->prefDirect = SO_VERT; 
        } else {
            if(vSlack > hSlack) parmsPtr->prefDirect = SO_HORZ;
            else parmsPtr->prefDirect = SO_VERT;
        }
    } else { 
        if(parmsPtr->prefDirect == SO_HORZ && hTilePtr == NIL(Tile)) 
            return(critTileInfo);
        if(parmsPtr->prefDirect == SO_VERT && vTilePtr == NIL(Tile)) 
            return(critTileInfo);
    }

    switch(parmsPtr->option) {
    case BEST_SLACK :
        if(parmsPtr->prefDirect == SO_HORZ) {
            critTileInfo.tilePtr = hTilePtr;
            critTileInfo.planeDir = SO_HORZ;
            critTileInfo.slk = (vSlack - v2Slack <= SWITCH_OPT) ? 
                vSlack / 2 : vSlack - v2Slack;
        } else {
            critTileInfo.tilePtr = vTilePtr;
            critTileInfo.planeDir = SO_VERT;
            critTileInfo.slk = (hSlack - h2Slack <= SWITCH_OPT) ? 
                hSlack / 2 : hSlack - h2Slack;
        }
        break;
    case HALF_SLACK:
        if(parmsPtr->prefDirect == SO_HORZ) {
            critTileInfo.tilePtr = hTilePtr;
            critTileInfo.planeDir = SO_HORZ;
            critTileInfo.slk = (vSlack <= SWITCH_OPT) ? 
                vSlack : vSlack / 2;
        } else {
            critTileInfo.tilePtr = vTilePtr;
            critTileInfo.planeDir = SO_VERT;
            critTileInfo.slk = (hSlack <= SWITCH_OPT) ? 
                hSlack : hSlack / 2;
        }
        break;
    case FULL_SLACK:
        if(parmsPtr->prefDirect == SO_HORZ) {
            critTileInfo.tilePtr = hTilePtr;
            critTileInfo.planeDir = SO_HORZ;
            critTileInfo.slk = vSlack;
        } else {
            critTileInfo.tilePtr = vTilePtr;
            critTileInfo.planeDir = SO_VERT;
            critTileInfo.slk = hSlack;
        }
        break;
    default:
        Bear_PrintFatalErr("SoGetCritTileInfo() failed, invalid option.");
    break;
    }

    if(critTileInfo.slk < 0)
        Bear_PrintErr("SoFindCritTileInfo() failed. Not suppose to happen.");

    return(critTileInfo);
}

