		
/* tmHSplitByZones.c --
 *
 *          Tim Collins
 *     (Professor E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"
#include "tileMappingInt.h"


static void DoLeftDCell();
static void DoRightDCell();
static void DoLeftVFace();
static void DoRightVFace();
static void DoLeftVTile();
static void DoRightVTile();
static void DoVLine();
static void DoCLine();
static void PropagateNewId();
static void CopyHExtToExt();

void
TmHSplitByZones(numOfZones, tablePtr, zoneDataPtr)
int      numOfZones;
Table    *tablePtr;
ZoneData *zoneDataPtr;
{
    /*
     * This procedure uses a table of starting and ending zones of pins and
     * pseudo pins to determine the pseudo pins to be put on the dummy cells,
     * and their correct id's.
     */
    int           i;
    Table         *entryPtr;
    Table         *placePtr;
    BearId        newId;
    ZoneData      *zonePlacePtr;

    if (numOfZones <= 1)
    {
	TmFreeZoneData(&zoneDataPtr);
	return;
    }
    /* 
     * The first one is skipped, because we allocated one too many for the 
     * sake of homogeneity in the code.
     */
    /* make first junction consistent */
    CopyHExtToExt(zoneDataPtr);
    zonePlacePtr = zoneDataPtr->nextPtr;
	TmFreeZoneData(zoneDataPtr);
    for (i = 2; i <= numOfZones; i++)
    {
	entryPtr = tablePtr;
	while (entryPtr != NIL(Table))
	{
	    /* 
	     * This means that a net crosses the side of a zone
	     * therefore a pseudo pin must be put there.
	     */
	    if ((entryPtr->startZone <= i) && (entryPtr->endZone > 1))
	    {
		newId = Mac_GenerateId();
		if ((zonePlacePtr->zoneType == V_LINE) && 
		  (entryPtr->startZone < i))
		{
		    DoVLine(entryPtr, newId, zonePlacePtr);
		    entryPtr->currentId = newId;
		}
		else if ((zonePlacePtr->zoneType == C_LINE) && 
			 (entryPtr->startZone < i))
		{
		    DoCLine(entryPtr, newId, zonePlacePtr);
		    entryPtr->currentId = newId;
		}
		else
		{
		    if (entryPtr->startZone < i)
		    {
			if (zonePlacePtr->zoneType == DCELL)
			{
			    DoLeftDCell(entryPtr, newId, zonePlacePtr);
			}
			if (zonePlacePtr->zoneType == V_TILE)
			{
			    DoLeftVTile(entryPtr, newId, zonePlacePtr);
			}
			if (zonePlacePtr->zoneType == V_FACE)
			{
			    DoLeftVFace(entryPtr, newId, zonePlacePtr);
			}
			if ((zonePlacePtr->zoneType == JUNCTION) ||
			    (zonePlacePtr->zoneType == DJUNCTION))
			{
			    PropagateNewId(entryPtr, 
			      entryPtr->currentId, zonePlacePtr);
			}
			else
			{
			    PropagateNewId(entryPtr, newId, zonePlacePtr);
			}
		    }
		    else
		    {
		    /*
		     * if this is the starting zone we want to keep the 
		     * current id.
		     */
			newId = entryPtr->currentId;
			PropagateNewId(entryPtr, newId, zonePlacePtr);
		    }
		    /*
		     * Delete the entry from the table, if it ends in this zone.
		     */
		    if (entryPtr->endZone == i)
		    {
			placePtr = tablePtr;
			if (placePtr == entryPtr)
			{
			    tablePtr = tablePtr->nextPtr;
			    FREE((char *)entryPtr);
			    entryPtr = tablePtr;
			}
			else
			{
			    while (placePtr->nextPtr != entryPtr)
			    {
				placePtr = placePtr->nextPtr;
			    }
			    placePtr->nextPtr = placePtr->nextPtr->nextPtr;
			    FREE((char *)entryPtr);
			    entryPtr = placePtr->nextPtr;
			}
			continue;
		    }
		    else
		    {
			if (zonePlacePtr->zoneType == DCELL)
			{
			    entryPtr->currentId = newId;
			    newId = Mac_GenerateId();
			    DoRightDCell(entryPtr, newId, zonePlacePtr);
			    entryPtr->currentId = newId;
			}
			if (zonePlacePtr->zoneType == V_TILE)
			{
			    entryPtr->currentId = newId;
			    newId = Mac_GenerateId();
			    DoRightVTile(entryPtr, newId, zonePlacePtr);
			    entryPtr->currentId = newId;
			}
			if (zonePlacePtr->zoneType == V_FACE)
			{
			    entryPtr->currentId = newId;
			    newId = Mac_GenerateId();
			    DoRightVFace(entryPtr, newId, zonePlacePtr);
			    entryPtr->currentId = newId;
			}
		    }
		}
	    }
	    entryPtr = entryPtr->nextPtr;
	}
	zoneDataPtr  = zonePlacePtr;
	zonePlacePtr = zonePlacePtr->nextPtr;
	TmFreeZoneData(zoneDataPtr);
    }
}


static void
DoLeftDCell(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;

	(void) Tp_CreatePPinOfDCell(zonePlacePtr->dCellPtr,
	  LEFT_SIDE, &newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}


static void
DoRightDCell(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;

	(void) Tp_CreatePPinOfDCell(zonePlacePtr->dCellPtr,
	  RIGHT_SIDE, &newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}


static void
DoLeftVFace(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;

	(void) Tp_CreatePPin(zonePlacePtr->bnPtr->unValue.bnTilePtr, 
	  BOTTOM_LEFT_SIDE, TP_BOTTOM, &newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}


static void
DoRightVFace(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;

	(void) Tp_CreatePPin(zonePlacePtr->bnPtr->unValue.bnTilePtr,
	  TOP_RIGHT_SIDE, TP_BOTTOM, &newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}


static void
DoLeftVTile(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;

	(void) Tp_CreatePPin(zonePlacePtr->bnPtr->unValue.bnTilePtr, 
	  BOTTOM_LEFT_SIDE, TP_BOTTOM, &newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}


static void
DoRightVTile(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;

	(void) Tp_CreatePPin(zonePlacePtr->bnPtr->unValue.bnTilePtr,
	  TOP_RIGHT_SIDE, TP_BOTTOM, &newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}


static void
DoVLine(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;
	
	(void) Tp_CreatePPinOfBnLine(zonePlacePtr->bnPtr->unValue.bnLinePtr,
	  &newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}

static void
DoCLine(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
	PseudoPin     *newPPinPtr;
	
	(void) Tp_CreatePPinOfBnLine(zonePlacePtr->bnPtr->unValue.bnLinePtr,
	  &newPPinPtr);
	(void) Tp_PutExtNetIdOfPPin(entryPtr->currentId, newPPinPtr);
	(void) Tp_PutIntNetIdOfPPin(newId, newPPinPtr);
	(void) Tp_PutNetOfPPin(entryPtr->netPtr, newPPinPtr);
	(void) Tp_PutWidthOfPPin(entryPtr->width, newPPinPtr);
}


static void
CopyHExtToExt(zonePlacePtr)
ZoneData      *zonePlacePtr;
{
	BearId        hExtId;
	LinkedPinInst *pinInstPtr;
	LinkedRPin    *rPinPtr;

	/* 
	 * Update the ext from the hext of pins in this zone.
	 */
	pinInstPtr = zonePlacePtr->pinInstList;
	while (pinInstPtr != NIL(LinkedPinInst))
	{
		(void) Mac_PutExtNetIdOfPin(
		  Mac_GetHExtNetIdOfPin(pinInstPtr->pinInstPtr), 
		  pinInstPtr->pinInstPtr);
		pinInstPtr = pinInstPtr->nextPtr;
	}
	rPinPtr = zonePlacePtr->rPinList;
	while (rPinPtr != NIL(LinkedRPin))
	{
	    if ((hExtId = Mac_GetHExtNetIdOfRPin(rPinPtr->rPinPtr)) != 0)
	    {
		(void) Mac_PutExtNetIdOfRPin(hExtId, rPinPtr->rPinPtr);
	    }
	    rPinPtr = rPinPtr->nextPtr;
	}
}


static void
PropagateNewId(entryPtr, newId, zonePlacePtr)
Table         *entryPtr;
BearId        newId;
ZoneData      *zonePlacePtr;
{
    BearId        intId;
    BearId        extId;
    BearId        hExtId;
    LinkedPPin    *pPinPtr;
    LinkedRPin    *rPinPtr;
    LinkedPinInst *pinInstPtr;
    float          width;
    float          *widthVect;

    /* 
     * Update the id of pins in this zone that are on the
     * current net.
     */
    pinInstPtr = zonePlacePtr->pinInstList;
    while (pinInstPtr != NIL(LinkedPinInst))
    {
	hExtId = Mac_GetHExtNetIdOfPin(pinInstPtr->pinInstPtr);
	extId = Mac_GetExtNetIdOfPin(pinInstPtr->pinInstPtr);
	if ((extId == entryPtr->id) || (hExtId == entryPtr->id))
	{
	    (void) Mac_PutExtNetIdOfPin(newId, pinInstPtr->pinInstPtr);
	    (void) Mac_PutVExtNetIdOfPin(newId, pinInstPtr->pinInstPtr);
	    widthVect = Mac_GetWidthVectOfPin(pinInstPtr->pinInstPtr);
	    width = widthVect[METAL2_LAYER];
	    if (width > entryPtr->width)
	    {
		entryPtr->width = width;
	    }
	}
	pinInstPtr = pinInstPtr->nextPtr;
    }
    rPinPtr = zonePlacePtr->rPinList;
    while (rPinPtr != NIL(LinkedRPin))
    {
	hExtId = Mac_GetHExtNetIdOfRPin(rPinPtr->rPinPtr);
	extId = Mac_GetExtNetIdOfRPin(rPinPtr->rPinPtr);
	if ((extId == entryPtr->id) || (hExtId == entryPtr->id))
	{
	    (void) Mac_PutExtNetIdOfRPin(newId, rPinPtr->rPinPtr);
	    (void) Mac_PutVExtNetIdOfRPin(newId, rPinPtr->rPinPtr);
	    widthVect = Mac_GetWidthVectOfRPin(rPinPtr->rPinPtr);
	    width = widthVect[METAL2_LAYER];
	    if (width > entryPtr->width)
	    {
		entryPtr->width = width;
	    }
	}
	rPinPtr = rPinPtr->nextPtr;
    }
    /* 
     * Update the id of pseudo pins in this zone that are on the
     * current net.
     */
    pPinPtr = zonePlacePtr->pPinList;
    while (pPinPtr != NIL(LinkedPPin))
    {
	intId = Tp_GetIntNetIdOfPPin(pPinPtr->ppinPtr);
	extId = Tp_GetExtNetIdOfPPin(pPinPtr->ppinPtr);
	if (zonePlacePtr->zoneType == JUNCTION)
	{
	  if (extId == entryPtr->id)
	  {
	    (void) Tp_PutExtNetIdOfPPin(newId, pPinPtr->ppinPtr);
	    (void) Tp_PutNetOfPPin(entryPtr->netPtr, pPinPtr->ppinPtr);
	  }
	  else if ((intId == entryPtr->id) && 
		   (pPinPtr->ppinPtr->tilePtr != NIL(Tile)) &&
		   (Tp_IsSBNTile(pPinPtr->ppinPtr->tilePtr)))
	  {
	    (void) Tp_PutIntNetIdOfPPin(newId, pPinPtr->ppinPtr);
	    (void) Tp_PutNetOfPPin(entryPtr->netPtr, pPinPtr->ppinPtr);
	  }
	}
	else
	{
	  if (intId == entryPtr->id)
	  {
	    (void) Tp_PutIntNetIdOfPPin(newId, pPinPtr->ppinPtr);
	    (void) Tp_PutNetOfPPin(entryPtr->netPtr, pPinPtr->ppinPtr);
	  }
          else if ((extId == entryPtr->id) &&
		   (pPinPtr->ppinPtr->tilePtr != NIL(Tile)) &&
		   (Tp_IsDCellTile(pPinPtr->ppinPtr->tilePtr)))
	  {
	    (void) Tp_PutExtNetIdOfPPin(newId, pPinPtr->ppinPtr);
	    (void) Tp_PutNetOfPPin(entryPtr->netPtr, pPinPtr->ppinPtr);
	  }
	}
	pPinPtr = pPinPtr->nextPtr;
    }
}


void
TmNumberTable(tableList, id, width, netPtr, numOfZone)
Table  **tableList;
BearId id;
float  width;
Net    *netPtr;
int    numOfZone;
{
	/*
	 * This procedure puts the pin in the table.  Its start zone, end zone,
	 * and current id are all saved.
	 */
	Table  *tablePtr;

	tablePtr = *tableList;
	while (tablePtr != NIL(Table))
	{
		if (tablePtr->id == id)
		{
			tablePtr->endZone = numOfZone;
			break;
		}
		tablePtr = tablePtr->nextPtr;
	}
	if (tablePtr == NIL(Table))
	{
		MALLOC(Table *, tablePtr, sizeof (Table));
		tablePtr->startZone = numOfZone;
		tablePtr->endZone = numOfZone;
		tablePtr->id = id;
		tablePtr->width = width;
		tablePtr->currentId = id;
		tablePtr->nextPtr = *tableList;
		*tableList = tablePtr;
	}
	tablePtr->netPtr = netPtr;
}

