
/* tmHVFUnify.c --
 *
 *           Tim Collins
 *      (Professor E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"
#include "tileMappingInt.h"


#define EXT 1
#define HEXT 1
#define VEXT 0
#define NOT_EXT 0

typedef struct _SaveInfo
{
    BearId  tempIntId;
    BearId  tempExtId;
    BearId  saveIntId;
    BearId  saveExtId;
} SaveInfo;

typedef struct _ItrData {
    BearStatus  (*proc)();
    ClientData  clientData;
} ItrData;


extern void ItrPPinsOfHPlane();
extern void ItrPPinsOfVPlane();
extern void CreateSaveSpace();
extern void FreeHSaveSpace();
extern void FreeVSaveSpace();
extern void TransferHTempValues();
extern void TransferVTempValues();
extern void TransferHSaveValues();
extern void CopyHTempToSave();
extern void MovePPinValuesToTemp();
extern void CopyExtToHExt();
extern void CopyExtToVExt();
extern int DoEachHTile();
extern int DoEachVTile();
extern int DoEachSolidTile();
extern BearStatus DoEachBnLine();
extern BearStatus MakePPinSaveSpace();
extern BearStatus FreePPinSaveSpace();
extern BearStatus TransferTempValues();
extern BearStatus TransferSaveValues();
extern BearStatus CopyValues();
extern BearStatus MoveValues();
extern BearStatus CopyExtIds();
extern BearStatus CopyExtRIds();


extern ViewType viewTypeFlag;


/*
 * ----------------------------------------------------------------------------
 *
 * Tm_HVFUnify () --
 *
 * This procedure facilitates the calling of HFUnify, VFUnify, or both.
 * The pseudo pin information is saved, and upon exit the actual pseudo
 * pin on the appropriate plane or planes is updated.
 *
 * Results:
 *
 * Side effects:
 * ----------------------------------------------------------------------------
 */

void
Tm_HFOnlyUnify(parentCellPtr, planeDataPtr)
CellMaster *parentCellPtr;
PlaneData *planeDataPtr;
{

    tmGlobalParentCellPtr = parentCellPtr;
    (void) CreateSaveSpace(planeDataPtr);
    (void) MovePPinValuesToTemp(planeDataPtr);
    (void) CopyExtToHExt(planeDataPtr);
    (void) Tm_HFUnify(EXT);
    (void) TransferHTempValues(planeDataPtr);
    (void) TransferVTempValues(planeDataPtr);
    (void) FreeHSaveSpace(planeDataPtr);
    (void) FreeVSaveSpace(planeDataPtr);
    viewTypeFlag = H_DONE;
}


void
Tm_HRectUnify(parentCellPtr, planeDataPtr, dmntRect)
CellMaster *parentCellPtr;
PlaneData *planeDataPtr;
Rect      dmntRect;
{

    tmGlobalParentCellPtr = parentCellPtr;
    (void) CreateSaveSpace(planeDataPtr);
    (void) MovePPinValuesToTemp(planeDataPtr);
    (void) CopyExtToHExt(planeDataPtr);
    (void) TmHRectUnify(planeDataPtr, dmntRect, EXT);
    (void) TransferHTempValues(planeDataPtr);
    (void) TransferVTempValues(planeDataPtr);
    (void) FreeHSaveSpace(planeDataPtr);
    (void) FreeVSaveSpace(planeDataPtr);
    viewTypeFlag = BOTH_DONE;
}


void
Tm_VFOnlyUnify(parentCellPtr, planeDataPtr)
CellMaster *parentCellPtr;
PlaneData *planeDataPtr;
{

    tmGlobalParentCellPtr = parentCellPtr;
    (void) CreateSaveSpace(planeDataPtr);
    (void) MovePPinValuesToTemp(planeDataPtr);
    (void) CopyExtToVExt(planeDataPtr);
    (void) Tm_VFUnify(EXT);
    (void) TransferVTempValues(planeDataPtr);
    (void) TransferHTempValues(planeDataPtr);
    (void) FreeHSaveSpace(planeDataPtr);
    (void) FreeVSaveSpace(planeDataPtr);
    viewTypeFlag = V_DONE;
}


void
Tm_VRectUnify(parentCellPtr, planeDataPtr, dmntRect)
CellMaster *parentCellPtr;
PlaneData *planeDataPtr;
Rect      dmntRect;
{

    tmGlobalParentCellPtr = parentCellPtr;
    (void) CreateSaveSpace(planeDataPtr);
    (void) MovePPinValuesToTemp(planeDataPtr);
    (void) CopyExtToVExt(planeDataPtr);
    (void) TmVRectUnify(planeDataPtr, dmntRect, EXT);
    (void) TransferVTempValues(planeDataPtr);
    (void) TransferHTempValues(planeDataPtr);
    (void) FreeHSaveSpace(planeDataPtr);
    (void) FreeVSaveSpace(planeDataPtr);
    viewTypeFlag = BOTH_DONE;
}


void
Tm_HVFUnify(parentCellPtr, planeDataPtr)
CellMaster *parentCellPtr;
PlaneData *planeDataPtr;
{

    tmGlobalParentCellPtr = parentCellPtr;
    (void) CreateSaveSpace(planeDataPtr);
    (void) MovePPinValuesToTemp(planeDataPtr);
    (void) Tm_HFUnify(NOT_EXT);
    (void) CopyHTempToSave(planeDataPtr);
    (void) MovePPinValuesToTemp(planeDataPtr);
    (void) Tm_VFUnify(NOT_EXT);
    (void) TransferHSaveValues(planeDataPtr);
    (void) TransferVTempValues(planeDataPtr);
    (void) FreeHSaveSpace(planeDataPtr);
    (void) FreeVSaveSpace(planeDataPtr);
    viewTypeFlag = BOTH_DONE;
}


static void
CopyExtToHExt(planeDataPtr)
PlaneData  *planeDataPtr;
{
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
      Tp_GetHorzPlaneBnd(planeDataPtr),
      DoEachSolidTile,
      (ClientData) HEXT);
}


static void
CopyExtToVExt(planeDataPtr)
PlaneData  *planeDataPtr;
{
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
      Tp_GetHorzPlaneBnd(planeDataPtr),
      DoEachSolidTile,
      (ClientData) VEXT);
}


static void
ItrPPinsOfHPlane(planeDataPtr, proc)
PlaneData  *planeDataPtr;
BearStatus (*proc)();
{
    ItrData itrData;

    itrData.proc = proc;
    itrData.clientData = (ClientData) NULL;

    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
      Tp_GetHorzPlaneBnd(planeDataPtr),
      DoEachHTile,
      (ClientData) &itrData);

    (void) Tp_ItrHorzBnLine(planeDataPtr, DoEachBnLine, (ClientData) &itrData);
}


static void
ItrPPinsOfVPlane(planeDataPtr, proc)
PlaneData  *planeDataPtr;
BearStatus (*proc)();
{
    ItrData itrData;

    itrData.proc = proc;
    itrData.clientData = (ClientData) NULL;

    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr),
      Tp_GetVertPlaneBnd(planeDataPtr),
      DoEachVTile,
      (ClientData) &itrData);

    (void) Tp_ItrVertBnLine(planeDataPtr, DoEachBnLine, (ClientData) &itrData);
}


static BearStatus
DoEachBnLine(bnLinePtr, itrDataPtr)
BNLine     *bnLinePtr;
ItrData    *itrDataPtr;
{
    (void) Tp_ItrPPinsOfBnLine(Tp_GetTLTileOfBnLine(bnLinePtr), 
        Tp_GetBRTileOfBnLine(bnLinePtr), itrDataPtr->proc, 
        itrDataPtr->clientData);
    return (BEAR_CONTINUE_SEARCH);
}


static int
DoEachSolidTile(tilePtr, planeFlag)
Tile   *tilePtr;
int    planeFlag;
{
    CellInst      *cellInstPtr;
    RouteCell     *routeCellPtr;

    if (Tp_IsCellInstTile(tilePtr) == TRUE)
    {
        cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
        (void) Mac_ItrPinOfCellInst(cellInstPtr, CopyExtIds,
          (ClientData) planeFlag);
    }
    if (Tp_IsRCellTile(tilePtr) == TRUE)
    {
        routeCellPtr = Tp_GetRCellOfTile(tilePtr);
        (void) Mac_ItrSidePinOfRCell(routeCellPtr, CopyExtRIds,
          (ClientData) planeFlag);
    }
    return (MAGIC_CONTINUE_SEARCH);
}


static BearStatus
CopyExtIds(pinInstPtr, planeFlag)
PinInst  *pinInstPtr;
int      planeFlag;
{
    if (planeFlag == HEXT)
    {
        (void) Mac_PutHExtNetIdOfPin(Mac_GetExtNetIdOfPin(pinInstPtr),
          pinInstPtr);
    }
    else
    {
        (void) Mac_PutVExtNetIdOfPin(Mac_GetExtNetIdOfPin(pinInstPtr),
          pinInstPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
CopyExtRIds(rPinPtr, planeFlag)
RoutePin *rPinPtr;
int      planeFlag;
{
    if (planeFlag == HEXT)
    {
        (void) Mac_PutHExtNetIdOfRPin(Mac_GetExtNetIdOfRPin(rPinPtr),
          rPinPtr);
    }
    else
    {
        (void) Mac_PutVExtNetIdOfRPin(Mac_GetExtNetIdOfRPin(rPinPtr),
          rPinPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}


static int
DoEachVTile(tilePtr, itrDataPtr)
Tile       *tilePtr;
ItrData    *itrDataPtr;
{
    if (Tp_IsBottleneckTile(tilePtr))
    {
        (void) Tp_ItrPPinsOfSide(tilePtr, BOTTOM_LEFT_SIDE, 
            itrDataPtr->proc, itrDataPtr->clientData);
        (void) Tp_ItrPPinsOfSide(tilePtr, TOP_RIGHT_SIDE,
            itrDataPtr->proc, itrDataPtr->clientData);
    }
    return (MAGIC_CONTINUE_SEARCH);
}


static int
DoEachHTile(tilePtr, itrDataPtr)
Tile       *tilePtr;
ItrData    *itrDataPtr;
{
    DummyCell *dCellPtr;

    if (Tp_IsBottleneckTile(tilePtr))
    {
        (void) Tp_ItrPPinsOfSide(tilePtr, BOTTOM_LEFT_SIDE,
            itrDataPtr->proc, itrDataPtr->clientData);
        (void) Tp_ItrPPinsOfSide(tilePtr, TOP_RIGHT_SIDE,
            itrDataPtr->proc, itrDataPtr->clientData);
    }
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, LEFT_SIDE,
            itrDataPtr->proc, itrDataPtr->clientData);
        (void) Mac_ItrPPinOfDCell(dCellPtr, RIGHT_SIDE,
            itrDataPtr->proc, itrDataPtr->clientData);
        (void) Mac_ItrPPinOfDCell(dCellPtr, TOP_SIDE,
            itrDataPtr->proc, itrDataPtr->clientData);
        (void) Mac_ItrPPinOfDCell(dCellPtr, BOTTOM_SIDE,
            itrDataPtr->proc, itrDataPtr->clientData);
    }
    return (MAGIC_CONTINUE_SEARCH);
}


static void
CreateSaveSpace(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfHPlane(planeDataPtr, MakePPinSaveSpace);
    ItrPPinsOfVPlane(planeDataPtr, MakePPinSaveSpace);
}


/* ARGSUSED */
static BearStatus
MakePPinSaveSpace(ppinPtr, clientData)
PseudoPin   *ppinPtr;
ClientData  clientData;
{
    SaveInfo *saveInfoPtr;

    MALLOC(SaveInfo *, saveInfoPtr, sizeof (SaveInfo));
    ppinPtr->clientData = (ClientData) saveInfoPtr;
    return (BEAR_CONTINUE_SEARCH);
}


static void
MovePPinValuesToTemp(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfHPlane(planeDataPtr, MoveValues);
    ItrPPinsOfVPlane(planeDataPtr, MoveValues);
}


/* ARGSUSED */
static BearStatus
MoveValues(pPinPtr, clientData)
PseudoPin   *pPinPtr;
ClientData  clientData;
{
    SaveInfo *saveInfoPtr;

    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
    saveInfoPtr->tempIntId = Tp_GetIntNetIdOfPPin(pPinPtr);
    saveInfoPtr->tempExtId = Tp_GetExtNetIdOfPPin(pPinPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static void
FreeHSaveSpace(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfHPlane(planeDataPtr, FreePPinSaveSpace);
}


static void
FreeVSaveSpace(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfVPlane(planeDataPtr, FreePPinSaveSpace);
}


/* ARGSUSED */
static BearStatus
FreePPinSaveSpace(ppinPtr, clientData)
PseudoPin   *ppinPtr;
ClientData  clientData;
{

    FREE((char *) ppinPtr->clientData);
    ppinPtr->clientData = (ClientData) NULL;
    return (BEAR_CONTINUE_SEARCH);
}


static void
TransferHTempValues(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfHPlane(planeDataPtr, TransferTempValues);
}


static void
TransferVTempValues(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfVPlane(planeDataPtr, TransferTempValues);
}


static void
TransferHSaveValues(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfHPlane(planeDataPtr, TransferSaveValues);
}


static void
CopyHTempToSave(planeDataPtr)
PlaneData *planeDataPtr;
{
    ItrPPinsOfHPlane(planeDataPtr, CopyValues);
}


/* ARGSUSED */
static BearStatus
TransferTempValues(pPinPtr, clientData)
PseudoPin   *pPinPtr;
ClientData  clientData;
{
    SaveInfo *saveInfoPtr;

    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
if (saveInfoPtr->tempIntId != 0)
{
    (void) Tp_PutIntNetIdOfPPin(saveInfoPtr->tempIntId, pPinPtr);
}
if (saveInfoPtr->tempExtId != 0)
{
    (void) Tp_PutExtNetIdOfPPin(saveInfoPtr->tempExtId, pPinPtr);
}
    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
static BearStatus
TransferSaveValues(pPinPtr, clientData)
PseudoPin   *pPinPtr;
ClientData  clientData;
{
    SaveInfo *saveInfoPtr;

    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
    (void) Tp_PutIntNetIdOfPPin(saveInfoPtr->saveIntId, pPinPtr);
    (void) Tp_PutExtNetIdOfPPin(saveInfoPtr->saveExtId, pPinPtr);
    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
static BearStatus
CopyValues(pPinPtr, clientData)
PseudoPin   *pPinPtr;
ClientData  clientData;
{
    SaveInfo *saveInfoPtr;

    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
    saveInfoPtr->saveIntId = saveInfoPtr->tempIntId;
    saveInfoPtr->saveExtId = saveInfoPtr->tempExtId;
    return (BEAR_CONTINUE_SEARCH);
}


BearId
TmGetTempIntNetIdOfPPin(pPinPtr)
PseudoPin  *pPinPtr;
{
    SaveInfo *saveInfoPtr;

    if (pPinPtr == NIL(PseudoPin)) 
    {
        TmPrintFatalErr("TmGetTempIntNetIdOfPPin() failed : no pseudo pin given");
    }
    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
    return (saveInfoPtr->tempIntId);
}


BearId
TmGetTempExtNetIdOfPPin(pPinPtr)
PseudoPin  *pPinPtr;
{
    SaveInfo *saveInfoPtr;

    if (pPinPtr == NIL(PseudoPin)) 
    {
        TmPrintFatalErr("TmGetTempIntNetIdOfPPin() failed : no pseudo pin given");
    }
    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
    return (saveInfoPtr->tempExtId);
}


void
TmPutTempIntNetIdOfPPin(intNetId, pPinPtr)
BearId      intNetId;
PseudoPin   *pPinPtr;
{
    SaveInfo *saveInfoPtr;

    if (pPinPtr == NIL(PseudoPin)) 
    {
        TmPrintFatalErr("TmPutTempIntNetIdOfPPin() failed");
    } 
    else 
    {
        if (intNetId == 0) 
        {
            TmPrintFatalErr("TmPutTempIntNetIdOfPPin() failed : id value cannot be 0");
        }
    }
    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
    saveInfoPtr->tempIntId = intNetId;
}


void
TmPutTempExtNetIdOfPPin(extNetId, pPinPtr)
BearId      extNetId;
PseudoPin   *pPinPtr;
{
    SaveInfo *saveInfoPtr;

    if (pPinPtr == NIL(PseudoPin)) 
    {
        TmPrintFatalErr("TmPutTempExtNetIdOfPPin() failed");
    } 
    else 
    {
        if (extNetId == 0) 
        {
            TmPrintFatalErr("TmPutTempExtNetIdOfPPin() failed : id value cannot be 0");
        }
    }
    saveInfoPtr = (SaveInfo *) pPinPtr->clientData;
    saveInfoPtr->tempExtId = extNetId;
}
