/* tmPinsArndCircle.c --
 *
 *         Sherry Solden
 *     (Professor E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"
#include "tileMappingInt.h"

void TmPinsAroundCircle();
int DoNorthWallSeg();
int DoSouthWallSeg();
int DoEastWallSeg();
int DoWestWallSeg();
static void AddPPinsToList();
static BearStatus AddPins();
static void AppendPinsToList();
BearStatus UnmarkWallSeg();
static void GetBAndTTiles();
static void GetLAndRTiles();
int GetStatusOfWallSegment();

int circleStatus;

#define alive             1
#define leftDead          2
#define botDead           3
#define rightDead         4
#define topDead           5
#define bothDead          6
#define halfDead          7
#define noSeg             8
#define clockWise         9
#define counterClockWise 10

void
Tm_PinsAroundCircle(parentCellPtr, planeDataPtr, junctPtr, pinListsPtr, 
	includeInsidePPins)

CellMaster         *parentCellPtr;
PlaneData          *planeDataPtr;
WallJunction       *junctPtr;
PinsAroundJunction *pinListsPtr;
bool               includeInsidePPins;
{
	tmGlobalParentCellPtr = parentCellPtr;
	tmGlobalPlaneDataPtr = planeDataPtr;

	pinListsPtr->horzPPinList = NIL(LinkedPPin);
	pinListsPtr->vertPPinList = NIL(LinkedPPin);
	pinListsPtr->pinInstList = NIL(LinkedPinInst);
	pinListsPtr->rPinList = NIL(LinkedRPin);

	/* Clear all marks on wall segments */
	Fpg_ItrWallSegSeq(Tp_GetFpg(planeDataPtr), UnmarkWallSeg,
		(ClientData) NULL);

	/* Clear the status of the circle */
        circleStatus = alive;

        TmPinsAroundCircle(junctPtr, pinListsPtr);
}



void
TmPinsAroundCircle(junctPtr, pinListsPtr)
WallJunction       *junctPtr;
PinsAroundJunction *pinListsPtr;
{
  int stat;

  stat = DoNorthWallSeg(junctPtr, pinListsPtr, FALSE);
  stat = DoSouthWallSeg(junctPtr, pinListsPtr, FALSE);
  stat = DoEastWallSeg(junctPtr, pinListsPtr, FALSE);
  stat = DoWestWallSeg(junctPtr, pinListsPtr, FALSE);
}

int
DoNorthWallSeg(junctPtr, pinListsPtr, mustBeAlive)
WallJunction       *junctPtr;
PinsAroundJunction *pinListsPtr;
bool               mustBeAlive;
{
  int          stat;
  bool         wrongDirection;
  bool         willBeAlive;
  WallSegment  *wallSegPtr;
  BottleNeck   *bottleNeckPtr;
  Tile         *bnTilePtr;
  Rect         bnRect;
  int          status;
  WallJunction *topJunctPtr;
  WallJunction *botJunctPtr;
  Point        *topPointPtr;
  Point        *botPointPtr;
  Point        vPoint;
  Tile         *dmntTilePtr;
  Rect         dmntRect;
  PinsAroundJunction *faceListPtr;

  
  if ((wallSegPtr = Fpg_GetNorthSegOfWallJunct(junctPtr)) != NIL(WallSegment))
  {
    /* If wallSegPtr has not been visited */
    if (wallSegPtr->mark == BEAR_UNMARKED)
    {
      Fpg_MarkWallSeg(wallSegPtr);
      status = GetStatusOfWallSegment(wallSegPtr);
      wrongDirection = (((circleStatus == clockWise) && 
			 (status == leftDead)) ||
			((circleStatus == counterClockWise) && 
			 (status == rightDead)));
      if ((status == alive) || wrongDirection || mustBeAlive)
      {
	/*    get bottleNeckPtr */
	/*    Add botPPinList to pinListPtr.horzPPinList */
	bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
	if (bottleNeckPtr->unType == TP_BN_LINE)
	  bnTilePtr=Tp_GetTLTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr);
	else
	  bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
	(void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, 
				 bnTilePtr, &bnRect);
	if (Tp_IsSBNTile(bnTilePtr))
	{
	  (void) Tp_ItrPPinsOfBnLine(bnTilePtr,
		      Tp_GetBRTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr), 
		      AddPins, (ClientData) &pinListsPtr->horzPPinList);
	}
	else
	{
	  (void) AddPPinsToList(bnTilePtr, BOTTOM_LEFT_SIDE, 
			        &pinListsPtr->horzPPinList);
	}
        return(alive);
      }
      else if ((status == leftDead) || (status == rightDead))
      {
	if (status == rightDead)
	  circleStatus = clockWise;
        else circleStatus = counterClockWise;

	/* Get PinInst and routePins Around VFace */
	topJunctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
	botJunctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
	topPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, topJunctPtr);
	botPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, botJunctPtr);
	Tp_HPointToVPoint(botPointPtr, &vPoint);
	dmntTilePtr = TiSrPointNoHint(
			      Tp_GetVertPlane(tmGlobalPlaneDataPtr),&vPoint);
	(void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, 
				 dmntTilePtr, &dmntRect);
	dmntRect.r_ytop = topPointPtr->p_y;
	dmntRect.r_ybot = botPointPtr->p_y;
	
	faceListPtr = Tm_PinsAroundVFace(tmGlobalParentCellPtr, 
					 tmGlobalPlaneDataPtr,
					 dmntRect, FALSE);
	(void) AppendPinsToList(*faceListPtr, pinListsPtr);

	willBeAlive = FALSE;	
	if (circleStatus == clockWise) 
	{
	  stat = DoEastWallSeg(topJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
          stat = DoNorthWallSeg(topJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoWestWallSeg(topJunctPtr, pinListsPtr, willBeAlive);
	}
	else
	{
	  stat = DoWestWallSeg(topJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoNorthWallSeg(topJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoEastWallSeg(topJunctPtr, pinListsPtr, willBeAlive);
	}
	return(halfDead);
      }
      /* status = bothDead - do nothing */
      return(bothDead);
    }

    /* segment already marked */
    /* must determine status of segment to return correct value */
    status = GetStatusOfWallSegment(wallSegPtr);
    if (status == alive)
    {
      return(alive);
    }
    else if ((status == leftDead) || (status == rightDead))
    {
      return(halfDead);
    }
    else return(bothDead);
  }
  return(noSeg);
}



int
DoSouthWallSeg(junctPtr, pinListsPtr, mustBeAlive)
WallJunction       *junctPtr;
PinsAroundJunction *pinListsPtr;
bool               mustBeAlive;
{
  int          stat;
  bool         wrongDirection;
  bool         willBeAlive;
  WallSegment  *wallSegPtr;
  BottleNeck   *bottleNeckPtr;
  Tile         *bnTilePtr;
  Rect         bnRect;
  int          status;
  WallJunction *topJunctPtr;
  WallJunction *botJunctPtr;
  Point        *topPointPtr;
  Point        *botPointPtr;
  Point        vPoint;
  Tile         *dmntTilePtr;
  Rect         dmntRect;
  PinsAroundJunction *faceListPtr;

  if ((wallSegPtr = Fpg_GetSouthSegOfWallJunct(junctPtr)) != NIL(WallSegment))
  {
    /* If wallSegPtr has not been visited */
    if (wallSegPtr->mark == BEAR_UNMARKED)
    {
      Fpg_MarkWallSeg(wallSegPtr);
      status = GetStatusOfWallSegment(wallSegPtr);
      wrongDirection = (((circleStatus == clockWise) && 
			 (status == rightDead)) ||
			((circleStatus == counterClockWise) && 
			 (status == leftDead)));
      if ((status == alive) || wrongDirection || mustBeAlive)
      {
	/*    get bottleNeckPtr */
	/*    Add botPPinList to pinListPtr.horzPPinList */
	bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
	if (bottleNeckPtr->unType == TP_BN_LINE)
	  bnTilePtr=Tp_GetTLTileOfBnLine(
					 bottleNeckPtr->unValue.bnLinePtr);
	else bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
	(void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, 
				 bnTilePtr, &bnRect);
	if (Tp_IsSBNTile(bnTilePtr))
	{
	  (void) Tp_ItrPPinsOfBnLine(bnTilePtr,
		       Tp_GetBRTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr), 
		       AddPins, (ClientData) &pinListsPtr->horzPPinList);
	}
	else
	{
	  (void) AddPPinsToList(bnTilePtr, TOP_RIGHT_SIDE, 
			        &pinListsPtr->horzPPinList);
	}
	return(alive);
      }
      else if ((status == leftDead) || (status == rightDead))
      {
	if (status == leftDead)
	  circleStatus = clockWise;
        else circleStatus = counterClockWise;


	/* Get PinInst and routePins Around VFace */
	topJunctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
	botJunctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
	topPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, topJunctPtr);
	botPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, botJunctPtr);
	Tp_HPointToVPoint(botPointPtr, &vPoint);
	dmntTilePtr = TiSrPointNoHint(
			       Tp_GetVertPlane(tmGlobalPlaneDataPtr),&vPoint);
	(void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, 
				 dmntTilePtr, &dmntRect);
	dmntRect.r_ytop = topPointPtr->p_y;
	dmntRect.r_ybot = botPointPtr->p_y;
	
	faceListPtr = Tm_PinsAroundVFace(tmGlobalParentCellPtr, 
					 tmGlobalPlaneDataPtr,
					 dmntRect, FALSE);
	(void) AppendPinsToList(*faceListPtr, pinListsPtr);
	
	/*    Call TmPinsAroundCircle on next junctPtr */
	willBeAlive = FALSE;
	if (circleStatus == clockWise) 
	{
	  stat = DoWestWallSeg(botJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoSouthWallSeg(botJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoEastWallSeg(botJunctPtr, pinListsPtr, willBeAlive);
	}
	else
	{
	  stat = DoEastWallSeg(botJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoSouthWallSeg(botJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoWestWallSeg(botJunctPtr, pinListsPtr, willBeAlive);
	}
	return(halfDead);
      }
      /* status = bothDead - do nothing */
      return(bothDead);
    }
    /* segment already marked */
    /* must determine status of segment to return correct value */
    status = GetStatusOfWallSegment(wallSegPtr);
    if (status == alive)
    {
      return(alive);
    }
    else if ((status == leftDead) || (status == rightDead))
    {
      return(halfDead);
    }
    else return(bothDead);
  }
  return(noSeg);
}


int
DoEastWallSeg(junctPtr, pinListsPtr, mustBeAlive)
WallJunction       *junctPtr;
PinsAroundJunction *pinListsPtr;
bool               mustBeAlive;
{
  int          stat;
  bool         wrongDirection;
  bool         willBeAlive;
  WallSegment  *wallSegPtr;
  Tile         *bnTilePtr;
  Tile         *dmntTilePtr;
  Rect         bnRect;
  Rect         dmntRect;
  BottleNeck   *bottleNeckPtr;
  int          status;
  WallJunction *rightJunctPtr;
  WallJunction *leftJunctPtr;
  Point        *rightPointPtr;
  Point        *leftPointPtr;
  PinsAroundJunction *faceListPtr;

  if ((wallSegPtr = Fpg_GetEastSegOfWallJunct(junctPtr)) != NIL(WallSegment))
  {
    /* If wallSegPtr has not been visited */
    if (wallSegPtr->mark == BEAR_UNMARKED)
    {
      Fpg_MarkWallSeg(wallSegPtr);
      status = GetStatusOfWallSegment(wallSegPtr);
      wrongDirection = (((circleStatus == clockWise) && 
			 (status == topDead)) ||
			((circleStatus == counterClockWise) && 
			 (status == botDead)));
      if ((status == alive) || wrongDirection || mustBeAlive)
      {
	/*    get bottleNeckPtr */
	/*    Add botPPinList to pinListPtr.horzPPinList */
	bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
	if (bottleNeckPtr->unType == TP_BN_LINE)
	  bnTilePtr=Tp_GetTLTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr);
	else bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
	(void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, 
				 bnTilePtr, &bnRect);
	if (Tp_IsSBNTile(bnTilePtr))
	{
	  (void) Tp_ItrPPinsOfBnLine(bnTilePtr,
		      Tp_GetBRTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr), 
		      AddPins, (ClientData) &pinListsPtr->vertPPinList);
	}
	else
	{
	  (void) AddPPinsToList(bnTilePtr, BOTTOM_LEFT_SIDE,
			        &pinListsPtr->vertPPinList);
	}
	return(alive);
      }
      else if ((status == topDead) || (status == botDead))
      {
	if (status == botDead)
	  circleStatus = clockWise;
        else circleStatus = counterClockWise;

	/* Get PinInst and routePins Around HFace */
	leftJunctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
	rightJunctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
	leftPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, leftJunctPtr);
	rightPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER,rightJunctPtr);
	dmntTilePtr = TiSrPointNoHint(
		 	  Tp_GetHorzPlane(tmGlobalPlaneDataPtr),leftPointPtr);
	(void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, 
				 dmntTilePtr, &dmntRect);
	dmntRect.r_xbot = leftPointPtr->p_x;
	dmntRect.r_xtop = rightPointPtr->p_x;
	
	faceListPtr = Tm_PinsAroundHFace(tmGlobalParentCellPtr, 
					 tmGlobalPlaneDataPtr,
					 dmntRect, FALSE);
	(void) AppendPinsToList(*faceListPtr, pinListsPtr);
	
	/*    Call TmPinsAroundCircle on next junctPtr */
	willBeAlive = FALSE;
	if (circleStatus == clockWise) 
	{
	  stat = DoSouthWallSeg(rightJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoEastWallSeg(rightJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoNorthWallSeg(rightJunctPtr, pinListsPtr, willBeAlive);
	}
	else
	{
	  stat = DoNorthWallSeg(rightJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoEastWallSeg(rightJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoSouthWallSeg(rightJunctPtr, pinListsPtr, willBeAlive);
	}
	return(halfDead);
      }
      /* status = bothDead - do nothing */
      return(bothDead);
    }
    /* segment already marked */
    /* must determine status of segment to return correct value */
    status = GetStatusOfWallSegment(wallSegPtr);
    if (status == alive)
    {
      return(alive);
    }
    else if ((status == topDead) || (status == botDead))
    {
      return(halfDead);
    }
    else return(bothDead);
  }
  return(noSeg);
}


int
DoWestWallSeg(junctPtr, pinListsPtr, mustBeAlive)
WallJunction       *junctPtr;
PinsAroundJunction *pinListsPtr;
bool               mustBeAlive;
{
  int          stat;
  bool         wrongDirection;
  bool         willBeAlive;
  WallSegment  *wallSegPtr;
  Tile         *bnTilePtr;
  Tile         *dmntTilePtr;
  Rect         bnRect;
  Rect         dmntRect;
  BottleNeck   *bottleNeckPtr;
  int          status;
  WallJunction *rightJunctPtr;
  WallJunction *leftJunctPtr;
  Point        *rightPointPtr;
  Point        *leftPointPtr;
  PinsAroundJunction *faceListPtr;

  if ((wallSegPtr = Fpg_GetWestSegOfWallJunct(junctPtr)) != NIL(WallSegment))
  {
    /* If wallSegPtr has not been visited */
    if (wallSegPtr->mark == BEAR_UNMARKED)
    {
      Fpg_MarkWallSeg(wallSegPtr);
      status = GetStatusOfWallSegment(wallSegPtr);
      wrongDirection = (((circleStatus == clockWise) && 
			 (status == botDead)) ||
			((circleStatus == counterClockWise) && 
			 (status == topDead)));
      if ((status == alive) || wrongDirection || mustBeAlive)
      {
	/*    get bottleNeckPtr */
	/*    Add botPPinList to pinListPtr.horzPPinList */
	bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
	if (bottleNeckPtr->unType == TP_BN_LINE)
	  bnTilePtr=Tp_GetTLTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr);
	else bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
	(void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, 
				 bnTilePtr, &bnRect);
	if (Tp_IsSBNTile(bnTilePtr))
	{
	  (void) Tp_ItrPPinsOfBnLine(bnTilePtr,
		       Tp_GetBRTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr), 
	               AddPins, (ClientData) &pinListsPtr->vertPPinList);
	}
	else
	{
	  (void) AddPPinsToList(bnTilePtr, TOP_RIGHT_SIDE,
			        &pinListsPtr->vertPPinList);
	}
	return(alive);
      }
      else if ((status == topDead) || (status == botDead))
      {
	if (status == topDead)
	  circleStatus = clockWise;
        else circleStatus = counterClockWise;

	/* Get PinInst and routePins Around HFace */
	leftJunctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
	rightJunctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
	leftPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, leftJunctPtr);
	rightPointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER,rightJunctPtr);
	dmntTilePtr = TiSrPointNoHint(
       			Tp_GetHorzPlane(tmGlobalPlaneDataPtr),leftPointPtr);
	(void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, 
				 dmntTilePtr, &dmntRect);
	dmntRect.r_xbot = leftPointPtr->p_x;
	dmntRect.r_xtop = rightPointPtr->p_x;

	faceListPtr = Tm_PinsAroundHFace(tmGlobalParentCellPtr, 
					 tmGlobalPlaneDataPtr,
					 dmntRect, FALSE);
	(void) AppendPinsToList(*faceListPtr, pinListsPtr);

	/*    Call TmPinsAroundCircle on next junctPtr */
	willBeAlive = FALSE;
	if (circleStatus == clockWise) 
	{
	  stat = DoNorthWallSeg(leftJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoWestWallSeg(leftJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoSouthWallSeg(leftJunctPtr, pinListsPtr, willBeAlive);
	}
	else
	{
	  stat = DoSouthWallSeg(leftJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoWestWallSeg(leftJunctPtr, pinListsPtr, willBeAlive);
	  if (stat == halfDead)
	  {
	    willBeAlive = TRUE;
	  }
	  stat = DoNorthWallSeg(leftJunctPtr, pinListsPtr, willBeAlive);
	}
	return(halfDead);
      }
      /* status = bothDead - do nothing */
      return(bothDead);
    }
    /* segment already marked */
    /* must determine status of segment to return correct value */
    status = GetStatusOfWallSegment(wallSegPtr);
    if (status == alive)
    {
      return(alive);
    }
    else if ((status == topDead) || (status == botDead))
    {
      return(halfDead);
    }
    else return(bothDead);

  }
  return(noSeg);
}



static void
AddPPinsToList(bnTilePtr, direction, lnPPinListPtr)
Tile            *bnTilePtr;
SideDirection   direction;
LinkedPPin      **lnPPinListPtr;
{
	(void) Tp_ItrPPinsOfSide(bnTilePtr, (SideDirection) direction,
  	  AddPins, (ClientData) lnPPinListPtr);
}



static BearStatus
AddPins(ppinPtr, lnPPinListPtr)
PseudoPin       *ppinPtr;
LinkedPPin      **lnPPinListPtr;
{
	LinkedPPin *lnPPinPtr;

   	MALLOC(LinkedPPin *, lnPPinPtr, sizeof (LinkedPPin));
	lnPPinPtr->ppinPtr = ppinPtr;
	lnPPinPtr->nextPtr = *lnPPinListPtr;
	*lnPPinListPtr = lnPPinPtr;
	return (BEAR_CONTINUE_SEARCH);
}



static void
AppendPinsToList(pinLists, faceListsPtr)
PinsAroundJunction pinLists;
PinsAroundJunction *faceListsPtr;
{
   LinkedPinInst *pinPtr;
   LinkedRPin    *rPinPtr;
   LinkedPPin    *pPinPtr;

   /* Copy all pins (pinInsts, RoutePins and Pseudo_pins) */
   if (faceListsPtr->horzPPinList == NIL(LinkedPPin))
   {
     faceListsPtr->horzPPinList = pinLists.horzPPinList;
   }
   else if (pinLists.horzPPinList != NIL(LinkedPPin))
   {
     for (pPinPtr = pinLists.horzPPinList;
	  pPinPtr->nextPtr != NIL(LinkedPPin); pPinPtr = pPinPtr->nextPtr);

     pPinPtr->nextPtr = faceListsPtr->horzPPinList;
     faceListsPtr->horzPPinList = pinLists.horzPPinList;
   }

   if (faceListsPtr->vertPPinList == NIL(LinkedPPin))
   {
     faceListsPtr->vertPPinList = pinLists.vertPPinList;
   }
   else if (pinLists.vertPPinList != NIL(LinkedPPin))
   {
     for (pPinPtr = pinLists.vertPPinList;
	  pPinPtr->nextPtr != NIL(LinkedPPin); pPinPtr = pPinPtr->nextPtr);

     pPinPtr->nextPtr = faceListsPtr->vertPPinList;
     faceListsPtr->vertPPinList = pinLists.vertPPinList;
   }

   if (faceListsPtr->pinInstList == NIL(LinkedPinInst))
   {
     faceListsPtr->pinInstList = pinLists.pinInstList;
   }
   else if (pinLists.pinInstList != NIL(LinkedPinInst))
   {
     for (pinPtr = pinLists.pinInstList;
	  pinPtr->nextPtr != NIL(LinkedPinInst); pinPtr = pinPtr->nextPtr);

     pinPtr->nextPtr = faceListsPtr->pinInstList;
     faceListsPtr->pinInstList = pinLists.pinInstList;
   }

   if (faceListsPtr->rPinList == NIL(LinkedRPin))
   {
     faceListsPtr->rPinList = pinLists.rPinList;
   }
   else if (pinLists.rPinList != NIL(LinkedRPin))
   {
     for (rPinPtr = pinLists.rPinList;
	  rPinPtr->nextPtr != NIL(LinkedRPin); rPinPtr = rPinPtr->nextPtr);
     rPinPtr->nextPtr = faceListsPtr->rPinList;
     faceListsPtr->rPinList = pinLists.rPinList;
   }
 }

int
GetStatusOfWallSegment(segPtr)
WallSegment *segPtr;
{
   BottleNeck *bottleNeckPtr;
   Tile *bnTilePtr;
   Tile *rightTilePtr;
   Tile *topTilePtr;
   Tile *botTilePtr;
   Tile *leftTilePtr;

   bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(segPtr);
   if (bottleNeckPtr->unType == TP_BN_TILE)
   {
     bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
     if (segPtr->direction == FPG_VERTICAL)
     {
	 GetLAndRTiles(bnTilePtr, &leftTilePtr, &rightTilePtr);
	 if (Tp_IsDCellTile(leftTilePtr) && Tp_IsDCellTile(rightTilePtr))
	   return (bothDead);
	 else if (Tp_IsDCellTile(leftTilePtr))
           return (leftDead);
	 else if (Tp_IsDCellTile(rightTilePtr))
	   return (rightDead);
         else return (alive);
     }
     else  /*horizontal segment */
     {
	 GetBAndTTiles(bnTilePtr, &botTilePtr, &topTilePtr);
	 if (Tp_IsDCellTile(botTilePtr) && Tp_IsDCellTile(topTilePtr))
	   return (bothDead);
         else if (Tp_IsDCellTile(botTilePtr))
	   return(botDead);
         else if (Tp_IsDCellTile(topTilePtr))
	   return (topDead);
         else return (alive);
     }
   }
   return (alive);
}

static void
GetLAndRTiles(tilePtr, leftTilePtrPtr, rightTilePtrPtr)
Tile  *tilePtr;
Tile  **leftTilePtrPtr;
Tile  **rightTilePtrPtr;
{
	Point        searchPnt;
	Rect         rect;
	Plane        *planePtr;

    planePtr = Tp_GetHorzPlane(tmGlobalPlaneDataPtr);
    (void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, tilePtr, &rect);
    searchPnt.p_x = rect.r_ll.p_x - 1;
    searchPnt.p_y = rect.r_ll.p_y;
    *leftTilePtrPtr = TiSrPointNoHint(planePtr, &searchPnt);
    searchPnt.p_x = rect.r_ur.p_x;
    searchPnt.p_y = rect.r_ll.p_y;
    *rightTilePtrPtr = TiSrPointNoHint(planePtr, &searchPnt);
}

static void
GetBAndTTiles(tilePtr, bottomTilePtrPtr, topTilePtrPtr)
Tile  *tilePtr;
Tile  **bottomTilePtrPtr;
Tile  **topTilePtrPtr;
{
	Point        searchPnt;
	Point        vSearchPnt;
	Rect         rect;
	Plane        *planePtr;

	planePtr = Tp_GetVertPlane(tmGlobalPlaneDataPtr);
	(void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, tilePtr, &rect);
	searchPnt.p_x = rect.r_ur.p_x;
	searchPnt.p_y = rect.r_ll.p_y - 1;
        Tp_HPointToVPoint(&searchPnt, &vSearchPnt);
	*bottomTilePtrPtr = TiSrPointNoHint(planePtr, &vSearchPnt);
	searchPnt.p_x = rect.r_ur.p_x;
	searchPnt.p_y = rect.r_ur.p_y;
    Tp_HPointToVPoint(&searchPnt, &vSearchPnt);
    *topTilePtrPtr = TiSrPointNoHint(planePtr, &vSearchPnt);
}



/* ARGSUSED */
BearStatus
UnmarkWallSeg(segPtr, clientData)
WallSegment *segPtr;
ClientData clientData;
{
   Fpg_UnmarkWallSeg(segPtr);
   return (BEAR_CONTINUE_SEARCH);
}


