
/* tmUtils.c --
 *
 *           Tim Collins
 *      (Professor E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"
#include "tileMappingInt.h"


/* ---------- Local Procedures ---------- */
static int FindBnTile();


void
TmGetLAndRBnTilesOfDCell(dCellPtr, leftBnTilePtrPtr, rightBnTilePtrPtr)
DummyCell *dCellPtr;
Tile      **leftBnTilePtrPtr;
Tile      **rightBnTilePtrPtr;
{
	Rect searchRect;

	searchRect.r_ll.p_x = dCellPtr->bbox.r_ur.p_x;
	searchRect.r_ll.p_y = dCellPtr->bbox.r_ll.p_y;
	searchRect.r_ur.p_x = dCellPtr->bbox.r_ur.p_x + 1;
	searchRect.r_ur.p_y = dCellPtr->bbox.r_ur.p_y;
	(void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
		 &searchRect, FindBnTile, (ClientData)rightBnTilePtrPtr);
	if (!(Tp_IsBottleneckTile(*rightBnTilePtrPtr)))
	{
	printf("Error in getting bn in TmGetLAndRBnTilesOfDCell\n");fflush(stdout);
	}
	searchRect.r_ll.p_x = dCellPtr->bbox.r_ll.p_x -1;
	searchRect.r_ll.p_y = dCellPtr->bbox.r_ll.p_y;
	searchRect.r_ur.p_x = dCellPtr->bbox.r_ll.p_x;
	searchRect.r_ur.p_y = dCellPtr->bbox.r_ur.p_y;
	(void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
		 &searchRect, FindBnTile, (ClientData)leftBnTilePtrPtr);
	if (!(Tp_IsBottleneckTile(*leftBnTilePtrPtr)))
	{
	printf("Error in getting bn in TmGetLAndRBnTilesOfDCell\n");fflush(stdout);
	}
}


void
TmGetBAndTBnTilesOfDCell(dCellPtr, bottomBnTilePtrPtr, topBnTilePtrPtr)
DummyCell *dCellPtr;
Tile      **bottomBnTilePtrPtr;
Tile      **topBnTilePtrPtr;
{
	Rect hSearchRect;
	Rect vSearchRect;

	hSearchRect.r_ll.p_x = dCellPtr->bbox.r_ll.p_x;
	hSearchRect.r_ll.p_y = dCellPtr->bbox.r_ur.p_y;
	hSearchRect.r_ur.p_x = dCellPtr->bbox.r_ur.p_x;
	hSearchRect.r_ur.p_y = dCellPtr->bbox.r_ur.p_y + 1;
	(void) Tp_HRectToVRect(&hSearchRect, &vSearchRect);
	(void) TiSrArea(NIL(Tile), Tp_GetVertPlane(tmGlobalPlaneDataPtr),
		 &vSearchRect, FindBnTile, (ClientData)topBnTilePtrPtr);
	if (!(Tp_IsBottleneckTile(*topBnTilePtrPtr)))
	{
	printf("Error in getting bn in TmGetTAndBBnTilesOfDCell\n");fflush(stdout);
	}
	hSearchRect.r_ll.p_x = dCellPtr->bbox.r_ll.p_x;
	hSearchRect.r_ll.p_y = dCellPtr->bbox.r_ll.p_y -1;
	hSearchRect.r_ur.p_x = dCellPtr->bbox.r_ur.p_x;
	hSearchRect.r_ur.p_y = dCellPtr->bbox.r_ll.p_y;
	(void) Tp_HRectToVRect(&hSearchRect, &vSearchRect);
	(void) TiSrArea(NIL(Tile), Tp_GetVertPlane(tmGlobalPlaneDataPtr),
		 &vSearchRect, FindBnTile, (ClientData)bottomBnTilePtrPtr);
	if (!(Tp_IsBottleneckTile(*bottomBnTilePtrPtr)))
	{
	    printf("Error in getting bn in TmGetTAndBBnTilesOfDCell\n");
	    fflush(stdout);
	}
}


static int
FindBnTile(tilePtr,bnTilePtrPtr)
Tile   *tilePtr;
Tile   **bnTilePtrPtr;
{
    if (Tp_IsBottleneckTile(tilePtr))
    {
	*bnTilePtrPtr = tilePtr;
	return (1);
    }
    return (0);
}


void
TmGetHSBnTilesOfDCell(dCellPtr, tSBnTilePtr, bSBnTilePtr)
DummyCell *dCellPtr;
Tile      **tSBnTilePtr;
Tile      **bSBnTilePtr;
{
	Point searchPnt;

	searchPnt.p_x = dCellPtr->bbox.r_ll.p_x;
	/* new change: r_ur.p_y -> r_ur.p_y - 1 */
	searchPnt.p_y = dCellPtr->bbox.r_ur.p_y - 1;
	*tSBnTilePtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
	  &searchPnt);
	searchPnt.p_x = dCellPtr->bbox.r_ll.p_x;
	searchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
	*bSBnTilePtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
	  &searchPnt);
}


void
TmGetVSBnTilesOfDCell(dCellPtr, rSBnTilePtr, lSBnTilePtr)
DummyCell *dCellPtr;
Tile      **rSBnTilePtr;
Tile      **lSBnTilePtr;
{
	Point searchPnt;
	Point searchVPnt;

	searchPnt.p_x = dCellPtr->bbox.r_ll.p_x + 1;
	searchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
	Tp_HPointToVPoint(&searchPnt, &searchVPnt);
	*lSBnTilePtr = TiSrPointNoHint(Tp_GetVertPlane(tmGlobalPlaneDataPtr),
	  &searchVPnt);
	searchPnt.p_x = dCellPtr->bbox.r_ur.p_x;
	searchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
	Tp_HPointToVPoint(&searchPnt, &searchVPnt);
	*rSBnTilePtr = TiSrPointNoHint(Tp_GetVertPlane(tmGlobalPlaneDataPtr),
	  &searchVPnt);
}


void
TmGetLAndRTiles(dCellPtr, leftBnTilePtrPtr, rightBnTilePtrPtr)
DummyCell *dCellPtr;
Tile      **leftBnTilePtrPtr;
Tile      **rightBnTilePtrPtr;
{
	Point searchPnt;

	searchPnt.p_x = dCellPtr->bbox.r_ur.p_x;
	searchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
	*rightBnTilePtrPtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
	  &searchPnt);
	searchPnt.p_x = dCellPtr->bbox.r_ll.p_x - 1;
	searchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
	*leftBnTilePtrPtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
	  &searchPnt);
}


void
TmGetTAndBTiles(dCellPtr, topBnTilePtrPtr, bottomBnTilePtrPtr)
DummyCell *dCellPtr;
Tile      **topBnTilePtrPtr;
Tile      **bottomBnTilePtrPtr;
{
	Point hSearchPnt;
	Point vSearchPnt;

	hSearchPnt.p_x = dCellPtr->bbox.r_ur.p_x;
	hSearchPnt.p_y = dCellPtr->bbox.r_ur.p_y;
	(void) Tp_HPointToVPoint(&hSearchPnt, &vSearchPnt);
	*topBnTilePtrPtr = TiSrPointNoHint(Tp_GetVertPlane(tmGlobalPlaneDataPtr),
	  &vSearchPnt);
	hSearchPnt.p_x = dCellPtr->bbox.r_ur.p_x;
	hSearchPnt.p_y = dCellPtr->bbox.r_ll.p_y - 1;
	(void) Tp_HPointToVPoint(&hSearchPnt, &vSearchPnt);
	*bottomBnTilePtrPtr = TiSrPointNoHint(Tp_GetVertPlane(tmGlobalPlaneDataPtr),
	  &vSearchPnt);
}


ZoneData *
TmMakeZoneData()
{
    ZoneData *zoneDataPtr;

    MALLOC(ZoneData *, zoneDataPtr, sizeof (ZoneData));
	zoneDataPtr->pPinList = NIL(LinkedPPin);
	zoneDataPtr->rPinList = NIL(LinkedRPin);
	zoneDataPtr->pinInstList = NIL(LinkedPinInst);
	zoneDataPtr->zoneType = NULL_ZONE_TYPE;
	zoneDataPtr->dCellPtr = NIL(DummyCell);
	zoneDataPtr->bnPtr = NIL(BottleNeck);
	zoneDataPtr->nextPtr = NIL(ZoneData);

    return (zoneDataPtr);
}


void
TmFreeZoneData(zoneDataPtrPtr)
ZoneData **zoneDataPtrPtr;
{
    FREE((char *) *zoneDataPtrPtr);
	*zoneDataPtrPtr = NIL(ZoneData);
}


PinsAroundJunction *
Tm_MakePinsAroundJunction()
{
    PinsAroundJunction *pajPtr;

    MALLOC(PinsAroundJunction *, pajPtr, sizeof (PinsAroundJunction));
    pajPtr->pinInstList = NIL(LinkedPinInst);
    pajPtr->rPinList = NIL(LinkedRPin);
    pajPtr->horzPPinList = NIL(LinkedPPin);
    pajPtr->vertPPinList = NIL(LinkedPPin);

    return (pajPtr);
}


void
Tm_FreePinsAroundJunction(pajPtr)
PinsAroundJunction *pajPtr;
{
    LinkedPinInst *lnPinInstPtr;
    LinkedPPin    *lnPPinPtr;
    LinkedRPin    *lnRPinPtr;

    for (lnPPinPtr = pajPtr->horzPPinList;
		lnPPinPtr != NIL(LinkedPPin);
        lnPPinPtr = pajPtr->horzPPinList) {

		pajPtr->horzPPinList = lnPPinPtr->nextPtr;
		FREE((char *) lnPPinPtr);
    }

    for (lnPPinPtr = pajPtr->vertPPinList;
		lnPPinPtr != NIL(LinkedPPin);
        lnPPinPtr = pajPtr->vertPPinList) {

	    pajPtr->vertPPinList = lnPPinPtr->nextPtr;
	    FREE((char *) lnPPinPtr);
    }

    for (lnPinInstPtr = pajPtr->pinInstList;
        lnPinInstPtr != NIL(LinkedPinInst);
        lnPinInstPtr = pajPtr->pinInstList) {

        pajPtr->pinInstList = lnPinInstPtr->nextPtr;
	    FREE((char *) lnPinInstPtr);
    }

    for (lnRPinPtr = pajPtr->rPinList;
        lnRPinPtr != NIL(LinkedRPin);
	    lnRPinPtr = pajPtr->rPinList) {

        pajPtr->rPinList = lnRPinPtr->nextPtr;
	    FREE((char *) lnRPinPtr);
    }
}


void
TmPrintFatalErr(message)
BearString message;
{
	errRaise(TM_MOD_NAME, ((int) -1), message);
}


