
/* tmVBnSplit.c --
 *
 *          Tim Collins
 *     (Professor E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"
#include "tileMappingInt.h"


typedef struct _BnLineData
{
    PseudoPin          *pPinList;
    DummyCell          *dCellPtr;
    struct _BnLineData *nextPtr;
} BnLineData;


typedef struct _LocalData
{
    Rect          *area;
    Table         *table;
    int           zone;
    SideDirection side;
    LinkedPinInst **pinList;
    LinkedPPin    **pPinList;
    LinkedRPin    **rPinList;
} LocalData;



/* ---------- Local Procedures ---------- */
static void         VBnLineSetup();
static void         VBnTileSetup();
static void         VBnLineSplit();
static void         VBnTileSplit();
static void         AddPPinToBnLineTile();
static void         HandleZonePins();
static void         OperateOnBottomPins();
static void         OperateOnBottomBn();
static void         OperateOnDCell();
static void         OperateOnTopBn();
static void         OperateOnTopPins();
static void         DividePPins();
static int          SearchSolidTile();
static BearStatus   GetPinsInRect();
static BearStatus   GetRPinsInRect();
static BearStatus   AddPPinsToTable();
static BearStatus   AddDCellPPinsToTable();


/* ---------- Local Global Variables ---------- */
static Table *tableHeadPtr;


/*
 * ----------------------------------------------------------------------------
 *
 * Tm_InsDCellOnVTP () --
 *
 *    This procedure inserts all the dummy cells, and distributes the pseudo
 *    pins that were on the vertical bottleneck tile, evenly among the two
 *    new bottleneck tiles, and the dummy cell.
 *
 * Results:
 *    All the dummy cells are inserted.
 *
 * Side effects:
 *    The dummy cells have vertical pseudo pins.
 *
 * ----------------------------------------------------------------------------
 */

void
Tm_InsDCellOnVTP(parentCellPtr)
CellMaster *parentCellPtr;
{
    BnLineData      *bnLineDataPtr;
    DummyCell       *dCellPtr;
    DCellClass      class;
    PlaneData       *planeDataPtr;
    WallJunction    *llJunctPtr;

    tmGlobalParentCellPtr = parentCellPtr;
    tmGlobalPlaneDataPtr = planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);
    bnLineDataPtr = NIL(BnLineData);
    if (planeDataPtr->pd_dcellList == NIL(DummyCell))
    {
        (void) Tp_CreateDCells(planeDataPtr);
    }
    dCellPtr = planeDataPtr->pd_dcellList;
    while (dCellPtr != NIL(DummyCell))
    {
        class = Mac_GetClassOfDCell(dCellPtr);
        if ((class == HBNTI_AND_VBNTI) || (class == HBNLN_AND_VBNTI))
        {
            (void) VBnTileSetup(dCellPtr);
        }
        dCellPtr = dCellPtr->nextPtr;
    }

    dCellPtr = planeDataPtr->pd_dcellList;
    while (dCellPtr != NIL(DummyCell))
    {
        tableHeadPtr = NIL(Table);
        class = Mac_GetClassOfDCell(dCellPtr);
        if ((class != HBNTI_AND_VBNLN) && (class != HBNLN_AND_VBNLN))
        {
            (void) VBnTileSplit(dCellPtr);
        }
        dCellPtr = dCellPtr->nextPtr;
    }

    dCellPtr = planeDataPtr->pd_dcellList;
    while (dCellPtr != NIL(DummyCell))
    {
        class = Mac_GetClassOfDCell(dCellPtr);
        if ((class == HBNTI_AND_VBNLN) || (class == HBNLN_AND_VBNLN))
        {
            (void) VBnLineSetup(dCellPtr, &bnLineDataPtr);
        }
        dCellPtr = dCellPtr->nextPtr;
    }

    while (bnLineDataPtr != NIL(BnLineData))
    {
        (void) VBnLineSplit(bnLineDataPtr);
        bnLineDataPtr = bnLineDataPtr->nextPtr;
    }
    Tp_IdentifySpaceTiles(planeDataPtr);
    (void) Fpg_CreateFloorPlanGraph(planeDataPtr, &llJunctPtr);
}


static void
VBnLineSetup(dCellPtr, bnLineDataPtrPtr)
DummyCell  *dCellPtr;
BnLineData **bnLineDataPtrPtr;
{
    /* 
     * Since by inserting the dummy cell the tile plane is changed, we must
     * first save all the information from the semi-bottleneck tiles, and
     * then paint the dummy cells.
     */

    BnLineData *bnLineDataPtr;
    BNLineType bnType;
    Line       line;
    Tile       *lSBnTilePtr;
    Tile       *rSBnTilePtr;

    MALLOC(BnLineData *, bnLineDataPtr, sizeof (BnLineData));
    (void) TmGetVSBnTilesOfDCell(dCellPtr, &rSBnTilePtr, &lSBnTilePtr);
    (void) Tp_FindBnLine(lSBnTilePtr, rSBnTilePtr, &line);
    /*
     * The length will be used to distribute the pseudo pin evenly after the
     * insertion of the dummy cell.
     */
    if (Tp_IsPairOfSBNTiles(&lSBnTilePtr, &rSBnTilePtr, &bnType) == FALSE)
    {
         Bear_PrintErr("VBnLineSetup(): not a pair of sbn tile.");   
    }
    /*
     * Save the pseudo pins from the correct fragment of the semi-bottleneck
     * tile.
     */
    if (bnType == TP_VERT_R_DOWN)
    {
        (void) Tp_RemovePPinListOfTile(rSBnTilePtr, 
          BOTTOM_LEFT_SIDE, TP_TOP, &bnLineDataPtr->pPinList);
    }
    else
    {
        (void) Tp_RemovePPinListOfTile(rSBnTilePtr, 
            BOTTOM_LEFT_SIDE, TP_BOTTOM,
            &bnLineDataPtr->pPinList);
    }
    (void) Tp_FreeTileProp(lSBnTilePtr);
    (void) Tp_FreeTileProp(rSBnTilePtr);
    Tp_IdentifySpaceTiles(tmGlobalPlaneDataPtr);
    bnLineDataPtr->dCellPtr = dCellPtr;
    bnLineDataPtr->nextPtr = *bnLineDataPtrPtr;
    *bnLineDataPtrPtr = bnLineDataPtr;
}


static void
VBnLineSplit(bnLineDataPtr)
BnLineData *bnLineDataPtr;
{
    /*
     * Paint the dummy cell and then distribute the pseudo pins evenly among
     * the two horizontal bottleneck tiles, and the dummy cell.
     */
    Tile           *topBnTilePtr;
    Tile           *bottomBnTilePtr;
    Tile           *leftBnTilePtr;
    Tile           *rightBnTilePtr;
    /* Rect           hRect; */
    WallJunction   *fpgPtr;

    (void) Tp_PaintDCellOnHorzPlane(bnLineDataPtr->dCellPtr, 
      tmGlobalPlaneDataPtr);
    Tp_IdentifySpaceTiles(tmGlobalPlaneDataPtr);
        (void) Fpg_CreateFloorPlanGraph(tmGlobalPlaneDataPtr, &fpgPtr);
    (void) TmGetLAndRBnTilesOfDCell(bnLineDataPtr->dCellPtr, &leftBnTilePtr, 
      &rightBnTilePtr);
    /*
    (void) Tp_CreateSpaceTileProp(leftBnTilePtr);
    (void) Tp_PutTypeOfSpaceTile(TP_HORZ_BN_TILE, leftBnTilePtr);
    (void) Tp_CreateSpaceTileProp(rightBnTilePtr);
    (void) Tp_PutTypeOfSpaceTile(TP_HORZ_BN_TILE, rightBnTilePtr);
    */
    (void) Tp_PaintDCellOnVertPlane(bnLineDataPtr->dCellPtr, 
      tmGlobalPlaneDataPtr);
    Tp_IdentifySpaceTiles(tmGlobalPlaneDataPtr);
        (void) Fpg_CreateFloorPlanGraph(tmGlobalPlaneDataPtr, &fpgPtr);
    (void) TmGetBAndTBnTilesOfDCell(bnLineDataPtr->dCellPtr, &bottomBnTilePtr, 
      &topBnTilePtr);
    /*
    (void) Tp_CreateSpaceTileProp(topBnTilePtr);
    (void) Tp_PutTypeOfSpaceTile(TP_VERT_BN_TILE, topBnTilePtr);
    (void) Tp_CreateSpaceTileProp(bottomBnTilePtr);
    (void) Tp_PutTypeOfSpaceTile(TP_VERT_BN_TILE, bottomBnTilePtr);
    (void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, topBnTilePtr, &hRect);
    */

    while (bnLineDataPtr->pPinList != NIL(PseudoPin))
    {
        (void) AddPPinToBnLineTile(bottomBnTilePtr, bnLineDataPtr);
    }
}



static void
AddPPinToBnLineTile(bnTilePtr, bnLineDataPtr)
Tile       *bnTilePtr;
BnLineData *bnLineDataPtr;
{
    PseudoPin *pPinPtr;
    PseudoPin *newPPinPtr;
    BearId    newId;
    BearId    intId;
    Net       *netPtr;
    float     width;

    pPinPtr = bnLineDataPtr->pPinList;
    bnLineDataPtr->pPinList = bnLineDataPtr->pPinList->nextPtr;
    pPinPtr->nextPtr = NIL(PseudoPin);
    Tp_InsertPPinOfBnTile(bnTilePtr, TOP_RIGHT_SIDE, pPinPtr);
    Tp_CreatePPin(bnTilePtr, BOTTOM_LEFT_SIDE, TP_BOTTOM, &newPPinPtr);
    newId = Mac_GenerateId();
    intId = Tp_GetIntNetIdOfPPin(pPinPtr);
    netPtr = Tp_GetNetOfPPin(pPinPtr);
    width = Tp_GetWidthOfPPin(pPinPtr);

    (void) Tp_PutIntNetIdOfPPin(newId, pPinPtr);
    (void) Tp_PutIntNetIdOfPPin(newId, newPPinPtr);
    (void) Tp_PutExtNetIdOfPPin(intId, newPPinPtr);
    (void) Tp_PutNetOfPPin(netPtr, newPPinPtr);
    (void) Tp_PutWidthOfPPin(width, newPPinPtr);
}



static void
VBnTileSetup(dCellPtr)
DummyCell *dCellPtr;
{
    Tile        *hbnTilePtr;
        Point       hSearchPnt;
        WallJunction *fpgPtr;

        /* Paint dummy cell on Horz Plane */

    hSearchPnt.p_x = dCellPtr->bbox.r_ll.p_x;
    hSearchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
    hbnTilePtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
                                     &hSearchPnt);
        (void) Tp_FreeTileProp(hbnTilePtr);
    (void) Tp_PaintDCellOnHorzPlane(dCellPtr, tmGlobalPlaneDataPtr);
    Tp_IdentifySpaceTiles(tmGlobalPlaneDataPtr);
    (void) Fpg_CreateFloorPlanGraph(tmGlobalPlaneDataPtr, &fpgPtr);
}



static void
VBnTileSplit(dCellPtr)
DummyCell *dCellPtr;
{
    Tile         *vbnTilePtr;
    Tile         *hbnTilePtr;
    Rect         hrect;
    Rect         vrect;
    Tile         *topBnTilePtr;
    Tile         *bottomBnTilePtr;
    PseudoPin    *leftList;
    PseudoPin    *rightList;
    Point        hSearchPnt;
    Point        vSearchPnt;
    ZoneData     *zoneDataPtr;
    WallJunction *fpgPtr;
    WallSegment  *wallSegPtr;
    WallJunction *junct1Ptr;
    WallJunction *junct2Ptr;
    Point        *j1PointPtr;
    Point        *j2PointPtr;

    zoneDataPtr = TmMakeZoneData();
    zoneDataPtr->nextPtr = TmMakeZoneData();
    zoneDataPtr->nextPtr->nextPtr = TmMakeZoneData();
    zoneDataPtr->nextPtr->nextPtr->nextPtr = NIL(ZoneData);

    hSearchPnt.p_x = dCellPtr->bbox.r_ur.p_x;
    hSearchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
    (void) Tp_HPointToVPoint(&hSearchPnt, &vSearchPnt);
    vbnTilePtr = TiSrPointNoHint(Tp_GetVertPlane(tmGlobalPlaneDataPtr),
      &vSearchPnt);
    hSearchPnt.p_x = dCellPtr->bbox.r_ll.p_x;
    hSearchPnt.p_y = dCellPtr->bbox.r_ll.p_y;
    hbnTilePtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
      &hSearchPnt);
    (void) Mac_PutTileOfDCell(hbnTilePtr, dCellPtr);
    Tp_HorzTileToRect(tmGlobalPlaneDataPtr, hbnTilePtr, &hrect);
    wallSegPtr = Tp_GetWallSegmentOfBnTile(vbnTilePtr);
    junct1Ptr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
    junct2Ptr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
    j1PointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct1Ptr);
    j2PointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, junct2Ptr);
    hrect.r_xbot = j1PointPtr->p_x;
    hrect.r_xtop = j2PointPtr->p_x;
    Tp_HRectToVRect(&hrect, &vrect);
        /* Unify the circle (left face, vertBnTile, right face) */
    Tm_VCircleUnify(tmGlobalParentCellPtr, tmGlobalPlaneDataPtr, vrect);
    /* Delete old PPins from horzBnTile */
    (void) Tp_FreePPinsOfTile(vbnTilePtr);
        /* Reassign ppins to VertBnTile by circle split */
    Tm_VCSplit(tmGlobalParentCellPtr, tmGlobalPlaneDataPtr, vrect);

        /* Save ppin list since painting will overwrite the list */
    (void) Tp_RemovePPinListOfTile(vbnTilePtr, 
                       TOP_RIGHT_SIDE,
                       TP_BOTTOM, 
                       &rightList);
    (void) Tp_RemovePPinListOfTile(vbnTilePtr, 
                       BOTTOM_LEFT_SIDE,
                       TP_BOTTOM, 
                       &leftList);

        /* Paint dummy cell on vertical plane */
    (void) Tp_FreeTileProp(vbnTilePtr);
    (void) Tp_PaintDCellOnVertPlane(dCellPtr, tmGlobalPlaneDataPtr);
    Tp_IdentifySpaceTiles(tmGlobalPlaneDataPtr);
    (void) TmGetBAndTBnTilesOfDCell(dCellPtr, &bottomBnTilePtr, 
                    &topBnTilePtr);

        /* Proportionally divide the list of pins between the bottom */
        /* and top of the dummy cell */
    (void) DividePPins(bottomBnTilePtr,topBnTilePtr,
               TOP_RIGHT_SIDE,rightList);
    (void) DividePPins(bottomBnTilePtr,topBnTilePtr,
               BOTTOM_LEFT_SIDE,leftList);
        /* Create ppins on the dummy cell to connect nets in bnRegion */
    (void) HandleZonePins(topBnTilePtr, dCellPtr, bottomBnTilePtr,
        zoneDataPtr);
    (void) TmVSplitByZones(3, tableHeadPtr, zoneDataPtr); 
    (void) Fpg_CreateFloorPlanGraph(tmGlobalPlaneDataPtr, &fpgPtr);
}



static void
DividePPins(bottomBnTilePtr,topBnTilePtr,side,pPinList)
Tile           *bottomBnTilePtr;
Tile           *topBnTilePtr;
SideDirection  side;
PseudoPin      *pPinList;
{
    Rect     rect;
    int      bottomDY;
    int      topDY;
    int      pPinNum;
    int      bottomPPinNum;
    PseudoPin   *pPinPtr;

    pPinNum = 0;
    bottomPPinNum = 0;
    for(pPinPtr=pPinList ; pPinPtr!=NIL(PseudoPin) ; pPinPtr=pPinPtr->nextPtr)
    {
    pPinNum++;
    }
    (void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, bottomBnTilePtr, &rect);
    bottomDY = rect.r_ur.p_y - rect.r_ll.p_y;
    (void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, topBnTilePtr, &rect);
    topDY = rect.r_ur.p_y - rect.r_ll.p_y;

    bottomPPinNum = pPinNum * ((float)bottomDY/(bottomDY + topDY)) + 0.5;
    if (bottomPPinNum <= 0)
    {
    (void) Tp_AppendPPinListOfTile(NIL(PseudoPin),bottomBnTilePtr,
                       side,TP_BOTTOM);
    (void) Tp_AppendPPinListOfTile(pPinList,topBnTilePtr,
                       side,TP_BOTTOM);
    }
    else
    {
    (void) Tp_AppendPPinListOfTile(pPinList,bottomBnTilePtr,side,TP_BOTTOM);
    pPinPtr = pPinList;
    for (bottomPPinNum-- ; bottomPPinNum > 0 ; bottomPPinNum--)
    {
        pPinPtr = pPinPtr->nextPtr;
    }
    (void) Tp_AppendPPinListOfTile(pPinPtr->nextPtr,topBnTilePtr,
                       side,TP_BOTTOM);
    pPinPtr->nextPtr = NIL(PseudoPin);
    }
}


static void
HandleZonePins(tBnTilePtr, dCellPtr, bBnTilePtr, zoneDataPtr)
Tile        *tBnTilePtr;
DummyCell   *dCellPtr;
Tile        *bBnTilePtr;
ZoneData    *zoneDataPtr;
{
    /*
     * This procedure operates on the pins in each of the 3 zones :
     * adding them to the table, and saving the pin insts and the pseudo pins
     * so that their id's can be changed if needed.
     */
    ZoneData *zonePlacePtr;

    zonePlacePtr = zoneDataPtr;

    zonePlacePtr->pinInstList = NIL(LinkedPinInst);
    zonePlacePtr->rPinList = NIL(LinkedRPin);
    zonePlacePtr->pPinList = NIL(LinkedPPin);
    (void) OperateOnBottomPins(bBnTilePtr, zonePlacePtr);
    (void) OperateOnBottomBn(bBnTilePtr, &(zonePlacePtr->pPinList));
    zonePlacePtr->zoneType = DJUNCTION;
    MALLOC(BottleNeck *, zonePlacePtr->bnPtr, sizeof (BottleNeck));
    zonePlacePtr->bnPtr->unValue.bnTilePtr = bBnTilePtr;

    zonePlacePtr = zoneDataPtr->nextPtr;

    zonePlacePtr->pinInstList = NIL(LinkedPinInst);
    zonePlacePtr->rPinList = NIL(LinkedRPin);
    zonePlacePtr->pPinList = NIL(LinkedPPin);
    (void) OperateOnDCell(dCellPtr, &(zonePlacePtr->pPinList));
    zonePlacePtr->zoneType = DCELL;
    zonePlacePtr->dCellPtr = dCellPtr;

    zonePlacePtr = zoneDataPtr->nextPtr->nextPtr;

    zonePlacePtr->pPinList = NIL(LinkedPPin);
    (void) OperateOnTopBn(tBnTilePtr, &(zonePlacePtr->pPinList));
    zonePlacePtr->pinInstList = NIL(LinkedPinInst);
    zonePlacePtr->rPinList = NIL(LinkedRPin);
    (void) OperateOnTopPins(tBnTilePtr, zonePlacePtr);
    zonePlacePtr->zoneType = DJUNCTION;
    MALLOC(BottleNeck *, zonePlacePtr->bnPtr, sizeof (BottleNeck));
    zonePlacePtr->bnPtr->unValue.bnTilePtr = tBnTilePtr;

    zonePlacePtr->nextPtr = NIL(ZoneData);
}


static void
OperateOnTopPins(tBnTilePtr, zonePlacePtr)
Tile      *tBnTilePtr;
ZoneData  *zonePlacePtr;
{
    /* 
     * Add the pins to the table, and save them for future access.
     */
    Rect      searchRect;
    LocalData localData;

    (void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, tBnTilePtr, &searchRect);
    searchRect.r_ybot = searchRect.r_ytop - 1;
    searchRect.r_ytop += 1;
    localData.area = &searchRect;
    localData.pinList = &(zonePlacePtr->pinInstList);
    localData.rPinList = &(zonePlacePtr->rPinList);
    localData.pPinList = &(zonePlacePtr->pPinList);
    localData.side = BOTTOM_SIDE;
    localData.zone = 3;
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
      &searchRect,
      SearchSolidTile,
      (ClientData) &localData);
    /* for i/o pins */
    (void) Mac_ItrPinOfCellInst(Mac_GetInstOfCoreMaster(tmGlobalParentCellPtr),
        GetPinsInRect, (ClientData) &localData);
}


static void
OperateOnTopBn(tBnTilePtr, pPinListPtr)
Tile          *tBnTilePtr;
LinkedPPin    **pPinListPtr;
{
    /* 
     * Add the pseudo pins to the table, and save them for future access.
     */
    LocalData localData;

    localData.pPinList = pPinListPtr;
    localData.zone = 3;
    (void) Tp_ItrPPinsOfSide(tBnTilePtr, TOP_RIGHT_SIDE, AddPPinsToTable,
      (ClientData) &localData);
    (void) Tp_ItrPPinsOfSide(tBnTilePtr, BOTTOM_LEFT_SIDE, AddPPinsToTable,
      (ClientData) &localData);
}


static void
OperateOnDCell(dCellPtr, pPinListPtr)
DummyCell     *dCellPtr;
LinkedPPin    **pPinListPtr;
{
    /* 
     * Add the pseudo pins to the table, and save them for future access.
     */
    LocalData localData;

    localData.pPinList = pPinListPtr;
    localData.zone = 2;
/*
 * code change : by george : on 18 oct 89
 *
    (void) Mac_ItrPPinOfDCell(dCellPtr, LEFT_SIDE,
        AddPPinsToTable, (ClientData) &localData);
    (void) Mac_ItrPPinOfDCell(dCellPtr, RIGHT_SIDE,
        AddPPinsToTable, (ClientData) &localData);
 */
    (void) Mac_ItrPPinOfDCell(dCellPtr, LEFT_SIDE, AddDCellPPinsToTable,
        (ClientData) &localData);
    (void) Mac_ItrPPinOfDCell(dCellPtr, RIGHT_SIDE, AddDCellPPinsToTable,
        (ClientData) &localData);
}


static void
OperateOnBottomBn(bBnTilePtr, pPinListPtr)
Tile          *bBnTilePtr;
LinkedPPin    **pPinListPtr;
{
    /* 
     * Add the pseudo pins to the table, and save them for future access.
     */
    LocalData localData;

    localData.pPinList = pPinListPtr;
    localData.zone = 1;
    (void) Tp_ItrPPinsOfSide(bBnTilePtr, TOP_RIGHT_SIDE, AddPPinsToTable,
      (ClientData) &localData);
    (void) Tp_ItrPPinsOfSide(bBnTilePtr, BOTTOM_LEFT_SIDE, AddPPinsToTable,
      (ClientData) &localData);
}


static void
OperateOnBottomPins(bBnTilePtr, zonePlacePtr)
Tile      *bBnTilePtr;
ZoneData  *zonePlacePtr;
{
    /* 
     * Add the pins to the table, and save them for future access.
     */
    Rect      searchRect;
    LocalData localData;

    (void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, bBnTilePtr, &searchRect);
    searchRect.r_ytop = searchRect.r_ybot + 1;
    searchRect.r_ybot -= 1;
    localData.area = &searchRect;
    localData.pinList = &(zonePlacePtr->pinInstList);
    localData.rPinList = &(zonePlacePtr->rPinList);
    localData.pPinList = &(zonePlacePtr->pPinList);
    localData.side = TOP_SIDE;
    localData.zone = 1;
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
      &searchRect,
      SearchSolidTile,
      (ClientData) &localData);
    /* for i/o pins */
    (void) Mac_ItrPinOfCellInst(Mac_GetInstOfCoreMaster(tmGlobalParentCellPtr),
        GetPinsInRect, (ClientData) &localData);
}


static BearStatus
AddDCellPPinsToTable(pPinPtr, localDataPtr)
PseudoPin *pPinPtr;
LocalData *localDataPtr;
{
    LinkedPPin *pinPtr;
    BearId     id;
    Net        *netPtr;
    float      width;

    MALLOC(LinkedPPin *, pinPtr, sizeof (LinkedPPin));
    pinPtr->ppinPtr = pPinPtr;
    pinPtr->nextPtr = *(localDataPtr->pPinList);
    *(localDataPtr->pPinList) = pinPtr;
    id = Tp_GetExtNetIdOfPPin(pPinPtr);
    netPtr = Tp_GetNetOfPPin(pPinPtr);
    width = Tp_GetWidthOfPPin(pPinPtr);
    (void) TmNumberTable(&tableHeadPtr, id, width, netPtr, localDataPtr->zone);

    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
AddPPinsToTable(pPinPtr, localDataPtr)
PseudoPin *pPinPtr;
LocalData *localDataPtr;
{
    LinkedPPin *pinPtr;
    BearId     id;
    Net        *netPtr;
    float      width;

    MALLOC(LinkedPPin *, pinPtr, sizeof (LinkedPPin));
    pinPtr->ppinPtr = pPinPtr;
    pinPtr->nextPtr = *(localDataPtr->pPinList);
    *(localDataPtr->pPinList) = pinPtr;
    id = Tp_GetIntNetIdOfPPin(pPinPtr);
    netPtr = Tp_GetNetOfPPin(pPinPtr);
    width = Tp_GetWidthOfPPin(pPinPtr);
    (void) TmNumberTable(&tableHeadPtr, id, width, netPtr, localDataPtr->zone);

    return (BEAR_CONTINUE_SEARCH);
}


static int
SearchSolidTile(tilePtr, localDataPtr)
Tile      *tilePtr;
LocalData *localDataPtr;
{
    CellInst    *cellInstPtr;
    RouteCell   *routeCellPtr;
    DummyCell   *dCellPtr;

    if (Tp_IsCellInstTile(tilePtr))
    {
        cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
        (void) Mac_ItrPinOfCellInst(cellInstPtr, GetPinsInRect,
          (ClientData) localDataPtr);
    }
#ifdef USE_DR_CODE
    if (Tp_IsRCellTile(tilePtr))
    {
        routeCellPtr = Tp_GetRCellOfTile(tilePtr);
        (void) Mac_ItrSidePinOfRCell(routeCellPtr, GetRPinsInRect,
          (ClientData) localDataPtr);
    }
#endif /* USE_DR_CODE */
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, localDataPtr->side, 
          AddDCellPPinsToTable, (ClientData) localDataPtr);
    }

    return (MAGIC_CONTINUE_SEARCH);
}


static BearStatus
GetRPinsInRect(rPinPtr, localDataPtr)
RoutePin  *rPinPtr;
LocalData *localDataPtr;
{
    LinkedRPin  *linkRPinPtr;
    Point       absPos;
    BearId      vId;
    Net         *netPtr;
    float       width;
    float       *widthVect;

    (void) Mac_GetAbsPosOfRPin(rPinPtr, &absPos);
    if ((absPos.p_x >= localDataPtr->area->r_xbot) &&
      (absPos.p_x <= localDataPtr->area->r_xtop) &&
      (absPos.p_y >= localDataPtr->area->r_ybot) &&
      (absPos.p_y <= localDataPtr->area->r_ytop))
    {
        MALLOC(LinkedRPin *, linkRPinPtr, sizeof (LinkedRPin));
        linkRPinPtr->rPinPtr = rPinPtr;
        linkRPinPtr->nextPtr = *(localDataPtr->rPinList);
        *(localDataPtr->rPinList) = linkRPinPtr;
        vId = Mac_GetVExtNetIdOfRPin(rPinPtr);
        netPtr = Mac_GetNetPtrOfRPin(rPinPtr);
        widthVect = Mac_GetWidthVectOfRPin(rPinPtr);
        width = widthVect[METAL2_LAYER];
        (void) TmNumberTable(&tableHeadPtr, vId, width, netPtr, 
          localDataPtr->zone);
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
GetPinsInRect(pinInstPtr, localDataPtr)
PinInst   *pinInstPtr;
LocalData *localDataPtr;
{
    LinkedPinInst *pinPtr;
    Point         absPos;
    BearId        vId;
    Net           *netPtr;
    float         width;
    float         *widthVect;

    (void) Mac_GetAbsPosOfPin(pinInstPtr, &absPos);
    if ((absPos.p_x >= localDataPtr->area->r_xbot) &&
      (absPos.p_x <= localDataPtr->area->r_xtop) &&
      (absPos.p_y >= localDataPtr->area->r_ybot) &&
      (absPos.p_y <= localDataPtr->area->r_ytop))
    {
        MALLOC(LinkedPinInst *, pinPtr, sizeof (LinkedPinInst));
        pinPtr->pinInstPtr = pinInstPtr;
        pinPtr->nextPtr = *(localDataPtr->pinList);
        *(localDataPtr->pinList) = pinPtr;
        vId = Mac_GetVExtNetIdOfPin(pinInstPtr);
        netPtr = Mac_GetNetOfPin(pinInstPtr);
        widthVect = Mac_GetWidthVectOfPin(pinInstPtr);
        width = widthVect[METAL2_LAYER];
        (void) TmNumberTable(&tableHeadPtr, vId, width, netPtr, 
          localDataPtr->zone);
    }
    return (BEAR_CONTINUE_SEARCH);
}


/*
 * old code
 *

static void AddPPinToLeftOfBnTile();
static void AddPPinToRightOfBnTile();

static void
AddPPinToLeftOfBnTile(bnTilePtr, leftList)
Tile      *bnTilePtr;
PseudoPin **leftList;
{
    PseudoPin *ppinPtr;

    ppinPtr = *leftList;
    *leftList = ppinPtr->nextPtr;
    ppinPtr->nextPtr = NIL(PseudoPin);
    Tp_InsertPPinOfBnTile(bnTilePtr, BOTTOM_LEFT_SIDE, ppinPtr);
}


static void
AddPPinToRightOfBnTile(bnTilePtr, rightList)
Tile      *bnTilePtr;
PseudoPin **rightList;
{
    PseudoPin *pPinPtr;

    pPinPtr = *rightList;
    *rightList = pPinPtr->nextPtr;
    pPinPtr->nextPtr = NIL(PseudoPin);
    Tp_InsertPPinOfBnTile(bnTilePtr, TOP_RIGHT_SIDE, pPinPtr);
}
*/


