
/* tmVFUnify.c --
 *
 *          Tim Collins
 *     (Professor E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "tile.h"
#include "bear.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "mac.h"
#include "tileMapping.h"
#include "tileMappingInt.h"

typedef struct _LocalData
{
    Rect         rect;
    WallJunction *junctPtr;
} LocalData;

typedef struct _LocalInfo
{
    Rect               *area;
    PseudoPin          *pseudoPin;
    Tile               *tilePtr;
    PinsAroundJunction *pinListsPtr;
} LocalInfo;

typedef struct _TileQueue
{
    Tile              *tilePtr;
    int               bottom;
    int               paired;
    struct _TileQueue *nextPtr;
} TileQueue;


#define bnTileLeftDead  1
#define bnTileRightDead 2
#define bnTileBothDead  3
#define bnTileAlive     4


/* ---------- Local Procedures ---------- */
static int UnifyEachDominantTile();
/* static BearStatus FixPPinsInTile(); */
/* static BearStatus InsertPPinsInList(); */
static int ItrSolidTile();
static int GetPinsOfSolidTile();
static BearStatus CheckAgainstJunctPins();
static BearStatus CheckAgainstBottomPin();
static BearStatus CheckTopBottomPin();
static BearStatus CheckLeftRightPin();
static BearStatus CheckLeftRightRPin();
static int GetAndSortBN();
static BearStatus CheckDCellPPins();
static BearStatus GetPPinsOfDCell();
static BearStatus GetRPinsInRect();
static BearStatus GetPinsInRect();
static BearStatus GetPPins();
static BearStatus FindBnLine();
static void ItrPinsOnLeftAndRight();
static void InsertBnInList();
static void PropagateExtToVExt();
static BearStatus PropagateExtIdOfIOPads();
static void CopyPinListsToFaceLists();
static void FixTileList();
static BNLine * TmGetHBnLineOfSBnTiles();
int    GetStatusOfHBnTile();
void Tm_GetPinsInHBottleNeckRegion2();
static BearStatus FindJunctInRect();


/* ---------- Local Global Variables ---------- */
static int globalCopyFlag;
static Rect tmGlobalDmntRect;


/*
 * ----------------------------------------------------------------------------
 *
 * Tm_VFUnify () --
 *
 * This procedure looks at the vertical dominant tiles one by one, and
 * translates the global routing information that is on both planes, and puts
 * it on the vertical plane.
 *
 * Results:
 *   All the routing information is on the vertical plane.
 *
 * Side effects:
 *   All the routing information on the horizontal plane is no good anymore.
 * ----------------------------------------------------------------------------
 */

void
Tm_VFUnify(copyFlag)
int copyFlag;
{

    tmGlobalPlaneDataPtr = Mac_GetCellPlaneOfCell(tmGlobalParentCellPtr);
    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(tmGlobalPlaneDataPtr),
      Tp_GetVertPlaneBnd(tmGlobalPlaneDataPtr),
      UnifyEachDominantTile,
      (ClientData) copyFlag);
    /* This is not ready to be put in the source
       It is an attempt to delete duplicate pseudoPins
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
      Tp_GetHorzPlaneBnd(tmGlobalPlaneDataPtr),
      FixPPinsInTile,
      (ClientData) NULL);
     */
}


static int
UnifyEachDominantTile(tilePtr, copyFlag)
Tile      *tilePtr;
int       copyFlag;
{
    Rect dmntRect;

    if (Tp_IsDominantTile(tilePtr))
    {
        (void) Tp_VertTileToRect(tmGlobalPlaneDataPtr, tilePtr, 
          &dmntRect);
        (void) TmVRectUnify(tmGlobalPlaneDataPtr, dmntRect, copyFlag);
    }
    return (0);
}



#ifdef BETTY_BOOP
/*
 * Under developement
 */
static BearStatus
FixPPinsInTile(tilePtr, clientData)
Tile        *tilePtr;
ClientData  clientData;
{
    PinsAroundJunction  *faceListsPtr;
    PseudoPin           *pinList;
    LinkedPPin          *lnPPinList;
    LinkedPPin          *lnPPinPtr;
    LinkedPPin          *tmpLnPPinPtr;
    BearId              intId;
    BearId              extId;

    if (Tp_IsBottleneckTile(tilePtr))
    {
/*
 * code change : by george : on 25 oct 89
 *
        MALLOC(PinsAroundJunction *, faceListsPtr, sizeof (PinsAroundJunction));
        faceListsPtr->vertPPinList = NIL(LinkedPPin);
 */
        faceListsPtr = Tm_MakePinsAroundJunction();

        (void) Tp_ItrPPinsOfSide(tilePtr, (SideDirection) TOP_RIGHT_SIDE, 
            InsertPPinsInList, (ClientData) faceListsPtr);

        lnPPinList = faceListsPtr->vertPPinList;
        if (lnPPinList != NIL(LinkedPPin))
        {
        for (lnPPinPtr = lnPPinList; lnPPinPtr != NIL(LinkedPPin);
             lnPPinPtr = lnPPinPtr->nextPtr)
        {
            intId = TmGetTempIntNetIdOfPPin(lnPPinPtr->ppinPtr);
            extId = TmGetTempExtNetIdOfPPin(lnPPinPtr->ppinPtr);
            for (tmpLnPPinPtr = lnPPinPtr->nextPtr;
                tmpLnPPinPtr != NIL(LinkedPPin);
                tmpLnPPinPtr = tmpLnPPinPtr->nextPtr)
            {
               if ((TmGetTempIntNetIdOfPPin(tmpLnPPinPtr->ppinPtr) == intId) &&
                   (TmGetTempExtNetIdOfPPin(tmpLnPPinPtr->ppinPtr) == extId))
               {
                  if (tmpLnPPinPtr->prevPtr != NIL(LinkedPPin))
                  {
                     tmpLnPPinPtr->prevPtr->nextPtr = tmpLnPPinPtr->nextPtr;
                  }
                  if (tmpLnPPinPtr->nextPtr != NIL(LinkedPPin))
                  {
                     tmpLnPPinPtr->nextPtr->prevPtr = tmpLnPPinPtr->prevPtr;
                  }
               }
            }
        }
        Tp_RemovePPinListOfTile(tilePtr, (SideDirection) TOP_RIGHT_SIDE, 
             (int) TP_BOTTOM, &pinList);
        for (lnPPinPtr = lnPPinList; lnPPinPtr != NIL(LinkedPPin);
             lnPPinPtr = lnPPinPtr->nextPtr)
        {
            Tp_InsertPPinOfBnTile(tilePtr, (SideDirection) TOP_RIGHT_SIDE, 
             lnPPinPtr->ppinPtr);
        }
        }
        /* Do the bottomLeft side */
        faceListsPtr->vertPPinList = NIL(LinkedPPin);
        (void) Tp_ItrPPinsOfSide(tilePtr, (SideDirection) BOTTOM_LEFT_SIDE,
            GetPPins, (ClientData)faceListsPtr);
        lnPPinList = faceListsPtr->vertPPinList;
        if (lnPPinList != NIL(LinkedPPin))
        {
        for (lnPPinPtr = lnPPinList; lnPPinPtr != NIL(LinkedPPin);
             lnPPinPtr = lnPPinPtr->nextPtr)
        {
            intId = TmGetTempIntNetIdOfPPin(lnPPinPtr->ppinPtr);
            extId = TmGetTempExtNetIdOfPPin(lnPPinPtr->ppinPtr);
            for (tmpLnPPinPtr = lnPPinPtr->nextPtr;
                tmpLnPPinPtr != NIL(LinkedPPin);
                tmpLnPPinPtr = tmpLnPPinPtr->nextPtr)
            {
               if ((TmGetTempIntNetIdOfPPin(tmpLnPPinPtr->ppinPtr) == intId) &&
                   (TmGetTempExtNetIdOfPPin(tmpLnPPinPtr->ppinPtr) == extId))
               {
                  if (tmpLnPPinPtr->prevPtr != NIL(LinkedPPin))
                  {
                     tmpLnPPinPtr->prevPtr->nextPtr = tmpLnPPinPtr->nextPtr;
                  }
                  if (tmpLnPPinPtr->nextPtr != NIL(LinkedPPin))
                  {
                     tmpLnPPinPtr->nextPtr->prevPtr = tmpLnPPinPtr->prevPtr;
                  }
               }
            }
        }
        Tp_RemovePPinListOfTile(tilePtr, (SideDirection) BOTTOM_LEFT_SIDE, 
            (int) TP_BOTTOM, &pinList);
        for (lnPPinPtr = lnPPinList; lnPPinPtr != NIL(LinkedPPin);
             lnPPinPtr = lnPPinPtr->nextPtr)
        {
            Tp_InsertPPinOfBnTile(tilePtr, (SideDirection) BOTTOM_LEFT_SIDE, 
                lnPPinPtr->ppinPtr);
        }
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
InsertPPinsInList(pPinPtr,pinListPtr)
PseudoPin          *pPinPtr;
PinsAroundJunction *pinListPtr;
{
    LinkedPPin *linkedPPinPtr;

    MALLOC(LinkedPPin *, linkedPPinPtr, sizeof (LinkedPPin));
    linkedPPinPtr->ppinPtr = pPinPtr;
    linkedPPinPtr->nextPtr = pinListPtr->vertPPinList;
    if (pinListPtr->vertPPinList != NIL(LinkedPPin))
       pinListPtr->vertPPinList->prevPtr = linkedPPinPtr;
    pinListPtr->vertPPinList = linkedPPinPtr;
    return (BEAR_CONTINUE_SEARCH);
}

#endif BETTY_BOOP


PinsAroundJunction *
Tm_PinsAroundVFace(parentCellPtr, planeDataPtr, rect, includeInsidePPins)
CellMaster *parentCellPtr;
PlaneData  *planeDataPtr;
/* rect is a horizontal rectangle */
Rect       rect;
bool       includeInsidePPins;
{
    Point              *jPoint;
    TileQueue          *bnTileList;
    TileQueue          *bnTileQue;
    WallSegment        *wallSegPtr;
    WallJunction       *junctPtr;
    WallJunction       *tmpJunctPtr;
    PinsAroundJunction pinLists;
    PinsAroundJunction *faceListsPtr;
    BNLine             *bnLinePtr;
    BottleNeck         *bottleNeckPtr;
    Tile               *tlTilePtr;
    Tile               *brTilePtr;
    Tile               *bnTilePtr;
    LocalData          localData;

    tmGlobalPlaneDataPtr = planeDataPtr;
    bnTileList = NIL(TileQueue);
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
      &rect,
      GetAndSortBN,
      (ClientData) &bnTileList);
    FixTileList(&bnTileList);
/*
 * code change : by george : on 24 oct 89
 *
    MALLOC(PinsAroundJunction *, faceListsPtr, sizeof (PinsAroundJunction));
    faceListsPtr->vertPPinList = NIL(LinkedPPin);
    faceListsPtr->horzPPinList = NIL(LinkedPPin);
    faceListsPtr->pinInstList = NIL(LinkedPinInst);
    faceListsPtr->rPinList = NIL(LinkedRPin);
 */
    faceListsPtr = Tm_MakePinsAroundJunction();
    if (bnTileList == NIL(TileQueue))
    {
        localData.rect = rect;
/*
 * code change : by george : on 25 oct 89
 *
        Fpg_ItrWallJunct(Mac_GetFpgOfCell(parentCellPtr), 
            FindJunctInRect, (ClientData)&localData);
 */
        (void) Fpg_ItrWallJunctSeq(Tp_GetFpg(planeDataPtr), FindJunctInRect,
            (ClientData) &localData);

        Tm_PinsAroundJunction(parentCellPtr, planeDataPtr, 
            localData.junctPtr, faceListsPtr);
    return (faceListsPtr);
    }
    if (Tp_IsSBNTile(bnTileList->tilePtr))
    {
    bnLinePtr = TmGetHBnLineOfSBnTiles(bnTileList->tilePtr,
      bnTileList->nextPtr->tilePtr);
    wallSegPtr = Tp_GetWallSegmentOfBnLine(bnLinePtr);
    }
    else
    {
    wallSegPtr = Tp_GetWallSegmentOfBnTile(bnTileList->tilePtr);
    }
    junctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
    /* Circle Unify/Split calls this procedure when empty cells */
    /* were already deleted - therefore could get ambiguous junction */
    jPoint = Fpg_GetPointOfWallJunct(FPG_CENTER, junctPtr);
    while (GEO_ENCLOSE(jPoint, &rect) == FALSE)
    {
       tmpJunctPtr = Fpg_GetWestJunctOfWallJunct(junctPtr);
       junctPtr = tmpJunctPtr;
       jPoint = Fpg_GetPointOfWallJunct(FPG_CENTER, junctPtr);
    }
    Tm_PinsAroundJunction(parentCellPtr, planeDataPtr, junctPtr, &pinLists);
    CopyPinListsToFaceLists(pinLists, faceListsPtr, includeInsidePPins);
    wallSegPtr = Fpg_GetSouthSegOfWallJunct(junctPtr);
    if (wallSegPtr != NIL(WallSegment))
    {
    bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
    if (bottleNeckPtr->unType == TP_BN_LINE)
    {
        tlTilePtr = Tp_GetTLTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr);
        brTilePtr = Tp_GetBRTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr);
        (void) Tp_ItrPPinsOfBnLine(tlTilePtr, brTilePtr, GetPPins, 
                           (ClientData)faceListsPtr);
    }
    else
    {
        bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
        (void) Tp_ItrPPinsOfSide(bnTilePtr, (SideDirection) TOP_RIGHT_SIDE,
            GetPPins, (ClientData)faceListsPtr);
    }
    }
    while (bnTileList != NIL(TileQueue))
    {
    if (Tp_IsSBNTile(bnTileList->tilePtr))
    {
        bnLinePtr = TmGetHBnLineOfSBnTiles(bnTileList->tilePtr,
        bnTileList->nextPtr->tilePtr);
        wallSegPtr = Tp_GetWallSegmentOfBnLine(bnLinePtr);
    }
    else
    {
        wallSegPtr = Tp_GetWallSegmentOfBnTile(bnTileList->tilePtr);
    }
    junctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
        /* Could get ambiguous junction when empty cells are deleted */
    jPoint = Fpg_GetPointOfWallJunct(FPG_CENTER, junctPtr);
    while (GEO_ENCLOSE(jPoint, &rect) == FALSE)
    {
       tmpJunctPtr = Fpg_GetWestJunctOfWallJunct(junctPtr);
       junctPtr = tmpJunctPtr;
       jPoint = Fpg_GetPointOfWallJunct(FPG_CENTER, junctPtr);
    }
    Tm_PinsAroundJunction(parentCellPtr, planeDataPtr, junctPtr, &pinLists);
    CopyPinListsToFaceLists(pinLists, faceListsPtr, includeInsidePPins);
    if (Tp_IsBottleneckTile(bnTileList->tilePtr))
    {
        (void) Tm_GetPinsInHBottleNeckRegion2(parentCellPtr, 
                          planeDataPtr,
                          bnTileList->tilePtr,
                          junctPtr,
                          &pinLists);
        CopyPinListsToFaceLists(pinLists, faceListsPtr, includeInsidePPins);
    }
    else
    /* semi-bottleneck tile */
    {
        /* no pins on bnline, so go on to next region */
        if (bnTileList->nextPtr->paired == 0)
        {
        bnTileList = bnTileList->nextPtr;
        }
    }
    bnTileQue = bnTileList->nextPtr;
    FREE((char *)bnTileList);
    bnTileList = bnTileQue;
    }
    wallSegPtr = Fpg_GetNorthSegOfWallJunct(junctPtr);
    if (wallSegPtr != NIL(WallSegment))
    {
    bottleNeckPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr);
    if (bottleNeckPtr->unType == TP_BN_LINE)
    {
        tlTilePtr = Tp_GetTLTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr);
        brTilePtr = Tp_GetBRTileOfBnLine(bottleNeckPtr->unValue.bnLinePtr);
        (void) Tp_ItrPPinsOfBnLine(tlTilePtr, brTilePtr, GetPPins, 
                        (ClientData)faceListsPtr);
    }
    else
    {
        bnTilePtr = bottleNeckPtr->unValue.bnTilePtr;
        (void) Tp_ItrPPinsOfSide(bnTilePtr, (SideDirection) BOTTOM_LEFT_SIDE,
            GetPPins, (ClientData)faceListsPtr);
    }
    }
    return (faceListsPtr);
}



typedef struct _BnLineInfo {
    Tile *bTilePtr;
    Tile *tTilePtr;
    BNLine *bnLinePtr;
} BnLineInfo;


static BNLine *
TmGetHBnLineOfSBnTiles(bTilePtr, tTilePtr)
Tile *bTilePtr;
Tile *tTilePtr;
{
    BnLineInfo bnLineInfo;

    bnLineInfo.bTilePtr = bTilePtr;
    bnLineInfo.tTilePtr = tTilePtr;
    (void) Tp_ItrHorzBnLine(tmGlobalPlaneDataPtr, FindBnLine, 
        (ClientData) &bnLineInfo);
    return (bnLineInfo.bnLinePtr);
}


static BearStatus
FindBnLine(bnLinePtr, bnLineInfoPtr)
BNLine     *bnLinePtr;
BnLineInfo *bnLineInfoPtr;
{
    if ((Tp_GetBRTileOfBnLine(bnLinePtr) == bnLineInfoPtr->bTilePtr) &&
      (Tp_GetTLTileOfBnLine(bnLinePtr) == bnLineInfoPtr->tTilePtr))
    {
        bnLineInfoPtr->bnLinePtr = bnLinePtr;
        return (BEAR_STOP_SEARCH);
    }
    return (BEAR_CONTINUE_SEARCH);
}


static void
CopyPinListsToFaceLists(pinLists, faceListsPtr, includeInsidePPins)
PinsAroundJunction pinLists;
PinsAroundJunction *faceListsPtr;
bool includeInsidePPins;
{
    LinkedPPin    *pPinPtr;
    LinkedRPin    *rPinPtr;
    LinkedPinInst *pinPtr;

    if (faceListsPtr->vertPPinList == NIL(LinkedPPin))
    {
       faceListsPtr->vertPPinList = pinLists.vertPPinList;
       pinLists.vertPPinList = NIL(LinkedPPin);
    }
    if (includeInsidePPins)
    {
       if (faceListsPtr->horzPPinList == NIL(LinkedPPin))
       {
          faceListsPtr->horzPPinList = pinLists.horzPPinList;
          pinLists.horzPPinList = NIL(LinkedPPin);
       }
    }
    if (faceListsPtr->pinInstList == NIL(LinkedPinInst))
    {
       faceListsPtr->pinInstList = pinLists.pinInstList;
       pinLists.pinInstList = NIL(LinkedPinInst);
    }
    if (faceListsPtr->rPinList == NIL(LinkedRPin))
    {
       faceListsPtr->rPinList = pinLists.rPinList;
       pinLists.rPinList = NIL(LinkedRPin);
    }


    if (pinLists.vertPPinList != NIL(LinkedPPin))
    {
       for (pPinPtr = pinLists.vertPPinList;
            pPinPtr->nextPtr != NIL(LinkedPPin); 
            pPinPtr = pPinPtr->nextPtr);
       pPinPtr->nextPtr = faceListsPtr->vertPPinList;
       faceListsPtr->vertPPinList = pinLists.vertPPinList;
    }
    if (includeInsidePPins)
    {
       if (pinLists.horzPPinList != NIL(LinkedPPin))
       {
          for (pPinPtr = pinLists.horzPPinList;
               pPinPtr->nextPtr != NIL(LinkedPPin); 
               pPinPtr = pPinPtr->nextPtr);
          pPinPtr->nextPtr = faceListsPtr->horzPPinList;
          faceListsPtr->horzPPinList = pinLists.horzPPinList;
       }
    }
    if (pinLists.pinInstList != NIL(LinkedPinInst))
    {
       for (pinPtr = pinLists.pinInstList;
            pinPtr->nextPtr != NIL(LinkedPinInst); 
            pinPtr = pinPtr->nextPtr);
       pinPtr->nextPtr = faceListsPtr->pinInstList;
       faceListsPtr->pinInstList = pinLists.pinInstList;
    }
    if (pinLists.rPinList != NIL(LinkedRPin))
    {
       for (rPinPtr = pinLists.rPinList;
            rPinPtr->nextPtr != NIL(LinkedRPin); 
            rPinPtr = rPinPtr->nextPtr);
       rPinPtr->nextPtr = faceListsPtr->rPinList;
       faceListsPtr->rPinList = pinLists.rPinList;
    }
}

void
Tm_GetPinsInHBottleNeckRegion2(parentCellPtr,
                   planeDataPtr,
                   bnTilePtr, 
                   junctPtr,
                   pinListsPtr)
CellMaster *parentCellPtr;
PlaneData  *planeDataPtr;
Tile       *bnTilePtr;
WallJunction *junctPtr;
PinsAroundJunction *pinListsPtr;
{
/* Get pins on the cell instances that bound a horz bottleneck tile */

    Rect      bnRect;
    Rect      vRect;
    Point     point;
    Tile      *tilePtr;
    DummyCell *dCellPtr;
    LocalInfo *localInfoPtr;
        int       bnTileStatus;

        MALLOC(LocalInfo *, localInfoPtr, sizeof (LocalInfo));
        localInfoPtr->pinListsPtr = pinListsPtr;
        pinListsPtr->vertPPinList = NIL(LinkedPPin);
        pinListsPtr->horzPPinList = NIL(LinkedPPin);
        pinListsPtr->pinInstList = NIL(LinkedPinInst);
        pinListsPtr->rPinList = NIL(LinkedRPin);

    (void) Tp_HorzTileToRect(planeDataPtr, bnTilePtr, &bnRect);
    localInfoPtr->area = &bnRect;
    point.p_x = bnRect.r_ll.p_x - 1;
    point.p_y = bnRect.r_ll.p_y;
    tilePtr = TiSrPointNoHint(Tp_GetHorzPlane(planeDataPtr), &point);
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, (SideDirection) RIGHT_SIDE,
            GetPPinsOfDCell, (ClientData) pinListsPtr);
    }
    point.p_x = bnRect.r_ur.p_x;
    point.p_y = bnRect.r_ll.p_y;
    tilePtr = TiSrPointNoHint(Tp_GetHorzPlane(planeDataPtr), &point);
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, (SideDirection) LEFT_SIDE,
            GetPPinsOfDCell, (ClientData) pinListsPtr);
    }
        bnTileStatus = GetStatusOfHBnTile(bnTilePtr, junctPtr);

        /* Look at pins on right-edge of bnTile */
        if ((bnTileStatus != bnTileRightDead) &&
        (bnTileStatus != bnTileBothDead))
    {
           bnRect.r_ur.p_x += 1 ;
    }
    else
    {
           bnRect.r_ur.p_x -= 1; 
    }
       
        /* Look at pins on left-edge of bnTile */
        if ((bnTileStatus != bnTileLeftDead) &&
        (bnTileStatus != bnTileBothDead))
    {
       bnRect.r_ll.p_x -= 1;
    }
    else
    {
       bnRect.r_ll.p_x += 1;
    }

    Tp_HRectToVRect(&bnRect, &vRect);
    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr),
      &vRect,
      GetPinsOfSolidTile,
      (ClientData) localInfoPtr);
    /* for i/o pins */
    (void) Mac_ItrPinOfCellInst(Mac_GetInstOfCoreMaster(parentCellPtr),
                       GetPinsInRect, (ClientData) localInfoPtr);
}

void
Tm_GetPinsInHBottleNeckRegion(parentCellPtr, planeDataPtr, 
                             bnTilePtr, pinListsPtr)
CellMaster *parentCellPtr;
PlaneData  *planeDataPtr;
Tile       *bnTilePtr;
PinsAroundJunction *pinListsPtr;
{
    /* Get pins on the cell instances that bound a horz bottleneck tile */

    Rect      bnRect;
    Rect      vRect;
    Point     point;
    Tile      *tilePtr;
    DummyCell *dCellPtr;
    LocalInfo *localInfoPtr;

        MALLOC(LocalInfo *, localInfoPtr, sizeof (LocalInfo));
        localInfoPtr->pinListsPtr = pinListsPtr;
        pinListsPtr->vertPPinList = NIL(LinkedPPin);
        pinListsPtr->horzPPinList = NIL(LinkedPPin);
        pinListsPtr->pinInstList = NIL(LinkedPinInst);
        pinListsPtr->rPinList = NIL(LinkedRPin);

    (void) Tp_HorzTileToRect(planeDataPtr, bnTilePtr, &bnRect);
    localInfoPtr->area = &bnRect;
    point.p_x = bnRect.r_ll.p_x - 1;
    point.p_y = bnRect.r_ll.p_y;
    tilePtr = TiSrPointNoHint(Tp_GetHorzPlane(planeDataPtr), &point);
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, (SideDirection) RIGHT_SIDE,
            GetPPinsOfDCell, (ClientData) pinListsPtr);
    }
    point.p_x = bnRect.r_ur.p_x;
    point.p_y = bnRect.r_ll.p_y;
    tilePtr = TiSrPointNoHint(Tp_GetHorzPlane(planeDataPtr), &point);
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, (SideDirection) LEFT_SIDE,
            GetPPinsOfDCell, (ClientData) pinListsPtr);
    }
    bnRect.r_ur.p_x += 1;
    bnRect.r_ll.p_x -= 1;
    Tp_HRectToVRect(&bnRect, &vRect);
    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr),
      &vRect,
      GetPinsOfSolidTile,
      (ClientData) localInfoPtr);
    /* for i/o pins */
    (void) Mac_ItrPinOfCellInst(Mac_GetInstOfCoreMaster(parentCellPtr),
                       GetPinsInRect, (ClientData) localInfoPtr);
}


static BearStatus
GetPPinsOfDCell(pPinPtr, faceListsPtr)
PseudoPin          *pPinPtr;
PinsAroundJunction *faceListsPtr;
{
    LinkedPPin *linkPPinPtr;

    MALLOC(LinkedPPin *, linkPPinPtr, sizeof (LinkedPPin));
    linkPPinPtr->ppinPtr = pPinPtr;
    linkPPinPtr->nextPtr = faceListsPtr->vertPPinList;
    faceListsPtr->vertPPinList = linkPPinPtr;
    return (BEAR_CONTINUE_SEARCH);
}


static int
GetPinsOfSolidTile(tilePtr, localInfoPtr)
Tile      *tilePtr;
LocalInfo *localInfoPtr;
{
    CellInst    *cellInstPtr;
    RouteCell   *routeCellPtr;

    if (Tp_IsCellInstTile(tilePtr)) 
    {
        cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
        (void) Mac_ItrPinOfCellInst (cellInstPtr, GetPinsInRect,
          (ClientData) localInfoPtr);
    }
#ifdef USE_DR_CODE
    if (Tp_IsRCellTile(tilePtr)) 
    {
        routeCellPtr = Tp_GetRCellOfTile(tilePtr);
        (void) Mac_ItrSidePinOfRCell(routeCellPtr, GetRPinsInRect,
          (ClientData) localInfoPtr);
    }
#endif /* USE_DR_CODE */

    return (0);
}


static BearStatus
GetRPinsInRect(rPinPtr, localInfoPtr)
RoutePin  *rPinPtr;
LocalInfo *localInfoPtr;
{
    LinkedRPin *linkRPinPtr;
    Point      absPos;

    (void) Mac_GetAbsPosOfRPin(rPinPtr, &absPos);
    if ((absPos.p_x >= localInfoPtr->area->r_xbot) &&
      (absPos.p_x <= localInfoPtr->area->r_xtop) &&
      (absPos.p_y >= localInfoPtr->area->r_ybot) &&
      (absPos.p_y <= localInfoPtr->area->r_ytop))
    {
        MALLOC(LinkedRPin *, linkRPinPtr, sizeof (LinkedRPin));
        linkRPinPtr->rPinPtr = rPinPtr;
        linkRPinPtr->nextPtr = localInfoPtr->pinListsPtr->rPinList;
        localInfoPtr->pinListsPtr->rPinList = linkRPinPtr;
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
GetPinsInRect(pinInstPtr, localInfoPtr)
PinInst   *pinInstPtr;
LocalInfo *localInfoPtr;
{
    LinkedPinInst *linkPinPtr;
    Point         absPos;

    (void) Mac_GetAbsPosOfPin(pinInstPtr, &absPos);
    if ((absPos.p_x >= localInfoPtr->area->r_xbot) &&
      (absPos.p_x <= localInfoPtr->area->r_xtop) &&
      (absPos.p_y >= localInfoPtr->area->r_ybot) &&
      (absPos.p_y <= localInfoPtr->area->r_ytop))
    {
        MALLOC(LinkedPinInst *, linkPinPtr, sizeof (LinkedPinInst));
        linkPinPtr->pinInstPtr = pinInstPtr;
        linkPinPtr->nextPtr = localInfoPtr->pinListsPtr->pinInstList;
        localInfoPtr->pinListsPtr->pinInstList = linkPinPtr;
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
GetPPins(pPinPtr,pinListPtr)
PseudoPin          *pPinPtr;
PinsAroundJunction *pinListPtr;
{
    LinkedPPin *linkedPPinPtr;

    MALLOC(LinkedPPin *, linkedPPinPtr, sizeof (LinkedPPin));
    linkedPPinPtr->ppinPtr = pPinPtr;
    linkedPPinPtr->nextPtr = pinListPtr->horzPPinList;
    pinListPtr->horzPPinList = linkedPPinPtr;
    return (BEAR_CONTINUE_SEARCH);
}



static void
FixBadNetUnification(oldId, newId, curTilePtr)
BearId oldId;
BearId newId;
Tile   *curTilePtr;
{
    PinsAroundJunction *faceListsPtr;
    BearId             extId;
    BearId             intId;
    LinkedPPin         *pPinPtr;
    LinkedRPin         *rPinPtr;
    LinkedPinInst      *pinPtr;
    TileQueue          *bnTileList;
    TileQueue          *bnTileQue;
    Tile               *pinTilePtr;

    faceListsPtr =
      Tm_PinsAroundVFace(tmGlobalParentCellPtr, tmGlobalPlaneDataPtr, 
        tmGlobalDmntRect, FALSE);

    for (pPinPtr = faceListsPtr->vertPPinList; pPinPtr != NIL(LinkedPPin);
      pPinPtr = faceListsPtr->vertPPinList)
    {
    extId = TmGetTempExtNetIdOfPPin(pPinPtr->ppinPtr);
    intId = TmGetTempIntNetIdOfPPin(pPinPtr->ppinPtr);
    if (extId == oldId)
    {
        (void) TmPutTempExtNetIdOfPPin(newId, pPinPtr->ppinPtr);
    }
    if ((intId == oldId) &&
      Tp_IsSBNTile(Tp_GetTileOfPPin(pPinPtr->ppinPtr)))
    {
        (void) TmPutTempIntNetIdOfPPin(newId, pPinPtr->ppinPtr);
    }
    faceListsPtr->vertPPinList = pPinPtr->nextPtr;
    FREE((char *)pPinPtr);
    }
    for (pinPtr = faceListsPtr->pinInstList;
      pinPtr != NIL(LinkedPinInst); pinPtr = faceListsPtr->pinInstList)
    {
    extId = Mac_GetVExtNetIdOfPin(pinPtr->pinInstPtr);
    if (extId == oldId)
    {
        (void) Mac_PutVExtNetIdOfPin(newId, pinPtr->pinInstPtr);
        if (globalCopyFlag)
        {
        (void) Mac_PutExtNetIdOfPin(newId, pinPtr->pinInstPtr);
        }
    }
    faceListsPtr->pinInstList = pinPtr->nextPtr;
    FREE((char *)pinPtr);
    }
    for (rPinPtr = faceListsPtr->rPinList;
      rPinPtr != NIL(LinkedRPin); rPinPtr = faceListsPtr->rPinList)
    {
    extId = Mac_GetVExtNetIdOfRPin(rPinPtr->rPinPtr);
    if (extId == oldId)
    {
        (void) Mac_PutVExtNetIdOfRPin(newId, rPinPtr->rPinPtr);
        if (globalCopyFlag)
        {
        (void) Mac_PutExtNetIdOfRPin(newId, rPinPtr->rPinPtr);
        }
    }
    faceListsPtr->rPinList = rPinPtr->nextPtr;
    FREE((char *)rPinPtr);
    }

    bnTileList = NIL(TileQueue);
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
      &tmGlobalDmntRect,
      GetAndSortBN,
      (ClientData) &bnTileList);
    FixTileList(&bnTileList);
    while (bnTileList != NIL(TileQueue))
    {
    if (Tp_IsSBNTile(bnTileList->tilePtr))
    {
        (void) Tp_ItrPPinsOfBnLine(bnTileList->tilePtr,
                bnTileList->nextPtr->tilePtr , GetPPins, 
                (ClientData)faceListsPtr);
        if (bnTileList->nextPtr->paired == 0)
        {
        bnTileList = bnTileList->nextPtr;
        }
    }
    else
    {
        (void) Tp_ItrPPinsOfBnTile(bnTileList->tilePtr, GetPPins,
                        (ClientData)faceListsPtr);
    }
    bnTileQue = bnTileList->nextPtr;
    FREE((char *)bnTileList);
    bnTileList = bnTileQue;
    }

    for (pPinPtr = faceListsPtr->horzPPinList; pPinPtr != NIL(LinkedPPin);
      pPinPtr = faceListsPtr->horzPPinList)
    {
    extId = TmGetTempExtNetIdOfPPin(pPinPtr->ppinPtr);
    intId = TmGetTempIntNetIdOfPPin(pPinPtr->ppinPtr);
    if (extId == oldId)
    {
        (void) TmPutTempExtNetIdOfPPin(newId, pPinPtr->ppinPtr);
        /* Sherry add - to insure the int Ids also are updated */
        if (intId == oldId)
        {
           (void) TmPutTempIntNetIdOfPPin(newId, pPinPtr->ppinPtr);
        }
    }
    if (intId == oldId) 
    {
      pinTilePtr = Tp_GetTileOfPPin(pPinPtr->ppinPtr);
      if ((pinTilePtr == curTilePtr) || Tp_IsSBNTile(pinTilePtr))
      {
        (void) TmPutTempIntNetIdOfPPin(newId, pPinPtr->ppinPtr);
      } 
    }
    faceListsPtr->horzPPinList = pPinPtr->nextPtr;
    FREE((char *)pPinPtr);
    }
    FREE((char *)faceListsPtr);
}

 
void
TmVRectUnify(planeDataPtr, rect, copyFlag)
PlaneData *planeDataPtr;
Rect      rect;
int       copyFlag;
{
    /*
     * By search the dominant tile for horizontal bottlenecks tiles and lines
     * from bottom to top, we propogate across the face the new horizontal 
     * values for the pseudo pin id's
     */

    Rect               bnRect;
    TileQueue          *bnTileList;
    WallSegment        *wallSegPtr;
    WallJunction       *junctPtr;
    PinsAroundJunction pinLists;
    LocalInfo          localInfo;
    BNLine             *bnLinePtr;

    tmGlobalDmntRect.r_xbot = rect.r_xbot;
    tmGlobalDmntRect.r_ybot = rect.r_ybot;
    tmGlobalDmntRect.r_xtop = rect.r_xtop;
    tmGlobalDmntRect.r_ytop = rect.r_ytop;
    tmGlobalPlaneDataPtr = planeDataPtr;
    globalCopyFlag = copyFlag;
    bnTileList = NIL(TileQueue);
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
      &tmGlobalDmntRect,
      GetAndSortBN,
      (ClientData) &bnTileList);
    (void) FixTileList(&bnTileList);
    if (bnTileList != NIL(TileQueue))
    {
        if (Tp_IsSBNTile(bnTileList->tilePtr))
        {
            bnLinePtr = TmGetHBnLineOfSBnTiles(bnTileList->tilePtr,
              bnTileList->nextPtr->tilePtr);
            wallSegPtr = Tp_GetWallSegmentOfBnLine(bnLinePtr);
        }
        else
        {
            wallSegPtr = Tp_GetWallSegmentOfBnTile(bnTileList->tilePtr);
        }
        junctPtr = Fpg_GetSEJunctOfWallSeg(wallSegPtr);
        (void)Tm_PinsAroundJunction(tmGlobalParentCellPtr, planeDataPtr, 
          junctPtr, &pinLists);
        (void)PropagateExtToVExt(&pinLists);
    }
    while (bnTileList != NIL(TileQueue))
    {
        if (Tp_IsSBNTile(bnTileList->tilePtr))
        {
            bnLinePtr = TmGetHBnLineOfSBnTiles(bnTileList->tilePtr,
              bnTileList->nextPtr->tilePtr);
            wallSegPtr = Tp_GetWallSegmentOfBnLine(bnLinePtr);
        }
        else
        {
            wallSegPtr = Tp_GetWallSegmentOfBnTile(bnTileList->tilePtr);
        }
        junctPtr = Fpg_GetNWJunctOfWallSeg(wallSegPtr);
        (void)Tm_PinsAroundJunction(tmGlobalParentCellPtr, planeDataPtr, 
          junctPtr, &pinLists);
        (void)PropagateExtToVExt(&pinLists);
        if (Tp_IsBottleneckTile(bnTileList->tilePtr))
        {
            (void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, 
                                     bnTileList->tilePtr,
                                     &bnRect);
            localInfo.tilePtr = bnTileList->tilePtr;
            localInfo.area = &bnRect;
            /* Sherry added: I/O pads bounding the bnTile need vExtId set */
            (void) Mac_ItrPinOfCellInst(Mac_GetInstOfCoreMaster(
                                                    tmGlobalParentCellPtr), 
                                        PropagateExtIdOfIOPads, 
                                        (ClientData) &localInfo);
            (void) Tp_ItrPPinsOfSide(bnTileList->tilePtr,
              (SideDirection) BOTTOM_LEFT_SIDE,
              CheckAgainstBottomPin, (ClientData) &localInfo);
            (void) Tp_ItrPPinsOfSide(bnTileList->tilePtr,
              (SideDirection) TOP_RIGHT_SIDE,
              CheckAgainstJunctPins, (ClientData) &pinLists);
        }
        else
        /* semi-bottleneck tile */
        {
            (void) Tp_ItrPPinsOfBnLine(bnTileList->tilePtr,
              bnTileList->nextPtr->tilePtr, CheckAgainstJunctPins, 
              (ClientData) &pinLists);
            if (bnTileList->nextPtr->paired == 0)
            {
                bnTileList = bnTileList->nextPtr;
            }
        }
        bnTileList = bnTileList->nextPtr;
    }
}


static void
FixTileList(bnTileListPtr)
TileQueue **bnTileListPtr;
{
    TileQueue  *bnTileList;
    TileQueue  *placePtr;
    BNLineType bnLineType;
    int        removeElt;
    Tile       *tmpTilePtr;

    bnTileList = *bnTileListPtr;
    while (bnTileList != NIL(TileQueue))
    {
    if (Tp_IsSBNTile(bnTileList->tilePtr))
    {
        if (bnTileList->nextPtr == NIL(TileQueue))
        {
            removeElt = 1;
        }
        else
        {
        if (Tp_IsSBNTile(bnTileList->nextPtr->tilePtr))
        {
            if (Tp_IsPairOfSBNTiles(&(bnTileList->tilePtr), 
                &(bnTileList->nextPtr->tilePtr), &bnLineType))
            {
            removeElt = 0;
            bnTileList->paired = 1;
            /*
             * the tiles have been switched, so me must unswitch
             * them.
             */
            tmpTilePtr = bnTileList->tilePtr; 
            bnTileList->tilePtr = bnTileList->nextPtr->tilePtr; 
            bnTileList->nextPtr->tilePtr = tmpTilePtr;
            /*
             * there may be a string of sbntiles that are 
             * consecutively pairs.
             */
            if (bnTileList->nextPtr->nextPtr == NIL(TileQueue))
            {
                bnTileList = bnTileList->nextPtr;
            }
            else
            {
                if (Tp_IsPairOfSBNTiles(
                  &(bnTileList->nextPtr->tilePtr), 
                  &(bnTileList->nextPtr->nextPtr->tilePtr), 
                  &bnLineType))
                {
                /*
                 * the tiles have been switched, so me must 
                 * unswitch them.
                 */
                tmpTilePtr = bnTileList->nextPtr->tilePtr; 
                bnTileList->nextPtr->tilePtr = 
                  bnTileList->nextPtr->nextPtr->tilePtr; 
                bnTileList->nextPtr->nextPtr->tilePtr = 
                  tmpTilePtr;
                }
                else
                {
                bnTileList = bnTileList->nextPtr;
                }
            }
            }
            else
            {
            removeElt = 1;
            bnTileList->paired = 0;
            }
        }
        else
        {
            removeElt = 1;
        }
        }
    }
    else
    {
        removeElt = 0;
    }
    if (removeElt == 1)
    {
        if (*bnTileListPtr == bnTileList)
        {
        *bnTileListPtr = (*bnTileListPtr)->nextPtr;
        }
        else
        {
        for (placePtr = *bnTileListPtr;placePtr->nextPtr != bnTileList;
          placePtr = placePtr->nextPtr);
        placePtr->nextPtr = placePtr->nextPtr->nextPtr;
        }
    }
    bnTileList = bnTileList->nextPtr;
    }
}


static void
PropagateExtToVExt(pinListsPtr)
PinsAroundJunction *pinListsPtr;
{
    LinkedPinInst *pinPosPtr;
    LinkedRPin    *rPinPtr;

    pinPosPtr = pinListsPtr->pinInstList;
    while (pinPosPtr != NIL(LinkedPinInst))
    {
        (void) Mac_PutVExtNetIdOfPin(
          Mac_GetExtNetIdOfPin(pinPosPtr->pinInstPtr), 
          pinPosPtr->pinInstPtr);
        pinPosPtr = pinPosPtr->nextPtr;
    }
    rPinPtr = pinListsPtr->rPinList;
    while (rPinPtr != NIL(LinkedRPin))
    {
        (void) Mac_PutVExtNetIdOfRPin(
          Mac_GetExtNetIdOfRPin(rPinPtr->rPinPtr),
          rPinPtr->rPinPtr);
        rPinPtr = rPinPtr->nextPtr;
    }
}

static BearStatus
PropagateExtIdOfIOPads(pinPtr,localInfoPtr)
PinInst   *pinPtr;
LocalInfo *localInfoPtr;
{
    /*
     * Make sure that pin is on the boundary area, and then propagate the
     * external id to the vExt id
     */

    Point  absPos;

    (void) Mac_GetAbsPosOfPin(pinPtr, &absPos);
    if ((absPos.p_x >= localInfoPtr->area->r_ll.p_x) &&
        (absPos.p_x <= localInfoPtr->area->r_ur.p_x) &&
        (absPos.p_y >= localInfoPtr->area->r_ll.p_y) &&
        (absPos.p_y <= localInfoPtr->area->r_ur.p_y))
    {
        (void) Mac_PutVExtNetIdOfPin(Mac_GetExtNetIdOfPin(pinPtr), pinPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
CheckAgainstJunctPins(pPinPtr, pinListsPtr)
PseudoPin          *pPinPtr;
PinsAroundJunction *pinListsPtr;
{
    /* Propogates id values across a junction region */

    LinkedPinInst *pinPosPtr;
    LinkedPPin    *pPinPosPtr;
    LinkedRPin    *rPinPosPtr;
    BearId        tIntId;
    BearId        tExtId;
    BearId        intId;
    BearId        extId;
    Tile          *curTilePtr;

    tIntId = TmGetTempIntNetIdOfPPin(pPinPtr);
    tExtId = TmGetTempExtNetIdOfPPin(pPinPtr);
    curTilePtr = Tp_GetTileOfPPin(pPinPtr);

    pinPosPtr = pinListsPtr->pinInstList;
    while (pinPosPtr != NIL(LinkedPinInst))
    {
    extId = Mac_GetExtNetIdOfPin(pinPosPtr->pinInstPtr);
    if (extId == tExtId)
    {
        (void) Mac_PutVExtNetIdOfPin(tIntId, pinPosPtr->pinInstPtr);
        if (globalCopyFlag)
        {
        (void) Mac_PutExtNetIdOfPin(tIntId, pinPosPtr->pinInstPtr);
        }
    }
    pinPosPtr = pinPosPtr->nextPtr;
    }
    rPinPosPtr = pinListsPtr->rPinList;
    while (rPinPosPtr != NIL(LinkedRPin))
    {
    extId = Mac_GetExtNetIdOfRPin(rPinPosPtr->rPinPtr);
    if (extId == tExtId)
    {
        (void) Mac_PutVExtNetIdOfRPin(tIntId, rPinPosPtr->rPinPtr);
        if (globalCopyFlag)
        {
        (void) Mac_PutExtNetIdOfRPin(tIntId, rPinPosPtr->rPinPtr);
        }
    }
    rPinPosPtr = rPinPosPtr->nextPtr;
    }
    pPinPosPtr = pinListsPtr->vertPPinList;
    while (pPinPosPtr != NIL(LinkedPPin))
    {
    intId = TmGetTempIntNetIdOfPPin(pPinPosPtr->ppinPtr);
    extId = TmGetTempExtNetIdOfPPin(pPinPosPtr->ppinPtr);
    if (extId == tExtId)
    {
        (void) TmPutTempExtNetIdOfPPin(tIntId, pPinPosPtr->ppinPtr);
    }
    /* This is needed for bottleneck lines */
    if ((intId == tExtId) &&
      Tp_IsSBNTile(Tp_GetTileOfPPin(pPinPosPtr->ppinPtr)))
    {
        (void) TmPutTempIntNetIdOfPPin(tIntId, pPinPosPtr->ppinPtr);
    }
    pPinPosPtr = pPinPosPtr->nextPtr;
    }
    pPinPosPtr = pinListsPtr->horzPPinList;
    while (pPinPosPtr != NIL(LinkedPPin))
    {
    if (pPinPosPtr->ppinPtr != pPinPtr)
    {
        intId = TmGetTempIntNetIdOfPPin(pPinPosPtr->ppinPtr);
        extId = TmGetTempExtNetIdOfPPin(pPinPosPtr->ppinPtr);
        if (extId == tExtId)
        {
        if ((Tp_GetTileOfPPin(pPinPosPtr->ppinPtr) == curTilePtr)
            && (intId != tIntId))
        {
            (void) FixBadNetUnification(intId, tIntId, curTilePtr);
        }
        else
        {
            (void) TmPutTempExtNetIdOfPPin(tIntId, pPinPosPtr->ppinPtr);
        }
        }
        /* This is needed for bottleneck lines */
        if (intId == tExtId)
        if ((intId == tExtId) &&
          Tp_IsSBNTile(Tp_GetTileOfPPin(pPinPosPtr->ppinPtr)))
        {
        (void) TmPutTempIntNetIdOfPPin(tIntId, pPinPosPtr->ppinPtr);
        }
    }
    pPinPosPtr = pPinPosPtr->nextPtr;
    }
    (void) TmPutTempExtNetIdOfPPin(tIntId, pPinPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
CheckAgainstBottomPin(pPinPtr, localInfoPtr)
PseudoPin *pPinPtr;
LocalInfo *localInfoPtr;
{
    /* Propogates id values across a horizontal bottleneck tile */

    (void) Tp_ItrPPinsOfSide(localInfoPtr->tilePtr,
      (SideDirection) BOTTOM_LEFT_SIDE,
      CheckTopBottomPin, (ClientData) pPinPtr);
    (void) Tp_ItrPPinsOfSide(localInfoPtr->tilePtr,
      (SideDirection) TOP_RIGHT_SIDE,
      CheckTopBottomPin, (ClientData) pPinPtr);
    (void) ItrPinsOnLeftAndRight(localInfoPtr, pPinPtr);
    (void) TmPutTempIntNetIdOfPPin(TmGetTempExtNetIdOfPPin(pPinPtr), pPinPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static void
ItrPinsOnLeftAndRight(localInfoPtr, pPinPtr)
LocalInfo *localInfoPtr;
PseudoPin *pPinPtr;
{
    /* Get pins on the cell instances that bound a vertical bottleneck tile */

    Rect      bnRect;
    Rect      vRect;
    Point     point;
    Tile      *tilePtr;
    DummyCell *dCellPtr;

    (void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, localInfoPtr->tilePtr,
      &bnRect);
    localInfoPtr->area = &bnRect;
    localInfoPtr->pseudoPin = pPinPtr;
    point.p_x = bnRect.r_ll.p_x - 1;
    point.p_y = bnRect.r_ll.p_y;
    tilePtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
      &point);
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, (SideDirection) RIGHT_SIDE,
          CheckDCellPPins, (ClientData) localInfoPtr);
    }
    point.p_x = bnRect.r_ur.p_x;
    point.p_y = bnRect.r_ll.p_y;
    tilePtr = TiSrPointNoHint(Tp_GetHorzPlane(tmGlobalPlaneDataPtr),
      &point);
    if (Tp_IsDCellTile(tilePtr))
    {
        dCellPtr = Tp_GetDCellOfTile(tilePtr);
        (void) Mac_ItrPPinOfDCell(dCellPtr, (SideDirection) LEFT_SIDE,
          CheckDCellPPins, (ClientData) localInfoPtr);
    }
    bnRect.r_ur.p_x += 1;
    bnRect.r_ll.p_x -= 1;
    Tp_HRectToVRect(&bnRect, &vRect);
    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(tmGlobalPlaneDataPtr),
      &vRect,
      ItrSolidTile,
      (ClientData) localInfoPtr);
    /* for i/o pins */
    (void) Mac_ItrPinOfCellInst(Mac_GetInstOfCoreMaster(
      tmGlobalParentCellPtr), CheckLeftRightPin, (ClientData) localInfoPtr);
}


static int
ItrSolidTile(tilePtr, localInfoPtr)
Tile      *tilePtr;
LocalInfo *localInfoPtr;
{
    CellInst    *cellInstPtr;
    RouteCell   *routeCellPtr;

    if (Tp_IsCellInstTile(tilePtr))
    {
        cellInstPtr = Tp_GetCellInstOfTile(tilePtr);
        (void) Mac_ItrPinOfCellInst (cellInstPtr, CheckLeftRightPin,
          (ClientData) localInfoPtr);
    }
#ifdef USE_DR_CODE
    if (Tp_IsRCellTile(tilePtr))
    {
        routeCellPtr = Tp_GetRCellOfTile(tilePtr);
        (void) Mac_ItrSidePinOfRCell(routeCellPtr, CheckLeftRightRPin,
          (ClientData) localInfoPtr);
    }
#endif /* USE_DR_CODE */

    return (0);
}


static BearStatus
CheckLeftRightRPin(rPinPtr,localInfoPtr)
RoutePin  *rPinPtr;
LocalInfo *localInfoPtr;
{
    /*
     * Make sure that pin is on the boundary area, and then change the
     * external id value so that it is consistent
     */

    Point  absPos;
    BearId tIntId;
    BearId tExtId;
    BearId extId;

    (void) Mac_GetAbsPosOfRPin(rPinPtr, &absPos);
    if (Mac_GetVExtNetIdOfRPin(rPinPtr) == 0)
    {
        (void) Mac_PutVExtNetIdOfRPin(Mac_GetExtNetIdOfRPin(rPinPtr), rPinPtr);
    }
    if ((absPos.p_x >= localInfoPtr->area->r_ll.p_x) &&
      (absPos.p_x <= localInfoPtr->area->r_ur.p_x) &&
      (absPos.p_y >= localInfoPtr->area->r_ll.p_y) &&
      (absPos.p_y <= localInfoPtr->area->r_ur.p_y))
    {
        tIntId = TmGetTempIntNetIdOfPPin(localInfoPtr->pseudoPin);
        extId = Mac_GetExtNetIdOfRPin(rPinPtr);
        tExtId = TmGetTempExtNetIdOfPPin(localInfoPtr->pseudoPin);
        if (extId == tIntId)
        {
            (void) Mac_PutVExtNetIdOfRPin(tExtId, rPinPtr);
            if (globalCopyFlag)
            {
                (void) Mac_PutExtNetIdOfRPin(tExtId, rPinPtr);
            }
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}

static BearStatus
CheckLeftRightPin(pinPtr,localInfoPtr)
PinInst   *pinPtr;
LocalInfo *localInfoPtr;
{
    /*
     * Make sure that pin is on the boundary area, and then change the
     * external id value so that it is consistent
     */

    Point  absPos;
    BearId tIntId;
    BearId tExtId;
    BearId extId;

    (void) Mac_GetAbsPosOfPin(pinPtr, &absPos);
    if (Mac_GetVExtNetIdOfPin(pinPtr) == 0)
    {
        (void) Mac_PutVExtNetIdOfPin(Mac_GetExtNetIdOfPin(pinPtr), pinPtr);
    }
    if ((absPos.p_x >= localInfoPtr->area->r_ll.p_x) &&
      (absPos.p_x <= localInfoPtr->area->r_ur.p_x) &&
      (absPos.p_y >= localInfoPtr->area->r_ll.p_y) &&
      (absPos.p_y <= localInfoPtr->area->r_ur.p_y))
    {
        tIntId = TmGetTempIntNetIdOfPPin(localInfoPtr->pseudoPin);
        extId = Mac_GetExtNetIdOfPin(pinPtr);
        tExtId = TmGetTempExtNetIdOfPPin(localInfoPtr->pseudoPin);
        if (extId == tIntId)
        {
            (void) Mac_PutVExtNetIdOfPin(tExtId, pinPtr);
            if (globalCopyFlag)
            {
                (void) Mac_PutExtNetIdOfPin(tExtId, pinPtr);
            }
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
CheckDCellPPins(pPinPtr,localInfoPtr)
PseudoPin *pPinPtr;
LocalInfo *localInfoPtr;
{
    BearId lIntId;
    BearId extId;

    extId = TmGetTempExtNetIdOfPPin(pPinPtr);
    lIntId = TmGetTempIntNetIdOfPPin(localInfoPtr->pseudoPin);
    if (extId == lIntId)
    {
        (void) TmPutTempExtNetIdOfPPin(
          TmGetTempExtNetIdOfPPin(localInfoPtr->pseudoPin), pPinPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
CheckTopBottomPin(pPinPtr,bottomPPinPtr)
PseudoPin *pPinPtr;
PseudoPin *bottomPPinPtr;
{
    BearId bIntId;
    BearId intId;
    BearId extId;
    BearId bExtId;

    bIntId = TmGetTempIntNetIdOfPPin(bottomPPinPtr);
    bExtId = TmGetTempExtNetIdOfPPin(bottomPPinPtr);
    intId = TmGetTempIntNetIdOfPPin(pPinPtr);
    extId = TmGetTempExtNetIdOfPPin(pPinPtr);
    if (pPinPtr != bottomPPinPtr)
    {
    if (extId != bExtId)
    {
        if (intId == bIntId)
        {
        if (Tp_GetSideOfPPin(pPinPtr) == TOP_RIGHT_SIDE)
        {
            (void) TmPutTempIntNetIdOfPPin(bExtId, pPinPtr);
        }
        /* pPinPtr on the bottom side */
        else
        {
            (void) TmPutTempExtNetIdOfPPin(bExtId, pPinPtr);
            (void) TmPutTempIntNetIdOfPPin(bExtId, pPinPtr);
            /* Backward unification */
            (void) FixBadNetUnification(extId, bExtId,
                                        Tp_GetTileOfPPin(bottomPPinPtr));
/*
printf("Backward unification for (%d, %d) \n", intId, extId);
fflush(stdout);
*/
        }
        }
    }
    }
    return (BEAR_CONTINUE_SEARCH);
}


static int
GetAndSortBN(tilePtr, bnTileList)
Tile       *tilePtr;
TileQueue  **bnTileList;
{
    if (Tp_IsBottleneckTile(tilePtr))
    {
        (void) InsertBnInList(tilePtr, bnTileList);
    }
    if (Tp_IsSBNTile(tilePtr))
    {
        (void) InsertBnInList(tilePtr, bnTileList);
    }
    return (MAGIC_CONTINUE_SEARCH);
}


static BearStatus
FindJunctInRect(junctPtr, localDataPtr)
WallJunction *junctPtr;
LocalData *localDataPtr;
{
    Point *pointPtr;

    pointPtr = Fpg_GetPointOfWallJunct(FPG_CENTER, junctPtr);
    if ((pointPtr->p_x < localDataPtr->rect.r_xtop) &&
    (pointPtr->p_y < localDataPtr->rect.r_ytop) &&
    (pointPtr->p_x > localDataPtr->rect.r_xbot) &&
    (pointPtr->p_y > localDataPtr->rect.r_ybot))
    {
    localDataPtr->junctPtr = junctPtr;
    return (BEAR_STOP_SEARCH);
    }
    return (BEAR_CONTINUE_SEARCH);
}

static void
InsertBnInList(bnTilePtr, bnTileList)
Tile       *bnTilePtr;
TileQueue  **bnTileList;
{
    TileQueue *tilePtr;
    TileQueue *currentTilePtr;
    Rect      bnRect;

    MALLOC(TileQueue *, tilePtr, sizeof (TileQueue));
    (void) Tp_HorzTileToRect(tmGlobalPlaneDataPtr, bnTilePtr, &bnRect);
    tilePtr->tilePtr = bnTilePtr;
    tilePtr->paired = 0;
    tilePtr->bottom = bnRect.r_ybot;
    if ((*bnTileList == NIL(TileQueue)) ||
         (bnRect.r_ybot < (*bnTileList)->bottom))
    {
        tilePtr->nextPtr = *bnTileList;
        *bnTileList = tilePtr;
    }
    else
    {
        currentTilePtr = *bnTileList;
        while ((currentTilePtr->nextPtr != NIL(TileQueue)) && 
          (bnRect.r_ybot > currentTilePtr->nextPtr->bottom))
        {
            currentTilePtr = currentTilePtr->nextPtr;
        }
        tilePtr->nextPtr = currentTilePtr->nextPtr;
        currentTilePtr->nextPtr = tilePtr;
    }
}

int
GetStatusOfHBnTile(bnTilePtr, junctPtr)
Tile *bnTilePtr;
WallJunction *junctPtr;
{
    WallSegment  *wallSegPtr2;
    WallJunction *junctPtr2;
    BottleNeck   *bnPtr;
    int          bnTileStatus;

    bnTileStatus = bnTileAlive;

    /* Check the right of the bnTile */
    junctPtr2 = Fpg_GetEastJunctOfWallJunct(junctPtr);
    if (junctPtr2)
    {
       wallSegPtr2 = Fpg_GetSouthSegOfWallJunct(junctPtr2);
       if (wallSegPtr2)
       {
           bnPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr2);
       if (bnPtr->unType == TP_BN_TILE)
           {
              if (bnTilePtr == bnPtr->unValue.bnTilePtr)
              {
                bnTileStatus = bnTileRightDead;
              }
           }
       }
    }

    /* Check the left of the bnTile */
    junctPtr2 = Fpg_GetWestJunctOfWallJunct(junctPtr);
    if (junctPtr2)
    {
        wallSegPtr2 = Fpg_GetSouthSegOfWallJunct(junctPtr2);
        if (wallSegPtr2)
        {
           bnPtr = Fpg_GetBottleNeckOfWallSeg(wallSegPtr2);
       if (bnPtr->unType == TP_BN_TILE)
           {
          if (bnTilePtr == bnPtr->unValue.bnTilePtr)
              {
         if (bnTileStatus == bnTileRightDead)
                    bnTileStatus = bnTileBothDead;
                 else bnTileStatus = bnTileLeftDead;
              }
           }
        }
    }
   return (bnTileStatus);

}
