
/* tpDCell.c -
 *
 *     George Carvalho
 *     (Advising Professor - E. S. Kuh)
 *
 * Dummy cell stuff.
 *
 */

#ifndef lint
static char rcsid[] = "$Header$";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"

typedef struct _MiscData {
    Line        bnSearchLine;           /* bottleneck line search area      */
    Rect        bnSearchArea;           /* bottleneck tile search area      */

    DummyCell   **dCellListPtr;         /* pointer to a list of dummy cells */
    BNLine      *horzBnLineList;
    BNLine      *vertBnLineList;
    Plane       *horzPlanePtr;
    Plane       *vertPlanePtr;
    Rect        *horzPlaneBndPtr;
    Rect        *vertPlaneBndPtr;
} MiscData;

/* ---------- Local Procedures ---------- */
int         FindVertBnTile();
int         FindEmptyRoomTypeOne(), FindEmptyRoomTypeFour();
BearStatus  FindVertBnLine();
BearStatus  FindEmptyRoomTypeTwo(), FindEmptyRoomTypeThree();
void        StoreDCellRect();


/*
 * ----------------------------------------------------------------------------
 *
 * Tp_CreateDCells() --
 *
 * All of the dummy cells are found and put into a list.  Note that
 * this procedure does not alter the tile plane in any way.
 *
 * IMPORTANT:
 *   Tp_IdentifySpaceTiles() MUST be called before yippie yi aye this
 *   procedure or bad things will happen.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_CreateDCells(planeDataPtr)
PlaneData *planeDataPtr;
{
    DummyCell   *dcellList = NIL(DummyCell);
    MiscData    miscData;

#ifdef DEBUG
    if (planeDataPtr == NIL(PlaneData)) {
        TpPrintFatalErr("Tp_CreateDCells() failed");
    }
#endif DEBUG

    /*
     * These fields of the 'miscData' structure are set here
     * and then are not changed.
     */
    miscData.dCellListPtr = &dcellList;
    miscData.horzBnLineList = Tp_GetHorzBnLineList(planeDataPtr);
    miscData.vertBnLineList = Tp_GetVertBnLineList(planeDataPtr);
    miscData.horzPlanePtr = Tp_GetHorzPlane(planeDataPtr);
    miscData.vertPlanePtr = Tp_GetVertPlane(planeDataPtr);
    miscData.horzPlaneBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    miscData.vertPlaneBndPtr = Tp_GetVertPlaneBnd(planeDataPtr);

    if (TiSrArea(NIL(Tile), miscData.vertPlanePtr, miscData.vertPlaneBndPtr,
        FindVertBnTile, (ClientData) &miscData) == 1) {
        return (BEAR_ERROR);
    }
    if (Tp_ItrBnLine(miscData.vertBnLineList,
        FindVertBnLine, (ClientData) &miscData) == BEAR_STOP_SEARCH) {
        return (BEAR_ERROR);
    }
    Tp_PutDCellList(planeDataPtr, dcellList);

    return (BEAR_OK);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_DeleteDCells() --
 *
 * Delete an entire dummy cell list first from the tile plane and then from
 * the MAC database.  Note that this includes freeing all of the respective
 * memory.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Tp_DeleteDCells(planeDataPtr)
PlaneData *planeDataPtr;
{
    DummyCell *dcellList;

#ifdef DEBUG
    if (planeDataPtr == NIL(PlaneData)) {
        TpPrintFatalErr("Tp_DeleteDCells() failed");
    }
#endif DEBUG

    dcellList = Tp_GetDCellList(planeDataPtr);
    (void) Mac_ItrDCell(dcellList, Tp_UnpaintDCell, (ClientData) planeDataPtr);

    Tp_FreeDCellList(planeDataPtr);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_FreeDCellList() --
 *
 * Delete an entire dummy cell list from the MAC database.  Note that this
 * includes freeing all of the respective memory.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

void
Tp_FreeDCellList(planeDataPtr)
PlaneData *planeDataPtr;
{
    DummyCell *dcellList;

#ifdef DEBUG
    if (planeDataPtr == NIL(PlaneData)) {
        TpPrintFatalErr("Tp_DeleteDCells() failed");
    }
#endif DEBUG

    if ((dcellList = Tp_GetDCellList(planeDataPtr)) != NIL(DummyCell)) {

        Mac_DeleteDCellList(&dcellList);
        Tp_PutDCellList(planeDataPtr, NIL(DummyCell));
    }
}



static int
FindEmptyRoomTypeOne(horzTilePtr, miscDataPtr)
register Tile   *horzTilePtr;
MiscData        *miscDataPtr;
/*
 * Find empty rooms that are the result of the intersection of a 
 * vertical bottleneck tile and a horizontal bottleneck tile.
 */
{
    DummyCell   *dcellPtr;
    Rect        dcellRect, horzRect;

    if (Tp_IsBottleneckTile(horzTilePtr)) {
        TITORECT(horzTilePtr, &horzRect);
        GeoClip(&horzRect, miscDataPtr->horzPlaneBndPtr);

        if (Mac_CreateDCell(miscDataPtr->dCellListPtr, &dcellPtr) ==
            BEAR_ERROR) {
            return (1); /* stop search */
        }
        Mac_PutClassOfDCell((DCellClass) HBNTI_AND_VBNTI, dcellPtr);

        dcellRect.r_xbot = miscDataPtr->bnSearchArea.r_xbot;
        dcellRect.r_xtop = miscDataPtr->bnSearchArea.r_xtop;
        dcellRect.r_ybot = horzRect.r_ybot;
        dcellRect.r_ytop = horzRect.r_ytop;
        StoreDCellRect(&dcellRect, dcellPtr);
    }
    return (0); /* continue search */
}



static BearStatus
FindEmptyRoomTypeTwo(horzBnLinePtr, miscDataPtr)
BNLine      *horzBnLinePtr;
MiscData    *miscDataPtr;
/*
 * Find empty rooms that are the result of the intersection of a
 * vertical bottleneck tile and a horizontal bottleneck line.
 */
{
    DummyCell           *dcellPtr;
    Line                *horzLinePtr;
    Rect                dcellRect;
    int                 horzX1, horzX2, horzY;
    register Rect       *vertRectPtr;

    horzLinePtr = Tp_GetLineOfBnLine(horzBnLinePtr);
    horzX1 = horzLinePtr->point1.p_x;
    horzX2 = horzLinePtr->point2.p_x;
    horzY = horzLinePtr->point1.p_y;

    vertRectPtr = &miscDataPtr->bnSearchArea;

    if ((vertRectPtr->r_ybot < horzY && vertRectPtr->r_ytop > horzY) &&

        ((horzX1 < vertRectPtr->r_xbot && horzX2 > vertRectPtr->r_xtop) ||
        (horzX2 < vertRectPtr->r_xbot && horzX1 > vertRectPtr->r_xtop))) {

        if (Mac_CreateDCell(miscDataPtr->dCellListPtr, &dcellPtr) ==
            BEAR_ERROR) {
            return (BEAR_STOP_SEARCH);
        }
        Mac_PutClassOfDCell((DCellClass) HBNLN_AND_VBNTI, dcellPtr);

        dcellRect.r_xbot = vertRectPtr->r_xbot;
        dcellRect.r_xtop = vertRectPtr->r_xtop;
        dcellRect.r_ybot = horzY - 1;
        dcellRect.r_ytop = horzY + 1;
        StoreDCellRect(&dcellRect, dcellPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}



static BearStatus
FindEmptyRoomTypeThree(horzBnLinePtr, miscDataPtr)
BNLine      *horzBnLinePtr;
MiscData    *miscDataPtr;
/*
 * Find empty rooms that are the result of the intersection of a
 * vertical bottleneck line and a horizontal bottleneck line.
 */
{
    DummyCell           *dcellPtr;
    Line                *horzLinePtr;
    Rect                dcellRect;
    int                 horzX1, horzX2, horzY;
    int                 vertX, vertY1, vertY2;

    vertX = miscDataPtr->bnSearchLine.point1.p_x;
    vertY1 = miscDataPtr->bnSearchLine.point1.p_y;
    vertY2 = miscDataPtr->bnSearchLine.point2.p_y;

    horzLinePtr = Tp_GetLineOfBnLine(horzBnLinePtr);
    horzX1 = horzLinePtr->point1.p_x;
    horzX2 = horzLinePtr->point2.p_x;
    horzY = horzLinePtr->point1.p_y;
 
    if (((horzX1 < vertX && horzX2 > vertX) ||
        (horzX2 < vertX && horzX1 > vertX)) &&

        ((vertY1 < horzY && vertY2 > horzY) ||
        (vertY2 < horzY && vertY1 > horzY))) {

        if (Mac_CreateDCell(miscDataPtr->dCellListPtr, &dcellPtr) ==
            BEAR_ERROR) {
            return (BEAR_STOP_SEARCH);
        }
        Mac_PutClassOfDCell((DCellClass) HBNLN_AND_VBNLN, dcellPtr);

        dcellRect.r_xbot = vertX - 1;
        dcellRect.r_xtop = vertX + 1;
        dcellRect.r_ybot = horzY - 1;
        dcellRect.r_ytop = horzY + 1;
        StoreDCellRect(&dcellRect, dcellPtr);
    }
    return (BEAR_CONTINUE_SEARCH);
}



static int
FindEmptyRoomTypeFour(horzTilePtr, miscDataPtr)
Tile        *horzTilePtr;
MiscData    *miscDataPtr;
/*
 * Find empty rooms that are the result of the intersection of a
 * vertical bottleneck line and a horizontal bottleneck tile.
 */
{
    DummyCell           *dcellPtr;
    Rect                dcellRect, horzRect;
    int                 vertX, vertY1, vertY2;

    if (Tp_IsBottleneckTile(horzTilePtr)) {
        TITORECT(horzTilePtr, &horzRect);
        GeoClip(&horzRect, miscDataPtr->horzPlaneBndPtr);

        vertX = miscDataPtr->bnSearchLine.point1.p_x;
        vertY1 = miscDataPtr->bnSearchLine.point1.p_y;
        vertY2 = miscDataPtr->bnSearchLine.point2.p_y;

        if ((horzRect.r_xbot < vertX && horzRect.r_xtop > vertX) &&

            ((vertY1 < horzRect.r_ybot && vertY2 > horzRect.r_ytop) ||
            (vertY2 < horzRect.r_ybot && vertY1 > horzRect.r_ytop))) {

            if (Mac_CreateDCell(miscDataPtr->dCellListPtr, &dcellPtr) ==
                BEAR_ERROR) {
                return (1); /* stop search */
            }
            Mac_PutClassOfDCell((DCellClass) HBNTI_AND_VBNLN, dcellPtr);

            dcellRect.r_xbot = vertX - 1;
            dcellRect.r_xtop = vertX + 1;
            dcellRect.r_ybot = horzRect.r_ybot;
            dcellRect.r_ytop = horzRect.r_ytop;
            StoreDCellRect(&dcellRect, dcellPtr);
        }
    }
    return (0); /* continue search */
}



static BearStatus
FindVertBnLine(vertBnLinePtr, miscDataPtr)
BNLine      *vertBnLinePtr;
MiscData    *miscDataPtr;
{
    Line *vertLinePtr;

    vertLinePtr = Tp_GetLineOfBnLine(vertBnLinePtr);

    /* Fill in the bottleneck LINE search area. */
    miscDataPtr->bnSearchLine.point1.p_x = vertLinePtr->point1.p_x;
    miscDataPtr->bnSearchLine.point1.p_y = vertLinePtr->point1.p_y;
    miscDataPtr->bnSearchLine.point2.p_x = vertLinePtr->point2.p_x;
    miscDataPtr->bnSearchLine.point2.p_y = vertLinePtr->point2.p_y;

    if (Tp_ItrBnLine(miscDataPtr->horzBnLineList, FindEmptyRoomTypeThree,
        (ClientData) miscDataPtr) == BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }

    /* Fill in the bottleneck TILE search area. */
    miscDataPtr->bnSearchArea.r_xbot = vertLinePtr->point1.p_x - 1;
    miscDataPtr->bnSearchArea.r_xtop = vertLinePtr->point1.p_x + 1;
    if (vertLinePtr->point1.p_y < vertLinePtr->point2.p_y) {
        miscDataPtr->bnSearchArea.r_ybot = vertLinePtr->point1.p_y;
        miscDataPtr->bnSearchArea.r_ytop = vertLinePtr->point2.p_y;
    } else {
        miscDataPtr->bnSearchArea.r_ybot = vertLinePtr->point2.p_y;
        miscDataPtr->bnSearchArea.r_ytop = vertLinePtr->point1.p_y;
    }

    if (TiSrArea(NIL(Tile), miscDataPtr->horzPlanePtr,
        &miscDataPtr->bnSearchArea, FindEmptyRoomTypeFour,
        (ClientData) miscDataPtr) == 1) {
        return (BEAR_STOP_SEARCH);
    }
    return (BEAR_CONTINUE_SEARCH);
}



/* ARGSUSED */
static int
FindVertBnTile(vertTilePtr, miscDataPtr)
Tile        *vertTilePtr;
MiscData    *miscDataPtr;
{
    Rect vertRect;

    if (Tp_IsBottleneckTile(vertTilePtr)) {
        TITORECT(vertTilePtr, &vertRect);
        GeoClip(&vertRect, miscDataPtr->vertPlaneBndPtr);

        /* Fill in the bottleneck TILE search area. */
        Tp_VRectToHRect(&vertRect, &miscDataPtr->bnSearchArea);
        if (TiSrArea(NIL(Tile), miscDataPtr->horzPlanePtr,
            &miscDataPtr->bnSearchArea, FindEmptyRoomTypeOne,
            (ClientData) miscDataPtr) == 1) {
            return (1); /* stop search */
        }

        if (Tp_ItrBnLine(miscDataPtr->horzBnLineList, FindEmptyRoomTypeTwo,
            (ClientData) miscDataPtr) == BEAR_STOP_SEARCH) {
            return (1); /* stop search */
        }
    }
    return (0); /* continue search */
}



static void
StoreDCellRect(dcellRectPtr, dcellPtr)
Rect        *dcellRectPtr;
DummyCell   *dcellPtr;
{
    GenericCell         gcell;
    register LinkedRect *rectList;

    gcell.unType = (CellClass) DUMMY_CELL;
    gcell.unValue.dummyCellPtr = dcellPtr;

    MALLOC(LinkedRect *, rectList, sizeof (LinkedRect));
    rectList->r_next = NIL(LinkedRect);
    Mac_CopyRect(&rectList->r_r, dcellRectPtr);
    Mac_PutHorzRectListOfGCell(rectList, &gcell);

    MALLOC(LinkedRect *, rectList, sizeof (LinkedRect));
    rectList->r_next = NIL(LinkedRect);
    Mac_CopyRect(&rectList->r_r, dcellRectPtr);
    Mac_PutVertRectListOfGCell(rectList, &gcell);

    Mac_PutBBoxOfDCell(dcellRectPtr, dcellPtr);
}



