
/* tpDuplicate.c --
 *
 *     Sangjin Hong
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpDuplicate.c,v 1.1 88/06/14 10:24:10 sangjin Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "rectSlice.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"

/*
 * ----------------------------------------------------------------------------
 *
 * Tp_DuplicateCellInst() --
 *
 * Duplicate a cell instance.
 *
 * Results:
 *    BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *    The cell is painted onto the horizontal and vertical
 *    tile planes and entered into the data base.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_DuplicateCellInst(cellInstPtr, moveDeltaPtr)
CellInst        *cellInstPtr;
Point           *moveDeltaPtr;
{
    CellMaster      *parentCellPtr;
    CellInst        *copyCellInstPtr;
    CheckData       checkData;
    GenericCell     gcell,origGCell;
    PlaneData       *planeDataPtr;
    Point           *originPtr, newOrigin;
    Transform       *origHorzTransPtr,horzTrans,trans;
    char            cellInstName[80];


    parentCellPtr = Mac_GetParentOfCell(cellInstPtr);
    planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);
    origGCell.unType = CELL_INST;
    origGCell.unValue.cellInstPtr = cellInstPtr;
    origHorzTransPtr = Mac_GetTransformOfGCell(&origGCell);
    GeoTranslateTrans(origHorzTransPtr,moveDeltaPtr->p_x,moveDeltaPtr->p_y,
                      &horzTrans);

    checkData.strong = TRUE;
    checkData.tileType = TT_SPACE;
    checkData.namePtr = NIL(char);
    checkData.planePtr = Tp_GetHorzPlane(planeDataPtr);
    checkData.bndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    checkData.rectList = cellInstPtr->masterPtr->horzRectList;
    checkData.transPtr = &horzTrans;
    if (TpCheckPlaneUnderRectList(&checkData)) 
    {
        sprintf(cellInstName, "B%d", Mac_GenerateId());
        if (Mac_DuplicateCell(parentCellPtr, cellInstPtr,
            cellInstName, &copyCellInstPtr, moveDeltaPtr) == BEAR_ERROR) 
        {
            Bear_PrintErr("Tp_DuplicateCellInst() failed\n");
            return (BEAR_ERROR);
        } 
        if (TpDuplicateLineSeg(cellInstPtr,copyCellInstPtr) == BEAR_ERROR)
        {
            Bear_PrintErr("Tp_DuplicateCellInst() failed\n");
            return (BEAR_ERROR);
        }
        gcell.unType = CELL_INST;
        gcell.unValue.cellInstPtr = copyCellInstPtr;
        Mac_PutTransformOfGCell(&horzTrans, &gcell);
        GeoTranslateTrans(&GeoIdentityTransform,moveDeltaPtr->p_x,
                          moveDeltaPtr->p_y, &trans);
        (void) Mac_MoveBoundaryOfCell(&trans, copyCellInstPtr);
        originPtr = Mac_GetOriginOfCell(cellInstPtr);
        GeoTransPoint(&trans, originPtr, &newOrigin);
        Mac_PutOriginOfCell(&newOrigin, copyCellInstPtr);
        if (TpDuplicatePinOfLineSeg(cellInstPtr,copyCellInstPtr) == BEAR_ERROR)
        {
            Bear_PrintErr("Tp_DuplicateCellInst() failed\n");
            return (BEAR_ERROR);
        }
    (void) Tp_PaintGCell(&gcell, planeDataPtr);
    return (BEAR_OK);
    } 
    else 
    {
        return (BEAR_ERROR);
    }
}



