
/* tpMisc.c -
 *
 *     George Carvalho
 *     Andrew Purshottam
 *     (Advising Professor - E. S. Kuh)
 *
 * Assorted cell stuff.
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpMisc.c,v 1.1 87/09/24 10:24:12 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"


/* ---------- External global variables for the tileProp module ---------- */
bool tp_DebugFlag = FALSE;


/* ---------- Local Procedures ---------- */
static BearStatus    AddCellToTilePlane();
static int           FindBBox();


/*
 * ----------------------------------------------------------------------------
 *
 * Tp_CreateTiPlaneForCellMaster() --
 *
 * Create the horizontal and vertical tile planes for the given 
 * cell master.  This envolves painting all of the cell masters
 * children onto both planes.
 *
 * Results:
 *   BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_CreateTiPlaneForCellMaster(cellMasterPtr)
CellMaster *cellMasterPtr;
{
    BearStatus  bearStatus;
    PlaneData   *planeDataPtr;
    Rect        *horzBndPtr, vertBnd;

    planeDataPtr = Tp_MakePlaneData();

    horzBndPtr = Mac_GetBBoxOfCellMaster(cellMasterPtr);
    Tp_PutHorzPlaneBnd(planeDataPtr, horzBndPtr);
    Tp_HRectToVRect(horzBndPtr, &vertBnd);
    Tp_PutVertPlaneBnd(planeDataPtr, &vertBnd);

#ifdef DEBUG
    if (tp_DebugFlag == TRUE) {
        printf ("-------------\n");
        printf ("plane boundary:\n");
        Mac_PrintRect(horzBndPtr);
        printf ("\n");
        fflush (stdout);
    }
#endif DEBUG

    if (Mac_GetPlacedFlagOfCell(cellMasterPtr) == TRUE) {
        bearStatus = Mac_ItrChildrenOfCell(cellMasterPtr, AddCellToTilePlane,
            (ClientData) planeDataPtr);

#ifdef DEBUG
        if (tp_DebugFlag == TRUE) {
            printf ("-------------\n");
            fflush (stdout);
        }
#endif DEBUG

        if (bearStatus == BEAR_COMPLETE_SEARCH) {
            Mac_PutCellPlaneOfCell(planeDataPtr, cellMasterPtr);
            return (BEAR_OK);
        } 
        else {
            Tp_FreePlaneData(planeDataPtr);
            Mac_PutCellPlaneOfCell(NIL(PlaneData), cellMasterPtr);
            return (BEAR_ERROR);
        }
    }
    else {
#ifdef DEBUG
        if (tp_DebugFlag == TRUE) {
            printf ("Cells aren't placed.\n");
            printf ("-------------\n");
            fflush (stdout);
        }
#endif DEBUG

        Mac_PutCellPlaneOfCell(planeDataPtr, cellMasterPtr);
    }

    return (BEAR_OK);
}



static BearStatus
AddCellToTilePlane(cellInstPtr, planeDataPtr)
CellInst    *cellInstPtr;
PlaneData   *planeDataPtr;
{
    GenericCell gcell;

    gcell.unType = (CellClass) CELL_INST;
    gcell.unValue.cellInstPtr = cellInstPtr;

#ifdef DEBUG
    if (tp_DebugFlag == TRUE) {
        printf ("cell instance name: %s\n", Mac_GetNameOfCellInst(cellInstPtr));
        fflush (stdout);
        printf ("horz. rect list:\n");
        Mac_PrintRectList(&cellInstPtr->transform,
            Mac_GetHorzRectListOfGCell(&gcell));
    }
#endif DEBUG

    return (Tp_PaintGCell(&gcell, planeDataPtr) ?
        BEAR_CONTINUE_SEARCH : BEAR_STOP_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_FreeTileList() --
 *
 * Free the memory associated with each tile in the given list.
 *
 * For this to work the memory for the tile list must have been allocated
 * one element at a time !!
 *
 * Results:
 *   none
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

void
Tp_FreeTileList(headPtr)
register LinkedTile *headPtr;
{
    register LinkedTile *tmpPtr;

    while ((tmpPtr = headPtr) != NIL(LinkedTile)) {
        headPtr = tmpPtr->nextPtr;
        FREE((char *) tmpPtr);
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_FindMinWidth() --
 *
 *     Goes through each tile in 'horzSpaceTileList' which must be a
 * list of space tiles from the horizontal plane and looks for the
 * narrowest one.
 *
 * Results:
 *     Returns the width of the narrowest tile.
 *
 * Side effects:
 *     None.
 * ----------------------------------------------------------------------------
 */

int
Tp_FindMinWidth(planeDataPtr, horzSpaceTileList)
PlaneData *planeDataPtr;
LinkedTile *horzSpaceTileList;    /* list of tiles from the horizontal plane */
{
    Rect *horzPlaneBndPtr;
    Tile *leftPseudoTilePtr, *rightPseudoTilePtr;
    int leftBnd, rightBnd;
    register LinkedTile *tmpPtr;
    register int minWidth, width;

#ifdef DEBUG
    if (planeDataPtr == NIL(PlaneData) ||
        horzSpaceTileList == NIL(LinkedTile)) {

        TpPrintFatalErr("Tp_FindMinWidth() failed");
    }
#endif DEBUG

    horzPlaneBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    leftBnd = horzPlaneBndPtr->r_xbot;
    rightBnd = horzPlaneBndPtr->r_xtop;
    leftPseudoTilePtr = planeDataPtr->pd_horzPlanePtr->pl_left;
    rightPseudoTilePtr = planeDataPtr->pd_horzPlanePtr->pl_right;
    tmpPtr = horzSpaceTileList;
    minWidth = width = ((TR(tmpPtr->tilePtr) == rightPseudoTilePtr) ?
        rightBnd : RIGHT(tmpPtr->tilePtr)) -
        ((BL(tmpPtr->tilePtr) == leftPseudoTilePtr) ?
        leftBnd : LEFT(tmpPtr->tilePtr));

    for (tmpPtr = tmpPtr->nextPtr;
        tmpPtr != NIL(LinkedTile);
        tmpPtr = tmpPtr->nextPtr) {

        width = ((TR(tmpPtr->tilePtr) == rightPseudoTilePtr) ?
            rightBnd : RIGHT(tmpPtr->tilePtr)) -
            ((BL(tmpPtr->tilePtr) == leftPseudoTilePtr) ?
            leftBnd : LEFT(tmpPtr->tilePtr));

        if (width < minWidth) {
            minWidth = width;
        }
    }
    return (minWidth);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_FindMinHeight() --
 *
 *     Goes through each tile in 'vertSpaceTileList' which must be a
 * list of space tiles from the vertical plane and looks for the
 * shortest one.
 *
 * Results:
 *     Returns the height of the shortest tile.
 *
 * Side effects:
 *     None.
 * ----------------------------------------------------------------------------
 */

int
Tp_FindMinHeight(planeDataPtr, vertSpaceTileList)
PlaneData *planeDataPtr;
LinkedTile *vertSpaceTileList;    /* list of tiles from the vertical plane */
{
    Rect *vertPlaneBndPtr;
    Tile *leftPseudoTilePtr, *rightPseudoTilePtr;
    int leftBnd, rightBnd;
    register LinkedTile *tmpPtr;
    register int minHeight, height;

#ifdef DEBUG
    if (planeDataPtr == NIL(PlaneData) ||
        vertSpaceTileList == NIL(LinkedTile)) {

        TpPrintFatalErr("Tp_FindMinHeight() failed");
    }
#endif DEBUG

    vertPlaneBndPtr = Tp_GetVertPlaneBnd(planeDataPtr);
    leftBnd = vertPlaneBndPtr->r_xbot;
    rightBnd = vertPlaneBndPtr->r_xtop;
    leftPseudoTilePtr = planeDataPtr->pd_vertPlanePtr->pl_left;
    rightPseudoTilePtr = planeDataPtr->pd_vertPlanePtr->pl_right;

    tmpPtr = vertSpaceTileList;
    minHeight = height = ((TR(tmpPtr->tilePtr) == rightPseudoTilePtr) ?
        rightBnd : RIGHT(tmpPtr->tilePtr)) -
        ((BL(tmpPtr->tilePtr) == leftPseudoTilePtr) ? 
        leftBnd : LEFT(tmpPtr->tilePtr));
    for (tmpPtr = tmpPtr->nextPtr;
        tmpPtr != NIL(LinkedTile);
        tmpPtr = tmpPtr->nextPtr) {

        height = ((TR(tmpPtr->tilePtr) == rightPseudoTilePtr) ?
        rightBnd : RIGHT(tmpPtr->tilePtr)) -
        ((BL(tmpPtr->tilePtr) == leftPseudoTilePtr) ? 
        leftBnd : LEFT(tmpPtr->tilePtr));

        if (height < minHeight) {
            minHeight = height;
        }
    }
    return (minHeight);
}



void
Tp_FindBBoxForAllCells(planeDataPtr, bboxPtr)
PlaneData   *planeDataPtr;
Rect        *bboxPtr;
{
    Mac_CopyRect(bboxPtr, &GeoNullRect);
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
        &TiPlaneRect, FindBBox, (ClientData) bboxPtr);
}



/* ARGSUSED */
static int
FindBBox(tilePtr, bboxPtr)
register Tile *tilePtr;
register Rect *bboxPtr;
{
    Rect rect;

    if (Tp_IsSolidTile(tilePtr)) {
        TiToRect(tilePtr, &rect);
        (void) GeoInclude(&rect, bboxPtr);
    }
    return (0); /* continue search */
}



void
Tp_StretchRect(rectPtr, percent)
register Rect *rectPtr;
double percent;
{
    register int deltaX, deltaY;

    deltaX = (rectPtr->r_xtop - rectPtr->r_xbot) * (percent - 1.0) / 2.0;
    deltaY = (rectPtr->r_ytop - rectPtr->r_ybot) * (percent - 1.0) / 2.0;
    rectPtr->r_xbot -= deltaX;
    rectPtr->r_ybot -= deltaY;
    rectPtr->r_xtop += deltaX;
    rectPtr->r_ytop += deltaY;
}



void
TpPrintFatalErr(message)
BearString message;
{
    errRaise(TP_MOD_NAME, ((int) -1), message);
}


