
/* tpMove.c --
 *
 *     M. Kobayashi
 *     (Advising Professor - E. S. Kuh)
 *
 * Movement of cell or cell group.
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpMove.c,v 1.1 87/09/24 10:24:17 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"
#include "tileMapping.h"

typedef struct _PPinInfo 
{
    PseudoPin         *trPPinList;
    PseudoPin         *blPPinList;
    int               fragment;
    int               yPos;
    struct _PPinInfo  *nextPtr;
} PPinInfo;

typedef struct _BnTileInfo 
{
    LinkedTile  *blBnTileList;
    LinkedTile  *trBnTileList;
} BnTileInfo;

static   int        FindBnTile();
static   bool       IsTileInTileList();
static   BearStatus PaintGCell();
static   BearStatus UnPaintGCell();
static   BearStatus GetLAndRBnTiles();
static   BearStatus GetBAndTBnTiles();
static   void       SavePPinList();
static   void       RestorePPinList();
static   void       FreePPinInfoList();
static   bool       CheckPlaneUnderCellGroup();
static   BearStatus CheckPlaneUnderGCell();
static   int        CheckTileType();
static   void       HorzMoveCellGroup();
static   void       VertMoveCellGroup();
static   void       HorzMoveCellInst();
static   void       VertMoveCellInst();
static   BearStatus MoveGCell();

CellMaster   *parentCellPtr;
PlaneData    *planeDataPtr;


/*
 * ----------------------------------------------------------------------------
 *
 * Tp_MoveCell() --
 *
 *    Move a cell or a cell group.
 *
 * Results:
 *    BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *    The tile planes and the data base are updated.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_MoveCell(gcellPtr, deltaPtr)
GenericCell *gcellPtr;
Point       *deltaPtr;
{
    CellGroup *cellGroupPtr;

    if (Mac_IsGroupedCell(gcellPtr->unType,
        (ClientData) gcellPtr->unValue.cellInstPtr)) {
        if (gcellPtr->unType == CELL_INST) {
            cellGroupPtr = Mac_GetGroupOfCell(gcellPtr->unValue.cellInstPtr);
        }
        else {
            cellGroupPtr = Mac_GetGroupOfRCell(gcellPtr->unValue.routeCellPtr);
        }
        return (Tp_MoveCellGroup(cellGroupPtr, deltaPtr));
    }
    else {
        if (gcellPtr->unType != CELL_INST) {
            Bear_PrintErr("Tp_MoveCell(): cell instance is not given.");
        }
        return (Tp_MoveCellInst(gcellPtr->unValue.cellInstPtr, deltaPtr));
    }
    /* NOTREACHED */
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_MoveCellGroup() --
 *
 *    Move a cell group.
 *
 * Results:
 *    BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *    The tile planes and the data base are updated.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_MoveCellGroup(cellGroupPtr, deltaPtr)
CellGroup   *cellGroupPtr;
Point       *deltaPtr;
{
    GenericCell  *gCellPtr;

    gCellPtr = cellGroupPtr->cellList->cellPtr;
    if (gCellPtr->unType == CELL_INST)
    {
    parentCellPtr = Mac_GetParentOfCell(gCellPtr->unValue.cellInstPtr);
    }
    else
    {
    parentCellPtr = Mac_GetCellOfRCell(gCellPtr->unValue.routeCellPtr);
    }
    planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);

    if (!CheckPlaneUnderCellGroup(cellGroupPtr,deltaPtr->p_x,deltaPtr->p_y))
    {
    Bear_PrintErr("Tp_MoveCellGroup(): The new position is not completely");
    Bear_PrintErr("                    within the planes boundary, or the");
    Bear_PrintErr("                    area under it is not free.");
    return (BEAR_ERROR);
    }
    if (CheckPlaneUnderCellGroup(cellGroupPtr,deltaPtr->p_x,0))
    {
    HorzMoveCellGroup(cellGroupPtr, deltaPtr->p_x);
    VertMoveCellGroup(cellGroupPtr, deltaPtr->p_y);
    return (BEAR_OK);
    }
    if (CheckPlaneUnderCellGroup(cellGroupPtr, 0, deltaPtr->p_y))
    {
    VertMoveCellGroup(cellGroupPtr, deltaPtr->p_y);
    HorzMoveCellGroup(cellGroupPtr, deltaPtr->p_x);
    return (BEAR_OK);
    }
    Bear_PrintErr("Tp_MoveCellGroup(): No path to the new position");
    return (BEAR_ERROR);
}



static bool
CheckPlaneUnderCellGroup(cellGroupPtr,deltaX,deltaY)
CellGroup   *cellGroupPtr;
int         deltaX;
int         deltaY;
{
    Point   delta;

    delta.p_x = deltaX;
    delta.p_y = deltaY;
    if (Mac_ItrCellGroup(cellGroupPtr, CheckPlaneUnderGCell,
                   (ClientData)&delta) == BEAR_COMPLETE_SEARCH)
    {
    return (TRUE);
    }
    return (FALSE);
}


static BearStatus 
CheckPlaneUnderGCell(linkedCellPtr, deltaPtr)
LinkedCell  *linkedCellPtr;
Point       *deltaPtr;
{
    GenericCell *gCellPtr;
    CellGroup   *cellGroupPtr;
    LinkedRect  *linkedRectPtr;
    Transform   *transPtr;
    Transform   newTrans;
    Rect        checkRect;
    
    gCellPtr = linkedCellPtr->cellPtr;
    if (gCellPtr->unType == CELL_INST)
    {
    cellGroupPtr = Mac_GetGroupOfCell(gCellPtr->unValue.cellInstPtr);
    }
    else
    {
    cellGroupPtr = Mac_GetGroupOfRCell(gCellPtr->unValue.routeCellPtr);
    }
    transPtr = Mac_GetTransformOfGCell(gCellPtr);
    GeoTranslateTrans(transPtr, deltaPtr->p_x, deltaPtr->p_y, &newTrans);
    for (linkedRectPtr  = Mac_GetHorzRectListOfGCell(gCellPtr);
         linkedRectPtr != NIL(LinkedRect);
         linkedRectPtr  = linkedRectPtr->r_next)
    {
        GeoTransRect(&newTrans, &linkedRectPtr->r_r, &checkRect);
        if (!GEO_SURROUND(Tp_GetHorzPlaneBnd(planeDataPtr), &checkRect))
        {
            return (BEAR_STOP_SEARCH);
        }
        if (TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr), &checkRect,
                CheckTileType, (ClientData) cellGroupPtr) != 0)
        {
            return (BEAR_STOP_SEARCH);
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}


static int
CheckTileType(tilePtr, cellGroupPtr)
Tile       *tilePtr;
CellGroup  *cellGroupPtr;
{
    if (Tp_IsSpaceTile(tilePtr) || Tp_IsDCellTile(tilePtr))
    {
        return (0); /* continue search */
    }
    if (Tp_IsCellInstTile(tilePtr))
    {
        if (cellGroupPtr == Mac_GetGroupOfCell(Tp_GetCellInstOfTile(tilePtr)))
        {
            return (0); /* continue search */
        }
    }
    else if (Tp_IsRCellTile(tilePtr))
    {
        if (cellGroupPtr == Mac_GetGroupOfRCell(Tp_GetRCellOfTile(tilePtr)))
        {
            return (0); /* continue search */
        }
    }
    return (1); /* stop search */
}


static void
HorzMoveCellGroup(cellGroupPtr, deltaX)
CellGroup  *cellGroupPtr;
int        deltaX;
{
    BnTileInfo   bnTileInfo;
    PPinInfo     *lPPinInfoList;
    PPinInfo     *rPPinInfoList;
    Point        delta;

    if (deltaX == 0)
    {
    return;
    }
    Tm_HUnify(parentCellPtr);

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) Mac_ItrCellGroup(cellGroupPtr, GetLAndRBnTiles,
                           (ClientData)&bnTileInfo);
    lPPinInfoList = rPPinInfoList = NIL(PPinInfo);
    SavePPinList(bnTileInfo.blBnTileList, BOTTOM_LEFT_SIDE, &lPPinInfoList);
    SavePPinList(bnTileInfo.trBnTileList, TOP_RIGHT_SIDE,   &rPPinInfoList);
    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);
    (void) Mac_ItrCellGroup(cellGroupPtr, UnPaintGCell, (ClientData)NULL);
    delta.p_x = deltaX;
    delta.p_y = 0;
    (void) Mac_ItrCellGroup(cellGroupPtr, MoveGCell, (ClientData)&delta);
    (void) Mac_ItrCellGroup(cellGroupPtr, PaintGCell, (ClientData)NULL);

    (void) Tp_IdentifySpaceTiles(planeDataPtr);
    (void) Fpg_CreateFloorPlanGraph(planeDataPtr, (WallJunction **) NULL);

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) Mac_ItrCellGroup(cellGroupPtr, GetLAndRBnTiles,
                           (ClientData)&bnTileInfo);
    RestorePPinList(bnTileInfo.blBnTileList, lPPinInfoList);
    RestorePPinList(bnTileInfo.trBnTileList, rPPinInfoList);
    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);
    FreePPinInfoList(lPPinInfoList);
    FreePPinInfoList(rPPinInfoList);

    Tm_HSplit(parentCellPtr);
}



static void
VertMoveCellGroup(cellGroupPtr, deltaY)
CellGroup  *cellGroupPtr;
int        deltaY;
{
    BnTileInfo   bnTileInfo;
    PPinInfo     *bPPinInfoList;
    PPinInfo     *tPPinInfoList;
    Point        delta;

    if (deltaY == 0)
    {
    return;
    }
    Tm_VUnify(parentCellPtr);

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) Mac_ItrCellGroup(cellGroupPtr, GetBAndTBnTiles,
                           (ClientData)&bnTileInfo);
    bPPinInfoList = tPPinInfoList = NIL(PPinInfo);
    SavePPinList(bnTileInfo.blBnTileList, BOTTOM_LEFT_SIDE, &bPPinInfoList);
    SavePPinList(bnTileInfo.trBnTileList, TOP_RIGHT_SIDE,   &tPPinInfoList);
    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);
    (void) Mac_ItrCellGroup(cellGroupPtr, UnPaintGCell, (ClientData)NULL);
    delta.p_x = 0;
    delta.p_y = deltaY;
    (void) Mac_ItrCellGroup(cellGroupPtr, MoveGCell, (ClientData)&delta);
    (void) Mac_ItrCellGroup(cellGroupPtr, PaintGCell, (ClientData)NULL);

    (void) Tp_IdentifySpaceTiles(planeDataPtr);
    (void) Fpg_CreateFloorPlanGraph(planeDataPtr, (WallJunction **) NULL);

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) Mac_ItrCellGroup(cellGroupPtr, GetBAndTBnTiles,
                           (ClientData)&bnTileInfo);
    RestorePPinList(bnTileInfo.blBnTileList, bPPinInfoList);
    RestorePPinList(bnTileInfo.trBnTileList, tPPinInfoList);
    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);
    FreePPinInfoList(bPPinInfoList);
    FreePPinInfoList(tPPinInfoList);

    Tm_VSplit(parentCellPtr);
}


static BearStatus
GetLAndRBnTiles(linkedCellPtr, bnTileInfoPtr)
LinkedCell  *linkedCellPtr;
BnTileInfo  *bnTileInfoPtr;
{
    Rect         searchRect;
    Rect         rect;
    Transform    *transPtr;
    LinkedRect   *linkedRectPtr;

    transPtr  = Mac_GetTransformOfGCell(linkedCellPtr->cellPtr);
    for (linkedRectPtr  = Mac_GetVertRectListOfGCell(linkedCellPtr->cellPtr);
     linkedRectPtr != NIL(LinkedRect) ;
     linkedRectPtr  = linkedRectPtr->r_next)
    {
    GeoTransRect(transPtr, &linkedRectPtr->r_r, &rect);
    searchRect.r_ll.p_x = rect.r_ur.p_x;
    searchRect.r_ll.p_y = rect.r_ll.p_y;
    searchRect.r_ur.p_x = rect.r_ur.p_x + 1;
    searchRect.r_ur.p_y = rect.r_ur.p_y;
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr), &searchRect,
               FindBnTile, (ClientData)&bnTileInfoPtr->trBnTileList);
    searchRect.r_ll.p_x = rect.r_ll.p_x - 1;
    searchRect.r_ll.p_y = rect.r_ll.p_y;
    searchRect.r_ur.p_x = rect.r_ll.p_x;
    searchRect.r_ur.p_y = rect.r_ur.p_y;
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr), &searchRect,
               FindBnTile, (ClientData)&bnTileInfoPtr->blBnTileList);
    }
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
GetBAndTBnTiles(linkedCellPtr, bnTileInfoPtr)
LinkedCell  *linkedCellPtr;
BnTileInfo  *bnTileInfoPtr;
{
    Rect         searchRect;
    Rect         vSearchRect;
    Rect         rect;
    Transform    *transPtr;
    LinkedRect   *linkedRectPtr;

    transPtr  = Mac_GetTransformOfGCell(linkedCellPtr->cellPtr);
    for (linkedRectPtr  = Mac_GetVertRectListOfGCell(linkedCellPtr->cellPtr);
     linkedRectPtr != NIL(LinkedRect) ;
     linkedRectPtr  = linkedRectPtr->r_next)
    {
    GeoTransRect(transPtr, &linkedRectPtr->r_r, &rect);
    searchRect.r_ll.p_x = rect.r_ll.p_x;
    searchRect.r_ll.p_y = rect.r_ur.p_y;
    searchRect.r_ur.p_x = rect.r_ur.p_x;
    searchRect.r_ur.p_y = rect.r_ur.p_y + 1;
    (void) Tp_HRectToVRect(&searchRect, &vSearchRect);
    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr), &vSearchRect,
               FindBnTile, (ClientData)&bnTileInfoPtr->trBnTileList);
    searchRect.r_ll.p_x = rect.r_ll.p_x;
    searchRect.r_ll.p_y = rect.r_ll.p_y - 1;
    searchRect.r_ur.p_x = rect.r_ur.p_x;
    searchRect.r_ur.p_y = rect.r_ll.p_y;
    (void) Tp_HRectToVRect(&searchRect, &vSearchRect);
    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr), &vSearchRect,
               FindBnTile, (ClientData)&bnTileInfoPtr->blBnTileList);
    }
    return (BEAR_CONTINUE_SEARCH);
}



static void
SavePPinList(bnTileList, side, pPinInfoListPtr)
LinkedTile    *bnTileList;
SideDirection side;
PPinInfo      **pPinInfoListPtr;
{
    Tile    *tilePtr;
    PPinInfo    *newPPinInfoPtr;
    LinkedTile  *curLinkedTilePtr;

    curLinkedTilePtr = bnTileList;
    while (curLinkedTilePtr != NIL(LinkedTile))
    {
    MALLOC(PPinInfo *, newPPinInfoPtr, sizeof (PPinInfo));
    tilePtr = curLinkedTilePtr->tilePtr;
    if (Tp_IsBottleneckTile(tilePtr))
    {
        newPPinInfoPtr->fragment = TP_BOTTOM;
    }
    else if (Tp_IsSBNTile(tilePtr))
    {
        if (side == BOTTOM_LEFT_SIDE)
        {
        newPPinInfoPtr->fragment = TP_TOP;
        }
        else
        {
        newPPinInfoPtr->fragment = TP_BOTTOM;
        }
    }
    else
    {
        TpPrintFatalErr("SavePPinList(): not a BN or SBN tile");
    }
    (void) Tp_RemovePPinListOfTile(tilePtr, TOP_RIGHT_SIDE,
                       newPPinInfoPtr->fragment,
                       &newPPinInfoPtr->trPPinList);
    (void) Tp_RemovePPinListOfTile(tilePtr, BOTTOM_LEFT_SIDE,
                       newPPinInfoPtr->fragment,
                       &newPPinInfoPtr->blPPinList);
    newPPinInfoPtr->yPos = BOTTOM(tilePtr);
    newPPinInfoPtr->nextPtr = *pPinInfoListPtr;
    *pPinInfoListPtr = newPPinInfoPtr;
    curLinkedTilePtr = curLinkedTilePtr->nextPtr;
    }
    return;
}



static void
RestorePPinList(bnTileList, pPinInfoList)
LinkedTile  *bnTileList;
PPinInfo    *pPinInfoList;
{
    Tile       *tilePtr;
    PPinInfo   *curPPinInfoPtr;
    LinkedTile *curLinkedTilePtr;

    curLinkedTilePtr = bnTileList;
    while (curLinkedTilePtr != NIL(LinkedTile))
    {
    tilePtr = curLinkedTilePtr->tilePtr;
    curPPinInfoPtr = pPinInfoList;
    while (curPPinInfoPtr != NIL(PPinInfo))
    {
        if (BOTTOM(tilePtr) == curPPinInfoPtr->yPos)
        {
        break;
        }
        curPPinInfoPtr = curPPinInfoPtr->nextPtr;
    }
    if (curPPinInfoPtr == NIL(PPinInfo))
    {
        TpPrintFatalErr("RestorePPinList():can't find any bn tile");
    }
    (void) Tp_AppendPPinListOfTile(curPPinInfoPtr->trPPinList, tilePtr,
                    TOP_RIGHT_SIDE,
                    curPPinInfoPtr->fragment); 
    (void) Tp_AppendPPinListOfTile(curPPinInfoPtr->blPPinList, tilePtr,
                    BOTTOM_LEFT_SIDE,
                    curPPinInfoPtr->fragment); 
    curLinkedTilePtr = curLinkedTilePtr->nextPtr;
    }
    return;
}



static void
FreePPinInfoList(pPinInfoList)
PPinInfo  *pPinInfoList;
{
    PPinInfo  *curPPinInfoPtr;

    curPPinInfoPtr = pPinInfoList;
    while (curPPinInfoPtr != NIL(PPinInfo))
    {
    pPinInfoList = curPPinInfoPtr->nextPtr;
    FREE((char *)curPPinInfoPtr);
    curPPinInfoPtr = pPinInfoList;
    }
    return;
}



/* ARGSUSED */
static BearStatus
UnPaintGCell(linkedCellPtr, clientData)
LinkedCell  *linkedCellPtr;
ClientData  clientData;
{
    Tp_UnpaintGCell(linkedCellPtr->cellPtr, planeDataPtr);
    return (BEAR_CONTINUE_SEARCH);
}


/* ARGSUSED */
static BearStatus
PaintGCell(linkedCellPtr, clientData)
LinkedCell  *linkedCellPtr;
ClientData  clientData;
{
    (void) Tp_PaintGCell(linkedCellPtr->cellPtr, planeDataPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
MoveGCell(linkedCellPtr, deltaPtr)
LinkedCell  *linkedCellPtr;
Point       *deltaPtr;
{
    if (linkedCellPtr->cellPtr->unType == CELL_INST) {
        (void) Mac_MoveCell(deltaPtr->p_x, deltaPtr->p_y,
            linkedCellPtr->cellPtr->unValue.cellInstPtr);
    }
#ifdef USE_DR_CODE
    else {
        Mac_MoveRouteCell(deltaPtr->p_x, deltaPtr->p_y,
            linkedCellPtr->cellPtr->unValue.routeCellPtr);
    }
#endif /* USE_DR_CODE */

    return (BEAR_CONTINUE_SEARCH);
}



static int
FindBnTile(tilePtr,bnTileListPtr)
Tile         *tilePtr;
LinkedTile   **bnTileListPtr;
{
    LinkedTile *newLinkedTilePtr;

    if (Tp_IsBottleneckTile(tilePtr) || Tp_IsSBNTile(tilePtr)) {
        if (!IsTileInTileList(*bnTileListPtr, tilePtr)) {
            MALLOC(LinkedTile *, newLinkedTilePtr, sizeof (LinkedTile));
            newLinkedTilePtr->nextPtr = *bnTileListPtr;
            newLinkedTilePtr->tilePtr = tilePtr;
            *bnTileListPtr = newLinkedTilePtr;
        }
    }

    return (0);
}


static bool
IsTileInTileList(tileList, tilePtr)
LinkedTile  *tileList;
Tile        *tilePtr;
{
    for ( ; tileList != NIL(LinkedTile) ; tileList = tileList->nextPtr)
    {
    if (tilePtr == tileList->tilePtr)
    {
        return (TRUE);
    }
    }
    return (FALSE);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_MoveCellInst() --
 *
 *    Move a cell instance.
 *
 * Results:
 *    BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *    The tile planes and the data base are updated.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_MoveCellInst(cellInstPtr, deltaPtr)
CellInst    *cellInstPtr;
Point       *deltaPtr;
{
    CheckData   checkData;
    GenericCell gcell;
    Transform   *oldTransPtr, newTrans;

    parentCellPtr = Mac_GetParentOfCell(cellInstPtr);
    planeDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);

    gcell.unType = CELL_INST;
    gcell.unValue.cellInstPtr = cellInstPtr;
    oldTransPtr = Mac_GetTransformOfGCell(&gcell);
    GeoTranslateTrans(oldTransPtr, deltaPtr->p_x, deltaPtr->p_y, &newTrans);

    checkData.strong   = FALSE;
    checkData.tileType = TT_SPACE;
    checkData.namePtr  = Mac_GetNameOfCellInst(cellInstPtr);
    checkData.planePtr = Tp_GetHorzPlane(planeDataPtr);
    checkData.bndPtr   = Tp_GetHorzPlaneBnd(planeDataPtr);
    checkData.rectList = Mac_GetHorzRectListOfGCell(&gcell);
    checkData.transPtr = &newTrans;
    if (!TpCheckPlaneUnderRectList(&checkData))
    {
        /*
         * The cells new position is not completely within the planes
         * boundary, or the area under it is not free.
         */
        return (BEAR_ERROR);
    }
    if (!Mac_IsGlobalRoutingDone(parentCellPtr))
    {
        Tp_UnpaintGCell(&gcell, planeDataPtr);
        (void) Mac_MoveCell(deltaPtr->p_x, deltaPtr->p_y, cellInstPtr);
        (void) Tp_PaintGCell(&gcell, planeDataPtr);
        return (BEAR_OK);
    }
    GeoTranslateTrans(oldTransPtr, deltaPtr->p_x, 0, &newTrans);
    if (TpCheckPlaneUnderRectList(&checkData))
    {
        HorzMoveCellInst(cellInstPtr, deltaPtr->p_x);
        VertMoveCellInst(cellInstPtr, deltaPtr->p_y);
        return (BEAR_OK);
    }
    GeoTranslateTrans(oldTransPtr, 0, deltaPtr->p_y, &newTrans);
    if (TpCheckPlaneUnderRectList(&checkData))
    {
        VertMoveCellInst(cellInstPtr, deltaPtr->p_y);
        HorzMoveCellInst(cellInstPtr, deltaPtr->p_x);
        return (BEAR_OK);
    }
    Bear_PrintErr("Tp_MoveCellInst(): No path to the new position.");
    return (BEAR_ERROR);
}



static void
HorzMoveCellInst(cellInstPtr, deltaX)
CellInst   *cellInstPtr;
int        deltaX;
{
    BnTileInfo   bnTileInfo;
    PPinInfo     *lPPinInfoList;
    PPinInfo     *rPPinInfoList;
    LinkedCell   linkedCell;
    GenericCell  gCell;

    if (deltaX == 0)
    {
    return;
    }
    Tm_HUnify(parentCellPtr);

    linkedCell.cellPtr        = &gCell;
    gCell.unType              = CELL_INST;
    gCell.unValue.cellInstPtr = cellInstPtr;

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) GetLAndRBnTiles(&linkedCell, &bnTileInfo);
    lPPinInfoList = rPPinInfoList = NIL(PPinInfo);
    SavePPinList(bnTileInfo.blBnTileList, BOTTOM_LEFT_SIDE, &lPPinInfoList);
    SavePPinList(bnTileInfo.trBnTileList, TOP_RIGHT_SIDE,   &rPPinInfoList);

    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);

    Tp_UnpaintGCell(&gCell, planeDataPtr);
    (void) Mac_MoveCell(deltaX, 0, cellInstPtr);
    (void) Tp_PaintGCell(&gCell, planeDataPtr);

    (void) Tp_IdentifySpaceTiles(planeDataPtr);
    (void) Fpg_CreateFloorPlanGraph(planeDataPtr, (WallJunction **) NULL);

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) GetLAndRBnTiles(&linkedCell, &bnTileInfo);
    RestorePPinList(bnTileInfo.blBnTileList, lPPinInfoList);
    RestorePPinList(bnTileInfo.trBnTileList, rPPinInfoList);

    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);
    FreePPinInfoList(lPPinInfoList);
    FreePPinInfoList(rPPinInfoList);

    Tm_HSplit(parentCellPtr);
}


static void
VertMoveCellInst(cellInstPtr, deltaY)
CellInst   *cellInstPtr;
int        deltaY;
{
    BnTileInfo   bnTileInfo;
    PPinInfo     *bPPinInfoList;
    PPinInfo     *tPPinInfoList;
    LinkedCell   linkedCell;
    GenericCell  gCell;

    if (deltaY == 0)
    {
    return;
    }
    Tm_VUnify(parentCellPtr);

    linkedCell.cellPtr        = &gCell;
    gCell.unType              = CELL_INST;
    gCell.unValue.cellInstPtr = cellInstPtr;

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) GetBAndTBnTiles(&linkedCell, &bnTileInfo);
    bPPinInfoList = tPPinInfoList = NIL(PPinInfo);
    SavePPinList(bnTileInfo.blBnTileList, BOTTOM_LEFT_SIDE, &bPPinInfoList);
    SavePPinList(bnTileInfo.trBnTileList, TOP_RIGHT_SIDE,   &tPPinInfoList);

    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);

    Tp_UnpaintGCell(&gCell, planeDataPtr);
    (void) Mac_MoveCell(0, deltaY, cellInstPtr);
    (void) Tp_PaintGCell(&gCell, planeDataPtr);

    (void) Tp_IdentifySpaceTiles(planeDataPtr);
    (void) Fpg_CreateFloorPlanGraph(planeDataPtr, (WallJunction **) NULL);

    bnTileInfo.blBnTileList = bnTileInfo.trBnTileList = NIL(LinkedTile);
    (void) GetBAndTBnTiles(&linkedCell, &bnTileInfo);
    RestorePPinList(bnTileInfo.blBnTileList, bPPinInfoList);
    RestorePPinList(bnTileInfo.trBnTileList, tPPinInfoList);

    Tp_FreeTileList(bnTileInfo.blBnTileList);
    Tp_FreeTileList(bnTileInfo.trBnTileList);
    FreePPinInfoList(bPPinInfoList);
    FreePPinInfoList(tPPinInfoList);

    Tm_VSplit(parentCellPtr);
}
