
/* tpPPinItr.c --
 *
 *     June Wang
 *     (Advising Professor - E. S. Kuh)
 *
 * This file contains the pseudo pin iterators.
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpPPinItr.c,v 1.1 87/09/24 10:24:25 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"


typedef struct _UsersData {
    BearStatus (*func)();
    ClientData clientData;
} UsersData;


/* ---------- Local Procedures ---------- */
static BearStatus DoPPin();
static BearStatus CountNumOfPPins();


/* ARGSUSED */
static BearStatus
CountNumOfPPins(ppinPtr, numPtr)
PseudoPin   *ppinPtr;
int         *numPtr;
{
    (*numPtr)++;
    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * DoPPin() --
 *
 * Apply the user function to the pseudo pins attached to the graph edge if any.
 *
 * Results:
 *   None.
 *
 * Side effects:
 *   Whatever the side effects the user function has.
 *
 * ----------------------------------------------------------------------------
 */

static BearStatus
DoPPin(edge, userDataPtr)
Gt_GraphEdge    edge;
UsersData       *userDataPtr;
{
    Mac_EdgeData *edgeDataPtr;

    edgeDataPtr = (Mac_EdgeData *) Gt_GetEdgeDataOfEdge(edge);
    if (edgeDataPtr != NIL(Mac_EdgeData)) {
        if (edgeDataPtr->ppin1Ptr != NIL(PseudoPin)) {
            if ((*(userDataPtr->func))(edgeDataPtr->ppin1Ptr,
                userDataPtr->clientData) == BEAR_STOP_SEARCH) {
                return (BEAR_STOP_SEARCH);
            }
        }
        if (edgeDataPtr->ppin2Ptr != NIL(PseudoPin)) {
            if ((*(userDataPtr->func))(edgeDataPtr->ppin2Ptr,
                userDataPtr->clientData) == BEAR_STOP_SEARCH) {
                return (BEAR_STOP_SEARCH);
            }
        }
    }
    return (BEAR_CONTINUE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_ItrPPinsOfBnTile() --
 *
 * It takes a bottle neck tile and iterates ppins on the BOTTOM_LEFT
 * side and on the TOP_RIGHT side by calling Tp_ItrPPinsOfSide().
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, when successful.
 *   BEAR_STOP_SEARCH, when the funciton returns the same status.
 *
 * Side effects:
 *   Whatever side effects result from application of the 
 *   supplied procedure.
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_ItrPPinsOfBnTile(bnTilePtr, func, clientData)
Tile        *bnTilePtr;             /* Pointer to a bottle neck tile. */
BearStatus  (*func)();
ClientData  clientData;
{
#ifdef DEBUG
    if (bnTilePtr == NIL(Tile)) 
    {
        TpPrintFatalErr("Tp_ItrPinsOfBnTile() failed : bnTilePtr not given");
    } else if (Tp_IsBottleneckTile (bnTilePtr) == FALSE &&
        Tp_IsSBNTile(bnTilePtr) == FALSE) {
TpPrintFatalErr("Tp_ItrPinsOfBnTile() failed : tile must be bottleneck or sbn tile");
    } 
    else if (func == (BearStatus (*)()) NULL) 
    {
        TpPrintFatalErr("Tp_ItrPinsOfBnTile() failed : func not given");
    }
#endif DEBUG

    if ((Tp_ItrPPinsOfSide(bnTilePtr, BOTTOM_LEFT_SIDE, func, clientData) ==
        BEAR_COMPLETE_SEARCH) &&
        (Tp_ItrPPinsOfSide(bnTilePtr, TOP_RIGHT_SIDE, func, clientData) ==
        BEAR_COMPLETE_SEARCH)) {

        return (BEAR_COMPLETE_SEARCH);
    } else {
        return (BEAR_STOP_SEARCH);
    }
    /* NOTREACHED */
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_ItrPPinsOfSide() --
 *
 * It takes a tile and iterates the ppins on the designated 
 * side.  Since a side is composed by TP_TOP and TP_BOTTOM fragments, 
 * Tp_ItrPPinsOfFragment() is called twice to achieve the goal of 
 * iterating the ppins on the given side.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, when successful.
 *   BEAR_STOP_SEARCH, when the function returns the same status.
 *
 * Side effects:
 *   Whatever side effects result from application of the 
 *   supplied procedure.
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_ItrPPinsOfSide(tilePtr, side, func, clientData)
Tile            *tilePtr;
SideDirection   side;
BearStatus      (*func)();
ClientData      clientData;
{

#ifdef DEBUG
    if (tilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_ItrPinsOfSide() failed : tilePtr not given");
    } else if (Tp_IsBottleneckTile (tilePtr) == FALSE &&
        Tp_IsSBNTile(tilePtr) == FALSE) {
TpPrintFatalErr("Tp_ItrPinsOfSide() failed : tile must be bottleneck or sbn tile");
    } else if ((side != BOTTOM_LEFT_SIDE) && (side != TOP_RIGHT_SIDE)) {
        TpPrintFatalErr("Tp_ItrPinsOfSide() failed : side has illegal value");
    } else if (func == (BearStatus (*)()) NULL) {
        TpPrintFatalErr("Tp_ItrPinsOfSide() failed : func not given");
    }
#endif DEBUG


    /*
     * If there are no pseudo pins on the fragment of this side, 
     * go for the other fragment.
     */
    if (Tp_ItrPPinsOfFragment(tilePtr, side, TP_BOTTOM,
        func, clientData) == BEAR_COMPLETE_SEARCH) 
    {
        if (Tp_ItrPPinsOfFragment(tilePtr, side, TP_TOP,
            func, clientData) == BEAR_COMPLETE_SEARCH)
        {
            return (BEAR_COMPLETE_SEARCH);
        }
        else
        {
            return (BEAR_STOP_SEARCH);
        }
    }
    else
    {
        return (BEAR_STOP_SEARCH);
    }
    /* NOTREACHED */
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_ItrPPinsOfFragment() --
 *
 * Iterates the ppins on the designated fragment of the designated side 
 * of the bottle neck tile.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, when successful.
 *   BEAR_STOP_SEARCH, when the function returns the same status.
 *
 * Side effects:
 *   Whatever side effects result from application of the 
 *   supplied procedure.
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_ItrPPinsOfFragment(tilePtr, side, fragment, func, clientData)
Tile            *tilePtr;
SideDirection   side;
int             fragment;
BearStatus      (*func)();
ClientData      clientData;

{
    PseudoPin   *ppinList;
    PseudoPin   *ppinPtr;

#ifdef DEBUG
    if (tilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_ItrPinsOfFragment() failed : tilePtr not given");
    } else if (Tp_IsBottleneckTile(tilePtr) == FALSE &&
        Tp_IsSBNTile(tilePtr) == FALSE) {
TpPrintFatalErr("Tp_ItrPinsOfFragment() failed : tile must be bottleneck or sbn tile");
    } else if ((side != BOTTOM_LEFT_SIDE) && (side != TOP_RIGHT_SIDE)) {
        TpPrintFatalErr("Tp_ItrPinsOfFragment() failed : side has illegal value");
    } else if ((fragment != TP_TOP) && (fragment != TP_BOTTOM)) {
        TpPrintFatalErr("Tp_ItrPinsOfFragment() failed : fragment has illegal value");
    } else if (func == (BearStatus (*)()) NULL) {
        TpPrintFatalErr("Tp_ItrPinsOfFragment() failed : func not given");
    }
#endif DEBUG

    ppinList = (side == (SideDirection) BOTTOM_LEFT_SIDE) ?
        *(Tp_GetSpacePropOfTile(tilePtr)->blPPinList[fragment]) :
        *(Tp_GetSpacePropOfTile(tilePtr)->trPPinList[fragment]);
    for (ppinPtr = ppinList;
        ppinPtr != NIL(PseudoPin);
        ppinPtr = ppinPtr->nextPtr) {

        if ((*func)(ppinPtr, clientData) == BEAR_STOP_SEARCH) {
            return (BEAR_STOP_SEARCH);
        }
    }
    return (BEAR_COMPLETE_SEARCH);
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_ItrPPinsOfBnLine() --
 *
 * It takes a pair of semi-bottleneck tiles and iterates the ppins on the 
 * (implied) fragment which the two tiles share. 
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, when successful.
 *   BEAR_STOP_SEARCH, when the function returns the same status.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_ItrPPinsOfBnLine(tlTilePtr, brTilePtr, func, clientData)
Tile            *tlTilePtr;
Tile            *brTilePtr;
BearStatus      (*func)();
ClientData      clientData;
{
    BNLineType      bnLineType;
    int             fragment;
    SideDirection   side;

#ifdef DEBUG
    if (tlTilePtr == NIL(Tile)) 
    {
        TpPrintFatalErr("Tp_ItrPinsOfBnLine() failed : tlTile not given");
    } 
    else if (brTilePtr == NIL(Tile)) 
    {
        TpPrintFatalErr("Tp_ItrPinsOfBnLine() failed : brTile not given");
    } 
#endif DEBUG

    if (Tp_IsPairOfSBNTiles(&tlTilePtr, &brTilePtr, &bnLineType) == FALSE)
    {
        TpPrintFatalErr("Tp_ItrPinsOfBnLine() failed : not pair of sbn tiles");
    }

    switch (bnLineType) {
    case TP_VERT_R_DOWN:
        side = TOP_RIGHT_SIDE;
        fragment = TP_BOTTOM;
        break;
    case TP_VERT_R_UP:
        side = TOP_RIGHT_SIDE;
        fragment = TP_TOP;
        break;
    case TP_HORZ_B_RIGHT:
        side = BOTTOM_LEFT_SIDE;
        fragment = TP_TOP;
        break;
    case TP_HORZ_B_LEFT:
        side = BOTTOM_LEFT_SIDE;
        fragment = TP_BOTTOM;
        break;
    }

    if (Tp_ItrPPinsOfFragment(tlTilePtr, side, fragment,
        func, clientData) == BEAR_STOP_SEARCH) {

        return (BEAR_STOP_SEARCH);
    }
    else {
        return (BEAR_COMPLETE_SEARCH);
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_ItrPPinsOfGraph() --
 *
 * Given a graph, it traverses once every pseudo pin attached to the 
 * edges of the graph.
 *
 * Results:
 *   BEAR_COMPLETE_SEARCH, if successful.
 *   BEAR_STOP_SEARCH, otherwise.
 *
 * Side effects:
 *   None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_ItrPPinsOfGraph(graph, func, clientData)
Gt_Graph   graph;
BearStatus (*func)();
ClientData clientData;
{
    UsersData   userData;

#ifdef DEBUG
    if (graph == (Gt_Graph) NULL) {
        TpPrintFatalErr("Tp_ItrPPinsOfGraph() failed: no graph given");
    }
    else if (func == (BearStatus (*)()) NULL) {
        TpPrintFatalErr("Tp_ItrPPinsOfGraph() failed: no function given");
    }
#endif DEBUG

    userData.func = func;
    userData.clientData = clientData;
    if (Gt_ItrEdgesOfGraph(graph, DoPPin, (ClientData) &userData) ==
        BEAR_STOP_SEARCH) {
        return (BEAR_STOP_SEARCH);
    }
    return (BEAR_COMPLETE_SEARCH);
}



BearStatus
Tp_NumOfPPinsOfGraph(graph, numPtr)
Gt_Graph    graph;
int         *numPtr;
{
    *numPtr = 0;
    if (Tp_ItrPPinsOfGraph(graph, CountNumOfPPins, (ClientData) numPtr) ==
        BEAR_COMPLETE_SEARCH) {
        return (BEAR_OK);
    }
    return (BEAR_ERROR);
}



