
/* tpPrint.c --
 *
 *     June Wang
 *     George Carvalho
 *     (Advising Professor - E. S. Kuh)
 *
 * This file contains debugging stuff.
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpDebug.c,v 1.1 87/09/24 10:23:58 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"


typedef struct _PrintPPinData {
    Net         *netPtr;
    int         count;
    bool        oneNetFlag;
} PrintPPinData;


/* ---------- Local Procedures ---------- */
BearStatus PrintPPinIds();



void
Tp_PrintPPinListsOfTile(tilePtr, netPtr)
Tile *tilePtr;
Net  *netPtr;
/*
 * 'tilePtr' must be either a bottleneck tile, a semi-bottleneck
 * tile, or a dummycell tile.
 *
 * if netPtr != NIL(Net) then
 *     Print out information about only those ppin's that belong
 *     to this net.
 */
{
    DummyCell       *dcellPtr;
    PrintPPinData   prtData;
    Ring            *ringPtr;
    BearString      ppinListName;

    if (Tp_IsBottleneckTile(tilePtr) || Tp_IsSBNTile(tilePtr)) {
        if (netPtr != NIL(Net)) {
            prtData.oneNetFlag = TRUE;
            prtData.netPtr = netPtr;
        }
        else {
            prtData.oneNetFlag = FALSE;
            prtData.netPtr = NIL(Net);
        }

        printf ("----------\n");
        printf ("tile id: %d\n", Tp_GetIdOfTile(tilePtr));

        if (prtData.oneNetFlag) {
        printf ("\nOnly ppin's belonging to the net named %s are shown.\n\n",
                Mac_GetNameOfNet(prtData.netPtr));
        }

        ppinListName = Tp_IsHorzTile(tilePtr) ? "Top" : "Right";

        /*
         * Print out a list of the pseudo pins in the bottom fragment
         * of the top/right side.
         */
        printf ("%s pseudo pin list (bottom fragment) :\n",
            ppinListName);
        prtData.count = 1;
        (void) Tp_ItrPPinsOfFragment(tilePtr,
            (SideDirection) TOP_RIGHT_SIDE, TP_BOTTOM,
            PrintPPinIds, (ClientData) &prtData);

        /*
         * Print out a list of the pseudo pins in the top fragment
         * of the top/right side.
         */
        printf ("%s pseudo pin list (top fragment) :\n",
            ppinListName);
        prtData.count = 1;
        (void) Tp_ItrPPinsOfFragment(tilePtr,
            (SideDirection) TOP_RIGHT_SIDE, TP_TOP,
            PrintPPinIds, (ClientData) &prtData);

        ppinListName = Tp_IsHorzTile(tilePtr) ? "Bottom" : "Left";

        /*
         * Print out a list of the pseudo pins in the bottom fragment
         * of the bottom/left side.
         */
        printf ("%s pseudo pin list (bottom fragment) :\n",
            ppinListName);
        prtData.count = 1;
        (void) Tp_ItrPPinsOfFragment(tilePtr,
            (SideDirection) BOTTOM_LEFT_SIDE, TP_BOTTOM,
            PrintPPinIds, (ClientData) &prtData);

        /*
         * Print out a list of the pseudo pins in the top fragment
         * of the bottom/left side.
         */
        printf ("%s pseudo pin list (top fragment) :\n",
            ppinListName);
        prtData.count = 1;
        (void) Tp_ItrPPinsOfFragment(tilePtr,
            (SideDirection) BOTTOM_LEFT_SIDE, TP_TOP,
            PrintPPinIds, (ClientData) &prtData);

        printf ("\n"); fflush(stdout);
    }
    else if (Tp_IsDCellTile(tilePtr)) {
        if (netPtr != NIL(Net)) {
            prtData.oneNetFlag = TRUE;
            prtData.netPtr = netPtr;
        }
        else {
            prtData.oneNetFlag = FALSE;
            prtData.netPtr = NIL(Net);
        }

        printf ("----------\n");
        dcellPtr = Tp_GetDCellOfTile(tilePtr);

        printf ("dummy cell id : %d\n\n", Mac_GetIdOfDCell(dcellPtr));

        if (prtData.oneNetFlag) {
        printf ("\nOnly ppin's belonging to the net named %s are shown.\n\n",
                Mac_GetNameOfNet(prtData.netPtr));
        }

        printf ("Top pseudo pin list :\n");
        prtData.count = 1;
        (void) Mac_ItrPPinOfDCell(dcellPtr, TOP_SIDE,
            PrintPPinIds, (ClientData) &prtData);

        printf ("Right pseudo pin list :\n");
        prtData.count = 1;
        (void) Mac_ItrPPinOfDCell(dcellPtr, RIGHT_SIDE,
            PrintPPinIds, (ClientData) &prtData);

        printf ("Bottom pseudo pin list :\n");
        prtData.count = 1;
        (void) Mac_ItrPPinOfDCell(dcellPtr, BOTTOM_SIDE,
            PrintPPinIds, (ClientData) &prtData);

        printf ("Left pseudo pin list :\n");
        prtData.count = 1;
        (void) Mac_ItrPPinOfDCell(dcellPtr, LEFT_SIDE,
            PrintPPinIds, (ClientData) &prtData);

        printf ("\n"); fflush(stdout);
    }

#ifdef USE_DR_CODE
    else if (Tp_IsRCellTile(tilePtr)) {
        printf ("----------\n");
        printf ("tilePtr = 0x%x\n", tilePtr);

        Mac_PrintPPinListsOfRouteCell(Tp_GetRCellOfTile(tilePtr), netPtr);

        fflush(stdout);
    }
    else if (Tp_IsRingTile(tilePtr)) {
        printf ("----------\n");
        printf ("tilePtr = 0x%x\n", tilePtr);

        ringPtr = Tp_GetRingOfTile(tilePtr);
        printf ("%s\n","inner bound pin list :\n");
        (void) Mac_ItrInBoundPinOfRing(ringPtr, Mac_PrintGPin,
            (ClientData) NULL);

        printf ("%s\n","outer bound pin list :\n");
        (void) Mac_ItrOutBoundPinOfRing(ringPtr, Mac_PrintGPin,
            (ClientData) NULL);
        printf ("\n"); fflush(stdout);
    } 
#endif /* USE_DR_CODE */

    else {
        (void) sprintf (bear_ErrorMsg, "%s\n%s",
       "you must choose a bottleneck tile, a dummy/route cell or ring tile\n");
        Bear_PrintErr(bear_ErrorMsg);
        return;
    }
}



void
Tp_PrintPPinListOfBnLine(bnLinePtr, netPtr)
BNLine *bnLinePtr;
Net    *netPtr;
/*
 * if netPtr != NIL(Net) then
 *     print out information about only those ppin's that belong
 *     to this net.
 */
{
    PrintPPinData prtData;

    if (netPtr != NIL(Net)) {
        prtData.oneNetFlag = TRUE;
        prtData.netPtr = netPtr;
    }
    else {
        prtData.oneNetFlag = FALSE;
        prtData.netPtr = NIL(Net);
    }

    printf ("----------\n");
    printf ("bottle neck line id : %d\n\n", Tp_GetIdOfBnLine(bnLinePtr));

    if (prtData.oneNetFlag) {
    printf ("\nOnly ppin's belonging to the net named %s are shown.\n\n",
            Mac_GetNameOfNet(prtData.netPtr));
    }

    printf ("pseudo pin list :\n");
    prtData.count = 1;
    (void) Tp_ItrPPinsOfBnLine(bnLinePtr->tlTilePtr,
        bnLinePtr->brTilePtr, PrintPPinIds, (ClientData) &prtData);
    printf ("\n"); fflush(stdout);
}



static BearStatus
PrintPPinIds(ppinPtr, prtDataPtr)
PseudoPin       *ppinPtr;
PrintPPinData   *prtDataPtr;
{
    Net *netPtr;

    netPtr = Tp_GetNetOfPPin(ppinPtr);
    if ((prtDataPtr->oneNetFlag == TRUE) && (netPtr != prtDataPtr->netPtr)) {
        return (BEAR_CONTINUE_SEARCH);
    }

    printf ("%4d) ", prtDataPtr->count);
    if (netPtr == NIL(Net)) {
        printf ("[net name = UNDEFINED] ");
        printf ("[net id = UNDEFINED] ");
    } else {
        printf ("[net name = %s] ", Mac_GetNameOfNet(netPtr));
        printf ("[net id = %d] ", Mac_GetIdOfNet(netPtr));
    }
    printf ("(int net id = %d) (ext net id = %d)\n",
        Tp_GetIntNetIdOfPPin(ppinPtr), Tp_GetExtNetIdOfPPin(ppinPtr));
    printf ("      (width = %f) (tilePtr = 0x%x)\n",
        Tp_GetWidthOfPPin(ppinPtr), Tp_GetTileOfPPin(ppinPtr));

    prtDataPtr->count++;

    return (BEAR_CONTINUE_SEARCH);
}



/* ARGSUSED */
BearStatus
Tp_PrintPPin(ppinPtr, clientData)
PseudoPin   *ppinPtr;
ClientData  *clientData;
{
    Net *netPtr;

    printf ("[ppin id = %d] (width = %f) (order = %d)\n",
        Tp_GetIdOfPPin(ppinPtr),
        Tp_GetWidthOfPPin(ppinPtr),
        Tp_GetOrderOfPPin(ppinPtr));

    if ((netPtr = Tp_GetNetOfPPin(ppinPtr)) == NIL(Net)) 
    {
        printf ("No net attached to the psuedo pin\n");
    }
    else 
    {
        printf ("(global net id = %d) (int net id = %d) (ext net id = %d)\n",
            Mac_GetIdOfNet(netPtr),
            Tp_GetIntNetIdOfPPin(ppinPtr), 
            Tp_GetExtNetIdOfPPin(ppinPtr));
    }
    return (BEAR_CONTINUE_SEARCH);
}



