
/* tpRectPaint.c --
 *
 *     George Carvalho
 *     Andrew Purshottam
 *     (Advising Professor - E. S. Kuh)
 *
 * Routines to paint rectangles into a given plane.
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpRectPaint.c,v 1.1 87/09/24 10:24:42 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "geometry.h"
#include "malloc.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"

/* ---------- Local Procedures ---------- */
int CheckTileType();

#ifdef TABLE
static PaintResultType paintTable[2][2] = {
    { TT_SPACE, TT_SPACE },
    { TT_SOLID, TT_SOLID }
};
#endif TABLE

int
TpPaintProc(oldTileType, paintTileType)
int oldTileType, paintTileType;
{
    if (oldTileType == TT_SPACE) {
        return (paintTileType);
    } else if (paintTileType == TT_SPACE) {
        return (TT_SPACE);
    } else if (oldTileType == paintTileType) {
        return (oldTileType);
    } else {
        TpPrintFatalErr("TpPaintProc() failed");
    }
    /* NOTREACHED */
}



void
TpPaintRectList(planePtr, rectList, transPtr, gcellPtr)
Plane       *planePtr;
LinkedRect  *rectList;
Transform   *transPtr;
GenericCell *gcellPtr;
{
    Rect                paintRect;
    TileType            tileType;
    register Tile       *tilePtr;
    register LinkedRect *lnRectPtr;

    tileType = TP_CELL_BASE + Mac_GetIdOfGCell(gcellPtr);
    for (lnRectPtr = rectList;
        lnRectPtr != NIL(LinkedRect);
        lnRectPtr = lnRectPtr->r_next) {

        GeoTransRect(transPtr, &lnRectPtr->r_r, &paintRect);
#ifdef TABLE
        DBPaintPlane(planePtr, &paintRect,
            paintTable[TT_SOLID],
            NIL(PaintUndoInfo));
#else
        DBPaintPlaneByProc(planePtr, &paintRect,
            TpPaintProc, tileType,
            NIL(PaintUndoInfo));
#endif TABLE
    }

    for (lnRectPtr = rectList;
        lnRectPtr != NIL(LinkedRect);
        lnRectPtr = lnRectPtr->r_next) {

        GeoTransRect(transPtr, &lnRectPtr->r_r, &paintRect);
        tilePtr = TiSrPointNoHint(planePtr, &paintRect.r_ll);
        Tp_CreateSolidTileProp(tilePtr, gcellPtr);
    }
}



void 
TpUnpaintRectList(planePtr, rectList, transPtr, tileType)
Plane       *planePtr;
LinkedRect  *rectList;
Transform   *transPtr;
TileType    tileType;
{
    Rect                unPaintRect;
    Tile                *tilePtr;
    register LinkedRect *lnRectPtr;

    for (lnRectPtr = rectList;
        lnRectPtr != NIL(LinkedRect);
        lnRectPtr = lnRectPtr->r_next) {

        GeoTransRect(transPtr, &lnRectPtr->r_r, &unPaintRect);
        tilePtr = TiSrPointNoHint(planePtr, &unPaintRect.r_ll);
        FREE((char *) tilePtr->ti_client);
#ifdef TABLE
        DBPaintPlane(planePtr, &unPaintRect,
            paintTable[tileType],
            NIL(PaintUndoInfo));
#else
        DBPaintPlaneByProc(planePtr, &unPaintRect,
            TpPaintProc, tileType,
            NIL(PaintUndoInfo));
#endif TABLE
    }
}



/*
 * ----------------------------------------------------------------------------
 *
 * TpCheckPlaneUnderRectList() --
 *
 * Always check to see that the rectangle checkDataPtr->bndPtr completely
 * surrounds each rectangle in the rectangle list checkDataPtr->rectList.
 * Touching between the two of them IS allowed.
 *
 * If checkDataPtr->strong is TRUE, then do a strong check which means
 * that each tile under the rectangle list must have the same type as
 * checkDataPtr->tileType.
 *
 * If checkDataPtr->strong is FALSE, then do a weak check which means
 * that each tile under the rectangle list may have the same type as
 * checkDataPtr->tileType, or belong to a cell instance named
 * checkDataPtr->namePtr.
 *
 * Results:
 *   TRUE if the above conditions are met, FALSE otherwise.
 *
 * Side effects:
 *   none
 * ----------------------------------------------------------------------------
 */

bool 
TpCheckPlaneUnderRectList(checkDataPtr)
CheckData *checkDataPtr;
{
    Rect                checkRect;
    bool                stillOkay = TRUE;
    register LinkedRect *lnRectPtr;

    for (lnRectPtr = checkDataPtr->rectList;
        (lnRectPtr != NIL(LinkedRect)) && stillOkay;
        lnRectPtr = lnRectPtr->r_next) {

        GeoTransRect(checkDataPtr->transPtr, &lnRectPtr->r_r, &checkRect);
        stillOkay = GEO_SURROUND(checkDataPtr->bndPtr, &checkRect) &&
            !TiSrArea(NIL(Tile), checkDataPtr->planePtr, &checkRect,
            CheckTileType, (ClientData) checkDataPtr);
    }
    return (stillOkay);
}



static int
CheckTileType(tilePtr, checkDataPtr)
register Tile   *tilePtr;
CheckData       *checkDataPtr;
{
    if (TiGetType(tilePtr) == checkDataPtr->tileType) {
        return (0); /* continue search */
    } else if (checkDataPtr->strong == FALSE) {

        if (Tp_IsCellInstTile(tilePtr) &&
            (strcmp(Mac_GetNameOfCellInst(Tp_GetCellInstOfTile(tilePtr)),
            checkDataPtr->namePtr) == 0)) {

            return (0); /* continue search */
        }
        else if (Tp_IsDCellTile(tilePtr))
        {
            return (0); /* continue search */
        }
    }
    return (1); /* stop search */
}


    
