
/* tpResizeParent.c --
 *
 *     Sangjin Hong
 *     (Advising Professor - E. S. Kuh)
 *
 * Resizing of core cell.
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpResizeParent.c,v 1.1 88/06/27 14:43:57 sangjin Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"


typedef struct _MovePinData {
    Point vector;
    Rect  *bboxPtr;
    double xRatio, yRatio;
} MovePinData;


/* ---------- Local Procedures ---------- */
static void         ResizeCellT();
static void         ResizeCellR();
static void         ResizeCellB();
static void         ResizeCellL();
static BearStatus   MovePinT();
static BearStatus   MovePinR();
static BearStatus   MovePinB();
static BearStatus   MovePinL();


/*
 * ----------------------------------------------------------------------------
 *
 * Tp_ResizeParentCell() --
 *
 *    Resize a core cell instance.
 *
 * Results:
 *    BEAR_OK is returned if everything went down okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *    The tile planes and the data base are updated.
 *
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_ResizeParentCell(moveDeltaX,moveDeltaY,resizeDeltaX,resizeDeltaY,
    coreInstPtr, corePlaneDataPtr)

int         moveDeltaX;
int         moveDeltaY;
int         resizeDeltaX;
int         resizeDeltaY;
CellInst    *coreInstPtr;
PlaneData   *corePlaneDataPtr;
{
    CellMaster      *parentCellPtr;
    GenericCell     gcell;
    PlaneData       *parentPlaneDataPtr;
    Rect            *horzBndPtr, *vertBndPtr;
    WallJunction    *llJunctPtr;

    horzBndPtr = Tp_GetHorzPlaneBnd(corePlaneDataPtr);
    vertBndPtr = Tp_GetVertPlaneBnd(corePlaneDataPtr);
    if (moveDeltaX == 0) {
        horzBndPtr->r_xtop += resizeDeltaX;
    }
    else {
        horzBndPtr->r_xbot += moveDeltaX;
    }
    Tp_HRectToVRect(horzBndPtr, vertBndPtr);

    if (moveDeltaY == 0) {
        horzBndPtr->r_ytop += resizeDeltaY;
    }
    else {
        horzBndPtr->r_ybot += moveDeltaY;
    }
    Tp_HRectToVRect(horzBndPtr, vertBndPtr);

    gcell.unType = CELL_INST;
    gcell.unValue.cellInstPtr = coreInstPtr;

    parentCellPtr = Mac_GetParentOfCell(coreInstPtr);
    parentPlaneDataPtr = Mac_GetCellPlaneOfCell(parentCellPtr);

    Tp_UnpaintGCell(&gcell, parentPlaneDataPtr);

    if (moveDeltaX == 0) {
        (void) ResizeCellR(resizeDeltaX, coreInstPtr);
    }
    else {
        (void) ResizeCellL(moveDeltaX, coreInstPtr);
    }

    if (moveDeltaY == 0) {
        (void) ResizeCellT(resizeDeltaY, coreInstPtr);
    }
    else {
        (void) ResizeCellB(moveDeltaY, coreInstPtr);
    }

    (void) Tp_PaintGCell(&gcell, parentPlaneDataPtr);
    Tp_IdentifySpaceTiles(parentPlaneDataPtr);
    if (Fpg_CreateFloorPlanGraph(parentPlaneDataPtr, &llJunctPtr) == 
        BEAR_ERROR) {
        return (BEAR_ERROR);
    }

    return (BEAR_OK);
}



static void
ResizeCellT(deltaY, cellInstPtr)
int         deltaY;
CellInst    *cellInstPtr;
{
    LinkedLineSeg   *boundary; 
    LinkedLineSeg   *alineSeg;
    Rect            bbox;
    Rect            *rectPtr;
    MovePinData     movePinData;
        
    boundary = Mac_GetBoundaryOfCell(cellInstPtr);
    bbox.r_ybot = bbox.r_ytop = boundary->lineSeg->line.point1.p_y;
    for (alineSeg = boundary->nextPtr; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_y > bbox.r_ytop) 
        {
            bbox.r_ytop = alineSeg->lineSeg->line.point1.p_y;
        }
    }
    for (alineSeg = boundary; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_y == bbox.r_ytop) 
        {
            alineSeg->lineSeg->line.point1.p_y += deltaY;
        }
        if (alineSeg->lineSeg->line.point2.p_y == bbox.r_ytop) 
        {
            alineSeg->lineSeg->line.point2.p_y += deltaY;
        }
    }
    Mac_PutBoundaryOfCell(boundary, cellInstPtr);
    movePinData.vector.p_y = deltaY;
    movePinData.bboxPtr = Mac_GetBBoxOfCell(cellInstPtr);
    movePinData.yRatio =
        (double) (movePinData.bboxPtr->r_ytop - movePinData.bboxPtr->r_ybot
        + movePinData.vector.p_y)
        / (double) (movePinData.bboxPtr->r_ytop - movePinData.bboxPtr->r_ybot);
    (void) Mac_ItrPinOfCellInst(cellInstPtr, MovePinT,  
        (ClientData) &movePinData);
        
    rectPtr = movePinData.bboxPtr;
    rectPtr->r_ytop += deltaY;
    Mac_PutBBoxOfCell(rectPtr, cellInstPtr);
    cellInstPtr->masterPtr->horzRectList->r_r.r_ytop += deltaY;
    cellInstPtr->masterPtr->vertRectList->r_r.r_ytop += deltaY; 
}



static void
ResizeCellR(deltaX, cellInstPtr)
int         deltaX;
CellInst    *cellInstPtr;
{
    LinkedLineSeg   *boundary; 
    LinkedLineSeg   *alineSeg;
    Rect            bbox;
    Rect            *rectPtr;
    MovePinData     movePinData;
        
    boundary = Mac_GetBoundaryOfCell(cellInstPtr);
    bbox.r_xbot = bbox.r_xtop = boundary->lineSeg->line.point1.p_x;
    for (alineSeg = boundary->nextPtr; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_x > bbox.r_xtop) 
        {
            bbox.r_xtop = alineSeg->lineSeg->line.point1.p_x;
        }
    }
    for (alineSeg = boundary; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_x == bbox.r_xtop) 
        {
            alineSeg->lineSeg->line.point1.p_x += deltaX;
        }
        if (alineSeg->lineSeg->line.point2.p_x == bbox.r_xtop) 
        {
            alineSeg->lineSeg->line.point2.p_x += deltaX;
        }
    }
    Mac_PutBoundaryOfCell(boundary, cellInstPtr);
    movePinData.vector.p_x = deltaX;
    movePinData.bboxPtr = Mac_GetBBoxOfCell(cellInstPtr);
    movePinData.xRatio =
        (double) (movePinData.bboxPtr->r_xtop - movePinData.bboxPtr->r_xbot
        + movePinData.vector.p_x)
        / (double) (movePinData.bboxPtr->r_xtop - movePinData.bboxPtr->r_xbot);
    (void) Mac_ItrPinOfCellInst(cellInstPtr, MovePinR,
        (ClientData) &movePinData);
        
    rectPtr = movePinData.bboxPtr;
    rectPtr->r_xtop += deltaX;
    Mac_PutBBoxOfCell(rectPtr, cellInstPtr);
    cellInstPtr->masterPtr->horzRectList->r_r.r_xtop += deltaX;
    cellInstPtr->masterPtr->vertRectList->r_r.r_xtop += deltaX;
}



static void
ResizeCellB(deltaY, cellInstPtr)
int         deltaY;
CellInst    *cellInstPtr;
{
    LinkedLineSeg   *boundary; 
    LinkedLineSeg   *alineSeg;
    Rect            bbox;
    Rect            *rectPtr;
    MovePinData     movePinData;
        
    boundary = Mac_GetBoundaryOfCell(cellInstPtr);
    bbox.r_ybot = bbox.r_ytop = boundary->lineSeg->line.point1.p_y;
    for (alineSeg = boundary->nextPtr; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_y < bbox.r_ybot) 
        {
            bbox.r_ybot = alineSeg->lineSeg->line.point1.p_y;
        }
    }
    for (alineSeg = boundary; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_y == bbox.r_ybot) 
        {
            alineSeg->lineSeg->line.point1.p_y += deltaY;
        }
        if (alineSeg->lineSeg->line.point2.p_y == bbox.r_ybot) 
        {
            alineSeg->lineSeg->line.point2.p_y += deltaY;
        }
    }
    Mac_PutBoundaryOfCell(boundary, cellInstPtr);
    movePinData.vector.p_y = deltaY;
    movePinData.bboxPtr = Mac_GetBBoxOfCell(cellInstPtr);
    movePinData.yRatio =
        (double) (movePinData.bboxPtr->r_ytop - movePinData.bboxPtr->r_ybot
        - movePinData.vector.p_y)
        / (double) (movePinData.bboxPtr->r_ytop - movePinData.bboxPtr->r_ybot);
    (void) Mac_ItrPinOfCellInst(cellInstPtr, MovePinB,
        (ClientData) &movePinData);
        
    rectPtr = movePinData.bboxPtr;
    rectPtr->r_ybot += deltaY;
    Mac_PutBBoxOfCell(rectPtr, cellInstPtr);
    cellInstPtr->masterPtr->horzRectList->r_r.r_ybot += deltaY;
    cellInstPtr->masterPtr->vertRectList->r_r.r_ybot += deltaY; 
}



static void
ResizeCellL(deltaX, cellInstPtr)
int         deltaX;
CellInst    *cellInstPtr;
{
    LinkedLineSeg   *boundary; 
    LinkedLineSeg   *alineSeg;
    Rect            bbox;
    Rect            *rectPtr;
    MovePinData     movePinData;
    
    boundary = Mac_GetBoundaryOfCell(cellInstPtr);
    bbox.r_xbot = bbox.r_xtop = boundary->lineSeg->line.point1.p_x;
    for (alineSeg = boundary->nextPtr; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_x < bbox.r_xbot)
        {
            bbox.r_xbot = alineSeg->lineSeg->line.point1.p_x;
        }
    }
    for (alineSeg = boundary; alineSeg != NIL(LinkedLineSeg);
        alineSeg = alineSeg->nextPtr) 
    {
        if (alineSeg->lineSeg->line.point1.p_x == bbox.r_xbot) 
        {
            alineSeg->lineSeg->line.point1.p_x += deltaX;
        }
        if (alineSeg->lineSeg->line.point2.p_x == bbox.r_xbot) 
        {
            alineSeg->lineSeg->line.point2.p_x += deltaX;
        }
    }
    Mac_PutBoundaryOfCell(boundary, cellInstPtr);
    movePinData.vector.p_x = deltaX;
    movePinData.bboxPtr = Mac_GetBBoxOfCell(cellInstPtr);
    movePinData.xRatio =
        (double) (movePinData.bboxPtr->r_xtop - movePinData.bboxPtr->r_xbot
        - movePinData.vector.p_x)
        / (double) (movePinData.bboxPtr->r_xtop - movePinData.bboxPtr->r_xbot);
    (void) Mac_ItrPinOfCellInst(cellInstPtr, MovePinL,
        (ClientData) &movePinData);
        
    rectPtr = movePinData.bboxPtr;
    rectPtr->r_xbot += deltaX;
    Mac_PutBBoxOfCell(rectPtr, cellInstPtr);
    cellInstPtr->masterPtr->horzRectList->r_r.r_xbot += deltaX;
    cellInstPtr->masterPtr->vertRectList->r_r.r_xbot += deltaX;
}



static BearStatus
MovePinT(pinPtr, movePinDataPtr)
PinInst *pinPtr;
MovePinData  *movePinDataPtr;
{
    Point *pinPosPtr;

    pinPosPtr = Mac_GetRelPosOfPin(pinPtr);
    if (pinPosPtr->p_x == movePinDataPtr->bboxPtr->r_xbot) 
    {
        pinPosPtr->p_y = (pinPosPtr->p_y - movePinDataPtr->bboxPtr->r_ybot)
                         * movePinDataPtr->yRatio 
             + movePinDataPtr->bboxPtr->r_ybot;
    }
    else if (pinPosPtr->p_x == movePinDataPtr->bboxPtr->r_xtop) 
    {
        pinPosPtr->p_y = (pinPosPtr->p_y - movePinDataPtr->bboxPtr->r_ybot)
                         * movePinDataPtr->yRatio 
             + movePinDataPtr->bboxPtr->r_ybot;
    } 
    else if (pinPosPtr->p_y == movePinDataPtr->bboxPtr->r_ytop) 
    {
        pinPosPtr->p_y += movePinDataPtr->vector.p_y;
    }
    Mac_PutRelPosOfPin(pinPosPtr, pinPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
MovePinR(pinPtr, movePinDataPtr)
PinInst *pinPtr;
MovePinData  *movePinDataPtr;
{
    Point *pinPosPtr;

    pinPosPtr = Mac_GetRelPosOfPin(pinPtr);
    if (pinPosPtr->p_y == movePinDataPtr->bboxPtr->r_ybot) 
    {
        pinPosPtr->p_x = (pinPosPtr->p_x - movePinDataPtr->bboxPtr->r_xbot)
                         * movePinDataPtr->xRatio 
             + movePinDataPtr->bboxPtr->r_xbot;
    }
    else if (pinPosPtr->p_y == movePinDataPtr->bboxPtr->r_ytop) 
    {
        pinPosPtr->p_x = (pinPosPtr->p_x - movePinDataPtr->bboxPtr->r_xbot)
                         * movePinDataPtr->xRatio 
             + movePinDataPtr->bboxPtr->r_xbot;
    } 
    else if (pinPosPtr->p_x == movePinDataPtr->bboxPtr->r_xtop) 
    {
        pinPosPtr->p_x += movePinDataPtr->vector.p_x;
    }
    Mac_PutRelPosOfPin(pinPosPtr, pinPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
MovePinB(pinPtr, movePinDataPtr)
PinInst *pinPtr;
MovePinData  *movePinDataPtr;
{
    Point *pinPosPtr;

    pinPosPtr = Mac_GetRelPosOfPin(pinPtr);
    if (pinPosPtr->p_x == movePinDataPtr->bboxPtr->r_xbot) 
    {
        pinPosPtr->p_y = movePinDataPtr->bboxPtr->r_ytop 
             - (movePinDataPtr->bboxPtr->r_ytop - pinPosPtr->p_y)
                         * movePinDataPtr->yRatio;
    } 
    else if (pinPosPtr->p_x == movePinDataPtr->bboxPtr->r_xtop) 
    {
        pinPosPtr->p_y = movePinDataPtr->bboxPtr->r_ytop 
             - (movePinDataPtr->bboxPtr->r_ytop - pinPosPtr->p_y)
                         * movePinDataPtr->yRatio;
    } 
    else if (pinPosPtr->p_y == movePinDataPtr->bboxPtr->r_ybot) 
    {
        pinPosPtr->p_y += movePinDataPtr->vector.p_y;
    }
    Mac_PutRelPosOfPin(pinPosPtr, pinPtr);
    return (BEAR_CONTINUE_SEARCH);
}


static BearStatus
MovePinL(pinPtr, movePinDataPtr)
PinInst *pinPtr;
MovePinData  *movePinDataPtr;
{
    Point *pinPosPtr;

    pinPosPtr = Mac_GetRelPosOfPin(pinPtr);
    if (pinPosPtr->p_y == movePinDataPtr->bboxPtr->r_ybot) 
    {
        pinPosPtr->p_x = movePinDataPtr->bboxPtr->r_xtop
             - (movePinDataPtr->bboxPtr->r_xtop - pinPosPtr->p_x)
                         * movePinDataPtr->xRatio;
    } 
    else if (pinPosPtr->p_y == movePinDataPtr->bboxPtr->r_ytop) 
    {
        pinPosPtr->p_x = movePinDataPtr->bboxPtr->r_xtop
             - (movePinDataPtr->bboxPtr->r_xtop - pinPosPtr->p_x)
                         * movePinDataPtr->xRatio;
    }
    if (pinPosPtr->p_x == movePinDataPtr->bboxPtr->r_xbot) 
    {
        pinPosPtr->p_x += movePinDataPtr->vector.p_x;
    }
    Mac_PutRelPosOfPin(pinPosPtr, pinPtr);
    return (BEAR_CONTINUE_SEARCH);
}
