
/* tpTile.c --
 *
 *     June Wang
 *     (Advising Professor - E. S. Kuh)
 *
 */

#ifndef lint
static char rcsid[] = "$Header: tpTile.c,v 1.1 87/09/24 10:24:46 winjamar Exp $";
#endif  not lint

#include "bearCopyRight.h"
#include <stdio.h>
#include "magic.h"
#include "malloc.h"
#include "geometry.h"
#include "bear.h"
#include "tile.h"
#include "fpg.h"
#include "gtGraph.h"
#include "tileProp.h"
#include "tilePropInt.h"
#include "mac.h"

/* ---------- Local Procedures ---------- */
BNLine  *MakeBnLine();
int     FreePPinsOfSpacePropTile();
int     FreeTilePropOfTile();


/* ARGSUSED */
static int
FreePPinsOfSpacePropTile(tilePtr, clientData)
Tile        *tilePtr;
ClientData  clientData;
{
    if ((Tp_IsTileFilled(tilePtr) == TRUE) &&
        (Tp_IsSpacePropTile(tilePtr) == TRUE)) {
        Tp_FreePPinsOfTile(tilePtr);
    }

    return (MAGIC_CONTINUE_SEARCH);
}



/* ARGSUSED */
static int
FreeTilePropOfTile(tilePtr, clientData)
Tile        *tilePtr;
ClientData  clientData;
{
    if (Tp_IsTileFilled(tilePtr) == TRUE) {
        if (TILE_PROP_PTR->unType == TP_SPACE_PROP) {
            /*
             * Free the memory for the 'SpaceProp' structure.
             */
            FREE((char *) TILE_PROP_PTR->unValue.spacePropPtr);
        }

        /*
         * Free the memory for the 'TileProp' structure.
         */
        FREE((char *) TILE_PROP_PTR);
        tilePtr->ti_client = (ClientData) MINFINITY;
    }

    return (MAGIC_CONTINUE_SEARCH);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * MakeBnLine() --
 *
 *    Allocates memory for a new bottleneck line, and initializes
 *    the fields.
 *
 * Results:
 *    A pointer to a bottleneck line is returned.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

static BNLine *
MakeBnLine()
{
    register BNLine *bnLinePtr;

    MALLOC(BNLine *, bnLinePtr, sizeof (BNLine));
    bnLinePtr->id = 0;
    bnLinePtr->bnLineType = TP_NULL_BNL;
    bnLinePtr->tlTilePtr = NIL(Tile);
    bnLinePtr->brTilePtr = NIL(Tile);
    bnLinePtr->segPtr = NIL(WallSegment);
    bnLinePtr->nextPtr = NIL(BNLine);
    return (bnLinePtr);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * Tp_MakePlaneData() --
 *
 *    Allocates memory for a new plane data structure, and initializes
 *    the fields.
 *
 * Results:
 *    A pointer to a plane data structure is returned.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

PlaneData *
Tp_MakePlaneData()
{
    register PlaneData *planeDataPtr;

    MALLOC(PlaneData *, planeDataPtr, sizeof (PlaneData));

    /* Set up tile planes. */
    planeDataPtr->pd_status = 0;

    Tp_PutHorzPlaneBnd(planeDataPtr, &GeoNullRect);
    Tp_PutVertPlaneBnd(planeDataPtr, &GeoNullRect);

    planeDataPtr->pd_horzPlanePtr = DBNewPlane((ClientData) TT_SPACE);
    planeDataPtr->pd_vertPlanePtr = DBNewPlane((ClientData) TT_SPACE);

    Tp_PutHorzBnLineList(planeDataPtr, NIL(BNLine));
    Tp_PutVertBnLineList(planeDataPtr, NIL(BNLine));

    Tp_PutDCellList(planeDataPtr, NIL(DummyCell));

    Tp_PutFpg(planeDataPtr, NIL(FloorPlanGraph));

    return (planeDataPtr);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * Tp_MakeSpaceProp() --
 *
 *    Allocates memory for a new space property structure, and initializes
 *    the fields.
 *
 * Results:
 *    A pointer to a space property structure is returned.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

SpaceProp *
Tp_MakeSpaceProp()
{
    register SpaceProp *spacePropPtr;

    MALLOC(SpaceProp *, spacePropPtr, sizeof (SpaceProp));
    spacePropPtr->id = 0;
    spacePropPtr->junctTableIndex = -1;
    spacePropPtr->capacity = 0;
    spacePropPtr->density = 0;
    spacePropPtr->ridgeWidth = 0;
    spacePropPtr->prevTilePtr = NIL(Tile);
    spacePropPtr->segPtr = NIL(WallSegment);

    MALLOC(PseudoPin **,
        spacePropPtr->blPPinList[TP_BOTTOM], sizeof (PseudoPin *));
    *spacePropPtr->blPPinList[TP_BOTTOM] = NIL(PseudoPin);

    MALLOC(PseudoPin **,
        spacePropPtr->blPPinList[TP_TOP], sizeof (PseudoPin *));
    *spacePropPtr->blPPinList[TP_TOP] = NIL(PseudoPin);

    MALLOC(PseudoPin **,
        spacePropPtr->trPPinList[TP_BOTTOM], sizeof (PseudoPin *));
    *spacePropPtr->trPPinList[TP_BOTTOM] = NIL(PseudoPin);

    MALLOC(PseudoPin **,
        spacePropPtr->trPPinList[TP_TOP], sizeof (PseudoPin *));
    *spacePropPtr->trPPinList[TP_TOP] = NIL(PseudoPin);

    spacePropPtr->mark = BEAR_UNMARKED;
    spacePropPtr->clientData = (ClientData) NULL;

    return (spacePropPtr);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * Tp_MakeTileProp() --
 *
 *    Allocates memory for a new tile property structure, and initializes
 *    the fields.
 *
 * Results:
 *    A pointer to a tile property structure is returned.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

TileProp *
Tp_MakeTileProp()
{
    TileProp *tilePropPtr;

    MALLOC(TileProp *, tilePropPtr, sizeof (TileProp));
    tilePropPtr->mark = BEAR_UNMARKED;

    return (tilePropPtr);
}



void
Tp_CreateSpaceTileProp(tilePtr)
Tile *tilePtr;
/*
 * Allocates memory for both the 'TileProp' and 'SpaceProp' structures and
 * attaches them to the tile.
 */
{
#ifdef DEBUG
    if (tilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_CreateSpaceTileProp() failed: no tile given");
    }
#endif DEBUG
    
    tilePtr->ti_client = (ClientData) Tp_MakeTileProp();
    Tp_PutSpacePropOfTile(Tp_MakeSpaceProp(), tilePtr);
    Tp_PutIdOfTile(Mac_GenerateId(), tilePtr);
}



void
Tp_CreateSolidTileProp(tilePtr, gcellPtr)
Tile        *tilePtr;
GenericCell *gcellPtr;
/*
 * Allocates memory for the 'TileProp' structure and attaches it and the
 * cell to the tile.
 */
{
#ifdef DEBUG
    if (tilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_CreateSolidTileProp() failed: no tile given");
    } else if (gcellPtr == NIL(GenericCell)) {
    TpPrintFatalErr("Tp_CreateSolidTileProp() failed: no generic cell given");
    }
#endif DEBUG
    
#ifdef TABLE
    tilePtr->ti_body = (ClientData) TT_SOLID;
#else
    tilePtr->ti_body = (ClientData) (TP_CELL_BASE + Mac_GetIdOfGCell(gcellPtr));
#endif TABLE

    tilePtr->ti_client = (ClientData) Tp_MakeTileProp();
    switch (gcellPtr->unType) {
    case (CellClass) CELL_INST:
        Tp_PutCellInstOfTile(gcellPtr->unValue.cellInstPtr, tilePtr);
        break;
    case (CellClass) DUMMY_CELL:
        Tp_PutDCellOfTile(gcellPtr->unValue.dummyCellPtr, tilePtr);
        break;
    case (CellClass) ROUTE_CELL:
        Tp_PutRCellOfTile(gcellPtr->unValue.routeCellPtr, tilePtr);
        break;
    default:
        TpPrintFatalErr("Tp_CreateSolidTileProp() failed : unknown cell class");
        break;
    }
}



#ifdef USE_DR_CODE
void
Tp_CreateSolidTilePropForRing(tilePtr, ringPtr)
Tile    *tilePtr;
Ring    *ringPtr;
/*
 * Allocates memory for the 'TileProp' structure and attaches it and the
 * ring to the tile.
 */
{
#ifdef DEBUG
    if (tilePtr == NIL(Tile)) {
   TpPrintFatalErr("Tp_CreateSolidTilePropForRing() failed: no tile is given");
    } else if (ringPtr == NIL(Ring)) {
   TpPrintFatalErr("Tp_CreateSolidTilePropForRing() failed: no ring is given");
    }
#endif DEBUG
    
    tilePtr->ti_body = (ClientData) (TP_CELL_BASE + Mac_GetIdOfRing(ringPtr));

    tilePtr->ti_client = (ClientData) Tp_MakeTileProp();
    Tp_PutRingOfTile(ringPtr, tilePtr);
}
#endif /* USE_DR_CODE */



/*
 * ----------------------------------------------------------------------------
 *
 * Tp_CreateBnLine() --
 *
 *  Creates an empty list of pseudo pins and sets both tiles to point
 *  at that list.  Also creates and fills in '*bnLinePtrPtr' and then
 *  adds it to the top of '*bnLineListPtr'.  contains partially hydro-
 *  genated palm oil  Note that 'bnLineType' indicates which fragment
 *  of the side of 'tlTilePtr' is to become part of the BN line.
 *
 * Note:
 *     As long as 'tlTilePtr' and 'brTilePtr' are semi-bottleneck
 *     tiles they don't have to be handed to this routine in any
 *     particular order.
 *
 * Results:
 *     BEAR_OK is returned if everything went okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *     None.
 * ----------------------------------------------------------------------------
 */

BearStatus
Tp_CreateBnLine(tlTilePtr, brTilePtr, bnLineType, bnLineListPtr, bnLinePtrPtr)
Tile        *tlTilePtr, *brTilePtr;
BNLineType  bnLineType;                 /* bottle neck line type            */
BNLine      **bnLineListPtr, **bnLinePtrPtr;

{
    PseudoPin **newPPinPtrPtr;
    SpaceProp *tlPropPtr;
    SpaceProp *brPropPtr;

#ifdef DEBUG
    if (tlTilePtr == NIL(Tile) || brTilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_CreateBnLine() failed");
    } else if ((bnLineType != TP_VERT_R_UP) && (bnLineType != TP_VERT_R_DOWN) &&
        (bnLineType != TP_HORZ_B_RIGHT) && (bnLineType != TP_HORZ_B_LEFT)) {
        TpPrintFatalErr("Tp_CreateBnLine() failed : illegal bn line type");
    }
#endif DEBUG

    if (Tp_IsPairOfSBNTiles(&tlTilePtr, &brTilePtr, &bnLineType) == FALSE) {
        return (BEAR_ERROR);
    }

    tlPropPtr = Tp_GetSpacePropOfTile(tlTilePtr);
    brPropPtr = Tp_GetSpacePropOfTile(brTilePtr);
    switch (bnLineType) {
    case TP_VERT_R_UP:
        if (tlPropPtr->trPPinList[TP_TOP] == (PseudoPin **) NULL ||
            tlPropPtr->trPPinList[TP_TOP] != brPropPtr->blPPinList[TP_BOTTOM])
        {
            MALLOC(PseudoPin **, newPPinPtrPtr, sizeof (PseudoPin *));
            *newPPinPtrPtr = NIL(PseudoPin);
            tlPropPtr->trPPinList[TP_TOP] =
                brPropPtr->blPPinList[TP_BOTTOM] = newPPinPtrPtr;
        }
        break;

    case TP_VERT_R_DOWN:
        if (tlPropPtr->trPPinList[TP_BOTTOM] == (PseudoPin **) NULL ||
            tlPropPtr->trPPinList[TP_BOTTOM] != brPropPtr->blPPinList[TP_TOP])
        {
            MALLOC(PseudoPin **, newPPinPtrPtr, sizeof (PseudoPin *));
            *newPPinPtrPtr = NIL(PseudoPin);
            tlPropPtr->trPPinList[TP_BOTTOM] =
                brPropPtr->blPPinList[TP_TOP] = newPPinPtrPtr;
        }
        break;

    case TP_HORZ_B_RIGHT:
        if (tlPropPtr->blPPinList[TP_TOP] == (PseudoPin **) NULL ||
            tlPropPtr->blPPinList[TP_TOP] != brPropPtr->trPPinList[TP_BOTTOM])
        {
            MALLOC(PseudoPin **, newPPinPtrPtr, sizeof (PseudoPin *));
            *newPPinPtrPtr = NIL(PseudoPin);
            tlPropPtr->blPPinList[TP_TOP] =
                brPropPtr->trPPinList[TP_BOTTOM] = newPPinPtrPtr;
            /* 
             * PROPOSED CODE
             *
            if (brPropPtr->trPPinList[TP_BOTTOM]) != (PseudoPin **) NULL) {
                FREE((char *) brPropPtr->trPPinList[TP_BOTTOM]);
            }

            if (tlPropPtr->blPPinList[TP_TOP]) != (PseudoPin **) NULL) {
                brPropPtr->trPPinList[TP_BOTTOM] = 
                    tlPropPtr->blPPinList[TP_TOP];
            } else {
                MALLOC(PseudoPin **, newPPinPtrPtr, sizeof (PseudoPin *));
                brPropPtr->trPPinList[TP_BOTTOM] = 
                    tlPropPtr->blPPinList[TP_TOP] = newPPinPtrPtr;
            }
             */
        }
        break;

    case TP_HORZ_B_LEFT:
        if (tlPropPtr->blPPinList[TP_BOTTOM] == (PseudoPin **) NULL ||
            tlPropPtr->blPPinList[TP_BOTTOM] != brPropPtr->trPPinList[TP_TOP])
        {
            MALLOC(PseudoPin **, newPPinPtrPtr, sizeof (PseudoPin *));
            *newPPinPtrPtr = NIL(PseudoPin);
            tlPropPtr->blPPinList[TP_BOTTOM] =
                brPropPtr->trPPinList[TP_TOP] = newPPinPtrPtr;
        }
        break;
    }
    *bnLinePtrPtr = MakeBnLine();
    (*bnLinePtrPtr)->id = Mac_GenerateId();
    (*bnLinePtrPtr)->bnLineType = bnLineType;
    (*bnLinePtrPtr)->tlTilePtr = tlTilePtr;
    (*bnLinePtrPtr)->brTilePtr = brTilePtr;
    (void) Tp_FindBnLine(tlTilePtr, brTilePtr, &(*bnLinePtrPtr)->line);
    (*bnLinePtrPtr)->nextPtr = *bnLineListPtr;
    *bnLineListPtr = *bnLinePtrPtr;

    return (BEAR_OK);
}



BearStatus
Tp_FindBnLine(tlTilePtr, brTilePtr, linePtr)
Tile *tlTilePtr;                    /* Pointer to the top (left) tile of
                                     * a semi-bottleneck pair.
                                     */
Tile *brTilePtr;                    /* Pointer to the bottom (right) tile of
                                     * a semi-bottleneck pair.
                                     */
Line *linePtr;

/*
 * Find the xy coordinates of the bottle neck line that exists between
 * 'tlTilePtr' and 'brTilePtr', and place them in 'linePtr'.
 *
 * Note:
 *     As long as 'tlTilePtr' and 'brTilePtr' are semi-bottleneck
 *     tiles they don't whang-chung have to be handed to this
 *     routine in any particular order.
 *
 * Results:
 *    BEAR_OK is returned if everything went okay, BEAR_ERROR otherwise.
 *
 * Side effects:
 *    None.
 *
 */
{
    BNLineType  bnLineType;         /* bottle neck line type                */
    Rect        tlRect;             /* top (left) rectangle                 */
    Rect        brRect;             /* bottom (right) rectangle             */
    Rect        tmpRect; 

#ifdef DEBUG
    if (tlTilePtr == NIL(Tile) || brTilePtr == NIL(Tile) ||
        linePtr == NIL(Line)) {
        TpPrintFatalErr("Tp_FindBnLine() failed");
    }
#endif DEBUG
    
    if (Tp_IsPairOfSBNTiles(&tlTilePtr, &brTilePtr, &bnLineType) == FALSE) {
        return (BEAR_ERROR);
    }

    /*
     * Convert the tiles to rectangles.
     */
    if (bnLineType == TP_HORZ_B_LEFT || bnLineType == TP_HORZ_B_RIGHT) {
        TITORECT(tlTilePtr, &tlRect);
        TITORECT(brTilePtr, &brRect);
    } else {
        TITORECT(tlTilePtr, &tmpRect);
        Tp_VRectToHRect(&tmpRect, &tlRect);
        TITORECT(brTilePtr, &tmpRect);
        Tp_VRectToHRect(&tmpRect, &brRect);
    }

    /*
     * Determine the coordinates of the bottleneck line.
     */
    switch (bnLineType)
    {
    case TP_VERT_R_UP:
        linePtr->point1.p_x = linePtr->point2.p_x = tlRect.r_xtop;
        linePtr->point1.p_y = tlRect.r_ytop;
        linePtr->point2.p_y = brRect.r_ybot;
        break;
    case TP_VERT_R_DOWN:
        linePtr->point1.p_x = linePtr->point2.p_x = tlRect.r_xtop;
        linePtr->point1.p_y = brRect.r_ytop;
        linePtr->point2.p_y = tlRect.r_ybot;
        break;
    case TP_HORZ_B_RIGHT:
        linePtr->point1.p_x = brRect.r_xbot;
        linePtr->point2.p_x = tlRect.r_xtop;
        linePtr->point1.p_y = linePtr->point2.p_y = tlRect.r_ybot;
        break;
    case TP_HORZ_B_LEFT:
        linePtr->point1.p_x = tlRect.r_xbot;
        linePtr->point2.p_x = brRect.r_xtop;
        linePtr->point1.p_y = linePtr->point2.p_y = tlRect.r_ybot;
        break;
    }

    return (BEAR_OK);
}



void
Tp_FreePlaneData(planeDataPtr)
PlaneData *planeDataPtr;
/*
 * Still need to do something with pseudo pins of bottle neck lines,
 * as well as the bottle neck lines themselves.
 */
{
    Plane *horzPlanePtr, *vertPlanePtr; 

    if (planeDataPtr == NIL(PlaneData)) {
        return;
    }

    /*
     * Free all the dummy cells.
     */
    Tp_FreeDCellList(planeDataPtr);

    /*
     * Free all the pseudo pins in the space tiles.
     */
    Tp_FreeAllPPinsOnHorzPlane(planeDataPtr);
    Tp_FreeAllPPinsOnVertPlane(planeDataPtr);

    /*
     * Free the floor plan graph.
     */
    Fpg_DeleteFloorPlanGraph(planeDataPtr);

    horzPlanePtr = Tp_GetHorzPlane(planeDataPtr);
    vertPlanePtr = Tp_GetVertPlane(planeDataPtr);

    /*
     * Free all of the tile property structures associated with each tile.
     */
    (void) TiSrArea(NIL(Tile), horzPlanePtr,
        Tp_GetHorzPlaneBnd(planeDataPtr),
        FreeTilePropOfTile,
        (ClientData) NULL);
    (void) TiSrArea(NIL(Tile), vertPlanePtr,
        Tp_GetVertPlaneBnd(planeDataPtr),
        FreeTilePropOfTile,
        (ClientData) NULL);

    /*
     * Free all of the tiles
     */
    DBFreePaintPlane(horzPlanePtr);
    DBFreePaintPlane(vertPlanePtr);

    /*
     * Free the tile planes.
     */
    TiFreePlane(horzPlanePtr);
    TiFreePlane(vertPlanePtr);

    FREE((char *) planeDataPtr);
}



void
Tp_FreeTileProp(tilePtr)
Tile *tilePtr;
/*
 * Free all the memory that we have allocated for this tile.
 */
{
#ifdef DEBUG
    if (tilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_FreeTileProp() failed : no tile given");
    }
#endif DEBUG

    if (Tp_IsTileFilled(tilePtr) == TRUE) {
        switch (TILE_PROP_PTR->unType) {
        case (TileKind) TP_SPACE_PROP:
            /*
             * Free the memory for the 'SpaceProp' structure.
             */
            FREE((char *) TILE_PROP_PTR->unValue.spacePropPtr);
            break;
        default:
            Bear_PrintErr("Tp_FreeTileProp() failed : not a space tile");
            return;
        }

        /*
         * Free the memory for the 'TileProp' structure.
         */
        FREE((char *) TILE_PROP_PTR);
        tilePtr->ti_client = (ClientData) MINFINITY;
    }
}



void
Tp_FreeAllPPinsOnHorzPlane(planeDataPtr)
PlaneData *planeDataPtr;
{
    (void) TiSrArea(NIL(Tile), Tp_GetHorzPlane(planeDataPtr),
        Tp_GetHorzPlaneBnd(planeDataPtr),
        FreePPinsOfSpacePropTile,
        (ClientData) NULL);
}



void
Tp_FreeAllPPinsOnVertPlane(planeDataPtr)
PlaneData *planeDataPtr;
{
    (void) TiSrArea(NIL(Tile), Tp_GetVertPlane(planeDataPtr),
        Tp_GetVertPlaneBnd(planeDataPtr),
        FreePPinsOfSpacePropTile,
        (ClientData) NULL);
}



void
Tp_RemoveTileProp(tilePtr, tilePropPtrPtr)
Tile        *tilePtr;
TileProp    **tilePropPtrPtr;
{
#ifdef DEBUG
    if (tilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_RemoveTileProp() failed: no tile given");
    }
#endif DEBUG
    
    *tilePropPtrPtr = TILE_PROP_PTR;
    tilePtr->ti_client = (ClientData) MINFINITY;
}



void
Tp_AddTileProp(tilePropPtr, tilePtr)
TileProp    *tilePropPtr;
Tile        *tilePtr;
{
#ifdef DEBUG
    if (tilePropPtr == NIL(TileProp)) {
        TpPrintFatalErr("Tp_AddTileProp() failed: tile properties not given");
    } else if (tilePtr == NIL(Tile)) {
        TpPrintFatalErr("Tp_AddTileProp() failed: no tile given");
    }
#endif DEBUG
    
    tilePtr->ti_client = (ClientData) tilePropPtr;
}



void
Tp_HorzTileToRect(planeDataPtr, horzTilePtr, rectPtr)
register PlaneData  *planeDataPtr;
register Tile       *horzTilePtr;
register Rect       *rectPtr;
{
#ifdef DEBUG
    if ((planeDataPtr == NIL(PlaneData)) || (horzTilePtr == NIL(Tile)) ||
        (rectPtr == NIL(Rect))) {
        TpPrintFatalErr("Tp_HorzTileToRect() failed");
    }
#endif DEBUG

    TITORECT(horzTilePtr, rectPtr);
    GeoClip(rectPtr, Tp_GetHorzPlaneBnd(planeDataPtr));
}



void
Tp_VertTileToRect(planeDataPtr, vertTilePtr, rectPtr)
register PlaneData  *planeDataPtr;
register Tile       *vertTilePtr;
register Rect       *rectPtr;
{
    Rect rect;

#ifdef DEBUG
    if ((planeDataPtr == NIL(PlaneData)) || (vertTilePtr == NIL(Tile)) ||
        (rectPtr == NIL(Rect))) {
        TpPrintFatalErr("Tp_VertTileToRect() failed");
    }
#endif DEBUG

    TITORECT(vertTilePtr, rectPtr);
    GeoClip(rectPtr, Tp_GetVertPlaneBnd(planeDataPtr));
    Tp_VRectToHRect(rectPtr, &rect);
    Mac_CopyRect(rectPtr, &rect);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * Tp_ResizeTiPlane() --
 *
 * Resize the tile plane by adjusting the corners
 * of the plane's boundary.  Note that such an adjustment must be
 * made to both the horizontal and the vertical boundaries.
 *
 * Results:
 *    None.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

void
Tp_ResizeTiPlane(planeDataPtr, deltaX, deltaY)
PlaneData   *planeDataPtr;
int         deltaX, deltaY;
{
    Rect *horzBndPtr, *vertBndPtr;

    horzBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    horzBndPtr->r_xtop += deltaX;
    horzBndPtr->r_ytop += deltaY;

    vertBndPtr = Tp_GetVertPlaneBnd(planeDataPtr);
    Tp_HRectToVRect(horzBndPtr, vertBndPtr);
}



/* 
 * ----------------------------------------------------------------------------
 *
 * Tp_MoveTiPlane() --
 *
 * Moves the tile plane of the plane's boundary.  
 * Note that such an adjustment must be made to both the horizontal 
 * and the vertical boundaries.
 *
 * Results:
 *    None.
 *
 * Side effects:
 *    None.
 *
 * ----------------------------------------------------------------------------
 */

void
Tp_MoveTiPlane(planeDataPtr, deltaX, deltaY)
PlaneData   *planeDataPtr;
int         deltaX, deltaY;
{
    Rect *horzBndPtr, *vertBndPtr;

    horzBndPtr = Tp_GetHorzPlaneBnd(planeDataPtr);
    vertBndPtr = Tp_GetVertPlaneBnd(planeDataPtr);
    horzBndPtr->r_xbot += deltaX;
    horzBndPtr->r_ybot += deltaY;
    horzBndPtr->r_xtop += deltaX;
    horzBndPtr->r_ytop += deltaY;

    Tp_HRectToVRect(horzBndPtr, vertBndPtr);
}



